/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *      
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *      
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: FilesInterface.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Last modified: 21.7.2008
 */

package ue.edit;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;

import java.util.zip.Adler32;
import java.util.zip.CRC32;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.SwingWorker;

import ue.Language;
import ue.UE;
import ue.edit.snd.SndFile;
import ue.util.CRC16;
import ue.util.Fletcher16;
import ue.util.GenericFile;

/**
 * This class is used to add/remove internal files to/from CanBeEdited classes
 */
public class FilesInterface
{
    private CanBeEdited MOTHERSHIP = null;
    private Vector<String[]> filelist;

    /**
     * Constructor
     * @param file the file to work with
     */
    public FilesInterface(CanBeEdited file)
    {
        this.MOTHERSHIP = file;
    }
    
    /**
     * @return an array of intefnal filesthe open file will contain when it's saved
     */
    public synchronized String[][] getFiles() throws Exception
    {
        // variables
        Vector<String[]> answer = new Vector<String[]>();
        String[] item;
        
        Enumeration<String[]> en = this.MOTHERSHIP.getFiles(new java.io.FilenameFilter()
        {
            @Override
            public boolean accept(File dir, String name)
            {
                return true;
            }
        });
        
        //go thru items
        while(en.hasMoreElements())
        {
            item = en.nextElement();
            answer.add(item);
        }
        
        String[][] ret = new String[3][answer.size()];
        String glt = null;
        String file;

        for (int i = 0; i < answer.size(); i++)
        {
            item = answer.get(i);

            ret[0][i] = item[0];
            
            if (glt != null)
            {
                ret[1][i] = glt;
            }
            else
            {
                if (item[0].toLowerCase().endsWith(".tga")) //$NON-NLS-1$
                {
                    ret[1][i] = Language.getString("FilesInterface.5"); //$NON-NLS-1$
                }
                else if (item[0].toLowerCase().endsWith(".gif")) //$NON-NLS-1$
                {
                    ret[1][i] = Language.getString("FilesInterface.6"); //$NON-NLS-1$
                }
                else if (item[0].toLowerCase().endsWith(".wtf")) //$NON-NLS-1$
                {
                    ret[1][i] = Language.getString("FilesInterface.7"); //$NON-NLS-1$
                }
                else if (item[0].toLowerCase().endsWith(".wdf")) //$NON-NLS-1$
                {
                    ret[1][i] = Language.getString("FilesInterface.8"); //$NON-NLS-1$
                }
                else if (item[0].toLowerCase().endsWith(".ani")) //$NON-NLS-1$
                {
                    ret[1][i] = Language.getString("FilesInterface.9"); //$NON-NLS-1$
                }
                else if (item[0].toLowerCase().endsWith(".cur")) //$NON-NLS-1$
                {
                    ret[1][i] = Language.getString("FilesInterface.10"); //$NON-NLS-1$
                }
                else if (item[0].toLowerCase().endsWith(".dic")) //$NON-NLS-1$
                {
                    ret[1][i] = Language.getString("FilesInterface.11"); //$NON-NLS-1$
                }
                else if (item[0].toLowerCase().endsWith(".fnt")) //$NON-NLS-1$
                {
                    ret[1][i] = Language.getString("FilesInterface.12"); //$NON-NLS-1$
                }
                else if (item[0].toLowerCase().endsWith(".tdb")) //$NON-NLS-1$
                {
                    ret[1][i] = Language.getString("FilesInterface.13"); //$NON-NLS-1$
                }
                else
                {
                    if (item[0].toLowerCase().lastIndexOf(".") >= 0) //$NON-NLS-1$
                    {
                        String a = item[0];
                        ret[1][i] = Language.getString("FilesInterface.14").replace("%1", a.substring(a.lastIndexOf(".") + 1).toUpperCase()); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                    }
                    else
                    {
                        ret[1][i] = Language.getString("FilesInterface.15"); //$NON-NLS-1$
                    }
                }
            }

            // size
            ret[2][i] = item[1];
        }
        
        return ret;
    }
    
    /**
     * Discards changes.
     */
    public synchronized void discard()
    {
        this.MOTHERSHIP.discard();
    }
    
    /**
     * Removes item.
     * @param a the name of the internal file to be removed.
     */
    public synchronized void remove(String a){
        this.MOTHERSHIP.removeInternalFile(a);
    }
    
    public synchronized InternalFile getInternalFile(String file) throws Exception
    {
        if (!(MOTHERSHIP instanceof SndFile))
        {
            return MOTHERSHIP.getInternalFile(file, false);
        }
        else
        {
            return null;
        }
    }
    
    /**
     * Adds internal file.
     * @param a the files to be added
     */
    public synchronized void add(File[] a) throws IOException{	
        final FilesTask TASK = new FilesTask(a);
        
        final JOptionPane opti = new JOptionPane(Language.getString("FilesInterface.0"), //$NON-NLS-1$
                JOptionPane.PLAIN_MESSAGE);

        opti.setOptions(new String[]{Language.getString("FilesInterface.1")}); //$NON-NLS-1$
        
        final JDialog dia = opti.createDialog(UE.WINDOW, UE.APP_NAME);

        TASK.addPropertyChangeListener(new PropertyChangeListener()
        {
            public void propertyChange(PropertyChangeEvent evt)
            {
                opti.setMessage(evt.getNewValue().toString());
                if (TASK.isDone())
                {
                    dia.dispose();
                }
            }
        });
        
        TASK.execute();
        
        dia.setVisible(true);
        
        if (Language.getString("FilesInterface.1").equals(opti.getValue())) //$NON-NLS-1$
        {
            TASK.cancel(false);
        }
    }
    
    /**
     * Extracts file.
     * @param inFile the names of the files to extract.
     * @param dest the destination where to extract the file to.
     * @throws Exception if an exception occurs
     */
    public synchronized void extract(String[] inFile, File dest) throws Exception
    {
        File file;
        ArrayList<String> files = new ArrayList<String>();
        
        for (String name : inFile)
        {
            file = new File(dest, name);
            
            if (file.isFile() && file.exists())
            {
                String msg = Language.getString("FilesInterface.3"); //$NON-NLS-1$
                
                int respon = JOptionPane.showConfirmDialog(UE.WINDOW, msg.replace("%1", name), //$NON-NLS-1$
                        UE.APP_NAME, JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
                
                if (respon==JOptionPane.YES_OPTION)
                {
                    file.delete();
                    files.add(name);
                }
                else
                {
                    continue;
                }
            }
            else
            {
                files.add(name);
            }
        }
        
        final FilesTask TASK = new FilesTask(files.toArray(new String[0]), dest);
        
        final JOptionPane opti = new JOptionPane(Language.getString("FilesInterface.0"), //$NON-NLS-1$
                JOptionPane.PLAIN_MESSAGE);
        
        opti.setOptions(new String[]{Language.getString("FilesInterface.1")}); //$NON-NLS-1$
        
        final JDialog dia = opti.createDialog(UE.WINDOW, UE.APP_NAME);
        
        TASK.addPropertyChangeListener(new PropertyChangeListener()
        {
            public void propertyChange(PropertyChangeEvent evt)
            {
                opti.setMessage(evt.getNewValue().toString());
            
                if (TASK.isDone())
                {
                    dia.dispose();
                }
            }
        });
        
        TASK.execute();
        
        dia.setVisible(true);
        
        if (Language.getString("FilesInterface.1").equals(opti.getValue())) //$NON-NLS-1$
        {
            TASK.cancel(false);
        }
    }
    
    /**
     * Returns an enumeration of loaded files - those currently in buffer.
     */
    public synchronized Enumeration<String> getListOfLoadedFiles()
    {
        return this.MOTHERSHIP.getListOfLoadedFiles();
    }
    
    /**
     * Returns an iterator of files to be removed/ignored in the next save.
     */
    public synchronized Iterator<String> getRemovedFilesList()
    {
        return this.MOTHERSHIP.getRemovedFilesList();
    }

    private class FilesTask extends SwingWorker<Void, Void>
    {
        private File[] FILES = null;
        private String[] NAMES = null;
        private File DEST = null;
        
        // use this for adding files
        public FilesTask(File[] files)
        {
            super();
            this.FILES = files;
        }
        
        // use this for extracting files
        public FilesTask(String[] files, File dest)
        {
            super();
            this.NAMES = files;
            this.DEST = dest;
        }
        
        @Override
        public Void doInBackground()
        {
            try
            {
                if (this.FILES != null)
                {   // add files
                    String name;
                    String msg = Language.getString("FilesInterface.2"); //$NON-NLS-1$
                    FileInputStream fis;
                    byte[] data;
                 
                    for (File file : this.FILES)
                    {
                        if (this.isCancelled())
                        {
                            return null;
                        }
                        
                        // do not lower case the name, let MOTHERSHIP decide if this is needed
                        name = file.getName();

                        this.firePropertyChange(new String(), null, msg.replace("%1", name));  //$NON-NLS-1$
                        
                        fis = new FileInputStream(file);
			
                        data = new byte[fis.available()];

                        for (int start = 0; fis.available() > 0; start += fis.read(data, start, fis.available()))
                        {
                        }

                        fis.close();
                        
                        GenericFile guf = new GenericFile(name, data);

                        FilesInterface.this.MOTHERSHIP.addInternalFile(guf);
                    }
                }
                else
                {   // extract stuff
                    String msg;
                    FileOutputStream out;
                    FileInputStream in;
                    File file;
                    InternalFile ifile;
                    
                    for (String name : this.NAMES)
                    {
                        if (this.isCancelled())
                        {
                            return null;
                        }
                        
                        file = new File(this.DEST.getAbsolutePath(), name);
			
                        msg = Language.getString("FilesInterface.4"); //$NON-NLS-1$
                        
                        this.firePropertyChange(new String(), null, msg.replace("%1", name)); //$NON-NLS-1$
                        
                        ifile = FilesInterface.this.MOTHERSHIP.getInternalFile(name, false);
			
                        if (ifile == null)
                        {
                            continue;
                        }
			
                        if (ifile.madeChanges())
                        {
                            FilesInterface.this.MOTHERSHIP.markChanged();
                        }
			
                        file.createNewFile();
                        out = new FileOutputStream(file);
                        ifile.save(out);
                        out.close();

                        if (UE.DEBUG_MODE)
                        {
                            byte[] data = new byte[2048];
                            int len;
                            in = new FileInputStream(file);
                            Fletcher16 crc = new Fletcher16();

                            while (in.available() > 0)
                            {
                                len = in.read(data);
                                crc.update(data, 0, len);
                            }

                            in.close();

                            System.out.println(file.getName() + " -> CRC: " + crc.getValue());
                        }
                    }
                }
            }
            catch (Exception t)
            {
                UE.displayError(t);
                return null;
            }
            return null;
        }
    }
}