/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2008 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: PrefixToStringList.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 9.9.2008
 */
package ue.edit;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import ue.util.Tools;

/**
 * @author Alan Podlesek
 */
public class PrefixToStringList extends InternalFile
{
    private Hashtable<String, String> table = new Hashtable<String, String>();
    
    public PrefixToStringList()
    {
    }
    
    public PrefixToStringList(InputStream in) throws Exception
    {
        byte[] key = new byte[4];
        byte[] cls = new byte[40];
        
        while (in.available() > 0)
        {
            in.read(key);
            in.read(cls);
            table.put(Tools.toASCII(key).toUpperCase(), Tools.toASCII(cls));
        }
    }
    
    public void put(String prefix, String cls)
    {
        table.put(prefix.toUpperCase(), cls);
    }
    
    public void remove(String prefix)
    {
        table.remove(prefix.toUpperCase());
    }
    
    public String getClassByPrefix(String prefix)
    {
        return table.get(prefix.toUpperCase());
    }
    /**
     * @return file size
     */
    @Override
    public long getSize()
    {
        return table.size() * 44;
    }
    
    @Override
    public void save(OutputStream out) throws Exception
    {
        Enumeration<String> en = table.keys();
        String key;
         
        while (en.hasMoreElements())
        {
            key = en.nextElement();
            
            out.write(Tools.toByte(key.toUpperCase(), 4, 3));
            out.write(Tools.toByte(table.get(key), 40, 39));
        }
    }

    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        return new Vector<String>();
    }
}
