/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *      
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *      
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: Special.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *      
 * Last modified: 21.7.2008
 */
package ue.edit;

import ue.Language;
import ue.UE;
import ue.gui.MainPanel;
import ue.gui.special.CalcGUI;
import ue.gui.special.ConvertAddressGUI;
import ue.gui.special.Image2TgaGUI;
import ue.gui.special.MonitorGameGUI;
import ue.gui.special.Tga2AniGUI;
import ue.util.StringGroup;

/**
 * This class's function is to provide a list of available special commands which are reachable under
 * the Tools menu and returns the coresponding panels if demanded. All methods are static so it's pointless to
 * make an instance of this class.
 */
public class Special
{
    // commands listed
    private static String cmdCREATE_ANI_CUR = Language.getString("Special.0"); //$NON-NLS-1$
    private static String cmdCONVERT_IMAGE = Language.getString("Special.1"); //$NON-NLS-1$
    private static String cmdSUPERVISE_MULTI = Language.getString("Special.2"); //$NON-NLS-1$
    private static String cmdCALCULATOR = Language.getString("Special.3"); //$NON-NLS-1$
    private static String cmdCONVERT_ADDRESSES = "Convert Addresses";
    
    //commands
    private static String[] COMMANDS;
    
    /**
     * @return a String array of available commands that can be used with the edit function.
     */
    public static StringGroup[] getCommands()
    {
        StringGroup[] ret = new StringGroup[3];

        try
        {
            Special.cmdCREATE_ANI_CUR = Language.getString("Special.0"); //$NON-NLS-1$
            Special.cmdCONVERT_IMAGE = Language.getString("Special.1"); //$NON-NLS-1$
            Special.cmdSUPERVISE_MULTI = Language.getString("Special.2"); //$NON-NLS-1$
            Special.cmdCALCULATOR = Language.getString("Special.3"); //$NON-NLS-1$
            cmdCONVERT_ADDRESSES = "Convert Addresses";
            
            Special.COMMANDS = new String[]{Special.cmdCREATE_ANI_CUR,
                    Special.cmdCONVERT_IMAGE, Special.cmdSUPERVISE_MULTI, Special.cmdCALCULATOR,
                    cmdCONVERT_ADDRESSES};
            
            ret[0] = new StringGroup(Language.getString("Special.4"), new String[]{Special.cmdCREATE_ANI_CUR, Special.cmdCONVERT_IMAGE}); //$NON-NLS-1$
            ret[1] = new StringGroup(Language.getString("Special.5"), new String[]{Special.cmdSUPERVISE_MULTI}); //$NON-NLS-1$
            ret[2] = new StringGroup(Language.getString("Special.6"), new String[]{Special.cmdCALCULATOR, cmdCONVERT_ADDRESSES}); //$NON-NLS-1$
        }
        catch (Exception tz)
        {
            UE.displayError(tz);
            return null;
        }
        return ret;
    }
    
    /**
     * @param command the command - should be one of the available commands listed by the getCommands() function.
     * @return a MainPanel instance which corresponds to the command or null if the commands is invalid.
     */
    public static MainPanel edit(String command)
    {
        int z = -1;

        // not using Arrays.binarySearch because it requires the array to be sorted
        for (int i = 0; i < Special.COMMANDS.length; i++)
        {
            if (Special.COMMANDS[i].equals(command))
            {
                z = i;
                break;
            }
        }

        if (z < 0)
        {
            return null;
        }
        
        switch (z)
        {
            case 0:
            {
                return new Tga2AniGUI();
            }
            case 1:
            {
                return new Image2TgaGUI();
            }
            case 2:
            {
                return new MonitorGameGUI();
            }
            case 3:
            {
                return new CalcGUI();
            }
            case 4:
            {
                return new ConvertAddressGUI();
            }
        }
        return null;
    }
}