/**
 * Ultimate Editor
 * Copyright (C) 2009 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: GenericSegment.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 2.6.2009
 */
package ue.edit.exe;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

/**
 * @author Alan Podlesek
 */
public class GenericSegment extends InternalSegment
{
    private byte[] data;
    
    public GenericSegment(long address, byte[] data)
    {
        this.ADDRESS = address;
        this.setName(Long.toString(ADDRESS));
        this.data = data;
    }
    
    public GenericSegment(long address, byte[] data, boolean mark_as_modified)
    {
        this.ADDRESS = address;
        this.setName(Long.toString(ADDRESS));
        this.data = data;

        if (mark_as_modified)
        {
            this.markChanged();
        }
    }

    public GenericSegment(InputStream in, long address, int size) throws IOException
    {
        data = new byte[size];
        int read = 0;
        
        while ((read += in.read(data, read, data.length - read)) != data.length)
        {}
        
        this.ADDRESS = address;
        this.setName(Long.toString(ADDRESS));
    }    
    
    public byte[] getData()
    {
        byte[] d = new byte[data.length];
        System.arraycopy(data, 0, d, 0, data.length);
        return d;
    }
    
    public void setData(byte[] d)
    {
        if (data.length == d.length)
        {
            System.arraycopy(d, 0, data, 0, data.length);
            this.markChanged();
        }
    }
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        return null;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        return data.length;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        out.write(data);
    }
    
}
