/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *      
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *      
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: InternalSegment.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Last modified: 21.7.2008
 */
package ue.edit.exe;

import java.util.Arrays;

import ue.Language;
import ue.edit.InternalFile;
import ue.util.Tools;

/**
 * Every editable internal segment must extend this class.*/
public abstract class InternalSegment extends InternalFile implements Comparable<InternalSegment>
{
    /**
     * The address/location in file.
     */
    protected long ADDRESS = 0;

    /**
     * Returns the address in trek.exe where this segment originates from/is written to.
     */
    public long getAddress()
    {
        return this.ADDRESS;
    }
    
    @Override
    public int compareTo(InternalSegment o)
    {
        return (int)(this.ADDRESS - o.ADDRESS);
    }
    
    public static boolean checkIfValid(byte[] read, byte[] expected, boolean exception) throws Exception
    {
        if (read.length != expected.length || !Arrays.equals(read, expected))
        {
            if (!exception)
            {
                return false;
            }
            
            System.err.println("\nExpected: " + Tools.toHex(expected));
            System.err.println("Read: " + Tools.toHex(read));
            throw new Exception(Language.getString("InternalSegment.0")); //$NON-NLS-1$
        }
        
        return true;
    }

    public void testLastBytes(byte[] read, byte[] expected, boolean exception) throws Exception
    {
        if (expected != null)
        {
            byte[] t = new byte[expected.length];
            System.arraycopy(read, read.length - t.length, t, 0, t.length);
            InternalSegment.checkIfValid(t, expected, exception);
        }
    }
    
    @Override
    public String getCheckIntegrityString(int type, String msg)
    {
        String ret;
        
        switch(type)
        {
            case INTEGRITY_CHECK_WARNING:
            {
                ret = Language.getString("InternalSegment.1"); //$NON-NLS-1$
                break;
            }
            case INTEGRITY_CHECK_ERROR:
            {
                ret = Language.getString("InternalSegment.2"); //$NON-NLS-1$
                break;
            }
            default:
            {
                ret = Language.getString("InternalSegment.3"); //$NON-NLS-1$
            }
        }
        
        return ret.replace("%1", Long.toString(ADDRESS)).replace("%s", msg); //$NON-NLS-1$
    }
}
