/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: Trek.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Last modified: 21.7.2008
 */
package ue.edit.exe;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.security.MessageDigest;
import java.util.Arrays;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import ue.Language;
import ue.UE;
import ue.edit.CanBeEdited;
import ue.edit.FileManager;
import ue.edit.InternalFile;
import ue.edit.exe.trek.AIShpSets;
import ue.edit.exe.trek.Alien;
import ue.edit.exe.trek.AlienStartingID;
import ue.edit.exe.trek.AverageNumberOfSpaceObjects;
import ue.edit.exe.trek.BaseMapRanges;
import ue.edit.exe.trek.BasicScannerID;
import ue.edit.exe.trek.CDprotection;
import ue.edit.exe.trek.CompareBuildingID;
import ue.edit.exe.trek.CompareByteToMemOffset;
import ue.edit.exe.trek.CompareIntReg;
import ue.edit.exe.trek.CompareShortReg;
import ue.edit.exe.trek.EmpireImgPrefixes;
import ue.edit.exe.trek.HorizontalMapSize1;
import ue.edit.exe.trek.Jump;
import ue.edit.exe.trek.MEStartingConditions;
import ue.edit.exe.trek.MainBldStartingIDs;
import ue.edit.exe.trek.MainIndustryFarmsAndEnergyBld;
import ue.edit.exe.trek.MainIntelBld;
import ue.edit.exe.trek.MainIntelBld2;
import ue.edit.exe.trek.MainResearchBld;
import ue.edit.exe.trek.MainResearchBld2;
import ue.edit.exe.trek.MapRangeTechBonuses;
import ue.edit.exe.trek.MartialLawBld;
import ue.edit.exe.trek.MartialLawBld2;
import ue.edit.exe.trek.MartialLawBld3;
import ue.edit.exe.trek.MartialLawBld4;
import ue.edit.exe.trek.MinorRaceBldUpgradeIDs;
import ue.edit.exe.trek.MinorRaceMainBldIDLimits;
import ue.edit.exe.trek.MinorRaceSpBuildingIDs;
import ue.edit.exe.trek.MinorRacesNumbers;
import ue.edit.exe.trek.NebulaHCGraphic;
import ue.edit.exe.trek.Placeholder;
import ue.edit.exe.trek.PlanetBonusTypes;
import ue.edit.exe.trek.PunyFactories;
import ue.edit.exe.trek.PunyFarms;
import ue.edit.exe.trek.PushInt;
import ue.edit.exe.trek.RaceList;
import ue.edit.exe.trek.RaceListAddresses;
import ue.edit.exe.trek.ScreensaverModelList;
import ue.edit.exe.trek.MovIntToReg;
import ue.edit.exe.trek.SelectPhaserHobFile;
import ue.edit.exe.trek.SelectPlasmaHobFile;
import ue.edit.exe.trek.SelectTorpedoHobFile;
import ue.edit.exe.trek.ShipLoadLimit;
import ue.edit.exe.trek.ShipModelData;
import ue.edit.exe.trek.ShipnameCode;
import ue.edit.exe.trek.Shipnum;
import ue.edit.exe.trek.ShipyardBldIDs;
import ue.edit.exe.trek.SpecialShips;
import ue.edit.exe.trek.SpecialShipsOffset;
import ue.edit.mod.ModList;
import ue.edit.res.Stbof;
import ue.exception.InvalidArgumentsException;
import ue.gui.CheckGUI;
import ue.gui.MainPanel;
import ue.gui.exe.AIShipSetsGUI;
import ue.gui.exe.CDProtectionGUI;
import ue.gui.exe.GalacticMapGUI;
import ue.gui.exe.MEStartCondGUI;
import ue.gui.exe.ShipBuildingGUI;
import ue.gui.exe.ShipMapRangesGUI;
import ue.gui.exe.TrekPatchesGUI;
import ue.util.StringGroup;
import ue.util.Tools;
import ue.util.UEWorker;


/**
 * This class is a representation of the trek.exe file,
 * it communicates with other classes in the edit.exe.trek subpackages.
 * Also it sends MainPanel instances to use for editing the file.
 * Unlike other files this one deosn't have a modlist.
 * The reason for this is because changing trek.exe's size would break things.
 */
public class Trek implements CanBeEdited
{
    // SEGMENTS
    public final static String fileSHIPNUM1 = "shipnum1"; //$NON-NLS-1$
    public final static String fileSHIPNUM2 = "shipnum2"; //$NON-NLS-1$
    public final static String fileALIEN1 = "alien1"; //$NON-NLS-1$
    public final static String fileALIEN2 = "alien2"; //$NON-NLS-1$
    public final static String fileALIEN3 = "alien3"; //$NON-NLS-1$
    public final static String fileALIEN4 = "alien4"; //$NON-NLS-1$
    public final static String fileALIEN5 = "alien5"; //$NON-NLS-1$
    public final static String fileALIEN6 = "alien6"; //$NON-NLS-1$
    public final static String fileALIEN7 = "alien7"; //$NON-NLS-1$
    public final static String fileALIEN8 = "alien8"; //$NON-NLS-1$
    public final static String fileALIEN9 = "alien9"; //$NON-NLS-1$
    public final static String fileCDPROTECT = "cdprotection"; //$NON-NLS-1$
    public final static String fileMESTARTCOND = "mestartingconditions"; //$NON-NLS-1$
    public final static String fileALIEN_STARTING_ID = "alienstartingid"; //$NON-NLS-1$
    public final static String fileSPECIAL_SHIPS = "specialships"; //$NON-NLS-1$
    public final static String fileSPECIAL_STRUCT_JUMP = "specialstructurejump"; //$NON-NLS-1$
    public final static String fileSHIP_MODEL_DATA = "shipmodeldata"; //$NON-NLS-1$
    public final static String fileALIEN_STARTING_ID2 = "alienstartingid2"; //$NON-NLS-1$
    public final static String fileALIEN_STARTING_ID3 = "alienstartingid3"; //$NON-NLS-1$
    public final static String fileALIEN_STARTING_ID4 = "alienstartingid4"; //$NON-NLS-1$
    public final static String fileSCREENSAVER_MODEL_LIST = "screensavermodellist"; //$NON-NLS-1$
    public final static String fileRACE_LIST = "racelist"; //$NON-NLS-1$
    public final static String fileSHIP_LOAD_LIMIT1 = "shiploadlimit1"; //$NON-NLS-1$
    public final static String fileSHIP_LOAD_LIMIT2 = "shiploadlimit2"; //$NON-NLS-1$
    public final static String fileBASE_MAP_RANGES = "basemapranges"; //$NON-NLS-1$
    public final static String fileMAP_RANGE_BONUSES = "maprangetechbonuses"; //$NON-NLS-1$
    public final static String fileRACE_LIST_ADDRESSES = "racelistaddresses"; //$NON-NLS-1$
    public final static String fileMAIN_RESEARCH_BLD = "mainresearchbld"; //$NON-NLS-1$
    public final static String fileMAIN_INTEL_BLD = "mainintelbld"; //$NON-NLS-1$
    public final static String fileMAIN_INDUSTRY_FARMS_AND_ENERGY_BLD = "mainindustryfarmsandenergybld"; //$NON-NLS-1$
    public final static String fileMAIN_RESEARCH_BLD2 = "mainresearchbld2"; //$NON-NLS-1$
    public final static String fileMAIN_INTEL_BLD2 = "mainintelbld2"; //$NON-NLS-1$
    public final static String filePUNY_FARMS = "punyFarms"; //$NON-NLS-1$
    public final static String filePUNY_FACTORIES = "punyFactories"; //$NON-NLS-1$
    public final static String fileAI_SHIP_SETS = "aiShpSets"; //$NON-NLS-1$
    public final static String fileHORIZONTAL_MAP_SIZE_1 = "horizontalMapSize1"; //$NON-NLS-1$
    public final static String fileSPECIAL_SHIPS_OFFSET = "specialshipsOffset"; //$NON-NLS-1$
    public final static String fileUNKNOWN1 = "unknown1"; //$NON-NLS-1$
    public final static String fileUNKNOWN2 = "unknown2"; //$NON-NLS-1$
    public final static String fileUNKNOWN3 = "unknown3"; //$NON-NLS-1$
    public final static String fileUNKNOWN4 = "unknown4"; //$NON-NLS-1$
    public final static String fileUNKNOWN5 = "unknown5"; //$NON-NLS-1$
    public final static String fileUNKNOWN6 = "unknown6"; //$NON-NLS-1$
    public final static String fileMIN_MINORS_NUMBER = "minMinorsNumber"; //$NON-NLS-1$
    public final static String fileMAX_MINORS_NUMBER = "maxMinorsNumber"; //$NON-NLS-1$
    public final static String fileAVG_SPACE_OBJECT_NUM_S = "AvgSpaceObjectNumSmall"; //$NON-NLS-1$
    public final static String fileAVG_SPACE_OBJECT_NUM_M = "AvgSpaceObjectNumMedium"; //$NON-NLS-1$
    public final static String fileAVG_SPACE_OBJECT_NUM_L = "AvgSpaceObjectNumLarge"; //$NON-NLS-1$
    public final static String fileEMPIRE_IMAGE_PREFIXES = "EmpireImgPrefixes"; //$NON-NLS-1$
    public final static String fileMAIN_ENERGY_BLD = "mainEnergyBld"; //$NON-NLS-1$
    public final static String fileMARTIAL_LAW_BLD = "martialLawBld"; //$NON-NLS-1$
    public final static String fileBASIC_SCANNER_ID = "basicScannerID"; //$NON-NLS-1$
    public final static String fileMAIN_INTEL_BLD3 = "mainIntelBld3"; //$NON-NLS-1$
    public final static String fileMAIN_RESEARCH_BLD3 = "mainResearchBld3"; //$NON-NLS-1$
    public final static String fileMARTIAL_LAW_BLD2 = "martialLawBld2"; //$NON-NLS-1$
    public final static String fileORBITAL_DEFENSE_BLD = "orbitalDefenseBld"; //$NON-NLS-1$
    public final static String fileORBITAL_DEFENSE_BLD2 = "orbitalDefenseBld2"; //$NON-NLS-1$
    public final static String fileORBITAL_DEFENSE_BLD3 = "orbitalDefenseBld3"; //$NON-NLS-1$
    public final static String fileSHIELD_GEN_BLD = "shieldGenBld"; //$NON-NLS-1$
    public final static String fileORBITAL_DEFENSE_BLD4 = "orbitalDefenseBld4"; //$NON-NLS-1$
    public final static String fileORBITAL_DEFENSE_BLD5 = "orbitalDefenseBld5"; //$NON-NLS-1$
    public final static String fileSPECIAL_SHIPS_BLD = "specialShipsBld"; //$NON-NLS-1$
    public final static String fileTRADE_GOODS_BLD = "tradeGoodsBld"; //$NON-NLS-1$
    public final static String fileTRADE_GOODS_BLD2 = "tradeGoodsBld2"; //$NON-NLS-1$
    public final static String fileTRADE_GOODS_BLD3 = "tradeGoodsBld3"; //$NON-NLS-1$
    public final static String fileMINOR_MAIN_BLD_IDS = "minorMainBldIDs"; //$NON-NLS-1$
    public final static String fileCMP_BASIC_SHIPYARD_ID = "cmpBasicShipyardID"; //$NON-NLS-1$
    public final static String fileCMP_BASIC_SHIPYARD_ID2 = "cmpBasicShipyardID2"; //$NON-NLS-1$
    public final static String fileCMP_BASIC_SHIPYARD_ID3 = "cmpBasicShipyardID3"; //$NON-NLS-1$
    public final static String fileMINOR_SP_BUILDING_IDS = "minorRaceSpBuildingIDs"; //$NON-NLS-1$
    public final static String fileMINORS_BLD_UPGRADE_IDS = "minorsBldUpgradeIDs"; //$NON-NLS-1$
    public final static String fileMAX_BLD_ID1 = "maxBldID1"; //$NON-NLS-1$
    public final static String fileMAX_BLD_ID2 = "maxBldID2"; //$NON-NLS-1$
    public final static String fileMAX_BLD_ID3 = "maxBldID3"; //$NON-NLS-1$
    public final static String fileMAX_BLD_ID4 = "maxBldID4"; //$NON-NLS-1$
    public final static String fileMAX_BLD_ID5 = "maxBldID5"; //$NON-NLS-1$
    public final static String fileMAX_BLD_ID6 = "maxBldID6"; //$NON-NLS-1$
    public final static String fileMAX_BLD_ID7 = "maxBldID7"; //$NON-NLS-1$
    public final static String fileMAX_BLD_ID8 = "maxBldID8"; //$NON-NLS-1$
    public final static String fileMAX_BLD_ID9 = "maxBldID9"; //$NON-NLS-1$
    public final static String fileBLD_NUM1 = "bldNum1"; //$NON-NLS-1$
    public final static String fileBLD_NUM2 = "bldNum2"; //$NON-NLS-1$
    public final static String fileBLD_NUM3 = "bldNum3"; //$NON-NLS-1$
    public final static String fileBLD_NUM4 = "bldNum4"; //$NON-NLS-1$
    public final static String filePLANET_BONUS_TYPES = "planetBonusTypes"; //$NON-NLS-1$
    public final static String fileNEBULA_HC_GRAPHIC = "nebulaHCGraphic"; //$NON-NLS-1$
    public final static String fileORBITAL_DEFENSE_BLD6 = "orbitalDefenseBld6"; //$NON-NLS-1$
    public final static String fileDILITHIUM_BLD1 = "dilithiumBld1"; //$NON-NLS-1$
    public final static String fileDILITHIUM_BLD2 = "dilithiumBld2"; //$NON-NLS-1$
    public final static String fileBASIC_SHIPYARD_ID4 = "basicShipyardID4"; //$NON-NLS-1$
    public final static String fileDILITHIUM_BLD3 = "dilithiumBld3"; //$NON-NLS-1$
    public final static String fileORBITAL_DEFENSE_BLD7 = "orbitalDefenseBld7"; //$NON-NLS-1$
    public final static String fileFED_FARM_LVL1_BLD1 = "fedFarmLvL1Bld1"; //$NON-NLS-1$
    public final static String fileORBITAL_DEFENSE_BLD8 = "orbitalDefenseBld8"; //$NON-NLS-1$
    public final static String fileBASIC_GROUND_DEFENSE_ID1 = "fileBASIC_GROUND_DEFENSE_ID1"; //$NON-NLS-1$
    public final static String fileSHIELD_GEN_BLD2 = "fileSHIELD_GEN_BLD2"; //$NON-NLS-1$
    public final static String fileORBITAL_DEFENSE_BLD9 = "orbitalDefenseBld9"; //$NON-NLS-1$
    public final static String fileDILITHIUM_BLD4 = "dilithiumBld4"; //$NON-NLS-1$
    public final static String fileDILITHIUM_BLD5 = "dilithiumBld5"; //$NON-NLS-1$
    public final static String fileDILITHIUM_BLD6 = "dilithiumBld6"; //$NON-NLS-1$
    public final static String fileBASIC_SHIPYARD_ID5 = "basicShipyardID5"; //$NON-NLS-1$
    public final static String fileBASIC_SHIPYARD_ID6 = "basicShipyardID6"; //$NON-NLS-1$
    public final static String fileDILITHIUM_BLD7 = "dilithiumBld7"; //$NON-NLS-1$
    public final static String fileDILITHIUM_BLD8 = "dilithiumBld8"; //$NON-NLS-1$
    public final static String fileBASIC_GROUND_DEFENSE_ID2 = "fileBASIC_GROUND_DEFENSE_ID2"; //$NON-NLS-1$
    public final static String fileBASIC_GROUND_DEFENSE_ID3 = "fileBASIC_GROUND_DEFENSE_ID3"; //$NON-NLS-1$
    public final static String fileSHIELD_GEN_BLD3 = "fileSHIELD_GEN_BLD3"; //$NON-NLS-1$
    public final static String fileORBITAL_DEFENSE_BLD10 = "orbitalDefenseBld10"; //$NON-NLS-1$
    public final static String fileORBITAL_DEFENSE_BLD11 = "orbitalDefenseBld11"; //$NON-NLS-1$
    public final static String fileORBITAL_DEFENSE_BLD12 = "orbitalDefenseBld12"; //$NON-NLS-1$
    public final static String fileORBITAL_DEFENSE_BLD13 = "orbitalDefenseBld13"; //$NON-NLS-1$
    public final static String fileSHIPYARD_IDS = "fileSHIPYARDIDS"; //$NON-NLS-1$
    public final static String fileMARTIAL_LAW_BLD3 = "martialLawBld3"; //$NON-NLS-1$
    public final static String fileMARTIAL_LAW_BLD4 = "martialLawBld4"; //$NON-NLS-1$
    public final static String fileSHIPNAME_CODE = "shipnameCode"; //$NON-NLS-1$
    public final static String fileSELECTPLASMAHOBFILE = "selectPlasmaHobFile"; //$NON-NLS-1$
    public final static String fileSELECTPHASERHOBFILE = "selectPhaserHobFile"; //$NON-NLS-1$
    public final static String fileSELECTTORPEDOHOBFILE = "selectTorpedoHobFile"; //$NON-NLS-1$

    /*
     * A list of at least partialy editable segments.
     * CAREFULL!!! only add new segments of the file to the end of the list!
     */
    private static String[] SEGMENTLIST =  new String[]{Trek.fileSHIPNUM1,
        Trek.fileSHIPNUM2, Trek.fileALIEN1, Trek.fileALIEN2, Trek.fileALIEN3,
        Trek.fileALIEN4, Trek.fileALIEN5, Trek.fileALIEN6, Trek.fileALIEN7,
        Trek.fileALIEN8, Trek.fileALIEN9, Trek.fileCDPROTECT, Trek.fileMESTARTCOND,
        Trek.fileALIEN_STARTING_ID, Trek.fileSPECIAL_SHIPS, 
        Trek.fileSPECIAL_STRUCT_JUMP, Trek.fileSHIP_MODEL_DATA,
        Trek.fileALIEN_STARTING_ID2, Trek.fileALIEN_STARTING_ID3,
        Trek.fileALIEN_STARTING_ID4, Trek.fileSCREENSAVER_MODEL_LIST,
        Trek.fileRACE_LIST, Trek.fileSHIP_LOAD_LIMIT1, Trek.fileSHIP_LOAD_LIMIT2,
        Trek.fileBASE_MAP_RANGES, Trek.fileMAP_RANGE_BONUSES,
        Trek.fileRACE_LIST_ADDRESSES, Trek.fileMAIN_RESEARCH_BLD, Trek.fileMAIN_INTEL_BLD,
        Trek.fileMAIN_INDUSTRY_FARMS_AND_ENERGY_BLD, Trek.fileMAIN_RESEARCH_BLD2,
        Trek.fileMAIN_INTEL_BLD2, Trek.filePUNY_FARMS, Trek.filePUNY_FACTORIES,
        Trek.fileAI_SHIP_SETS, Trek.fileHORIZONTAL_MAP_SIZE_1, Trek.fileSPECIAL_SHIPS_OFFSET,
        Trek.fileUNKNOWN1, Trek.fileUNKNOWN2, Trek.fileUNKNOWN3, Trek.fileUNKNOWN4,
        Trek.fileUNKNOWN5, Trek.fileUNKNOWN6, Trek.fileMIN_MINORS_NUMBER, Trek.fileMAX_MINORS_NUMBER,
        Trek.fileAVG_SPACE_OBJECT_NUM_S, Trek.fileAVG_SPACE_OBJECT_NUM_M, Trek.fileAVG_SPACE_OBJECT_NUM_L,
        Trek.fileEMPIRE_IMAGE_PREFIXES, Trek.fileMAIN_ENERGY_BLD, Trek.fileMARTIAL_LAW_BLD,
        Trek.fileBASIC_SCANNER_ID, Trek.fileMAIN_INTEL_BLD3, Trek.fileMAIN_RESEARCH_BLD3,
        Trek.fileMARTIAL_LAW_BLD2, Trek.fileORBITAL_DEFENSE_BLD,
        Trek.fileORBITAL_DEFENSE_BLD2, Trek.fileORBITAL_DEFENSE_BLD3, Trek.fileSHIELD_GEN_BLD,
        Trek.fileORBITAL_DEFENSE_BLD4, Trek.fileORBITAL_DEFENSE_BLD5, Trek.fileSPECIAL_SHIPS_BLD,
        Trek.fileTRADE_GOODS_BLD, Trek.fileTRADE_GOODS_BLD2, Trek.fileTRADE_GOODS_BLD3,
        Trek.fileMINOR_MAIN_BLD_IDS, Trek.fileCMP_BASIC_SHIPYARD_ID, Trek.fileCMP_BASIC_SHIPYARD_ID2,
        Trek.fileCMP_BASIC_SHIPYARD_ID3, Trek.fileMINOR_SP_BUILDING_IDS, Trek.fileMINORS_BLD_UPGRADE_IDS,
        Trek.fileMAX_BLD_ID1, Trek.fileMAX_BLD_ID2, Trek.fileMAX_BLD_ID3, Trek.fileMAX_BLD_ID4,
        Trek.fileMAX_BLD_ID5, Trek.fileMAX_BLD_ID6, Trek.fileMAX_BLD_ID7, Trek.fileMAX_BLD_ID8,
        Trek.fileMAX_BLD_ID9, Trek.fileBLD_NUM1, Trek.fileBLD_NUM2, Trek.fileBLD_NUM3,
        Trek.fileBLD_NUM4, Trek.filePLANET_BONUS_TYPES, Trek.fileNEBULA_HC_GRAPHIC,
        fileORBITAL_DEFENSE_BLD6, fileDILITHIUM_BLD1, fileDILITHIUM_BLD2, fileBASIC_SHIPYARD_ID4,
        fileDILITHIUM_BLD3, fileORBITAL_DEFENSE_BLD7, fileFED_FARM_LVL1_BLD1,
        fileORBITAL_DEFENSE_BLD8, fileBASIC_GROUND_DEFENSE_ID1, fileSHIELD_GEN_BLD2,
        fileORBITAL_DEFENSE_BLD9, fileDILITHIUM_BLD4, fileDILITHIUM_BLD5, fileDILITHIUM_BLD6,
        fileBASIC_SHIPYARD_ID5, fileBASIC_SHIPYARD_ID6, fileDILITHIUM_BLD7, fileDILITHIUM_BLD8,
        fileBASIC_GROUND_DEFENSE_ID2, fileBASIC_GROUND_DEFENSE_ID3, fileSHIELD_GEN_BLD3,
        fileORBITAL_DEFENSE_BLD10, fileORBITAL_DEFENSE_BLD11, fileORBITAL_DEFENSE_BLD12,
        fileORBITAL_DEFENSE_BLD13, fileSHIPYARD_IDS, fileMARTIAL_LAW_BLD3, fileMARTIAL_LAW_BLD4,
        fileSHIPNAME_CODE, fileSELECTPLASMAHOBFILE, fileSELECTPHASERHOBFILE, fileSELECTTORPEDOHOBFILE
    };
    
    // trek.exe segment address, addresses are in the same order as segment names in segmentlist
    // only Joker's patch is supported
    private static final long[] SEGMENT_ADDRESS = new long[]
    {
        454537, // 0x0006EF89 - number of ships in shiplist.sst - fileSHIPNUM1
        454703, // 0x0006F02F - number of ships in shiplist.sst - fileSHIPNUM2
        323291, // 0x0004EEDB - Borg Cube ID (1)                - fileALIEN1
        323803, // 0x0004F0DB - Calamarain ID (4)               - fileALIEN2
        323674, // 0x0004F05A - Chodak ID (3)                   - fileALIEN3
        323611, // 0x0004F01B - Crystal Entity ID (2)           - fileALIEN4
        324087, // 0x0004F1F7 - Combat Drone ID (8)             - fileALIEN5
        324018, // 0x0004F1B2 - Edo Guardian ID (7)             - fileALIEN6
        323873, // 0x0004F121 - Gomtuu ID (5)                   - fileALIEN7
        324146, // 0x0004F232 - Husnock ID (9)                  - fileALIEN8
        323955, // 0x0004F173 - Tarellian ID (6)                - fileALIEN9
        1442,   // 0x000005A2 - CD Protection Jump              - fileCDPROTECT
        1629212,// 0x0018DC1C - ME Starting Conditions          - fileMESTARTCOND
        324439, // 0x0004F357 - Alien Starting ID               - fileALIEN_STARTING_ID
        995696, // 0x000F3170 - Special Ships                   - fileSPECIAL_SHIPS
        241336, // 0x0003AEB8 - Special Ship Building Jump      - fileSPECIAL_STRUCT_JUMP
        1636348,// 0x0018F7FC - Ship Model Data                 - fileSHIP_MODEL_DATA
        303045, // 0x00049FC5 - Alien Starting ID               - fileALIEN_STARTING_ID2
        302501, // 0x00049DA5 - Alien Starting ID               - fileALIEN_STARTING_ID3
        321656, // 0x0004E878 - Alien Starting ID               - fileALIEN_STARTING_ID4
        1575336,// 0x001809A8 - Screensaver model list          - fileSCREENSAVER_MODEL_LIST
        1525480,// 0x001746e8 - Race list                       - fileRACE_LIST
        453640, // 0x0006EC08 - ship load limit                 - fileSHIP_LOAD_LIMIT1
        453734, // 0x0006EC66 - ship load limit                 - fileSHIP_LOAD_LIMIT2
        1631788,// 0x0018E62C - base range values               - fileBASE_MAP_RANGES
        1632384,// 0x0018E880 - map range tech bonus values     - fileMAP_RANGE_BONUSES
        1610816,// 0x00189440 - race list addresses             - fileRACE_LIST_ADDRESSES
        545008, // 0x000850F0 - main research buildings         - fileMAIN_RESEARCH_BLD
        545952, // 0x000854A0 - main intel buildings            - fileMAIN_INTEL_BLD
        547152, // 0x00085950 - main ind, food and en buildings - fileMAIN_INDUSTRY_FARMS_AND_ENERGY_BLD
        548944, // 0x00086050 - main research buildings         - fileMAIN_RESEARCH_BLD2
        549824, // 0x000863C0 - main intel buildings            - fileMAIN_INTEL_BLD2
        279855, // 0x0004452F - puny farms                      - filePUNY_FARMS
        279663, // 0x0004446F - puny factories                  - filePUNY_FACTORIES
        1614016,// 0x0018A0C0 - AI ship sets                    - fileAI_SHIP_SETS
        249008, // 0x0003CCB0 - Horizontal map size             - fileHORIZONTAL_MAP_SIZE_1
        999831, // 0x000F4197 - Special Ships Offset            - fileSPECIAL_SHIPS_OFFSET
        346368, // 0x00054900 - called by special ships         - fileUNKNOWN1
        1158912,// 0x0011AF00 - called by special ships         - fileUNKNOWN2
        454080, // 0x0006EDC0 - called by special ships         - fileUNKNOWN3
        241280, // 0x0003AE80 - called by special ships         - fileUNKNOWN4
        1155568,// 0x0011A1F0 - called by special ships         - fileUNKNOWN5
        1158048,// 0x0011ABA0 - called by special ships         - fileUNKNOWN6
        726816, // 0x000B1720 - min number of minor races       - fileMIN_MINORS_NUMBER
        726640, // 0x000B1670 - max number of minor races       - fileMAX_MINORS_NUMBER
        719015, // 0x000AF8A7 - small gal avg number of objects - fileAVG_SPACE_OBJECT_NUM_S
        719393, // 0x000AFA21 - med gal avg number of objects   - fileAVG_SPACE_OBJECT_NUM_M
        719415, // 0x000AFA37 - lrg gal avg number of objects   - fileAVG_SPACE_OBJECT_NUM_L
        1641036,// 0x00190A4C - empire image prefixes           - fileEMPIRE_IMAGE_PREFIXES
        279471, // 0x000443AF - main energy buildings           - fileMAIN_ENERGY_BLD
        236573, // 0x00039C1D - martial law buildings           - fileMARTIAL_LAW_BLD
        278973, // 0x000441BD - basic scanner id                - fileBASIC_SCANNER_ID
        279087, // 0x0004422F - main intel buildings            - fileMAIN_INTEL_BLD3
        279279, // 0x000442EF - main intel buildings            - fileMAIN_RESEARCH_BLD3
        409531, // 0x00063FBB - martial law buildings           - fileMARTIAL_LAW_BLD2
        442046, // 0x0006BEBE - orbital batery                  - fileORBITAL_DEFENSE_BLD
        444171, // 0x0006C70B - orbital batery                  - fileORBITAL_DEFENSE_BLD2
        445471, // 0x0006CC1F - orbital batery                  - fileORBITAL_DEFENSE_BLD3
        445677, // 0x0006CCED - shield generator                - fileSHIELD_GEN_BLD
        985177, // 0x000F0859 - orbital batery                  - fileORBITAL_DEFENSE_BLD4
        985351, // 0x000F0907 - orbital batery                  - fileORBITAL_DEFENSE_BLD4
        992000, // 0x000F2300 - special ships building          - fileSPECIAL_SHIPS_BLD
        1019228,// 0x000F8D5C - trade goods id                  - fileTRADE_GOODS_BLD
        1019409,// 0x000F8E11 - trade goods id                  - fileTRADE_GOODS_BLD2
        1019645,// 0x000F8EFD - trade goods id                  - fileTRADE_GOODS_BLD3
        1629612,// 0x0018DDAC - minor race main building ids    - fileMINOR_MAIN_BLD_IDS
        863427, // 0x000D2CC3 - basic shipyard id               - fileCMP_BASIC_SHIPYARD_ID
        855471, // 0x000D0DAF - basic shipyard id               - fileCMP_BASIC_SHIPYARD_ID2
        878061, // 0x000D65ED - basic shipyard id               - fileCMP_BASIC_SHIPYARD_ID3
        792337, // 0x000C1711 - minor race special building ids - fileMINOR_SP_BUILDING_IDS
        1633768,// 0x0018EDE8 - minor main building upgrade ids - fileMINORS_BLD_UPGRADE_IDS
        325859, // 0x0004F8E3 - max building id                 - fileMAX_BLD_ID1
        326357, // 0x0004FAD5 - max building id                 - fileMAX_BLD_ID2
        340226, // 0x00053102 - max building id                 - fileMAX_BLD_ID3
        548904, // 0x00086028 - max building id                 - fileMAX_BLD_ID4
        550924, // 0x0008680C - max building id                 - fileMAX_BLD_ID5
        551145, // 0x000868E9 - max building id                 - fileMAX_BLD_ID6
        551371, // 0x000869CB - max building id                 - fileMAX_BLD_ID7
        551611, // 0x00086ABB - max building id                 - fileMAX_BLD_ID8
        325901, // 0x0004F90D - max building id                 - fileMAX_BLD_ID9
        44506,  // 0x0000ADDA - number of buildings             - fileBLD_NUM1
        52050,  // 0x0000CB52 - number of buildings             - fileBLD_NUM2
        237154, // 0x00039E62 - number of buildings             - fileBLD_NUM3
        997574, // 0x000F38C6 - number of buildings             - fileBLD_NUM4
        254560, // 0x0003E260 - planet bonus types              - filePLANET_BONUS_TYPES
        1025983,// 0x000FA7BF - disable nebula hc graphic       - fileNEBULA_HC_GRAPHIC
        987093, // 0x000F0FD5 - orbital id                      - fileORBITAL_DEFENSE_BLD6
        329173, // 0x000505D5 - dilithium refinery id           - fileDILITHIUM_BLD1
        329838, // 0x0005086E - dilithium refinery id           - fileDILITHIUM_BLD2
        330853, // 0x00050C65 - basic shipyard id               - fileBASIC_SHIPYARD_ID4
        330924, // 0x00050CAC - dilithium refinery id           - fileDILITHIUM_BLD3
        333149, // 0x0005155D - orbital id                      - fileORBITAL_DEFENSE_BLD7
        333182, // 0x0005157E - empire 1 lvl 1 farm id          - fileFED_FARM_LVL1_BLD1
        333543, // 0x000516E7 - orbital id                      - fileORBITAL_DEFENSE_BLD8
        333728, // 0x000517A0 - ground defense id               - fileBASIC_GROUND_DEFENSE_ID1
        333885, // 0x0005183D - shield generator                - fileSHIELD_GEN_BLD2
        44052,  // 0x0000AC14 - orbital id                      - fileORBITAL_DEFENSE_BLD9
        329192, // 0x000505E8 - dilithium refinery id           - fileDILITHIUM_BLD4
        329820, // 0x0005085C - dilithium refinery id           - fileDILITHIUM_BLD5
        329850, // 0x0005087A - dilithium refinery id           - fileDILITHIUM_BLD6
        330837, // 0x00050C55 - basic shipyard id               - fileBASIC_SHIPYARD_ID5
        330865, // 0x00050C71 - basic shipyard id               - fileBASIC_SHIPYARD_ID6
        330887, // 0x00050C87 - dilithium refinery id           - fileDILITHIUM_BLD7
        330936, // 0x00050CB8 - dilithium refinery id           - fileDILITHIUM_BLD8
        333712, // 0x00051790 - ground defense id               - fileBASIC_GROUND_DEFENSE_ID2
        333740, // 0x000517AC - ground defense id               - fileBASIC_GROUND_DEFENSE_ID3
        333869, // 0x0005182D - shield generator                - fileSHIELD_GEN_BLD3
        442016, // 0x0006BEA0 - orbital id                      - fileORBITAL_DEFENSE_BLD10
        444141, // 0x0006C6ED - orbital id                      - fileORBITAL_DEFENSE_BLD11
        985330, // 0x000F08F2 - orbital id                      - fileORBITAL_DEFENSE_BLD12
        1029735,// 0x000FB667 - orbital id                      - fileORBITAL_DEFENSE_BLD13
        236182, // 0x00039A96 - shipyard ids                    - fileSHIPYARD_IDS
        312555, // 0x0004C4EB - martial law buildings           - fileMARTIAL_LAW_BLD3
        236398, // 0x00039B6E - martial law buildings           - fileMARTIAL_LAW_BLD4
        477223, // 0x00074827 - handling of ship names          - fileSHIPNAME_CODE
        612185, // 0x00095759 - plasma hob loading code         - fileSELECTPLASMAHOBFILE
        612287, // 0x000957BF - phaser hob loading code         - fileSELECTPHASERHOBFILE
        611334  // 0x00095406 - torpedo hob loading code        - fileSELECTTORPEDOHOBFILE
    };

    /*
     * A list of valid commands to use with the edit function.
     */
    private static String[] COMMANDLIST;
    // A hash table of all loaded objects
    private Hashtable<String, InternalSegment> INTERNAL_SEGMENTS;
    // Trek.exe file
    private File TREK_FILE = null;
    // If the save process has been cancelled
    private boolean CANCELLED_SAVE = false;
    // stbof
    private Stbof STBOF = null;
    // known trek.exe sizes
    private static final long TREK_SIZE = 1781248;
    private static final String TREK_HASH = "a6f686cba6d3b7936b90103a41f92173";
    
    // COMMANDS
    private static String cmdCHECK = Language.getString("Trek.0"); //$NON-NLS-1$
    private static String cmdCD_PROTECTION = Language.getString("Trek.16"); //$NON-NLS-1$
    private static String cmdSTART_COND = Language.getString("Trek.18"); //$NON-NLS-1$
    private static String cmdMAP_RANGES = Language.getString("Trek.20"); //$NON-NLS-1$
    private static String cmdAI_SHP_BLD_SETS = Language.getString("Trek.22"); //$NON-NLS-1$
    private static String cmdSHIP_BUILDING = Language.getString("Trek.1"); //$NON-NLS-1$
    private static String cmdGALAXY_GEN = Language.getString("Trek.2"); //$NON-NLS-1$
    private static String cmdPATCHES = "Patches and bug fixes";
    
    /**
     * This creates an instance of the class, but does not open a file.
     * To open the file use the open method.
     */
    public Trek()
    {
        this.INTERNAL_SEGMENTS = new Hashtable<String, InternalSegment>();
    }
    
    @Override
    public void markChanged()
    {
        this.CANCELLED_SAVE = true;
    }
    
    /**
     * Make sure file is actually an InternalSegment object or this function will ignore your call.
     */
    @Override
    public void addInternalFile(InternalFile file)
    {
        if (file instanceof InternalSegment)
        {
            // only internal segments allowed
            this.INTERNAL_SEGMENTS.put(file.getName(), (InternalSegment)file);
        }
    }
    
    /**
     * Discards loaded internal segments.
     */
    @Override
    public void discard()
    {
        this.INTERNAL_SEGMENTS.clear();
    }
    
    /**
     * Returns a list of available internal segments.
     */
    @Override
    public Enumeration<String[]> getFiles(FilenameFilter filter)
    {
        Vector <String[]> lst = new Vector<String[]>();

        for (String segment : Trek.SEGMENTLIST)
        {
            if (filter.accept(this.TREK_FILE, segment) &&
                    !this.INTERNAL_SEGMENTS.contains(segment))
            {
                lst.add(new String[]{segment,
                        Integer.toString(INTERNAL_SEGMENTS.size())});
            }
        }
        
        Enumeration<InternalSegment> un = this.INTERNAL_SEGMENTS.elements();
        InternalSegment test = null;

        while(un.hasMoreElements())
        {
            test = un.nextElement();
            if (filter.accept(this.TREK_FILE, test.getName()))
            {
                lst.add(
                        new String[]{
                            test.getName(),
                            Long.toString(test.getSize())
                });
            }
        }
        
        return lst.elements();
    }
    
    /**
     * Returns a list of loaded internal segments.
     */
    @Override
    public Enumeration<String> getListOfLoadedFiles()
    {
        return this.INTERNAL_SEGMENTS.keys();
    }
    
    /**
     * NOT IMPLEMENTED - RETURNS NULL!
     */
    @Override
    public Iterator<String> getRemovedFilesList()
    {
        // parts of trek.exe are not allowed to be removed
        return null;
    }
    
    /**
     * Only removes loaded segment from cache, does not remove the segment from trek.exe!
     */
    @Override
    public void removeInternalFile(String name)
    {
        INTERNAL_SEGMENTS.remove(name);
    }
    
    /**
     * @param command Must be one of the options returned by the open function else null is returned.
     * @return MainPanel object or null
     */
    @Override
    public MainPanel edit(String command) throws Exception
    {
//        try
//        {
            int z=-1;
            
            for (int i = 0; i < Trek.COMMANDLIST.length; i++)
            {
                if (command.equals(Trek.COMMANDLIST[i]))
                {
                    z = i;
                    break;
                }
            }
            
            if (z == -1)
            {
                return null;
            }
            
            switch (z)
            {
                case 0:
                {
                    return new CheckGUI(this.check());
                }
                case 1:
                {
                    return new CDProtectionGUI(this.TREK_FILE.getParentFile(),
                            (CDprotection)this.getInternalFile(Trek.fileCDPROTECT, true));
                }
                case 2:
                {
                    return new MEStartCondGUI((MEStartingConditions)this.getInternalFile(Trek.fileMESTARTCOND, true),
                            this.STBOF);
                }
                case 3:
                {
                    return new ShipMapRangesGUI(this);
                }
                case 4:
                {
                    return new AIShipSetsGUI(this);
                }
                case 5:
                {
                    return new ShipBuildingGUI();
                }
                case 6:
                {
                    return new GalacticMapGUI(this);
                }
                case 7:
                {
                    return new TrekPatchesGUI(this);
                }
            }
//        }
//        catch (Exception er)
//        {
//            UE.displayError(er);
//        }
        return null;
    }
    
    /**
     * Saves changes to the specified file.
     * This creates a new thread with low priority (so it won't eat all resources)
     * and shows a dialog box with the option to cancel the operation.
     * @param dest The file to which changes are to be saved. If the file already exists, it is overwritten without confirmation
     * @return true if succesful else false.
     */
    @Override
    public boolean save(File dest, boolean backup, String extension)
    {
        this.CANCELLED_SAVE = false;

        if (dest == null)
        {
            dest = new File(this.TREK_FILE.getAbsolutePath());
        }

        SaveTask TSK = new SaveTask(dest, backup, extension);
        
        this.CANCELLED_SAVE = !(Boolean)Tools.runUEWorker(TSK);

        if (!CANCELLED_SAVE)
        {
            TREK_FILE = dest;
        }
        
        return !this.CANCELLED_SAVE;
    }
    
    /**
     * Opens the specified file and tests it, if it is a valid trek.exe file.
     * Should anything be wrong an error will be reported and null returned.
     * @param whereFrom The file to open. Must be a valid stbof.res file!
     * @return a String array of valid command that can be used with the edit function or null if an error occured.
     */
    @Override
    public StringGroup[] open(File file)
    {
        if (UE.FILES.getFile(FileManager.P_STBOF) != null)
        {
            this.STBOF = (Stbof)UE.FILES.getFile(FileManager.P_STBOF);
        }
        
        this.TREK_FILE = file;

        try
        {
            // check size
            if (this.TREK_FILE.length() != Trek.TREK_SIZE)
            {
                throw new Exception(
                        Language.getString("Trek.3")); //$NON-NLS-1$
            }
        }
        catch (Exception e)
        {
            UE.displayError(e);
            this.TREK_FILE = null;
        }
        
        return this.getCommands(); // if TREK_FILE == null then this will return null
    }
    
    /**
     * Returns segment address
     * @param segName name of the segment
     * @return
     */
    public long getSegmentAddress(String segName)
    {
        int z = -1;
        
        for (int i = 0; i < Trek.SEGMENTLIST.length; i++)
        {
            if (Trek.SEGMENTLIST[i].equals(segName))
            {
                z = i;
                break;
            }
        }

        if (z >= 0)
        {
            return Trek.SEGMENT_ADDRESS[z];
        }
        
        return -1;
    }
    
    /**
     * Returns a generic segment.
     * @param address address of the segment
     * @param size size of segment
     * @return
     */
    public GenericSegment getSegment(long address, int size) throws FileNotFoundException
    {
        GenericSegment seg = null;

        seg = (GenericSegment)INTERNAL_SEGMENTS.get(Long.toString(address));
            
        if (seg != null && seg.getSize() != size)
        {
            // this is bad because merging/splitting segments won't work since some segments aren't generic
            // so lets make sure we throw an ugly error if we mess up the size
            // this still doesn't protect against mocking around with non generic segments
            // but if they are changed via a generic segment (very bad idea), saving will fail
            String err = Language.getString("Trek.17"); //$NON-NLS-1$
            err = err.replace("%1", Long.toString(seg.getSize())); //$NON-NLS-1$
            err = err.replace("%2", Integer.toString(size)); //$NON-NLS-1$
            throw new FileNotFoundException(err);
        }
        else if (seg == null)
        {
            if ((address + size) <= this.TREK_FILE.length() && address >= 0 && size >= 1)
            {
                FileInputStream fis = null;
                
                try
                {
                    fis = new FileInputStream(this.TREK_FILE);
    
                    fis.skip(address);
                    
                    seg = new GenericSegment(fis, address, size);
                    
                    fis.close();
                }
                catch (Exception e)
                {
                    UE.displayError(e);
                    
                    if (fis != null)
                    {
                        try
                        {
                            fis.close();
                        }
                        catch (Exception f)
                        {}
                    }
                }
            }
        }
        
        if (seg == null)
        {
            String msg = Language.getString("Trek.4"); //$NON-NLS-1$
            msg = msg.replace("%1", "0x" + Long.toHexString(address)); //$NON-NLS-1$ //$NON-NLS-2$
            throw new FileNotFoundException(msg);
        }
        else
        {
            return seg;
        }
    }
    
    /**
     * Returns the internal file object.
     */
    @Override
    public InternalFile getInternalFile(String segName, boolean cache) throws FileNotFoundException
    {
        if (this.INTERNAL_SEGMENTS.containsKey(segName))
        {
            return this.INTERNAL_SEGMENTS.get(segName);
        }
        else
        {
            InternalSegment a = null;
            int z = -1;
            long address;
            
            for (int i = 0; i < Trek.SEGMENTLIST.length; i++)
            {
                if (Trek.SEGMENTLIST[i].equals(segName))
                {
                    z = i;
                    break;
                }
            }
            
            FileInputStream fis = null;
            
            try
            {
                if (z >= 0)
                {
                    fis = new FileInputStream(this.TREK_FILE);
                    address = Trek.SEGMENT_ADDRESS[z];

                    fis.skip(address);

                    switch (z)
                    {
                        case 0:
                        {   // number of ships in shiplist.sst
                            a = new Shipnum(fis);
                            break;
                        }
                        case 1:
                        {   // number of ships in shiplist.sst
                            a = new Shipnum(fis);
                            break;
                        }
                        case 2:
                        {
                            a = new Alien(fis); // Borg Cube ID
                            break;
                        }
                        case 3:
                        {
                            a = new Alien(fis); // Calamarain ID
                            break;
                        }
                        case 4:
                        {
                            a = new Alien(fis); // Chodak ID
                            break;
                        }
                        case 5:
                        {
                            a = new Alien(fis); // Crystal Entity ID
                            break;
                        }
                        case 6:
                        {
                            a = new Alien(fis); // Combat Drone ID
                            break;
                        }
                        case 7:
                        {
                            a = new Alien(fis); // Edo Guardian ID
                            break;
                        }
                        case 8:
                        {
                            a = new Alien(fis); // Gomtuu ID
                            break;
                        }
                        case 9:
                        {
                            a = new Alien(fis); // Husnock ID
                            break;
                        }
                        case 10:
                        {
                            a = new Alien(fis); // Tarellian
                            break;
                        }
                        case 11:
                        {
                            a = new CDprotection(fis);
                            break;
                        }
                        case 12:
                        {
                            a = new MEStartingConditions(fis);
                            break;
                        }
                        case 13:
                        {
                            a = new AlienStartingID(fis);
                            break;
                        }
                        case 14:
                        {
                            a = new SpecialShips(fis); // special1
                            break;
                        }
                        case 15:
                        { // specialstructjump
                            a = new Jump(fis);
                            break;
                        }
                        case 16:
                        { // shipmodeldata
                            a = new ShipModelData(fis, 74);
                            break;
                        }
                        case 17: case 18: case 19:
                        { // starting alien id
                            a = new AlienStartingID(fis);
                            break;
                        }
                        case 20:
                        { // screensaver model list
                            a = new ScreensaverModelList(fis, 71);
                            break;
                        }
                        case 21:
                        { // race list
                            a = new RaceList(fis, (RaceListAddresses)
                                    this.getInternalFile(Trek.fileRACE_LIST_ADDRESSES, true));
                            break;
                        }
                        case 22:
                        { // ship load limit
                            a = new ShipLoadLimit(fis);
                            break;
                        }
                        case 23:
                        { // ship load limit
                            a = new ShipLoadLimit(fis);
                            break;
                        }
                        case 24:
                        { // base map ranges
                            a = new BaseMapRanges(fis);
                            break;
                        }
                        case 25:
                        { // map range tech bonuses
                            a = new MapRangeTechBonuses(fis);
                            break;
                        }
                        case 26:
                        { // race list addresses
                            a = new RaceListAddresses(fis, 35);
                            break;
                        }
                        case 27:
                        { // main research buildings
                            a = new MainResearchBld(fis);
                            break;
                        }
                        case 28:
                        { // main intel buildings
                            a = new MainIntelBld(fis);
                            break;
                        }
                        case 29:
                        { // main industry buildings
                            a = new MainIndustryFarmsAndEnergyBld(fis);
                            break;
                        }
                        case 30:
                        { // main research buildings
                            a = new MainResearchBld2(fis);
                            break;
                        }
                        case 31:
                        { // main intel buildings
                            a = new MainIntelBld2(fis);
                            break;
                        }
                        case 32:
                        { // puny farms
                            a = new PunyFarms(fis);
                            break;
                        }
                        case 33:
                        { // puny factories
                            a = new PunyFactories(fis);
                            break;
                        }
                        case 34:
                        { // AI ship sets
                            a = new AIShpSets(fis);
                            break;
                        }
                        case 35:
                        { // Horizontal map size 1
                            a = new HorizontalMapSize1(fis);
                            break;
                        }
                        case 36:
                        { // Special skips function offset
                            a = new SpecialShipsOffset(fis);
                            break;
                        }
                        case 37: case 38: case 39: case 40: case 41: case 42:
                        { // Unknown 1, 2, 3, 4, 5, 6
                            a = new Placeholder();
                            break;
                        }
                        case 43: case 44:
                        { // min/max number of minors
                            a = new MinorRacesNumbers(fis);
                            break;
                        }
                        case 45: case 46: case 47:
                        { // avg number of space objects
                            a = new AverageNumberOfSpaceObjects(fis);
                            break;
                        }
                        case 48:
                        { // empire image prefixes
                            a = new EmpireImgPrefixes(fis);
                            break;
                        }
                        case 49: case 52: case 53:
                        { // main buildings
                            a = new MainBldStartingIDs(fis);
                            break;
                        }
                        case 50:
                        { // martial law buildings
                            a = new MartialLawBld(fis);
                            break;
                        }
                        case 51:
                        { // basic scanner id
                            a = new BasicScannerID(fis);
                            break;
                        }
                        case 54:
                        { // martial law buildings
                            a = new MartialLawBld2(fis);
                            break;
                        }
                        case 55: case 56: case 57: case 58:
                        case 59: case 60: case 61: case 62:
                        case 63: case 64:
                        // orbital batery, shield gen, sp bld,
                          // trade goods
                        {
                            a = new MovIntToReg(MovIntToReg.EAX, fis);
                            break;
                        }
                        case 65:
                        { // minor race main building id limits
                            a = new MinorRaceMainBldIDLimits(fis);
                            break;
                        }
                        case 66: case 67:
                        { // basic shipyard id, eax
                            a = new CompareBuildingID(fis, CompareBuildingID.ADDRESS_EAX);
                            break;
                        }
                        case 68:
                        { // basic shipyard id, edx
                            a = new CompareBuildingID(fis, CompareBuildingID.ADDRESS_EDX);
                            break;
                        }
                        case 69:
                        { // minor race special building ids
                            a = new MinorRaceSpBuildingIDs(fis);
                            break;
                        }
                        case 70:
                        { // minor race main building upgrade ids
                            a = new MinorRaceBldUpgradeIDs(fis);
                            break;
                        }
                        case 71:
                        { // max building id
                            a = new CompareShortReg(fis, CompareShortReg.REGISTER_AX);
                            break;
                        }
                        case 72:
                        { // max building id
                            a = new CompareShortReg(fis, CompareShortReg.REGISTER_DX);
                            break;
                        }
                        case 73: case 74: case 75:
                        case 76:
                        { // max building id
                            a = new CompareShortReg(fis, CompareShortReg.REGISTER_BX);
                            break;
                        }
                        case 77: case 78:
                        { // max building id
                            a = new CompareIntReg(fis, CompareIntReg.REGISTER_ECX);
                            break;
                        }
                        case 79:
                        { // max building id
                            a = new PushInt(fis);
                            break;
                        }
                        case 80:
                        { // number of buildings
                            a = new CompareShortReg(fis, CompareShortReg.REGISTER_AX);
                            break;
                        }
                        case 81: case 82:
                        { // number of buildings
                            a = new CompareShortReg(fis, CompareShortReg.REGISTER_BX);
                            break;
                        }
                        case 83:
                        { // number of buildings
                            a = new CompareShortReg(fis, CompareShortReg.REGISTER_DX);
                            break;
                        }
                        case 84:
                        { // planet bonus types
                            a = new PlanetBonusTypes(fis);
                            break;
                        }
                        case 85:
                        { // use nebula hardcoded graphic
                            a = new NebulaHCGraphic(fis);
                            break;
                        }
                        case 86:
                        { // orbital battery id
                            a = new CompareByteToMemOffset(fis);
                            break;
                        }
                        case  87: case  88:   // dilithium ref id
                        case  90: case  97:
                        case  98: case  99:
                        case 102: case 103:
                        case  89: case 100:  // basic shipyard id
                        case 101:
                        case  91: case  93:   // orbital batery id
                        case  96: case 107:
                        case 108: case 109:
                        case 110:
                        case  94: case 104:  // ground defense id
                        case 105:
                        case  95: case 106:  // shield generator id
                        {
                            a = new MovIntToReg(MovIntToReg.EDX, fis);
                            break;
                        }
                        case 92:            // fed farm id
                        {
                            a = new MovIntToReg(MovIntToReg.ESI, fis);
                            break;
                        }
                        case 111:
                        {
                            a = new ShipyardBldIDs(fis);
                            break;
                        }
                        case 112:
                        {
                            a = new MartialLawBld3(fis);
                            break;
                        }
                        case 113:
                        {
                            a = new MartialLawBld4(fis);
                            break;
                        }
                        case 114:
                        {
                            a = new ShipnameCode(fis);
                            break;
                        }
                        case 115:
                        {
                            a = new SelectPlasmaHobFile(fis);
                            break;
                        }
                        case 116:
                        {
                            a = new SelectPhaserHobFile(fis);
                            break;
                        }
                        case 117:
                        {
                            a = new SelectTorpedoHobFile(fis);
                            break;
                        }
                    }

                    long size = TREK_FILE.length() - fis.available() - address;

                    fis.close();

                    // check if the reported size equals actual size
                    if (size != a.getSize())
                    {
                        throw new InvalidArgumentsException(
                                "Segment size mismatch: %1".replace("%1", segName));
                    }
                    
                    if (a != null)
                    {
                        a.setName(segName);
                        a.ADDRESS = address;

                        if (cache) {
                            this.INTERNAL_SEGMENTS.put(a.getName(), a);
                        }
                    }
                }
            }
            catch (Exception e)
            {
                UE.displayError(e);
                
                if (fis != null)
                {
                    try
                    {
                        fis.close();
                    }
                    catch (Exception f)
                    {}
                }
            }
            
            if (a == null)
            {
                String msg = Language.getString("Trek.4"); //$NON-NLS-1$
                msg = msg.replace("%1", segName); //$NON-NLS-1$
                throw new FileNotFoundException(msg);
            }
            else
            {
                return a;
            }
        }
    }
    
    /**
     * @return true if changes were made to the opened file and false if not.
     */
    @Override
    public boolean madeChanges() {
        if (this.CANCELLED_SAVE) {
            return true;
        }

        for (Enumeration<InternalSegment> en = this.INTERNAL_SEGMENTS.elements(); en.hasMoreElements();)
        {
            InternalSegment se = (InternalSegment)en.nextElement();

            if (se.madeChanges())
            {
                if (UE.DEBUG_MODE)
                {
                    System.out.println(se.getName() + ": " + se.getAddress() + ".madeChanges == TRUE");
                }
                return true;
            }
        }
        return false;
    }

    /**
     * Checks internal files for unusal data and other problems.
     * @return a Vector containing the check results.
     */
    @SuppressWarnings("unchecked") //$NON-NLS-1$
    @Override
    public Vector<String> check(){
        Vector<String> response;
        
        final CheckTask TSK = new CheckTask();

        response = (Vector<String>)Tools.runUEWorker(TSK); // yes, this is on purpose
        
        //return
        if (TSK.isCancelled())
        {
            response.add(Language.getString("Trek.7")); //$NON-NLS-1$
        }
        
        if (response.isEmpty())
        {
            response.add(Language.getString("Trek.5")); //$NON-NLS-1$
        }
 
        return response;
    }

    @Override
    public StringGroup[] getCommands(){
        if ((this.TREK_FILE != null) && (this.STBOF != null))
        {
            // reset command strings
            Trek.cmdCHECK = Language.getString("Trek.0"); //$NON-NLS-1$
            Trek.cmdCD_PROTECTION = Language.getString("Trek.16"); //$NON-NLS-1$
            Trek.cmdSTART_COND = Language.getString("Trek.18"); //$NON-NLS-1$
            cmdMAP_RANGES = Language.getString("Trek.20"); //$NON-NLS-1$
            cmdAI_SHP_BLD_SETS = Language.getString("Trek.22"); //$NON-NLS-1$
            cmdSHIP_BUILDING = Language.getString("Trek.1"); //$NON-NLS-1$
            cmdGALAXY_GEN = Language.getString("Trek.2"); //$NON-NLS-1$
            cmdPATCHES = "Patches and bug fixes";
            
            Trek.COMMANDLIST = new String[]{Trek.cmdCHECK, Trek.cmdCD_PROTECTION, Trek.cmdSTART_COND,
                    Trek.cmdMAP_RANGES, cmdAI_SHP_BLD_SETS, cmdSHIP_BUILDING, cmdGALAXY_GEN,
                    Trek.cmdPATCHES
            };
            
            //HERE WE GO//
            StringGroup[] ret = new StringGroup[3];
            try
            {
                ret[0] = new StringGroup(Language.getString("Trek.6"), new String[]{ //$NON-NLS-1$ 
                    Trek.cmdCHECK, Trek.cmdCD_PROTECTION, cmdPATCHES});
                ret[1] = new StringGroup(Language.getString("Trek.19"), new String[]{ //$NON-NLS-1$
                    Trek.cmdSTART_COND, Trek.cmdMAP_RANGES, cmdSHIP_BUILDING, cmdGALAXY_GEN});
                ret[2] = new StringGroup(Language.getString("Trek.23"), new String[]{cmdAI_SHP_BLD_SETS}); //$NON-NLS-1$
            }
            catch (Exception tz)
            {
                UE.displayError(tz);
                return null;
            }
            
            return ret;
        }
        else
        {
            return null;
        }
    }

    @Override
    public long getInternalFileSize(String filename) throws IOException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public byte[] getInternalFileHash(String filename) throws Exception {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    private class CheckTask extends UEWorker<Vector<String>>
    {
        public CheckTask()
        {
            super();
            this.RESULT = new Vector<String>();
        }
        
        @Override
        public Vector<String> doInBackground()
        {
            String msg;
            InternalSegment test = null;
            Vector<String> temp;
            
            this.updateStatus(UEWorker.STATUS_WORKING);
            
            for (String segment : Trek.SEGMENTLIST)
            {
                if (this.isCancelled())
                {
                    this.updateStatus(UEWorker.STATUS_CANCEL);
                    return this.RESULT;
                }
                
                try
                {
                    Trek.this.getInternalFile(segment, true);
                }
                catch (FileNotFoundException e)
                {
                    this.RESULT.add(e.getMessage());
                }
            }
            
            // check segments for errors
            Enumeration<InternalSegment> un = Trek.this.INTERNAL_SEGMENTS.elements();
	
            while(un.hasMoreElements())
            {
                if (this.isCancelled())
                {
                    this.updateStatus(UEWorker.STATUS_CANCEL);
                    return this.RESULT;
                }

                try
                {
                    test = un.nextElement();
                    
                    msg = Language.getString("Trek.8"); //$NON-NLS-1$
                    msg = msg.replace("%1", test.getName()); //$NON-NLS-1$
                    this.sendMessage(msg);

                    temp = test.check();

                    if (temp != null && !temp.isEmpty())
                    {   // add to global results
                        this.RESULT.addAll(temp);
                    }
                }
                catch (Exception f)
                {
                    f.printStackTrace();
                    
                    msg = Language.getString("Trek.9"); //$NON-NLS-1$
                    msg = msg.replace("%1", test.getName()); //$NON-NLS-1$
                    msg = msg.replace("%2", f.getMessage()); //$NON-NLS-1$
                    
                    this.RESULT.add(msg);
                }
            }
            
            this.updateStatus(UEWorker.STATUS_DONE);
            return this.RESULT;
        }
    }
    
    private class SaveTask extends UEWorker<Boolean>
    {
        File DEST;
        boolean bkp;
        String ext;
        
        public SaveTask(File dest, boolean backup, String extension)
        {
            super();
            this.DEST = dest;
            bkp = backup;
            ext = extension;
        }
        
        @Override
        public Boolean doInBackground()
        {
            String msg;
            
            // preparation
            Vector<InternalSegment> TODO = new Vector<InternalSegment>(Trek.this.INTERNAL_SEGMENTS.values());
            
            // this will sort segments by address
            Collections.sort(TODO);

            this.RESULT = false;
            
            this.updateStatus(UEWorker.STATUS_WORKING);
            
            try
            {
                //where to
                if (this.DEST.equals(Trek.this.TREK_FILE))
                {
                    // do backup?
                    if (bkp)
                    {
                        msg = Language.getString("Trek.21"); //$NON-NLS-1$
                        this.sendMessage(msg);
                        File file = new File(TREK_FILE.getPath() + ext);
                        
                        int i = 1;
                        
                        while (file.exists())
                        {
                            file = new File(TREK_FILE.getPath() + i + ext);
                            i++;
                        }
                        
                        Tools.copyFile(TREK_FILE, file);
                    }
                    
                    msg = Language.getString("Trek.10"); //$NON-NLS-1$
                    msg = msg.replace("%1", "trek.tmp"); //$NON-NLS-1$ //$NON-NLS-2$
                    
                    this.sendMessage(msg);
                    this.DEST = FileManager.getWritableLocalFile("trek.tmp"); //$NON-NLS-1$
                }
                
                FileOutputStream fos = new FileOutputStream(this.DEST);
                FileInputStream fis = new FileInputStream(Trek.this.TREK_FILE);
                byte[] b;
                int len;
                long addy = 0;
                long size = 0;
                InternalSegment seg;

                for (int i = 0; i < TODO.size(); i++)
                {
                    if (this.isCancelled())
                    {
                        fos.close();
                        fis.close();
                        this.DEST.delete();
                        return this.RESULT;
                    }

                    seg = TODO.get(i);
                    
                    msg = Language.getString("Trek.11"); //$NON-NLS-1$
                    msg = msg.replace("%1", seg.getName()); //$NON-NLS-1$
                    
                    this.sendMessage(msg);

                    size = seg.getAddress() - addy; // calculate num of bytes from cur pos to seg start
                    addy = seg.getAddress() + seg.getSize(); // new address

                    if (size < 0)
                    {
                        msg = "Segment overlapping detected: Segments %1 and %2.";
                        msg = msg.replace("%2", seg.getName());
                        msg = msg.replace("%1", TODO.get(i - 1).getName());

                        this.reportError(new Exception(msg));
                        this.DEST.delete();
                        return this.RESULT;
                    }

                    b = new byte[(int)size];
                    
                    fis.read(b);
                    fos.write(b);

                    seg.save(fos);
                    seg.markSaved();
                    fis.skip(seg.getSize());

                    fos.flush();

                    if (fos.getChannel().position() - seg.getAddress() != seg.getSize())
                    {
                        throw new InvalidArgumentsException(
                                "Segment size mismatch: %1".replace("%1", Long.toString(seg.getAddress())));
                    }
                }
                
                // copy the rest from source
                b = new byte[1024];
                while (fis.available() > 0){
                    len = fis.read(b);
                    fos.write(b, 0, len);
                }
                
                this.sendMessage(Language.getString("Trek.12")); //$NON-NLS-1$
                
                fos.close();
                fis.close();
            }
            catch (Exception ze)
            {
                this.reportError(ze);
            }
            
            if (this.isCancelled())
            {
                this.DEST.delete();
                return this.RESULT;
            }
            
            //replace?
            if (this.DEST.getName().equals("trek.tmp")) //$NON-NLS-1$
            {
                msg = Language.getString("Trek.15"); //$NON-NLS-1$
                msg = msg.replace("%1", this.DEST.getName()); //$NON-NLS-1$

                this.sendMessage(msg);
	    
                try
                {
                    Tools.copyFile(this.DEST, Trek.this.TREK_FILE);
                }
                catch (IOException er)
                {
                    this.reportError(er);
                }
                
                msg = Language.getString("Trek.13"); //$NON-NLS-1$
                msg = msg.replace("%1", this.DEST.getName()); //$NON-NLS-1$
                
                this.sendMessage(msg);
                
                boolean deleted = this.DEST.delete();
                
                if (!deleted)
                {
                    msg = Language.getString("Trek.14"); //$NON-NLS-1$
                    msg = msg.replace("%1", this.DEST.getName()); //$NON-NLS-1$
                    
                    this.reportError(new IOException(msg));
                    return this.RESULT;
                }
            }
            
            this.updateStatus(UEWorker.STATUS_DONE);
            this.RESULT = true;
            return this.RESULT;
        }
    }

    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#getKnownChanges()
     */
    @Override
    public ModList getKnownChanges()
    {
        return null;
    }

    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#hasBeenModified()
     */
    @Override
    public boolean hasBeenModified()
    {
        try
        {
            FileInputStream fis = new FileInputStream(TREK_FILE);
        
            byte[] data = new byte[fis.available()];
            int i = 0;
            int len = 0;
            
            while ( (len = fis.read(data, i, data.length - i)) > 0 )
            {
                i += len;
            }
            
            fis.close();
            
            MessageDigest m = MessageDigest.getInstance("MD5"); //$NON-NLS-1$
            m.update(data, 0, data.length);
            
            byte[] hash = m.digest();

            byte[] h = Tools.hexToByte(TREK_HASH, 16);
            
            return !Arrays.equals(hash, h);
        }
        catch (Exception e)
        {
            UE.displayError(e);
        }

        return false;
    }

    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#toFile()
     */
    @Override
    public File toFile()
    {
        return TREK_FILE;
    }
}
