/**
 * Ultimate Editor (dev)
 * Copyright (C) 2009 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: AIShpSets.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 24.2.2009
 */
package ue.edit.exe.trek;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.edit.exe.InternalSegment;
import ue.util.Tools;

/**
 * @author Alan Podlesek
 */
public class AIShpSets extends InternalSegment
{
    public static final int NORMAL_EXPANSION = 0;
    public static final int UNOPPOSED_EXPANSION = 1;
    public static final int NORMAL_CONSOLIDATION = 2;
    public static final int DEFENSIVE_CONSOLIDATION = 3;
    public static final int OFFENSIVE_CONSOLIDATION = 4;
    public static final int COLD_WAR = 5;
    public static final int DEFENSIVE_WAR = 6;
    public static final int OFFENSIVE_WAR = 7;
    public static final int JIHAD = 8;
    
    public static final int LIST_END = -1;

    private short[][][] buildItemShipType;
    private short[][][] unknown;
    
    public AIShpSets(InputStream in) throws IOException
    {
        buildItemShipType = new short[5][9][36];
        unknown = new short[5][9][36];
        
        byte[] b = new byte[4];
        
        for (int emp = 0; emp < 5; emp++)
        {
            for (int agenda = 0; agenda < 9; agenda++)
            {
                for (int item = 0; item < 36; item++)
                {
                    in.read(b);
                    buildItemShipType[emp][agenda][item] = Tools.toShort(b, true);
                    in.read(b);
                    unknown[emp][agenda][item] = Tools.toShort(b, true);
                }
            }
        }
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();
        
        return response;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        return 12960;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        byte[] b = new byte[2];
        
        for (int emp = 0; emp < 5; emp++)
        {
            for (int agenda = 0; agenda < 9; agenda++)
            {
                for (int item = 0; item < 36; item++)
                {
                    out.write(Tools.toByte(buildItemShipType[emp][agenda][item], true));
                    out.write(b);
                    out.write(Tools.toByte(unknown[emp][agenda][item], true));
                    out.write(b);
                }
            }
        }
    }
    
    /**
     * Sets ship type id
     * @param empire (0-4)
     * @param agenda (0-9)
     * @param index (0-35)
     * @param type (0-5 or 9)
     */
    public void setShipType(int empire, int agenda, int index, int type)
    {
        if (buildItemShipType[empire][agenda][index] != type)
        {
            buildItemShipType[empire][agenda][index] = (short)type;
            this.markChanged();
        }
    }
    
    /**
     * Returns ship type.
     * @param empire (0-4)
     * @param agenda (0-9)
     * @param index (0-35)
     * @return
     */
    public int getShipType(int empire, int agenda, int index)
    {
        return buildItemShipType[empire][agenda][index];
    }
}
