/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: Alien.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Last modified: 23.7.2008
 */
package ue.edit.exe.trek;

import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.Language;
import ue.UE;
import ue.edit.FileManager;
import ue.edit.exe.InternalSegment;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.sst.ShipList;
import ue.exception.InvalidArgumentsException;
import ue.util.ID;
import ue.util.Tools;

/**
 * This class contains alien id.
 */
public class Alien extends InternalSegment
{
    private short SHIP_INDEX; // the value is stored as a short in file as well
    private byte[] TEST_DATA = new byte[3];

    private static byte[] test = new byte[]{(byte)0xB8};
    private static byte[] testExtendedVersion = new byte[]{0, 0, (byte)0xB8};
    private static byte[] testExtendedVersion2 = new byte[]{0, 0, (byte)0xBA};
    private static byte[] testExtendedVersion3 = new byte[]{0, 0, (byte)0x52};
    
    /**
     * @param in the InputStream to read from
     * @param address the location in file
     */
    public Alien(InputStream in) throws Exception
    {
        //read num
        byte[] b = new byte[2];
        in.read(b);
        this.SHIP_INDEX = Tools.toShort(b, true);

        in.read(TEST_DATA);

        if (!checkIfValid(TEST_DATA, testExtendedVersion, false)
                && !checkIfValid(TEST_DATA, testExtendedVersion2, false)
                && !checkIfValid(TEST_DATA, testExtendedVersion3, false))
        {
            b = new byte[]{TEST_DATA[0]};

            checkIfValid(b, test, true);
        }
    }
    
    /**
     * Used to save changes.
     * @param out the OutputStream to write the file to.
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        out.write(Tools.toByte(this.SHIP_INDEX, true));
        out.write(TEST_DATA);
    }
    
    /**
     * @return uncompressed size of the file.
     */
    @Override
    public long getSize()
    {
        long size = 5;
        return size;
    }
    
    @Override
    public Vector<String> check(){ 
        Vector<String> response = new Vector<String>();
        
        if (this.SHIP_INDEX < 5){
            String msg = Language.getString("Alien.0"); //$NON-NLS-1$
            msg = msg.replace("%1", Long.toString(this.ADDRESS)); //$NON-NLS-1$
            msg = msg.replace("%2", Integer.toString(5)); //$NON-NLS-1$
            response.add(msg);
        }
        
        // compare value with the one in shiplist.sst
        if (UE.FILES.getFile(FileManager.P_STBOF) != null)
        {
            Stbof st = (Stbof)UE.FILES.getFile(FileManager.P_STBOF);
            
            try
            {
                ShipList sl = (ShipList)st.getInternalFile(Stbof.fileSHIPLIST_SST, true);
                
                ID[] id = sl.getIDs(ShipList.RACE_ALIEN);
                
                int i = Integer.parseInt(this.getName().substring(5));
                
                if (i < 9)
                {
                    i--;
                }
                
                if (this.SHIP_INDEX != id[i].hashCode())
                {
                    String msg = Language.getString("Alien.2"); //$NON-NLS-1$
                    msg = msg.replace("%1", Long.toString(this.ADDRESS)); //$NON-NLS-1$
                    msg = msg.replace("%2", Short.toString(this.SHIP_INDEX)); //$NON-NLS-1$
                    msg = msg.replace("%3", Integer.toString(id[i].hashCode())); //$NON-NLS-1$
                    
                    response.add(msg);
                }
            }
            catch (FileNotFoundException e)
            {
                e.printStackTrace();
                response.add(e.getLocalizedMessage());
            }
        }

        return response;
    }
    
    /**
     * Sets ship index.
     */
    public void setIndex(short index) throws InvalidArgumentsException
    {
        if (index < 5){
            String msg = Language.getString("Alien.1"); //$NON-NLS-1$
            msg.replace("%1", Integer.toString(5)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
        this.SHIP_INDEX = index;
        this.markChanged();
    }

    /**
     * @return
     */
    public int getIndex()
    {
        return SHIP_INDEX;
    }
}
