/**
 * Ultimate Editor
 * Copyright (C) 2009 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: AverageNumberOfSpaceObjects.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 7.6.2009
 */
package ue.edit.exe.trek;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.edit.exe.InternalSegment;
import ue.util.Tools;

/**
 * @author Alan Podlesek
 */
public class AverageNumberOfSpaceObjects extends InternalSegment
{
    private static final byte[] mov = new byte[]{(byte) 0xC7, 0x44, 0x24, 0x6C};
    private int num;
    
    public AverageNumberOfSpaceObjects(InputStream in) throws Exception
    {
        byte[] b = new byte[4];
        
        in.read(b);
        
        InternalSegment.checkIfValid(b, mov, true);
        
        in.read(b);
        num = Tools.toInt(b, true);
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();

        if (num < 0)
        {
            response.add(this.getCheckIntegrityString(InternalSegment.INTEGRITY_CHECK_ERROR,
                    "Average number of objects on the map is negative."));
        }

        return response;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        return 8;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        out.write(mov);
        out.write(Tools.toByte(num, true));
    }

    public int getNumber()
    {
        return num;
    }

    public void setNumber(int num)
    {
        if (this.num != num && num > 0)
        {
            this.num = num;
            this.markChanged();
        }
    }
}
