/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2008 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: BaseMapRanges.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 9.10.2008
 */
package ue.edit.exe.trek;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.Language;
import ue.edit.exe.InternalSegment;
import ue.util.Tools;

/**
 * Contains map base ranges.
 */
public class BaseMapRanges extends InternalSegment
{
    public static final int SHORT_RANGE = 0;
    public static final int MEDIUM_RANGE = 1;
    public static final int LONG_RANGE = 2;
    
    // 0 <= empire <= 4
    // 0 <= range <= 2
    private int[] base = new int[15];   // [empire * 3 + range]

    
    public BaseMapRanges(InputStream in) throws IOException
    {
        byte[] b = new byte[4];
        
        for (int i = 0; i < base.length; i++)
        {
            in.read(b);
            base[i] = Tools.toInt(b, true);
        }
    }
    
    public int getBaseRange(int race, int range)
    {
        return base[race * 3 + range];
    }
    
    public void setBaseRange(int race, int range, int value)
    {
        int i = race * 3 + range;
        
        if (base[i] == value) return;
        
        base[i] = value;
        this.markChanged();
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        Vector<String> result = new Vector<String>();
        
        for (int i = 0; i < base.length; i++)
        {
            if (base[i] <= 0)
            {
                String msg = Language.getString("BaseMapRanges.0"); //$NON-NLS-1$
                msg = msg.replace("%1", Long.toString(ADDRESS)); //$NON-NLS-1$
                int rc = i % 3;
                int rn = i - rc * 3;
                msg = msg.replace("%2", Integer.toString(rn)); //$NON-NLS-1$
                msg = msg.replace("%3", Integer.toString(rc)); //$NON-NLS-1$
                result.add(msg);
            }
        }
        
        return result;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        return base.length * 4;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        for (int i = 0; i < base.length; i++)
        {
            out.write(Tools.toByte(base[i], true));
        }
    }
    
}
