/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: CDprotection.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Last modified: 23.7.2008
 */
package ue.edit.exe.trek;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.Language;
import ue.edit.exe.InternalSegment;

/**
 * This class is used for turning cd protection off/on.
 */
public class CDprotection extends InternalSegment
{
    private byte[] DATA;
    
    /**
     * @param in the InputStream to read from
     */
    public CDprotection(InputStream in) throws IOException
    {
        //read data
        this.DATA = new byte[2];
        in.read(this.DATA);
    }
    
    /**
     * Used to save changes.
     * @param out the OutputStream to write the file to.
     */
    @Override
    public void save(OutputStream out) throws Exception
    { 
        out.write(this.DATA);
    }
    
    /**
     * @return uncompressed size of the file.
     */
    @Override
    public long getSize()
    {
        return 2;
    }
    
    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();
        
        String msg = Language.getString("CDprotection.0"); //$NON-NLS-1$
        msg = msg.replace("%1", Long.toString(this.ADDRESS)); //$NON-NLS-1$
        
        if ((this.DATA[1] | 0x74) != this.DATA[1] && (this.DATA[1] | 0xEB) != this.DATA[1])
        {
            response.add(msg);
        }
        
        return response;
    }
    
    /**
     * Enables/disabled cd protection.
     */
    public void setEnabled(boolean enabled)
    {
        if (enabled)
        {
            this.DATA[1] = (byte)0x74;
        }
        else
        {
            this.DATA[1] = (byte)0xEB;
        }
        this.markChanged();
    }

    /**
     * Returns cd protection's state.
     */
    public boolean isEnabled()
    {
        return (this.DATA[1] == 0x74);
    }
}