/**
 * Ultimate Editor
 * Copyright (C) 2005-2011 Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: CompareByteReg.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 21.9.2010
 */

package ue.edit.exe.trek;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;
import ue.edit.exe.InternalSegment;

/**
 *
 */
public class CompareByteReg extends InternalSegment
{
    public static final int REGISTER_DX = 0;

    private static final byte[] test = new byte[]{(byte)0x66};
    private static final byte[] test_dx = new byte[]{(byte)0x83, (byte)0xFA};

    private int register;
    private int value;

    public CompareByteReg(InputStream in, int register) throws Exception
    {
        this.register = register;

        byte[] b = new byte[1];

        in.read(b);
        checkIfValid(b, test, true);

        switch (register)
        {
            case REGISTER_DX:
            {
                b = new byte[2];
                in.read(b);
                checkIfValid(b, test_dx, true);
                break;
            }
        }

        value = in.read() & 0xFF;
    }

    @Override
    public void save(OutputStream out) throws Exception
    {
        out.write(test);

        switch (register)
        {
            case REGISTER_DX:
            {
                out.write(test_dx);
                break;
            }
        }

        out.write(value);
    }

    @Override
    public Vector<String> check()
    {
        return null;
    }

    @Override
    public long getSize()
    {
        switch (register)
        {
            case REGISTER_DX:
            {
                return 4;
            }
        }

        return -1;
    }

    public void setValue(int value)
    {
        if (this.value != value)
        {
            this.value = value;
            this.markChanged();
        }
    }
}