/**
 * Ultimate Editor
 * Copyright (C) 2008 Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: CompareByteToMemOffset.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 2.7.2011
 */

package ue.edit.exe.trek;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;
import ue.edit.exe.InternalSegment;
import ue.exception.InvalidArgumentsException;

/**
 *
 */
public class CompareByteToMemOffset extends InternalSegment
{
    public static final int EBP = 0x7D;
    
    private static final byte[] cmp = new byte[]{0x66, (byte)0x83};

    private int register;
    private int offset;
    private int value;

    public CompareByteToMemOffset(InputStream in) throws IOException, Exception
    {
        byte[] b = new byte[2];
        in.read(b);

        checkIfValid(b, cmp, true);

        register = in.read();
        offset = in.read();
        value = in.read();
    }

    @Override
    public void save(OutputStream out) throws Exception {
        out.write(cmp);
        out.write(register);
        out.write(offset);
        out.write(value);
    }

    @Override
    public Vector<String> check() {
        return null;
    }

    @Override
    public long getSize() {
        return 5;
    }

    public int getValue()
    {
        return value;
    }

    public void setValue(int value) throws InvalidArgumentsException
    {
        if (value > Byte.MAX_VALUE)
        {
            throw new InvalidArgumentsException("Value exceeds the max value of a signed byte.");
        }

        if (this.value == value)
        {
            return;
        }

        this.value = value;
        this.markChanged();
    }
}
