/**
 * Ultimate Editor
 * Copyright (C) 2005-2010 Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: CompareIntReg.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 21.9.2010
 */

package ue.edit.exe.trek;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;
import ue.edit.exe.InternalSegment;
import ue.util.Tools;

/**
 *
 */
public class CompareIntReg extends InternalSegment
{
    public static final int REGISTER_ECX = 0;

    private static final byte[] test_ecx = new byte[]{(byte)0x81, (byte)0xF9};

    private int register;
    private int value;

    public CompareIntReg(InputStream in, int register) throws Exception
    {
        this.register = register;

        byte[] b = new byte[2];

        switch (register)
        {
            case REGISTER_ECX:
            {
                in.read(b);
                checkIfValid(b, test_ecx, true);
                break;
            }
        }

        b = new byte[4];
        in.read(b);
        value = Tools.toInt(b, true);
    }

    @Override
    public void save(OutputStream out) throws Exception
    {
        switch (register)
        {
            case REGISTER_ECX:
            {
                out.write(test_ecx);
                break;
            }
        }

        out.write(Tools.toByte(value, true));
    }

    @Override
    public Vector<String> check()
    {
        return null;
    }

    @Override
    public long getSize()
    {
        return 6;
    }

    public void setValue(int value)
    {
        if (this.value != value)
        {
            this.value = value;
            this.markChanged();
        }
    }
}