/**
 * Ultimate Editor
 * Copyright (C) 2005-2010 Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: CompareShortReg.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 21.9.2010
 */

package ue.edit.exe.trek;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;
import ue.edit.exe.InternalSegment;
import ue.util.Tools;

/**
 *
 */
public class CompareShortReg extends InternalSegment
{
    public static final int REGISTER_AX = 0;
    public static final int REGISTER_BX = 3;
    public static final int REGISTER_DX = 1;

    private static final byte[] test = new byte[]{(byte)0x66};
    private static final byte[] test_ax = new byte[]{(byte)0x3D};
    private static final byte[] test_dx = new byte[]{(byte)0x81, (byte)0xFA};
    private static final byte[] test_bx = new byte[]{(byte)0x81, (byte)0xFB};

    private int register;
    private short value;

    public CompareShortReg(InputStream in, int register) throws Exception
    {
        this.register = register;

        byte[] b = new byte[1];

        in.read(b);
        checkIfValid(b, test, true);

        switch (register)
        {
            case REGISTER_AX:
            {
                in.read(b);
                checkIfValid(b, test_ax, true);
                break;
            }
            case REGISTER_DX:
            {
                b = new byte[2];
                in.read(b);
                checkIfValid(b, test_dx, true);
                break;
            }
            case REGISTER_BX:
            {
                b = new byte[2];
                in.read(b);
                checkIfValid(b, test_bx, true);
                break;
            }
        }

        b = new byte[2];
        in.read(b);
        value = Tools.toShort(b, true);
    }

    @Override
    public void save(OutputStream out) throws Exception
    {
        out.write(test);

        switch (register)
        {
            case REGISTER_AX:
            {
                out.write(test_ax);
                break;
            }
            case REGISTER_BX:
            {
                out.write(test_bx);
                break;
            }
            case REGISTER_DX:
            {
                out.write(test_dx);
                break;
            }
        }

        out.write(Tools.toByte(value, true));
    }

    @Override
    public Vector<String> check()
    {
        return null;
    }

    @Override
    public long getSize()
    {
        switch (register)
        {
            case REGISTER_AX:
            {
                return 4;
            }
            case REGISTER_DX: case REGISTER_BX:
            {
                return 5;
            }
        }

        return -1;
    }

    public void setValue(short value)
    {
        if (this.value != value)
        {
            this.value = value;
            this.markChanged();
        }
    }
}