/**
 * Ultimate Editor (dev)
 * Copyright (C) 2010 Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: EmpireImgPrefixes.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 22.5.2010
 */

package ue.edit.exe.trek;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;
import ue.Language;
import ue.edit.exe.InternalSegment;
import ue.exception.InvalidArgumentsException;
import ue.util.CommonStrings;

public class EmpireImgPrefixes extends InternalSegment
{
    private char[] PREFIX;

    public EmpireImgPrefixes(InputStream in) throws IOException
    {
        PREFIX = new char[5];

        for (int i = 0; i < PREFIX.length; i++)
        {
            PREFIX[i] = (char)in.read();
        }
    }

    @Override
    public void save(OutputStream out) throws Exception
    {
        for (int i = 0; i < PREFIX.length; i++)
        {
            out.write(PREFIX[i]);
        }
    }

    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();
        String msg;

        for (int i = 0; i < PREFIX.length; i++)
        {
            if (!Character.isLetterOrDigit(PREFIX[i]))
            {
                msg = Language.getString("EmpireImgPrefixes.1");
                msg = msg.replace("%1", "0x" + Integer.toHexString(PREFIX[i]));
                response.add(msg);
            }
        }

        return response;
    }

    public char getEmpirePrefix(int empire_index) throws InvalidArgumentsException
    {
        checkIndex(empire_index);
        return PREFIX[empire_index];
    }

    public void setEmpirePrefix(int empire_index, char prefix) throws InvalidArgumentsException
    {
        checkIndex(empire_index);
        
        if (PREFIX[empire_index] != prefix)
        {
            PREFIX[empire_index] = prefix;
            this.markChanged();
        }
    }

    private void checkIndex(int index) throws InvalidArgumentsException
    {
        if (index < 0 || index >= PREFIX.length)
        {
            throw new InvalidArgumentsException(
                    CommonStrings.getLocalizedString(CommonStrings.InvalidIndex_1,
                    new String[]{Integer.toString(index)}));
        }
    }

    @Override
    public long getSize()
    {
        return PREFIX.length;
    }
}
