/**
 * Ultimate Editor
 * Copyright (C) 2009 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: HorizontalMapSize1.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 7.4.2009
 */
package ue.edit.exe.trek;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.Language;
import ue.edit.exe.InternalSegment;
import ue.util.Tools;

/**
 * @author Alan Podlesek
 */
public class HorizontalMapSize1 extends InternalSegment
{
    private static byte[] CODE_BLOCK_1 = new byte[]{0x52, (byte)0x8B, 0x15, (byte)0xB8,
            0x36, 0x5A, 0x00, (byte)0xB8};
    private static byte[] CODE_BLOCK_2 = new byte[]{(byte)0x83, (byte)0xFA, 0x01, 0x73,
            0x04, (byte)0x85, (byte)0xD2, 0x5A, (byte)0xC3, 0x76, 0x0C,
            (byte)0x83, (byte)0xFA, 0x02, 0x75, (byte)0xF7, (byte)0xB8};
    private static byte[] CODE_BLOCK_3 = new byte[]{0x5A, (byte)0xC3, (byte)0xB8};
    private static byte[] CODE_BLOCK_4 = new byte[]{0x5A, (byte)0xC3};

    private int size[] = new int[3];
    
    public HorizontalMapSize1(InputStream in) throws Exception
    {
        byte[] b = new byte[CODE_BLOCK_1.length];
        byte[] d = new byte[4];
             
        in.read(b);
        InternalSegment.checkIfValid(b, CODE_BLOCK_1, true);
        
        in.read(d);
        size[0] = Tools.toInt(d, true);
        
        b = new byte[CODE_BLOCK_2.length];
        in.read(b);
        InternalSegment.checkIfValid(b, CODE_BLOCK_2, true);
        
        in.read(d);
        size[2] = Tools.toInt(d, true);
        
        b = new byte[CODE_BLOCK_3.length];
        in.read(b);
        InternalSegment.checkIfValid(b, CODE_BLOCK_3, true);
        
        in.read(d);
        size[1] = Tools.toInt(d, true);

        b = new byte[CODE_BLOCK_4.length];
        in.read(b);
        InternalSegment.checkIfValid(b, CODE_BLOCK_4, true);
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();
        
        for (int i = 0; i < 2; i++)
        {
            if (size[i] > size[i+1])
            {
                response.add(
                        getCheckIntegrityString(InternalSegment.INTEGRITY_CHECK_WARNING,
                        Language.getString("HorizontalMapSize1.0") //$NON-NLS-2$
                        .replace("%1", Integer.toString(i+1)).replace("%2", Integer.toString(i)))
                        );
            }
        }
        
        return response;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        return 42;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        out.write(CODE_BLOCK_1);
        out.write(Tools.toByte(size[0], true));
        out.write(CODE_BLOCK_2);
        out.write(Tools.toByte(size[2], true));
        out.write(CODE_BLOCK_3);
        out.write(Tools.toByte(size[1], true));
        out.write(CODE_BLOCK_4);
    }
    
    public void setMapSize(int galSize, int mapSize)
    {
        if (size[galSize] == mapSize)
        {
            size[galSize] = mapSize;
            this.markChanged();
        }
    }
    
    public int getMapSize(int galSize)
    {
        return size[galSize];
    }
}
