/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: Jump.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Last modified: 23.7.2008
 */
package ue.edit.exe.trek;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.Language;
import ue.edit.exe.InternalSegment;
import ue.exception.InvalidArgumentsException;

/**
 * Jump instruction -> only type, not the entire thing.
 */
public class Jump extends InternalSegment
{
    // more will be added as needed
    public final static int JZ = 116; // 74h
    public final static int JNZ = 117;// 75h
    public final static int JLE = 126;// 7Eh
    public final static int JMP = 235;// EBh
    public final static int JBE = 118;// 76h
    
    private int data;
    
    public Jump(InputStream in) throws IOException
    {
        this.data = in.read();
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        Vector<String> ret = new Vector<String>();

        if ((this.data != Jump.JZ) && (this.data != Jump.JNZ) && (this.data != Jump.JLE)
                && (this.data != Jump.JMP) && (this.data != Jump.JBE))
        {
            ret.add(Language.getString("Jump.0").replace("%1", Long.toString(this.ADDRESS))); //$NON-NLS-1$ //$NON-NLS-2$
        }
       
        return ret;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        return 1;
    }
    
    /**
     * Sets the jump, use one of the static field values.
     */
    public void set(int data) throws InvalidArgumentsException
    {
        if ((data < 0) || (data > 255))
        {
            throw new InvalidArgumentsException(Language.getString("Jump.1").replace("%1", Integer.toString(data))); //$NON-NLS-1$ //$NON-NLS-2$
        }
        if (this.data == data)
        {
            return;
        }
        this.data = data;
        this.markChanged();
    }
    
    /**
     * Returns the byte.
     */
    public int get()
    {
        return this.data;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        out.write(this.data);
    }
}
