/**
 * Ultimate Editor
 * Copyright (C) 2009 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: MainIndustryFarmsAndEnergyBld.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 3.1.2009
 */
package ue.edit.exe.trek;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.Language;
import ue.edit.exe.InternalSegment;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;

/**
 * @author Alan Podlesek
 */
public class MainIndustryFarmsAndEnergyBld extends InternalSegment
{
    private static final byte[] TEST_MOV_ECX = new byte[]{(byte)0xB9};
    private static final byte[] TEST_CMP_ECX_BYTE = new byte[]{(byte)0x83, (byte)0xF9};
    private static final byte[] TEST_CMP_ECX_WORD = new byte[]{(byte)0x81, (byte)0xF9};
    private static final byte[] TEST_CMP_CX = new byte[]{(byte)0x66, (byte)0x81, (byte)0xF9};

    private byte[][] UNKNOWN = new byte[94][];
    private int[] if_begin_id = new int[5];
    private int[] if_end_id = new int[5];
    private int[] e_begin_id = new int[5];
    private int[] e_end_id = new int[5];
    private int[] f_begin_id = new int[5];
    private int[] f_end_id = new int[5];
    private short max_id;
    
    public MainIndustryFarmsAndEnergyBld(InputStream in) throws Exception
    {
        int u = 0;
        readUnknown(in, u++, 3, null);

        byte[] b = new byte[4];

        // industry, farms and energy
        for (int j = 0; j < 3; j++)
        {
            if (j == 0)
            {
                readUnknown(in, u++, 40, TEST_MOV_ECX);
            }
            else if (j == 1)
            {
                readUnknown(in, u++, 108, TEST_MOV_ECX);
            }
            else
            {
                readUnknown(in, u++, 132, TEST_MOV_ECX);
            }
            
            // 4
            int i = 0;
            in.read(b);
            if_begin_id[i] = Tools.toInt(b, true);
            
            readUnknown(in, u++, 32, TEST_CMP_ECX_BYTE);
            
            if_end_id[i++] = in.read();
            
            for (; i < 10; i++)
            {
                // 3
                readUnknown(in, u++, 3, TEST_MOV_ECX);
                
                // 4
                in.read(b);
                if (i < 5)
                {
                    if_begin_id[i] = Tools.toInt(b, true);
                }
                else
                {
                    e_begin_id[i-5] = Tools.toInt(b, true);
                }
                
                readUnknown(in, u++, 13, null);
                
                if (i < 5)
                {
                    testLastBytes(UNKNOWN[u - 1], TEST_CMP_ECX_WORD, true);

                    in.read(b);
                    if_end_id[i] = Tools.toInt(b, true);
                }
                else
                {
                    if (i == 5)
                    {
                        testLastBytes(UNKNOWN[u - 1], TEST_CMP_ECX_BYTE, true);
                        e_end_id[i-5] = in.read();
                    }
                    else
                    {
                        testLastBytes(UNKNOWN[u - 1], TEST_CMP_ECX_WORD, true);

                        in.read(b);
                        e_end_id[i-5] = Tools.toInt(b, true);
                    }
                }
            }
        }

        readUnknown(in, u++, 64, null);
        
        // farms and energy
        for (int j = 0; j < 2; j++)
        {
            readUnknown(in, u++, 8, TEST_MOV_ECX);
            
            // 4
            int i = 0;
            in.read(b);
            f_begin_id[i] = Tools.toInt(b, true);
            
            if (j == 0)
            {
                readUnknown(in, u++, 32, TEST_CMP_ECX_BYTE);
            }
            else
            {
                readUnknown(in, u++, 15, TEST_CMP_ECX_BYTE);
            }
            
            f_end_id[i++] = in.read();
            
            for (; (i < 10 && j == 0) || (i < 5); i++)
            {
                // 3
                readUnknown(in, u++, 3, TEST_MOV_ECX);
                
                in.read(b);
                if (i < 5)
                {
                    f_begin_id[i] = Tools.toInt(b, true);
                }
                else
                {
                    e_begin_id[i-5] = Tools.toInt(b, true);
                }
                
                readUnknown(in, u++, 13, null);
                    
                // 4
                if (i < 5)
                {
                    testLastBytes(UNKNOWN[u - 1], TEST_CMP_ECX_WORD, true);
                    in.read(b);
                    f_end_id[i] = Tools.toInt(b, true);
                }
                else
                {
                    if (i == 5)
                    {
                        testLastBytes(UNKNOWN[u - 1], TEST_CMP_ECX_BYTE, true);
                        e_end_id[i-5] = in.read();
                    }
                    else
                    {
                        testLastBytes(UNKNOWN[u - 1], TEST_CMP_ECX_WORD, true);
                        in.read(b);
                        e_end_id[i-5] = Tools.toInt(b, true);
                    }
                }
            }
        }

        readUnknown(in, u++, 34, TEST_CMP_CX);

        b = new byte[2];
        in.read(b);
        max_id = Tools.toShort(b, true);

        readUnknown(in, u++, 29, null);
    }

    private void readUnknown(InputStream in, int u, int size, final byte[] test) throws Exception
    {
        UNKNOWN[u] = new byte[size];
        in.read(UNKNOWN[u]);
        testLastBytes(UNKNOWN[u], test, true);
    }
    
    private void writeUnknown(OutputStream out, int u) throws IOException
    {
        out.write(UNKNOWN[u]);
    }

    /**
     * Sets maximum id
     * @param id
     */
    public void setMaxID(short id)
    {
        if (max_id != id)
        {
            max_id = id;
            this.markChanged();
        }
    }

    /**
     * @param empire empire id (0-5)
     * @param first_id building id
     * @param last_id building id
     */
    public void setIndustryAndFoodIDSpan(int empire, int first_id, int last_id) throws InvalidArgumentsException
    {
        if (first_id > max_id) first_id = max_id;
        if (last_id > max_id) last_id = max_id;
        
        if (empire == 0 && last_id > 0x7f)
        {
            throw new InvalidArgumentsException(Language.getString("MainIndustryFarmsAndEnergyBld.0").replace("%1", Integer.toString(last_id))); //$NON-NLS-1$ //$NON-NLS-2$
        }
        
        if (if_begin_id[empire] != first_id || if_end_id[empire] != last_id)
        {
            if_begin_id[empire] = first_id;
            if_end_id[empire] = last_id;
            this.markChanged();
        }
    }

    /**
     * @param empire empire id (0-5)
     * @param first_id building id
     * @param last_id building id
     */
    public void setEnergyIDSpan(int empire, int first_id, int last_id) throws InvalidArgumentsException
    {
        if (first_id > max_id) first_id = max_id;
        if (last_id > max_id) last_id = max_id;
        
        if (empire == 0 && last_id > 0x7f)
        {
            throw new InvalidArgumentsException(Language.getString("MainIndustryFarmsAndEnergyBld.1").replace("%1", Integer.toString(last_id))); //$NON-NLS-1$ //$NON-NLS-2$
        }
        
        if (e_begin_id[empire] != first_id || e_end_id[empire] != last_id)
        {
            e_begin_id[empire] = first_id;
            e_end_id[empire] = last_id;
            this.markChanged();
        }
    }

    /**
     * @param empire empire id (0-5)
     * @param first_id building id
     * @param last_id building id
     */
    public void setFoodIDSpan(int empire, int first_id, int last_id) throws InvalidArgumentsException
    {
        if (first_id > max_id) first_id = max_id;
        if (last_id > max_id) last_id = max_id;
        
        if (empire == 0 && last_id > 0x7f)
        {
            throw new InvalidArgumentsException(Language.getString("MainIndustryFarmsAndEnergyBld.2").replace("%1", Integer.toString(last_id))); //$NON-NLS-1$ //$NON-NLS-2$
        }
        
        if (f_begin_id[empire] != first_id || f_end_id[empire] != last_id)
        {
            f_begin_id[empire] = first_id;
            f_end_id[empire] = last_id;
            this.markChanged();
        }
    }

    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        Vector<String> ret = new Vector<String>();
        return ret;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        return 1544;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        int u = 0;
        writeUnknown(out, u++);

        // industry, farms and energy
        for (int j = 0; j < 3; j++)
        {
            writeUnknown(out, u++);
            
            // 4
            int i = 0;
            out.write(Tools.toByte(if_begin_id[i], true));
            
            writeUnknown(out, u++);
            
            out.write(if_end_id[i++]);
            
            for (; i < 10; i++)
            {
                // 3
                writeUnknown(out, u++);
                
                // 4
                if (i < 5)
                {
                    out.write(Tools.toByte(if_begin_id[i], true));
                }
                else
                {
                    out.write(Tools.toByte(e_begin_id[i-5], true));
                }
                
                writeUnknown(out, u++);
                
                if (i < 5)
                {
                    out.write(Tools.toByte(if_end_id[i], true));
                }
                else
                {
                    if (i == 5)
                    {
                        out.write(e_end_id[i-5]);
                    }
                    else
                    {
                        out.write(Tools.toByte(e_end_id[i-5], true));
                    }
                }
            }
        }

        writeUnknown(out, u++);
        
        // farms
        for (int j = 0; j < 2; j++)
        {
            writeUnknown(out, u++);
            
            // 4
            int i = 0;
            out.write(Tools.toByte(f_begin_id[i], true));
            
            writeUnknown(out, u++);
            
            out.write(f_end_id[i++]);
            
            for (; (i < 10 && j == 0) || (i < 5); i++)
            {
                // 3
                writeUnknown(out, u++);
                
                if (i < 5)
                {
                    out.write(Tools.toByte(f_begin_id[i], true));
                }
                else
                {
                    out.write(Tools.toByte(e_begin_id[i-5], true));
                }
                
                writeUnknown(out, u++);
                    
                // 4
                if (i < 5)
                {
                    out.write(Tools.toByte(f_end_id[i], true));
                }
                else
                {
                    if (i == 5)
                    {
                        out.write(e_end_id[i-5]);
                    }
                    else
                    {
                        out.write(Tools.toByte(e_end_id[i-5], true));
                    }
                }
            }
        }

        writeUnknown(out, u++);

        out.write(Tools.toByte(max_id, true));

        writeUnknown(out, u++);
    }
    
}
