/**
 * Ultimate Editor
 * Copyright (C) 2009 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: MainResearchBld.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 2.1.2009
 */
package ue.edit.exe.trek;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.edit.exe.InternalSegment;
import ue.util.Tools;

/**
 * @author Alan Podlesek
 */
public class MainResearchBld extends InternalSegment
{
    private static final byte[] TEST_MOV_ECX = new byte[]{(byte)0xB9};
    private static final byte[] TEST_CMP_ECX_WORD = new byte[]{(byte)0x81, (byte)0xF9};
    private static final byte[] TEST_CMP_CX = new byte[]{(byte)0x66, (byte)0x81, (byte)0xF9};

    private byte[][] UNKNOWN = new byte[53][];
    private int[] begin_id = new int[5];
    private int[] end_id = new int[5];
    private short max_id;
    
    public MainResearchBld(InputStream in) throws Exception
    {
        // total of 932 bytes
        // 4
        int u = 0;
        readUnknown(in, u++, 4, null);

        byte[] b = new byte[4];

        for (int j = 0; j < 5; j++) // 5 * 211 = 
        {
            // 75/39
            if (j < 2)
            {
                readUnknown(in, u++, 75, TEST_MOV_ECX);
            }
            else if (j == 2)
            {
                readUnknown(in, u++, 39, TEST_MOV_ECX);
            }
            else if (j == 3)
            {
                readUnknown(in, u++, 15, TEST_MOV_ECX);
            }
            else
            {
                readUnknown(in, u++, 8, TEST_MOV_ECX);
            }
                
            // 4
            int i = 0;
            in.read(b);
            begin_id[i] = Tools.toInt(b, true);
            
            // 32/20
            if (j < 3)
            {
                readUnknown(in, u++, 32, TEST_CMP_ECX_WORD);
            }
            else if (j == 3)
            {
                readUnknown(in, u++, 20, TEST_CMP_ECX_WORD);
            }
            else
            {
                readUnknown(in, u++, 15, TEST_CMP_ECX_WORD);
            }
            
            // 4
            in.read(b);
            end_id[i++] = Tools.toInt(b, true);
            
            for (; i < 5; i++) // 4 * 24 = 96
            {
                // 3
                readUnknown(in, u++, 3, TEST_MOV_ECX);
        
                // 4
                in.read(b);
                begin_id[i] = Tools.toInt(b, true);
                
                // 13
                readUnknown(in, u++, 13, TEST_CMP_ECX_WORD);
                
                // 4
                in.read(b);
                end_id[i] = Tools.toInt(b, true);
            }
        }

        readUnknown(in, u++, 34, TEST_CMP_CX);

        b = new byte[2];
        in.read(b);
        max_id = Tools.toShort(b, true);

        readUnknown(in, u++, 29, null);
    }
    
    private void readUnknown(InputStream in, int u, int size, final byte[] test) throws Exception
    {
        UNKNOWN[u] = new byte[size];
        in.read(UNKNOWN[u]);
        testLastBytes(UNKNOWN[u], test, true);
    }
    
    private void writeUnknown(OutputStream out, int u) throws IOException
    {
        out.write(UNKNOWN[u]);
    }

    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        Vector<String> ret = new Vector<String>();
        return ret;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        return 932;
    }
    
    /**
     * @param empire empire id (0-5)
     * @param first_id building id
     * @param last_id building id
     */
    public void setIDSpan(int empire, int first_id, int last_id)
    {
        if (first_id > max_id) first_id = max_id;
        if (last_id > max_id) last_id = max_id;
        
        if (begin_id[empire] != first_id || end_id[empire] != last_id)
        {
            begin_id[empire] = first_id;
            end_id[empire] = last_id;
            this.markChanged();
        }
    }
    
    /**
     * Sets maximum id
     * @param id
     */
    public void setMaxID(short id)
    {
        if (max_id != id)
        {
            max_id = id;
            this.markChanged();
        }
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        int u = 0;
        writeUnknown(out, u++);

        for (int j = 0; j < 5; j++) // 5 * 211 = 
        {
            for (int i = 0; i < 5; i++) // 4 * 24 = 96
            {
                writeUnknown(out, u++);
        
                out.write(Tools.toByte(begin_id[i], true));
                
                writeUnknown(out, u++);
                
                out.write(Tools.toByte(end_id[i], true));
            }
        }

        writeUnknown(out, u++);

        out.write(Tools.toByte(max_id, true));

        writeUnknown(out, u++);
    }
}
