/**
 * Ultimate Editor (dev)
 * Copyright (C) 2008 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: MapRangeTechBonuses.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 9.10.2008
 */
package ue.edit.exe.trek;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.Language;
import ue.UE;
import ue.edit.CanBeEdited;
import ue.edit.FileManager;
import ue.edit.exe.InternalSegment;
import ue.edit.exe.Trek;
import ue.util.Tools;

/**
 * Contains map range tech bonus values.
 */
public class MapRangeTechBonuses extends InternalSegment
{
    public static final int SHORT_RANGE = 0;
    public static final int MEDIUM_RANGE = 1;
    public static final int LONG_RANGE = 2;

    // empire * 33 + (tech - 1) * 3 + range
    private int[] bonus = new int[165];
    
    public MapRangeTechBonuses(InputStream in) throws IOException
    {
        byte[] b = new byte[4];
        
        for (int i = 0; i < bonus.length; i++)
        {
            in.read(b);
            bonus[i] = Tools.toInt(b, true);
        }
    }
    
    public int getRangeBonus(int level, int race, int range)
    {
        return bonus[race * 33 + (level - 1) * 3 + range];
    }
    
    public void setRangeBonus(int level, int race, int range, int value)
    {
        int i = race * 33 + (level - 1) * 3 + range;
        
        if (bonus[i] == value) return;
        
        bonus[i] = value;
        this.markChanged();
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        Vector<String> result = new Vector<String>();
        
        CanBeEdited trek = UE.FILES.getFile(FileManager.P_TREK);
        
        if (trek != null)
        {
            try
            {
                BaseMapRanges base = (BaseMapRanges)trek.getInternalFile(Trek.fileBASE_MAP_RANGES, true);
                
                int[] last = new int[3];
                int range;
                String msg;
                
                for (int empire = 0; empire < 5; empire++)
                {
                    last[0] = last[1] = last[2] = Integer.MIN_VALUE;

                    for (int lvl = 1; lvl < 12; lvl++)
                    {
                        for (int r = 0; r < 3; r++)
                        {
                            range = base.getBaseRange(empire, r) + getRangeBonus(lvl, empire, r);
                            
                            if (range <= 0)
                            {
                                msg = Language.getString("MapRangeTechBonuses.0"); //$NON-NLS-1$
                                msg = msg.replace("%1", Long.toString(ADDRESS)); //$NON-NLS-1$
                                msg = msg.replace("%2", Integer.toString(r)); //$NON-NLS-1$
                                msg = msg.replace("%3", Integer.toString(empire)); //$NON-NLS-1$
                                result.add(msg.replace("%4", Integer.toString(lvl))); //$NON-NLS-1$
                            }
                            else if (range < last[r])
                            {
                                msg = Language.getString("MapRangeTechBonuses.1"); //$NON-NLS-1$
                                msg = msg.replace("%1", Long.toString(ADDRESS)); //$NON-NLS-1$
                                msg = msg.replace("%2", Integer.toString(r)); //$NON-NLS-1$
                                msg = msg.replace("%3", Integer.toString(empire)); //$NON-NLS-1$
                                msg = msg.replace("%4", Integer.toString(lvl)); //$NON-NLS-1$
                                result.add(msg.replace("%5", Integer.toString(lvl - 1))); //$NON-NLS-1$
                            }
                            
                            last[r] = range;
                        }
                    }
                }
            }
            catch (Exception e)
            {
                result.add(Language.getString("MapRangeTechBonuses.2").replace("%1", Long.toString(ADDRESS)).replace("%2", e.getLocalizedMessage())); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
            }
        }
        
        return result;
    }

    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        return bonus.length * 4;
    }

    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        for (int i = 0; i < bonus.length; i++)
        {
            out.write(Tools.toByte(bonus[i], true));
        }
    }
    
}