/**
 * Ultimate Editor
 * Copyright (C) 2009 Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: MinorRaceBldUpgradeIDs.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 10.9.2010
 */

package ue.edit.exe.trek;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;
import ue.edit.exe.InternalSegment;
import ue.util.Tools;

/**
 * Main building ids used for the building screen, likely whenever the system pop isn't native
 */
public class MinorRaceBldUpgradeIDs extends InternalSegment
{
    private short[][] food_ids = new short[5][9];
    private short[][] industry_ids = new short[5][9];
    private short[][] energy_ids = new short[5][9];
    private short[][] science_ids = new short[5][9];
    private short[][] intel_ids = new short[5][9];

    public MinorRaceBldUpgradeIDs(InputStream in) throws Exception
    {
        byte[] b2 = new byte[2];

        for (int i = 0; i < 9; i++)
        {
            for (int emp = 0; emp < 5; emp++)
            {
                in.read(b2);
                food_ids[emp][i] = Tools.toShort(b2, true);
            }
        }
        for (int i = 0; i < 9; i++)
        {
            for (int emp = 0; emp < 5; emp++)
            {
                in.read(b2);
                industry_ids[emp][i] = Tools.toShort(b2, true);
            }
        }
        for (int i = 0; i < 9; i++)
        {
            for (int emp = 0; emp < 5; emp++)
            {
                in.read(b2);
                energy_ids[emp][i] = Tools.toShort(b2, true);
            }
        }
        for (int i = 0; i < 9; i++)
        {
            for (int emp = 0; emp < 5; emp++)
            {
                in.read(b2);
                science_ids[emp][i] = Tools.toShort(b2, true);
            }
        }
        for (int i = 0; i < 9; i++)
        {
            for (int emp = 0; emp < 5; emp++)
            {
                in.read(b2);
                intel_ids[emp][i] = Tools.toShort(b2, true);
            }
        }
    }

    @Override
    public void save(OutputStream out) throws Exception
    {
        for (int i = 0; i < 9; i++)
        {
            for (int emp = 0; emp < 5; emp++)
            {
                out.write(Tools.toByte(food_ids[emp][i], true));
            }
        }
        for (int i = 0; i < 9; i++)
        {
            for (int emp = 0; emp < 5; emp++)
            {
                out.write(Tools.toByte(industry_ids[emp][i], true));
            }
        }
        for (int i = 0; i < 9; i++)
        {
            for (int emp = 0; emp < 5; emp++)
            {
                out.write(Tools.toByte(energy_ids[emp][i], true));
            }
        }
        for (int i = 0; i < 9; i++)
        {
            for (int emp = 0; emp < 5; emp++)
            {
                out.write(Tools.toByte(science_ids[emp][i], true));
            }
        }
        for (int i = 0; i < 9; i++)
        {
            for (int emp = 0; emp < 5; emp++)
            {
                out.write(Tools.toByte(intel_ids[emp][i], true));
            }
        }
    }

    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();
        return response;
    }

    @Override
    public long getSize()
    {
        return 450; // 2 * 5 * 5 * 9
    }

    /**
     *
     * @param empire 0 to 4
     * @param level 1 to 9
     * @param id
     */
    public void setFoodID(int empire, int level, short id)
    {
        level--;

        if (food_ids[empire][level] != id)
        {
//            System.out.println(empire + ": " + level + ": " + food_ids[empire][level] + " -> " + id);
            food_ids[empire][level] = id;
            this.markChanged();
        }
    }

    /**
     * 
     * @param empire 0 to 4
     * @param level 1 to 9
     * @param id
     */
    public void setIndustryID(int empire, int level, short id)
    {
        level--;

        if (industry_ids[empire][level] != id)
        {
//            System.out.println(empire + ": " + level + ": " + industry_ids[empire][level] + " -> " + id);
            industry_ids[empire][level] = id;
            this.markChanged();
        }
    }

    /**
     *
     * @param empire 0 to 4
     * @param level 1 to 9
     * @param id
     */
    public void setEnergyID(int empire, int level, short id)
    {
        level--;

        if (energy_ids[empire][level] != id)
        {
//            System.out.println(empire + ": " + level + ": " + energy_ids[empire][level] + " -> " + id);
            energy_ids[empire][level] = id;
            this.markChanged();
        }
    }

    /**
     * @param empire 0 to 4
     * @param level 1 to 9
     * @param id
     */
    public void setScienceID(int empire, int level, short id)
    {
        level--;

        if (science_ids[empire][level] != id)
        {
//            System.out.println(empire + ": " + level + ": " + science_ids[empire][level] + " -> " + id);
            science_ids[empire][level] = id;
            this.markChanged();
        }
    }

    /**
     * @param empire 0 to 4
     * @param level 1 to 9
     * @param id
     */
    public void setIntelID(int empire, int level, short id)
    {
        level--;

        if (intel_ids[empire][level] != id)
        {
//            System.out.println(empire + ": " + level + ": " + intel_ids[empire][level] + " -> " + id);
            intel_ids[empire][level] = id;
            this.markChanged();
        }
    }
}
