/**
 * Ultimate Editor
 * Copyright (C) 2009 Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: MinorRaceMainBldIDLimits.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 1.9.2010
 */

package ue.edit.exe.trek;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;
import ue.UE;
import ue.edit.CanBeEdited;
import ue.edit.FileManager;
import ue.edit.exe.InternalSegment;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.bst.Edifice;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;

/**
 *
 * @author Alan
 */
public class MinorRaceMainBldIDLimits extends InternalSegment
{
    private short[] startingIDs = new short[20];
    private short[] endingIDs = new short[20];

    public MinorRaceMainBldIDLimits(InputStream in) throws Exception
    {
        byte[] b2 = new byte[2];

        // starting ids
        for (int i = 0; i < startingIDs.length; i++)
        {
            in.read(b2);
            startingIDs[i] = Tools.toShort(b2, true);
        }

        // ending ids
        for (int i = 0; i < endingIDs.length; i++)
        {
            in.read(b2);
            endingIDs[i] = Tools.toShort(b2, true);
        }
    }

    @Override
    public void save(OutputStream out) throws Exception
    {
        for (short id : startingIDs)
        {
            out.write(Tools.toByte(id, true));
        }

        for (short id : endingIDs)
        {
            out.write(Tools.toByte(id, true));
        }
    }

    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();

        CanBeEdited file = UE.FILES.getFile(FileManager.P_STBOF);
        String msg;

        if (file != null)
        {
            try
            {
                Edifice edifice = (Edifice) file.getInternalFile(Stbof.fileEDIFICE_BST, false);

                int num = edifice.getNumberOfEntries();

                msg = "Minor race building id (%1) is out of range. (now fixed)";
                boolean error = false;

                for (short id : startingIDs)
                {
                    if (num <= id || id < 0)
                    {
                        response.add(this.getCheckIntegrityString(InternalSegment.INTEGRITY_CHECK_ERROR,
                                msg.replace("%1", Integer.toString(id))));
                        error = true;
                    }
                }

                for (short id : endingIDs)
                {
                    if (num <= id || id < 0)
                    {
                        response.add(this.getCheckIntegrityString(InternalSegment.INTEGRITY_CHECK_ERROR,
                                msg.replace("%1", Integer.toString(id))));
                        error = true;
                    }
                }

                if (error)
                {
                    edifice.updateExe();
                }
            }
            catch (Exception ex)
            {
                ex.printStackTrace();
                response.add(this.getCheckIntegrityString(INTEGRITY_CHECK_ERROR, ex.getMessage()));
            }
        }

        return response;
    }

    @Override
    public long getSize()
    {
        return 80;
    }

    public void setIndustryIDSpan(int empire, short min, short max) throws InvalidArgumentsException
    {
        int index = empire * 4;
        checkIndex(index);

        if (startingIDs[index] != min || endingIDs[index] != max)
        {
            startingIDs[index] = min;
            endingIDs[index] = max;
            this.markChanged();
        }
    }

    public void setFoodIDSpan(int empire, short min, short max) throws InvalidArgumentsException
    {
        int index = empire * 4 + 1;
        checkIndex(index);

        if (startingIDs[index] != min || endingIDs[index] != max)
        {
            startingIDs[index] = min;
            endingIDs[index] = max;
            this.markChanged();
        }
    }

    public void setEnergyIDSpan(int empire, short min, short max) throws InvalidArgumentsException
    {
        int index = empire * 4 + 2;
        checkIndex(index);

        if (startingIDs[index] != min || endingIDs[index] != max)
        {
            startingIDs[index] = min;
            endingIDs[index] = max;
            this.markChanged();
        }
    }

    public void setResearchIDSpan(int empire, short min, short max) throws InvalidArgumentsException
    {
        int index = empire * 4 + 3;
        checkIndex(index);

        if (startingIDs[index] != min || endingIDs[index] != max)
        {
            startingIDs[index] = min;
            endingIDs[index] = max;
            this.markChanged();
        }
    }

    private void checkIndex(int index) throws InvalidArgumentsException
    {
         if (index < 0 || index >= 20)
         {
             throw new InvalidArgumentsException("Invalid index calculated from empire id: %1"
                     .replace("%1", Integer.toString(index)));
         }
    }
}