/**
 * Ultimate Editor
 * Copyright (C) 2009 Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: MinorRaceSpBuildingIDs.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 26.8.2010
 */

package ue.edit.exe.trek;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;
import ue.edit.exe.InternalSegment;
import ue.exception.InvalidArgumentsException;

/**
 *
 */
public class MinorRaceSpBuildingIDs extends InternalSegment
{
    private static final byte[] test = new byte[]{(byte)0xC3};

    private MovIntToReg[] values = new MovIntToReg[30];

    public MinorRaceSpBuildingIDs(InputStream in) throws Exception
    {
        byte[] b1 = new byte[1];

        for (int i = 0; i < 30; i++)
        {
            values[i] = new MovIntToReg(MovIntToReg.EAX, in);
            
            in.read(b1);
            checkIfValid(b1, test, true);   // 1
        }
    }

    @Override
    public void save(OutputStream out) throws Exception
    {
        for (MovIntToReg se : values)
        {
            se.save(out);
            out.write(test);
        }
    }

    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();
        return response;
    }

    @Override
    public long getSize()
    {
        return 180;
    }

    public void setID(int race, int id) throws InvalidArgumentsException
    {
        if (race < 5 || race > 34)
        {
            throw new InvalidArgumentsException(
                    "Invalid minor race id: %1".replace("%1", Integer.toString(race)));
        }

        // unfortunetly the entries aren't in order
        if (race == 8)
        {
            race = 9;
        }
        else if (race == 9)
        {
            race = 8;
        }
        else if (race >= 19 && race <= 30)
        {
            race++;
        }
        else if (race == 31)
        {
            race = 19;
        }

        race = race - 5;

        values[race].setValue(id);

        if (values[race].madeChanges())
        {
            this.markChanged();
        }
    }

    public int getID(int race) throws InvalidArgumentsException
    {
        if (race < 5 || race > 34)
        {
            throw new InvalidArgumentsException(
                    "Invalid minor race id: %1".replace("%1", Integer.toString(race)));
        }

        // unfortunetly the entries aren't in order
        if (race == 8)
        {
            race = 9;
        }
        else if (race == 9)
        {
            race = 8;
        }
        else if (race >= 19 && race <= 30)
        {
            race++;
        }
        else if (race == 31)
        {
            race = 19;
        }

        race = race - 5;

        return values[race].getValue();
    }
}