/**
 * Ultimate Editor
 * Copyright (C) 2009 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: MinorRacesNumbers.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 6.6.2009
 */
package ue.edit.exe.trek;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Arrays;
import java.util.Vector;

import ue.edit.exe.InternalSegment;
import ue.util.Tools;

/**
 * @author Alan Podlesek
 */
public class MinorRacesNumbers extends InternalSegment
{
    private int[] offset;
    
    private static final byte[] code_block1 = new byte[]
                                                      {
        0x51, (byte)0x89, (byte)0xC1, (byte)0x89, (byte)0xD0, (byte)0xC1, (byte)0xE2, 0x02, (byte)0x83, (byte)0xF9,
        0x01, 0x73, 0x13, (byte)0x85, (byte)0xC9, 0x75, 0x0B, (byte)0x83, (byte)0xF8, 0x03, 0x77, 0x06, (byte)0xFF,
        (byte)0xA2
                                                      };
        
    private static final byte[] code_block2 = new byte[]
                                                       {
        0x31, (byte)0xC0, 0x59, (byte)0xC3, 0x77, 0x0B, (byte)0x83, (byte)0xF8,
        0x03, 0x77, (byte)0xF5, (byte)0xFF, (byte)0xA2
                                                       };

    private static final byte[] code_block3 = new byte[]
                                                       {
        (byte)0x83, (byte)0xF9, 0x02,
        0x75, (byte)0xEA, (byte)0x83, (byte)0xF8, 0x03, 0x77, (byte)0xE5, (byte)0xFF, (byte)0xA2
                                                       };

    private static final byte[] mov_num = new byte[]
                                                   {
        (byte)0xB8
                                                   };
    
    private static final byte[] pop_and_retn = new byte[]
                                                   {
        0x59, (byte)0xC3
                                                   };

    private int[] jmp_offset;
    
    private int[] minor_num;
    
    public MinorRacesNumbers(InputStream in) throws Exception
    {
        // offsets
        byte[] b = new byte[4];
        offset = new int[12];
        
        for (int i = 0; i < offset.length; i++)
        {
            in.read(b);
            offset[i] = Tools.toInt(b, true);
        }
        // block1
        b = new byte[code_block1.length];
        in.read(b);
        InternalSegment.checkIfValid(b, code_block1, true);
        
        // jmp_offset 1
        jmp_offset = new int[3];
        b = new byte[4];
        in.read(b);
        jmp_offset[0] = Tools.toInt(b, true);
        
        // block 2
        b = new byte[code_block2.length];
        in.read(b);
        InternalSegment.checkIfValid(b, code_block2, true);

        // jmp_offset 2
        b = new byte[4];
        in.read(b);
        jmp_offset[1] = Tools.toInt(b, true);
        
        // block 3
        b = new byte[code_block3.length];
        in.read(b);
        InternalSegment.checkIfValid(b, code_block3, true);

        // jmp_offset 3
        b = new byte[4];
        in.read(b);
        jmp_offset[2] = Tools.toInt(b, true);

        // minors
        minor_num = new int[9];
        
        for (int i = 0; i < 9; i++)
        {       // 1+4+2=7
            b = new byte[1];
            in.read(b);
            
            if (b[0] == 0)
            {
                b = new byte[6];
                in.read(b);
                InternalSegment.checkIfValid(b, new byte[6], true);
            }
            else if (InternalSegment.checkIfValid(b, mov_num, true))
            {
                b = new byte[4];
                in.read(b);
                minor_num[i] = Tools.toInt(b, true);
                
                b = new byte[2];
                in.read(b);
                InternalSegment.checkIfValid(b, pop_and_retn, true);
            }
        }
        
        // now fix
        int[] ind = new int[12];
        int[] tmp = new int[12];
        System.arraycopy(offset, 0, ind, 0, 12);
        System.arraycopy(offset, 0, tmp, 0, 12);
        ind[0] = tmp[0] = tmp[1];
        ind[4] = tmp[4] = tmp[5];
        ind[8] = tmp[8] = tmp[9];
        Arrays.sort(tmp);
        int[] new_min = new int[12];
        
        for (int j = 0; j < 3; j++)
        {
            for (int i = 1; i < 4; i++)
            {
                ind[j*4 + i] = (ind[j*4 + i] - tmp[0])/7;
                new_min[j*3 + i - 1] = minor_num[ind[j*4 + i]];
            }
        }
        
        minor_num = new_min;
    }
    
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        return null;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        return 172;
    }
    
    public int getMinorsNumber(int galsize, int setting)
    {
        return minor_num[galsize * 3 + setting];
    }

    public void setMinorsNumber(int galsize, int setting, int val)
    {
        if (val > 31)
        {
            val = 31;
        }
        else if (val < 0)
        {
            val = 0;
        }
        
        if (minor_num[galsize * 3 + setting] != val)
        {
            minor_num[galsize * 3 + setting] = val;
            this.markChanged();
        }
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        int off = (int)ADDRESS;
        
        if (off >= 1024 && off <= 1518080)
        {
            off += 4197376;
        }
        else if (off >= 1523200 && off <= 1700864)
        {
            off += 4203008;
        }
        else if (off >= 1700864 && off <= 1781248)
        {
            off += 5155840;
        }
        int fix = 0;
        
        // offsets
        for (int i = 0; i < offset.length; i++)
        {
            if (i % 4 != 0)
            {
                offset[i] = off + 109 + (i - fix) * 7;
            }
            else
            {
                fix++;
            }
            
            out.write(Tools.toByte(offset[i], true));
        }

        // block1
        out.write(code_block1);
        
        // jmp_offset 1
        out.write(Tools.toByte(jmp_offset[0], true));
        
        // block 2
        out.write(code_block2);

        // jmp_offset 2
        out.write(Tools.toByte(jmp_offset[1], true));
        
        // block 3
        out.write(code_block3);

        // jmp_offset 3
        out.write(Tools.toByte(jmp_offset[2], true));
        
        // minors
        for (int i = 0; i < 9; i++)
        {
            out.write(mov_num);
            out.write(Tools.toByte(minor_num[i], true));
            out.write(pop_and_retn);
        }
    }
}
