/**
 * Ultimate Editor
 * Copyright (C) 2009 Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: MovIntToReg.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 26.8.2010
 */

package ue.edit.exe.trek;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;
import ue.edit.exe.InternalSegment;
import ue.util.Tools;

/**
 *
 * @author Alan
 */
public class MovIntToReg extends InternalSegment
{
    public static final int EAX = 184;  // 0xB8
    public static final int EDX = 186;  // 0xBA
    public static final int ESI = 190;  // 0xBE

    private int reg;
    private int value;

    public MovIntToReg(int register, InputStream in) throws Exception
    {
        reg = register;

        // register
        byte[] b = new byte[1];
        in.read(b);
        checkIfValid(b, new byte[]{(byte)reg}, true);

        // value
        b = new byte[4];
        in.read(b);
        value = Tools.toInt(b, true);
    }

    @Override
    public void save(OutputStream out) throws Exception
    {
        out.write(reg);                             // 1
        out.write(Tools.toByte(value, true));       // 5
    }

    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();
        return response;
    }

    @Override
    public long getSize()
    {
        return 5;
    }

    public void setValue(int value)
    {
        if (this.value != value)
        {
            this.value = value;
            this.markChanged();
        }
    }

    public int getValue()
    {
        return value;
    }
}