/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: NebulaHCGraphic.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 6.4.2011
 */
package ue.edit.exe.trek;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Arrays;
import java.util.Vector;

import ue.edit.exe.InternalSegment;

/**
 * This class contains code with hardcoded neb-.tga image for the nebula object.
 */
public class NebulaHCGraphic extends InternalSegment
{
    private static final byte[] code = new byte[]
    {
        0x68, (byte)0xC4, 0x27, 0x58, 0x00, (byte)0x8D, 0x44, 0x24, 0x08,
        0x50, (byte)0xE8, (byte)0xCB, (byte)0x89, 0x01, 0x00, (byte)0x83,
        (byte)0xC4, 0x08
    };

    private boolean enabled;

    /**
     * @param in the InputStream to read from
     * @param address the location in file
     */
    public NebulaHCGraphic(InputStream in) throws Exception
    {
        byte[] b = new byte[code.length];
        byte[] test = new byte[code.length];
        Arrays.fill(test, (byte)0x90);

        int len = 0;

        while (len < code.length)
        {
            len += in.read(b, len, code.length - len);
        }

        if (Arrays.equals(b, test))
        {
            enabled = false;
        }
        else
        {
            NebulaHCGraphic.checkIfValid(b, code, true);
            enabled = true;
        }
    }

    /**
     * Used to save changes.
     * @param out the OutputStream to write the file to.
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        if (enabled)
        {
            out.write(code);
        }
        else
        {
            byte[] test = new byte[code.length];
            Arrays.fill(test, (byte)0x90);
            out.write(test);
        }
    }

    /**
     * @return uncompressed size of the file.
     */
    @Override
    public long getSize()
    {
        return code.length;
    }

    @Override
    public Vector<String> check()
    {
        return null;
    }

    /**
     * Sets ship index.
     */
    public void setHCGraphicEnabled(boolean value)
    {
        if (enabled != value)
        {
            enabled = value;
            this.markChanged();
        }
    }

    /**
     * @return
     */
    public boolean isHCGraphicEnabled()
    {
        return enabled;
    }
}
