/**
 * Ultimate Editor
 * Copyright (C) 2008 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: RaceList.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 20.9.2008
 */
package ue.edit.exe.trek;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.edit.exe.InternalSegment;
import ue.util.Tools;

/**
 * List of races stored in trek.exe
 */
public class RaceList extends InternalSegment
{
    private RaceListAddresses addresses;
    private String[] race = new String[35];
    private static final int my_offset = 5728488;
    private int data_length = 0;
    
    public RaceList(InputStream in, RaceListAddresses rla) throws Exception
    {
        addresses = rla;

        byte[] b = new byte[372];

        in.read(b);
        
        // now get strings
        int offset = 0;
        int begin = 0;
        for (int i = 0; i < 35; i++)
        {
            begin = addresses.getRaceAddress(i) - my_offset;
            
            if (begin < 0 || begin > 371)
            {
                race[i] = new String();
            }
            else
            {
                race[i] = Tools.toASCII(b, begin, b.length - begin);
                data_length += race[i].length();
            }

            addresses.setRaceAddress(i, my_offset + offset);
            
            offset += race[i].length() + 1;
            
            data_length++;
        }
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        return new Vector<String>();
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        return 372;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        for (int i = 0; i < 35; i++)
        {
            out.write(Tools.toByte(race[i], race[i].length() + 1, race[i].length()));
        }
        
        out.write(new byte[372 - data_length]);
    }
    
    /**
     * @param i race id (0 - 35)
     * @param r race string
     * @return true if the race was set
     */
    public boolean setRaceString(int i, String r)
    {
        if (i < 0 || i > 34) return false;
        
        if (race[i].equals(r)) return false;
        
        int dif = r.length() - race[i].length();
        
        if (data_length + dif > 372)
        {
            r = r.substring(0, 372 - data_length - dif);
            dif = r.length() - race[i].length();
        }
        
        data_length += dif;

        race[i] = r;
        
        for (int j = i + 1; j < 35; j++)
        {
            addresses.setRaceAddress(j, addresses.getRaceAddress(j) + dif);
        }
            
        this.markChanged();
        return true;
    }
    
    /**
     * If this returns -1 then botf won't identify the race either.
     * @param r race string
     * @return race id or -1 if not found
     */
    public int getRaceIndex(String r)
    {
        for (int i = 34; i >= 0; i--)
        {
            if (race[i].equals(r)) return i;
        }
        return -1;
    }

    /**
     * @param i race id
     * @return null or race string
     */
    public String getRaceString(int i)
    {
        if (i < 0 || i > 34) return null;
        
        return race[i];
    }
}
