/**
 * Ultimate Editor
 * Copyright (C) 2008 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: RaceListAddresses.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 30.11.2008
 */
package ue.edit.exe.trek;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Vector;

import ue.edit.exe.InternalSegment;
import ue.util.Tools;

/**
 * Reads the list of addresses that point to race names in trek.exe
 * @author Alan Podlesek
 */
public class RaceListAddresses extends InternalSegment
{
    private ArrayList<Integer> addresses;
    
    /**
     * @param in the InputStream to read from
     * @param num number of addresses to read, if < 0 read to the end of stream
     * @throws IOException
     */
    public RaceListAddresses(InputStream in, int num) throws IOException
    {
        addresses = new ArrayList<Integer>();
        
        byte[] b = new byte[4];
        
        while (num != 0 && in.available() > 0)
        {
            in.read(b);
            addresses.add(Tools.toInt(b, true));
            num--;
        }
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        return new Vector<String>();
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        return addresses.size()*4;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        for (int i = 0; i < addresses.size(); i++)
        {
            out.write(Tools.toByte(addresses.get(i), true));
        }
    }
    
    /**
     * @param index address index
     * @param address 
     */
    public void setRaceAddress(int index, int address)
    {
        addresses.set(index, address);
    }
    
    /**
     * @param index address index
     * @return
     */
    public int getRaceAddress(int index)
    {
        return addresses.get(index);
    }
}
