/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: ScreensaverModelList.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Last modified: 4.8.2008
 */
package ue.edit.exe.trek;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Vector;

import ue.edit.exe.InternalSegment;
import ue.util.Tools;

/**
 * Contains a list of ship models to be displayed in screensaver.
 */
public class ScreensaverModelList extends InternalSegment
{
    private int MAX = 71;
    private ArrayList<String> MODELS = new ArrayList<String>();
    
    /**
     * @param in the InputStream to read from
     * @param max the maximum number of entries
     */
    public ScreensaverModelList(InputStream in, int max) throws Exception
    {
        MAX = max;
        
        byte[] b = new byte[8];
        String m;
        
        for (int i = 0; i < MAX && in.available() > 0; i++)
        {
            in.read(b);
            m = Tools.toASCII(b).toLowerCase();
            
            if (!MODELS.contains(m)) MODELS.add(m);
        }
    }
    
    /**
     * 
     */
    public boolean set(String[] models)
    {
        if (models.length > MAX && MAX > 0) return false;
        
        MODELS.clear();
        
        for (String m : models)
        {
            if (!MODELS.contains(m)) MODELS.add(m);
        }

        this.markChanged();
        return true;
    }
    
    public String get(int i)
    {
        return MODELS.get(i);
    }
    
    /**
     * 
     * @return the maximum allowed entries
     */
    public int getMax()
    {
        return MAX;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();
        
        
        
        return response;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        if (MAX >= 0 ) return MAX*8;
        return MODELS.size()*8;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        if (MODELS.size() < 1)
        {
            // write empty strings
            byte[] b = new byte[8];

            for (int i = 0; i < MAX; i++)
            {
                out.write(b);
            }
        }
        else
        {
            int index = 0;

            for (int i = 0; i < MAX; i++)
            {
                out.write(Tools.toByte(MODELS.get(index++), 8, 7));

                if (index >= MODELS.size())
                {
                    index = 0;
                }
            }
        }
    }

    /**
     * @return
     */
    public int getNumberofEntries()
    {
        return MODELS.size();
    }

    /**
     * @param string
     */
    public void add(String m)
    {
        if (!MODELS.contains(m)) MODELS.add(m);
        this.markChanged();
    }

    /**
     * @param string
     */
    public boolean remove(String string)
    {
        if (MODELS.size() < 2) return false;
        
        this.markChanged();
        return MODELS.remove(string);
    }
    
}
