/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: SelectPlasmaHobFile.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 12.12.2011
 */
package ue.edit.exe.trek;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.edit.exe.InternalSegment;
import ue.exception.InvalidArgumentsException;

/**
 * This class contains alien id.
 */
public class SelectTorpedoHobFile extends InternalSegment
{
    private boolean extendedShiplistPatch;

    private static byte[] testOriginal = new byte[]{
        0x66, (byte)0x8B, (byte)0x84, 0x02, (byte)0xDC, 0x01, 0x00, 0x00, 0x25, (byte)0xFF,
        (byte)0xFF, 0x00, 0x00, (byte)0x89, (byte)0x84, 0x24, (byte)0xCC, 0x01, 0x00, 0x00,
        (byte)0x83, (byte)0xF8, 0x05, 0x7D, 0x47, 0x31, (byte)0xC0, (byte)0x8A, (byte)0x84,
        0x24, (byte)0xCC, 0x01, 0x00, 0x00, (byte)0xE8, 0x53, 0x4B, (byte)0xFA, (byte)0xFF};
    private static byte[] testExtended = new byte[]{
        0x03, (byte)0xC2, (byte)0x8B, (byte)0xD4, (byte)0xE8, 0x51, 0x28, 0x09, 0x00, (byte)0x8A,
        (byte)0x84, 0x24, 0x1B, 0x01, 0x00, 0x00, (byte)0x90, (byte)0x90, (byte)0x90,
        (byte)0x90, (byte)0x90, (byte)0x90, (byte)0x90, (byte)0x90, (byte)0x90, (byte)0x90,
        (byte)0x90, (byte)0x90, (byte)0x90, (byte)0x90, (byte)0x90, (byte)0x90, (byte)0x90,
        (byte)0x90, (byte)0x90, (byte)0x90, (byte)0x90, (byte)0x90, (byte)0x90};
    
    /**
     * @param in the InputStream to read from
     * @param address the location in file
     */
    public SelectTorpedoHobFile(InputStream in) throws Exception
    {
        //read num
        byte[] b = new byte[testOriginal.length];
        in.read(b);

        if (!checkIfValid(b, testOriginal, false))
        {
            extendedShiplistPatch = checkIfValid(b, testExtended, true);
        }
        else
        {
            extendedShiplistPatch = false;
        }
    }
    
    /**
     * Used to save changes.
     * @param out the OutputStream to write the file to.
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        if (extendedShiplistPatch)
        {
            out.write(testExtended);
        }
        else
        {
            out.write(testOriginal);
        }
    }
    
    /**
     * @return uncompressed size of the file.
     */
    @Override
    public long getSize()
    {
        return testOriginal.length;
    }
    
    @Override
    public Vector<String> check()
    {
        return null;
    }
    
    public void setPatched(boolean patch)
    {
        if (extendedShiplistPatch != patch)
        {
            extendedShiplistPatch = patch;
            this.markChanged();
        }
    }

    /**
     * @return
     */
    public boolean isPatched()
    {
        return extendedShiplistPatch;
    }
}
