/**
 * Ultimate Editor (dev)
 * Copyright (C) 2008 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: ShipLoadLimit.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 10.9.2008
 */
package ue.edit.exe.trek;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.UE;
import ue.edit.FileManager;
import ue.edit.exe.InternalSegment;
import ue.edit.exe.Trek;

/**
 * The instructions used to load ships are limited to a number of ships. Set limit here.
 * @author Alan Podlesek
 */
public class ShipLoadLimit extends InternalSegment
{
    private int[] DATA = new int[5];
    
    public ShipLoadLimit(InputStream in) throws Exception
    {
        for (int i = 0; i < DATA.length; i++)
        {
            DATA[i] = in.read();
        }
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        return new Vector<String>();
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        return 5;
    }
    
    public void setID(int id)
    {
        if (id < 0 || id > 255)
        {
            return;
        }
        DATA[2] = id;
        this.markChanged();
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        // fix compare instruction
        DATA[0] = 0x80; // compare the low byte instead of the whole register
        
        // fix jump instruction
        // jge -> jz
        // jl -> jnz
        if (DATA[3] == 0x7C || DATA[3] == 0x75) // jl
        {
            DATA[1] = 0xFA;
            DATA[3] = 0x75;
            DATA[4] = 0xA2;
        }
        else
        { // jge
            DATA[1] = 0xF8;
            DATA[3] = 0x74;
            DATA[4] = 0x5E;
        }
        
        // cheating
        if (UE.FILES.getFile(FileManager.P_TREK) != null)
        {
            Trek t = (Trek)UE.FILES.getFile(FileManager.P_TREK);
            
            try
            {
                Shipnum sn = (Shipnum)t.getInternalFile(Trek.fileSHIPNUM1, true);
                DATA[2] = sn.getNum();
            }
            catch (Exception e)
            {
                
            }
        }
        
        for (int i = 0; i < DATA.length; i++)
        {
            out.write(DATA[i]);
        }
    }
    
}
