/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: Shipnum.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Last modified: 23.7.2008
 */
package ue.edit.exe.trek;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.Language;
import ue.UE;
import ue.edit.FileManager;
import ue.edit.exe.InternalSegment;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.sst.ShipList;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;

/**
 * A short value containing the number of ships in shiplist.sst
 */
public class Shipnum extends InternalSegment
{
    private static byte[] TEST_JZ = new byte[]{0x74};
    
    private short SHIP_NUMBER;
    
    /**
     * @param in the InputStream to read from
     * @param address location in file
     */
    public Shipnum(InputStream in) throws Exception
    {
        //read num
        byte[] b = new byte[2];
        in.read(b);
        this.SHIP_NUMBER = Tools.toShort(b, true);
        b = new byte[1];
        in.read(b);
        checkIfValid(b, TEST_JZ, true);
    }
    
    /**Used to save changes.
     * @param out the OutputStream to write the file to.
     */
    @Override
    public void save(OutputStream out) throws Exception
    { 
        out.write(Tools.toByte(this.SHIP_NUMBER, true));
        out.write(TEST_JZ);
    }
    
    /**
     * @return uncompressed size of the file.
     */
    @Override
    public long getSize()
    {
        return 3;
    }
    
    @Override
    public Vector<String> check(){ 
        Vector<String> response = new Vector<String>();
        
        // msg = Error at %1: Number of ships is less than %2.
        String msg = Language.getString("Shipnum.0"); //$NON-NLS-1$
        msg = msg.replace("%1", Long.toString(this.ADDRESS)); //$NON-NLS-1$
        msg = msg.replace("%2", Integer.toString(6)); //$NON-NLS-1$
        
        if (this.SHIP_NUMBER < 6) {
            response.add(msg);
        }
        
        // check with shiplist.sst
        if (UE.FILES.getFile(FileManager.P_STBOF) != null)
        {
            Stbof st = (Stbof)UE.FILES.getFile(FileManager.P_STBOF);
            
            try
            {
                ShipList sl = (ShipList)st.getInternalFile(Stbof.fileSHIPLIST_SST, true);
                
                if (this.SHIP_NUMBER != sl.getNumberOfEntries())
                {
                    // msg = Error at %1: Number of registered ships is %2 should be %3.
                    msg = Language.getString("Shipnum.2"); //$NON-NLS-1$
                    msg = msg.replace("%1", Long.toString(this.ADDRESS)); //$NON-NLS-1$
                    msg = msg.replace("%2", Integer.toString(this.SHIP_NUMBER)); //$NON-NLS-1$
                    msg = msg.replace("%3", Integer.toString(sl.getNumberOfEntries())); //$NON-NLS-1$

                    response.add(msg);
                }
            }
            catch (Exception e)
            {
                
            }
        }
        
        return response;
    }

    /**
     * Sets ship number.
     */
    public void setNum(short num) throws InvalidArgumentsException
    {
        if (num < 6)
        {
            // msg = Number of ships is less than %1.
            String msg = Language.getString("Shipnum.1"); //$NON-NLS-1$
            msg.replace("%1", Integer.toString(6)); //$NON-NLS-1$
            
            throw new InvalidArgumentsException(msg);
        }
        this.SHIP_NUMBER = num;
        this.markChanged();
    }

    /**
     * @return
     */
    public int getNum()
    {
        return SHIP_NUMBER;
    }
}
