/**
 * Ultimate Editor
 * Copyright (C) 2008 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: SpecialShips.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 23.11.2008
 */
package ue.edit.exe.trek;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;

import ue.UE;
import ue.edit.CanBeEdited;
import ue.edit.FileManager;
import ue.edit.exe.InternalSegment;
import ue.edit.exe.Trek;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.rst.Race;
import ue.edit.res.stbof.sst.ShipList;
import ue.edit.res.stbof.sst.ShipTech;
import ue.util.ID;
import ue.util.Tools;

/**
 * Reads/writes all of the trek.exe's function responsible for special ships.
 * @author Alan Podlesek
 */
public class SpecialShips extends InternalSegment
{
    private int BUILDING;       // integer
    private int SP_REG[] = new int[5];        // byte
    private int SP[] = new int[5];            // byte
    private int CALL_ADDRESS[] = new int[6];
    // settings
    private boolean SPECIAL_SHIPS = true;
    private boolean MINOR_SHIPS = false;
    private boolean MINOR_SHIPS_SIMPLE = false;
    
    private int STARBASE_CLASS;
    
    private int max_slots = 0;
    
    /**
     * @param in stream to read from
     * @throws Exception
     */
    public SpecialShips(InputStream in) throws Exception
    {
        // expected code
        final byte[] PUSH_BLOCK = new byte[]{           // 5 bytes
                0x53, 0x51, 0x52, 0x56, 0x57
            };
        final byte[] PUSH_EBP = new byte[]{             // 1 byte
                0x55
            };
        final byte[] PREPARE_STACK = new byte[]{        // 6
            (byte)0x81, (byte)0xEC, 0x30, 0x01, 0x00,
            0x00
        };
        
        // read and test input
        byte[] b;
        
        testInput(in, PUSH_BLOCK, true, null);          // 5    9
        
        if ( (b = testInput(in, PUSH_EBP, false, null)) == null)
        {
            SPECIAL_SHIPS = false;                      // 1    10
            MINOR_SHIPS = true;
            loadMinorShipsMod(in);                      // 234  244
        }
        else
        {
            MINOR_SHIPS = false;                        
            
            testInput(in, PREPARE_STACK, true, b);      // 6    15
            
            loadNormal(in);                             // 229  244
        }
    }
    
    /**
     * @param in stream to read from
     * @param expected expected data
     * @param throw_exception if an exception should be thrown on data mismatch
     * @param leftovers data to be put in front of the stream data
     * @return read data on mismatch or null
     * @throws Exception
     */
    private byte[] testInput(InputStream in, byte[] expected, boolean throw_exception, byte[] leftovers) throws Exception
    {
        byte[] b = new byte[expected.length];
        
        if (leftovers != null)
        {
            System.arraycopy(leftovers, 0, b, 0, leftovers.length);
            in.read(b, leftovers.length, b.length - leftovers.length);
        }
        else
        {
            in.read(b);
        }
        
        if (checkIfValid(b, expected, throw_exception))
        {
            return null; // input valid no mismathed bytes to return
        }
        else
        {
            return b; // return stuff we read
        }
    }
    
    /**
     * @param in
     * @throws Exception 
     */
    private void loadMinorShipsMod(InputStream in) throws Exception    // 
    {
        final byte[] PREPARE_STACK = new byte[]{        // 6
                (byte)0x81, (byte)0xEC, 0x30, 0x01, 0x00,
                0x00
        };
        final byte[] SET_EBP_TO_MINORS = new byte[]{    // 5
            (byte)0xBD, 0x44, 0x00, 0x00, 0x00
        };
        final byte[] PREPARE_ESI = new byte[]{          // 27 bytes
            (byte)0x8B, 0x3D, (byte)0xA8, 0x5D, 0x59,
            0x00, (byte)0x8B, 0x77, 0x1A, (byte)0xC1,
            (byte)0xFE, 0x10, 0x69, (byte)0xF6, 0x28,
            0x03, 0x00, 0x00, (byte)0xA1, (byte)0xC8,
            0x36, 0x5A, 0x00, 0x01, (byte)0xC6, 0x31,
            (byte)0xC0
        };
        final byte[] READ_SYS_EBP = new byte[]{         // 3
            (byte)0x8A, 0x04, 0x2E
        };
        final byte[] CHECK_AND_SET_EBP_FOR_RERUN = new byte[]{ // 8
            (byte)0x83, (byte)0xFD, 0x44, 0x75, 0x08,
            (byte)0x83, (byte)0xC5, 0x08
        };
        final byte[] JMP = new byte[]{                  // 1
            (byte)0xE9
        };
        final byte[] CLEAR_STACK = new byte[]{          // 6
                (byte) 0x81, (byte) 0xC4, 0x30, 0x01,
                0x00, 0x00
        };
        final byte[] POP_EBP = new byte[]{              // 1
                0x5D
        };
        final byte[] POP_BLOCK = new byte[]{            // 6
                0x5F, 0x5E, 0x5A, 0x59, 0x5B,
                (byte) 0xC3
        };
        final byte[] BLANK_BLOCK_END = new byte[]{      // 1
                0x00
        };

        testInput(in, PREPARE_STACK, true, null);       // 6    6
        
        testInput(in, SET_EBP_TO_MINORS, true, null);   // 5    11
        
        testInput(in, PREPARE_ESI, true, null);         // 27   38
        
        testInput(in, READ_SYS_EBP, true, null);        // 3    41
        
        loadMRMRaceTest(in);                            // 35   78

        loadShipLoadAndBasicChecks(in);                 // 51   127
        
        loadAddShipToList(in);                          // 65   192
        
        testInput(in, CHECK_AND_SET_EBP_FOR_RERUN, true, null);        // 8    200

        testInput(in, JMP, true, null);                 // 1    201
        in.skip(4);                                     // 4    205

        testInput(in, CLEAR_STACK, true, null);         // 6    211
        
        testInput(in, POP_EBP, true, null);             // 1    212

        testInput(in, POP_BLOCK, true, null);           // 6    218

        testInput(in, BLANK_BLOCK_END, true, null);     // 16   234
    }

    private void loadMRMRaceTest(InputStream in) throws Exception       // 35
    {
        final byte[] IS_EMPIRE = new byte[]{            // 2
                0x3C, 0x04
        };
        final byte[] JA = new byte[]{                   // 2
                0x77, 0x0A
        };
        final byte[] IS_SYS_OWNER = new byte[]{         // 3
                (byte)0x83, (byte)0xFD, 0x4C
        };
        final byte[] JZ = new byte[]{                   // 2
                0x74, 0x18
        };
        final byte[] JMP = new byte[]{                  // 1
                (byte)0xE9
        };
        final byte[] PUSH_EBP = new byte[]{             // 1 byte
                0x55
            };
        final byte[] GET_SUBJUDGED_STATUS = new byte[]{ // 12
                (byte)0xBD, 0x10, 0x20, 0x5A, 0x00,
                0x6B, (byte)0xC0, 0x40, (byte)0x8B,
                0x44, 0x28, 0x3C
        };
        final byte[] POP_EBP = new byte[]{              // 1
                0x5D
        };
        final byte[] TEST_EAX = new byte[]{             // 2
                (byte)0x83, (byte)0xF8, 0x01
        };
        final byte[] JZ2 = new byte[]{                  // 2
                0x74, (byte)0xE8
        };
        final byte[] READ_SYS_EBP = new byte[]{         // 3
                (byte)0x8A, 0x04, 0x2E
        };
        
        testInput(in, IS_EMPIRE, true, null);           // 2    2

        testInput(in, JA, true, null);                  // 2    4
        
        testInput(in, IS_SYS_OWNER, true, null);        // 3    7

        testInput(in, JZ, true, null);                  // 2    9
        
        testInput(in, JMP, true, null);                 // 1    10
        in.skip(4);                                     // 4    14
        
        testInput(in, PUSH_EBP, true, null);            // 1    15
        
        testInput(in, GET_SUBJUDGED_STATUS, true, null);// 12   27

        testInput(in, POP_EBP, true, null);             // 1    25

        testInput(in, TEST_EAX, true, null);            // 2    27

        testInput(in, JZ2, true, null);                 // 2    29
        
        testInput(in, READ_SYS_EBP, true, null);        // 3    32
    }
    
    /**
     * @param in
     * @throws Exception 
     */
    private void loadNormal(InputStream in) throws Exception    // 229
    {
        final byte[] PREPARE_ESI = new byte[]{          // 27 bytes
            (byte)0x8B, 0x3D, (byte)0xA8, 0x5D, 0x59,
            0x00, (byte)0x8B, 0x77, 0x1A, (byte)0xC1,
            (byte)0xFE, 0x10, 0x69, (byte)0xF6, 0x28,
            0x03, 0x00, 0x00, (byte)0xA1, (byte)0xC8,
            0x36, 0x5A, 0x00, 0x01, (byte)0xC6, 0x31,
            (byte)0xC0
        };
        final byte[] READ_SYS_OWNER = new byte[]{       // 3
                (byte)0x8A, 0x46, 0x4C
        };
        final byte[] READ_SYS_RACE = new byte[]{        // 3
            (byte)0x8A, 0x46, 0x44
        };
        final byte[] CLEAR_STACK = new byte[]{          // 6
            (byte) 0x81, (byte) 0xC4, 0x30, 0x01,
            0x00, 0x00
        };
        final byte[] POP_BLOCK = new byte[]{            // 6
            0x5F, 0x5E, 0x5A, 0x59, 0x5B,
            (byte) 0xC3
        };
        
        byte[] b;
        
        testInput(in, PREPARE_ESI, true, null);         // 27   27
        
        if ( (b = testInput(in, READ_SYS_RACE, false, null)) == null)
        {
            // minor ships are turned on, but player can't build empire ships in minor race system
            // UE will fix it on save
            // I'm not very fond of adding yet another variation,
            // but if there's demand this behaviour could be changed
            MINOR_SHIPS_SIMPLE = true;                         // 3    30
        }
        else
        {
            testInput(in, READ_SYS_OWNER, true, b);     // 3    30
        }
        
        loadShipLoadAndBasicChecks(in);                 // 51   81
        
        loadRaceAndSP1Check(in);                        // 12   93
        
        loadSpecialBuildingCheck(in);                   // 19   112
        
        loadAddShipToList(in);                          // 65   177

        testInput(in, CLEAR_STACK, true, null);         // 6    183

        testInput(in, POP_BLOCK, true, null);           // 6    189

        loadSpecialShipChecks(in);                      // 26   215
    }
    
    private void loadShipLoadAndBasicChecks(InputStream in) throws Exception // 51
    {
        final byte[] CALL = new byte[]{                 // 1
            (byte)0xE8
        };
        final byte[] TEST_CALL_RESULTS = new byte[]{    // 4
            (byte)0x89, (byte)0xC1, (byte)0x85,
            (byte)0xC0
        };
        final byte[] JZ = new byte[]{                   // 2
            (byte)0x0F, (byte)0x84
        };
        final byte[] UNKNOWN = new byte[]{              // 10
            0x66, (byte)0x8B, 0x59, 0x04, 0x31,
            (byte)0xC0, (byte)0x89, (byte)0xE2,
            (byte)0x88, (byte)0xD8
        };
        final byte[] CHECK_IF_OUTPOST = new byte[]{     // 8
            (byte)0x8B, 0x54, 0x24, 0x68, 0x66,
            (byte)0x83, (byte)0xFA, 0x06
        };
        final byte[] SJZ = new byte[]{                  // 1
                0x74
        };
        final byte[] CHECK_IF_STARBASE = new byte[]{    // 3
                0x66, (byte)0x83, (byte)0xFA
        };

        byte[] b = new byte[4];
        
        testInput(in, CALL, true, null);                // 1    1
        
        in.read(b);
        CALL_ADDRESS[0] = Tools.toInt(b, true);         // 4    5
        
        testInput(in, CALL, true, null);                // 1    6
    
        in.read(b);
        CALL_ADDRESS[1] = Tools.toInt(b, true);         // 4    10
        
        testInput(in, TEST_CALL_RESULTS, true, null);   // 4    14
    
        testInput(in, JZ, true, null);                  // 2    16

        in.skip(4);     // skip address                 // 4    20
        
        testInput(in, UNKNOWN, true, null);             // 10   30
        
        testInput(in, CALL, true, null);                // 1    31
    
        in.read(b);
        CALL_ADDRESS[2] = Tools.toInt(b, true);         // 4    35
        
        testInput(in, CHECK_IF_OUTPOST, true, null);    // 8    43
        
        testInput(in, SJZ, true, null);                 // 1    44
        
        in.skip(1); // skip address                     // 1    45
        
        testInput(in, CHECK_IF_STARBASE, true, null);   // 4    49
        
        STARBASE_CLASS = in.read();  // or some other class
        
        testInput(in, SJZ, true, null);                 // 1    50
        
        in.skip(1); // skip address                     // 1    51
    }

    private void loadRaceAndSP1Check(InputStream in) throws Exception // 12
    {
        final byte[] COMPARE_SP = new byte[]{           // 2
                0x66, (byte) 0x83
        };

        in.skip(6);     // skip race check - don't care // 6    6
        
        byte[] b;
        
        if ( (b = testInput(in, COMPARE_SP, false, null)) != null)
        {
            SPECIAL_SHIPS = false;                      // 3    9
            in.skip(1); // reg
        }
        else
        {
            SP_REG[0] = in.read();                      // 3    9

            if (SP_REG[0] == 0xFB)
            {
                max_slots++;
            }
        }
        
        SP[0] = in.read();                              // 1    10

        b = new byte[2];
        in.read(b);                                     // 2    12

        if (b[0] != 0x75)
        {
            SPECIAL_SHIPS = false;
        }
    }
    
    private void loadSpecialBuildingCheck(InputStream in) throws Exception // 19
    {
        final byte[] TEST_SP_BUILDING_1 = new byte[]{   // 3
            0x31, (byte) 0xC0, (byte) 0xBA
        };
        final byte[] TEST_SP_BUILDING_2 = new byte[]{   // 3
            (byte) 0x8A, 0x46, 0x4C
        };
        final byte[] TEST_SP_BUILDING_3 = new byte[]{   // 2
            (byte) 0x85, (byte) 0xC0
        };
        final byte[] CALL = new byte[]{                 // 1
                (byte)0xE8
        };
        final byte[] SJZ = new byte[]{                  // 1
                0x74
        };

        testInput(in, TEST_SP_BUILDING_1, true, null);  // 3    3

        byte[] b_int = new byte[4];
        in.read(b_int);                                 // 4    7
        BUILDING = Tools.toInt(b_int, true);
        
        testInput(in, TEST_SP_BUILDING_2, true, null);  // 3    10

        testInput(in, CALL, true, null);                // 1    11

        byte[] b = new byte[4];
        in.read(b);
        CALL_ADDRESS[3] = Tools.toInt(b, true);         // 4    15

        testInput(in, TEST_SP_BUILDING_3, true, null);  // 2    17

        testInput(in, SJZ, true, null);                 // 1    18
        
        in.skip(1); // skip address                     // 1    19
    }
    
    private void loadAddShipToList(InputStream in) throws Exception // 65
    {
        final byte[] ADD_SHIP_TO_LIST_1 = new byte[]{   // 47
            (byte) 0xBA, 0x03, 0x00, 0x00, 0x00, 0x31,
            (byte) 0xC0, 0x66, (byte) 0x89, (byte) 0x9C,
            0x24, 0x28, 0x01, 0x00, 0x00, 0x66,
            (byte) 0x8B, (byte) 0x84, 0x24, 0x1C, 0x01,
            0x00, 0x00, (byte) 0x89, (byte) 0x94, 0x24,
            0x20, 0x01, 0x00, 0x00, (byte) 0x89,
            (byte) 0x84, 0x24, 0x2C, 0x01, 0x00, 0x00,
            (byte) 0x8D, (byte) 0x94, 0x24, 0x20, 0x01,
            0x00, 0x00, (byte) 0x8B, 0x47, 0x38
        };
        final byte[] ADD_SHIP_TO_LIST_2 = new byte[]{   // 2
            (byte) 0x89, (byte) 0xC8
        };
        final byte[] ADD_SHIP_TO_LIST_3 = new byte[]{   // 4
            (byte) 0x89, (byte) 0xC1, (byte) 0x85,
            (byte) 0xC0
        };
        byte[] SJNZ = new byte[]{                       // 1
                0x75
        };
        final byte[] CALL = new byte[]{                 // 1
                (byte)0xE8
        };

        testInput(in, ADD_SHIP_TO_LIST_1, true, null);  // 47   47
        
        testInput(in, CALL, true, null);                // 1    48
    
        byte[] b = new byte[4];
        in.read(b);
        CALL_ADDRESS[4] = Tools.toInt(b, true);         // 4    52
    
        testInput(in, ADD_SHIP_TO_LIST_2, true, null);  // 2    54
    
        testInput(in, CALL, true, null);                // 1    55
        
        in.read(b);
        CALL_ADDRESS[5] = Tools.toInt(b, true);         // 4    59
    
        testInput(in, ADD_SHIP_TO_LIST_3, true, null);  // 4    63
    
        testInput(in, SJNZ, true, null);                // 1    64
        
        in.skip(1); // skip address                     // 1    65
    }
    
    private void loadSpecialShipChecks(InputStream in) throws Exception // 26
    {
        final byte[] COMPARE_SP = new byte[]{           // 2
                0x66, (byte) 0x83
        };
        final byte[] SJMP = new byte[]{                 // 1
                (byte)0xEB
        };

        for (int i = 1; i < 5; i++)             // total:  24   24
        {
            testInput(in, COMPARE_SP, true, null);      // 2    2

            SP_REG[i] = in.read();                      // 1    3
        
            if (SP_REG[i] == 0xFB)
            {
                max_slots++;
            }

            SP[i] = in.read();                          // 1    4

            in.skip(2);  // jump                        // 2    6
        }
        
        testInput(in, SJMP, true, null);                // 1    25

        in.skip(1); // address                          // 1    26
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        Vector<String> ret = new Vector<String>();

        if (MINOR_SHIPS_SIMPLE)
        {
            CanBeEdited st = UE.FILES.getFile(FileManager.P_STBOF);

            if (st != null && st instanceof Stbof)
            {
                Stbof stbof = (Stbof)st;

                try
                {
                    Race races = (Race) stbof.getInternalFile(Stbof.fileRACE_RST, true);
                    ShipList ships = (ShipList)stbof.getInternalFile(Stbof.fileSHIPLIST_SST, true);
                    ShipTech tech = (ShipTech)stbof.getInternalFile(Stbof.fileSHIPTECH_SST, true);

                    String msg; ID[] ids; int num;

                    msg = "%1 have less than two ships available at tech. level 1.";

                    for (int r = 0; r < races.getNumberOfEntries(); r++)
                    {
                        num = 0;
                        ids = ships.getIDs(r);

                        for (ID id : ids)
                        {
                            if (Tools.getMax(tech.getTechLvls(id.hashCode())) == 1)
                            {
                                num++;

                                if (num >= 2)
                                {
                                    break;
                                }
                            }
                        }

                        if (num < 2)
                        {

                            ret.add(getCheckIntegrityString(InternalSegment.INTEGRITY_CHECK_ERROR,
                                msg.replace("%1", races.getName(r))));
                        }
                    }
                }
                catch (Exception ex)
                {
                    ex.printStackTrace();

                    ret.add(getCheckIntegrityString(InternalSegment.INTEGRITY_CHECK_ERROR,
                            ex.getMessage()));
                }
            }
        }

        return ret;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        return 226;
    }
    
    /**
     * 
     * @param b if true special ships are enabled
     */
    public void setSPEnabled(boolean b)
    {
        if (areSPEnabled() != b)
        {
            this.markChanged();
        }
        
        SPECIAL_SHIPS = b;
        
        if (SPECIAL_SHIPS)
        {
            MINOR_SHIPS_SIMPLE = MINOR_SHIPS || MINOR_SHIPS_SIMPLE;
            MINOR_SHIPS = false;
        }
    }
    
    /**
     * @param id building id of the building that allows building special ships
     */
    public void setSPBuilding(int id)
    {
        if (id != BUILDING)
        {
            if (id >= 0 && id <= 255)
            {
                BUILDING = id;
                this.markChanged();
            }
        }
    }
    
    /**
     * 
     * @return building id
     */
    public int getSPBuilding()
    {
        return BUILDING;
    }

    /**
     * @return true if special ships are enabled
     */
    public boolean areSPEnabled()
    {
        return SPECIAL_SHIPS;
    }
    
    public HashSet<Integer> getShipIDs()
    {
        HashSet<Integer> ret = new HashSet<Integer>();
        
        if (areSPEnabled())
        {
            for (int i = 0; i < 5; i++)
            {
                if (SP_REG[i] == 0xFB) ret.add(SP[i]);
            }
        }
        
        return ret;
    }
    
    /**
     * 
     * @param ids ship ids of special ships (max 5)
     */
    public void setShipIDs(HashSet<Integer> ids)
    {
        setSPEnabled(ids.size() > 0 && ids.size() <= max_slots);
        
        if (areSPEnabled())
        {
            int[] ships = new int[ids.size()];
            
            int i = 0;
            
            Iterator<Integer> it = ids.iterator();
            
            while (it.hasNext())
            {
                ships[i++] = it.next();
            }
            
            i = 0;
            
            for (int slot = 0; slot < 5; slot++)
            {
                if (SP_REG[slot] == 0xFB && SP[slot] != ships[i])
                {
                    this.markChanged();
                    SP[slot] = ships[i];
                    
                    if (i < ships.length - 1)
                    {
                        i++;
                    }
                }
            }
        }
    }
    
    public int getSPIdSlotsNumber()
    {
        return max_slots;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception      // 244
    {
        final byte[] PUSH_BLOCK = new byte[]{           // 5 bytes
                0x53, 0x51, 0x52, 0x56, 0x57
            };
        
        updateAddresses();
        
        out.write(PUSH_BLOCK);
        
        if ( MINOR_SHIPS )
        {
            saveAsUsingMinorShipMod(out);
        }
        else
        {
            saveNormal(out);
        }
    }

    /**
     * @param out
     * @throws IOException 
     */
    private void saveNormal(OutputStream out) throws IOException
    {
        final byte[] PREPARE_STACK = new byte[]{        // 6
                (byte)0x81, (byte)0xEC, 0x30, 0x01,
                0x00, 0x00
            };
        final byte[] PREPARE_ESI = new byte[]{          // 27 bytes
           (byte)0x8B, 0x3D, (byte)0xA8, 0x5D, 0x59,
           0x00, (byte)0x8B, 0x77, 0x1A, (byte)0xC1,
           (byte)0xFE, 0x10, 0x69, (byte)0xF6, 0x28,
           0x03, 0x00, 0x00, (byte)0xA1, (byte)0xC8,
           0x36, 0x5A, 0x00, 0x01, (byte)0xC6, 0x31,
           (byte)0xC0
        };
        final byte[] READ_SYS_OWNER = new byte[]{       // 3
                (byte)0x8A, 0x46, 0x4C
        };
        final byte[] CLEAR_STACK = new byte[]{          // 6
            (byte) 0x81, (byte) 0xC4, 0x30, 0x01,
            0x00, 0x00
        };
        final byte[] POP_BLOCK = new byte[]{            // 6
            0x5F, 0x5E, 0x5A, 0x59, 0x5B,
            (byte) 0xC3
        };
        final byte[] READ_SYS_RACE = new byte[]{        // 3
                (byte)0x8A, 0x46, 0x44
        };

        out.write(PREPARE_STACK);       // 6
        
        out.write(PREPARE_ESI);         // 27

        if (MINOR_SHIPS_SIMPLE)
        {
            out.write(READ_SYS_RACE);      // 3
        }
        else
        {
            out.write(READ_SYS_OWNER);      // 3
        }
        
        saveShipLoadAndBasicChecks(out, 134, 147);
        
        saveRaceAndSP1Check(out, 108, 31);
        
        saveSpecialBuildingCheck(out, 33);
        
        saveAddShipToList(out, -62);

        out.write(CLEAR_STACK);
        
        out.write(POP_BLOCK);
        
        saveSpecialShipChecks(out, -129, -96);
    }
    
    private void saveShipLoadAndBasicChecks(OutputStream out, int jump_to_next_address, int jump_to_exit_address) throws IOException // 51
    {
        final byte[] CALL = new byte[]{                 // 1
                (byte)0xE8
        };
        final byte[] TEST_CALL_RESULTS = new byte[]{    // 4
                (byte)0x89, (byte)0xC1, (byte)0x85,
                (byte)0xC0
        };
        final byte[] JZ = new byte[]{                   // 2
                (byte)0x0F, (byte)0x84
        };
        final byte[] UNKNOWN = new byte[]{              // 10
                0x66, (byte)0x8B, 0x59, 0x04, 0x31,
                (byte)0xC0, (byte)0x89, (byte)0xE2,
                (byte)0x88, (byte)0xD8
        };
        final byte[] CHECK_IF_OUTPOST = new byte[]{     // 8
                (byte)0x8B, 0x54, 0x24, 0x68, 0x66,
                (byte)0x83, (byte)0xFA, 0x06
        };
        final byte[] SJZ = new byte[]{                  // 1
                0x74
        };
        final byte[] CHECK_IF_STARBASE = new byte[]{    // 3
                0x66, (byte)0x83, (byte)0xFA
        };

        out.write(CALL);
        out.write(Tools.toByte(CALL_ADDRESS[0], true)); //     5        5
        
        out.write(CALL);
        out.write(Tools.toByte(CALL_ADDRESS[1], true)); //      5       10

        out.write(TEST_CALL_RESULTS);                   //      4       14
        
        out.write(JZ);
        out.write(Tools.toByte(jump_to_exit_address - 20, true));             //      6       20
        
        out.write(UNKNOWN);                             //      10      30
        
        out.write(CALL);
        out.write(Tools.toByte(CALL_ADDRESS[2], true)); //      5       35
        
        out.write(CHECK_IF_OUTPOST);                    //      8       43

        out.write(SJZ);
        out.write(jump_to_next_address - 45);           //      2       45
        
        out.write(CHECK_IF_STARBASE);                   //      4       49
        out.write(STARBASE_CLASS);    
        
        out.write(SJZ);
        out.write(jump_to_next_address - 51);           //      2       51
    }
    
    private void saveRaceAndSP1Check(OutputStream out, int jump_next_address, int jump_add_address) throws IOException
    {
        final byte[] COMPARE_SP = new byte[]{           // 2
                0x66, (byte) 0x83
        };

        // disable race check
        for (int i = 0; i < 6; i++)     // 6
        {
            out.write(0x90);
        }
        
        if (SPECIAL_SHIPS)
        {
            out.write(COMPARE_SP);      // 3 -> 8
            out.write(0xFB);            // 1 -> 9
            out.write(SP[0]);           // 1 -> 10
            out.write(0x75);            // 11
            out.write(jump_next_address - 12);     // 12
        }
        else
        {
            for (int i = 0; i < 4; i++)
            {
                out.write(0x90);
            }

            out.write(0xEB);
            out.write(jump_add_address - 12);     // 12
        }
    }
    
    private void saveSpecialBuildingCheck(OutputStream out, int jump_next_address) throws IOException
    {
        final byte[] TEST_SP_BUILDING_1 = new byte[]{   // 3
                0x31, (byte) 0xC0, (byte) 0xBA
        };
        final byte[] TEST_SP_BUILDING_2 = new byte[]{   // 3
                (byte) 0x8A, 0x46, 0x4C
        };
        final byte[] TEST_SP_BUILDING_3 = new byte[]{   // 2
                (byte) 0x85, (byte) 0xC0
        };
        final byte[] CALL = new byte[]{                 // 1
                (byte)0xE8
        };
        final byte[] SJZ = new byte[]{                  // 1
                0x74
        };

        out.write(TEST_SP_BUILDING_1);  // 3
        
        out.write(Tools.toByte(BUILDING, true));        // 7
            
        out.write(TEST_SP_BUILDING_2);  // 10

        out.write(CALL);        // 11
        out.write(Tools.toByte(CALL_ADDRESS[3], true)); // 15

        out.write(TEST_SP_BUILDING_3);  // 17

        out.write(SJZ);                 // 18
        out.write(jump_next_address + 19); // 19
    }
    
    private void saveAddShipToList(OutputStream out, int jump_next_address) throws IOException // 65
    {
        final byte[] ADD_SHIP_TO_LIST_1 = new byte[]{   // 47
                (byte) 0xBA, 0x03, 0x00, 0x00, 0x00, 0x31,
                (byte) 0xC0, 0x66, (byte) 0x89, (byte) 0x9C,
                0x24, 0x28, 0x01, 0x00, 0x00, 0x66,
                (byte) 0x8B, (byte) 0x84, 0x24, 0x1C, 0x01,
                0x00, 0x00, (byte) 0x89, (byte) 0x94, 0x24,
                0x20, 0x01, 0x00, 0x00, (byte) 0x89,
                (byte) 0x84, 0x24, 0x2C, 0x01, 0x00, 0x00,
                (byte) 0x8D, (byte) 0x94, 0x24, 0x20, 0x01,
                0x00, 0x00, (byte) 0x8B, 0x47, 0x38
        };
        final byte[] ADD_SHIP_TO_LIST_2 = new byte[]{   // 2
                (byte) 0x89, (byte) 0xC8
        };
        final byte[] ADD_SHIP_TO_LIST_3 = new byte[]{   // 4
                (byte) 0x89, (byte) 0xC1, (byte) 0x85,
                (byte) 0xC0
        };
        byte[] SJNZ = new byte[]{                       // 1
                    0x75
        };
        final byte[] CALL = new byte[]{                 // 1
                    (byte)0xE8
        };

        out.write(ADD_SHIP_TO_LIST_1);                  // 47

        out.write(CALL);
        out.write(Tools.toByte(CALL_ADDRESS[4], true)); // 52
        
        out.write(ADD_SHIP_TO_LIST_2);                  // 54
        
        out.write(CALL);
        out.write(Tools.toByte(CALL_ADDRESS[5], true)); // 59
        
        out.write(ADD_SHIP_TO_LIST_3);                  // 63
        
        out.write(SJNZ);
        out.write(jump_next_address - 65);              // 65
    }
    
    private void saveSpecialShipChecks(OutputStream out, int jump_next_address, int jump_add_address) throws IOException
    {
        final byte[] COMPARE_SP = new byte[]{           // 2
                0x66, (byte) 0x83
        };
        final byte[] SJMP = new byte[]{                 // 1
                (byte)0xEB
        };

        for (int i = 1; i < 5; i++)             // total:  24   24
        {
            out.write(COMPARE_SP);
            out.write(SP_REG[i]);
            out.write(SP[i]);
            out.write(0x74);
            out.write(jump_add_address - i*6);
        }
        
        out.write(SJMP);
        out.write(jump_next_address + 26);
    }
    
    /**
     * @param out
     * @throws IOException 
     */
    private void saveAsUsingMinorShipMod(OutputStream out) throws Exception   // 244
    {
        final byte[] PREPARE_STACK = new byte[]{        // 6
                (byte)0x81, (byte)0xEC, 0x30, 0x01, 0x00,
                0x00
        };
        final byte[] SET_EBP_TO_MINORS = new byte[]{    // 5
            (byte)0xBD, 0x44, 0x00, 0x00, 0x00
        };
        final byte[] PREPARE_ESI = new byte[]{          // 27 bytes
            (byte)0x8B, 0x3D, (byte)0xA8, 0x5D, 0x59,
            0x00, (byte)0x8B, 0x77, 0x1A, (byte)0xC1,
            (byte)0xFE, 0x10, 0x69, (byte)0xF6, 0x28,
            0x03, 0x00, 0x00, (byte)0xA1, (byte)0xC8,
            0x36, 0x5A, 0x00, 0x01, (byte)0xC6, 0x31,
            (byte)0xC0
        };
        final byte[] READ_SYS_EBP = new byte[]{         // 3
            (byte)0x8A, 0x04, 0x2E
        };
        final byte[] CHECK_AND_SET_EBP_FOR_RERUN = new byte[]{ // 8
            (byte)0x83, (byte)0xFD, 0x44, 0x75, 0x08,
            (byte)0x83, (byte)0xC5, 0x08
        };
        final byte[] JMP = new byte[]{                  // 1
            (byte)0xE9
        };
        final byte[] CLEAR_STACK = new byte[]{          // 6
                (byte) 0x81, (byte) 0xC4, 0x30, 0x01,
                0x00, 0x00
        };
        final byte[] POP_EBP = new byte[]{              // 1
                0x5D
        };
        final byte[] PUSH_EBP = new byte[]{             // 1
                0x55
        };
        final byte[] POP_BLOCK = new byte[]{            // 6
                0x5F, 0x5E, 0x5A, 0x59, 0x5B,
                (byte) 0xC3
        };
        final byte[] BLANK_BLOCK_END = new byte[]{      // 1
                0x00
        };

        out.write(PUSH_EBP);                    // 1
        out.write(PREPARE_STACK);               // 6
        out.write(SET_EBP_TO_MINORS);           // 5
        out.write(PREPARE_ESI);                 // 27
        out.write(READ_SYS_EBP);                // 3
        
        saveMRMRaceTest(out, 152);              // 35

        saveShipLoadAndBasicChecks(out, 103, 116);   // 51
        
        saveAddShipToList(out, -31);            // 65
        
        out.write(CHECK_AND_SET_EBP_FOR_RERUN); // 8

        out.write(JMP);
        out.write(Tools.toByte(-195, true));    // 5
        
        out.write(CLEAR_STACK);                 // 6
        out.write(POP_EBP);                     // 1
        out.write(POP_BLOCK);                   // 6
        out.write(BLANK_BLOCK_END);             // 1
    }
    
    private void saveMRMRaceTest(OutputStream out, int exit_address) throws Exception       // 36
    {
        final byte[] IS_EMPIRE = new byte[]{            // 2
                0x3C, 0x04
        };
        final byte[] JA = new byte[]{                   // 2
                0x77, 0x0A
        };
        final byte[] IS_SYS_OWNER = new byte[]{         // 3
                (byte)0x83, (byte)0xFD, 0x4C
        };
        final byte[] JZ = new byte[]{                   // 2
                0x74, 0x18
        };
        final byte[] JMP = new byte[]{                  // 1
                (byte)0xE9
        };
        final byte[] PUSH_EBP = new byte[]{             // 1 byte
                0x55
            };
        final byte[] GET_SUBJUDGED_STATUS = new byte[]{ // 12
                (byte)0xBD, 0x10, 0x20, 0x5A, 0x00,
                0x6B, (byte)0xC0, 0x40, (byte)0x8B,
                0x44, 0x28, 0x3C
        };
        final byte[] POP_EBP = new byte[]{              // 1
                0x5D
        };
        final byte[] TEST_EAX = new byte[]{             // 2
                (byte)0x83, (byte)0xF8, 0x01
        };
        final byte[] JZ2 = new byte[]{                  // 2
                0x74, (byte)0xE8
        };
        final byte[] READ_SYS_EBP = new byte[]{         // 3
                (byte)0x8A, 0x04, 0x2E
        };
        
        out.write(IS_EMPIRE);   // 2
        out.write(JA);          // 2
        out.write(IS_SYS_OWNER);// 3
        out.write(JZ);          // 2
        out.write(JMP);         // 5
        out.write(Tools.toByte(exit_address - 14, true));
        out.write(PUSH_EBP);
        out.write(GET_SUBJUDGED_STATUS);
        out.write(POP_EBP);
        out.write(TEST_EAX);
        out.write(JZ2);
        out.write(READ_SYS_EBP);
    }

    public void setMinorShipsMod(boolean b)
    {
        if (MINOR_SHIPS != b)
        {
            markChanged();
            MINOR_SHIPS = b;
            
            if (MINOR_SHIPS)
            {
                SPECIAL_SHIPS = false;
                MINOR_SHIPS_SIMPLE = false;
            }
        }
    }
    
    public boolean areMinorShipsEnabled()
    {
        return MINOR_SHIPS;
    }
    
    public void setMinorShipsSimpleMod(boolean b)
    {
        if (MINOR_SHIPS_SIMPLE != b)
        {
            markChanged();
            MINOR_SHIPS_SIMPLE = b;
            
            if (MINOR_SHIPS_SIMPLE)
            {
                MINOR_SHIPS = false;
            }
        }
    }
    
    public boolean areMinorShipsSimpleEnabled()
    {
        return MINOR_SHIPS_SIMPLE;
    }

    private void updateAddresses()
    {
        Trek trek = (Trek)UE.FILES.getFile(FileManager.P_TREK);
        
        CALL_ADDRESS[0] = (int)trek.getSegmentAddress(Trek.fileUNKNOWN1);
        CALL_ADDRESS[1] = (int)trek.getSegmentAddress(Trek.fileUNKNOWN2);
        CALL_ADDRESS[2] = (int)trek.getSegmentAddress(Trek.fileUNKNOWN3);
        CALL_ADDRESS[3] = (int)trek.getSegmentAddress(Trek.fileUNKNOWN4);
        CALL_ADDRESS[4] = (int)trek.getSegmentAddress(Trek.fileUNKNOWN5);
        CALL_ADDRESS[5] = (int)trek.getSegmentAddress(Trek.fileUNKNOWN6);
        
        for (int i = 0; i < CALL_ADDRESS.length; i++)
        {
            CALL_ADDRESS[i] -= this.ADDRESS;
        }
        
        if (MINOR_SHIPS)
        {
            CALL_ADDRESS[0] -= 88;
            CALL_ADDRESS[1] -= 93;
            CALL_ADDRESS[2] -= 118;
            // CALL_ADDRESS[3] = unused
            CALL_ADDRESS[4] -= 186;
            CALL_ADDRESS[5] -= 193;
        }
        else
        {
            CALL_ADDRESS[0] -= 46;
            CALL_ADDRESS[1] -= 51;
            CALL_ADDRESS[2] -= 76;
            CALL_ADDRESS[3] -= 119;
            CALL_ADDRESS[4] -= 175;
            CALL_ADDRESS[5] -= 182;
        }
    }
}
