/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *      
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *      
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: Alt.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *      
 * Last modified: 18.9.2008
 */

package ue.edit.res;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import java.util.logging.Level;
import java.util.logging.Logger;
import ue.Language;
import ue.UE;
import ue.edit.FileManager;
import ue.edit.FilesInterface;
import ue.edit.InternalFile;
import ue.edit.mod.ModList;
import ue.edit.sav.SavEntry;
import ue.edit.sav.SavFile;
import ue.exception.NotAPanel;
import ue.gui.CheckGUI;
import ue.gui.FilesGUI;
import ue.gui.MainPanel;
import ue.gui.ModifiedFilesGUI;
import ue.util.GenericFile;
import ue.util.StringGroup;
import ue.util.Tools;
import ue.util.UEWorker;

/**Used for alt.res.*/
public class Alt extends ResFile
{
    private File ALT;
    private boolean CANCELLED_SAVE = false;
    public boolean ADD_CHANGES_2_MODLIST = true;
    //a list of generic files
    private Hashtable<String, InternalFile> INTERNAL_FILES = new Hashtable<String, InternalFile>();
    private HashSet <String> REMOVED = new HashSet<String>();
    
    @Override
    public Enumeration<String> getListOfLoadedFiles()
    {
        return this.INTERNAL_FILES.keys();
    }
    
    @Override
    public Iterator<String> getRemovedFilesList()
    {
        return this.REMOVED.iterator();
    }
    
    /**
     * Retruns a list of files in alt.res
     * @return List of files as enumeration
     */
    @Override
    public Enumeration<String[]> getFiles(FilenameFilter filter)
    {
        Vector <String[]> result = new Vector<String[]>();
        SavFile source = null;
        SavEntry ze = null;
        
        try
        {
            source = new SavFile(this.ALT, null);
            
            //GO THRU EXISTING FILES
            for (Enumeration<SavEntry> e = source.entries(); e.hasMoreElements();)
            {
                ze = e.nextElement();
                if (ze.getName().equals("mod.lst") || this.REMOVED.contains(ze.getName())) //$NON-NLS-1$
                {
                    continue;
                }
                if (filter.accept(this.ALT, ze.getName()) && !this.INTERNAL_FILES.containsKey(ze.getName()))
                {
                    result.add(new String[]{ze.getName(), Integer.toString(ze.getUncompressedSize())});
                }
            }

            //AND NOW THE REST
            Enumeration<InternalFile> itg = this.INTERNAL_FILES.elements();
            while (itg.hasMoreElements())
            {
                InternalFile gef = itg.nextElement();
                if (gef.getName().equals("mod.lst") || this.REMOVED.contains(gef.getName())) //$NON-NLS-1$
                {
                    continue;
                }
                if (filter.accept(this.ALT, gef.getName()))
                {
                    result.add(new String[]{gef.getName(), Long.toString(gef.getSize())});
                }
            }
        }
        catch (Exception e)
        {
            UE.displayError(e);
        }
        return result.elements();
    }
    
    /**
     * Discards all changes.
     */
    @Override
    public void discard()
    {
        this.REMOVED.clear();
        this.INTERNAL_FILES.clear();
    }
    
    /**
     * @param command Must be one of the options returned by the open function else null is returned.
     * @return MainPanel object or null
     * @throws may throw NotAPanel if the command opens a window or does a background task instead of returning a MainPanel
     */
    @Override
    public MainPanel edit(String command) throws NotAPanel, Exception
    {
        if (Language.getString("Alt.0").equals(command)) //$NON-NLS-1$
        {
            return new FilesGUI(new FilesInterface(this));
        }
        if (Language.getString("Alt.1").equals(command)) //$NON-NLS-1$
        {
            return new CheckGUI(this.check());
        }
        if (Language.getString("Alt.2").equals(command)) //$NON-NLS-1$
        {
            // update modified files list first
            Enumeration<InternalFile> itg = this.INTERNAL_FILES.elements();
            InternalFile gef;
            while (itg.hasMoreElements())
            {
                gef = itg.nextElement();
                if (gef.getName().equals("mod.lst")) //$NON-NLS-1$
                {
                    continue;
                }
                if (gef.madeChanges())
                {
                    this.MODS.add(gef.getName());
                }
            }
            // disable main window
            UE.WINDOW.setEnabled(false);

            new ModifiedFilesGUI(this);
            throw new NotAPanel();
        }
        return null;
    }
    
    @Override
    public void markChanged()
    {
        this.CANCELLED_SAVE = true;
    }
    
    /**
     * Saves changes to the specified file.
     * @param whereTo The file to which changes are to be saved.
     * @return true if succesful else false.
     */
    @Override
    public boolean save(File whereTo, boolean bkp, String ext) throws Exception
    {
        if (whereTo == null)
        {
            // save to the open file
            whereTo = new File(this.ALT.getAbsolutePath());
        }
        
        this.CANCELLED_SAVE = true;

        final SaveTask TSK = new SaveTask(whereTo, bkp, ext);
        
    	Object saved_ok = Tools.runUEWorker(TSK);

        if (saved_ok instanceof Boolean)
        {
            if (((Boolean)saved_ok).booleanValue())
            {
                this.CANCELLED_SAVE = false;
                this.REMOVED.clear();
                this.ALT = whereTo;
            }
        }

        return !this.CANCELLED_SAVE;
    }
    
    /**
     * This adds an internal file to alt.res.
     * @param gf the InternalFile to be added.
     */
    @Override
    public void addInternalFile(InternalFile gf)
    {
        this.INTERNAL_FILES.put(gf.getName(), gf);
        this.REMOVED.remove(gf.getName());
        gf.markChanged();
    }
    
    /**
     * This removes a file from alt.res.
     * @param name the name of the file to be removed.
     */
    @Override
    public void removeInternalFile(String name)
    {
        this.INTERNAL_FILES.remove(name);
        this.REMOVED.add(name);
    }
    
    /**
     * @param whereFrom The file to open. Must be a valid alt.res file!
     * @return a String array of valid commands that can be used with the edit function.
     */
    @Override
    public StringGroup[] open(File whereFrom)
    {
        this.ALT = whereFrom;
        try
        {
            SavFile sf = new SavFile(this.ALT, null);
            SavEntry se = sf.getEntry("mod.lst"); //$NON-NLS-1$
            if (se!=null)
            {
                this.MODS = new ModList(sf.getInputStream(se));
                if (this.MODS==null)
                {
                    UE.displayError(new Exception(Language.getString("Alt.5"))); //$NON-NLS-1$
                    this.MODS = new ModList();
                }
            }
            else
            {
                this.MODS = new ModList();
            }
            
            sf.close();
        }
        catch (Exception e)
        {
            UE.displayError(e);
            this.ALT = null;
            return null;
        }
        return this.getCommands();
    }
    
    /**
     * @return a String array of available commands that can be used with the edit function.
     */
    @Override
    public StringGroup[] getCommands()
    {
        if (this.ALT == null)
        {
            return null;
        }
        
        StringGroup[] ret = new StringGroup[1];
        try
        {
            ret[0] = new StringGroup(Language.getString("Alt.6"), new String[]{Language.getString("Alt.0"),//$NON-NLS-1$ //$NON-NLS-2$
                Language.getString("Alt.1"), Language.getString("Alt.2")}); //$NON-NLS-1$ //$NON-NLS-2$
        }
        catch (Exception tz)
        {
            UE.displayError(tz);
            return null;
        }
        return ret;
    }
    
    /**
     * @return true if changes were made to the opened file and false if not.
     */
    @Override
    public boolean madeChanges()
    {
        if (this.CANCELLED_SAVE || !this.REMOVED.isEmpty())
        {
            return true;
        }
        
        if (!this.INTERNAL_FILES.isEmpty())
        {
            Enumeration<InternalFile> en = this.INTERNAL_FILES.elements();
            
            while (en.hasMoreElements())
            {
                if (en.nextElement().madeChanges())
                {
                    return true;
                }
            }
        }
        return false;
    }
    
    /**
     * Checks internal files for unusal data and other problems.
     * @return a Vector containing the check results.
     */
    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();
        
        CheckTask TSK = new CheckTask();
        
        Object resp = Tools.runUEWorker(TSK);

        try
        {
            Vector<?> v  = (Vector<?>)resp;

            for (int i = 0; i < v.size(); i++)
            {
                response.add(v.get(i).toString());
            }
            
            if (response.isEmpty())
            {
                response.add(Language.getString("Alt.7")); //$NON-NLS-1$
            }
        }
        catch (Exception e)
        {
            UE.displayError(e);
        }

        return response;
    }
    
    /**
     * Returns a requested generic file.
     * @throws FileNotFoundExeption if the file can't be found.
     */
    @Override
    public InternalFile getInternalFile(String name, boolean cache) throws FileNotFoundException
    {
        String msg;
        if (this.REMOVED.contains(name))
        {
            msg = Language.getString("Alt.8"); //$NON-NLS-1$
            msg = msg.replace("%1", name); //$NON-NLS-1$
            throw new FileNotFoundException(msg);
        }
        InternalFile gen = null;
        
        gen = this.INTERNAL_FILES.get(name);
        
        if (gen == null)
        {
            InputStream in = null;
            SavEntry ze = null;
            try
            {
                SavFile zf = new SavFile(this.ALT, null);
                ze = zf.getEntry(name);
                
                if (ze==null)
                {
                    zf.close();
                    msg = Language.getString("Alt.8"); //$NON-NLS-1$
                    msg = msg.replace("%1", name); //$NON-NLS-1$
                    throw new FileNotFoundException(msg);
                }
                else
                {
                    in = zf.getInputStream(ze);
                    byte[] data = new byte[in.available()];
                    in.read(data);
                    gen = new GenericFile(name, data);

                    if (cache)
                        this.INTERNAL_FILES.put(gen.getName(), gen);
                }
                zf.close();
            }
            catch (FileNotFoundException e)
            {
                throw e;
            }
            catch (Exception e)
            {
                UE.displayError(e);
            }
        }
        return gen;
    }

    @Override
    public long getInternalFileSize(String name) throws IOException
    {
        String msg;
        if (this.REMOVED.contains(name))
        {
            msg = Language.getString("Alt.8"); //$NON-NLS-1$
            msg = msg.replace("%1", name); //$NON-NLS-1$
            throw new FileNotFoundException(msg);
        }
        InternalFile gen = null;

        gen = this.INTERNAL_FILES.get(name);

        long size = 0;

        if (gen == null)
        {
            SavEntry ze = null;

            try
            {
                SavFile zf = new SavFile(this.ALT, null);
                ze = zf.getEntry(name);

                if (ze == null)
                {
                    zf.close();
                    msg = Language.getString("Alt.8"); //$NON-NLS-1$
                    msg = msg.replace("%1", name); //$NON-NLS-1$
                    throw new FileNotFoundException(msg);
                }
                else
                {
                    size = ze.getUncompressedSize();
                }
                zf.close();
            }
            catch (FileNotFoundException e)
            {
                throw e;
            }
            catch (Exception e)
            {
                UE.displayError(e);
            }
        }
        else
        {
            size = gen.getSize();
        }

        return size;
    }

    private class SaveTask extends UEWorker<Boolean>
    {
        File DESTIN;
        boolean bkp;
        String ext;
        
        public SaveTask(File dest, boolean back, String exte)
        {
            super();
            this.DESTIN = dest;
            bkp = back;
            ext = exte;
        }
        
        @Override
        public Boolean doInBackground()
        {
            this.RESULT = false;
            String msg;
            
            this.updateStatus(UEWorker.STATUS_WORKING);
            
            SavFile source = null;
            SavFile dest = null;
            
            try
            {
                source = new SavFile(Alt.this.ALT, null);
                
                //where to
                if (this.DESTIN.equals(Alt.this.ALT))
                {
                    // do backup?
                    if (bkp)
                    {
                        this.sendMessage(Language.getString("Alt.19")); //$NON-NLS-1$
                        
                        File file = new File(ALT.getPath() + ext);
                        int i = 1;
                        
                        while (file.exists())
                        {
                            file = new File(ALT.getPath() + i + ext);
                            i++;
                        }
                        
                        Tools.copyFile(ALT, file);
                    }
                    
                    this.DESTIN = FileManager.getWritableLocalFile("alt.tmp"); //$NON-NLS-1$
                }
                
                this.DESTIN.delete();
                dest = new SavFile(this.DESTIN, source.getName()); //savfile creates the file 
                
                if (this.isCancelled())
                {
                    dest.close();
                    this.DESTIN.delete();
                    source.close();
                    this.updateStatus(UEWorker.STATUS_CANCEL);
                    return this.RESULT;
                }
            }
            catch (Exception e)
            {
                this.reportError(e);
                return this.RESULT;
            }
            
            msg = Language.getString("Alt.9"); //$NON-NLS-1$
            
            //GO THRU EXISTING FILES
            for (Enumeration<SavEntry> e = source.entries(); e.hasMoreElements() && !this.isCancelled(); )
            {
                try
                {
                    SavEntry ze = e.nextElement();
                    
                    if (ze.getName().equals("mod.lst") || Alt.this.REMOVED.contains(ze.getName()))//$NON-NLS-1$
                    {
                        continue;
                    }
                    
                    if (!Alt.this.INTERNAL_FILES.containsKey(ze.getName()))
                    {
                        this.sendMessage(msg.replace("%1", ze.getName())); //$NON-NLS-1$
                        
                        InputStream is = source.getInputStream(ze, false);
                        dest.putNextEntry(ze);
                        OutputStream zos = dest.getOutputStream(ze, false);
                        int len;
                        byte[] buf = new byte[1024];
                        
                        while ((len = is.read(buf)) > 0)
                        {
                            zos.write(buf, 0, len);
                        }
                        is.close();
                        zos.close();

                        // Complete the entry
                        dest.closeEntry();
                    }
                    if (this.isCancelled())
                    {
                        dest.close();
                        this.DESTIN.delete();
                        source.close();
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        return this.RESULT;
                    }
                }
                catch (Exception zne)
                {
                    this.reportError(zne);
                    return this.RESULT;
                }
            }
            
            msg = Language.getString("Alt.10"); //$NON-NLS-1$
            //AND NOW THE REST
            Enumeration<InternalFile> itg = Alt.this.INTERNAL_FILES.elements();
            SavEntry zeg;
            
            while (itg.hasMoreElements())
            {
                try
                {
                    if (this.isCancelled())
                    {
                        source.close();
                        dest.close();
                        this.DESTIN.delete();
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        return this.RESULT;
                    }
                    
                    InternalFile gef = itg.nextElement();
                    if (gef.getName().equals("mod.lst") || Alt.this.REMOVED.contains(gef.getName())) //$NON-NLS-1$
                    {
                        continue;
                    }
                    
                    this.sendMessage(msg.replace("%1", gef.getName())); //$NON-NLS-1$
                    
                    if (Alt.this.ADD_CHANGES_2_MODLIST)
                    {   // save changes to modlist
                        Alt.this.MODS.add(gef.getName());
                    }
                    
                    int ln = (int)gef.getSize();
                    
                    zeg = new SavEntry(gef.getName());
                    zeg.setCompressedSize(-1);
                    zeg.setUncompressedSize(ln);
                    
                    dest.putNextEntry(zeg);
                    OutputStream zos = dest.getOutputStream(zeg, true);
                    gef.save(zos);
                    
                    gef.markSaved();
                    
                    zos.close();
                    dest.closeEntry();
                }
                catch (Exception ze)
                {
                    this.reportError(ze);
                    return this.RESULT;
                }
            }
            
            this.sendMessage(msg.replace("%1", "mod.lst")); //$NON-NLS-1$ //$NON-NLS-2$
            
            try
            {
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                Alt.this.MODS.save(baos);
                byte[] bub = baos.toByteArray();
                int sizi = bub.length;
                
                zeg = new SavEntry("mod.lst"); //$NON-NLS-1$
                zeg.setCompressedSize(-1);
                zeg.setUncompressedSize(sizi);
                
                dest.putNextEntry(zeg);
                OutputStream zos = dest.getOutputStream(zeg, true);
                
                zos.write(bub, 0, sizi);
                zos.close();
                dest.closeEntry();
            }
            catch (Exception ze)
            {
                this.reportError(ze);
                return this.RESULT;
            }			
            
            //close files
            this.sendMessage(Language.getString("Alt.11")); //$NON-NLS-1$
            
            try
            {
                source.close();
                dest.close();
            }
            catch (Exception e)
            {
                this.reportError(e);
                return this.RESULT;
            }
            
            if (this.isCancelled())
            {
                // last chance to cancel action
                return this.RESULT;
            }
            
            //replace?
            if (this.DESTIN.getName().equals("alt.tmp")) //$NON-NLS-1$
            {
                msg = Language.getString("Alt.9"); //$NON-NLS-1$
                this.sendMessage(msg.replace("%1", this.DESTIN.getName())); //$NON-NLS-1$
                
                try
                {
                    Tools.copyFile(this.DESTIN, Alt.this.ALT);
                }
                catch (IOException er)
                {
                    this.reportError(er);
                    return this.RESULT;
                }
                
                msg = Language.getString("Alt.12"); //$NON-NLS-1$
                
                this.sendMessage(msg.replace("%1", this.DESTIN.getName())); //$NON-NLS-1$
                
                boolean test = this.DESTIN.delete();
                
                if (!test)
                {
                    msg = Language.getString("Alt.13"); //$NON-NLS-1$
                    
                    this.reportError(new Exception(msg.replace("%1", this.DESTIN.getName()))); //$NON-NLS-1$
                }
            }
            this.RESULT = true;
            this.updateStatus(UEWorker.STATUS_DONE);
            return this.RESULT;
        }
    }
    
    private class CheckTask extends UEWorker<Vector<String>>
    {
        public CheckTask()
        {
            super();
        }
        
        @Override
        public Vector<String> doInBackground()
        {
            String msg;
            //check all
            InternalFile test = null;
            
            this.RESULT = new Vector<String>();
            this.updateStatus(UEWorker.STATUS_WORKING);
            
            this.sendMessage(Language.getString("Alt.3")); //$NON-NLS-1$
            
            Vector<String> temp;
            
            try
            {
                //of the file as whole
                SavFile su = new SavFile(Alt.this.ALT, null);
                temp = su.check();
                if (!temp.isEmpty())
                {
                    this.RESULT.addAll(temp);
                }
                su.close();
            }
            catch (Exception f)
            {
                msg = Language.getString("Alt.16"); //$NON-NLS-1$
                msg = msg.replace("%1", Alt.this.ALT.getName()); //$NON-NLS-1$
                msg = msg.replace("%2", f.getMessage()); //$NON-NLS-1$
                f.printStackTrace();
                this.RESULT.add(msg);
            }
            
            Enumeration<InternalFile> un = Alt.this.INTERNAL_FILES.elements();
            
            msg = Language.getString("Alt.17"); //$NON-NLS-1$
            
            while(un.hasMoreElements() && !this.isCancelled())
            {
                try
                {
                    if (this.isCancelled())
                    {
                        this.RESULT.add(Language.getString("Alt.18")); //$NON-NLS-1$
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        return this.RESULT;
                    }
                    
                    test = un.nextElement();
                    
                    this.sendMessage(msg.replace("%1", test.getName())); //$NON-NLS-1$
                    
                    temp = test.check();
                    if (!temp.isEmpty())
                    {
                        this.RESULT.addAll(temp);
                    }
                }
                catch (Exception f)
                {
                    msg = Language.getString("Alt.16"); //$NON-NLS-1$
                    msg = msg.replace("%1", test.getName()); //$NON-NLS-1$
                    msg = msg.replace("%2", f.getMessage()); //$NON-NLS-1$
                    f.printStackTrace();
                    this.RESULT.add(msg);
                }
            }
            
            this.updateStatus(UEWorker.STATUS_DONE);
            return this.RESULT;
        }
    }
    
    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#getKnownChanges()
     */
    @Override
    public ModList getKnownChanges()
    {
        return this.MODS;
    }

    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#hasBeenModified()
     */
    @Override
    public boolean hasBeenModified()
    {
        return !this.MODS.isEmpty() || this.madeChanges();
    }
    
    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#toFile()
     */
    @Override
    public File toFile()
    {
        return this.ALT;
    }
}
