/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *      
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *      
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: Stbof.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: ??.??.2005
 * 
 * Modified: 18.9.2008
 * 
 * Code clean up. Formating etc. Stbof now uses UEWorker.
 */

package ue.edit.res;

import java.awt.Dimension;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import ue.Language;
import ue.UE;
import ue.edit.FileManager;
import ue.edit.FilesInterface;
import ue.edit.InternalFile;
import ue.edit.exe.Trek;
import ue.edit.mod.ModList;
import ue.edit.res.stbof.StringFile;
import ue.edit.res.stbof.bin.AIBldReq;
import ue.edit.res.stbof.bin.BldSet;
import ue.edit.res.stbof.bin.Converse;
import ue.edit.res.stbof.bin.Intel;
import ue.edit.res.stbof.bin.Meplanet;
import ue.edit.res.stbof.bin.Morale;
import ue.edit.res.stbof.bin.Planboni;
import ue.edit.res.stbof.bin.Planspac;
import ue.edit.res.stbof.bin.Shipname;
import ue.edit.res.stbof.bin.Starname;
import ue.edit.res.stbof.bst.Edifbnft;
import ue.edit.res.stbof.bst.Edifice;
import ue.edit.res.stbof.dic.Lexicon;
import ue.edit.res.stbof.est.Environ;
import ue.edit.res.stbof.fnt.TgaFont;
import ue.edit.res.stbof.hob.HobFile;
import ue.edit.res.stbof.lst.PaletteList;
import ue.edit.res.stbof.lst.Texture;
import ue.edit.res.stbof.pst.Planet;
import ue.edit.res.stbof.rst.Race;
import ue.edit.res.stbof.smt.Objstruc;
import ue.edit.res.stbof.sst.ShipList;
import ue.edit.res.stbof.sst.ShipRace;
import ue.edit.res.stbof.sst.ShipTech;
import ue.edit.res.stbof.tec.RaceTech;
import ue.edit.res.stbof.tec.TecField;
import ue.edit.res.stbof.tec.TechTree;
import ue.edit.res.stbof.wdf.WindowDefinitionFile;
import ue.edit.res.stbof.wtf.Aiempire;
import ue.edit.res.stbof.wtf.Aiminor;
import ue.edit.res.stbof.wtf.Colval;
import ue.exception.CorruptedFile;
import ue.exception.InvalidArgumentsException;
import ue.exception.NotAPanel;
import ue.gui.CheckGUI;
import ue.gui.FilesGUI;
import ue.gui.MainPanel;
import ue.gui.ModifiedFilesGUI;
import ue.gui.stbof.AIBldReqGUI;
import ue.gui.stbof.IntelGUI;
import ue.gui.stbof.AiminorGUI;
import ue.gui.stbof.AniGUI;
import ue.gui.stbof.BldSetGUI;
import ue.gui.stbof.BuildingsGUI;
import ue.gui.stbof.BuildingsMultiSetGUI;
import ue.gui.stbof.ColValGUI;
import ue.gui.stbof.EnvironGUI;
import ue.gui.stbof.HobGUI;
import ue.gui.stbof.HtmlReportGUI;
import ue.gui.stbof.InGameTextGUI;
import ue.gui.stbof.LexiconGUI;
import ue.gui.stbof.MaxPopOrBonGUI;
import ue.gui.stbof.MeplanetGUI;
import ue.gui.stbof.MoraleGUI;
import ue.gui.stbof.ObjstrucGUI;
import ue.gui.stbof.PlanetGUI;
import ue.gui.stbof.RaceInfoGUI;
import ue.gui.stbof.ShipGUI;
import ue.gui.stbof.ShipModelGUI;
import ue.gui.stbof.ShipMultiGUI;
import ue.gui.stbof.ShipnameGUI;
import ue.gui.stbof.StarnameGUI;
import ue.gui.stbof.StartConGUI;
import ue.gui.stbof.TechGUI;
import ue.gui.stbof.TextureGUI;
import ue.gui.stbof.WDFEditorGUI;
import ue.util.AniOrCur;
import ue.util.CommonStrings;
import ue.util.GenericFile;
import ue.util.Pair;
import ue.util.StringGroup;
import ue.util.TargaImage;
import ue.util.Tools;
import ue.util.UEWorker;
import ue.util.ZipReader;

/**
 * This class is a representation of the stbof.res file,
 * it communicates with other classes in the edit.res.stbof subpackages.
 * it also sends MainPanel instances to use for editing the file.
 */
public class Stbof extends ResFile
{
    /**
     * List of recognized internal files.
     */
    private static String[] FILELIST;

    /*
     * A list of valid commands to use with the edit function.
     * CAREFULL!!! add new only to the end of the list!
     */
    private static String[] COMMANDLIST;
    
    // A hash table of all loaded files.
    private Hashtable<String, InternalFile> INTERNAL_FILES;

    // List of files to be removed
    private HashSet <String> REMOVED = new HashSet<String>();

    // File opened
    private File STBOF_FILE = null;

    // If the save process has been cancelled this is true else false
    // Is also set to true when markChanged is called
    private boolean CANCELLED_SAVE = false;

    // Trek.exe - some options require this, but if null, stbof.java hould still work without problems
    // as should all the panels it returns
    private Trek TREK = null;
    
    // used as a temporary file during saving
    private final static String fileSTBOF_TMP = "stbof.tmp"; //$NON-NLS-1$
    
    // If set to false no changes will be saved to the modlist
    // This is an artifact of the past, still works, but there's no reason to set it to false
    private boolean ADD_CHANGES_2_MODLIST = true;

    private ArrayList<String> files = new ArrayList<String>();

    // LIST OF INTERNAL FILES
    // Use these to load appropriate files from outside stbof.res
    public final static String fileAIBLDREQ_BIN = "aibldreq.bin"; //$NON-NLS-1$
    public final static String fileAIEMPIRE_WTF = "aiempire.wtf"; //$NON-NLS-1$
    public final static String fileAIMINOR_WTF = "aiminor.wtf"; //$NON-NLS-1$
    public final static String fileBLDSET1_BIN = "bldset1.bin"; //$NON-NLS-1$
    public final static String fileBLDSET2_BIN = "bldset2.bin"; //$NON-NLS-1$
    public final static String fileBLDSET3_BIN = "bldset3.bin"; //$NON-NLS-1$
    public final static String fileBLDSET4_BIN = "bldset4.bin"; //$NON-NLS-1$
    public final static String fileBLDSET5_BIN = "bldset5.bin"; //$NON-NLS-1$
    public final static String fileCCOLVAL_WTF = "ccolval.wtf"; //$NON-NLS-1$
    public final static String fileCONVERSE_BIN = "converse.bin"; //$NON-NLS-1$
    public final static String fileEDIFBNFT_BST = "edifbnft.bst"; //$NON-NLS-1$
    public final static String fileEDIFDESC_BST = "edifdesc.bst"; //$NON-NLS-1$
    public final static String fileEDIFICE_BST = "edifice.bst"; //$NON-NLS-1$
    public final static String fileENVIRON_EST = "environ.est"; //$NON-NLS-1$
    public final static String fileFCOLVAL_WTF = "fcolval.wtf"; //$NON-NLS-1$
    public final static String fileHCOLVAL_WTF = "hcolval.wtf"; //$NON-NLS-1$
    public final static String fileKCOLVAL_WTF = "kcolval.wtf"; //$NON-NLS-1$
    public final static String fileLEXICON_DIC = "lexicon.dic"; //$NON-NLS-1$
    public final static String fileMEPLANET_BIN = "meplanet.bin"; //$NON-NLS-1$
    public final static String fileMORALE_BIN = "morale.bin"; //$NON-NLS-1$
    public final static String fileOBJSTRUC_SMT = "objstruc.smt"; //$NON-NLS-1$
    public final static String filePALETTE_LST = "palette.lst"; //$NON-NLS-1$
    public final static String filePLANBONI_BIN = "planboni.bin"; //$NON-NLS-1$
    public final static String filePLANET_PST = "planet.pst"; //$NON-NLS-1$
    public final static String filePLANSPAC_BIN = "planspac.bin"; //$NON-NLS-1$
    public final static String fileRACE_RST = "race.rst"; //$NON-NLS-1$
    public final static String fileRACEDESC_RST = "racedesc.rst"; //$NON-NLS-1$
    public final static String fileRACETECH_TEC = "racetech.tec"; //$NON-NLS-1$
    public final static String fileRCOLVAL_WTF = "rcolval.wtf"; //$NON-NLS-1$
    public final static String fileSHIPDESC_SST = "shipdesc.sst"; //$NON-NLS-1$
    public final static String fileSHIPLIST_SST = "shiplist.sst"; //$NON-NLS-1$
    public final static String fileSHIPNAME_BIN = "shipname.bin"; //$NON-NLS-1$
    public final static String fileSHIPRACE_SST = "shiprace.sst"; //$NON-NLS-1$
    public final static String fileSHIPTECH_SST = "shiptech.sst"; //$NON-NLS-1$
    public final static String fileSTARNAME_BIN = "starname.bin"; //$NON-NLS-1$
    public final static String fileTECFIELD_TEC = "tecfield.tec"; //$NON-NLS-1$
    public final static String fileTECHDESC_TEC = "techdesc.tec"; //$NON-NLS-1$
    public final static String fileTECHTREE_TEC = "techtree.tec"; //$NON-NLS-1$
    public final static String fileTEXTURE_LST = "texture.lst"; //$NON-NLS-1$
    public final static String fileINTEL_BIN = "intel.bin"; //$NON-NLS-1$

    // LIST OF AVAILABLE COMMANDS
    // These are retranslated on each getCommands call
    private static String cmdANIMATIONS = Language.getString("Stbof.0"); //$NON-NLS-1$
    private static String cmdBUILDING_STATS = Language.getString("Stbof.1"); //$NON-NLS-1$
    private static String cmdBUILDING_GROUP_EDIT = Language.getString("Stbof.2"); //$NON-NLS-1$
    private static String cmdCHECK = Language.getString("Stbof.3"); //$NON-NLS-1$
    private static String cmdCOLONY_VAL = Language.getString("Stbof.4"); //$NON-NLS-1$
    private static String cmdENVIRONMENTS = Language.getString("Stbof.6"); //$NON-NLS-1$
    private static String cmdFILES = Language.getString("Stbof.7"); //$NON-NLS-1$
    private static String cmdGAME_TEXT = Language.getString("Stbof.8"); //$NON-NLS-1$
    private static String cmdHOB_FILES = Language.getString("Stbof.9"); //$NON-NLS-1$
    private static String cmdLEXICON = Language.getString("Stbof.10"); //$NON-NLS-1$
    private static String cmdMAX_PLANET_POP = Language.getString("Stbof.11"); //$NON-NLS-1$
    private static String cmdME_SYSTEMS = Language.getString("Stbof.12"); //$NON-NLS-1$
    private static String cmdMINOR_ATTITUDES = Language.getString("Stbof.14"); //$NON-NLS-1$
    private static String cmdMODLIST = Language.getString("Stbof.15"); //$NON-NLS-1$
    private static String cmdMORALE_PENALTIES = Language.getString("Stbof.16"); //$NON-NLS-1$
    private static String cmdPLANET_BONUSES = Language.getString("Stbof.17"); //$NON-NLS-1$
    private static String cmdPLANETS = Language.getString("Stbof.18"); //$NON-NLS-1$
    private static String cmdRACES_INFO = Language.getString("Stbof.19"); //$NON-NLS-1$
    private static String cmdSHIP_GROUP_EDIT = Language.getString("Stbof.20"); //$NON-NLS-1$
    private static String cmdSHIP_NAMES = Language.getString("Stbof.21"); //$NON-NLS-1$
    private static String cmdSHIP_STATS = Language.getString("Stbof.22"); //$NON-NLS-1$
    private static String cmdSTAR_NAMES = Language.getString("Stbof.23"); //$NON-NLS-1$
    private static String cmdSTART_BUILDINGS = Language.getString("Stbof.24"); //$NON-NLS-1$
    private static String cmdSTART_TECH = Language.getString("Stbof.25"); //$NON-NLS-1$
    private static String cmdSTELLAR_OBJ = Language.getString("Stbof.26"); //$NON-NLS-1$
    private static String cmdTECH_FIELDS = Language.getString("Stbof.27"); //$NON-NLS-1$
    private static String cmdTEXTURES = Language.getString("Stbof.28"); //$NON-NLS-1$
    private static String cmdINSTALL_MODELS = Language.getString("Stbof.57"); //$NON-NLS-1$
    private static String cmdAI_BLD_REQ = Language.getString("Stbof.60"); //$NON-NLS-1$
    private static String cmdEDIFICE_REPORT = Language.getString("Stbof.62"); //$NON-NLS-1$
    private static String cmdSHIPLIST_REPORT = "Ships Overview";
    private static String cmdWDF_EDITOR = "Graphical User Interface";
    private static String cmdINTEL_MULT = "Intel multipliers";
    
    /**
     * This creates an instance of the class, but does not open a file.
     * To open the file use the open method.
     * DON'T USE THIS CLASS WITHOUT OPENING A FILE!
     */
    public Stbof()
    {
        // List of files
        // files are recognized by their index in filelist
        // don't modify it unless you add new entries to the end of it
        Stbof.FILELIST =  new String[]{Stbof.fileMORALE_BIN, Stbof.fileRACE_RST,
                Stbof.fileRACEDESC_RST, Stbof.fileAIMINOR_WTF, Stbof.fileSHIPNAME_BIN,
                Stbof.fileSTARNAME_BIN, Stbof.fileTEXTURE_LST, Stbof.fileLEXICON_DIC,
                Stbof.fileOBJSTRUC_SMT, Stbof.fileENVIRON_EST, Stbof.filePLANSPAC_BIN,
                Stbof.filePLANBONI_BIN, Stbof.filePLANET_PST, Stbof.fileMEPLANET_BIN,
                Stbof.fileTECHTREE_TEC, Stbof.fileTECHDESC_TEC, Stbof.fileTECFIELD_TEC,
                Stbof.fileEDIFBNFT_BST, Stbof.fileEDIFDESC_BST, Stbof.fileEDIFICE_BST,
                Stbof.fileBLDSET1_BIN, Stbof.fileBLDSET2_BIN, Stbof.fileBLDSET3_BIN,
                Stbof.fileBLDSET4_BIN, Stbof.fileBLDSET5_BIN, Stbof.fileRACETECH_TEC,
                Stbof.fileSHIPLIST_SST, Stbof.fileSHIPDESC_SST, Stbof.fileSHIPRACE_SST,
                Stbof.fileSHIPTECH_SST, Stbof.fileAIEMPIRE_WTF, Stbof.filePALETTE_LST,
                Stbof.fileCCOLVAL_WTF, Stbof.fileHCOLVAL_WTF, Stbof.fileFCOLVAL_WTF,
                Stbof.fileKCOLVAL_WTF, Stbof.fileRCOLVAL_WTF, Stbof.fileAIBLDREQ_BIN,
                Stbof.fileCONVERSE_BIN, Stbof.fileINTEL_BIN};
        
        // List of commands.
        // Same as filelist, only add to the end of the list.
        Stbof.COMMANDLIST = new String[]{Stbof.cmdMORALE_PENALTIES, Stbof.cmdRACES_INFO,
                Stbof.cmdMINOR_ATTITUDES, Stbof.cmdFILES, Stbof.cmdCHECK,
                Stbof.cmdSHIP_NAMES, Stbof.cmdSTAR_NAMES, Stbof.cmdLEXICON,
                Stbof.cmdSTELLAR_OBJ, Stbof.cmdANIMATIONS, Stbof.cmdENVIRONMENTS,
                Stbof.cmdMAX_PLANET_POP, Stbof.cmdPLANET_BONUSES, Stbof.cmdPLANETS,
                Stbof.cmdME_SYSTEMS, Stbof.cmdTECH_FIELDS, Stbof.cmdGAME_TEXT,
                Stbof.cmdBUILDING_STATS, Stbof.cmdSTART_BUILDINGS, Stbof.cmdSTART_TECH,
                Stbof.cmdMODLIST, Stbof.cmdSHIP_STATS,
                Stbof.cmdSHIP_GROUP_EDIT, Stbof.cmdTEXTURES, Stbof.cmdCOLONY_VAL,
                Stbof.cmdHOB_FILES, Stbof.cmdBUILDING_GROUP_EDIT,
                Stbof.cmdINSTALL_MODELS, Stbof.cmdAI_BLD_REQ, cmdEDIFICE_REPORT,
                cmdSHIPLIST_REPORT, cmdWDF_EDITOR, cmdINTEL_MULT};
        
        this.INTERNAL_FILES = new Hashtable<String, InternalFile>();
        
        if (UE.FILES.getFile(FileManager.P_TREK) != null)
        {   // if trek.exe is loaded
            this.TREK = (Trek)UE.FILES.getFile(FileManager.P_TREK);
        }
    }

    @Override
    public void markChanged()
    {
        this.CANCELLED_SAVE = true;
    }
    
    /**
     * Returns an enumeration of loaded files.
     */
    @Override
    public Enumeration<String> getListOfLoadedFiles()
    {
        return this.INTERNAL_FILES.keys();
    }
    
    /**
     * Returns an iterator of files to be removed/ignored in the next save.
     */
    @Override
    public Iterator<String> getRemovedFilesList()
    {
        return this.REMOVED.iterator();
    }
    
    /**
     * This adds a generic file to stbof.res. The file is saved in a hashset, therefore there can't be two files
     * with the same name added to the archive. Should a file with the same name already been added before it will
     * be overwritten. If the file is editable the one from internal files is removed (if present).
     * @param gf the GenericFile to be added.
     */
    @Override
    public void addInternalFile(InternalFile gf)
    {
        String msg;
        
        // Java is case sensitive, this is why all names must be converted to lower case
        // to make sure "Somefile" and "somefile" are equal.
        gf.setName(gf.getName().toLowerCase());
        gf.markChanged();
        
        if (gf.getName().length() > 12)
        {   // botf can't handle files with larger file names
            // 8 - name, 1 - dot, 3 - extension
            msg = Language.getString("Stbof.29"); //$NON-NLS-1$
            msg = msg.replace("%1", gf.getName()); //$NON-NLS-1$
            UE.displayError(new Exception(msg));
            return;
        }
	
        // we got GenericFile, however if the file is one of the known ones, we want it's class loaded
        // not the GenericFile, this check the name and reloads the file as the proper type
        int z = -1;
        
        for (int i = 0; i < Stbof.FILELIST.length; i++)
        {
            if (Stbof.FILELIST[i].equals(gf.getName()))
            {
                z = i;
            }
        }
        
        if (z != -1)
        { // known file
            try
            {
                ByteArrayInputStream bais = new ByteArrayInputStream(((GenericFile)gf).getData());
                InternalFile a = gf; // if an error occurs we'll still have the GenericFile
                
                switch (z)
                {
                    case 0:
                    {
                        a = new Morale(bais);
                        break;
                    }
                    case 1:
                    {
                        a = new Race(bais, this);
                        break;
                    }
                    case 2:
                    {
                        a = new StringFile(bais);
                        break;
                    }
                    case 3:
                    {
                        a = new Aiminor(bais);
                        break;
                    }
                    case 4:
                    {
                        a = new Shipname(bais);
                        break;
                    }
                    case 5:
                    {
                        a = new Starname(bais);
                        break;
                    }
                    case 6:
                    {
                        a = new Texture(bais, this);
                        break;
                    }
                    case 7:
                    {
                        a = new Lexicon(bais);
                        break;
                    }
                    case 8:
                    {
                        a = new Objstruc(bais, this);
                        break;
                    }
                    case 9:
                    {
                        a = new Environ(bais);
                        break;
                    }
                    case 10:
                    {
                        a = new Planspac(bais);
                        break;
                    }
                    case 11:
                    {
                        a = new Planboni(bais);
                        break;
                    }
                    case 12:
                    {
                        a = new Planet(bais, this);
                        break;
                    }
                    case 13:
                    {
                        a = new Meplanet(bais);
                        break;
                    }
                    case 14:
                    {
                        a = new TechTree(bais);
                        break;
                    }
                    case 15:
                    {
                        a = new StringFile(bais);
                        break;
                    }
                    case 16:
                    {
                        a = new TecField(bais);
                        break;
                    }
                    case 17:
                    {
                        a = new Edifbnft(bais);
                        break;
                    }
                    case 18:
                    {
                        a = new StringFile(bais);
                        break;
                    }
                    case 19:
                    {
                        a = new Edifice(bais, this);
                        break;
                    }
                    case 20:
                    {
                        a = new BldSet(bais);
                        break;
                    }
                    case 21:
                    {
                        a = new BldSet(bais);
                        break;
                    }
                    case 22:
                    {
                        a = new BldSet(bais);
                        break;
                    }
                    case 23:
                    {
                        a = new BldSet(bais);
                        break;
                    }
                    case 24:
                    {
                        a = new BldSet(bais);
                        break;
                    }
                    case 25:
                    {
                        a = new RaceTech(bais, (Edifice)this.getInternalFile(Stbof.fileEDIFICE_BST, true));
                        break;
                    }
                    case 26:
                    {
                        a = new ShipList(bais, this);
                        break;
                    }
                    case 27:
                    {
                        a = new StringFile(bais);
                        break;
                    }
                    case 28:
                    {
                        a = new ShipRace(bais);
                        break;
                    }
                    case 29:
                    {
                        a = new ShipTech(bais);
                        break;
                    }
                    case 30:
                    {
                        a = new Aiempire(bais);
                        break;
                    }
                    case 31:
                    {
                        a = new PaletteList(bais);
                        break;
                    }
                    case 32:
                    case 33:
                    case 34:
                    case 35:
                    case 36:
                    {
                        a = new Colval(bais);
                        break;
                    }
                    case 37:
                    {
                        a = new AIBldReq(bais);
                        break;
                    }
                    case 38:
                    {
                        a = new Converse(bais);
                        break;
                    }
                    case 39:
                    {
                        a = new Intel(bais);
                        break;
                    }
                }
                
                // gf name is already lower case
                a.setName(gf.getName());

                // added files are considered changed, to make sure the save dialog pops up
                a.markChanged();
                this.INTERNAL_FILES.put(a.getName(), a);
                bais.close();
            }
            catch (Exception u)
            {
                // should not happen
                UE.displayError(u);
            }
        }
        else
        {
            // if this is not a known file
            this.INTERNAL_FILES.put(gf.getName(), gf);
            files.add(gf.getName());
            gf.markChanged();
        }
        
        if (this.REMOVED.contains(gf.getName()))
        {
            // remove from to be removed list in case it is listed there
            this.REMOVED.remove(gf.getName());
        }
    }
    
    /**
     * This removes the specified file from the generic files which were added.
     * If the provided file is not present in the list, no action is taken.
     * @param name the name of the file to remove.
     * Note: All files are identified by name as there can't be two files
     * with the same name in stbof.res
     */
    @Override
    public void removeInternalFile(String name)
    {
        name = name.toLowerCase();
        this.INTERNAL_FILES.remove(name);
        this.MODS.remove(name);
        this.REMOVED.add(name);
        files.remove(name);
    }
    
    /**
     * Call this function to discard changes.
     */
    @Override
    public void discard()
    {
        this.REMOVED.clear();
        this.INTERNAL_FILES.clear();

        ZipFile zf = null;

        try
        {
            zf = new ZipFile(this.STBOF_FILE);

            Enumeration<? extends ZipEntry> entries = zf.entries();

            while (entries.hasMoreElements())
            {
                files.add(entries.nextElement().getName().toLowerCase());
            }
            files.trimToSize();

            zf.close();
        }
        catch (Exception e)
        {
            if (zf != null)
            {
                try
                {
                    zf.close();
                }
                catch (IOException ex)
                {
                }
            }
            UE.displayError(e);
        }
    }
    
    /**
     * @param command Must be one of the options returned by the open function else null is returned.
     * @return MainPanel object or null
     */
    @Override
    public MainPanel edit(String command) throws Exception
    {
        int z = -1;

        for (int i = 0; i < Stbof.COMMANDLIST.length; i++)
        {
            if (command.equals(Stbof.COMMANDLIST[i]))
            {
                z = i;
            }
        }

        if (z == -1)
        {
            // command not found
            return null;
        }

        switch (z)
        {
            case 0:
            {
                return new MoraleGUI((Morale)this.getInternalFile(Stbof.fileMORALE_BIN, true),
                        (Race)this.getInternalFile(Stbof.fileRACE_RST, true));
            }
            case 1:
            {
                return new RaceInfoGUI(this);
            }
            case 2:
            {
                return new AiminorGUI((Aiminor)this.getInternalFile(Stbof.fileAIMINOR_WTF, true),
                        (Race)this.getInternalFile(Stbof.fileRACE_RST, true),
                        (Lexicon)this.getInternalFile(Stbof.fileLEXICON_DIC, true));
            }
            case 3:
            {
                return new FilesGUI(new FilesInterface(this));
            }
            case 4:
            {
                return new CheckGUI(this.check());
            }
            case 5:
            {
                return new ShipnameGUI((Shipname)this.getInternalFile(Stbof.fileSHIPNAME_BIN, true));
            }
            case 6:
            {
                return new StarnameGUI((Starname)this.getInternalFile(Stbof.fileSTARNAME_BIN, true));
            }
            case 7:
            {
                return new LexiconGUI((Lexicon)this.getInternalFile(Stbof.fileLEXICON_DIC, true));
            }
            case 8:
            {
                return new ObjstrucGUI((Objstruc)this.getInternalFile(Stbof.fileOBJSTRUC_SMT, true),
                        (Lexicon)this.getInternalFile(Stbof.fileLEXICON_DIC, true));
            }
            case 9:
            {
                return new AniGUI(this);
            }
            case 10:
            {
                return new EnvironGUI((Environ)this.getInternalFile(Stbof.fileENVIRON_EST, true));
            }
            case 11:
            {
                return new MaxPopOrBonGUI((Planspac)this.getInternalFile(Stbof.filePLANSPAC_BIN, true),
                        (Environ)this.getInternalFile(Stbof.fileENVIRON_EST, true));
            }
            case 12:
            {
                return new MaxPopOrBonGUI((Planboni)this.getInternalFile(Stbof.filePLANBONI_BIN, true),
                        (Environ)this.getInternalFile(Stbof.fileENVIRON_EST, true));
            }
            case 13:
            {
                return new PlanetGUI((Planet)this.getInternalFile(Stbof.filePLANET_PST, true),
                        (Planspac)this.getInternalFile(Stbof.filePLANSPAC_BIN, true),
                        (Planboni)this.getInternalFile(Stbof.filePLANBONI_BIN, true),
                        this, (Environ)this.getInternalFile(Stbof.fileENVIRON_EST, true));
            }
            case 14:
            {
                return new MeplanetGUI((Meplanet)this.getInternalFile(Stbof.fileMEPLANET_BIN, true));
            }
            case 15:
            {
                return new TechGUI(this);
            }
            case 16:
            {
                return new InGameTextGUI(this);
            }
            case 17:
            {
                return new BuildingsGUI(this, this.TREK);
            }
            case 18:
            {
                return new BldSetGUI(this);
            }
            case 19:
            {
                return new StartConGUI(this);
            }
            case 20:
            {
                // update modified files list first
                Enumeration<InternalFile> itg = this.INTERNAL_FILES.elements();
                InternalFile gef;
                while (itg.hasMoreElements())
                {
                    gef = itg.nextElement();

                    if (this.REMOVED.contains(gef.getName()))
                    {
                        continue;
                    }

                    if (gef.getName().equals("mod.lst")) { //$NON-NLS-1$
                        continue;
                    }
                    if (this.ADD_CHANGES_2_MODLIST && gef.madeChanges())
                    {
                        this.MODS.add(gef.getName());
                    }
                }
                // disable main window
                UE.WINDOW.setEnabled(false);
                new ModifiedFilesGUI(this);
                // this is a seperate window not panel
                throw new NotAPanel();
            }
            case 21:
            {
                return new ShipGUI(this, this.TREK);
            }
            case 22:
            {
                return new ShipMultiGUI(this);
            }
            case 23:
            {
                return new TextureGUI(this, (Texture)this.getInternalFile(Stbof.fileTEXTURE_LST, true),
                        (PaletteList)this.getInternalFile(Stbof.filePALETTE_LST, true));
            }
            case 24:
            {
                return new ColValGUI(this);
            }
            case 25:
            {
                return new HobGUI(this, (Texture)this.getInternalFile(Stbof.fileTEXTURE_LST, true));
            }
            case 26:
            {
                return new BuildingsMultiSetGUI(this);
            }
            case 27:
            {
                return new ShipModelGUI(this, this.TREK);
            }
            case 28:
            {
                return new AIBldReqGUI((AIBldReq)this.getInternalFile(Stbof.fileAIBLDREQ_BIN, true));
            }
            case 29:
                return new HtmlReportGUI(
                        (Edifice)this.getInternalFile(Stbof.fileEDIFICE_BST, true),
                        (Edifbnft)this.getInternalFile(Stbof.fileEDIFBNFT_BST, true),
                        (Race)this.getInternalFile(Stbof.fileRACE_RST, true),
                        (Lexicon)this.getInternalFile(Stbof.fileLEXICON_DIC, true),
                        (TecField)this.getInternalFile(Stbof.fileTECFIELD_TEC, true)
                );
            case 30:
                return new HtmlReportGUI(
                        (ShipList)this.getInternalFile(Stbof.fileSHIPLIST_SST, true),
                        (ShipTech)this.getInternalFile(Stbof.fileSHIPTECH_SST, true),
                        (Race)this.getInternalFile(Stbof.fileRACE_RST, true),
                        (Lexicon)this.getInternalFile(Stbof.fileLEXICON_DIC, true)
                );
            case 31:
                return new WDFEditorGUI(this);
            case 32:
                return new IntelGUI(this);
        }

        return null;
    }
    
    /**
     * Saves changes to the specified file.
     * @param whereTo The file to which changes are to be saved.
     * If the file already exists, it is overwritten without confirmation.
     * @return true if succesful else false.
     */
    @Override
    public boolean save(File whereTo, boolean bkp, String ext)
    {
        if (whereTo == null)
        { // means save to the open file
            whereTo = new File(this.STBOF_FILE.getAbsolutePath());
        }
        
        SaveTask TSK = new SaveTask(whereTo, bkp, ext);
	
        this.CANCELLED_SAVE = !(Boolean)Tools.runUEWorker(TSK);

        if (!CANCELLED_SAVE)
        {
            this.STBOF_FILE = whereTo;
        }
        
        return !this.CANCELLED_SAVE;
    }
    
    /**
     * Opens the specified file and tests it, if it is a valid stbof.res archive.
     * Also it checks whether stbof contains all editable files.
     * Should a file be missing or anything else be wrong an error will be reported and null returned.
     * @param whereFrom The file to open. Must be a valid stbof.res file!
     * @return a String array of valid command that can be used with the edit function
     * or null if an error occured.
     */
    @Override
    public StringGroup[] open(File whereFrom)
    {
        String msg;
        this.STBOF_FILE = whereFrom;

        ZipFile zf = null;
        
        try
        {
            zf = new ZipFile(this.STBOF_FILE);
            
            for (String element : Stbof.FILELIST)
            {
                if (zf.getEntry(element) == null)
                {
                    msg = Language.getString("Stbof.32"); //$NON-NLS-1$
                    msg = msg.replace("%1", element); //$NON-NLS-1$
                    UE.displayError(new CorruptedFile(msg));
                    zf.close();
                    this.STBOF_FILE = null;
                    return null;
                }
            }
            
            // load modlist or create an empty one
            ZipEntry ze = zf.getEntry("mod.lst"); //$NON-NLS-1$
            if (ze != null)
            {
                this.MODS = new ModList(new ZipReader(zf.getInputStream(ze)));
                
                if (this.MODS == null)
                {
                    UE.displayError(new Exception(Language.getString("Stbof.33"))); //$NON-NLS-1$
                    this.MODS = new ModList();
                }
            }
            else
            {
                this.MODS = new ModList();
            }

            Enumeration<? extends ZipEntry> entries = zf.entries();

            while (entries.hasMoreElements())
            {
                files.add(entries.nextElement().getName().toLowerCase());
            }
            files.trimToSize();

            zf.close();
        }
        catch (Exception e)
        {
            if (zf != null)
            {
                try
                {
                    zf.close();
                }
                catch (IOException ex)
                {
                }
            }

            UE.displayError(e);
            this.STBOF_FILE = null;
            return null;
        }

        return this.getCommands();
    }
    
    /**
     * @return a String array of available commands that can be used with the edit function.
     */
    @Override
    public StringGroup[] getCommands()
    {
        if (this.STBOF_FILE == null)
        { // file not open
            return null;
        }
        
        Stbof.cmdANIMATIONS = Language.getString("Stbof.0"); //$NON-NLS-1$
        Stbof.cmdBUILDING_STATS = Language.getString("Stbof.1"); //$NON-NLS-1$
        Stbof.cmdBUILDING_GROUP_EDIT = Language.getString("Stbof.2"); //$NON-NLS-1$
        Stbof.cmdCHECK = Language.getString("Stbof.3"); //$NON-NLS-1$
        Stbof.cmdCOLONY_VAL = Language.getString("Stbof.4"); //$NON-NLS-1$
        Stbof.cmdENVIRONMENTS = Language.getString("Stbof.6"); //$NON-NLS-1$
        Stbof.cmdFILES = Language.getString("Stbof.7"); //$NON-NLS-1$
        Stbof.cmdGAME_TEXT = Language.getString("Stbof.8"); //$NON-NLS-1$
        Stbof.cmdHOB_FILES = Language.getString("Stbof.9"); //$NON-NLS-1$
        Stbof.cmdLEXICON = Language.getString("Stbof.10"); //$NON-NLS-1$
        Stbof.cmdMAX_PLANET_POP = Language.getString("Stbof.11"); //$NON-NLS-1$
        Stbof.cmdME_SYSTEMS = Language.getString("Stbof.12"); //$NON-NLS-1$
        Stbof.cmdMINOR_ATTITUDES = Language.getString("Stbof.14"); //$NON-NLS-1$
        Stbof.cmdMODLIST = Language.getString("Stbof.15"); //$NON-NLS-1$
        Stbof.cmdMORALE_PENALTIES = Language.getString("Stbof.16"); //$NON-NLS-1$
        Stbof.cmdPLANET_BONUSES = Language.getString("Stbof.17"); //$NON-NLS-1$
        Stbof.cmdPLANETS = Language.getString("Stbof.18"); //$NON-NLS-1$
        Stbof.cmdRACES_INFO = Language.getString("Stbof.19"); //$NON-NLS-1$
        Stbof.cmdSHIP_GROUP_EDIT = Language.getString("Stbof.20"); //$NON-NLS-1$
        Stbof.cmdSHIP_NAMES = Language.getString("Stbof.21"); //$NON-NLS-1$
        Stbof.cmdSHIP_STATS = Language.getString("Stbof.22"); //$NON-NLS-1$
        Stbof.cmdSTAR_NAMES = Language.getString("Stbof.23"); //$NON-NLS-1$
        Stbof.cmdSTART_BUILDINGS = Language.getString("Stbof.24"); //$NON-NLS-1$
        Stbof.cmdSTART_TECH = Language.getString("Stbof.25"); //$NON-NLS-1$
        Stbof.cmdSTELLAR_OBJ = Language.getString("Stbof.26"); //$NON-NLS-1$
        Stbof.cmdTECH_FIELDS = Language.getString("Stbof.27"); //$NON-NLS-1$
        Stbof.cmdTEXTURES = Language.getString("Stbof.28"); //$NON-NLS-1$
        Stbof.cmdINSTALL_MODELS = Language.getString("Stbof.57"); //$NON-NLS-1$
        Stbof.cmdAI_BLD_REQ = Language.getString("Stbof.60"); //$NON-NLS-1$
        cmdEDIFICE_REPORT = Language.getString("Stbof.62"); //$NON-NLS-1$
        cmdSHIPLIST_REPORT = "Ships Overview";
        cmdWDF_EDITOR = "Graphical User Interface";
        cmdINTEL_MULT = "Intel Multipliers";

        Stbof.COMMANDLIST = new String[]{Stbof.cmdMORALE_PENALTIES, Stbof.cmdRACES_INFO,
                Stbof.cmdMINOR_ATTITUDES, Stbof.cmdFILES, Stbof.cmdCHECK,
                Stbof.cmdSHIP_NAMES, Stbof.cmdSTAR_NAMES, Stbof.cmdLEXICON,
                Stbof.cmdSTELLAR_OBJ, Stbof.cmdANIMATIONS, Stbof.cmdENVIRONMENTS,
                Stbof.cmdMAX_PLANET_POP, Stbof.cmdPLANET_BONUSES, Stbof.cmdPLANETS,
                Stbof.cmdME_SYSTEMS, Stbof.cmdTECH_FIELDS, Stbof.cmdGAME_TEXT,
                Stbof.cmdBUILDING_STATS, Stbof.cmdSTART_BUILDINGS, Stbof.cmdSTART_TECH,
                Stbof.cmdMODLIST, Stbof.cmdSHIP_STATS,
                Stbof.cmdSHIP_GROUP_EDIT, Stbof.cmdTEXTURES, Stbof.cmdCOLONY_VAL,
                Stbof.cmdHOB_FILES, Stbof.cmdBUILDING_GROUP_EDIT,
                Stbof.cmdINSTALL_MODELS, Stbof.cmdAI_BLD_REQ, cmdEDIFICE_REPORT,
                cmdSHIPLIST_REPORT, cmdWDF_EDITOR, cmdINTEL_MULT};

        StringGroup[] ret = new StringGroup[11];
        try{
            // Graphics
            ret[0] = new StringGroup(Language.getString("Stbof.34"), //$NON-NLS-1$
                    new String[]{Stbof.cmdANIMATIONS, Stbof.cmdTEXTURES, Stbof.cmdHOB_FILES,
                    Stbof.cmdINSTALL_MODELS, Stbof.cmdWDF_EDITOR});
            // Starting Conditions
            ret[1] = new StringGroup(Language.getString("Stbof.35"), //$NON-NLS-1$
                    new String[]{Stbof.cmdSTART_BUILDINGS, Stbof.cmdSTART_TECH, Stbof.cmdME_SYSTEMS});
            // Text
            ret[2] = new StringGroup(Language.getString("Stbof.36"), //$NON-NLS-1$
                    new String[]{Stbof.cmdLEXICON, Stbof.cmdGAME_TEXT});
            // Galaxy
            ret[3] = new StringGroup(Language.getString("Stbof.37"), //$NON-NLS-1$
                    new String[]{Stbof.cmdPLANETS, Stbof.cmdPLANET_BONUSES,
                                Stbof.cmdMAX_PLANET_POP, Stbof.cmdENVIRONMENTS,
                                Stbof.cmdSTELLAR_OBJ, Stbof.cmdSTAR_NAMES});
            // AI
            ret[4] = new StringGroup(Language.getString("Stbof.38"), //$NON-NLS-1$
                    new String[]{Stbof.cmdMINOR_ATTITUDES,
                                Stbof.cmdCOLONY_VAL, Stbof.cmdAI_BLD_REQ});
            // Technology
            ret[5] = new StringGroup(Language.getString("Stbof.39"), //$NON-NLS-1$
                    new String[]{Stbof.cmdTECH_FIELDS});
            // Ships
            ret[6] = new StringGroup(Language.getString("Stbof.40"), //$NON-NLS-1$
                    new String[]{Stbof.cmdSHIP_NAMES, Stbof.cmdSHIP_STATS,
                                Stbof.cmdSHIP_GROUP_EDIT, cmdSHIPLIST_REPORT});
            // Buildings
            ret[7] = new StringGroup(Language.getString("Stbof.41"), //$NON-NLS-1$
                    new String[]{Stbof.cmdBUILDING_STATS, Stbof.cmdBUILDING_GROUP_EDIT,
                    cmdEDIFICE_REPORT});
            // Races
            ret[8] = new StringGroup(Language.getString("Stbof.42"), //$NON-NLS-1$
                    new String[]{Stbof.cmdRACES_INFO, Stbof.cmdMORALE_PENALTIES});
            // Other
            ret[9] = new StringGroup(Language.getString("Stbof.43"), //$NON-NLS-1$
                    new String[]{Stbof.cmdFILES, Stbof.cmdMODLIST, Stbof.cmdCHECK});
            // Intel
            ret[10] = new StringGroup("Intel",
                    new String[]{cmdINTEL_MULT});
        }
        catch (Exception tz)
        {
            UE.displayError(tz);
            return null;
        }
        return ret;
    }

    /**
     * Returns true if the ship graphics string is being used in stbof.res
     * @throws IOException 
     * @throws ZipException 
     */
    // TODO: make this method accept arguments of type String[] for checking multiple strings at the same time
    public boolean isShipGraphicsStringUsed(String str) throws ZipException, IOException
    {
        str = str.toLowerCase();
        
        ArrayList<String> array = new ArrayList<String>();
        
        array.add(str + "_a.hob"); //$NON-NLS-1$
        array.add(str + "_b.hob"); //$NON-NLS-1$
        array.add(str + "_c.hob"); //$NON-NLS-1$
        array.add("i_" + str + "w.tga"); //$NON-NLS-1$ //$NON-NLS-2$
        array.add("i_" + str + "60.tga"); //$NON-NLS-1$ //$NON-NLS-2$
        array.add("i_" + str + "30.tga"); //$NON-NLS-1$ //$NON-NLS-2$
        array.add("i_" + str + "120.tga"); //$NON-NLS-1$ //$NON-NLS-2$
        array.add("i_" + str + "170.tga"); //$NON-NLS-1$ //$NON-NLS-2$
            
        for (int i = 0; i < array.size(); i++)
        {
            if (this.REMOVED.contains(array.get(i)))
            {
                array.remove(i);
                i--;
            }
            else if (this.INTERNAL_FILES.containsKey(array.get(i)))
            {
                return true;
            }   
        }
        
        for (String img : array)
        {
            if (this.hasInternalFile(img))
            {
                return true;
            }
        }
        
        return false;
    }
    
    /**
     * @return unused ship graphics string
     * @throws IOException 
     * @throws ZipException 
     */
    public String getUnusedShipGraphicsString() throws ZipException, IOException
    {
        int i = 0;          // try numbers first
        char c = 'A';
        String g = Integer.toString(i++);
        
        while ((c <= 'Z') && (this.isShipGraphicsStringUsed(g) || this.isShipGraphicsStringUsed(g + "_") //$NON-NLS-1$
                || this.isShipGraphicsStringUsed(g + "__"))) //$NON-NLS-1$
        { // search from 0-9 and A to Z
            if (i < 10)
            {
                g = Integer.toString(i++);
            }
            else
            {
                g = Character.toString(c++);
            }
        }
        
        if (c > 'Z')
        {   // search from 00 to 99 and AA to ZZ
            i = 0;
            int j = 0;
            c = 'A';
            char d = 'A';
            
            g = Integer.toString(i) + Integer.toString(j);
            
            while ((c <= 'Z') && (this.isShipGraphicsStringUsed(g) || this.isShipGraphicsStringUsed(g + "_"))) //$NON-NLS-1$
            {
                if (i < 10)
                {
                    g = Integer.toString(i) + Integer.toString(j++);
                    
                    if (j == 10)
                    {
                        j = 0;
                        i++;
                    }
                }
                else
                {
                    g = Character.toString(c) + Character.toString(d++);
                    
                    if (d == 'Z')
                    {
                        d = 'A';
                        c++;
                    }
                }
            }
        }
        
        if (c <= 'Z')
        {
            return g;
        }
        else
        {
            return null; // none found
        }
    }
    
    /**
     * Renames model graphic files and entries in shiplist.sst.
     * @param old old graphics string
     * @param now new graphics string
     * @return true if successful
     * @throws Exception
     */
    public boolean renameModelGraphicFiles(String old, String now) throws Exception
    {
        now = now.toLowerCase();
        old = old.toLowerCase();
        
        ShipList SHIPS = (ShipList)this.getInternalFile(Stbof.fileSHIPLIST_SST, true);
        
        // check
        if (this.isShipGraphicsStringUsed(now))
        {
            return false;
        }
        
        for (int i = 0; i < SHIPS.getNumberOfEntries(); i++)
        {
            try
            {
                if (SHIPS.getImageName(i).equals(old))
                {
                    SHIPS.setImageName(i, now);
                }
            }
            catch (InvalidArgumentsException e)
            {
                UE.displayError(e);
            }
        }
        
        InternalFile file;
        String sfile;
        char c = 'a';
        
        while( c <= 'c')
        {
            try
            {
                sfile = old + "_" + c + ".hob"; //$NON-NLS-1$ //$NON-NLS-2$
                file = this.getInternalFile(sfile, false);
                this.removeInternalFile(sfile);
                file.setName(now + "_" + c + ".hob"); //$NON-NLS-1$ //$NON-NLS-2$
                this.addInternalFile(file);
            }
            catch (FileNotFoundException fnf){}
            c++;
        }
        
        try
        {
            sfile = "i_" + old + "w.tga"; //$NON-NLS-1$ //$NON-NLS-2$
            file = this.getInternalFile(sfile, false);
            this.removeInternalFile(sfile);
            file.setName("i_" + now + "w.tga"); //$NON-NLS-1$ //$NON-NLS-2$
            this.addInternalFile(file);
        }
        catch (FileNotFoundException fnf){}
        
        try
        {
            sfile = "i_" + old + "30.tga"; //$NON-NLS-1$ //$NON-NLS-2$
            file = this.getInternalFile(sfile, false);
            this.removeInternalFile(sfile);
            file.setName("i_" + now + "30.tga"); //$NON-NLS-1$ //$NON-NLS-2$
            this.addInternalFile(file);
        }
        catch (FileNotFoundException fnf){}
        
        try
        {
            sfile = "i_" + old + "60.tga"; //$NON-NLS-1$ //$NON-NLS-2$
            file = this.getInternalFile(sfile, false);
            this.removeInternalFile(sfile);
            file.setName("i_" + now + "60.tga"); //$NON-NLS-1$ //$NON-NLS-2$
            this.addInternalFile(file);
        }
        catch (FileNotFoundException fnf){}
        
        try
        {
            sfile = "i_" + old + "120.tga"; //$NON-NLS-1$ //$NON-NLS-2$
            file = this.getInternalFile(sfile, false);
            this.removeInternalFile(sfile);
            file.setName("i_" + now + "120.tga"); //$NON-NLS-1$ //$NON-NLS-2$
            this.addInternalFile(file);
        }
        catch (FileNotFoundException fnf){}
        
        try
        {
            sfile = "i_" + old + "170.tga"; //$NON-NLS-1$ //$NON-NLS-2$
            file = this.getInternalFile(sfile, false);
            this.removeInternalFile(sfile);
            file.setName("i_" + now + "170.tga"); //$NON-NLS-1$ //$NON-NLS-2$
            this.addInternalFile(file);
        }
        catch (FileNotFoundException fnf){}
        
        return true;
    }
    
    /**
     * Returns the internal file
     */
    @Override
    public InternalFile getInternalFile(String fileName, boolean cache) throws FileNotFoundException
    {
        String msg;
        
        fileName = fileName.toLowerCase();
        
        if (this.REMOVED.contains(fileName))
        {
            msg = Language.getString("Stbof.44"); //$NON-NLS-1$
            msg = msg.replace("%1", fileName); //$NON-NLS-1$
            throw new FileNotFoundException(msg);
        }
        
        if (this.INTERNAL_FILES.containsKey(fileName))
        {
            return this.INTERNAL_FILES.get(fileName);
        }
        else
        {
            InternalFile a = null;
            int z = -1;
            
            for (int i = 0; i < Stbof.FILELIST.length; i++)
            {
                if (Stbof.FILELIST[i].equals(fileName))
                {
                    z = i;
                    break;
                }
            }
            
            try
            {
                ZipFile zf = new ZipFile(this.STBOF_FILE);
                
                Enumeration<? extends ZipEntry> zenum = zf.entries();
                ZipEntry ze = null;
                
                while (zenum.hasMoreElements())
                {
                    ze = zenum.nextElement();
                    if (fileName.equals(ze.getName().toLowerCase()))
                    {
                        break;
                    }
                }
                
                if ((ze == null) || !fileName.equals(ze.getName().toLowerCase()))
                {
                    zf.close();
                    msg = Language.getString("Stbof.44"); //$NON-NLS-1$
                    msg = msg.replace("%1", fileName); //$NON-NLS-1$
                    throw new FileNotFoundException(msg);
                }
                
                switch (z)
                {
                    case 0:
                    {
                        a = new Morale(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 1:
                    {
                        a = new Race(new ZipReader(zf.getInputStream(ze)), this);
                        break;
                    }
                    case 2:
                    {
                        a = new StringFile(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 3:
                    {
                        a = new Aiminor(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 4:
                    {
                        a = new Shipname(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 5:
                    {
                        a = new Starname(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 6:
                    {
                        a = new Texture(new ZipReader(zf.getInputStream(ze)), this);
                        break;
                    }
                    case 7:
                    {
                        a = new Lexicon(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 8:
                    {
                        a = new Objstruc(new ZipReader(zf.getInputStream(ze)), this);
                        break;
                    }
                    case 9:
                    {
                        a = new Environ(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 10:
                    {
                        a = new Planspac(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 11:
                    {
                        a = new Planboni(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 12:
                    {
                        a = new Planet(new ZipReader(zf.getInputStream(ze)), this);
                        break;
                    }
                    case 13:
                    {
                        a = new Meplanet(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 14:
                    {
                        a = new TechTree(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 15:
                    {
                        a = new StringFile(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 16:
                    {
                        a = new TecField(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 17:
                    {
                        a = new Edifbnft(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 18:
                    {
                        a = new StringFile(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 19:
                    {
                        a = new Edifice(new ZipReader(zf.getInputStream(ze)), this);
                        break;
                    }
                    case 20:
                    {
                        a = new BldSet(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 21:
                    {
                        a = new BldSet(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 22:
                    {
                        a = new BldSet(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 23:
                    {
                        a = new BldSet(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 24:
                    {
                        a = new BldSet(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 25:
                    {
                        a = new RaceTech(new ZipReader(zf.getInputStream(ze)),
                                (Edifice)this.getInternalFile(Stbof.fileEDIFICE_BST, true));
                        break;
                    }
                    case 26:
                    {
                        a = new ShipList(new ZipReader(zf.getInputStream(ze)), this);
                        break;
                    }
                    case 27:
                    {
                        a = new StringFile(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 28:
                    {
                        a = new ShipRace(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 29:
                    {
                        a = new ShipTech(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 30:
                    {
                        a = new Aiempire(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 31:
                    {
                        a = new PaletteList(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 32:
                    {
                        a = new Colval(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 33:
                    {
                        a = new Colval(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 34:
                    {
                        a = new Colval(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 35:
                    {
                        a = new Colval(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 36:
                    {
                        a = new Colval(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 37:
                    {
                        a = new AIBldReq(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 38:
                    {
                        a = new Converse(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    case 39:
                    {
                        a = new Intel(new ZipReader(zf.getInputStream(ze)));
                        break;
                    }
                    default:
                    {
                        InputStream inu = new ZipReader(zf.getInputStream(ze));
                        byte[] bib = new byte[inu.available()];
                        int len = 0;
                        int k = 0;
                        while ((len = inu.read(bib, k, inu.available())) > 0)
                        {
                            k +=len;
                        }
                        a = new GenericFile("", bib); //$NON-NLS-1$
                        a.setName(ze.getName().toLowerCase());
                        zf.close();
                        if (a==null)
                        {
                            msg = Language.getString("Stbof.44"); //$NON-NLS-1$
                            msg = msg.replace("%1", fileName); //$NON-NLS-1$
                            throw new FileNotFoundException(msg);
                        }
                        return a;
                    }
                }
                
                a.setName(ze.getName().toLowerCase());

                if (cache) {
                    this.INTERNAL_FILES.put(a.getName().toLowerCase(), a);
                }
                
                zf.close();
            }
            catch (FileNotFoundException e)
            {
                throw e;
            }
            catch (Exception e)
            {
                UE.displayError(e);
            }
            
            if (a == null)
            {
                msg = Language.getString("Stbof.44"); //$NON-NLS-1$
                msg = msg.replace("%1", fileName); //$NON-NLS-1$
                throw new FileNotFoundException(msg);
            }
            
            return a;
        }
    }
    
    /**
     * Returns true if internal file exists in the archive.
     */
    public boolean hasInternalFile(String fileName)
    {
        fileName = fileName.toLowerCase();
        
        if (this.REMOVED.contains(fileName))
        {
            return false;
        }
        
        if (this.INTERNAL_FILES.containsKey(fileName))
        {
            return true;
        }
        else
        {
            return files.contains(fileName);
        }
    }	
    
    /**
     * @return true if changes were made to the opened file and false if not.
     */
    @Override
    public boolean madeChanges()
    {
        if (!this.REMOVED.isEmpty() || this.CANCELLED_SAVE)
        {
            return true;
        }
        
        for (Enumeration<InternalFile> en = this.INTERNAL_FILES.elements(); en.hasMoreElements();)
        {
            InternalFile se = en.nextElement();
            
            if (se.madeChanges())
            {
                if (UE.DEBUG_MODE)
                {
                    System.out.println(se.getName() + ".madeChanges == TRUE");
                }

                return true;
            }
        }

        return false;
    }
    
    /**
     * Loads an animation represented by the AniOrCur class.
     * @param name the name of the animation file to load and return.
     * @return an animation loaded from specified file, null if the animation can't be loaded.
     * @throws FileNotFoundExeption if the file can't be found.
     */
    public AniOrCur getAnimation(String name, boolean cache) throws FileNotFoundException
    {
        String msg;
        
        name = name.toLowerCase();
        
        if ((name==null) || this.REMOVED.contains(name))
        {
            msg = Language.getString("Stbof.44"); //$NON-NLS-1$
            msg = msg.replace("%1", name); //$NON-NLS-1$
            throw new FileNotFoundException(msg);
        }
        
        InputStream in = null;
        AniOrCur img = null;
        
        try
        {
            if (!this.INTERNAL_FILES.containsKey(name))
            {
                ZipFile zf = new ZipFile(this.STBOF_FILE);
                
                Enumeration<? extends ZipEntry> zenum = zf.entries();
                ZipEntry ze = null;
                
                while (zenum.hasMoreElements())
                {
                    ze = zenum.nextElement();
                    if (name.equals(ze.getName().toLowerCase()))
                    {
                        break;
                    }
                }
                
                if ((ze == null) || !name.equals(ze.getName().toLowerCase()))
                {
                    zf.close();
                    msg = Language.getString("Stbof.44"); //$NON-NLS-1$
                    msg = msg.replace("%1", name); //$NON-NLS-1$
                    throw new FileNotFoundException(msg);
                }
                
                in = new ZipReader(zf.getInputStream(ze));
                img = new AniOrCur(in, name);
                
                zf.close();

                if (cache)
                    INTERNAL_FILES.put(name, img);
            }
            else
            {
                InternalFile fimg = INTERNAL_FILES.get(name);

                if (fimg instanceof AniOrCur)
                {
                    img = (AniOrCur)fimg;
                }
                else
                {
                    GenericFile gun = (GenericFile)fimg;
                    ByteArrayInputStream baos = new ByteArrayInputStream(gun.getData());
                    img = new AniOrCur(baos, name);

                    if (gun.madeChanges())
                    {
                        img.markChanged();
                    }

                    INTERNAL_FILES.remove(name);
                    INTERNAL_FILES.put(name, img);
                }
            }
        }
        catch (FileNotFoundException fnf)
        {
            msg = Language.getString("Stbof.44"); //$NON-NLS-1$
            msg = msg.replace("%1", name); //$NON-NLS-1$
            throw new FileNotFoundException(msg);
        }
        catch (IOException e)
        {
            UE.displayError(e);
        }
        return img;
    }
    
    /**
     * Loads a wdf file represented by the WindowDefinitionFile object.
     * @param name the name of the file to load and return.
     * @throws FileNotFoundExeption if the file can't be found.
     */
    public WindowDefinitionFile getWDF(String name, boolean cache) throws FileNotFoundException
    {
        String msg;

        name = name.toLowerCase();

        if ((name==null) || this.REMOVED.contains(name))
        {
            msg = Language.getString("Stbof.44"); //$NON-NLS-1$
            msg = msg.replace("%1", name); //$NON-NLS-1$
            throw new FileNotFoundException(msg);
        }

        InputStream in = null;
        WindowDefinitionFile wdf = null;

        try
        {
            if (!this.INTERNAL_FILES.containsKey(name))
            {
                ZipFile zf = new ZipFile(this.STBOF_FILE);

                Enumeration<? extends ZipEntry> zenum = zf.entries();
                ZipEntry ze = null;

                while (zenum.hasMoreElements())
                {
                    ze = zenum.nextElement();
                    if (name.equals(ze.getName().toLowerCase()))
                    {
                        break;
                    }
                }

                if ((ze == null) || !name.equals(ze.getName().toLowerCase()))
                {
                    zf.close();
                    msg = Language.getString("Stbof.44"); //$NON-NLS-1$
                    msg = msg.replace("%1", name); //$NON-NLS-1$
                    throw new FileNotFoundException(msg);
                }

                in = new ZipReader(zf.getInputStream(ze));
                wdf = new WindowDefinitionFile(in);
                wdf.setName(name);

                zf.close();

                if (cache)
                    INTERNAL_FILES.put(name, wdf);
            }
            else
            {
                InternalFile fwdf = INTERNAL_FILES.get(name);

                if (fwdf instanceof WindowDefinitionFile)
                {
                    wdf = (WindowDefinitionFile)fwdf;
                }
                else
                {
                    GenericFile gun = (GenericFile)fwdf;
                    ByteArrayInputStream baos = new ByteArrayInputStream(gun.getData());
                    wdf = new WindowDefinitionFile(baos);
                    wdf.setName(name);

                    if (gun.madeChanges())
                    {
                        wdf.markChanged();
                    }

                    INTERNAL_FILES.remove(name);
                    INTERNAL_FILES.put(name, wdf);
                }
            }
        }
        catch (FileNotFoundException fnf)
        {
            msg = Language.getString("Stbof.44"); //$NON-NLS-1$
            msg = msg.replace("%1", name); //$NON-NLS-1$
            throw new FileNotFoundException(msg);
        }
        catch (IOException e)
        {
            UE.displayError(e);
        }
        return wdf;
    }

    /**
     * Loads a font file represented by the TgaFont object.
     * @param name the name of the file to load and return.
     * @throws FileNotFoundExeption if the file can't be found.
     */
    public TgaFont getTgaFont(String name, boolean cache) throws FileNotFoundException
    {
        String msg;

        name = name.toLowerCase();

        if ((name==null) || this.REMOVED.contains(name))
        {
            msg = Language.getString("Stbof.44"); //$NON-NLS-1$
            msg = msg.replace("%1", name); //$NON-NLS-1$
            throw new FileNotFoundException(msg);
        }

        InputStream in = null;
        TgaFont font = null;

        try
        {
            if (!this.INTERNAL_FILES.containsKey(name))
            {
                ZipFile zf = new ZipFile(this.STBOF_FILE);

                Enumeration<? extends ZipEntry> zenum = zf.entries();
                ZipEntry ze = null;

                while (zenum.hasMoreElements())
                {
                    ze = zenum.nextElement();
                    if (name.equals(ze.getName().toLowerCase()))
                    {
                        break;
                    }
                }

                if ((ze == null) || !name.equals(ze.getName().toLowerCase()))
                {
                    zf.close();
                    msg = Language.getString("Stbof.44"); //$NON-NLS-1$
                    msg = msg.replace("%1", name); //$NON-NLS-1$
                    throw new FileNotFoundException(msg);
                }

                in = new ZipReader(zf.getInputStream(ze));
                font = new TgaFont(in);
                font.setName(name);

                zf.close();

                if (cache)
                    INTERNAL_FILES.put(name, font);
            }
            else
            {
                InternalFile ffont = INTERNAL_FILES.get(name);
                
                if (ffont instanceof TgaFont)
                {
                    font = (TgaFont)ffont;
                }
                else
                {
                    GenericFile gun = (GenericFile)ffont;
                    ByteArrayInputStream baos = new ByteArrayInputStream(gun.getData());
                    font = new TgaFont(baos);
                    font.setName(name);

                    if (gun.madeChanges())
                    {
                        font.markChanged();
                    }

                    INTERNAL_FILES.remove(name);
                    INTERNAL_FILES.put(name, font);
                }
            }
        }
        catch (FileNotFoundException fnf)
        {
            msg = Language.getString("Stbof.44"); //$NON-NLS-1$
            msg = msg.replace("%1", name); //$NON-NLS-1$
            throw new FileNotFoundException(msg);
        }
        catch (Exception e)
        {
            UE.displayError(e);
        }
        return font;
    }

    /**
     * @return a list of files located in stbof.res
     */
    @Override
    public Enumeration <String[]> getFiles(java.io.FilenameFilter fil)
    {
        Vector <String[]> result = new Vector<String[]>();

        String file;

        try
        {
            ZipFile zf = new ZipFile(this.STBOF_FILE);

            Enumeration<? extends ZipEntry> zenum = zf.entries();
            ZipEntry ze = null;

            while (zenum.hasMoreElements())
            {
                ze = zenum.nextElement();

                file = ze.getName().toLowerCase();

                if (file == null
                        || !fil.accept(STBOF_FILE, file)
                        || file.equals("mod.lst")
                        || this.REMOVED.contains(file)
                        || INTERNAL_FILES.containsKey(file)) //$NON-NLS-1$
                {
                    continue;
                }

                result.add(new String[]{file, Long.toString(ze.getSize())});
            }

            zf.close();
        }
        catch (Exception ex)
        {
            UE.displayError(ex);
        }

        //AND NOW THE REST
        Enumeration<InternalFile> itg = this.INTERNAL_FILES.elements();
        
        while (itg.hasMoreElements())
        {
            InternalFile gef = itg.nextElement();

            if (!fil.accept(STBOF_FILE, gef.getName()) ||
                    gef.getName().equals("mod.lst") || this.REMOVED.contains(gef.getName())) //$NON-NLS-1$
            {
                continue;
            }

            result.add(new String[]{gef.getName(), Long.toString(gef.getSize())});
        }
        
        return result.elements();
    }
    
    /**
     * @return a list of file names located in stbof.res
     */
    public ArrayList<String> getFileList(java.io.FilenameFilter fil)
    {
        ArrayList <String> result = new ArrayList <String>();

        String file;

        try
        {
            ZipFile zf = new ZipFile(this.STBOF_FILE);

            Enumeration<? extends ZipEntry> zenum = zf.entries();
            ZipEntry ze = null;

            while (zenum.hasMoreElements())
            {
                ze = zenum.nextElement();

                file = ze.getName().toLowerCase();

                if (file == null
                        || !fil.accept(STBOF_FILE, file)
                        || file.equals("mod.lst")
                        || this.REMOVED.contains(file)
                        || INTERNAL_FILES.containsKey(file)) //$NON-NLS-1$
                {
                    continue;
                }

                result.add(file);
            }

            zf.close();
        }
        catch (Exception ex)
        {
            UE.displayError(ex);
        }

        //AND NOW THE REST
        Enumeration<InternalFile> itg = this.INTERNAL_FILES.elements();

        while (itg.hasMoreElements())
        {
            InternalFile gef = itg.nextElement();

            if (!fil.accept(STBOF_FILE, gef.getName()) ||
                    gef.getName().equals("mod.lst") || this.REMOVED.contains(gef.getName())) //$NON-NLS-1$
            {
                continue;
            }

            result.add(gef.getName());
        }

        Collections.sort(result);
        
        return result;
    }

    /**
     * Loads a tga image file from stbof.res using the TargaImage class.
     * @param name the name of the tga image to load and return.
     * @return a TargaImage loaded from specified file, null if the targa image can't be loaded.
     * @throws FileNotFoundExeption if the file can't be found.
     */ 
    public TargaImage getTargaImage(String name, boolean cache) throws FileNotFoundException
    {
        String msg;
        
        name = name.toLowerCase();
        
        if (this.REMOVED.contains(name))
        {
            msg = Language.getString("Stbof.44"); //$NON-NLS-1$
            msg = msg.replace("%1", name); //$NON-NLS-1$
            throw new FileNotFoundException(msg);
        }
        
        InputStream in = null;
        TargaImage img = null;
        ZipEntry ze = null;
        
        try
        {
            if (!this.INTERNAL_FILES.containsKey(name))
            {
                ZipFile zf = new ZipFile(this.STBOF_FILE);
                
                Enumeration<? extends ZipEntry> zenum = zf.entries();
                
                while (zenum.hasMoreElements())
                {
                    ze = zenum.nextElement();
                    if (name.equals(ze.getName().toLowerCase()))
                    {
                        break;
                    }
                }
                
                if ((ze == null) || !name.equals(ze.getName().toLowerCase()))
                {
                    zf.close();
                }
                else
                {
                    in = new ZipReader(zf.getInputStream(ze));
                    img = new TargaImage(in);
                    img.setName(name);
                    zf.close();

                    if (cache)
                        INTERNAL_FILES.put(name, img);
                }
            }
            else
            {
                InternalFile fimg = INTERNAL_FILES.get(name);

                if (fimg instanceof TargaImage)
                {
                    img = (TargaImage)fimg;
                }
                else
                {
                    GenericFile gun = (GenericFile) fimg;
                    ByteArrayInputStream baos = new ByteArrayInputStream(gun.getData());
                    img = new TargaImage(baos);
                    img.setName(name);
                    if (gun.madeChanges())
                    {
                        img.markChanged();
                    }

                    INTERNAL_FILES.remove(name);
                    INTERNAL_FILES.put(name, img);
                }
            }
        }
        catch (Exception e)
        {
            UE.displayError(e);
            return img;
        }

        if (img == null)
        {
            msg = Language.getString("Stbof.44"); //$NON-NLS-1$
            msg = msg.replace("%1", name); //$NON-NLS-1$
            throw new FileNotFoundException(msg);
        }
        
        return img;
    }
    
    /**
     * Checks internal files for unusal data and other problems.
     * @return a Vector containing the check results.
     */
    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();
        
        CheckTask TSK = new CheckTask();
        
        Vector<?> r = (Vector<?>)Tools.runUEWorker(TSK);
        
        for (int i = 0; i < r.size(); i++)
        {
            response.add(r.get(i).toString());
        }
        
        //return
        if (response.isEmpty())
        {
            response.add(Language.getString("Stbof.45")); //$NON-NLS-1$
        }
        
        return response;
    }

    private class CheckTask extends UEWorker<Vector<String>>
    {
        public CheckTask()
        {
            super();
        }
        
        @Override
        public Vector<String> doInBackground()
        {
            try
            {
                String msg;
                this.RESULT = new Vector<String>();
                InternalFile test;
                Vector<String> temp;
                
                this.updateStatus(UEWorker.STATUS_WORKING);
                int i=0;
                
                for (i = 0; i < Stbof.FILELIST.length; i++)
                {
                    if (this.isCancelled())
                    {
                        this.RESULT.add(Language.getString("Stbof.46")); //$NON-NLS-1$
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        return this.RESULT;
                    }
                    
                    try
                    {
                        Stbof.this.getInternalFile(Stbof.FILELIST[i], true);
                    }
                    catch (FileNotFoundException e)
                    {
                        msg = Language.getString("Stbof.56"); //$NON-NLS-1$
                        msg = msg.replace("%1", Stbof.FILELIST[i]); //$NON-NLS-1$
                        msg = msg.replace("%2", e.getMessage()); //$NON-NLS-1$
                        this.RESULT.add(msg);
                    }
                }
                
                test = null;
                
                msg = Language.getString("Stbof.47"); //$NON-NLS-1$
                
                ArrayList<Pair<String, Dimension>> images;
                TargaImage tar;
                
                Enumeration<InternalFile> un = Stbof.this.INTERNAL_FILES.elements();
                while(un.hasMoreElements())
                {
                    if (this.isCancelled())
                    {
                        this.RESULT.add(Language.getString("Stbof.46")); //$NON-NLS-1$
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        return this.RESULT;
                    }
                    
                    try
                    {
                        test = un.nextElement();
                        
                        this.sendMessage(msg.replace("%1", test.getName())); //$NON-NLS-1$
                        temp = test.check();

                        if (temp != null)
                        {
                            this.RESULT.addAll(temp);
                        }
                        
                        // now check images
                        images = test.getImagesListedInThisFile();
                        
                        if (images != null)
                        {
                            for (Pair<String, Dimension> img : images)
                            {
                                try
                                {
                                    tar = getTargaImage(img.LEFT, false);
                                    
                                    if (tar.getPixelDepth() != 16)
                                    {
                                        RESULT.add(
                                                test.getCheckIntegrityString(
                                                        InternalFile.INTEGRITY_CHECK_ERROR,
                                                        CommonStrings.getLocalizedString(
                                                                CommonStrings.Image_1_HasInvalidPixelDepth_2,
                                                                new String[]{
                                                                        img.LEFT,
                                                                        Short.toString(tar.getPixelDepth())
                                                                }
                                                        )
                                                )
                                        );
                                    }
                                    
                                    if (tar.getType() != 2)
                                    {
                                        RESULT.add(
                                                test.getCheckIntegrityString(
                                                        InternalFile.INTEGRITY_CHECK_ERROR,
                                                        CommonStrings.getLocalizedString(
                                                                CommonStrings.TGAImage_1_IsNotUncompressed,
                                                                new String[]{
                                                                        img.LEFT
                                                                }
                                                        )
                                                )
                                        );
                                    }
                                    
                                    if (!tar.getImageSize().equals(img.RIGHT))
                                    {
                                        RESULT.add(
                                                test.getCheckIntegrityString(
                                                        InternalFile.INTEGRITY_CHECK_WARNING,
                                                        CommonStrings.getLocalizedString(
                                                                CommonStrings.WrongImage_1_Size_23_Expected_45,
                                                                new String[]{
                                                                        img.LEFT,
                                                                        Integer.toString(tar.getImageSize().width),
                                                                        Integer.toString(tar.getImageSize().height),
                                                                        Integer.toString(img.RIGHT.width),
                                                                        Integer.toString(img.RIGHT.height)
                                                                }
                                                        )
                                                )
                                        );
                                    }
                                    
                                    tar = null;
                                }
                                catch (FileNotFoundException fnf)
                                {
                                    RESULT.add(
                                            test.getCheckIntegrityString(
                                                    InternalFile.INTEGRITY_CHECK_ERROR,
                                                    CommonStrings.getLocalizedString(
                                                            CommonStrings.FileNotFound_1,
                                                            new String[]{img.LEFT}
                                                    )
                                            )
                                    );
                                }
                            }
                        }
                    }
                    catch (Exception f)
                    {
                        msg = Language.getString("Stbof.48"); //$NON-NLS-1$
                        if (test != null)
                        {
                            msg = msg.replace("%1", test.getName()); //$NON-NLS-1$
                        }
                        else
                        {
                            msg = msg.replace("%1", Stbof.this.STBOF_FILE.getName()); //$NON-NLS-1$ 
                        }
                        
                        msg = msg.replace("%2", f.getMessage()); //$NON-NLS-1$
                        this.RESULT.add(msg);
                        f.printStackTrace();
                    }
                    
                    images = null;
                }
            }
            catch (Throwable e)
            {
                String msg = Language.getString("Stbof.48"); //$NON-NLS-1$
                msg = msg.replace("%1", Stbof.this.STBOF_FILE.getName()); //$NON-NLS-1$
                msg = msg.replace("%2", e.getMessage()); //$NON-NLS-1$
                this.RESULT.add(msg);
                e.printStackTrace();
            }
            
            this.updateStatus(UEWorker.STATUS_DONE);
            return this.RESULT;
        }
    }
    
    private class SaveTask extends UEWorker<Boolean>
    {
        File whereTo;
        boolean bkp;
        String ext;
        
        public SaveTask(File dest, boolean bkp, String ext)
        {
            super();
            this.whereTo = dest;
            this.bkp = bkp;
            this.ext = ext;
        }
        
        @Override
        public Boolean doInBackground()
        {
            this.RESULT = false;
            String msg;
            
            this.updateStatus(UEWorker.STATUS_WORKING);
            
            ZipFile source = null;
            ZipOutputStream zos = null;
            
            try
            {
                source = new ZipFile(Stbof.this.STBOF_FILE);
                
                //where to
                if (this.whereTo.equals(Stbof.this.STBOF_FILE))
                {
                    if (bkp)
                    {
                        this.sendMessage(Language.getString("Stbof.61")); //$NON-NLS-1$

                        File file = new File(STBOF_FILE.getPath() + ext);
                        
                        int i = 1;
                        
                        while (file.exists())
                        {
                            file = new File(STBOF_FILE.getPath() + i + ext);
                            i++;
                        }
                        
                        Tools.copyFile(STBOF_FILE, file);
                    }

                    msg = Language.getString("Stbof.49"); //$NON-NLS-1$
                    msg = msg.replace("%1", Stbof.fileSTBOF_TMP); //$NON-NLS-1$
                    this.sendMessage(msg);
                    this.whereTo = FileManager.getWritableLocalFile(Stbof.fileSTBOF_TMP);
                }
                this.whereTo.delete();
                this.whereTo.createNewFile();
                zos = new ZipOutputStream(new FileOutputStream(this.whereTo));
            }
            catch (Exception ze)
            {
                try
                {
                    source.close();
                }
                catch (Exception e){}

                try
                {
                    zos.close();
                }
                catch (Exception e){}

                this.reportError(ze);

                return this.RESULT;
            }
            
            for (Enumeration<? extends ZipEntry> e = source.entries(); e.hasMoreElements(); )
            {
                try
                {
                    if (this.isCancelled())
                    {
                        source.close();
                        zos.close();
                        this.whereTo.delete();
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        return this.RESULT;
                    }
                    
                    ZipEntry ze = (ZipEntry)e.nextElement();
                    String mfilename = ze.getName();
                    
                    if (mfilename.equals("mod.lst")  //$NON-NLS-1$
                            || Stbof.this.REMOVED.contains(mfilename.toLowerCase()))
                    {
                        continue;
                    }
                    
                    msg = Language.getString("Stbof.50"); //$NON-NLS-1$

                    if (!Stbof.this.INTERNAL_FILES.containsKey(mfilename.toLowerCase()))
                    {
                        // copy
                        ZipEntry zfl = new ZipEntry(mfilename);
                        
                        this.sendMessage(msg.replace("%1", mfilename)); //$NON-NLS-1$
                        
                        InputStream is = source.getInputStream(zfl);
                        int ln = is.available();
                        zfl.setSize(ln);
                        zfl.setCompressedSize(-1);
                        zfl.setMethod(8);
                        zos.putNextEntry(zfl);
                        
                        int len;
                        byte[] buf = new byte[1024];
                        
                        while ((len = is.read(buf)) != -1) {
                            zos.write(buf, 0, len);
                        }
                        
                        is.close();
                        zos.flush();
                    }
                    // Complete the entry
                    zos.closeEntry();
                }
                catch (Exception ze)
                {
                    this.reportError(ze);
                    return this.RESULT;
                }
            }
            
            //go thru remaining files
            Enumeration<InternalFile> itg = Stbof.this.INTERNAL_FILES.elements();
            InternalFile gef;
            ZipEntry zeg;
            msg = Language.getString("Stbof.51"); //$NON-NLS-1$
            
            while (itg.hasMoreElements())
            {
                try
                {
                    if (this.isCancelled())
                    {
                        source.close();
                        zos.close();
                        this.whereTo.delete();
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        return this.RESULT;
                    }
                    
                    gef = itg.nextElement();
                    
                    if (Stbof.this.REMOVED.contains(gef.getName())
                            || gef.getName().equals("mod.lst")) //$NON-NLS-1$
                    { // this should be redundant, but just in case...
                        continue;
                    }
                    
                    this.sendMessage(msg.replace("%1", gef.getName())); //$NON-NLS-1$
                    
                    if (Stbof.this.ADD_CHANGES_2_MODLIST && gef.madeChanges())
                    {
                        Stbof.this.MODS.add(gef.getName());
                    }
                    
                    gef.markSaved();
                    zeg = new ZipEntry(gef.getName().toLowerCase());
                    zeg.setCompressedSize(-1);
                    zeg.setMethod(8);
                    zeg.setSize(gef.getSize());
                    zos.putNextEntry(zeg);
                    
                    gef.save(zos);
                    zos.flush();
                    zos.closeEntry();
                }
                catch (Exception ze)
                {
                    this.reportError(ze);
                    return this.RESULT;
                }
            }

            this.sendMessage(msg.replace("%1", "mod.lst")); //$NON-NLS-1$ //$NON-NLS-2$
            
            try
            {
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                Stbof.this.MODS.save(baos);
                byte[] bub = baos.toByteArray();
                int sizi = bub.length;
                
                zeg = new ZipEntry("mod.lst"); //$NON-NLS-1$
                zeg.setCompressedSize(-1);
                zeg.setMethod(8);
                zeg.setSize(sizi);
                zos.putNextEntry(zeg);
                
                zos.write(bub, 0, sizi);
                zos.flush();
                zos.closeEntry();
            }
            catch (Exception ze)
            {
                this.reportError(ze);
                return this.RESULT;
            }
            
            try
            {
                this.sendMessage(Language.getString("Stbof.52")); //$NON-NLS-1$
                source.close();
                zos.close();
            }
            catch (Exception ze)
            {
                this.whereTo.delete();
                this.reportError(ze);
                return this.RESULT;
            }
            
            if (this.isCancelled())
            {
                this.whereTo.delete();
                return this.RESULT;
            }
            
            //replace?
            if (this.whereTo.getName().equals(Stbof.fileSTBOF_TMP))
            {
                msg = Language.getString("Stbof.50"); //$NON-NLS-1$
                msg = msg.replace("%1", this.whereTo.getName()); //$NON-NLS-1$
                this.sendMessage(msg);
                
                try
                {
                    Tools.copyFile(this.whereTo, Stbof.this.STBOF_FILE);
                }
                catch (IOException er)
                {
                    this.reportError(er);
                    return this.RESULT;
                }
                
                msg = Language.getString("Stbof.53"); //$NON-NLS-1$
                msg = msg.replace("%1", this.whereTo.getName()); //$NON-NLS-1$
                this.sendMessage(msg);

                boolean deleted = this.whereTo.delete();
                if (!deleted)
                {
                    msg = Language.getString("Stbof.54"); //$NON-NLS-1$
                    msg = msg.replace("%1", this.whereTo.getName()); //$NON-NLS-1$
                    this.reportError(new IOException(msg));
                    return this.RESULT;
                }
            }

            try
            {
                source = new ZipFile(Stbof.this.STBOF_FILE);
                Enumeration<? extends ZipEntry> entries = source.entries();
                files.clear();
                while (entries.hasMoreElements())
                {
                    files.add(entries.nextElement().getName().toLowerCase());
                }
                files.trimToSize();
                source.close();
            }
            catch (IOException er)
            {
                try
                {
                    source.close();
                }
                catch (IOException ex)
                {
                    ex.printStackTrace();
                }

                this.reportError(er);
                return this.RESULT;
            }

            Stbof.this.REMOVED.clear();
            this.RESULT = true;
            this.updateStatus(UEWorker.STATUS_DONE);
            return this.RESULT;
        }
    }
    
    /**
     * @param string
     * @throws IOException 
     * @throws InvalidArgumentsException 
     */
    public boolean removeModelGraphicFiles(String[] strings) throws Exception
    {
        RemoveModelGraphicFilesTask tsk = new RemoveModelGraphicFilesTask(strings);
        
        return (Boolean)Tools.runUEWorker(tsk);
    }

    private class RemoveModelGraphicFilesTask extends UEWorker<Boolean>
    {
        private String[] strings;
        
        public RemoveModelGraphicFilesTask(String[] s){
            super();
            this.strings = s;
        }

        /* (non-Javadoc)
         * @see javax.swing.SwingWorker#doInBackground()
         */
        @Override
        protected Boolean doInBackground()
        {
            ArrayList <String> textures = new ArrayList<String>();
            
            String sfile;
            HobFile hob;
            String msg;
            String t;
            ByteArrayInputStream bais;
            ByteArrayOutputStream baos;
            
            this.RESULT = false;
            
            this.updateStatus(UEWorker.STATUS_WORKING);

            msg = Language.getString("Stbof.58"); //$NON-NLS-1$
            
            try
            {
                Texture texturelist = (Texture)Stbof.this.getInternalFile(Stbof.fileTEXTURE_LST, true);

                for (String str : this.strings)
                {
                    if (this.isCancelled())
                    {
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        return this.RESULT;
                    }
                    
                    str = str.toLowerCase();
                    
                    char c = 'a';
                    
                    while( c <= 'c')
                    {
                        sfile = str + "_" + c + ".hob"; //$NON-NLS-1$ //$NON-NLS-2$
                        
                        this.sendMessage(msg.replace("%1", sfile)); //$NON-NLS-1$
    
                        baos = new ByteArrayOutputStream();
                        
                        Stbof.this.getInternalFile(str + "_" + c + ".hob", true).save(baos); //$NON-NLS-1$ //$NON-NLS-2$
                        bais = new ByteArrayInputStream(baos.toByteArray());
                        hob = new HobFile(bais);
                            
                        for (int j = 0; j < hob.numberOfTextures(); j++)
                        {
                            t = texturelist.get(hob.getTextureID(j));

                            if (!textures.contains(t))
                            {
                                textures.add(t);
                            }
                        }
                        
                        Stbof.this.removeInternalFile(sfile);
                        c++;
                    }
                    
                    sfile = "i_" + str + "w.tga"; //$NON-NLS-1$ //$NON-NLS-2$
                    this.sendMessage(msg.replace("%1", sfile)); //$NON-NLS-1$
                    Stbof.this.removeInternalFile(sfile);
            
                    sfile = "i_" + str + "30.tga"; //$NON-NLS-1$ //$NON-NLS-2$
                    this.sendMessage(msg.replace("%1", sfile)); //$NON-NLS-1$
                    Stbof.this.removeInternalFile(sfile);
            
                    sfile = "i_" + str + "60.tga"; //$NON-NLS-1$ //$NON-NLS-2$
                    this.sendMessage(msg.replace("%1", sfile)); //$NON-NLS-1$
                    Stbof.this.removeInternalFile(sfile);
            
                    sfile = "i_" + str + "120.tga"; //$NON-NLS-1$ //$NON-NLS-2$
                    this.sendMessage(msg.replace("%1", sfile)); //$NON-NLS-1$
                    Stbof.this.removeInternalFile(sfile);
            
                    sfile = "i_" + str + "170.tga"; //$NON-NLS-1$ //$NON-NLS-2$
                    this.sendMessage(msg.replace("%1", sfile)); //$NON-NLS-1$
                    Stbof.this.removeInternalFile(sfile);
                }
                
                // test and remove textures
                Enumeration<String[]> hobfiles = Stbof.this.getFiles(new FilenameFilter(){
                    @Override
                    public boolean accept(File arg0, String arg1)
                    {
                        return arg1.toLowerCase().endsWith(".hob"); //$NON-NLS-1$
                    }
                });
                
                this.sendMessage(Language.getString("Stbof.59")); //$NON-NLS-1$
    
                while (hobfiles.hasMoreElements() && !textures.isEmpty())
                {
                    if (this.isCancelled())
                    {
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        return this.RESULT;
                    }

                    baos = new ByteArrayOutputStream();
    
                    Stbof.this.getInternalFile(hobfiles.nextElement()[0], true).save(baos);
                    bais = new ByteArrayInputStream(baos.toByteArray());
                    hob = new HobFile(bais);
                        
                    for (int j = 0; j < hob.numberOfTextures(); j++)
                    {
                        t = texturelist.get(hob.getTextureID(j));

                        if (textures.contains(t))
                        {
                            textures.remove(t);
                        }
                    }
                }
                
                int index;
                
                while (!textures.isEmpty())
                {
                    if (this.isCancelled())
                    {
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        return this.RESULT;
                    }

                    t = textures.remove(0);
                    this.sendMessage(msg.replace("%1", t)); //$NON-NLS-1$
                    Stbof.this.removeInternalFile(t);
                    index = texturelist.getTextureIndex(t);
                    if (index >= 0)
                    {
                        texturelist.remove(index);
                    }
                    
                }
                
                this.RESULT = true;
            }
            catch (Exception f)
            {
                this.reportError(f);
                return this.RESULT;
            }
            
            this.updateStatus(UEWorker.STATUS_DONE);
            return this.RESULT;
        }
    }

    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#getKnownChanges()
     */
    @Override
    public ModList getKnownChanges()
    {
        return this.MODS;
    }

    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#hasBeenModified()
     */
    @Override
    public boolean hasBeenModified()
    {
        return !this.MODS.isEmpty() || this.madeChanges();
    }
    
    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#toFile()
     */
    @Override
    public File toFile()
    {
        return this.STBOF_FILE;
    }

    /**
     * @param string
     * @return file size
     * @throws IOException 
     * @throws ZipException 
     */
    @Override
    public long getInternalFileSize(String fileName) throws IOException
    {
        String msg;
        
        fileName = fileName.toLowerCase();
        
        if (this.REMOVED.contains(fileName))
        {
            msg = Language.getString("Stbof.44"); //$NON-NLS-1$
            msg = msg.replace("%1", fileName); //$NON-NLS-1$
            throw new FileNotFoundException(msg);
        }
        
        if (this.INTERNAL_FILES.containsKey(fileName))
        {
            return this.INTERNAL_FILES.get(fileName).getSize();
        }
        else
        {
            ZipFile zf = new ZipFile(this.STBOF_FILE);
            
            Enumeration<? extends ZipEntry> zenum = zf.entries();
            ZipEntry ze = null;
            
            while (zenum.hasMoreElements())
            {
                ze = zenum.nextElement();
                if (fileName.equals(ze.getName().toLowerCase()))
                {
                    zf.close();
                    return ze.getSize();
                }
            }
            zf.close();
        }
        
        msg = Language.getString("Stbof.44"); //$NON-NLS-1$
        msg = msg.replace("%1", fileName); //$NON-NLS-1$
        throw new FileNotFoundException(msg);
    }
}
