/**
 * Ultimate Editor
 * Copyright (C) 2009 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: Converse.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 4.5.2009
 */
package ue.edit.res.stbof.bin;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;

/**
 * Converse.bin
 * @author Alan Podlesek
 */
public class Converse extends InternalFile
{
    private int voice_groups;
    private int map_address;
    private int voice_lists;
    private byte[] unknown1 = new byte[2];
    // followed by VoiceLists, VoiceGroups, VoiceGroup map entries, VoiceList map entries
    
    private ArrayList<VoiceGroup> groups = new ArrayList<VoiceGroup>();
    private ArrayList<VoiceList> lists = new ArrayList<VoiceList>();
    
    public Converse(InputStream in) throws IOException
    {
        byte[] intb = new byte[4];
        in.read(intb);
        voice_groups = Tools.toInt(intb, true);
        
        in.read(intb);
        map_address = Tools.toInt(intb, true);

        in.read(intb);
        voice_lists = Tools.toInt(intb, true);
        
        in.read(unknown1);
        
        byte[] data = new byte[map_address - 14];
        in.read(data);
        ByteArrayInputStream bais = new ByteArrayInputStream(data);
        
        // read VoiceGroups
        int type, num, address;
        
        for (int i = 0; i < voice_groups; i++)
        {
            // read map entry
            in.read(intb); // type
            type = Tools.toInt(intb, true);
            in.skip(4);
            in.read(intb); // number
            num = Tools.toInt(intb, true);
            in.read(intb); // address
            address = Tools.toInt(intb, true);
            
            // read data
            address -= 14;
            bais.reset();
            bais.skip(address);
            groups.add(new VoiceGroup(bais, num, type));
        }
        
        // read VoiceLists
        for (int i = 0; i < voice_lists; i++)
        {
            // read map entry
            in.read(intb); // address
            address = Tools.toInt(intb, true);
            in.read(intb); // number
            num = Tools.toInt(intb, true);
            in.read(intb); // type
            type = Tools.toInt(intb, true);
            in.skip(4);
            
            // read data
            address -= 14;
            bais.reset();
            bais.skip(address);
            lists.add(new VoiceList(bais, num, type));
        }
        
        bais.close();
    }
    
    @Override
    public String toString()
    {
        String str = "";
        
        for (int i = 0; i < groups.size(); i++)
        {
            str = str + "Group " + i + ":\n" + groups.get(i).toString() + "\n";
        }
        
        return str;
    }

    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();
        
        VoiceGroup conf;
        
        for (int i = 0; i < groups.size(); i++)
        {
            conf = groups.get(i);

            response.addAll(conf.check());
        }
        
        return response;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        long size = 14;
        
        for (int i = 0; i < groups.size(); i++)
        {
            size += 16 + groups.get(i).getSize();
        }
        
        for (int i = 0; i < lists.size(); i++)
        {
            size += 16 + lists.get(i).getSize();
        }

        return size;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        voice_groups = groups.size();
        
        out.write(Tools.toByte(voice_groups, true));
        
        map_address = 14;
        
        for (int i = 0; i < groups.size(); i++)
        {
            map_address += groups.get(i).getSize();
        }
        
        for (int i = 0; i < lists.size(); i++)
        {
            map_address += lists.get(i).getSize();
        }
        
        
        out.write(Tools.toByte(map_address, true));
        
        voice_lists = lists.size();
        
        out.write(Tools.toByte(voice_lists, true));
        out.write(unknown1);
        
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        
        int address = 14;
        byte[] skip = new byte[4];

        // write VoiceLists
        VoiceList seg;
        
        for (int i = 0; i < lists.size(); i++)
        {
            seg = lists.get(i);
            seg.save(out);
            address += seg.getSize();
        }

        // write VoiceGroups
        VoiceGroup conf;
        for (int i = 0; i < groups.size(); i++)
        {
            conf = groups.get(i);
            conf.save(out);
            
            // save map entry
            baos.write(Tools.toByte(conf.type, true));
            baos.write(skip);
            baos.write(Tools.toByte(conf.getSize() - 1, true));
            baos.write(Tools.toByte(address, true));
            
            // adjust address
            address += conf.getSize();
        }
                
        // write voicelist map entries
        address = 14;
        
        for (int i = 0; i < lists.size(); i++)
        {
            seg = lists.get(i);
            // save map entry
            baos.write(Tools.toByte(address, true));
            baos.write(Tools.toByte(seg.unknown1.length, true));
            baos.write(Tools.toByte(seg.type, true));
            baos.write(skip);
            
            // adjust address
            address += seg.getSize();
        }
        
        // write map
        out.write(baos.toByteArray());
    }
    
    /**
     * Return -1 if not found. Otherwise returns race index. 36 is alien, 37 is neutral.
     * @param snd_index index of the voice in english.snd
     * @return
     */
    public int getVoiceOwner(short snd_index)
    {
        int race = -1;
        VoiceGroup group;
        
        for (int i = 0; i < groups.size(); i++)
        {
            group = groups.get(i);
            
            if ((race = group.getRace(snd_index)) >= 0)
            {
                return race;
            }
        }
        
        return race;
    }
    
    /**
     * Will switch sound indexes of given races.
     * @param race1
     * @param race2
     * @param both if false will set race1 to use race2 voices
     * @throws InvalidArgumentsException
     */
    public void switchRaces(int race1, int race2, boolean both) throws InvalidArgumentsException
    {
        if (race1 == race2)
        {
            return;
        }
        
        ArrayList<VoiceGroup.VoiceGroupEntry> todo1 = new ArrayList<VoiceGroup.VoiceGroupEntry>();
        ArrayList<VoiceGroup.VoiceGroupEntry> todo2 = new ArrayList<VoiceGroup.VoiceGroupEntry>();
        VoiceGroup grp;
        int direct[] = new int[2], list[] = new int[2];
        VoiceGroup.VoiceGroupEntry a, b = null;
        
        for (int i = 0; i < groups.size(); i++)
        {
            a = b = null;
            grp = groups.get(i);
            
            for (int j = 0; j < grp.voices.length; j++)
            {
                if (grp.voices[j].race == race1)
                {
                    todo1.add(grp.voices[j]);
                    
                    if (grp.type == VoiceGroup.TYPE_DIRECT_SND_INDEX)
                    {
                        direct[0]++;
                    }
                    else if (grp.type == VoiceGroup.TYPE_VOICE_LIST_INDEX)
                    {
                        list[0]++;
                    }
                }
                else if (grp.voices[j].race == race2)
                {
                    todo2.add(grp.voices[j]);

                    if (grp.type == VoiceGroup.TYPE_DIRECT_SND_INDEX)
                    {
                        direct[1]++;
                    }
                    else if (grp.type == VoiceGroup.TYPE_VOICE_LIST_INDEX)
                    {
                        list[1]++;
                    }
                }
            }
        }
        
        if (todo1.size() != todo2.size())
        {
            throw new InvalidArgumentsException(
                    Language.getString("Converse.0").replace( //$NON-NLS-1$
                            "%1", Integer.toString(todo1.size())).replace( //$NON-NLS-1$
                                    "%2", Integer.toString(todo2.size()))); //$NON-NLS-1$
        }
        
        if (direct[0] != direct[1] || list[0] != list[1])
        {
            throw new InvalidArgumentsException(
                    Language.getString("Converse.1").replace("%1", this.getName())); //$NON-NLS-1$ //$NON-NLS-2$
        }
        
        int n = 0;
        short tmp;
        
        while (todo1.size() > 0)
        {
            a = todo1.remove(0);
            n = 0;
            
            while (b == null || (b.index >= VoiceGroup.offset && a.index < VoiceGroup.offset)
                    || (a.index >= VoiceGroup.offset && b.index < VoiceGroup.offset))
            {
                b = todo2.get(n++);
            }
            
            todo2.remove(n - 1);
            
            tmp = a.index;
            a.index = b.index;
            
            if (both) b.index = tmp;
            
            a = b = null;
        }
        
        this.markChanged();
    }
    
    private class VoiceGroup
    {
        private final static int TYPE_DIRECT_SND_INDEX = 0; // sound_index is index in .snd
        private final static int TYPE_VOICE_LIST_INDEX = 1; // sound_index is VoiceList index + 15000
        private final static int offset = 15000;
        
        private int type;
        
        private final static int delimiter = 254;
        private final static int end_of_group = 252;
        
        private VoiceGroupEntry[] voices;

        private int extra;      // either an end_mark or -1
        
        public VoiceGroup(InputStream in, int num, int type) throws IOException
        {
            this.type = type;
            
            num = (num-1)/11;
            
            voices = new VoiceGroupEntry[num];
            
            in.skip(1); // delimiter
            
            for (int i = 0; i < num; i++)
            {
                voices[i] = new VoiceGroupEntry(in);    // 10n
                in.skip(1);                             // delimiter or end of group
            }
            
            extra = in.read();  // end_of_group or -1 11n + 2            
        }
        
        /**
         * @param snd_index
         * @return
         */
        public int getRace(short snd_index)
        {
            if (type == TYPE_DIRECT_SND_INDEX)
            {
                for (int i = 0; i < voices.length; i++)
                {
                    if (voices[i].index == snd_index)
                    {
                        return voices[i].race;
                    }
                }
            }
            else if (type == TYPE_VOICE_LIST_INDEX)
            {
                int ilist;
                VoiceList vlist;
                
                for (int i = 0; i < voices.length; i++)
                {
                    ilist = voices[i].index - offset;
                    
                    if (ilist < lists.size() && ilist >= 0)
                    {
                        vlist = lists.get(ilist);
                        
                        for (int j = 0; j < vlist.snd_index.length; j++)
                        {
                            if (vlist.snd_index[j] == snd_index)
                            {
                                return voices[i].race;
                            }
                        }
                    }
                }
            }
            
            return -1;
        }

        public int getSize()
        {
            return voices.length * 11 + 2;
        }
        
        public void save(OutputStream out) throws IOException
        {
            out.write(delimiter);       // 1

            for (int i = 0; i < voices.length; i++)
            {
                voices[i].save(out);    // 10n + 1
                
                if (i + 1 == voices.length)
                {
                    out.write(end_of_group);    // 11n + 1
                }
                else
                {
                    out.write(delimiter);
                }
            }
            
            out.write(extra);   // 11n + 2
        }
        
        protected Vector<String> check()
        {
            Vector<String> response = new Vector<String>();
            String msg;
            
            if (type == TYPE_DIRECT_SND_INDEX)
            {
                for (int i = 0; i < voices.length; i++)
                {
                    if (voices[i].index > 1363 || voices[i].index < 0)
                    {
                        msg = Language.getString("Converse.2").replace("%1", Short.toString(voices[i].index)); //$NON-NLS-1$ //$NON-NLS-2$
                        msg = getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg);
                        
                        response.add(msg);
                    }
                }
            }
            else if (type == TYPE_VOICE_LIST_INDEX)
            {
                int ilist;
                VoiceList vlist;
                
                for (int i = 0; i < voices.length; i++)
                {
                    ilist = voices[i].index - offset;
                    
                    if (ilist >= lists.size() || ilist < 0)
                    {
                        msg = Language.getString("Converse.3").replace("%1", Short.toString(voices[i].index)); //$NON-NLS-1$ //$NON-NLS-2$
                        msg = getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg);
                        response.add(msg);
                    }
                    else
                    {
                        vlist = lists.get(ilist);
                        
                        for (int j = 0; j < vlist.snd_index.length; j++)
                        {
                            if (vlist.snd_index[j] > 1363 || vlist.snd_index[j] < 0)
                            {
                                msg = Language.getString("Converse.4").replace("%1", Short.toString(vlist.snd_index[j])); //$NON-NLS-1$ //$NON-NLS-2$
                                msg = getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg);
                                
                                response.add(msg);
                            }
                        }
                    }
                }
            }
            else
            {
                msg = Language.getString("Converse.5").replace("%1", Integer.toString(type)); //$NON-NLS-1$ //$NON-NLS-2$
                msg = getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg);
                response.add(msg);
            }
            
            return response;
        }
        
        @Override
        public String toString()
        {
            String str = "";
            
            for (int i = 0; i < voices.length; i++)
            {
                str = str + "Race " + voices[i].race + ": ";
                
                if (this.type == TYPE_DIRECT_SND_INDEX)
                {
                    str = str + voices[i].index + "\n";
                }
                else
                {
                    VoiceList lst = lists.get(voices[i].index - offset);
                    
                    for (int j = 0; j < lst.snd_index.length; j++)
                    {
                        if (j + 1 < lst.snd_index.length)
                        {
                            str = str + lst.snd_index[j] + ", ";
                        }
                        else
                        {
                            str = str + lst.snd_index[j] + "\n";
                        }
                    }
                }
            }
            
            return str;
        }
        
        private class VoiceGroupEntry
        {
            int race;
            byte[] unknown = new byte[7]; // seems to always be the same
            short index;
            
            public VoiceGroupEntry(InputStream in) throws IOException
            {
                race = in.read();
                in.read(unknown);
                byte[] b = new byte[2];
                in.read(b);
                index = Tools.toShort(b, true);
            }
            
            public void save(OutputStream out) throws IOException
            {
                out.write(race);
                out.write(unknown);
                out.write(Tools.toByte(index, true));
            }
        }
    }
    
    private class VoiceList
    {
        // known types
        public static final int DIPLOMATIC_INTRO = 4;
        
        int type; // taken from map, no idea
        
        short unknown1[];       // my guess is sound list offset where the index is saved on file load
        short unknown2[];       // always 1 (?)
        short snd_index[];
        
        public VoiceList(InputStream in, int num, int type) throws IOException
        {
            this.type = type;
            
            unknown1 = new short[num];
            unknown2 = new short[num];
            snd_index = new short[num];
            
            byte[] b = new byte[2];

            for (int i = 0; i < num; i++)
            {
                in.read(b);                             // 2n
                unknown1[i] = Tools.toShort(b, true);
            }
            
            for (int i = 0; i < num; i++)
            {
                in.read(b);                             // 4n
                unknown2[i] = Tools.toShort(b, true);
                in.read(b);
                snd_index[i] = Tools.toShort(b, true);  // 6n
            }
        }
        
        public int getSize()
        {
            return 6 * unknown1.length;
        }
        
        public void save(OutputStream out) throws IOException
        {
            for (int i = 0; i < unknown1.length; i++)
            {
                out.write(Tools.toByte(unknown1[i], true));
            }
            
            for (int i = 0; i < unknown2.length; i++)
            {
                out.write(Tools.toByte(unknown2[i], true));
                out.write(Tools.toByte(snd_index[i], true));
            }
        }
    }
}
