/**
 * Ultimate Editor
 * Copyright (C) 2011 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: Intel.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 18.10.2011
 */
package ue.edit.res.stbof.bin;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.edit.InternalFile;
import ue.exception.InvalidArgumentsException;
import ue.util.CommonStrings;
import ue.util.Tools;

/**
 *
 */
public class Intel extends InternalFile
{
    private IntelEntry[] entries;

    public Intel(InputStream in) throws IOException
    {
        entries = new IntelEntry[5];

        if (in.available() < getSize())
        {
            throw new IOException(CommonStrings.getLocalizedString(
                    CommonStrings.File_Is_Corrupted_1, new String[]{"intel.bin"}));
        }

        for (int i = 0; i < 5 && in.available() > 0; i++)
        {
            entries[i] = new IntelEntry(in);
        }
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        return null;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        return 96 * entries.length;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        for (int i = 0; i < entries.length; i++)
        {
            entries[i].save(out);
        }
    }

    public int getBaseEspAtkMul(int empire) throws InvalidArgumentsException
    {
        checkIndex(empire, entries.length - 1);
        return entries[empire].base_esp_attack_mult;
    }

    public int getBaseSabAtkMul(int empire) throws InvalidArgumentsException
    {
        checkIndex(empire, entries.length - 1);
        return entries[empire].base_sab_attack_mult;
    }

    public int getTargetEmpAtkMul(int empire, int target) throws InvalidArgumentsException
    {
        checkIndex(empire, entries.length - 1);
        checkIndex(target, entries.length - 1);
        return entries[empire].target_emp_attack_mult[target];
    }

    public int getDiffLvlAtkMul(int empire, int level) throws InvalidArgumentsException
    {
        checkIndex(empire, entries.length - 1);
        checkIndex(level, 4);
        return entries[empire].diff_level_attack_mult[level];
    }

    public int getMoraleLvlAtkMul(int empire, int level) throws InvalidArgumentsException
    {
        checkIndex(empire, entries.length - 1);
        checkIndex(level, 7);
        return entries[empire].morale_level_attack_mult[level];
    }

    public int getTargetTreatyAtkMul(int empire, int treaty) throws InvalidArgumentsException
    {
        checkIndex(empire, entries.length - 1);
        checkIndex(treaty, 3);
        return entries[empire].target_treaty_attack_mult[treaty];
    }

    public void setBaseEspAtkMul(int empire, int mult) throws InvalidArgumentsException
    {
        checkIndex(empire, entries.length - 1);

        if (entries[empire].base_esp_attack_mult != mult)
        {
            entries[empire].base_esp_attack_mult = mult;
            markChanged();
        }
    }

    public void setBaseSabAtkMul(int empire, int mult) throws InvalidArgumentsException
    {
        checkIndex(empire, entries.length - 1);
        if (entries[empire].base_sab_attack_mult != mult)
        {
            entries[empire].base_sab_attack_mult = mult;
            markChanged();
        }
    }

    public void setTargetEmpAtkMul(int empire, int target, int mult) throws InvalidArgumentsException
    {
        checkIndex(empire, entries.length - 1);
        checkIndex(target, entries.length - 1);
        if (entries[empire].target_emp_attack_mult[target] != mult)
        {
            entries[empire].target_emp_attack_mult[target] = mult;
            markChanged();
        }
    }

    public void setDiffLvlAtkMul(int empire, int level, int mult) throws InvalidArgumentsException
    {
        checkIndex(empire, entries.length - 1);
        checkIndex(level, 4);

        if (entries[empire].diff_level_attack_mult[level] != mult)
        {
            entries[empire].diff_level_attack_mult[level] = mult;
            markChanged();
        }
    }

    public void setMoraleLvlAtkMul(int empire, int level, int mult) throws InvalidArgumentsException
    {
        checkIndex(empire, entries.length - 1);
        checkIndex(level, 7);
        
        if (entries[empire].morale_level_attack_mult[level] != mult)
        {
            entries[empire].morale_level_attack_mult[level] = mult;
            markChanged();
        }
    }

    public void setTargetTreatyAtkMul(int empire, int treaty, int mult) throws InvalidArgumentsException
    {
        checkIndex(empire, entries.length - 1);
        checkIndex(treaty, 3);

        if (entries[empire].target_treaty_attack_mult[treaty] != mult)
        {
            entries[empire].target_treaty_attack_mult[treaty] = mult;
            markChanged();
        }
    }

    private void checkIndex(int index, int max) throws InvalidArgumentsException
    {
        if (index > max)
        {
            throw new InvalidArgumentsException(CommonStrings.getLocalizedString(
                    CommonStrings.InvalidIndex_1, new String[]{Integer.toString(index)}));
        }
    }

    private class IntelEntry
    {
        public int base_esp_attack_mult;          // base multiplier for espionage attack threshold (4)
        public int base_sab_attack_mult;          // base multiplier for sabotage attack threshold (8)
        public int[] target_emp_attack_mult;      // 5x enemy empires multiplier for attack threshold (28)
        public int[] diff_level_attack_mult;      // 5x difficulty level = multiplier for attack threshold (48)
        public int[] morale_level_attack_mult;    // 8x empsInfo morale labels = security modifier%
                                                  // written to intelInfo+2C (80)
        public int[] target_treaty_attack_mult;   // 4x empsInfo treaty labels = attack threshold modifier
                                                  // written to intelInfo+4C (96)

        public IntelEntry(InputStream in) throws IOException
        {
            target_emp_attack_mult      = new int[5];
            diff_level_attack_mult      = new int[5];
            morale_level_attack_mult    = new int[8];
            target_treaty_attack_mult   = new int[4];

            byte[] b = new byte[4];
            in.read(b);
            base_esp_attack_mult = Tools.toInt(b, true);
            in.read(b);
            base_sab_attack_mult = Tools.toInt(b, true);

            int i;
            for (i = 0; i < 5; i++)
            {
                in.read(b);
                target_emp_attack_mult[i] = Tools.toInt(b, true);
            }
            for (i = 0; i < 5; i++)
            {
                in.read(b);
                diff_level_attack_mult[i] = Tools.toInt(b, true);
            }
            for (i = 0; i < 8; i++)
            {
                in.read(b);
                morale_level_attack_mult[i] = Tools.toInt(b, true);
            }
            for (i = 0; i < 4; i++)
            {
                in.read(b);
                target_treaty_attack_mult[i] = Tools.toInt(b, true);
            }
        }

        public void save(OutputStream out) throws IOException
        {
            out.write(Tools.toByte(base_esp_attack_mult, true));
            out.write(Tools.toByte(base_sab_attack_mult, true));

            int i;
            for (i = 0; i < 5; i++)
            {
                out.write(Tools.toByte(target_emp_attack_mult[i], true));
            }
            for (i = 0; i < 5; i++)
            {
                out.write(Tools.toByte(diff_level_attack_mult[i], true));
            }
            for (i = 0; i < 8; i++)
            {
                out.write(Tools.toByte(morale_level_attack_mult[i], true));
            }
            for (i = 0; i < 4; i++)
            {
                out.write(Tools.toByte(target_treaty_attack_mult[i], true));
            }
        }
    }
}
