/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: Meplanet.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.edit.res.stbof.bin;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import ue.Language;
import ue.UE;
import ue.edit.CanBeEdited;
import ue.edit.FileManager;
import ue.edit.InternalFile;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.pst.Planet;
import ue.edit.res.stbof.rst.Race;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;

public class Meplanet extends InternalFile
{
    //fields
    private CanBeEdited STBOF;

    private HashMap<String, ArrayList<MeplanetEntry>> ENTRY = new HashMap<String, ArrayList<MeplanetEntry>>(); //contains indexes

    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();
        String msg;
        
        try
        {
            if (ENTRY.size() != 5)
            {
                msg = Language.getString("Meplanet.0"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", Integer.toString(ENTRY.size())); //$NON-NLS-1$
                
                response.add(msg);
            }
            
            Iterator<String> sute = (ENTRY.keySet()).iterator();
            
            String key;
            ArrayList<MeplanetEntry> array;
            int size;
            
            String[] pstnam = new String[0];
            Race rst = null;
            
            if (STBOF != null)
            {
                Planet pst = (Planet)STBOF.getInternalFile(Stbof.filePLANET_PST, true);
                pstnam = pst.getNames();
                rst = (Race)STBOF.getInternalFile(Stbof.fileRACE_RST, true);
            }
            
            MeplanetEntry me;
            
            while (sute.hasNext())
            {
                key = sute.next();
                array = ENTRY.get(key);
                size = array.size();
                
                for (int i = 0; i < size; i++)
                {
                    try
                    {
                        me = array.get(i);
                        
                        if (me.PLANET.length() > 39)
                        {
                            msg = Language.getString("Meplanet.1"); //$NON-NLS-1$
                            msg = msg.replace("%1", getName()); //$NON-NLS-1$
                            msg = msg.replace("%2", me.PLANET); //$NON-NLS-1$
                            
                            response.add(msg);
                        }
                        if (me.RACE.length() > 39)
                        {
                            msg = Language.getString("Meplanet.2"); //$NON-NLS-1$
                            msg = msg.replace("%1", getName()); //$NON-NLS-1$
                            msg = msg.replace("%2", me.RACE); //$NON-NLS-1$
                            
                            response.add(msg);
                        }
                        if (me.COUNT != size)
                        {
                            me.COUNT = size; // auto-fix
                        }
                        if (me.LEVEL < 0)
                        {
                            msg = Language.getString("Meplanet.3"); //$NON-NLS-1$
                            msg = msg.replace("%1", getName()); //$NON-NLS-1$
                            msg = msg.replace("%2", Integer.toString(me.LEVEL)); //$NON-NLS-1$
                            msg = msg.replace("%3", me.PLANET); //$NON-NLS-1$
                            
                            response.add(msg);
                        }
                        
                        //check if planetes and races are in planet.pst, race.pst
                        if (STBOF != null)
                        {
                            int p = -1;
                            
                            for (int k = 0; k < pstnam.length; k++)
                            {
                                if (pstnam[k].equals(me.PLANET))
                                {
                                    p = k;
                                    break;
                                }
                            }
                            
                            if (p < 0)
                            {
                                msg = Language.getString("Meplanet.4"); //$NON-NLS-1$
                                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                                msg = msg.replace("%2", me.PLANET); //$NON-NLS-1$
                                
                                response.add(msg);
                            }
                            int test = -1;
                            for (int h = 0; h < 5; h++)
                            {
                                if ((rst.getName(h)).equals(me.RACE))
                                {
                                    test = h;
                                    break;
                                }
                            }
                            if (test < 0)
                            {
                                msg = Language.getString("Meplanet.5"); //$NON-NLS-1$
                                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                                msg = msg.replace("%2", me.RACE); //$NON-NLS-1$
                                
                                response.add(msg);
                            }
                        }
                    }
                    catch (Exception z2)
                    {
                        z2.printStackTrace();
                        msg = Language.getString("Meplanet.6"); //$NON-NLS-1$
                        msg = msg.replace("%1", getName()); //$NON-NLS-1$
                        msg = msg.replace("%2", z2.getMessage()); //$NON-NLS-1$
                        
                        response.add(msg);
                    }
                }
            }
        }
        catch (Exception z)
        {
            z.printStackTrace();
            msg = Language.getString("Meplanet.6"); //$NON-NLS-1$
            msg = msg.replace("%1", getName()); //$NON-NLS-1$
            msg = msg.replace("%2", z.getMessage()); //$NON-NLS-1$
            
            response.add(msg);
        }
    
        return response;
    }
    
    @Override
    public long getSize()
    {
        long size = 0;
        Iterator<String> sute = (ENTRY.keySet()).iterator();
    
        String key;
        ArrayList<MeplanetEntry> array;
        
        while (sute.hasNext())
        {
            key = sute.next();
            array = ENTRY.get(key);
            size += array.size()*88;
        }
        
        return size;
    }
    
    @Override
    public void save(OutputStream out) throws IOException
    {
        Iterator<String> sute = (ENTRY.keySet()).iterator();
        String key;
        ArrayList<MeplanetEntry> array;
        int size;
        MeplanetEntry me;
        
        while (sute.hasNext())
        {
            key = sute.next();
            array = ENTRY.get(key);
            size = array.size();
            for (int i = 0; i < size; i++)
            {
                me = array.get(i);
                me.COUNT = size;
                me.save(out);
            }
        }
    }
    
    /**
     * Constructor
     * @param in the InputStream to read from
     */
    public Meplanet(InputStream in) throws IOException
    {
        STBOF = UE.FILES.getFile(FileManager.P_STBOF);
        
        MeplanetEntry me;
        ArrayList<MeplanetEntry> array;
	
        while (in.available() > 0)
        {
            me = new MeplanetEntry(in);
            if (ENTRY.containsKey(me.RACE))
            {
                array = ENTRY.get(me.RACE);
            }
            else
            {
                array = new ArrayList<MeplanetEntry>();
            }
            array.add(me);
            ENTRY.put(me.RACE, array);
        }
    }
    
    /**
     * Returns a list of planets
     * @return race the race name
     * @return an array of planet names for the race or null if there are none
     */
    public String[] getPlanets(String race)
    {
        String[] ret = new String[0];
        ArrayList<MeplanetEntry> array = ENTRY.get(race);
	
        if (array != null)
        {
            int size = array.size();
            ret = new String[size];
            MeplanetEntry me;
		
            for (int i = 0; i < size; i++)
            {
                me = array.get(i);
                ret[i] = me.PLANET;
            }
        }
        
        return ret;
    }
    /**
     * Returns all owners of listed planets
     */
    public String[] getOwners()
    {
        Iterator<String> sute = (ENTRY.keySet()).iterator();
        int size = ENTRY.size();
        String key[] = new String[size];
        for (int i = 0; i < size && sute.hasNext(); i++)
        {
            key[i] = new String(sute.next());
        }
        return key;
    }
    
    /**Sets the owner of a group of planets
     * @param old the old owner
     * @param race the new owner
     */
	public boolean setOwner(String old, String race) throws InvalidArgumentsException{
		if (old.equals(race)) {
			return false;
		}
		String msg;
		
		ArrayList<MeplanetEntry> array = ENTRY.get(old);
		if (array == null){
				msg = Language.getString("Meplanet.7"); //$NON-NLS-1$
				msg = msg.replace("%1", old); //$NON-NLS-1$

				throw new InvalidArgumentsException(msg);
		}
		MeplanetEntry me;
		int size = array.size();
		for (int i=0; i<size; i++){
			me = array.get(i);
			me.RACE = new String(race);
		}
		ENTRY.remove(old);
		ENTRY.put(race, array);
                this.markChanged();
		return true;
	}
	/**Sets the planet name
	@param owner the owner
	@param index the planet
	@param name planet name*/
	public boolean setPlanet(String owner, int index, String name) throws InvalidArgumentsException{
		String msg;
		ArrayList<MeplanetEntry> array = ENTRY.get(owner);
		if (array==null){
			msg = Language.getString("Meplanet.7"); //$NON-NLS-1$
			msg = msg.replace("%1", owner); //$NON-NLS-1$

			throw new InvalidArgumentsException(msg);
		}
		MeplanetEntry me = array.get(index);
		if (me==null){
			msg = Language.getString("Meplanet.8"); //$NON-NLS-1$
			msg = msg.replace("%1", owner); //$NON-NLS-1$
			msg = msg.replace("%2", Integer.toString(index)); //$NON-NLS-1$

			throw new InvalidArgumentsException(msg);
		}
		if (me.PLANET.equals(name)) {
			return false;
		}
		me.PLANET = new String(name);
                this.markChanged();
		return true;
	}
	/**Adds owner to list.
	@param owner the new owner to be added*/
	public boolean addOwner(String owner) throws InvalidArgumentsException{
		String msg;
		
		if (ENTRY.containsKey(owner)){
			msg = Language.getString("Meplanet.9"); //$NON-NLS-1$
			msg = msg.replace("%1", owner); //$NON-NLS-1$

			throw new InvalidArgumentsException(msg);
		}
		ArrayList<MeplanetEntry> array = new ArrayList<MeplanetEntry>();
		ENTRY.put(owner, array);
                this.markChanged();
		return true;
	}
	/**Adds planet to list.
	@param owner the new owner off the planet.
	@param planet the planet name*/
	public boolean addPlanet(String owner, String planet) throws InvalidArgumentsException{
		ArrayList<MeplanetEntry> array = ENTRY.get(owner);
		String msg;
		if (array==null){
			msg = Language.getString("Meplanet.7"); //$NON-NLS-1$
			msg = msg.replace("%1", owner); //$NON-NLS-1$

			throw new InvalidArgumentsException(msg);
		}
		MeplanetEntry me = new MeplanetEntry(owner, planet);
		array.add(me);
		ENTRY.put(owner, array);
                this.markChanged();
		return true;
	}
	/**Removes an owner from the list.
	@param owner the owner to be removed*/
	public boolean removeOwner(String owner){
		ENTRY.remove(owner);
                this.markChanged();
		return true;
	}
	/**Removes a planet of an owner from the list.
	@param owner the owner
	@param index the index of the planet to be removed*/
	public boolean removePlanet(String owner, int index) throws InvalidArgumentsException{
		ArrayList<MeplanetEntry> array = ENTRY.get(owner);
		String msg;
		if (array==null){
			msg = Language.getString("Meplanet.7"); //$NON-NLS-1$
			msg = msg.replace("%1", owner); //$NON-NLS-1$

			throw new InvalidArgumentsException(msg);
		}
		array.remove(index);
                this.markChanged();
		return true;
	}
	/**Sets the tech at which the planet will be colonized.
	@param owner the owner
	@param index the index of the planet
	@param level the tech level, 5 for never*/
	public boolean setColonized(String owner, int index, int level) throws InvalidArgumentsException{
		ArrayList<MeplanetEntry> array = ENTRY.get(owner);
		String msg;
		if (array==null){
			msg = Language.getString("Meplanet.7"); //$NON-NLS-1$
			msg = msg.replace("%1", owner); //$NON-NLS-1$

			throw new InvalidArgumentsException(msg);
		}
		MeplanetEntry me = array.get(index);
		if (me==null){
			msg = Language.getString("Meplanet.8"); //$NON-NLS-1$
			msg = msg.replace("%1", owner); //$NON-NLS-1$
			msg = msg.replace("%2", Integer.toString(index)); //$NON-NLS-1$

			throw new InvalidArgumentsException(msg);
		}
		if (level<0 || level>4) {
			level=5;
		}
		if (me.LEVEL == level) {
			return false;
		}
		me.LEVEL = level;
                this.markChanged();
		return true;
	}
	/**Gets the tech at which the planet will be colonized.
	@param owner the owner
	@param index the index of the planet*/
	public int getColonized(String owner, int index) throws InvalidArgumentsException{
		ArrayList<MeplanetEntry> array = ENTRY.get(owner);
		String msg;
		if (array==null){
			msg = Language.getString("Meplanet.7"); //$NON-NLS-1$
			msg = msg.replace("%1", owner); //$NON-NLS-1$

			throw new InvalidArgumentsException(msg);
		}
		MeplanetEntry me = array.get(index);
		if (me==null){
			msg = Language.getString("Meplanet.8"); //$NON-NLS-1$
			msg = msg.replace("%1", owner); //$NON-NLS-1$
			msg = msg.replace("%2", Integer.toString(index)); //$NON-NLS-1$

			throw new InvalidArgumentsException(msg);
		}
		if (me.LEVEL<0 || me.LEVEL>4) {
			me.LEVEL=5;
		}
		return me.LEVEL;
	}

    public void renamePlanet(String old, String name)
    {
        Collection<ArrayList<MeplanetEntry>> c = ENTRY.values();

        for (ArrayList<MeplanetEntry> list : c)
        {
            for (MeplanetEntry e : list)
            {
                if (e.PLANET.equals(old))
                {
                    e.PLANET = name;
                }
            }
        }
    }
	
	
	private class MeplanetEntry{
		public String PLANET;
		public String RACE;
		public int LEVEL;
		public int COUNT;
		//constructors
		public MeplanetEntry(InputStream in) throws IOException{
			byte[] b = new byte[40];
			//planet
			in.read(b);
			PLANET = Tools.toASCII(b);
			//race
			in.read(b);
			RACE = Tools.toASCII(b);
			//colonize level
			b = new byte[4];
			in.read(b);
			LEVEL = Tools.toInt(b, true);
			//counter
			in.read(b);
			COUNT = Tools.toInt(b, true);
		}
		public MeplanetEntry(MeplanetEntry me){
			PLANET = new String(me.PLANET);
			RACE = new String(me.RACE);
			LEVEL = me.LEVEL;
			COUNT = me.COUNT;
		}
		public MeplanetEntry(String owner, String name){
			PLANET = new String(name);
			RACE = new String(owner);
			LEVEL = 0;
			COUNT = 0;
		}
		//save
		public boolean save(OutputStream out) throws IOException{
			byte[] b = Tools.toByte(new String(PLANET), 40, 39);
			out.write(b);
			b = Tools.toByte(new String(RACE), 40, 39);
			out.write(b);
			//colonize level
			b = Tools.toByte(LEVEL, true);
			out.write(b);
			//counter
			b = Tools.toByte(COUNT, true);
			out.write(b);
			return true;
		}
	}
}

