/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: Starname.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.edit.res.stbof.bin;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.util.Tools;

/**This class is the representation of the starname.bin file,
it contains names of generic stars - those that aren't home systems to any race.*/
public class Starname extends InternalFile{
	private ArrayList<String> STARS;
	
	/**@param in the InputStream to read from*/
	public Starname(InputStream in) throws IOException{
		//init
		STARS = new ArrayList<String>();
		//read num and throw away
		byte[] b = new byte[4];
		in.read(b);
		//read entries
		byte[] nameb = new byte[24];
		String strName;

		while (in.available()>0){
			//star name
			in.read(nameb);
			strName = Tools.toASCII(nameb, 0, 24);
			STARS.add(strName);
		}
	}

	/**Used to save changes.
	@param out the OutputStream to write the file to.*/
	@Override
	public void save(OutputStream out) throws Exception{ 
		byte[] b;
		//actualy ignores saved data and counts num of entries instead
		int lu = STARS.size();
		b = Tools.toByte(lu, true);
		out.write(b);
		//entries
		for (int i=0; i<STARS.size(); i++){
			//name
			String name = STARS.get(i);
			byte[] writeBuffer = Tools.toByte(name, 20, 19);
			out.write(writeBuffer);
			int ix = writeBuffer.length + 1;
			out.write(Tools.toByte(ix, true));
		}
	}
	
	/**@return uncompressed size of the file.**/
	@Override
	public long getSize(){
		long size = 4 + STARS.size()*24;
		return size;
	}
	
	/**@return number of entries (star names)*/
	public long entries(){
		return STARS.size();
	}
	
	@Override
	public Vector<String> check(){ 
		Vector<String> response = new Vector<String>();
		String msg;
		String str;
		for (int i=0; i<STARS.size(); i++){
			str = STARS.get(i);
			if (str.length() > 19){
				msg = Language.getString(Language.getString("Starname.0")); //$NON-NLS-1$
				msg = msg.replace("%1", getName()); //$NON-NLS-1$
				msg = msg.replace("%2", str); //$NON-NLS-1$
				response.add(msg);
			}
		}
		return response;
	}
	/**Adds a name to the list.
	@param name the name to add
	@return true if the name was added else false*/
	public boolean add(String name){
		if (STARS.contains(name)) {
			return false;
		}
		if (name.length()>19) {
			return false;
		}
		STARS.add(name);
		Collections.sort(STARS);
                this.markChanged();
		return true;
	}
	/**Remove a name to the list.
	@param name the name to remove
	*/
	public void remove(String name){
		STARS.remove(name);
		Collections.sort(STARS);
                this.markChanged();
	}
	/**Changes a name in the list.
	@param oldName the name to be changed
	@param name the new name
	@return true if the name was changed else false*/
	public boolean set(String oldName, String name){
		if (name.length()>19) {
			return false;
		}
		int index = STARS.indexOf(oldName);
		if (index==-1) {
			return false;
		}
		STARS.set(index, name);
		Collections.sort(STARS);
                this.markChanged();
		return true;
	}
	/**@return an array of all the star names in the file*/
	public String[] getEntries(){
		Collections.sort(STARS);
		return STARS.toArray(new String[1]);
	}
}
