/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: Building.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.edit.res.stbof.bst;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import java.util.Arrays;
import ue.UE;
import ue.util.Tools;

/**This class is a representation of a building.*/
public class Building implements Comparable<Building>{
	//fields
	public String NAME_SINGULAR;            //40 bytes	40
	public String NAME_PLURAL;              //40 bytes	80
	public short ID;                        //02 bytes	82
	public short UPGRADE_FOR;               //02 bytes	84
	private byte[] UNKNOWN_1 = new byte[4];	//04 bytes	88
	public String PICTURE;                  //08 bytes	96
	private byte[] UNKNOWN_2 = new byte[5];	//05 bytes	101
	public byte SCI_TYPE;                   //01 bytes	102
	public byte SCI_LEVEL;                  //01 byte	103
	public byte EXCLUDE_MAJOR;              //01 byte	104
	public long RACE_REQ;                   //08 bytes	112
	public byte POLITIC_REQ;                //01 byte	113
	public byte SYSTEM_REQ;                 //01 byte	114
	public byte BONI_TYPE;                  //01 byte	115
	public byte FREQ;                       //01 byte	116
	public byte GROUP;                      //01 byte	117
	public byte CATEGORY;                   //01 byte	118
	private byte[] UNKNOWN_4 = new byte[2];	//02 bytes	120
	public int INDUSTRY_COST;               //02 bytes	122
	private byte[] UNKNOWN_5 = new byte[2];	//02 bytes	124
	public short ENERGY_COST;               //02 bytes	126
	private byte[] UNKNOWN_6 = new byte[2];	//02 bytes	128
	public short PPL_COST;                  //02 bytes	130
	public short MORALE_BONI;               //02 bytes	132
	private byte[] UNKNOWN_7 = new byte[2];	//02 bytes	134
	public short[] BONUS = new short[6];	//12 bytes	146
	public short[] BONUS_2 = new short[6];	//12 bytes	158
	private byte[] UNKNOWN_8 = new byte[14];//12 bytes	172
	public int DESC_ADDRESS;                //04 bytes	176
	
	public Building(InputStream in) throws IOException{
		//name 0-80
		byte[] b = new byte[40];
		in.read(b);
		NAME_SINGULAR = Tools.toASCII(b);
		in.read(b);
		NAME_PLURAL = Tools.toASCII(b);
		//id 80-2
		b = new byte[2];
		in.read(b);
		ID = Tools.toShort(b, true);
		//upgrade for 82-2
		in.read(b);
		UPGRADE_FOR = Tools.toShort(b, true);
		//unknown 84-4
		in.read(UNKNOWN_1);
		//picture 88-8
		b = new byte[8]; in.read(b); PICTURE = Tools.toASCII(b);
		//unknown 96-5
		in.read(UNKNOWN_2);
		//science type 101-1
		SCI_TYPE = (byte)in.read();
		//science level 102-1
		SCI_LEVEL = (byte)in.read();
		//major that can't build it? (mask) 103-1
		EXCLUDE_MAJOR = (byte)in.read();
		//race that needs to be on the system (mask) 104-8
		b = new byte[8]; in.read(b); RACE_REQ = Tools.toLong(b, true);
		//politic requirement/or rather needed faction 112-1
		POLITIC_REQ = (byte)in.read();
		//system requirement/what the system needs to be like 113-1
		SYSTEM_REQ = (byte)in.read();
		//bonus type 114-1
		BONI_TYPE = (byte)in.read();
		//frequency - how many times the building can be built in a system 115-1
		FREQ = (byte)in.read();
		//unknown 116-1
		GROUP = (byte)in.read();
		//category 117-1
		CATEGORY = (byte)in.read();
		//unknown
		in.read(UNKNOWN_4);
		//industry cost
		byte[] v = new byte[4];
		v[0] = (byte)in.read();
		v[1] = (byte)in.read();
		INDUSTRY_COST = Tools.toInt(v, true);
		//unknown
		in.read(UNKNOWN_5);
		//energy cost
		b = new byte[2]; in.read(b); ENERGY_COST = Tools.toShort(b, true);
		//unknown
		in.read(UNKNOWN_6);
		//people cost
		in.read(b);
		PPL_COST = Tools.toShort(b, true);
		//morale boni
		b = new byte[2]; in.read(b); MORALE_BONI = Tools.toShort(b, true);
		//unknown
		in.read(UNKNOWN_7);
		//bonus
		b = new byte[2];
		for (int i=0; i<6; i++){
			in.read(b);
			BONUS[i] = Tools.toShort(b, true);
		}
		//bonus #2
		b = new byte[2];
		for (int i=0; i<6; i++){
			in.read(b);
			BONUS_2[i] = Tools.toShort(b, true);
		}
		//unknown
		in.read(UNKNOWN_8);
		//description address
		b = new byte[4]; in.read(b); DESC_ADDRESS = Tools.toInt(b, true);

        if (UE.DEBUG_MODE)
        {
            System.out.println(NAME_SINGULAR + ": Unknown 8:" + Arrays.toString(UNKNOWN_8));
        }
	}
	public Building(Building bg){
		NAME_SINGULAR = new String(bg.NAME_SINGULAR);
		NAME_PLURAL = new String(bg.NAME_PLURAL);
		//id
		ID = bg.ID;
		UPGRADE_FOR = bg.UPGRADE_FOR;
		//unknown
		UNKNOWN_1 = bg.UNKNOWN_1;
		//picture
		PICTURE = new String(bg.PICTURE);
		//science type
		SCI_TYPE = bg.SCI_TYPE;
		//science level
		SCI_LEVEL = bg.SCI_LEVEL;
		//major that can't build it? (mask)
		EXCLUDE_MAJOR = bg.EXCLUDE_MAJOR;
		//race
		RACE_REQ = bg.RACE_REQ;
		//politic requirement/or rather needed faction
		POLITIC_REQ = bg.POLITIC_REQ;
		//system requirement/what the system needs to be like
		SYSTEM_REQ = bg.SYSTEM_REQ;
		//bonus type
		BONI_TYPE = bg.BONI_TYPE;
		//frequency - how many times the building can be built in a system
		FREQ = bg.FREQ;
		//unknown
		UNKNOWN_2 = bg.UNKNOWN_2;
		GROUP = bg.GROUP;
		//category
		CATEGORY = bg.CATEGORY;
		//unknown
		UNKNOWN_8 = bg.UNKNOWN_8;
		//industry cost
		INDUSTRY_COST = bg.INDUSTRY_COST;
		//unknown
		UNKNOWN_4 = bg.UNKNOWN_4;
		//energy cost
		ENERGY_COST = bg.ENERGY_COST;
		//unknown
		UNKNOWN_5 = bg.UNKNOWN_5;
		//people cost
		PPL_COST = bg.PPL_COST;
		//morale boni
		MORALE_BONI = bg.MORALE_BONI;
		//unknown
		UNKNOWN_6 = bg.UNKNOWN_6;
		//bonus
		for (int i=0; i<6; i++) {
			BONUS[i]=bg.BONUS[i];
		}
		for (int i=0; i<6; i++) {
			BONUS_2[i]=bg.BONUS_2[i];
		}
		//unknown
		UNKNOWN_7 = bg.UNKNOWN_7;
		//description address
		DESC_ADDRESS = bg.DESC_ADDRESS;
	}
	public void save(OutputStream out) throws IOException{
		//name
		byte[] b = Tools.toByte(NAME_SINGULAR, 40, 39);
		out.write(b);
		b = Tools.toByte(NAME_PLURAL, 40, 39);
		out.write(b);
		//id
		b = Tools.toByte(ID, true);
		out.write(b);
		//upgrade
		b = Tools.toByte(UPGRADE_FOR, true);
		out.write(b);
		//unknown
		out.write(UNKNOWN_1);
		//picture
		b = Tools.toByte(PICTURE, 8, 7); out.write(b);
		//unknown
		out.write(UNKNOWN_2);
		//science type
		out.write(SCI_TYPE);
		//science level
		out.write(SCI_LEVEL);
		//major that can't build it? (mask)
		out.write(EXCLUDE_MAJOR);
		//race
		out.write(Tools.toByte(RACE_REQ, true));
		//politic requirement/or rather needed faction
		out.write(POLITIC_REQ);
		//system requirement/what the system needs to be like
		out.write(SYSTEM_REQ);
		//bonus type
		out.write(BONI_TYPE);
		//frequency - how many times the building can be built in a system
		out.write(FREQ);
		//unknown
		out.write(GROUP);
		//category
		out.write(CATEGORY);
		//unknown
		out.write(UNKNOWN_4);
		//industry cost
		b = Tools.toByte(INDUSTRY_COST, true);
		out.write(b[0]);
		out.write(b[1]);
		//unknown
		out.write(UNKNOWN_5);
		//energy cost
		b = Tools.toByte(ENERGY_COST, true); out.write(b);
		//unknown
		out.write(UNKNOWN_6);
		//people cost
		b = Tools.toByte(PPL_COST, true); out.write(b);
		//morale boni
		b = Tools.toByte(MORALE_BONI, true); out.write(b);
		//unknown
		out.write(UNKNOWN_7);
		//bonus
		for (int i=0; i<6; i++){
			b = Tools.toByte(BONUS[i], true); out.write(b);
		}
		//bonus 2
		for (int i=0; i<6; i++){
			b = Tools.toByte(BONUS_2[i], true); out.write(b);
		}
		//unknown
		out.write(UNKNOWN_8);
		//description addres
		b = Tools.toByte(DESC_ADDRESS, true); out.write(b);
	}
	@Override
	public boolean equals(Object o){
		if (!(o instanceof Building)) {
			return false;
		}
		Building gu = (Building)o;
		return ID==gu.ID;
	}
	/**Returns the file's hashcode.*/
	@Override
	public int hashCode(){
		return ID;
	}
    @Override
	public int compareTo(Building obj){
		return (ID - obj.ID);
	}
	@Override
	public String toString(){ return NAME_SINGULAR;}
}
