/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: Edifbnft.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.edit.res.stbof.bst;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;
import ue.util.RandomReader;
import ue.util.RandomWriter;

/**This class is a representation of the edifbnft.bst file. It holds a list of building's bonus descriptions.*/
public class Edifbnft extends InternalFile{
	//fields
	private ArrayList<String> ENTRY = new ArrayList<String>();
	//specified by StbofEdit
	@Override
	public Vector<String> check(){
		Vector<String> response = new Vector<String>();
		String msg;
		for (int i=0; i<ENTRY.size(); i++){
			String str = ENTRY.get(i);
			if (str.length()==0){
					msg = Language.getString("Edifbnft.0"); //$NON-NLS-1$
					msg = msg.replace("%1", getName()); //$NON-NLS-1$
					msg = msg.replace("%2", Integer.toString(i)); //$NON-NLS-1$
					response.add(msg);
			}
		}
		return response;
	}
	@Override
	public long getSize(){
		long size = 4 + ENTRY.size()*4;
		String str;
		for (int i=0; i<ENTRY.size(); i++){
			str = ENTRY.get(i);
			size += str.length()+1;
		}
		return size;
	}
	@Override
	public void save(OutputStream out) throws IOException{
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		RandomWriter WRITER = new RandomWriter(baos, (int)getSize());
		//set up addresses
		int size = ENTRY.size();
		int[] addy = new int[size];
		//write strings
		WRITER.seek(4);
		String str;
		for (int z=0; z<size; z++){
			addy[z] = WRITER.getPointer();
			str=ENTRY.get(z);
			WRITER.write(Tools.toByte(str, str.length()+1, str.length()));
		}
		//save address
		int address = WRITER.getPointer();
		//save map
		for (int z=0; z<size; z++){
			WRITER.write(Tools.toByte(addy[z], true));
		}
		//sava map addy
		WRITER.seek(0);
		WRITER.write(Tools.toByte(address, true));
		WRITER.close();
		out.write(baos.toByteArray());
	}
	/**Constructor.*/
	public Edifbnft(InputStream in) throws IOException{
		RandomReader READER = new RandomReader(in);
		//address
		READER.seek(0);
		byte[] b = new byte[4];
		READER.read(b);
		int address = Tools.toInt(b, true);
		//string addresses
		READER.seek(address);
		int t = (READER.available() / 4);
		int addy;
		//strings
		byte[] c;
		for (int i=0; i<t; i++){
			READER.seek(address + i*4);
			READER.read(b);
			addy = Tools.toInt(b, true);
			READER.seek(addy);
			c = new byte[READER.available()];
			READER.read(c);
			ENTRY.add(Tools.toASCII(c));
		}
	}
	/**Returns bonus description.*/
	public String getBoniDesc(int index) throws InvalidArgumentsException{
		if (index<0 || index>=ENTRY.size()){
			String msg = Language.getString("Edifbnft.1"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
		return new String(ENTRY.get(index));
	}
	/**Sets bonus description.*/
	public void setBoniDesc(int index, String desc) throws InvalidArgumentsException{
		if (index<0 || index>=ENTRY.size()){
			String msg = Language.getString("Edifbnft.1"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
		String str = ENTRY.get(index);
		if (str.equals(desc)) {
			return;
		}
		ENTRY.set(index, new String(desc));
                this.markChanged();
	}
	/**Returns the number of entries.*/
	public int getNumberOfEntries(){
		return ENTRY.size();
	}
}