/*
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *
 * File: Edifice.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 */
package ue.edit.res.stbof.bst;

import java.awt.Dimension;
import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Vector;

import ue.Language;
import ue.UE;
import ue.edit.CanBeEdited;
import ue.edit.FileManager;
import ue.edit.InternalFile;
import ue.edit.exe.Trek;
import ue.edit.exe.trek.BasicScannerID;
import ue.edit.exe.trek.CompareBuildingID;
import ue.edit.exe.trek.CompareByteToMemOffset;
import ue.edit.exe.trek.CompareIntReg;
import ue.edit.exe.trek.CompareShortReg;
import ue.edit.exe.trek.MainBldStartingIDs;
import ue.edit.exe.trek.MainIndustryFarmsAndEnergyBld;
import ue.edit.exe.trek.MainIntelBld;
import ue.edit.exe.trek.MainIntelBld2;
import ue.edit.exe.trek.MainResearchBld;
import ue.edit.exe.trek.MainResearchBld2;
import ue.edit.exe.trek.MartialLawBld;
import ue.edit.exe.trek.MartialLawBld2;
import ue.edit.exe.trek.MartialLawBld3;
import ue.edit.exe.trek.MartialLawBld4;
import ue.edit.exe.trek.MinorRaceBldUpgradeIDs;
import ue.edit.exe.trek.MinorRaceMainBldIDLimits;
import ue.edit.exe.trek.MinorRaceSpBuildingIDs;
import ue.edit.exe.trek.PunyFactories;
import ue.edit.exe.trek.PunyFarms;
import ue.edit.exe.trek.PushInt;
import ue.edit.exe.trek.MovIntToReg;
import ue.edit.exe.trek.ShipyardBldIDs;
import ue.edit.exe.trek.SpecialShips;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.bin.AIBldReq;
import ue.edit.res.stbof.rst.Race;
import ue.edit.res.stbof.tec.TechTree;
import ue.exception.InvalidArgumentsException;
import ue.util.Pair;
import ue.util.Tools;
import ue.util.ID;
import ue.util.RandomWriter;

// TODO: make class more memory efficient
/**This class is a representation of the edifice.bst file. It holds a list of buildings.*/
public class Edifice extends InternalFile
{
    Stbof STBOF;
    //fields
    private long[][] IGNORE_LIST = new long[5][6];//5 major races, 6*8*8 = 384 bits
    //buildings
    private ArrayList<Building> ENTRY = new ArrayList<Building>();
    // masks
    public static final byte MASK_ALL = 0;
    // bonus types
    public static final byte BONUS_BUILDS_SHIPS = 18;
    public static final byte BONUS_MORALE_MARTIAL_LAW = 36;
    // building groups simplified
    public static final int GROUP_MAIN_FOOD = 0;
    public static final int GROUP_MAIN_INDUSTRY = 1;
    public static final int GROUP_MAIN_ENERGY = 2;
    public static final int GROUP_MAIN_INTEL = 3;
    public static final int GROUP_MAIN_RESEARCH = 4;
    public static final int GROUP_SCANNERS = 5;
    public static final int GROUP_OTHER = 6;
    // political req
    public static final byte POLITIC_REQ_HOME_SYSTEM = 0;
    public static final byte POLITIC_REQ_NATIVE_MEMBER = 1;
    public static final byte POLITIC_REQ_NON_NATIVE_MEMBER = 2;
    public static final byte POLITIC_REQ_SUBJUGATED = 3;
    public static final byte POLITIC_REQ_AFFILIATED = 4;
    public static final byte POLITIC_REQ_INDEPENDANT_MINOR = 5;
    public static final byte POLITIC_REQ_CONQUERED_HOME = 6;
    public static final byte POLITIC_REQ_NONE = 9;
    public static final byte POLITIC_REQ_REBEL = 10;
    public static final byte POLITIC_REQ_EMPTY_SYSTEM = 11;
    // freq req
    public static final byte FREQ_LIMIT_NONE = 0;
    public static final byte FREQ_LIMIT_ONE_PER_SYSTEM = 1;
    public static final byte FREQ_LIMIT_ONE_PER_EMPIRE = 2;
    // building ids
    public static final int BASIC_SCANNER_ID = 0;
    public static final int ORBITAL_BATTERY_ID = 1;
    public static final int BASIC_DILITHIUM_SOURCE_ID = 2;
    public static final int BASIC_SHIELD_GENERATOR_ID = 3;
    public static final int TRADE_GOODS_ID = 4;
    public static final int BASIC_SHIPYARD_ID = 5;
    public static final int BASIC_GROUND_DEFENSE_ID = 6;
    // building id sets
    public static final int MARTIAL_LAW_ID_SET = 100;
    public static final int SHIPYARD_ID_SET = 101;
    public static final int MINOR_SP_BLD_ID_SET = 102;

    // system requirements
    public static final int SYS_REQ_ARCTIC_PLANET   = 0;
    public static final int SYS_REQ_BARREN_PLANET   = 1;
    public static final int SYS_REQ_DESERT_PLANET   = 2;
    public static final int SYS_REQ_GAS_GIANT       = 3;
    public static final int SYS_REQ_JUNGLE_PLANET   = 4;
    public static final int SYS_REQ_OCEANIC_PLANET  = 5;
    public static final int SYS_REQ_TERRAN_PLANET   = 6;
    public static final int SYS_REQ_VOLCANIC_PLANET = 7;
    public static final int SYS_REQ_NONE            = 8;
    public static final int SYS_REQ_ASTEROID        = 9;
    public static final int SYS_REQ_ASTEROID_BELT   = 10;
    public static final int SYS_REQ_DILITHIUM       = 11;
    public static final int SYS_REQ_WORM_HOLE       = 12;
    public static final int SYS_REQ_RADIO_PULSAR    = 13;
    public static final int SYS_REQ_XRAY_PULSAR     = 14;


    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();
        String msg;

        TechTree tech = null;
        Race race = null;

        try
        {
            tech = (TechTree) STBOF.getInternalFile(Stbof.fileTECHTREE_TEC, true);
        }
        catch (FileNotFoundException ex)
        {
            response.add(this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, ex.getMessage()));
        }

        try
        {
            race = (Race) this.STBOF.getInternalFile(Stbof.fileRACE_RST, false);
        }
        catch (FileNotFoundException ex)
        {
            response.add(this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, ex.getMessage()));
        }

        Building test = null; 	//the building currently being tested
        Building b, b2;

        for (int i = 0; i < ENTRY.size(); i++) {
            try {
                test = ENTRY.get(i);

                if (test.NAME_SINGULAR.length() > 39) {
                    msg = Language.getString("Edifice.0"); //$NON-NLS-1$
                    msg = msg.replace("%1", getName()); //$NON-NLS-1$
                    msg = msg.replace("%2", test.NAME_SINGULAR); //$NON-NLS-1$
                    response.add(msg);
                }
                if (test.NAME_PLURAL.length() > 39) {
                    msg = Language.getString("Edifice.0"); //$NON-NLS-1$
                    msg = msg.replace("%1", getName()); //$NON-NLS-1$
                    msg = msg.replace("%2", test.NAME_PLURAL); //$NON-NLS-1$
                    response.add(msg);
                }
                if (test.ID < 0) {
                    msg = Language.getString("Edifice.1"); //$NON-NLS-1$
                    msg = msg.replace("%1", getName()); //$NON-NLS-1$
                    msg = msg.replace("%2", test.NAME_SINGULAR); //$NON-NLS-1$
                    msg = msg.replace("%3", Short.toString(test.ID)); //$NON-NLS-1$
                    response.add(msg);
                }
                if (test.PICTURE.length() > 12) {
                    msg = Language.getString("Edifice.2"); //$NON-NLS-1$
                    msg = msg.replace("%1", getName()); //$NON-NLS-1$
                    msg = msg.replace("%2", test.PICTURE); //$NON-NLS-1$

                    response.add(msg);
                }
                if ((test.SCI_TYPE < 0) || (test.SCI_TYPE > 6)) {
                    msg = Language.getString("Edifice.3"); //$NON-NLS-1$
                    msg = msg.replace("%1", getName()); //$NON-NLS-1$
                    msg = msg.replace("%2", test.NAME_SINGULAR); //$NON-NLS-1$
                    msg = msg.replace("%3", Integer.toString(test.SCI_TYPE)); //$NON-NLS-1$

                    response.add(msg);
                }
                if (tech != null) {
                    if ((test.SCI_LEVEL < 0) || (test.SCI_LEVEL > tech.getMaxTechLevel(test.SCI_TYPE))) {
                        msg = Language.getString("Edifice.4"); //$NON-NLS-1$
                        msg = msg.replace("%1", getName()); //$NON-NLS-1$
                        msg = msg.replace("%2", test.NAME_SINGULAR); //$NON-NLS-1$
                        msg = msg.replace("%3", Integer.toString(test.SCI_LEVEL)); //$NON-NLS-1$

                        response.add(msg);
                    }
                }
                if ((test.SYSTEM_REQ < 0) || (test.SYSTEM_REQ > 14)) {
                    msg = Language.getString("Edifice.5"); //$NON-NLS-1$
                    msg = msg.replace("%1", getName()); //$NON-NLS-1$
                    msg = msg.replace("%2", test.NAME_SINGULAR); //$NON-NLS-1$
                    msg = msg.replace("%3", Integer.toString(test.SYSTEM_REQ)); //$NON-NLS-1$

                    response.add(msg);
                }
                if ((test.INDUSTRY_COST < 0) && (test.INDUSTRY_COST > 65535)) {
                    msg = Language.getString("Edifice.6"); //$NON-NLS-1$
                    msg = msg.replace("%1", getName()); //$NON-NLS-1$
                    msg = msg.replace("%2", test.NAME_SINGULAR); //$NON-NLS-1$
                    msg = msg.replace("%3", Integer.toString(test.INDUSTRY_COST)); //$NON-NLS-1$

                    response.add(msg);
                }
                if (test.ENERGY_COST < 0) {
                    msg = Language.getString("Edifice.7"); //$NON-NLS-1$
                    msg = msg.replace("%1", getName()); //$NON-NLS-1$
                    msg = msg.replace("%2", test.NAME_SINGULAR); //$NON-NLS-1$
                    msg = msg.replace("%3", Integer.toString(test.ENERGY_COST)); //$NON-NLS-1$

                    response.add(msg);
                }
                if ((test.CATEGORY < 0) || (test.CATEGORY > 4)) {
                    msg = Language.getString("Edifice.8"); //$NON-NLS-1$
                    msg = msg.replace("%1", getName()); //$NON-NLS-1$
                    msg = msg.replace("%2", test.NAME_SINGULAR); //$NON-NLS-1$
                    msg = msg.replace("%3", Integer.toString(test.CATEGORY)); //$NON-NLS-1$

                    response.add(msg);
                }
                if ((test.POLITIC_REQ < 0) || (test.POLITIC_REQ > 11) || (test.POLITIC_REQ == 7) || (test.POLITIC_REQ == 8)) {
                    msg = Language.getString("Edifice.9"); //$NON-NLS-1$
                    msg = msg.replace("%1", getName()); //$NON-NLS-1$
                    msg = msg.replace("%2", test.NAME_SINGULAR); //$NON-NLS-1$
                    msg = msg.replace("%3", Integer.toString(test.POLITIC_REQ)); //$NON-NLS-1$

                    response.add(msg);
                }
                if ((test.FREQ < 0) || (test.FREQ > 2)) {
                    msg = Language.getString("Edifice.10"); //$NON-NLS-1$
                    msg = msg.replace("%1", getName()); //$NON-NLS-1$
                    msg = msg.replace("%2", test.NAME_SINGULAR); //$NON-NLS-1$
                    msg = msg.replace("%3", Integer.toString(test.FREQ)); //$NON-NLS-1$

                    response.add(msg);
                }
                if ((test.PPL_COST != 0) && (test.PPL_COST != 10)) {
                    msg = Language.getString("Edifice.11"); //$NON-NLS-1$
                    msg = msg.replace("%1", getName()); //$NON-NLS-1$
                    msg = msg.replace("%2", test.NAME_SINGULAR); //$NON-NLS-1$
                    msg = msg.replace("%3", Integer.toString(test.PPL_COST)); //$NON-NLS-1$

                    response.add(msg);
                }

                int mask;

                for (int emp = 0; emp < 5; emp++) {
                    mask = (byte) Math.pow(2, emp);

                    if ((test.EXCLUDE_MAJOR & mask) == 0) {
                        if (test.BONUS[emp] == 0) {
                            if (test.BONI_TYPE != 18) {
                                msg = Language.getString("Edifice.22"); //$NON-NLS-1$
                                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                                msg = msg.replace("%2", test.NAME_SINGULAR); //$NON-NLS-1$

                                response.add(msg);
                                break;
                            }
                        }
                    }
                }

                if ((test.UPGRADE_FOR >= 0) && (test.UPGRADE_FOR < ENTRY.size()) && ((test.EXCLUDE_MAJOR & (byte) 0x1F) != (byte) 0x1F)) {
                    b = ENTRY.get(test.UPGRADE_FOR);

                    if (((test.EXCLUDE_MAJOR & b.EXCLUDE_MAJOR) != 0) && ((test.RACE_REQ & b.RACE_REQ) != 0)) {
                        if (b.INDUSTRY_COST > test.INDUSTRY_COST) {
                            msg = Language.getString("Edifice.20"); //$NON-NLS-1$
                            msg = msg.replace("%1", getName()); //$NON-NLS-1$
                            msg = msg.replace("%2", b.NAME_SINGULAR); //$NON-NLS-1$
                            msg = msg.replace("%3", test.NAME_SINGULAR); //$NON-NLS-1$

                            response.add(msg);
                        } else if (b.INDUSTRY_COST == test.INDUSTRY_COST) {
                            msg = Language.getString("Edifice.21"); //$NON-NLS-1$
                            msg = msg.replace("%1", getName()); //$NON-NLS-1$
                            msg = msg.replace("%2", b.NAME_SINGULAR); //$NON-NLS-1$
                            msg = msg.replace("%3", test.NAME_SINGULAR); //$NON-NLS-1$

                            response.add(msg);
                        }

                        // if this is a main building, check buildings from other races
                        if ((test.BONI_TYPE == 0) || (test.BONI_TYPE == 2) || (test.BONI_TYPE == 7) || (test.BONI_TYPE == 10) || (test.BONI_TYPE == 13)) {
                            if (test.BONI_TYPE == b.BONI_TYPE) {
                                for (int j = 0; j < ENTRY.size(); j++) {
                                    b2 = ENTRY.get(j);

                                    if ((b.BONI_TYPE == b2.BONI_TYPE) && (b.SCI_LEVEL == b2.SCI_LEVEL) && (b.ID != b2.ID) && ((b2.EXCLUDE_MAJOR & (byte) 0x1F) != (byte) 0x1F)) {
                                        if (b2.INDUSTRY_COST > test.INDUSTRY_COST) {
                                            msg = Language.getString("Edifice.20"); //$NON-NLS-1$
                                            msg = msg.replace("%1", getName()); //$NON-NLS-1$
                                            msg = msg.replace("%2", b2.NAME_SINGULAR); //$NON-NLS-1$
                                            msg = msg.replace("%3", test.NAME_SINGULAR); //$NON-NLS-1$

                                            response.add(msg);
                                        } else if (b2.INDUSTRY_COST == test.INDUSTRY_COST) {
                                            msg = Language.getString("Edifice.21"); //$NON-NLS-1$
                                            msg = msg.replace("%1", getName()); //$NON-NLS-1$
                                            msg = msg.replace("%2", b2.NAME_SINGULAR); //$NON-NLS-1$
                                            msg = msg.replace("%3", test.NAME_SINGULAR); //$NON-NLS-1$

                                            response.add(msg);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                setGroup(i, getGroup(i));
            } catch (Exception ftz) {
                msg = Language.getString("Edifice.16"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", ftz.getMessage()); //$NON-NLS-1$

                response.add(msg);
            }
        }

        int f = -1;
        int l = -1;
        int group1 = 0, group2 = 0;
        byte mask;
        byte exemp;
        int id;

        try
        {
            id = getTrekExeID(Edifice.BASIC_SCANNER_ID);

            if (id < 0)
            {
                msg = "Missing basic scanner available to all races.";
                response.add(this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg));
            }

            id = getTrekExeID(Edifice.ORBITAL_BATTERY_ID);

            if (id < 0)
            {
                msg = "Missing orbital battery available to all races.";
                response.add(this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg));
            }
            else if (id > Byte.MAX_VALUE)
            {
                msg = "Orbital battery found but has id > %1.".replace("%1", Byte.toString(Byte.MAX_VALUE));
                response.add(this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg));
            }

            id = getTrekExeID(Edifice.BASIC_DILITHIUM_SOURCE_ID);

            if (id < 0)
            {
                msg = "Missing basic dilithium source available to all races.";
                response.add(this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg));
            }

            id = getTrekExeID(Edifice.BASIC_SHIELD_GENERATOR_ID);

            if (id < 0)
            {
                msg = "Missing basic shield generator available to all races.";
                response.add(this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg));
            }

            id = getTrekExeID(Edifice.TRADE_GOODS_ID);

            if (id < 0)
            {
                msg = "Missing trade goods available to all races.";
                response.add(this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg));
            }

            id = getTrekExeID(Edifice.BASIC_GROUND_DEFENSE_ID);

            if (id < 0)
            {
                msg = "Missing basic ground defense available to all races.";
                response.add(this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg));
            }

            id = getTrekExeID(Edifice.BASIC_SHIPYARD_ID);

            if (id < 0)
            {
                msg = "Missing basic shipyard available to all races.";
                response.add(this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg));
            }
        }
        catch (Exception ex)
        {
            msg = Language.getString("Edifice.16"); //$NON-NLS-1$
            msg = msg.replace("%1", getName()); //$NON-NLS-1$
            msg = msg.replace("%2", ex.getMessage()); //$NON-NLS-1$
            response.add(msg);
        }

        for (int emp = 0; emp < 5; emp++)
        {
            // check martial law
            try
            {
                id = this.getTrekExeID(MARTIAL_LAW_ID_SET, emp);

                if (id < 0)
                {
                    msg = "Missing martial law building for the %1.";

                    if (race != null)
                    {
                        try {
                            msg = msg.replace("%1", race.getName(emp));
                        } catch (InvalidArgumentsException ex) {
                            response.add(race.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, ex.getMessage()));
                        }
                    }
                    else
                    {
                         msg = msg.replace("%1", "empire with id %1.".replace("%1", Integer.toString(emp)));
                    }

                    response.add(this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg));
                }

                for (int g = 0; g < 5; g++) // find main buildings
                {
                    f = 0;
                    l = 0;

                    switch (g) // searching for a continous line of buildings belonging to group 1 or 2
                    {
                        case 0: {
                            group1 = group2 = Edifice.GROUP_MAIN_RESEARCH;
                            break;
                        }
                        case 1: {
                            group1 = group2 = Edifice.GROUP_MAIN_INTEL;
                            break;
                        }
                        case 2: {
                            group1 = Edifice.GROUP_MAIN_INDUSTRY;
                            group2 = Edifice.GROUP_MAIN_FOOD;
                            break;
                        }
                        case 3: {
                            group1 = group2 = Edifice.GROUP_MAIN_ENERGY;
                            break;
                        }
                        case 4: {
                            group1 = group2 = Edifice.GROUP_MAIN_FOOD;
                            break;
                        }
                    }

                    mask = (byte) Math.pow(2, emp);

                    for (int i = 0; i < this.getNumberOfEntries(); i++)
                    {
                        exemp = this.getExEmpire(i);

                        // check empire and if pop requirement > 0
                        if (((exemp & mask) == 0) && (this.getPopCost(i) > 0)) {
                            if ((this.getGroup(i) == group1) || (this.getGroup(i) == group2)) {
                                // found first building
                                f = i;

                                // read next until not the same
                                while ((i < this.getNumberOfEntries()) && ((exemp & mask) == 0) && (this.getPopCost(i) > 0) && ((this.getGroup(i) == group1) || (this.getGroup(i) == group2))) {
                                    i++;
                                }

                                l = i - 1; // found last

                                break; // quit search - we're done
                            }
                        }
                    }

                    msg = ""; //$NON-NLS-1$

                    switch (group1) {
                        case GROUP_MAIN_RESEARCH: {
                            msg = Language.getString("Edifice.23"); //$NON-NLS-1$
                            msg = msg.replace("%1", getName()); //$NON-NLS-1$
                            msg = msg.replace("%2", Integer.toString(emp)); //$NON-NLS-1$
                            msg = msg.replace("%3", Integer.toString(f)); //$NON-NLS-1$
                            msg = msg.replace("%4", Integer.toString(l)); //$NON-NLS-1$
                            break;
                        }
                        case GROUP_MAIN_INTEL: {
                            msg = Language.getString("Edifice.24"); //$NON-NLS-1$
                            msg = msg.replace("%1", getName()); //$NON-NLS-1$
                            msg = msg.replace("%2", Integer.toString(emp)); //$NON-NLS-1$
                            msg = msg.replace("%3", Integer.toString(f)); //$NON-NLS-1$
                            msg = msg.replace("%4", Integer.toString(l)); //$NON-NLS-1$
                            break;
                        }
                        case GROUP_MAIN_INDUSTRY: {   // industry and food (group2 == food)
                            msg = Language.getString("Edifice.25"); //$NON-NLS-1$
                            msg = msg.replace("%1", getName()); //$NON-NLS-1$
                            msg = msg.replace("%2", Integer.toString(emp)); //$NON-NLS-1$
                            msg = msg.replace("%3", Integer.toString(f)); //$NON-NLS-1$
                            msg = msg.replace("%4", Integer.toString(l)); //$NON-NLS-1$
                            break;
                        }
                        case GROUP_MAIN_ENERGY: {
                            msg = Language.getString("Edifice.26"); //$NON-NLS-1$
                            msg = msg.replace("%1", getName()); //$NON-NLS-1$
                            msg = msg.replace("%2", Integer.toString(emp)); //$NON-NLS-1$
                            msg = msg.replace("%3", Integer.toString(f)); //$NON-NLS-1$
                            msg = msg.replace("%4", Integer.toString(l)); //$NON-NLS-1$
                            break;
                        }
                        case GROUP_MAIN_FOOD:
                        {
                            while ((this.getTechLvl(f) == 0) && (f < l)) {   // ignore primitive farms
                                f++;
                            }

                            msg = Language.getString("Edifice.27"); //$NON-NLS-1$
                            msg = msg.replace("%1", getName()); //$NON-NLS-1$
                            msg = msg.replace("%2", Integer.toString(emp)); //$NON-NLS-1$
                            msg = msg.replace("%3", Integer.toString(f)); //$NON-NLS-1$
                            msg = msg.replace("%4", Integer.toString(l)); //$NON-NLS-1$
                            break;
                        }
                    }

                    response.add(msg);
                }
            } catch (Exception e)
            {
                response.add(this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR,
                        e.getLocalizedMessage()));
            }
        }

        for (int r = 5; r < race.getNumberOfEntries(); r++)
        {
            try
            {
                // check minors special building
                id = this.getTrekExeID(Edifice.MINOR_SP_BLD_ID_SET, r);

                if (id < 0)
                {
                    msg = "Missing special minor race building for the %1.";

                    if (race != null)
                    {
                        try
                        {
                            msg = msg.replace("%1", race.getName(r));
                        }
                        catch (InvalidArgumentsException ex)
                        {
                            response.add(race.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, ex.getMessage()));
                        }
                    }
                    else
                    {
                         msg = msg.replace("%1", "empire with id %1.".replace("%1", Integer.toString(r)));
                    }

                    response.add(this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_WARNING, msg));
                }
            } catch (Exception e)
            {
                response.add(this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR,
                        e.getLocalizedMessage()));
            }
        }

        return response;
    }

    @Override
    public ArrayList<Pair<String, Dimension>> getImagesListedInThisFile()
    {
        ArrayList<Pair<String, Dimension>> images = new ArrayList<Pair<String, Dimension>>();

        Dimension size1 = new Dimension(30, 30);
        Dimension size2 = new Dimension(60, 50);
        Dimension size3 = new Dimension(270, 225);
        Dimension size4 = new Dimension(170, 142);
        Dimension size5 = new Dimension(120, 100);

        for (Building test : ENTRY)
        {
            images.add(new Pair<String, Dimension>(test.PICTURE + ".tga", size5));
            images.add(new Pair<String, Dimension>(test.PICTURE + "_30.tga", size1));
            images.add(new Pair<String, Dimension>(test.PICTURE + "_60.tga", size2));
            images.add(new Pair<String, Dimension>(test.PICTURE + "_b.tga", size3));
            images.add(new Pair<String, Dimension>(test.PICTURE + "170.tga", size4));

            if (test.BONI_TYPE == 36 &&
                test.ENERGY_COST == 0 &&
                test.INDUSTRY_COST == 0 &&
                test.PPL_COST == 0)
            {   // don't add morale buildings *w.tga image
                continue;
            }
            
            images.add(new Pair<String, Dimension>(test.PICTURE + "w.tga", size5));
        }

        return images;
    }

    /**Returns the size of the file*/
    @Override
    public long getSize() {
        long size = 740 + ENTRY.size() * 180;
        return size;
    }

    /**Saves file to OutputStream.*/
    @Override
    public void save(OutputStream out) throws IOException, Exception {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        RandomWriter writer = new RandomWriter(baos, (int) this.getSize());

        final int buildings_num = ENTRY.size();

        writer.seek(0);

        //the tech map
        int[] sorted = new int[77];	//contains addresses where buildings with the same tech and level are (7 techs * 10 lvls)
        short[] num = new short[77];	//contains the number of buildings with the same tech and level (7 techs * 10 levels)

        //number of buildings 0-4
        writer.write(Tools.toByte(buildings_num, true));

        //skip tech and map addresses 4-496
        writer.write(new byte[496]);

        //ignore list 500-240
        this.updateIgnoreList();
        this.updateExe();

        for (int i = 0; i < 5; i++) {//for each major race
            for (int j = 0; j < 6; j++) {
                writer.write(Tools.toByte(IGNORE_LIST[i][j], true));
            }
        }

        //buildings 740-740+176*x
        Building building;

        int[] addresses = new int[ENTRY.size()]; //this array will contain addresses of buildings

        for (byte tech = 0; tech < 7; tech++) { 						//go thru all technologies
            for (byte lvl = 0; lvl < 10; lvl++) { 					//go thru all levels
                int block_address = writer.getPointer(); 			//get current address
                short bld_num = 0;					//number of buildings with this tech and level
                for (int i = 0; i < buildings_num; i++) {			//go thru all buildings
                    building = ENTRY.get(i);				//load building
                    if ((building.SCI_TYPE == tech) && (building.SCI_LEVEL == lvl)) {//if level and tech match
                        addresses[building.ID] = writer.getPointer();	//save current (building) address
                        bld_num++;					//increase counter

                        // last minute fixes
                        if (building.UPGRADE_FOR >= ENTRY.size())
                        {
                            building.UPGRADE_FOR = 32767;
                        }
                        if (this.getGroup(i) == Edifice.GROUP_MAIN_FOOD && building.SCI_LEVEL == 0)
                        {
                            building.GROUP = 5;
                        }


                        building.save(writer);				//writes the building data
                    }
                }
                //save data for tech-map
                if (bld_num != 0) {
                    sorted[tech * 11 + lvl] = block_address;
                } else {
                    sorted[tech * 11 + lvl] = 0;
                }

                num[tech * 11 + lvl] = bld_num; //number
            }

            // levels above 9
            int block_address = writer.getPointer(); 			//get current address
            short bld_num = 0;

            for (int i = 0; i < buildings_num; i++) {			//go thru all buildings
                building = ENTRY.get(i);				//load building

                if ((building.SCI_TYPE == tech) && (building.SCI_LEVEL > 9)) {//if level and tech match
                    addresses[building.ID] = writer.getPointer();	//save current (building) address
                    bld_num++;					//increase counter
                    building.save(writer);				//writes the building data
                }
            }

            //save data for tech-map
            if (bld_num != 0) {
                sorted[tech * 11 + 10] = block_address;
            } else {
                sorted[tech * 11 + 10] = 0;
            }

            num[tech * 11 + 10] = bld_num; //number
        }

        //map address - this is where buildings addresses begin
        //this is difrent from the tech map and is located at the end of file
        int add = writer.getPointer();			//get address where the map begins

        for (int i = 0; i < buildings_num; i++) {	//write address of each building
            writer.write(Tools.toByte(addresses[i], true));
        }
        writer.seek(4);					//go to begining of file (index 4)
        writer.write(Tools.toByte(add, true));	//write address of the map


        //tech map

        //number of buildings in each group
        for (int i = 0; i < 7; i++) { //tech
            short cmp = 0;
            for (int j = 0; j < 11; j++) { //count all levels
                cmp += num[i * 11 + j];
            }
            writer.write(Tools.toByte(cmp, true));
        }

        //addresses and numbers
        writer.write(new byte[2]); //this is just a separator
        for (int tech = 0; tech < 7; tech++) {
            //addresses
            for (int i = 0; i < 11; i++) {
                writer.write(Tools.toByte(sorted[tech * 11 + i], true));
            }
            //void
//			writer.write(new byte[4]); //another separator
            //numbers
            for (int i = 0; i < 11; i++) {
                writer.write(Tools.toByte(num[tech * 11 + i], true));
            }
            //void
//			writer.write(new byte[4]); //again a separator
            writer.write(new byte[2]); //again a separator
        }
        writer.close();
        baos.writeTo(out);
    }

    /**Constructor.*/
    public Edifice(InputStream in, Stbof st) throws Exception {
        STBOF = st;

        in.skip(4);	// skip number of buildings (can read building addresses to the end of the file instead)

        byte[] b = new byte[4];
        in.read(b);	// read address of the map where buildings addresses are located
        int map_address = Tools.toInt(b, true);

        in.skip(492);	//skip the tech map, it will be set when saving anyway

        b = new byte[8];
        for (int i = 0; i < 5; i++) {	//for each empire
            for (int j = 0; j < 6; j++) {//5*8 bytes = 40
                in.read(b);
                //the bits flow from left to right in file, but here they are reversed to ease calculation of masks
                //they are then again reversed when saving
                IGNORE_LIST[i][j] = Tools.toLong(b, true);
            }
        }

        int deadline = in.available() - map_address + 740; //we already read 740 bytes

        Building bld;
        while (in.available() > deadline) {
            bld = new Building(in);
            ENTRY.add(bld);
        }
        //sort
        Collections.sort(ENTRY); //sorted by building's ID

        for (short i = 0; i < ENTRY.size(); i++) { //ID's might be currupted, fix this / each building must have it's own unique ID
            bld = ENTRY.get(i);
            bld.ID = i;
        }
    }

    /**Returns a list of building's short names.*/
    public String[] getShortNames() {
        String[] ret = new String[ENTRY.size()];
        Building[] bt = ENTRY.toArray(new Building[0]);
        for (int i = 0; i < ENTRY.size(); i++) {
            if (bt[i].NAME_SINGULAR.length() > 20) {
                ret[i] = bt[i].NAME_SINGULAR.substring(0, 20) + "..."; //$NON-NLS-1$
            } else {
                ret[i] = new String(bt[i].NAME_SINGULAR);
            }
        }
        return ret;
    }

    /**Returns a list of building's short names.
    0-All
    1-Cards
    2-Feds
    3-Fergs
    4-Klins
    5-Roms
    6-All Empires
    7-Cards Only
    8-Feds Only
    9-Fergs Only
    10-Klins Only
    11-Roms Only
    12-One Resident race only
    @param mask race mask*/
    public ID[] getShortIDs(byte mask, int maxLen)
    {
        ArrayList<ID> ret = new ArrayList<ID>();

        if (mask < 6)
        {
            if (mask != 0) {
                mask = (byte) Math.pow(2, mask - 1);
            } else {
                mask = (byte) 0;
            }
            mask = (byte) ((mask ^ ((byte) 0x1f) & 0x1f));
            Building[] bt = ENTRY.toArray(new Building[0]);
            for (int i = 0; i < ENTRY.size(); i++) {
                if ((bt[i].EXCLUDE_MAJOR & mask) == bt[i].EXCLUDE_MAJOR) {
                    if (bt[i].NAME_SINGULAR.length() > maxLen) {
                        ret.add(new ID(bt[i].NAME_SINGULAR.substring(0, maxLen - 3) + "...", i)); //$NON-NLS-1$
                    } else {
                        ret.add(new ID(new String(bt[i].NAME_SINGULAR), i));
                    }
                }
            }
        } else if (mask == 6) {
            mask = 0;
            Building[] bt = ENTRY.toArray(new Building[0]);
            for (int i = 0; i < ENTRY.size(); i++) {
                if ((bt[i].EXCLUDE_MAJOR | mask) == 0) {
                    if (bt[i].NAME_SINGULAR.length() > maxLen) {
                        ret.add(new ID(bt[i].NAME_SINGULAR.substring(0, maxLen - 3) + "...", i)); //$NON-NLS-1$
                    } else {
                        ret.add(new ID(new String(bt[i].NAME_SINGULAR), i));
                    }
                }
            }
        } else if (mask == 12) {
            Building[] bt = ENTRY.toArray(new Building[0]);
            for (int i = 0; i < ENTRY.size(); i++) {
                boolean ok = true;
                boolean one = false;
                for (int n = 0; (n < 63) && ok; n++) {
                    long z = (long) Math.pow(2, n);
                    if ((bt[i].RACE_REQ & z) == z) {
                        if (!one) {
                            one = true;
                        } else {
                            ok = false;
                        }
                    }
                }
                if (ok) {
                    if (bt[i].NAME_SINGULAR.length() > maxLen) {
                        ret.add(new ID(bt[i].NAME_SINGULAR.substring(0, maxLen - 3) + "...", i)); //$NON-NLS-1$
                    } else {
                        ret.add(new ID(new String(bt[i].NAME_SINGULAR), i));
                    }
                }
            }
        } else {
            mask = (byte) (mask - 6);
            if (mask != 0) {
                mask = (byte) Math.pow(2, mask - 1);
            } else {
                mask = (byte) 0;
            }
            mask = (byte) ((mask ^ ((byte) 0x1f) & 0x1f));
            Building[] bt = ENTRY.toArray(new Building[0]);
            for (int i = 0; i < ENTRY.size(); i++) {
                if ((bt[i].EXCLUDE_MAJOR & mask) == mask) {
                    if (bt[i].NAME_SINGULAR.length() > maxLen) {
                        ret.add(new ID(bt[i].NAME_SINGULAR.substring(0, maxLen - 3) + "...", i)); //$NON-NLS-1$
                    } else {
                        ret.add(new ID(new String(bt[i].NAME_SINGULAR), i));
                    }
                }
            }
        }
        return ret.toArray(new ID[0]);
    }

    public boolean canBeBuiltBy(int index, int controling, int resident) throws InvalidArgumentsException
    {
        this.checkIndex(index);
        Building bt = ENTRY.get(index);

        boolean result = true;

        if (controling < 5)
        {
            byte emp = (byte) Math.pow(2, controling);
            emp = (byte) ((emp ^ ((byte) 0x1f) & 0x1f));

            result = ((bt.EXCLUDE_MAJOR & emp) == bt.EXCLUDE_MAJOR);
        }

        long mask = (long)Math.pow(2, resident);

        return result && ((mask & bt.RACE_REQ) == mask);
    }

    /**Returns building name*/
    public String getName(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        return new String(bg.NAME_SINGULAR);
    }

    /**Returns building plural name*/
    public String getPluralName(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        return new String(bg.NAME_PLURAL);
    }

    /**Sets building name*/
    public void setName(int index, String str) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        if (bg.NAME_SINGULAR.equals(str)) {
            return;
        }
        bg.NAME_SINGULAR = new String(str);
        this.markChanged();
    }

    /**Sets building plural name*/
    public void setPluralName(int index, String str) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        if (bg.NAME_PLURAL.equals(str)) {
            return;
        }
        bg.NAME_PLURAL = new String(str);
        this.markChanged();
    }

    /**Adds a building to the end of the list.
    @param index the index of the building to copy settings from.*/
    public int add(int index) throws InvalidArgumentsException
    {
        this.checkIndex(index);
        if (ENTRY.size() >= 384)
        {
            throw new InvalidArgumentsException(Language.getString("Edifice.17")); //$NON-NLS-1$
        }
        Building ne = new Building(ENTRY.get(index));
        ne.ID = (short) ENTRY.size();
        ENTRY.add(ne);
        ne.UPGRADE_FOR = (short) ENTRY.size();

        // fix other indexes
        for (Building bld : ENTRY)
        {
            if (bld.UPGRADE_FOR == ne.ID)
            {
                bld.UPGRADE_FOR = (short) ENTRY.size();
            }
        }

        this.markChanged();
        return ne.ID;
    }

    /**Returns the number of entries.*/
    public int getNumberOfEntries() {
        return ENTRY.size();
    }

    /**Sets the description address.*/
    public void setDescAddress(int index, int desc) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        if (bg.DESC_ADDRESS == desc) {
            return;
        }
        bg.DESC_ADDRESS = desc;
        this.markChanged();
    }

    /**Removes a building from the list.
    @param index the index of the building to be removed.*/
    public void remove(int index) throws Exception
    {
        this.checkIndex(index);
        ENTRY.remove(index);

        // fix ids in trek.exe
        relocateTrekID(index, -1);

        //sort
        Collections.sort(ENTRY);
        Building bg;
        for (short i = 0; i < ENTRY.size(); i++)
        {
            bg = ENTRY.get(i);

            // fix ids in trek.exe
            relocateTrekID(bg.ID, i);

            bg.ID = i;
        }
        this.markChanged();
    }

    /**Returns the description address.*/
    public int getDescAddress(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        return bg.DESC_ADDRESS;
    }

    /**Sets picture prefix.*/
    public void setPicture(int index, String pic) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        if (bg.PICTURE.compareToIgnoreCase(pic) == 0) {
            return;
        }
        bg.PICTURE = new String(pic);
        this.markChanged();
    }

    /**Returns picture prefix.*/
    public String getPicture(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        return bg.PICTURE.toLowerCase();
    }

    /**Returns required tech index.*/
    public byte getTechReq(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        return bg.SCI_TYPE;
    }

    /**Returns requred tech level.*/
    public byte getTechLvl(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        return bg.SCI_LEVEL;
    }

    /**Sets tech index.
    0-biotech
    1-computer
    2-construction
    3-energy
    4-propulsion
    5-weapons
    6-all
     */
    public void setTechReq(int index, byte tech) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        if (bg.SCI_TYPE == tech) {
            return;
        }
        bg.SCI_TYPE = tech;
        this.markChanged();
    }

    /**Sets tech level.*/
    public void setTechLvl(int index, byte tech) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        if (bg.SCI_LEVEL == tech) {
            return;
        }
        bg.SCI_LEVEL = tech;
        this.markChanged();
    }

    /**Returns requred industry/build cost.*/
    public int getIndCost(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        return bg.INDUSTRY_COST;
    }

    /**Sets industry/build cost.*/
    public void setIndCost(int index, int cost) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        if (bg.INDUSTRY_COST == cost) {
            return;
        }
        bg.INDUSTRY_COST = cost;
        this.markChanged();
    }

    /**Returns requred energy cost.*/
    public short getEnCost(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        return bg.ENERGY_COST;
    }

    /**Sets energy cost.*/
    public void setEnCost(int index, short cost) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);

        if (bg.ENERGY_COST == cost) {
            return;
        }

        try {
            CanBeEdited file;

            if ((file = UE.FILES.getFile(FileManager.P_STBOF)) != null) {
                AIBldReq br = (AIBldReq) file.getInternalFile(Stbof.fileAIBLDREQ_BIN, true);

                if (br.contains(bg.ID)) {
                    if (br.getEnergyReq(bg.ID) == bg.ENERGY_COST) {
                        br.setEnergyReq(bg.ID, cost);
                    }
                }
            }
        } catch (Exception e) {
            // do nothing
        }

        bg.ENERGY_COST = cost;
        this.markChanged();
    }

    /**Returns requred system.*/
    public byte getSysReq(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        return bg.SYSTEM_REQ;
    }

    /**Sets tech index.*/
    public void setSysReq(int index, byte req) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        if (bg.SYSTEM_REQ == req) {
            return;
        }
        bg.SYSTEM_REQ = req;
        this.markChanged();
    }

    /**Returns the category.*/
    public byte getCategory(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        return bg.CATEGORY;
    }

    /**Sets category.*/
    public void setCategory(int index, byte cat) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        if (bg.CATEGORY == cat) {
            return;
        }
        bg.CATEGORY = cat;
        this.markChanged();
    }

    /**Returns the politic req.*/
    public byte getPoliticReq(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        return bg.POLITIC_REQ;
    }

    /**Sets politic req.*/
    public void setPoliticReq(int index, byte req) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        if (bg.POLITIC_REQ == req) {
            return;
        }
        bg.POLITIC_REQ = req;
        this.markChanged();
    }

    /**Returns freq.*/
    public byte getFreq(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        return bg.FREQ;
    }

    /**Sets freq.*/
    public void setFreq(int index, byte freq) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        if (bg.FREQ == freq) {
            return;
        }
        bg.FREQ = freq;
        this.markChanged();
    }

    /**Returns morale bonis.*/
    public short getMoraleBoni(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        return bg.MORALE_BONI;
    }

    /**Sets morale bonis.*/
    public void setMoraleBoni(int index, short boni) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        if (bg.MORALE_BONI == boni) {
            return;
        }
        bg.MORALE_BONI = boni;
        this.markChanged();
    }

    /**Returns bonus type.*/
    public byte getBoniType(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        return bg.BONI_TYPE;
    }

    /**Sets bonus type.*/
    public void setBoniType(int index, byte boni) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        if (bg.BONI_TYPE == boni) {
            return;
        }
        bg.BONI_TYPE = boni;

        this.markChanged();
    }

    /**Returns bonus.*/
    public short getBoni(int index, int maj) throws InvalidArgumentsException {
        this.checkIndex(index);
        this.checkBoniIndex(maj);
        Building bg = ENTRY.get(index);
        return bg.BONUS[maj];
    }

    /**Sets bonus.*/
    public void setBoni(int index, int maj, short val) throws InvalidArgumentsException {
        this.checkIndex(index);
        this.checkBoniIndex(maj);
        Building bg = ENTRY.get(index);
        if (bg.BONUS[maj] == val) {
            return;
        }
        bg.BONUS[maj] = val;
        this.markChanged();
    }

    /**Returns mask indicating which races can't build the building.*/
    public byte getExEmpire(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        return bg.EXCLUDE_MAJOR;
    }

    /**Sets major races that can't build the building.*/
    public void setExEmpire(int index, byte mask) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        if (bg.EXCLUDE_MAJOR == mask) {
            return;
        }
        bg.EXCLUDE_MAJOR = mask;

        this.setGroup(index, this.getGroup(index));
        this.markChanged();
    }

    /**Returns mask indicating which races are needed. A positive bit sets the race as required.*/
    public long getReqRace(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        return bg.RACE_REQ;
    }

    /**Sets which races are needed.*/
    public void setReqRace(int index, long req) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        if (bg.RACE_REQ == req) {
            return;
        }
        bg.RACE_REQ = req;
        this.markChanged();
    }

    /**Returns population maintenance.*/
    public long getPopCost(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        return bg.PPL_COST;
    }

    /**Sets population maintenance.*/
    public void setPopCost(int index, short cost) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);
        if (bg.PPL_COST == cost) {
            return;
        }
        bg.PPL_COST = cost;
        this.markChanged();
    }

    /**Returns downgrade id.*/
    public short getUpgradedFrom(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);

        if (bg.UPGRADE_FOR > ENTRY.size() - 1)
        {
            bg.UPGRADE_FOR = (short) ENTRY.size();
        }

        return bg.UPGRADE_FOR;
    }

    /**Sets downgrade id.*/
    public void setUpgradedFrom(int index, short dw) throws InvalidArgumentsException {
        this.checkIndex(index);
        Building bg = ENTRY.get(index);

        if (bg.UPGRADE_FOR == dw)
        {
            return;
        }
        
        bg.UPGRADE_FOR = dw;
        this.markChanged();
    }

    /**Returns bonus.*/
    public short getBoni2(int index, int maj) throws InvalidArgumentsException {
        this.checkIndex(index);
        this.checkBoniIndex(maj);
        Building bg = ENTRY.get(index);
        return bg.BONUS_2[maj];
    }

    /**Sets bonus.*/
    public void setBoni2(int index, int maj, short val) throws InvalidArgumentsException {
        this.checkIndex(index);
        this.checkBoniIndex(maj);
        Building bg = ENTRY.get(index);
        if (bg.BONUS_2[maj] == val) {
            return;
        }
        bg.BONUS_2[maj] = val;
        this.markChanged();
    }

    /**
     * @param index building index
     * @return true if the building is one of the main buildings
     * @throws InvalidArgumentsException if index is invalid
     */
    public boolean isMainBuilding(int index) throws InvalidArgumentsException {
        int group = this.getGroup(index);

        return ((group == Edifice.GROUP_MAIN_FOOD) || (group == Edifice.GROUP_MAIN_INDUSTRY) || (group == Edifice.GROUP_MAIN_ENERGY) || (group == Edifice.GROUP_MAIN_INTEL) || (group == Edifice.GROUP_MAIN_RESEARCH));
    }

    /**
     * Returns one of the group fields value.
     * @param index building index
     * @return group
     * @throws InvalidArgumentsException
     */
    public int getGroup(int index) throws InvalidArgumentsException {
        this.checkIndex(index);

        Building bld = ENTRY.get(index);

        // this is warped a bit, my guess is the additional slot is meant for neutral
        // it should be:
        //- main food (bonus id 0): group ids 0-4 (cards - roms)
        //- main industry (bonus id 13): 6-10
        //- main energy (bonus id 2): 12-16
        //- main intel (id 10): 18-22
        //- main research (id 7): 24-28
        //- scanners (id 29): 30
        //- others: 34

        if (bld.GROUP <= 5) {
            return Edifice.GROUP_MAIN_FOOD;
        } else if (bld.GROUP <= 11) {
            return Edifice.GROUP_MAIN_INDUSTRY;
        } else if (bld.GROUP <= 17) {
            return Edifice.GROUP_MAIN_ENERGY;
        } else if (bld.GROUP <= 23) {
            return Edifice.GROUP_MAIN_INTEL;
        } else if (bld.GROUP <= 29) {
            return Edifice.GROUP_MAIN_RESEARCH;
        } else if (bld.GROUP == 30) {
            return Edifice.GROUP_SCANNERS;
        } else {
            return Edifice.GROUP_OTHER;
        }
    }

    public void setGroup(int index, int group) throws InvalidArgumentsException {
        this.checkIndex(index);

        Building bld = ENTRY.get(index);

        switch (group) {
            case GROUP_SCANNERS: {
                if (bld.GROUP != 30) {
                    bld.GROUP = 30;
                    this.markChanged();
                }
                return;
            }
            case GROUP_OTHER: {
                if (bld.GROUP != 34) {
                    bld.GROUP = 34;
                    this.markChanged();
                }
                return;
            }
            default: {
                // default value
                byte def = (byte) (group * 6);
                byte old = bld.GROUP;
                bld.GROUP = def;

                // reset to actual value
                for (byte emp = 4; emp >= 0; emp--) {
                    byte mask = (byte) Math.pow(2, emp);

                    if ((bld.EXCLUDE_MAJOR & mask) == 0)
                    {
                        bld.GROUP = (byte) (def + emp);
                        break;
                    }
                }

                // fix for primitive farms
                if (group == Edifice.GROUP_MAIN_FOOD && bld.SCI_LEVEL == 0)
                {
                    bld.GROUP = 5;
                }

                if (old != bld.GROUP)
                {
                    this.markChanged();
                }
            }
        }
    }

    public int getTrekExeID(int idType) throws Exception
    {
        CanBeEdited file;

        if ((file = UE.FILES.getFile(FileManager.P_TREK)) != null)
        {
            String ifileName = getTrekExeInternalFileName(idType);

            if (ifileName != null)
            {
                InternalFile ifile = file.getInternalFile(ifileName, true);

                if (ifile instanceof BasicScannerID)
                {
                    return ((BasicScannerID)ifile).getID();
                }
                else if (ifile instanceof MovIntToReg)
                {
                    return ((MovIntToReg)ifile).getValue();
                }
                else if (ifile instanceof CompareBuildingID)
                {
                    return ((CompareBuildingID)ifile).getID();
                }
                else
                {
                    throw new InvalidArgumentsException("Unexpected data type.");
                }
            }
            else
            {
                throw new Exception("Invalid id type.");
            }
        }
        else
        {
            throw new Exception("No trek.exe open.");
        }
    }

    public void setTrekExeID(int idType, int id) throws Exception
    {
        CanBeEdited file;

        if ((file = UE.FILES.getFile(FileManager.P_TREK)) != null)
        {
            String ifileName = getTrekExeInternalFileName(idType);

            if (ifileName != null)
            {
                InternalFile ifile = file.getInternalFile(ifileName, true);

                if (ifile instanceof BasicScannerID)
                {
                    ((BasicScannerID)ifile).setID(id);
                }
                else if (ifile instanceof MovIntToReg)
                {
                    ((MovIntToReg)ifile).setValue(id);
                }
                else if (ifile instanceof CompareBuildingID)
                {
                    ((CompareBuildingID)ifile).setID(id);
                }
                else
                {
                    throw new InvalidArgumentsException("Unexpected data type.");
                }
            }
            else
            {
                throw new Exception("Invalid id type.");
            }
        }
        else
        {
            throw new Exception("No trek.exe open.");
        }
    }

    public int getTrekExeID(int idSetType, int race) throws Exception
    {
        CanBeEdited file;

        if ((file = UE.FILES.getFile(FileManager.P_TREK)) != null)
        {
            String ifileName = getTrekExeInternalFileName(idSetType);

            if (ifileName != null)
            {
                InternalFile ifile = file.getInternalFile(ifileName, true);

                if (ifile instanceof MartialLawBld)
                {
                    return ((MartialLawBld)ifile).getID(race);
                }
                else if (ifile instanceof ShipyardBldIDs)
                {
                    return ((ShipyardBldIDs)ifile).getID(race);
                }
                else if (ifile instanceof MinorRaceSpBuildingIDs)
                {
                    return ((MinorRaceSpBuildingIDs)ifile).getID(race);
                }
                else
                {
                    throw new InvalidArgumentsException("Unexpected data type.");
                }
            }
            else
            {
                throw new Exception("Invalid id type.");
            }
        }
        else
        {
            throw new Exception("No trek.exe open.");
        }
    }

    public void setTrekExeID(int idSetType, int race, int id) throws Exception
    {
        CanBeEdited file;

        if ((file = UE.FILES.getFile(FileManager.P_TREK)) != null)
        {
            String ifileName = getTrekExeInternalFileName(idSetType);

            if (ifileName != null)
            {
                InternalFile ifile = file.getInternalFile(ifileName, true);

                if (ifile instanceof MartialLawBld)
                {
                    ((MartialLawBld)ifile).setID(race, id);
                }
                else if (ifile instanceof ShipyardBldIDs)
                {
                    ((ShipyardBldIDs)ifile).setID(race, id);
                }
                else if (ifile instanceof MinorRaceSpBuildingIDs)
                {
                    ((MinorRaceSpBuildingIDs)ifile).setID(race, id);
                }
                else
                {
                    throw new InvalidArgumentsException("Unexpected data type.");
                }
            }
            else
            {
                throw new Exception("Invalid id type.");
            }
        }
        else
        {
            throw new Exception("No trek.exe open.");
        }
    }

    public void updateExe() throws Exception
    {
        try
        {
            CanBeEdited file;

            if ((file = UE.FILES.getFile(FileManager.P_TREK)) != null)
            {
                Race race = (Race) this.STBOF.getInternalFile(Stbof.fileRACE_RST, false);

                MinorRaceMainBldIDLimits minors = (MinorRaceMainBldIDLimits) file.getInternalFile(Trek.fileMINOR_MAIN_BLD_IDS, true);

                MainResearchBld main_res = (MainResearchBld) file.getInternalFile(Trek.fileMAIN_RESEARCH_BLD, true);
                MainResearchBld2 main_res2 = (MainResearchBld2) file.getInternalFile(Trek.fileMAIN_RESEARCH_BLD2, true);
                MainIntelBld main_int = (MainIntelBld) file.getInternalFile(Trek.fileMAIN_INTEL_BLD, true);
                MainIntelBld2 main_int2 = (MainIntelBld2) file.getInternalFile(Trek.fileMAIN_INTEL_BLD2, true);
                MainIndustryFarmsAndEnergyBld main_ife = (MainIndustryFarmsAndEnergyBld) file.getInternalFile(Trek.fileMAIN_INDUSTRY_FARMS_AND_ENERGY_BLD, true);
                PunyFarms farms = (PunyFarms) file.getInternalFile(Trek.filePUNY_FARMS, true);
                PunyFactories factories = (PunyFactories) file.getInternalFile(Trek.filePUNY_FACTORIES, true);
                MainBldStartingIDs main_en = (MainBldStartingIDs) file.getInternalFile(Trek.fileMAIN_ENERGY_BLD, true);
                MainBldStartingIDs main_int3 = (MainBldStartingIDs) file.getInternalFile(Trek.fileMAIN_INTEL_BLD3, true);
                MainBldStartingIDs main_res3 = (MainBldStartingIDs) file.getInternalFile(Trek.fileMAIN_RESEARCH_BLD3, true);
                MovIntToReg race1FarmID = (MovIntToReg) file.getInternalFile(Trek.fileFED_FARM_LVL1_BLD1, true);
                
                MartialLawBld martial = (MartialLawBld) file.getInternalFile(Trek.fileMARTIAL_LAW_BLD, true);
                MartialLawBld2 martial2 = (MartialLawBld2) file.getInternalFile(Trek.fileMARTIAL_LAW_BLD2, true);
                MartialLawBld3 martial3 = (MartialLawBld3) file.getInternalFile(Trek.fileMARTIAL_LAW_BLD3, true);
                MartialLawBld4 martial4 = (MartialLawBld4) file.getInternalFile(Trek.fileMARTIAL_LAW_BLD4, true);

                MovIntToReg groundDefense1 = (MovIntToReg) file.getInternalFile(Trek.fileBASIC_GROUND_DEFENSE_ID1, true);
                MovIntToReg groundDefense2 = (MovIntToReg) file.getInternalFile(Trek.fileBASIC_GROUND_DEFENSE_ID2, true);
                MovIntToReg groundDefense3 = (MovIntToReg) file.getInternalFile(Trek.fileBASIC_GROUND_DEFENSE_ID3, true);

                BasicScannerID scanner = (BasicScannerID) file.getInternalFile(Trek.fileBASIC_SCANNER_ID, true);

                MovIntToReg orbital = (MovIntToReg) file.getInternalFile(Trek.fileORBITAL_DEFENSE_BLD, true);
                MovIntToReg orbital2 = (MovIntToReg) file.getInternalFile(Trek.fileORBITAL_DEFENSE_BLD2, true);
                MovIntToReg orbital3 = (MovIntToReg) file.getInternalFile(Trek.fileORBITAL_DEFENSE_BLD3, true);
                MovIntToReg orbital4 = (MovIntToReg) file.getInternalFile(Trek.fileORBITAL_DEFENSE_BLD4, true);
                MovIntToReg orbital5 = (MovIntToReg) file.getInternalFile(Trek.fileORBITAL_DEFENSE_BLD5, true);
                CompareByteToMemOffset orbital6 = (CompareByteToMemOffset) file.getInternalFile(Trek.fileORBITAL_DEFENSE_BLD6, true);
                MovIntToReg orbital7 = (MovIntToReg) file.getInternalFile(Trek.fileORBITAL_DEFENSE_BLD7, true);
                MovIntToReg orbital8 = (MovIntToReg) file.getInternalFile(Trek.fileORBITAL_DEFENSE_BLD8, true);
                MovIntToReg orbital9 = (MovIntToReg) file.getInternalFile(Trek.fileORBITAL_DEFENSE_BLD9, true);
                MovIntToReg orbital10 = (MovIntToReg) file.getInternalFile(Trek.fileORBITAL_DEFENSE_BLD10, true);
                MovIntToReg orbital11 = (MovIntToReg) file.getInternalFile(Trek.fileORBITAL_DEFENSE_BLD11, true);
                MovIntToReg orbital12 = (MovIntToReg) file.getInternalFile(Trek.fileORBITAL_DEFENSE_BLD12, true);
                MovIntToReg orbital13 = (MovIntToReg) file.getInternalFile(Trek.fileORBITAL_DEFENSE_BLD13, true);

                MovIntToReg shield = (MovIntToReg) file.getInternalFile(Trek.fileSHIELD_GEN_BLD, true);
                MovIntToReg shield2 = (MovIntToReg) file.getInternalFile(Trek.fileSHIELD_GEN_BLD2, true);
                MovIntToReg shield3 = (MovIntToReg) file.getInternalFile(Trek.fileSHIELD_GEN_BLD3, true);

                SpecialShips sp = (SpecialShips) file.getInternalFile(Trek.fileSPECIAL_SHIPS, false);
                MovIntToReg spbld = (MovIntToReg) file.getInternalFile(Trek.fileSPECIAL_SHIPS_BLD, true);

                MovIntToReg trade = (MovIntToReg) file.getInternalFile(Trek.fileTRADE_GOODS_BLD, true);
                MovIntToReg trade2 = (MovIntToReg) file.getInternalFile(Trek.fileTRADE_GOODS_BLD2, true);
                MovIntToReg trade3 = (MovIntToReg) file.getInternalFile(Trek.fileTRADE_GOODS_BLD3, true);

                CompareBuildingID basic_shipyard = (CompareBuildingID) file.getInternalFile(Trek.fileCMP_BASIC_SHIPYARD_ID, true);
                CompareBuildingID basic_shipyard2 = (CompareBuildingID) file.getInternalFile(Trek.fileCMP_BASIC_SHIPYARD_ID2, true);
                CompareBuildingID basic_shipyard3 = (CompareBuildingID) file.getInternalFile(Trek.fileCMP_BASIC_SHIPYARD_ID3, true);
                MovIntToReg basic_shipyard4 = (MovIntToReg) file.getInternalFile(Trek.fileBASIC_SHIPYARD_ID4, true);
                MovIntToReg basic_shipyard5 = (MovIntToReg) file.getInternalFile(Trek.fileBASIC_SHIPYARD_ID5, true);
                MovIntToReg basic_shipyard6 = (MovIntToReg) file.getInternalFile(Trek.fileBASIC_SHIPYARD_ID6, true);

                ShipyardBldIDs shipyards = (ShipyardBldIDs) file.getInternalFile(Trek.fileSHIPYARD_IDS, true);

                MinorRaceSpBuildingIDs minor_sps = (MinorRaceSpBuildingIDs) file.getInternalFile(Trek.fileMINOR_SP_BUILDING_IDS, true);
                MinorRaceBldUpgradeIDs main_upgrades = (MinorRaceBldUpgradeIDs) file.getInternalFile(Trek.fileMINORS_BLD_UPGRADE_IDS, true);

                MovIntToReg dilithiumBld1 = (MovIntToReg) file.getInternalFile(Trek.fileDILITHIUM_BLD1, true);
                MovIntToReg dilithiumBld2 = (MovIntToReg) file.getInternalFile(Trek.fileDILITHIUM_BLD2, true);
                MovIntToReg dilithiumBld3 = (MovIntToReg) file.getInternalFile(Trek.fileDILITHIUM_BLD3, true);
                MovIntToReg dilithiumBld4 = (MovIntToReg) file.getInternalFile(Trek.fileDILITHIUM_BLD4, true);
                MovIntToReg dilithiumBld5 = (MovIntToReg) file.getInternalFile(Trek.fileDILITHIUM_BLD5, true);
                MovIntToReg dilithiumBld6 = (MovIntToReg) file.getInternalFile(Trek.fileDILITHIUM_BLD6, true);
                MovIntToReg dilithiumBld7 = (MovIntToReg) file.getInternalFile(Trek.fileDILITHIUM_BLD7, true);
                MovIntToReg dilithiumBld8 = (MovIntToReg) file.getInternalFile(Trek.fileDILITHIUM_BLD8, true);

                CompareShortReg maxID1 = (CompareShortReg) file.getInternalFile(Trek.fileMAX_BLD_ID1, true);
                CompareShortReg maxID2 = (CompareShortReg) file.getInternalFile(Trek.fileMAX_BLD_ID2, true);
                CompareShortReg maxID3 = (CompareShortReg) file.getInternalFile(Trek.fileMAX_BLD_ID3, true);
                CompareShortReg maxID4 = (CompareShortReg) file.getInternalFile(Trek.fileMAX_BLD_ID4, true);
                CompareShortReg maxID5 = (CompareShortReg) file.getInternalFile(Trek.fileMAX_BLD_ID5, true);
                CompareShortReg maxID6 = (CompareShortReg) file.getInternalFile(Trek.fileMAX_BLD_ID6, true);
                CompareIntReg maxID7 = (CompareIntReg) file.getInternalFile(Trek.fileMAX_BLD_ID7, true);
                CompareIntReg maxID8 = (CompareIntReg) file.getInternalFile(Trek.fileMAX_BLD_ID8, true);
                PushInt maxID9 = (PushInt) file.getInternalFile(Trek.fileMAX_BLD_ID9, true);
                CompareShortReg numBld1 = (CompareShortReg) file.getInternalFile(Trek.fileBLD_NUM1, true);
                CompareShortReg numBld2 = (CompareShortReg) file.getInternalFile(Trek.fileBLD_NUM2, true);
                CompareShortReg numBld3 = (CompareShortReg) file.getInternalFile(Trek.fileBLD_NUM3, true);
                CompareShortReg numBld4 = (CompareShortReg) file.getInternalFile(Trek.fileBLD_NUM4, true);


                // utopia planitia
                spbld.setValue(sp.getSPBuilding());

                int id = this.getNumberOfEntries() - 1;
                short num = (short) id;

                main_res.setMaxID(num);
                main_res2.setMaxID(num);
                main_int.setMaxID(num);
                main_int2.setMaxID(num);
                main_ife.setMaxID(num);
                maxID1.setValue(num);
                maxID2.setValue(num);
                maxID3.setValue(num);
                maxID4.setValue(num);
                maxID5.setValue(num);
                maxID6.setValue(num);
                maxID7.setValue(num);
                maxID8.setValue(num);
                maxID9.setValue(num);

                num++;
                numBld1.setValue(num);
                numBld2.setValue(num);
                numBld3.setValue(num);
                numBld4.setValue(num);

                // basic scanner
                id = getTrekExeID(Edifice.BASIC_SCANNER_ID);

                if (id < 0)
                {
                    throw new Exception("Missing basic scanner available to all races.");
                }
                else
                {
                    scanner.setID(id);
                }

                // ground defense building
                id = getTrekExeID(Edifice.BASIC_GROUND_DEFENSE_ID);

                if (id < 0)
                {
                    throw new Exception("Missing basic ground defense building available to all races.");
                }
                else
                {
                    groundDefense1.setValue(id);
                    groundDefense2.setValue(id);
                    groundDefense3.setValue(id);
                }

                // orbital batery
                id = getTrekExeID(Edifice.ORBITAL_BATTERY_ID);

                if (id < 0 || id > Byte.MAX_VALUE)
                {
                    throw new Exception("Missing valid orbital battery available to all races.");
                }
                else
                {
                    orbital.setValue(id);
                    orbital2.setValue(id);
                    orbital3.setValue(id);
                    orbital4.setValue(id);
                    orbital5.setValue(id);
                    orbital6.setValue(id);
                    orbital7.setValue(id);
                    orbital8.setValue(id);
                    orbital9.setValue(id);
                    orbital10.setValue(id);
                    orbital11.setValue(id);
                    orbital12.setValue(id);
                    orbital13.setValue(id);
                }

                id = getTrekExeID(Edifice.BASIC_DILITHIUM_SOURCE_ID);

                if (id < 0)
                {
                    throw new Exception("Missing valid basic dilithium source available to all races.");
                }
                else
                {
                    dilithiumBld1.setValue(id);
                    dilithiumBld2.setValue(id);
                    dilithiumBld3.setValue(id);
                    dilithiumBld4.setValue(id);
                    dilithiumBld5.setValue(id);
                    dilithiumBld6.setValue(id);
                    dilithiumBld7.setValue(id);
                    dilithiumBld8.setValue(id);
                }

                id = getTrekExeID(Edifice.BASIC_SHIELD_GENERATOR_ID);

                if (id < 0)
                {
                    throw new Exception("Missing basic shield generator available to all races.");
                }
                else
                {
                    shield.setValue(id);
                    shield2.setValue(id);
                    shield3.setValue(id);
                }

                id = getTrekExeID(Edifice.TRADE_GOODS_ID);

                if (id < 0)
                {
                    throw new Exception("Missing trade goods available to all races.");
                }
                else
                {
                    trade.setValue(id);
                    trade2.setValue(id);
                    trade3.setValue(id);
                }

                // basic shipyard
                id = getTrekExeID(Edifice.BASIC_SHIPYARD_ID);

                if (id < 0)
                {
                    throw new Exception("Missing basic shipyard available to all races.");
                }
                else
                {
                    basic_shipyard.setID(id);
                    basic_shipyard2.setID(id);
                    basic_shipyard3.setID(id);
                    basic_shipyard4.setValue(id);
                    basic_shipyard5.setValue(id);
                    basic_shipyard6.setValue(id);
                    shipyards.setID(5, id);
                }

                int f = -1;
                int l = -1;
                int group1 = 0, group2 = 0;
                byte mask;
                byte exemp;

                for (int emp = 0; emp < 5; emp++)
                {
                    // martial law
                    id = this.getTrekExeID(MARTIAL_LAW_ID_SET, emp);

                    if (id < 0)
                    {
                        throw new Exception("Missing martial law building for the %1"
                                .replace("%1", race.getName(emp)));
                    }
                    else
                    {
                        martial.setID(emp, id);
                        martial2.setID(emp, id);
                        martial3.setID(emp, (short)id);
                        martial4.setID(emp, id);
                    }

                    // shipyard
                    id = this.getTrekExeID(SHIPYARD_ID_SET, emp);

                    if (id < 0)
                    {
                        id = getTrekExeID(Edifice.BASIC_SHIPYARD_ID);
                    }
                    
                    shipyards.setID(emp, id);

                    // main buildings
                    for (int g = 0; g < 6; g++) // find main buildings
                    {
                        f = 0;
                        l = 0;

                        switch (g) // searching for a continous line of buildings belonging to group 1 or 2
                        {
                            case 0: {
                                group1 = group2 = Edifice.GROUP_MAIN_RESEARCH;
                                break;
                            }
                            case 1: {
                                group1 = group2 = Edifice.GROUP_MAIN_INTEL;
                                break;
                            }
                            case 2: {
                                group1 = Edifice.GROUP_MAIN_INDUSTRY;
                                group2 = Edifice.GROUP_MAIN_FOOD;
                                break;
                            }
                            case 3: {
                                group1 = group2 = Edifice.GROUP_MAIN_ENERGY;
                                break;
                            }
                            case 4: {
                                group1 = group2 = Edifice.GROUP_MAIN_FOOD;
                                break;
                            }
                            case 5: {
                                group1 = group2 = Edifice.GROUP_MAIN_INDUSTRY;
                                break;
                            }
                        }

                        mask = (byte) Math.pow(2, emp);
                        int prevlvl = 1, level;

                        for (int i = 0; i < this.getNumberOfEntries(); i++)
                        {
                            exemp = this.getExEmpire(i);

                            // check empire and if pop requirement > 0
                            if (((exemp & mask) == 0) && (this.getPopCost(i) > 0)) {
                                if ((this.getGroup(i) == group1) || (this.getGroup(i) == group2)) {
                                    // found first building
                                    f = i;

                                    level = this.getTechLvl(i);

                                    // read next until not the same
                                    while ((i < this.getNumberOfEntries()) && ((exemp & mask) == 0) && (this.getPopCost(i) > 0) && ((this.getGroup(i) == group1) || (this.getGroup(i) == group2)))
                                    {
                                        // set upgrades
                                        level = this.getTechLvl(i);

                                        if (level > 0 && level < 10)
                                        {
                                            switch (g)
                                            {
                                                case 0: // research
                                                {
                                                    // stick prev bld for prev levels
                                                    for ( ; prevlvl < level - 1; prevlvl++)
                                                    {
                                                        main_upgrades.setScienceID(emp, prevlvl, (short)(i - 1));
                                                    }
                                                    main_upgrades.setScienceID(emp, level, (short)i);
                                                    break;
                                                }
                                                case 1: // intel
                                                {
                                                    // stick prev bld for prev levels
                                                    for ( ; prevlvl < level - 1; prevlvl++)
                                                    {
                                                        main_upgrades.setIntelID(emp, prevlvl, (short)(i - 1));
                                                    }
                                                    main_upgrades.setIntelID(emp, level, (short)i);
                                                    break;
                                                }
                                                case 3: // energy
                                                {
                                                    // stick prev bld for prev levels
                                                    for ( ; prevlvl < level - 1; prevlvl++)
                                                    {
                                                        main_upgrades.setEnergyID(emp, prevlvl, (short)(i - 1));
                                                    }
                                                    main_upgrades.setEnergyID(emp, level, (short)i);
                                                    break;
                                                }
                                                case 4: // food
                                                {
                                                    // stick prev bld for prev levels
                                                    for ( ; prevlvl < level - 1; prevlvl++)
                                                    {
                                                        main_upgrades.setFoodID(emp, prevlvl, (short)(i - 1));
                                                    }
                                                    main_upgrades.setFoodID(emp, level, (short)i);

                                                    if (level == 1 && emp == 1)
                                                    {
                                                        // set fed farm id
                                                        race1FarmID.setValue(i);
                                                    }

                                                    break;
                                                }
                                                case 5: // industry
                                                {
                                                    // stick prev bld for prev levels
                                                    for ( ; prevlvl < level - 1; prevlvl++)
                                                    {
                                                        main_upgrades.setIndustryID(emp, prevlvl, (short)(i - 1));
                                                    }
                                                    main_upgrades.setIndustryID(emp, level, (short)i);
                                                    break;
                                                }
                                                default:
                                            }
                                        }

                                        prevlvl = level;
                                        i++;
                                    }

                                    l = i - 1; // found last

                                    if (level < 9)
                                    {
                                        switch (g)
                                        {
                                            case 0: // research
                                            {
                                                // stick prev bld for prev levels
                                                for (level = level + 1; level < 10; level++)
                                                {
                                                    main_upgrades.setScienceID(emp, level, (short)l);
                                                }
                                                break;
                                            }
                                            case 1: // intel
                                            {
                                                // stick prev bld for prev levels
                                                for (level = level + 1; level < 10; level++)
                                                {
                                                    main_upgrades.setIntelID(emp, level, (short)l);
                                                }
                                                break;
                                            }
                                            case 3: // energy
                                            {
                                                // stick prev bld for prev levels
                                                for (level = level + 1; level < 10; level++)
                                                {
                                                    main_upgrades.setEnergyID(emp, level, (short)l);
                                                }
                                                break;
                                            }
                                            case 4: // food
                                            {
                                                // stick prev bld for prev levels
                                                for (level = level + 1; level < 10; level++)
                                                {
                                                    main_upgrades.setFoodID(emp, level, (short)l);
                                                }
                                                break;
                                            }
                                            case 5: // industry
                                            {
                                                // stick prev bld for prev levels
                                                for (level = level + 1; level < 10; level++)
                                                {
                                                    main_upgrades.setIndustryID(emp, level, (short)l);
                                                }
                                                break;
                                            }
                                            default:
                                        }
                                    }

                                    break; // quit search - we're done
                                }
                            }
                        }

                        // set trek.exe values
                        try {
                            switch (group1) {
                                case GROUP_MAIN_RESEARCH: {
                                    main_res.setIDSpan(emp, f, l);
                                    main_res2.setIDSpan(emp, f, l);
                                    main_res3.setStartID(emp, f);
                                    minors.setResearchIDSpan(emp, (short)f, (short)l);
                                    break;
                                }
                                case GROUP_MAIN_INTEL: {
                                    main_int.setIDSpan(emp, f, l);
                                    main_int2.setIDSpan(emp, f, l);
                                    main_int3.setStartID(emp, f);
                                    break;
                                }
                                case GROUP_MAIN_INDUSTRY: {
                                    if (group2 == GROUP_MAIN_FOOD)
                                    {
                                        // industry and food (group2 == food)
                                        main_ife.setIndustryAndFoodIDSpan(emp, f, l);
                                        factories.setPunyFactory(emp, f);
                                    }
                                    else if (group2 == GROUP_MAIN_INDUSTRY)
                                    {
                                        minors.setIndustryIDSpan(emp, (short)f, (short)l);
                                    }
                                    break;
                                }
                                case GROUP_MAIN_ENERGY: {
                                    main_ife.setEnergyIDSpan(emp, f, l);
                                    main_en.setStartID(emp, f);
                                    minors.setEnergyIDSpan(emp, (short)f, (short)l);
                                    break;
                                }
                                case GROUP_MAIN_FOOD: {
                                    minors.setFoodIDSpan(emp, (short)f, (short)l);

                                    // ignore primitive farms for major races
                                    while ((this.getTechLvl(f) == 0) && (f < l))
                                    {
                                        f++;
                                    }

                                    farms.setPunyFarm(emp, f);

                                    main_ife.setFoodIDSpan(emp, f, l);
                                    break;
                                }
                            }
                        } catch (Exception e) {
                            UE.displayError(e);
                        }
                    }
                }

                for (int r = 5; r < race.getNumberOfEntries(); r++)
                {
                    id = this.getTrekExeID(MINOR_SP_BLD_ID_SET, r);

                    if (id >= 0)
                    {
                        minor_sps.setID(r, id);
                    }
                }
            }
        }
        catch (Exception e)
        {
            throw e;
        }
    }

    /**Updates the ignore list and groups.*/
    public void updateIgnoreList() throws Exception {
        for (int i = 0; i < ENTRY.size(); i++)
        {
            //ignore list
            byte emp_mask = this.getExEmpire(i);
            int bld_mask_index = 0;
            int id = i;

            while (id > 63)
            {
                bld_mask_index++; //the part (0-5)
                id = id - 64;
            }

            long bld_bit = (long) Math.pow(2, id); //the bit used for this building is positive

            if (id == 63)
            {
                bld_bit = ~bld_bit;	//if the id is 63 the above goes out of range (long is signed)
                //set last bit to positive, others to 0
            }

            for (int empire = 0; empire < 5; empire++)
            {
                long bld_mask = IGNORE_LIST[empire][bld_mask_index];       //get current buildings mask
                byte race_mask = (byte) Math.pow(2, empire);		       //create race mask for current race

                if ((race_mask & emp_mask) == 0) {
                    //means the building is buildable by this empire
                    //don't ignore-set as 0
	                /*AND
                    /*bld_mask	~bld_bit     result
                    /*  0         0	     =      0	// means used, stays the same
                    /*  0         1	     =      0	// stays the same for rest of ids
                    //  1         0      =      0	// change to not be ignored
                    /*  1         1	     =      1	// stays the same for rest of ids
                    //in ~bld_bit all bits are set as 1 except the one now being set
                    //in bld_mask there are both bits 1 and 0
                    //all bits must remain as in the original, except the bit to be set */
                    IGNORE_LIST[empire][bld_mask_index] = bld_mask & (~bld_bit);
                } else {
                    //ignore-set as 1
        			/*OR
                    /*bld_mask	bld_bit     result
                    /*  0           0         =0	// stays as it was
                    /*  0           1         =1	// set to ignore
                    /*  1           0         =1	// stays as it was
                    //  1       	1         =1	// set to ignore   */
                    //in bld_bit all bits are set as 0 except the one now being set
                    //in bld_mask there are both bits 1 and 0
                    //all bits must remain as in the original, except the bit to be set
                    IGNORE_LIST[empire][bld_mask_index] = bld_mask | bld_bit;
                }
            }
        }
    }

    /**
     * sets the bonus for all buildings of the specified type.
     * @param boni_type the type of building to edit
     * @param boni_calc bonus - a calculation with variable L (level) - see Tools.calculate()
     * @param empire the required empire (built by)
     */
    public void multiSetBonus(byte boni_type, String[] calc, byte empire) throws InvalidArgumentsException {
        Building bld;
        double val;
        String c;

        for (int i = 0; i < ENTRY.size(); i++) {
            bld = ENTRY.get(i);

            if ((bld.BONI_TYPE == boni_type) && ((bld.EXCLUDE_MAJOR & empire) == 0)) {
                for (int j = 0; j < bld.BONUS.length; j++) {
                    c = this.prepareCalc(bld, calc[j]);

                    val = Tools.calculate(c);
                    val = Math.min(val, Short.MAX_VALUE);
                    val = Math.max(val, Short.MIN_VALUE);

                    bld.BONUS[j] = (short) val;
                    this.markChanged();
                }
            }
        }
    }

    /**
     * Sets the industry cost for all buildings of the specified type.
     * @param boni_type the type of building to edit
     * @param cost_calc industry cost - a calculation with variable L (level) - see Tools.calculate()
     * @param empire the required empire (built by)
     */
    public void multiSetIndustryCost(byte boni_type, String calc, byte empire) throws InvalidArgumentsException {
        Building bld;
        double val;
        String c;

        for (int i = 0; i < ENTRY.size(); i++) {
            bld = ENTRY.get(i);

            if ((bld.BONI_TYPE == boni_type) && ((bld.EXCLUDE_MAJOR & empire) == 0)) {
                c = this.prepareCalc(bld, calc);

                val = Tools.calculate(c);
                val = Math.min(val, Integer.MAX_VALUE);
                val = Math.max(val, Integer.MIN_VALUE);

                bld.INDUSTRY_COST = (int) val;
                this.markChanged();
            }
        }
    }

    /**
     * Sets the morale bonus for all buildings of the specified type.
     * @param boni_type the type of building to edit
     * @param calc bonus - a calculation with variable L (level) - see Tools.calculate()
     * @param empire the required empire (built by)
     */
    public void multiSetMorale(byte boni_type, String calc, byte empire) throws InvalidArgumentsException {
        Building bld;
        double val;
        String c;

        for (int i = 0; i < ENTRY.size(); i++) {
            bld = ENTRY.get(i);

            if ((bld.BONI_TYPE == boni_type) && ((bld.EXCLUDE_MAJOR & empire) == 0)) {
                c = this.prepareCalc(bld, calc);

                val = Tools.calculate(c);
                val = Math.min(val, Short.MAX_VALUE);
                val = Math.max(val, Short.MIN_VALUE);

                bld.MORALE_BONI = (short) val;
                this.markChanged();
            }
        }
    }

    /**
     * Sets the energy maintainance for all buildings of the specified type.
     * @param boni_type the type of building to edit
     * @param calc bonus - a calculation with variable L (level) - see Tools.calculate()
     * @param empire the required empire (built by)
     */
    public void multiSetEnergy(byte boni_type, String calc, byte empire) throws InvalidArgumentsException {
        Building bld;
        double val;
        String c;

        for (int i = 0; i < ENTRY.size(); i++) {
            bld = ENTRY.get(i);

            if ((bld.BONI_TYPE == boni_type) && ((bld.EXCLUDE_MAJOR & empire) == 0)) {
                c = this.prepareCalc(bld, calc);

                val = Tools.calculate(c);
                val = Math.min(val, Short.MAX_VALUE);
                val = Math.max(val, Short.MIN_VALUE);

                this.setEnCost(i, (short) val);
            }
        }
    }

    /**
     * Sets the unrest bonus for all buildings of the specified type.
     * @param boni_type the type of building to edit
     * @param calc bonus - a calculation with variable L (level) - see Tools.calculate()
     * @param empire the required empire (built by)
     */
    public void multiSetUnrestBonus(byte boni_type, String[] calc, byte empire) throws InvalidArgumentsException {
        Building bld;
        double val;
        String c;

        for (int i = 0; i < ENTRY.size(); i++) {
            bld = ENTRY.get(i);

            if ((bld.BONI_TYPE == boni_type) && ((bld.EXCLUDE_MAJOR & empire) == 0)) {
                for (int j = 0; j < bld.BONUS_2.length; j++) {
                    c = this.prepareCalc(bld, calc[j]);

                    val = Tools.calculate(c);
                    val = Math.min(val, Short.MAX_VALUE);
                    val = Math.max(val, Short.MIN_VALUE);

                    bld.BONUS_2[j] = (short) val;
                    this.markChanged();
                }
            }
        }
    }

    // replace varibles with values
    private String prepareCalc(Building bld, String calc) {
        String c = calc;

        c = c.replaceAll("L", Integer.toString(bld.SCI_LEVEL)); //$NON-NLS-1$
        c = c.replaceAll("U0", Integer.toString(bld.BONUS_2[0])); //$NON-NLS-1$
        c = c.replaceAll("U1", Integer.toString(bld.BONUS_2[1])); //$NON-NLS-1$
        c = c.replaceAll("U2", Integer.toString(bld.BONUS_2[2])); //$NON-NLS-1$
        c = c.replaceAll("U3", Integer.toString(bld.BONUS_2[3])); //$NON-NLS-1$
        c = c.replaceAll("U4", Integer.toString(bld.BONUS_2[4])); //$NON-NLS-1$
        c = c.replaceAll("U5", Integer.toString(bld.BONUS_2[5])); //$NON-NLS-1$
        c = c.replaceAll("E", Integer.toString(bld.ENERGY_COST)); //$NON-NLS-1$
        c = c.replaceAll("M", Integer.toString(bld.MORALE_BONI)); //$NON-NLS-1$
        c = c.replaceAll("C", Integer.toString(bld.INDUSTRY_COST)); //$NON-NLS-1$
        c = c.replaceAll("B0", Integer.toString(bld.BONUS[0])); //$NON-NLS-1$
        c = c.replaceAll("B1", Integer.toString(bld.BONUS[1])); //$NON-NLS-1$
        c = c.replaceAll("B2", Integer.toString(bld.BONUS[2])); //$NON-NLS-1$
        c = c.replaceAll("B3", Integer.toString(bld.BONUS[3])); //$NON-NLS-1$
        c = c.replaceAll("B4", Integer.toString(bld.BONUS[4])); //$NON-NLS-1$
        c = c.replaceAll("B5", Integer.toString(bld.BONUS[5])); //$NON-NLS-1$

        return c;
    }

    private void checkIndex(int index) throws InvalidArgumentsException {
        if ((index < 0) || (index >= ENTRY.size())) {
            String msg = Language.getString("Edifice.18"); //$NON-NLS-1$
            msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
    }

    private void checkBoniIndex(int index) throws InvalidArgumentsException {
        if ((index < 0) || (index > 5)) {
            String msg = Language.getString("Edifice.19"); //$NON-NLS-1$
            msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
    }

    /**
     * @param index index of the building to be moved up (towards 0) one spot
     * @return new building index
     * @throws InvalidArgumentsException
     */
    public int moveUp(int index) throws Exception
    {
        checkIndex(index);

        if (index > 0) {
            int new_index = index - 1;

            Building bld = ENTRY.remove(index);
            Building bld2 = ENTRY.remove(new_index);
            Building bld3;

            bld.ID = (short) (new_index);
            bld2.ID = (short) index;

            ENTRY.add(new_index, bld);
            ENTRY.add(index, bld2);

            for (int i = 0; i < ENTRY.size(); i++)
            {
                bld3 = ENTRY.get(i);

                if (bld3.UPGRADE_FOR == bld.ID) {
                    bld3.UPGRADE_FOR = bld2.ID;
                } else if (bld3.UPGRADE_FOR == bld2.ID) {
                    bld3.UPGRADE_FOR = bld.ID;
                }
            }

            // fix ids in trek.exe
            switchTrekIDs(index, new_index);

            this.markChanged();

            return new_index;
        }

        return index;
    }

    /**
     * @param index index of the building to be moved down (towards num - 1) one spot
     * @return new building index
     * @throws InvalidArgumentsException
     */
    public int moveDown(int index) throws Exception
    {
        checkIndex(index);

        if (index < ENTRY.size() - 1)
        {
            int new_index = index + 1;

            Building bld2 = ENTRY.remove(new_index);
            Building bld = ENTRY.remove(index);
            Building bld3;

            bld.ID = (short) (new_index);
            bld2.ID = (short) index;

            ENTRY.add(index, bld2);
            ENTRY.add(new_index, bld);

            for (int i = 0; i < ENTRY.size(); i++) {
                bld3 = ENTRY.get(i);

                if (bld3.UPGRADE_FOR == bld.ID) {
                    bld3.UPGRADE_FOR = bld2.ID;
                } else if (bld3.UPGRADE_FOR == bld2.ID) {
                    bld3.UPGRADE_FOR = bld.ID;
                }
            }

            // fix ids in trek.exe
            switchTrekIDs(index, new_index);

            this.markChanged();

            return new_index;
        }

        return index;
    }

    private void relocateTrekID(int oldID, int newID) throws Exception
    {
        for (int idType = BASIC_SCANNER_ID; idType <= BASIC_GROUND_DEFENSE_ID; idType++)
        {
            if (this.getTrekExeID(idType) == oldID)
            {
                this.setTrekExeID(idType, newID);
            }
        }

        int id;

        for (int idType = MARTIAL_LAW_ID_SET; idType <= SHIPYARD_ID_SET; idType++)
        {
            for (int emp = 0; emp < 5; emp++)
            {
                id = this.getTrekExeID(idType, emp);

                if (id == oldID)
                {
                    this.setTrekExeID(idType, emp, newID);
                }
            }
        }

        CanBeEdited file;

        if ((file = UE.FILES.getFile(FileManager.P_STBOF)) != null)
        {
            Race races = (Race) file.getInternalFile(Stbof.fileRACE_RST, true);

            for (int idType = MINOR_SP_BLD_ID_SET; idType <= MINOR_SP_BLD_ID_SET; idType++)
            {
                for (int race = 5; race < races.getNumberOfEntries(); race++)
                {
                    id = this.getTrekExeID(idType, race);

                    if (id == oldID)
                    {
                        this.setTrekExeID(idType, race, newID);
                    }
                }
            }
        }
    }

    private void switchTrekIDs(int oldID, int newID) throws Exception
    {
        int id;

        for (int idType = BASIC_SCANNER_ID; idType <= BASIC_GROUND_DEFENSE_ID; idType++)
        {
            id = this.getTrekExeID(idType);

            if (id == oldID)
            {
                this.setTrekExeID(idType, newID);
            }
            else if (id == newID)
            {
                this.setTrekExeID(idType, oldID);
            }
        }

        for (int idType = MARTIAL_LAW_ID_SET; idType <= SHIPYARD_ID_SET; idType++)
        {
            for (int emp = 0; emp < 5; emp++)
            {
                id = this.getTrekExeID(idType, emp);

                if (id == oldID)
                {
                    this.setTrekExeID(idType, emp, newID);
                }
                else if (id == newID)
                {
                    this.setTrekExeID(idType, emp, oldID);
                }
            }
        }

        CanBeEdited file;

        if ((file = UE.FILES.getFile(FileManager.P_STBOF)) != null)
        {
            Race races = (Race) file.getInternalFile(Stbof.fileRACE_RST, true);

            for (int idType = MINOR_SP_BLD_ID_SET; idType <= MINOR_SP_BLD_ID_SET; idType++)
            {
                for (int race = 5; race < races.getNumberOfEntries(); race++)
                {
                    id = this.getTrekExeID(idType, race);

                    if (id == oldID)
                    {
                        this.setTrekExeID(idType, race, newID);
                    }
                    else if (id == newID)
                    {
                        this.setTrekExeID(idType, race, oldID);
                    }
                }
            }
        }
    }

    private String getTrekExeInternalFileName(int idType)
    {
        switch (idType)
        {
            case BASIC_SCANNER_ID:
                return Trek.fileBASIC_SCANNER_ID;
            case ORBITAL_BATTERY_ID:
                return Trek.fileORBITAL_DEFENSE_BLD;
            case BASIC_DILITHIUM_SOURCE_ID:
                return Trek.fileDILITHIUM_BLD1;
            case BASIC_SHIELD_GENERATOR_ID:
                return Trek.fileSHIELD_GEN_BLD;
            case TRADE_GOODS_ID:
                return Trek.fileTRADE_GOODS_BLD;
            case BASIC_SHIPYARD_ID:
                return Trek.fileCMP_BASIC_SHIPYARD_ID;
            case BASIC_GROUND_DEFENSE_ID:
                return Trek.fileBASIC_GROUND_DEFENSE_ID1;

            /// id sets
            case MARTIAL_LAW_ID_SET:
                return Trek.fileMARTIAL_LAW_BLD;
            case SHIPYARD_ID_SET:
                return Trek.fileSHIPYARD_IDS;
            case MINOR_SP_BLD_ID_SET:
                return Trek.fileMINOR_SP_BUILDING_IDS;
            default:
                return null;
        }
    }

    public String getTrekExeIDDescription(int idType)
    {
        switch (idType)
        {
            case BASIC_SCANNER_ID:
                return "Basic scanner";
            case ORBITAL_BATTERY_ID:
                return "Orbital battery";
            case BASIC_DILITHIUM_SOURCE_ID:
                return "Basic dilithium source";
            case BASIC_SHIELD_GENERATOR_ID:
                return "Basic shield generator";
            case TRADE_GOODS_ID:
                return "Trade goods";
            case BASIC_SHIPYARD_ID:
                return "Basic shipyard";
            case BASIC_GROUND_DEFENSE_ID:
                return "Basic ground defense";

            /// id sets
            case MARTIAL_LAW_ID_SET:
                return "%1 martial law";
            case SHIPYARD_ID_SET:
                return "%1 shipyard";
            case MINOR_SP_BLD_ID_SET:
                return "%1 special building";
            default:
                return null;
        }
    }
}
