/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: Lexicon.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.edit.res.stbof.dic;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;

/**This class is a representation of the lexicon.dic file.
It contains strings used by botf's interface.*/
public class Lexicon extends InternalFile{
	private int MAPADDRESS;
	private int unk;
	private short ENTRIES;
	private ArrayList<LecEntry> ENTRY = new ArrayList<LecEntry>();
	
	/**@param in the input stream to read value from
	*/
	public Lexicon(InputStream in) throws IOException
        {
            int skip = 0;
	    //wrap
            BufferedInputStream bis = new BufferedInputStream(in);
            bis.mark(bis.available());
            //address
            byte[] b = new byte[4];
            bis.read(b);
            MAPADDRESS = Tools.toInt(b, true);
            //unknown
            bis.reset();

            while (skip < MAPADDRESS)
            {
                skip += bis.skip(MAPADDRESS - skip);
            }
            
            bis.read(b);
            unk = Tools.toInt(b, true);
            
            //entries
            b = new byte[2];
            bis.read(b);
            ENTRIES = Tools.toShort(b, true);
            
            //loop
            int loc;
            int len;
            String str;
            
            for (int i = 0; i < ENTRIES; i++)
            {
                bis.reset();
                bis.mark(bis.available());
                
                b = new byte[4];
                
                skip = MAPADDRESS + 6 + (i * 8);
                while (skip > 0)
                {
                    skip -= bis.skip(skip);
                }
                
                //address
                bis.read(b);
                loc = Tools.toInt(b, true);
                //length
                bis.read(b);
                len = Tools.toInt(b, true);
                //string
                b = new byte[len];
                
                bis.reset();
                bis.mark(bis.available());

                skip = loc;
                while (skip > 0)
                {
                    skip -= bis.skip(skip);
                }

                bis.read(b);
                str = Tools.toASCII(b, 0, len);
                //add
                ENTRY.add(new LecEntry(str, len));
            }
	}

	/**@param out the output stream to write to*/
	@Override
	public void save(OutputStream out) throws Exception{
		ENTRIES = (short)ENTRY.size();
		
		LecEntry le;
		MAPADDRESS = 4;
		for (int i=0; i < ENTRIES; i++){
			le = ENTRY.get(i);
			MAPADDRESS += le.getSize();
		}
		out.write(Tools.toByte(MAPADDRESS, true));
		
		String str;
		for (int i=0; i < ENTRIES; i++){
			le = ENTRY.get(i);
			str = le.getString();
			out.write(Tools.toByte(str, str.length()+1, str.length()));
		}
		//write map
		out.write(Tools.toByte(unk, true));
		out.write(Tools.toByte(ENTRIES, true));
		
		int ad = 4;
		int se;
		for (int i=0; i < ENTRIES; i++){
			out.write(Tools.toByte(ad, true));
			le = ENTRY.get(i);
			se = le.getSize();
			out.write(Tools.toByte(se, true));
			ad += se;
		}
	}

	/**@return an array of short strings the lexicon file contains, if a string is larger
	than 20 characters it will not be complete in this array.*/
	public String[] getEntries(){
		int n = ENTRY.size();
		String[] str = new String[n];
		LecEntry le;
		for (int i=0; i < n; i++){
			le = ENTRY.get(i);
			str[i] = new String(le.getString());
			if (str[i].length() > 20){
				str[i] = str[i].substring(0, 17) + "..."; //$NON-NLS-1$
			}
		}
		return str;
	}
	/**@return an array of strings the lexicon file contains.*/
	public String[] getFullEntries(){
		int n = ENTRY.size();
		String[] str = new String[n];
		LecEntry le;
		for (int i=0; i < n; i++){
			le = ENTRY.get(i);
			str[i] = new String(le.getString());
		}
		return str;
	}
	/**@param index the index of the string to return
	@return full string requested
	@throws InvalidArgumentsException if the index is invalid*/
	public String getEntry(int index) throws InvalidArgumentsException{
		try{
			LecEntry le = ENTRY.get(index);
			return new String(le.getString());
		}catch (Exception e){
			String msg = Language.getString("Lexicon.0"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
	}
	/**Changes entry.
	@param index the index of the entry to change
	@param ent the string to replace the previous one
	@return true if value has been changed
	@throws InvalidArgumentsException if the index is invalid*/
	public boolean setEntry(int index, String ent) throws InvalidArgumentsException{
		try{
			LecEntry le = ENTRY.get(index);
			if (ent.equals(le.getString())) {
				return false;
			}
			le.setString(ent);
                        this.markChanged();
			return true;
		}catch (Exception e){
			String msg = Language.getString("Lexicon.0"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
	}

	/**@return the uncompressed size of the file**/
	@Override
	public long getSize() {
		int n = ENTRY.size();
		int size = 10 + n * 8;
		LecEntry le;
		for (int i=0; i < n; i++){
			le = ENTRY.get(i);
			size += le.getSize();
		}
		return size;
	}

	@Override
	public Vector<String> check(){
		Vector<String> response = new Vector<String>();
		if (ENTRY.size() != 1460){
			String msg = Language.getString("Lexicon.1"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(ENTRY.size())); //$NON-NLS-1$
			response.add(msg);
		}
		return response;
	}
	
	private class LecEntry{
		private String entry;
		private int address;
		public LecEntry(String e, int a){ entry = new String(e); address = a;}
		public String getString() { return new String(entry);}
		public int getAddress() { return address; }
		public void setString(String g) { entry = new String(g);}
		public void setAddress(int ad){ address = ad; }
		public int getSize() { int h = entry.length() + 1; return h; }
	}
}
