/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: HobFile.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.edit.res.stbof.hob;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;

/**This class reprisents the file format *.hob. The hob files contain the models of ships and other objects.
If anyone has ANY useful info on this file, please contact me.*/
public class HobFile extends InternalFile{
	private int	    FILE_SIZE;	        //   0 -   3 (4 bytes)
	private int 	TEXTURE_ID_ADDRESS;	//   4 -   7 (4 bytes)
	private byte[]	UNKNOWN_1;		    //   8 -  23 (16 bytes)
	private int	    TEXTURE_NUM;	    //  24 -  27 (4 bytes)
	private byte[]	UNKNOWN_2;		    //  28 -  79 (52 bytes)
	private float[]	MIN_XYZ;            //  80 -  91 (12 bytes)
	private float[]	MAX_XYZ;            //  92 - 103 (12 bytes)
	private float	MODEL_SIZE;         // 104 - 107 (4 bytes)
	private	byte[]	UNKNOWN_3;		    // 108 - ??? (texture_id_address - 108 bytes)
	private	int[]	TEXTURE_ID;		    // ??? - ??? (texture_num*4 bytes)
	private	byte[]	UNKNOWN_4;		    // ??? - EOF

	public HobFile(HobFile hob){
		FILE_SIZE = hob.FILE_SIZE;
		TEXTURE_ID_ADDRESS = hob.TEXTURE_ID_ADDRESS;
		UNKNOWN_1 = new byte[16];
		System.arraycopy(hob.UNKNOWN_1, 0, UNKNOWN_1, 0, 16);
		TEXTURE_NUM = hob.TEXTURE_NUM;
		UNKNOWN_2 = new byte[52];
		System.arraycopy(hob.UNKNOWN_2, 0, UNKNOWN_2, 0, 52);
		MIN_XYZ = new float[3];
		System.arraycopy(hob.MIN_XYZ, 0, MIN_XYZ, 0, 3);
		MAX_XYZ = new float[3];
		System.arraycopy(hob.MAX_XYZ, 0, MAX_XYZ, 0, 3);
		MODEL_SIZE = hob.MODEL_SIZE;
		UNKNOWN_3 = new byte[hob.UNKNOWN_3.length];
		System.arraycopy(hob.UNKNOWN_3, 0, UNKNOWN_3, 0, UNKNOWN_3.length);
		TEXTURE_ID = new int[hob.TEXTURE_ID.length];
		System.arraycopy(hob.TEXTURE_ID, 0, TEXTURE_ID, 0, TEXTURE_ID.length);
		UNKNOWN_4 = new byte[hob.UNKNOWN_4.length];
		System.arraycopy(hob.UNKNOWN_4, 0, UNKNOWN_4, 0, UNKNOWN_4.length);
	}
	
	/**@param in the InputStream to read from*/
	public HobFile(InputStream in) throws IOException{		
		byte[] b = new byte[4];
		//get file size
		in.read(b);
		FILE_SIZE = Tools.toInt(b, true);
		//get position where texture id's are found
		in.read(b);
		TEXTURE_ID_ADDRESS = Tools.toInt(b, true);
		if (TEXTURE_ID_ADDRESS < 8){
			String msg = Language.getString("HobFile.5"); //$NON-NLS-1$
			msg = msg.replace("%1", getName()); //$NON-NLS-1$
			throw new IOException(msg);
		}
		//unknown 1
		UNKNOWN_1 = new byte[16];
		in.read(UNKNOWN_1);
		//number of texture id's
		in.read(b);
		TEXTURE_NUM = Tools.toInt(b, true);
		if (TEXTURE_NUM < 0){
			String msg = Language.getString("HobFile.0"); //$NON-NLS-1$
			msg.replace("%1", Integer.toString(TEXTURE_NUM)); //$NON-NLS-1$
			throw new IOException(msg);
		}
		//unknown 2
		UNKNOWN_2 = new byte[52];
		in.read(UNKNOWN_2);
        // minimum x,y,z (placement)
        MIN_XYZ = new float[3];
        for (int i = 0; i < 3; i++){
            in.read(b);
            MIN_XYZ[i] = Tools.toFloat(b, true);
        }
        // maximum x,y,z (placement)
        MAX_XYZ = new float[3];
        for (int i = 0; i < 3; i++){
            in.read(b);
            MAX_XYZ[i] = Tools.toFloat(b, true);
        }
		//model size (radius)
		in.read(b);
		MODEL_SIZE = Tools.toFloat(b, true);
		//unknown 3
		UNKNOWN_3 = new byte[TEXTURE_ID_ADDRESS - 108];
		in.read(UNKNOWN_3);
		//texture id's
		TEXTURE_ID = new int[TEXTURE_NUM];
		for (int i=0; i<TEXTURE_NUM; i++){
			in.read(b);
			TEXTURE_ID[i] = Tools.toInt(b, true);
		}
		//unknown 4
		UNKNOWN_4 = new byte[in.available()];
		in.read(UNKNOWN_4);
	}

	/**Used to save changes.
	@param out the OutputStream where the data should be written to
	@return true if data has been written*/
	@Override
	public void save(OutputStream out) throws Exception{ 
		//file size
		out.write(Tools.toByte(FILE_SIZE, true));
		//position where texture id's are found
		out.write(Tools.toByte(TEXTURE_ID_ADDRESS, true));
		//unknown 1
		out.write(UNKNOWN_1);
		//number of texture id's
		out.write(Tools.toByte(TEXTURE_NUM, true));
		//unknown 2
		out.write(UNKNOWN_2);
		//minimal x,y,z
        for (int i = 0; i < 3; i++){
		    out.write(Tools.toByte(MIN_XYZ[i], true));
        }
		//maximal x,y,z
        for (int i = 0; i < 3; i++){
		    out.write(Tools.toByte(MAX_XYZ[i], true));
        }
		//model size
		out.write(Tools.toByte(MODEL_SIZE, true));
		//unknown 3
		out.write(UNKNOWN_3);
		//texture id's
		for (int i = 0; i < TEXTURE_NUM; i++){
			out.write(Tools.toByte(TEXTURE_ID[i], true));
		}
		//unknown 4
		out.write(UNKNOWN_4);
	}
	
	/**Used to get the uncompressed size of the file.**/
	@Override
	public long getSize(){
		long size = 108 + UNKNOWN_3.length + 4*TEXTURE_NUM + UNKNOWN_4.length;
		return size;
	}
	/**@return the number of textures used by this model.*/
	public int numberOfTextures(){
		return TEXTURE_NUM;
	}

	/**
        @return the model size (radius).
	*/
	public float getModelSize(){
		return MODEL_SIZE;
	}

        /**Sets the model size.*/
        public void setModelSize(float size) throws InvalidArgumentsException
        {
            if (size < 0)
            {
                String msg = Language.getString("HobFile.1"); //$NON-NLS-1$
                msg = msg.replace("%1", Float.toString(size)); //$NON-NLS-1$
                throw new InvalidArgumentsException(msg);
            }
            
            if (Math.abs(Float.compare(MODEL_SIZE, size)) < 0.01f) return;
            
            MODEL_SIZE = size;
            this.markChanged();
        }

	/**
        @return the placement (min_x, max_x, min_y, max_y, min_z, max_z <- in this order).
	*/
	public float[] getModelPlacement(){
        float[] ret = new float[6];        
        for (int i = 0; i < 3; i++){
            ret[2 * i] = MIN_XYZ[i];
            ret[2 * i + 1] = MAX_XYZ[i];
        }
        return ret;
	}

        /**Sets the model placement.  (min_x, max_x, min_y, max_y, min_z, max_z <- in this order)*/
        public void setModelPlacement(float[] place) throws InvalidArgumentsException
        {
            if (place.length < 6)
            {
                String msg = Language.getString("HobFile.2"); //$NON-NLS-1$
                throw new InvalidArgumentsException(msg);
            }
            for (int i = 0; i < 3; i++)
            {
                if (Math.abs(Float.compare(MIN_XYZ[i], place[2*i])) > 0.01f)
                {
                    MIN_XYZ[i] = place[2*i];
                    this.markChanged();
                }
                if (Math.abs(Float.compare(MAX_XYZ[i], place[2*i+1])) > 0.01f)
                {
                    MAX_XYZ[i] = place[2*i+1];
                    this.markChanged();
                }
            }
        }

	/**@return the textures id.*/
	public int getTextureID(int index) throws InvalidArgumentsException{
		if (index<0 || index >= TEXTURE_NUM){
			String msg = Language.getString("HobFile.3"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
		return TEXTURE_ID[index];
	}

        /**Set the textures id.*/
        public void setTextureID(int index, int newID) throws InvalidArgumentsException
        {
            if (index<0 || index >= TEXTURE_NUM)
            {
                String msg = Language.getString("HobFile.3"); //$NON-NLS-1$
                msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
                throw new InvalidArgumentsException(msg);
            }
            if (TEXTURE_ID[index] == newID) return;
            TEXTURE_ID[index] = newID;
            this.markChanged();
        }

	@Override
	public Vector<String> check(){ 
		Vector<String> response = new Vector<String>();
        if (MODEL_SIZE <= 0){
        	String msg = Language.getString("HobFile.4"); //$NON-NLS-1$
        	msg = msg.replace("%1", Float.toString(MODEL_SIZE)); //$NON-NLS-1$
        	response.add(msg);
        }
		return response;
	}
}
