/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: Texture.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.edit.res.stbof.lst;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.edit.res.Stbof;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;

/**This class represents texture.lst.*/
public class Texture extends InternalFile{
	private ArrayList<String> ENTRY = new ArrayList<String>();
	private Stbof STBOF;	
	private int END = -1;
	
	public Texture()
        {
		this.STBOF = null;
	}

	/**@param in the InputStream to read from*/
	public Texture(InputStream in, Stbof st) throws IOException
        {
		this.STBOF = st;
		byte[] b = new byte[4];
		//num of entries
		in.read(b);
		//entries
		String entry = null;
		int index = 0;
		byte[] be = new byte[32];
		while (in.available() > 0){
			//index
			in.read(b);
			index = Tools.toInt(b, true);
			if (index != -1){
				//entry
				in.read(be);
				entry = Tools.toASCII(be, 0, 32);
				entry = entry.toLowerCase();
				// add - this can be really really bad
				// fixing this won't be easy because other classes depend on this behaviour
				// does botf read this file in a similiar fashion?
				this.ENTRY.add(index, entry);
			}
		}
	}
	/**
         * Adds the specified string to the end of the list.
         * @param n the string to be added.
         * @return index of the added texture
         */
	public int add(String n)
        {
		if ((n.length() > 31) || (n.length() < 1))
                {
			return -1;
		}
		this.ENTRY.add(n);
                this.markChanged();
                return this.ENTRY.size() - 1;
	}

	public String get(int index) throws InvalidArgumentsException
        {
		if ((this.ENTRY.size() <= index) || (index < 0))
        {
            throw new InvalidArgumentsException(Language.getString("Texture.2").replace("%1", Integer.toString(index))); //$NON-NLS-1$ //$NON-NLS-2$
        }
		return this.ENTRY.get(index);
	}
	
	/**Returns true if the texture is in the list.
	@param n the string to be removed.*/
	public boolean contains(String n)
        {
		return this.ENTRY.contains(n);
	}
        
    public boolean anyTextureFileBeginsWith(String prefix)
    {
        prefix = prefix.toLowerCase();

        for (int i = 0; i < ENTRY.size(); i++)
        {
            if (ENTRY.get(i).toLowerCase().startsWith(prefix))
            {
                return true;
            }
        }
        return false;
    }

	/**Sets the value at the specified index.
	@param index the index of the entry to change.
	@param n the string to replace the old one.
	*/
	public void set(int index, String n)
        {
		if ((n.length() > 31) || (n.length() < 1)) {
			return;
		}
		this.ENTRY.set(index, n);
                this.markChanged();
	}

        public boolean isTextureValid(int index)
        {
            if (index < 0 || index >= ENTRY.size())
            {
                return false;
            }

            return ENTRY.get(index).toLowerCase().endsWith(".gif");
        }

	/**Returns a list of all the entries in the file.*/
	public String[] getEntries()
        {
            ArrayList<String> valid = new ArrayList<String>();

            String[] x = new String[1];
            
            for (int i = 0; i < ENTRY.size(); i++)
            {
                x[0] = ENTRY.get(i);

                if (isTextureValid(i))
                {
                    valid.add(x[0]);
                }
            }

            x = valid.toArray(new String[0]);

            return x;
	}

	/**used to save changes.*/
	@Override
	public void save(OutputStream out) throws IOException
        {
		//num of entries
		int num = this.ENTRY.size();

		out.write(Tools.toByte(num, true));
		//entries
		String entry;
		for (int i = 0; i < num; i++)
                {
			//index
			out.write(Tools.toByte(i, true));
			//entry
			entry = this.ENTRY.get(i);
			out.write(Tools.toByte(entry, 32, 31));
		}
                
		out.write(Tools.toByte(this.END, true));
	}
	
	/**Used to get the uncompressed size of the file.**/
	@Override
	public long getSize()
        {
		long siz = 8 + this.ENTRY.size() * 36;
		return siz;
	}

        public int getMaxIndex()
        {
            return ENTRY.size() - 1;
	}

	public int getTextureNum()
    {
        return this.getEntries().length;
	}
	
        public int getTextureIndex(String texture)
        {
            for (int i = 0; i < this.ENTRY.size(); i++)
            {
                if (texture.toLowerCase().equals(this.ENTRY.get(i).toLowerCase()))
                {
                    return i;
                }
            }
            
            return -1;
        }
        
        /**
         * @param index index of the texture to remove
         */
        public void remove(int index)
        {
            this.ENTRY.set(index, "X"); //$NON-NLS-1$
            this.markChanged();
        }
        
        public int getUnusedIndex()
        {
            for (int i = 0; i < ENTRY.size(); i++)
            {
                if (!ENTRY.get(i).toLowerCase().endsWith(".gif"))
                {
                    return i;
                }
            }

            return -1;
        }
        
	@Override
	public Vector<String> check()
        {
		Vector<String> response = new Vector<String>();

		if (this.STBOF == null)
                {
                    return response;
                }
		
		try{
			//get paletteList
			PaletteList PALETTE = (PaletteList)this.STBOF.getInternalFile(Stbof.filePALETTE_LST, true);
			int num = PALETTE.getNumberOfEntries(); //let's see how many textures have their palettes registered
			if (ENTRY.size() > num)
                        {
				//not all textures are registered!
				String msg = Language.getString("Texture.0"); //$NON-NLS-1$
				msg = msg.replace("%1", getName()); //$NON-NLS-1$
				msg = msg.replace("%2", Integer.toString(this.ENTRY.size() - num)); //$NON-NLS-1$
				response.add(msg);
			}
		}
                catch (Exception e)
                {
			String msg = Language.getString("Texture.1"); //$NON-NLS-1$
			msg = msg.replace("%1", getName()); //$NON-NLS-1$
			msg = msg.replace("%2", e.getMessage()); //$NON-NLS-1$
			response.add(msg);
		}

		return response;
	}
    
}
