/*
Ultimate Editor for Birth Of The Federation 1.02
Copyright (C) 2005-2007  Alan Podlesek

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


File: Planet.java

Written by: Alan Podlesek <seqdcer@yahoo.com>

Last modified: 6.9.2007
 */
package ue.edit.res.stbof.pst;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.est.Environ;
import ue.exception.InvalidArgumentsException;
import ue.util.AniOrCur;
import ue.util.Tools;
import ue.util.ID;

/**This class is a representation of planet.pst, which contains a list of planets (surprise!).
Some of the planets are home planets others are placed randomly on the map.*/
public class Planet extends InternalFile
{
    Stbof STBOF;

    //fields
    byte[] UNKNOWN_1 = new byte[258];
    private ArrayList<PlanetEntry> PLANETS = new ArrayList<PlanetEntry>();

    //specified by StbofEdit
    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();
        String msg;
        PlanetEntry pe;

        //animations
        ArrayList<String> tmp = getAnimationFiles();
        AniOrCur AOC = null;
        
        for (int i = 0; i < tmp.size(); i++)
        {
            try
            {
                AOC = STBOF.getAnimation(tmp.get(i), false);
            }
            catch (FileNotFoundException fnf)
            {
                msg = Language.getString("Planet.0"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", fnf.getMessage()); //$NON-NLS-1$
                response.add(msg);
            }
            if (AOC == null)
            {
                msg = Language.getString("Planet.1"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", tmp.get(i)); //$NON-NLS-1$
                response.add(msg);
            }
        }
        //entries
        Hashtable<Short, String> idset = new Hashtable<Short, String>();
        ArrayList<PlanetEntry> fix_id = new ArrayList<PlanetEntry>();
        
        for (int i = 0; i < PLANETS.size(); i++)
        {
            pe = PLANETS.get(i);
            //id
            if (idset.containsKey(pe.INDEX))
            {
                msg = Language.getString("Planet.2"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", idset.get(pe.INDEX)); //$NON-NLS-1$
                msg = msg.replace("%3", pe.NAME); //$NON-NLS-1$
                msg = msg.replace("%4", Short.toString(pe.INDEX)); //$NON-NLS-1$
                fix_id.add(pe);
                response.add(msg);
            }
            else
            {
                idset.put(pe.INDEX, pe.NAME);
            }

            //name
            if (pe.NAME.length() > 39)
            {
                msg = Language.getString("Planet.3"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$

                response.add(msg);
            }
            //ani
            if (pe.PIC.length() > 11)
            {
                msg = Language.getString("Planet.4"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$

                response.add(msg);
            }
            //size
            if (pe.SIZE < 0 || pe.SIZE > 3)
            {
                msg = Language.getString("Planet.5"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Short.toString(pe.SIZE)); //$NON-NLS-1$

                response.add(msg);
            }
            //terra points
            if (pe.TERRA_POINTS < 0)
            {
                msg = Language.getString("Planet.6"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Integer.toString(pe.TERRA_POINTS)); //$NON-NLS-1$

                response.add(msg);
            }
            //terraformed
            if (pe.TERRAFORMED < 0)
            {
                msg = Language.getString("Planet.6"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Integer.toString(pe.TERRA_POINTS)); //$NON-NLS-1$

                response.add(msg);
            }
            //bonuses
            if (pe.PLT_BONI < 0 || pe.PLT_BONI > 3)
            {
                msg = Language.getString("Planet.7"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Short.toString(pe.PLT_BONI)); //$NON-NLS-1$
                response.add(msg);
            }
            if (pe.UNI_BONI < 0 || pe.UNI_BONI > 3)
            {
                msg = Language.getString("Planet.8"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Short.toString(pe.UNI_BONI)); //$NON-NLS-1$
                response.add(msg);
            }
            if (pe.POP_BONI < 0 || pe.POP_BONI > 3)
            {
                msg = Language.getString("Planet.9"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Short.toString(pe.POP_BONI)); //$NON-NLS-1$
                response.add(msg);
            }
            //type
            if (pe.TYPE < 0 || pe.TYPE > 7)
            {
                msg = Language.getString("Planet.10"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Short.toString(pe.TYPE)); //$NON-NLS-1$
                response.add(msg);
            }
            //atmosphere
            if (pe.ATMO < 8 || pe.ATMO > 12)
            {
                msg = Language.getString("Planet.11"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Short.toString(pe.ATMO)); //$NON-NLS-1$
                response.add(msg);
            }
        }

        // fix ids
        while (!fix_id.isEmpty())
        {
            pe = fix_id.get(0);
            short id = 0;

            while (pe != null)
            {
                if (!idset.containsKey(id))
                {
                    pe.INDEX = id;
                    fix_id.remove(0);
                    pe = null;
                }
                id++;
            }
        }

        return response;
    }

    @Override
    public long getSize()
    {
        long size = 260 + (long) PLANETS.size() * 76;
        return size;
    }

    @Override
    public void save(OutputStream out) throws Exception
    {
        sortEntries();
        //entries 4
        short lol = (short) PLANETS.size();
        byte[] b = Tools.toByte(lol, true);
        out.write(b);
        //probability and number plus unknown bytes
        Environ env = (Environ) STBOF.getInternalFile(Stbof.fileENVIRON_EST, true);
        for (byte k = 0; k < 8; k++)
        { //8*8=64: 4-68
            //unknown
            out.write(UNKNOWN_1[k * 8]);
            out.write(UNKNOWN_1[k * 8 + 1]);
            //frequency
            float freq = env.getFrequency(k);
            out.write(Tools.toByte(freq, true));
            //number
            short num = getNumberOfPlanets(k);
            out.write(Tools.toByte(num, true));
        }
        //unknown
        for (int k = 0; k < 194; k++)
        {
            out.write(UNKNOWN_1[k + 64]);
        }
        //planet entries
        PlanetEntry pa;
        for (int i = 0; i < lol; i++)
        {
            pa = PLANETS.get(i);
            pa.save(out);
        }
    }

    /**sorts the entries.*/
    public void sortEntries() throws Exception
    {
        PlanetEntry[] plot = new PlanetEntry[PLANETS.size()];
        int i = 0;
        PlanetEntry plan;
        
        for (byte typ = 0; typ < 8; typ++)
        {
            for (byte atm = 8; atm < 13; atm++)
            {
                for (short siz = 0; siz < 3; siz++)
                {
                    for (int co = 0; co < PLANETS.size(); co++)
                    {
                        plan = PLANETS.get(co);
                        if (plan.TYPE == typ && plan.SIZE == siz && plan.ATMO == atm)
                        {
                            plot[i++] = plan;
                        }
                    }
                }
            }
        }
        //clear and add
        PLANETS.clear();
        for (int k = 0; k < plot.length; k++)
        {
            PLANETS.add(plot[k]);
        }
    }

    /**returns the number of consequential planets of the same type.*/
    private short getNumberOfPlanets(byte type)
    {
        short num = 0;
        boolean oky = true;
        PlanetEntry plan;

        for (int i = 0; i < PLANETS.size() && oky; i++)
        {
            plan = PLANETS.get(i);
            if (plan.TYPE != type && num != 0)
            {
                oky = false;
            }
            else if (plan.TYPE == type)
            {
                num++;
            }
        }
        return num;
    }

    /**Constructor.
    @param in the InputStream to read from*/
    public Planet(InputStream in, Stbof st) throws IOException
    {
        STBOF = st;
        //entries
        in.skip(2);
        //unknown
        in.read(UNKNOWN_1);
        //planet entries
        PlanetEntry pa;
        while (in.available() > 0)
        {
            pa = new PlanetEntry(in);
            PLANETS.add(pa);
        }
    }
    //get stuff

    /**returns an array of planet names.*/
    public String[] getNames()
    {
        int num = PLANETS.size();
        String[] ret = new String[num];
        PlanetEntry planet;
        
        for (int i = 0; i < num; i++)
        {
            planet = PLANETS.get(i);
            ret[i] = new String(planet.NAME);
        }
        return ret;
    }

    /**returns a list of animation files used*/
    public ArrayList<String> getAnimationFiles()
    {
        PlanetEntry pe;
        ArrayList<String> ret = new ArrayList<String>();
        
        for (int i = 0; i < PLANETS.size(); i++)
        {
            pe = PLANETS.get(i);
            if (!ret.contains(pe.PIC))
            {
                ret.add(new String(pe.PIC));
            }
        }
        return ret;
    }

    /**Returns the name of the planet at the specified index.
    @param index the index of the planet to get the name of.
     */
    public String getName(int index) throws InvalidArgumentsException
    {
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        return new String(planet.NAME);
    }

    /**Returns the index of the planet with the specified ID.
    @param ID the id of the planet.
    @return -1 if the planet is not found
     */
    public int getIndex(short ID)
    {
        PlanetEntry planet;
        int index = -1;
        int counter = 0;

        while (counter < PLANETS.size() && index == -1)
        {
            planet = PLANETS.get(counter);
            if (planet.INDEX == ID)
            {
                index = counter;
            }
            counter++;
        }
        return index;
    }

    /**Returns the IDs of planets.
     */
    public ID[] getIDs(int i)
    {
        PlanetEntry planet;
        if (i == 0) {
            ID[] ret = new ID[PLANETS.size()];
            for (int j = 0; j < PLANETS.size(); j++)
            {
                planet = PLANETS.get(j);
                ret[j] = new ID(planet.NAME, planet.INDEX);
            }
            return ret;
        }
        else
        {
            ArrayList<ID> hmp = new ArrayList<ID>();
            for (int j = 0; j < PLANETS.size(); j++)
            {
                planet = PLANETS.get(j);
                if (planet.TYPE == (byte) (i - 1))
                {
                    hmp.add(new ID(planet.NAME, planet.INDEX));
                }
            }
            ID[] ret = new ID[hmp.size()];
            for (int j = 0; j < hmp.size(); j++)
            {
                ret[j] = hmp.get(j);
            }
            return ret;
        }
    }

    /**Returns the ID of the planet at the specified index.
    @param index the index of the planet.
     */
    public short getID(int index) throws InvalidArgumentsException
    {
        checkIndex(index);
        PlanetEntry planet;
        planet = PLANETS.get(index);
        return planet.INDEX;
    }

    /**Returns the name of the planet's animation file at the specified index.
    @param index the index of the planet to get the name of the animation file.
     */
    public String getAnimation(int index) throws InvalidArgumentsException
    {
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        return new String(planet.PIC);
    }

    /**Returns the size of the planet at the specified index.
    @param index the index of the planet to get the size of
     */
    public short getSize(int index) throws InvalidArgumentsException
    {
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        return planet.SIZE;
    }

    /**Returns the terraforming points of the planet at the specified index.
    @param index the index of the planet to get the terraforming points of
     */
    public int getTerraPoints(int index) throws InvalidArgumentsException
    {
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        return planet.TERRA_POINTS;
    }

    /**Returns how much of the planet at the specified index is terraformed.
    @param index the index of the planet
     */
    public int getTerraformed(int index) throws InvalidArgumentsException
    {
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        return planet.TERRAFORMED;
    }

    /**Returns the planet bonus of the planet at the specified index.
    @param index the index of the planet
     */
    public byte getPltBoni(int index) throws InvalidArgumentsException
    {
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        return planet.PLT_BONI;
    }

    /**Returns the universal bonus of the planet at the specified index.
    @param index the index of the planet
     */
    public byte getUniBoni(int index) throws InvalidArgumentsException
    {
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        return planet.UNI_BONI;
    }

    /**Returns the population bonus of the planet at the specified index.
    @param index the index of the planet
     */
    public byte getPopBoni(int index) throws InvalidArgumentsException
    {
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        return planet.POP_BONI;
    }

    /**Returns the type of the planet at the specified index.
    @param index the index of the planet
     */
    public byte getType(int index) throws InvalidArgumentsException
    {
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        return planet.TYPE;
    }

    /**Returns the atmosphere of the planet at the specified index.
    @param index the index of the planet
     */
    public byte getAtmosphere(int index) throws InvalidArgumentsException
    {
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        return planet.ATMO;
    }
    //set stuff

    /**Sets the name of the planet at the specified index.
    @param index the index of the planet.
    @param newName the new name of the planet
    @return true if changes were made
     */
    public boolean setName(int index, String newName) throws InvalidArgumentsException
    {
        if (newName.length() > 39)
        {
            return false;
        }
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        if (newName.equals(planet.NAME))
        {
            return false;
        }
        planet.NAME = new String(newName);
        this.markChanged();
        return true;
    }

    /**Sets the name of the planet's animation file at the specified index.
    @param index the index of the planet.
    @param newAni the new animation file
    @return true if changes were made
     */
    public boolean setAnimation(int index, String newAni) throws InvalidArgumentsException {
        if (newAni.length() > 12) {
            return false;
        }
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        if (newAni.equals(planet.PIC)) {
            return false;
        }
        planet.PIC = new String(newAni);
        this.markChanged();
        return true;
    }

    /**Sets the size of the planet at the specified index.
    @param index the index of the planet to get the size of
    @param size the new size, can be 0, 1 or 2
    @return true if changes were made
     */
    public boolean setSize(int index, short size) throws InvalidArgumentsException {
        if (size < 0 || size > 2) {
            String msg = Language.getString("Planet.12"); //$NON-NLS-1$
            msg = msg.replace("%1", Short.toString(size)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        if (size == planet.SIZE) {
            return false;
        }
        planet.SIZE = size;
        this.markChanged();
        return true;
    }

    /**Sets the terraforming points of the planet at the specified index.
    @param index the index of the planet to set the terraforming points of
    @param terra the terraforming points
    @return true if data has been changed
     */
    public void setTerraPoints(int index, int terra) throws InvalidArgumentsException {
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);

        terra = terra & 0xFFFF;
        if (planet.TERRA_POINTS == terra) {
            return;
        }
        planet.TERRA_POINTS = terra;
        this.markChanged();
    }

    /**Sets how much of the planet at the specified index is terraformed.
    @param index the index of the planet
    @param terra how many terraforming points have been terraformed
    @return true if data has been changed
     */
    public void setTerraformed(int index, int terra) throws InvalidArgumentsException {
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        terra = terra & 0xFFFF;
        if (planet.TERRAFORMED == terra) {
            return;
        }
        planet.TERRAFORMED = terra;
        this.markChanged();
    }

    /**Sets the planet bonus of the planet at the specified index.
    @param index the index of the planet
    @param boni the bonus
    @return true if data has been changed
     */
    public boolean setPltBoni(int index, byte boni) throws InvalidArgumentsException {
        if (boni < 0 || boni > 2) {
            String msg = Language.getString("Planet.13"); //$NON-NLS-1$
            msg = msg.replace("%1", Short.toString(boni)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        if (planet.PLT_BONI == boni) {
            return false;
        }
        planet.PLT_BONI = boni;
        this.markChanged();
        return true;
    }

    /**Sets the universal bonus of the planet at the specified index.
    @param index the index of the planet
    @param boni the bonus
    @return true if data has been changed
     */
    public boolean setUniBoni(int index, byte boni) throws InvalidArgumentsException {
        if (boni < 0 || boni > 2) {
            String msg = Language.getString("Planet.13"); //$NON-NLS-1$
            msg = msg.replace("%1", Short.toString(boni)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        if (planet.UNI_BONI == boni) {
            return false;
        }
        planet.UNI_BONI = boni;
        this.markChanged();
        return true;
    }

    /**Sets the population bonus of the planet at the specified index.
    @param index the index of the planet
    @param boni the bonus
    @return true if data has been changed
     */
    public boolean setPopBoni(int index, byte boni) throws InvalidArgumentsException {
        if (boni < 0 || boni > 2) {
            String msg = Language.getString("Planet.13"); //$NON-NLS-1$
            msg = msg.replace("%1", Short.toString(boni)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        if (planet.POP_BONI == boni) {
            return false;
        }
        planet.POP_BONI = boni;
        this.markChanged();
        return true;
    }

    /**Sets the type of the planet at the specified index.
    @param index the index of the planet
    @param type the planet type
    @return true if data has been changed
     */
    public boolean setType(int index, byte type) throws InvalidArgumentsException {
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        if (planet.TYPE == type) {
            return false;
        }
        planet.TYPE = type;
        this.markChanged();
        return true;
    }

    /**Sets the atmosphere of the planet at the specified index.
    @param index the index of the planet
    @param atmo the planet atmosphere
    @return true if data has been changed
     */
    public boolean setAtmosphere(int index, byte atmo) throws InvalidArgumentsException {
        checkIndex(index);
        PlanetEntry planet = PLANETS.get(index);
        if (planet.ATMO == atmo) {
            return false;
        }
        planet.ATMO = atmo;
        this.markChanged();
        return true;
    }

    private void checkIndex(int index) throws InvalidArgumentsException {
        if (index < 0 || index >= PLANETS.size()) {
            String msg = Language.getString("Planet.14"); //$NON-NLS-1$
            msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
    }

    public void addPlanet(byte type) throws Exception
    {
        if (PLANETS.size() == Short.MAX_VALUE)
        {
            throw new Exception("Maximum number of planets reached.");
        }

        // find unused id
        short id;

        for (id = 1;; id++) {
            if (this.getIndex(id) < 0) {
                break;
            }
        }

        PlanetEntry planet = new PlanetEntry(this.PLANETS.get(PLANETS.size() - 1));

        // fix
        planet.INDEX = id;
        planet.NAME = Short.toString(id) + " (added by UE)";
        planet.TYPE = type;

        // add
        PLANETS.add(planet);

        this.markChanged();
    }

    public void removePlanet(short id) throws Exception
    {
        if (PLANETS.size() == 1)
        {
            throw new Exception("Minimum number of planets reached.");
        }

        int index = this.getIndex(id);

        if (index >= 0)
        {
            PLANETS.remove(index);
        }

        this.markChanged();
    }
}

