/*
Ultimate Editor for Birth Of The Federation 1.02
Copyright (C) 2005-2007  Alan Podlesek

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


File: PlanetEntry.java

Written by: Alan Podlesek <seqdcer@yahoo.com>

Last modified: 6.9.2007
 */
package ue.edit.res.stbof.pst;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import ue.util.Tools;

/**This class is a reprisentation of a planet entry in planet.pst, which contains info on particular planet.
This class is used by planet.pst as well as systInfo.*/
public class PlanetEntry implements Comparable<PlanetEntry> {
    //fields

    /**The name of the planet. Can be max of 39 chars.*/
    public  String  NAME;
    /**The animation for the planet. Can be max of 11 chars.*/
    public  String PIC;
    private byte[]  UNKNOWN_1 = new byte[2];
    /**The index of the planet. This is like an ID ;)*/
    public  short   INDEX;
    /**Size of the planet, can be 0 (small), 1 (medium) or 2 (large).*/
    public  short   SIZE;
    /**Terraforming points of the planet - these are the points that need to be terraformed to make the planet colonizable.*/
    public  int     TERRA_POINTS;
    /**How many terraforming points have been *terraformed*.*/
    public  int     TERRAFORMED;
    /**Planet bonus - can be 0, 1 or 2*/
    public  byte    PLT_BONI;
    /**Universal bonus - can be 0, 1 or 2*/
    public  byte    UNI_BONI;
    /**Population bonus - can be 0, 1 or 2*/
    public  byte    POP_BONI;
    private byte[]  UNKNOWN_2 = new byte[4];
    /**Type of the planet.*/
    public  byte    TYPE;
    public  byte    ATMO; //sphere
    private byte[]  UNKNOWN_3 = new byte[2];

    public long getSize()
    {
        return 76;
    }

    public void save(OutputStream out) throws IOException
    {
        //name
        out.write(Tools.toByte(NAME, 40, 39));
        //pic
        out.write(Tools.toByte(PIC, 12, 11));
        //unknown
        out.write(UNKNOWN_1);
        //index
        out.write(Tools.toByte(INDEX, true));
        //size
        out.write(Tools.toByte(SIZE, true));
        //terra_points
        out.write(Tools.toByte((short)(TERRA_POINTS & 0xFFFF), true));
        //terraformed
        out.write(Tools.toByte((short)(TERRAFORMED & 0xFFFF), true));
        //planet bonus
        short boni;
        if (PLT_BONI == 0x02) {
            boni = 620;
        } else {
            if (PLT_BONI == 0x01) {
                boni = 310;
            } else {
                boni = 0;
            }
        }
        out.write(Tools.toByte(boni, true));
        //universal bonus
        if (UNI_BONI == 0x02) {
            boni = 620;
        } else {
            if (UNI_BONI == 0x01) {
                boni = 310;
            } else {
                boni = 0;
            }
        }
        out.write(Tools.toByte(boni, true));
        //population bonus
        if (POP_BONI == 0x02) {
            boni = 620;
        } else {
            if (POP_BONI == 0x01) {
                boni = 310;
            } else {
                boni = 0;
            }
        }
        out.write(Tools.toByte(boni, true));
        //unknown
        out.write(UNKNOWN_2);
        //type
        out.write(TYPE);
        //atmosphere
        out.write(ATMO);
        //unknown
        out.write(UNKNOWN_3);
    }

    @Override
    public int compareTo(PlanetEntry obj) {
        return INDEX - obj.INDEX;
    }

    @Override
    public boolean equals(Object obj)
    {
        if (!(obj instanceof PlanetEntry))
        {
            return false;
        }

        PlanetEntry pe = (PlanetEntry) obj;
        return INDEX == pe.INDEX;
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 59 * hash + this.INDEX;
        return hash;
    }

    /**constructor
    @param in the InputStream to fetch data from*/
    public PlanetEntry(InputStream in) throws IOException
    {
        //name        40
        byte[] b = new byte[40];
        in.read(b);
        NAME = Tools.toASCII(b, 0, 40);
        //pic         52
        b = new byte[12];
        in.read(b);
        PIC = Tools.toASCII(b, 0, 12);
        //unknown     54
        in.read(UNKNOWN_1);
        //index       56
        b = new byte[2];
        in.read(b);
        INDEX = Tools.toShort(b, true);
        //size        58
        in.read(b);
        SIZE = Tools.toShort(b, true);
        //terra_points        60
        in.read(b);
        TERRA_POINTS = Tools.toShort(b, true) & 0xFFFF;
        //terraformed 62
        in.read(b);
        TERRAFORMED = Tools.toShort(b, true) & 0xFFFF;
        //planet bonus        64
        in.read(b);
        int lol = Tools.toShort(b, true);
        if (lol >= 620) {
            PLT_BONI = 0x02;
        } else {
            if (lol >= 310) {
                PLT_BONI = 0x01;
            } else {
                PLT_BONI = 0x00;
            }
        }
        //universal bonus     66
        in.read(b);
        lol = Tools.toShort(b, true);
        if (lol >= 620) {
            UNI_BONI = 0x02;
        } else {
            if (lol >= 310) {
                UNI_BONI = 0x01;
            } else {
                UNI_BONI = 0x00;
            }
        }
        //population bonus    68
        in.read(b);
        lol = Tools.toShort(b, true);
        if (lol >= 620) {
            POP_BONI = 0x02;
        } else {
            if (lol >= 310) {
                POP_BONI = 0x01;
            } else {
                POP_BONI = 0x00;
            }
        }
        //unknown     72
        in.read(UNKNOWN_2);
        //type
        b = new byte[1];
        in.read(b);
        TYPE = b[0];
        //atmosphere
        in.read(b);
        ATMO = b[0];
        //unknown
        in.read(UNKNOWN_3);
    }

    /**constructor
    @param data the byte array to fetch data from*/
    public PlanetEntry(byte[] data) throws IOException {
        byte[] b;
        //name 0-40
        NAME = Tools.toASCII(data, 0, 40);
        //pic 40-12
        PIC = Tools.toASCII(data, 40, 12);
        //unknown 52-2
        UNKNOWN_1[0] = data[52];
        UNKNOWN_1[1] = data[53];
        //index 54-2
        b = new byte[2];
        b[0] = data[54];
        b[1] = data[55];
        INDEX = Tools.toShort(b, true);
        //size 56-2
        b[0] = data[56];
        b[1] = data[57];
        SIZE = Tools.toShort(b, true);
        //terra_points 58-2
        b[0] = data[58];
        b[1] = data[59];
        TERRA_POINTS = Tools.toShort(b, true) & 0xFFFF;
        //terraformed 60-2
        b[0] = data[60];
        b[1] = data[61];
        TERRAFORMED = Tools.toShort(b, true) & 0xFFFF;
        //planet bonus 62-2
        b[0] = data[62];
        b[1] = data[63];
        int lol = Tools.toShort(b, true);
        if (lol >= 620) {
            PLT_BONI = 0x02;
        } else {
            if (lol >= 310) {
                PLT_BONI = 0x01;
            } else {
                PLT_BONI = 0x00;
            }
        }
        //universal bonus 64-2
        b[0] = data[64];
        b[1] = data[65];
        lol = Tools.toShort(b, true);
        if (lol >= 620) {
            UNI_BONI = 0x02;
        } else {
            if (lol >= 310) {
                UNI_BONI = 0x01;
            } else {
                UNI_BONI = 0x00;
            }
        }
        //population bonus 66-2
        b[0] = data[66];
        b[1] = data[67];
        lol = Tools.toShort(b, true);
        if (lol >= 620) {
            POP_BONI = 0x02;
        } else {
            if (lol >= 310) {
                POP_BONI = 0x01;
            } else {
                POP_BONI = 0x00;
            }
        }
        //unknown 68-4
        for (int i = 0; i < 4; i++) {
            UNKNOWN_2[i] = data[68 + i];
        }
        //type 72-1
        TYPE = data[72];
        //atmosphere 73-1
        ATMO = data[73];
        //unknown 74-2
        for (int i = 0; i < 2; i++) {
            UNKNOWN_3[i] = data[74 + i];
        }
    }

    /**constructor
    @param source the PlanetEntry to fetch data from*/
    public PlanetEntry(PlanetEntry source) {
        //name
        NAME = new String(source.NAME);
        //pic
        PIC = new String(source.PIC);
        //unknown
        UNKNOWN_1 = source.UNKNOWN_1;
        //index
        INDEX = source.INDEX;
        //size
        SIZE = source.SIZE;
        //terra_points
        TERRA_POINTS = source.TERRA_POINTS;
        //terraformed
        TERRAFORMED = source.TERRAFORMED;
        //planet bonus
        PLT_BONI = source.PLT_BONI;
        //universal bonus
        UNI_BONI = source.UNI_BONI;
        //population bonus
        POP_BONI = source.POP_BONI;
        //unknown
        UNKNOWN_2 = source.UNKNOWN_2;
        //type
        TYPE = source.TYPE;
        //type
        ATMO = source.ATMO;
        //unknown
        UNKNOWN_3 = source.UNKNOWN_3;
    }
}
