/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: Objstruc.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.edit.res.stbof.smt;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.edit.res.Stbof;
import ue.exception.InvalidArgumentsException;
import ue.util.AniOrCur;
import ue.util.Tools;
import ue.util.TargaImage;

public class Objstruc extends InternalFile{
	private short ENTRIES;
	private Stbof STBOF;
	private ArrayList<ObjEntry> ENTRY = new ArrayList<ObjEntry>();
	
	/**@param in the input stream to read value from*/
	public Objstruc(InputStream in, Stbof st) throws IOException{
		STBOF = st;
		//numoentries
		byte[] b = new byte[2];
		in.read(b);
		ENTRIES = Tools.toShort(b, true);
		//loop
		b = new byte[24];
		while (in.available()>0){
			in.read(b);
			ENTRY.add(new ObjEntry(b));
		}
		in.close();
	}
	
	/**@param out the output stream to write to*/
	@Override
	public void save(OutputStream out) throws Exception{
		//set up
		ENTRIES = (short)ENTRY.size();
		out.write(Tools.toByte(ENTRIES, true));
		ObjEntry oe;
		byte[] b;
		for (short i = 0; i<ENTRIES; i++){
			oe = ENTRY.get(i);
			//index
			b = Tools.toByte(i, true);
			out.write(b);
			//unknown
			out.write(oe.unknown);
			//frequency
			out.write(Tools.toByte(oe.FREQUENCY, true));
			//graphics
			b = Tools.toByte(oe.GRAPH, 16, 15);
			out.write(b);
		}
	}

	public float allFreq(){
		float flt = 0;
		int n = ENTRY.size();
		for (int i=0; i<n; i++){
			ObjEntry le = ENTRY.get(i);
			flt = flt + le.FREQUENCY;
		}
		return flt;
	}
	
	public int numberOfEntries(){
		int n = ENTRY.size();
		return n;
	}
	
	public String getGraph(int index) throws InvalidArgumentsException{
		checkIndex(index);
		ObjEntry le = ENTRY.get(index);
		return new String(le.GRAPH);
	}
	
	public boolean setGraph(int index, String ent) throws InvalidArgumentsException{
		if (ent.length() < 1 || ent.length() > 15) {
			return false;
		}
		checkIndex(index);
		ObjEntry le = ENTRY.get(index);
		if (ent.equals(le.GRAPH)) {
			return false;
		}
		le.GRAPH = new String(ent);
                this.markChanged();
		return true;
	}
	
	public float getFreq(int index) throws InvalidArgumentsException{
		checkIndex(index);
		ObjEntry le = ENTRY.get(index);
		return le.FREQUENCY;
	}
	
	public boolean setFreq(int index, float ent) throws InvalidArgumentsException{
		checkIndex(index);
		ObjEntry le = ENTRY.get(index);
		if (le.FREQUENCY == ent) {
			return false;
		}
		le.FREQUENCY = ent;
                this.markChanged();
		return true;
	}

	public String[] getGraphList(){
		int n = ENTRY.size();
		ArrayList<String> tut = new ArrayList<String>();
		for (int i=0; i<n; i++){
			ObjEntry le = ENTRY.get(i);
			if (le.GRAPH.equals("neb-")) { //$NON-NLS-1$
				tut.add(le.GRAPH + ".tga"); //$NON-NLS-1$
			} else {
				tut.add(le.GRAPH + ".ani"); //$NON-NLS-1$
			}
			tut.add(le.GRAPH + "map.tga"); //$NON-NLS-1$
			tut.add(le.GRAPH + "maps.tga"); //$NON-NLS-1$
		}
		return tut.toArray(new String[0]);
	}
	
	/**@return the uncompressed size of the file**/
	@Override
	public long getSize() {
		int n = ENTRY.size();
		int size = 2 + n * 24;
		return size;
	}

	@Override
	public Vector<String> check(){
		Vector<String> response = new Vector<String>();
		String msg;
		
		float flt = 1;
		if (allFreq() != flt){
			msg = Language.getString("Objstruc.0"); //$NON-NLS-1$
			msg = msg.replace("%1", getName()); //$NON-NLS-1$
			response.add(msg);
		}
		int n = numberOfEntries();
		while (n > 0){
			n--;
			try{
				if (getFreq(n) <= 0){
					msg = Language.getString("Objstruc.1"); //$NON-NLS-1$
					msg = msg.replace("%1", getName()); //$NON-NLS-1$
					msg = msg.replace("%2", Integer.toString(n)); //$NON-NLS-1$
					msg = msg.replace("%3", Float.toString(getFreq(n))); //$NON-NLS-1$
					response.add(msg);
				}
				if (getGraph(n).length() > 15){
					msg = Language.getString("Objstruc.2"); //$NON-NLS-1$
					msg = msg.replace("%1", getName()); //$NON-NLS-1$
					msg = msg.replace("%2", Integer.toString(n)); //$NON-NLS-1$
					msg = msg.replace("%3", getGraph(n)); //$NON-NLS-1$
					response.add(msg);
				}
			}catch (InvalidArgumentsException er){
				msg = Language.getString("Objstruc.3"); //$NON-NLS-1$
				msg = msg.replace("%1", getName()); //$NON-NLS-1$
				msg = msg.replace("%2", er.getMessage()); //$NON-NLS-1$
				response.add(msg);
			}
		}
		String[] pt = getGraphList();
		AniOrCur AOC = null;
		TargaImage tar = null;
		for (int i=0; i<pt.length; i++){
			if (pt[i].endsWith(".ani")){ //$NON-NLS-1$
				try{
					AOC = STBOF.getAnimation(pt[i], false);
				}catch (FileNotFoundException fnf){
					msg = Language.getString("Objstruc.3"); //$NON-NLS-1$
					msg = msg.replace("%1", getName()); //$NON-NLS-1$
					msg = msg.replace("%2", fnf.getMessage()); //$NON-NLS-1$
					response.add(msg);
				}
				if (AOC==null){
					msg = Language.getString("Objstruc.4"); //$NON-NLS-1$
					msg = msg.replace("%1", getName()); //$NON-NLS-1$
					msg = msg.replace("%2", pt[i]); //$NON-NLS-1$
					response.add(msg);
				}
			}else{
				try{
					tar = STBOF.getTargaImage(pt[i], false);
				}catch (FileNotFoundException fnf){
					msg = Language.getString("Objstruc.3"); //$NON-NLS-1$
					msg = msg.replace("%1", getName()); //$NON-NLS-1$
					msg = msg.replace("%2", fnf.getMessage()); //$NON-NLS-1$
					response.add(msg);
				}
				if (tar==null){
					msg = Language.getString("Objstruc.5"); //$NON-NLS-1$
					msg = msg.replace("%1", getName()); //$NON-NLS-1$
					msg = msg.replace("%2", pt[i]); //$NON-NLS-1$
					response.add(msg);
				}
			}
		}
		return response;
	}
	
	private void checkIndex(int index) throws InvalidArgumentsException{
		if (index<0 || index>=ENTRY.size()){
			String msg = Language.getString("Objstruc.6"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
	}
	
	private class ObjEntry{
		public short INDEX; //2
		public byte[] unknown = new byte[2]; //2
		public float FREQUENCY; //4
		public String GRAPH; //16
		public ObjEntry(byte[] b){
			try{
				byte[] c = new byte[2];
				c[0] = b[0];
				c[1] = b[1];
				INDEX = Tools.toShort(c, true);
				//tictac
				unknown[0] = b[2];
				unknown[1] = b[3];
				c = new byte[4];
				for (int i = 0; i<4; i++) {
					c[i] = b[4+i];
				}
				FREQUENCY = Tools.toFloat(c, true);
				GRAPH = Tools.toASCII(b, 8, 16);
			}catch (Exception ex){ ex.printStackTrace();}
		}
	}
}
