/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: ShipList.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 28.9.2007
*/

package ue.edit.res.stbof.sst;

import java.awt.Dimension;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.StringFile;
import ue.exception.InvalidArgumentsException;
import ue.util.ID;
import ue.util.Pair;

/**Represents shiplist.sst.*/
public class ShipList extends InternalFile
{
	private Stbof STBOF = null;
	private ArrayList<Ship> SHIP = new ArrayList<Ship>();
	// fields
        public static final int RACE_ALIEN = 36;
        
	public ShipList(InputStream in, Stbof st) throws Exception
        {
            STBOF = st;
            ShipTech tech = (ShipTech)STBOF.getInternalFile(Stbof.fileSHIPTECH_SST, true);
            Ship sh;
            
            while (in.available() > 0)
            {
                sh = new Ship(in);
                SHIP.add(sh);
                sh.RACE_OVERRIDE = sh.RACE;
                sh.RACE = (short)tech.getRace(sh.ID);
            }
	}
	
	public void sort(){
		Collections.sort(SHIP);
	}
	
	/**Saves changes.
	@return true if save has been successful.
	@throws Exception the function may throw an exception if an error occures.*/
	@Override
	public void save(OutputStream out) throws Exception
    {
	    Ship sh;

        for (short i = 0; i < SHIP.size(); i++)
        {
            sh = SHIP.get(i);
            sh.save(out);
        }
	}

	/**This function checks the file for known problems/errors that may occur while editing the file.
	@return a Vector object containing results of the check as Strings.*/
	@Override
	public Vector<String> check()
        {
		Vector<String> response = new Vector<String>();
		
		Ship sh;

                /*
                HobFile hob;
                Dimension texture_size = null;
                GenericFile gfil;
                
                Texture texture = null;
                
                try
                {
                    texture = (Texture)STBOF.getInternalFile(Stbof.fileTEXTURE_LST);
                }
                catch (Exception e)
                {}
                */
                
		for (int i=0; i<SHIP.size(); i++){
			sh = SHIP.get(i);
			Vector<String> tmp = sh.check(getName());
			if (!tmp.isEmpty()) {
				response.addAll(tmp);
			}
			//graphics
			
			if (sh.GRAPHICS.toLowerCase().equals("t02")) {
                continue; //$NON-NLS-1$
            } //$NON-NLS-1$

			String str = sh.GRAPHICS + "_a.hob"; //$NON-NLS-1$
			str = str.toLowerCase();
			if (!STBOF.hasInternalFile(str)) {
				response.add(getFileNotFoundString(str));
			}
/*                        else if (texture != null)
                        {
                            try
                            {
                                gfil = (GenericFile)STBOF.getInternalFile(sh.GRAPHICS + "_a.hob");
                                ByteArrayInputStream bais = new ByteArrayInputStream(gfil.getData());
                                hob = new HobFile(bais);
                                
                                for (int t = 0; t < hob.numberOfTextures(); t++)
                                {
                                    int tID = hob.getTextureID(t);
                                    String tex = texture.get(tID);
                                    gfil = (GenericFile)STBOF.getInternalFile(tex);
                                    bais = new ByteArrayInputStream(gfil.getData());
                                    GifImage img = new GifImage(bais);
                                    
                                    if (texture_size == null)
                                    {
                                        texture_size = img.getSize();
                                    }
                                    else if (!texture_size.equals(img.getSize()))
                                    {
                                        response.add(getImageSizeWarningString(tex, texture_size, img.getSize()));
                                    }
                                }
                            }
                            catch (Exception e)
                            {
                                response.add(getGenericErrorString(e.getMessage()));
                            }
                        }
*/                        
			str = sh.GRAPHICS + "_b.hob"; //$NON-NLS-1$
			str = str.toLowerCase();
			if (!STBOF.hasInternalFile(str)) {
				response.add(getFileNotFoundString(str));
			}
			str = sh.GRAPHICS + "_c.hob"; //$NON-NLS-1$
			str = str.toLowerCase();
			if (!STBOF.hasInternalFile(str)) {
				response.add(getFileNotFoundString(str));
			}
		}
		//sinchron check
		try{
			ShipTech sht = (ShipTech)STBOF.getInternalFile(Stbof.fileSHIPTECH_SST, true);
			for (int i=0; i<SHIP.size(); i++){
				sh = SHIP.get(i);
				if (sh.RACE!=sht.getRace(i)){
					String msg = Language.getString("ShipList.0"); //$NON-NLS-1$
					msg = msg.replace("%1", getName()); //$NON-NLS-1$
					msg = msg.replace("%2", Integer.toString(sh.RACE)); //$NON-NLS-1$
					msg = msg.replace("%3", Integer.toString(sht.getRace(i))); //$NON-NLS-1$
					msg = msg.replace("%4", Stbof.fileSHIPTECH_SST); //$NON-NLS-1$
					response.add(msg);
					sht.setRace(i, sh.RACE);
                                        this.markChanged();
				}
			}
		}catch (Exception et){
			response.add(getGenericErrorString(et.getMessage()));
		}
		// duplicate description addresses
		// some modders might use same address for multiple ships for convenience
		// however this can make editing descriptions in UE difficult (read: hocus pocus type)
		// there is no way to know if the user wants to "divide" descriptions
		// or if (s)he's even aware the descriptions are shared
		// this alerts the user if ships are using same description
		// hopefully not many modders do this
		try{
			for (int i=0; i<SHIP.size(); i++){
				sh = SHIP.get(i);
				for (int j=i+1; j<SHIP.size(); j++){
					Ship sh2 = SHIP.get(j);
					if (sh.DESCRIPTION==sh2.DESCRIPTION){
						String msg = Language.getString("ShipList.1"); //$NON-NLS-1$
						msg = msg.replace("%1", getName()); //$NON-NLS-1$
						msg = msg.replace("%2", sh.NAME); //$NON-NLS-1$
						msg = msg.replace("%3", sh2.NAME); //$NON-NLS-1$
						msg = msg.replace("%4", Integer.toString(sh.DESCRIPTION)); //$NON-NLS-1$
						response.add(msg);
					}
				}
			}
		}catch (Exception et){
			response.add(getGenericErrorString(et.getMessage()));
		}
		//find unused descriptions
		try{
			StringFile sht = (StringFile)STBOF.getInternalFile(Stbof.fileSHIPDESC_SST, true);
			for (int i=0; i<SHIP.size(); i++){
				try{
					sh = SHIP.get(i);
					sht.setIndex(sh.DESCRIPTION, i);
				}catch (Exception et){
					response.add(getGenericErrorString(et.getMessage()));
				}
			}
		}catch (Exception et){
			response.add(getGenericErrorString(et.getMessage()));
		}
		return response;
	}
	/**Used to get the uncompressed size of the file.**/
	@Override
	public long getSize(){
		long size = SHIP.size()*288;
		return size;
	}
	/**Returns an array of ship ID's according to either race or ship type.
         * 45 -> type == 9
         * 37 + race -> type == race - 37
         * 35 -> all ships
         */
	public ID[] getIDs(int race){
		if (race==35){//all
			ID[] ret = new ID[SHIP.size()];
			Ship shp;
			for (int i=0; i<SHIP.size(); i++){
				shp = SHIP.get(i);
				ret[i] = new ID(shp.NAME, i);
			}
			return ret;
		}else if (race<35 || race == 36){
			ArrayList<ID> ret = new ArrayList<ID>();
			Ship shp;
			for (int i=0; i<SHIP.size(); i++){
				shp = SHIP.get(i);
				if (shp.RACE==race) {
					ret.add(new ID(shp.NAME, i));
				}
			}
			return ret.toArray(new ID[0]);
		}else if (race==45){
			race = 9;
			ArrayList<ID> ret = new ArrayList<ID>();
			Ship shp;
			for (int i=0; i<SHIP.size(); i++){
				shp = SHIP.get(i);
				if (shp.SHIP_TYPE==race) {
					ret.add(new ID(shp.NAME, i));
				}
			}
			return ret.toArray(new ID[0]);
		}else{
			race = race - 37;
			ArrayList<ID> ret = new ArrayList<ID>();
			Ship shp;
			for (int i=0; i<SHIP.size(); i++){
				shp = SHIP.get(i);
				if (shp.SHIP_TYPE==race) {
					ret.add(new ID(shp.NAME, i));
				}
			}
			return ret.toArray(new ID[0]);
		}
	}
        
    @Override
    public ArrayList<Pair<String, Dimension>> getImagesListedInThisFile()
    {
        ArrayList<Pair<String, Dimension>> images = new ArrayList<Pair<String, Dimension>>();

        Dimension size1 = new Dimension(120, 100);
        Dimension size2 = new Dimension(60, 50);
        Dimension size3 = new Dimension(30, 30);
        Dimension size4 = new Dimension(170, 142);

        for (Ship ship : SHIP)
        {
            if (ship.SHIP_TYPE != 8)
            {
                images.add(new Pair<String, Dimension>("i_" + ship.GRAPHICS + "w.tga", size1));
            }
            images.add(new Pair<String, Dimension>("i_" + ship.GRAPHICS + "60.tga", size2));
            images.add(new Pair<String, Dimension>("i_" + ship.GRAPHICS + "30.tga", size3));
            images.add(new Pair<String, Dimension>("i_" + ship.GRAPHICS + "120.tga", size1));

            if (ship.SHIP_TYPE != 6 && ship.SHIP_TYPE != 7)
            {
                images.add(new Pair<String, Dimension>("i_" + ship.GRAPHICS + "170.tga", size4));
            }
        }

        return images;
    }
        
	/**Returns graphics prefix.*/
	public String getImageName(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return new String(sh.GRAPHICS);
	}
	/**Sets graphics prefix.*/
	public void setImageName(int index, String name) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.GRAPHICS.equals(name)) {
			return;
		}
		sh.GRAPHICS = new String(name);
                this.markChanged();
	}
	/**Returns race index.*/
	public short getRace(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.RACE;
	}
	/**Return number of races*/
	public int getNumberOfEntries()
	{
		return SHIP.size();
	}
	/**Returns description address of a ship.*/
	public int getDescAddress(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.DESCRIPTION;
	}
	/**Sets description address.*/
	public void setDescAddress(int index, int address) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.DESCRIPTION==address) {
			return;
		}
		sh.DESCRIPTION=address;
                this.markChanged();
	}
	/**Returns name of a ship.*/
	public String getName(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return new String(sh.NAME);
	}
	/**Sets name.*/
	public void setName(int index, String name) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.NAME.equals(name)) {
			return;
		}
		sh.NAME=new String(name);
                this.markChanged();
	}
	/**Returns class of a ship.*/
	public String getClass(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return new String(sh.CLASS);
	}
	/**Sets class.*/
	public void setClass(int index, String clas) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.CLASS.equals(clas)) {
			return;
		}
		sh.CLASS=new String(clas);
                this.markChanged();
	}
	/**Returns function of a ship.*/
	public short getFunction(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.SHIP_TYPE;
	}
	/**Sets function.*/
	public void setFunction(int index, short type) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.SHIP_TYPE==type) {
			return;
		}
		sh.SHIP_TYPE = type;
                this.markChanged();
	}
	/**Returns name group of a ship.*/
	public short getNameGroup(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.SHIPNAME_GROUP;
	}
	/**Sets name group.*/
	public void setNameGroup(int index, short type) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.SHIPNAME_GROUP==type) {
			return;
		}
		sh.SHIPNAME_GROUP = type;
                this.markChanged();
	}
	/**Returns map range of a ship.*/
	public double getMapRange(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.MAP_RANGE;
	}
	/**Sets map range.*/
	public void setMapRange(int index, double range) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.MAP_RANGE==range) {
			return;
		}
		sh.MAP_RANGE = range;
        this.markChanged();
	}
	/**Returns map speed of a ship.*/
	public double getMapSpeed(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.MAP_SPEED;
	}
	/**Sets map speed.*/
	public void setMapSpeed(int index, double speed) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.MAP_SPEED==speed) {
			return;
		}
		sh.MAP_SPEED = speed;
        this.markChanged();
	}
	/**Returns scan range of a ship.*/
	public int getScanRange(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.SCAN_RANGE;
	}
	/**Sets scan range.*/
	public void setScanRange(int index, int scan) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.SCAN_RANGE==scan) {
			return;
		}
		sh.SCAN_RANGE = scan;
        this.markChanged();
	}
	/**Sets race.
	@return true if changed*/
	public boolean setRace(int index, short race, boolean adjust_ovr) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.RACE==race) {
			return false;
		}
		if (adjust_ovr && sh.RACE == sh.RACE_OVERRIDE) {
            sh.RACE_OVERRIDE = race; // race override is not being used
        }
		sh.RACE = race;
        this.markChanged();
		if (adjust_ovr) {
            sort();
        }
		return true;
	}
	/**Returns ship starting id's, this doesn't include the alien ships.*/
	public int[] getStartingIDs(){
		Collections.sort(SHIP);
		int[] ret = new int[35];
		for (int i=0; i<35; i++){
            ID[] x	= getIDs(i);
			if (x.length<1){
				ret[i] = -1;
			}else{
				ret[i] = x[0].hashCode();
			}
		}
		return ret;
	}
	/**Returns ship starting id's*/
	public int[] allignIDs(){
		Collections.sort(SHIP);
		int[] ret = new int[SHIP.size()];
		for (int i=0; i<SHIP.size(); i++){
			Ship sh = SHIP.get(i);
			ret[i] = sh.ID;
			sh.ID = (short)i;
		}
		return ret;
	}
	/**Returns build cost of a ship.*/
	public int getBuildCost(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.BUILD_COST;
	}
	/**Sets build cost.*/
	public void setBuildCost(int index, int cost) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.BUILD_COST==cost) {
			return;
		}
		sh.BUILD_COST = cost;
                this.markChanged();
	}
	/**Returns maintenance of a ship.*/
	public int getMaintenance(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.MAINTENANCE;
	}
	/**Sets maintenance.*/
	public void setMaintenance(int index, int cost) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.MAINTENANCE==cost) {
			return;
		}
		sh.MAINTENANCE = cost;
                this.markChanged();
	}
	/**Returns cloak level of a ship.*/
	public int getCloak(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.CLOAK_LVL;
	}
	/**Sets cloak level.*/
	public void setCloak(int index, int clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.CLOAK_LVL==clk) {
			return;
		}
		sh.CLOAK_LVL = clk;
                this.markChanged();
	}
	/**Returns beam accuracy of a ship.*/
	public int getBeamAccu(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.BEAM_ACCURACY;
	}
	/**Sets beam accuracy.*/
	public void setBeamAccu(int index, int clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.BEAM_ACCURACY==clk) {
			return;
		}
		sh.BEAM_ACCURACY = clk;
                this.markChanged();
	}
	/**Returns beam number of a ship.*/
	public int getBeamNum(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.BEAM_NUMBER;
	}
	/**Sets beam number.*/
	public void setBeamNum(int index, int clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.BEAM_NUMBER==clk) {
			return;
		}
		sh.BEAM_NUMBER = clk;
                this.markChanged();
	}
	/**Returns beam multiplier of a ship.*/
	public int getBeamMulti(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.BEAM_MULTIPLIER;
	}
	/**Sets beam multiplier.*/
	public void setBeamMulti(int index, int clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.BEAM_MULTIPLIER==clk) {
			return;
		}
		sh.BEAM_MULTIPLIER = clk;
                this.markChanged();
	}
	/**Returns beam damage of a ship.*/
	public int getBeamDamage(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.BEAM_DAMAGE;
	}
	/**Sets beam damage.*/
	public void setBeamDamage(int index, int clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.BEAM_DAMAGE==clk) {
			return;
		}
		sh.BEAM_DAMAGE = clk;
                this.markChanged();
	}
	/**Returns beam min range of a ship.*/
	public double getBeamMin(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.BEAM_MIN_RANGE;
	}
	/**Sets beam min range.*/
	public void setBeamMin(int index, double clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.BEAM_MIN_RANGE==clk) {
			return;
		}
		sh.BEAM_MIN_RANGE = clk;
                this.markChanged();
	}
	/**Returns beam max range of a ship.*/
	public double getBeamMax(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.BEAM_MAX_RANGE;
	}
	/**Sets beam max range.*/
	public void setBeamMax(int index, double clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.BEAM_MAX_RANGE==clk) {
			return;
		}
		sh.BEAM_MAX_RANGE = clk;
                this.markChanged();
	}
	/**Returns torp accuracy of a ship.*/
	public int getTorpAccu(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.TORP_ACCURACY;
	}
	/**Sets torp accuracy.*/
	public void setTorpAccu(int index, int clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.TORP_ACCURACY==clk) {
			return;
		}
		sh.TORP_ACCURACY = clk;
                this.markChanged();
	}
	/**Returns torp number of a ship.*/
	public int getTorpNum(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.TORP_NUMBER;
	}
	/**Sets torp number.*/
	public void setTorpNum(int index, int clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.TORP_NUMBER==clk) {
			return;
		}
		sh.TORP_NUMBER = clk;
                this.markChanged();
	}
	/**Returns torp multiplier of a ship.*/
	public int getTorpMulti(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.TORP_MULTIPLIER;
	}
	/**Sets torp multiplier.*/
	public void setTorpMulti(int index, int clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.TORP_MULTIPLIER==clk) {
			return;
		}
		sh.TORP_MULTIPLIER = clk;
                this.markChanged();
	}
	/**Returns torp damage of a ship.*/
	public int getTorpDamage(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.TORP_DAMAGE;
	}
	/**Sets torp damage.*/
	public void setTorpDamage(int index, int clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.TORP_DAMAGE==clk) {
			return;
		}
		sh.TORP_DAMAGE = clk;
                this.markChanged();
	}
	/**Returns torp min range of a ship.*/
	public double getTorpMin(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.TORP_MIN_RANGE;
	}
	/**Sets torp min range.*/
	public void setTorpMin(int index, double clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.TORP_MIN_RANGE==clk) {
			return;
		}
		sh.TORP_MIN_RANGE = clk;
                this.markChanged();
	}
	/**Returns torp max range of a ship.*/
	public double getTorpMax(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.TORP_MAX_RANGE;
	}
	/**Sets torp max range.*/
	public void setTorpMax(int index, double clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.TORP_MAX_RANGE==clk) {
			return;
		}
		sh.TORP_MAX_RANGE = clk;
                this.markChanged();
	}
	/**Returns shield penetration of a ship.*/
	public int getShieldPen(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.SHIELD_PENETRATION;
	}
	/**Sets shield penetration.*/
	public void setShieldPen(int index, int clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.SHIELD_PENETRATION==clk) {
			return;
		}
		sh.SHIELD_PENETRATION = clk;
                this.markChanged();
	}
	/**Returns shield level of a ship.*/
	public int getShieldLvl(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.SHIELD_LEVEL;
	}
	/**Sets shield level.*/
	public void setShieldLvl(int index, int clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.SHIELD_LEVEL==clk) {
			return;
		}
		sh.SHIELD_LEVEL = clk;
                this.markChanged();
	}
	/**Returns shield strength of a ship.*/
	public int getShieldStr(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.SHIELD_STRENGTH;
	}
	/**Sets shield strength.*/
	public void setShieldStr(int index, int clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.SHIELD_STRENGTH==clk) {
			return;
		}
		sh.SHIELD_STRENGTH = clk;
                this.markChanged();
	}
	/**Returns shield recharge of a ship.*/
	public int getShieldRec(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.SHIELD_RECHARGE;
	}
	/**Sets shield recharge.*/
	public void setShieldRec(int index, int clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.SHIELD_RECHARGE==clk) {
			return;
		}
		sh.SHIELD_RECHARGE = clk;
                this.markChanged();
	}
	/**Returns hull strength of a ship.*/
	public int getHullStr(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.HULL_STRENGTH;
	}
	/**Sets hull strength.*/
	public void setHullStr(int index, int clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.HULL_STRENGTH==clk) {
			return;
		}
		sh.HULL_STRENGTH = clk;
                this.markChanged();
	}
	/**Returns defense of a ship.*/
	public int getDefense(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.DEFENSE;
	}
	/**Sets defense.*/
	public void setDefense(int index, int clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.DEFENSE==clk) {
			return;
		}
		sh.DEFENSE = clk;
                this.markChanged();
	}
	/**Returns production of a ship.*/
	public int getProduction(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.PRODUCTION;
	}
	/**Sets production.*/
	public void setProduction(int index, int clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.PRODUCTION==clk) {
			return;
		}
		sh.PRODUCTION = clk;
                this.markChanged();
	}
	/**Returns agility of a ship.*/
	public double getAgility(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.AGILITY;
	}
	/**Sets agility.*/
	public void setAgility(int index, double clk) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.AGILITY==clk) {
			return;
		}
		sh.AGILITY = clk;
                this.markChanged();
	}
	
	/**Returns overriden race of a ship.*/
	public short getRaceOverride(int index) throws InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		return sh.RACE_OVERRIDE;
	}
	/**Sets agility.*/
	public void setRaceOverride(int index, short race) throws  InvalidArgumentsException{
		checkShipIndex(index);
		Ship sh = SHIP.get(index);
		if (sh.RACE_OVERRIDE==race) {
			return;
		}
		sh.RACE_OVERRIDE = race;
                this.markChanged();
	}

	/**Moves the ship up. (lowers it's id)*/
	public int[] moveUp(int index) throws  InvalidArgumentsException{
		checkShipIndex(index);
		if (index-1<0) {
			return allignIDs();
		}
		Ship sh = SHIP.get(index);
		Ship sh2 = SHIP.get(index-1);
		if (sh.RACE!=sh2.RACE) {
			return allignIDs();
		}
		int[] ids = new int[SHIP.size()];
		for (int i = 0; i<SHIP.size(); i++){
			if (i == index){
				ids[i] = index -1;
			}else if (i == index -1){
				ids[i] = index;
			}else{
				ids[i]=i;
			}
		}
		short id = sh2.ID;
		sh2.ID = sh.ID;
		sh.ID = id;
                this.markChanged();
		return ids;
	}
	/**Moves the ship down. (increases it's id)*/
	public int[] moveDown(int index) throws  InvalidArgumentsException{
		checkShipIndex(index);
		if (index+1>=SHIP.size()) {
			return allignIDs();
		}
		Ship sh = SHIP.get(index);
		Ship sh2 = SHIP.get(index+1);
		if (sh.RACE!=sh2.RACE) {
			return allignIDs();
		}
		int[] ids = new int[SHIP.size()];
		for (int i = 0; i<SHIP.size(); i++){
			if (i == index){
				ids[i] = index +1;
			}else if (i == index +1){
				ids[i] = index;
			}else{
				ids[i]=i;
			}
		}
		short id = sh2.ID;
		sh2.ID = sh.ID;
		sh.ID = id;
                this.markChanged();
		return ids;
	}

    /** Creates a copy of the ship with the specified index, moves ships after the id down.*/
    public void createShip(int index, int desc_address) throws Exception{
        checkShipIndex(index);
        Ship sh = SHIP.get(index);
        sh = new Ship(sh);
        // fix new ship
        sh.DESCRIPTION = desc_address;
        SHIP.add(index, sh);
        
        // add and fix id's
        // other files that depend on these ids must be fixed seperatly.
        // you need to: add a description, fix shiptech and shiprace
        for (short i = 0; i<SHIP.size(); i++){
            sh = SHIP.get(i);
            sh.ID = i;
        }
        this.markChanged();
    }
    
    /** Removes the ship with the specified id. */
    public void removeShip(int index) throws Exception{
		checkShipIndex(index);
        SHIP.remove(index);

        // add and fix id's
        // other files that depend on these ids must be fixed seperatly.
        // you need to: fix shiptech and shiprace, remove description
		Ship sh;
        for (short i = 0; i<SHIP.size(); i++)
        {
            sh = SHIP.get(i);
            sh.ID = i;
        }
        this.markChanged();
    }
    
    public ArrayList<String> getGraphics(){
    	ArrayList<String> g = new ArrayList<String>();
    	Ship sh;
    	for (short i = 0; i<SHIP.size(); i++){
            sh = SHIP.get(i);
        	if (!g.contains(sh.GRAPHICS)) {
                g.add(sh.GRAPHICS);
            }
        }
    	return g;
    }
    
    private String getFileNotFoundString(String file){
    	String msg = Language.getString("ShipList.2"); //$NON-NLS-1$
		msg = msg.replace("%1", getName()); //$NON-NLS-1$
		msg = msg.replace("%2", file); //$NON-NLS-1$
    	return msg;
    }
    
    private String getGenericErrorString(String error){
    	String msg = Language.getString("ShipList.3"); //$NON-NLS-1$
		msg = msg.replace("%1", getName()); //$NON-NLS-1$
		msg = msg.replace("%2", error); //$NON-NLS-1$
    	return msg;
    }
    
    private void checkShipIndex(int index) throws InvalidArgumentsException{
		if (index<0 || index>=SHIP.size()){
			String msg = Language.getString("ShipList.8"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
    }

    /**
     * @param g
     * @return
     */
    public String getClassByPrefix(String g)
    {
        Ship sh;
        
        g = g.toLowerCase();
        
        for (int i = 0; i < SHIP.size(); i++)
        {
            sh = SHIP.get(i);
            if (sh.GRAPHICS.toLowerCase().equals(g)) {
                return sh.CLASS;
            }
        }
        
        return null;
    }
    
    /**
     * returns -1 if not found
     * @param g
     * @return
     */
    public int getFirstIDWithPrefix(String g)
    {
        g = g.toLowerCase();
        Ship sh;
        for (int i = 0; i < SHIP.size(); i++)
        {
            sh = SHIP.get(i);
            if (sh.GRAPHICS.toLowerCase().equals(g)) {
                return i;
            }
        }
        return -1;
    }

    public char getPhaserPrefix(int index) throws InvalidArgumentsException
    {
        checkShipIndex(index);
        return SHIP.get(index).PHASER_PREFIX;
    }

    public void setPhaserPrefix(int index, char prefix) throws InvalidArgumentsException
    {
        checkShipIndex(index);

        Ship sh = SHIP.get(index);

        if (sh.PHASER_PREFIX != prefix)
        {
            sh.PHASER_PREFIX = prefix;
            this.markChanged();
        }
    }

    public char getTorpedoPrefix(int index) throws InvalidArgumentsException
    {
        checkShipIndex(index);
        return SHIP.get(index).TORPEDO_PREFIX;
    }

    public void setTorpedoPrefix(int index, char prefix) throws InvalidArgumentsException
    {
        checkShipIndex(index);

        Ship sh = SHIP.get(index);

        if (sh.TORPEDO_PREFIX != prefix)
        {
            sh.TORPEDO_PREFIX = prefix;
            this.markChanged();
        }
    }
}