/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: ShipTech.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.edit.res.stbof.sst;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;

/**Represents shiptech.sst.*/
public class ShipTech extends InternalFile{
	private byte[][] TECH_LVL;
	private int[] RACE;
	private short[] PARENT_ID;
	
	public ShipTech(InputStream in) throws Exception{
		byte[] b = new byte[2];
		TECH_LVL = new byte[6][in.available()/16];
		RACE = new int[in.available()/16];
		PARENT_ID = new short[in.available()/16];
		int rac;
		short i;
		while (in.available()>0){
			//race 2
			b = new byte[4];
			in.read(b);
			rac = Tools.toInt(b, true);
			//id 4
			b = new byte[2];
			in.read(b);
			i = Tools.toShort(b, true);
			//set race
			RACE[i] = rac;
			//upgrade 6
			in.read(b);
			PARENT_ID[i] = Tools.toShort(b, true);
			//req tech lvls 12
			for (int j=0; j<6; j++){
				TECH_LVL[j][i] = (byte)in.read();
			}
			//ship 2
			in.skip(2);
		}
	}
	
	/**Saves changes.
	@return true if save has been successful.
	@throws Exception the function may throw an exception if an error occures.*/
	@Override
	public void save(OutputStream out) throws Exception{
		for (short i=0; i<RACE.length; i++){
			//race 4
			out.write(Tools.toByte(RACE[i], true));
			//id 6
			out.write(Tools.toByte(i, true));
			//upgrade 8
			out.write(Tools.toByte(PARENT_ID[i], true));
			//req tech lvls 14
			for (int j=0; j<6; j++){
				out.write(TECH_LVL[j][i]);
			}
			//ship 16
			out.write(new byte[2]);
		}
	}
	/**This function checks the file for known problems/errors that may occur while editing the file.
	@return a Vector object containing results of the check as Strings.*/
	@Override
	public Vector<String> check(){
		Vector<String> response = new Vector<String>();
		String msg;
		for (int i=0; i<RACE.length; i++){
			if (RACE[i]<0 || RACE[i]>36){
				msg = Language.getString("ShipTech.0"); //$NON-NLS-1$
				msg = msg.replace("%1", getName()); //$NON-NLS-1$
				msg = msg.replace("%2", Integer.toString(RACE[i])); //$NON-NLS-1$
				msg = msg.replace("%3", Integer.toString(i)); //$NON-NLS-1$
				response.add(msg);
			}
			for (int j=0; j<6; j++){
				if (TECH_LVL[j][i]<0){
					msg = Language.getString("ShipTech.1"); //$NON-NLS-1$
					msg = msg.replace("%1", getName()); //$NON-NLS-1$
					msg = msg.replace("%2", Integer.toString(TECH_LVL[j][i])); //$NON-NLS-1$
					msg = msg.replace("%3", Integer.toString(i)); //$NON-NLS-1$
					response.add(msg);
				}
			}
			if (PARENT_ID[i]==i){
				msg = Language.getString("ShipTech.2"); //$NON-NLS-1$
				msg = msg.replace("%1", getName()); //$NON-NLS-1$
				msg = msg.replace("%2", Integer.toString(PARENT_ID[i])); //$NON-NLS-1$
				msg = msg.replace("%3", Integer.toString(i)); //$NON-NLS-1$
				response.add(msg);
			}
			if (PARENT_ID[i]>=0){
				for (int j=0; j<6; j++){
					if (TECH_LVL[j][i]<TECH_LVL[j][PARENT_ID[i]]){
						msg = Language.getString("ShipTech.3"); //$NON-NLS-1$
						msg = msg.replace("%1", getName()); //$NON-NLS-1$
						msg = msg.replace("%2", Integer.toString(i)); //$NON-NLS-1$
						response.add(msg);
					}
				}
				if (RACE[i]!=RACE[PARENT_ID[i]]){
					msg = Language.getString("ShipTech.4"); //$NON-NLS-1$
					msg = msg.replace("%1", getName()); //$NON-NLS-1$
					msg = msg.replace("%2", Integer.toString(RACE[i])); //$NON-NLS-1$
					msg = msg.replace("%3", Integer.toString(PARENT_ID[i])); //$NON-NLS-1$
					response.add(msg);
				}
			}
		}
		return response;
	}
	/**Used to get the uncompressed size of the file.**/
	@Override
	public long getSize(){
		long size = RACE.length*16;
		return size;
	}
	/**Reorders data.*/
	public void reOrder(int[] id) throws InvalidArgumentsException{
		if (id.length!=RACE.length){
			throw new InvalidArgumentsException(Language.getString("ShipTech.5")); //$NON-NLS-1$
		}
		byte[][] tech = new byte[6][id.length];
		int[] race = new int[id.length];
		short[] parent = new short[id.length];
		
		for (int i=0; i<id.length; i++){//for each entry
			if (i != id[i])
                        {
			    this.markChanged();
                        }
                        
			for (int j=0; j<6; j++){//switch each level requirement
				tech[j][i] = TECH_LVL[j][id[i]];
			}
			race[i] = RACE[id[i]];//switch owner
			short u = PARENT_ID[id[i]];//get previous parent
			if (u<0){
				parent[i] = -1;
			}else{
				short gum = -1;
				for (short hi=0; hi<id.length &&gum<0; hi++){
					if (id[hi]==u){
						gum = hi;
					}
				}
				parent[i] = gum;
			}
		}
		
		TECH_LVL = tech;
		RACE = race;
		PARENT_ID = parent;
	}
	/**Sets race*/
	public void setRace(int index, int race) throws InvalidArgumentsException{
		checkRaceIndex(index);
		if (RACE[index]==race) {
			return;
		}
		RACE[index] = race;
                this.markChanged();
	}
	/**Returns race*/
	public int getRace(int index) throws InvalidArgumentsException{
		checkRaceIndex(index);
		return RACE[index];
	}
	/**Returns parent id*/
	public int getParent(int id) throws InvalidArgumentsException{
		checkParentIndex(id);
		return PARENT_ID[id];
	}
	/**Sets parent*/
	public void setParent(int id, short par) throws InvalidArgumentsException{
		checkParentIndex(id);
		if (PARENT_ID[id]==par) {
			return;
		}
		PARENT_ID[id] = par;
                this.markChanged();
	}
	/**Returns tech levels*/
	public byte[] getTechLvls(int id) throws InvalidArgumentsException{
		checkParentIndex(id);
		byte[] ret = new byte[6];
		for (int i=0; i<6; i++){
			ret[i] = TECH_LVL[i][id];
		}
		return ret;
	}
	/**Sets tech levels*/
	public void setTechLvls(int id, byte[] par) throws InvalidArgumentsException{
		checkParentIndex(id);
		for (int i=0; i<6; i++){
			if (TECH_LVL[i][id]!=par[i]){
				TECH_LVL[i][id]=par[i];
                                this.markChanged();
			}
		}
	}

	/**Inserts new slot.*/
	public void insert(int id) throws InvalidArgumentsException{
		checkParentIndex(id);

		byte[][] tech = new byte[6][PARENT_ID.length + 1];
		int[] race = new int[PARENT_ID.length + 1];
		short[] parent = new short[PARENT_ID.length + 1];
		
		for (int i=0; i<id; i++){ // copy entries from 0 to not including id
			for (int j=0; j<6; j++){
				tech[j][i] = TECH_LVL[j][i];
			}
			race[i] = RACE[i];
			if (PARENT_ID[i] < id) {
				parent[i] = PARENT_ID[i];
			} else {
				parent[i] = (short)(PARENT_ID[i] + 1);
			}
        }
		for (int i=id; i<PARENT_ID.length; i++){ // copy entries from id to end
			for (int j=0; j<6; j++){
				tech[j][i+1] = TECH_LVL[j][i];
			}
			race[i+1] = RACE[i];
			if (PARENT_ID[i] < id) {
				parent[i+1] = PARENT_ID[i];
			} else {
				parent[i+1] = (short)(PARENT_ID[i] + 1);
			}
        }
        // the new one
    	for (int j=0; j<6; j++){
			tech[j][id] = TECH_LVL[j][id];
		}
		race[id] = RACE[id];
		parent[id] = PARENT_ID[id];
		parent[id + 1] = -1; 
		
		TECH_LVL = tech;
		RACE = race;
		PARENT_ID = parent;
                this.markChanged();
    }
	
	/**Removes slot.*/
	public void remove(int id) throws InvalidArgumentsException{
		checkParentIndex(id);

		byte[][] tech = new byte[6][PARENT_ID.length - 1];
		int[] race = new int[PARENT_ID.length - 1];
		short[] parent = new short[PARENT_ID.length - 1];
		
		for (int i=0; i<id; i++){
			for (int j=0; j<6; j++){
				tech[j][i] = TECH_LVL[j][i];
			}
			race[i] = RACE[i];
			if (PARENT_ID[i] < id) {
				parent[i] = PARENT_ID[i];
			} else {
				parent[i] = (short)(PARENT_ID[i] - 1);
			}
        }
		
		for (int i= id + 1; i< PARENT_ID.length; i++){
			for (int j=0; j<6; j++){
				tech[j][i-1] = TECH_LVL[j][i];
			}
			race[i-1] = RACE[i];
			if (PARENT_ID[i] < id) {
				parent[i-1] = PARENT_ID[i];
			} else {
				parent[i-1] = (short)(PARENT_ID[i] - 1);
			}
        }

		TECH_LVL = tech;
		RACE = race;
		PARENT_ID = parent;
                this.markChanged();
    }
	
	private void checkRaceIndex(int index) throws InvalidArgumentsException{
		if (index<0 || index>=RACE.length){
			String msg = Language.getString("ShipTech.6"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
	}

	private void checkParentIndex(int index) throws InvalidArgumentsException{
		if (index<0 || index>=PARENT_ID.length){
			String msg = Language.getString("ShipTech.7"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
	}

    public int getMaxTechLevel()
    {
        int max = 0;

        for (int i = 0; i < TECH_LVL.length; i++)
        {
            for (int j = 0; j < TECH_LVL[i].length; j++)
            {
                if (TECH_LVL[i][j] > max)
                {
                    max = TECH_LVL[i][j];
                }
            }
        }


        return max;
    }
}
