/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: TextDataBase.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.edit.res.stbof.tdb;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;
import ue.util.RandomReader;

public class TextDataBase extends InternalFile{
	private byte[] UNKNOWN_1 = new byte[4];
	private ArrayList<TdbEntry> MAP = new ArrayList<TdbEntry>();

        /**Constructor.*/
	public TextDataBase(String name, InputStream in) throws IOException{
		//number of entries
		byte[] b = new byte[4];
		in.read(b);
		int num = Tools.toInt(b, true);
		//unknown
		in.read(UNKNOWN_1);
		//largest string size
		in.read(b);
		byte[] lar = new byte[Tools.toInt(b, true)];
		in.skip(4);
		//read map
		b = new byte[16];
		for (int i=0; i<num; i++){
			in.read(b);
			MAP.add(new TdbEntry(b));
		}
		//read strings
		RandomReader rand = new RandomReader(in);
		TdbEntry te;
		String str;
		for (int i=0; i<num; i++){
			te = MAP.get(i);
			rand.seek(te.START_POS);
			rand.read(lar);
			str = Tools.toASCII(lar);
			te.STR = str.substring(1, str.length()-1);
		}
                this.setName(name);
	}
	/**Returns string.*/
	public String getString(int index) throws InvalidArgumentsException{
		checkIndex(index);
		TdbEntry t = MAP.get(index);
		return new String(t.STR);
	}
	/**Sets a string.*/
	public boolean setString(int index, String newString) throws InvalidArgumentsException{
		checkIndex(index);
		TdbEntry tdb = MAP.get(index);
		String h = new String(tdb.STR);
		if (h.equals(newString)) {
			return false;
		}
		tdb.STR = new String(newString);
		tdb.LENGTH = newString.length() + 3;
                this.markChanged();
		return true;
	}
	
	/*Specified by StbofEdit*/
	@Override
	public Vector<String> check(){
		Vector<String> response = new Vector<String>();
		return response;
	}
	@Override
	public long getSize(){
		long size = 16 + MAP.size()*19;
		TdbEntry t;
		for (int i=0; i<MAP.size(); i++){
			t = MAP.get(i);
			size+=t.STR.length();
		}
		return size;
	}

	@Override
	public void save(java.io.OutputStream out) throws IOException{
		//number
		out.write(Tools.toByte(MAP.size(), true));
		//unknown
		out.write(UNKNOWN_1);
		//largest string size and clean up
		long u = 0;
		TdbEntry t;
		for (int i=0; i<MAP.size(); i++){
			t = MAP.get(i);
			t.LENGTH = t.STR.length() + 3;
			if (t.LENGTH>u) {
				u=t.LENGTH;
			}
		}
		out.write(Tools.toByte(u, true));
		u = 0; //address
		//map
		for (int i=0; i<MAP.size(); i++){
			t=MAP.get(i);
			t.START_POS = (int)u;
			u += t.LENGTH;
			t.dump(out);
		}
		//strings
		for (int i=0; i<MAP.size(); i++){
			t=MAP.get(i);
			out.write(32);
			out.write(t.STR.getBytes("ISO-8859-1")); //$NON-NLS-1$
			out.write(10);
			out.write(0);
		}
	}
	
	public String[] getShortStrings(){
		String[] ret = new String[MAP.size()];
		String str;
		TdbEntry t;
		for (int i=0; i<MAP.size(); i++){
			t = MAP.get(i);
			str = new String(t.STR);
			if (str.length()<20){
				ret[i] = str;
			}else{
				ret[i] = str.substring(0, 17) + "..."; //$NON-NLS-1$
			}
		}
		return ret;
	}
	public String[] getFullStrings(){
		String[] ret = new String[MAP.size()];
		TdbEntry t;
		for (int i=0; i<MAP.size(); i++){
			t = MAP.get(i);
			ret[i] = new String(t.STR);
		}
		return ret;
	}
	
	private void checkIndex(int index) throws InvalidArgumentsException{
		if (index<0 || index>=MAP.size()){
			String msg = Language.getString("TextDataBase.0"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
	}
}
