/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: RaceTech.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
*/

package ue.edit.res.stbof.tec;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.edit.res.stbof.bst.Edifice;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;

public class RaceTech extends InternalFile
{
    Edifice edifice;
	ArrayList<TechEntry> ENTRY = new ArrayList<TechEntry>();
	
	public RaceTech(InputStream in, Edifice edifice) throws Exception{
        this.edifice = edifice;

		byte[] b = new byte[4];
		in.read(b);
		int end = Tools.toInt(b, true);
		in.skip(end - 4);

		while (in.available() > 0)
        {
			ENTRY.add(new TechEntry(in));
		}
	}
	
	/**Saves changes.
	@return true if save has been successful.
	@throws Exception the function may throw an exception if an error occures.*/
	@Override
	public void save(OutputStream out) throws Exception
    {
        // disable non-existing builings
        for (int entry = 0; entry < ENTRY.size(); entry++)
        {
            for (int id = edifice.getNumberOfEntries(); id < 384; id++)
            {
                this.setAvailable(entry, id, false);
            }
        }

		int add = ENTRY.size() * 4;

		for (int i = 0; i < ENTRY.size(); i++)
        {
			out.write(Tools.toByte(add, true));
			add += 108;
		}
		for (int i = 0; i < ENTRY.size(); i++)
        {
			TechEntry tech = ENTRY.get(i);
			tech.save(out);
		}
    }

	/**This function checks the file for known problems/errors that may occur while editing the file.
	@return a Vector object containing results of the check as Strings.*/
	@Override
	public Vector<String> check()
    {
		Vector<String> response = new Vector<String>();
		Vector<String> temp = new Vector<String>();
		for (int i=0; i<ENTRY.size(); i++){
			try{
				TechEntry test = ENTRY.get(i);
				temp = test.check(this);
				if (!temp.isEmpty()) {
					response.addAll(temp);
				}
			} catch (Exception f){
				String msg = Language.getString("RaceTech.0"); //$NON-NLS-1$
				msg = msg.replace("%1", f.getMessage()); //$NON-NLS-1$
				response.add(msg);
			}
		}
		return response;
	}
	
	/**Used to get the uncompressed size of the file.**/
	@Override
	public long getSize()
    {
		long size = ENTRY.size()*112;
		return size;
	}

	public short getTechLvl(int index, int tech) throws InvalidArgumentsException{
		checkIndex(index);
		checkTechType(tech);
		TechEntry tec = ENTRY.get(index);
		return tec.TECH_LEVEL[tech];
	}
	public short getRaceID(int index) throws InvalidArgumentsException{
		checkIndex(index);
		TechEntry tec = ENTRY.get(index);
		return tec.RACE;
	}
	public int getTechPnt(int index, int tech) throws InvalidArgumentsException{
		checkIndex(index);
		checkTechType(tech);
		TechEntry tec = ENTRY.get(index);
		return tec.TECH_POINTS[tech];
	}
	public void setTechLvl(int index, int tech, short lvl) throws InvalidArgumentsException{
		checkIndex(index);
		checkTechType(tech);
		TechEntry tec = ENTRY.get(index);
		
        if (tec.TECH_LEVEL[tech] == lvl)
        {
			return;
		}
		tec.TECH_LEVEL[tech]=lvl;
        this.markChanged();
	}
	public void setTechPnt(int index, int tech, int pnt) throws InvalidArgumentsException{
		checkIndex(index);
		checkTechType(tech);
		TechEntry tec = ENTRY.get(index);
		if (tec.TECH_POINTS[tech]==pnt) {
			return;
		}
		tec.TECH_POINTS[tech]=pnt;
        this.markChanged();
	}
	
    public short[] getAll()
    {
        short[] res = new short[ENTRY.size()];
        for (int i=0; i<ENTRY.size(); i++)
        {
            TechEntry tec = ENTRY.get(i);
            res[i] = tec.RACE;
        }
        return res;
	}
        
    /**
     * @return number of entries in this file
     */
    public int getNumberOfEntries()
    {
        return ENTRY.size();
    }

	public boolean isAvailable(int index, int id) throws InvalidArgumentsException{
		checkIndex(index);
		TechEntry tec = ENTRY.get(index);
		int i=0;
		
        while ( id > 63)
        {
			i++;
			id -= 64;
		}

        long lng = 1;
        lng = lng << id;

		long hump = tec.BUILDINGS[i];
		
        return (hump&lng)==lng;
	}

    public void setAvailable(int index, int id, boolean value) throws InvalidArgumentsException{
		checkIndex(index);
		TechEntry tec = ENTRY.get(index);
		int i = 0;

		while (id > 63)
        {
			i++;
			id = id - 64;
		}

        long lng = 1;
        lng = lng << id;
        
		long hump = tec.BUILDINGS[i];
		if (value)
        {
			if ((hump&lng)==lng)
            {
				return;
			}
			tec.BUILDINGS[i] = hump | lng; // enable
            this.markChanged();
		}
        else
        {
			if ((hump&lng)==lng)
            {
				tec.BUILDINGS[i] = hump^lng; // disable
                this.markChanged();
			}
		}
	}
	
    private void checkTechType(int tech) throws InvalidArgumentsException{
		if (tech<0 || tech>6){
			String msg = Language.getString("RaceTech.1"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(tech)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
    }
    
    private void checkIndex(int index) throws InvalidArgumentsException{
		if (index<0 || index>=ENTRY.size()){
			String msg = Language.getString("RaceTech.2"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
    }
}


