/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: TecField.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.10.2007
*/

package ue.edit.res.stbof.tec;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;
import ue.util.ID;

/**This class is a representation of the tecfield.tec file. It holds the tech fields of the game.*/
public class TecField extends InternalFile{
	//fields
	private ArrayList<ID> ENTRY = new ArrayList<ID>();

	@Override
	public Vector<String> check(){
		Vector<String> response = new Vector<String>();
		String msg;
		if (ENTRY.size()==0){
			msg = Language.getString("TecField.0"); //$NON-NLS-1$
			msg = msg.replace("%1", getName()); //$NON-NLS-1$
			response.add(msg);
		}
		ID tec;
		for (int i=0; i<ENTRY.size(); i++){
			tec = ENTRY.get(i);
			if ((tec.toString()).length()>39){
				msg = Language.getString("TecField.1"); //$NON-NLS-1$
				msg = msg.replace("%1", getName()); //$NON-NLS-1$
				msg = msg.replace("%2", tec.toString()); //$NON-NLS-1$
				response.add(msg);
			}
		}
		return response;
	}
	@Override
	public long getSize(){
		long size = ENTRY.size()*41 + 2;
		return size;
	}

	@Override
	public void save(OutputStream out) throws IOException{
		short size = (short)ENTRY.size();
		byte[] b = Tools.toByte(size, true);
		byte[] a = new byte[1];
		out.write(b);
		ID se;
		for (byte i=0; i<size; i++){
			se = ENTRY.get(i);
			//string
			b = Tools.toByte(se.toString(), 40, 39);
			out.write(b);
			//index
			a[0] = i;
			out.write(a);
		}
	}
	/**Constructor
	@param in the InputStream to read from*/
	public TecField(InputStream in) throws IOException{
		ID se;
		in.skip(2);
		byte[] b = new byte[40];
		int i=0;
		String nam = null;
		while (in.available()>0){
			in.read(b);
			nam = Tools.toASCII(b);
			in.skip(1);
			se = new ID(nam, i);
			ENTRY.add(se);
			i++;
		}
	}
	//gets
	/**Return a list of all tech fields.*/
	public String[] getFields(){
		int s = ENTRY.size();
		String[] ret = new String[s];
		ID d;
		for (int i = 0; i < s; i++){
			d = ENTRY.get(i);
			ret[i] = d.toString();
		}
		return ret;
	}
        
    /**
     * @param index
     * @return
     */
    public String getField(int index) throws InvalidArgumentsException
    {
        if (ENTRY.size() <= index)
        {
            throw new InvalidArgumentsException(Language.getString("TecField.2").replace("%1", Integer.toString(index))); //$NON-NLS-1$ //$NON-NLS-2$
        }
        
        return ENTRY.get(index).toString();
    }
    /**
     * @param index
     * @param name
     */
    public void setField(int index, String name) throws InvalidArgumentsException
    {
        if (ENTRY.size() <= index)
        {
            throw new InvalidArgumentsException(Language.getString("TecField.2").replace("%1", Integer.toString(index))); //$NON-NLS-1$ //$NON-NLS-2$
        }
        ENTRY.set(index, new ID(name, index));
        this.markChanged();
    }

    public int getNumberOfFields()
    {
        return ENTRY.size();
    }
}

