/**
 * Ultimate Editor
 * Copyright (C) 2005-2011 Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: TechEntry.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 13.2.2011
 */

package ue.edit.res.stbof.tec;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;
import ue.Language;
import ue.edit.InternalFile;
import ue.util.Tools;

public class TechEntry
{

    public short RACE;
    private byte[] UNKNOWN_1 = new byte[2];
    //biotech, computer, construction, energy, propulsion, weapons, sociology[not used]
    public short[] TECH_LEVEL = new short[7];
    private byte[] UNKNOWN_2 = new byte[2];
    private byte[] UNKNOWN_3 = new byte[2];
    private byte[] UNKNOWN_4 = new byte[2];
    private byte[] UNKNOWN_5 = new byte[2];
    private byte[] UNKNOWN_6 = new byte[2];
    private byte[] UNKNOWN_7 = new byte[2];
    private byte[] UNKNOWN_8 = new byte[2];
    public int[] TECH_POINTS = new int[7];
    public long[] BUILDINGS = new long[6];

    public Vector<String> check(InternalFile parent)
    {
        Vector<String> response = new Vector<String>();
        String msg;
        
        if (RACE < 0 || RACE > 34)
        {
            msg = Language.getString("TechEntry.1");
            msg = msg.replace("%1", Integer.toString(RACE));
            response.add(parent.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg));
        }
        for (int i = 0; i < 7; i++)
        {
            if (TECH_LEVEL[i] < 0) {
                msg = Language.getString("TechEntry.2");
                msg = msg.replace("%1", Integer.toString(RACE));
                msg = msg.replace("%2", Integer.toString(TECH_LEVEL[i]));
                response.add(parent.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg));
            }
            if (TECH_LEVEL[i] == 0 && RACE < 5) {
                msg = Language.getString("TechEntry.3");
                msg = msg.replace("%1", Integer.toString(RACE));
                response.add(parent.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg));
            }
            if (TECH_POINTS[i] < 0) {
                msg = Language.getString("TechEntry.4");
                msg = msg.replace("%1", Integer.toString(RACE));
                response.add(parent.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg));
            }
        }
        return response;
    }

    public TechEntry(InputStream in) throws Exception
    {
        //race 0-2
        byte[] b = new byte[2];
        in.read(b);
        RACE = Tools.toShort(b, true);
        //unknown 2-2
        in.read(UNKNOWN_1);
        //System.out.println(RACE + ":" + Tools.toHex(UNKNOWN_1));
        //tech era/unknown/points 4-8*7=56
        for (int i = 0; i < 7; i++)
        {
            b = new byte[2];
            in.read(b);
            TECH_LEVEL[i] = Tools.toShort(b, true);

            switch (i)
            {
                case 0:
                {
                    in.read(UNKNOWN_2);
                    //System.out.println(RACE + ":" + i + ":" + Tools.toHex(UNKNOWN_2));
                    break;
                }
                case 1:
                {
                    in.read(UNKNOWN_3);
                    //System.out.println(RACE + ":" + i + ":" + Tools.toHex(UNKNOWN_3));
                    break;
                }
                case 2:
                {
                    in.read(UNKNOWN_4);
//                                                if (RACE < 5) System.out.println("3:" + Tools.toHex(UNKNOWN_4));
                    break;
                }
                case 3:
                {
                    in.read(UNKNOWN_5);
//                                                if (RACE < 5) System.out.println("4:" + Tools.toHex(UNKNOWN_5));
                    break;
                }
                case 4:
                {
                    in.read(UNKNOWN_6);
//                                                if (RACE < 5) System.out.println("5:" + Tools.toHex(UNKNOWN_6));
                    break;
                }
                case 5:
                {
                    in.read(UNKNOWN_7);
//                                                if (RACE < 5) System.out.println("6:" + Tools.toHex(UNKNOWN_7));
                    break;
                }
                case 6:
                {
                    in.read(UNKNOWN_8);
//                                                if (RACE < 5) System.out.println("7:" + Tools.toHex(UNKNOWN_8));
                    break;
                }
            }
            b = new byte[4];
            in.read(b);
            TECH_POINTS[i] = Tools.toInt(b, true);
        }
        //buildings masks 60-6*8=48
        b = new byte[8];
        for (int i = 0; i < 6; i++)
        {
            in.read(b);
            BUILDINGS[i] = Tools.toLong(b, true);
        }
    }

    public void save(OutputStream out) throws Exception {
        //race 0-2
        out.write(Tools.toByte(RACE, true));
        //unknown 2-2
        out.write(UNKNOWN_1);
        //tech era/unknown/points 4-8*7=56
        for (int i = 0; i < 7; i++)
        {
            out.write(Tools.toByte(TECH_LEVEL[i], true));
            switch (i)
            {
                case 0:
                {
                    out.write(UNKNOWN_2);
                    break;
                }
                case 1:
                {
                    out.write(UNKNOWN_3);
                    break;
                }
                case 2:
                {
                    out.write(UNKNOWN_4);
                    break;
                }
                case 3:
                {
                    out.write(UNKNOWN_5);
                    break;
                }
                case 4:
                {
                    out.write(UNKNOWN_6);
                    break;
                }
                case 5:
                {
                    out.write(UNKNOWN_7);
                    break;
                }
                case 6:
                {
                    out.write(UNKNOWN_8);
                    break;
                }
            }
            out.write(Tools.toByte(TECH_POINTS[i], true));
        }
        //buildings masks 60-6*8=48
        for (int i = 0; i < 6; i++)
        {
            out.write(Tools.toByte(BUILDINGS[i], true));
        }
    }
}
