/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: TechTree.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.10.2007
*/

package ue.edit.res.stbof.tec;

import java.awt.Dimension;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.exception.InvalidArgumentsException;
import ue.util.Pair;
import ue.util.Tools;

/**This class is a representation of the techtree.tec file. It holds the tech tree of the game.*/
public class TechTree extends InternalFile{
	//fields
	private ArrayList<TechEntry> ENTRY = new ArrayList<TechEntry>();

	@Override
	public Vector<String> check(){
		Vector<String> response = new Vector<String>();
		String msg;
		try{
			if (ENTRY.size()==0){
				msg = Language.getString("TechTree.0"); //$NON-NLS-1$
				msg = msg.replace("%1", getName()); //$NON-NLS-1$
				response.add(msg);
			}
			int size = ENTRY.size();
			TechEntry se;
			HashSet<String> targ = new HashSet<String>();
			for (short i=0; i<size; i++){
				se = ENTRY.get(i);
				if (se.FIELD<0 || se.FIELD>7){
					msg = Language.getString("TechTree.1"); //$NON-NLS-1$
					msg = msg.replace("%1", getName()); //$NON-NLS-1$
					msg = msg.replace("%2", se.NAME); //$NON-NLS-1$
					msg = msg.replace("%3", Integer.toString(se.FIELD)); //$NON-NLS-1$
					response.add(msg);
				}
				if (se.LEVEL<0 || se.LEVEL>10){
					msg = Language.getString("TechTree.2"); //$NON-NLS-1$
					msg = msg.replace("%1", getName()); //$NON-NLS-1$
					msg = msg.replace("%2", se.NAME); //$NON-NLS-1$
					msg = msg.replace("%3", Integer.toString(se.LEVEL)); //$NON-NLS-1$
					response.add(msg);
				}
				if (se.COST<0){
					msg = Language.getString("TechTree.3"); //$NON-NLS-1$
					msg = msg.replace("%1", getName()); //$NON-NLS-1$
					msg = msg.replace("%2", se.NAME); //$NON-NLS-1$
					msg = msg.replace("%3", Long.toString(se.COST)); //$NON-NLS-1$
					response.add(msg);
				}
				if (se.DESCRIPTION<0){
					msg = Language.getString("TechTree.4"); //$NON-NLS-1$
					msg = msg.replace("%1", getName()); //$NON-NLS-1$
					msg = msg.replace("%2", se.NAME); //$NON-NLS-1$
					msg = msg.replace("%3", Long.toString(se.DESCRIPTION)); //$NON-NLS-1$
					response.add(msg);
				}
				if (se.NAME.length()>39){
					msg = Language.getString("TechTree.5"); //$NON-NLS-1$
					msg = msg.replace("%1", getName()); //$NON-NLS-1$
					msg = msg.replace("%2", se.NAME); //$NON-NLS-1$
					response.add(msg);
				}
				targ.add(se.IMAGE);
			}
		}catch (Exception ze){
			ze.printStackTrace();
			response.add(getGenericErrorString(ze.getMessage()));
		}
		return response;
	}
        
	public ArrayList<Pair<String, Dimension>> getImagesListedInThisFile()
	{
	    ArrayList<Pair<String, Dimension>> images = new ArrayList<Pair<String, Dimension>>();
	    
            Dimension size1 = new Dimension(120, 100);
            Dimension size2 = new Dimension(60, 50);
            Dimension size3 = new Dimension(270, 225);
	    
	    for (TechEntry test : ENTRY)
	    {
	        images.add(new Pair<String, Dimension>(test.IMAGE + ".tga", size1));
                images.add(new Pair<String, Dimension>(test.IMAGE + "_60.tga", size2));
                images.add(new Pair<String, Dimension>(test.IMAGE + "_b.tga", size3));
	    }
	    
	    return images;
	}
            
	@Override
	public long getSize(){
		long size = ENTRY.size()*72;
		return size;
	}

	@Override
	public void save(OutputStream out) throws IOException{
		int size = ENTRY.size();
		TechEntry se;
		for (short i=0; i<size; i++){
			se = ENTRY.get(i);
			se.INDEX=i;
			se.save(out);
		}
	}
	/**Constructor
	@param in the InputStream to read from*/
	public TechTree(InputStream in) throws IOException{
		TechEntry se;
		while (in.available() >0){
			se = new TechEntry(in);
			ENTRY.add(se);
		}
                sort();
	}
	//gets
        public int getMaxTechLevel(byte field)
        {
            int ret = 0;
            TechEntry te;

            for (int i = 0; i < ENTRY.size(); i++)
            {
                te = ENTRY.get(i);

                if (te.FIELD == field && ret < te.LEVEL)
                {
                    ret = te.LEVEL;
                }
            }

            return ret;
        }

	/**Returns a list of techs of the specified field.
	@param field the tech field index*/
	public String[] getTechList(byte field)
        {
		String[] ret = new String[0];
		ArrayList<String> areta = new ArrayList<String>();
		int s = ENTRY.size();
		TechEntry te;
		int au=0;
		for (int i=0; i<s; i++){
			te = ENTRY.get(i);
			if (te.FIELD==field){
				areta.add(new String(te.NAME));
				te.LEVEL=(byte)au++;
			}
		}
		ret = areta.toArray(ret);
		return ret;
	}
	/**Returns the number of techs
	*/
	public int getNumberOfEntries()
        {
            return ENTRY.size();
	}

	/**Return the cost of a tech.
	@param field the tech field
	@param level the level of the tech*/
	public int getCost(byte field, byte level) throws InvalidArgumentsException{
		int s = ENTRY.size();
		TechEntry te;
		int a = -1;
		for (int i=0; i<s && a==-1; i++){
			te = ENTRY.get(i);
			if (te.FIELD==field && te.LEVEL==level) {
				a = (int)te.COST;
			}
		}
		if (a<0) {
			throw new InvalidArgumentsException(getIndexNotFoundString(field, level));
		}
		return a;
	}
	/**Return the index of a tech.
	@param field the tech field
	@param level the level of the tech*/
	public int getIndex(byte field, byte level) throws InvalidArgumentsException{
		int s = ENTRY.size();
		TechEntry te;
		int a = -1;
		for (int i=0; i<s && a==-1; i++){
			te = ENTRY.get(i);
			if (te.FIELD==field && te.LEVEL==level) {
				a = i;
			}
		}
		if (a<0) {
			throw new InvalidArgumentsException(getIndexNotFoundString(field, level));
		}
		return a;
	}
	/**Returns the image of a tech.
	@param field the tech field
	@param level the level of the tech*/
	public String getImage(byte field, byte level) throws InvalidArgumentsException{
		int s = ENTRY.size();
		TechEntry te;
		String a = null;
		for (int i=0; i<s && a==null; i++){
			te = ENTRY.get(i);
			if (te.FIELD==field && te.LEVEL==level) {
				a = new String(te.IMAGE);
			}
		}
		if (a==null) {
			throw new InvalidArgumentsException(getIndexNotFoundString(field, level));
		}
		return a;
	}
	/**Returns the name of a tech.
	@param field the tech field
	@param level the level of the tech*/
	public String getName(byte field, byte level) throws InvalidArgumentsException{
		int s = ENTRY.size();
		TechEntry te;
		String a = null;
		for (int i=0; i<s && a==null; i++){
			te = ENTRY.get(i);
			if (te.FIELD==field && te.LEVEL==level) {
				a = new String(te.NAME);
			}
		}
		if (a==null) {
			throw new InvalidArgumentsException(getIndexNotFoundString(field, level));
		}
		return a;
	}
	/**Returns the address of the description of a tech.
	@param index the index of the tech*/
	public int getDesc(int index) throws InvalidArgumentsException{
		TechEntry te = ENTRY.get(index);
		if (te==null) {
			throw new InvalidArgumentsException(
					getIndexNotFoundString(index));
		}
		return te.DESCRIPTION;
	}
	/**Returns the address of the description of a tech.
	@param field the tech field
	@param level the level of the tech*/
	public int getDesc(byte field, byte level) throws InvalidArgumentsException{
		int s = ENTRY.size();
		TechEntry te;
		int a = -1;
		for (int i=0; i<s && a==-1; i++){
			te = ENTRY.get(i);
			if (te.FIELD==field && te.LEVEL==level) {
				a = te.DESCRIPTION;
			}
		}
		if (a<0) {
			throw new InvalidArgumentsException(getIndexNotFoundString(field, level));
		}
		return a;
	}
	//sets
	/**Sets the cost of a tech.
	@param field the tech field
	@param cost the tech cost
	@param level the level of the tech*/
	public void setCost(byte field, byte level, int cost) throws InvalidArgumentsException{
		if (cost<0){
			String msg = Language.getString("TechTree.6"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(cost)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
		int s = ENTRY.size();
		TechEntry te;
		boolean a = true;
		for (int i=0; i<s && a; i++){
			te = ENTRY.get(i);
			if (te.FIELD==field && te.LEVEL==level){
				a = false;
				if (te.COST==cost) {
					return;
				}
				te.COST=cost;
                                this.markChanged();
			}
		}
		if (a) {
			throw new InvalidArgumentsException(getIndexNotFoundString(field, level));
		}
	}
	/**Sets the image of a tech.
	@param field the tech field
	@param image the tech image
	@param level the level of the tech*/
	public void setImage(byte field, byte level, String image) throws InvalidArgumentsException{
		int s = ENTRY.size();
		TechEntry te;
		boolean a = true;
		for (int i=0; i<s && a; i++){
			te = ENTRY.get(i);
			if (te.FIELD==field && te.LEVEL==level){
				a = false;
				if (te.IMAGE.equals(image)) {
					return;
				}
				te.IMAGE=new String(image);
                                this.markChanged();
			}
		}
		if (a) {
			throw new InvalidArgumentsException(getIndexNotFoundString(field, level));
		}
	}
	/**Sets the name of a tech.
	@param field the tech field
	@param name the tech name
	@param level the level of the tech*/
	public void setName(byte field, byte level, String name) throws InvalidArgumentsException{
		int s = ENTRY.size();
		TechEntry te;
		boolean a = true;
		for (int i=0; i<s && a; i++){
			te = ENTRY.get(i);
			if (te.FIELD==field && te.LEVEL==level){
				a = false;
				if (te.NAME.equals(name)) {
					return;
				}
				te.NAME=new String(name);
                                this.markChanged();
			}
		}
		if (a) {
			throw new InvalidArgumentsException(getIndexNotFoundString(field, level));
		}
	}
	/**Sets the description address of a tech.
	@param field the tech field
	@param desc the tech description address
	@param level the level of the tech*/
	public void setDesc(byte field, byte level, int desc) throws InvalidArgumentsException{
		if (desc<0){
			String msg = Language.getString("TechTree.7"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(desc)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
		int s = ENTRY.size();
		TechEntry te;
		boolean a = true;
		for (int i=0; i<s && a; i++){
			te = ENTRY.get(i);
			if (te.FIELD==field && te.LEVEL==level){
				a = false;
				if (te.DESCRIPTION==desc) {
					return;
				}
				te.DESCRIPTION=desc;
                                this.markChanged();
			}
		}
		if (a) {
			throw new InvalidArgumentsException(getIndexNotFoundString(field, level));
		}
	}
	/**Sets the description address of a tech.
	@param desc the tech description address
	@param index the index of the tech*/
	public void setDesc(int index, int desc) throws InvalidArgumentsException{
		if (desc<0){
			String msg = Language.getString("TechTree.7"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(desc)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
		TechEntry te = ENTRY.get(index);
		if (te == null) {
			throw new InvalidArgumentsException(
					getIndexNotFoundString(index));
		}
		if (te.DESCRIPTION==desc) {
			return;
		}
		te.DESCRIPTION=desc;
                this.markChanged();
	}

	private String getGenericErrorString(String error){
		String msg = Language.getString("TechTree.8"); //$NON-NLS-1$
		msg = msg.replace("%1", getName()); //$NON-NLS-1$
		msg = msg.replace("%2", error); //$NON-NLS-1$
		return msg;
	}
    
    private String getIndexNotFoundString(int field, int level){
    	String msg = Language.getString("TechTree.13"); //$NON-NLS-1$
		msg = msg.replace("%1", "(" + field + ", " + level + ")"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
    	return msg;
    }
    
    private String getIndexNotFoundString(int index){
    	String msg = Language.getString("TechTree.13"); //$NON-NLS-1$
		msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
    	return msg;
    }

    public void setCost(int index, int cost) throws InvalidArgumentsException
    {
        TechEntry te = ENTRY.get(index);

        if (te == null)
        {
            throw new InvalidArgumentsException(
					getIndexNotFoundString(index));
        }

        if (te.COST != cost)
        {
            te.COST = cost;
            this.markChanged();
        }
    }

    public int getCost(int index) throws InvalidArgumentsException
    {
        TechEntry te = ENTRY.get(index);

        if (te == null)
        {
            throw new InvalidArgumentsException(
					getIndexNotFoundString(index));
        }

        return (int)te.COST;
    }

    public byte getLevel(int index) throws InvalidArgumentsException
    {
        TechEntry te = ENTRY.get(index);

        if (te == null)
        {
            throw new InvalidArgumentsException(
					getIndexNotFoundString(index));
        }

        return te.LEVEL;
    }

    public byte getField(int index) throws InvalidArgumentsException
    {
        TechEntry te = ENTRY.get(index);

        if (te == null)
        {
            throw new InvalidArgumentsException(
					getIndexNotFoundString(index));
        }

        return te.FIELD;
    }

    public int add(String name, byte field)
    {
        TechEntry newTech = null;

        for (TechEntry tech : ENTRY)
        {
            if (tech.FIELD == field &&
                    (newTech == null || tech.LEVEL >= newTech.LEVEL))
            {
                newTech = new TechEntry(tech);

                // fix
                newTech.LEVEL++;
                newTech.NAME = name;
                newTech.COST++;
            }
        }

        if (newTech != null)
        {
            ENTRY.add(newTech);
            sort();

            return newTech.INDEX;
        }
        
        return -1;
    }

    private void sort()
    {
        Collections.sort(ENTRY);

        short index = 0;

        for (TechEntry entry : ENTRY)
        {
            entry.INDEX = index;
            index++;
        }
    }

    public int getMaxTechLevel()
    {
        int ret = 0;
        TechEntry te;

        for (int i = 0; i < ENTRY.size(); i++)
        {
            te = ENTRY.get(i);

            if (ret < te.LEVEL)
            {
                ret = te.LEVEL;
            }
        }

        return ret;
    }

    //class #1
	private class TechEntry implements Comparable<TechEntry>
        {
		public String NAME;
		public short INDEX;
		public String IMAGE;
		public byte FIELD;
		public byte LEVEL;
		private byte[] UNKNOWN_1 = new byte[3];
		public long COST;
		public int DESCRIPTION;
		//constructors
		public TechEntry(InputStream in) throws IOException{
			byte[] b = new byte[40];
			//name 0-40
			in.read(b);
			NAME = Tools.toASCII(b);
			//index 40-2
			b = new byte[2];
			in.read(b);
			INDEX = Tools.toShort(b, true);
			//image 42-13
			b = new byte[13];
			in.read(b);
			IMAGE = Tools.toASCII(b);
			//field & level 55-2
			b = new byte[2];
			in.read(b);
			FIELD = b[0];
			LEVEL = b[1];
			//unknown 57-3
			in.read(UNKNOWN_1);
			//cost 60-8
			b = new byte[8];
			in.read(b);
			COST = Tools.toLong(b, true);
			//vertical 68-4
			b = new byte[4];
			in.read(b);
			DESCRIPTION = Tools.toInt(b, true);
		}
		public TechEntry(TechEntry me){
			NAME = new String(me.NAME);
			INDEX = me.INDEX;
			IMAGE = new String(me.IMAGE);
			FIELD = me.FIELD;
			LEVEL = me.LEVEL;
			COST = me.COST;
			DESCRIPTION = me.DESCRIPTION;
			UNKNOWN_1 = me.UNKNOWN_1;
		}
		//save
		public void save(OutputStream out) throws IOException{
			//name
			byte[] b = Tools.toByte(NAME, 40, 39);
			out.write(b);
			//index
			b = Tools.toByte(INDEX, true);
			out.write(b);
			//image
			b = Tools.toByte(IMAGE, 13, 12);
			out.write(b);
			//field & level
			b = new byte[2];
			b[0] = FIELD;
			b[1] = LEVEL;
			out.write(b);
			//unknown
			out.write(UNKNOWN_1);
			//cost
			b = Tools.toByte(COST, true);
			out.write(b);
			//description
			b = Tools.toByte(DESCRIPTION, true);
			out.write(b);
		}

        @Override
        public int compareTo(TechEntry arg)
        {
            // -1 if before, 0 is same and +1 if after
            return (FIELD * 256 + LEVEL) - (arg.FIELD * 256 + arg.LEVEL);
        }
    }
}

