/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WDFButton.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 30.7.2011
 */

package ue.edit.res.stbof.wdf;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import ue.util.Tools;

/**
 *
 */
public class WDFButton extends WDFWidget
{
    public static final int TYPE_PUSH   = 0;
    public static final int TYPE_TOGGLE = 2;

    protected int unknown;
    protected int buttonType;         // single byte
    protected boolean repeat;         // repeating action
    protected int repeatsPerSecond;   // either RepeatInterval/RepeatsPerSecond/RepeatIncrement
    protected String normalImage;
    protected String selectedImage;

    public WDFButton(InputStream in) throws IOException
    {
        super(WDFWidget.TYPE_BUTTON, in);

        byte[] b4 = new byte[4];

        in.read(b4);
        unknown = Tools.toInt(b4, true);
        buttonType = in.read();
        in.read(b4);
        repeat = Tools.toInt(b4, true) != 0;
        in.read(b4);
        repeatsPerSecond = Tools.toInt(b4, true);
        b4 = new byte[13];
        in.read(b4);
        normalImage = Tools.toASCII(b4);
        normalImage = normalImage.trim().toLowerCase();
        in.read(b4);
        selectedImage = Tools.toASCII(b4);
        selectedImage = selectedImage.trim().toLowerCase();
    }

    @Override
    public void save(OutputStream out) throws IOException
    {
        super.save(out);

        out.write(Tools.toByte(unknown, true)); // 4
        out.write(buttonType);                  // 5
        out.write((repeat) ? 1 : 0);            //
        out.write(0);
        out.write(0);
        out.write(0);                           // 9
        out.write(Tools.toByte(repeatsPerSecond, true)); // 13
        out.write(Tools.toByte(normalImage, 13, 12));
        out.write(Tools.toByte(selectedImage, 13, 12));  // +26 = 39
    }

    @Override
    public long getSize()
    {
        return super.getSize() + 39;
    }

    @Override
    public String toString()
    {
        return "Widget: Type: " + typeID + " Size: " + getSize() + " Files: "
                + this.backgroundImage + ", " + this.normalImage + ", " + this.selectedImage;

    }

    public int getButtonType() {
        return buttonType;
    }

    public boolean setButtonType(int buttonType) {
        if (this.buttonType != buttonType)
        {
            this.buttonType = buttonType;
            return true;
        }
        return false;
    }

    public String getNormalImage() {
        return normalImage;
    }

    public boolean setNormalImage(String normalImage) {
        if (!this.normalImage.equalsIgnoreCase(normalImage))
        {
            this.normalImage = normalImage;
            return true;
        }
        return false;
    }

    public boolean isRepeatingClicks() {
        return repeat;
    }

    public boolean setRepeatingClicks(boolean repeat) {
        if (this.repeat = repeat)
        {
            this.repeat = repeat;
            return true;
        }
        return false;
    }

    public int getRepeatsPerSecond() {
        return repeatsPerSecond;
    }

    public boolean setRepeatsPerSecond(int repeatsPerSecond) {
        if (this.repeatsPerSecond != repeatsPerSecond)
        {
            this.repeatsPerSecond = repeatsPerSecond;
            return true;
        }
        return false;
    }

    public String getSelectedImage() {
        return selectedImage;
    }

    public boolean setSelectedImage(String selectedImage) {
        if (!this.selectedImage.equalsIgnoreCase(selectedImage))
        {
            this.selectedImage = selectedImage;
            return true;
        }
        return false;
    }

    public int getUnknown() {
        return unknown;
    }

    public boolean setUnknown(int unknown) {
        if (this.unknown != unknown)
        {
            this.unknown = unknown;
            return true;
        }
        return false;
    }
}
