/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WDFFont.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 31.7.2011
 */

package ue.edit.res.stbof.wdf;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Arrays;
import ue.util.Tools;

/**
 *
 * @author Alan
 */
public class WDFFont
{
    protected int[] rgb;     // 3 bytes
    protected int[] unknown;      // 7 * integer, 28 bytes
    protected boolean overrideExeDefaults;
    protected String fontFile;
    
    public WDFFont(InputStream in) throws IOException
    {
        rgb = new int[3];

        for (int i = 0; i < 3; i++)
        {
            rgb[i] = in.read() & 0xFF;
        }

        unknown = new int[7];
        byte[] b4 = new byte[4];

        for (int i = 0; i < unknown.length; i++)
        {
            in.read(b4);
            unknown[i] = Tools.toInt(b4, true);
        }

        in.read(b4);
        overrideExeDefaults = !(Tools.toInt(b4, true) == 0);
        
        byte[] b = new byte[13];
        in.read(b);
        fontFile = Tools.toASCII(b);
        fontFile = fontFile.trim().toLowerCase();
    }

    public WDFFont(WDFFont font)
    {
        rgb = new int[3];
        unknown = new int[7];

        setValues(font);
    }

    public void setValues(WDFFont font)
    {
        System.arraycopy(font.rgb, 0, rgb, 0, 3);
        System.arraycopy(font.unknown, 0, unknown, 0, 7);

        overrideExeDefaults = font.overrideExeDefaults;
        fontFile = font.fontFile;
    }

    public void save(OutputStream out) throws IOException
    {
        for (int i = 0; i < 3; i++)
        {
            out.write(rgb[i]);
        }

        for (int i = 0; i < unknown.length; i++)
        {
            out.write(Tools.toByte(unknown[i], true));
        }

        out.write(Tools.toByte((overrideExeDefaults) ? 1 : 0, true));

        out.write(Tools.toByte(fontFile, 13, 12));  // 48
    }

    @Override
    public String toString()
    {
        return "Font: " + fontFile;
    }

    public String getFontFile() {
        return fontFile;
    }

    public void setFontFile(String fontFile) {
        this.fontFile = fontFile;
    }

    public int getUnknown(int index) {
        return unknown[index];
    }

    public void setUnknown(int index, int value) {
        this.unknown[index] = value;
    }

    public int getRGBColor() {
        return (rgb[0] << 16) | (rgb[2] << 8) | (rgb[1]);
    }

    public void setRGBColor(int rgb) {
        this.rgb[0] = (rgb >> 16) & 0xFF;
        this.rgb[2] = (rgb >> 8) & 0xFF;
        this.rgb[1] = rgb & 0xFF;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final WDFFont other = (WDFFont) obj;
        if (!Arrays.equals(this.rgb, other.rgb)) {
            return false;
        }
        if (!Arrays.equals(this.unknown, other.unknown)) {
            return false;
        }
        if (this.overrideExeDefaults != other.overrideExeDefaults)
        {
            return false;
        }
        if ((this.fontFile == null) ? (other.fontFile != null) : !this.fontFile.equals(other.fontFile)) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 79 * hash + Arrays.hashCode(this.rgb);
        hash = 79 * hash + Arrays.hashCode(this.unknown);
        hash = 79 * hash + (this.fontFile != null ? this.fontFile.hashCode() : 0);
        return hash;
    }

    public boolean overrideExeDefaults()
    {
        return overrideExeDefaults;
    }

    public boolean setOverrideExeDefaults(boolean overrideExeDefaults) {
        if (this.overrideExeDefaults != overrideExeDefaults)
        {
            this.overrideExeDefaults = overrideExeDefaults;
            return true;
        }
        return false;
    }
}
