/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WDFListBox.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 31.7.2011
 */

package ue.edit.res.stbof.wdf;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import ue.util.Tools;

/**
 *
 */
public class WDFListBox extends WDFWidget
{
    protected int unknown1;
    protected WDFFont font1;
    protected WDFFont font2;
    protected int[] unknown2;  // 11 values

    public WDFListBox(InputStream in) throws IOException
    {
        super(WDFWidget.TYPE_LISTBOX, in);

        byte[] b4 = new byte[4];

        in.read(b4);
        unknown1 = Tools.toInt(b4, true);

        font1 = new WDFFont(in);
        font2 = new WDFFont(in);

        unknown2 = new int[11];

        for (int i = 0; i < unknown2.length; i++)
        {
            in.read(b4);
            unknown2[i] = Tools.toInt(b4, true);
        }
    }

    @Override
    public void save(OutputStream out) throws IOException
    {
        super.save(out);

        out.write(Tools.toByte(unknown1, true)); // 4

        font1.save(out);    // +48 = 52
        font2.save(out);    // +48 = 100

        for (int i = 0; i < unknown2.length; i++)   // + 11*4 = 144
        {
            out.write(Tools.toByte(unknown2[i], true));
        }
    }

    @Override
    public long getSize()
    {
        return super.getSize() + 144;
    }

    @Override
    public String toString()
    {
        return "Widget: Type: " + typeID + " Size: " + getSize() + " Files: "
                + this.backgroundImage + ", " + font1 + ", " + font2;
    }

    public WDFFont getFont1() {
        return new WDFFont(font1);
    }

    public boolean setFont1(WDFFont font1) {
        if (!this.font1.equals(font1))
        {
            this.font1.setValues(font1);
            return true;
        }
        return false;
    }

    public WDFFont getFont2() {
        return new WDFFont(font2);
    }

    public boolean setFont2(WDFFont font2) {
        if (!this.font2.equals(font2))
        {
            this.font2.setValues(font2);
            return true;
        }
        return false;
    }

    public int getUnknown1() {
        return unknown1;
    }

    public boolean setUnknown1(int unknown1) {
        if (this.unknown1 != unknown1)
        {
            this.unknown1 = unknown1;
            return true;
        }
        return false;
    }

    public int getUnknown2(int index) {
        return unknown2[index];
    }

    public boolean setUnknown2(int index, int value) {
        if (this.unknown2[index] != value)
        {
            this.unknown2[index] = value;
            return true;
        }
        return false;
    }
}
