/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WDFMenu.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 31.7.2011
 */

package ue.edit.res.stbof.wdf;

import java.awt.Dimension;
import java.awt.Point;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import ue.util.Tools;

/**
 *
 */
public class WDFMenu extends WDFWidget
{
    protected Point menuPosition;
    protected Dimension menuDimension;

    protected WDFFont font1;
    protected WDFFont font2;
    protected String menuFile;
    protected String leftImage;
    protected String topImage;
    protected String rightImage;
    protected String bottomImage;
    protected String unknownFile;

    public WDFMenu(InputStream in) throws IOException
    {
        super(WDFWidget.TYPE_MENU, in);

        byte[] b4 = new byte[4];
        int xw, yh;

        in.read(b4);
        xw = Tools.toInt(b4, true);
        in.read(b4);
        yh = Tools.toInt(b4, true);

        menuPosition = new Point(xw, yh);

        in.read(b4);
        xw = Tools.toInt(b4, true);
        in.read(b4);
        yh = Tools.toInt(b4, true);

        menuDimension = new Dimension(xw, yh);

        font1 = new WDFFont(in);
        font2 = new WDFFont(in);

        byte[] b13 = new byte[13];

        in.read(b13);
        menuFile = Tools.toASCII(b13);
        menuFile = menuFile.trim().toLowerCase();
        in.read(b13);
        leftImage = Tools.toASCII(b13);
        leftImage = leftImage.trim().toLowerCase();
        in.read(b13);
        topImage = Tools.toASCII(b13);
        topImage = topImage.trim().toLowerCase();
        in.read(b13);
        rightImage = Tools.toASCII(b13);
        rightImage = rightImage.trim().toLowerCase();
        in.read(b13);
        bottomImage = Tools.toASCII(b13);
        bottomImage = bottomImage.trim().toLowerCase();
        in.read(b13);
        unknownFile = Tools.toASCII(b13);
        unknownFile = unknownFile.trim().toLowerCase();
    }

    @Override
    public void save(OutputStream out) throws IOException
    {
        super.save(out);

        out.write(Tools.toByte(menuPosition.x, true));    // 4
        out.write(Tools.toByte(menuPosition.y, true));    // 4
        out.write(Tools.toByte(menuDimension.width, true));    // 4
        out.write(Tools.toByte(menuDimension.height, true));    // 4

        font1.save(out);        // +48 = 64
        font2.save(out);        // +48 = 112

        out.write(Tools.toByte(menuFile, 13, 12));
        out.write(Tools.toByte(leftImage, 13, 12)); // +26 = 138
        out.write(Tools.toByte(topImage, 13, 12));
        out.write(Tools.toByte(rightImage, 13, 12));    // +26 = 164
        out.write(Tools.toByte(bottomImage, 13, 12));
        out.write(Tools.toByte(unknownFile, 13, 12));   // + 26 = 190
    }

    @Override
    public long getSize()
    {
        return super.getSize() + 190;
    }

    @Override
    public String toString()
    {
        return "Widget: Type: " + typeID + " Size: " + getSize() + " Files: "
                + this.backgroundImage + ", " + font1 + ", " + font2 + ", " + menuFile
                + ", " + leftImage + ", " + topImage + ", " + rightImage + ", "
                + bottomImage + ", " + unknownFile;
    }

    public String getBottomImage() {
        return bottomImage;
    }

    public boolean setBottomImage(String bottomImage) {
        if (!this.bottomImage.equals(bottomImage))
        {
            this.bottomImage = bottomImage;
            return true;
        }
        return false;
    }

    public WDFFont getFont1() {
        return new WDFFont(font1);
    }

    public boolean setFont1(WDFFont font1)
    {
        if (!this.font1.equals(font1))
        {
            this.font1.setValues(font1);
            return true;
        }
        return false;
    }

    public WDFFont getFont2() {
        return new WDFFont(font2);
    }

    public boolean setFont2(WDFFont font2)
    {
        if (!this.font2.equals(font2))
        {
            this.font2.setValues(font2);
            return true;
        }
        return false;
    }

    public String getLeftImage() {
        return leftImage;
    }

    public boolean setLeftImage(String leftImage) {
        if (!this.leftImage.equals(leftImage))
        {
            this.leftImage = leftImage;
            return true;
        }
        return false;
    }

    public String getMenuFile() {
        return menuFile;
    }

    public boolean setMenuFile(String menuFile) {
        if (!this.menuFile.equals(menuFile))
        {
            this.menuFile = menuFile;
            return true;
        }
        return false;
    }

    public String getRightImage() {
        return rightImage;
    }

    public boolean setRightImage(String rightImage) {
        if (!this.rightImage.equals(rightImage))
        {
            this.rightImage = rightImage;
            return true;
        }
        return false;
    }

    public String getTopImage() {
        return topImage;
    }

    public boolean setTopImage(String topImage) {
        if (!this.topImage.equals(topImage))
        {
            this.topImage = topImage;
            return true;
        }
        return false;
    }

    public Point getMenuPosition()
    {
        return new Point(menuPosition);
    }

    public boolean setMenuPosition(Point value)
    {
        if (!menuPosition.equals(value))
        {
            this.menuPosition = new Point(value);
            return true;
        }
        return false;
    }

    public Dimension getMenuDimension()
    {
        return new Dimension(menuDimension);
    }

    public boolean setMenuDimension(Dimension value)
    {
        if (!menuDimension.equals(value))
        {
            this.menuDimension = new Dimension(value);
            return true;
        }
        return false;
    }

    public String getUnknownFile() {
        return unknownFile;
    }

    public void setUnknownFile(String unknownFile) {
        this.unknownFile = unknownFile;
    }
}
