/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WDFScrollBar.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 31.7.2011
 */

package ue.edit.res.stbof.wdf;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import ue.util.Tools;

/**
 *
 */
public class WDFScrollBar extends WDFWidget
{
    protected int unknown;
    protected String handleOnImage;
    protected String handleOffImage;
    protected int minValue;
    protected int maxValue;

    public WDFScrollBar(InputStream in) throws IOException
    {
        super(WDFWidget.TYPE_SCROLLBAR, in);

        byte[] b4 = new byte[4];
        byte[] b13 = new byte[13];

        in.read(b4);
        unknown = Tools.toInt(b4, true);

        in.read(b13);
        handleOnImage = Tools.toASCII(b13);
        handleOnImage = handleOnImage.trim().toLowerCase();

        in.read(b13);
        handleOffImage = Tools.toASCII(b13);
        handleOffImage = handleOffImage.trim().toLowerCase();

        in.read(b4);
        minValue = Tools.toInt(b4, true);

        in.read(b4);
        maxValue = Tools.toInt(b4, true);
    }

    @Override
    public void save(OutputStream out) throws IOException
    {
        super.save(out);

        out.write(Tools.toByte(unknown, true)); // 4
        out.write(Tools.toByte(handleOnImage, 13, 12));
        out.write(Tools.toByte(handleOffImage, 13, 12));  // +26 = 30
        out.write(Tools.toByte(minValue, true));
        out.write(Tools.toByte(maxValue, true));    // +8 = 38
    }

    @Override
    public long getSize()
    {
        return super.getSize() + 38;
    }

    @Override
    public String toString()
    {
        return "Widget: Type: " + typeID + " Size: " + getSize() + " Files: "
                + this.backgroundImage + ", " + handleOffImage + ", " + handleOnImage;
    }

    public String getHandleOffImage() {
        return handleOffImage;
    }

    public boolean setHandleOffImage(String handleOffImage) {
        if (!this.handleOffImage.equalsIgnoreCase(handleOffImage))
        {
            this.handleOffImage = handleOffImage;
            return true;
        }
        return false;
    }

    public String getHandleOnImage() {
        return handleOnImage;
    }

    public boolean setHandleOnImage(String handleOnImage) {
        if (!this.handleOnImage.equalsIgnoreCase(handleOnImage))
        {
            this.handleOnImage = handleOnImage;
            return true;
        }
        return false;
    }

    public int getMaxValue() {
        return maxValue;
    }

    public boolean setMaxValue(int maxValue) {
        if (this.maxValue == maxValue)
        {
            this.maxValue = maxValue;
            return true;
        }
        return false;
    }

    public int getMinValue() {
        return minValue;
    }

    public boolean setMinValue(int minValue) {
        if (this.minValue == minValue)
        {
            this.minValue = minValue;
            return true;
        }
        return false;
    }

    public int getUnknown() {
        return unknown;
    }

    public boolean setUnknown(int unknown) {
        if (this.unknown == unknown)
        {
            this.unknown = unknown;
            return true;
        }
        return false;
    }
}
