/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WDFSlider.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 31.7.2011
 */

package ue.edit.res.stbof.wdf;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import ue.util.Tools;

/**
 *
 */
public class WDFSlider extends WDFWidget
{
    protected byte[] unknown1; // 2 bytes
    protected int[] unknown2;  // 4 integers

    protected String cellOffImage;
    protected String cellOnImage;
    protected String markerOffImage;
    protected String markerOnImage;

    protected int[] unknown3;  // 3 integers

    public WDFSlider(InputStream in) throws IOException
    {
        super(WDFWidget.TYPE_SLIDER, in);

        byte[] b4 = new byte[4];
        byte[] b13 = new byte[13];

        unknown1 = new byte[2];
        in.read(unknown1);

        unknown2 = new int[4];
        for (int i = 0; i < unknown2.length; i++)
        {
            in.read(b4);
            unknown2[i] = Tools.toInt(b4, true);
        }

        in.read(b13);
        cellOffImage = Tools.toASCII(b13);
        cellOffImage = cellOffImage.trim().toLowerCase();
        in.read(b13);
        cellOnImage = Tools.toASCII(b13);
        cellOnImage = cellOnImage.trim().toLowerCase();
        in.read(b13);
        markerOffImage = Tools.toASCII(b13);
        markerOffImage = markerOffImage.trim().toLowerCase();
        in.read(b13);
        markerOnImage = Tools.toASCII(b13);
        markerOnImage = markerOnImage.trim().toLowerCase();

        unknown3 = new int[3];
        for (int i = 0; i < unknown3.length; i++)
        {
            in.read(b4);
            unknown3[i] = Tools.toInt(b4, true);
        }
    }

    @Override
    public void save(OutputStream out) throws IOException
    {
        super.save(out);

        out.write(unknown1);    // 2

        for (int i = 0; i < unknown2.length; i++)   // +4*4 = 18
        {
            out.write(Tools.toByte(unknown2[i], true));
        }

        out.write(Tools.toByte(cellOffImage, 13, 12));
        out.write(Tools.toByte(cellOnImage, 13, 12)); // +26 = 44
        out.write(Tools.toByte(markerOffImage, 13, 12));
        out.write(Tools.toByte(markerOnImage, 13, 12));    // +26 = 70

        for (int i = 0; i < unknown3.length; i++)   // +3*4 = 82
        {
            out.write(Tools.toByte(unknown3[i], true));
        }
    }

    @Override
    public long getSize()
    {
        return super.getSize() + 82;
    }

    @Override
    public String toString()
    {
        return "Widget: Type: " + typeID + " Size: " + getSize() + " Files: "
                + this.backgroundImage + ", " + cellOffImage + ", " + cellOnImage + ", " + markerOffImage
                + ", " + markerOnImage;
    }

    public String getCellOffImage() {
        return cellOffImage;
    }

    public boolean setCellOffImage(String cellOffImage)
    {
        if (!this.cellOffImage.equals(cellOffImage))
        {
            this.cellOffImage = cellOffImage;
            return true;
        }
        return false;
    }

    public String getCellOnImage() {
        return cellOnImage;
    }

    public boolean setCellOnImage(String cellOnImage) {
        if (!this.cellOnImage.equals(cellOnImage))
        {
            this.cellOnImage = cellOnImage;
            return true;
        }
        return false;
    }

    public String getMarkerOffImage() {
        return markerOffImage;
    }

    public boolean setMarkerOffImage(String markerOffImage) {
        if (!this.markerOffImage.equals(markerOffImage))
        {
            this.markerOffImage = markerOffImage;
            return true;
        }
        return false;
    }

    public String getMarkerOnImage() {
        return markerOnImage;
    }

    public boolean setMarkerOnImage(String markerOnImage) {
        if (!this.markerOnImage.equals(markerOnImage))
        {
            this.markerOnImage = markerOnImage;
            return true;
        }
        return false;
    }

    public byte getUnknown1(int index) {
        return unknown1[index];
    }

    public boolean setUnknown1(int index, byte value)
    {
        if (unknown1[index] != value)
        {
            this.unknown1[index] = value;
            return true;
        }
        return false;
    }

    public int getUnknown2(int index) {
        return unknown2[index];
    }

    public boolean setUnknown2(int index, int value) {
        if (unknown2[index] != value)
        {
            this.unknown2[index] = value;
            return true;
        }
        return false;
    }

    public int getUnknown3(int index) {
        return unknown3[index];
    }

    public boolean setUnknown3(int index, int value) {
        if (unknown3[index] != value)
        {
            this.unknown3[index] = value;
            return true;
        }
        return false;
    }
}
