/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WDFTextBox.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 31.7.2011
 */

package ue.edit.res.stbof.wdf;

import java.awt.Dimension;
import java.awt.Point;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import ue.util.Tools;

/**
 *
 */
public class WDFTextBox extends WDFWidget
{
    protected Point textPosition;
    protected Dimension textDimension;
    protected WDFFont font;

    public WDFTextBox(InputStream in) throws IOException
    {
        super(WDFWidget.TYPE_TEXTBOX, in);

        byte[] b4 = new byte[4];
        int xw, yh;

        in.read(b4);
        xw = Tools.toInt(b4, true);
        in.read(b4);
        yh = Tools.toInt(b4, true);
        textPosition = new Point(xw, yh);

        in.read(b4);
        xw = Tools.toInt(b4, true);
        in.read(b4);
        yh = Tools.toInt(b4, true);
        textDimension = new Dimension(xw, yh);

        font = new WDFFont(in);
    }

    @Override
    public void save(OutputStream out) throws IOException
    {
        super.save(out);

        out.write(Tools.toByte(textPosition.x, true)); // 4
        out.write(Tools.toByte(textPosition.y, true)); // 8
        out.write(Tools.toByte(textDimension.width, true)); // 12
        out.write(Tools.toByte(textDimension.height, true)); // 16
        font.save(out); // +48 = 64
    }

    @Override
    public long getSize()
    {
        return super.getSize() + 64;
    }

    @Override
    public String toString()
    {
        return "Widget: Type: " + typeID + " Size: " + getSize() + " Files: "
                + this.backgroundImage + ", " + font;
    }

    public WDFFont getFont() {
        return new WDFFont(font);
    }

    public boolean setFont(WDFFont font) {
        if (!this.font.equals(font))
        {
            this.font.setValues(font);
            return true;
        }
        return false;
    }

    public Dimension getTextDimension() {
        return new Dimension(textDimension);
    }

    public boolean setTextDimension(Dimension textDimension) {
        if (!this.textDimension.equals(textDimension))
        {
            this.textDimension = new Dimension(textDimension);
            return true;
        }
        return false;
    }

    public Point getTextPosition() {
        return new Point(textPosition);
    }

    public boolean setTextPosition(Point textPosition) {
        if (!this.textPosition.equals(textPosition))
        {
            this.textPosition = new Point(textPosition);
            return true;
        }
        return false;
    }
}
