/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WDFWidget.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 30.7.2011
 */


package ue.edit.res.stbof.wdf;

import java.awt.Dimension;
import java.awt.Point;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import ue.util.Tools;

/**
 *
 */
public abstract class WDFWidget implements Comparable<WDFWidget>
{
    public static final int TYPE_WINDOW     = 1;
    public static final int TYPE_BUTTON     = 2;
    public static final int TYPE_MENU       = 5;
    public static final int TYPE_SLIDER     = 7;
    public static final int TYPE_SCROLLBAR  = 8;
    public static final int TYPE_LISTBOX    = 12;
    public static final int TYPE_PICTUREBOX = 13;
    public static final int TYPE_TEXTBOX    = 14;

    public static final String NO_IMAGE = "none";

    protected int          typeID;         // widget type
    protected int          instanceID;     // widget id
    protected Point        position;       // widget position
    protected Dimension    dimension;      // widget dimensions
    protected String       backgroundImage;// filename of background image, may be "none"

    public WDFWidget(int typeID, InputStream in) throws IOException
    {
        this.typeID = typeID;

        byte[] b4 = new byte[4];
        int xw, yh;

        // instance id
        in.read(b4);
        instanceID = Tools.toInt(b4, true);

        // position
        in.read(b4);
        xw = Tools.toInt(b4, true);
        in.read(b4);
        yh = Tools.toInt(b4, true);
        position = new Point(xw, yh);

        // dimensions
        in.read(b4);
        xw = Tools.toInt(b4, true);
        in.read(b4);
        yh = Tools.toInt(b4, true);
        dimension = new Dimension(xw, yh);

        b4 = new byte[13];
        in.read(b4);
        backgroundImage = Tools.toASCII(b4);
        backgroundImage = backgroundImage.trim().toLowerCase();
    }

    public void save(OutputStream out) throws IOException
    {
        out.write(Tools.toByte(typeID, true));
        out.write(Tools.toByte(instanceID, true));
        out.write(Tools.toByte(position.x, true));
        out.write(Tools.toByte(position.y, true));
        out.write(Tools.toByte(dimension.width, true));
        out.write(Tools.toByte(dimension.height, true));
        out.write(Tools.toByte(backgroundImage, 13, 12));
    }

    @Override
    public int compareTo(WDFWidget o)
    {
        return (typeID - o.typeID) * 100000 + (instanceID - o.instanceID);
    }

    public long getSize()
    {
        return 37;
    }

    public String getBackgroundImage() {
        return backgroundImage;
    }

    public void setBackgroundImage(String backgroundImage)
    {
        if (!this.backgroundImage.equalsIgnoreCase(backgroundImage))
        {
            this.backgroundImage = backgroundImage;
        }
    }

    public Dimension getDimension() {
        return new Dimension(dimension);
    }

    public boolean setDimension(Dimension dimension)
    {
        if (!this.dimension.equals(dimension))
        {
            this.dimension = new Dimension(dimension);
            return true;
        }
        return false;
    }

    public int getInstanceID() {
        return instanceID;
    }

    public boolean setInstanceID(int instanceID) {
        if (this.instanceID != instanceID)
        {
            this.instanceID = instanceID;
            return true;
        }
        return false;
    }

    public Point getPosition() {
        return new Point(position);
    }

    public boolean setPosition(Point position)
    {
        if (!this.position.equals(position))
        {
            this.position = new Point(position);
            return true;
        }

        return false;
    }

    public int getTypeID() {
        return typeID;
    }

    public void setTypeID(int typeID) {
        this.typeID = typeID;
    }

    public String getTypeAsString()
    {
        switch (typeID)
        {
            case TYPE_WINDOW:
                return "Window";
            case TYPE_BUTTON:
                return "Button";
            case TYPE_MENU:
                return "Menu";
            case TYPE_SLIDER:
                return "Slider";
            case TYPE_SCROLLBAR:
                return "Scrollbar";
            case TYPE_LISTBOX:
                return "Listbox";
            case TYPE_PICTUREBOX:
                return "Picture placeholder";
            case TYPE_TEXTBOX:
                return "Textbox";
            default:
                return "Invalid type";
        }
    }
}
