/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WDFWindow.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 30.7.2011
 */

package ue.edit.res.stbof.wdf;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import ue.UE;
import ue.util.Tools;

/**
 *
 */
public class WDFWindow extends WDFWidget
{
    protected ArrayList[] widgets = new ArrayList[16];

    @SuppressWarnings("unchecked")
    public WDFWindow(InputStream in) throws IOException
    {
        super(WDFWidget.TYPE_WINDOW, in);

        int widgetType;
        WDFWidget widget;
        byte[] b4 = new byte[4];

        in.read(b4);
        int sum = Tools.toInt(b4, true);

        in.skip(60);

        for (; sum > 0 && in.available() > 0; sum--)
        {
            in.read(b4);
            widgetType = Tools.toInt(b4, true);

            switch (widgetType)
            {
                case WDFWidget.TYPE_WINDOW:
                {
                    widget = new WDFWindow(in);
                    break;
                }
                case WDFWidget.TYPE_BUTTON:
                {
                    widget = new WDFButton(in);
                    break;
                }
                case WDFWidget.TYPE_MENU:
                {
                    widget = new WDFMenu(in);
                    break;
                }
                case WDFWidget.TYPE_SLIDER:
                {
                    widget = new WDFSlider(in);
                    break;
                }
                case WDFWidget.TYPE_SCROLLBAR:
                {
                    widget = new WDFScrollBar(in);
                    break;
                }
                case WDFWidget.TYPE_LISTBOX:
                {
                    widget = new WDFListBox(in);
                    break;
                }
                case WDFWidget.TYPE_PICTUREBOX:
                {
                    widget = new WDFPictureBox(in);
                    break;
                }
                case WDFWidget.TYPE_TEXTBOX:
                {
                    widget = new WDFTextBox(in);
                    break;
                }
                default:
                    throw new IOException("Unknown widget type encountered: %1"
                            .replace("%1", Integer.toString(widgetType)));
            }

            if (widgets[widgetType] == null)
            {
                widgets[widgetType] = new ArrayList<WDFWidget>();
            }

            widgets[widgetType].add(widget);

            if (UE.DEBUG_MODE)
            {
                System.out.println("Read widget: " + widgetType + " Bytes: " + widget.getSize());
            }
        }
    }

    @Override
    @SuppressWarnings("unchecked")
    public void save(OutputStream out) throws IOException
    {
        super.save(out);

        // numbers
        int[] sum = new int[16];

        for (int i = 1; i < widgets.length; i++)
        {
            if (widgets[i] != null)
            {
                sum[i] = widgets[i].size();
                sum[0] += sum[i];
            }
        }

        for (int i = 0; i < sum.length; i++)
        {
            out.write(Tools.toByte(sum[i], true));
        }

        // widgets
        WDFWidget widget;

        for (int i = 1; i < widgets.length; i++)
        {
            if (widgets[i] != null)
            {
                for (int n = 0; n < widgets[i].size(); n++)
                {
                    widget = (WDFWidget)widgets[i].get(n);
                    widget.save(out);
                }
            }
        }
    }

    @Override
    public long getSize()
    {
        long size = super.getSize() + 64;
        WDFWidget widget;

        for (int i = 1; i < widgets.length; i++)
        {
            if (widgets[i] != null)
            {
                for (int n = 0; n < widgets[i].size(); n++)
                {
                    widget = (WDFWidget)widgets[i].get(n);
                    size += widget.getSize();
                }
            }
        }
        
        return size;
    }

    @Override
    public String toString()
    {
        String str =  "Window: Size: " + getSize() + " Widgets: ";

        for (int i = 0; i < widgets.length; i++)
        {
            if (widgets[i] != null)
            {
                for (int n = 0; n < widgets[i].size(); n++)
                {
                    str = str + "\n" + widgets[i].get(n);
                }
            }
        }

        return str;
    }

    @SuppressWarnings("unchecked")
    public Enumeration<WDFWidget> getWidgets(int type)
    {
        if (type > 0 && type < widgets.length && widgets[type] != null)
        {
            return Collections.enumeration(widgets[type]);
        }
        else
        {
            return Collections.enumeration(new ArrayList<WDFWidget>(0));
        }
    }
}
