/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: Aiempire.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.10.2007
*/

package ue.edit.res.stbof.wtf;


import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;

/**Represents aiempire.wtf.*/
public class Aiempire extends InternalFile{
	private byte[] UNKNOWN_1 = new byte[4];
	private short SETTINGS = 0;
	private short EMPIRES = 0;
	private int VALUES = 0;
	private String[] SETTING;
	private String[] EMPIRE;
	//[race index][setting]
	private double[][] VALUE;
	
	public Aiempire(InputStream in) throws Exception{
		byte[] b;
		//unknown 4
		in.read(UNKNOWN_1);
		//settings 2
		b = new byte[2];
		in.read(b);
		SETTINGS = Tools.toShort(b, true);
		//minors 2
		in.read(b);
		EMPIRES = Tools.toShort(b, true);
		//values 4
		b = new byte[4];
		in.read(b);
		VALUES = Tools.toInt(b, true);
		//check for error and declare
		VALUES = SETTINGS*EMPIRES;
		SETTING = new String[SETTINGS];
		EMPIRE = new String[EMPIRES];
		VALUE = new double[EMPIRES][SETTINGS];
		//read settings
		b = new byte[33];
		for (int i=0; i<SETTINGS; i++){
			in.read(b);
			SETTING[i] = Tools.toASCII(b);
		}
		//read minors
		for (int i=0; i<EMPIRES; i++){
			in.read(b);
			EMPIRE[i] = Tools.toASCII(b);
		}
		//read values
		b = new byte[8];
		for (int i=0; i<EMPIRES; i++){
			for (int j=0; j<SETTINGS; j++){
				in.read(b);
				VALUE[i][j] = Tools.toDouble(b, true);
			}
		}
	}
	
	/**Saves changes.
	@return true if save has been successful.
	@throws Exception the function may throw an exception if an error occures.*/
	@Override
	public void save(OutputStream out) throws Exception{
		//unknown
		out.write(UNKNOWN_1);
		//settings
		byte[] b = Tools.toByte(SETTINGS, true);
		out.write(b);
		//minors
		b = Tools.toByte(EMPIRES, true);
		out.write(b);
		//values
		b = Tools.toByte(VALUES, true);
		out.write(b);
		//write settings
		for (int i=0; i<SETTINGS; i++){
			b = Tools.toByte(SETTING[i], 33, 33);
			out.write(b);
		}
		//write minors
		for (int i=0; i<EMPIRES; i++){
			b = Tools.toByte(EMPIRE[i], 33, 33);
			out.write(b);
		}
		//write values
		for (int i=0; i<EMPIRES; i++){
			for (int j=0; j<SETTINGS; j++){
				b = Tools.toByte(VALUE[i][j], true);
				out.write(b);
			}
		}
	}
	/**This function checks the file for known problems/errors that may occur while editing the file.
	@return a Vector object containing results of the check as Strings.*/
	@Override
	public Vector<String> check(){
		Vector<String> response = new Vector<String>();
		String msg;
		if (SETTINGS!=9){
			msg = Language.getString("Aiempire.0"); //$NON-NLS-1$
			msg = msg.replace("%1", getName()); //$NON-NLS-1$
			msg = msg.replace("%2", Integer.toString(SETTINGS)); //$NON-NLS-1$
			response.add(msg);
		}
		return response;
	}
	/**Used to get the uncompressed size of the file.**/
	@Override
	public long getSize(){
		return (12 + (SETTINGS+EMPIRES)*33 + VALUES*8);
	}
	/**used to set a value of an entry
	@param index the index of the entry to set values from
	@param setting the setting you want to set
	@param value the new value to assign*/
	public void setValue(int index, int setting, double value) throws InvalidArgumentsException{
		if (value<0){
			String msg = Language.getString("Aiempire.1"); //$NON-NLS-1$
			msg = msg.replace("%1", Double.toString(value)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
		try{
			if (VALUE[index][setting] == value) {
				return;
			}
			VALUE[index][setting] = value;
                        this.markChanged();
		}catch (Exception e){
			String msg = Language.getString("Aiempire.2"); //$NON-NLS-1$
			msg = msg.replace("%1", "(" + index + ", " + setting + ")"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			throw new InvalidArgumentsException(msg);
		}
	}
	/**used to get value of entry
	@param index the index of the entry to get values from
	@param setting the setting you want to get*/
	public double getValue(int index, int setting) throws InvalidArgumentsException{
		try{
			return VALUE[index][setting];
		}catch (Exception e){
			String msg = Language.getString("Aiempire.2"); //$NON-NLS-1$
			msg = msg.replace("%1", "(" + index + ", " + setting + ")"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			throw new InvalidArgumentsException(msg);
		}
	}
	/**Returns empire names:*/
	public String[] getEmpires(){
		String[] arry = new String[EMPIRE.length];
		for (int i=0; i<EMPIRE.length; i++){
			arry[i] = new String(EMPIRE[i]);
		}
		return arry;
	}

}