/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: Aiminor.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.edit.res.stbof.wtf;

import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.Language;
import ue.UE;
import ue.edit.FileManager;
import ue.edit.InternalFile;
import ue.edit.exe.Trek;
import ue.edit.exe.trek.RaceList;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.rst.Race;
import ue.exception.CorruptedFile;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;

public class Aiminor extends InternalFile
{
    public static final int PERSONALITY_SETTING = 0;
    public static final int EXPLORER_SETTING = 1;
    public static final int HOSTILITY1_SETTING = 2;
    public static final int HOSTILITY2_SETTING = 3;
    public static final int HOSTILITY3_SETTING = 4;
    public static final int HOSTILITY4_SETTING = 5;
    public static final int HOSTILITY5_SETTING = 6;

    private byte[] UNKNOWN_1 = new byte[4];
    private String[] SETTING;
    private String[] MINOR;
    //[race index][setting]
    private double[][] VALUE;
    
    /**
     * Constructor
     */
    public Aiminor(InputStream in) throws CorruptedFile, IOException, EOFException
    {
        byte[] b;
        //unknown
        in.read(UNKNOWN_1);
        //settings
        b = new byte[2];
        in.read(b);
        int SETTINGS = Tools.toShort(b, true);
        //minors
        in.read(b);
        int MINORS = Tools.toShort(b, true);
        //values
        b = new byte[4];
        in.read(b);
        //set
        SETTING = new String[SETTINGS];
        MINOR = new String[MINORS];
        VALUE = new double[MINORS][SETTINGS];
        //read settings
        b = new byte[33];
        for (int i = 0; i < SETTINGS; i++)
        {
            in.read(b);
            SETTING[i] = Tools.toASCII(b, 0, 33);
        }
        
        //read minors
        for (int i = 0; i < MINORS; i++)
        {
            in.read(b);
            MINOR[i] = Tools.toASCII(b, 0, 33);
        }
	
        //read values
        b = new byte[8];
        for (int i = 0; i < MINORS; i++)
        {
            for (int j = 0; j < SETTINGS; j++)
            {
                in.read(b);
                VALUE[i][j] = Tools.toDouble(b, true);
            }
        }
    }
    
    public boolean containsRace(String race)
    {
        for (String r : MINOR)
        {
            if (r.equals(race))
            {
                return true;
            }
        }
        
        return false;
    }
    
    /**
     * Used to change entry name
     * @param index the index of the entry to be changed
     * @param name the new name to be aplied
     */
    public void setName(int index, String name) throws InvalidArgumentsException
    {
        checkIndex(index);
        if (name.length() == 0 || name.length() > 32)
        {
            return;
        }
        if (name.equals(getName(index)))
        {
            return;
        }
        MINOR[index]=name;
        this.markChanged();
    }
    
    /**
     * Used to get entry name
     * @param index the index of the entry
     */
    public String getName(int index) throws InvalidArgumentsException
    {
        checkIndex(index);
        return MINOR[index];
    }

    /**
     * Used to get entry index
     */
    public int getIndex(String name) throws InvalidArgumentsException
    {
        for (int i = 0; i < MINOR.length; i++)
        {
            if (MINOR[i].equalsIgnoreCase(name))
            {
                return i;
            }
        }

        return -1;
    }

    public String[] getNames()
    {
        return MINOR;
    }
    
    /**
     * used to remove entry
     * @param index the index of the entry
     */
    public void remove(int index) throws InvalidArgumentsException
    {
        checkIndex(index);
        
        short num = (short)(MINOR.length - 1);
        
        String[] min = new String[num];
        double[][] val = new double[num][SETTING.length];
        
        System.arraycopy(MINOR, 0, min, 0, index);
        
        for (int i = 0; i < index; i++)
        {
            System.arraycopy(VALUE[i], 0, val[i], 0, SETTING.length);
        }
        
        System.arraycopy(MINOR, index + 1, min, index, min.length - index);

        for (int i = index; i < val.length; i++)
        {
            System.arraycopy(VALUE[i + 1], 0, val[i], 0, SETTING.length);
        }

        MINOR = min;
        VALUE = val;
        
        this.markChanged();
    }
    
    /**
     * Used to add entry
     * @param index the index of the entry to copy values from
     */
    public int add(int index) throws InvalidArgumentsException
    {
        checkIndex(index);
        
        short num = (short)(MINOR.length + 1);
        
        String[] min = new String[num];
        
        double[][] val = new double[num][SETTING.length];
	
        System.arraycopy(MINOR, 0, min, 0, MINOR.length);
        System.arraycopy(VALUE, 0, val, 0, VALUE.length);
        
        min[MINOR.length] = MINOR[index];

        for (int x = 0; x < SETTING.length; x++)
        {
            val[MINOR.length][x]=VALUE[index][x];
        }
        
        MINOR = min;
        VALUE = val;
        
        this.markChanged();

        return MINOR.length - 1;
    }
    
    /**
     * Used to get value of entry
     * @param index the index of the entry to get values from
     * @param setting the setting you want to get
     */
    public double getValue(int index, int setting) throws InvalidArgumentsException
    {
        checkIndex(index);
        checkSettingIndex(setting);
        return VALUE[index][setting];
    }
    
    /**
     * Used to get number of entries
     */
    public int getNumber()
    {
        return MINOR.length;
    }

    /**
     * Used to set a value of an entry
     * @param index the index of the entry to set values from
     * @param setting the setting you want to set
     * @param value the new value to assign
     */
    public void setValue(int index, int setting, double value) throws InvalidArgumentsException
    {
        checkIndex(index);
        checkSettingIndex(setting);
	
        if (value == getValue(index, setting))
        {
            return;
        }
        
        if (value < 0 || value > 8)
        {
            String msg = Language.getString("Aiminor.0"); //$NON-NLS-1$
            msg = msg.replace("%1", Double.toString(value)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
	
        if (setting == 0 && value > 4)
        {
            String msg = Language.getString("Aiminor.0"); //$NON-NLS-1$
            msg = msg.replace("%1", Double.toString(value)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
        
        if (setting == 1 && value > 1)
        {
            String msg = Language.getString("Aiminor.0"); //$NON-NLS-1$
            msg = msg.replace("%1", Double.toString(value)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
        
        VALUE[index][setting] = value;
        this.markChanged();
    }
    
    /**
     * Used to save changes.
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        //unknown
        out.write(UNKNOWN_1);
        //settings
        byte[] b = Tools.toByte((short)SETTING.length, true);
        out.write(b);
        //minors
        b = Tools.toByte((short)MINOR.length, true);
        out.write(b);
        //values
        b = Tools.toByte(SETTING.length * MINOR.length, true);
        out.write(b);
        //write settings
        for (int i = 0; i < SETTING.length; i++)
        {
            b = Tools.toByte(SETTING[i], 33, 32);
            out.write(b);
        }
        //write minors
        for (int i = 0; i < MINOR.length; i++)
        {
            b = Tools.toByte(MINOR[i], 33, 32);
            out.write(b);
        }
        //write values
        for (int i = 0; i < MINOR.length; i++)
        {
            for (int j = 0; j < SETTING.length; j++)
            {
                b = Tools.toByte(VALUE[i][j], true);
                out.write(b);
            }
        }
    }

    /**
     * Used to get the uncompressed size of the file.
     */
    @Override
    public long getSize()
    {
        return (12 + (SETTING.length + MINOR.length) * 33 + (SETTING.length * MINOR.length) * 8);
    }
    
    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>(); 
        
        String msg;
        
        // check number of settings
        if (SETTING.length != 7)
        {
            msg = Language.getString("Aiminor.1"); //$NON-NLS-1$
            msg = msg.replace("%1", Integer.toString(SETTING.length)); //$NON-NLS-1$
            msg = this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg);
            response.add(msg);
        }
        
        // check sanity of set values
        for (int i = 0; i < MINOR.length; i++)
        {
            for (int j = 0; j < SETTING.length; j++)
            {
                if (j == 0)
                {
                    if (VALUE[i][j] < 0 || VALUE[i][j] > 4)
                    {
                        msg = Language.getString("Aiminor.2"); //$NON-NLS-1$
                        msg = msg.replace("%1", MINOR[i]); //$NON-NLS-1$
                        msg = msg.replace("%2", Double.toString(VALUE[i][j])); //$NON-NLS-1$
                        msg = this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg);
                        response.add(msg);
                    }
                }
                else if (j == 1)
                {
                    if (VALUE[i][j] < 0 || VALUE[i][j] > 1)
                    {
                        msg = Language.getString("Aiminor.3"); //$NON-NLS-1$
                        msg = msg.replace("%1", MINOR[i]); //$NON-NLS-1$
                        msg = msg.replace("%2", Double.toString(VALUE[i][j])); //$NON-NLS-1$
                        msg = this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg);
                        response.add(msg);
                    }
                }
                else
                {
                    if (VALUE[i][j] < 0 || VALUE[i][j] > 8)
                    {
                        msg = Language.getString("Aiminor.4"); //$NON-NLS-1$
                        msg = msg.replace("%1", MINOR[i]); //$NON-NLS-1$
                        msg = msg.replace("%2", Double.toString(VALUE[i][j])); //$NON-NLS-1$
                        msg = this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, msg);
                        response.add(msg);
                    }
                }
            }
        }
        
        if (UE.FILES.getFile(FileManager.P_STBOF) != null)
        {
            try
            {
                // check if races match race.rst
                Race race = (Race)UE.FILES.getFile(FileManager.P_STBOF).getInternalFile(Stbof.fileRACE_RST, true);

                boolean found[] = new boolean[race.getNumberOfEntries()];
                found[0] = true;
                
                if (race.getNumberOfEntries() - 5 != MINOR.length)
                {
                    msg = Language.getString("Aiminor.7").replace("%1", Stbof.fileRACE_RST);  //$NON-NLS-1$ //$NON-NLS-2$
                    msg = this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_WARNING, msg);
                    response.add(msg);
                }
                
                String minor;
                
                for (int i = 5; i < race.getNumberOfEntries(); i++)
                {
                    minor = race.getName(i);
                    
                    if (!this.containsRace(minor))
                    {
                        found[0] = found[i] = false;    // not found in aiminor
                        
                        msg = Language.getString("Aiminor.8").replace("%1", minor);  //$NON-NLS-1$ //$NON-NLS-2$
                        msg = this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_WARNING, msg);
                        response.add(msg);
                    }
                    else
                    {
                        found[i] = true;
                    }
                }
                
                // get trek.exe
                if (UE.FILES.getFile(FileManager.P_TREK) != null)
                {
                    RaceList rlst = (RaceList)UE.FILES.getFile(FileManager.P_TREK).getInternalFile(Trek.fileRACE_LIST, true);
                    
                    for (int i = 0; i < race.getNumberOfEntries(); i++)
                    {
                        if (!rlst.getRaceString(i).equals(race.getName(i)))
                        {
                            msg = Language.getString("Aiminor.9").replace("%1", rlst.getRaceString(i)); //$NON-NLS-1$ //$NON-NLS-2$
                            
                            rlst.setRaceString(i, race.getName(i)); // rename entry in trek.exe to match race.rst
                            
                            msg = msg.replace("%2", race.getName(i));
                            msg = this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_INFO, msg);
                            response.add(msg);
                        }
                        
                        if (i > 4 && !found[i])
                        {
                            for (int j = 0; j < MINOR.length; j++)
                            {
                                if (!race.containsRace(MINOR[j]))
                                {
                                    msg = Language.getString("Aiminor.10").replace("%1", MINOR[j]); //$NON-NLS-1$ //$NON-NLS-2$

                                    MINOR[j] = race.getName(i);
                                    this.markChanged();
                                    
                                    msg = msg.replace("%2", MINOR[j]); //$NON-NLS-1$
                                    msg = this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_INFO, msg);
                                    response.add(msg);
                                    
                                    found[i] = true;
                                    
                                    break;
                                }
                            }
                            
                            if (!found[i])
                            {
                                this.add(0);
                                MINOR[MINOR.length - 1] = race.getName(i);
                                this.markChanged();
                                found[i] = true;

                                msg = Language.getString("Aiminor.11").replace("%1", race.getName(i)); //$NON-NLS-1$ //$NON-NLS-2$
                                msg = this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_INFO, msg);
                                response.add(msg);
                            }
                        }
                    }

                    // now remove the extras
                    for (int j = 0; j < MINOR.length; j++)
                    {
                        if (!race.containsRace(MINOR[j]))
                        {
                            msg = Language.getString("Aiminor.13").replace("%1", MINOR[j]); //$NON-NLS-1$ //$NON-NLS-2$
                            msg = this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_INFO, msg);
                            response.add(msg);

                            this.remove(j);
                            j--;
                        }
                    }
                }
                else if (!found[0])
                {
                    msg = Language.getString("Aiminor.12"); //$NON-NLS-1$
                    msg = this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_INFO, msg);
                    response.add(msg);
                }
            }
            catch (Exception e)
            {
                response.add(this.getCheckIntegrityString(InternalFile.INTEGRITY_CHECK_ERROR, e.getMessage()));
            }
        }
        
        return response;
    }
    
    private void checkIndex(int index) throws InvalidArgumentsException
    {
        if (index < 0 || index >= MINOR.length)
        {
            String msg = Language.getString("Aiminor.5"); //$NON-NLS-1$
            msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
    }
    
    private void checkSettingIndex(int setting) throws InvalidArgumentsException
    {
        if (setting < 0 || setting >= SETTING.length)
        {
            String msg = Language.getString("Aiminor.6"); //$NON-NLS-1$
            msg = msg.replace("%1", Integer.toString(setting)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
    }
}