/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: Colval.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.10.2007
*/

package ue.edit.res.stbof.wtf;

import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.exception.CorruptedFile;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;

public class Colval extends InternalFile{
	private byte[] UNKNOWN_1 = new byte[4];
	private short SETTINGS = 0;
	private short SETS = 0;
	private int VALUES = 0;
	private String[] SETTING;
	private String[] SET;
	//[set][setting]
	private double[][] VALUE;
	
	/**constructor*/
	public Colval(InputStream in) throws CorruptedFile, IOException, EOFException{
		byte[] b;
		//unknown
		in.read(UNKNOWN_1);
		//settings
		b = new byte[2];
		in.read(b);
		SETTINGS = Tools.toShort(b, true);
		//sets
		in.read(b);
		SETS = Tools.toShort(b, true);
		//values
		b = new byte[4];
		in.read(b);
		VALUES = Tools.toInt(b, true);
		//check for error and declare
		VALUES = SETTINGS*SETS;
		SETTING = new String[SETTINGS];
		SET = new String[SETS];
		VALUE = new double[SETS][SETTINGS];
		//read settings
		b = new byte[33];
		for (int i=0; i<SETTINGS; i++){
			in.read(b);
			try{
				SETTING[i] = Tools.toASCII(b, 0, 33);
			}catch (Exception ef){ ef.printStackTrace();}
		}
		//read sets
		for (int i=0; i<SETS; i++){
			in.read(b);
			try{
				SET[i] = Tools.toASCII(b, 0, 33);
			}catch (Exception ef){ ef.printStackTrace();}
		}
		//read values
		b = new byte[8];
		for (int i=0; i<SETS; i++){
			for (int j=0; j<SETTINGS; j++){
				in.read(b);
				VALUE[i][j] = Tools.toDouble(b, true);
			}
		}
	}
	/**used to get value of entry
	@param index the index of the entry to get values from
	@param setting the setting you want to get*/
	public double getValue(int index, int setting) throws InvalidArgumentsException{
		checkSetIndex(index);
		checkSettingIndex(setting);
		return VALUE[index][setting];
	}
	/**used to set a value of an entry
	@param index the index of the entry to set values from
	@param setting the setting you want to set
	@param value the new value to assign*/
	public void setValue(int index, int setting, double value) throws InvalidArgumentsException{
		checkSetIndex(index);
		checkSettingIndex(setting);
		if (value == getValue(index, setting)) {
			return;
		}
		VALUE[index][setting] = value;
                this.markChanged();
	}
	/**used to save changes.*/
	@Override
	public void save(OutputStream out) throws Exception{
		//unknown
		out.write(UNKNOWN_1);
		//settings
		byte[] b = Tools.toByte(SETTINGS, true);
		out.write(b);
		//sets
		b = Tools.toByte(SETS, true);
		out.write(b);
		//values
		b = Tools.toByte(VALUES, true);
		out.write(b);
		//write settings
		for (int i=0; i<SETTINGS; i++){
			b = Tools.toByte(SETTING[i], 33, 32);
			out.write(b);
		}
		//write sets
		for (int i=0; i<SETS; i++){
			b = Tools.toByte(SET[i], 33, 32);
			out.write(b);
		}
		//write values
		for (int i=0; i<SETS; i++){
			for (int j=0; j<SETTINGS; j++){
				b = Tools.toByte(VALUE[i][j], true);
				out.write(b);
			}
		}
	}
	/**Used to get the uncompressed size of the file.**/
	@Override
	public long getSize(){
		return (12 + (SETTINGS+SETS)*33 + VALUES*8);
	}
	
	@Override
	public Vector<String> check(){
		Vector<String> response = new Vector<String>();
		if (SETTINGS!=13){
			String msg = Language.getString("Colval.0"); //$NON-NLS-1$
			msg = msg.replace("%1", getName()); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(SETTINGS)); //$NON-NLS-1$
			response.add(msg);
		}
		return response;
	}
	
	private void checkSetIndex(int index) throws InvalidArgumentsException{
		if (index < 0 || index >= SETS){
			String msg = Language.getString("Colval.1"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
	}
	
	private void checkSettingIndex(int setting) throws InvalidArgumentsException{
		if (setting < 0 || setting >= SETTINGS){
			String msg = Language.getString("Colval.2"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(setting)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
	}
}
