/**
 * Ultimate Editor
 * Copyright (C) 2005-2010 Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: GListHead.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 22.07.2010
 */

package ue.edit.sav;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;
import ue.edit.InternalFile;
import ue.util.Tools;

public class GListHead extends InternalFile
{
    private byte[] UNKNOWN_1 = new byte[16];    // 0x00 - 0x0F (16 bytes)
    private int SLOT_SIZE;                      // 0x10 - 0x13 (4 bytes)
    private int TOTAL_SLOTS;                    // 0x14 - 0x17 (4 bytes)
    private int OCCUPIED_SLOTS;                 // 0x18 - 0x1B (4 bytes)
    private byte[] UNKNOWN_2 = new byte[12];    // 0x1C - 0x27 (12 bytes)

    public GListHead(InputStream in) throws IOException
    {
        int len = 0;

        while ((len += in.read(UNKNOWN_1, len, UNKNOWN_1.length - len)) != UNKNOWN_1.length)
        {}

        byte[] b = new byte[4];
        in.read(b);
        SLOT_SIZE = Tools.toInt(b, true);
        in.read(b);
        TOTAL_SLOTS = Tools.toInt(b, true);
        in.read(b);
        OCCUPIED_SLOTS = Tools.toInt(b, true);

        len = 0;
        
        while ((len += in.read(UNKNOWN_2, len, UNKNOWN_2.length - len)) != UNKNOWN_2.length)
        {}
    }

    @Override
    public void save(OutputStream out) throws Exception
    {
        out.write(UNKNOWN_1);
        out.write(Tools.toByte(SLOT_SIZE, true));
        out.write(Tools.toByte(TOTAL_SLOTS, true));
        out.write(Tools.toByte(OCCUPIED_SLOTS, true));
        out.write(UNKNOWN_2);
    }

    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();

        return response;
    }

    @Override
    public long getSize()
    {
        return 40;
    }

    public int getOccupiedSlotsNum()
    {
        return OCCUPIED_SLOTS;
    }

    public void setOccupiedSlotsNum(int num)
    {
        if (OCCUPIED_SLOTS != num)
        {
            OCCUPIED_SLOTS = num;

            if (OCCUPIED_SLOTS > TOTAL_SLOTS)
            {
                TOTAL_SLOTS = OCCUPIED_SLOTS;
            }
            this.markChanged();
        }
    }

    public int getSlotSize()
    {
        return SLOT_SIZE;
    }

    public void setSlotSize(int size)
    {
        if (SLOT_SIZE != size)
        {
            SLOT_SIZE = size;
            this.markChanged();
        }
    }

    public int getTotalSlotsNum()
    {
        return TOTAL_SLOTS;
    }

    public void setTotalSlotNum(int num)
    {
        if (TOTAL_SLOTS != num)
        {
            TOTAL_SLOTS = num;

            if (OCCUPIED_SLOTS > TOTAL_SLOTS)
            {
                TOTAL_SLOTS = OCCUPIED_SLOTS;
            }
            this.markChanged();
        }
    }
}
