/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2010  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: GShipList.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 5.8.2010
 */

package ue.edit.sav;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Vector;
import ue.edit.InternalFile;
import ue.exception.InvalidArgumentsException;
import ue.util.CommonStrings;
import ue.util.Tools;

/**
 *
 * @author Alan
 */
public class GShipList extends InternalFile
{
    private HashMap<Short, Ship> entries;
    private GListHead header;
    private int slots_num;

    public GShipList(GListHead header, InputStream in) throws IOException
    {
        this.header = header;
        
        entries = new HashMap<Short, Ship>();

        int num = header.getOccupiedSlotsNum();

        Ship sh;

        for (int i = 0; i < num; i++)
        {
            sh = new Ship(in);
            entries.put(sh.shipID, sh);
        }

        // fix header
        slots_num = header.getTotalSlotsNum();

        if (slots_num < num)
        {
            slots_num = (num / 16 + 1) * 16;
            header.setTotalSlotNum(slots_num);
        }
        header.setSlotSize(116);
    }

    @Override
    public void save(OutputStream out) throws Exception
    {
        Ship sh;
        Short[] IDs = entries.keySet().toArray(new Short[0]);
        Arrays.sort(IDs);
        byte[] padding = new byte[116];

        for (int i = 0; i < slots_num; i++)
        {
            if (i < IDs.length)
            {
                sh = entries.get(IDs[i]);
                sh.save(out);
            }
            else
            {
                out.write(padding);
            }
        }
    }

    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();
        return response;
    }

    @Override
    public long getSize()
    {
        return slots_num * 116;
    }

    public int getShipTypeID(short index) throws InvalidArgumentsException
    {
        testIndex(index);
        Ship sh = entries.get(index);
        return sh.shipType;
    }

    public String getShipName(short index) throws InvalidArgumentsException
    {
        testIndex(index);
        Ship sh = entries.get(index);
        return sh.name;
    }

    public int getHullStr(short index) throws InvalidArgumentsException
    {
        testIndex(index);
        Ship sh = entries.get(index);
        return sh.hullStr;
    }

    private void testIndex(short index) throws InvalidArgumentsException
    {
        if (!entries.containsKey(index))
        {
            throw new InvalidArgumentsException(
                    CommonStrings.getLocalizedString(
                    CommonStrings.InvalidIndex_1, new String[]{Integer.toString(index)}));
        }
    }

    private class Ship
    {
        String name;                    // 040
        short raceID;                   // 042
        short shipID;                   // 044
        int beamAccuBonus;              // 048
        int torpAccuBonus;              // 052
        int defenseBonus;               // 056
        int shieldStr;                  // 060
        int hullStr;                    // 064
        int crewExp;                    // 068
        int crewExpLevel;               // 072
        byte[] unknown1 = new byte[8];  // 080
        short fleetID;                  // 082
        int shipType;                   // 086  -> id in shiplist.sst
        byte[] unknown2 = new byte[30]; // 116

        Ship(InputStream in) throws IOException
        {
            byte[] b4 = new byte[40];
            in.read(b4);
            name = Tools.toASCII(b4);
            
            b4 = new byte[4];
            byte[] b2 = new byte[2];
            in.read(b2);
            raceID = Tools.toShort(b2, true);
            in.read(b2);
            shipID = Tools.toShort(b2, true);
            in.read(b4);
            beamAccuBonus = Tools.toShort(b4, true);
            in.read(b4);
            torpAccuBonus = Tools.toShort(b4, true);
            in.read(b4);
            defenseBonus = Tools.toShort(b4, true);
            in.read(b4);
            shieldStr = Tools.toShort(b4, true);
            in.read(b4);
            hullStr = Tools.toShort(b4, true);
            in.read(b4);
            crewExp = Tools.toShort(b4, true);
            in.read(b4);
            crewExpLevel = Tools.toShort(b4, true);
            in.read(unknown1);
            in.read(b2);
            fleetID = Tools.toShort(b2, true);
            in.read(b4);
            shipType = Tools.toShort(b4, true);
            in.read(unknown2);
        }

        void save(OutputStream out) throws IOException
        {
            out.write(Tools.toByte(name, 40, 39));
            out.write(Tools.toByte(raceID, true));
            out.write(Tools.toByte(shipID, true));
            out.write(Tools.toByte(beamAccuBonus, true));
            out.write(Tools.toByte(torpAccuBonus, true));
            out.write(Tools.toByte(defenseBonus, true));
            out.write(Tools.toByte(shieldStr, true));
            out.write(Tools.toByte(hullStr, true));
            out.write(Tools.toByte(crewExp, true));
            out.write(Tools.toByte(crewExpLevel, true));
            out.write(unknown1);
            out.write(Tools.toByte(fleetID, true));
            out.write(Tools.toByte(shipType, true));
            out.write(unknown2);
        }
    }
}
