/**
 * Ultimate Editor
 * Copyright (C) 2008 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: GalInfo.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 17.11.2008
 */
package ue.edit.sav;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.util.Tools;

/**
 * galInfo
 */
public class GalInfo extends InternalFile
{
    private byte[] UNKNOWN_1 = new byte[8];     //  8
    private int GAL_SIZE;                       // 12
    private short STAR_COUNT;                   // 14
    private short ANOMALY_COUNT;                // 16
    private short OUTPOST_COUNT;                // 18
    private byte[] UNKNOWN_2 = new byte[2];     // 20
    private short SECTOR_COUNT;                 // 22
    private byte[] UNKNOWN_3 = new byte[26];    // 48
    
    private static String strError = Language.getString("GalInfo.0"); //$NON-NLS-1$
    private static String strGalSize = Language.getString("GalInfo.1"); //$NON-NLS-1$
    private static String strStarCnt = Language.getString("GalInfo.2"); //$NON-NLS-1$
    private static String strAnomCnt = Language.getString("GalInfo.3"); //$NON-NLS-1$
    private static String strOutCnt = Language.getString("GalInfo.4"); //$NON-NLS-1$
    private static String strSectorCnt = Language.getString("GalInfo.5"); //$NON-NLS-1$

    /**
     * @param in the InputStream to read from
     * @throws IOException 
     */
    public GalInfo(InputStream in) throws IOException
    {
        byte[] b = new byte[4];
        
        in.read(UNKNOWN_1);
        in.read(b);
        GAL_SIZE = Tools.toInt(b, true);
        b = new byte[2];
        in.read(b);
        STAR_COUNT = Tools.toShort(b, true);
        in.read(b);
        ANOMALY_COUNT = Tools.toShort(b, true);
        in.read(b);
        OUTPOST_COUNT = Tools.toShort(b, true);
        in.read(UNKNOWN_2);
        in.read(b);
        SECTOR_COUNT = Tools.toShort(b, true);
        in.read(UNKNOWN_3);

        System.out.println("Debug info: Object count: " + (ANOMALY_COUNT + STAR_COUNT));
    }

    /**
     * @param num number of anomalies
     */
    public void setAnomalyCount(short num)
    {
        if (ANOMALY_COUNT != num)
        {
            ANOMALY_COUNT = num;
            this.markChanged();
        }
    }
    
    /**
     * @return number of anomalies
     */
    public short getAnomalyCount()
    {
        return ANOMALY_COUNT;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.InternalFile#check()
     */
    @Override
    public Vector<String> check()
    {
        Vector<String> ret = new Vector<String>();
        
        if (GAL_SIZE < 0 || GAL_SIZE > 2)
        {
            String msg = strError.replace("%1", getName()); //$NON-NLS-1$
            msg.replace("%2", strGalSize); //$NON-NLS-1$
            msg.replace("%1", Integer.toString(GAL_SIZE)); //$NON-NLS-1$
            
            ret.add(msg);
        }
        if (STAR_COUNT < 0)
        {
            String msg = strError.replace("%1", getName()); //$NON-NLS-1$
            msg.replace("%2", strStarCnt); //$NON-NLS-1$
            msg.replace("%1", Integer.toString(STAR_COUNT)); //$NON-NLS-1$
            
            ret.add(msg);
        }
        if (ANOMALY_COUNT < 0)
        {
            String msg = strError.replace("%1", getName()); //$NON-NLS-1$
            msg.replace("%2", strAnomCnt); //$NON-NLS-1$
            msg.replace("%1", Integer.toString(ANOMALY_COUNT)); //$NON-NLS-1$
            
            ret.add(msg);
        }
        if (OUTPOST_COUNT < 0)
        {
            String msg = strError.replace("%1", getName()); //$NON-NLS-1$
            msg.replace("%2", strOutCnt); //$NON-NLS-1$
            msg.replace("%1", Integer.toString(OUTPOST_COUNT)); //$NON-NLS-1$
            
            ret.add(msg);
        }
        if (SECTOR_COUNT < 0)
        {
            String msg = strError.replace("%1", getName()); //$NON-NLS-1$
            msg.replace("%2", strSectorCnt); //$NON-NLS-1$
            msg.replace("%1", Integer.toString(SECTOR_COUNT)); //$NON-NLS-1$
            
            ret.add(msg);
        }
        return ret;
    }

    /* (non-Javadoc)
     * @see ue.edit.InternalFile#getSize()
     */
    @Override
    public long getSize()
    {
        return 48;
    }

    /* (non-Javadoc)
     * @see ue.edit.InternalFile#save(java.io.OutputStream)
     */
    @Override
    public void save(OutputStream out) throws Exception
    {
        out.write(UNKNOWN_1);
        out.write(Tools.toByte(GAL_SIZE, true));
        out.write(Tools.toByte(STAR_COUNT, true));
        out.write(Tools.toByte(ANOMALY_COUNT, true));
        out.write(Tools.toByte(OUTPOST_COUNT, true));
        out.write(UNKNOWN_2);
        out.write(Tools.toByte(SECTOR_COUNT, true));
        out.write(UNKNOWN_3);
    }

    /**
     * @param num number of outposts
     */
    public void setBaseCount(short num)
    {
        if (OUTPOST_COUNT != num)
        {
            OUTPOST_COUNT = num;
            this.markChanged();
        }
    }
}
