/*
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2007  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *
 * File: GameInfo.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Last modified: 6.10.2007
 */

package ue.edit.sav;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;

/**This class is a representation of the gameInfo file. It holds info on the game.*/
public class GameInfo extends InternalFile{
	//data
	private String FILENAME; //irrelevant
	private String GAME_NAME;
	private int TURNS;
	private boolean MP;
	private short EMPIRE_MASK;
	private byte PLAYER_EMPIRE;
	private byte DIFFICULTY;
	
	private byte[] UNKNOWN_1 = new byte[8];
	private byte[] UNKNOWN_2 = new byte[512];
	private byte[] UNKNOWN_3 = new byte[2924];
	
	//specified by SavEdit
	@Override
	public Vector<String> check(){
		Vector<String> response = new Vector<String>();
		String msg;
		if (FILENAME.length()>12){
			msg = Language.getString("GameInfo.0"); //$NON-NLS-1$
			msg = msg.replace("%1", getName()); //$NON-NLS-1$
			msg = msg.replace("%2", FILENAME); //$NON-NLS-1$
			response.add(msg);
		}
		if (GAME_NAME.length()>49){
			msg = Language.getString("GameInfo.1"); //$NON-NLS-1$
			msg = msg.replace("%1", getName()); //$NON-NLS-1$
			msg = msg.replace("%2", GAME_NAME); //$NON-NLS-1$
			response.add(msg);
		}
		if (TURNS<0){
			msg = Language.getString("GameInfo.2"); //$NON-NLS-1$
			msg = msg.replace("%1", getName()); //$NON-NLS-1$
			response.add(msg);
		}
		if (TURNS>9998){
			msg = Language.getString("GameInfo.3"); //$NON-NLS-1$
			msg = msg.replace("%1", getName()); //$NON-NLS-1$
			response.add(msg);
		}
		if (PLAYER_EMPIRE<0 || PLAYER_EMPIRE>4){
			msg = Language.getString("GameInfo.4"); //$NON-NLS-1$
			msg = msg.replace("%1", getName()); //$NON-NLS-1$
			msg = msg.replace("%2", Integer.toString(PLAYER_EMPIRE)); //$NON-NLS-1$
			response.add(msg);
		}
		short t = (short)Math.pow(2, PLAYER_EMPIRE);
		if (t!=EMPIRE_MASK){
			msg = Language.getString("GameInfo.5"); //$NON-NLS-1$
			msg = msg.replace("%1", getName()); //$NON-NLS-1$
			msg = msg.replace("%2", Integer.toString(EMPIRE_MASK)); //$NON-NLS-1$
			response.add(msg);
			EMPIRE_MASK = t;
		}
			
		return response;
	}
	@Override
	public long getSize(){
		return 3520;
	}
	@Override
	public void save(OutputStream out) throws IOException{
		//filename
		byte[] b = Tools.toByte(FILENAME, 13, 12);
		out.write(b);
		//name
		b = Tools.toByte(GAME_NAME, 51, 49);
		out.write(b);
		//turns
		b = Tools.toByte(TURNS, true);
		out.write(b);
		//unknown
		out.write(UNKNOWN_1);
		//multi
		int x;
		if (MP) {
			x = 1;
		} else {
			x = 0;
		}
		b = Tools.toByte(x, true);
		out.write(b);
		//unknown
		out.write(UNKNOWN_2);
		//empire mask 592-2
		out.write(Tools.toByte(EMPIRE_MASK, true));
		//player empire
		out.write(PLAYER_EMPIRE);
		//difficulty
		out.write(DIFFICULTY);
		//unknown
		out.write(UNKNOWN_3);
	}
	/**Constructor
	@param in the InputStream to read from*/
	public GameInfo(InputStream in) throws IOException{
		//filename (irrelevant, change has no effect) 0-13
		byte[] b = new byte[13];
		in.read(b);
		FILENAME = Tools.toASCII(b);
		//name 13-51
		b = new byte[51];
		in.read(b);
		GAME_NAME = Tools.toASCII(b);
		//turns 64-4
		b = new byte[4];
		in.read(b);
		TURNS = Tools.toInt(b, true);
		//unknown 68-8
		in.read(UNKNOWN_1);
		//multi 76-4
		in.read(b);
		int x = Tools.toInt(b, true);
		if (x==0) {
			MP = false;
		} else {
			MP = true;
		}
		//unknown 80-512
		in.read(UNKNOWN_2);
		//empire mask 592-2
		b = new byte[2];
		in.read(b);
		EMPIRE_MASK = Tools.toShort(b, true);
		//player empire 594-1
		PLAYER_EMPIRE = (byte)in.read();
		//player empire 595-1
		DIFFICULTY = (byte)in.read();
		//unknown 596-2925
		in.read(UNKNOWN_3);
	}
	//gets
	/**Return the name of the game.*/
	public String getGameName(){ return GAME_NAME;}
	/**Return the race index of the players empire.*/
	public byte getPlayerEmpire(){ return PLAYER_EMPIRE;}
	/**Return difficulty.*/
	public byte getDifficulty(){ return DIFFICULTY;}
	/**Returns turn number.*/
	public int getTurn(){ return (TURNS+1);}
	/**Returns true if the game is multiplayer.*/
	public boolean isMultiplayer(){ return MP;}
	//sets
	/**Sets the name of the game.*/
	public void setGameName(String name){
		if (name.length()>49) {
			name = name.substring( 0, 48);
		}
		if (GAME_NAME.equals(name)) {
			return;
		}
		GAME_NAME = name;
                this.markChanged();
	}
	/**Sets the turn number of the game.*/
	public void setTurn(int turn){
		turn--;
		if (turn<0) {
			turn = 0;
		}
		if (TURNS == turn) {
			return;
		}
		TURNS = turn;
                this.markChanged();
	}
	/**Sets the player empire.*/
	public void setPlayerEmpire(byte emp) throws InvalidArgumentsException{
		if (emp<0 || emp>4){
			String msg = Language.getString("GameInfo.6"); //$NON-NLS-1$
			msg = msg.replace("%1", Byte.toString(emp)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
		if (PLAYER_EMPIRE == emp) {
			return;
		}
		PLAYER_EMPIRE = emp;
		EMPIRE_MASK = (short)Math.pow(2, emp);
                this.markChanged();
	}
	/**Sets difficulty.*/
	public void setDifficulty(byte diff) throws InvalidArgumentsException{
		if (diff<0 || diff>4){
			String msg = Language.getString("GameInfo.7"); //$NON-NLS-1$
			msg = msg.replace("%1", Byte.toString(diff)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
		if (DIFFICULTY == diff) {
			return;
		}
		DIFFICULTY = diff;
                this.markChanged();
	}
	/**Sets if the game is multiplayer.*/
	public void setMultiplayer(boolean mp){
		if (MP==mp) {
			return;
		}
		MP = mp;
                this.markChanged();
	}
}