/*
	Author of the original algoritm written in C is Riky S <afax at afax.cjb.net>.
	
	File: LzssInputStream.java
	
	Modified by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.10.2007
*/

package ue.edit.sav;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;

// FIXME: add error messages

/**This InputStream is used to decompress read botf compressed data using the modified lzss algorithm.
This class was 'ported' from C code given by Riky S.*/
public class LzssInputStream extends InputStream{
	private InputStream IN;
	private ArrayList<Byte> DATA = new ArrayList<Byte>();
	private int INDEX = 0;
	
	private final int N = 4096;
	private final int F = 17;
	private int[] text_buf = new int[N + F - 1];
	private final int THRESHOLD = 1;

	/**This is a test constructor.*/
	public LzssInputStream(InputStream in, int[] text) throws IOException{
		text_buf = text;
		IN = in;
		
		decode(in);
	}
	
	public int[] getTextBuf(){ return text_buf;}
	
	/**Main constructor.*/
	public LzssInputStream(InputStream in) throws IOException{
		IN = in;
		decode(in);
	}
	
	//works like a charm :) thanks Riky, 'copied' this from lzss.c
	private void decode(InputStream in) throws IOException{
		//read and decode
		int  i, j, k, r, c, i2, j2, flags;
		
		for (i = 0; i < N - F; i++) {
			text_buf[i] = 0;
		}
		r = N - F;  flags = 0;
		
		while (in.available() > 0) {
			if (((flags >>= 1) & 256) == 0) {
				try{
					c = in.read();
				}catch (Exception e){
					 break;
				}
				flags = c | 0xff00;		/* uses higher byte cleverly */
			}							/* to count eight */
			if ((flags & 1) != 0) {
				try{
					c = in.read();
				}catch (Exception e){
					 break;
				}
//                                DATA.add(c);
                                if (c>127){
					DATA.add(new Byte(Integer.toString(c-256)));
					text_buf[r++] = Byte.parseByte(Integer.toString(c-256));
				}else{
					DATA.add(new Byte(Integer.toString(c)));  
					text_buf[r++] = Byte.parseByte(Integer.toString(c));
				}
				r &= (N - 1);
			} else {
				try{
					i2 = in.read();
					j2 = in.read();
				}catch (Exception e){
					break;
				}
				i = (((i2 & 0x0f) << 8) | j2) + N - F - 1;
				j = ((i2 & 0xf0) >> 4) + THRESHOLD;

				for (k = 0; k <= j; k++) {
					c = text_buf[(i + k) & (N - 1)];
//                                        DATA.add(c);
					if (c>127){
						DATA.add(new Byte(Integer.toString(c-256)));
						text_buf[r++] = Byte.parseByte(Integer.toString(c-256));
					}else{
						DATA.add(new Byte(Integer.toString(c)));  
						text_buf[r++] = Byte.parseByte(Integer.toString(c));
					}
					r &= (N - 1);
				}
			}
		}
		
	}
	
	/**Returns the number of bytes avalable in this stream.*/
	@Override
	public int available() throws IOException{
		try{
			if (DATA.isEmpty()) {
				return -1;
			}
			if (DATA.size() - INDEX <= 0) {
				return -1;
			}
			return (DATA.size() - INDEX);
		}catch (Exception e){
			throw new IOException();
		}
	}
	
	/**Closes this and the underlying stream.*/
	@Override
	public void close() throws IOException{ IN.close(); DATA = null;}
	
	/**Not supported.*/
	@Override
	public boolean markSupported() { return false;}
	
	@Override
	public int read() throws IOException{
		try{
			return (DATA.get(INDEX) < 0) ? DATA.get(INDEX++).intValue() + 256 : DATA.get(INDEX++).intValue();
		}catch (Exception e){
			throw new IOException();
		}
	}
	
	@Override
	public int read(byte[] b) throws IOException{
		try{
			int i;
			for (i = 0; i<b.length && available()>0; i++){
				b[i] = Integer.valueOf(read()).byteValue();
			}
			return i;
		}catch (Exception e){
			throw new IOException();
		}
	}
	
	@Override
	public int read(byte[] b, int off, int len) throws IOException{
		try{
			int i;
			for (i = 0; i < b.length - off && i < len && available()>0; i++){
				b[off + i] = Integer.valueOf(read()).byteValue();
			}
			return i;
		}catch (Exception e){
			throw new IOException();
		}
	}
	
	@Override
	public long skip(long n) throws IOException{ 
		try{
			int i;
			for (i = 0; i < n && available()>0; i++){ INDEX++; }
			return i;
		}catch (Exception e){
			throw new IOException();
		}
	}
}