/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: MixInputStream.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.10.2007
*/

package ue.edit.sav;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import ue.Language;
import ue.util.Tools;

/**
For files partly encoded.
Known files that use this partial compression:
result.lst
*/
public class MixInputStream extends InputStream{
	private boolean closed = false;
	private byte[] DATA;
	private int INDEX = 0;
	
	public MixInputStream(InputStream in) throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		SavSubEntry sav;

		int old_address = 0;
		int new_address = 0;
		int size = 0;
		byte[] b = new byte[4];
		
		while (in.available() > 0){
			in.read(b);
			new_address = Tools.toInt(b, true);
			
			if (new_address - old_address != size + 12) {
				throw new IOException(Language.getString("MixInputStream.0")); //$NON-NLS-1$
			}
			
			sav = new SavSubEntry(in, true);

			size = sav.getSize();
			old_address = new_address;
			
			sav.decompress();
			sav.store(baos);			
		}
		
		DATA = baos.toByteArray();
		
		in.close();
	}
	public boolean closed(){ return closed;}
	@Override
	public int available() throws IOException{
		checkClosed();
		int rev = DATA.length - INDEX;
		if (rev <= 0) {
			return -1;
		} else {
			return rev;
		}
	}

	@Override
	public void close() throws IOException{
		checkClosed();
		DATA = null; // bye bye data
		closed = true;
	}
	@Override
	public void mark(int readlimit){}
	@Override
	public boolean markSupported(){ return false;}
	@Override
	public int read() throws IOException{
		checkClosed();
		checkEOF();
		byte b = DATA[INDEX++];
		return b;
	}
	@Override
	public int read(byte[] b) throws IOException{
		checkClosed();
		checkEOF();
		int i;
		for (i = 0; i < b.length && available()>0; i++){
			b[i] = DATA[INDEX++];
		}
		return i;
	}

	@Override
	public int read(byte[] b, int off, int len) throws IOException{
		checkClosed();
		checkEOF();
		if (available() < len) {
			len = available();
		}
		int i;
		for ( i = off; i < b.length - off && i < len + off; i++){
			b[i] = DATA[INDEX++];
		}
		return i;
	}
	  
	@Override
	public void reset() throws IOException{ 
		throw new IOException(Language.getString("MixInputStream.1")); //$NON-NLS-1$
	}
	
	@Override
	public long skip(long n) throws IOException{
		checkClosed();
		checkEOF();
		if (available() < n) {
			n = available();
		}
		INDEX += n;
		return n;
	}
	
	private void checkClosed() throws IOException{
		if (closed){
			throw new IOException(Language.getString("MixInputStream.2")); //$NON-NLS-1$
		}
	}
	
	private void checkEOF() throws IOException{
		if (available()<=0){
			if (available()<=0) {
				throw new IOException(Language.getString("MixInputStream.3")); //$NON-NLS-1$
			}
		}
	}
}