/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: MixOutputStream.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.10.2007
*/

package ue.edit.sav;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import ue.Language;
import ue.util.Tools;

/**
For files partly encoded.
Known files that use this compression:
result.lst
*/
public class MixOutputStream extends OutputStream{
	private ByteArrayOutputStream DATA = new ByteArrayOutputStream();
	private boolean closed = false;
	private int entry_num = 1;
	private SavOutputStream out;
	
	public MixOutputStream(SavOutputStream out){
		this.out = out;
	}
	@Override
	public void close() throws IOException{
		checkClosed();

		ByteArrayInputStream bais = new ByteArrayInputStream(DATA.toByteArray());
		
		int address = 12;
		
		SavSubEntry sav;
		entry_num = 0;
		while (bais.available()>0){
			sav = new SavSubEntry(bais, false);
			sav.compress();
			
			out.write(Tools.toByte(address, true));
			sav.store(out);
			
			address += sav.getSize() + 12;
			entry_num++;
		}
		
		DATA.close(); bais.close();
		out.setSubEntries(entry_num);
		out.close();
		closed = true;
	}
	@Override
	public void flush() throws IOException{
		checkClosed();
	}
	
	@Override
	public void write(byte[] b) throws IOException{
		checkClosed();
		DATA.write(b);
	}
	@Override
	public void write(byte[] b, int off, int len) throws IOException{
		checkClosed();
		DATA.write(b, off, len);
	}
	@Override
	public void write(int b) throws IOException{
		checkClosed();
		DATA.write(b);
	}
	public boolean closed() { return closed; }
	
	private void checkClosed() throws IOException{
		if (closed){
			throw new IOException(Language.getString("MixOutputStream.0")); //$NON-NLS-1$
		}
	}
}
