/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: SavEntry.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.10.2007
*/

package ue.edit.sav;

/**This class reprisents an entry in a saved game.
It holds all data used for the map. Inspired by ZipEntry ^_^*/
public class SavEntry{
	//data
	private String name = ""; //entry name //$NON-NLS-1$
	private int address = 0; //address
	private int index = 0; //index - handled by SavFile
	private int uncom = 0; //uncompressed size
	private int com = 0; //compressed size
	private int subentries = 1; // number of compressed subentries
	private byte[] unknown = new byte[4];
	/*constructors*/
	/**@param n the name of the entry.*/
	public SavEntry(String n){ name = n; }
	/*set*/
	/**Sets the name of the entry.*/
	public void setName(String n){ name = n; }
	/**Sets the address of the entry.*/
	public void setAddress(int n){ address = n; }
	/**Sets the index of the entry.*/
	public void setIndex(int n){ index = n; }
	/**Sets the uncompressed size of the entry.*/
	public void setUncompressedSize(int n){ uncom = n; }
	/**Sets the compressed size of the entry.*/
	public void setCompressedSize(int n){ com = n; }
	/**Sets thenumber of sub entries.*/
	public void setSubEntries(int n){ subentries = n; }
	/**Sets the unknown data of the entry.*/
	public void setUnknown(byte[] n){
		if (n.length == 4){
			unknown = n;
		}else{
			unknown = new byte[4];
		}
	}
	//get
	/**Returns the name of the entry.*/
	public String getName(){ return name; }
	/**Returns the address of the entry.*/
	public int getAddress(){ return address; }
	/**Returns the index of the entry.*/
	public int getIndex(){ return index; }
	/**Returns the uncompressed size of the entry.*/
	public int getUncompressedSize(){ return uncom; }
	/**Returns the compressed size of the entry.*/
	public int getCompressedSize(){ return com; }
	/**Returns the number of subentries.*/
	public int getSubEntries(){ return subentries; }
	/**Returns the unknown data of the entry.*/
	public byte[] getUnknown(){ return unknown;}
	//other
	@Override
	public boolean equals(Object t){
		if (t instanceof SavEntry){
			SavEntry ot = (SavEntry)t;
			if ((ot.getName()).equals(name)) {
				return true;
			}
		}
		return false;
	}

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 43 * hash + (this.name != null ? this.name.hashCode() : 0);
        return hash;
    }
	@Override
	public String toString(){ return new String(name);}
}
