/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: SavFile.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.10.2007
*/

package ue.edit.sav;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;

import ue.Language;
import ue.exception.InvalidArgumentsException;
import ue.exception.SavException;
import ue.util.Tools;

/**This class is used to process saved games files.*/
public class SavFile{
	//open source, files, read from it
	//open dest file, write to it.
	//when closing, rewrite header, replace file if required
	private File SOURCE = null;
	private File DEST = null;
	private OutputStream D_OUTPUT = null; //destination, opened until file is closed
	private SavOutputStream E_OUTPUT = null; //one for each entry
	private InputStream S_INPUT = null; //source, opened only when read
	private String S_SAVNAME;
	private String D_SAVNAME;
	private int S_MAPADDRESS;
	private int D_MAPADDRESS;
	private SavEntry CURRENT_ENTRY = null;
	private ArrayList<SavEntry> S_ENTRY = new ArrayList<SavEntry>();
	private ArrayList<SavEntry> D_ENTRY = new ArrayList<SavEntry>();
	
	/**Opens a *.sav file or creates one if the file does not exist yet.
	If the file already exists the name argument is not needed (can be null).*/
	public SavFile(File go, String n) throws IOException, InvalidArgumentsException{
		//if exist
		if (go.exists()){
			SOURCE = go;
			S_INPUT = new FileInputStream(SOURCE);
			//name 16
			byte[] b = new byte[16];
			S_INPUT.read(b);
			S_SAVNAME = Tools.toASCII(b, 0, 16);
			D_SAVNAME = new String(S_SAVNAME);
			//map address 4
			b = new byte[4];
			S_INPUT.read(b);
			S_MAPADDRESS = Tools.toInt(b, true);
			//number of map entries 4
			S_INPUT.read(b);
			int ENTRIES = Tools.toInt(b, true); //no need for making this global
			//entries
			SavEntry gen;
			String name;
			int ad;
			int un;
			int co;
			int en;
			byte[] unknown;
			S_INPUT.skip(S_MAPADDRESS - 24);
			for (int i = 0; i < ENTRIES; i++){
				unknown = new byte[4];
				//name 16
				b = new byte[16];
				S_INPUT.read(b);
				name = Tools.toASCII(b, 0, 16);
				gen = new SavEntry(name);
				//address 4
				b = new byte[4];
				S_INPUT.read(b);
				ad = Tools.toInt(b, true);
				gen.setAddress(ad);
				//index 4
				gen.setIndex(i);
				S_INPUT.skip(4);
				//uncompressed 4
				S_INPUT.read(b);
				un = Tools.toInt(b, true);
				gen.setUncompressedSize(un);
				//compressed 4
				S_INPUT.read(b);
				co = Tools.toInt(b, true);
				gen.setCompressedSize(co);
				//subentries 4
				S_INPUT.read(b);
				en = Tools.toInt(b, true);
				gen.setSubEntries(en);
				//unknown 4
				S_INPUT.read(unknown);
				gen.setUnknown(unknown);
				//add to set
				S_ENTRY.add(gen);
			}
			S_INPUT.close();
		}else{
			//set up
			D_SAVNAME = n;
			if (D_SAVNAME == null){
				throw new InvalidArgumentsException(Language.getString("SavFile.0")); //$NON-NLS-1$
			}
			D_MAPADDRESS = 24;
			int ENTRIES = 0;
			//create
			DEST = go;
			DEST.createNewFile();
			D_OUTPUT = new FileOutputStream(DEST);
			SOURCE = null;
			//write
			 //name
			byte[] writeBuffer = Tools.toByte(D_SAVNAME, 16, 15);
			D_OUTPUT.write(writeBuffer);
			 //values
			byte[] b = Tools.toByte(D_MAPADDRESS, true);
			D_OUTPUT.write(b);
			b = Tools.toByte(ENTRIES, true);
			D_OUTPUT.write(b);
			//ok, I'm ready now
		}
	}
	/**Returns the internal name of the saved game.*/
	public String getName(){ return D_SAVNAME; }
	/**Sets the internal name of the saved game.*/
	public void setName(String n) throws InvalidArgumentsException{
		if (n.length() > 15 || n.length() < 1){
			String msg = Language.getString("SavFile.1"); //$NON-NLS-1$
			msg = msg.replace("%1", n); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}else{
			D_SAVNAME = n;
		}
	}
	/**Returns an enumeration of all files.*/
	public Enumeration<SavEntry> entries(){
		return new Enume(S_ENTRY.listIterator(0));
	}
	
	private class Enume implements Enumeration<SavEntry>{
		private Iterator<SavEntry> it;
		public Enume(Iterator<SavEntry> i){ it = i; }
        @Override
		public boolean hasMoreElements(){
			return it.hasNext();
		}
        @Override
		public SavEntry nextElement(){
			return it.next();
		}
	}
	
	/**Adds a new SavEntry to the file.*/
	public void putNextEntry(SavEntry se) throws SavException{
		if (E_OUTPUT != null){
			throw new SavException(Language.getString("SavFile.2")); //$NON-NLS-1$
		}
		if (D_ENTRY == null){
			throw new SavException(getSavFileClosedString());
		}
		if (D_ENTRY.contains(se)){
			String msg = Language.getString("SavFile.3"); //$NON-NLS-1$
			msg = msg.replace("%1", se.toString()); //$NON-NLS-1$
			throw new SavException(msg);
		}else{
			CURRENT_ENTRY = se;
			D_ENTRY.add(se);
		}
	}
	/**Returns an OutputStream used to write entry data.*/
	public OutputStream getOutputStream(SavEntry se) throws SavException, IOException{
		return getOutputStream(se, true);
	}
	/**Returns an OutputStream used to write entry data.*/
	public OutputStream getOutputStream(SavEntry se, boolean comp) throws SavException, IOException{
		if (D_ENTRY == null){
			throw new SavException(getSavFileClosedString());
		}
		if (!D_ENTRY.contains(se)){
			getFileNotFoundString(se);
		}
		if (S_INPUT != null){
			S_INPUT.close();
		}
		if (E_OUTPUT!=null){
			throw new SavException(Language.getString("SavFile.4")); //$NON-NLS-1$
		}
		int index = D_ENTRY.indexOf(se);
		if (index!=D_ENTRY.size()-1){
			String msg = Language.getString("SavFile.5"); //$NON-NLS-1$
			msg = msg.replace("%1", se.toString()); //$NON-NLS-1$
			throw new SavException(msg);
		}
		E_OUTPUT = new SavOutputStream(D_OUTPUT, se.getSubEntries());
		if (comp){
			OutputStream l;
			//care for a mix?
			if ((se.getName()).equals(SavGame.fileRESULT_LST)){
				l = new MixOutputStream(E_OUTPUT);
			}else{
				l = new LzssOutputStream(E_OUTPUT);
			}
			return l;
		}else{
			return E_OUTPUT;
		}
	}
	/**Returns an InputStream used to read SavEntry data.*/
	public InputStream getInputStream(SavEntry se) throws SavException, IOException{
		return getInputStream(se, true);
	}
	/**Returns an InputStream used to read SavEntry data.*/
	public InputStream getInputStream(SavEntry se, boolean decom) throws SavException, IOException{
		if (D_ENTRY == null){
			throw new SavException(getSavFileClosedString());
		}
		if (!S_ENTRY.contains(se)){
			getFileNotFoundString(se);
		}
		int index = S_ENTRY.indexOf(se);
		SavEntry sev = S_ENTRY.get(index);
		int destAddress = sev.getAddress();
		int length = sev.getCompressedSize();
		
		S_INPUT = new FileInputStream(SOURCE);
		S_INPUT.skip(destAddress);

		if (decom){
			InputStream l;
			//care for a mix?
			if (sev.getSubEntries()!=1){
				l = new MixInputStream(new SavInputStream(S_INPUT, length));
			}else{
				l = new LzssInputStream(new SavInputStream(S_INPUT, length));
			}
			return l;
		}else{
			return new SavInputStream(S_INPUT, length);
		}
	}
	/**Closes the currently opened entry.*/
	public void closeEntry() throws SavException, IOException{
		if (D_ENTRY == null){
			throw new SavException(getSavFileClosedString());
		}
		if (E_OUTPUT == null){
			throw new SavException(Language.getString("SavFile.6")); //$NON-NLS-1$
		}
		E_OUTPUT.close();
		
		CURRENT_ENTRY.setCompressedSize(E_OUTPUT.getLength());
		CURRENT_ENTRY.setSubEntries(E_OUTPUT.getSubEntries());
		E_OUTPUT = null;
		CURRENT_ENTRY = null;
		D_OUTPUT.flush();
	}
	/**Returns true if there's an entry with the name in the file.*/
	public boolean contains(String n) throws SavException{
		if (S_ENTRY == null){
			throw new SavException(getSavFileClosedString());
		}
		SavEntry se = new SavEntry(n);
		return S_ENTRY.contains(se);
	}
	/**Flushes all data to the file and closes it.*/
	public void close() throws IOException{
		if (D_ENTRY.isEmpty()){
			//close all
			if (D_OUTPUT!=null) {
				D_OUTPUT.close();
			}
			if (S_INPUT!=null) {
				S_INPUT.close();
			}
			D_OUTPUT = null; //destination, opened until file is closed
			E_OUTPUT = null; //one for each entry
			S_INPUT = null; //source, opened only when read
			S_ENTRY = null;
			D_ENTRY = null;
			//delete first temp
			if (DEST!=null) {
				DEST.delete();
			}
			return;
		}
		if (SOURCE!=null){
			String msg = Language.getString("SavFile.7"); //$NON-NLS-1$
			msg = msg.replace("%1", SOURCE.getName()); //$NON-NLS-1$
			throw new IOException(msg);
		}
		if (D_ENTRY == null){
			throw new IOException(getSavFileClosedString());
		}
		//write map
		Iterator<SavEntry> it = D_ENTRY.iterator();
		SavEntry sev;
		int index = 0;
		while (it.hasNext()){
			sev = it.next();
			//name
			byte[] b = Tools.toByte(sev.getName(), 16, 15);
			D_OUTPUT.write(b);
			//address
			sev.setAddress(D_MAPADDRESS);
			D_OUTPUT.write(Tools.toByte(D_MAPADDRESS, true));
			//index
			sev.setIndex(index);
			D_OUTPUT.write(Tools.toByte(index++, true));
			//uncompressed
			D_OUTPUT.write(Tools.toByte(sev.getUncompressedSize(), true));
			//compressed
			D_OUTPUT.write(Tools.toByte(sev.getCompressedSize(), true));
			D_MAPADDRESS += sev.getCompressedSize();
			// subentries
			D_OUTPUT.write(Tools.toByte(sev.getSubEntries(), true));			
			//unknown
			D_OUTPUT.write(sev.getUnknown());
		}
		D_OUTPUT.close();
		//rewrite header
		File xDEST = new File("xsav.tmp"); //$NON-NLS-1$
		D_OUTPUT = new FileOutputStream(xDEST);
		D_OUTPUT.write(Tools.toByte(D_SAVNAME, 16, 15));
		D_OUTPUT.write(Tools.toByte(D_MAPADDRESS, true));
		D_OUTPUT.write(Tools.toByte(D_ENTRY.size(), true));
		//copy all info
		S_INPUT = new FileInputStream(DEST);
		S_INPUT.skip(24);
		byte[] b = new byte[1024];
		while (S_INPUT.available()>0){
		        Arrays.fill(b, (byte)0);
			S_INPUT.read(b);
			D_OUTPUT.write(b);
		}
		//close all
		D_OUTPUT.close();
		S_INPUT.close();
		D_OUTPUT = null; //destination, opened until file is closed
		E_OUTPUT = null; //one for each entry
		S_INPUT = null; //source, opened only when read
		S_ENTRY = null;
		D_ENTRY = null;
		//delete first temp
		DEST.delete();
		//copy
		xDEST.renameTo(DEST);
	}
	
	public Vector<String> check() throws Exception{
		Vector<String> response = new Vector<String>();
		String msg;
		//check
		Iterator<SavEntry> it = S_ENTRY.iterator();
		boolean first = true;
		SavEntry fi = null;
		SavEntry sec;
		while (it.hasNext()){
			if (first){
				fi = it.next();
			}else{
				sec = it.next();
				
				int fblock = fi.getAddress() + fi.getCompressedSize();
				if (fblock > sec.getAddress() && fblock <= (sec.getAddress() + sec.getCompressedSize())){
					if (sec.getUncompressedSize()!=0 || sec.getCompressedSize()!=0){
						msg = Language.getString("SavFile.8"); //$NON-NLS-1$
						msg = msg.replace("%1", SOURCE.getName()); //$NON-NLS-1$
						msg = msg.replace("%2", fi.getName()); //$NON-NLS-1$
						msg = msg.replace("%3", sec.getName()); //$NON-NLS-1$
						response.add(msg);
					}
				}
				fi = sec;
			}
			first = false;
		};
		if (first){
			msg = Language.getString("SavFile.9"); //$NON-NLS-1$
			msg = msg.replace("%1", SOURCE.getName()); //$NON-NLS-1$
			response.add(msg);
		}
		return response;
	}
	/**Returns the SavEntry with the given name or null if not found.*/
	public SavEntry getEntry(String n) throws SavException{
		if (S_ENTRY == null){
			throw new SavException(getSavFileClosedString());
		}
		SavEntry se = new SavEntry(n);
		Iterator<SavEntry> it = S_ENTRY.iterator();
		boolean found = false;
		SavEntry sea = null;
		while (it.hasNext() && !found){
			sea = it.next();
			if (sea.equals(se)) {
				found = true;
			}
		}
		if (found) {
			return sea;
		} else {
			return null;
		}
	}
	
	private String getSavFileClosedString(){
		return Language.getString("SavFile.10"); //$NON-NLS-1$
	}
	
	private String getFileNotFoundString(SavEntry se){
		String msg = Language.getString("SavFile.11"); //$NON-NLS-1$
		msg = msg.replace("%1", se.toString()); //$NON-NLS-1$
		return msg;
	}
}
