/*
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2007  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *
 * File: SavGame.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Last modified: 6.10.2007
 */
package ue.edit.sav;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.MessageDigest;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.SwingWorker;

import ue.Language;
import ue.UE;
import ue.edit.CanBeEdited;
import ue.edit.FileManager;
import ue.edit.FilesInterface;
import ue.edit.InternalFile;
import ue.edit.exe.Trek;
import ue.edit.mod.ModList;
import ue.edit.res.Stbof;
import ue.edit.sav.lst.Result;
import ue.edit.sav.lst.Sector;
import ue.gui.CheckGUI;
import ue.gui.FilesGUI;
import ue.gui.MainPanel;
import ue.gui.sav.EmpireInfoGUI;
import ue.gui.sav.GameInfoGUI;
import ue.gui.sav.MapGUIv2;
import ue.gui.sav.StarBaseInfoGUI;
import ue.gui.sav.StellInfoGUI;
import ue.gui.sav.StrcInfoGUI;
import ue.gui.sav.SystemsInfoGUI;
import ue.util.GenericFile;
import ue.util.StringGroup;
import ue.util.Tools;

/**This class is a middle-man between the GUI and sav files. Opens, edits, saves saved games.*/
public class SavGame implements CanBeEdited {

    /**A list of at least partialy editable files.*/
    //CAREFULL!!! only add new files to the end of the list!
    private static String[] FILELIST;
    /**A list of valid commands to use with the edit function.*/
    //if you add a command be sure to change the edit, save, getInternalFile as well
    //CAREFULL!!! add new only to the end of the list!
    private static String[] COMMANDLIST;
    //a hash table of all objects
    private Hashtable<String, InternalFile> INTERNAL_FILES;
    private HashSet<String> REMOVED;
    //file opened
    private File SAV_FILE = null;
    //other
    private Stbof STBOF = null;
    private Trek EXE = null;
    private boolean CANCELLED_SAVE = false;
    // list of known files
    public final static String fileGAMEINFO = "gameInfo"; //$NON-NLS-1$
    public final static String fileRESULT_LST = "result.lst"; //$NON-NLS-1$
    public final static String fileSECTOR_LST = "sector.lst"; //$NON-NLS-1$
    public final static String fileSTARBASEINFO = "starBaseInfo"; //$NON-NLS-1$
    public final static String fileSTELLINFO = "stellInfo"; //$NON-NLS-1$
    public final static String fileSYSTINFO = "systInfo"; //$NON-NLS-1$
    public final static String fileGALINFO = "galInfo"; //$NON-NLS-1$
    public final static String fileEMPSINFO = "empsInfo"; //$NON-NLS-1$
    public final static String fileGSHIPHEAD = "GShipHead"; //$NON-NLS-1$
    public final static String fileGTFORCEHD = "GTForceHd"; //$NON-NLS-1$
    public final static String fileGWTFORCEHD = "GWTForceHd"; //$NON-NLS-1$
    public final static String fileGTFSTASISHD = "GTFStasisHd"; //$NON-NLS-1$
    public final static String fileGTFORCELIST = "GTForceList"; //$NON-NLS-1$
    public final static String fileGWTFORCE = "GWTForce"; //$NON-NLS-1$
    public final static String fileORDINFO = "ordInfo"; //$NON-NLS-1$
    public final static String fileGSHIPLIST = "GShipList"; //$NON-NLS-1$
    public final static String fileSTRCINFO = "strcInfo"; //$NON-NLS-1$
    public final static String fileALIENINFO = "alienInfo"; //$NON-NLS-1$
    public final static String fileTECHINFO = "techinfo"; //$NON-NLS-1$
    public final static String fileSPHOFIN0 = "SphOfIn0"; //$NON-NLS-1$
    public final static String fileSPHOFIN1 = "SphOfIn1"; //$NON-NLS-1$
    public final static String fileSPHOFIN2 = "SphOfIn2"; //$NON-NLS-1$
    public final static String fileSPHOFIN3 = "SphOfIn3"; //$NON-NLS-1$
    public final static String fileSPHOFIN4 = "SphOfIn4"; //$NON-NLS-1$

    // list of commands
    public static String cmdCHECK = Language.getString("SavGame.0"); //$NON-NLS-1$
    public static String cmdGAME_INFO = Language.getString("SavGame.1"); //$NON-NLS-1$
    public static String cmdFILES = Language.getString("SavGame.2"); //$NON-NLS-1$
    public static String cmdMAPv2 = Language.getString("SavGame.3"); //$NON-NLS-1$
    public static String cmdSTARBASES = Language.getString("SavGame.4"); //$NON-NLS-1$
    public static String cmdSTELLAR_OBJ = Language.getString("SavGame.5"); //$NON-NLS-1$
    public static String cmdSYSTEM_INFO = Language.getString("SavGame.6"); //$NON-NLS-1$
    public static String cmdEMPIRE_INFO = "Empire info";
    public static String cmdSTRCINFO = "Lists of built structures";

    /**
     * Creates an instance of this class.
     */
    public SavGame() {
        FILELIST = new String[]{fileSYSTINFO, fileSTELLINFO,
                    fileSTARBASEINFO, fileSECTOR_LST, fileGAMEINFO,
                    fileGALINFO, fileEMPSINFO, fileRESULT_LST,
                    fileGSHIPHEAD, fileGTFORCEHD, fileGWTFORCEHD, fileGTFSTASISHD,
                    fileGTFORCELIST, fileGWTFORCE, fileORDINFO, fileGSHIPLIST,
                    fileSTRCINFO, fileALIENINFO, fileTECHINFO, fileSPHOFIN0,
                    fileSPHOFIN1, fileSPHOFIN2, fileSPHOFIN3, fileSPHOFIN4};
        // check Strings.java on how commands are translated
        COMMANDLIST = new String[]{cmdFILES, cmdCHECK,
                    cmdSYSTEM_INFO, cmdSTELLAR_OBJ, cmdSTARBASES,
                    cmdGAME_INFO, cmdEMPIRE_INFO, cmdMAPv2, cmdSTRCINFO};
        INTERNAL_FILES = new Hashtable<String, InternalFile>(FILELIST.length);
        REMOVED = new HashSet<String>();
    }

    @Override
    public void markChanged() {
        CANCELLED_SAVE = true;
    }

    /**
     *  This funtion expects a translated command to be passed. for translation use UE.TRANSLATOR
     *  @param command Must be one of the options returned by the open function else null is returned.
     *  @return JPanel object or null
     */
    @Override
    public MainPanel edit(String command) {
        try {
            int z = -1;

            for (int i = 0; i < COMMANDLIST.length; i++) {
                if (command.equals(COMMANDLIST[i])) {
                    z = i;
                }
            }

            switch (z) {
                case 0: {
                    return new FilesGUI(new FilesInterface(this));
                }
                case 1: {
                    return new CheckGUI(check());
                }
                case 2: {
                    return new SystemsInfoGUI(this, STBOF, EXE);
                }
                case 3: {
                    return new StellInfoGUI((StellInfo) getInternalFile(fileSTELLINFO, true));
                }
                case 4: {
                    return new StarBaseInfoGUI((StarBaseInfo) getInternalFile(fileSTARBASEINFO, true),
                            STBOF);
                }
                case 5: {
                    return new GameInfoGUI((GameInfo) getInternalFile(fileGAMEINFO, true), STBOF);
                }
                case 6: {
                    return new EmpireInfoGUI(STBOF, this);
                }
                case 7: {
                    return new MapGUIv2(STBOF, this, EXE);
                }
                case 8:
                {
                    return new StrcInfoGUI(this, STBOF, EXE);
                }
            }
        } catch (Exception e) {
            UE.displayError(e);
        }
        return null;
    }

    /**Saves changes to the specified file.
     * @param whereTo The file to which changes are to be saved.
     * @return true if succesful else false.
     */
    @Override
    public boolean save(File whereTo, boolean bkp, String ext) throws Exception
    {
        CANCELLED_SAVE = false;
        
        if (whereTo == null)
        {
            whereTo = new File(SAV_FILE.getAbsolutePath());
        }

        // FIX ME: Use UEWorker
        final SaveTask TSK = new SaveTask(whereTo, bkp, ext);

        final JOptionPane opti = new JOptionPane(getStartingTaskString(),
                JOptionPane.PLAIN_MESSAGE);
        opti.setOptions(new String[]{getCancelString()});
        final JDialog dia = opti.createDialog(UE.WINDOW, UE.APP_NAME);

        TSK.addPropertyChangeListener(new PropertyChangeListener() {

            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                if (new String().equals(evt.getPropertyName())) {
                    opti.setMessage(evt.getNewValue());
                }
                if (TSK.isDone()) {
                    dia.dispose();
                }
            }
        });

        TSK.execute();

        dia.setVisible(true);

        if (getCancelString().equals(opti.getValue())) {
            TSK.cancel(false);
            CANCELLED_SAVE = true;
        }

        if (!CANCELLED_SAVE) {
            this.SAV_FILE = whereTo;
        }

        return !CANCELLED_SAVE;
    }

    /**
    @param whereFrom The file to open. Must be a valid saved game.
    @return a String array of valid command that can be used with the edit function.*/
    @Override
    public StringGroup[] open(File whereFrom) {
        if (UE.FILES.getFile(FileManager.P_STBOF) != null) {
            STBOF = (Stbof) UE.FILES.getFile(FileManager.P_STBOF);
        }

        if (UE.FILES.getFile(FileManager.P_TREK) != null) {
            EXE = (Trek) UE.FILES.getFile(FileManager.P_TREK);
        }

        SAV_FILE = whereFrom;
        try {
            SavFile sf = new SavFile(SAV_FILE, null);

            for (int i = 0; i < FILELIST.length; i++) {
                if (!sf.contains(FILELIST[i])) {
                    UE.displayError(new FileNotFoundException(getFileNotFoundString(FILELIST[i])));
                    sf.close();
                    SAV_FILE = null;
                    return null;
                }
            }
            sf.close();
        } catch (Exception e) {
            SAV_FILE = null;
            UE.displayError(e);
        }

        return getCommands();
    }

    /**@return a String array of available commands that can be used with the edit function.*/
    @Override
    public StringGroup[] getCommands() {
        if (SAV_FILE == null || STBOF == null) {
            return null;
        }
        cmdCHECK = Language.getString("SavGame.0"); //$NON-NLS-1$
        cmdGAME_INFO = Language.getString("SavGame.1"); //$NON-NLS-1$
        cmdFILES = Language.getString("SavGame.2"); //$NON-NLS-1$
        cmdMAPv2 = Language.getString("SavGame.3"); //$NON-NLS-1$
        cmdSTARBASES = Language.getString("SavGame.4"); //$NON-NLS-1$
        cmdSTELLAR_OBJ = Language.getString("SavGame.5"); //$NON-NLS-1$
        cmdSYSTEM_INFO = Language.getString("SavGame.6"); //$NON-NLS-1$
        cmdEMPIRE_INFO = "Empire info";
        cmdSTRCINFO = "Lists of built structures";

        COMMANDLIST = new String[]{cmdFILES, cmdCHECK,
                    cmdSYSTEM_INFO, cmdSTELLAR_OBJ, cmdSTARBASES,
                    cmdGAME_INFO, cmdEMPIRE_INFO, cmdMAPv2, cmdSTRCINFO};

        StringGroup[] ret = new StringGroup[5];
        try {
            // Galaxy
            ret[0] = new StringGroup(Language.getString("SavGame.7"), //$NON-NLS-1$
                    new String[]{cmdSTELLAR_OBJ, cmdMAPv2});
            // Game
            ret[1] = new StringGroup(Language.getString("SavGame.8"), new String[]{cmdGAME_INFO, cmdEMPIRE_INFO}); //$NON-NLS-1$
            // Other
            ret[2] = new StringGroup(Language.getString("SavGame.9"), //$NON-NLS-1$
                    new String[]{cmdCHECK, cmdFILES});
            // Ships
            ret[3] = new StringGroup(Language.getString("SavGame.10"), new String[]{cmdSTARBASES}); //$NON-NLS-1$
            // System
            ret[4] = new StringGroup(Language.getString("SavGame.11"), new String[]{cmdSYSTEM_INFO, cmdSTRCINFO}); //$NON-NLS-1$
        } catch (Exception tz) {
            UE.displayError(tz);
            return null;
        }
        return ret;
    }

    /**Returns the internal file object.
    @param fileName the name of the file*/
    @Override
    public InternalFile getInternalFile(String fileName, boolean cache) throws FileNotFoundException {
        if (REMOVED.contains(fileName)) {
            throw new FileNotFoundException(getFileNotFoundString(fileName));
        }
        if (INTERNAL_FILES.containsKey(fileName)) {
            return INTERNAL_FILES.get(fileName);
        } else {
            InternalFile a = null;
            int z = -1;

            for (int i = 0; i < FILELIST.length; i++) {
                if (FILELIST[i].equals(fileName)) {
                    z = i;
                }
            }

            try {
                SavFile zf = new SavFile(SAV_FILE, null);
                SavEntry ze;
                if (z >= 0) {
                    ze = zf.getEntry(FILELIST[z]);
                } else {
                    ze = new SavEntry(fileName);
                }

                switch (z) {
                    case 0: {
                        a = new SystInfo(zf.getInputStream(ze), STBOF);
                        break;
                    }
                    case 1: {
                        a = new StellInfo(zf.getInputStream(ze), STBOF);
                        break;
                    }
                    case 2: {
                        a = new StarBaseInfo(zf.getInputStream(ze));
                        break;
                    }
                    case 3: {
                        a = new Sector(zf.getInputStream(ze), this);
                        break;
                    }
                    case 4: {
                        a = new GameInfo(zf.getInputStream(ze));
                        break;
                    }
                    case 5: {
                        a = new GalInfo(zf.getInputStream(ze));
                        break;
                    }
                    case 6: {
                        a = new EmpsInfo(fileName.toLowerCase(), zf.getInputStream(ze));
                        break;
                    }
                    case 7: {
                        a = new Result(zf.getInputStream(ze));
                        break;
                    }
                    case 8:
                    case 9:
                    case 10:
                    case 11:
                    {
                        a = new GListHead(zf.getInputStream(ze));
                        break;
                    }
                    case 12:
                    {
                        a = new TaskForceList((GListHead)getInternalFile(SavGame.fileGTFORCEHD, true),
                                zf.getInputStream(ze));
                        break;
                    }
                    case 13:
                    {
                        a = new TaskForceList((GListHead)getInternalFile(SavGame.fileGWTFORCEHD, true),
                                zf.getInputStream(ze));
                        break;
                    }
                    case 14:
                    {
                        a = new OrderInfo(zf.getInputStream(ze));
                        break;
                    }
                    case 15:
                    {
                        a = new GShipList((GListHead)getInternalFile(SavGame.fileGSHIPHEAD, true),
                                zf.getInputStream(ze));
                        break;
                    }
                    case 16:
                    {
                        a = new StrcInfo(zf.getInputStream(ze));
                        break;
                    }
                    case 17:
                    {
                        a = new AlienInfo(zf.getInputStream(ze), this);
                        break;
                    }
                    case 18:
                    {
                        a = new TechInfo(zf.getInputStream(ze));
                        break;
                    }
                    case 19: case 20: case 21: case 22: case 23:
                    {
                        a = new SphereOfInfluence(zf.getInputStream(ze));
                        break;
                    }
                    default: {
                        InputStream inu = zf.getInputStream(ze);
                        byte[] bib = new byte[0];
                        if (inu.available() > 0) {
                            bib = new byte[inu.available()];
                            inu.read(bib);
                        }
                        a = new GenericFile("", bib); //$NON-NLS-1$
                    }
                }
                a.setName(ze.getName());

                if (cache)
                    INTERNAL_FILES.put(a.getName(), a);
                zf.close();
            } catch (Exception e) {
                UE.displayError(e);
            }

            return a;
        }
    }

    /**@return true if changes were made to the opened file and false if not.*/
    @Override
    public boolean madeChanges()
    {
        if (!REMOVED.isEmpty() || CANCELLED_SAVE)
        {
            if (UE.DEBUG_MODE)
            {
                System.out.println(this.SAV_FILE.getName() + ": !REMOVED.isEmpty() || CANCELLED_SAVE");
            }

            return true;
        }

        InternalFile se;
        
        for (Enumeration<InternalFile> en = INTERNAL_FILES.elements(); en.hasMoreElements();)
        {
            se = en.nextElement();
            
            if (se.madeChanges())
            {
                if (UE.DEBUG_MODE)
                {
                    System.out.println(this.SAV_FILE.getName() + ": Made changes to: " + se.getName());
                }

                return true;
            }
        }
        return false;
    }

    /**Returns a vector containing result of the check.*/
    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();

        final CheckTask TSK = new CheckTask();

        final JOptionPane opti = new JOptionPane(getStartingTaskString(),
                JOptionPane.PLAIN_MESSAGE);
        opti.setOptions(new String[]{getCancelString()});
        final JDialog dia = opti.createDialog(UE.WINDOW, UE.APP_NAME);

        TSK.addPropertyChangeListener(new PropertyChangeListener() {

            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                if (new String().equals(evt.getPropertyName())) {
                    opti.setMessage(evt.getNewValue());
                }
                if (TSK.isDone()) {
                    dia.dispose();
                }
            }
        });

        TSK.execute();

        dia.setVisible(true);

        if (getCancelString().equals(opti.getValue())) {
            TSK.cancel(true);
        }

        try {
            response = TSK.getResult();
        } catch (Exception e) {
            UE.displayError(e);
        }

        //return
        if (response.isEmpty()) {
            response.add(Language.getString("SavGame.12")); //$NON-NLS-1$
        }
        return response;
    }

    @Override
    public long getInternalFileSize(String fileName) throws IOException
    {
        if (REMOVED.contains(fileName))
        {
            throw new FileNotFoundException(getFileNotFoundString(fileName));
        }

        if (INTERNAL_FILES.containsKey(fileName))
        {
            return INTERNAL_FILES.get(fileName).getSize();
        }
        else
        {
            try {
                SavFile zf = new SavFile(SAV_FILE, null);
                SavEntry ze = zf.getEntry(fileName);
                zf.close();
                return ze.getUncompressedSize();
            }
            catch (Exception ex)
            {
                throw new IOException(ex.getMessage());
            }
        }
    }

    @Override
    public byte[] getInternalFileHash(String filename) throws Exception
    {
        MessageDigest m = MessageDigest.getInstance("MD5"); //$NON-NLS-1$

        ByteArrayOutputStream baos = new ByteArrayOutputStream();

        InternalFile file = this.getInternalFile(filename, false);
        file.save(baos);

        m.update(baos.toByteArray(), 0, baos.size());
        baos.close();

        return m.digest();
    }

    private class CheckTask extends SwingWorker<Vector<String>, Vector<String>> {

        private Vector<String> RESPONSE = new Vector<String>();

        public CheckTask() {
            super();
        }

        public Vector<String> getResult() {
            Vector<String> ret = new Vector<String>(RESPONSE);
            if (this.isCancelled()) {
                ret.add(Language.getString("SavGame.13")); //$NON-NLS-1$
            }
            return ret;
        }

        @Override
        public Vector<String> doInBackground() {
            //check all
            InternalFile test = null;
            Vector<String> temp;

            try {
                //of the file as whole
                SavFile su = new SavFile(SAV_FILE, null);
                temp = su.check();
                if (!temp.isEmpty()) {
                    RESPONSE.addAll(temp);
                }
                su.close();
            } catch (Exception f) {
                RESPONSE.add(getGenericErrorString(f.getMessage()));
                f.printStackTrace();
            }

            for (int i = 0; i < FILELIST.length && !isCancelled(); i++) {
                try {
                    test = getInternalFile(FILELIST[i], true);
                    // TODO: test fire
                    firePropertyChange("", null, "Performing self-check on " + test.getName() + "...");
                    temp = test.check();
                    if (!temp.isEmpty()) {
                        RESPONSE.addAll(temp);
                    }
                } catch (Exception f) {
                    RESPONSE.add(getGenericErrorString(f.getMessage()));
                    f.printStackTrace();
                }
            }
            return RESPONSE;
        }
    }

    private class SaveTask extends SwingWorker<Void, Void> {

        File DESTIN;
        String ext;
        boolean bkp;

        public SaveTask(File dest, boolean bkp, String ext) {
            super();
            DESTIN = dest;
            this.bkp = bkp;
            this.ext = ext;
        }

        @Override
        public Void doInBackground() {
            setProgress(0);
            String msg;

            SavFile source = null;
            SavFile dest = null;
            try {
                source = new SavFile(SAV_FILE, null);

                //where to
                if (DESTIN.equals(SAV_FILE)) {
                    if (bkp) {
                        firePropertyChange(new String(), null, Language.getString("SavGame.24")); //$NON-NLS-1$

                        File file = new File(SAV_FILE.getPath() + ext);
                        int i = 1;

                        while (file.exists()) {
                            file = new File(SAV_FILE.getPath() + i + ext);
                            i++;
                        }

                        Tools.copyFile(SAV_FILE, file);
                    }
                    DESTIN = FileManager.getWritableLocalFile("sav.tmp"); //$NON-NLS-1$
                    }
                DESTIN.delete();
                dest = new SavFile(DESTIN, source.getName()); //savfile creates the file
                if (isCancelled()) {
                    dest.close();
                    DESTIN.delete();
                    source.close();
                    return null;
                }
            } catch (Exception e) {
                UE.displayError(e);
                cancel(false);
            }

            for (Enumeration<SavEntry> e = source.entries(); e.hasMoreElements() && !isCancelled();) {
                try {
                    SavEntry ze = e.nextElement();
                    // skip file if removed
                    if (REMOVED.contains(ze.getName()) || INTERNAL_FILES.containsKey(ze.getName())) {
                        continue;
                    }
                    // copy
                    msg = Language.getString("SavGame.14"); //$NON-NLS-1$
                    msg = msg.replace("%1", ze.getName()); //$NON-NLS-1$
                    firePropertyChange(new String(), null, msg);

                    InputStream is = source.getInputStream(ze, false);
                    ze.setCompressedSize(-1);
                    dest.putNextEntry(ze);
                    OutputStream zos = dest.getOutputStream(ze, false);
                    int len;
                    byte[] buf = new byte[1024];

                    while (is.available() > 0) {
                        len = is.read(buf);
                        zos.write(buf, 0, len);
                    }
                    is.close();
                    zos.close();
                    // Complete the entry
                    dest.closeEntry();
                    if (isCancelled()) {
                        dest.close();
                        DESTIN.delete();
                        source.close();
                        return null;
                    }
                } catch (Exception zne) {
                    UE.displayError(zne);
                    cancel(false);
                }
            }

            // add files
            Enumeration<InternalFile> enu = INTERNAL_FILES.elements();
            SavEntry old;

            while (enu.hasMoreElements()) {
                try {
                    InternalFile stfil = enu.nextElement();

                    msg = Language.getString("SavGame.15"); //$NON-NLS-1$
                    msg = msg.replace("%1", stfil.getName()); //$NON-NLS-1$
                    firePropertyChange(new String(), null, msg);

                    SavEntry ze = new SavEntry(stfil.getName());
                    ze.setCompressedSize(-1);
                    ze.setUncompressedSize((int) stfil.getSize());

                    old = source.getEntry(stfil.getName());

                    if (old != null) {
                        ze.setUnknown(old.getUnknown());
                    }

                    dest.putNextEntry(ze);

                    OutputStream zos = dest.getOutputStream(ze, true);

                    stfil.save(zos);

                    stfil.markSaved();

                    zos.close();
                    // Complete the entry
                    dest.closeEntry();
                } catch (Exception zne) {
                    UE.displayError(zne);
                    cancel(false);
                }
            }

            try {
                source.close();
                dest.close();
            } catch (Exception e) {
                UE.displayError(e);
                cancel(false);
            }
            if (isCancelled()) {
                return null;
            }
            //replace?
            if (DESTIN.getName().equals("sav.tmp")) { //$NON-NLS-1$

                try {
                    Tools.copyFile(DESTIN, SAV_FILE);
                } catch (IOException er) {
                    UE.displayError(er);
                    return null;
                }

                boolean check = DESTIN.delete();
                if (!check) {
                    msg = Language.getString(Language.getString("SavGame.23")); //$NON-NLS-1$
                    msg = msg.replace("%1", DESTIN.getName()); //$NON-NLS-1$
                    UE.displayError(new IOException(msg));
                }
            }
            CANCELLED_SAVE = false;
            return null;
        }
    }

    @Override
    public void addInternalFile(InternalFile file) {
        String msg;
        file.markChanged();
        if (file.getName().length() > 15) {
            msg = Language.getString("SavGame.18"); //$NON-NLS-1$
            msg = msg.replace("%1", file.getName()); //$NON-NLS-1$
            UE.displayError(new Exception(msg));
            return;
        }

        if (file instanceof GenericFile) {
            int z = -1;
            for (int i = 0; i < FILELIST.length; i++) {
                if (FILELIST[i].equals(file.getName())) {
                    z = i;
                }
            }
            if (z >= 0) {
                try {
                    ByteArrayInputStream bais = new ByteArrayInputStream(((GenericFile) file).getData());
                    InternalFile a = file;

                    switch (z) {
                        case 0: {
                            a = new SystInfo(bais, STBOF);
                            break;
                        }
                        case 1: {
                            a = new StellInfo(bais, STBOF);
                            break;
                        }
                        case 2: {
                            a = new StarBaseInfo(bais);
                            break;
                        }
                        case 3: {
                            a = new Sector(bais, this);
                            break;
                        }
                        case 4: {
                            a = new GameInfo(bais);
                            break;
                        }
                        case 5: {
                            a = new GalInfo(bais);
                            break;
                        }
                        case 6: {
                            a = new EmpsInfo(file.getName(), bais);
                            break;
                        }
                        case 7: {
                            a = new Result(bais);
                            break;
                        }
                        case 8: case 9: case 10: case 11:
                        {
                            a = new GListHead(bais);
                            break;
                        }
                        case 12:
                        {
                            a = new TaskForceList((GListHead)getInternalFile(SavGame.fileGTFORCEHD, true),
                                    bais);
                            break;
                        }
                        case 13:
                        {
                            a = new TaskForceList((GListHead)getInternalFile(SavGame.fileGWTFORCEHD, true),
                                    bais);
                            break;
                        }
                        case 14:
                        {
                            a = new OrderInfo(bais);
                            break;
                        }
                        case 15:
                        {
                            a = new GShipList((GListHead)getInternalFile(SavGame.fileGSHIPHEAD, true),
                                    bais);
                            break;
                        }
                        case 16:
                        {
                            a = new StrcInfo(bais);
                            break;
                        }
                        case 17:
                        {
                            a = new AlienInfo(bais, this);
                            break;
                        }
                        case 18:
                        {
                            a = new TechInfo(bais);
                            break;
                        }
                        case 19: case 20: case 21: case 22: case 23:
                        {
                            a = new SphereOfInfluence(bais);
                            break;
                        }
                    }
                    a.setName(file.getName());
                    // added files are considered changed, to make sure the save dialog pops up
                    a.markChanged();
                    INTERNAL_FILES.put(a.getName(), a);
                    bais.close();
                } catch (Exception u) {
                    u.printStackTrace();
                }
            } else {
                INTERNAL_FILES.put(file.getName(), file);
            }
        } else {
            INTERNAL_FILES.put(file.getName(), file);
        }

        REMOVED.remove(file.getName());
    }

    @Override
    public void discard() {
        REMOVED.clear();
        INTERNAL_FILES.clear();
    }

    @Override
    public Enumeration<String[]> getFiles(FilenameFilter filter)
    {
        Vector<String[]> lst = new Vector<String[]>();

        try
        {
            SavFile source = new SavFile(SAV_FILE, null);
            SavEntry ze;

            for (Enumeration<SavEntry> e = source.entries(); e.hasMoreElements();)
            {
                ze = e.nextElement();

                if (REMOVED.contains(ze.getName()))
                {
                    continue;
                }
                if (filter.accept(SAV_FILE, ze.getName()) &&
                        !INTERNAL_FILES.containsKey(ze.getName())) {
                    
                    lst.add(new String[]{ze.getName(), Long.toString(ze.getUncompressedSize())});
                }
            }

            InternalFile test;
            Enumeration<InternalFile> un = INTERNAL_FILES.elements();
            while (un.hasMoreElements()) {
                test = un.nextElement();
                if (REMOVED.contains(test.getName())) {
                    continue;
                }
                if (filter.accept(SAV_FILE, test.getName())) {
                    lst.add(new String[]{test.getName(), Long.toString(test.getSize())});
                }
            }
        } catch (Exception e) {
            UE.displayError(e);
        }

        return lst.elements();
    }

    @Override
    public Enumeration<String> getListOfLoadedFiles() {
        return INTERNAL_FILES.keys();
    }

    @Override
    public Iterator<String> getRemovedFilesList() {
        return REMOVED.iterator();
    }

    @Override
    public void removeInternalFile(String name) {
        INTERNAL_FILES.remove(name);
        REMOVED.add(name);
    }

    private String getCancelString() {
        return Language.getString("SavGame.19"); //$NON-NLS-1$
    }

    private String getStartingTaskString() {
        return Language.getString("SavGame.20"); //$NON-NLS-1$
    }

    private String getFileNotFoundString(String fileName) {
        String msg = Language.getString("SavGame.21"); //$NON-NLS-1$
        msg = msg.replace("%1", fileName); //$NON-NLS-1$
        return msg;
    }

    private String getGenericErrorString(String error) {
        String msg = Language.getString("SavGame.22"); //$NON-NLS-1$
        msg = msg.replace("%1", SAV_FILE.getName()); //$NON-NLS-1$
        msg = msg.replace("%2", error); //$NON-NLS-1$
        return msg;
    }
    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#getKnownChanges()
     */

    @Override
    public ModList getKnownChanges() {
        return null;
    }
    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#hasBeenModified()
     */

    @Override
    public boolean hasBeenModified() {
        return false;
    }

    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#toFile()
     */
    @Override
    public File toFile() {
        return SAV_FILE;
    }
}

