/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: SavInputStream.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.10.2007
*/

package ue.edit.sav;

import java.io.IOException;
import java.io.InputStream;

import ue.Language;

/**This InputStream encapsulates InputStream.
 * It is used to set up artificial read limits.*/
public class SavInputStream extends InputStream{
	private InputStream RAN;
	private int length;
	
	public SavInputStream(InputStream raf, int end) throws IOException{
		RAN = raf;
		length = end;
	}
	@Override
	public int available() throws IOException{
		return length;
	}

	@Override
	public void close() throws IOException{
		RAN.close();
		RAN = null;
	}
	@Override
	public void mark(int readlimit){}
	@Override
	public boolean markSupported(){ return false;}
	@Override
	public int read() throws IOException{
		checkEOF();
		length--;
		return RAN.read();
	}
	@Override
	public int read(byte[] b) throws IOException{
		int a = b.length;
		if (available() < b.length) {
			a = available();
		}
		length-=a;
		return RAN.read(b, 0, a);
	}

	@Override
	public int read(byte[] b, int off, int len) throws IOException{
		checkEOF();
		if (available() < len) {
			len = available();
		}
		length-=len;
		return RAN.read(b, off, len);
	}
	
	@Override
	public void reset() throws IOException{
		throw new IOException(Language.getString("SavInputStream.0")); //$NON-NLS-1$
	}
	
	@Override
	public long skip(long n) throws IOException{
		if (available()<=0) {
			return -1;
		}
		if (available() < n) {
			n = available();
		}
		length-=n;
		return RAN.skip(n);
	}

	private void checkEOF() throws IOException{
		if (available()<=0){
			throw new IOException(Language.getString("SavInputStream.1")); //$NON-NLS-1$
		}
	}
}
