/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: SavEntry.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.10.2007
*/

package ue.edit.sav;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import ue.Language;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;

/**This class reprisents a subentry in a save entry.
 * Contains metedata as well as the actual data.*/
public class SavSubEntry {
	private boolean compressed = true;

	private int uncomp = 0;
	private int comp = -1;
	private byte[] data;

	public SavSubEntry(InputStream in, boolean compressed) throws IOException{
		this.compressed = compressed;
		
		byte[] b = new byte[4];
		in.read(b);
		uncomp = Tools.toInt(b, true);
		if (compressed){
			in.read(b);
			comp = Tools.toInt(b, true);
			if (comp > in.available()){
				String msg = Language.getString("SavSubEntry.0"); //$NON-NLS-1$
				msg = msg.replace("%1", Integer.toString(comp)); //$NON-NLS-1$
				throw new IOException(msg);
			}
			data = new byte[comp];
		}else{
			if (uncomp > in.available()){
				String msg = Language.getString("SavSubEntry.1"); //$NON-NLS-1$
				msg = msg.replace("%1", Integer.toString(uncomp)); //$NON-NLS-1$
				throw new IOException(msg);
			}
			data = new byte[uncomp];
		}
		in.read(data);
	}
	
	public void store(OutputStream out) throws IOException{		
		out.write(Tools.toByte(uncomp, true));
		if (compressed) {
			out.write(Tools.toByte(comp, true));
		}
		out.write(data);
	}
	
	public byte[] getData(){
		int len;
		if (compressed) {
			len = comp;
		} else {
			len = uncomp;
		}
		byte[] b = new byte[len];
		System.arraycopy(data, 0, b, 0, len);
		return b;
	}
	
	public void setData(byte[] b) throws InvalidArgumentsException{
		int len;
		if (compressed) {
			len = comp;
		} else {
			len = uncomp;
		}
		if (b.length != len){
			String msg = Language.getString("SavSubEntry.2"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(b.length)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
		System.arraycopy(b, 0, data, 0, len);
	}
	
	public void decompress() throws IOException{
		if (!compressed) {
			return;
		}
		LzssInputStream lis = new LzssInputStream(new ByteArrayInputStream(data));
		int len = lis.available();
		if (len != uncomp) {
			throw new IOException(Language.getString("SavSubEntry.3")); //$NON-NLS-1$
		}
		data = new byte[len];
		lis.read(data);
		compressed = false;
	}
	
	public void compress() throws IOException{
		if (compressed) {
			return;
		}
		ByteArrayOutputStream  baos = new ByteArrayOutputStream();
		LzssOutputStream los = new LzssOutputStream(baos);
		los.write(data);
		los.close();
		if (baos.size() != comp && comp >= 0) {
			throw new IOException(Language.getString("SavSubEntry.4")); //$NON-NLS-1$
		}
		data = baos.toByteArray();
		baos.close();
		comp = data.length;
		compressed = true;
	}
	
	public int getSize(){
		return data.length;
	}
}