/**
 * Ultimate Editor
 * Copyright (C) 2008 Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: SphereOfInfluence.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 27.5.2011
 */
package ue.edit.sav;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;
import ue.edit.InternalFile;
import ue.exception.InvalidArgumentsException;
import ue.util.CommonStrings;

/**
 * Used for loading SphOfIn0-SphOfIn4 files.
 */
public class SphereOfInfluence extends InternalFile
{
    private byte[] map;

    public SphereOfInfluence(InputStream in) throws IOException
    {
        map = new byte[in.available()];

        int read = 0;

        while (in.available() > 0)
        {
            read += in.read(map, read, map.length - read);
        }
    }

    public int getInfluence(int sectorIndex) throws InvalidArgumentsException
    {
        testIndex(sectorIndex);

        return map[sectorIndex] & 0xFF;
    }

    public void setInfluence(int sectorIndex, int influence) throws InvalidArgumentsException
    {
        testIndex(sectorIndex);

        if ((map[sectorIndex] & 0xFF) == influence)
        {
            return;
        }

        map[sectorIndex] = (byte)influence;
        this.markChanged();
    }

    private void testIndex(int sectorIndex) throws InvalidArgumentsException
    {
        if (sectorIndex < 0 || sectorIndex >= map.length)
        {
            throw new InvalidArgumentsException(
                    CommonStrings.getLocalizedString(
                        CommonStrings.InvalidIndex_1,
                        new String[]{Integer.toString(sectorIndex)}));
        }
    }

    @Override
    public void save(OutputStream out) throws Exception
    {
        out.write(map);
    }

    @Override
    public Vector<String> check() {
        //throw new UnsupportedOperationException("Not supported yet.");
        return null;
    }

    @Override
    public long getSize() {
        return map.length;
    }
}
