/*
Ultimate Editor for Birth Of The Federation 1.02
Copyright (C) 2005-2007  Alan Podlesek

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


File: StarBaseInfo.java

Written by: Alan Podlesek <seqdcer@yahoo.com>

Last modified: 6.9.2007
 */
package ue.edit.sav;

import java.awt.Point;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Vector;

import ue.Language;
import ue.UE;
import ue.edit.CanBeEdited;
import ue.edit.FileManager;
import ue.edit.InternalFile;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.sst.ShipList;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;

/**This class is a representation of the starBaseInfo file. It holds a list of outposts/starbases in the game.*/
public class StarBaseInfo extends InternalFile {
    //fields

    private ArrayList<PostEntry> ENTRY = new ArrayList<PostEntry>();
    //specified by SavEdit

    @Override
    public Vector<String> check() {
        Vector<String> response = new Vector<String>();
        String msg;
        PostEntry post;
        Hashtable<Short, String> test = new Hashtable<Short, String>();

        ShipList ship = null;
        CanBeEdited file;
        if ((file = UE.FILES.getFile(FileManager.P_STBOF)) != null) {
            try {
                ship = (ShipList) file.getInternalFile(Stbof.fileSHIPLIST_SST, true);
            } catch (Exception e) {
            }
        }

        for (int i = 0; i < this.ENTRY.size(); i++) {
            post = this.ENTRY.get(i);
            if (post.NAME.length() > 39) {
                msg = Language.getString("StarBaseInfo.0"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", post.NAME); //$NON-NLS-1$
                response.add(msg);
            }
            if ((post.OWNER < 0) || (post.OWNER > 4)) {
                msg = Language.getString("StarBaseInfo.1"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", post.NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Short.toString(post.OWNER)); //$NON-NLS-1$
                response.add(msg);
            }
            if (test.containsKey(post.ID)) {
                msg = Language.getString("StarBaseInfo.2"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", post.NAME); //$NON-NLS-1$
                msg = msg.replace("%3", test.get(post.ID)); //$NON-NLS-1$
                msg = msg.replace("%4", Short.toString(post.ID)); //$NON-NLS-1$
                response.add(msg);
            } else {
                test.put(post.ID, post.NAME);
            }
            if ((post.TYPE < 6) || (post.TYPE > 7)) {
                msg = Language.getString("StarBaseInfo.3"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", post.NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Short.toString(post.TYPE)); //$NON-NLS-1$
                response.add(msg);
            }
            if (ship != null) {
                try {
                    if (ship.getHullStr(post.SHIP_ID) < post.HULL) {
                        msg = Language.getString("StarBaseInfo.8"); //$NON-NLS-1$
                        msg = msg.replace("%1", getName()); //$NON-NLS-1$
                        msg = msg.replace("%2", post.NAME); //$NON-NLS-1$

                        post.HULL = ship.getHullStr(post.SHIP_ID);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    msg = Language.getString("StarBaseInfo.9"); //$NON-NLS-1$
                    msg = msg.replace("%1", getName()); //$NON-NLS-1$
                    msg = msg.replace("%2", e.getLocalizedMessage()); //$NON-NLS-1$
                }
            }

            if (post.HULL <= 0) {
                msg = Language.getString("StarBaseInfo.10"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", post.NAME); //$NON-NLS-1$

                post.HULL = 1;
            }
        }
        return response;
    }

    @Override
    public long getSize() {
        long size = this.ENTRY.size() * 92;
        return size;
    }

    @Override
    public void save(OutputStream out) throws IOException {
        int size = this.ENTRY.size();
        PostEntry se;
        for (int i = 0; i < size; i++) {
            se = this.ENTRY.get(i);
            se.save(out);
        }
    }

    /**Constructor
    @param in the InputStream to read from*/
    public StarBaseInfo(InputStream in) throws IOException {
        PostEntry se;
        while (in.available() > 0) {
            se = new PostEntry(in);
            this.ENTRY.add(se);
        }
    }
    //gets

    /**Returns a list of names*/
    public String[] getNames() {
        int siz = this.ENTRY.size();
        String[] res = new String[siz];
        PostEntry se;
        for (int i = 0; i < siz; i++) {
            se = this.ENTRY.get(i);
            res[i] = new String(se.NAME);
        }
        return res;
    }

    /**
     * @param id outpost id
     * @return -1 if not found
     */
    public int getIndex(short id) {
        PostEntry pe;
        for (int i = 0; i < ENTRY.size(); i++) {
            pe = ENTRY.get(i);
            if (pe.ID == id) {
                return i;
            }
        }
        return -1;
    }

    /**
     * @param index outpost index
     */
    public short getID(int index) throws InvalidArgumentsException
    {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);
        return pe.ID;
    }

    /**Returns the name of the outpost.
    @param index the index of the outpost*/
    public String getName(int index) throws InvalidArgumentsException
    {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);
        return pe.NAME;
    }

    /**Returns owner of the outpost.
    @param index the index of the outpost*/
    public short getOwner(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);
        return pe.OWNER;
    }

    /**Returns type of the outpost(ship).
    @param index the index of the outpost*/
    public short getType(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);
        return pe.TYPE;
    }

    /**Returns ship id (for shiplist.sst) of the outpost.
    @param index the index of the outpost*/
    public short getShipID(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);
        return pe.SHIP_ID;
    }

    /**
     * @param index outpost index
     * @return outpost hull
     * @throws InvalidArgumentsException
     */
    public int getHull(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);
        return pe.HULL;
    }

    /**
     * @param index outpost index
     * @return production cost
     * @throws InvalidArgumentsException
     */
    public double getProductionCost(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);
        return pe.PRODUCTION_COST;
    }

    /**
     * @param index outpost index
     * @return production invested
     * @throws InvalidArgumentsException
     */
    public double getProductionInvested(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);
        return pe.PRODUCTION_INVESTED;
    }

    /**
     * @param index outpost index
     * @return scan range
     * @throws InvalidArgumentsException
     */
    public double getScanRange(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);
        return pe.SCAN_RANGE;
    }

    //sets
    /**Sets the name of the outpost.
    @param index the index of the outpost
     */
    public void setName(int index, String name) throws InvalidArgumentsException {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);
        if (pe.NAME.equals(name)) {
            return;
        }
        pe.NAME = new String(name);
        this.markChanged();
    }

    /**Sets the owner of the outpost.
    @param index the index of the outpost
    @param owner the owner*/
    public void setOwner(int index, short owner) throws InvalidArgumentsException {
        this.checkIndex(index);
        this.checkOwner(owner);
        PostEntry pe = this.ENTRY.get(index);
        if (pe.OWNER == owner) {
            return;
        }
        pe.OWNER = owner;
        this.markChanged();
    }

    /**Sets type of the outpost(ship).
    @param index the index of the outpost
    @param type type of ship/ship function*/
    public void setType(int index, short type) throws InvalidArgumentsException {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);
        if (pe.TYPE == type) {
            return;
        }
        pe.TYPE = type;
        this.markChanged();
    }

    /**
     *
     * @param index outpost index
     * @param id ship id
     * @throws InvalidArgumentsException
     */
    public void setShipID(int index, short id) throws InvalidArgumentsException {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);
        if (pe.SHIP_ID != id) {
            pe.SHIP_ID = id;
            this.markChanged();
        }
    }

    public void setHull(int index, int hull) throws InvalidArgumentsException {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);
        if ((pe.HULL != hull) && (hull >= 1)) {
            pe.HULL = hull;
            this.markChanged();
        }
    }

    /**Sets outpost position.
    @param index the index of the outpost
    @param vert vertical position
    @param hori horizontal position
    @param hsize number of sectors in a horizontal line
    @return new starbase id
     */
    public short setPosition(int index, int vert, int hori, int hsize) throws InvalidArgumentsException {
        this.checkIndex(index);
        if ((vert < 0) || (hori < 0) || (hsize < 0)) {
            String msg = Language.getString("StarBaseInfo.4"); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
        int id = vert * hsize + hori;
        PostEntry pe = this.ENTRY.get(index);
        pe.V_SEC = vert;
        pe.H_SEC = hori;
        pe.ID = (short) id;
        this.markChanged();
        return pe.ID;
    }

    public Point getPosition(int index) throws InvalidArgumentsException
    {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);
        return new Point(pe.H_SEC, pe.V_SEC);
    }

    /**
     * @param index outpost index
     * @param cost production cost
     * @throws InvalidArgumentsException
     */
    public void setProductionCost(int index, double cost) throws InvalidArgumentsException {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);

        if (pe.PRODUCTION_COST != cost)
        {
            pe.PRODUCTION_COST = cost;
            this.markChanged();
        }
    }

    /**
     * @param index outpost index
     * @param invested production invested
     * @throws InvalidArgumentsException
     */
    public void setProductionInvested(int index, double invested) throws InvalidArgumentsException {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);

        if (pe.PRODUCTION_INVESTED != invested)
        {
            pe.PRODUCTION_INVESTED = invested;
            this.markChanged();
        }
    }

    /**
     * @param index outpost index
     * @param range scan range
     * @throws InvalidArgumentsException
     */
    public void setScanRange(int index, int range) throws InvalidArgumentsException {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);

        if (pe.SCAN_RANGE != range)
        {
            pe.SCAN_RANGE = range;
            this.markChanged();
        }
    }

    /**Returns the descriptive name of the sector where the starbase is
    @param index the index of the outpost*/
    public String getDescription(int index) throws InvalidArgumentsException {
        this.checkIndex(index);
        PostEntry pe = this.ENTRY.get(index);
        byte[] but = new byte[1];
        but[0] = (byte) (97 + pe.H_SEC);
        String ret = Language.getString("StarBaseInfo.5"); //$NON-NLS-1$
        try {
            ret = ret.replace("%1", Tools.toASCII(but).toUpperCase() + (pe.V_SEC + 1)); //$NON-NLS-1$
        } catch (Exception ex) {
            ret = ""; //$NON-NLS-1$
            ex.printStackTrace();
        }
        return ret;
    }

    private void checkIndex(int index) throws InvalidArgumentsException {
        if ((index < 0) || (index > this.ENTRY.size())) {
            String msg = Language.getString("StarBaseInfo.6"); //$NON-NLS-1$
            msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
    }

    private void checkOwner(int owner) throws InvalidArgumentsException {
        if ((owner < 0) || (owner > 4)) {
            String msg = Language.getString("StarBaseInfo.7"); //$NON-NLS-1$
            msg = msg.replace("%1", Integer.toString(owner)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
    }

    /**
     * @param func outpost (6) or starbase (7)
     * @param v vertical pos
     * @param h horizontal pos
     * @return index
     */
    public int add(short id, short func, int v, int h) {
        PostEntry pe = new PostEntry(func, v, h);
        pe.ID = id;
        ENTRY.add(pe);
        this.markChanged();
        return ENTRY.size() - 1;
    }

    /**
     * Removes the outpost and resets ids of others
     * @param index outpost index
     */
    public void remove(int index) {
        ENTRY.remove(index);

        this.markChanged();
    }

    /**
     * @return Number of outposts in this file
     */
    public int getNumberOfEntries() {
        return ENTRY.size();
    }

    private class PostEntry {

        public String NAME;
        public short OWNER;
        public short ID; //ensures only one outpost per sector
        public short TYPE;
        public short SHIP_ID;
        //position in sectors
        public int V_SEC;
        public int H_SEC;
        public double PRODUCTION_COST;
        public double PRODUCTION_INVESTED;
        public int SCAN_RANGE;
        public int HULL;
        private byte[] UNKNOWN_2 = new byte[12];

        //constructors
        public PostEntry(InputStream in) throws IOException {
            byte[] b = new byte[40];
            //name 0-40
            in.read(b);
            this.NAME = Tools.toASCII(b);
            //owner 40-2
            b = new byte[2];
            in.read(b);
            this.OWNER = Tools.toShort(b, true);
            //ID 42-2
            in.read(b);
            this.ID = Tools.toShort(b, true);
            //type 44-2
            in.read(b);
            this.TYPE = Tools.toShort(b, true);
            //ship id 46-2
            in.read(b);
            this.SHIP_ID = Tools.toShort(b, true);
            //vertical 48-4
            b = new byte[4];
            in.read(b);
            this.V_SEC = Tools.toInt(b, true);
            //horizontal 52-4
            in.read(b);
            this.H_SEC = Tools.toInt(b, true);
            //production cost 56-8
            b = new byte[8];
            in.read(b);
            this.PRODUCTION_COST = Tools.toDouble(b, true);
            //production invested 64-8
            in.read(b);
            this.PRODUCTION_INVESTED = Tools.toDouble(b, true);
            //unknown 56-20
            b = new byte[4];
            in.read(b);
            this.SCAN_RANGE = Tools.toInt(b, true);
            // hull
            in.read(b);
            this.HULL = Tools.toInt(b, true);
            //unknown 12
            in.read(this.UNKNOWN_2);
        }

        public PostEntry(short type, int v, int h) {
            this.OWNER = 0;
            this.NAME = "";
            this.ID = -1;
            this.TYPE = type;
            this.SHIP_ID = 0;
            this.V_SEC = v;
            this.H_SEC = h;
            this.HULL = 1;
            this.PRODUCTION_COST = 1;
            this.PRODUCTION_INVESTED = 1;
            this.SCAN_RANGE = (type == 6) ? 1 : 2;
        }
        //save

        public void save(OutputStream out) throws IOException {
            //name 0-40
            byte[] b = Tools.toByte(this.NAME, 40, 39);
            out.write(b);
            //owner
            b = Tools.toByte(this.OWNER, true);
            out.write(b);
            //ID 42-2
            b = Tools.toByte(this.ID, true);
            out.write(b);
            //type
            b = Tools.toByte(this.TYPE, true);
            out.write(b);
            //ship id
            b = Tools.toByte(this.SHIP_ID, true);
            out.write(b);
            //vertical
            b = Tools.toByte(this.V_SEC, true);
            out.write(b);
            //horizontal
            b = Tools.toByte(this.H_SEC, true);
            out.write(b);
            // production cost
            b = Tools.toByte(this.PRODUCTION_COST, true);
            out.write(b);
            // production invested
            b = Tools.toByte(this.PRODUCTION_INVESTED, true);
            out.write(b);
            //unknown
            out.write(Tools.toByte(SCAN_RANGE, true));
            // hull
            out.write(Tools.toByte(this.HULL, true));
            //unknown
            out.write(this.UNKNOWN_2);
        }
    }
}

