/*
Ultimate Editor for Birth Of The Federation 1.02
Copyright (C) 2005-2007  Alan Podlesek

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


File: SystInfo.java

Written by: Alan Podlesek <seqdcer@yahoo.com>

Last modified: 6.9.2007
 */
package ue.edit.sav;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.EOFException;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Vector;

import ue.Language;
import ue.UE;
import ue.edit.InternalFile;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.pst.PlanetEntry;
import ue.exception.InvalidArgumentsException;
import ue.util.AniOrCur;
import ue.util.ID;
import ue.util.Tools;
import ue.util.TargaImage;

/**Represents systInfo**/
public class SystInfo extends InternalFile {

    public static final byte SYS_TYPE_HOME = 0;
    public static final byte SYS_TYPE_NATIVE = 1;
    public static final byte SYS_TYPE_NON_NATIVE = 2;
    public static final byte SYS_TYPE_SUBJUGATED = 3;
    public static final byte SYS_TYPE_AFFILIATED = 4;
    public static final byte SYS_TYPE_INDEPENDANT_MINOR = 5;
    public static final byte SYS_TYPE_CONQUERED_HOME = 6;
    public static final byte SYS_TYPE_ANY = 9;
    public static final byte SYS_TYPE_REBEL = 10;
    public static final byte SYS_TYPE_EMPTY = 11;

    public static final short MORALE_REBELLIOUS = -4;
    public static final short MORALE_DEFIANT = -3;
    public static final short MORALE_DISGRUNTLED = -2;
    public static final short MORALE_APATHETIC = -1;
    public static final short MORALE_CONTENT = 0;
    public static final short MORALE_PLEASED = 1;
    public static final short MORALE_LOYAL = 2;
    public static final short MORALE_FANATIC = 3;

    public static final int QUEUE_ORDER_TYPE_BUILDING = 1;
    public static final int QUEUE_ORDER_TYPE_UPGRADE_BUILDING = 2;
    public static final int QUEUE_ORDER_TYPE_SHIP = 3;
    public static final int QUEUE_ORDER_TYPE_TRADE_GOODS = 5;
    public static final int QUEUE_ORDER_TYPE_FIX_UNREST = 7;
    
    private ArrayList<SystemEntry> SYSTEMS = new ArrayList<SystemEntry>();
    private Stbof STBOF;

    //constructor
    public SystInfo(InputStream in, Stbof st) throws IOException, EOFException
    {
        STBOF = st;

        if (!in.markSupported())
        {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();

            byte[] b = new byte[1024];
            int len;

            while (in.available() > 0) {
                len = in.read(b);
                baos.write(b, 0, len);
            }

            in = new ByteArrayInputStream(baos.toByteArray());
        }

        ByteArrayOutputStream extra = new ByteArrayOutputStream();
        byte[] b = new byte[4];

        SystemEntry sys = new SystemEntry(in);
        SYSTEMS.add(sys);

        if (UE.DEBUG_MODE)
        {
            System.out.println("Read: " + sys.NAME + " (" + sys.INDEX + ")");
        }

        SystemEntryTest systest = new SystemEntryTest();

        in.mark(Integer.MAX_VALUE);

        while (in.available() > 0) {
            in.mark(Integer.MAX_VALUE);

            try {
                systest.test(in, sys.INDEX + 1);

                in.reset();
                // add
                sys.setExtra(extra.toByteArray());

                if (UE.DEBUG_MODE)
                {
                    System.out.println("Skipped: " + extra.size());
                }

                extra.reset();
                sys = new SystemEntry(in);

                if (UE.DEBUG_MODE)
                {
                    System.out.println("Read: " + sys.NAME + " (" + sys.INDEX + ")");
                }
                
                SYSTEMS.add(sys);
                continue;
            } catch (Exception e) {
                // problem
                in.reset();
                // add 4 bytes to extra
                in.read(b);
                extra.write(b);
            }
        }

        // last
        sys.setExtra(extra.toByteArray());
        System.out.println("Skipped: " + extra.size());

        Collections.sort(SYSTEMS);
    }

    /**Returns file size**/
    @Override
    public long getSize() {
        long retval = 0;
        int i = SYSTEMS.size();
        for (int j = 0; j < i; j++) {
            retval += (SYSTEMS.get(j)).getSize();
        }
        return retval;
    }

    /**Returns number of entries**/
    public int getNumberOfEntries() {
        return SYSTEMS.size();
    }

    /**CAUTION! After adding the system, you must set image position or you'll have two systems drawn in the same sector*/
    //indexes!!!
    public int add(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry re = new SystemEntry(SYSTEMS.get(index));
        re.INDEX = SYSTEMS.size();
        SYSTEMS.add(re);
        Collections.sort(SYSTEMS);
        SystemEntry z;
        int ent = SYSTEMS.size();
        for (int x = 0; x < ent; x++) {
            z = SYSTEMS.get(x);
            z.INDEX = x;
        }
        this.markChanged();
        return SYSTEMS.size();
    }

    public void remove(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SYSTEMS.remove(index);
        Collections.sort(SYSTEMS);
        SystemEntry z;
        int ent = SYSTEMS.size();
        for (int x = 0; x < ent; x++) {
            z = SYSTEMS.get(x);
            z.INDEX = x;
        }
        this.markChanged();
    }

    /**used to save changes.*/
    @Override
    public void save(OutputStream out) throws Exception {
        SystemEntry z;
        int ent = SYSTEMS.size();
        for (int x = 0; x < ent; x++) {
            z = SYSTEMS.get(x);
            z.save(out);
        }
    }

    @Override
    public Vector<String> check() {
        Vector<String> response = new Vector<String>();
        String msg;
        SystemEntry z;
        Vector<String> temp;
        int ent = SYSTEMS.size();
        Hashtable<Integer, String> idset = new Hashtable<Integer, String>();
        for (int x = 0; x < ent; x++) {
            z = SYSTEMS.get(x);
            //index
            if (idset.containsKey(z.INDEX)) {
                msg = Language.getString("SystInfo.0"); //$NON-NLS-1$
                msg = msg.replace("%1", getName()); //$NON-NLS-1$
                msg = msg.replace("%2", z.NAME); //$NON-NLS-1$
                msg = msg.replace("%3", idset.get(z.INDEX)); //$NON-NLS-1$
                msg = msg.replace("%4", Integer.toString(z.INDEX)); //$NON-NLS-1$
                response.add(msg);
            } else {
                idset.put(z.INDEX, z.NAME);
            }
            temp = z.check(getName());
            if (!temp.isEmpty()) {
                response.addAll(temp);
            }
        }
        ArrayList<String> qpt = getGraphicFiles();
        String pt = null;
        AniOrCur AOC = null;
        TargaImage tar = null;
        for (int i = 0; i < qpt.size(); i++) {
            pt = qpt.get(i);
            if (pt.endsWith(".ani")) { //$NON-NLS-1$
                try {
                    AOC = STBOF.getAnimation(pt, false);
                } catch (FileNotFoundException fnf) {
                    response.add(getGenericErrorString(fnf.getMessage()));
                }
                if (AOC == null) {
                    msg = Language.getString("SystInfo.1"); //$NON-NLS-1$
                    msg = msg.replace("%1", getName()); //$NON-NLS-1$
                    msg = msg.replace("%2", pt); //$NON-NLS-1$
                    response.add(msg);
                }
            } else {
                try {
                    tar = STBOF.getTargaImage(pt, false);
                } catch (FileNotFoundException fnf) {
                    response.add(getGenericErrorString(fnf.getMessage()));
                }
                if (tar == null) {
                    msg = Language.getString("SystInfo.2"); //$NON-NLS-1$
                    msg = msg.replace("%1", getName()); //$NON-NLS-1$
                    msg = msg.replace("%2", pt); //$NON-NLS-1$
                    response.add(msg);
                }
            }
        }

        return response;
    }
    //gets :)

    /**Returns morale of the system.
    @param index system index*/
    public short getMorale(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        return z.MORALE;
    }

    /**Returns an array of system names.*/
    public String[] getNames() {
        int ent = SYSTEMS.size();
        String[] ret = new String[ent];
        SystemEntry z;
        for (int x = 0; x < ent; x++) {
            z = SYSTEMS.get(x);
            ret[x] = z.NAME;
        }
        return ret;
    }

    /**
     * Returns an array of system ids
     * @param filter < 0 means no filter, 0-4 set major race id, 5 - neutral
     * @return
     */
    public ID[] getSystemIDs(int filter, boolean populated_only)
    {
        ArrayList<ID> ids = new ArrayList<ID>();

        SystemEntry z;

        for (int x = 0; x < SYSTEMS.size(); x++)
        {
            z = SYSTEMS.get(x);

            if (populated_only && z.RESIDENT < 0)
            {
                continue;
            }

            if (filter < 0)
            {
                ids.add(new ID(z.NAME, x));
            }
            else if (filter >= 0 && filter < 5)
            {
                if (z.CONTROLING == filter)
                {
                    ids.add(new ID(z.NAME, x));
                }
            }
            else
            {
                if (z.CONTROLING >= 5)
                {
                    ids.add(new ID(z.NAME, x));
                }
            }
        }
        
        return ids.toArray(new ID[0]);
    }

    /**Returns the systems name
    @param index system index*/
    public String getName(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        return new String(z.NAME);
    }

    /**Returns the systems animation file
    @param index system index*/
    public String getAni(int index) throws InvalidArgumentsException {
        // TODO: what if it's a tga instead?
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        return new String(z.STARANI + ".ani"); //$NON-NLS-1$
    }

    /**Returns the systems graphic prefix
    @param index system index*/
    public String getGraphic(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        return new String(z.STARANI);
    }

    /**Returns the manager
    @param index system index*/
    public int getMan(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        if (z.MANAGE < 0 || z.MANAGE > 2) {
            z.MANAGE = 0;
        }
        return z.MANAGE;
    }

    /**Returns the star type
    @param index system index*/
    public byte getType(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        return z.STARTYPE;
    }

    /**Returns the population
    @param index system index*/
    public int getPopulation(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        return z.POPULATION;
    }

    /**Returns the resident race
    @param index system index*/
    public short getResident(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        return z.RESIDENT;
    }

    /**Returns the controling race
    @param index system index*/
    public int getControling(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        return z.CONTROLING;
    }

    /**Returns the population allocated to industry
    @param index system index*/
    public int getPop2Ind(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        return z.POP2IND;
    }

    /**Returns the population allocated to energy
    @param index system index*/
    public int getPop2Ene(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        return z.POP2ENE;
    }

    /**Returns the population allocated to food
    @param index system index*/
    public int getPop2Food(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        return z.POP2FOO;
    }

    /**Returns the population allocated to research
    @param index system index*/
    public int getPop2Res(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        return z.POP2RES;
    }

    /**Returns the population allocated to intel
    @param index system index*/
    public int getPop2Intel(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        return z.POP2INT;
    }

    /**
     * Returns the system type (politics)
     * @param index system index
     */
    public byte getSystemType(int index) throws InvalidArgumentsException
    {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        return z.SYSTEM_TYPE;
    }

    /**
     * Returns the system build queue order type
     * @param index system index
     */
    public int getBuildQueueOrderType(int index) throws InvalidArgumentsException
    {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        return z.BUILD_QUEUE_ORDER_TYPE;
    }

    /**
     * Returns if the system was explored by given empire
     * @param index system index
     * @param empire empire index
     * @returns true if the mepire has explored this system
     */
    public boolean isExploredBy(int index, byte empire) throws InvalidArgumentsException
    {
        checkSystemIndex(index);

        SystemEntry z = SYSTEMS.get(index);

        return (z.EXPLORED_BY & empire) == 0;
    }

    /**Returns true if dilithium is present
    @param index system index*/
    public boolean hasDilithium(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        return (z.SYS_REQ & 0x0800) != 0;
    }

    /**Returns the number of planets
    @param index system index*/
    public int planets(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        return z.PLANETS;
    }

    /**Returns the planet names
    @param index system index*/
    public String[] planetNames(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        String[] ret = null;
        if (z.PLANETS > 0) {
            ret = new String[z.PLANETS];
            PlanetEntry pe;
            for (int i = 0; i < z.PLANETS; i++) {
                pe = z.PLANET.get(i);
                ret[i] = new String(pe.NAME);
            }
        }
        return ret;
    }
    //sets :)

    /**Sets the system name
    @param index system index
    @param name the new name*/
    public void setName(int index, String name) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        if (name.length() > 39) {
            name = name.substring(0, 39);
        }
        if (z.NAME.equals(name)) {
            return;
        }
        z.NAME = new String(name);
        this.markChanged();
    }

    /**Sets the systems animation file
    @param index system index
    @param ani animation file (example: 'vtos.ani')*/
    public void setAni(int index, String ani) throws InvalidArgumentsException {
        checkSystemIndex(index);
        ani = ani.toLowerCase();
        if (!ani.endsWith(".ani")) { //$NON-NLS-1$
            String msg = Language.getString("SystInfo.3"); //$NON-NLS-1$
            msg = msg.replace("%1", ani); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
        if (ani.length() > 8) {
            return;
        }
        ani = ani.substring(0, ani.length() - 4);
        SystemEntry z = SYSTEMS.get(index);
        if (z.STARANI.equals(ani)) {
            return;
        }
        z.STARANI = new String(ani);
        this.markChanged();
    }

    /**Sets the manager
    @param index system index
    @param man the manager (0-none, 1-ai, 2-player)*/
    public void setMan(int index, int man) throws InvalidArgumentsException {
        checkSystemIndex(index);
        if (man < 0 || man > 2) {
            man = 0;
        }
        SystemEntry z = SYSTEMS.get(index);
        if (z.MANAGE == man) {
            return;
        }
        z.MANAGE = man;
        this.markChanged();
    }

    /**Sets the star type
    @param index system index
    @param star the star(object) type*/
    public void setType(int index, byte star) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        if (z.STARTYPE == star) {
            return;
        }
        z.STARTYPE = star;
        this.markChanged();
    }

    /**Sets the morale
    @param index system index
    @param morale the morale value (1-194)*/
    public void setMorale(int index, short morale) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        if (z.MORALE == morale) {
            return;
        }
        if (morale < 0) {
            morale = 0;
        }
        z.MORALE = morale;
        if (morale == 0) {
            z.MORALE_DESC = 0;
        } else if (morale < 25) {
            z.MORALE_DESC = -4;
        } else if (morale < 51) {
            z.MORALE_DESC = -3;
        } else if (morale < 76) {
            z.MORALE_DESC = -2;
        } else if (morale < 100) {
            z.MORALE_DESC = -1;
        } else if (morale < 126) {
            z.MORALE_DESC = 0;
        } else if (morale < 151) {
            z.MORALE_DESC = 1;
        } else if (morale < 176) {
            z.MORALE_DESC = 2;
        } else {
            z.MORALE_DESC = 3;
        }
        this.markChanged();
    }

    /**Sets the population
    @param index system index
    @param pop population*/
    public void setPopulation(int index, int pop) throws InvalidArgumentsException {
        checkSystemIndex(index);
        if (pop < 0) {
            return;
        }
        SystemEntry z = SYSTEMS.get(index);
        if (z.POPULATION == pop) {
            return;
        }
        z.POPULATION = pop;
        this.markChanged();
    }

    /**Sets the resident race
    @param index system index
    @param race the resident race*/
    public void setResident(int index, short race) throws InvalidArgumentsException {
        checkSystemIndex(index);
        if (race < 0) {
            return;
        }
        SystemEntry z = SYSTEMS.get(index);
        if (z.RESIDENT == race) {
            return;
        }
        z.RESIDENT = race;
        this.markChanged();
    }

    /**Sets the controling race
    @param index system index
    @param race the controling race*/
    public void setControling(int index, short race) throws InvalidArgumentsException {
        checkSystemIndex(index);
        if (race < 0) {
            return;
        }
        SystemEntry z = SYSTEMS.get(index);
        if (z.CONTROLING == race) {
            return;
        }
        z.CONTROLING = race;
        this.markChanged();
    }

    /**Sets the population allocated to industry
    @param index system index
    @param pop the population allocated*/
    public void setPop2Ind(int index, int pop) throws InvalidArgumentsException {
        checkSystemIndex(index);
        if (pop < 0) {
            return;
        }
        int other = getPop2Ene(index) + getPop2Food(index) + getPop2Res(index) + getPop2Intel(index);
        other = getPopulation(index) - other;
        if (other >= 0 && other < pop) {
            pop = other;
        }
        SystemEntry z = SYSTEMS.get(index);
        if (z.POP2IND == pop) {
            return;
        }
        z.POP2IND = pop;
        this.markChanged();
    }

    /**Sets the population allocated to energy
    @param index system index
    @param pop the population allocated*/
    public void setPop2Ene(int index, int pop) throws InvalidArgumentsException {
        checkSystemIndex(index);
        if (pop < 0) {
            return;
        }
        int other = getPop2Ind(index) + getPop2Food(index) + getPop2Res(index) + getPop2Intel(index);
        other = getPopulation(index) - other;
        if (other >= 0 && other < pop) {
            pop = other;
        }
        SystemEntry z = SYSTEMS.get(index);
        if (z.POP2ENE == pop) {
            return;
        }
        z.POP2ENE = pop;
        this.markChanged();
    }

    /**Sets the population allocated to food
    @param index system index
    @param pop the population allocated*/
    public void setPop2Food(int index, int pop) throws InvalidArgumentsException {
        checkSystemIndex(index);
        if (pop < 0) {
            return;
        }
        int other = getPop2Ind(index) + getPop2Ene(index) + getPop2Res(index) + getPop2Intel(index);
        other = getPopulation(index) - other;
        if (other >= 0 && other < pop) {
            pop = other;
        }
        SystemEntry z = SYSTEMS.get(index);
        if (z.POP2FOO == pop) {
            return;
        }
        z.POP2FOO = pop;
        this.markChanged();
    }

    /**Sets the population allocated to research
    @param index system index
    @param pop the population allocated*/
    public void setPop2Res(int index, int pop) throws InvalidArgumentsException {
        checkSystemIndex(index);
        if (pop < 0) {
            return;
        }
        int other = getPop2Ind(index) + getPop2Ene(index) + getPop2Food(index) + getPop2Intel(index);
        other = getPopulation(index) - other;
        if (other >= 0 && other < pop) {
            pop = other;
        }
        SystemEntry z = SYSTEMS.get(index);
        if (z.POP2RES == pop) {
            return;
        }
        z.POP2RES = pop;
        this.markChanged();
    }

    /**Sets the population allocated to intel
    @param index system index
    @param pop the population allocated*/
    public void setPop2Intel(int index, int pop) throws InvalidArgumentsException {
        checkSystemIndex(index);
        if (pop < 0) {
            return;
        }
        int other = getPop2Ind(index) + getPop2Ene(index) + getPop2Food(index) + getPop2Res(index);
        other = getPopulation(index) - other;
        if (other >= 0 && other < pop) {
            pop = other;
        }
        SystemEntry z = SYSTEMS.get(index);
        if (z.POP2INT == pop) {
            return;
        }
        z.POP2INT = pop;
        this.markChanged();
    }

    /**
     * Sets the system type (politics)
     * @param index system index
     * @param type use one of the SYS_TYPE fields.
     */
    public void setSystemType(int index, byte type) throws InvalidArgumentsException
    {
        checkSystemIndex(index);

        SystemEntry z = SYSTEMS.get(index);

        if (z.SYSTEM_TYPE != type)
        {
            z.SYSTEM_TYPE = type;
            this.markChanged();
        }
    }

    /**
     * Sets the systems build queue order type
     * @param index system index
     * @param type use one of the QUEUE_ORDER_TYPE fields.
     */
    public void setBuildQueueOrderType(int index, int type) throws InvalidArgumentsException
    {
        checkSystemIndex(index);

        SystemEntry z = SYSTEMS.get(index);

        if (z.BUILD_QUEUE_ORDER_TYPE != type)
        {
            z.BUILD_QUEUE_ORDER_TYPE = type;
            this.markChanged();
        }
    }

    /**Sets if dilithium is present
    @param index system index
    @param dil true if dilithium should be present*/
    public void setDilithium(int index, boolean dil) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        if (hasDilithium(index) == dil) {
            return;
        }
        if (dil) {
            z.SYS_REQ = z.SYS_REQ | 0x0800;
            z.DIL_ICON = 1;
        } else {
            z.SYS_REQ = z.SYS_REQ & ~0x0800;
            z.DIL_ICON = 0;
        }
        this.markChanged();
    }

    /**
     * Sets if the system was explored by given empire
     * @param index system index
     * @param empire empire index
     * @param explored set to true if the mepire has explored this system
     */
    public void setExploredBy(int index, byte empire, boolean explored) throws InvalidArgumentsException
    {
        checkSystemIndex(index);

        SystemEntry z = SYSTEMS.get(index);

        if (((z.EXPLORED_BY & empire) == 0) != explored)
        {
            if (explored)
            {
                z.EXPLORED_BY = (byte)(z.EXPLORED_BY & (~empire));
            }
            else
            {
                z.EXPLORED_BY = (byte)(z.EXPLORED_BY | empire);
            }
            
            this.markChanged();
        }
    }

    public void setExploredByMask(int index, byte mask) throws InvalidArgumentsException
    {
        checkSystemIndex(index);

        SystemEntry z = SYSTEMS.get(index);

        if (z.EXPLORED_BY != mask)
        {
            z.EXPLORED_BY = mask;
            this.markChanged();
        }
    }

    /**Sets the position of the system.
    @param index system index
    @param v_row the index of the vertical row (0+)
    @param h_row the index of the horizontal row (0+)
     */
    public void setPosition(int index, int v_row, int h_row) throws InvalidArgumentsException {
        checkSystemIndex(index);
        int vpos = v_row * 5 + (int) (Math.random() * 3) + 1;
        int hpos = h_row * 5 + (int) (Math.random() * 3) + 1;
        SystemEntry z = SYSTEMS.get(index);
        z.V_POS = vpos;
        z.H_POS = hpos;
        this.markChanged();
    }
    //planet manipulation
    //get stuff

    /**returns an array of planet names.
    @param index system index
    @return an array of planet names*/
    public String[] getPlanetNames(int index) throws InvalidArgumentsException {
        checkSystemIndex(index);
        SystemEntry z = SYSTEMS.get(index);
        int num = z.PLANETS;
        String[] ret = new String[num];
        PlanetEntry planet;
        for (int i = 0; i < num; i++) {
            planet = z.PLANET.get(i);
            ret[i] = new String(planet.NAME);
        }
        return ret;
    }

    /**returns a list of animation files used by system stars and planets*/
    public ArrayList<String> getGraphicFiles() {
        ArrayList<String> ret = new ArrayList<String>();
        int siz = SYSTEMS.size();
        SystemEntry z;
        String str;
        for (int index = 0; index < siz; index++) {
            z = SYSTEMS.get(index);
            if (z.STARANI.equals("neb-")) { //$NON-NLS-1$
                str = z.STARANI + ".tga"; //$NON-NLS-1$
            } else {
                str = z.STARANI + ".ani"; //$NON-NLS-1$
            }
            if (!ret.contains(str)) {
                ret.add(str);
            }
            if (!ret.contains(z.STARANI + "map.tga")) { //$NON-NLS-1$
                ret.add(z.STARANI + "map.tga"); //$NON-NLS-1$ //$NON-NLS-2$
            }
            if (!ret.contains(z.STARANI + "maps.tga")) { //$NON-NLS-1$
                ret.add(z.STARANI + "maps.tga"); //$NON-NLS-1$ //$NON-NLS-2$
            }
            PlanetEntry pe;
            for (int i = 0; i < z.PLANETS; i++) {
                pe = z.PLANET.get(i);
                if (!ret.contains(pe.PIC)) {
                    ret.add(new String(pe.PIC));
                }
            }
        }
        return ret;
    }

    /**Returns the name of the planet at the specified index.
    @param sys the system index.
    @param index the index of the planet to get the name of.
     */
    public String getName(int sys, int index) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        return new String(planet.NAME);
    }

    /**Returns the name of the planet's animation file at the specified index.
    @param sys the system index.
    @param index the index of the planet to get the name of the animation file.
     */
    public String getAnimation(int sys, int index) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        return new String(planet.PIC);
    }

    /**Returns the size of the planet at the specified index.
    @param sys the system index.
    @param index the index of the planet to get the size of
     */
    public short getSize(int sys, int index) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        return planet.SIZE;
    }

    /**Returns the terraforming points of the planet at the specified index.
    @param sys the system index.
    @param index the index of the planet to get the terraforming points of
     */
    public int getTerraPoints(int sys, int index) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        return planet.TERRA_POINTS;
    }

    /**Returns how much of the planet at the specified index is terraformed.
    @param sys the system index.
    @param index the index of the planet
     */
    public int getTerraformed(int sys, int index) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        return planet.TERRAFORMED;
    }

    /**Returns the planet bonus of the planet at the specified index.
    @param sys the system index.
    @param index the index of the planet
     */
    public byte getPltBoni(int sys, int index) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        return planet.PLT_BONI;
    }

    /**Returns the universal bonus of the planet at the specified index.
    @param sys the system index.
    @param index the index of the planet
     */
    public byte getUniBoni(int sys, int index) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        return planet.UNI_BONI;
    }

    /**Returns the population bonus of the planet at the specified index.
    @param sys the system index.
    @param index the index of the planet
     */
    public byte getPopBoni(int sys, int index) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        return planet.POP_BONI;
    }

    /**Returns the type of the planet at the specified index.
    @param sys the system index.
    @param index the index of the planet
     */
    public byte getType(int sys, int index) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        return planet.TYPE;
    }

    /**Returns the atmosphere of the planet at the specified index.
    @param sys the system index.
    @param index the index of the planet
     */
    public byte getAtmosphere(int sys, int index) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        return planet.ATMO;
    }
    //set stuff

    /**Sets the name of the planet at the specified index.
    @param sys the system index.
    @param index the index of the planet.
    @param newName the new name of the planet
    @return true if changes were made
     */
    public boolean setName(int sys, int index, String newName) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        if (newName.length() > 39) {
            newName = newName.substring(0, 39);
        }
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        if (newName.equals(planet.NAME)) {
            return false;
        }
        planet.NAME = new String(newName);
        this.markChanged();
        return true;
    }

    /**Sets the name of the planet's animation file at the specified index.
    @param sys the system index.
    @param index the index of the planet.
    @param newAni the new animation file
    @return true if changes were made
     */
    public boolean setAnimation(int sys, int index, String newAni) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        if (newAni.length() > 12) {
            return false;
        }
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        if (newAni.equals(planet.PIC)) {
            return false;
        }
        planet.PIC = new String(newAni);
        this.markChanged();
        return true;
    }

    /**Sets the size of the planet at the specified index.
    @param sys the system index.
    @param index the index of the planet to get the size of
    @param size the new size, can be 0, 1 or 2
    @return true if changes were made
     */
    public boolean setSize(int sys, int index, short size) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        if (size < 0 || size > 2) {
            String msg = Language.getString("SystInfo.4"); //$NON-NLS-1$
            msg = msg.replace("%1", Short.toString(size)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        if (size == planet.SIZE) {
            return false;
        }
        planet.SIZE = size;
        this.markChanged();
        return true;
    }

    /**Sets the terraforming points of the planet at the specified index.
    @param sys the system index.
    @param index the index of the planet to set the terraforming points of
    @param terra the terraforming points
    @return true if data has been changed
     */
    public void setTerraPoints(int sys, int index, int terra) throws InvalidArgumentsException {
        checkSystemIndex(sys);

        SystemEntry z = SYSTEMS.get(sys);
        PlanetEntry planet = z.PLANET.get(index);

        if (planet == null)
        {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }

        terra = terra & 0xFFFF;
        
        if (planet.TERRA_POINTS == terra)
        {
            return;
        }
        planet.TERRA_POINTS = terra;
        this.markChanged();
    }

    /**Sets how much of the planet at the specified index is terraformed.
    @param sys the system index.
    @param index the index of the planet
    @param terra how many terraforming points have been terraformed
    @return true if data has been changed
     */
    public void setTerraformed(int sys, int index, int terra) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }

        terra = terra & 0xFFFF;

        if (planet.TERRAFORMED == terra)
        {
            return;
        }
        planet.TERRAFORMED = terra;
        this.markChanged();
    }

    /**Sets the planet bonus of the planet at the specified index.
    @param sys the system index.
    @param index the index of the planet
    @param boni the bonus
    @return true if data has been changed
     */
    public boolean setPltBoni(int sys, int index, byte boni) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        if (boni < 0 || boni > 2) {
            String msg = Language.getString("SystInfo.5"); //$NON-NLS-1$
            msg = msg.replace("%1", Byte.toString(boni)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        if (planet.PLT_BONI == boni) {
            return false;
        }
        planet.PLT_BONI = boni;
        this.markChanged();
        return true;
    }

    /**Sets the universal bonus of the planet at the specified index.
    @param sys the system index.
    @param index the index of the planet
    @param boni the bonus
    @return true if data has been changed
     */
    public boolean setUniBoni(int sys, int index, byte boni) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        if (boni < 0 || boni > 2) {
            String msg = Language.getString("SystInfo.6"); //$NON-NLS-1$
            msg = msg.replace("%1", Byte.toString(boni)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        if (planet.UNI_BONI == boni) {
            return false;
        }
        planet.UNI_BONI = boni;
        this.markChanged();
        return true;
    }

    /**Sets the population bonus of the planet at the specified index.
    @param sys the system index.
    @param index the index of the planet
    @param boni the bonus
    @return true if data has been changed
     */
    public boolean setPopBoni(int sys, int index, byte boni) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        if (boni < 0 || boni > 2) {
            String msg = Language.getString("SystInfo.7"); //$NON-NLS-1$
            msg = msg.replace("%1", Byte.toString(boni)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        if (planet.POP_BONI == boni) {
            return false;
        }
        planet.POP_BONI = boni;
        this.markChanged();
        return true;
    }

    /**Sets the type of the planet at the specified index.
    @param sys the system index.
    @param index the index of the planet
    @param type the planet type
    @return true if data has been changed
     */
    public boolean setType(int sys, int index, byte type) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        if (planet.TYPE == type) {
            return false;
        }
        planet.TYPE = type;
        resetSysReq(sys);
        this.markChanged();
        return true;
    }

    /**Sets the atmosphere of the planet at the specified index.
    @param sys the system index.
    @param index the index of the planet
    @param atmo the planet atmosphere
    @return true if data has been changed
     */
    public boolean setAtmosphere(int sys, int index, byte atmo) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        PlanetEntry planet = z.PLANET.get(index);
        if (planet == null) {
            throw new InvalidArgumentsException(
                    getPlanetIndexErrorString(sys, index));
        }
        if (planet.ATMO == atmo) {
            return false;
        }
        planet.ATMO = atmo;
        this.markChanged();
        return true;
    }
    //add, remove, position(up, down) planet

    /**Adds a planet to the end of a systems planet list
    @param sys the system index
     */
    public void addPlanet(int sys) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        PlanetEntry pe = null;
        PlanetEntry pew = null;
        if (z.PLANETS > 0) {
            //there must be a planet at index 0
            pe = z.PLANET.get(0);
        } else {
            int ia = 0;
            SystemEntry se;
            do {
                se = SYSTEMS.get(ia);
                if (se.PLANETS > 0) {
                    pe = se.PLANET.get(0);
                }
                ia++;
            } while (se.PLANETS < 1 && ia < SYSTEMS.size());
            if (pe == null) {
                throw new InvalidArgumentsException(Language.getString("SystInfo.8")); //$NON-NLS-1$
            }
        }
        pew = new PlanetEntry(pe);
        pew.INDEX = (short) z.PLANETS;
        z.PLANET.add(pew);
        z.PLANETS = z.PLANET.size();
        resetSysReq(sys);
        this.markChanged();
    }

    /**Removes a planet from a systems planet list
    @param sys the system index
    @param index the planet index
     */
    public void removePlanet(int sys, int index) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        z.PLANET.remove(index);
        z.PLANETS = z.PLANET.size();
        //set indexes
        PlanetEntry pe;
        for (int i = index; i < z.PLANETS; i++) {
            pe = z.PLANET.get(i);
            pe.INDEX = (short) i;
        }
        resetSysReq(sys);
        this.markChanged();
    }

    /**Moves a planet in systems planet list one position up
    @param sys the system index
    @param index the planet index
     */
    public void movePlanetUp(int sys, short index) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        if (index == 0) {
            return;
        }
        PlanetEntry temp = z.PLANET.get(index);
        PlanetEntry temp2 = z.PLANET.get(index - 1);
        if (temp == null) {
            throw new InvalidArgumentsException(getPlanetIndexErrorString(sys, index));
        }
        if (temp2 == null) {
            throw new InvalidArgumentsException(getPlanetIndexErrorString(sys, index - 1));
        }
        temp2.INDEX = index;
        temp.INDEX = (short) (index - 1);
        z.PLANET.set(index, temp2);
        z.PLANET.set(index - 1, temp);
        z.PLANETS = z.PLANET.size();
        this.markChanged();
    }

    /**Moves a planet in systems planet list one position down
    @param sys the system index
    @param index the planet index
     */
    public void movePlanetDown(int sys, short index) throws InvalidArgumentsException {
        checkSystemIndex(sys);
        SystemEntry z = SYSTEMS.get(sys);
        if (index == z.PLANETS - 1) {
            return;
        }
        PlanetEntry temp = z.PLANET.get(index);
        PlanetEntry temp2 = z.PLANET.get(index + 1);
        if (temp == null) {
            throw new InvalidArgumentsException(getPlanetIndexErrorString(sys, index));
        }
        if (temp2 == null) {
            throw new InvalidArgumentsException(getPlanetIndexErrorString(sys, index + 1));
        }
        temp2.INDEX = index;
        temp.INDEX = (short) (index + 1);
        z.PLANET.set(index, temp2);
        z.PLANET.set(index + 1, temp);
        z.PLANETS = z.PLANET.size();
        this.markChanged();
    }

    private void checkSystemIndex(int index) throws InvalidArgumentsException {
        if (index < 0 || index >= SYSTEMS.size()) {
            String msg = Language.getString("SystInfo.9"); //$NON-NLS-1$
            msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
            throw new InvalidArgumentsException(msg);
        }
    }

    private String getGenericErrorString(String error) {
        String msg = Language.getString("SystInfo.10"); //$NON-NLS-1$
        msg = msg.replace("%1", getName()); //$NON-NLS-1$
        msg = msg.replace("%2", error); //$NON-NLS-1$
        return msg;
    }

    private String getPlanetIndexErrorString(int sys, int index) {
        String msg = Language.getString("SystInfo.11"); //$NON-NLS-1$
        msg = msg.replace("%1", getName()); //$NON-NLS-1$
        msg = msg.replace("%2", "(" + sys + ", " + index + ")"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
        return msg;
    }

    private void resetSysReq(int sys)
    {
        SystemEntry system = SYSTEMS.get(sys);

        system.SYS_REQ = system.SYS_REQ & 0xFFFFFF00;

        PlanetEntry planet;

        for (int p = 0; p < system.PLANET.size(); p++)
        {
            planet = system.PLANET.get(p);
            system.SYS_REQ = system.SYS_REQ | (int)Math.pow(2, planet.TYPE);
        }

        this.markChanged();
    }

    //reprisents each system entry
    class SystemEntry implements Comparable<SystemEntry>
    {
        //identified and editable
        public int INDEX;                           //   4 system index
        public int MANAGE;                          //   8 managed by (0-noone, 1-human, 2-AI)
        public String NAME;                         //  48 system name
        public byte SYSTEM_TYPE;                    //  49 0=home, 1=native, 2=non native, 3=subjugated, 4=affiliated, 5=independent minor, 6=conquered home, 7&8=invalid?, 9=any, A=rebel, B=empty
        public byte STARTYPE;                       //  50 type of star (object)
        public String STARANI;                      //  55 graphics files
        private byte[] UNKNOWN_2 = new byte[8];     //  63
        public byte EXPLORED_BY;                    //  64 XOR empire index
        public int POPULATION;                      //  68 system population
        public short RESIDENT;                      //  70 resident race
        private byte[] UNKNOWN_3 = new byte[6];     //  76
        public int CONTROLING;                      //  80 controling race
    //    private byte[] UNKNOWN_4 = new byte[4];   //  84 - index, again
        public int POP2IND;                         //  88 population allocated to industry (NOTE: this IS population NOT units!)
        public int POP2ENE;                         //  92 population allocated to energy
        public int POP2FOO;                         //  96 population allocated to food
        public int POP2RES;                         // 100 population allocated to research
        public int POP2INT;                         // 104 population allocated to intel
        private byte[] UNKNOWN_5 = new byte[2];     // 106
        public int PLANETS;                         // 110 number of planets
        public byte DIL_ICON;                       // 111 dilithium (0-no, 1-yes)
        private byte[] UNKNOWN_6 = new byte[1];     // 112
        public int SYS_REQ;                         // 116 sys req (8 - dilithium)
        private byte[] UNKNOWN_7 = new byte[4];     // 120
        public int H_POS;                           // 124 horizontal position (of the ????map.tga) on map
        public int V_POS;                           // 128 vertical position (of the ????map.tga) on map
        public int BUILD_QUEUE_ORDER_TYPE;          // 132 1=build building, 2=upgrade building(s), 3=build ship, 5=tradegoods, 7=unrest order
        public int INVESTED_INDUSTRY;               // 136
        
        private byte[] UNKNOWN_8 = new byte[596];   // 732
        public short MORALE;                        // 734 system morale (1-194)
        public short MORALE_DESC;                   // 736 -4:rebellious(1-24),-3:defiant(25-50),-2:disgruntled(51-75), -1:apathetic(76-99),0:content(100-125),1:pleased(126-150),2:loyal(151-175),3:fanatic(176-194)*/
        private byte[] UNKNOWN_9 = new byte[72];    // 808

        public ArrayList<PlanetEntry> PLANET = new ArrayList<PlanetEntry>();
        private byte[] EXTRA;

        //constructor
        public SystemEntry(InputStream in) throws IOException {
            byte[] fourB = new byte[4];
            byte[] twoB = new byte[2];
            byte[] str = new byte[40];

            //index 0-4
            in.read(fourB);
            INDEX = Tools.toInt(fourB, true);
            //man 4-4
            in.read(fourB);
            MANAGE = Tools.toInt(fourB, true);
            //name: 8-40 bytes
            in.read(str);
            NAME = Tools.toASCII(str, 0, 40);
            //system type 48-1
            SYSTEM_TYPE = (byte) in.read();
            //startype 49-1
            STARTYPE = (byte) in.read();
            //starani 50-5
            in.read(str, 0, 5);
            STARANI = Tools.toASCII(str, 0, 5);
            //unknown_2 55-8
            in.read(UNKNOWN_2);
            // explored by
            EXPLORED_BY = (byte)in.read();
            //population 64-4
            in.read(fourB);
            POPULATION = Tools.toInt(fourB, true);
            //resident 68-2
            in.read(twoB);
            RESIDENT = Tools.toShort(twoB, true);
            //unknown_3 70-6
            in.read(UNKNOWN_3);
            //controling race 76-4
            in.read(fourB);
            CONTROLING = Tools.toInt(fourB, true);
            // repeated index 4 80-4
            in.skip(4);
            //population allocated to industry 84-4
            in.read(fourB);
            POP2IND = Tools.toInt(fourB, true);
            //population allocated to energy 88-4
            in.read(fourB);
            POP2ENE = Tools.toInt(fourB, true);
            //population allocated to food 92-4
            in.read(fourB);
            POP2FOO = Tools.toInt(fourB, true);
            //population allocated to research 96-4
            in.read(fourB);
            POP2RES = Tools.toInt(fourB, true);
            //population allocated to intel 100-4
            in.read(fourB);
            POP2INT = Tools.toInt(fourB, true);
            //unknown_5 104-2
            in.read(UNKNOWN_5);
            //planets 106-4
            in.read(fourB);
            PLANETS = Tools.toInt(fourB, true);
            DIL_ICON = (byte) in.read();
            //unknown_6 111-1
            in.read(UNKNOWN_6);
            //dilithium 112-4
            in.read(fourB);
            SYS_REQ = Tools.toInt(fourB, true);
            //unknown_7 116-4
            in.read(UNKNOWN_7);
            //horizontal position 120-4
            in.read(fourB);
            H_POS = Tools.toInt(fourB, true);
            //vertical position 124-4
            in.read(fourB);
            V_POS = Tools.toInt(fourB, true);
            //vertical position 128-4
            in.read(fourB);
            BUILD_QUEUE_ORDER_TYPE = Tools.toInt(fourB, true);
            // industry invested
            in.read(fourB);
            INVESTED_INDUSTRY = Tools.toInt(fourB, true);
            //unknown_8 136-596
            in.read(UNKNOWN_8);
            //morale 732-2
            in.read(twoB);
            MORALE = Tools.toShort(twoB, true);
            //morale description 734-2
            in.read(twoB);
            MORALE_DESC = Tools.toShort(twoB, true);
            //unknown_9 736-72
            in.read(UNKNOWN_9);
            //planet data
            byte[] d = new byte[76];
            PlanetEntry pe;
            for (int i = 0; i < PLANETS; i++) {
                in.read(d);
                pe = new PlanetEntry(d);
                PLANET.add(pe);
            }
            //EXTRA
            EXTRA = new byte[0];

            System.out.println(NAME + ": " + BUILD_QUEUE_ORDER_TYPE);
        }
        //copy constructor

        public SystemEntry(SystemEntry source) {
            INDEX = source.INDEX;
            MANAGE = source.MANAGE;
            NAME = source.NAME;
            PLANETS = source.PLANETS;
            STARTYPE = source.STARTYPE;
            STARANI = source.STARANI;
            EXPLORED_BY = source.EXPLORED_BY;
            POPULATION = source.POPULATION;
            RESIDENT = source.RESIDENT;
            CONTROLING = source.CONTROLING;
            POP2IND = source.POP2IND;
            POP2ENE = source.POP2ENE;
            POP2FOO = source.POP2FOO;
            POP2RES = source.POP2RES;
            POP2INT = source.POP2INT;
            SYS_REQ = source.SYS_REQ;
            DIL_ICON = source.DIL_ICON;
            H_POS = source.H_POS;
            V_POS = source.V_POS;
            BUILD_QUEUE_ORDER_TYPE = source.BUILD_QUEUE_ORDER_TYPE;
            MORALE = source.MORALE;
            MORALE_DESC = source.MORALE_DESC;
            INVESTED_INDUSTRY = source.INVESTED_INDUSTRY;

            PlanetEntry pe;

            for (int i = 0; i < source.PLANET.size(); i++) {
                pe = source.PLANET.get(i);
                PLANET.add(new PlanetEntry(pe));
            }
            SYSTEM_TYPE = source.SYSTEM_TYPE;
            UNKNOWN_2 = source.UNKNOWN_2;
            UNKNOWN_3 = source.UNKNOWN_3;
            UNKNOWN_5 = source.UNKNOWN_5;
            UNKNOWN_6 = source.UNKNOWN_6;
            UNKNOWN_7 = source.UNKNOWN_7;
            UNKNOWN_8 = source.UNKNOWN_8;
            UNKNOWN_9 = source.UNKNOWN_9;
            EXTRA = source.EXTRA;
        }

        @Override
        public int compareTo(SystemEntry se) throws ClassCastException {
            return (INDEX - se.INDEX);
        }

        public boolean equals(SystemEntry se) {
            return (se.INDEX == INDEX);
        }
        //saves

        public boolean save(OutputStream out) throws IOException {
            byte[] b;
            //index 0-4
            b = Tools.toByte(INDEX, true);
            out.write(b);
            //man 4-4
            b = Tools.toByte(MANAGE, true);
            out.write(b);
            //name: 8-40 bytes
            b = Tools.toByte(NAME, 40, 39);
            out.write(b);
            //unknown_1 48-1
            out.write(SYSTEM_TYPE);
            //startype 49-1
            out.write(STARTYPE);
            //starani 50-5
            b = Tools.toByte(STARANI, 5, 4);
            out.write(b);
            //unknown_2 55-8
            out.write(UNKNOWN_2);
            // explored by
            out.write(EXPLORED_BY);
            //population 64-4
            b = Tools.toByte(POPULATION, true);
            out.write(b);
            //resident 68-2
            b = Tools.toByte(RESIDENT, true);
            out.write(b);
            //unknown_3 70-6
            out.write(UNKNOWN_3);
            //controling race 76-4
            b = Tools.toByte(CONTROLING, true);
            out.write(b);
            //repeated index 80-4
            out.write(Tools.toByte(INDEX, true));
            //population allocated to industry 84-4
            b = Tools.toByte(POP2IND, true);
            out.write(b);
            //population allocated to energy 88-4
            b = Tools.toByte(POP2ENE, true);
            out.write(b);
            //population allocated to food 92-4
            b = Tools.toByte(POP2FOO, true);
            out.write(b);
            //population allocated to research 96-4
            b = Tools.toByte(POP2RES, true);
            out.write(b);
            //population allocated to intel 100-4
            b = Tools.toByte(POP2INT, true);
            out.write(b);
            //unknown_5 104-2
            out.write(UNKNOWN_5);
            //planets 106-4
            PLANETS = PLANET.size();
            b = Tools.toByte(PLANETS, true);
            out.write(b);
            //dilithium icon 110-1
            out.write(DIL_ICON);
            //unknown_6 111-2
            out.write(UNKNOWN_6);
            //dilithium 113-1
            out.write(Tools.toByte(SYS_REQ, true));
            //unknown_7 114-6
            out.write(UNKNOWN_7);
            //horizontal position 120-4
            b = Tools.toByte(H_POS, true);
            out.write(b);
            //vertical position 124-4
            b = Tools.toByte(V_POS, true);
            out.write(b);
            // build queue order type 128-4
            out.write(Tools.toByte(BUILD_QUEUE_ORDER_TYPE, true));
            // industry invested 132-4
            out.write(Tools.toByte(INVESTED_INDUSTRY, true));
            //unknown_8 136-596
            out.write(UNKNOWN_8);
            //morale 732-2
            b = Tools.toByte(MORALE, true);
            out.write(b);
            //morale description 734-2
            b = Tools.toByte(MORALE_DESC, true);
            out.write(b);
            //unknown_9 736-72
            out.write(UNKNOWN_9);
            //planet data
            PlanetEntry pe;
            for (int i = 0; i < PLANETS; i++) {
                pe = PLANET.get(i);
                pe.save(out);
            }
            //EXTRA
            if (EXTRA != null) {
                out.write(EXTRA);
            }
            return true;
        }

        public long getSize() {
            if (EXTRA == null) {
                return (808 + PLANETS * 76);
            }
            return (808 + PLANETS * 76 + EXTRA.length);
        }

        @Override
        public String toString() {
            return NAME;
        }

        public Vector<String> check(String name) {
            Vector<String> response = new Vector<String>();
            String msg;

            if (MANAGE < 0 || MANAGE > 2) {
                msg = Language.getString("SystInfo.12"); //$NON-NLS-1$
                msg = msg.replace("%1", name); //$NON-NLS-1$
                msg = msg.replace("%2", NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Integer.toString(MANAGE)); //$NON-NLS-1$
                response.add(msg);
            }
            if (NAME.length() > 39) {
                msg = Language.getString("SystInfo.13"); //$NON-NLS-1$
                msg = msg.replace("%1", name); //$NON-NLS-1$
                msg = msg.replace("%2", NAME); //$NON-NLS-1$
                response.add(msg);
            }
            if (STARTYPE < 0 || STARTYPE > 11) {
                msg = Language.getString("SystInfo.14"); //$NON-NLS-1$
                msg = msg.replace("%1", name); //$NON-NLS-1$
                msg = msg.replace("%2", NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Integer.toString(STARTYPE)); //$NON-NLS-1$
                response.add(msg);
            }
            if (STARANI.length() > 4) {
                msg = Language.getString("SystInfo.15"); //$NON-NLS-1$
                msg = msg.replace("%1", name); //$NON-NLS-1$
                msg = msg.replace("%2", NAME); //$NON-NLS-1$
                msg = msg.replace("%3", STARANI); //$NON-NLS-1$
                response.add(msg);
            }
            if (POPULATION < 0) {
                msg = Language.getString("SystInfo.16"); //$NON-NLS-1$
                msg = msg.replace("%1", name); //$NON-NLS-1$
                msg = msg.replace("%2", NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Integer.toString(POPULATION)); //$NON-NLS-1$
                response.add(msg);
            }
            if (POP2IND < 0) {
                msg = Language.getString("SystInfo.17"); //$NON-NLS-1$
                msg = msg.replace("%1", name); //$NON-NLS-1$
                msg = msg.replace("%2", NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Integer.toString(POP2IND)); //$NON-NLS-1$
                response.add(msg);
            }
            if (POP2ENE < 0) {
                msg = Language.getString("SystInfo.18"); //$NON-NLS-1$
                msg = msg.replace("%1", name); //$NON-NLS-1$
                msg = msg.replace("%2", NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Integer.toString(POP2ENE)); //$NON-NLS-1$
                response.add(msg);
            }
            if (POP2FOO < 0) {
                msg = Language.getString("SystInfo.19"); //$NON-NLS-1$
                msg = msg.replace("%1", name); //$NON-NLS-1$
                msg = msg.replace("%2", NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Integer.toString(POP2FOO)); //$NON-NLS-1$
                response.add(msg);
            }
            if (POP2RES < 0) {
                msg = Language.getString("SystInfo.20"); //$NON-NLS-1$
                msg = msg.replace("%1", name); //$NON-NLS-1$
                msg = msg.replace("%2", NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Integer.toString(POP2RES)); //$NON-NLS-1$
                response.add(msg);
            }
            if (POP2INT < 0) {
                msg = Language.getString("SystInfo.21"); //$NON-NLS-1$
                msg = msg.replace("%1", name); //$NON-NLS-1$
                msg = msg.replace("%2", NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Integer.toString(POP2INT)); //$NON-NLS-1$
                response.add(msg);
            }
                    if (MORALE < 0 || MORALE > 195) {
                msg = Language.getString("SystInfo.23"); //$NON-NLS-1$
                msg = msg.replace("%1", name); //$NON-NLS-1$
                msg = msg.replace("%2", NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Integer.toString(MORALE)); //$NON-NLS-1$
                response.add(msg);
            }
            if (MORALE_DESC < -4 || MORALE_DESC > 3) {
                msg = Language.getString("SystInfo.24"); //$NON-NLS-1$
                msg = msg.replace("%1", name); //$NON-NLS-1$
                msg = msg.replace("%2", NAME); //$NON-NLS-1$
                msg = msg.replace("%3", Integer.toString(MORALE_DESC)); //$NON-NLS-1$
                response.add(msg);
            }
            if (EXTRA != null) {
                if (EXTRA.length >= 808) {
                    msg = Language.getString("SystInfo.25"); //$NON-NLS-1$
                    msg = msg.replace("%1", name); //$NON-NLS-1$
                    response.add(msg);
                }
            }
            PlanetEntry pe;

            Hashtable<Short, String> idset = new Hashtable<Short, String>();
            for (int i = 0; i < PLANET.size(); i++) {
                pe = PLANET.get(i);
                //id
                if (idset.containsKey(pe.INDEX)) {
                    msg = Language.getString("SystInfo.26"); //$NON-NLS-1$
                    msg = msg.replace("%1", name); //$NON-NLS-1$
                    msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                    msg = msg.replace("%3", idset.get(pe.INDEX)); //$NON-NLS-1$
                    response.add(msg);
                } else {
                    idset.put(pe.INDEX, pe.NAME);
                }
                //name
                if (pe.NAME.length() > 39) {
                    msg = Language.getString("SystInfo.27"); //$NON-NLS-1$
                    msg = msg.replace("%1", name); //$NON-NLS-1$
                    msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                    response.add(msg);
                }
                //ani
                if (pe.PIC.length() > 11) {
                    msg = Language.getString("SystInfo.28"); //$NON-NLS-1$
                    msg = msg.replace("%1", name); //$NON-NLS-1$
                    msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                    msg = msg.replace("%3", pe.PIC); //$NON-NLS-1$
                    response.add(msg);
                }
                //size
                if (pe.SIZE < 0 || pe.SIZE > 3) {
                    msg = Language.getString("SystInfo.29"); //$NON-NLS-1$
                    msg = msg.replace("%1", name); //$NON-NLS-1$
                    msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                    msg = msg.replace("%3", Integer.toString(pe.SIZE)); //$NON-NLS-1$
                    response.add(msg);
                }
                //terra point
                if (pe.TERRA_POINTS < 0) {
                    msg = Language.getString("SystInfo.30"); //$NON-NLS-1$
                    msg = msg.replace("%1", name); //$NON-NLS-1$
                    msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                    msg = msg.replace("%3", Integer.toString(pe.TERRA_POINTS)); //$NON-NLS-1$
                    response.add(msg);
                }
                //terra point
                if (pe.TERRAFORMED < 0) {
                    msg = Language.getString("SystInfo.31"); //$NON-NLS-1$
                    msg = msg.replace("%1", name); //$NON-NLS-1$
                    msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                    msg = msg.replace("%3", Integer.toString(pe.TERRAFORMED)); //$NON-NLS-1$
                    response.add(msg);
                }
                //bonuses
                if (pe.PLT_BONI < 0 || pe.PLT_BONI > 3) {
                    msg = Language.getString("SystInfo.32"); //$NON-NLS-1$
                    msg = msg.replace("%1", name); //$NON-NLS-1$
                    msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                    msg = msg.replace("%3", Integer.toString(pe.PLT_BONI)); //$NON-NLS-1$
                    response.add(msg);
                }
                if (pe.UNI_BONI < 0 || pe.UNI_BONI > 3) {
                    msg = Language.getString("SystInfo.33"); //$NON-NLS-1$
                    msg = msg.replace("%1", name); //$NON-NLS-1$
                    msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                    msg = msg.replace("%3", Integer.toString(pe.UNI_BONI)); //$NON-NLS-1$
                    response.add(msg);
                }
                if (pe.POP_BONI < 0 || pe.POP_BONI > 3) {
                    msg = Language.getString("SystInfo.34"); //$NON-NLS-1$
                    msg = msg.replace("%1", name); //$NON-NLS-1$
                    msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                    msg = msg.replace("%3", Integer.toString(pe.POP_BONI)); //$NON-NLS-1$
                    response.add(msg);
                }
                //type
                if (pe.TYPE < 0 || pe.TYPE > 10) {
                    msg = Language.getString("SystInfo.35"); //$NON-NLS-1$
                    msg = msg.replace("%1", name); //$NON-NLS-1$
                    msg = msg.replace("%2", pe.NAME); //$NON-NLS-1$
                    msg = msg.replace("%3", Integer.toString(pe.TYPE)); //$NON-NLS-1$
                    response.add(msg);
                }
                else if ((SYS_REQ & (int)Math.pow(2, pe.TYPE)) == 0)
                {
                    msg = "Warning: in systInfo: System requirement flag in system " + NAME + " for planet type " + pe.TYPE + " not set. (now fixed)";
                    SYS_REQ = SYS_REQ | (int)Math.pow(2, pe.TYPE);
                    response.add(msg);
                    markChanged();
                }
            }

            for (int t = 0; t <= 10; t++)
            {
                if ((SYS_REQ & (int)Math.pow(2, t)) != 0)
                {
                    pe = null;

                    for (int i = 0; i < PLANET.size(); i++)
                    {
                        pe = PLANET.get(i);

                        if (pe.TYPE == t)
                        {
                            break;
                        }
                        else
                        {
                            pe = null;
                        }
                    }

                    if (pe == null)
                    {
                        msg = "Warning: in systInfo: System requirement flag in system " + NAME + " for planet type " + pe.TYPE + " is set despite no such planet in system. (now fixed)";
                        SYS_REQ = SYS_REQ & ~(int)Math.pow(2, t);
                        response.add(msg);
                        markChanged();
                    }
                }
            }

            return response;
        }

        void setExtra(byte[] ex) {
            EXTRA = ex;
        }
    }

    //reprisents each system entry
    class SystemEntryTest {
        //identified and editable

        public int INDEX = 0; //system index
        public int MANAGE = 0; //managed by (0-noone, 1-human, 2-AI)
        public String NAME = ""; //system name //$NON-NLS-1$
        public int PLANETS; //number of planets
        public String STARANI; //graphics files
        public int POPULATION; //system population
        byte[] fourB = new byte[4];
        byte[] str = new byte[40];

        //constructor
        public SystemEntryTest() {
        }

        public void test(InputStream in, int expected_index) throws Exception {
            in.read(fourB);
            INDEX = Tools.toInt(fourB, true);
            if (INDEX != expected_index) {
                throw new Exception("Wrong index: " + INDEX + "(Expected: " + expected_index + ")");
            }

            //man 4-4
            in.read(fourB);
            MANAGE = Tools.toInt(fourB, true);
            if (MANAGE < 0 && MANAGE >= 2) {
                throw new Exception("Wrong manager: " + MANAGE);
            }

            //name: 8-40 bytes
            in.read(str);
            NAME = Tools.toASCII(str, 0, 40);
            if (NAME.isEmpty() || !Tools.isSensible(NAME)) {
                throw new Exception("Invalid name: " + NAME);
            }

            in.skip(2);
            //starani 50-5
            in.read(str, 0, 5);
            STARANI = Tools.toASCII(str, 0, 5);
            if (STARANI.isEmpty() || !Tools.isSensible(STARANI)) {
                throw new Exception("Invalid star ani: " + STARANI);
            }

            in.skip(9);
            //population 64-4
            in.read(fourB);
            POPULATION = Tools.toInt(fourB, true);
            if (POPULATION < 0) {
                throw new Exception("Invalid population: " + POPULATION);
            }
            in.skip(38);
            //planets 106-4
            in.read(fourB);
            PLANETS = Tools.toInt(fourB, true);
            if (PLANETS < 0 || PLANETS > 10) {
                throw new Exception("Invalid planets: " + PLANETS);
            }
        }
    }
}
