/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: Sector.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.edit.sav.lst;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Vector;

import ue.Language;
import ue.edit.InternalFile;
import ue.edit.sav.SavGame;
import ue.edit.sav.StarBaseInfo;
import ue.edit.sav.StellInfo;
import ue.edit.sav.SystInfo;
import ue.exception.InvalidArgumentsException;
import ue.util.Tools;

/**This class is a representation of the sector.lst file. It holds a list of sectors in the game.*/
public class Sector extends InternalFile{
	private ArrayList<SectorEntry> ENTRY = new ArrayList<SectorEntry>();
	private int height;
	private int weight;
	private SavGame GAME;
        
    public int getNumberOfSectors()
    {
        return ENTRY.size();
    }
        
	//specified by SavEdit
	@Override
	public Vector<String> check(){
		Vector<String> response = new Vector<String>();
		String msg;
		
		if (ENTRY.size()==0){
			msg = Language.getString("Sector.0"); //$NON-NLS-1$
			msg = msg.replace("%1", getName()); //$NON-NLS-1$
			response.add(msg);
		}
		try{
			StarBaseInfo base = (StarBaseInfo)GAME.getInternalFile(SavGame.fileSTARBASEINFO, true);
			StellInfo stell = (StellInfo)GAME.getInternalFile(SavGame.fileSTELLINFO, true);
			SystInfo syst = (SystInfo)GAME.getInternalFile(SavGame.fileSYSTINFO, true);
			SectorEntry sec;
			for (int i=0; i<ENTRY.size(); i++){
				sec = ENTRY.get(i);
				if (sec.STARBASE!=-1)
                                {
				    if (base.getIndex(sec.STARBASE) < 0)
                                    {
				        msg = Language.getString("Sector.1"); //$NON-NLS-1$
				        msg = msg.replace("%1", getName()); //$NON-NLS-1$
				        msg = msg.replace("%2", getDescription(i, true)); //$NON-NLS-1$
				        msg = msg.replace("%3", Integer.toString(sec.STARBASE)); //$NON-NLS-1$
				        response.add(msg);
                                    }
				}
				try{
					if (sec.STELLAR!=-1 && sec.SYSTEM!=-1){
						msg = Language.getString("Sector.2"); //$NON-NLS-1$
						msg = msg.replace("%1", getName()); //$NON-NLS-1$
						msg = msg.replace("%2", getDescription(i, true)); //$NON-NLS-1$
						response.add(msg);
					}
				}catch (InvalidArgumentsException ke){
					ke.printStackTrace();
				}
				if (sec.STELLAR!=-1){
					try{
						stell.getType(sec.STELLAR);
					}catch (InvalidArgumentsException ze){
						try{
							msg = Language.getString("Sector.3"); //$NON-NLS-1$
							msg = msg.replace("%1", getName()); //$NON-NLS-1$
							msg = msg.replace("%2", getDescription(i, true)); //$NON-NLS-1$
							msg = msg.replace("%3", Integer.toString(sec.SYSTEM)); //$NON-NLS-1$
							response.add(msg);
						}catch (InvalidArgumentsException ke){
							ke.printStackTrace();
						}
					}
				}
				if (sec.SYSTEM!=-1){
					try{
						syst.getMan(sec.SYSTEM);
					}catch (InvalidArgumentsException ze){
						try{
							msg = Language.getString("Sector.4"); //$NON-NLS-1$
							msg = msg.replace("%1", getName()); //$NON-NLS-1$
							msg = msg.replace("%2", getDescription(i, true)); //$NON-NLS-1$
							msg = msg.replace("%3", Integer.toString(sec.SYSTEM)); //$NON-NLS-1$
							response.add(msg);
						}catch (InvalidArgumentsException ke){
							ke.printStackTrace();
						}
					}
				}
			}
		}catch (Exception e){
			response.add(e.getMessage());
		}
		return response;
	}
	@Override
	public long getSize(){
		long size = ENTRY.size()*104;
		return size;
	}
	@Override
	public void save(OutputStream out) throws IOException{
		int size = ENTRY.size();
		SectorEntry se;
		for (int i=0; i<size; i++){
			se = ENTRY.get(i);
			se.save(out);
		}
	}
	/**Constructor
	@param in the InputStream to read from*/
	public Sector(InputStream in, SavGame gam) throws IOException{
		GAME = gam;
		SectorEntry se;
		while (in.available() >0){
			se = new SectorEntry(in);
			ENTRY.add(se);
		}
		reset();
	}
	private void reset(){
		int size = ENTRY.size();
		SectorEntry se = ENTRY.get(size-1);
		weight = se.H_INDEX;
		height = se.V_INDEX;
		for (int i=0; i<size; i++){
			se = ENTRY.get(i);
			se.MAX_H = weight+1;
		}
		Collections.sort(ENTRY);
	}
	//gets
	/**Returns the number of sectors in a horizontal line*/
	public int getHSectors(){
		return (weight+1);
	}
	/**Returns the number of sectors in a vertical line*/
	public int getVSectors(){
		return (height+1);
	}
	/**Returns the descriptive name of the sector
	@param index the index of the sector*/
	public String getDescription(int index, boolean numeric) throws InvalidArgumentsException
    {
	    String ret = Language.getString("Sector.5"); //$NON-NLS-1$
	    
	    if (index < 0)
        {
            return ret.replace("%1", "?"); //$NON-NLS-1$
        }

        checkIndex(index);
        SectorEntry se = ENTRY.get(index);

        if (this.getHSectors() <= ('Z' - 65) && !numeric)
        {
            try
            {
                byte[] but = new byte[1];
                but[0] = (byte)(65 + se.H_INDEX);

                ret = ret.replace("%1", Tools.toASCII(but).toUpperCase() + (se.V_INDEX+1)); //$NON-NLS-1$
            }
            catch (Exception ex)
            {
                ret = ""; //$NON-NLS-1$
                ex.printStackTrace();
            }
        }
        else
        {
            ret = ret.replace("%1", Integer.toString(se.H_INDEX + 1) + "." + Integer.toString(se.V_INDEX + 1));
        }

        return ret;
	}
	/**Returns the vertical index of a sector.
	@param index the index of the sector*/
	public int getVertical(int index) throws InvalidArgumentsException{
		checkIndex(index);
		SectorEntry se = ENTRY.get(index);
		return se.V_INDEX;
	}
	/**Returns the horizontal index of a sector.
	@param index the index of the sector*/
	public int getHorizontal(int index) throws InvalidArgumentsException{
		checkIndex(index);
		SectorEntry se = ENTRY.get(index);
		return se.H_INDEX;
	}
	/**Returns the revealed mask of a sector.
	@param index the index of the sector*/
	public long getRevealed(int index) throws InvalidArgumentsException{
		checkIndex(index);
		SectorEntry se = ENTRY.get(index);
		return se.REVEALEDBY;
	}
	/**Returns the explorer mask of a sector.
	@param index the index of the sector*/
	public long getExplored(int index) throws InvalidArgumentsException{
		checkIndex(index);
		SectorEntry se = ENTRY.get(index);
		return se.EXPLOREDBY;
	}

	public boolean getExploredBy(int index, int race) throws InvalidArgumentsException
    {
		checkIndex(index);
		SectorEntry se = ENTRY.get(index);
        long mask = (long)Math.pow(2, race);
		return (se.EXPLOREDBY & mask) != 0;
	}
    public boolean getScannedBy(int index, int race) throws InvalidArgumentsException
    {
		checkIndex(index);
		SectorEntry se = ENTRY.get(index);
        long mask = (long)Math.pow(2, race);
		return (se.REVEALEDBY & mask) != 0;
    }


	/**Returns the system index of a sector.
	@param index the index of the sector
	@return the index of the system in this sector or -1 if none*/
	public short getSystem(int index) throws InvalidArgumentsException{
		checkIndex(index);
		SectorEntry se = ENTRY.get(index);
		return se.SYSTEM;
	}
	/**Returns the index of the stellar object in sector.
	@param index the index of the sector
	@return the index of the stellar object in this sector or -1 if none*/
	public short getStellarObject(int index) throws InvalidArgumentsException{
		checkIndex(index);
		SectorEntry se = ENTRY.get(index);
		return se.STELLAR;
	}
	/**Sets the index of the stellar object in sector.
	@param index the index of the sector
	@param obj the index of the stellar object in this sector or -1 if none*/
	public void setStellarObject(int index, short obj) throws InvalidArgumentsException{
		checkIndex(index);
		SectorEntry se = ENTRY.get(index);
		
        if (se.STELLAR != obj)
        {
            se.STELLAR = obj;
            this.markChanged();
        }
	}
	/**Returns the id of the outpost in sector.
	@param index the index of the sector
	@return the index of the outpost in this sector or -1 if none*/
	public short getOutpost(int index) throws InvalidArgumentsException{
		checkIndex(index);
		SectorEntry se = ENTRY.get(index);
		return se.STARBASE;
	}
	/**Returns the owner mask of the sector.
	@param index the index of the sector*/
	public short getOwnerMask(int index) throws InvalidArgumentsException{
		checkIndex(index);
		SectorEntry se = ENTRY.get(index);
		return se.OWNER_CLAIM;
	}
	/**Returns the race of the owner of the sector and -1 if the sector is in dispute or not owned by anyone.
	@param index the index of the sector*/
	public int getOwnerRace(int index) throws InvalidArgumentsException{
		checkIndex(index);
		int t = getOwnerMask(index);
		SectorEntry se = ENTRY.get(index);

		switch (t)
        {
			case 1:{
                t = 0;
                break;
			}
			case 2:{
                t = 1;
                break;
			}
			case 4:{
                t = 2;
                break;
			}
			case 8:{
                t = 3;
                break;
			}
			case 16:{
                t = 4;
                break;
			}
			case 0:{
                t = se.RACE[5];
                break;
			}
			default:{
				return -1;
			}
		}

        if (t <= 4)
        {   // clean
    		for (int i = 0; i < 6; i++)
            {
                if (i != t)
                {
                    se.RACE[i] = -1;
                }
                else
                {
                    se.RACE[i] = t;
                }
            }
        }

        return t;
	}
	/**Returns the scan strength of the sector.
	@param index the index of the sector
	@param view the race*/
	public int getScanStrength(int index, int view) throws InvalidArgumentsException{
		checkIndex(index);
		SectorEntry se = ENTRY.get(index);
		checkView(view);
		return se.SCAN_STRENGTH[view];
	}
	
	
	//sets
	/**Sets the scan strength of the sector.
	@param index the index of the sector
	@param view the race
	@param value the new scan strength*/
	public void setScanStrength(int index, int view, int value) throws InvalidArgumentsException{
		checkIndex(index);
		SectorEntry se = ENTRY.get(index);
		if (view<0 || view>5) {
			return;
		}
		if (se.SCAN_STRENGTH[view] == value)
        {
			return;
		}
		se.SCAN_STRENGTH[view]=value;
        this.markChanged();
	}
	/**Sets the revealed mask of a sector.
	@param index the index of the sector
	@param rev new reveal mask*/
	public void setRevealed(int index, long rev) throws InvalidArgumentsException{
		checkIndex(index);
		SectorEntry se = ENTRY.get(index);
		if (se.REVEALEDBY==rev) {
			return;
		}
		se.REVEALEDBY = rev;
                this.markChanged();
	}

	/**Sets the explored mask of a sector.
	@param index the index of the sector
	@param rev new explore mask*/
	public void setExplored(int index, long rev) throws InvalidArgumentsException{
		checkIndex(index);
		SectorEntry se = ENTRY.get(index);
		if (se.EXPLOREDBY==rev) {
			return;
		}
		se.EXPLOREDBY = rev;
                this.markChanged();
	}

	public void setExploredBy(int index, int race, boolean explored) throws InvalidArgumentsException
    {
		checkIndex(index);

		SectorEntry se = ENTRY.get(index);
        long mask = (long)Math.pow(2, race);

		if (((se.EXPLOREDBY & mask) != 0) != explored)
        {
            if (explored)
            {
                se.EXPLOREDBY = se.EXPLOREDBY | mask;
            }
            else
            {
                mask = ~mask;
                se.EXPLOREDBY = se.EXPLOREDBY & mask;
            }

            this.markChanged();
        }
	}

    public void setScannedBy(int index, int race, boolean revealed) throws InvalidArgumentsException
    {
		checkIndex(index);

		SectorEntry se = ENTRY.get(index);
        long mask = (long)Math.pow(2, race);

		if (((se.REVEALEDBY & mask) != 0) != revealed)
        {
            if (revealed)
            {
                se.REVEALEDBY = se.REVEALEDBY | mask;
            }
            else
            {
                mask = ~mask;
                se.REVEALEDBY = se.REVEALEDBY & mask;
            }

            this.markChanged();
        }
    }

	/**Sets the owner mask of a sector auto owner race.
	If mask is 0, the race parameter is used. Don't do this if there's a system in the sector!
	@param index the index of the sector
	@param rev new owner mask
	@param race new owner race*/
	public void setOwnerMask(int index, short rev, int race) throws InvalidArgumentsException
    {
		checkIndex(index);
		SectorEntry se = ENTRY.get(index);

		if (se.OWNER_CLAIM == rev && rev != 0)
        {
			return;
		}
        
		if ((rev & (short)1) == (short)1) {
			se.RACE[0] = race;
		} else {
			se.RACE[0] = -1;
		}
		if ((rev & (short)2) == (short)2) {
			se.RACE[1] = race;
		} else {
			se.RACE[1] = -1;
		}
		if ((rev & (short)4) == (short)4) {
			se.RACE[2] = race;
		} else {
			se.RACE[2] = -1;
		}
		if ((rev & (short)8) == (short)8) {
			se.RACE[3] = race;
		} else {
			se.RACE[3] = -1;
		}
		if ((rev & (short)16) == (short)16) {
			se.RACE[4] = race;
		} else {
			se.RACE[4] = -1;
		}
		if (rev == 0) {
			se.RACE[5] = race;
		} else {
			se.RACE[5] = -1;
		}
		se.OWNER_CLAIM = rev;
        this.markChanged();
	}
	/**Sets the starbase of a sector.
	@param index the index of the sector
	@param rev new starbase*/
	public void setStarbase(int index, short rev) throws InvalidArgumentsException{
		checkIndex(index);
		SectorEntry se = ENTRY.get(index);
		if (se.STARBASE==rev) {
			return;
		}
		se.STARBASE = rev;
                this.markChanged();
	}
	//other
	/**Switches two sectors.
	@param index the index of the first sector
	@param rev index of the second sector*/
	public void switchSectors(int index, int rev) throws InvalidArgumentsException{
		checkIndex(index);
		checkIndex(rev);
		SectorEntry sea = ENTRY.get(index);
		SectorEntry se2 = ENTRY.get(rev);
		int a = sea.V_INDEX;
		int b = sea.H_INDEX;
		sea.V_INDEX = se2.V_INDEX;
		sea.H_INDEX = se2.H_INDEX;
		se2.V_INDEX = a;
		se2.H_INDEX = b;
		Collections.sort(ENTRY);
        this.markChanged();
	}
	/**Adds a horizontal line.*/
	public void addHorizontalLine(){
		height = height+1;
		for (int i=0; i<=weight; i++){
			ENTRY.add(new SectorEntry(i, height));
		}
		for (int i=0; i<ENTRY.size(); i++){
			SectorEntry se = ENTRY.get(i);
			se.MAX_H = weight+1;
		}
		Collections.sort(ENTRY);
                this.markChanged();
	}
	/**Adds a vertical line.*/
	public void addVerticalLine(){
		for (int i=0; i<=height; i++){
			ENTRY.add(new SectorEntry(weight+1, i));
		}
		weight = weight+1;
		for (int i=0; i<ENTRY.size(); i++){
			SectorEntry se = ENTRY.get(i);
			se.MAX_H = weight+1;
		}
		Collections.sort(ENTRY);
                this.markChanged();
	}

    public void setSystem(int index, short system) throws InvalidArgumentsException
    {
        checkIndex(index);
        SectorEntry se = ENTRY.get(index);
        if (se.SYSTEM != system)
        {
            se.SYSTEM = system;
            this.markChanged();
        }
    }


	private void checkIndex(int index) throws InvalidArgumentsException{
		if (index<0 || index>=ENTRY.size()){
			String msg = Language.getString("Sector.6"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
	}
	
	private void checkView(int view) throws InvalidArgumentsException{
		if (view<0 || view > 5){
			String msg = Language.getString("Sector.7"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(view)); //$NON-NLS-1$
			throw new InvalidArgumentsException(msg);
		}
	}

	//class #1
	private class SectorEntry implements Comparable<SectorEntry>
        {           
	    public int MAX_H = 0;
	    //data
	    public int V_INDEX;
	    public int H_INDEX;
	    public short STARBASE;
	    public short STELLAR;
	    public short SYSTEM;
	    public short OWNER_CLAIM; /*just the mask*/
	    //cards, feds, fergs, klings, roms, other
		private int[] RACE = new int[6];
		public long EXPLOREDBY;
		public long REVEALEDBY;
		private byte[] UNKNOWN_1 = new byte[8];
		public int[] SCAN_STRENGTH = new int[5];
		private byte[] UNKNOWN_2 = new byte[20];
		//constructors
		public SectorEntry(InputStream in) throws IOException{
			byte[] b = new byte[4];
			//vertical index 0-4
			in.read(b);
			V_INDEX = Tools.toInt(b, true);
			//horizontal index 4-4
			in.read(b);
			H_INDEX = Tools.toInt(b, true);
			//starbase 8-2
			b = new byte[2];
			in.read(b);
			STARBASE = Tools.toShort(b, true);
			//stellar object 10-2
			in.read(b);
			STELLAR = Tools.toShort(b, true);
			//system 12-2
			in.read(b);
			SYSTEM = Tools.toShort(b, true);
			//owner? 14-2
			in.read(b);
			OWNER_CLAIM = Tools.toShort(b, true);
			//race owners 16-24
			b = new byte[4];
			for (int i = 0; i < 6; i++)
            {
				in.read(b);
				RACE[i] = Tools.toInt(b, true);
			}
			//explored by 40-8
			b = new byte[8];
			in.read(b);
			EXPLOREDBY = Tools.toLong(b, true);
			//revealed by 48-8
			in.read(b);
			REVEALEDBY = Tools.toLong(b, true);
			//unknown 56-8
			in.read(UNKNOWN_1);
			//scan strength 64-20
			b = new byte[4];
			for (int i=0; i<5; i++){
				in.read(b);
				SCAN_STRENGTH[i] = Tools.toInt(b, true);
			}
			//unknown 84-20
			in.read(UNKNOWN_2);
		}
		public SectorEntry(SectorEntry me){
			V_INDEX = me.V_INDEX;
			H_INDEX = me.H_INDEX;
			STARBASE = me.STARBASE;
			STELLAR = me.STELLAR;
			SYSTEM = me.SYSTEM;
			OWNER_CLAIM = me.OWNER_CLAIM;
			EXPLOREDBY = me.EXPLOREDBY;
			REVEALEDBY = me.REVEALEDBY;
			for (int i=0; i<6; i++){
				if (i<5) {
					SCAN_STRENGTH[i] = me.SCAN_STRENGTH[i];
				}
				RACE[i] = me.RACE[i];
			}
			UNKNOWN_1 = me.UNKNOWN_1;
			UNKNOWN_2 = me.UNKNOWN_2;
		}
		public SectorEntry(int h_index, int v_index){
			V_INDEX = v_index;
			H_INDEX = h_index;
			STARBASE = -1;
			STELLAR = -1;
			SYSTEM = -1;
			OWNER_CLAIM = 0;
			EXPLOREDBY = 0;
			REVEALEDBY = 0;
			for (int i=0; i<6; i++){
				if (i<5) {
					SCAN_STRENGTH[i] = 0;
				}
				RACE[i] = -1;
			}
			UNKNOWN_1 = new byte[8];
			UNKNOWN_2 = new byte[20];
		}
		//save
		public void save(OutputStream out) throws IOException{
			//vertical index
			byte[] b = Tools.toByte(V_INDEX, true);
			out.write(b);
			//horizontal index
			b = Tools.toByte(H_INDEX, true);
			out.write(b);
			//starbase
			b = Tools.toByte(STARBASE, true);
			out.write(b);
			//stellar
			b = Tools.toByte(STELLAR, true);
			out.write(b);
			//system
			b = Tools.toByte(SYSTEM, true);
			out.write(b);
			//owner
			b = Tools.toByte(OWNER_CLAIM, true);
			out.write(b);
			//unknown
			for (int i = 0; i < 6; i++)
            {
				b = Tools.toByte(RACE[i], true);
				out.write(b);
			}
			//explered
			b = Tools.toByte(EXPLOREDBY, true);
			out.write(b);
			//revealed
			b = Tools.toByte(REVEALEDBY, true);
			out.write(b);
			//unknown
			out.write(UNKNOWN_1);
			//scan strength
			for (int i=0; i<5; i++){
				b = Tools.toByte(SCAN_STRENGTH[i], true);
				out.write(b);
			}
			//unknown
			out.write(UNKNOWN_2);
		}
		//for comparing
		@Override
		public boolean equals(Object o){
			if (!(o instanceof SectorEntry))
            {
                return false;
            }

			SectorEntry se = (SectorEntry)o;

			return (se.V_INDEX==V_INDEX && se.H_INDEX==H_INDEX);
		}
		@Override
		public int hashCode(){
			return (V_INDEX*MAX_H + H_INDEX);
		}
        @Override
		public int compareTo(SectorEntry se) throws ClassCastException{
			return (hashCode() - se.hashCode());
		}
	}
}
