/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: SavEdit.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.edit.snd;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import ue.Language;
import ue.UE;
import ue.edit.FileManager;
import ue.edit.mod.ModList;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.bin.Converse;
import ue.exception.InvalidArgumentsException;
import ue.exception.NotAPanel;
import ue.gui.CheckGUI;
import ue.gui.MainPanel;
import ue.gui.ModifiedFilesGUI;
import ue.gui.snd.SoundGUI;
import ue.gui.snd.SwitchRaceGUI;
import ue.util.RandomReader;
import ue.util.SettingsManager;
import ue.util.StringGroup;
import ue.util.Tools;
import ue.util.UEWorker;
import ue.util.Wave;

/**
 * English.snd or german.snd file.
 * @author Alan Podlesek
 */
public class English extends SndFile
{
    private File ENGLISH;
    private boolean isEnglish;
    private Stbof STBOF;
    private Converse CONVERSE;
    private boolean CHANGED = false;
    private boolean CANCELLED_SAVE = false;
    private ArrayList<SndEntry> MAP = new ArrayList<SndEntry>();
    private Hashtable<Integer, Wave> WAVE = new Hashtable<Integer, Wave>();
    public boolean ADD_CHANGES_2_MODLIST=true;
    
    // commands
    private static String cmdCHECK = Language.getString("English.0"); //$NON-NLS-1$
    private static String cmdMODLIST = Language.getString("English.1"); //$NON-NLS-1$
    private static String cmdVOICES = Language.getString("English.2"); //$NON-NLS-1$
    private static String cmdSWITCH_RACES = Language.getString("English.10"); //$NON-NLS-1$
    
    
    public English(boolean english)
    {
        isEnglish=english;
    }

    //specified by SndFile
    @Override
    public void markChanged()
    {
        CHANGED = true;
    }
    
    /**
     * Returns a MainPanel to edit the file.
     * @param command must be one of the commands specified by the open function.
     * @return a MainPanel object according to the command, null if command is invalid.
     */
    @Override
    public MainPanel edit(String command) throws Exception
    {
        if (command.equals(cmdVOICES))
        {
            return new SoundGUI(this, STBOF);
        }
        if (command.equals(cmdCHECK))
        {
            return new CheckGUI(check());
        }
        if (command.equals(cmdMODLIST))
        {
            // update modified files list first
            Enumeration<Integer> keys = WAVE.keys();
            
            while (keys.hasMoreElements())
            {
                MODS.add(keys.nextElement().toString());
            }
            // disable main window
            UE.WINDOW.setEnabled(false);
            new ModifiedFilesGUI(this);
            throw new NotAPanel();
        }
        if (command.equals(cmdSWITCH_RACES))
        {
            return new SwitchRaceGUI(this);
        }
        
        return null;
    }

    @Override
    public int getNumberOfSounds()
    {
        return MAP.size();
    }

    /**
     * Saves changes.
     * @param whereTo File where everything should be saved to. If null save to the same file.
     * @return true if save has been successful.
     * @throws Exception the function may throw an exception if an error occures.
     */
    @Override
    public boolean save(File whereTo, boolean bkp, String ext) throws Exception
    {
        if (whereTo == null)
        {
            whereTo = new File(ENGLISH.getAbsolutePath());
        }
        
        SaveTask TSK = new SaveTask(whereTo, bkp, ext);
        
        this.CANCELLED_SAVE = !(Boolean)Tools.runUEWorker(TSK);

        if (!CANCELLED_SAVE)
        {
            this.ENGLISH = whereTo;
        }

        CHANGED = false;
        return !this.CANCELLED_SAVE;
    }
    
    /**
     * There you go, returns Wave.
     */
    @Override
    public Wave getWave(int i) throws InvalidArgumentsException
    {
        SndEntry snd = MAP.get(i);
        
        if (snd == null)
        {
            throw new InvalidArgumentsException(getIndexNotFoundString(i));
        }
	
        Wave t;
        
        if (WAVE.containsKey(i))
        {
            t = WAVE.get(i);
            
            if (t != null)
            {
                return t;
            }
        }
	
        try
        {
            FileInputStream reader = new FileInputStream(ENGLISH);
            reader.skip(snd.ADDRESS);
            byte[] b = new byte[snd.SIZE];
            reader.read(b);
            t = new Wave(1, b, snd.FRAMES);
            reader.close();
            return t;
        }
        catch (Exception ew)
        {
            UE.displayError(ew);
        }
        
        throw new InvalidArgumentsException(getIndexNotFoundString(i));
    }
    
    /**
     * Opens the provided file.
     * @param whereFrom the File to be opened.
     * @return a String array of edit options specific for the opened file.
     */
    @Override
    public StringGroup[] open(File whereFrom)
    {
        if (UE.FILES.getFile(FileManager.P_STBOF) != null)
        {
            STBOF = (Stbof)UE.FILES.getFile(FileManager.P_STBOF);
        }
        
        try
        {
            CONVERSE = (Converse)STBOF.getInternalFile(Stbof.fileCONVERSE_BIN, true);
            
            ENGLISH = whereFrom;
            FileInputStream reader = new FileInputStream(ENGLISH);
            byte[] b = new byte[4];
            reader.read(b);
            int mapaddy = Tools.toInt(b, true);
            reader.skip(mapaddy - 4);
            
            int count = 0;
            while (reader.available() > 0 && count < 1364)
            {
                MAP.add(new SndEntry(reader));
                count++;
            }
            
            try
            {
                if (reader.available() > 0)
                {
                    SndEntry snd = new SndEntry(reader);
                    reader.close();
                    reader = new FileInputStream(ENGLISH);
                    reader.skip(snd.ADDRESS);
                    b = new byte[snd.SIZE];
                    reader.read(b);
                    ByteArrayInputStream bais = new ByteArrayInputStream(b);
                    MODS = new ModList(bais);
                }
                else
                {
                    MODS = new ModList();
                }
            }
            catch (Exception g)
            {
                MODS = new ModList();
                g.printStackTrace();
            }
            
            try
            {
                reader.close();
                reader = new FileInputStream(ENGLISH);
                reader.skip(mapaddy + getNumberOfSounds()*16);
                if (reader.available() > 0)
                {
                    reader.skip(16);
                }
                if (reader.available() > 0)
                {
                    SndEntry snd = new SndEntry(reader);
                    reader.close();
                    reader = new FileInputStream(ENGLISH);
                    reader.skip(snd.ADDRESS);
                    b = new byte[snd.SIZE];
                    reader.read(b);
                    ByteArrayInputStream bais = new ByteArrayInputStream(b);
                    DESCRIPTION = new SndDesc(bais);
                }
                else
                {
                    String fn;
                    if (isEnglish)
                    {
                        fn = "english.dsc"; //$NON-NLS-1$
                    }
                    else
                    {
                        fn = "german.dsc"; //$NON-NLS-1$ //$NON-NLS-2$
                    }
                    
                    File fdesc = new File(fn);
                    if (!fdesc.exists())
                    {
                        fdesc.createNewFile();
                    }
                    
                    FileInputStream filod = new FileInputStream(fdesc);
                    DESCRIPTION = new SndDesc(filod);
                }
            }
            catch (Exception g)
            {
                DESCRIPTION = new SndDesc();
            }
            reader.close();
            
        }
        catch (Exception ef)
        {
            UE.displayError(ef);
            ENGLISH = null;
            return null;
        }
        return getCommands();
    }
    
    /**
     * @return a String array of available commands that can be used with the edit function.
     */
    @Override
    public StringGroup[] getCommands()
    {
        if (ENGLISH == null || STBOF == null)
        {
            return null;
        }
        cmdCHECK = Language.getString("English.0"); //$NON-NLS-1$
        cmdMODLIST = Language.getString("English.1"); //$NON-NLS-1$
        cmdVOICES = Language.getString("English.2"); //$NON-NLS-1$
        cmdSWITCH_RACES = Language.getString("English.10"); //$NON-NLS-1$

        StringGroup[] ret = new StringGroup[2];
        try
        {
            ret[0] = new StringGroup(Language.getString("English.4"), //$NON-NLS-1$
                    new String[]{cmdCHECK, cmdMODLIST});
            ret[1] = new StringGroup(Language.getString("English.5"), new String[]{cmdVOICES, cmdSWITCH_RACES}); //$NON-NLS-1$
        }
        catch (Exception tz)
        {
            UE.displayError(tz);
            return null;
        }
        return ret;
    }
    
    @Override
    public short getRaceDescriptor(int index)
    {
        short race = 0;
        
        race = (short) CONVERSE.getVoiceOwner((short)index);
        
        if (race >= 0 && race < 5)
        {
            race += 2;
        }
        else if (race == 36 || race == 37)
        {
            race = 1;
        }
        else if (race < 0)
        {
            race = 7;
        }
        else
        {
            race = 0;
        }
        
        return race;
    }
    
    /**
     * @return true if changes were made, else false.
     */
    @Override
    public boolean madeChanges()
    { 
        return (CHANGED || DESCRIPTION.madeChanges() || CANCELLED_SAVE);
    }
    
    /**
     * This function checks the file for known problems/errors that may occur while editing the file.
     * @return a Vector object containing results of the check as Strings.
     */
    @Override
    public Vector<String> check()
    {
        Vector<String> response = new Vector<String>();
        String msg;
        
        if (MAP.size() != 1364)
        {
            msg = Language.getString("English.6"); //$NON-NLS-1$
            msg = msg.replace("%1", ENGLISH.getName()); //$NON-NLS-1$
            msg = msg.replace("%2", Integer.toString(1364)); //$NON-NLS-1$
            response.add(msg);
        }
        SndEntry snd = MAP.get(0);
        
        for (int i = 1; i < MAP.size(); i++)
        {
            SndEntry snd2 = MAP.get(i);
            if (snd.ADDRESS+snd.SIZE!=snd2.ADDRESS)
            {
                msg = Language.getString("English.7"); //$NON-NLS-1$
                msg = msg.replace("%1", ENGLISH.getName()); //$NON-NLS-1$
                msg = msg.replace("%2", Integer.toString(snd.ADDRESS)); //$NON-NLS-1$
                response.add(msg);
            }
            snd = snd2;
        }
        
        response.add("Info:\n\nSound groups:\n\n" + CONVERSE.toString());
        
        if (response.isEmpty())
        {
            response.add(Language.getString("English.8")); //$NON-NLS-1$
        }
    
        return response;
    }
    
    /**
     * Extracts a playable wav file to the specified destination.
     * @param index the index of the sound file
     * @param dest the destination file
     */
    @Override
    public void extract(int index, File dest) throws IOException, InvalidArgumentsException
    {
        Wave tmp = getWave(index);
        FileOutputStream nu = new FileOutputStream(dest);
        tmp.save(nu);
        nu.close();
    }
    
    /**
     * replaces a sound in snd with the specified one.
     * @param index the index of the sound file
     * @param source the source file
     */
    @Override
    public void replace(int index, File source) throws IOException, InvalidArgumentsException, Exception
    {
        if (index < 0 || index > getNumberOfSounds())
        {
            throw new InvalidArgumentsException(getIndexNotFoundString(index));
        }
        
        FileInputStream fis = new FileInputStream(source);
        int read = 0;
        byte[] data = new byte[fis.available()];
        
        while(fis.available() > 0)
        {
            read += fis.read(data, read, fis.available());
        }
        
        Wave m = new Wave(data);
        
        if (m.getSndType() != 1)
        {
            throw new Exception(Language.getString("English.9")); //$NON-NLS-1$
        }
        WAVE.put(index, m);
        
        CHANGED = true;
    }
    
    /**
     * replaces a sound in snd with the specified one.
     * @param index the index of the sound file
     * @param source the source file
     */
    public void replace(int index, Wave source) throws InvalidArgumentsException
    {
        if (index < 0 || index > getNumberOfSounds())
        {
            throw new InvalidArgumentsException(getIndexNotFoundString(index));
        }
        
        if (source.getSndType() != 1)
        {
            throw new InvalidArgumentsException(Language.getString("English.9")); //$NON-NLS-1$
        }
        
        WAVE.put(index, source);
        CHANGED = true;
    }

    /**
     * Creates one big playable file.
     * @param whereTo the destination file
     */
    @Override
    public void thinkBig(File whereTo) throws Exception
    {
        ThinkBigTask TSK = new ThinkBigTask(whereTo);
        
        Tools.runUEWorker(TSK);
    }
    
    /**
     * Plays the specified sound.
     * @param index the index of the sound
     */
    @Override
    public void play(int index) throws Exception
    {
        if (index < 0 || index > getNumberOfSounds())
        {
            throw new InvalidArgumentsException(getIndexNotFoundString(index));
        }

        throw new Exception(Language.getString("English.11")); //$NON-NLS-1$
    }
    
    /**
     * Returns true if the play function is supported by the class.
     */
    @Override
    public boolean isPlayable()
    {
        return false;
    }
    
    private String getIndexNotFoundString(int index)
    {
        String msg = Language.getString("English.13"); //$NON-NLS-1$
        msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
        return msg;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#getKnownChanges()
     */
    @Override
    public ModList getKnownChanges()
    {
        return MODS;
    }
    
    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#hasBeenModified()
     */
    @Override
    public boolean hasBeenModified()
    {
        return !MODS.isEmpty() || madeChanges();
    }
    
    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#toFile()
     */
    @Override
    public File toFile()
    {
        return ENGLISH;
    }

    /**
     * Read Converse switchRaces method documentation.
     */
    public void switchRaces(short race1, short race2, boolean both) throws InvalidArgumentsException
    {
        CONVERSE.switchRaces(race1, race2, both);        
    }
    
    private class ThinkBigTask extends UEWorker<Void>
    {
        File path;
        
        public ThinkBigTask(File dest)
        {
            super();
            path = dest;
        }
        
        @Override
        public Void doInBackground()
        {
            this.updateStatus(UEWorker.STATUS_WORKING);
            
            String msg;
            
            try
            {
                int strRIFF = 1179011410;
                int strWAVE = 1163280727;
                int strFMT = 544501094;
                int WAVE_CHUNK_SIZE = 20;
                short FORMAT = 17;
                short CHANNELS = 1;
                int FREQUENCY = 22050;
                int BYTES_PER_SECOND = 11100;
                short BLOCK_ALIGMENT = 512;
                short BITS_PER_SAMPLE = 4;
                short EXTRA_FMT = 2;
                short SAMPLES_PER_BLOCK = 1017;
                int strFACT = 1952670054;
                int intFACT = 4;
                int strDATA = 1635017060;
                
                int j = MAP.size();
                int frames = 0;
                int size = 0;
                
                // calculate
                for (int i = 0; i < j && i < getNumberOfSounds(); i++)
                {
                    if (WAVE.containsKey(i))
                    {
                        Wave w = getWave(i);
                        size += w.getDataSize();
                        frames += w.getFrames();
                    }
                    else
                    {
                        SndEntry se = MAP.get(i);
                        frames += se.FRAMES;
                        size += se.SIZE;
                    }
                }
                
                // write
                FileOutputStream out = new FileOutputStream(path);
                RandomReader reader = new RandomReader(new FileInputStream(ENGLISH));
                //RIFF
                out.write(Tools.toByte(strRIFF, true));
                //size-8
                out.write(Tools.toByte((size+52), true));
                //WAVE
                out.write(Tools.toByte(strWAVE, true));
                //FMT
                out.write(Tools.toByte(strFMT, true));
                //wave chunk size
                out.write(Tools.toByte(WAVE_CHUNK_SIZE, true));
                //format
                out.write(Tools.toByte(FORMAT, true));
                //channels
                out.write(Tools.toByte(CHANNELS, true));
                //frequency
                out.write(Tools.toByte(FREQUENCY, true));
                //bytes per sec
                out.write(Tools.toByte(BYTES_PER_SECOND, true));
                //block aligment
                out.write(Tools.toByte(BLOCK_ALIGMENT, true));
                //bits per sample
                out.write(Tools.toByte(BITS_PER_SAMPLE, true));
                //extra chunk size
                out.write(Tools.toByte(EXTRA_FMT, true));
                //samples per block
                out.write(Tools.toByte(SAMPLES_PER_BLOCK, true));
                //FACT
                out.write(Tools.toByte(strFACT, true));
                //intFACT
                out.write(Tools.toByte(intFACT, true));
                //frames/4
                out.write(Tools.toByte((frames), true));
                //DATA
                out.write(Tools.toByte(strDATA, true));
                //data size
                out.write(Tools.toByte(size, true));
                //write data
                Wave w;
                
                for (int i = 0; i < j && i < getNumberOfSounds(); i++)
                {
                    if (isCancelled())
                    {
                        out.close();
                        path.delete();
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        return null;
                    }
                    
                    msg = Language.getString("English.14"); //$NON-NLS-1$
                    msg = msg.replace("%1", (int)(((double)(i*100))/((double)getNumberOfSounds())) + "%"); //$NON-NLS-1$ //$NON-NLS-2$
                    this.sendMessage(msg);
                    
                    if (WAVE.containsKey(i))
                    {
                        w = getWave(i);
                        byte[] b = w.getRawData();
                        out.write(b);
                    }
                    else
                    {
                        SndEntry snd = MAP.get(i);
                        reader.seek(snd.ADDRESS);
                        byte[] b = new byte[snd.SIZE];
                        reader.read(b);
                        out.write(b);
                    }
                }
                
                reader.close();
                out.close();
            }
            catch (Exception e)
            {
                this.reportError(e);
            }
            
            this.updateStatus(UEWorker.STATUS_DONE);
            return null;
        }
    }
    
    private class SaveTask extends UEWorker<Boolean>
    {
        File whereTo;
        boolean bkp;
        String ext;
        
        /**
         * @param dest destination file
         * @param mbkp make backup?
         * @param bext backup extension
         */
        public SaveTask(File dest, boolean mbkp, String bext)
        {
            super();
            whereTo = dest;
            bkp = mbkp;
            ext = bext;
        }
        
        @Override
        public Boolean doInBackground()
        {
            this.RESULT = false;
            String msg;
            
            this.updateStatus(UEWorker.STATUS_WORKING);
            
            try
            {
                //open
                if (whereTo.equals(ENGLISH))
                {
                    if (bkp)
                    {
                        this.sendMessage(Language.getString("English.22")); //$NON-NLS-1$
                        
                        File file = new File(ENGLISH.getPath() + ext);
                        
                        int i = 1;
                        
                        while (file.exists())
                        {
                            file = new File(ENGLISH.getPath() + i + ext);
                            i++;
                        }
                        
                        Tools.copyFile(ENGLISH, file);
                    }
                
                    whereTo = FileManager.getWritableLocalFile("english.tmp"); //$NON-NLS-1$
                }
                
                msg = Language.getString("English.14"); //$NON-NLS-1$
                msg = msg.replace("%1", whereTo.getName()); //$NON-NLS-1$
                this.sendMessage(msg);
                
                whereTo.delete();
                whereTo.createNewFile();
                
                // get file size
                int size = 4; // map address
                
                Wave w;
                
                // calculate data size
                for (int i = 0; i < getNumberOfSounds(); i++)
                {
                    msg = Language.getString("English.15"); //$NON-NLS-1$
                    msg = msg.replace("%1", (int)((double)(i*100)/getNumberOfSounds()) + "%"); //$NON-NLS-1$ //$NON-NLS-2$
                    this.sendMessage(msg);
                    
                    if (WAVE.containsKey(i))
                    {
                        Wave wu = WAVE.get(i);
                        size += wu.getDataSize();
                        if (ADD_CHANGES_2_MODLIST)
                        {
                            MODS.add(Integer.toString(i));
                        }
                    }
                    else
                    {
                        SndEntry sus = MAP.get(i);
                        size += sus.SIZE;
                    }
                }
                
                // mod list size
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                MODS.save(baos);
                byte[] mldata = baos.toByteArray();
                size += mldata.length;
                
                //descriptions size
                ByteArrayOutputStream baos2 = new ByteArrayOutputStream();
                DESCRIPTION.save(baos2);
                byte[] ddata = baos2.toByteArray();
                size += ddata.length;
                
                if (UE.SETTINGS.getProperty(SettingsManager.STORE_SND_DESC).equals(SettingsManager.YES))
                {
                    String fn;
                    if (isEnglish)
                    {
                        fn= "english.dsc"; //$NON-NLS-1$
                    }
                    else
                    {
                        fn= "german.dsc"; //$NON-NLS-1$ //$NON-NLS-2$
                    }
                    FileOutputStream gre = new FileOutputStream(FileManager.getWritableLocalFile(fn));
                    gre.write(ddata);
                    gre.close();
                }
                
                // write
                FileOutputStream writer = new FileOutputStream(whereTo);
                writer.write(Tools.toByte(size, true));
                
                SndEntry snd;
                int index = 4;
                
                for (int i = 0; i < getNumberOfSounds(); i++)
                {
                    msg = Language.getString("English.16"); //$NON-NLS-1$
                    msg = msg.replace("%1", (int)((double)(i*100)/getNumberOfSounds()) + "%"); //$NON-NLS-1$ //$NON-NLS-2$
                    this.sendMessage(msg);
                    
                    if (isCancelled())
                    {
                        writer.close();
                        whereTo.delete();
                        
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        return this.RESULT;
                    }
                    
                    w = getWave(i);
                    snd = MAP.get(i);
                    snd.ADDRESS = index;
                    snd.SIZE = w.getDataSize();
                    snd.FRAMES = w.getFrames();
                    byte[] b = w.getRawData();
                    writer.write(b);
                    index += snd.SIZE;
                }
                
                //mod list
                SndEntry m = new SndEntry();
                m.ADDRESS = index;
                m.SIZE = mldata.length;
                index += m.SIZE;
                writer.write(mldata);
                
                //descriptions
                SndEntry m2 = new SndEntry();
                m2.ADDRESS = index;
                m2.SIZE = ddata.length;
                index += m2.SIZE;
                writer.write(ddata);
                
                // write map
                for (int i = 0; i < getNumberOfSounds(); i++)
                {
                    msg = Language.getString("English.17"); //$NON-NLS-1$
                    msg = msg.replace("%1", (int)((double)(i*100)/getNumberOfSounds()) + "%"); //$NON-NLS-1$ //$NON-NLS-2$
                    this.sendMessage(msg);
                    
                    snd = MAP.get(i);
                    snd.save(writer);
                }
                
                m.save(writer);
                m2.save(writer);

                this.sendMessage(Language.getString("English.18")); //$NON-NLS-1$
                writer.close();
                
                //replace?
                if (whereTo.getName().equals("english.tmp")) //$NON-NLS-1$
                {
                    msg = Language.getString("English.20"); //$NON-NLS-1$
                    msg = msg.replace("%1", whereTo.getName()); //$NON-NLS-1$
                    this.sendMessage(msg);
                    
                    try
                    {
                        Tools.copyFile(whereTo, ENGLISH);
                    }
                    catch (IOException er)
                    {
                        this.reportError(er);
                        return RESULT;
                    }
                    
                    msg = Language.getString("English.19"); //$NON-NLS-1$
                    msg = msg.replace("%1", whereTo.getName()); //$NON-NLS-1$
                    this.sendMessage(msg);        
                    
                    boolean del = whereTo.delete();
                    
                    if (!del)
                    {
                        msg = Language.getString(Language.getString("English.21")); //$NON-NLS-1$
                        msg = msg.replace("%1", whereTo.getName()); //$NON-NLS-1$
                        this.reportError(new IOException(msg));
                        return RESULT;
                    }
                }

                WAVE.clear();
            }
            catch (Exception er)
            {
                this.reportError(er);
                return RESULT;
            }
            
            RESULT = true;
            this.updateStatus(UEWorker.STATUS_DONE);
            return RESULT;
        }
    }
}
