/*
Ultimate Editor for Birth Of The Federation 1.02
Copyright (C) 2005-2007  Alan Podlesek

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


File: Music.java

Written by: Alan Podlesek <seqdcer@yahoo.com>

Last modified: 6.9.2007
 */
package ue.edit.snd;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.SwingWorker;

import ue.Language;
import ue.UE;
import ue.edit.FileManager;
import ue.edit.mod.ModList;
import ue.edit.res.Stbof;
import ue.exception.InvalidArgumentsException;
import ue.exception.NotAPanel;
import ue.exception.UnsupportedWave;
import ue.gui.CheckGUI;
import ue.gui.MainPanel;
import ue.gui.ModifiedFilesGUI;
import ue.gui.snd.SoundGUI;
import ue.util.SettingsManager;
import ue.util.Tools;
import ue.util.Pair;
import ue.util.StringGroup;
import ue.util.Wave;

// TODO: clean up
@SuppressWarnings("unchecked")
public class Music extends SndFile {

    private File MUSIC;
    private Stbof STBOF;
    private boolean CHANGED = false;
    private boolean CANCELLED_SAVE = false;
    private ArrayList<SndEntry> MAP = new ArrayList<SndEntry>();
    private Pair<Integer, Integer>[] MINI_MAP = (Pair<Integer, Integer>[]) new Pair[16];
    private Hashtable<Integer, Wave> WAVE = new Hashtable<Integer, Wave>();
    public boolean ADD_CHANGES_2_MODLIST = true;
    // commands
    private static String cmdCHECK = Language.getString("Music.0"); //$NON-NLS-1$
    private static String cmdMODLIST = Language.getString("Music.1"); //$NON-NLS-1$
    private static String cmdMUSIC = Language.getString("Music.2"); //$NON-NLS-1$

    public Music() {
    }
    //specified by SndFile

    @Override
    public void markChanged() {
        CHANGED = true;
    }

    /**Returns a MainPanel to edit the file.
    @param command must be one of the commands specified by the open function.
    @return a MainPanel object according to the command, null if command is invalid.**/
    @Override
    public MainPanel edit(String command) throws Exception {
        if (command.equals(cmdMUSIC)) {
            return new SoundGUI(this, STBOF);
        }
        if (command.equals(cmdCHECK)) {
            return new CheckGUI(check());
        }
        if (command.equals(cmdMODLIST)) {
            // update modified files list first
            Enumeration<Integer> keys = WAVE.keys();
            while (keys.hasMoreElements()) {
                MODS.add(keys.nextElement().toString());
            }
            // disable main window
            UE.WINDOW.setEnabled(false);
            new ModifiedFilesGUI(this);
            throw new NotAPanel();
        }
        return null;
    }

    @Override
    public int getNumberOfSounds() {
        return MAP.size();
    }

    /**Saves changes.
    @param whereTo File where everything should be saved to. If null save to the same file.
    @return true if save has been successful.
    @throws Exception the function may throw an exception if an error occures.**/
    @Override
    public boolean save(File whereTo, boolean bkp, String ext) throws Exception {
        if (whereTo == null)
        {
            whereTo = new File(MUSIC.getAbsolutePath());
        }

        // FIX ME: use UEWorker
        final SaveTask TSK = new SaveTask(whereTo, bkp, ext);

        final JOptionPane opti = new JOptionPane(Language.getString("Music.3"), //$NON-NLS-1$
                JOptionPane.PLAIN_MESSAGE);
        opti.setOptions(new String[]{Language.getString("Music.4")}); //$NON-NLS-1$
        final JDialog dia = opti.createDialog(UE.WINDOW, UE.APP_NAME);

        TSK.addPropertyChangeListener(new PropertyChangeListener() {

            public void propertyChange(PropertyChangeEvent evt) {
                if (new String().equals(evt.getPropertyName())) {
                    opti.setMessage(evt.getNewValue());
                }
                if (TSK.isDone()) {
                    dia.dispose();
                }
            }
        });

        TSK.execute();

        dia.setVisible(true);

        if (Language.getString("Music.4").equals(opti.getValue())) { //$NON-NLS-1$
            TSK.cancel(false);
            CHANGED = true;
        } else {
            CHANGED = false;
        }

        if (!CHANGED)
        {
            this.MUSIC = whereTo;
        }

        return !CHANGED;
    }

    /**There you go, returns Wave.*/
    @Override
    public Wave getWave(int i) throws InvalidArgumentsException {
        SndEntry snd = MAP.get(i);
        if (snd == null) {
            throw new InvalidArgumentsException(getIndexNotFoundString(i));
        }
        Wave t;
        if (WAVE.containsKey(i)) {
            t = WAVE.get(i);
            if (t != null) {
                return t;
            }
        }
        try {
            FileInputStream reader = new FileInputStream(MUSIC);
            reader.skip(snd.ADDRESS);
            byte[] b = new byte[snd.SIZE];
            reader.read(b);
            t = new Wave(3, b, snd.FRAMES);
            reader.close();
            return t;
        } catch (IOException ew) {
            UE.displayError(ew);
        } catch (UnsupportedWave ew) {
            UE.displayError(ew);
        }
        throw new InvalidArgumentsException(getIndexNotFoundString(i));
    }

    /**Opens the provided file.
    @param whereFrom the File to be opened.
    @return a String array of edit options specific for the opened file.**/
    @Override
    public StringGroup[] open(File whereFrom) {
        if (UE.FILES.getFile(FileManager.P_STBOF) != null) {
            STBOF = (Stbof) UE.FILES.getFile(FileManager.P_STBOF);
        }
        try {
            MUSIC = whereFrom;
            FileInputStream reader = new FileInputStream(MUSIC);
            //read mini map
            byte[] b = new byte[4];
            for (int i = 0; i < 16; i++) {
                reader.read(b);
                int num = Tools.toInt(b, true);
                reader.read(b);
                int add = Tools.toInt(b, true);
                MINI_MAP[i] = new Pair<Integer, Integer>(num, add);
            }
            //read map
            int add = ((Integer) MINI_MAP[0].RIGHT).intValue();
            reader.close();
            reader = new FileInputStream(MUSIC);
            reader.skip(add);
            for (int j = 0; j < 151; j++) {
                MAP.add(new SndEntry(reader));
            }
            try {
                if (reader.available() > 0) {
                    SndEntry snd = new SndEntry(reader);
                    reader.close();
                    reader = new FileInputStream(MUSIC);
                    reader.skip(snd.ADDRESS);
                    b = new byte[snd.SIZE];
                    reader.read(b);
                    ByteArrayInputStream bais = new ByteArrayInputStream(b);
                    MODS = new ModList(bais);
                } else {
                    MODS = new ModList();
                }
            } catch (Exception g) {
                MODS = new ModList();
                g.printStackTrace();
            }
            try {
                System.gc();
                reader.close();
                reader = new FileInputStream(MUSIC);
                reader.skip(add + 151 * 16);
                if (reader.available() > 0) {
                    reader.skip(16);
                }
                if (reader.available() > 0) {
                    SndEntry snd = new SndEntry(reader);
                    reader.close();
                    reader = new FileInputStream(MUSIC);
                    reader.skip(snd.ADDRESS);
                    b = new byte[snd.SIZE];
                    reader.read(b);
                    ByteArrayInputStream bais = new ByteArrayInputStream(b);
                    DESCRIPTION = new SndDesc(bais);
                } else {
                    String fn = "music.dsc"; //$NON-NLS-1$

                    File fdesc = new File(fn);
                    if (!fdesc.exists()) {
                        fdesc.createNewFile();
                    }
                    FileInputStream filod = new FileInputStream(fdesc);
                    DESCRIPTION = new SndDesc(filod);
                }
            } catch (Exception g) {
                DESCRIPTION = new SndDesc();
            }
            reader.close();
        } catch (Exception ef) {
            UE.displayError(ef);
            MUSIC = null;
            return null;
        }
        return getCommands();
    }

    /**@return a String array of available commands that can be used with the edit function.*/
    public StringGroup[] getCommands() {
        if (MUSIC == null || STBOF == null) {
            return null;
        }
        cmdCHECK = Language.getString("Music.0"); //$NON-NLS-1$
        cmdMODLIST = Language.getString("Music.1"); //$NON-NLS-1$
        cmdMUSIC = Language.getString("Music.2"); //$NON-NLS-1$

        StringGroup[] ret = new StringGroup[2];
        try {
            ret[0] = new StringGroup(Language.getString("Music.5"), new String[]{ //$NON-NLS-1$
                        cmdCHECK, cmdMODLIST});
            ret[1] = new StringGroup(Language.getString("Music.6"), new String[]{cmdMUSIC}); //$NON-NLS-1$
        } catch (Exception tz) {
            UE.displayError(tz);
            return null;
        }
        return ret;
    }

    /**@return true if changes were made, else false.*/
    @Override
    public boolean madeChanges() {
        try {
            return (CHANGED || DESCRIPTION.madeChanges() || CANCELLED_SAVE);
        } catch (Exception f) {
            f.printStackTrace();
            return false;
        }
    }

    /**This function checks the file for known problems/errors that may occur while editing the file.
    @return a Vector object containing results of the check as Strings.
     */
    @Override
    public Vector<String> check() {
        Vector<String> response = new Vector<String>();
        String msg;

        if (MAP.size() != 151) {
            msg = Language.getString("Music.7"); //$NON-NLS-1$
            msg = msg.replace("%1", MUSIC.getName()); //$NON-NLS-1$
            msg = msg.replace("%2", Integer.toString(MAP.size())); //$NON-NLS-1$
            response.add(msg);
        }
        SndEntry snd = MAP.get(0);
        for (int i = 1; i < MAP.size(); i++) {
            SndEntry snd2 = MAP.get(i);
            if (snd.ADDRESS + snd.SIZE != snd2.ADDRESS) {
                msg = Language.getString("Music.8"); //$NON-NLS-1$
                msg = msg.replace("%1", MUSIC.getName()); //$NON-NLS-1$
                msg = msg.replace("%2", Integer.toString(snd.ADDRESS)); //$NON-NLS-1$
                response.add(msg);
            }
            snd = snd2;
        }
        if (response.isEmpty()) {
            response.add(Language.getString("Music.9")); //$NON-NLS-1$
        }
        return response;
    }

    /**Extracts a playable wav file to the specified destination.
    @param index the index of the sound file
    @param dest the destination file*/
    @Override
    public void extract(int index, File dest) throws IOException, InvalidArgumentsException {
        Wave tmp = getWave(index);
        FileOutputStream nu = new FileOutputStream(dest);
        tmp.save(nu);
        nu.close();
    }

    /**replaces a sound in snd with the specified one.
    @param index the index of the sound file
    @param source the source file*/
    @Override
    public void replace(int index, File source) throws IOException, InvalidArgumentsException, Exception {
        if (index < 0 || index > 151) {
            throw new InvalidArgumentsException(getIndexNotFoundString(index));
        }
        FileInputStream sig = new FileInputStream(source);
        int tz = 0;
        byte[] zuz = new byte[sig.available()];
        while (sig.available() > 0) {
            tz = sig.read(zuz, tz, sig.available());
        }
        Wave m = new Wave(zuz);
        if (m.getSndType() != 3) {
            throw new Exception(Language.getString("Music.10")); //$NON-NLS-1$
        }
        WAVE.put(index, m);
        CHANGED = true;
    }

    /**replaces a sound in snd with the specified one.
    @param index the index of the sound file
    @param source the source file*/
    public void replace(int index, Wave source) throws IOException, InvalidArgumentsException, Exception {
        if (index < 0 || index > 151) {
            throw new InvalidArgumentsException(getIndexNotFoundString(index));
        }
        if (source.getSndType() != 3) {
            throw new Exception(Language.getString("Music.10")); //$NON-NLS-1$
        }
        WAVE.put(index, source);
        //check memory, force save
        Runtime rn = Runtime.getRuntime();
        rn.gc();

        CHANGED = true;
    }

    /**Creates one big playable file.
    @param dest the destination file*/
    @Override
    public void thinkBig(File dest) throws Exception {
        final ThinkBigTask TSK = new ThinkBigTask(dest);

        final JOptionPane opti = new JOptionPane(Language.getString("Music.3"), //$NON-NLS-1$
                JOptionPane.PLAIN_MESSAGE);
        opti.setOptions(new String[]{Language.getString("Music.4")}); //$NON-NLS-1$
        final JDialog dia = opti.createDialog(UE.WINDOW, UE.APP_NAME);

        TSK.addPropertyChangeListener(new PropertyChangeListener() {

            public void propertyChange(PropertyChangeEvent evt) {
                if (new String().equals(evt.getPropertyName())) {
                    opti.setMessage(evt.getNewValue());
                }
                if (TSK.isDone()) {
                    dia.dispose();
                }
            }
        });

        TSK.execute();

        dia.setVisible(true);

        if (Language.getString("Music.4").equals(opti.getValue())) { //$NON-NLS-1$
            TSK.cancel(false);
        }
    }

    /**Plays the specified sound.
    @param index the index of the sound*/
    @Override
    public void play(int index) throws Exception {
        throw new Exception(Language.getString("Music.12")); //$NON-NLS-1$
    }

    /**Returns true if the play function is supported by the class.*/
    @Override
    public boolean isPlayable() {
        return false;
    }

    private String getIndexNotFoundString(int index) {
        String msg = Language.getString("Music.13"); //$NON-NLS-1$
        msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
        return msg;
    }

    // FIXME: remove usage of second tmp file, see english.java
    private class SaveTask extends SwingWorker<Void, Void> {

        File whereTo;
        boolean bkp;
        String ext;

        public SaveTask(File dest, boolean bkp, String ext) {
            super();
            whereTo = dest;
            this.bkp = bkp;
            this.ext = ext;
        }

        @Override
        public Void doInBackground() {
            setProgress(0);
            String msg;
            try {
                //open
                if (whereTo.equals(MUSIC)) {
                    if (bkp) {
                        firePropertyChange(new String(), null, Language.getString("Music.22")); //$NON-NLS-1$

                        File file = new File(MUSIC.getPath() + ext);
                        int i = 1;

                        while (file.exists()) {
                            file = new File(MUSIC.getPath() + i + ext);
                            i++;
                        }

                        Tools.copyFile(MUSIC, file);
                    }
                    whereTo = FileManager.getWritableLocalFile("music.tmp"); //$NON-NLS-1$
                }

                msg = Language.getString("Music.14"); //$NON-NLS-1$
                msg = msg.replace("%1", whereTo.getName()); //$NON-NLS-1$

                firePropertyChange(new String(), null, msg);
                whereTo.delete();
                whereTo.createNewFile();

                //write sounds
                File temp2 = new File("music2.tmp"); //$NON-NLS-1$
                FileOutputStream writer = new FileOutputStream(temp2);
                writer.write(new byte[16 * 8]);
                SndEntry snd;
                int index = 128;
                for (int i = 0; i < 151; i++) {
                    msg = Language.getString("Music.15"); //$NON-NLS-1$
                    msg = msg.replace("%1", (int) ((double) (i * 100) / 150) + "%"); //$NON-NLS-1$ //$NON-NLS-2$
                    firePropertyChange(new String(), null, msg);
                    if (isCancelled()) {
                        writer.close();
                        whereTo.delete();
                        CANCELLED_SAVE = true;
                        return null;
                    }
                    if (WAVE.containsKey(i) && ADD_CHANGES_2_MODLIST) {
                        MODS.add(Integer.toString(i));
                    }
                    snd = MAP.get(i);
                    snd.ADDRESS = index;
                    Wave w = getWave(i);
                    snd.SIZE = w.getDataSize();
                    snd.FRAMES = w.getFrames();
                    byte[] b = w.getRawData();
                    writer.write(b);
                    index += snd.SIZE;
                }
                //mod list
                SndEntry m1 = new SndEntry();
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                MODS.save(baos);
                byte[] ghjkjhg = baos.toByteArray();
                m1.SIZE = ghjkjhg.length;
                m1.ADDRESS = index;
                index += m1.SIZE;
                writer.write(ghjkjhg);
                //descriptions
                SndEntry m2 = new SndEntry();
                ByteArrayOutputStream baos2 = new ByteArrayOutputStream();
                DESCRIPTION.save(baos2);
                byte[] umbaya = baos2.toByteArray();
                m2.SIZE = umbaya.length;
                m2.ADDRESS = index;
                if (UE.SETTINGS.getProperty(SettingsManager.STORE_SND_DESC).equals(SettingsManager.YES)) {
                    String fn = "music.dsc"; //$NON-NLS-1$
                    FileOutputStream gre = new FileOutputStream(FileManager.getWritableLocalFile(fn));
                    gre.write(umbaya);
                    gre.close();
                }
                writer.write(umbaya);
                int sizu = index + umbaya.length;
                //write map
                for (int i = 0; i < 151; i++) {
                    msg = Language.getString("Music.16"); //$NON-NLS-1$
                    msg = msg.replace("%1", (int) ((double) (i * 100) / 150) + "%"); //$NON-NLS-1$ //$NON-NLS-2$

                    firePropertyChange(new String(), null, msg);
                    snd = MAP.get(i);
                    snd.save(writer);
                }
                m1.save(writer);
                m2.save(writer);
                writer.close();
                //mini map
                writer = new FileOutputStream(whereTo);
                int sr = 0;
                for (int i = 0; i < 16; i++) {
                    msg = Language.getString("Music.17"); //$NON-NLS-1$
                    msg = msg.replace("%1", (int) ((double) (i * 100) / 15) + "%"); //$NON-NLS-1$ //$NON-NLS-2$
                    firePropertyChange(new String(), null, msg);

                    switch (i) {
                        case 0: {
                            sr = sizu;
                            break;
                        }
                        case 1: {
                            sr = sizu + 5 * 16;
                            break;
                        }
                        case 2: {
                            sr = sizu + 20 * 16;
                            break;
                        }
                        case 3: {
                            sr = sizu + 36 * 16;
                            break;
                        }
                        case 4: {
                            sr = sizu + 46 * 16;
                            break;
                        }
                        case 5: {
                            sr = sizu + 55 * 16;
                            break;
                        }
                        case 6: {
                            sr = sizu + 66 * 16;
                            break;
                        }
                        case 7: {
                            sr = sizu + 70 * 16;
                            break;
                        }
                        case 8: {
                            sr = sizu + 73 * 16;
                            break;
                        }
                        case 9: {
                            sr = sizu + 88 * 16;
                            break;
                        }
                        case 10: {
                            sr = sizu + 103 * 16;
                            break;
                        }
                        case 11: {
                            sr = sizu + 118 * 16;
                            break;
                        }
                        case 12: {
                            sr = sizu + 128 * 16;
                            break;
                        }
                        case 13: {
                            sr = sizu + 140 * 16;
                            break;
                        }
                        case 14: {
                            sr = sizu + 144 * 16;
                            break;
                        }
                        case 15: {
                            sr = sizu + 147 * 16;
                            break;
                        }
                    }
                    ;

                    MINI_MAP[i].RIGHT = new Integer(sr);
                    writer.write(Tools.toByte(((Integer) MINI_MAP[i].LEFT).intValue(), true));
                    writer.write(Tools.toByte(sr, true));
                }
                //copy
                msg = Language.getString("Music.18"); //$NON-NLS-1$
                msg = msg.replace("%1", whereTo.getName()); //$NON-NLS-1$
                firePropertyChange(new String(), null, msg);

                FileInputStream reader = new FileInputStream(temp2);
                reader.skip(128);
                byte[] buh = new byte[2048];
                while (reader.available() > 0) {
                    int len = reader.read(buh);
                    writer.write(buh, 0, len);
                }

                firePropertyChange(new String(), null, Language.getString("Music.19")); //$NON-NLS-1$
                writer.close();
                reader.close();
                temp2.delete();
                //replace?
                if (whereTo.getName().equals("music.tmp")) { //$NON-NLS-1$

                    msg = Language.getString("Music.18"); //$NON-NLS-1$
                    msg = msg.replace("%1", whereTo.getName()); //$NON-NLS-1$
                    firePropertyChange(new String(), null, msg);

                    try {
                        Tools.copyFile(whereTo, MUSIC);
                    } catch (IOException er) {
                        UE.displayError(er);
                        return null;
                    }

                    msg = Language.getString("Music.20"); //$NON-NLS-1$
                    msg = msg.replace("%1", whereTo.getName()); //$NON-NLS-1$
                    firePropertyChange(new String(), null, msg);

                    boolean bu = whereTo.delete();

                    if (!bu) {
                        msg = Language.getString(Language.getString("Music.21")); //$NON-NLS-1$
                        msg = msg.replace("%1", whereTo.getName()); //$NON-NLS-1$
                        UE.displayError(new IOException(msg));
                        return null;
                    }
                }
                CANCELLED_SAVE = false;
                WAVE.clear();
            } catch (Exception er) {
                UE.displayError(er);
            }
            return null;
        }
    }

    private class ThinkBigTask extends SwingWorker<Void, Void> {

        File path;

        public ThinkBigTask(File dest) {
            super();
            path = dest;
        }

        @Override
        public Void doInBackground() {
            setProgress(0);

            try {
                int strRIFF = 1179011410;
                int strWAVE = 1163280727;
                int strFMT = 544501094;
                int WAVE_CHUNK_SIZE = 20;
                short FORMAT = 17;
                short CHANNELS = 2;
                int FREQUENCY = 22050;
                int BYTES_PER_SECOND = 11100;
                short BLOCK_ALIGMENT = 1024;
                short BITS_PER_SAMPLE = 4;
                short EXTRA_FMT = 2;
                short SAMPLES_PER_BLOCK = 1017;
                int strFACT = 1952670054;
                int intFACT = 4;
                int strDATA = 1635017060;

                int j = MAP.size();
                int frames = 0;
                int size = 0;
                for (int i = 0; i < j && i < 151; i++) {
                    if (WAVE.containsKey(i)) {
                        Wave w = getWave(i);
                        size += w.getDataSize();
                        frames += w.getFrames();
                    } else {
                        SndEntry se = MAP.get(i);
                        frames += se.FRAMES;
                        size += se.SIZE;
                    }
                }
                FileOutputStream out = new FileOutputStream(path);
                //RIFF
                out.write(Tools.toByte(strRIFF, true));
                //size-8
                out.write(Tools.toByte((size + 52), true));
                //WAVE
                out.write(Tools.toByte(strWAVE, true));
                //FMT
                out.write(Tools.toByte(strFMT, true));
                //wave chunk size
                out.write(Tools.toByte(WAVE_CHUNK_SIZE, true));
                //format
                out.write(Tools.toByte(FORMAT, true));
                //channels
                out.write(Tools.toByte(CHANNELS, true));
                //frequency
                out.write(Tools.toByte(FREQUENCY, true));
                //bytes per sec
                out.write(Tools.toByte(BYTES_PER_SECOND, true));
                //block aligment
                out.write(Tools.toByte(BLOCK_ALIGMENT, true));
                //bits per sample
                out.write(Tools.toByte(BITS_PER_SAMPLE, true));
                //extra chunk size
                out.write(Tools.toByte(EXTRA_FMT, true));
                //samples per block
                out.write(Tools.toByte(SAMPLES_PER_BLOCK, true));
                //FACT
                out.write(Tools.toByte(strFACT, true));
                //intFACT
                out.write(Tools.toByte(intFACT, true));
                //frames/4
                out.write(Tools.toByte((frames), true));
                //DATA
                out.write(Tools.toByte(strDATA, true));
                //data size
                out.write(Tools.toByte(size, true));
                //write data
                Wave w;
                String msg;
                for (int i = 0; i < j && i < 1365; i++) {
                    if (isCancelled()) {
                        out.close();
                        path.delete();
                        return null;
                    }
                    msg = Language.getString("Music.14"); //$NON-NLS-1$
                    msg = msg.replace("%1", (int) (((double) (i * 100) / (double) 150)) + "%"); //$NON-NLS-1$ //$NON-NLS-2$
                    firePropertyChange(new String(), null, msg);
                    w = getWave(i);
                    byte[] b = w.getRawData();
                    out.write(b);
                }
                out.close();
            } catch (Exception e) {
                UE.displayError(e);
            }
            return null;
        }
    }

    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#getKnownChanges()
     */
    public ModList getKnownChanges() {
        return MODS;
    }

    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#hasBeenModified()
     */
    public boolean hasBeenModified() {
        return !MODS.isEmpty() || madeChanges();
    }
    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#toFile()
     */

    public File toFile() {
        return MUSIC;
    }
}
