/*
Ultimate Editor for Birth Of The Federation 1.02
Copyright (C) 2005-2007  Alan Podlesek

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


File: Sfx.java

Written by: Alan Podlesek <seqdcer@yahoo.com>

Last modified: 6.9.2007
 */
package ue.edit.snd;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.SwingWorker;

import ue.Language;
import ue.UE;
import ue.edit.FileManager;
import ue.edit.mod.ModList;
import ue.edit.res.Stbof;
import ue.exception.InvalidArgumentsException;
import ue.exception.NotAPanel;
import ue.gui.CheckGUI;
import ue.gui.MainPanel;
import ue.gui.ModifiedFilesGUI;
import ue.gui.snd.SoundGUI;
import ue.util.RandomReader;
import ue.util.SettingsManager;
import ue.util.StringGroup;
import ue.util.Tools;
import ue.util.Wave;

public class Sfx extends SndFile {

    public File SFX;
    private Stbof STBOF;
    private boolean CHANGED = false;
    private boolean CANCELLED_SAVE = false;
    private ArrayList<SndEntry> MAP = new ArrayList<SndEntry>();
    private Hashtable<Integer, Wave> WAVE = new Hashtable<Integer, Wave>();
    public boolean ADD_CHANGES_2_MODLIST = true;
    // commands
    private static String cmdCHECK = Language.getString("Sfx.0"); //$NON-NLS-1$
    private static String cmdMODLIST = Language.getString("Sfx.1"); //$NON-NLS-1$
    private static String cmdGAME_SOUNDS = Language.getString("Sfx.2"); //$NON-NLS-1$

    //specified by SndFile
    @Override
    public void markChanged() {
        CHANGED = true;
    }

    /**Returns a MainPanel to edit the file.
    @param command must be one of the commands specified by the open function.
    @return a MainPanel object according to the command, null if command is invalid.**/
    @Override
    public MainPanel edit(String command) throws Exception {
        if (command.equals(cmdGAME_SOUNDS)) {
            return new SoundGUI(this, STBOF);
        }
        if (command.equals(cmdCHECK)) {
            return new CheckGUI(check());
        }
        if (command.equals(cmdMODLIST)) {
            // update modified files list first
            Enumeration<Integer> keys = WAVE.keys();
            while (keys.hasMoreElements()) {
                MODS.add(keys.nextElement().toString());
            }
            // disable main window
            UE.WINDOW.setEnabled(false);
            new ModifiedFilesGUI(this);
            throw new NotAPanel();
        }
        return null;
    }

    @Override
    public int getNumberOfSounds() {
        return MAP.size();
    }

    /**Saves changes.
    @param whereTo File where everything should be saved to. If null save to the same file.
    @return true if save has been successful.
    @throws Exception the function may throw an exception if an error occures.**/
    @Override
    public boolean save(File whereTo, boolean bkp, String ext) throws Exception {
        if (whereTo == null) {
            whereTo = new File(SFX.getAbsolutePath());
        }

        // FIX ME: UEWorker
        final SaveTask TSK = new SaveTask(whereTo, bkp, ext);

        final JOptionPane opti = new JOptionPane(Language.getString("Sfx.3"), //$NON-NLS-1$
                JOptionPane.PLAIN_MESSAGE);
        opti.setOptions(new String[]{Language.getString("Sfx.4")}); //$NON-NLS-1$
        final JDialog dia = opti.createDialog(UE.WINDOW, UE.APP_NAME);

        TSK.addPropertyChangeListener(new PropertyChangeListener() {

            public void propertyChange(PropertyChangeEvent evt) {
                if (new String().equals(evt.getPropertyName())) {
                    opti.setMessage(evt.getNewValue());
                }
                if (TSK.isDone()) {
                    dia.dispose();
                }
            }
        });

        TSK.execute();

        dia.setVisible(true);

        if (Language.getString("Sfx.4").equals(opti.getValue())) { //$NON-NLS-1$
            TSK.cancel(false);
            CHANGED = true;
        } else {
            CHANGED = false;
            this.SFX = whereTo;
        }

        return !CHANGED;
    }

    /**There you go, returns Wave.*/
    @Override
    public Wave getWave(int i) throws InvalidArgumentsException {
        String msg;
        SndEntry snd = MAP.get(i);
        if (snd == null) {
            throw new InvalidArgumentsException(getIndexNotFoundString(i));
        }
        Wave t;
        if (WAVE.containsKey(i)) {
            t = WAVE.get(i);
            if (t != null) {
                return t;
            }
        }
        try {
            RandomReader reader = new RandomReader(new FileInputStream(SFX));
            reader.seek(snd.ADDRESS);
            byte[] b = new byte[snd.SIZE];
            reader.read(b);
            t = new Wave(b);
            reader.close();
            return t;
        } catch (Exception ew) {
            UE.displayError(ew);
        }
        msg = Language.getString("Sfx.5"); //$NON-NLS-1$
        msg = msg.replace("%1", Integer.toString(i)); //$NON-NLS-1$
        throw new InvalidArgumentsException(msg);
    }

    /**Opens the provided file.
    @param whereFrom the File to be opened.
    @return a String array of edit options specific for the opened file.**/
    @Override
    public StringGroup[] open(File whereFrom) {
        if (UE.FILES.getFile(FileManager.P_STBOF) != null) {
            STBOF = (Stbof) UE.FILES.getFile(FileManager.P_STBOF);
        }
        try {
            SFX = whereFrom;
            FileInputStream in = new FileInputStream(SFX);
            byte[] b = new byte[4];
            in.read(b);
            int mapaddy = Tools.toInt(b, true);
            in.skip(mapaddy - 4);
            int count = 0;
            while (in.available() > 0 && count < 163) {
                MAP.add(new SndEntry(in));
                count++;
            }
            SndEntry snd1 = null;
            SndEntry snd2 = null;
            try {
                if (in.available() > 0) {
                    snd1 = new SndEntry(in);
                } else {
                    MODS = new ModList();
                }
            } catch (Exception g) {
                MODS = new ModList();
                g.printStackTrace();
            }
            try {
                if (in.available() > 0) {
                    snd2 = new SndEntry(in);
                } else {
                    String fn = "sfx.dsc"; //$NON-NLS-1$

                    File fdesc = new File(fn);
                    if (!fdesc.exists()) {
                        fdesc.createNewFile();
                    }
                    FileInputStream filod = new FileInputStream(fdesc);
                    DESCRIPTION = new SndDesc(filod);
                }
            } catch (Exception g) {
                DESCRIPTION = new SndDesc();
            }
            in.close();
            //mod list
            if (snd1 != null) {
                in = new FileInputStream(SFX);
                in.skip(snd1.ADDRESS);
                b = new byte[snd1.SIZE];
                in.read(b);
                ByteArrayInputStream bais = new ByteArrayInputStream(b);
                MODS = new ModList(bais);
                bais.close();
                in.close();
            }
            //descriptions
            if (snd2 != null) {
                in = new FileInputStream(SFX);
                in.skip(snd2.ADDRESS);
                b = new byte[snd2.SIZE];
                in.read(b);
                ByteArrayInputStream bais = new ByteArrayInputStream(b);
                DESCRIPTION = new SndDesc(bais);
                bais.close();
                in.close();
            }
        } catch (Exception ef) {
            UE.displayError(ef);
            SFX = null;
            return null;
        }
        return getCommands();
    }

    /**@return a String array of available commands that can be used with the edit function.*/
    public StringGroup[] getCommands() {
        if (SFX == null || STBOF == null) {
            return null;
        }

        cmdCHECK = Language.getString("Sfx.0"); //$NON-NLS-1$
        cmdMODLIST = Language.getString("Sfx.1"); //$NON-NLS-1$
        cmdGAME_SOUNDS = Language.getString("Sfx.2"); //$NON-NLS-1$

        StringGroup[] ret = new StringGroup[2];
        try {
            ret[0] = new StringGroup(Language.getString("Sfx.6"), new String[]{ //$NON-NLS-1$
                        cmdCHECK, cmdMODLIST});
            ret[1] = new StringGroup(Language.getString("Sfx.7"), new String[]{cmdGAME_SOUNDS}); //$NON-NLS-1$
        } catch (Exception tz) {
            UE.displayError(tz);
            return null;
        }
        return ret;
    }

    /**@return true if changes were made, else false.*/
    @Override
    public boolean madeChanges() {
        return (CHANGED || DESCRIPTION.madeChanges() || CANCELLED_SAVE);
    }

    /**This function checks the file for known problems/errors that may occur while editing the file.
    @return a Vector object containing results of the check as Strings.
     */
    @Override
    public Vector<String> check() {
        Vector<String> response = new Vector<String>();
        String msg;

        if (MAP.size() != 163) {
            msg = Language.getString("Sfx.8"); //$NON-NLS-1$
            msg = msg.replace("%1", SFX.getName()); //$NON-NLS-1$
            msg = msg.replace("%2", Integer.toString(163)); //$NON-NLS-1$
            response.add(msg);
        }
        SndEntry snd = MAP.get(0);
        for (int i = 1; i < MAP.size(); i++) {
            SndEntry snd2 = MAP.get(i);
            if (snd.ADDRESS + snd.SIZE != snd2.ADDRESS) {
                msg = Language.getString("Sfx.9"); //$NON-NLS-1$
                msg = msg.replace("%1", SFX.getName()); //$NON-NLS-1$
                msg = msg.replace("%2", Integer.toString(snd.ADDRESS)); //$NON-NLS-1$
                response.add(msg);
            }
            snd = snd2;
        }
        if (response.isEmpty()) {
            response.add(Language.getString("Sfx.10")); //$NON-NLS-1$
        }
        return response;
    }

    /**Extracts a playable wav file to the specified destination.
    @param index the index of the sound file
    @param dest the destination file*/
    @Override
    public void extract(int index, File dest) throws IOException, InvalidArgumentsException {
        Wave tmp = getWave(index);
        FileOutputStream nu = new FileOutputStream(dest);
        tmp.save(nu);
        nu.close();
    }

    /**replaces a sound in snd with the specified one.
    @param index the index of the sound file
    @param source the source file*/
    @Override
    public void replace(int index, File source) throws IOException, InvalidArgumentsException, Exception {
        if (index < 0 || index > 162) {
            throw new InvalidArgumentsException(getIndexNotFoundString(index));
        }
        FileInputStream sig = new FileInputStream(source);
        int tz = 0;
        byte[] zuz = new byte[sig.available()];
        while (sig.available() > 0) {
            tz = sig.read(zuz, tz, sig.available());
        }
        Wave m = new Wave(zuz);
        if (m.getSndType() != 2) {
            throw new Exception(Language.getString("Sfx.11")); //$NON-NLS-1$
        }
        WAVE.put(index, m);
        CHANGED = true;
    }

    /**replaces a sound in snd with the specified one.
    @param index the index of the sound file
    @param source the source file*/
    public void replace(int index, Wave source) throws IOException, InvalidArgumentsException, Exception {
        if (index < 0 || index > 162) {
            throw new InvalidArgumentsException(getIndexNotFoundString(index));
        }
        if (source.getSndType() != 2) {
            throw new Exception(Language.getString("Sfx.11")); //$NON-NLS-1$
        }
        WAVE.put(index, source);
        //check memory, force save
        Runtime rn = Runtime.getRuntime();
        rn.gc();
        CHANGED = true;
    }

    /**Creates one big playable file.
    @param dest the destination file*/
    @Override
    public void thinkBig(File dest) throws Exception {
        final ThinkBigTask TSK = new ThinkBigTask(dest);

        final JOptionPane opti = new JOptionPane(Language.getString("Sfx.3"), //$NON-NLS-1$
                JOptionPane.PLAIN_MESSAGE);
        opti.setOptions(new String[]{Language.getString("Sfx.4")}); //$NON-NLS-1$
        final JDialog dia = opti.createDialog(UE.WINDOW, UE.APP_NAME);

        TSK.addPropertyChangeListener(new PropertyChangeListener() {

            public void propertyChange(PropertyChangeEvent evt) {
                if (new String().equals(evt.getPropertyName())) {
                    opti.setMessage(evt.getNewValue());
                }
                if (TSK.isDone()) {
                    dia.dispose();
                }
            }
        });

        TSK.execute();

        dia.setVisible(true);

        if (Language.getString("Sfx.4").equals(opti.getValue())) { //$NON-NLS-1$
            TSK.cancel(false);
        }
    }

    /**Plays the specified sound.
    @param index the index of the sound*/
    @Override
    public void play(int index) throws InvalidArgumentsException, Exception {
        if (index < 0 || index > 162) {
            throw new InvalidArgumentsException(getIndexNotFoundString(index));
        }
        int count = 0;

        Tools.playSound(null); // stop playing
        File r = new File("test" + count + ".wav"); //$NON-NLS-1$ //$NON-NLS-2$
        if (r.exists()) {
            r.delete();
        }
        extract(index, r);
        Tools.playSound(r);
    }

    private String getIndexNotFoundString(int index) {
        String msg = Language.getString("Sfx.13"); //$NON-NLS-1$
        msg = msg.replace("%1", Integer.toString(index)); //$NON-NLS-1$
        return msg;
    }

    /**Returns true if the play function is supported by the class.*/
    @Override
    public boolean isPlayable() {
        return true;
    }

    private class ThinkBigTask extends SwingWorker<Void, Void> {

        File path;

        public ThinkBigTask(File dest) {
            super();
            path = dest;
        }

        @Override
        public Void doInBackground() {
            setProgress(0);
            String msg;

            try {
                int strRIFF = 1179011410;
                int strWAVE = 1163280727;
                int strFMT = 544501094;
                int WAVE_CHUNK_SIZE = 16;
                short FORMAT = 1;
                short CHANNELS = 1;
                int FREQUENCY = 22050;
                int BYTES_PER_SECOND = 44100;
                short BLOCK_ALIGMENT = 2;
                short BITS_PER_SAMPLE = 16;
                int strDATA = 1635017060;

                int j = MAP.size();
                int frames = 0;
                int size = 0;
                for (int i = 0; i < j && i < 163; i++) {
                    if (WAVE.containsKey(i)) {
                        Wave w = getWave(i);
                        size += w.getDataSize();
                        frames += w.getFrames();
                    } else {
                        SndEntry se = MAP.get(i);
                        frames += se.FRAMES;
                        size += se.SIZE;
                    }
                }
                FileOutputStream out = new FileOutputStream(path);
                RandomReader reader = new RandomReader(new FileInputStream(SFX));
                //RIFF
                out.write(Tools.toByte(strRIFF, true));
                //size-8
                out.write(Tools.toByte((size + 52), true));
                //WAVE
                out.write(Tools.toByte(strWAVE, true));
                //FMT
                out.write(Tools.toByte(strFMT, true));
                //wave chunk size
                out.write(Tools.toByte(WAVE_CHUNK_SIZE, true));
                //format
                out.write(Tools.toByte(FORMAT, true));
                //channels
                out.write(Tools.toByte(CHANNELS, true));
                //frequency
                out.write(Tools.toByte(FREQUENCY, true));
                //bytes per sec
                out.write(Tools.toByte(BYTES_PER_SECOND, true));
                //block aligment
                out.write(Tools.toByte(BLOCK_ALIGMENT, true));
                //bits per sample
                out.write(Tools.toByte(BITS_PER_SAMPLE, true));
                //DATA
                out.write(Tools.toByte(strDATA, true));
                //data size
                out.write(Tools.toByte(size, true));
                //write data
                Wave w;
                for (int i = 0; i < j && i < 163; i++) {
                    if (isCancelled()) {
                        out.close();
                        path.delete();
                        return null;
                    }
                    msg = Language.getString("Sfx.14"); //$NON-NLS-1$
                    msg = msg.replace("%1", (int) (((double) (i * 100) / (double) 162)) + "%"); //$NON-NLS-1$ //$NON-NLS-2$
                    firePropertyChange(new String(), null, msg);
                    if (WAVE.containsKey(i)) {
                        w = getWave(i);
                        byte[] b = w.getRawData();
                        out.write(b);
                    } else {
                        SndEntry snd = MAP.get(i);
                        reader.seek(snd.ADDRESS);
                        byte[] b = new byte[snd.SIZE];
                        reader.read(b);
                        out.write(b);
                    }
                }
                reader.close();
                out.close();
            } catch (Exception e) {
                UE.displayError(e);
            }
            return null;
        }
    }

    private class SaveTask extends SwingWorker<Void, Void> {

        File whereTo;
        boolean bkp;
        String ext;

        public SaveTask(File dest, boolean bkp, String ext) {
            super();
            whereTo = dest;
            this.bkp = bkp;
            this.ext = ext;
        }

        @Override
        public Void doInBackground() {
            setProgress(0);
            String msg;

            try {
                //open
                if (whereTo.equals(SFX)) {
                    if (bkp) {
                        firePropertyChange(new String(), null, Language.getString("Sfx.22")); //$NON-NLS-1$

                        File file = new File(SFX.getPath() + ext);
                        int i = 1;

                        while (file.exists()) {
                            file = new File(SFX.getPath() + i + ext);
                            i++;
                        }

                        Tools.copyFile(SFX, file);
                    }
                    whereTo = FileManager.getWritableLocalFile("sfx.tmp"); //$NON-NLS-1$
                }
                msg = Language.getString("Sfx.14"); //$NON-NLS-1$
                msg = msg.replace("%1", whereTo.getName()); //$NON-NLS-1$
                firePropertyChange(new String(), null, msg);
                whereTo.delete();
                whereTo.createNewFile();
                //get map address
                int mapad = 4; //map address
                int j = MAP.size();
                Wave w;
                //add size of wave files
                for (int i = 0; i < j && i < 163; i++) {
                    msg = Language.getString("Sfx.15"); //$NON-NLS-1$
                    msg = msg.replace("%1", (int) ((double) (i * 100) / 162) + "%"); //$NON-NLS-1$ //$NON-NLS-2$
                    firePropertyChange(new String(), null, msg);
                    if (WAVE.containsKey(i)) {
                        Wave wu = WAVE.get(i);
                        mapad += wu.getSize(); //full size
                        //System.out.println(wu.getSize());
                        if (ADD_CHANGES_2_MODLIST) {
                            MODS.add(Integer.toString(i));
                        }
                    } else {
                        SndEntry sus = MAP.get(i);
                        mapad += sus.SIZE;
                    }
                }
                //mod list size
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                MODS.save(baos);
                byte[] modsb = baos.toByteArray();
                mapad += modsb.length;
                //descriptions size
                ByteArrayOutputStream baos2 = new ByteArrayOutputStream();
                DESCRIPTION.save(baos2);
                byte[] umbaya = baos2.toByteArray();
                mapad += umbaya.length;
                if (UE.SETTINGS.getProperty(SettingsManager.STORE_SND_DESC).equals(SettingsManager.YES)) {
                    String fn = "sfx.dsc"; //$NON-NLS-1$
                    FileOutputStream gre = new FileOutputStream(FileManager.getWritableLocalFile(fn));
                    gre.write(umbaya);
                    gre.close();
                }
                //oh joy
                FileOutputStream out = new FileOutputStream(whereTo);
                out.write(Tools.toByte(mapad, true)); //writes map address
                //save sounds
                SndEntry snd = new SndEntry();
                snd.ADDRESS = 0;
                snd.SIZE = 4;
                SndEntry snd2;
                for (int i = 0; i < j && i < 163; i++) {
                    msg = Language.getString("Sfx.16"); //$NON-NLS-1$
                    msg = msg.replace("%1", (int) ((double) (i * 100) / 162) + "%"); //$NON-NLS-1$ //$NON-NLS-2$
                    firePropertyChange(new String(), null, msg);
                    if (isCancelled()) {
                        out.close();
                        whereTo.delete();
                        return null;
                    }
                    snd2 = MAP.get(i);
                    //if loaded
                    if (WAVE.containsKey(i)) {
                        w = WAVE.get(i);
                        snd2.SIZE = w.getSize();
                        snd2.FRAMES = w.getFrames();
                        w.save(out);
                    } else {
                        FileInputStream in = new FileInputStream(SFX);
                        in.skip(snd2.ADDRESS);
                        byte[] b = new byte[snd2.SIZE];
                        in.read(b);
                        out.write(b);
                        in.close();
                    }
                    snd2.ADDRESS = snd.ADDRESS + snd.SIZE;
                    snd = snd2;
                }
                //mod list
                SndEntry m = new SndEntry();
                m.ADDRESS = snd.ADDRESS + snd.SIZE; //set address
                m.SIZE = modsb.length;
                out.write(modsb);
                //descriptions
                SndEntry m2 = new SndEntry();
                m2.ADDRESS = m.ADDRESS + m.SIZE;
                
                m2.SIZE = umbaya.length;
                out.write(umbaya);

                for (int i = 0; i < j && i < 163; i++) {
                    msg = Language.getString("Sfx.17"); //$NON-NLS-1$
                    msg = msg.replace("%1", (int) ((double) (i * 100) / 162) + "%"); //$NON-NLS-1$ //$NON-NLS-2$
                    firePropertyChange(new String(), null, msg);
                    snd = MAP.get(i);
                    snd.save(out);
                }
                m.save(out);
                m2.save(out);
                firePropertyChange(new String(), null, Language.getString("Sfx.18")); //$NON-NLS-1$
                out.close();

                //replace?
                if (whereTo.getName().equals("sfx.tmp")) { //$NON-NLS-1$

                    msg = Language.getString("Sfx.20"); //$NON-NLS-1$
                    msg = msg.replace("%1", SFX.getName()); //$NON-NLS-1$
                    firePropertyChange(new String(), null, msg);

                    try {
                        Tools.copyFile(whereTo, SFX);
                    } catch (IOException er) {
                        UE.displayError(er);
                        return null;
                    }

                    msg = Language.getString("Sfx.19"); //$NON-NLS-1$
                    msg = msg.replace("%1", SFX.getName()); //$NON-NLS-1$
                    firePropertyChange(new String(), null, msg);

                    boolean bu = whereTo.delete();

                    if (!bu) {
                        msg = Language.getString(Language.getString("Sfx.21")); //$NON-NLS-1$
                        msg = msg.replace("%1", whereTo.getName()); //$NON-NLS-1$
                        UE.displayError(new IOException(msg));
                        return null;
                    }
                }
                CANCELLED_SAVE = false;
                WAVE.clear();
            } catch (Exception er) {
                UE.displayError(er);
            }
            return null;
        }
    }

    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#getKnownChanges()
     */
    @Override
    public ModList getKnownChanges() {
        return MODS;
    }

    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#hasBeenModified()
     */
    @Override
    public boolean hasBeenModified() {
        return !MODS.isEmpty() || madeChanges();
    }
    /* (non-Javadoc)
     * @see ue.edit.CanBeEdited#toFile()
     */

    @Override
    public File toFile() {
        return SFX;
    }
}
