/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: SavEdit.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.edit.snd;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Hashtable;

import ue.Language;
import ue.edit.res.stbof.rst.Race;
import ue.util.Tools;

public class SndDesc{
	private boolean CHANGED=false;
	private Hashtable<Integer, DescEntry> ENTRY = new Hashtable<Integer, DescEntry>();
	
	public SndDesc(){
	}
	public SndDesc(InputStream in) throws IOException{
		
		byte[] b = new byte[4];
		int index;
		DescEntry desc;
		while(in.available()>0){
			in.read(b);
			index = Tools.toInt(b, true);
			desc = new DescEntry(in);
			ENTRY.put(index, desc);
		}
	}
	public void save(OutputStream out) throws IOException{
		Enumeration<Integer> KEY = ENTRY.keys();
		int index;
		DescEntry desc;
		while (KEY.hasMoreElements()){
			index = KEY.nextElement();
			desc = ENTRY.get(index);
			out.write(Tools.toByte(index, true));
			desc.save(out);
		}
		CHANGED=false;
	}
	public void export(OutputStream out, int num, Race race) throws IOException{
		for (int i=0; i<num; i++){
			try{
				String str = i + ":\t"; //$NON-NLS-1$
				int g = getRace(i);
				switch (g){
					case 0:{
						str = str + Language.getString("SndDesc.0"); //$NON-NLS-1$
						break;
					}
					case 1:{
						str = str + Language.getString("SndDesc.1"); //$NON-NLS-1$
						break;
					}
					case 7:
					{
                                            str = str + Language.getString("SndDesc.12"); //$NON-NLS-1$
					    break;
					}
					default:{
						str = str + race.getName(g-2);
					}
				}
				str = str + ":\t\t"; //$NON-NLS-1$
				g = getType(i);
				switch (g){
					case 1:{
						str = str + Language.getString("SndDesc.2"); //$NON-NLS-1$
						break;
					}
					case 2:{
						str = str + Language.getString("SndDesc.3"); //$NON-NLS-1$
						break;
					}
					case 3:{
						str = str + Language.getString("SndDesc.4"); //$NON-NLS-1$
						break;
					}
					case 4:{
						str = str + Language.getString("SndDesc.5"); //$NON-NLS-1$
						break;
					}
					case 5:{
						str = str + Language.getString("SndDesc.6"); //$NON-NLS-1$
						break;
					}
					case 6:{
						str = str + Language.getString("SndDesc.7"); //$NON-NLS-1$
						break;
					}
					case 7:{
						str = str + Language.getString("SndDesc.8"); //$NON-NLS-1$
						break;
					}
					case 8:{
						str = str + Language.getString("SndDesc.9"); //$NON-NLS-1$
						break;
					}
					default:{
						str = str + Language.getString("SndDesc.10"); //$NON-NLS-1$
						break;
					}
				}
				str = str + ":\t\t" + getDesc(i) + "\n"; //$NON-NLS-1$ //$NON-NLS-2$
				out.write(Tools.toByte(str, str.length(), str.length()));
			}catch (Exception t){
				t.printStackTrace();
			}
		}
		CHANGED=false;
	}
	public short getRace(int index){
		DescEntry desc = ENTRY.get(index);
		if (desc==null){
			desc = new DescEntry(index);
			ENTRY.put(index, desc);
		}
		return desc.RACE;
	}
	public short getType(int index){
		DescEntry desc = ENTRY.get(index);
		if (desc==null){
			desc = new DescEntry(index);
			ENTRY.put(index, desc);
		}
		return desc.TYPE;
	}
	public String getDesc(int index){
		DescEntry desc = ENTRY.get(index);
		if (desc==null){
			desc = new DescEntry(index);
			ENTRY.put(index, desc);
		}
		return new String(desc.DESC);
	}
	public void setRace(int index, short race){
		DescEntry desc = ENTRY.get(index);
		if (desc==null){
			desc = new DescEntry(index);
			ENTRY.put(index, desc);
		}
		if (desc.RACE==race) {
			return;
		}
		desc.RACE=race;
		CHANGED=true;
	}
	public void setType(int index, short type){
		DescEntry desc = ENTRY.get(index);
		if (desc==null){
			desc = new DescEntry(index);
			ENTRY.put(index, desc);
		}
		if (desc.TYPE==type) {
			return;
		}
		desc.TYPE=type;
		CHANGED=true;
	}
	public void setDesc(int index, String desci){
		DescEntry desc = ENTRY.get(index);
		if (desc==null){
			desc = new DescEntry(index);
			ENTRY.put(index, desc);
		}
		if (desc.DESC.equals(desci)) {
			return;
		}
		desc.DESC=new String(desci);
		CHANGED=true;
	}
	public boolean madeChanges(){ return CHANGED;}
	
	private class DescEntry{
		/*RACE:
		0-unknown
		1-neutral
		*/
		public short RACE;
		/*TYPE:
		0-unknown
		1-tactical battle
		2-diplomatic report
		3-cpu
		4-weapon fire
		5-ship propulsion
		6-ambient music
		*/
		public short TYPE;
		public String DESC;
		public DescEntry(int i){
			RACE=0;
			TYPE=0;
			DESC = Language.getString("SndDesc.11"); //$NON-NLS-1$
			DESC = DESC.replace("%1", Integer.toString(i)); //$NON-NLS-1$
		}
		public DescEntry(InputStream in) throws IOException{
			byte[] b = new byte[2];
			in.read(b);
			RACE=Tools.toShort(b, true);
			in.read(b);
			TYPE=Tools.toShort(b, true);
			b = new byte[4];
			in.read(b);
			b = new byte[Tools.toInt(b, true)];
			in.read(b);
			DESC=Tools.toASCII(b); //FIXME: Should read/save as UTF!
		}
		public void save(OutputStream out) throws IOException{
			byte[] b = Tools.toByte(RACE, true);
			out.write(b);
			b = Tools.toByte(TYPE, true);
			out.write(b);
			b = Tools.toByte(DESC.length(), true);
			out.write(b);
			b = Tools.toByte(DESC, DESC.length(), DESC.length());
			out.write(b);
		}
	}
}
