/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: SndFile.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.edit.snd;

import java.io.*;
import java.util.*;

import ue.edit.*;
import ue.gui.*;
import ue.edit.mod.*;
import ue.edit.res.stbof.rst.Race;
import ue.exception.*;
import ue.util.*;

public abstract class SndFile implements CanBeEdited{
	public ModList MODS;
	protected SndDesc DESCRIPTION;

    /**Returns the number of sounds in this file.*/
	public abstract int getNumberOfSounds();
	/**Extracts a playable wav file to the specified destination.
	@param index the index of the sound file
	@param dest the destination file*/
	public abstract void extract(int index, File dest) throws Exception;
	
	/**replaces a sound in snd with the specified one.
	@param index the index of the sound file
	@param source the source file*/
	public abstract void replace(int index, File source) throws Exception;
	
	/**Creates one big playable file.
	@param dest the destination file*/
	public abstract void thinkBig(File dest) throws Exception;
	
	/**Plays the specified sound.
	@param index the index of the sound*/
	public abstract void play(int index) throws Exception;
	/**There you go, returns Wave.*/
	abstract public Wave getWave(int i) throws InvalidArgumentsException;
	/**Returns true if the play function is supported by the class.*/
	public abstract boolean isPlayable();
    @Override
	public abstract void markChanged();

	/** NOT IMPLEMENTED! This function makes no sense here.*/
    @Override
	public void addInternalFile(InternalFile file) {
        throw new UnsupportedOperationException("Not supported yet.");
	}
	/** NOT IMPLEMENTED! This function makes no sense here.*/
    @Override
	public void discard() {
	}
	/** NOT IMPLEMENTED! This function makes no sense here.*/
    @Override
	public Enumeration<String[]> getFiles(FilenameFilter filter) {
		return null;
	}
	/** NOT IMPLEMENTED! This function makes no sense here.*/
    @Override
	public InternalFile getInternalFile(String name, boolean cache) throws FileNotFoundException {
		throw new UnsupportedOperationException("Not supported yet.");
	}
	/** NOT IMPLEMENTED! This function makes no sense here.*/
    @Override
	public Enumeration<String> getListOfLoadedFiles() {
		return null;
	}
	/** NOT IMPLEMENTED! This function makes no sense here.*/
    @Override
	public Iterator<String> getRemovedFilesList() {
		return null;
	}
	/** NOT IMPLEMENTED! This function makes no sense here.*/
    @Override
	public void removeInternalFile(String name) {
	}
	
	public short getRaceDescriptor(int index)
	{
	    return DESCRIPTION.getRace(index);
	}
	
        public short getTypeDescriptor(int index)
        {
            return DESCRIPTION.getType(index);
        }
	
        public String getDescription(int index)
        {
            return DESCRIPTION.getDesc(index);
        }
        
        public void setTypeDescriptor(int index, short type)
        {
            DESCRIPTION.setType(index, type);
        }

        public void setRaceDescriptor(int index, short race)
        {
            DESCRIPTION.setRace(index, race);
        }
        
        public void exportDescriptions(OutputStream out, int num, Race race) throws IOException
        {
            DESCRIPTION.export(out, getNumberOfSounds(), race);
        }
        
        public void setDescription(int index, String desc)
        {
            DESCRIPTION.setDesc(index, desc);
        }

    @Override
    public long getInternalFileSize(String filename) throws IOException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public byte[] getInternalFileHash(String filename) throws Exception
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }
}
