/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: AniLabel.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui;

import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.SwingConstants;

import ue.UE;
import ue.exception.UnsupportedImage;
import ue.util.AniOrCur;

/**Animation.*/
@SuppressWarnings("serial") //$NON-NLS-1$
public class AniLabel extends JLabel
{
	private AniOrCur animation;
	private int delay; //in miliseconds
	private int index;
	private javax.swing.Timer timer;
    private	Image[] images;
    
	private ActionListener taskPerformer = new ActionListener()
    {
        @Override
        public void actionPerformed(ActionEvent evt)
        {
            synchronized(this)
            {
                if (animation != null)
                {
                    if (delay > 0)
                    {
                        if (images != null)
                        {
                            if (images.length == 0)
                            {
                                return;
                            }
                            if (index >= images.length)
                            {
                                index = 0;
                            }
                            if (images[index]!=null)
                            {
                                setIcon(new ImageIcon(images[index++]));
                            }
                            else
                            {
                                setIcon(null);
                            }
                            timer.setDelay(delay);
                        }
                    }
                }

                if (!isDisplayable())
                {
                    timer.stop();
                }
            }
        }
    };

	/**Constructs an empty AniLabel*/
	public AniLabel()
    {
		super();
		animation = null;
		delay = 1000;
		index = 0;
		setHorizontalAlignment(SwingConstants.CENTER);
		setVerticalAlignment(SwingConstants.CENTER);

		timer = new javax.swing.Timer(delay, taskPerformer);
        timer.setRepeats(true);
        timer.start();
	}

	/**Constructs an AniLabel showing specified animation.*/
	public AniLabel(AniOrCur ani, boolean showAlpha)
    {
		super();
		if (ani==null) {
			return;
		}
		animation = ani;
		delay = animation.getDelay();
		index = 0;
		setHorizontalAlignment(SwingConstants.CENTER);
		setVerticalAlignment(SwingConstants.CENTER);
		
        try
        {
			images = animation.getAllFrames(showAlpha);
		}
        catch (UnsupportedImage ze)
        {
			UE.displayError(ze);
		}

		timer = new javax.swing.Timer(delay, taskPerformer);
        timer.setRepeats(true);
		taskPerformer.actionPerformed(null);
        timer.start();
	}

    /**Constructs an AniLabel the uses a "still" image*/
	public AniLabel(Icon img)
    {
		super();
		animation = null;
		delay = 1000;
		index = 0;
		setHorizontalAlignment(SwingConstants.CENTER);
		setVerticalAlignment(SwingConstants.CENTER);

        this.setIcon(img);
        
		timer = new javax.swing.Timer(delay, taskPerformer);
        timer.setRepeats(true);
        timer.start();
	}

	/**Sets the animation.*/
	public void setAnimation(AniOrCur ani, boolean showAlpha)
    {
        timer.stop();
		animation = ani;
		
        if (animation==null)
        {
			setIcon(null);
			return;
		}
		try{
			images = animation.getAllFrames(showAlpha);
		}catch (UnsupportedImage ze){
			UE.displayError(ze);
		}
		delay = animation.getDelay();
		taskPerformer.actionPerformed(null);
        timer.start();
	}
}
