/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2010  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: CyclingHorizontalSpinner.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 2.12.2010
 */

package ue.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SpinnerListModel;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.event.ChangeListener;
import ue.util.ID;

/**
 *
 */
public class CyclingHorizontalSpinner extends JPanel
{
    private JButton[] btnCtrl;
    private JTextField txtField;

    private Object first;
    private Object last;
    private SpinnerListModel model;
    private boolean editable = true;

    public CyclingHorizontalSpinner()
    {
        first = null;

        btnCtrl = new JButton[2];
        btnCtrl[0] = new JButton("<");
        btnCtrl[1] = new JButton(">");
        txtField = new JTextField(40);
        txtField.setEditable(false);
        txtField.setBorder(null);

        btnCtrl[0].setIconTextGap(0);
        btnCtrl[0].setMargin(new Insets(0, 0, 0, 0));
        btnCtrl[1].setIconTextGap(0);
        btnCtrl[1].setMargin(new Insets(0, 0, 0, 0));
        int buttonheight = txtField.getPreferredSize().height;
        btnCtrl[0].setPreferredSize(new Dimension(buttonheight + 5, buttonheight));
        btnCtrl[1].setPreferredSize(new Dimension(buttonheight + 5, buttonheight));

        UIDefaults uidef = UIManager.getDefaults();
        Color bcolor = uidef.getColor("Button.toolBarBorderBackground");

        if (bcolor == null)
        {
            bcolor = new Color(-6710887);
        }
        txtField.setBorder(BorderFactory.createMatteBorder(1, 0, 1, 0, bcolor));
        txtField.setHorizontalAlignment(JTextField.CENTER);

        // actions
        // prev
        btnCtrl[0].addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                if (editable)
                {
                    commitEdit();
                }

                ID val = getPreviousValue();
                setValue(val);
            }
        });
        // next
        btnCtrl[1].addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                if (editable)
                {
                    commitEdit();
                }

                ID val = getNextValue();
                setValue(val);
            }
        });

        // layout
        this.setLayout(new BorderLayout(0, 0));
        this.add(btnCtrl[0], BorderLayout.LINE_START);
        this.add(txtField, BorderLayout.CENTER);
        this.add(btnCtrl[1], BorderLayout.LINE_END);

        model = new SpinnerListModel();
    }

    public CyclingHorizontalSpinner(SpinnerListModel listModel)
    {
        this();

        setModel(listModel);
    }

    public void setEditable(boolean val)
    {
        editable = val;
    }

    public synchronized ID getNextValue()
    {
        Object val = model.getNextValue();

        if (val == null)
        {
            val = first;
        }

        return (ID)val;
    }

    public synchronized ID getPreviousValue()
    {
        Object val = model.getPreviousValue();

        if (val == null)
        {
            val = last;
        }

        return (ID)val;
    }

    public synchronized ID getValue()
    {
        Object val = model.getValue();

        if (val instanceof ID)
        {
            return (ID) val;
        }
        else
        {
            return null;
        }
    }

    public void addChangeListener(ChangeListener listener)
    {
        model.addChangeListener(listener);
    }

    public void removeChangeListener(ChangeListener listener)
    {
        model.removeChangeListener(listener);
    }

    public void setModel(SpinnerListModel model)
    {
        if (model == null)
        {
            first = null;
            last = null;
        }

        sortList(model);

        first = model.getValue();

        if (first != null)
        {
            txtField.setEditable(editable);
            txtField.setText(first.toString());
        }

        // get last
        last = first;

        while (model.getNextValue() != null)
        {
            last = model.getNextValue();
            model.setValue(last);
        }

        model.setValue(first);

        if (this.model != null)
        {
            ChangeListener[] listeners = this.model.getChangeListeners();

            if (listeners != null)
            {
                for (ChangeListener l : listeners)
                {
                    this.model.removeChangeListener(l);
                    model.addChangeListener(l);
                }
            }
        }

        this.model = model;
    }

    public void setValue(ID val)
    {
        model.setValue(val);
        
        if (val != null)
        {
            txtField.setEditable(editable);
            txtField.setText(model.getValue().toString());
        }
    }

    public void commitEdit()
    {
        ID current = this.getValue();

        if (current != null)
        {
            String newname = txtField.getText();

            if (!newname.equals(current.toString()))
            {
                if (newname.length() > 0)
                {
                    current.setName(newname);
                    sortList(model);
                }
            }
        }
    }

    private void sortList(SpinnerListModel model)
    {
        if (model == null)
        {
            return;
        }

        List<?> list = model.getList();

        Collections.sort(list, new Comparator<Object>(){
            @Override
            public int compare(Object o1, Object o2)
            {
                return o1.toString().compareToIgnoreCase(o2.toString());
            }
        });

        this.model.setList(list);
    }

    public SpinnerListModel getModel()
    {
        return model;
    }

    @Override
    public void setFont(Font font)
    {
        super.setFont(font);

        if (btnCtrl != null)
        {
            btnCtrl[0].setFont(font);
            btnCtrl[1].setFont(font);
            txtField.setFont(font);
        }
    }
}