/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: StbofFilesGUI.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.util.Enumeration;
import java.util.Iterator;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.Popup;
import javax.swing.PopupFactory;
import javax.swing.RowSorter;
import javax.swing.SwingConstants;
import javax.swing.Timer;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;

import ue.Language;
import ue.UE;
import ue.edit.FilesInterface;
import ue.edit.InternalFile;
import ue.edit.res.stbof.fnt.TgaFont;
import ue.exception.InvalidArgumentsException;
import ue.util.AniOrCur;
import ue.util.GifImage;
import ue.util.SettingsManager;
import ue.util.TargaImage;

@SuppressWarnings("serial")  //$NON-NLS-1$
public class FilesGUI extends MainPanel
{
    private PreviewLoader LOADER;
    private FilesInterface INTERFACE;
    private int SELECTED = -1;
    //components
    private JTable tblFILES;
    private JTextArea txtSTATUS;
    private JButton btn_Add;
    private JButton btn_Remove;
    private JButton btn_Discard;
    private JButton btn_Extract;
    private JLabel lblPREVIEW;
    private JCheckBox chkPREVIEW;
    private JScrollPane tblScroll;
    private JTextField txtSearch;
    private Popup popSearch;

    private boolean popupShowing = false;

    final Timer searchReset = new Timer(2000, new ActionListener()
    {
        @Override
        public void actionPerformed(ActionEvent e)
        {
            synchronized (searchReset)
            {
                searchStr = "";

                if (popupShowing && popSearch != null)
                {
                    popSearch.hide();
                    popupShowing = false;
                }
            }
        }
    });

    private SearchStringListener ssl;

    String searchStr = "";

    /**constructor*/
    public FilesGUI(FilesInterface f)
    {
        if (f == null)
        {
            return;
        }

        INTERFACE = f;

        ssl = new SearchStringListener();

        //buttons
        btn_Add = new JButton(Language.getString("FilesGUI.0")); //$NON-NLS-1$
        btn_Add.addActionListener( new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent ae)
            {
                JFileChooser fc = new JFileChooser();
                fc.setCurrentDirectory(new File(UE.SETTINGS.getProperty(SettingsManager.WORK_PATH)));
                fc.setDialogTitle(UE.APP_NAME + " - " + Language.getString("FilesGUI.1")); //$NON-NLS-1$ //$NON-NLS-2$
                fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
                fc.setMultiSelectionEnabled(true);
                int returnVal = fc.showDialog(UE.WINDOW, Language.getString("FilesGUI.0")); //$NON-NLS-1$
                
                if(returnVal != JFileChooser.APPROVE_OPTION)
                {
                    return;
                }
                
                File[] fil = fc.getSelectedFiles();
                if (fil.length>0)
                {
                    try
                    {
                        UE.SETTINGS.setProperty(SettingsManager.WORK_PATH, fil[0].getParentFile().getAbsolutePath());
                        INTERFACE.add(fil);
                        String[][] xe = INTERFACE.getFiles();
                        tblFILES.setModel(new TableData(xe));
                        updateStatus();
                    }
                    catch (Exception e)
                    {
                        UE.displayError(e);
                    }
                }
            }
        });
        
        btn_Remove = new JButton(Language.getString("FilesGUI.2")); //$NON-NLS-1$
        btn_Remove.setEnabled(false);
        btn_Remove.addActionListener( new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent ae)
            {
                if (tblFILES.getSelectedRow() >= 0)
                {
                    try
                    {
                        int[] s = tblFILES.getSelectedRows();
                        for (int i=0; i < s.length; i++)
                        {
                            INTERFACE.remove((String)tblFILES.getValueAt(s[i], 0));
                        }
                        String[][] xe = INTERFACE.getFiles();
                        tblFILES.setModel(new TableData(xe));
                        updateStatus();
                    }
                    catch (Exception e)
                    {
                        UE.displayError(e);
                    }
                }
            }
        });
	
        btn_Discard = new JButton(Language.getString("FilesGUI.3")); //$NON-NLS-1$
        btn_Discard.addActionListener( new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent ae)
            {
                try
                {
                    INTERFACE.discard();
                    String[][] xe = INTERFACE.getFiles();
                    tblFILES.setModel(new TableData(xe));
                    tblFILES.invalidate();
                    updateStatus();
                }
                catch (Exception e)
                {
                    UE.displayError(e);
                }
            }
        });
	
        btn_Extract = new JButton(Language.getString("FilesGUI.4")); //$NON-NLS-1$
        btn_Extract.setEnabled(false);
        btn_Extract.addActionListener( new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent ae)
            {
                if (tblFILES.getSelectedRow() >= 0)
                {
                    JFileChooser fc = new JFileChooser();
                    fc.setCurrentDirectory(new File(UE.SETTINGS.getProperty(SettingsManager.WORK_PATH)));
                    fc.setDialogTitle(UE.APP_NAME + " - " + Language.getString("FilesGUI.5")); //$NON-NLS-1$ //$NON-NLS-2$
                    fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                    fc.setMultiSelectionEnabled(false);
                    int returnVal = fc.showDialog(UE.WINDOW, Language.getString("FilesGUI.4")); //$NON-NLS-1$
                    
                    if(returnVal != JFileChooser.APPROVE_OPTION)
                    {
                        return;
                    }
                    File fil = fc.getSelectedFile();
                    if (fil!=null)
                    {
                        UE.SETTINGS.setProperty(SettingsManager.WORK_PATH, fil.getAbsolutePath());

                        try
                        {
                            int[] s = tblFILES.getSelectedRows();
                            
                            String[] li = new String[s.length];
                            
                            for (int i=0; i < s.length; i++)
                            {
                                li[i] = (String)tblFILES.getValueAt(s[i], 0);
                            }
                            INTERFACE.extract(li, fil);
                        }
                        catch (InvalidArgumentsException e)
                        {
                            UE.displayError(e);
                        }
                        catch (Exception e)
                        {
                            UE.displayError(e);
                        }
                    }
                }
            }
        });
	
        //status
        txtSTATUS = new JTextArea();
        txtSTATUS.setLineWrap(true);
        txtSTATUS.setWrapStyleWord(true);
        txtSTATUS.setEditable(false);
        
        // preview
        lblPREVIEW = new JLabel();
        lblPREVIEW.setHorizontalAlignment(JLabel.CENTER);
        lblPREVIEW.setVerticalAlignment(JLabel.CENTER);
        lblPREVIEW.setBackground(Color.WHITE);
        lblPREVIEW.setVerticalTextPosition(JLabel.CENTER);
        lblPREVIEW.setHorizontalTextPosition(JLabel.CENTER);
        lblPREVIEW.setBorder(BorderFactory.createEtchedBorder());
        int width = btn_Extract.getPreferredSize().width;
        if (width < btn_Remove.getPreferredSize().width)
        {
            width = btn_Remove.getPreferredSize().width;
        }
        if (width < btn_Add.getPreferredSize().width)
        {
            width = btn_Add.getPreferredSize().width;
        }
        if (width < btn_Discard.getPreferredSize().width)
        {
            width = btn_Discard.getPreferredSize().width;
        }
        lblPREVIEW.setPreferredSize(new Dimension(width, width));
        lblPREVIEW.setHorizontalAlignment(SwingConstants.CENTER);
        lblPREVIEW.setVerticalAlignment(SwingConstants.CENTER);
        setPreview(null);

        // search string
        txtSearch = new JTextField(12);

        // chkPREVIEW
        chkPREVIEW = new JCheckBox(Language.getString("FilesGUI.9"), true); //$NON-NLS-1$
        chkPREVIEW.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                tblFILES.tableChanged(null);
            }
        });
        
        //list
        try
        {
            tblFILES = new JTable();
            tblFILES.setAutoCreateRowSorter(true);
            tblFILES.getSelectionModel().setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
            tblFILES.setShowGrid(false);
            tblFILES.setIntercellSpacing(new Dimension(0, 0));
            tblFILES.setRowSelectionAllowed(true);
            tblFILES.setColumnSelectionAllowed(false);
            tblFILES.setUpdateSelectionOnSort(true);

            tblFILES.getSelectionModel().addListSelectionListener(new ListSelectionListener()
            {
                private Boolean working = false;
                
                @Override
                public void valueChanged(ListSelectionEvent arg0)
                {
                    if (SELECTED == tblFILES.getSelectedRow()
                            || arg0.getValueIsAdjusting() || working)
                    {
                        return;
                    }
                    
                    working = true;
                    
                    SELECTED = tblFILES.getSelectedRow();
                    
                    btn_Extract.setEnabled(tblFILES.getSelectedRow() >= 0);
                    btn_Remove.setEnabled(tblFILES.getSelectedRow() >= 0);
                    
                    if (chkPREVIEW.isSelected() && tblFILES.getSelectedRowCount() == 1)
                    {
                        String name = tblFILES.getValueAt(tblFILES.getSelectedRow(), 0).toString();
                        
                        if (LOADER != null) {
                            LOADER.cancel();
                        }
                        LOADER = new PreviewLoader(name);
                        LOADER.start();
                    }
                    else
                    {
                        if (LOADER != null) {
                            LOADER.cancel();
                        }
                        setPreview(null);
                    }

                    tblFILES.revalidate();
                    working = false;
                }
            });

            tblFILES.addKeyListener(ssl);

            String[][] xe = INTERFACE.getFiles();
            tblFILES.setModel(new TableData(xe));
            tblFILES.setDefaultRenderer(tblFILES.getModel().getColumnClass(0), new CellRenderer());
        }
        catch (Exception e)
        {
            UE.displayError(e);
        }
        tblScroll = new JScrollPane(tblFILES);
        tblScroll.setPreferredSize(new Dimension(400, 250));

        txtSearch.addFocusListener(new FocusListener()
        {
            @Override
            public void focusGained(FocusEvent e)
            {
                // initial search
                searchStr = txtSearch.getText();

                synchronized (searchReset)
                {
                    searchReset.stop();
                    selectNextTableItem();
                    searchReset.start();
                }
            }

            @Override
            public void focusLost(FocusEvent e)
            {
                synchronized (searchReset)
                {
                    searchReset.stop();

                    searchStr = "";

                    if (popupShowing && popSearch != null)
                    {
                        popSearch.hide();
                        popupShowing = false;
                    }
                }
            }
        });

        txtSearch.addKeyListener(new KeyListener()
        {
            @Override
            public void keyTyped(KeyEvent e) {
                synchronized (searchReset)
                {
                    searchReset.stop();

                    if (e.getKeyChar() == KeyEvent.CHAR_UNDEFINED)
                    {
                        searchStr = "";

                        if (popupShowing && popSearch != null)
                        {
                            popSearch.hide();
                            popupShowing = false;
                        }
                    }
                    else
                    {
                        char ch = e.getKeyChar();
                        searchStr = searchStr + ch;

                        // find string
                        selectNextTableItem();

                        searchReset.start();
                    }
                }
            }
            @Override
            public void keyPressed(KeyEvent e) {
            }
            @Override
            public void keyReleased(KeyEvent e) {
            }
        });

        addKeyListener(ssl);
        txtSTATUS.addKeyListener(ssl);
        btn_Add.addKeyListener(ssl);
        btn_Remove.addKeyListener(ssl);
        btn_Discard.addKeyListener(ssl);
        btn_Extract.addKeyListener(ssl);
        lblPREVIEW.addKeyListener(ssl);
        chkPREVIEW.addKeyListener(ssl);
        tblScroll.addKeyListener(ssl);

        //font
        Font def = UE.SETTINGS.getDefaultFont();
        btn_Add.setFont(def);
        btn_Remove.setFont(def);
        btn_Discard.setFont(def);
        btn_Extract.setFont(def);			
        tblFILES.setFont(def);
        tblFILES.setRowHeight(UE.WINDOW.getGraphics().getFontMetrics(def).getHeight());
        txtSTATUS.setFont(def);
        lblPREVIEW.setFont(def);
        chkPREVIEW.setFont(def);
        txtSearch.setFont(def);
        //general
        setLayout(new BorderLayout(0, 0));
        setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        //add
        JPanel pnlCenter = new JPanel();

        pnlCenter.setLayout(new BorderLayout(5, 5));
        pnlCenter.add(tblScroll, BorderLayout.CENTER);

        //status
        JScrollPane scrSTATUS = new JScrollPane(txtSTATUS);
        scrSTATUS.setPreferredSize(new Dimension(400, 100));
        pnlCenter.add(scrSTATUS, BorderLayout.SOUTH);

        add(pnlCenter, BorderLayout.CENTER);

        //buttons
        GridBagConstraints c = new GridBagConstraints();
        c.anchor = GridBagConstraints.NORTH;
        c.fill = GridBagConstraints.HORIZONTAL;

        JPanel pnlEast = new JPanel();
        JPanel pnlOptions = new JPanel();
        pnlOptions.setLayout(new GridBagLayout());
        c.insets = new Insets(0, 0, 5, 0);
        c.gridx = 0;
        c.gridy = 0;
        pnlOptions.add(btn_Add, c);
        c.insets = new Insets(5, 0, 5, 0);
        c.gridy = 1;
        pnlOptions.add(btn_Remove, c);
        c.gridy = 2;
        pnlOptions.add(btn_Extract, c);
        c.gridy = 3;
        pnlOptions.add(btn_Discard, c);
        c.gridy = 4;
        pnlOptions.add(chkPREVIEW, c);
        c.gridy = 5;
        pnlOptions.add(lblPREVIEW, c);

        pnlEast.add(pnlOptions);
        add(pnlEast, BorderLayout.EAST);

        updateStatus();

        // sort table by name
        tblFILES.getRowSorter().toggleSortOrder(0);
    }
    
    public void updateStatus()
    {
        String msg = Language.getString("FilesGUI.6"); //$NON-NLS-1$
        
        txtSTATUS.setText(msg + "\n"); //$NON-NLS-1$
        boolean first = true;
        Enumeration<String> enu = INTERFACE.getListOfLoadedFiles();
        while (enu.hasMoreElements())
        {
            if (first)
            {
                first = false;
                txtSTATUS.setText(txtSTATUS.getText() + enu.nextElement());
            }
            else
            {
                txtSTATUS.setText(txtSTATUS.getText() + ", " + enu.nextElement()); //$NON-NLS-1$
            }
        }
        if (first)
        {
            txtSTATUS.setText(txtSTATUS.getText() + Language.getString("FilesGUI.7")); //$NON-NLS-1$
        }
        txtSTATUS.setText(txtSTATUS.getText() + "\n\n" //$NON-NLS-1$
                + Language.getString("FilesGUI.8") + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
        first = true;
        Iterator<String> it = INTERFACE.getRemovedFilesList();
        while (it.hasNext())
        {
            if (first)
            {
                first = false;
                txtSTATUS.setText(txtSTATUS.getText() + it.next());
            }
            else
            {
                txtSTATUS.setText(txtSTATUS.getText() + ", " + it.next()); //$NON-NLS-1$
            }
        }
        if (first)
        {
            txtSTATUS.setText(txtSTATUS.getText() + Language.getString("FilesGUI.7") //$NON-NLS-1$
                    + "\n"); //$NON-NLS-1$
        }
    }
    
    @Override
    public void finalWarning() throws Exception
    {
    }
    
    private synchronized void setPreview(ImageIcon icon)
    {
        if (icon == null)
        {
            lblPREVIEW.setIcon(null);
            lblPREVIEW.setText(Language.getString("FilesGUI.11")); //$NON-NLS-1$
        }
        else
        {
            lblPREVIEW.setIcon(icon);
            lblPREVIEW.setText(null);
        }
    }

    private void selectNextTableItem()
    {
        // find string
        RowSorter sorter = tblFILES.getRowSorter();

        int i = Math.max(0, tblFILES.getSelectedRow());

        for (int k = 0; k < tblFILES.getRowCount(); k++)
        {
            if (i == sorter.convertRowIndexToModel(k))
            {
                i = k;
                break;
            }
        }

        String item;
        int index;

        for (int k = 1; k < tblFILES.getRowCount(); k++)
        {
            index = sorter.convertRowIndexToModel((k + i) % tblFILES.getRowCount());

            item = tblFILES.getValueAt(index, 0).toString();

            if (item.startsWith(searchStr))
            {
                tblFILES.changeSelection(index, 0, false, false);
                break;
            }
        }
    }

    private class PreviewLoader extends Thread
    {
        private String NAME;
        private boolean stop = false;

        public PreviewLoader(String name)
        {
            NAME = name;
        }

        public void cancel()
        {
            stop = true;
        }

        @Override
        public void run()
        {
            setPreview(null);

            if (NAME == null)
            {
                return;
            }

            lblPREVIEW.setText(Language.getString("FilesGUI.10")); //$NON-NLS-1$

            try
            {
                if (NAME.toLowerCase().endsWith(".tga") //$NON-NLS-1$
                        || NAME.toLowerCase().endsWith(".gif") //$NON-NLS-1$
                        || NAME.toLowerCase().endsWith(".cur") //$NON-NLS-1$
                        || NAME.toLowerCase().endsWith(".fnt") //$NON-NLS-1$
                        || NAME.toLowerCase().endsWith(".ani")) //$NON-NLS-1$
                {
                    InternalFile f = INTERFACE.getInternalFile(NAME);

                    if (f == null)
                    {
                        if (!stop)
                        {
                            setPreview(null);
                        }
                    }
                    else
                    {
                        boolean b = f.madeChanges();
                        ByteArrayOutputStream baos = new ByteArrayOutputStream();
                        f.save(baos);
                        if (b)
                        {
                            f.markChanged();
                        }
                        ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
                        baos.close();

                        if (NAME.toLowerCase().endsWith(".gif")) //$NON-NLS-1$
                        {
                            GifImage gif = new GifImage(bais);

                            Image img = gif.getImage();
                            ImageIcon icon = new ImageIcon(img);

                            if (lblPREVIEW.getWidth() < icon.getIconWidth())
                            {
                                int width = lblPREVIEW.getWidth();
                                int height = icon.getIconHeight();
                                if (icon.getIconWidth() > lblPREVIEW.getWidth())
                                {
                                    height = (int)(((double)lblPREVIEW.getWidth())*icon.getIconHeight()/icon.getIconWidth());

                                    icon = new ImageIcon(img.getScaledInstance(width,
                                            height, Image.SCALE_DEFAULT));
                                }
                            }
                            if (!stop)
                            {
                                setPreview(icon);
                            }
                        }
                        else if (NAME.toLowerCase().endsWith(".tga")) //$NON-NLS-1$
                        {
                            TargaImage img = new TargaImage(bais);

                            // test image
                            if (img.getType()!=2)
                            {
                                String msg = Language.getString("FilesGUI.14"); //$NON-NLS-1$
                                msg = msg.replace("%1", NAME); //$NON-NLS-1$
                                UE.displayError(new Exception(msg));
                            }
                            if (img.getPixelDepth()!=16)
                            {
                                String msg = Language.getString("FilesGUI.15"); //$NON-NLS-1$
                                msg = msg.replace("%1", NAME); //$NON-NLS-1$
                                msg = msg.replace("%2", Integer.toString(img.getPixelDepth())); //$NON-NLS-1$
                                UE.displayError(new Exception(msg));
                            }

                            if (!stop)
                            {
                                ImageIcon icon;
                                if (img.getImageSize().width > lblPREVIEW.getWidth())
                                {
                                    icon = new ImageIcon(img.getThumbnail(lblPREVIEW.getWidth(), false, false));
                                }
                                else
                                {
                                    icon = new ImageIcon(img.getImage(false));
                                }
                                setPreview(icon);
                            }
                        }
                        else if (NAME.toLowerCase().endsWith(".fnt")) //$NON-NLS-1$
                        {
                            TgaFont fnt = new TgaFont(bais);

                            int column = (int)Math.round((Math.random() * 4));
                            int charind;

                            if (column == 0)
                            {
                                charind = (int)Math.round((Math.random() * 9));
                            }
                            else if (column == 3)
                            {
                                charind = (int)Math.round((Math.random() * 31));
                            }
                            else
                            {
                                charind = (int)Math.round((Math.random() * 25));
                            }

                            if (!stop)
                            {
                                Image img = fnt.getCharImage(column, charind);
                                ImageIcon icon = new ImageIcon(img);

                                int height = icon.getIconHeight();
                                int width = icon.getIconWidth();

                                while (lblPREVIEW.getHeight() > height * 2)
                                {
                                    height = height * 2;
                                    width = width * 2;
                                }

                                icon = new ImageIcon(img.getScaledInstance(width,
                                        height, Image.SCALE_DEFAULT));

                                setPreview(icon);
                            }
                        }
                        else if (NAME.toLowerCase().endsWith(".cur") //$NON-NLS-1$
                                || NAME.toLowerCase().endsWith(".ani")) //$NON-NLS-1$
                        {
                            AniOrCur a = new AniOrCur(bais, NAME);
                            TargaImage img = a.getTgaFrame(0);

                            if (!stop)
                            {
                                ImageIcon icon;
                                if (img.getImageSize().width > lblPREVIEW.getWidth())
                                {
                                    icon = new ImageIcon(img.getThumbnail(lblPREVIEW.getWidth(), false, false));
                                }
                                else
                                {
                                    icon = new ImageIcon(img.getImage(false));
                                }
                                setPreview(icon);
                            }
                        }
                    }
                }
                else
                {
                    if (!stop)
                    {
                        setPreview(null);
                    }
                }
            }
            catch (Exception e)
            {
                UE.displayError(e);
                if (!stop) {
                    setPreview(null);
                }
            }
        }
    }
        
    private class TableData extends AbstractTableModel
    {
        String[][] files;

        public TableData(String[][] data)
        {
            files = data;
        }

        /* (non-Javadoc)
         * @see javax.swing.table.TableModel#getColumnCount()
         */
        @Override
        public int getColumnCount()
        {
            return files.length;
        }

        /* (non-Javadoc)
         * @see javax.swing.table.TableModel#getRowCount()
         */
        @Override
        public int getRowCount()
        {
            return files[0].length;
        }

        /* (non-Javadoc)
         * @see javax.swing.table.TableModel#getValueAt(int, int)
         */
        @Override
        public Object getValueAt(int row, int col)
        {
            // try converting it to a number
            String obj = files[col][row];

            if (col == 2)
            {
                try
                {
                    return Double.parseDouble(obj);
                }
                catch (Exception ex)
                {}
            }

            return obj;
        }

        @Override
        public Class<?> getColumnClass(int columnIndex)
        {
            String obj = files[columnIndex][0];

            if (columnIndex == 2)
            {
                try
                {
                    Double.parseDouble(obj);
                    return Double.class;
                }
                catch (Exception ex)
                {}
            }

            return String.class;
        }

        @Override
        public String getColumnName(int columnIndex)
        {
            switch (columnIndex)
            {
                case 0:
                    return Language.getString("FilesGUI.12"); //$NON-NLS-1$
                case 1:
                    return Language.getString("FilesGUI.13"); //$NON-NLS-1$
                default:
                    return "Size in bytes";
            }
        }
   }

   private class CellRenderer extends DefaultTableCellRenderer
   {
    @Override
       public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
       {
            return super.getTableCellRendererComponent(table, value, isSelected, false, row, column);
       }
   }

   private class SearchStringListener implements KeyListener
   {
       @Override
       public void keyTyped(KeyEvent e)
       {
            char ch = e.getKeyChar();

            if (ch != KeyEvent.CHAR_UNDEFINED)
            {
                if (!popupShowing)
                {
                    txtSearch.setText(Character.toString(ch));

                    Point pt = tblScroll.getLocationOnScreen();
                    Dimension d = txtSearch.getPreferredSize();

                    int x = pt.x + tblScroll.getWidth() - d.width;
                    int y = pt.y + tblScroll.getHeight() - d.height;

                    popSearch = PopupFactory.getSharedInstance().getPopup(tblScroll, txtSearch,
                         x, y);

                    popSearch.show();
                    popupShowing = true;

                    txtSearch.requestFocus();

                    return;
                }
            }

            if (popupShowing && popSearch != null)
            {
                popSearch.hide();
                popupShowing = false;
            }
        }

        @Override
        public void keyPressed(KeyEvent e) {
        }

        @Override
        public void keyReleased(KeyEvent e) {
        }
    }
}