/*
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *
 * File: FontDialog.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 14.2.2011
 */

package ue.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.text.ParseException;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import ue.UE;

public class FontDialog extends JDialog
{
    private JLabel lblFont;
    private JLabel lblSize;
    private JList lstFont;
    private JSpinner spiSize;
    private JCheckBox chkItalic;
    private JCheckBox chkBold;
    private JScrollPane scrFont;

    private JButton btnOK;
    private JButton btnCancel;

    private JLabel lblPreview;

    private boolean selected = false;

    public FontDialog(Font def)
    {
        super(UE.WINDOW, UE.APP_NAME + " - Font Chooser", true);
        
        setupComponents(def);
        placeComponents();

        pack();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation(new Point((int)(screenSize.getWidth()/2-100), (int)(screenSize.getHeight()/2-100)));
    }

    private void setupComponents(Font def)
    {
        lblFont = new JLabel("Font:");
        lblFont.setFont(def);

        lblSize = new JLabel("Font size:");
        lblSize.setFont(def);

        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        String fontNames[] = ge.getAvailableFontFamilyNames();

        lstFont = new JList();
        lstFont.setListData(fontNames);
        lstFont.setFont(def);
        lstFont.addListSelectionListener(new ListSelectionListener()
        {
            @Override
            public void valueChanged(ListSelectionEvent e)
            {
                if (!e.getValueIsAdjusting() && lblPreview != null)
                {
                    lblPreview.setFont(getPreviewFont());
                }
            }
        });

        scrFont = new JScrollPane(lstFont);
        lstFont.setSelectedValue(def.getName(), true);
        lstFont.ensureIndexIsVisible(lstFont.getSelectedIndex());

        spiSize = new JSpinner(new SpinnerNumberModel(def.getSize(), 8, 72, 1));
        spiSize.setFont(def);
        spiSize.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent e)
            {
                if (lblPreview != null)
                {
                    lblPreview.setFont(getPreviewFont());
                }
            }
        });

        chkItalic = new JCheckBox("Italic", def.isItalic());
        chkItalic.setFont(new Font(def.getName(), def.getStyle() | Font.ITALIC, def.getSize()));
        chkItalic.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent e)
            {
                if (lblPreview != null)
                {
                    lblPreview.setFont(getPreviewFont());
                }
            }
        });

        chkBold = new JCheckBox("Bold", def.isBold());
        chkBold.setFont(new Font(def.getName(), def.getStyle() | Font.BOLD, def.getSize()));
        chkBold.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent e)
            {
                if (lblPreview != null)
                {
                    lblPreview.setFont(getPreviewFont());
                }
            }
        });

        btnOK = new JButton("Ok");
        btnOK.setFont(def);
        btnOK.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                selected = true;
                setVisible(false);
            }
        });

        btnCancel = new JButton("Cancel");
        btnCancel.setFont(def);
        btnCancel.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                setVisible(false);
            }
        });

        lblPreview = new JLabel("AaBbCcDdGgHhIiJj");
        lblPreview.setFont(def);
    }

    private void placeComponents()
    {
        JPanel pnlCenter = new JPanel(new BorderLayout());

            JPanel pnlTop = new JPanel(new BorderLayout());
            lblFont.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
            pnlTop.add(lblFont, BorderLayout.NORTH);

            JPanel pnlFunc = new JPanel(new BorderLayout());
                JPanel pnlList = new JPanel(new BorderLayout());
                pnlList.add(scrFont, BorderLayout.CENTER);
                pnlList.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

            pnlFunc.add(pnlList, BorderLayout.CENTER);
            
                JPanel pnlRight = new JPanel();
                pnlRight.setLayout(new BoxLayout(pnlRight, BoxLayout.Y_AXIS));

                    JPanel pnlRightTop = new JPanel(new GridLayout(4, 1, 5, 5));

                        JPanel pnlSize = new JPanel(new BorderLayout());
                        lblSize.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 5));
                        pnlSize.add(lblSize, BorderLayout.WEST);

                            JPanel pnlSpiSize = new JPanel(new BorderLayout());
                            pnlSpiSize.add(spiSize, BorderLayout.CENTER);
                            pnlSpiSize.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 5));

                        pnlSize.add(pnlSpiSize, BorderLayout.CENTER);

                    pnlRightTop.add(pnlSize);

                    chkItalic.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 5));
                    pnlRightTop.add(chkItalic);
                    chkBold.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 5));
                    pnlRightTop.add(chkBold);

                pnlRight.add(pnlRightTop);
                pnlRight.add(Box.createVerticalGlue());

            pnlFunc.add(pnlRight, BorderLayout.EAST);

                JPanel pnlPreview = new JPanel(new BorderLayout());
                pnlPreview.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

                lblPreview.setBorder(BorderFactory.createTitledBorder(
                        BorderFactory.createLineBorder(Color.GRAY),
                        "Preview:", TitledBorder.DEFAULT_JUSTIFICATION,
                        TitledBorder.DEFAULT_POSITION, lstFont.getFont()));

                pnlPreview.add(lblPreview, BorderLayout.CENTER);

            pnlFunc.add(pnlPreview, BorderLayout.SOUTH);

        pnlCenter.add(pnlTop, BorderLayout.NORTH);
        pnlCenter.add(pnlFunc, BorderLayout.CENTER);

        JPanel pnlBottom = new JPanel();
        pnlBottom.setLayout(new BoxLayout(pnlBottom, BoxLayout.X_AXIS));
        pnlBottom.add(Box.createHorizontalGlue());
        
            JPanel pnlButtons = new JPanel(new GridLayout(1, 2, 5, 5));
            pnlButtons.add(btnOK);
            pnlButtons.add(btnCancel);
            pnlButtons.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
            
        pnlBottom.add(pnlButtons);

        this.setLayout(new BorderLayout());
        add(pnlCenter, BorderLayout.CENTER);
        add(pnlBottom, BorderLayout.SOUTH);
    }

    public Font getSelectedFont()
    {
        if (!selected)
        {
            return null;
        }

        return getPreviewFont();
    }

    private Font getPreviewFont()
    {
        if (chkItalic == null || chkBold == null || spiSize == null || lstFont == null)
        {
            return getFont();
        }

        int style = Font.PLAIN;

        if (chkItalic.isSelected())
        {
            style = style | Font.ITALIC;
        }

        if (chkBold.isSelected())
        {
            style = style | Font.BOLD;
        }

        try {
            spiSize.commitEdit();
        } catch (ParseException ex) {

        }

        int size = (Integer)spiSize.getValue();

        return new Font(lstFont.getSelectedValue().toString(), style, size);
    }
}
