/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: InfoList.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Last modified: 5.8.2008
 */
package ue.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Vector;

import javax.swing.Icon;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import ue.util.AniOrCur;

/**
 *
 */
@SuppressWarnings("serial") //$NON-NLS-1$
public class InfoList extends JPanel
{
    public static final int MOUSE_CLICK = 0;
    public static final int MOUSE_MOVEMENT = 1;

    private Icon DEFAULT;
    private Color SEL_COLOR;
    private boolean imgOpaque = false;
    private int SELECTED = -1;
    private Vector<InfoPanel> DATA = new Vector<InfoPanel>();
    private Vector<ListSelectionListener> LISTENERS = new Vector<ListSelectionListener>();
    private Vector<SelectionListener> RIGHT_CLICK_LISTENERS = new Vector<SelectionListener>();
    private double total_height = 0;

    private InfoPanel.Orientation orientation;
    private int selection_activator;

    public InfoList(Icon def, boolean imgOpaque, int rows, int columns)
    {
        super();
        DEFAULT = def;
        GridLayout layout = new GridLayout(rows, columns, 0, 0);
        this.setLayout(layout);
        this.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);

        this.setAlignmentX(InfoList.LEFT_ALIGNMENT);
        JList list = new JList();
        SEL_COLOR = list.getSelectionBackground();
        this.imgOpaque = imgOpaque;
        orientation = InfoPanel.Orientation.Horizontal;
    }

    public void setRowsAndColumnsNumber(int rows, int columns)
    {
        GridLayout layout = new GridLayout(rows, columns, 0, 0);
        this.setLayout(layout);
    }

    public void setSelectionBackground(Color background)
    {
        SEL_COLOR = background;
    }

    @Override
    public void setBackground(Color background)
    {
        super.setBackground(background);

        for (Component c : this.getComponents())
        {
            c.setBackground(background);
        }
    }

    @Override
    public void setForeground(Color foreground)
    {
        super.setForeground(foreground);

        for (Component c : this.getComponents())
        {
            c.setForeground(foreground);
        }
    }

    public void setItemLine2Foreground(int index, Color foreground)
    {
        DATA.get(index).setLine2Foreground(foreground);
    }

    public void addItem(Icon img, String line1, String line2, String line3)
    {
        addItem(img, line1, line2, line3, null);
    }

    public void addItem(Icon img, String line1, String line2, String line3, int[] id)
    {
        if (img == null)
        {
            img = DEFAULT;
        }

        InfoPanel pnl = new InfoPanel(img, line1, line2, line3, id);
        pnl.setBackground(this.getBackground());
        pnl.setForeground(this.getForeground());
        pnl.setAlignmentX(InfoPanel.LEFT_ALIGNMENT);
        pnl.addMouseListener(new SelectListener(DATA.size()));
        //pnl.addKeyListener(keyListener);
        Dimension maxSize = pnl.getPreferredSize();
        maxSize.width = Short.MAX_VALUE;
        pnl.setMaximumSize(maxSize);
        pnl.setImageBackgroundOpaque(imgOpaque);
        pnl.setOrientation(orientation);
        pnl.setFont(getFont());
        total_height += maxSize.height;
        DATA.add(pnl);

        this.add(pnl);
        this.invalidate();
    }
    
    public void addItem(AniOrCur ani, String line1, String line2, String line3)
    {
        addItem(ani, line1, line2, line3, null);
    }

    public void addItem(AniOrCur ani, String line1, String line2, String line3, int[] id)
    {
        InfoPanel pnl;

        if (ani == null)
        {
            pnl = new InfoPanel(DEFAULT, line1, line2, line3, id);
        }
        else
        {
            pnl = new InfoPanel(ani, line1, line2, line3, id);
        }

        pnl.setBackground(this.getBackground());
        pnl.setForeground(this.getForeground());
        pnl.setAlignmentX(InfoPanel.LEFT_ALIGNMENT);
        pnl.addMouseListener(new SelectListener(DATA.size()));
        //pnl.addKeyListener(keyListener);
        Dimension maxSize = pnl.getPreferredSize();
        maxSize.width = Short.MAX_VALUE;
        pnl.setMaximumSize(maxSize);
        pnl.setImageBackgroundOpaque(imgOpaque);
        pnl.setOrientation(orientation);
        pnl.setFont(getFont());
        total_height += maxSize.height;
        DATA.add(pnl);

        this.add(pnl);
        this.invalidate();
    }

    public void removeAllItems()
    {
        this.removeAll();
        DATA.clear();
        total_height = 0;

        if (SELECTED != -1)
        {
            SELECTED = -1;
            fireSelectionChanged();
        }

        this.invalidate();
    }
    
    public int getSelectedItemIndex()
    {
        return SELECTED;
    }

    public int[] getSelectedItemID()
    {
        if (SELECTED < 0) return null;

        InfoPanel pnl;

        pnl = DATA.get(SELECTED);

        return pnl.getID();
    }

    public String getSelectedItemLine1()
    {
        if (SELECTED < 0) return null;
        
        InfoPanel pnl;
        
        pnl = DATA.get(SELECTED);
        
        return pnl.getLine1();
    }

    public String getSelectedItemLine2()
    {
        if (SELECTED < 0) return null;
        
        InfoPanel pnl;
        
        pnl = DATA.get(SELECTED);
        
        return pnl.getLine2();
    }

    public String getSelectedItemLine3()
    {
        if (SELECTED < 0) return null;
        
        InfoPanel pnl;
        
        pnl = DATA.get(SELECTED);
        
        return pnl.getLine3();
    }

    public synchronized void setSelectedItemIndex(int index)
    {
        InfoPanel pnl;

        if (SELECTED == index)
        {
            if (SELECTED >= 0)
            {
                pnl = DATA.get(SELECTED);
                pnl.setBackground(getBackground());
            }

            SELECTED = -1;

            this.setVisible(false);
            this.setVisible(true);
            fireSelectionChanged();
        }
        else
        {
            if (SELECTED >= 0)
            {
                pnl = DATA.get(SELECTED);
                pnl.setBackground(getBackground());
            }

            SELECTED = -1;

            if (index < DATA.size() && index >= 0)
            {
                SELECTED = index;
                pnl = DATA.get(SELECTED);
                pnl.setBackground(SEL_COLOR);

                this.setVisible(false);
                this.setVisible(true);
            }
            else
            {
              this.setVisible(false);
              this.setVisible(true);
            }

            fireSelectionChanged();
        }
    }

    /**
     * 
     */
    public synchronized void clearSelection()
    {
        InfoPanel pnl;
        
        if (SELECTED >= 0)
        {
            pnl = DATA.get(SELECTED);
            pnl.setBackground(getBackground());
        }
        
        SELECTED = -1;

        this.setVisible(false);
        this.setVisible(true);

        fireSelectionChanged();
    }

    /**
     * @param listener
     */
    public void addListSelectionListener(ListSelectionListener listener)
    {
        LISTENERS.add(listener);
    }

    public void addRightClickListener(SelectionListener listener)
    {
        RIGHT_CLICK_LISTENERS.add(listener);
    }

    public void setSelectionActivator(int activator)
    {
        selection_activator = activator;
    }

    protected void fireSelectionChanged()
    {
        for (int i = 0; i < LISTENERS.size(); i++)
        {
            LISTENERS.get(i).valueChanged(new ListSelectionEvent(this, SELECTED, SELECTED, true));
        }
    }
    
    protected void fireRightClick(SelectionEvent evt)
    {
        for (int i = 0; i < RIGHT_CLICK_LISTENERS.size(); i++)
        {
            RIGHT_CLICK_LISTENERS.get(i).selected(evt);
        }
    }

    public void setTooltip(int index, String tip)
    {
        if (index < 0 || index >= DATA.size()) return;
        DATA.get(index).setTooltip(tip);
    }

    public void setSelectedItemID(int[] id)
    {
        for (int i = 0; i < DATA.size(); i++)
        {
            if (Arrays.equals(DATA.get(i).getID(), id))
            {
                this.setSelectedItemIndex(i);
                this.fireSelectionChanged();
                return;
            }
        }
        this.fireSelectionChanged();
    }
    
    private class SelectListener implements MouseListener
    {
        private int INDEX;
        
        public SelectListener(int index)
        {
            INDEX = index;
        }
        
        @Override
        public void mouseClicked(MouseEvent evt)
        {
            if (evt.getButton() == MouseEvent.BUTTON3)
            {
                if (selection_activator == MOUSE_CLICK)
                {
                    if (SELECTED != INDEX)
                    {
                        setSelectedItemIndex(INDEX);
                    }
                }

                InfoPanel pnl = DATA.get(INDEX);

                fireRightClick(new SelectionEvent(pnl.getID(), evt.getLocationOnScreen()));
            }
        }

        /* (non-Javadoc)
         * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
         */
        @Override
        public void mouseEntered(MouseEvent evt)
        {
            if (SELECTED != INDEX)
            {
                if ((evt.getModifiersEx() & MouseEvent.BUTTON1_DOWN_MASK) != 0
                        || selection_activator == MOUSE_MOVEMENT)
                {
                    setSelectedItemIndex(INDEX);
                }
            }
        }

        /* (non-Javadoc)
         * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
         */
        @Override
        public void mouseExited(MouseEvent arg0)
        {
            if (selection_activator == MOUSE_MOVEMENT)
            {
                if (SELECTED == INDEX)
                {
                    setSelectedItemIndex(-1);
                }
            }
        }

        /* (non-Javadoc)
         * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
         */
        @Override
        public void mousePressed(MouseEvent evt)
        {
            if (selection_activator == MOUSE_CLICK)
            {
                if ((evt.getModifiersEx() & MouseEvent.BUTTON1_DOWN_MASK) != 0)
                {
                    setSelectedItemIndex(INDEX);
                }
            }
        }

        /* (non-Javadoc)
         * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
         */
        @Override
        public void mouseReleased(MouseEvent arg0)
        {
        }
    }

    public void sort(Comparator<InfoPanel> comp)
    {
        Collections.sort(DATA, comp);
        this.removeAll();
        InfoPanel pnl;
        
        for (int i = 0; i < DATA.size(); i++)
        {
            pnl = DATA.get(i);
            pnl.removeMouseListener(pnl.getMouseListeners()[0]);
            pnl.addMouseListener(new SelectListener(i));
            this.add(pnl);
        }
        this.invalidate();
    }

    /**
     * @return number of items in this list
     */
    public int getNumberOfItems()
    {
        return DATA.size();
    }

    /**
     * 
     * @param value either horizontal or vertival
     */
    public void setItemOrientation(InfoPanel.Orientation value)
    {
        this.orientation = value;

        for (InfoPanel pnl : DATA)
        {
            pnl.setOrientation(value);
        }
    }

    public Icon getDefaultIcon()
    {
        return DEFAULT;
    }

    @Override
    public void setFont(Font font)
    {
        Font oldFont = getFont();
        super.setFont(font);

        if (font != oldFont)
        {
            revalidate();
    	    repaint();
        }

        if (DATA != null)
        {
            for (InfoPanel pnl : DATA)
            {
                pnl.setFont(font);
            }
        }
    }
}
