/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: InfoPanel.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Last modified: 5.8.2008
 */
package ue.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.KeyListener;
import java.awt.event.MouseListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import ue.util.AniOrCur;

/**
 * A simple panel with an image and three lines of text.
 */
@SuppressWarnings("serial") //$NON-NLS-1$
public class InfoPanel extends JPanel
{
    public enum Orientation {Horizontal, Vertical }
    

    private AniLabel lblIMG;
    private JLabel lblLINE1;
    private JLabel lblLINE2;
    private JLabel lblLINE3;
    private JPanel pnl1, pnl2;
    private Orientation orientation;
    private int[] ID;
    
    public InfoPanel(Icon img, String line1, String line2, String line3, int[] id)
    {
        super();

        this.ID = id;
        orientation = Orientation.Horizontal;

        lblIMG = new AniLabel(img);

        initComponents(line1, line2, line3);
        placeComponents();
    }

    public InfoPanel(AniOrCur ani, String line1, String line2, String line3, int[] id)
    {
        super();

        this.ID = id;
        orientation = Orientation.Horizontal;

        lblIMG = new AniLabel(ani, true);

        initComponents(line1, line2, line3);
    }

    private void initComponents(String line1, String line2, String line3)
    {
        lblIMG.setBackground(Color.BLACK);
        lblIMG.setOpaque(false);
        lblIMG.setAlignmentY(InfoPanel.CENTER_ALIGNMENT);
        lblLINE1 = new JLabel(line1);
        lblLINE1.setAlignmentY(InfoPanel.CENTER_ALIGNMENT);
        lblLINE1.setOpaque(false);
        lblLINE2 = new JLabel(line2);
        lblLINE2.setAlignmentY(InfoPanel.CENTER_ALIGNMENT);
        lblLINE2.setOpaque(false);
        lblLINE3 = new JLabel(line3);
        lblLINE3.setAlignmentY(InfoPanel.CENTER_ALIGNMENT);
        lblLINE3.setOpaque(false);

        setOrientation(orientation);
    }

    public void setOrientation(InfoPanel.Orientation value)
    {
        orientation = value;

        float align = (orientation == Orientation.Horizontal)
                ? InfoPanel.LEFT_ALIGNMENT : InfoPanel.CENTER_ALIGNMENT;

        int textAlign = (orientation == Orientation.Horizontal)
                ? SwingConstants.LEFT : SwingConstants.CENTER;

        lblIMG.setAlignmentX(align);
        lblLINE1.setAlignmentX(align);
        lblLINE1.setHorizontalTextPosition(textAlign);
        lblLINE2.setAlignmentX(align);
        lblLINE2.setHorizontalTextPosition(textAlign);
        lblLINE3.setAlignmentX(align);
        lblLINE3.setHorizontalTextPosition(textAlign);

        placeComponents();
    }
    
    private void placeComponents()
    {
        this.removeAll();

        pnl1 = new JPanel();
        pnl1.setOpaque(false);
        pnl1.setLayout(new BoxLayout(pnl1, BoxLayout.Y_AXIS));
        pnl1.add(Box.createVerticalGlue());
        pnl1.add(lblLINE1);
        pnl1.add(Box.createVerticalGlue());
        pnl1.add(lblLINE2);
        pnl1.add(Box.createVerticalGlue());
        pnl1.add(lblLINE3);
        pnl1.add(Box.createVerticalGlue());

        pnl2 = new JPanel();
        pnl2.setLayout(new FlowLayout(FlowLayout.CENTER));
        pnl2.setOpaque(false);
        pnl2.add(lblIMG);

        if (orientation == Orientation.Horizontal)
        {
            this.setLayout(new FlowLayout(FlowLayout.LEFT));
            this.add(pnl2);
            this.add(pnl1);
        }
        else
        {
            this.setLayout(new GridBagLayout());
            GridBagConstraints c = new GridBagConstraints();
            c.insets = new Insets(5, 5, 5, 5);
            c.fill = GridBagConstraints.NONE;
            c.anchor = GridBagConstraints.CENTER;
            c.gridx = 0;
            c.gridy = 0;
            this.add(pnl2, c);
            c.gridy = 1;
            this.add(pnl1, c);
        }
    }

    @Override
    public void setBackground(Color background)
    {
        super.setBackground(background);
/*
        for (Component c : this.getComponents())
        {
            c.setBackground(background);
        }*/
    }

    @Override
    public void setForeground(Color foreground)
    {
        super.setForeground(foreground);
/*
        for (Component c : this.getComponents())
        {
            c.setForeground(foreground);
        }
*/
        if (lblLINE1 != null)
            lblLINE1.setForeground(foreground);
        if (lblLINE2 != null)
            lblLINE2.setForeground(foreground);
        if (lblLINE3 != null)
            lblLINE3.setForeground(foreground);
    }

    public void setLine2Foreground(Color foreground)
    {
        if (lblLINE2 != null)
            lblLINE2.setForeground(foreground);
    }

    /**
     * @return
     */
    public String getLine1()
    {
        return lblLINE1.getText();
    }

    /**
     * @return
     */
    public String getLine2()
    {
        return lblLINE2.getText();
    }
    
    /**
     * @return
     */
    public String getLine3()
    {
        return lblLINE3.getText();
    }

    /**
     * @return
     */
    public int[] getID()
    {
        return ID;
    }

    public void setTooltip(String tip)
    {
        Component[] components = this.getComponents();

        for (Component c : components)
        {
            if (c instanceof JComponent)
            {
                ((JComponent)c).setToolTipText(tip);
            }
        }
        
        this.setToolTipText(tip);

        if (lblLINE1 != null)
            lblLINE1.setToolTipText(tip);
        if (lblLINE2 != null)
            lblLINE2.setToolTipText(tip);
        if (lblLINE3 != null)
            lblLINE3.setToolTipText(tip);
        if (lblIMG != null)
            lblIMG.setToolTipText(tip);
    }

    public void setImageBackgroundOpaque(boolean opaque)
    {
        lblIMG.setOpaque(opaque);
    }

    @Override
    public void addMouseListener(MouseListener m)
    {
        super.addMouseListener(m);
        
        lblIMG.addMouseListener(m);
//        if (lblLINE1 != null)
            lblLINE1.addMouseListener(m);
//        if (lblLINE2 != null)
            lblLINE2.addMouseListener(m);
//        if (lblLINE3 != null)
          lblLINE3.addMouseListener(m);
          pnl1.addMouseListener(m);
          pnl2.addMouseListener(m);
    }

    @Override
    public void removeMouseListener(MouseListener m)
    {
        super.removeMouseListener(m);
        
        lblIMG.removeMouseListener(m);
        lblLINE1.removeMouseListener(m);
        lblLINE2.removeMouseListener(m);
        lblLINE3.removeMouseListener(m);
        pnl1.removeMouseListener(m);
        pnl2.removeMouseListener(m);
    }

    @Override
    public void addKeyListener(KeyListener m)
    {
        super.addKeyListener(m);

        pnl1.addKeyListener(m);
        pnl2.addKeyListener(m);
        lblLINE1.addKeyListener(m);
        lblLINE2.addKeyListener(m);
        lblLINE3.addKeyListener(m);
        lblIMG.addKeyListener(m);
    }

    @Override
    public void setFont(Font font)
    {
        Font oldFont = getFont();
        super.setFont(font);

        if (font != oldFont)
        {
            revalidate();
    	    repaint();
        }

        if (lblIMG != null)
        {
            lblIMG.setFont(font);
            lblLINE1.setFont(font);
            lblLINE2.setFont(font);
            lblLINE3.setFont(font);
        }
    }
}