/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: MainMenu.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;

import java.util.Date;
import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;

import ue.Language;
import ue.UE;
import ue.edit.FileManager;
import ue.edit.Special;
import ue.exception.NotAPanel;
import ue.gui.help.AboutGUI;
import ue.gui.help.HelpWindow;
import ue.util.SettingsManager;
import ue.util.StringGroup;

/**
The main menu bar.

Acts as a link between the MainWindow and the other parts of the editor.
*/
@SuppressWarnings("serial")  //$NON-NLS-1$
public class MainMenu extends JMenuBar implements ActionListener, WindowListener, PropertyChangeListener
{
    // some useful strings
    private String curSHFile = null;
    private static final String defSHFile = "blank.html"; //$NON-NLS-1$
    // commands
    private static String cmdAbout = Language.getString("MainMenu.0"); //$NON-NLS-1$
    private static String cmdClose = Language.getString("MainMenu.1"); //$NON-NLS-1$
    private static String cmdExit = Language.getString("MainMenu.2"); //$NON-NLS-1$
    private static String cmdFAQ = Language.getString("MainMenu.15"); //$NON-NLS-1$
    private static String cmdOpen = Language.getString("MainMenu.3"); //$NON-NLS-1$
    private static String cmdSave = Language.getString("MainMenu.4"); //$NON-NLS-1$
    private static String cmdSaveAs = Language.getString("MainMenu.5"); //$NON-NLS-1$
    private static String cmdRestore = Language.getString("MainMenu.16"); //$NON-NLS-1$
    private static String cmdScreen = Language.getString("MainMenu.14"); //$NON-NLS-1$
    private static String cmdSettings = Language.getString("MainMenu.6"); //$NON-NLS-1$
    
    /*MENU's*/
    private JMenu mFile = new JMenu(Language.getString("MainMenu.7")); //$NON-NLS-1$
    private JMenu mEdit = new JMenu(Language.getString("MainMenu.8")); //$NON-NLS-1$
    private JMenu mSpecial = new JMenu(Language.getString("MainMenu.9")); //$NON-NLS-1$
    private JMenu mHelp = new JMenu(Language.getString("MainMenu.10")); //$NON-NLS-1$
    //items (7)
        //FILE items (7)
        private JMenuItem fileOpen = new JMenuItem(cmdOpen);
        private JMenu fileRecentList = new JMenu(Language.getString("MainMenu.11")); //$NON-NLS-1$
        private JMenu fileRestore = new JMenu(cmdRestore);
        private JMenuItem fileSave = new JMenuItem(cmdSave);
        private JMenuItem fileSaveAs = new JMenuItem(cmdSaveAs);
        private JMenuItem fileClose = new JMenuItem(cmdClose);
        private JMenuItem fileExit = new JMenuItem(cmdExit);
        //RECENT items
        private JMenuItem[] fileRecent = new JMenuItem[10];
        //EDIT items
        private JMenuItem editSettings = new JMenuItem(cmdSettings);
        //HELP items
        private JMenuItem helpAbout = new JMenuItem(cmdAbout);
        private JMenuItem helpScreen = new JMenuItem(cmdScreen);
        private JMenuItem helpFAQ = new JMenuItem(cmdFAQ);
			
    /**Constructor.*/
    public MainMenu()
    {
        Font def = UE.SETTINGS.getDefaultBigFont();
        
        //add listener (this - used for *file* items and edit->settings and help)
        fileOpen.addActionListener(this);
        fileSave.addActionListener(this);
        fileSaveAs.addActionListener(this);
        fileClose.addActionListener(this);
        fileExit.addActionListener(this);
        editSettings.addActionListener(this);
        helpAbout.addActionListener(this);
        helpScreen.addActionListener(this);
        helpFAQ.addActionListener(this);
        
        //add items to sub menus
        refreshRecentFilesAndBackups();
        
        //add items to menus
        //file
        mFile.add(fileOpen);
        mFile.add(fileRecentList);
        mFile.addSeparator();
        mFile.add(fileSave);
        mFile.add(fileSaveAs);
        mFile.add(fileRestore);
        mFile.addSeparator();
        mFile.add(fileClose);
        mFile.addSeparator();
        mFile.add(fileExit);
        //edit
        mEdit.add(editSettings);
        //help
        mHelp.add(helpScreen);
        mHelp.add(helpFAQ);
        mHelp.add(helpAbout);
        /////////////////////////////////////
        //this gets available special commands
        StringGroup[] str = Special.getCommands();
        //ActionListener for special commands buttons
        OpenSpecial ospec = new OpenSpecial();
        //open and get options
        Arrays.sort(str);
        //add
        if (str != null)
        {
            for (int i = 0; i < str.length; i++)
            {
                JMenu tm = new JMenu(str[i].getName());
                tm.setFont(def);
                String[] strM = str[i].getStrings();
                Arrays.sort(strM);
                for (int j = 0; j < strM.length; j++)
                {
                    JMenuItem tmi = new JMenuItem(strM[j]);
                    tmi.addActionListener(ospec);
                    tmi.setFont(def);
                    tm.add(tmi);
                }
                mSpecial.add(tm);
            }
        }
        // set font
        mFile.setFont(def);
        mEdit.setFont(def);
        mSpecial.setFont(def);
        mHelp.setFont(def);
        fileOpen.setFont(def);
        fileRecentList.setFont(def);
        fileSave.setFont(def);
        fileSaveAs.setFont(def);
        fileRestore.setFont(def);
        fileClose.setFont(def);
        fileExit.setFont(def);
        editSettings.setFont(def);
        helpAbout.setFont(def);
        helpScreen.setFont(def);
        helpFAQ.setFont(def);
        //add to frame
        add(mFile);
        add(mEdit);
        add(mSpecial);
        add(mHelp);
        //setup
        fileSave.setEnabled(false);
        fileRestore.setEnabled(false);
        fileClose.setEnabled(false);
        fileSaveAs.setEnabled(false);
        // listen for language and font changes
        Language.addPropertyChangeListener(this);
        UE.SETTINGS.addPropertyChangeListener(this);
        //show
        setVisible(true);
    }
    
    /**Invoked when user clicks on buttons in the File menu or the 'Settings' button in the Edit menu.
	
	'File->Exit' calls the closeAll() function of the FileManager and saveSettings() of the SettingsManager,
	then exits the program.
	
	'File->Open...' calls the closeAll() function of the FileManager, because one file can be opened/edited at a time,
	then displays a JFileChooser where the user can choose the file to open. The file is opened using
	the FileManager's open(File file, boolean addToRecent) method which returns an array of commands.
	Those commands are then added to the 'Edit' menu.
	
	'File->Close' calls the closeAll() function of the FileManager and MainWindow's showPanel(null).
	
	'File->Save' calls FileManager's save() function.
	
	'File->Save As..' displays a JFileChooser where the user can choose the file to save to. A promt
	is displayed asking if the file should be overwriten if it already exists,
	then the FileManager's save(File file) function is called.
	
	'Edit->Settings' calls MainWindow's showPanel(MainPanel panel) function to display the SettingsPanel.
	
	'Help->About UE...' displays the about window

	'Help->Screen Help' displays panel help

	'Help->F.A.Q.' displays FAQ
     */
    @Override
    public void actionPerformed(ActionEvent e)
    {
        // font
        Font def = UE.SETTINGS.getDefaultBigFont();
        //save unsaved
        try
        {
            UE.WINDOW.finalWarning();
        }
        catch (Exception ex)
        {
            UE.displayError(ex);

            if (e.getActionCommand().equals(cmdExit))
            {
                UE.SETTINGS.saveSettings();
                System.exit(0);
            }
            
            return;
        }
        //say bye bye
        if (e.getActionCommand().equals(cmdExit))
        {
            if (!UE.FILES.closeAll())
            {
                return;
            }
            UE.SETTINGS.saveSettings();
            System.exit(0);
        }
        else
        //if open
        if (e.getActionCommand().equals(cmdOpen))
        {
            if (!UE.FILES.closeAll())
            {
                return;
            }
            fileSave.setEnabled(false);
            fileRestore.setEnabled(false);
            fileSaveAs.setEnabled(false);
            fileClose.setEnabled(false);
            UE.WINDOW.showPanel(null);
            JFileChooser jfc = new JFileChooser(new File(UE.SETTINGS.getProperty(SettingsManager.LAST)));
            javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter()
            {
                @Override
                public boolean accept(File f)
                {
                    if (f.isDirectory())
                    {
                        return true;
                    }
                    String name = f.getName();
                    name = name.toLowerCase();
                    if (name.endsWith(".res") || name.endsWith(".sav") //$NON-NLS-1$ //$NON-NLS-2$
                            || name.endsWith(".snd") || name.endsWith(".exe")) { //$NON-NLS-1$ //$NON-NLS-2$
                        return true; //$NON-NLS-1$
                    }
                    else
                    {
                        return false;
                    }
                }
                @Override
                public String getDescription()
                {
                    return "*.res, *.sav, *.snd, *.exe"; //$NON-NLS-1$
                }
            };
            jfc.setFileFilter(filter);
            int returnVal = jfc.showOpenDialog(this);
            if(returnVal == JFileChooser.APPROVE_OPTION)
            {
                //open and get options
                StringGroup[] editOptions = UE.FILES.open(jfc.getSelectedFile());
                mEdit.removeAll();
                mEdit.add(editSettings);
                //add
                if (editOptions != null)
                {
                    Arrays.sort(editOptions);
                    fileSave.setEnabled(true);
                    fileRestore.setEnabled(true);
                    fileSaveAs.setEnabled(true);
                    fileClose.setEnabled(true);
                    mEdit.addSeparator();
                    ActionListener act = new OpenEdit();

                    for (int i = 0; i < editOptions.length; i++)
                    {
                        JMenu tm = new JMenu(editOptions[i].getName());
                        tm.setFont(def);
                        String[] strM = editOptions[i].getStrings();
                        Arrays.sort(strM);
                        for (int j = 0; j < strM.length; j++)
                        {
                            JMenuItem tmi = new JMenuItem(strM[j]);
                            tmi.addActionListener(act);
                            tmi.setFont(def);
                            tm.add(tmi);
                        }
                        mEdit.add(tm);
                    }
                }
                //refresh recent
                refreshRecentFilesAndBackups();
            }
            curSHFile = null;
            helpScreen.setEnabled(true);
            
            // this resets the title - in case something was already open beforehand
            UE.WINDOW.setTitle(UE.APP_NAME);
        }
        else
        //if close
        if (e.getActionCommand().equals(cmdClose))
        {
            if (!UE.FILES.closeAll())
            {
                return;
            }
            UE.WINDOW.showPanel(null);
            mEdit.removeAll();
            mEdit.add(editSettings);
            fileSave.setEnabled(false);
            fileRestore.setEnabled(false);
            fileSaveAs.setEnabled(false);
            refreshRecentFilesAndBackups();
            fileClose.setEnabled(false);
            curSHFile = null;
            helpScreen.setEnabled(true);
        }
        else
        //if save
        if (e.getActionCommand().equals(cmdSave))
        {
            UE.FILES.save(null, true);
            refreshRecentFilesAndBackups();
        }
        else
        //if save as
        if (e.getActionCommand().equals(cmdSaveAs))
        {
            JFileChooser jfc = new JFileChooser(new File(UE.SETTINGS.getProperty(SettingsManager.LAST)));
            int returnVal = jfc.showSaveDialog(this);
            if(returnVal == JFileChooser.APPROVE_OPTION)
            {
                File filo = jfc.getSelectedFile();
                if (filo.exists())
                {
                    String msg = Language.getString("MainMenu.12"); //$NON-NLS-1$
                    msg = msg.replace("%1", filo.getName()); //$NON-NLS-1$
                    int respon = JOptionPane.showConfirmDialog(UE.WINDOW, msg, 
                            UE.APP_NAME, JOptionPane.YES_NO_OPTION,
                            JOptionPane.QUESTION_MESSAGE);
                    if (respon==JOptionPane.YES_OPTION)
                    {
                        UE.FILES.save(filo, false);
                    }
                }
                else
                {
                    UE.FILES.save(filo, false);
                }
            }
        }
        else
        //if settings
        if (e.getActionCommand().equals(cmdSettings))
        {
            MainPanel pnl = new SettingsPanel();
            UE.WINDOW.showPanel(pnl);
            UE.WINDOW.setTitle(UE.APP_NAME + " - " + cmdSettings); //$NON-NLS-1$
            curSHFile = null;
            helpScreen.setEnabled(curSHFile != null);
        }
        else
        //if about
        if (e.getActionCommand().equals(cmdAbout))
        {
            new AboutGUI();
        }
        else
        // if screen help
        if (e.getActionCommand().equals(cmdScreen))
        {
            if (curSHFile != null)
            {
                new HelpWindow(Language.getString("MainMenu.10"), FileManager.SHPath, curSHFile); //$NON-NLS-1$
            }
            else
            {
                new HelpWindow(Language.getString("MainMenu.10"), FileManager.SHPath, defSHFile); //$NON-NLS-1$
            }
        }
        else
        // if faq
        if (e.getActionCommand().equals(cmdFAQ))
        {
            new HelpWindow(Language.getString("MainMenu.10"), FileManager.FAQPath, "faq.html"); //$NON-NLS-1$ //$NON-NLS-2$
        }
    }
    
    /**
        Refreshes the recent files list in the File menu.
	
	Uses SettingsManager's getRecentFiles() function to get the list of recent files.
	Should the list be empty, a disabled JMenuItem("None") will be added to the menu.
     */
    public void refreshRecentFilesAndBackups()
    {
        // font
        Font def = UE.SETTINGS.getDefaultBigFont();
        //trash old ones
        fileRecentList.removeAll();
        fileRestore.removeAll();

        //get new ones
        String[] file = UE.SETTINGS.getRecentFiles();
        if (file == null)
        {
            fileRecent[0] = new JMenuItem(Language.getString("MainMenu.13")); //$NON-NLS-1$
            fileRecent[0].setFont(def);
            fileRecent[0].setEnabled(false);
            fileRecentList.add(fileRecent[0]);
        }
        else
        {
            OpenRecent ore = new OpenRecent();
            for (int i = 0; i < file.length; i++)
            {
                fileRecent[i] = new JMenuItem(file[i]);
                fileRecent[i].setFont(def);
                fileRecent[i].addActionListener(ore);
                fileRecentList.add(fileRecent[i]);
            }
        }
        
        ArrayList<Date> bkps = UE.FILES.getBackupDates();

        JMenuItem item;
        if (bkps.isEmpty())
        {
            item = new JMenuItem(Language.getString("MainMenu.13")); //$NON-NLS-1$
            item.setFont(def);
            item.setEnabled(false);
            fileRestore.add(item);
        }
        else
        {
            RestoreBackups ore = new RestoreBackups();
            for (int i = bkps.size() - 1; i >= 0; i--)
            {
                item = new JMenuItem(FileManager.simple_date_format.format(bkps.get(i)));
                item.setFont(def);
                item.addActionListener(ore);
                fileRestore.add(item);
            }
        }
        
        validate();
    }
    
    /**Leftover - does nothing.*/
    @Override
    public void windowActivated(WindowEvent e){}
    /**Leftover - does nothing.*/
    @Override
    public void windowClosed(WindowEvent e) {}
    /**
	Used to close the window and exit program. Envoked when x is clicked on window.
	Actualy calls actionPerformed(ActionEvent e) function in this class with 'Exit' as the command.
     */
    @Override
    public void windowClosing(WindowEvent e)
    {
        actionPerformed(new ActionEvent(this, 1, cmdExit));
    }
    /**Leftover - does nothing.*/
    @Override
    public void windowDeactivated(WindowEvent e) {}
    /**Leftover - does nothing.*/
    @Override
    public void windowDeiconified(WindowEvent e) {}
    /**Leftover - does nothing.*/
    @Override
    public void windowIconified(WindowEvent e) {}
    /**Leftover - does nothing.*/
    @Override
    public void windowOpened(WindowEvent e) {}
    
    /**Used with the recent files commands.*/
    private class OpenRecent implements ActionListener
    {
        @Override
        public void actionPerformed(ActionEvent e)
        {
            // font
            Font def = UE.SETTINGS.getDefaultBigFont();
            //save unsaved
            UE.WINDOW.showPanel(null);
            if (!UE.FILES.closeAll())
            {
                return;
            }
            fileSave.setEnabled(false);
            fileRestore.setEnabled(false);
            fileSaveAs.setEnabled(false);
            fileClose.setEnabled(false);
            //open and get options
            StringGroup[] editOptions = UE.FILES.open(new File(e.getActionCommand()));
            if (editOptions == null)
            {
                return;
            }
            Arrays.sort(editOptions);
            mEdit.removeAll();
            //add
            mEdit.add(editSettings);
            if (editOptions != null)
            {
                fileSave.setEnabled(true);
                fileRestore.setEnabled(true);
                fileSaveAs.setEnabled(true);
                fileClose.setEnabled(true);
                mEdit.addSeparator();
                ActionListener act = new OpenEdit();
                //JMenu[] others = new JMenu[editOptions.length];
                for (int i = 0; i < editOptions.length; i++)
                {
                    JMenu tm = new JMenu(editOptions[i].getName());
                    tm.setFont(def);
                    String[] strM = editOptions[i].getStrings();
                    Arrays.sort(strM);
                    for (int j = 0; j < strM.length; j++)
                    {
                        JMenuItem tmi = new JMenuItem(strM[j]);
                        tmi.addActionListener(act);
                        tmi.setFont(def);
                        tm.add(tmi);
                    }
                    mEdit.add(tm);
                }
            }
            refreshRecentFilesAndBackups();
            curSHFile = null;
            helpScreen.setEnabled(true);
            
            UE.WINDOW.setTitle(UE.APP_NAME);
        }
    }
    
    /**Used with the edit commands*/
    private class OpenEdit implements ActionListener
    {
        @Override
        public void actionPerformed(ActionEvent e)
        {
            //save unsaved
            UE.WINDOW.showPanel(null);
            try
            {
                MainPanel pnl = UE.FILES.edit(e.getActionCommand());
                UE.WINDOW.showPanel(pnl);
                curSHFile = pnl.getHelpFileName();
                helpScreen.setEnabled(curSHFile != null);
                UE.WINDOW.setTitle(UE.APP_NAME + " - " + e.getActionCommand()); //$NON-NLS-1$
            }
            catch (NotAPanel p)
            {
            }
            catch (Exception p)
            {
                UE.displayError(p);
            }
        }
    }
    
    /**Used with the special commands - those under Utils*/
    private class OpenSpecial implements ActionListener
    {
        @Override
        public void actionPerformed(ActionEvent e)
        {
            MainPanel pnl = Special.edit(e.getActionCommand());
            UE.WINDOW.showPanel(pnl);
            curSHFile = pnl.getHelpFileName();
            helpScreen.setEnabled(curSHFile != null);
            UE.WINDOW.setTitle(UE.APP_NAME + " - " + e.getActionCommand()); //$NON-NLS-1$
        }
    }
    
    private class RestoreBackups implements ActionListener
    {
        @Override
        public void actionPerformed(ActionEvent e)
        {
            // font
            Font def = UE.SETTINGS.getDefaultBigFont();
            // close
            UE.WINDOW.showPanel(null);
            mEdit.removeAll();
            mEdit.add(editSettings);
            fileSave.setEnabled(false);
            fileRestore.setEnabled(false);
            fileSaveAs.setEnabled(false);
            fileClose.setEnabled(false);
            curSHFile = null;
            helpScreen.setEnabled(true);
            // restore
            UE.FILES.restoreBackups(e.getActionCommand());
            // open
            StringGroup[] editOptions = UE.FILES.getCommands();
            mEdit.removeAll();
            mEdit.add(editSettings);
            //add
            if (editOptions != null)
            {
                Arrays.sort(editOptions);
                fileSave.setEnabled(true);
                fileRestore.setEnabled(true);
                fileSaveAs.setEnabled(true);
                fileClose.setEnabled(true);
                mEdit.addSeparator();
                ActionListener act = new OpenEdit();
                
                for (int i = 0; i < editOptions.length; i++)
                {
                    JMenu tm = new JMenu(editOptions[i].getName());
                    tm.setFont(def);
                    String[] strM = editOptions[i].getStrings();
                    Arrays.sort(strM);
                    for (int j = 0; j < strM.length; j++)
                    {
                        JMenuItem tmi = new JMenuItem(strM[j]);
                        tmi.addActionListener(act);
                        tmi.setFont(def);
                        tm.add(tmi);
                    }
                    mEdit.add(tm);
                }
            }
            
            refreshRecentFilesAndBackups();
        }
    }

    // call this when language changes and the menu needs updating
    private void resetMenu()
    {
        // font
        Font def = UE.SETTINGS.getDefaultBigFont();
        // menus
        mFile.setText(Language.getString("MainMenu.7")); //$NON-NLS-1$
        mEdit.setText(Language.getString("MainMenu.8")); //$NON-NLS-1$
        mSpecial.setText(Language.getString("MainMenu.9")); //$NON-NLS-1$
        mHelp.setText(Language.getString("MainMenu.10")); //$NON-NLS-1$
        // commands
        cmdAbout = Language.getString("MainMenu.0"); //$NON-NLS-1$
        cmdClose = Language.getString("MainMenu.1"); //$NON-NLS-1$
        cmdExit = Language.getString("MainMenu.2"); //$NON-NLS-1$
        cmdOpen = Language.getString("MainMenu.3"); //$NON-NLS-1$
        cmdSave = Language.getString("MainMenu.4"); //$NON-NLS-1$
        cmdSaveAs = Language.getString("MainMenu.5"); //$NON-NLS-1$
        cmdRestore = Language.getString("MainMenu.16"); //$NON-NLS-1$
        cmdSettings = Language.getString("MainMenu.6"); //$NON-NLS-1$
        cmdFAQ = Language.getString("MainMenu.15"); //$NON-NLS-1$
        cmdScreen = Language.getString("MainMenu.14"); //$NON-NLS-1$
        // items
        fileOpen.setText(cmdOpen);
        fileRecentList.setText(Language.getString("MainMenu.11")); //$NON-NLS-1$
        fileSave.setText(cmdSave);
        fileSaveAs.setText(cmdSaveAs);
        fileRestore.setText(cmdRestore);
        fileClose.setText(cmdClose);
        fileExit.setText(cmdExit);
        editSettings.setText(cmdSettings);
        helpAbout.setText(cmdAbout);
        helpScreen.setText(cmdScreen);
        helpFAQ.setText(cmdFAQ);
        // update commands
        StringGroup[] editOptions = UE.FILES.getCommands();
        mEdit.removeAll();
        //add
        mEdit.add(editSettings);
        if (editOptions != null)
        {
            Arrays.sort(editOptions);
            fileSave.setEnabled(true);
            fileRestore.setEnabled(true);
            fileSaveAs.setEnabled(true);
            fileClose.setEnabled(true);
            mEdit.addSeparator();
            ActionListener act = new OpenEdit();
            //JMenu[] others = new JMenu[editOptions.length];
            for (int i = 0; i < editOptions.length; i++)
            {
                JMenu tm = new JMenu(editOptions[i].getName());
                tm.setFont(def);
                String[] strM = editOptions[i].getStrings();
                Arrays.sort(strM);
                for (int j = 0; j < strM.length; j++)
                {
                    JMenuItem tmi = new JMenuItem(strM[j]);
                    tmi.addActionListener(act);
                    tmi.setFont(def);
                    tm.add(tmi);
                }
                mEdit.add(tm);
            }
        }
        // in case there are no recent files
        refreshRecentFilesAndBackups();
        /////////////////////////////////////
        //this gets available special commands
        mSpecial.removeAll();
        StringGroup[] str = Special.getCommands();
        //ActionListener for special commands buttons
        OpenSpecial ospec = new OpenSpecial();
        //open and get options
        Arrays.sort(str);
        //add
        if (str != null)
        {
            for (int i = 0; i < str.length; i++)
            {
                JMenu tm = new JMenu(str[i].getName());
                tm.setFont(def);
                String[] strM = str[i].getStrings();
                Arrays.sort(strM);
                for (int j = 0; j < strM.length; j++)
                {
                    JMenuItem tmi = new JMenuItem(strM[j]);
                    tmi.addActionListener(ospec);
                    tmi.setFont(def);
                    tm.add(tmi);
                }
                mSpecial.add(tm);
            }
        }
        // set font
        mFile.setFont(def);
        mEdit.setFont(def);
        mSpecial.setFont(def);
        mHelp.setFont(def);
        fileOpen.setFont(def);
        fileRecentList.setFont(def);
        fileSave.setFont(def);
        fileSaveAs.setFont(def);
        fileRestore.setFont(def);
        fileClose.setFont(def);
        fileExit.setFont(def);
        editSettings.setFont(def);
        helpAbout.setFont(def);		
        helpScreen.setFont(def);		
        helpFAQ.setFont(def);		
        // window title
        UE.WINDOW.setTitle(UE.APP_NAME + " - " + cmdSettings); //$NON-NLS-1$
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent e)
    {
        if (e.getSource().equals(Language.class))
        {
            resetMenu();
        }
        else if (e.getSource().equals(UE.SETTINGS) && e.getPropertyName().equals(SettingsManager.FONT_NAME))
        {
            resetMenu();
        }
    }	
}
