/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: MainWindow.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;

import javax.swing.JFrame;
import javax.swing.WindowConstants;

import ue.UE;

/**
The main window.

This is a subclass of JFrame. It shows demanded panels and contains a menu on top.
**/
@SuppressWarnings("serial") //$NON-NLS-1$
public class MainWindow extends JFrame
{
    private static MainPanel CURRENT_PANEL = null;
    
    /**
     * Constructa an instance of MainWindow.
     * 
     * Title is "Ultimate Editor"
     * Default size is 600x480 pixels.
     * Loads in the midle of the screen.
     */
    public MainWindow()
    {
        /*SETUP*/
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        setIconImage(UE.ICON);
        setTitle(UE.APP_NAME);
        MainMenu meni = new MainMenu();
        setJMenuBar(meni);
        setSize(600, 480);
        setMinimumSize(new Dimension(300, 60));
        setLayout(new BorderLayout());
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation(new Point((int)(screenSize.getWidth()/2-300), (int)(screenSize.getHeight()/2-240)));
        addWindowListener(meni);
        setVisible(true);
    }
    
    /**
     * Displays the provided panel in window.
     * 
     * Calls the currently showed panel's finalWarning() function if any,
     * then removes it	and adds the specified one.
     * If the new panel is larger than window space,
     * the window sets it's size to provide enough space.
     * Panels size is read with the getPreferredSize() function.
     * 
     * @param panel The MainPanel to display. If the parameter is null, only the current panel is removed.
     */
    public void showPanel(MainPanel panel)
    {
        // warn current panel that it's being replaced
        if (CURRENT_PANEL != null)
        {
            try
            {
                CURRENT_PANEL.finalWarning();
            }
            catch (Exception ex)
            {
                UE.displayError(ex);
                return;
            }
            
            remove(CURRENT_PANEL);
        }
        
        if (panel != null)
        {
            Dimension pan = panel.getPreferredSize();
            Dimension win = getSize();
            double w;
            double h;
            
            if (pan.getWidth() > this.getContentPane().getWidth())
            {
                w = pan.getWidth() + this.getContentPane().getWidth() - pan.getWidth();
            }
            else
            {
                w = win.getWidth();
            }
            
            if (pan.getHeight() > this.getContentPane().getHeight())
            {
                h = win.getHeight() + this.getContentPane().getHeight() - pan.getHeight();
            }
            else
            {
                h = win.getHeight();
            }
            
            setMinimumSize(new Dimension((int)pan.getWidth() + getWidth() - this.getContentPane().getWidth(), (int)pan.getHeight() + getHeight() - this.getContentPane().getHeight()));
            win.setSize(w, h);
            setSize(win);
            add(panel, BorderLayout.CENTER);
        }
        else
        {
            setMinimumSize(new Dimension(300, 60));
            //System.out.println("Main window received >null< for panel.");
        }

        if ((this.getExtendedState() & MAXIMIZED_BOTH) != MAXIMIZED_BOTH)
        {
            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

            if (this.getX() + this.getWidth() > screenSize.width
                    || this.getY() + this.getHeight() > screenSize.height)
            {
                setLocation(new Point(
                        (int)((screenSize.getWidth() - getWidth())/2),
                        (int)((screenSize.getHeight() - getHeight())/2)));
            }
        }

        CURRENT_PANEL = panel;
        repaint();
        validate();
    }
    
    /**
     * Calls the currently showed panel's finalWarning() function if any.
     */
    public void finalWarning() throws Exception
    {
        if (CURRENT_PANEL != null)
        {
            CURRENT_PANEL.finalWarning();
        }
    }
}

