/*
Ultimate Editor for Birth Of The Federation 1.02
Copyright (C) 2005-2007  Alan Podlesek

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


File: ModifiedFilesGUI.java

Written by: Alan Podlesek <seqdcer@yahoo.com>

Last modified: 6.9.2007
 */
package ue.gui;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import ue.Language;
import ue.UE;
import ue.edit.CanBeEdited;
import ue.edit.InternalFile;
import ue.edit.mod.Mod;
import ue.edit.mod.ModList;
import ue.edit.res.ResFile;
import ue.edit.res.Stbof;
import ue.edit.snd.SndFile;
import ue.util.SettingsManager;
import ue.util.Tools;
import ue.util.UEWorker;

/**This class is used to display the about window.*/
@SuppressWarnings("serial")  //$NON-NLS-1$
public class ModifiedFilesGUI extends JFrame implements WindowListener {

    private ModifiedFilesGUI ME;
    private JList lstFILE;
    private JButton btnADD;
    private JButton btnREM;
    private JButton btnEXT_ALL;
    private JButton btnEXT;
    private JButton btnEXP_ALL;
    private JComboBox cmbFILE;
    private CanBeEdited[] FILES;

    public ModifiedFilesGUI(CanBeEdited selected) {
        FILES = UE.FILES.openAllGameArchives();

        setup();
        init(selected);
    }

    private void setup() {
        ME = this;
        addWindowListener(this);
        /*CREATE ITEMS*/
        lstFILE = new JList();
        btnADD = new JButton(Language.getString("ModifiedFilesGUI.0")); //$NON-NLS-1$
        btnREM = new JButton(Language.getString("ModifiedFilesGUI.1")); //$NON-NLS-1$
        btnEXT_ALL = new JButton(Language.getString("ModifiedFilesGUI.2")); //$NON-NLS-1$
        btnEXT = new JButton(Language.getString("ModifiedFilesGUI.17")); //$NON-NLS-1$
        btnEXP_ALL = new JButton(Language.getString("ModifiedFilesGUI.18")); //$NON-NLS-1$
        cmbFILE = new JComboBox();
        /*SETUP ITEMS*/
        btnEXT.setEnabled(false);
        btnEXT_ALL.setEnabled(false);
        btnREM.setEnabled(false);
        Font def = UE.SETTINGS.getDefaultFont();

        btnEXT_ALL.setFont(def);
        btnEXP_ALL.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent ae) {
//             get destination
                JFileChooser fc = new JFileChooser();
                fc.setCurrentDirectory(new File(UE.SETTINGS.getProperty(SettingsManager.WORK_PATH)));
                fc.setDialogTitle(UE.APP_NAME + " - " //$NON-NLS-1$
                        + Language.getString("ModifiedFilesGUI.19")); //$NON-NLS-1$
                fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                fc.setMultiSelectionEnabled(false);
                int returnVal = fc.showDialog(ME, Language.getString("ModifiedFilesGUI.20")); //$NON-NLS-1$

                if (returnVal != JFileChooser.APPROVE_OPTION) {
                    return;
                }

                File dest = fc.getSelectedFile();

                // destination chosen
                if (dest != null) {
                    UE.SETTINGS.setProperty(SettingsManager.WORK_PATH, dest.getPath());

                    for (CanBeEdited cbe : FILES) {
                        if (!cbe.hasBeenModified()) {
                            continue;
                        }

                        try {
                            if (cbe instanceof Stbof) {
                                Vector<String> files = new Vector<String>();
                                Mod[] mods = new Mod[0];

                                mods = cbe.getKnownChanges().getAll();

                                //fill
                                String str;
                                for (int i = 0; i < mods.length; i++) {
                                    Iterator<String> it = mods[i].iterator();
                                    while (it.hasNext()) {
                                        str = it.next();
                                        if (!files.contains(str)) {
                                            files.add(str);
                                        }
                                    }
                                }

                                File f = new File(dest, cbe.toFile().getName());

                                if (f.exists() && f.isFile()) {
                                    f = new File(dest, cbe.getClass().getSimpleName());
                                }

                                if (!f.exists()) {
                                    f.mkdirs();
                                }

                                String[] inFile = files.toArray(new String[0]);
                                extract(cbe, inFile, f);
                            } else {
                                String name = cbe.toFile().getName();
                                File f = new File(dest, name.substring(name.lastIndexOf(".") + 1));

                                if (!f.exists()) {
                                    f.mkdirs();
                                }

                                copy(cbe, f);
                            }
                        } catch (Exception e) {
                            UE.displayError(e);
                        }
                    }
                }
            }
        });

        cmbFILE.setFont(def);
        cmbFILE.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                int sel = cmbFILE.getSelectedIndex();
                if (sel < 0) {
                    return;
                }

                btnADD.setEnabled(FILES[sel] instanceof SndFile || FILES[sel] instanceof ResFile);

                fillList();

                btnEXT_ALL.setEnabled(lstFILE.isEnabled());
            }
        });

        lstFILE.setFont(def);
        lstFILE.addListSelectionListener(new ListSelectionListener() {

            public void valueChanged(ListSelectionEvent arg0) {
                btnREM.setEnabled(!lstFILE.isSelectionEmpty());
                btnEXT.setEnabled(btnREM.isEnabled());
            }
        });
        // Add file to the list
        btnADD.setFont(def);
        btnADD.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent ae) {
                if (cmbFILE.getSelectedIndex() < 0) {
                    return;
                }

                String n = JOptionPane.showInputDialog(Language.getString("ModifiedFilesGUI.3"), //$NON-NLS-1$
                        Language.getString("ModifiedFilesGUI.4")); //$NON-NLS-1$

                if (n != null && n.length() > 0) {
                    if (FILES[cmbFILE.getSelectedIndex()] instanceof ResFile) {
                        ((ResFile) FILES[cmbFILE.getSelectedIndex()]).MODS.add(n);
                    } else if (FILES[cmbFILE.getSelectedIndex()] instanceof SndFile) {
                        ((SndFile) FILES[cmbFILE.getSelectedIndex()]).MODS.add(n);
                    }
                    FILES[cmbFILE.getSelectedIndex()].markChanged();
                    fillList();
                }
            }
        });
        // Remove from list
        btnREM.setFont(def);
        btnREM.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent ae) {
                if (lstFILE.isSelectionEmpty()) {
                    return;
                }
                Object[] obj = lstFILE.getSelectedValues();
                for (int i = 0; i < obj.length; i++) {
                    if (FILES[cmbFILE.getSelectedIndex()] instanceof ResFile) {
                        ((ResFile) FILES[cmbFILE.getSelectedIndex()]).MODS.remove(obj.toString());
                    } else if (FILES[cmbFILE.getSelectedIndex()] instanceof SndFile) {
                        ((SndFile) FILES[cmbFILE.getSelectedIndex()]).MODS.remove(obj.toString());
                    }
                }
                FILES[cmbFILE.getSelectedIndex()].markChanged();
                fillList();
            }
        });
        // Extract all modified files
        btnEXT_ALL.setFont(def);
        btnEXT_ALL.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent ae) {
                // get destination
                JFileChooser fc = new JFileChooser();
                fc.setCurrentDirectory(new File(UE.SETTINGS.getProperty(SettingsManager.WORK_PATH)));
                fc.setDialogTitle(UE.APP_NAME + " - " //$NON-NLS-1$
                        + Language.getString("ModifiedFilesGUI.5")); //$NON-NLS-1$
                fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                fc.setMultiSelectionEnabled(false);
                int returnVal = fc.showDialog(ME, Language.getString("ModifiedFilesGUI.6")); //$NON-NLS-1$
                if (returnVal != JFileChooser.APPROVE_OPTION) {
                    return;
                }
                File fil = fc.getSelectedFile();
                // destination chosen
                if (fil != null) {
                    UE.SETTINGS.setProperty(SettingsManager.WORK_PATH, fil.getPath());
                    // go thru all modified files
                    Vector<String> files = new Vector<String>();
                    Mod[] mods = new Mod[0];

                    if (FILES[cmbFILE.getSelectedIndex()] instanceof ResFile) {
                        mods = ((ResFile) FILES[cmbFILE.getSelectedIndex()]).MODS.getAll();
                    } else if (FILES[cmbFILE.getSelectedIndex()] instanceof SndFile) {
                        mods = ((SndFile) FILES[cmbFILE.getSelectedIndex()]).MODS.getAll();
                    }

                    //fill
                    String str;
                    for (int i = 0; i < mods.length; i++) {
                        Iterator<String> it = mods[i].iterator();
                        while (it.hasNext()) {
                            str = it.next();
                            if (!files.contains(str)) {
                                files.add(str);
                            }
                        }
                    }

                    try {
                        String[] inFile = files.toArray(new String[0]);
                        extract(FILES[cmbFILE.getSelectedIndex()], inFile, fil);
                    } catch (Exception e) {
                        UE.displayError(e);
                    }
                }
            }
        });
        // Extract selected files
        btnEXT.setFont(def);
        btnEXT.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent ae) {
                if (lstFILE.isSelectionEmpty()) {
                    return;
                }

                // get destination
                JFileChooser fc = new JFileChooser();
                fc.setCurrentDirectory(new File(UE.SETTINGS.getProperty(SettingsManager.WORK_PATH)));
                fc.setDialogTitle(UE.APP_NAME + " - " //$NON-NLS-1$
                        + Language.getString("ModifiedFilesGUI.5")); //$NON-NLS-1$
                fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                fc.setMultiSelectionEnabled(false);
                int returnVal = fc.showDialog(ME, Language.getString("ModifiedFilesGUI.6")); //$NON-NLS-1$
                if (returnVal != JFileChooser.APPROVE_OPTION) {
                    return;
                }
                File fil = fc.getSelectedFile();

                if (fil != null) {
                    UE.SETTINGS.setProperty(SettingsManager.WORK_PATH, fil.getPath());
                    String str;
                    Object[] sel_val = lstFILE.getSelectedValues();
                    Vector<String> files = new Vector<String>();

                    for (int i = 0; i < sel_val.length; i++) {
                        str = (String) sel_val[i];
                        if (!files.contains(str)) {
                            files.add(str);
                        }
                    }
                    try {
                        String[] inFile = files.toArray(new String[0]);
                        extract(FILES[cmbFILE.getSelectedIndex()], inFile, fil);
                    } catch (Exception e) {
                        UE.displayError(e);
                    }
                }
            }
        });
        /*ADD ITEMS TO FRAME*/
        //general layout settings
        setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.insets = new Insets(5, 5, 5, 5);
        c.fill = GridBagConstraints.BOTH;
        //adding items
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        add(cmbFILE, c);
        c.gridx = 1;
        add(btnEXP_ALL, c);
        c.gridx = 0;
        c.gridy = 1;
        c.gridheight = 5;
        JScrollPane scr = new JScrollPane(lstFILE);
        scr.setPreferredSize(new Dimension(200, 200));
        add(scr, c);
        c.gridheight = 1;
        c.gridx = 1;
        add(btnADD, c);
        c.gridy = 2;
        add(btnREM, c);
        c.gridy = 3;
        add(btnEXT, c);
        c.gridy = 4;
        add(btnEXT_ALL, c);
        /*FRAME SETUP*/
        //icon is the same as main
        setIconImage(UE.ICON);
        setTitle(Language.getString("ModifiedFilesGUI.7")); //$NON-NLS-1$
        pack();
        setResizable(false);	//this frame is not resizable
        //place at the center of screen
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation(new Point((int) (screenSize.getWidth() / 2 - 125), (int) (screenSize.getHeight() / 2 - 125)));
        setVisible(true);
    }

    private void fillList() {
        Vector<String> files = new Vector<String>();
        int sel = cmbFILE.getSelectedIndex();

        if (sel >= 0) {
            if (FILES[sel].hasBeenModified()) {
                ModList ml = FILES[sel].getKnownChanges();

                if (ml == null) {
                    Font def = UE.SETTINGS.getDefaultFont();
                    FontMetrics metrics = cmbFILE.getFontMetrics(def);

                    String msg = Language.getString("ModifiedFilesGUI.21"); //$NON-NLS-1$
                    int j;

                    while (msg.length() != 0) {
                        String w = msg;
                        while (metrics.stringWidth(w) > 175) {
                            j = w.lastIndexOf(" "); //$NON-NLS-1$

                            if (j <= 0) {
                                w = w.substring(0, w.length() - 1);
                            } else {
                                w = w.substring(0, j);
                            }
                        }
                        files.add(w.trim());
                        msg = msg.substring(w.length());
                        msg.trim();
                    }

                    lstFILE.setEnabled(false);
                } else {
                    Mod[] mods = ml.getAll();

                    //refill
                    String str;
                    for (int i = 0; i < mods.length; i++) {
                        Iterator<String> it = mods[i].iterator();
                        while (it.hasNext()) {
                            str = it.next();
                            if (!files.contains(str)) {
                                files.add(str);
                            }
                        }
                    }

                    Collections.sort(files);
                    lstFILE.setEnabled(true);
                }
            } else {
                Font def = UE.SETTINGS.getDefaultFont();
                FontMetrics metrics = cmbFILE.getFontMetrics(def);

                String msg = Language.getString("ModifiedFilesGUI.22"); //$NON-NLS-1$
                int j;

                while (msg.length() != 0) {
                    String w = msg;
                    while (metrics.stringWidth(w) > 175) {
                        j = w.lastIndexOf(" "); //$NON-NLS-1$

                        if (j <= 0) {
                            w = w.substring(0, w.length() - 1);
                        } else {
                            w = w.substring(0, j);
                        }
                    }
                    files.add(w.trim());
                    msg = msg.substring(w.length());
                    msg.trim();
                }

                lstFILE.setEnabled(false);
            }
        }

        lstFILE.setListData(files);
    }

    private void init(CanBeEdited selected) {
        String str;
        int j;
        int sel = 0;

        int max = 175;
        Font def = UE.SETTINGS.getDefaultFont();
        FontMetrics metrics = cmbFILE.getFontMetrics(def);

        for (int i = 0; i < FILES.length; i++) {
            if (FILES[i].equals(selected)) {
                sel = i;
            }
            str = FILES[i].toFile().getAbsolutePath();

            while (metrics.stringWidth(str) > max) {
                if (str.startsWith("...")) //$NON-NLS-1$
                {
                    str = str.substring(4);
                }

                j = str.indexOf("/"); //$NON-NLS-1$

                if (j < 0) {
                    j = str.indexOf("\\"); //$NON-NLS-1$
                }

                if (j <= 0) {
                    str = "..." + str.substring(1); //$NON-NLS-1$
                } else {
                    str = "..." + str.substring(j); //$NON-NLS-1$
                }
            }
            cmbFILE.addItem(str);
        }

        cmbFILE.setSelectedIndex(sel);
    }

    /**Extracts files.
     * @param edited
     * @param inFile the name of the file to extract.
     * @param to the destination where to extract the file to.
     */
    public void extract(CanBeEdited edited, String[] inFile, File to) throws Exception {
        File file;
        ArrayList<String> files = new ArrayList<String>();
        boolean overwrite_all = false;
        boolean skip_all = false;

        for (String name : inFile) {
            if (edited instanceof ResFile) {
                file = new File(to, name);
            } else {
                file = new File(to, name + ".wav"); //$NON-NLS-1$
            }

            if (file.isFile() && file.exists()) {
                if (overwrite_all) {
                    file.delete();
                    files.add(name);
                } else if (skip_all) {
                    continue;
                } else {
                    String msg = Language.getString("ModifiedFilesGUI.10"); //$NON-NLS-1$

                    int respon = JOptionPane.showOptionDialog(
                            UE.WINDOW,
                            msg.replace("%1", file.getName()), //$NON-NLS-1$
                            UE.APP_NAME,
                            JOptionPane.NO_OPTION,
                            JOptionPane.QUESTION_MESSAGE,
                            null,
                            new String[]{Language.getString("ModifiedFilesGUI.11"), // yes //$NON-NLS-1$
                                Language.getString("ModifiedFilesGUI.12"), // yes to all //$NON-NLS-1$
                                Language.getString("ModifiedFilesGUI.13"), // no //$NON-NLS-1$
                                Language.getString("ModifiedFilesGUI.14"), // No to all //$NON-NLS-1$
                            },
                            Language.getString("ModifiedFilesGUI.13") // no //$NON-NLS-1$
                            );

                    switch (respon) {
                        case 0: {
                            file.delete();
                            files.add(name);
                            break;
                        }
                        case 1: {
                            file.delete();
                            files.add(name);
                            overwrite_all = true;
                            break;
                        }
                        case 2: {
                            continue;
                        }
                        case 3: {
                            skip_all = true;
                        }
                    }
                    ;
                }
            } else {
                files.add(name);
            }
        }

        ExtractTask task = new ExtractTask(edited, to, files.toArray(new String[0]));

        Tools.runUEWorker(task);
    }

    private void copy(CanBeEdited cbe, File f) {
        File dest = new File(f, cbe.toFile().getName());

        if (dest.isFile() && dest.exists()) {
            String msg = Language.getString("ModifiedFilesGUI.10"); //$NON-NLS-1$

            int respon = JOptionPane.showOptionDialog(
                    UE.WINDOW,
                    msg.replace("%1", dest.getName()), //$NON-NLS-1$
                    UE.APP_NAME,
                    JOptionPane.NO_OPTION,
                    JOptionPane.QUESTION_MESSAGE,
                    null,
                    new String[]{Language.getString("ModifiedFilesGUI.11"), // yes //$NON-NLS-1$
                        Language.getString("ModifiedFilesGUI.13") // no //$NON-NLS-1$
                    },
                    Language.getString("ModifiedFilesGUI.13") // no //$NON-NLS-1$
                    );

            if (respon == 0) {
                dest.delete();
            } else {
                return;
            }
        }

        CopyTask task = new CopyTask(cbe, dest);

        Tools.runUEWorker(task);
    }

    private class ExtractTask extends UEWorker<Void> {

        private CanBeEdited src;
        private File to;
        private String[] files;

        public ExtractTask(CanBeEdited edited, File fil, String[] p) {
            src = edited;
            to = fil;
            files = p;
        }

        /* (non-Javadoc)
         * @see javax.swing.SwingWorker#doInBackground()
         */
        @Override
        protected Void doInBackground() throws Exception {
            String msg;

            this.updateStatus(UEWorker.STATUS_WORKING);

            try {
                for (int i = 0; i < files.length; i++) {
                    if (this.isCancelled()) {
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        return null;
                    }

                    File tor;

                    if (src instanceof ResFile) {
                        tor = new File(to.getAbsolutePath(), files[i]);
                    } else {
                        tor = new File(to.getAbsolutePath(), files[i] + ".wav"); //$NON-NLS-1$
                    }

                    msg = Language.getString("ModifiedFilesGUI.16"); //$NON-NLS-1$
                    msg = msg.replace("%1", tor.getName()); //$NON-NLS-1$

                    this.sendMessage(msg);

                    if (src instanceof ResFile) {
                        InternalFile fui = ((ResFile) src).getInternalFile(files[i], false);
                        tor.createNewFile();
                        FileOutputStream out = new FileOutputStream(tor);
                        fui.save(out);
                        out.close();
                    } else {
                        int index = -1;
                        try {
                            index = Integer.parseInt(files[i]);
                        } catch (Exception ef) {
                        }

                        if (index > -1) {
                            ((SndFile) src).extract(index, tor);
                        }
                    }
                }
            } catch (Exception t) {
                this.reportError(t);
                return null;
            }

            this.updateStatus(UEWorker.STATUS_DONE);
            return null;
        }
    }

    private class CopyTask extends UEWorker<Void> {

        private CanBeEdited src;
        private File to;

        public CopyTask(CanBeEdited edited, File fil) {
            src = edited;
            to = fil;
        }

        /* (non-Javadoc)
         * @see javax.swing.SwingWorker#doInBackground()
         */
        @Override
        protected Void doInBackground() throws Exception {
            String msg = Language.getString("ModifiedFilesGUI.23").replace("%1", src.toFile().getName()); //$NON-NLS-1$ //$NON-NLS-2$

            this.updateStatus(UEWorker.STATUS_WORKING);

            this.sendMessage(msg);

            try {
                FileInputStream fis = new FileInputStream(src.toFile());
                FileOutputStream fos = new FileOutputStream(to);

                byte[] b = new byte[1024];
                int len;

                while ((len = fis.read(b)) > 0) {
                    if (this.isCancelled()) {
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        fis.close();
                        fos.close();
                        to.delete();
                        return null;
                    }

                    fos.write(b, 0, len);
                }

                fis.close();
                fos.close();
            } catch (Exception t) {
                this.reportError(t);
                return null;
            }

            this.updateStatus(UEWorker.STATUS_DONE);
            return null;
        }
    }

    // SPECIFIED BY WINDOW LISTENER
    public void windowActivated(WindowEvent e) {
    }

    public void windowClosed(WindowEvent e) {
        UE.WINDOW.setEnabled(true);
        UE.WINDOW.toFront();
    }

    public void windowClosing(WindowEvent e) {
        UE.WINDOW.setEnabled(true);
        UE.WINDOW.toFront();
    }

    public void windowDeactivated(WindowEvent e) {
    }

    public void windowDeiconified(WindowEvent e) {
    }

    public void windowIconified(WindowEvent e) {
    }

    public void windowOpened(WindowEvent e) {
    }
}
