/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: SettingsPanel.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.text.ParseException;
import java.util.Locale;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import ue.Language;
import ue.UE;
import ue.util.ID;
import ue.util.SettingsManager;
import ue.util.Tools;

/**The user interface (MainPanel) for managing the settings*/
@SuppressWarnings("serial")  //$NON-NLS-1$
public class SettingsPanel extends MainPanel
{
    /*ITEMS*/
    private JTextField txtSTBOF;
    private JButton btnSTBR;
    private JCheckBox chkSTBOF;
    private JLabel lblSTBOF;
    private JCheckBox chkSAVE_SND;
    private JCheckBox chkUPDATE_CHECK;
    private JTextField txtTREK;
    private JButton btnTREK;
    private JCheckBox chkTREK;
    private JComboBox cmbLANGUAGE;
    private JButton btnFONT;
    private JLabel lblLANGUAGE;
    private JLabel lblFONT;
    private JLabel lblFONT_DESC;
    private JLabel lblTREK;
    private JLabel lblBACKUPS;
    private JCheckBox chkSECOND_FILES;
    private JSpinner spiBACKUPS;
    private JCheckBox chkEXPERIMENTAL;

    // other stuff
    private static String[] LANGUAGE_CODE = {"en", "de", "en", "sl"}; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$  //$NON-NLS-5$
    
    /**Constructor.*/
    public SettingsPanel()
    {
        /*CREATE*/
        //default font
        Font def = UE.SETTINGS.getDefaultFont();
        txtSTBOF = new JTextField(40);
        btnSTBR = new JButton(Language.getString("SettingsPanel.0")); //$NON-NLS-1$
        chkSTBOF = new JCheckBox(Language.getString("SettingsPanel.1"), UE.SETTINGS.getBooleanProperty(SettingsManager.AUTO_STBOF)); //$NON-NLS-1$
        chkSAVE_SND = new JCheckBox(Language.getString("SettingsPanel.2"), UE.SETTINGS.getBooleanProperty(SettingsManager.STORE_SND_DESC)); //$NON-NLS-1$
        lblSTBOF = new JLabel(Language.getString("SettingsPanel.3")); //$NON-NLS-1$
        lblBACKUPS = new JLabel(Language.getString("SettingsPanel.14")); //$NON-NLS-1$
        txtTREK = new JTextField(40);
        btnTREK = new JButton(Language.getString("SettingsPanel.0")); //$NON-NLS-1$
        chkTREK = new JCheckBox(Language.getString("SettingsPanel.4"), UE.SETTINGS.getBooleanProperty(SettingsManager.AUTO_EXE)); //$NON-NLS-1$
        cmbLANGUAGE = new JComboBox();
        btnFONT = new JButton("Change font");

        String str = def.getName() + ", " + def.getSize();
        if (def.isPlain())
        {
            str = str + ", plain";
        }
        if (def.isItalic())
        {
            str = str + ", italic";
        }
        if (def.isBold())
        {
            str = str + ", bold";
        }
        lblFONT_DESC = new JLabel(str);

        lblFONT = new JLabel(Language.getString("SettingsPanel.7")); //$NON-NLS-1$
        chkUPDATE_CHECK = new JCheckBox(Language.getString("SettingsPanel.12"), UE.SETTINGS.getBooleanProperty(SettingsManager.CHECK_FOR_UPDATES)); //$NON-NLS-1$
        chkSECOND_FILES = new JCheckBox(Language.getString("SettingsPanel.13"), UE.SETTINGS.getBooleanProperty(SettingsManager.LOAD_SECONDARY_FILES)); //$NON-NLS-1$
        chkEXPERIMENTAL = new JCheckBox(Language.getString("SettingsPanel.15"), UE.SETTINGS.getBooleanProperty(SettingsManager.EXPERIMENTAL)); //$NON-NLS-1$
        spiBACKUPS = new JSpinner(new SpinnerNumberModel(UE.SETTINGS.getNumberProperty(SettingsManager.NUMBER_OF_BACKUPS), -1, Integer.MAX_VALUE, 1));

        /*SETUP*/
        txtSTBOF.setText(UE.SETTINGS.getProperty(SettingsManager.STBOF));
        btnSTBR.setFont(def);
        chkSTBOF.setFont(def);
        chkSAVE_SND.setFont(def);
        txtSTBOF.setFont(def);
        lblSTBOF.setFont(def);
        spiBACKUPS.setFont(def);
        lblBACKUPS.setFont(def);

        Dimension d = spiBACKUPS.getPreferredSize();
        d.width = 50;
        spiBACKUPS.setPreferredSize(d);
        spiBACKUPS.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent e)
            {
                try
                {
                    spiBACKUPS.commitEdit();
                }
                catch (ParseException pe)
                {
                    // on error getValue will return last valid value
                    // which we are ok with
                }
                
                UE.SETTINGS.setProperty(SettingsManager.NUMBER_OF_BACKUPS, (Integer)spiBACKUPS.getValue());

                if (UE.SETTINGS.getBooleanProperty(SettingsManager.BACKUP_WARNING))
                {
                    JOptionPane.showMessageDialog(UE.WINDOW,
                            "Be warned, backup files consume a lot of disk space. They're essentialy additional copies of the game.",
                            UE.APP_NAME, JOptionPane.INFORMATION_MESSAGE);

                    UE.SETTINGS.setProperty(SettingsManager.BACKUP_WARNING, false);
                }
            }
        });

        chkSECOND_FILES.setFont(def);
        chkSECOND_FILES.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                boolean b = !UE.SETTINGS.getBooleanProperty(SettingsManager.LOAD_SECONDARY_FILES);
                UE.SETTINGS.setProperty(SettingsManager.LOAD_SECONDARY_FILES, b);
                
                File f = UE.FILES.getPrimaryFile();
                if (f != null)
                {
                    if (UE.FILES.closeAll())
                    {
                        UE.FILES.open(f);
                    }
                }
            }
        });
        chkUPDATE_CHECK.setFont(def);
        chkUPDATE_CHECK.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                boolean b = !UE.SETTINGS.getBooleanProperty(SettingsManager.CHECK_FOR_UPDATES);
                UE.SETTINGS.setProperty(SettingsManager.CHECK_FOR_UPDATES, b);
                if (b)
                {
                    Tools.checkForUpdate();
                }
            }
        });
        chkEXPERIMENTAL.setFont(def);
        chkEXPERIMENTAL.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                boolean b = !UE.SETTINGS.getBooleanProperty(SettingsManager.EXPERIMENTAL);
                UE.SETTINGS.setProperty(SettingsManager.EXPERIMENTAL, b);
            }
        });
        btnSTBR.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent ae)
            {
                JFileChooser fc = new JFileChooser();
                fc.setCurrentDirectory(new File(UE.SETTINGS.getProperty(SettingsManager.STBOF)));
                fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
                fc.setDialogTitle(UE.APP_NAME + " - " //$NON-NLS-1$
                        + Language.getString("SettingsPanel.5")); //$NON-NLS-1$
                fc.setMultiSelectionEnabled(false);
                
                javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter()
                {
                    @Override
                    public boolean accept(File f)
                    {
                        if (f.isDirectory())
                        {
                            return true;
                        }
                        
                        String name = f.getName();
                        name = name.toLowerCase();
                        return name.endsWith(".res"); //$NON-NLS-1$
                    }
                    @Override
                    public String getDescription()
                    {
                        return Language.getString("SettingsPanel.10"); //$NON-NLS-1$
                    }
                };
                
                fc.setFileFilter(filter);
                
                int retValue = fc.showDialog(UE.WINDOW, Language.getString("SettingsPanel.5")); //$NON-NLS-1$
                if (retValue == JFileChooser.APPROVE_OPTION)
                {
                    File fil = fc.getSelectedFile();
                    if (fil != null)
                    {
                        UE.SETTINGS.setProperty(SettingsManager.STBOF, fil.getPath());
                        txtSTBOF.setText(fil.getPath());
                    }
                }
            }
        });
        txtTREK.setText(UE.SETTINGS.getProperty(SettingsManager.EXE));
        btnTREK.setFont(def);
        chkTREK.setFont(def);
        txtTREK.setFont(def);
        btnTREK.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent ae)
            {
                JFileChooser fc = new JFileChooser();
                fc.setCurrentDirectory(new File(UE.SETTINGS.getProperty(SettingsManager.EXE)));
                fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
                fc.setDialogTitle(UE.APP_NAME + " - " //$NON-NLS-1$
                        + Language.getString("SettingsPanel.5")); //$NON-NLS-1$
                fc.setMultiSelectionEnabled(false);
                javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter(){
                    @Override
                    public boolean accept(File f)
                    {
                        if (f.isDirectory())
                        {
                            return true;
                        }
                        
                        String name = f.getName();
                        name = name.toLowerCase();
                        return name.endsWith(".exe"); //$NON-NLS-1$
                    }
                    @Override
                    public String getDescription()
                    {
                        return Language.getString("SettingsPanel.11"); //$NON-NLS-1$
                    }
                };
                
                fc.setFileFilter(filter);
                int retValue = fc.showDialog(UE.WINDOW, Language.getString("SettingsPanel.5")); //$NON-NLS-1$
                if (retValue == JFileChooser.APPROVE_OPTION)
                {
                    File fil = fc.getSelectedFile();
                    if (fil != null)
                    {
                        UE.SETTINGS.setProperty(SettingsManager.EXE, fil.getPath());
                        txtTREK.setText(fil.getPath());
                    }
                }
            }
        });
        cmbLANGUAGE.setFont(def);
        cmbLANGUAGE.addItem(new ID(Language.getString("SettingsPanel.9"), 0)); //$NON-NLS-1$
        Locale locale;
        for (int i = 1; i < LANGUAGE_CODE.length; i++)
        {
            locale = new Locale(LANGUAGE_CODE[i]);
            cmbLANGUAGE.addItem(new ID(locale.getDisplayLanguage(locale).substring(0, 1).toUpperCase() + locale.getDisplayLanguage(locale).substring(1), i));
        }
        // select current language
        str = Language.getLanguage();
        int k = -1;
        for (int i = 0; i < LANGUAGE_CODE.length; i++)
        {
            if (LANGUAGE_CODE[i].equals(str))
            {
                k = i;
                break;
            }
        }
        if (k >= 0)
        {
            cmbLANGUAGE.setSelectedIndex(k);
        }
        // listener
        cmbLANGUAGE.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent ae)
            {
                int i = cmbLANGUAGE.getSelectedIndex();
                if (i >= 0)
                {
                    try
                    {
                        UE.SETTINGS.setProperty(SettingsManager.LANGUAGE, LANGUAGE_CODE[i]);
                        Language.loadLanguage(LANGUAGE_CODE[i]);
                        updateText();
                    }
                    catch (Throwable e)
                    {
                        e.printStackTrace();
                    }
                }
            }
        });
        lblLANGUAGE = new JLabel(Language.getString("SettingsPanel.8")); //$NON-NLS-1$
        lblLANGUAGE.setFont(def);
        
        lblTREK = new JLabel(Language.getString("SettingsPanel.6")); //$NON-NLS-1$
        lblTREK.setFont(def);
		btnFONT.setFont(def);
        lblFONT_DESC.setFont(def);
        btnFONT.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent ae)
            {
                FontDialog dialog = new FontDialog(btnFONT.getFont());
                dialog.setVisible(true);

                if (dialog.getSelectedFont() != null)
                {
                    try
                    {
                        Font def = dialog.getSelectedFont();

                        UE.SETTINGS.setProperty(SettingsManager.FONT_SIZE, def.getSize());
                        UE.SETTINGS.setProperty(SettingsManager.FONT_STYLE, def.getStyle());
                        UE.SETTINGS.setProperty(SettingsManager.FONT_NAME, def.getName());
                        updateText();
                    }
                    catch (Throwable e)
                    {
                        e.printStackTrace();
                    }
                }
            }
        });
        lblFONT.setFont(def);
        /*LAYOUT*/
        setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.insets=new Insets(5, 5, 5, 5);
        c.fill=GridBagConstraints.BOTH;
        // gui
        JPanel pnlGUI = new JPanel(new GridBagLayout());
        /*ADD*/
        //labels
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        pnlGUI.add(lblLANGUAGE, c);
        c.gridx = 1;
        pnlGUI.add(cmbLANGUAGE, c);
        c.gridx = 2;
        pnlGUI.add(lblFONT, c);
        c.gridx = 3;
        pnlGUI.add(lblFONT_DESC, c);
        c.gridx = 4;
        pnlGUI.add(btnFONT, c);
        c.gridx = 0;
        c.gridy = 0;
        add(pnlGUI, c);
        // paths
        JPanel pnlPATHS = new JPanel(new GridBagLayout());
        /*ADD*/
        //labels
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 2;
        c.gridheight = 1;
        pnlPATHS.add(lblSTBOF, c);
        c.gridy = 3;
        pnlPATHS.add(lblTREK, c);
        //textfields
        c.gridy = 1;
        c.gridwidth = 1;
        pnlPATHS.add(txtSTBOF, c);
        c.gridy = 4;
        pnlPATHS.add(txtTREK, c);
        //buttons
        c.gridx = 1;
        c.gridy = 1;
        pnlPATHS.add(btnSTBR, c);
        c.gridy = 4;
        pnlPATHS.add(btnTREK, c);
        //checkboxes
        c.gridx = 0;
        c.gridy = 2;
        c.gridwidth = 2;
        pnlPATHS.add(chkSTBOF, c);
        c.gridy = 5;
        pnlPATHS.add(chkTREK, c);
        c.gridy = 6;
        pnlPATHS.add(chkSAVE_SND, c);
        c.gridy = 7;
        pnlPATHS.add(chkUPDATE_CHECK, c);
        c.gridy = 8;
        pnlPATHS.add(chkSECOND_FILES, c);
        c.gridy = 9;
        pnlPATHS.add(chkEXPERIMENTAL, c);
        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth = 2;
        c.gridheight = 1;
        add(pnlPATHS, c);
        JPanel pnlBKP = new JPanel(new GridBagLayout());
        pnlBKP.setAlignmentX(JPanel.LEFT_ALIGNMENT);
        c.gridwidth = 1;        
        c.gridy = 0;
        c.gridx = 0;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.WEST;
        pnlBKP.add(lblBACKUPS, c);
        c.gridx = 1;
        pnlBKP.add(spiBACKUPS, c);
        c.gridx = 0;
        c.gridy = 2;
        c.gridwidth = 2;
        c.gridheight = 1;
        add(pnlBKP, c);
    }
    
    @Override
    public void finalWarning() throws Exception
    {
        //stbof
        UE.SETTINGS.setProperty(SettingsManager.STBOF, txtSTBOF.getText());
        UE.SETTINGS.setProperty(SettingsManager.AUTO_STBOF, chkSTBOF.isSelected());
        //trek
        UE.SETTINGS.setProperty(SettingsManager.EXE, txtTREK.getText());
        UE.SETTINGS.setProperty(SettingsManager.AUTO_EXE, chkTREK.isSelected());
        // snd descriptions
        UE.SETTINGS.setProperty(SettingsManager.STORE_SND_DESC, chkSAVE_SND.isSelected());
    }
    
    private void updateText()
    {
        UE.WINDOW.showPanel(new SettingsPanel());
    }
}
