/**
 * Ultimate Editor (dev)
 * Copyright (C) 2009 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: AIShipSetsGUI.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 24.2.2009
 */
package ue.gui.exe;

import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileNotFoundException;

import javax.swing.BoxLayout;
import javax.swing.GroupLayout;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import ue.Language;
import ue.UE;
import ue.edit.CanBeEdited;
import ue.edit.FileManager;
import ue.edit.exe.Trek;
import ue.edit.exe.trek.AIShpSets;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.dic.Lexicon;
import ue.edit.res.stbof.rst.Race;
import ue.exception.InvalidArgumentsException;
import ue.gui.MainPanel;
import ue.util.ID;
import ue.util.SettingsManager;

/**
 * @author Alan Podlesek
 */
@SuppressWarnings("serial") //$NON-NLS-1$
public class AIShipSetsGUI extends MainPanel
{
    private final String strEmpire = Language.getString("AIShipSetsGUI.0"); //$NON-NLS-1$
    private final String[] strAgendas = new String[]
                                                   {
            Language.getString("AIShipSetsGUI.1"), //$NON-NLS-1$
            Language.getString("AIShipSetsGUI.2"), //$NON-NLS-1$
            Language.getString("AIShipSetsGUI.3"), //$NON-NLS-1$
            Language.getString("AIShipSetsGUI.4"), //$NON-NLS-1$
            Language.getString("AIShipSetsGUI.5"), //$NON-NLS-1$
            Language.getString("AIShipSetsGUI.6"), //$NON-NLS-1$
            Language.getString("AIShipSetsGUI.7"), //$NON-NLS-1$
            Language.getString("AIShipSetsGUI.8"), //$NON-NLS-1$
            Language.getString("AIShipSetsGUI.9") //$NON-NLS-1$
                                                   };
    private final String strNothing = Language.getString("AIShipSetsGUI.10"); //$NON-NLS-1$
    private final String strShipType = Language.getString("AIShipSetsGUI.11"); //$NON-NLS-1$

    private JComboBox cmbEmpire;
    private JComboBox cmbAgenda;
    private JComboBox[] cmbShipType;
    private JLabel lblEmpire;
    private JLabel lblAgenda;
    
    private AIShpSets shipSets;
    
    public AIShipSetsGUI(Trek exe)
    {
        try
        {
            shipSets = (AIShpSets)exe.getInternalFile(Trek.fileAI_SHIP_SETS, true);
            
            setupComponents();
            placeComponents();
        }
        catch (Exception e)
        {
            UE.displayError(e);
        }
    }
    
    private void setupComponents()
    {
        Font def = UE.SETTINGS.getDefaultFont();
        
        CanBeEdited stbof = UE.FILES.getFile(FileManager.P_STBOF);
        Race races = null;
        Lexicon lexicon = null;
        
        if (stbof != null)
        {
            try
            {
                races = (Race)stbof.getInternalFile(Stbof.fileRACE_RST, true);
                lexicon = (Lexicon)stbof.getInternalFile(Stbof.fileLEXICON_DIC, true);
            }
            catch (FileNotFoundException e)
            {
            }
        }
        
        // labels
        lblEmpire = new JLabel(Language.getString("AIShipSetsGUI.12")); //$NON-NLS-1$
        lblEmpire.setFont(def);
        lblAgenda = new JLabel(Language.getString("AIShipSetsGUI.13")); //$NON-NLS-1$
        lblAgenda.setFont(def);
        
        // combo boxes
        ActionListener set = new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                int i = -1;
                
                for (int j = 0; j < 36; j++)
                {
                    if (cmbShipType[j].equals(e.getSource()))
                    {
                        i = j;
                        break;
                    }
                }
                
                if (i >= 0)
                {
                    setShipType(i);
                }
            }
        };

        cmbShipType = new JComboBox[36];
        
        for (int i = 0; i < 36; i++)
        {
            cmbShipType[i] = new JComboBox();
            cmbShipType[i].setFont(def);
            
            if (lexicon != null)
            {
                cmbShipType[i].addItem(new ID(strNothing, -1));
                
                try
                {
                    cmbShipType[i].addItem(new ID(lexicon.getEntry(1103), 0));
                    cmbShipType[i].addItem(new ID(lexicon.getEntry(232), 1));
                    cmbShipType[i].addItem(new ID(lexicon.getEntry(204), 2));
                    cmbShipType[i].addItem(new ID(lexicon.getEntry(1183), 3));
                    cmbShipType[i].addItem(new ID(lexicon.getEntry(173), 4));
                    cmbShipType[i].addItem(new ID(lexicon.getEntry(166), 5));
                    cmbShipType[i].addItem(new ID(lexicon.getEntry(1366), 9));
                }
                catch (InvalidArgumentsException e1)
                {
                }
            }
            
            for (int j = cmbShipType[i].getItemCount(); j < 7; j++)
            {
                if (j == 6) j = 9;
                
                cmbShipType[i].addItem(new ID(strShipType.replace("%1", Integer.toString(j)), j));
            }
            
            cmbShipType[i].addActionListener(set);
        }
        
        cmbAgenda = new JComboBox();
        cmbAgenda.setFont(def);
        
        ActionListener update = new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                updateShipTypes();
            }
        };
        
        for (int i = 0; i < strAgendas.length; i++)
        {
            cmbAgenda.addItem(new ID(strAgendas[i], i));
        }
        
        cmbAgenda.addActionListener(update);

        cmbEmpire = new JComboBox();
        cmbEmpire.setFont(def);
        
        cmbEmpire.addActionListener(update);
        
        for (int r = 0; r < 5; r++)
        {
            if (races != null)
            {
                try
                {
                    cmbEmpire.addItem(new ID(races.getName(r), r));
                    continue;
                }
                catch (InvalidArgumentsException e)
                {
                }
            }
            
            cmbEmpire.addItem(new ID(strEmpire.replace("%1", Integer.toString(r)), r)); //$NON-NLS-1$
        }
    }
    
    private void placeComponents()
    {
        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        
        JPanel pnlTop = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        pnlTop.add(lblEmpire);
        pnlTop.add(cmbEmpire);
        pnlTop.add(lblAgenda);
        pnlTop.add(cmbAgenda);
        
        this.add(pnlTop);
        
        JPanel pnlSet = new JPanel();
        
        GroupLayout layout = new GroupLayout(pnlSet);
        pnlSet.setLayout(layout);
        layout.setAutoCreateGaps(true);
        layout.setAutoCreateContainerGaps(true);

        GroupLayout.SequentialGroup hGroup = layout.createSequentialGroup();
        GroupLayout.SequentialGroup vGroup = layout.createSequentialGroup();
      
        Font def = UE.SETTINGS.getDefaultFont();
        JLabel lbl;
        
        GroupLayout.ParallelGroup vpg1;
        GroupLayout.ParallelGroup vpg2;

        GroupLayout.ParallelGroup[] hpg = new GroupLayout.ParallelGroup[12];
        
        for (int i = 0; i < 12; i++)
        {
            hpg[i] = layout.createParallelGroup(GroupLayout.Alignment.BASELINE);
        }
        
        int n = 0;
        
        for (int i = 0; i < 3; i++)
        {
            vpg1 = layout.createParallelGroup();
            vpg2 = layout.createParallelGroup();
            
            for (int j = 0; j < 12; j++)
            {
                lbl = new JLabel(Language.getString("AIShipSetsGUI.14").replace("%1", Integer.toString(n + 1))); //$NON-NLS-1$ //$NON-NLS-2$
                lbl.setHorizontalTextPosition(SwingConstants.RIGHT);
                lbl.setFont(def);
                vpg1.addComponent(lbl);
                hpg[j].addComponent(lbl);

                vpg2.addComponent(cmbShipType[n]);
                hpg[j].addComponent(cmbShipType[n]);

                n++;
            }
            
            hGroup.addGroup(vpg1);
            hGroup.addGroup(vpg2);
        }
        
        for (int i = 0; i < 12; i++)
        {
            vGroup.addGroup(hpg[i]);
        }

        layout.setHorizontalGroup(hGroup);
        layout.setVerticalGroup(vGroup);

        this.add(pnlSet);
    }
    
    /* (non-Javadoc)
     * @see ue.gui.MainPanel#finalWarning()
     */
    @Override
    public void finalWarning() throws Exception
    {
        // nothing to do
    }
    
    private void updateShipTypes()
    {
        int emp = cmbEmpire.getSelectedIndex();
        int agenda = cmbAgenda.getSelectedIndex();
        
        if (emp >= 0 && agenda >= 0)
        {
            int type;
            
            for (int i = 0; i < 36; i++)
            {
                type = shipSets.getShipType(emp, agenda, i);
                
                cmbShipType[i].setSelectedItem(new ID(null, type));
            }
        }
    }
    
    private void setShipType(int index)
    {
        int emp = cmbEmpire.getSelectedIndex();
        int agenda = cmbAgenda.getSelectedIndex();
        
        Object id = cmbShipType[index].getSelectedItem();
        
        if (emp >= 0 && agenda >= 0 && id != null)
        {
            int type = id.hashCode();
            
            shipSets.setShipType(emp, agenda, index, type);
            
            int i = index + 1;
            
            if (i < 36)
            {
                if (type == AIShpSets.LIST_END)
                {
                    cmbShipType[i].setSelectedItem(new ID(null, -1));
                    cmbShipType[i].setEnabled(false);
                }
                else
                {
                    cmbShipType[i].setEnabled(true);
                }
            }
        }
    }
}
