/*
 *	Ultimate Editor for Birth Of The Federation 1.02
 *	Copyright (C) 2005-2008  Alan Podlesek
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *
 *	File: CrackGUI.java
 *
 *	Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 *	Last modified: 1.6.2008
 */

package ue.gui.exe;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import ue.Language;
import ue.UE;
import ue.edit.exe.trek.CDprotection;
import ue.gui.MainPanel;
import ue.util.ID;
import ue.util.SettingsManager;

/**
 * A gui used to enable/disable BotF CD protection.
 */
@SuppressWarnings("serial") //$NON-NLS-1$
public class CDProtectionGUI extends MainPanel {
	private CDprotection PROTECTION;
	// gui
	private JLabel lblCDProtection = new JLabel(Language.getString("CDProtectionGUI.0")); //$NON-NLS-1$
	private JLabel lblModifyIni = new JLabel(Language.getString("CDProtectionGUI.1")); //$NON-NLS-1$
	private JLabel lblPath = new JLabel(Language.getString("CDProtectionGUI.2")); //$NON-NLS-1$
	private JCheckBox chkCDProtection;
	private JButton btnModify = new JButton(Language.getString("CDProtectionGUI.3")); //$NON-NLS-1$
	private JButton btnBrowse = new JButton(Language.getString("CDProtectionGUI.4")); //$NON-NLS-1$
	private JButton btnEdit = new JButton(Language.getString("CDProtectionGUI.10")); //$NON-NLS-1$
	private JTextField txtPath = new JTextField(40);
	private JComboBox cmbRoots = new JComboBox();
	private JLabel lblDrives = new JLabel(Language.getString("CDProtectionGUI.8")); //$NON-NLS-1$
	private JTextArea txtINFO = new JTextArea();
	// strings
	// %1 - the name of the updated file
	private static String strFILE_UPDATED = Language.getString("CDProtectionGUI.5"); //$NON-NLS-1$
	private static String strSELECT_FILE = Language.getString("CDProtectionGUI.6"); //$NON-NLS-1$
	private static String strSELECT = Language.getString("CDProtectionGUI.7"); //$NON-NLS-1$
	private static String strINFO = Language.getString("CDProtectionGUI.9"); //$NON-NLS-1$
	
	public CDProtectionGUI(File path, CDprotection protection){
		if (path == null || protection == null) return;
		PROTECTION = protection;
		
		// CREATE
		chkCDProtection = new JCheckBox("", PROTECTION.isEnabled()); //$NON-NLS-1$
		txtPath.setText(path.getAbsolutePath() + File.separator + "stbof.ini"); //$NON-NLS-1$ //$NON-NLS-2$
		
		// SETUP
		
		Font def = UE.SETTINGS.getDefaultFont();
		
		lblCDProtection.setFont(def);
		lblModifyIni.setFont(def);
		lblPath.setFont(def);
		chkCDProtection.setFont(def);
		btnModify.setFont(def);
		btnBrowse.setFont(def);
		txtPath.setFont(def);
		lblDrives.setFont(def);
		cmbRoots.setFont(def);
		
		btnModify.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent a) {
				try{
					File path = new File(txtPath.getText());
					
					// read stbof.ini
					BufferedReader reader = new BufferedReader(
							new InputStreamReader(
									new FileInputStream(path), "ISO-8859-1")); //$NON-NLS-1$

					ArrayList<String> lines = new ArrayList<String>();
					String cdpath = "C:\\botf\\"; //$NON-NLS-1$
					while (reader.ready()){
						lines.add(reader.readLine());
					}
					
					reader.close();
					
					// copy hdpath to cdpath
					String str;
					if (!chkCDProtection.isSelected()){
						for (int i = 0; i < lines.size(); i++){
							str = lines.get(i);
							
							if (str.startsWith("HDPATH")){ //$NON-NLS-1$
								cdpath = str.substring(7);
							}
						}
					}else{
						cdpath = ((ID)cmbRoots.getSelectedItem()).toString();
						if (!cdpath.endsWith(File.separator))
							cdpath = cdpath + File.separator;
					}
					// write to stbof.ini
					BufferedWriter writer = new BufferedWriter(
							new OutputStreamWriter(
									new FileOutputStream(path), "ISO-8859-1")); //$NON-NLS-1$
					
					for (int i = 0; i < lines.size(); i++){
						str = lines.get(i);
						
						if (str.startsWith("CDPATH")){ //$NON-NLS-1$
							str = "CDPATH=" + cdpath; //$NON-NLS-1$
						}

						writer.write(str);
						writer.newLine();
					}
					
					writer.close();
					
					// show message
					JOptionPane.showMessageDialog(UE.WINDOW,
							strFILE_UPDATED.replace("%1", path.getName()), //$NON-NLS-1$
							UE.APP_NAME, JOptionPane.INFORMATION_MESSAGE);
				}catch (Exception e){
					UE.displayError(e);
				}
			}
		});
		
		chkCDProtection.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				PROTECTION.setEnabled(!PROTECTION.isEnabled());
			}
		});
		
		btnBrowse.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				File path = new File(txtPath.getText());

				JFileChooser fc = new JFileChooser();
				fc.setCurrentDirectory(path.getParentFile());
				fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
				fc.setDialogTitle(UE.APP_NAME + " - " + strSELECT_FILE); //$NON-NLS-1$
				fc.setMultiSelectionEnabled(false);
				
				javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter(){
					@Override
					public boolean accept(File f){
						if (f.isDirectory()) {
							return true;
						}
						String name = f.getName();
						name = name.toLowerCase();
						if (name.endsWith(".ini")) { //$NON-NLS-1$
							return true;
						} else {
							return false;
						}
					}
					@Override
					public String getDescription(){
						return "stbof.ini (*.ini)"; //$NON-NLS-1$
					}
				};
				fc.setFileFilter(filter);

				int retValue = fc.showDialog(UE.WINDOW, strSELECT);

				if (retValue == JFileChooser.APPROVE_OPTION){
					File fil = fc.getSelectedFile();
					if (fil!=null){
						txtPath.setText(fil.getPath());
					}
				}
			}
		});
		
		int i = 0;
		for (File f : File.listRoots()){
        	cmbRoots.addItem(new ID(f.getAbsolutePath(), i++));
        }
		cmbRoots.setSelectedIndex(cmbRoots.getItemCount() - 1);
		
		txtINFO.setFont(def);
		txtINFO.setEditable(false);
		txtINFO.setFocusable(false);
		txtINFO.setLineWrap(true);
		txtINFO.setOpaque(false);
		txtINFO.setWrapStyleWord(true);
		txtINFO.setText(strINFO.replace("%1", path.getAbsolutePath() + File.separator)); //$NON-NLS-1$
		
		btnEdit.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				String ret = (String)JOptionPane.showInputDialog(null, Language.getString("CDProtectionGUI.11"), //$NON-NLS-1$
						UE.APP_NAME, JOptionPane.PLAIN_MESSAGE);
				int i = cmbRoots.getItemCount();
				cmbRoots.addItem(new ID(ret, i));
				cmbRoots.setSelectedIndex(i);
			}
		});
		// ADD
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill=GridBagConstraints.BOTH;
		c.insets = new Insets(5, 5, 5, 5);
		c.gridx=0;
		c.gridy=0;
		c.gridwidth=1;
		c.gridheight=1;

		// pnlOne
		JPanel pnlOne = new JPanel(new GridBagLayout());
		
			pnlOne.add(lblCDProtection, c);
			c.gridx=1;
			pnlOne.add(chkCDProtection, c);
		
		c.gridx=0;
		add(pnlOne, c);
		// pnlTwo
		JPanel pnlTwo = new JPanel(new GridBagLayout());
		
			c.gridwidth=2;
			pnlTwo.add(lblModifyIni, c);
			c.gridx=2;
			c.gridwidth=1;
			pnlTwo.add(btnModify, c);
			c.gridx=0;
			c.gridy=1;
			pnlTwo.add(lblDrives, c);
			c.gridx=1;
			pnlTwo.add(cmbRoots, c);
			c.gridx=2;
			pnlTwo.add(btnEdit, c);
			c.gridx=0;
			c.gridy=2;
			pnlTwo.add(lblPath, c);
			c.gridx=1;
			pnlTwo.add(txtPath, c);
			c.gridx=2;
			pnlTwo.add(btnBrowse, c);
	
		c.gridx=0;
		c.gridy=1;
		add(pnlTwo, c);
		c.gridy=2;
		add(txtINFO, c);
	}
	
	@Override
	public void finalWarning() throws Exception
    {
	}

}
