/**
 * Ultimate Editor (dev)
 * Copyright (C) 2009 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: GalacticMapGUI.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 2.6.2009
 */
package ue.gui.exe;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileNotFoundException;
import java.util.Arrays;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import ue.Language;
import ue.UE;
import ue.edit.exe.GenericSegment;
import ue.edit.exe.InternalSegment;
import ue.edit.exe.Trek;
import ue.edit.exe.trek.AverageNumberOfSpaceObjects;
import ue.edit.exe.trek.MinorRacesNumbers;
import ue.gui.MainPanel;
import ue.util.Tools;

/**
 * @author Alan Podlesek
 */
@SuppressWarnings("serial")
public class GalacticMapGUI extends MainPanel
{
    private Trek TREK;
    private boolean init_ok = false;
    
    // code
    private static final long[][] width_address = new long[][]
    {
        new long[]  // small
                 {
                0x3cce8, 0x3ceba, 0x4e9fa, 0xae554, 0xae581,
                0xae5f0, 0xb15cb, 0xb5fe9, 0xb60a1, 0xb615c, 0xb61c9,
                0xb6571
                 },
        new long[]  // medium
                 {
                0x3cd04, 0x3cef5, 0x4eb24, 0xae4b1, 0xb160d, 0xb5fd4,
                0xb608f, 0xb6168, 0xb629f, 0xb6451
                 },
        new long[]  // large
                 {
                0x3ccfd, 0x3ceee, 0xae6e6, 0xb163f,
                0xb5ff0, 0xb60a8, 0xb6118, 0xb62a9, 0xb6591, 0x4eb2e
                 }
    };
    
    private static final long[][] height_address = new long[][]
    {
        new long[]  // small
                 {
                0x3ccb8, 0xb15d2, 0xb6161, 0xb61e1, 0xb6576, 0x4e9e3
                 },
        new long[]  // medium
                 {
                0x3ccd4, 0xb1614, 0xb616d, 0xb62b5, 0xb6456, 0x4eb0e
                 },
        new long[]  // large
                 {
                0x3cccd, 0xb1646, 0xb611d, 0xb62bf, 0xb6596, 0x4eb18
                 }
    };
    
    private static final byte[][][] numericSectorIDCode = new byte[][][]
                                                                       {
        new byte[][]
                   {
                new byte[]{(byte) 0x83, (byte) 0xC3, 0x01},
                new byte[]{0x68, 0x57, (byte)0xF9, 0x57, 0x00},
                new byte[]{0x25, 0x73, 0x20, 0x25, 0x64, 0x2E},
                new byte[]{(byte) 0x83, (byte) 0xC0, 0x01},
                new byte[]{0x25, 0x64, 0x2E, 0x25, 0x64}
                   },
                   new byte[][]
                              {
                new byte[]{(byte) 0x83, (byte) 0xC3, 0x41},
                new byte[]{0x68, 0x58, (byte)0xF9, 0x57, 0x00},
                new byte[]{0x00, 0x25, 0x73, 0x20, 0x25, 0x63},
                new byte[]{(byte) 0x83, (byte) 0xC0, 0x61},
                new byte[]{0x25, 0x63, 0x25, 0x64, 0x00}
                              }
                                                                   };
    
    private static final long[] numericSectorID_address = new long[]
    {
        0xB58DA, 0xB58E6, 0x17D757, 0xB5F0B, 0x17D8FC
    };

    private static final long[] width5_address = new long[]
    {
        0xae52a, 0xae483, 0xae6b8
    };
    
    private static final long[] height5_address = new long[]
    {
        0xae525, 0xae47e, 0xae6b3
    };

    private static final long[][] sectorSize_address = new long[][]
    {
        new long[]  // small
                 {
                0xdb8a8, 0xda42d, 0xdaf66, 0xd9c60, 0xda683, 0xda940, 0xdafd5, 0xdb08d, 0xdbcb5
                 },
        new long[]  // medium
                 {
                0xdb8b2, 0xda437, 0xdaf6c, 0xd9c6a, 0xda68d, 0xda94a, 0xdafdc, 0xdb094, 0xdbcbf
                 },
        new long[]  // large
                 {
                0xdb8bc, 0xda441, 0xdaf72, 0xd9c74, 0xda697, 0xda954, 0xdafe3, 0xdb09b, 0xdbcc9
                 },
        new long[] // zoom-in
                 {
                0xdb69b, 0xda2a5, 0xdaf60, 0xd9b71, 0xda4c0, 0xda75a, 0xdafa9, 0xdb01a, 0xdba0c
                 }
    };

    private static final long[][] systemAlignment_address = new long[][]
    {
        new long[]  // small
                 {
                0xd9c3c, 0xda97a, 0xdb185, 0xdb530
                 },
        new long[]  // medium
                 {
                0xd9c49, 0xda98a, 0xdb18f, 0xdb53d
                 },
        new long[]  // large
                 {
                0xD9C56, 0xDA99A, 0xDB19C, 0xDB54A
                 },
        new long[] // zoom-in
                 {
                0xd9ad4, 0xda7ff, 0xdb110, 0xdb3c6
                 }
    };
    
    private static final long irregularGalaxyDensity_address = 0xae53f;

    /*
    private static final long[] galacticDensity = new long[]
    {
        0x17D568,
        0x17D568
    };*/

    private static final long[] numberOfSpaceObjectsLimit_address = new long[]
    {
        0xAFA49, 0xAFA94
    };

    private static final long starAnomalyRatio_address = 0x17D538;

    private static final long[] minEmpireDistance_address = new long[]
    {
        0xae544, 0xae563, 0xae5b1, 0xae5e6,  // small (the third is actually 0xae590, but it's shared so it needs to be fixed to use the new address)
        0xae4a1, 0xae5ff, 0xae643, 0xae665,  // medium
        0xae6f5, 0xae6d2, 0xae727, 0xae759   // large (the third is actually 0xae706, but it's shared so it needs to be fixed to use the new address)
    };

    private static final long[] widthOfRing_address = new long[]
    {
        0xae590, 0xae623, 0xae706       // small, medium, large
    };
    
    private static final long[] sizeOfRingCore_address = new long[]
    {
        0xae595, 0xae628, 0xae70b       // small, medium, large
    };

    private static final long[] ellipseGalAxis_address = new long[]
    {
        0xae572, 0xae60e, 0xae6dc,      // small, medium, large (X)
        0xae568, 0xae604, 0xae6d7       // small, medium, large (Y)
    };
    
    private static final long[] spiralGalEllipseAxis_address = new long[]
    {
        0xae5d0, 0xae660, 0xae75d,      // small, medium, large (X) (last one would be 0xae741, but UE inserts a fix)
        0xae5cb, 0xae65b, 0xae741       // small, medium, large (Y)
    };

    private static final long[] spiralArmWidth_address = new long[]
    {
        0xae5c6, 0xae656, 0xae73c      // small, medium, large
    };
    
    private static final long[] spiralArmArc_address = new long[]
                                                                 {
        0x17D5A0, 0x17D5A8, 0x17D5B0      // inner, medium, outer
                                                                 };
    private static final long aiScanRange_address = 0x1C44F;


    /*
Gowron's starting pop codes                     
                        
Hex location    suggested value         vanilla value   
Free Systems                    
0xb3cc1         A6 0E   A0 0F   
0xb3c4d         C4 09   20 03   
0xb3c60         C4 09   20 03   
0xb3d30         07  00  07 00   
                        
Minor Race Systems                      
0xb3f0f         A0 0F   A0 0F   
0xb3e47         B8 0B   14 05   
0xb3e57         B8 0B   14 05   
0xb3f80         07 00   07 00   
                        
Additional Major Empire Starting Systems                
0xb4141         94 11   A0 0F   
0xb4071         A0 0F   14 05   
0xb4081         A0 0F   14 05   
0xb41fb         07 00   07 00   
                        
    */
    
    // strings
    private String strMapSize = Language.getString("GalacticMapGUI.0"); //$NON-NLS-1$
    private String strSystemAlignment = Language.getString("GalacticMapGUI.1"); //$NON-NLS-1$
    private String strIrregularGalaxy = Language.getString("GalacticMapGUI.2"); //$NON-NLS-1$
    private String strMinorRaces = Language.getString("GalacticMapGUI.3"); //$NON-NLS-1$
    private String strMapWidth = Language.getString("GalacticMapGUI.4"); //$NON-NLS-1$
    private String strMapHeight = Language.getString("GalacticMapGUI.5"); //$NON-NLS-1$
    private String strDensity_ = Language.getString("GalacticMapGUI.6"); //$NON-NLS-1$
    private String strSmall_ = Language.getString("GalacticMapGUI.7"); //$NON-NLS-1$
    private String strMedium_ = Language.getString("GalacticMapGUI.8"); //$NON-NLS-1$
    private String strLarge_ = Language.getString("GalacticMapGUI.9"); //$NON-NLS-1$
    private String strZoomIn_ = Language.getString("GalacticMapGUI.10"); //$NON-NLS-1$
    private String strFewMinors = Language.getString("GalacticMapGUI.11"); //$NON-NLS-1$
    private String strSomeMinors = Language.getString("GalacticMapGUI.12"); //$NON-NLS-1$
    private String strManyMinors = Language.getString("GalacticMapGUI.13"); //$NON-NLS-1$
    private String strAllNumericSectorID = Language.getString("GalacticMapGUI.14"); //$NON-NLS-1$
    private String strSectorSize = Language.getString("GalacticMapGUI.15"); //$NON-NLS-1$
    private String strNumSpaceObjects = Language.getString("GalacticMapGUI.16"); //$NON-NLS-1$
    private String strStarAnomalyRatio = Language.getString("GalacticMapGUI.17"); //$NON-NLS-1$
    private String strStarAnomalyRatio_ = Language.getString("GalacticMapGUI.18"); //$NON-NLS-1$
    private String strIrregular = Language.getString("GalacticMapGUI.19"); //$NON-NLS-1$
    private String strElliptic = Language.getString("GalacticMapGUI.20"); //$NON-NLS-1$
    private String strRing = Language.getString("GalacticMapGUI.21"); //$NON-NLS-1$
    private String strSpiral = Language.getString("GalacticMapGUI.22"); //$NON-NLS-1$
    private String strMinEmpDist = Language.getString("GalacticMapGUI.23"); //$NON-NLS-1$
    private String strEllipseAxes = Language.getString("GalacticMapGUI.24"); //$NON-NLS-1$
    private String strMajor = Language.getString("GalacticMapGUI.25"); //$NON-NLS-1$
    private String strMinor = Language.getString("GalacticMapGUI.26"); //$NON-NLS-1$
    private String strSpiralGalSettings = Language.getString("GalacticMapGUI.27"); //$NON-NLS-1$
    private String strArmWidth = Language.getString("GalacticMapGUI.28"); //$NON-NLS-1$
    private String strRingGalSettings = Language.getString("GalacticMapGUI.29"); //$NON-NLS-1$
    private String strRingWidth = Language.getString("GalacticMapGUI.30"); //$NON-NLS-1$
    private String strCoreSize = Language.getString("GalacticMapGUI.31"); //$NON-NLS-1$
    private String strSpiralArmArcInner = Language.getString("GalacticMapGUI.32"); //$NON-NLS-1$
    private String strSpiralArmArcMiddle = Language.getString("GalacticMapGUI.33"); //$NON-NLS-1$
    private String strSpiralArmArcOuter = Language.getString("GalacticMapGUI.34"); //$NON-NLS-1$
    private String strSpiralArmArcMultiplier = Language.getString("GalacticMapGUI.35"); //$NON-NLS-1$
    
    // Labels
    private JLabel lblMapWidth;
    private JLabel lblMapHeight;
    private JLabel[] lblZoomIn;
    private JLabel[] lblSmall;
    private JLabel[] lblMedium;
    private JLabel[] lblLarge;
    private JLabel[] lblMinorsSetting;
    private JLabel lblDensity;
    private JLabel lblStarAnomalyRatio;
    private JLabel[] lblGalaxyShape;
    private JLabel[] lblAxis;
    private JLabel[] lblSpiralGalSettings;
    private JLabel[] lblRingGalSettings;
    private JLabel[] lblSpiralArmArc;
    
    // Spinners
    private JSpinner[] spiMapWidth;
    private JSpinner[] spiMapHeight;
    private JSpinner[] spiSectorSize;
    private JSpinner[] spiAlignmentMultiplier;
    private JSpinner[] spiMaxMinors;
    private JSpinner spiDensity;
    private JSpinner[] spiSpaceObjectsNum;
    private JSpinner spiStarAnomalyRatio;
    private JSpinner[] spiMinEmpireDistance;
    private JSpinner[] spiEllipseAxis;
    private JSpinner[] spiSpiralEllipseAxis;
    private JSpinner[] spiSpiralArmWidth;
    private JSpinner[] spiRingWidth;
    private JSpinner[] spiCoreSize;
    private JSpinner[] spiSpiralArmArc;
    
    // check boxes
    private JCheckBox chkNumericSectorID;
    
    // panels
    private JPanel cards;
    
    // buttons
    private JButton[] btnSwitchCard;
    
    
    public GalacticMapGUI(Trek trek) throws Exception
    {
        TREK = trek;
        
        try
        {
            createComponents();
            placeComponents();
            init_ok = true;
        }
        catch (Exception e)
        {
            UE.displayError(e);
        }
    }
    
    private void createComponents() throws Exception
    {
        Font font = UE.SETTINGS.getDefaultFont();

        // labels
        lblMapWidth = new JLabel(strMapWidth);
        lblMapWidth.setFont(font);
        
        lblMapHeight = new JLabel(strMapHeight);
        lblMapHeight.setFont(font);

        lblDensity = new JLabel(strDensity_);
        lblDensity.setFont(font);
        
        lblMinorsSetting = new JLabel[3];
        this.createAndSetupLabels(lblMinorsSetting, font, new String[]{strFewMinors, strSomeMinors, strManyMinors}, SwingConstants.CENTER);
        
        lblZoomIn = new JLabel[2];
        this.createAndSetupLabels(lblZoomIn, font, new String[]{strZoomIn_}, SwingConstants.RIGHT);
        
        lblSmall = new JLabel[9];
        this.createAndSetupLabels(lblSmall, font, new String[]{strSmall_}, SwingConstants.RIGHT);
        
        lblMedium = new JLabel[9];
        this.createAndSetupLabels(lblMedium, font, new String[]{strMedium_}, SwingConstants.RIGHT);

        lblLarge = new JLabel[9];
        this.createAndSetupLabels(lblLarge, font, new String[]{strLarge_}, SwingConstants.RIGHT);

        lblStarAnomalyRatio = new JLabel(strStarAnomalyRatio_);
        lblStarAnomalyRatio.setFont(font);
        
        lblGalaxyShape = new JLabel[4];
        this.createAndSetupLabels(lblGalaxyShape, font, new String[]{strIrregular, strElliptic, strRing, strSpiral}, SwingConstants.CENTER);
        
        lblAxis = new JLabel[2];
        this.createAndSetupLabels(lblAxis, font, new String[]{strMajor, strMinor}, SwingConstants.CENTER);
        
        lblSpiralGalSettings = new JLabel[3];
        this.createAndSetupLabels(lblSpiralGalSettings, font, new String[]{strMajor, strMinor, strArmWidth}, SwingConstants.CENTER);
        
        lblRingGalSettings = new JLabel[2];
        this.createAndSetupLabels(lblRingGalSettings, font, new String[]{strRingWidth, strCoreSize}, SwingConstants.CENTER);

        lblSpiralArmArc = new JLabel[3];
        this.createAndSetupLabels(lblSpiralArmArc, font, new String[]{strSpiralArmArcInner, strSpiralArmArcMiddle, strSpiralArmArcOuter}, SwingConstants.RIGHT);

        // spinners
        ChangeListener lstMapWidth = new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent e)
            {
                int i, max, val, k;

                for (i = 0; i < 3; i++)
                {
                    if ((Integer)spiMapWidth[i].getValue() > 27)
                    {
                        chkNumericSectorID.setSelected(true);
                        chkNumericSectorID.setEnabled(false);
                        break;
                    }
                }

                if (i >= 3)
                {
                    chkNumericSectorID.setEnabled(true);
                }
                
                for (i = 0; i < 3; i++)
                {
                    spiSectorSize[i].setValue((int)getExpectedSectorSize(i));

                    if (i < 2)
                    {
                        if (((Number)spiMapWidth[i].getValue()).intValue() >
                            ((Number)spiMapWidth[i+1].getValue()).intValue())
                        {
                            spiMapWidth[i+1].setValue(spiMapWidth[i].getValue());
                        }
                    }
                    
                    
                    max = ((Number)spiMapWidth[i].getValue()).intValue() * ((Number)spiMapHeight[i].getValue()).intValue();
                    val = Tools.getValueWithinBounds((Number)spiSpaceObjectsNum[i].getValue(), 5, max).intValue();
                    spiSpaceObjectsNum[i].setModel(new SpinnerNumberModel(val, 5, max, 1));
                    
                    try
                    {
                        max = (int)Math.sqrt(
                                Math.pow(((Number)spiMapWidth[i].getValue()).intValue(), 2) +
                                Math.pow(((Number)spiMapHeight[i].getValue()).intValue(), 2)
                                ) / 2;
                        
                        k = i * 4;
    
                        for (int j = 0; j < 4; j++)
                        {
                            val = Tools.getValueWithinBounds((Number)spiMinEmpireDistance[k].getValue(), 0, max).intValue();
                            spiMinEmpireDistance[k].setModel(new SpinnerNumberModel(val, 0, max, 1));
                            k++;
                        }
                        
                        val = Tools.getValueWithinBounds((Number)spiRingWidth[i].getValue(), 1, max).intValue();
                        spiRingWidth[i].setModel(new SpinnerNumberModel(val, 1, max, 1));

                        val = Tools.getValueWithinBounds((Number)spiCoreSize[i].getValue(), 0, max).intValue();
                        spiCoreSize[i].setModel(new SpinnerNumberModel(val, 0, max, 1));
                    }
                    catch (Exception f)
                    {
                        UE.displayError(f);
                    }
                }
            }
        };
        
        int val, min, max;
        
        spiMapWidth = new JSpinner[3];
        min = 3; max = 209;
        for (int i = 0; i < spiMapWidth.length; i++)
        {
            val = Tools.getValueWithinBounds(getMapWidth(i, true), min, max).intValue();
            
            spiMapWidth[i] = new JSpinner(new SpinnerNumberModel(val, min, max, 1));
            spiMapWidth[i].setFont(font);
            spiMapWidth[i].addChangeListener(lstMapWidth);
        }

        ChangeListener lstMapHeight = new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent e)
            {
                int i, max, val, k;
                
                for (i = 0; i < 3; i++)
                {
                    spiSectorSize[i].setValue((int)getExpectedSectorSize(i));
                    
                    if (i < 2)
                    {
                        if (((Number)spiMapHeight[i].getValue()).intValue() >
                            ((Number)spiMapHeight[i+1].getValue()).intValue())
                        {
                            spiMapHeight[i+1].setValue(spiMapHeight[i].getValue());
                        }
                    }

                    max = ((Number)spiMapWidth[i].getValue()).intValue() * ((Number)spiMapHeight[i].getValue()).intValue();
                    val = Tools.getValueWithinBounds((Number)spiSpaceObjectsNum[i].getValue(), 5, max).intValue();
                    spiSpaceObjectsNum[i].setModel(new SpinnerNumberModel(val, 5, max, 1));

                    try
                    {
                        max = (int)Math.sqrt(
                                Math.pow(((Number)spiMapWidth[i].getValue()).intValue(), 2) +
                                Math.pow(((Number)spiMapHeight[i].getValue()).intValue(), 2)
                                ) / 2;

                        k = i * 4;
    
                        for (int j = 0; j < 4; j++)
                        {
                            val = Tools.getValueWithinBounds((Number)spiMinEmpireDistance[k].getValue(), 0, max).intValue();
                            spiMinEmpireDistance[k].setModel(new SpinnerNumberModel(val, 0, max, 1));
                            k++;
                        }
                        
                        val = Tools.getValueWithinBounds((Number)spiRingWidth[i].getValue(), 1, max).intValue();
                        spiRingWidth[i].setModel(new SpinnerNumberModel(val, 1, max, 1));

                        val = Tools.getValueWithinBounds((Number)spiCoreSize[i].getValue(), 0, max).intValue();
                        spiCoreSize[i].setModel(new SpinnerNumberModel(val, 1, max, 1));
                    }
                    catch (Exception f)
                    {
                        UE.displayError(f);
                    }
                }
            }
        };
        
        spiMapHeight = new JSpinner[3];
        min = 2; max = 156;
        for (int i = 0; i < spiMapHeight.length; i++)
        {
            val = Tools.getValueWithinBounds(getMapHeight(i, true), min, max).intValue();
            
            spiMapHeight[i] = new JSpinner(new SpinnerNumberModel(val, min, max, 1));
            spiMapHeight[i].setFont(font);
            spiMapHeight[i].addChangeListener(lstMapHeight);
        }
        
        ChangeListener lstSectorSize = new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent e)
            {
                for (int i = 0; i < spiSectorSize.length; i++)
                {
                    spiAlignmentMultiplier[i].setValue(0.2 * ((Number)spiSectorSize[i].getValue()).intValue());
                }
            }
        };
        spiSectorSize = new JSpinner[4];
        min = 5; max = 380;
        for (int i = 0; i < spiSectorSize.length; i++)
        {
            val = Tools.getValueWithinBounds(getSectorSize(i, true), min, max).intValue();
            
            spiSectorSize[i] = new JSpinner(new SpinnerNumberModel(val, min, max, 1));
            spiSectorSize[i].setFont(font);
            spiSectorSize[i].addChangeListener(lstSectorSize);
        }
        
        spiAlignmentMultiplier = new JSpinner[4];
        double dval, dmin, dmax;
        dmin = 0.1; dmax = 1000;
        for (int i = 0; i < spiAlignmentMultiplier.length; i++)
        {
            dval = Tools.getValueWithinBounds(getSystemAlignment(i, true), dmin, dmax).doubleValue();
            
            spiAlignmentMultiplier[i] = new JSpinner(new SpinnerNumberModel(dval, dmin, dmax, 0.1));
            spiAlignmentMultiplier[i].setFont(font);
        }
        
        dmin = 0.01; dmax = 1;
        dval = Tools.getValueWithinBounds(getIrrGalDensity(), dmin, dmax).doubleValue();
        spiDensity = new JSpinner(new SpinnerNumberModel(dval, dmin, dmax, 0.01));
        
        spiMaxMinors = new JSpinner[9];
        min = 1; max = 30;
        for (int i = 0; i < 3; i++)
        {
            for (int j = 0; j < 3; j++)
            {
                val = Tools.getValueWithinBounds(getMaxMinors(i, j), min, max).intValue();

                spiMaxMinors[i*3 + j] = new JSpinner(new SpinnerNumberModel(val, min, max, 1));
                spiMaxMinors[i*3 + j].setFont(font);
            }
        }

        spiSpaceObjectsNum = new JSpinner[3];
        min = 5;
        for (int i = 0; i < 3; i++)
        {
            max = this.getMapHeight(i, true) * this.getMapWidth(i, true);
            val = Tools.getValueWithinBounds(getSpaceObjectsNum(i), min, max).intValue();
            
            spiSpaceObjectsNum[i] = new JSpinner(new SpinnerNumberModel(val, min, max, 1));
            spiSpaceObjectsNum[i].setFont(font);
        }
        
        dmin = 0.01; dmax = 1;
        dval = Tools.getValueWithinBounds(getStarRatio(), dmin, dmax).doubleValue();
        spiStarAnomalyRatio = new JSpinner(new SpinnerNumberModel(dval, dmin, dmax, 0.01));
        
        spiMinEmpireDistance = new JSpinner[12];
        min = 1;
        int k = 0;
        for (int i = 0; i < 3; i++)
        {
            for (int j = 0; j < 4; j++)
            {
                max = (int)Math.sqrt(
                        Math.pow(((Number)spiMapWidth[i].getValue()).intValue(), 2) +
                        Math.pow(((Number)spiMapHeight[i].getValue()).intValue(), 2)
                        ) / 2;

                val = Tools.getValueWithinBounds(getMinEmpireDistance(i, j), 0, max).intValue();

                spiMinEmpireDistance[k] = new JSpinner(new SpinnerNumberModel(val, 0, max, 1));
                spiMinEmpireDistance[k].setFont(font);
                
                k++;
            }
        }
        
        spiEllipseAxis = new JSpinner[6];
        min = 1; max = 999; k = 0;
        for (int i = 0; i < 3; i++)
        {
            for (int j = 0; j < 2; j++)
            {
                val = Tools.getValueWithinBounds(getEllipseGalAxis(i, j), min, max).intValue();
                
                spiEllipseAxis[k] = new JSpinner(new SpinnerNumberModel(val, min, max, 1));
                spiEllipseAxis[k].setFont(font);
                
                k++;
            }
        }
        
        spiSpiralEllipseAxis = new JSpinner[6]; // TODO: min max
        for (int i = 0; i < 3; i++)
        {
            for (int j = 0; j < 2; j++)
            {
                val = Tools.getValueWithinBounds(getSpiralGalEllipseAxis(i, j), 1, 999).intValue();

                k = i * 2 + j;
                spiSpiralEllipseAxis[k] = new JSpinner(new SpinnerNumberModel(val, 1, 999, 1));
                spiSpiralEllipseAxis[k].setFont(font);
            }
        }
        
        spiSpiralArmWidth = new JSpinner[3];
        for (int i = 0; i < 3; i++)
        {
            val = Tools.getValueWithinBounds(getSpiralArmWidth(i), 1, 999).intValue();

            spiSpiralArmWidth[i] = new JSpinner(new SpinnerNumberModel(val, 1, 999, 1));
            spiSpiralArmWidth[i].setFont(font);
        }
        
        spiRingWidth = new JSpinner[3];
        for (int i = 0; i < 3; i++)
        {
            val = Tools.getValueWithinBounds(this.getRingWidth(i), 1, Math.min(this.getMapHeight(i, false), this.getMapWidth(i, false))).intValue();

            spiRingWidth[i] = new JSpinner(new SpinnerNumberModel(val, 1, Math.min(this.getMapHeight(i, false), this.getMapWidth(i, false)), 1));
            spiRingWidth[i].setFont(font);
        }

        spiCoreSize = new JSpinner[3];
        for (int i = 0; i < 3; i++)
        {
            val = Tools.getValueWithinBounds(this.getRingCoreSize(i), 0, Math.min(this.getMapHeight(i, false), this.getMapWidth(i, false))).intValue();

            spiCoreSize[i] = new JSpinner(new SpinnerNumberModel(val, 0, Math.min(this.getMapHeight(i, false), this.getMapWidth(i, false)), 1));
            spiCoreSize[i].setFont(font);
        }
        
        spiSpiralArmArc = new JSpinner[3];
        for (int i = 0; i < 3; i++)
        {
            spiSpiralArmArc[i] = new JSpinner(new SpinnerNumberModel(this.getSpiralArmArc(i), 0.0, 999.0, 0.01));
            spiSpiralArmArc[i].setFont(font);
        }
        
        // check boxes
        chkNumericSectorID = new JCheckBox(strAllNumericSectorID, this.getNumericSectorID(true));
        chkNumericSectorID.setFont(font);
        
        // buttons
        btnSwitchCard = new JButton[2];
        
        btnSwitchCard[0] = new JButton("<<<"); //$NON-NLS-1$
        btnSwitchCard[0].setFont(font);
        btnSwitchCard[0].addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                ((CardLayout)cards.getLayout()).previous(cards);
            }
        });
        
        btnSwitchCard[1] = new JButton(">>>"); //$NON-NLS-1$
        btnSwitchCard[1].setFont(font);
        btnSwitchCard[1].addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                ((CardLayout)cards.getLayout()).next(cards);
            }
        });
    }
    
    private void placeComponents()
    {
        this.setLayout(new BorderLayout());

        GridBagConstraints c = new GridBagConstraints();
        c.insets = new Insets(5, 5, 5, 5);
        c.fill = GridBagConstraints.BOTH;

        cards = new JPanel(new CardLayout());
        
        JPanel[] card = new JPanel[3];

        // card 0
        int index = 0;
        card[index] = new JPanel(new GridBagLayout());
        
        // map size
        JPanel pnlMapSize = new JPanel(new GridBagLayout());
        pnlMapSize.setBorder(BorderFactory.createTitledBorder(strMapSize));
            c.gridy = 0;
            c.gridx = 1;
            pnlMapSize.add(lblMapWidth, c);
            c.gridx++;
            pnlMapSize.add(lblMapHeight, c);
        
            c.gridy++;
            c.gridx = 0;
            pnlMapSize.add(lblSmall[0], c);
            c.gridy++;
            pnlMapSize.add(lblMedium[0], c);
            c.gridy++;
            pnlMapSize.add(lblLarge[0], c);
        
            c.gridy = 1;
            for (int i = 0; i < 3; i++)
            {
                c.gridx = 1;
                pnlMapSize.add(spiMapWidth[i], c);
                c.gridx = 2;
                pnlMapSize.add(spiMapHeight[i], c);
                c.gridy++;
            }
        
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        card[index].add(pnlMapSize, c);
        
        // sector size
        JPanel pnlSectorSize = new JPanel(new GridBagLayout());
        pnlSectorSize.setBorder(BorderFactory.createTitledBorder(strSectorSize));
            c.gridx = 0;
            c.gridy = 0;
            pnlSectorSize.add(lblSmall[1], c);
            c.gridy++;
            pnlSectorSize.add(lblMedium[1], c);
            c.gridy++;
            pnlSectorSize.add(lblLarge[1], c);
        
            c.gridy = 0;
            c.gridx = 1;
            for (int i = 0; i < 3; i++)
            {
                pnlSectorSize.add(spiSectorSize[i], c);
                c.gridy++;
            }
            
            c.gridx = 0;
            pnlSectorSize.add(lblZoomIn[0], c);
            c.gridx++;
            pnlSectorSize.add(spiSectorSize[3], c);
            
        c.gridx = 1;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        card[index].add(pnlSectorSize, c);
        
        // system alignment
        JPanel pnlSystemAlignment = new JPanel(new GridBagLayout());
        pnlSystemAlignment.setBorder(BorderFactory.createTitledBorder(strSystemAlignment));
            c.gridx = 0;
            c.gridy = 0;
            pnlSystemAlignment.add(lblSmall[2], c);
            c.gridy++;
            pnlSystemAlignment.add(lblMedium[2], c);
            c.gridy++;
            pnlSystemAlignment.add(lblLarge[2], c);
            
            c.gridy = 0;
            c.gridx = 1;
            for (int i = 0; i < 3; i++)
            {
                pnlSystemAlignment.add(spiAlignmentMultiplier[i], c);
                c.gridy++;
            }
            
            c.gridx = 0;
            pnlSystemAlignment.add(lblZoomIn[1], c);
            c.gridx++;
            pnlSystemAlignment.add(spiAlignmentMultiplier[3], c);
            
        c.gridx = 2;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        card[index].add(pnlSystemAlignment, c);

        // irregular gal
        JPanel pnlIrregularGalaxy = new JPanel(new GridBagLayout());
        pnlIrregularGalaxy.setBorder(BorderFactory.createTitledBorder(strIrregularGalaxy));
            c.gridx = 0;
            c.gridy = 0;
            pnlIrregularGalaxy.add(lblDensity, c);
            c.gridx = 1;
            pnlIrregularGalaxy.add(spiDensity, c);
            
        c.gridx = 2;
        c.gridy = 2;
        c.gridwidth = 1;
        c.gridheight = 1;
        card[index].add(pnlIrregularGalaxy, c);
        
        // num of space objects
        JPanel pnlSpaceObjectsNum = new JPanel(new GridBagLayout());
        pnlSpaceObjectsNum.setBorder(BorderFactory.createTitledBorder(strNumSpaceObjects));
            c.gridx = 0;
            c.gridy = 0;
            c.gridwidth = 1;
            pnlSpaceObjectsNum.add(lblSmall[3], c);
            c.gridy++;
            pnlSpaceObjectsNum.add(lblMedium[3], c);
            c.gridy++;
            pnlSpaceObjectsNum.add(lblLarge[3], c);
            
            c.gridy = 0;
            c.gridx = 1;
            for (int i = 0; i < 3; i++)
            {
                pnlSpaceObjectsNum.add(spiSpaceObjectsNum[i], c);
                c.gridy++;
            }
            
        c.gridx = 2;
        c.gridy = 1;
        c.gridwidth = 1;
        c.gridheight = 1;
        card[index].add(pnlSpaceObjectsNum, c);
        
        c.gridwidth = 2;
        c.gridx = 0;
        c.gridy = 2;
        card[index].add(chkNumericSectorID, c);
        
        // minimum empire distances
        JPanel pnlMinEmpDist = new JPanel(new GridBagLayout());
        pnlMinEmpDist.setBorder(BorderFactory.createTitledBorder(strMinEmpDist));
            c.gridwidth = 1;
            c.gridx = 0;
            c.gridy = 1;
            pnlMinEmpDist.add(lblSmall[4], c);
            c.gridy++;
            pnlMinEmpDist.add(lblMedium[4], c);
            c.gridy++;
            pnlMinEmpDist.add(lblLarge[4], c);
            
            c.gridy = 0;
            c.gridx = 1;
            for (int i = 0; i < 4; i++)
            {
                pnlMinEmpDist.add(lblGalaxyShape[i], c);
                c.gridx++;
            }
            
            // max
            for (int i = 0; i < 3; i++)
            {
                c.gridy = i + 1;

                for (int j = 0; j < 4; j++)
                {
                    c.gridx = j + 1;
                    pnlMinEmpDist.add(spiMinEmpireDistance[i * 4 + j], c);
                }
            }
            
        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth = 2;
        c.gridheight = 1;
        card[index].add(pnlMinEmpDist, c);
        
        // card 1
        index++;
        card[index] = new JPanel(new GridBagLayout());

        // minor races
        JPanel pnlMinors = new JPanel(new GridBagLayout());
        pnlMinors.setBorder(BorderFactory.createTitledBorder(strMinorRaces));
            c.gridwidth = 1;
            c.gridx = 0;
            c.gridy = 1;
            pnlMinors.add(lblSmall[5], c);
            c.gridy++;
            pnlMinors.add(lblMedium[5], c);
            c.gridy++;
            pnlMinors.add(lblLarge[5], c);
            
            c.gridy = 0;
            c.gridx = 1;

            for (int i = 0; i < 3; i++)
            {
                pnlMinors.add(lblMinorsSetting[i], c);
                c.gridx++;
            }
            
            // max
            for (int i = 0; i < 3; i++)
            {
                c.gridy = i + 1;
                
                for (int j = 0; j < 3; j++)
                {
                    c.gridx = j + 1;
                    pnlMinors.add(spiMaxMinors[i * 3 + j], c);
                }
            }
            
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 2;
        card[index].add(pnlMinors, c);
        
        // ellipse axes
        JPanel pnlEllipseAxes = new JPanel(new GridBagLayout());
        pnlEllipseAxes.setBorder(BorderFactory.createTitledBorder(strEllipseAxes));
            c.gridheight = 1;
            c.gridx = 0;
            c.gridy = 1;
            pnlEllipseAxes.add(lblSmall[6], c);
            c.gridy++;
            pnlEllipseAxes.add(lblMedium[6], c);
            c.gridy++;
            pnlEllipseAxes.add(lblLarge[6], c);
            
            c.gridy = 0;
            c.gridx = 1;

            for (int i = 0; i < 2; i++)
            {
                pnlEllipseAxes.add(lblAxis[i], c);
                c.gridx++;
            }

            for (int i = 0; i < 3; i++)
            {
                c.gridy = i + 1;
                
                for (int j = 0; j < 2; j++)
                {
                    c.gridx = j + 1;
                    pnlEllipseAxes.add(spiEllipseAxis[i * 2 + j], c);
                }
            }
            
        c.gridx = 1;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 2;
        card[index].add(pnlEllipseAxes, c);
      
        // star anomaly ratio
        JPanel pnlStarAnomalyRatio = new JPanel(new GridBagLayout());
        pnlStarAnomalyRatio.setBorder(BorderFactory.createTitledBorder(strStarAnomalyRatio));
            c.gridheight = 1;
            c.gridx = 0;
            c.gridy = 0;
            pnlStarAnomalyRatio.add(lblStarAnomalyRatio, c);
            c.gridx = 1;
            pnlStarAnomalyRatio.add(spiStarAnomalyRatio, c);
            
        c.gridx = 1;
        c.gridy = 3;
        c.gridwidth = 1;
        c.gridheight = 1;
        card[index].add(pnlStarAnomalyRatio, c);
        
        // spiral settings
        JPanel pnlSpiralSettings = new JPanel(new GridBagLayout());
        pnlSpiralSettings.setBorder(BorderFactory.createTitledBorder(strSpiralGalSettings));
            c.gridx = 0;
            c.gridy = 1;
            pnlSpiralSettings.add(lblSmall[7], c);
            c.gridy++;
            pnlSpiralSettings.add(lblMedium[7], c);
            c.gridy++;
            pnlSpiralSettings.add(lblLarge[7], c);
            
            c.gridy = 0;
            c.gridx = 1;
    
            for (int i = 0; i < 3; i++)
            {
                pnlSpiralSettings.add(lblSpiralGalSettings[i], c);
                c.gridx++;
            }
    
            for (int i = 0; i < 3; i++)
            {
                c.gridy = i + 1;
                
                for (int j = 0; j < 3; j++)
                {
                    c.gridx = j + 1;
                    if (j < 2)
                    {
                        pnlSpiralSettings.add(spiSpiralEllipseAxis[i * 2 + j], c);
                    }
                    else
                    {
                        pnlSpiralSettings.add(spiSpiralArmWidth[i], c);
                    }
                }
            }
            
        c.gridx = 0;
        c.gridy = 2;
        c.gridwidth = 1;
        c.gridheight = 1;
        card[index].add(pnlSpiralSettings, c);

        // spiral arm arc multipliers
        JPanel pnlSpiralArmArc = new JPanel(new GridBagLayout());
        pnlSpiralArmArc.setBorder(BorderFactory.createTitledBorder(strSpiralArmArcMultiplier));
            c.gridy = 0;
            c.gridx = 0;
            
            for (int i = 0; i < 3; i++)
            {
                c.gridx = 0;
                pnlSpiralArmArc.add(lblSpiralArmArc[i], c);
                c.gridx = 1;
                pnlSpiralArmArc.add(spiSpiralArmArc[i], c);
                c.gridy++;
            }
        
        c.gridx = 1;
        c.gridy = 2;
        c.gridwidth = 1;
        c.gridheight = 1;
        card[index].add(pnlSpiralArmArc, c);
        
        // card 2
        index = 2;
        card[index] = new JPanel(new GridBagLayout());
        
        // ring gal settings
        JPanel pnlRingSettings = new JPanel(new GridBagLayout());
        pnlRingSettings.setBorder(BorderFactory.createTitledBorder(strRingGalSettings));
            c.gridx = 0;
            c.gridy = 1;
            pnlRingSettings.add(lblSmall[8], c);
            c.gridy++;
            pnlRingSettings.add(lblMedium[8], c);
            c.gridy++;
            pnlRingSettings.add(lblLarge[8], c);
            
            c.gridy = 0;
            c.gridx = 1;
    
            for (int i = 0; i < 2; i++)
            {
                pnlRingSettings.add(lblRingGalSettings[i], c);
                c.gridx++;
            }
    
            for (int i = 0; i < 3; i++)
            {
                c.gridy = i + 1;
                
                for (int j = 0; j < 2; j++)
                {
                    c.gridx = j + 1;
                    if (j == 0)
                    {
                        pnlRingSettings.add(spiRingWidth[i], c);
                    }
                    else
                    {
                        pnlRingSettings.add(spiCoreSize[i], c);
                    }
                }
            }
        
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        card[index].add(pnlRingSettings, c);

        // finish
        for (index = 0; index < card.length; index++)
        {
            cards.add(card[index], Integer.toString(index));
        }

        add(cards, BorderLayout.CENTER);
        
        JPanel pnlSwitchCards = new JPanel();
        pnlSwitchCards.setLayout(new BoxLayout(pnlSwitchCards, BoxLayout.X_AXIS));
        
        Dimension dim = new Dimension(10, btnSwitchCard[0].getPreferredSize().height + 10);
        
        pnlSwitchCards.add(Box.createRigidArea(dim));
        pnlSwitchCards.add(btnSwitchCard[0]);
        pnlSwitchCards.add(Box.createHorizontalGlue());
        pnlSwitchCards.add(btnSwitchCard[1]);
        pnlSwitchCards.add(Box.createRigidArea(dim));

        add(pnlSwitchCards, BorderLayout.SOUTH);
        
        ((CardLayout)cards.getLayout()).first(cards);
    }
    
    /* (non-Javadoc)
     * @see ue.gui.MainPanel#finalWarning()
     */
    @Override
    public void finalWarning() throws Exception
    {
        if (init_ok)
        {
            // map size
            for (int i = 0; i < 3; i++)
            {
                if (getMapWidth(i, false) != (Integer)spiMapWidth[i].getValue())
                {
                    setMapWidth(i, (Integer)spiMapWidth[i].getValue());
                }

                if (getMapHeight(i, false) != (Integer)spiMapHeight[i].getValue())
                {
                    setMapHeight(i, (Integer)spiMapHeight[i].getValue());
                }
            }
            // numeric sector id
            this.setNumericSectorID(chkNumericSectorID.isSelected());
            // sector size
            for (int i = 0; i < 4; i++)
            {
                setSectorSize(i, ((Number)spiSectorSize[i].getValue()).shortValue());
            }
            // alignment multipliers
            for (int i = 0; i < 4; i++)
            {
                setSystemAlignment(i, ((Number)spiAlignmentMultiplier[i].getValue()).floatValue());
            }
            // irregular galaxy density
            this.setIrrGalDensity(((Number)spiDensity.getValue()).floatValue());
            // minors
            for (int i = 0; i < 3; i++)
            {
                for (int j = 0; j < 3; j++)
                {
                    this.setMaxMinors(i, j, ((Number)spiMaxMinors[i*3 + j].getValue()).intValue());
                }
            }
            // number of objects
            for (int i = 0; i < 3; i++)
            {
                setSpaceObjectsNum(i, ((Number)spiSpaceObjectsNum[i].getValue()).intValue());
            }

            // star anomaly ratio
            this.setStarRatio(((Number)spiStarAnomalyRatio.getValue()).doubleValue());

            // min empire distance
            for (int i = 0; i < 3; i++)
            {
                for (int j = 0; j < 4; j++)
                {
                    this.setMinEmpireDistance(i, j, ((Number)spiMinEmpireDistance[i * 4 + j].getValue()).intValue());
                }
            }

            // ellipse axes
            for (int i = 0; i < 3; i++)
            {
                for (int j = 0; j < 2; j++)
                {
                    this.setEllipseGalAxis(i, j, ((Number)spiEllipseAxis[i * 2 + j].getValue()).intValue());
                }
            }

            // spiral gal axes
            for (int i = 0; i < 3; i++)
            {
                for (int j = 0; j < 2; j++)
                {
                    this.setSpiralGalEllipseAxis(i, j, ((Number)spiSpiralEllipseAxis[i * 2 + j].getValue()).intValue());
                }
            }

            // spiral arm width
            for (int i = 0; i < 3; i++)
            {
                spiSpiralArmWidth[i].commitEdit();
                this.setSpiralArmWidth(i, ((Number)spiSpiralArmWidth[i].getValue()).intValue());
            }

            // ring width
            for (int i = 0; i < 3; i++)
            {
                this.setRingWidth(i, ((Number)spiRingWidth[i].getValue()).intValue());
            }

            // ring core size
            for (int i = 0; i < 3; i++)
            {
                this.setRingCoreSize(i, ((Number)spiCoreSize[i].getValue()).intValue());
            }

            // spiral arm arc multipliers
            for (int i = 0; i < 3; i++)
            {
                spiSpiralArmArc[i].commitEdit();
                this.setSpiralArmArc(i, ((Number)spiSpiralArmArc[i].getValue()).doubleValue());
            }
        }
    }
    
    private int getMapWidth(int size, boolean test) throws Exception
    {
        short width;
        int ds = 2;
        byte[] expected = TREK.getSegment(width_address[size][0], ds).getData();
        
        width = Tools.toShort(expected, true);
        
        if (test)
        {
            for (long address : width_address[size])
            {
                try
                {
                    InternalSegment.checkIfValid(TREK.getSegment(address, ds).getData(), expected, true);
                }
                catch (Exception e)
                {
                    throw new Exception(e.getMessage() + " (0x" + Long.toHexString(address) + ")"); //$NON-NLS-1$ //$NON-NLS-2$
                }
            }
            try
            {
                expected = Tools.toByte((short) (5*width), true);
                InternalSegment.checkIfValid(TREK.getSegment(width5_address[size], ds).getData(), expected, true);
            }
            catch (Exception e)
            {
                throw new Exception(e.getMessage() + " (0x" + Long.toHexString(width5_address[size]) + ")"); //$NON-NLS-1$ //$NON-NLS-2$
            }
        }
        
        return width;
    }

    private void setMapWidth(int size, int value) throws Exception
    {
        if (value != getMapWidth(size, false))
        {
            GenericSegment seg;
            byte[] width = Tools.toByte((short) value, true);
            
            for (long address : width_address[size])
            {
                seg = new GenericSegment(address, width);
                seg.markChanged();
                TREK.addInternalFile(seg);
            }

            width = Tools.toByte((short) (5*value), true);
            seg = new GenericSegment(width5_address[size], width);
            seg.markChanged();
            TREK.addInternalFile(seg);

            setSpaceObjectsLimit();
            setAIScanRange();
            
            // make sure fixes are in place else invalid value doesn't get overwritten
            int val = getMinEmpireDistance(0, 2);
            setMinEmpireDistance(0, 2, val + 1);
            setMinEmpireDistance(0, 2, val);
            val = getMinEmpireDistance(2, 2);
            setMinEmpireDistance(2, 2, val + 1);
            setMinEmpireDistance(2, 2, val);
            val = getSpiralGalEllipseAxis(2, 0);
            setSpiralGalEllipseAxis(2, 0, val + 1);
            setSpiralGalEllipseAxis(2, 0, val);
        }
    }

    private int getMapHeight(int size, boolean test) throws Exception
    {
        short height;
        int ds = 2;
        byte[] expected = TREK.getSegment(height_address[size][0], ds).getData();
        
        height = Tools.toShort(expected, true);
        
        if (test)
        {
            for (long address : height_address[size])
            {
                try
                {
                    InternalSegment.checkIfValid(TREK.getSegment(address, ds).getData(), expected, true);
                }
                catch (Exception e)
                {
                    throw new Exception(e.getMessage() + " (0x" + Long.toHexString(address) + ")"); //$NON-NLS-1$ //$NON-NLS-2$
                }
            }
            
            try
            {
                expected = Tools.toByte((short) (5*height), true);
                InternalSegment.checkIfValid(TREK.getSegment(height5_address[size], ds).getData(), expected, true);
            }
            catch (Exception e)
            {
                throw new Exception(e.getMessage() + " (0x" + Long.toHexString(height5_address[size]) + ")"); //$NON-NLS-1$ //$NON-NLS-2$
            }
        }
        
        return height;
    }
    
    private void setMapHeight(int size, int value) throws Exception
    {
        if (value != getMapHeight(size, false))
        {
            GenericSegment seg;
            byte[] height = Tools.toByte((short) value, true);
            
            for (long address : height_address[size])
            {
                seg = new GenericSegment(address, height);
                seg.markChanged();
                TREK.addInternalFile(seg);
            }
            
            height = Tools.toByte((short) (5*value), true);
            seg = new GenericSegment(height5_address[size], height);
            seg.markChanged();
            TREK.addInternalFile(seg);

            setSpaceObjectsLimit();
            setAIScanRange();
        }
    }
    
    private void setNumericSectorID(boolean enabled) throws Exception
    {
        if (enabled != getNumericSectorID(false))
        {
            GenericSegment seg;
            
            int nen = 1;
            
            if (enabled)
            {
                nen = 0;
            }
            
            for (int i = 0; i < numericSectorID_address.length; i++)
            {
                seg = new GenericSegment(numericSectorID_address[i], numericSectorIDCode[nen][i]);
                seg.markChanged();
                TREK.addInternalFile(seg);
            }
        }
    }
    
    private boolean getNumericSectorID(boolean test) throws Exception
    {
        GenericSegment seg;
        
        boolean enabled = InternalSegment.checkIfValid(TREK.getSegment(numericSectorID_address[0], numericSectorIDCode[0][0].length).getData(), numericSectorIDCode[0][0], false);
        
        int nen = 1;
        
        if (enabled)
        {
            nen = 0;
        }
        
        if (test)
        {
            for (int i = 3; i < 4; i++)
            {
                seg = TREK.getSegment(numericSectorID_address[i], numericSectorIDCode[nen][i].length);
                    
                try
                {
                    InternalSegment.checkIfValid(seg.getData(), numericSectorIDCode[nen][i], true);
                }
                catch (Exception e)
                {
                    throw new Exception(e.getMessage() + " (0x" + Long.toHexString(numericSectorID_address[i]) + ")"); //$NON-NLS-1$ //$NON-NLS-2$
                }
            }
        }
        
        return enabled;
    }
    
    private short getExpectedSectorSize(int gsize)
    {
        short x = (short) Math.floor(500/(Integer)spiMapWidth[gsize].getValue());
        short y = (short) Math.floor(380/(Integer)spiMapHeight[gsize].getValue());
        
        if (x <= 5 || y <= 5)
        {
            x = y = 5;
        }
        
        if (x < y)
        {
            return x;
        }
        else
        {
            return y;
        }
    }
    
    private void setSectorSize(int gsize, short size) throws Exception
    {
        if (size != getSectorSize(gsize, false))
        {
            GenericSegment seg;
            byte[] data = Tools.toByte(size, true);
            
            for (long address : sectorSize_address[gsize])
            {
                seg = new GenericSegment(address, data);
                seg.markChanged();
                TREK.addInternalFile(seg);
            }
        }
    }

    private short getSectorSize(int gsize, boolean test) throws Exception
    {
        short size;
        int ds = 2;
        byte[] expected = TREK.getSegment(sectorSize_address[gsize][0], ds).getData();
        
        size = Tools.toShort(expected, true);
        
        if (test)
        {
            for (long address : sectorSize_address[gsize])
            {
                try
                {
                    InternalSegment.checkIfValid(TREK.getSegment(address, ds).getData(), expected, true);
                }
                catch (Exception e)
                {
                    throw new Exception(e.getMessage() + " (0x" + Long.toHexString(address) + ")"); //$NON-NLS-1$ //$NON-NLS-2$
                }
            }
        }
        
        return size;
    }
    
    private void setSystemAlignment(int gsize, float align) throws Exception
    {
        if (align != getSystemAlignment(gsize, false))
        {
            GenericSegment seg;
            byte[] data = Tools.toByte(align, true);
            
            for (long address : systemAlignment_address[gsize])
            {
                seg = new GenericSegment(address, data);
                seg.markChanged();
                TREK.addInternalFile(seg);
            }
        }
    }

    private float getSystemAlignment(int gsize, boolean test) throws Exception
    {
        float align;
        int ds = 4;
        byte[] expected = TREK.getSegment(systemAlignment_address[gsize][0], ds).getData();
        
        align = Tools.toFloat(expected, true);
        
        if (test)
        {
            for (long address : systemAlignment_address[gsize])
            {
                try
                {
                    InternalSegment.checkIfValid(TREK.getSegment(address, ds).getData(), expected, true);
                }
                catch (Exception e)
                {
                    throw new Exception(e.getMessage() + " (0x" + Long.toHexString(address) + ")"); //$NON-NLS-1$ //$NON-NLS-2$
                }
            }
        }
        
        return align;
    }
    
    private void setIrrGalDensity(float density) throws Exception
    {
        if (density != getIrrGalDensity())
        {
            GenericSegment seg;
            byte[] data = Tools.toByte(density, true);
            
            seg = new GenericSegment(irregularGalaxyDensity_address, data);
            seg.markChanged();
            TREK.addInternalFile(seg);
        }
    }

    private float getIrrGalDensity() throws Exception
    {
        byte[] data = TREK.getSegment(irregularGalaxyDensity_address, 4).getData();
        
        return Tools.toFloat(data, true);        
    }

    private int getMaxMinors(int galsize, int setting) throws FileNotFoundException
    {
        MinorRacesNumbers num = (MinorRacesNumbers) TREK.getInternalFile(Trek.fileMAX_MINORS_NUMBER, true);
        int val = num.getMinorsNumber(galsize, setting) - 1;
        if (val > 30)
        {
            return 30;
        }
        else if (val < 1)
        {
            return 1;
        }
        
        return val;
    }

    private void setMaxMinors(int galsize, int setting, int value) throws FileNotFoundException
    {
        MinorRacesNumbers num = (MinorRacesNumbers) TREK.getInternalFile(Trek.fileMAX_MINORS_NUMBER, true);
        num.setMinorsNumber(galsize, setting, value + 1);
    }
    
    private void setSpaceObjectsLimit() throws Exception
    {
        short limit = 0, tmp;
        
        for (int i = 0; i < 3; i++)
        {
            tmp = (short)(this.getMapHeight(i, false) * this.getMapWidth(i, false));
            
            if (limit < tmp)
            {
                limit = tmp;
            }
        }
        
        if (limit != getSpaceObjectsLimit(false) && limit > 0)
        {
            GenericSegment seg;
            byte[] data = Tools.toByte(limit, true);
            
            for (long address : numberOfSpaceObjectsLimit_address)
            {
                seg = new GenericSegment(address, data);
                seg.markChanged();
                TREK.addInternalFile(seg);
            }
        }
    }

    private short getSpaceObjectsLimit(boolean test) throws Exception
    {
        byte[] data = TREK.getSegment(numberOfSpaceObjectsLimit_address[0], 2).getData();
        
        try
        {
            InternalSegment.checkIfValid(TREK.getSegment(numberOfSpaceObjectsLimit_address[1], 2).getData(), data, true);
        }
        catch (Exception e)
        {
            throw new Exception(e.getMessage() + " (0x" + Long.toHexString(numberOfSpaceObjectsLimit_address[1]) + ")"); //$NON-NLS-1$ //$NON-NLS-2$
        }
        
        return Tools.toShort(data, true);
    }    

    private int getAIScanRange(boolean test) throws Exception
    {
        byte[] data = TREK.getSegment(aiScanRange_address, 1).getData();

        if (test)
        {
            try
            {
                InternalSegment.checkIfValid(data, new byte[]{(byte)0xB9}, true);
            }
            catch (Exception e)
            {
                throw new Exception(e.getMessage() + " (0x" + Long.toHexString(numberOfSpaceObjectsLimit_address[1]) + ")"); //$NON-NLS-1$ //$NON-NLS-2$
            }
        }

        return Tools.toShort(TREK.getSegment(aiScanRange_address + 1, 4).getData(), true);
    }    

    private void setAIScanRange() throws Exception
    {
        int max = 0, tmp;

        for (int i = 0; i < 3; i++)
        {
            tmp = Math.max(this.getMapHeight(i, false), this.getMapWidth(i, false));

            if (max < tmp)
            {
                max = tmp;
            }
        }

        if (getAIScanRange(true) != max)
        {
            byte[] data = Tools.toByte(max, true);
            GenericSegment seg = new GenericSegment(aiScanRange_address + 1, data);
            seg.markChanged();
            TREK.addInternalFile(seg);
        }
    }

    private int getSpaceObjectsNum(int i) throws FileNotFoundException
    {
        AverageNumberOfSpaceObjects anso;
        String n;
        
        switch (i)
        {
            case 0:
            {
                n = Trek.fileAVG_SPACE_OBJECT_NUM_S;
                break;
            }
            case 1:
            {
                n = Trek.fileAVG_SPACE_OBJECT_NUM_M;
                break;
            }
            default:
            {
                n = Trek.fileAVG_SPACE_OBJECT_NUM_L;
            }
        }
        
        anso = (AverageNumberOfSpaceObjects) TREK.getInternalFile(n, true);
        
        return anso.getNumber();
    }
    
    private void setSpaceObjectsNum(int i, int num) throws Exception
    {
        AverageNumberOfSpaceObjects anso;
        String n;
        
        switch (i)
        {
            case 0:
            {
                n = Trek.fileAVG_SPACE_OBJECT_NUM_S;
                break;
            }
            case 1:
            {
                n = Trek.fileAVG_SPACE_OBJECT_NUM_M;
                break;
            }
            default:
            {
                n = Trek.fileAVG_SPACE_OBJECT_NUM_L;
            }
        }
        
        anso = (AverageNumberOfSpaceObjects) TREK.getInternalFile(n, true);
        
        anso.setNumber(num);
/*
        if (i == 2 && anso.madeChanges())
        {
            GenericSegment seg = TREK.getSegment(galacticDensity[TREK.getVersion()], 4);

            float sectors = this.getMapHeight(i, false) * this.getMapWidth(i, false);
            float density = anso.getNumber() * 2500 / sectors;

            seg.setData(Tools.toByte(density, true));
            TREK.addInternalFile(seg);
        }*/
    }
    
    private void setStarRatio(double ratio) throws Exception
    {
        if (ratio != getStarRatio())
        {
            GenericSegment seg;
            byte[] data = Tools.toByte(ratio, true);
            
            seg = new GenericSegment(starAnomalyRatio_address, data);
            seg.markChanged();
            TREK.addInternalFile(seg);
        }
    }

    private double getStarRatio() throws Exception
    {
        byte[] data = TREK.getSegment(starAnomalyRatio_address, 8).getData();
        
        return Tools.toDouble(data, true);        
    }

    private int getMinEmpireDistance(int galSize, int galShape) throws FileNotFoundException
    {
        int i = galSize * 4 + galShape;
        
        if (i == 2 || i == 10)
        {
            if (TREK.getSegment(minEmpireDistance_address[i], 1).getData()[0] == (byte)0xB9)
            {
                // fix is in place, read value
                return Tools.toInt(TREK.getSegment(minEmpireDistance_address[i] + 1, 4).getData(), true);
            }
            else
            {
                // read the shared value
                return Tools.toInt(TREK.getSegment(widthOfRing_address[galSize], 4).getData(), true);
            }
        }
        else
        {
            return Tools.toInt(TREK.getSegment(minEmpireDistance_address[i], 4).getData(), true);
        }
    }
    
    private void setMinEmpireDistance(int galSize, int galShape, int dist) throws FileNotFoundException
    {
        if (getMinEmpireDistance(galSize, galShape) == dist)
        {
            return;
        }
        
        int i = galSize * 4 + galShape;
        GenericSegment seg;
        
        if (i == 2 || i == 10)
        {
            seg = new GenericSegment(minEmpireDistance_address[i], new byte[]{(byte)0xB9}); // mov ecx instruction
            seg.markChanged();
            TREK.addInternalFile(seg);

            seg = new GenericSegment(minEmpireDistance_address[i] + 1, Tools.toByte(dist, true)); // value
            seg.markChanged();
            TREK.addInternalFile(seg);
            
            byte[] b = new byte[5];             // fill empty space with nop
            Arrays.fill(b, (byte)0x90);
            
            seg = new GenericSegment(minEmpireDistance_address[i] + 5, b);
            seg.markChanged();
            TREK.addInternalFile(seg);
            
            // jump to setting gal size
            b = new byte[]{(byte)0xE9, (byte)0x8E, (byte)0xFF, (byte)0xFF, (byte)0xFF};
            
            if (i == 10)
            {
                b[1] = (byte)0xAA;
            }
            
            seg = new GenericSegment(minEmpireDistance_address[i] + 10, b);
            seg.markChanged();
            TREK.addInternalFile(seg);
        }
        else
        {
            seg = new GenericSegment(minEmpireDistance_address[i], Tools.toByte(dist, true));
            seg.markChanged();
            TREK.addInternalFile(seg);
        }
    }
    
    private int getEllipseGalAxis(int galSize, int y) throws FileNotFoundException
    {
        int i = y * 3 + galSize;
        
        return Tools.toInt(TREK.getSegment(ellipseGalAxis_address[i], 4).getData(), true);
    }
    
    private void setEllipseGalAxis(int galSize, int y, int value) throws FileNotFoundException
    {
        if (getEllipseGalAxis(galSize, y) != value)
        {
            int i = y * 3 + galSize;
            
            GenericSegment seg = new GenericSegment(ellipseGalAxis_address[i], Tools.toByte(value, true));
            seg.markChanged();
            TREK.addInternalFile(seg);
        }
    }
    
    private int getSpiralGalEllipseAxis(int galSize, int y) throws FileNotFoundException
    {
        int i = y * 3 + galSize;
        
        if (i == 2)
        {
            if (TREK.getSegment(spiralGalEllipseAxis_address[i], 1).getData()[0] == (byte)0xBE)
            {
                // fix is in place, read value
                return Tools.toInt(TREK.getSegment(spiralGalEllipseAxis_address[i] + 1, 4).getData(), true);
            }
            else
            {
                // read the shared value
                return Tools.toInt(TREK.getSegment(spiralGalEllipseAxis_address[5], 4).getData(), true);
            }
        }
        else
        {
            return Tools.toInt(TREK.getSegment(spiralGalEllipseAxis_address[i], 4).getData(), true);
        }
    }
    
    private void setSpiralGalEllipseAxis(int galSize, int y, int value) throws FileNotFoundException
    {
        if (getSpiralGalEllipseAxis(galSize, y) != value)
        {
            int i = y * 3 + galSize;
            GenericSegment seg;
            
            if (i == 2)
            {
                seg = new GenericSegment(spiralGalEllipseAxis_address[i], new byte[]{(byte)0xBE}); // mov esi instruction
                seg.markChanged();
                TREK.addInternalFile(seg);

                seg = new GenericSegment(spiralGalEllipseAxis_address[i] + 1, Tools.toByte(value, true)); // value
                seg.markChanged();
                TREK.addInternalFile(seg);
                
                byte[] b = new byte[5];             // fill empty space with nop
                Arrays.fill(b, (byte)0x90);
                
                seg = new GenericSegment(spiralGalEllipseAxis_address[i] + 5, b);
                seg.markChanged();
                TREK.addInternalFile(seg);
                
                // jump to code that sets map width instead of exit
                b = new byte[]{(byte)0xE9, (byte)0x74, (byte)0xFF, (byte)0xFF, (byte)0xFF};
                
                seg = new GenericSegment(spiralGalEllipseAxis_address[i] + 10, b);
                seg.markChanged();
                TREK.addInternalFile(seg);
            }
            else
            {
                if (i == 5)
                {
                    // make sure the fix is in place
                    if (getSpiralGalEllipseAxis(2, 0) == getSpiralGalEllipseAxis(2, 1))
                    {
                        int val = getSpiralGalEllipseAxis(2, 0);
                        setSpiralGalEllipseAxis(2, 0, val + 1);
                        setSpiralGalEllipseAxis(2, 0, val);
                    }
                }
                
                seg = new GenericSegment(spiralGalEllipseAxis_address[i], Tools.toByte(value, true));
                seg.markChanged();
                TREK.addInternalFile(seg);
            }
        }
    }
    
    private int getSpiralArmWidth(int galSize) throws FileNotFoundException
    {
        return Tools.toInt(TREK.getSegment(spiralArmWidth_address[galSize], 4).getData(), true);
    }
    
    private void setSpiralArmWidth(int galSize, int value) throws FileNotFoundException
    {
        if (getSpiralArmWidth(galSize) != value)
        {
            GenericSegment seg = new GenericSegment(spiralArmWidth_address[galSize], Tools.toByte(value, true));
            seg.markChanged();
            TREK.addInternalFile(seg);
        }
    }
    
    private int getRingWidth(int galSize) throws FileNotFoundException
    {
        return Tools.toInt(TREK.getSegment(widthOfRing_address[galSize], 4).getData(), true);
    }
    
    private void setRingWidth(int galSize, int value) throws FileNotFoundException
    {
        if (getRingWidth(galSize) != value)
        {
            GenericSegment seg;

            if (galSize != 1)
            {
                    // make sure the fix is in place
                    if (getRingWidth(galSize) == getMinEmpireDistance(galSize, 2))
                    {
                        int val = getMinEmpireDistance(galSize, 2);
                        setMinEmpireDistance(galSize, 2, val + 1);
                        setMinEmpireDistance(galSize, 2, val);
                    }
            }
            
            seg = new GenericSegment(widthOfRing_address[galSize], Tools.toByte(value, true));
            seg.markChanged();
            TREK.addInternalFile(seg);
        }
    }
    
    private int getRingCoreSize(int galSize) throws FileNotFoundException
    {
        return Tools.toInt(TREK.getSegment(sizeOfRingCore_address[galSize], 4).getData(), true);
    }
    
    private void setRingCoreSize(int galSize, int value) throws FileNotFoundException
    {
        if (getRingCoreSize(galSize) != value)
        {
            GenericSegment seg = new GenericSegment(sizeOfRingCore_address[galSize], Tools.toByte(value, true));
            seg.markChanged();
            TREK.addInternalFile(seg);
        }
    }
    
    private double getSpiralArmArc(int arc) throws FileNotFoundException
    {
        return Tools.toDouble(TREK.getSegment(spiralArmArc_address[arc], 8).getData(), true) / Math.PI;
    }
    
    private void setSpiralArmArc(int arc, double value) throws FileNotFoundException
    {
        if (Math.abs(getSpiralArmArc(arc) - value) >= 0.005)
        {
            GenericSegment seg = new GenericSegment(spiralArmArc_address[arc], Tools.toByte(value * Math.PI, true));
            seg.markChanged();
            TREK.addInternalFile(seg);
        }
    }
    
    @Override
    public String getHelpFileName()
    {
            return "GalaxyGeneration.html"; //$NON-NLS-1$
    }
}
