/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * 
 * File: MEStartCondGUI.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Last modified: 7.5.2008
 */

package ue.gui.exe;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import ue.Language;
import ue.UE;
import ue.edit.exe.trek.MEStartingConditions;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.dic.Lexicon;
import ue.edit.res.stbof.rst.Race;
import ue.gui.MainPanel;
import ue.util.SettingsManager;

/**
 * Used for setting major empire starting conditions.
 */
@SuppressWarnings("serial") //$NON-NLS-1$
public class MEStartCondGUI  extends MainPanel {
	private MEStartingConditions CONDITIONS;
	private Race RACE;
	private Lexicon LEXICON;
	// gui
	private JTextField[] txtPOP = new JTextField[25];
	private JTextField[] txtCRED = new JTextField[25];
	private JLabel lblPOP = new JLabel(Language.getString("MEStartCondGUI.0")); //$NON-NLS-1$
	private JLabel lblCRED = new JLabel(Language.getString("MEStartCondGUI.1")); //$NON-NLS-1$
	private JLabel[] lblERA = new JLabel[10];
	private JLabel[] lblRACE = new JLabel[10];
	
	public MEStartCondGUI(MEStartingConditions cond, Stbof st){
		if (cond == null || st == null) return;
		CONDITIONS = cond;
		
		try{
			RACE = (Race)st.getInternalFile(Stbof.fileRACE_RST, true);
			LEXICON = (Lexicon)st.getInternalFile(Stbof.fileLEXICON_DIC, true);
			
			// CREATE
			for (int r = 0; r < 5; r++){
				for (int a = 0; a < 5; a++){
					txtPOP[(r*5) + a] = new JTextField(Integer.toString(CONDITIONS.getPopulation(r, a)));
					txtCRED[(r*5) + a] = new JTextField(Integer.toString(CONDITIONS.getCredits(r, a)));
					
				}
				lblRACE[r] = new JLabel(RACE.getName(r));
				lblRACE[r+5] = new JLabel(RACE.getName(r));
			}
			
			lblERA[0] = new JLabel(LEXICON.getEntry(88)); // beginning
			lblERA[5] = new JLabel(LEXICON.getEntry(88));
			lblERA[1] = new JLabel(LEXICON.getEntry(252)); // early
			lblERA[6] = new JLabel(LEXICON.getEntry(252));
			lblERA[2] = new JLabel(LEXICON.getEntry(234)); // developed
			lblERA[7] = new JLabel(LEXICON.getEntry(234));
			lblERA[3] = new JLabel(LEXICON.getEntry(312)); // expanded
			lblERA[8] = new JLabel(LEXICON.getEntry(312));
			lblERA[4] = new JLabel(LEXICON.getEntry(4)); // advanced
			lblERA[9] = new JLabel(LEXICON.getEntry(4));
			
			// SETUP
			
			Font def = UE.SETTINGS.getDefaultFont();
			
			for (int i = 0; i < 25; i++){
				txtPOP[i].setFont(def);
				txtCRED[i].setFont(def);
		        if (i < 10){
					lblRACE[i].setFont(def);
					lblERA[i].setFont(def);
				}
			}
			
			// ADD
			setLayout(new GridBagLayout());
			GridBagConstraints c = new GridBagConstraints();
			c.fill=GridBagConstraints.BOTH;
			c.insets = new Insets(5, 5, 5, 5);
			c.gridx=0;
			c.gridy=0;
			c.gridwidth=1;
			c.gridheight=1;
	
			// pnlPOP
			JPanel pnlPOP = new JPanel(new GridBagLayout());
	
				c.gridwidth=6;
				pnlPOP.add(lblPOP, c);
				c.gridwidth=1;
				c.gridy=1;
				for (int i = 0; i < 5; i++){
					c.gridx = i + 1;
					pnlPOP.add(lblERA[i], c);
				}
	
				c.gridy=2;
				c.gridx=0;
				for (int i = 0; i < 5; i++){
					c.gridy = i + 2;
					pnlPOP.add(lblRACE[i], c);
				}
				
				c.gridy=2;
				c.gridx=1;
				
				for (int i = 0; i < 5; i++){
					for (int j = 0; j < 5; j++){
						c.gridy=i+2;
						c.gridx=j+1;
						pnlPOP.add(txtPOP[i*5 + j], c);
					}
				}
				
			c.gridx=0;
			c.gridy=0;
			add(pnlPOP, c);
			// pnlCRED
			JPanel pnlCRED = new JPanel(new GridBagLayout());
	
				c.gridwidth=6;
				pnlCRED.add(lblCRED, c);
				c.gridwidth=1;
				c.gridy=1;
				for (int i = 0; i < 5; i++){
					c.gridx = i + 1;
					pnlCRED.add(lblERA[i+5], c);
				}
	
				c.gridy=2;
				c.gridx=0;
				for (int i = 0; i < 5; i++){
					c.gridy = i + 2;
					pnlCRED.add(lblRACE[i+5], c);
				}
				
				c.gridy=2;
				c.gridx=1;
				
				for (int i = 0; i < 5; i++){
					for (int j = 0; j < 5; j++){
						c.gridy=i+2;
						c.gridx=j+1;
						pnlCRED.add(txtCRED[i*5 + j], c);
					}
				}
				
			c.gridx=0;
			c.gridy=1;
			add(pnlCRED, c);
		}catch (Exception e){
			UE.displayError(e);
		}
	}
	
	@Override
	public void finalWarning() throws Exception
    {
		int val;
		for (int r = 0; r < 5; r++)
        {
			for (int a = 0; a < 5; a++)
            {
                val = Integer.parseInt(txtPOP[(r*5)+a].getText());
                CONDITIONS.setPopulation(r, a, val);

                val = Integer.parseInt(txtCRED[(r*5)+a].getText());
                CONDITIONS.setCredits(r, a, val);
			}
		}
	}

}
