/**
 * Ultimate Editor
 * Copyright (C) 2009 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: ShipBuildingGUI.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 14.4.2009
 */
package ue.gui.exe;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.FileNotFoundException;
import java.util.Arrays;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;

import ue.Language;
import ue.UE;
import ue.edit.FileManager;
import ue.edit.exe.GenericSegment;
import ue.edit.exe.Trek;
import ue.edit.exe.trek.SpecialShips;
import ue.gui.MainPanel;

/**
 * @author Alan Podlesek
 */
@SuppressWarnings("serial") //$NON-NLS-1$
public class ShipBuildingGUI extends MainPanel
{
    private static final long ship_auto_address = 0x51AA9;

    private static final long base_auto_address = 0x51AD8;

    private static final byte[] auto_disabled = new byte[]
    {
        (byte)0x90, (byte)0x90, (byte)0x90, (byte)0x90, (byte)0x90
    };

    private static final byte[] ship_auto_enabled = 
            new byte[]{(byte)0xE8, (byte)0xB2, (byte)0x7C, (byte)0xFF, (byte)0xFF};

    private static final byte[] base_auto_enabled = 
            new byte[]{(byte)0xE8, (byte)0xB3, (byte)0x7B, (byte)0xFF, (byte)0xFF};

    private Trek TREK;
    private SpecialShips SP;
    
    private JCheckBox chkSPECIAL;
    private JCheckBox chkMINOR_SHIPS;
    private JCheckBox chkMINOR_SHIPS_SIMPLE;
    private JCheckBox chkAUTO_UPGRADE_SHIPS;
    private JCheckBox chkAUTO_UPGRADE_BASE;
    
    public ShipBuildingGUI() throws FileNotFoundException
    {
        TREK = (Trek)UE.FILES.getFile(FileManager.P_TREK);
        
        SP = (SpecialShips)TREK.getInternalFile(Trek.fileSPECIAL_SHIPS, true);
        
        setupComponents();
        placeComponents();
    }
    
    private void setupComponents() throws FileNotFoundException
    {
        Font def = UE.SETTINGS.getDefaultFont();

        chkSPECIAL = new JCheckBox(Language.getString("ShipBuildingGUI.0"), SP.areSPEnabled()); //$NON-NLS-1$
        chkSPECIAL.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                if (SP != null)
                {
                    SP.setSPEnabled(chkSPECIAL.isSelected());
                }
                chkMINOR_SHIPS.setSelected(SP.areMinorShipsEnabled());
                chkMINOR_SHIPS_SIMPLE.setSelected(SP.areMinorShipsSimpleEnabled());
            }
        });
        chkMINOR_SHIPS = new JCheckBox(Language.getString("ShipBuildingGUI.1"), SP.areMinorShipsEnabled()); //$NON-NLS-1$
        chkMINOR_SHIPS.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                if (SP != null)
                {
                    SP.setMinorShipsMod(chkMINOR_SHIPS.isSelected());
                }
                chkSPECIAL.setSelected(SP.areSPEnabled());
                chkMINOR_SHIPS_SIMPLE.setSelected(SP.areMinorShipsSimpleEnabled());
            }
        });
        chkMINOR_SHIPS_SIMPLE = new JCheckBox(Language.getString("ShipBuildingGUI.2"), SP.areMinorShipsSimpleEnabled()); //$NON-NLS-1$
        chkMINOR_SHIPS_SIMPLE.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                if (SP != null)
                {
                    SP.setMinorShipsSimpleMod(chkMINOR_SHIPS_SIMPLE.isSelected());
                }
                chkSPECIAL.setSelected(SP.areSPEnabled());
                chkMINOR_SHIPS.setSelected(SP.areMinorShipsEnabled());
            }
        });

        GenericSegment gen = TREK.getSegment(ship_auto_address, 5);

        boolean enabled = true;

        if (Arrays.equals(gen.getData(), auto_disabled))
        {
            enabled = false;
        }

        chkAUTO_UPGRADE_SHIPS = new JCheckBox("Ship auto-upgrade", enabled); //$NON-NLS-1$
        chkAUTO_UPGRADE_SHIPS.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                if (chkAUTO_UPGRADE_SHIPS.isSelected())
                {
                    TREK.addInternalFile(
                            new GenericSegment(ship_auto_address,
                            ship_auto_enabled, true));
                }
                else
                {
                    TREK.addInternalFile(
                            new GenericSegment(ship_auto_address, auto_disabled, true));
                }
            }
        });

        gen = TREK.getSegment(base_auto_address, 5);

        enabled = true;

        if (Arrays.equals(gen.getData(), auto_disabled))
        {
            enabled = false;
        }

        chkAUTO_UPGRADE_BASE = new JCheckBox("Outpost auto-upgrade", enabled); //$NON-NLS-1$
        chkAUTO_UPGRADE_BASE.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                if (chkAUTO_UPGRADE_BASE.isSelected())
                {
                    TREK.addInternalFile(
                            new GenericSegment(base_auto_address,
                            base_auto_enabled, true));
                }
                else
                {
                    TREK.addInternalFile(
                            new GenericSegment(base_auto_address, auto_disabled, true));
                }
            }
        });

        chkSPECIAL.setFont(def);
        chkMINOR_SHIPS.setFont(def);
        chkMINOR_SHIPS_SIMPLE.setFont(def);
        chkAUTO_UPGRADE_SHIPS.setFont(def);
        chkAUTO_UPGRADE_BASE.setFont(def);
    }
    
    private void placeComponents()
    {
        BoxLayout blayout = new BoxLayout(this, BoxLayout.Y_AXIS);
        this.setLayout(blayout);
        
        add(chkSPECIAL);
        add(chkMINOR_SHIPS_SIMPLE);
        add(chkMINOR_SHIPS);
        add(chkAUTO_UPGRADE_SHIPS);
        add(chkAUTO_UPGRADE_BASE);
    }
    
    /* (non-Javadoc)
     * @see ue.gui.MainPanel#finalWarning()
     */
    @Override
    public void finalWarning() throws Exception
    {
        if (SP != null)
        {
            SP.setSPEnabled(chkSPECIAL.isSelected());
            SP.setMinorShipsMod(chkMINOR_SHIPS.isSelected());
            SP.setMinorShipsSimpleMod(chkMINOR_SHIPS_SIMPLE.isSelected());
        }
    }

    @Override
    public String getHelpFileName()
    {
            return "ShipBuilding.html"; //$NON-NLS-1$
    }
}
