/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2008 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: ShipMapRangesGUI.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 11.10.2008
 */
package ue.gui.exe;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JToggleButton;

import ue.Language;
import ue.UE;
import ue.edit.CanBeEdited;
import ue.edit.FileManager;
import ue.edit.exe.Trek;
import ue.edit.exe.trek.BaseMapRanges;
import ue.edit.exe.trek.MapRangeTechBonuses;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.rst.Race;
import ue.gui.MainPanel;
import ue.util.SettingsManager;

/**
 * Used for setting ship map ranges.
 */
@SuppressWarnings("serial") //$NON-NLS-1$
public class ShipMapRangesGUI extends MainPanel
{
    private ButtonGroup gEmpires = new ButtonGroup();
    private JToggleButton[] btnEmpire = new JToggleButton[5];
    private JTextField[] txtValue = new JTextField[33];
    private JLabel[] lblHeader = new JLabel[4];
    private JLabel[] lblLevel = new JLabel[11];
    
    private BaseMapRanges base;
    private MapRangeTechBonuses bonus;
    
    private int selected_empire = -1;
    
    public ShipMapRangesGUI(Trek trek)
    {
        try
        {
            // SETUP
            
            base = (BaseMapRanges)trek.getInternalFile(Trek.fileBASE_MAP_RANGES, true);
            bonus = (MapRangeTechBonuses)trek.getInternalFile(Trek.fileMAP_RANGE_BONUSES, true);
            
            CanBeEdited stbof = UE.FILES.getFile(FileManager.P_STBOF);
            
            this.setupComponents(stbof);
            this.placeComponents();
            
            btnEmpire[0].doClick();
        }
        catch (Exception e)
        {
            UE.displayError(e);
        }
    }
    
    private void setupComponents(CanBeEdited stbof) throws Exception
    {
        Font def = UE.SETTINGS.getDefaultFont();

        // buttons
        int maxLen = 0;
        
        String str = Language.getString("ShipMapRangesGUI.0"); //$NON-NLS-1$
        Race race = null;
        
        if (stbof != null)
        {
            race = (Race)stbof.getInternalFile(Stbof.fileRACE_RST, true);
        }
        
        for (int i = 0; i < 5; i++)
        {
            if (race == null)
            {
                btnEmpire[i] = new JToggleButton(str.replace("%1", Integer.toString(i + 1))); //$NON-NLS-1$
            }
            else
            {
                btnEmpire[i] = new JToggleButton(race.getName(i));
            }
            
            gEmpires.add(btnEmpire[i]);
            
            if (btnEmpire[i].getMinimumSize().width > maxLen)
            {
                maxLen = btnEmpire[i].getMinimumSize().width;
            }
            else
            {
                btnEmpire[i].setMaximumSize(new Dimension(maxLen, btnEmpire[i].getMinimumSize().height));
            }
            btnEmpire[i].setAlignmentX(Component.CENTER_ALIGNMENT);
            
            btnEmpire[i].addActionListener(new BtnListener(i));
            btnEmpire[i].setFont(def);
        }
        
        // range labels
        lblHeader[0] = new JLabel(Language.getString("ShipMapRangesGUI.1")); //$NON-NLS-1$
        lblHeader[1] = new JLabel(Language.getString("ShipMapRangesGUI.2")); //$NON-NLS-1$
        lblHeader[2] = new JLabel(Language.getString("ShipMapRangesGUI.3")); //$NON-NLS-1$
        lblHeader[3] = new JLabel(Language.getString("ShipMapRangesGUI.4")); //$NON-NLS-1$
        
        maxLen = 0;
        
        for (int i = 0; i < 4; i++)
        {
            if (lblHeader[i].getMinimumSize().width > maxLen)
            {
                maxLen = lblHeader[i].getMinimumSize().width;
            }
            else
            {
                lblHeader[i].setMaximumSize(new Dimension(maxLen, lblHeader[i].getMinimumSize().height));
            }
            lblHeader[i].setAlignmentX(Component.CENTER_ALIGNMENT);
            
            if (i > 0)
            {
                lblHeader[i].setHorizontalAlignment(JLabel.CENTER);
            }

            lblHeader[i].setFont(def);
        }

        // level labels
        str = Language.getString("ShipMapRangesGUI.5"); //$NON-NLS-1$
        
        for (int i = 0; i < 11; i++)
        {
            lblLevel[i] = new JLabel(str.replace("%1", Integer.toString(i + 1))); //$NON-NLS-1$
            
            if (lblLevel[i].getMinimumSize().width > maxLen)
            {
                maxLen = lblLevel[i].getMinimumSize().width;
            }
            else
            {
                lblLevel[i].setMaximumSize(new Dimension(maxLen, lblLevel[i].getMinimumSize().height));
            }
            
            lblLevel[i].setAlignmentX(Component.CENTER_ALIGNMENT);

            lblLevel[i].setFont(def);
        }
        
        // values
        for (int i = 0; i < 33; i++)
        {
            txtValue[i] = new JTextField();
            txtValue[i].setHorizontalAlignment(JTextField.RIGHT);
            txtValue[i].setMaximumSize(new Dimension(maxLen, txtValue[i].getMinimumSize().height));
            txtValue[i].setFont(def);
        }
    }
    
    private void placeComponents()
    {
        this.setLayout(new BorderLayout());
        
        // menu
        JPanel pnlMenu = new JPanel();
        BoxLayout blayout = new BoxLayout(pnlMenu, BoxLayout.Y_AXIS);
        
        pnlMenu.setLayout(blayout);
        pnlMenu.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
        
        for (JToggleButton btn : btnEmpire)
        {
            pnlMenu.add(btn);
        }
        
        
        
        // table
        JPanel pnlTable = new JPanel();
        GridLayout glayout = new GridLayout(12, 4, 0, 0);
        
        pnlTable.setLayout(glayout);
        pnlTable.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

        int j = 0, k = 0;
        
        for (int i = 0; i < 12; i++)
        {
            if (i == 0)
            {
                for (int l = 0; l < 4; l++)
                {
                    pnlTable.add(lblHeader[l]);
                }
            }
            else
            {
                pnlTable.add(lblLevel[j]);
                
                k = j * 3;

                pnlTable.add(txtValue[k]);
                pnlTable.add(txtValue[k + 1]);
                pnlTable.add(txtValue[k + 2]);
                
                j++;
            }
        }
        
        // add to main
        this.add(pnlMenu, BorderLayout.WEST);
        this.add(pnlTable, BorderLayout.CENTER);
    }
    
    /* (non-Javadoc)
     * @see ue.gui.MainPanel#finalWarning()
     */
    @Override
    public void finalWarning() throws Exception
    {
        int k = 0;
        int[] range = new int[33];
        int b; 
        
        for (int l = 1; l <= 11; l++)
        {
            for (int r = 0; r < 3; r++)
            {
                range[k] = Integer.parseInt(txtValue[k].getText());

                if (range[k] > 0)
                {
                    b = base.getBaseRange(selected_empire, r);

                    bonus.setRangeBonus(l, selected_empire, r, range[k] - b);
                }
                
                k++;
            }
        }
    }
    
    private class BtnListener implements ActionListener
    {
        private int empire;
        
        public BtnListener(int index)
        {
            empire = index;
        }
        
        @Override
        public void actionPerformed(ActionEvent ae)
        {
            try
            {
                if (selected_empire >= 0)
                {
                    finalWarning();
                }
                
                selected_empire = empire;
                
                int k = 0;
                int range = 0;

                for (int l = 1; l <= 11; l++)
                {
                    for (int r = 0; r < 3; r++)
                    {
                        range = base.getBaseRange(empire, r) + bonus.getRangeBonus(l, empire, r);
                        txtValue[k++].setText(Integer.toString(range));
                    }
                }
            }
            catch (Exception e)
            {
                UE.displayError(e);
            }
        }
    };
}
