/**
 * Ultimate Editor (dev)
 * Copyright (C) 2009 Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: TrekPatchesGUI.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 6.4.2011
 */

package ue.gui.exe;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.ByteArrayOutputStream;
import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import ue.UE;
import ue.edit.CanBeEdited;
import ue.edit.FileManager;
import ue.edit.exe.Trek;
import ue.edit.exe.trek.EmpireImgPrefixes;
import ue.edit.exe.trek.NebulaHCGraphic;
import ue.edit.exe.trek.SelectPhaserHobFile;
import ue.edit.exe.trek.SelectPlasmaHobFile;
import ue.edit.exe.trek.SelectTorpedoHobFile;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.smt.Objstruc;
import ue.edit.res.stbof.sst.ShipList;
import ue.gui.MainPanel;
import ue.util.AniOrCur;
import ue.util.GenericFile;
import ue.util.TargaImage;
import ue.util.Tga2Ani;

/**
 * @author Alan
 */
public class TrekPatchesGUI extends MainPanel
{
    private Trek TREK;

    private NebulaHCGraphic nebulaHCGraphic;
    private SelectPlasmaHobFile plasmaHobLoading;
    private SelectPhaserHobFile phaserHobLoading;
    private SelectTorpedoHobFile torpedoHobLoading;

    // components
    private JCheckBox chkNebulaHCGraphic;
    private JCheckBox chkPlasmaPhaserHobLoading;
    private JCheckBox chkTorpedoHobLoading;

    public TrekPatchesGUI(Trek exe) throws Exception
    {
        TREK = exe;

        // load required files
        nebulaHCGraphic = (NebulaHCGraphic)TREK.getInternalFile(Trek.fileNEBULA_HC_GRAPHIC, true);
        plasmaHobLoading = (SelectPlasmaHobFile)TREK.getInternalFile(Trek.fileSELECTPLASMAHOBFILE, true);
        phaserHobLoading = (SelectPhaserHobFile)TREK.getInternalFile(Trek.fileSELECTPHASERHOBFILE, true);
        torpedoHobLoading = (SelectTorpedoHobFile)TREK.getInternalFile(Trek.fileSELECTTORPEDOHOBFILE, true);

        setupComponents();
        placeComponents();
    }

    private void setupComponents()
    {
        Font font = UE.SETTINGS.getDefaultFont();

        chkNebulaHCGraphic = new JCheckBox(
                "Disable hardcoded nebula image - allowing the use of animations for the object (may modify stbof.res)");
        chkNebulaHCGraphic.setSelected(!nebulaHCGraphic.isHCGraphicEnabled());
        chkNebulaHCGraphic.setFont(font);
        chkNebulaHCGraphic.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                boolean enabled = !chkNebulaHCGraphic.isSelected();

                nebulaHCGraphic.setHCGraphicEnabled(enabled);

                if (!enabled)
                {
                    CanBeEdited file = UE.FILES.getFile(FileManager.P_STBOF);

                    if (file != null)
                    {
                        try
                        {
                            Stbof res = (Stbof) file;

                            Objstruc objstruc = (Objstruc) res.getInternalFile(Stbof.fileOBJSTRUC_SMT, false);

                            String graph = objstruc.getGraph(2);

                            if (res.hasInternalFile(graph + ".tga")
                                    && !res.hasInternalFile(graph + ".ani"))
                            {
                                TargaImage tga = res.getTargaImage(graph + ".tga", false);
                                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                                tga.save(baos);

                                Tga2Ani T2A = new Tga2Ani();
                                T2A.addFrame(graph + ".tga", baos.toByteArray());
                                baos.reset();

                                AniOrCur aoc = T2A.getAniOrCur(false);
                                aoc.save(baos);

                                res.addInternalFile(new GenericFile(graph + ".ani", baos.toByteArray()));
                                baos.close();
                            }
                        }
                        catch (Exception ex)
                        {
                            UE.displayError(ex);
                        }
                    }
                }
                else
                {
                    CanBeEdited file = UE.FILES.getFile(FileManager.P_STBOF);

                    if (file != null)
                    {
                        try
                        {
                            Stbof res = (Stbof) file;

                            Objstruc objstruc = (Objstruc) res.getInternalFile(Stbof.fileOBJSTRUC_SMT, false);

                            String graph = objstruc.getGraph(2);

                            if (res.hasInternalFile(graph + ".ani")
                                    && !res.hasInternalFile(graph + ".tga"))
                            {
                                AniOrCur aoc = res.getAnimation(graph + ".ani", false);
                                TargaImage tga = aoc.getTgaFrame(0);

                                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                                tga.save(baos);

                                res.addInternalFile(new GenericFile(graph + ".tga", baos.toByteArray()));
                                baos.close();
                            }
                        }
                        catch (Exception ex)
                        {
                            UE.displayError(ex);
                        }
                    }
                }
            }
        });

        chkPlasmaPhaserHobLoading = new JCheckBox(
                "Enable per ship plasma/phaser hob file selection (may modify stbof.res/shiplist.sst)");
        chkPlasmaPhaserHobLoading.setSelected(plasmaHobLoading.isPatched());
        chkPlasmaPhaserHobLoading.setFont(font);
        chkPlasmaPhaserHobLoading.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    boolean enabled = !plasmaHobLoading.isPatched();

                    plasmaHobLoading.setPatched(enabled);
                    phaserHobLoading.setPatched(enabled);

                    if (enabled)
                    {
                        // fix prefixes
                        CanBeEdited file = UE.FILES.getFile(FileManager.P_STBOF);
                        EmpireImgPrefixes prefixes = (EmpireImgPrefixes)TREK.getInternalFile(Trek.fileEMPIRE_IMAGE_PREFIXES, false);

                        if (file != null)
                        {
                            Stbof res = (Stbof) file;

                            ShipList ships = (ShipList) res.getInternalFile(Stbof.fileSHIPLIST_SST, true);
                            int r;

                            for (int i = 0; i < ships.getNumberOfEntries(); i++)
                            {
                                r = ships.getRace(i);
                                
                                if (r < 5)
                                {
                                    ships.setPhaserPrefix(i, prefixes.getEmpirePrefix(r));
                                }
                                else
                                {
                                    ships.setPhaserPrefix(i, 'M');
                                }
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    UE.displayError(ex);
                }
            }
        });
        chkTorpedoHobLoading = new JCheckBox(
                "Enable per ship torpedo hob file selection (may modify stbof.res/shiplist.sst)");
        chkTorpedoHobLoading.setSelected(torpedoHobLoading.isPatched());
        chkTorpedoHobLoading.setFont(font);
        chkTorpedoHobLoading.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    boolean enabled = !torpedoHobLoading.isPatched();

                    torpedoHobLoading.setPatched(enabled);

                    if (enabled)
                    {
                        // fix prefixes
                        CanBeEdited file = UE.FILES.getFile(FileManager.P_STBOF);
                        EmpireImgPrefixes prefixes = (EmpireImgPrefixes)TREK.getInternalFile(Trek.fileEMPIRE_IMAGE_PREFIXES, false);

                        if (file != null)
                        {
                            Stbof res = (Stbof) file;

                            ShipList ships = (ShipList) res.getInternalFile(Stbof.fileSHIPLIST_SST, true);
                            int r;

                            for (int i = 0; i < ships.getNumberOfEntries(); i++)
                            {
                                r = ships.getRace(i);

                                if (r < 5)
                                {
                                    ships.setTorpedoPrefix(i, prefixes.getEmpirePrefix(r));
                                }
                                else
                                {
                                    ships.setTorpedoPrefix(i, 'M');
                                }
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    UE.displayError(ex);
                }
            }
        });
    }

    private void placeComponents()
    {
        Font font = UE.SETTINGS.getDefaultBigFont();
        
        this.setLayout(new BorderLayout());
        this.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

        JPanel pnlOptions = new JPanel(new GridLayout(0, 1));
        pnlOptions.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

        JLabel lbl = new JLabel("Graphics");
        lbl.setFont(font);
        pnlOptions.add(lbl);

            pnlOptions.add(chkNebulaHCGraphic);
            pnlOptions.add(chkPlasmaPhaserHobLoading);
            pnlOptions.add(chkTorpedoHobLoading);

        JScrollPane pane = new JScrollPane(pnlOptions);

        add(pane, BorderLayout.NORTH);
    }

    @Override
    public void finalWarning() throws Exception
    {
    }
}
