/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: HelpWindow.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 25.6.2008
*/

package ue.gui.help;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.IOException;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLEditorKit;

import ue.UE;
import ue.util.Tools;

/**
 * Show a window with text.
 */
@SuppressWarnings("serial") //$NON-NLS-1$
public class HelpWindow extends JFrame implements HyperlinkListener {
	private JTextPane helpPane;
	private JScrollPane scrPane;
	
	/**
	 * Constructor.
	 * @param title The title of the panel whose help will be displayed.
	 * @param help The help text.
	 */
	public HelpWindow(String title, String path, String html){
		// window
		this.setTitle(title);
		this.setIconImage(UE.ICON);

		// layout
		BorderLayout layout = new BorderLayout(0, 0);

		helpPane = new JTextPane();
		helpPane.setEditorKit(new HTMLEditorKit());
		helpPane.setCaretColor(new Color(255, 255, 255, 255));
		helpPane.setEditable(false);
		helpPane.setOpaque(false);
		helpPane.addHyperlinkListener(this);
		loadText(title, path, html);
		
		scrPane = new JScrollPane(helpPane);
		scrPane.setOpaque(false);

		JPanel panel = new JPanel(layout);
		panel.add(scrPane, BorderLayout.CENTER);

		this.add(panel);
		
		// show
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		this.setSize(new Dimension(400, 480));
		
		this.setLocation((d.width - 400)/2, (d.height - 480)/2);
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

		this.setVisible(true);
	}
	
	private void loadText(String title, String path, String file)
    {
		try {
			
			String html = UE.FILES.getHelpFile(path, file);

            if (html != null)
            {
                int start = html.indexOf("<title>"); //$NON-NLS-1$
                int end = html.indexOf("</title>"); //$NON-NLS-1$


                if (start != -1 && end !=-1){
                     String html_title = html.substring(start + 7, end);

                     this.setTitle(title + " - " + html_title); //$NON-NLS-1$
                }
            }
            else
            {
                setTitle(title);
            }

		    helpPane.setText(html);
		    helpPane.setCaretPosition(0);
		} catch (IOException e) {
            setTitle(title);

		    helpPane.setText("<p><b>Could not load help file.</b></p><br>Error: " + e.getLocalizedMessage());
		    helpPane.setCaretPosition(0);
		}
	}
	
    @Override
	public void hyperlinkUpdate(HyperlinkEvent event) {
		if (event.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
			if (event.getURL() == null){
				// same page (help)
				helpPane.scrollToReference(event.getDescription().substring(1));
			}else{
				// external page (call default browser)
				Tools.openInDefaultBrowser(event.getURL().toString());
			}
		}
	}
}
