/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: GameinfoGUI.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui.sav;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JTextField;

import ue.Language;
import ue.UE;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.dic.Lexicon;
import ue.edit.res.stbof.rst.Race;
import ue.edit.sav.GameInfo;
import ue.gui.MainPanel;
import ue.util.ID;
import ue.util.SettingsManager;

/**Used to edit game info of a saved game.*/
@SuppressWarnings("serial")  //$NON-NLS-1$
public class GameInfoGUI extends MainPanel{
	private GameInfo GAME;
	private Race RACE;
	private Lexicon LEX;
	private boolean READY = false;
	//items
	JComboBox jcbMULTI = new JComboBox();
	JComboBox jcbPLEMP = new JComboBox();
	//JComboBox jcbDIFF = new JComboBox();
	JTextField txtNAME = new JTextField(50);
	JTextField txtTURN = new JTextField(5);
	JLabel[] lblLABEL = new JLabel[5];
	/**Constructor.*/
	public GameInfoGUI(GameInfo gi, Stbof st){
		if (gi==null || st==null) {
			return;
		}
		try{
			RACE = (Race)st.getInternalFile(Stbof.fileRACE_RST, true);
			LEX = (Lexicon)st.getInternalFile(Stbof.fileLEXICON_DIC, true);
		}catch (Exception er){
			UE.displayError(er);
			return;
		}
		if (RACE==null || LEX==null) {
			return;
		}
		//associate
		GAME=gi;
		//build
		lblLABEL[0] = new JLabel(Language.getString("GameInfoGUI.0")); //$NON-NLS-1$
		lblLABEL[1] = new JLabel(Language.getString("GameInfoGUI.1")); //$NON-NLS-1$
		lblLABEL[2] = new JLabel(Language.getString("GameInfoGUI.2")); //$NON-NLS-1$
		lblLABEL[3] = new JLabel(Language.getString("GameInfoGUI.3")); //$NON-NLS-1$
		lblLABEL[4] = new JLabel(Language.getString("GameInfoGUI.4")); //$NON-NLS-1$
		//set up
		Font def = UE.SETTINGS.getDefaultFont();
		jcbMULTI.setFont(def);
		txtNAME.setFont(def);
		jcbPLEMP.setFont(def);
		//jcbDIFF.setFont(def);
		txtTURN.setFont(def);
		for (int i=0; i<5; i++){
			lblLABEL[i].setFont(def);
		}
		//multiplayer
		jcbMULTI.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				if (READY){
					if (jcbMULTI.getSelectedIndex()==0){
						GAME.setMultiplayer(false);
					}else{
						GAME.setMultiplayer(true);
					}
				}
			}
		});
		//player empire
		jcbPLEMP.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				if (READY){
					try{
						GAME.setPlayerEmpire((byte)jcbPLEMP.getSelectedIndex());
					}catch (Exception ze){
						UE.displayError(ze);
					}
				}
			}
		});
		//difficulty
		//jcbDIFF.addActionListener(new ActionListener(){
		//	public void actionPerformed(ActionEvent ae){
		//		if (READY){
		//			try{
		//				GAME.setDifficulty((byte)jcbDIFF.getSelectedIndex());
		//			}catch (InvalidArgumentsException ze){
		//				displayError(ze, 1);
		//			}
		//		}
		//	}
		//});
		//lay out
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets=new Insets(5, 5, 5, 5);
		c.fill=GridBagConstraints.BOTH;
		//up
		c.gridx=1;
		c.gridy=0;
		c.gridwidth=2;
		add(txtNAME, c);
		c.gridwidth--;
		c.gridy++;
		add(txtTURN, c);
		c.gridy++;
		add(jcbMULTI, c);
		c.gridy++;
		add(jcbPLEMP, c);
		//c.gridy++;
		//add(jcbDIFF, c);
		c.gridx=0;
		for (int i=0; i<4; i++){
			c.gridy=i;
			add(lblLABEL[i], c);
		}
		//fill
		jcbMULTI.addItem(Language.getString("GameInfoGUI.5")); //$NON-NLS-1$
		jcbMULTI.addItem(Language.getString("GameInfoGUI.6")); //$NON-NLS-1$
		for (int i=0; i<5; i++){
			try{
				jcbPLEMP.addItem(new ID(RACE.getName(i), i));
			}catch (Exception ze){
				UE.displayError(ze);
			}
		}
		//int i=0;
		//try{
		//	jcbDIFF.addItem(new ID(LEX.getEntry(1150), i++));
		//	jcbDIFF.addItem(new ID(LEX.getEntry(256), i++));
		//	jcbDIFF.addItem(new ID(LEX.getEntry(873), i++));
		//	jcbDIFF.addItem(new ID(LEX.getEntry(433), i++));
		//	jcbDIFF.addItem(new ID(LEX.getEntry(482), i++));
		//}catch (InvalidArgumentsException ze){
		//	displayError(ze, 1);
		//}
		//set
		txtNAME.setText(GAME.getGameName());
		txtTURN.setText(Integer.toString(GAME.getTurn()));
		if (GAME.isMultiplayer()) {
			jcbMULTI.setSelectedIndex(1);
		} else {
			jcbMULTI.setSelectedIndex(0);
		}
		jcbPLEMP.setSelectedIndex(GAME.getPlayerEmpire());
		//jcbDIFF.setSelectedIndex(GAME.getDifficulty());
		READY=true;
	}
	//final warning
	@Override
	public void finalWarning() throws Exception{
        GAME.setGameName(txtNAME.getText());
        GAME.setTurn(Integer.parseInt(txtTURN.getText()));
	}
}
