/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package ue.gui.sav;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.image.BufferedImage;
import java.awt.image.RescaleOp;
import java.io.FileNotFoundException;
import java.util.Arrays;
import javax.swing.JButton;
import ue.Language;
import ue.UE;
import ue.edit.res.stbof.sst.ShipList;
import ue.exception.InvalidArgumentsException;
import ue.util.TargaImage;
import ue.util.Tools;

/**
 *
 * @author Alan
 */
public class MapButton extends JButton
{
    private MapPanel map;
    private int x, y, objType = -1, index;
    private Point link = null;

    private TargaImage check = null;
    private TargaImage main = null;
    private TargaImage base = null;
    private TargaImage[] tf_icon = new TargaImage[7];

    private boolean[][] mov_icon = new boolean[6][9];
    private boolean[][] movs_icon = new boolean[6][9];

    private String sector = null;
    private String title = null;
    private String objdesc = null;
    private String shipdesc = null;

    public MapButton(MapPanel pnl, int x, int y)
    {
        map = pnl;
        this.x = x;
        this.y = y;

        int h = map.sectors.getHSectors();
        index = y * h + x;

        setFocusPainted(false);
        setOpaque(true);
        setForeground(Color.WHITE);
        setIconTextGap(0);
        setPreferredSize(new Dimension(50, 50));
        setMinimumSize(new Dimension(20, 20));
        setBorder(map.normalBorder);

        try
        {
            sector = map.sectors.getDescription(index, false);
        }
        catch (Exception ex)
        {
            UE.displayError(ex);
        }
    }

    public void clearMovementFlags(boolean selection_only)
    {
        for (int r = 0; r < 6; r++)
        {
            if (!selection_only)
            {
                Arrays.fill(movs_icon[r], false);
            }

            Arrays.fill(mov_icon[r], false);
        }
    }

    public void refresh
            (
                boolean ownership,
                boolean object,
                boolean military,
                boolean paths
            )
            throws InvalidArgumentsException, FileNotFoundException
    {
        String pic;

        if (ownership)
        {
            check = null;

            // ownership
            int owner = map.sectors.getOwnerMask(index);

            switch (owner)
            {
                case 0:
                {
                    title = map.lexicon.getEntry(1382).replace("%s", sector);
                    break;
                }
                case 1:
                case 2:
                case 4:
                case 8:
                case 16:
                {
                    owner = map.sectors.getOwnerRace(index);

                    pic = map.empImgPrefixes.getEmpirePrefix(owner) + "_check.tga";

                    if (map.res.hasInternalFile(pic))
                    {
                        check = map.res.getTargaImage(pic, true);
                    }

                    pic = map.races.getName(owner);
                    title = map.lexicon.getEntry(1107).replace("%s", sector) + " (" + pic + ")";

                    break;
                }
                default:
                {
                    pic = "check.tga";

                    if (map.res.hasInternalFile(pic))
                    {
                        check = map.res.getTargaImage(pic, true);
                    }

                    title = map.lexicon.getEntry(187).replace("%s", sector);
                }
            }
        }

        if (object)
        {
            main = null;
            link = null;

            // draw star if present
            int item_index = map.sectors.getSystem(index);

            if (item_index >= 0)
            {
                //image
                pic = map.systems.getGraphic(item_index) + "map.tga"; //$NON-NLS-1$

                main = map.res.getTargaImage(pic, true);

                int r = map.systems.getControling(item_index);

                if (r >= 0 && r < map.races.getNumberOfEntries())
                {
                    pic = map.races.getName(r);
                    title = map.lexicon.getEntry(1107).replace("%s", sector) + " (" + pic + ")";
                }
                else
                {
                    r = map.systems.getResident(item_index);

                    if (r >= 0 && r < map.races.getNumberOfEntries())
                    {
                        pic = map.races.getName(r);
                        title = map.lexicon.getEntry(1107).replace("%s", sector) + " (" + pic + ")";
                    }
                }

                objdesc = "<b>" + map.systems.getName(item_index) + "</b>";
                objType = map.systems.getType(item_index);
                objdesc = objdesc + "<br>" + getObjType(objType);

                r = map.systems.getPlanetNames(item_index).length;

                objdesc = objdesc + "<br>" + r;

                if (r == 1)
                {
                    objdesc = objdesc + " " + map.lexicon.getEntry(1153);
                }
                else
                {
                    objdesc = objdesc + " " + map.lexicon.getEntry(623);
                }

                int pop = map.systems.getPopulation(item_index);

                if (pop > 0)
                {
                    objdesc = objdesc + "<br>" + map.lexicon.getEntry(919) + " " + pop;

                    // ground combat
                    r = map.systems.getResident(item_index);

                    if (r >= 0 && r < map.races.getNumberOfEntries())
                    {
                        double techMulti = Tools.toDouble(map.groundCombatTechMulti.getData(), true);

                        techMulti = techMulti * map.techInfo.getTechLevel((short)r, 6);

                        double defense = pop * map.races.getCombatMulti(r) * (1.0 + techMulti);

                        r = map.systems.getControling(item_index);
                        
                        if (r >= 0 && r < 5)
                        {
                            defense = defense *
                                    (1.0 + map.empireInfo.getGroundCombatBonus(r)/100.0);
                        }

                        int num;
                        
                        if (r < 0 || r > 4)
                        {
                            r = 5;
                        }

                        for (short bid : map.strcInfo.getBuildings((short)item_index))
                        {
                            num = map.strcInfo.getPoweredOnCount((short)item_index, bid);

                            if (num > 0 && map.edifice.getBoniType(bid) == 37)
                            {
                                defense = defense * (1.0 + num * map.edifice.getBoni(bid, r)/100.0);
                            }
                        }
                        
                        objdesc = objdesc + "<br>" + map.lexicon.getEntry(422) + " " +
                                (int)Math.floor(defense);
                    }

                    // defense
                    int num = 0;

                    objdesc = objdesc + "<br><b>" + map.lexicon.getEntry(218) + "</b>";

                    for (short bid : map.strcInfo.getBuildings((short)item_index))
                    {
                        if (map.edifice.getBoniType(bid) == 19 || map.edifice.getBoniType(bid) == 17)
                        {
                            num = map.strcInfo.getCount((short)item_index, bid);

                            if (num > 1)
                            {
                                objdesc = objdesc + "<br>" + num + " " + map.edifice.getPluralName(bid);
                            }
                            else
                            {
                                objdesc = objdesc + "<br>" + num + " " + map.edifice.getName(bid);
                            }
                        }
                    }

                    if (num == 0)
                    {
                        objdesc = objdesc + "<br><i>" + map.lexicon.getEntry(872) + "</i>";
                    }
                }
            }
            else
            {
                // stellar object image
                item_index = map.sectors.getStellarObject(index);

                if (item_index >= 0)
                {
                    //image
                    pic = map.objects.getAni(item_index) + "map.tga"; //$NON-NLS-1$

                    main = map.res.getTargaImage(pic, true);

                    objdesc = "<b>" + map.objects.getName(item_index) + "</b>";
                    objType = map.objects.getType(item_index);
                    objdesc = objdesc + "<br>" + getObjType(objType);

                    if (map.objects.getType(item_index) == 10)
                    {
                        objdesc = objdesc + "<br>" + map.lexicon.getEntry(1250) + ": ";

                        if (map.objects.getLink(item_index) >= 0)
                        {
                            objdesc = objdesc + map.objects.getName(map.objects.getLink(item_index));

                            int[] p = map.objects.getPosition(map.objects.getLink(item_index));

                            link = new Point(p[0], p[1]);
                        }
                        else
                        {
                            objdesc = objdesc + "<i>" + map.lexicon.getEntry(872) + "</i>";
                        }
                    }
                }
            }
        }

        if (military)
        {
            Arrays.fill(tf_icon, null);
            base = null;
            boolean military_present = false;

            // base
            shipdesc = "<b>" + map.lexicon.getEntry(642) + "</b>";

            if (objdesc != null)
            {
                shipdesc = "<br><br>" + shipdesc;
            }

            int item_index = map.sectors.getOutpost(index);

            if (item_index >= 0)
            {
                military_present = true;

                item_index = map.outposts.getIndex((short)item_index);

                int ship = map.outposts.getShipID(item_index);

                pic = Character.toString(map.empImgPrefixes.getEmpirePrefix(map.outposts.getOwner(item_index)));

                if (map.outposts.getType(item_index) == 6)
                {
                    pic = "i_" + pic.toLowerCase() + "o11.tga";
                }
                else
                {
                    pic = "i_" + pic.toLowerCase() + "s.tga";
                }

                if (!map.res.hasInternalFile(pic))
                {
                    pic = null;
                }

                if (pic != null)
                {
                    base = map.res.getTargaImage(pic, true);
                }

                shipdesc = shipdesc + "<br>" + map.outposts.getName(item_index)
                        + " (" + map.ships.getClass(ship) + ")";
            }

            // ships
            int[] tf = map.gWTForce.getSectorTaskForceNumbersByRace(map.races, y, x);
            int n, num = map.races.getNumberOfEntries();
            boolean minor = false, alien = false;
            int tf_index = 0;

            for (int r = 0; r < tf.length; r++)
            {
                n = tf[r];

                if (n > 0)
                {
                    military_present = true;

                    if (r < num)
                    {
                        shipdesc = shipdesc + "<br>" + n + " " + map.races.getName(r)
                                    + " " + map.lexicon.getEntry(1252);
                    }
                    else
                    {
                        shipdesc = shipdesc + "<br>" + n + " "
                                + map.ships.getIDs(ShipList.RACE_ALIEN)[r - num - 1];
                    }

                    // icon
                    if (r < 5)
                    {
                        pic = "i_" + map.empImgPrefixes.getEmpirePrefix(r) + "26.tga";
                        tf_icon[tf_index] = map.res.getTargaImage(pic, true);

                        tf_index++;
                    }
                    else if (r < num && !minor)
                    {
                        // minor race
                        pic = "i_m26.tga";
                        tf_icon[tf_index] = map.res.getTargaImage(pic, true);

                        minor = true;
                        tf_index++;
                    }
                    else if (!alien)
                    {
                        // alien
                        pic = "i_t26.tga";
                        tf_icon[tf_index] = map.res.getTargaImage(pic, true);

                        alien = true;
                        tf_index++;
                    }
                }
            }

            // no military
            if (!military_present)
            {
                shipdesc = shipdesc + "<br><i>" + map.lexicon.getEntry(872) + "</i>";
            }
        }

        // paths
        if (paths)
        {
            markPaths(true);
        }
    }

    public void markPaths(boolean select) throws InvalidArgumentsException
    {
        // ship movements
        int[] i_tf = map.gWTForce.getSectorTaskForces(y, x);
        boolean not_first, selected;

        int[] orders = map.ordInfo.findFleetOrders(i_tf);
        int i = -1;

        for (int tfIndex : i_tf)
        {
            i++;

            selected = tfIndex == map.selected_taskforce;

            int order = orders[i];

            if (order >= 0)
            {
                int mx = 0, my = 0;
                int w = map.ordInfo.getTargetSectorColumn(order) - x;
                int h = map.ordInfo.getTargetSectorRow(order) - y;
                not_first = false;

                if (w == 0 && h == 0) // not moving
                {
                    continue;
                }

                int r = map.gWTForce.getOwner(tfIndex);

                if (r > 4)
                {
                    r = 5;
                }

                while (w - mx != 0 || h - my != 0)
                {
                    if (Math.abs(w - mx) > Math.abs(h - my))
                    {
                        if (w < 0)
                        {
                            if (not_first)
                            {
                                map.map[x + mx][y + my].movs_icon[r][6] = true;

                                if (selected)
                                {
                                    map.map[x + mx][y + my].mov_icon[r][6] = select;
                                }
                            }
                            mx--;
                        }
                        else
                        {
                            if (not_first)
                            {
                                map.map[x + mx][y + my].movs_icon[r][2] = true;
                                if (selected)
                                {
                                    map.map[x + mx][y + my].mov_icon[r][2] = select;
                                }
                            }
                            mx++;
                        }
                    }
                    else if (Math.abs(w - mx) < Math.abs(h - my))
                    {
                        if (h < 0)
                        {
                            if (not_first)
                            {
                                map.map[x + mx][y + my].movs_icon[r][0] = true;
                                if (selected)
                                {
                                    map.map[x + mx][y + my].mov_icon[r][0] = select;
                                }
                            }
                            my--;
                        }
                        else
                        {
                            if (not_first)
                            {
                                map.map[x + mx][y + my].movs_icon[r][4] = true;
                                if (selected)
                                {
                                    map.map[x + mx][y + my].mov_icon[r][4] = select;
                                }
                            }
                            my++;
                        }
                    }
                    else
                    {
                        if (w < 0)
                        {
                            if (h < 0)
                            {
                                if (not_first)
                                {
                                    map.map[x + mx][y + my].movs_icon[r][7] = true;
                                    if (selected)
                                    {
                                        map.map[x + mx][y + my].mov_icon[r][7] = select;
                                    }
                                }
                                my--;
                            }
                            else
                            {
                                if (not_first)
                                {
                                    map.map[x + mx][y + my].movs_icon[r][5] = true;
                                    if (selected)
                                    {
                                        map.map[x + mx][y + my].mov_icon[r][5] = select;
                                    }
                                }
                                my++;
                            }

                            mx--;
                        }
                        else
                        {
                            if (h < 0)
                            {
                                if (not_first)
                                {
                                    map.map[x + mx][y + my].movs_icon[r][1] = true;
                                    if (selected)
                                    {
                                        map.map[x + mx][y + my].mov_icon[r][1] = select;
                                    }
                                }
                                my--;
                            }
                            else
                            {
                                if (not_first)
                                {
                                    map.map[x + mx][y + my].movs_icon[r][3] = true;
                                    if (selected)
                                    {
                                        map.map[x + mx][y + my].mov_icon[r][3] = select;
                                    }
                                }
                                my++;
                            }
                            mx++;
                        }
                    }

                    not_first = true;
                }

                map.map[x + mx][y + my].movs_icon[r][8] = true;
                if (selected)
                {
                    map.map[x + mx][y + my].mov_icon[r][8] = select;
                }
            }
        }
    }

    @Override
    protected void paintComponent(Graphics g)
    {
        super.paintComponent(g);

        Image tga;

        if (map.view >= 0)
        {
            String tip = "<html>";
            
            try
            {
                boolean revealed = map.view == 0 || (map.sectors.getRevealed(index) & map.mask) == map.mask;
                boolean explored = map.view == 0 || (map.sectors.getExplored(index) & map.mask) == map.mask;

                if (revealed || explored)
                {
                    this.setEnabled(true);

                    // color sector
                    setBackground(Color.BLACK);

                    int owner = map.sectors.getOwnerMask(index);
                    
                    switch (owner)
                    {
                        case 0:
                        {
                            if (explored)
                            {
                                tip = tip + "<h3>" + title + "</h3>";
                            }
                            else
                            {
                                String str = map.lexicon.getEntry(1385).replace("%s", sector);
                                tip = tip + "<h3>" + str + "</h3>";
                            }

                            break;
                        }
                        case 1:
                        case 2:
                        case 4:
                        case 8:
                        case 16:
                        {
                            owner = map.sectors.getOwnerRace(index);

                            if (explored || map.view - 1 == owner ||
                                    map.empireInfo.isRaceKnown(map.view - 1, owner))
                            {
                                if (check != null && map.drawOwnership)
                                {
                                    tga = check.getImage(true);

                                    BufferedImage img = new BufferedImage(
                                            tga.getWidth(null), tga.getHeight(null),
                                            BufferedImage.TYPE_INT_RGB);

                                    Graphics2D g2 = img.createGraphics();
                                    g2.drawImage(tga, 0, 0, null);
                                    g2.dispose();

                                    float scaleFactor = 2.5f;
                                    RescaleOp op = new RescaleOp(scaleFactor, -5, null);
                                    img = op.filter(img, null);

                                    g.drawImage(img.getScaledInstance(
                                            getHeight(), getHeight(), Image.SCALE_AREA_AVERAGING), 0, 0, null);
                                }

                                tip = tip + "<h3>" + title + "</h3>";
                            }
                            else
                            {
                                String str = map.lexicon.getEntry(1385).replace("%s", sector);
                                tip = tip + "<h3>" + str + "</h3>";
                            }

                            break;
                        }
                        default:
                        {
                            if (check != null && map.drawOwnership)
                            {
                                tga = check.getImage(true);

                                BufferedImage img = new BufferedImage(
                                            tga.getWidth(null), tga.getHeight(null),
                                            BufferedImage.TYPE_INT_RGB);

                                Graphics2D g2 = img.createGraphics();
                                g2.drawImage(tga, 0, 0, null);
                                g2.dispose();

                                float scaleFactor = 2.5f;
                                RescaleOp op = new RescaleOp(scaleFactor, -5, null);
                                img = op.filter(img, null);

                                g.drawImage(img.getScaledInstance(
                                    getHeight(), getHeight(), Image.SCALE_AREA_AVERAGING), 0, 0, null);
                            }

                            tip = tip + "<h3>" + title + "</h3>";
                        }
                    }

                    // draw star/object if present
                    if (main != null)
                    {
                        int size = getHeight() / 3;

                        if (map.drawObjects)
                        {
                            tga = main.getImage(true);

                            BufferedImage img = new BufferedImage(
                                            tga.getWidth(null), tga.getHeight(null),
                                            BufferedImage.TYPE_INT_ARGB);

                            Graphics2D g2 = img.createGraphics();
                            g2.drawImage(tga, 0, 0, null);
                            g2.dispose();

                            g.drawImage(img.getScaledInstance(
                                    size, size, Image.SCALE_AREA_AVERAGING), size, size, null);
                        }
                        
                        if (objdesc != null && explored)
                        {
                            tip = tip + objdesc;
                        }
                        else if (objType >= 0)
                        {
                            String str = getObjType(objType);
                            tip = tip + str;
                        }
                    }

                    // draw base
                    if (base != null && map.drawOutposts)
                    {
                        tga = base.getImage(true);

                        int size = getHeight() / 4;

                        BufferedImage img = new BufferedImage(
                                        tga.getWidth(null), tga.getHeight(null),
                                        BufferedImage.TYPE_INT_ARGB);

                        Graphics2D g2 = img.createGraphics();
                        g2.drawImage(tga, 0, 0, null);
                        g2.dispose();

                        g.drawImage(img.getScaledInstance(
                                size, size, Image.SCALE_AREA_AVERAGING), 0, 3 * size, null);
                    }

                    // draw task forces
                    int w = getWidth() / 4;

                    if (w > 0 && map.drawShips)
                    {
                        int offset = 0;
                        int h_offset = 0;
                        int h;

                        for (int i = 0; i < 7 && tf_icon[i] != null; i++)
                        {
                            tga = tf_icon[i].getImage(true);

                            h = (int)(w / (double)tga.getWidth(null) * tga.getHeight(null));
                            
                            if (i == 4)
                            {
                                offset = 0;
                                h_offset += h;
                            }

                            BufferedImage img = new BufferedImage(
                                            tga.getWidth(null), tga.getHeight(null),
                                            BufferedImage.TYPE_INT_ARGB);

                            Graphics2D g2 = img.createGraphics();
                            g2.drawImage(tga, 0, 0, null);
                            g2.dispose();

                            g.drawImage(img.getScaledInstance(
                                    w, h, Image.SCALE_AREA_AVERAGING), offset, h_offset, null);

                            offset += w;
                        }
                    }

                    if (shipdesc != null)
                    {
                        tip = tip + shipdesc;
                    }

                    // ship paths
                    if (map.drawPaths)
                    {
                        if (map.view == 0)
                        {
                            for (int race = 0; race < 6; race++)
                            {
                                this.drawPaths(race, movs_icon, map.movs_icon, g);
                            }
                        }
                        else
                        {
                            this.drawPaths(map.view - 1, movs_icon, map.movs_icon, g);
                        }
                        if (map.view == 0)
                        {
                            for (int race = 0; race < 6; race++)
                            {
                                this.drawPaths(race, mov_icon, map.mov_icon, g);
                            }
                        }
                        else
                        {
                            this.drawPaths(map.view - 1, mov_icon, map.mov_icon, g);
                        }
                    }
                }
                else
                {
                    setBackground(Color.WHITE);
//                    String str = map.lexicon.getEntry(1385).replace("%s", sector);
//                    tip = tip + "<h3>" + str + "</h3>";
                    this.setToolTipText(null);
                    this.setEnabled(false);
                    return;
                }
            }
            catch (Exception fn)
            {
                UE.displayError(fn);
            }

            tip = tip + "</html>";
            this.setToolTipText(tip);
        }
        else
        {
            setBackground(Color.WHITE);
            this.setToolTipText(null);
            this.setEnabled(false);
        }
    }

    private void drawPaths(int race, boolean[][] flags, Image[] icons, Graphics g)
    {
        int w = getWidth();
        int h = getHeight();
        int iw, ih, ix, iy;

        double scale = w / 5.0 / icons[8].getWidth(null);

        for (int i = 0; i < 9; i++)
        {
            if (flags[race][i])
            {
                iw = (int)(icons[i].getWidth(null) * scale);
                ih = (int)(icons[i].getHeight(null) * scale);

                if (i == 0 || i == 4 || i == 8)
                {
                    ix = (w - iw) / 2;
                }
                else if (i >= 1 && i <= 3)
                {
                    ix = w - iw;
                }
                else
                {
                    ix = 0;
                }

                if (i == 6 || i == 2 || i == 8)
                {
                    iy = (h - ih) / 2;
                }
                else if (i >= 3 && i <= 5)
                {
                    iy = h - ih;
                }
                else
                {
                    iy = 0;
                }

                BufferedImage img = new BufferedImage(
                                icons[i].getWidth(null), icons[i].getHeight(null),
                                BufferedImage.TYPE_INT_ARGB);

                Graphics2D g2 = img.createGraphics();
                g2.drawImage(icons[i], 0, 0, null);
                g2.dispose();

                g.drawImage(img.getScaledInstance(
                        iw, ih, Image.SCALE_AREA_AVERAGING), ix, iy, null);
            }
        }
    }

    private String getObjType(int type)
    {
        switch (type)
        {
            case 0:
                return Language.getString("SystemsInfoGUI.13");
            case 1:
                return Language.getString("SystemsInfoGUI.14");
            case 2:
                return Language.getString("SystemsInfoGUI.15");
            case 3:
                return Language.getString("SystemsInfoGUI.16");
            case 4:
                return Language.getString("SystemsInfoGUI.17");
            case 5:
                return Language.getString("SystemsInfoGUI.18");
            case 6:
                return Language.getString("SystemsInfoGUI.19");
            case 7:
                return Language.getString("SystemsInfoGUI.20");
            case 8:
                return Language.getString("SystemsInfoGUI.21");
            case 9:
                return Language.getString("SystemsInfoGUI.22");
            case 10:
                return Language.getString("SystemsInfoGUI.23");
            default:
                return Language.getString("SystemsInfoGUI.24");
        }
    }

    public Point getLink()
    {
        return link;
    }
}
