/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: MapGUIv2RightClickMenus.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 28.2.2011
 */

package ue.gui.sav;

import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import ue.UE;
import ue.edit.exe.Trek;
import ue.edit.exe.trek.NebulaHCGraphic;
import ue.edit.sav.SavGameTools;
import ue.exception.InvalidArgumentsException;
import ue.util.ID;

/**
 *
 */
public abstract class MapGUIv2RightClickMenus
{
    /**
     * Returns the menu shown when a sector is right-clicked
     * 
     * @param map
     * @param empire
     * @param sector
     * @return
     * @throws FileNotFoundException
     * @throws InvalidArgumentsException
     */
    public static ArrayList<JMenuItem> getSectorMenu(final MapPanel map,
            final int empire, final int sector)
            throws FileNotFoundException, InvalidArgumentsException
    {
        ArrayList<JMenuItem> items = new ArrayList<JMenuItem>();

        boolean visible = empire < 0;

        if (empire >= 0)
        {
            visible = map.sectors.getExploredBy(sector, empire) ||
                    map.sectors.getScannedBy(sector, empire);
        }
        
        ////////////////
        // add menues //
        ////////////////
        JMenu mnuAdd = new JMenu("Add");
        mnuAdd.setEnabled(visible);
        ArrayList<JMenuItem> addItems = new ArrayList<JMenuItem>();

        // outposts
        if (map.sectors.getOutpost(sector) < 0)
        {
            JMenu mnuOutpost = new JMenu("Outpost/Starbase");

            ArrayList<ID> ids  = new ArrayList<ID>();
            ID[] outposts = map.ships.getIDs(43); // 37 + 6
            for (int i = 0; i < outposts.length; i++)
            {
                ids.add(outposts[i]);
            }
            outposts = map.ships.getIDs(44); // 37 + 7
            for (int i = 0; i < outposts.length; i++)
            {
                ids.add(outposts[i]);
            }

            if (ids.size() > 0)
            {
                JMenuItem mnuBase;
                ID id;
                Collections.sort(ids);

                ActionListener addOutpost = new ActionListener()
                {
                    @Override
                    public void actionPerformed(ActionEvent e)
                    {
                        try{
                            JMenuItem mnu = (JMenuItem) e.getSource();
                            String text = mnu.getText();
                            short id = Short.parseShort(text.substring(0, text.indexOf(":")));
                            SavGameTools.addOutpost(map.game, map.ships, sector,
                                    id, false);
                            map.refreshMap();
                        } catch (Exception ex) {
                            UE.displayError(ex);
                        }
                    }
                };

                for (int i = 0; i < ids.size(); i++)
                {
                    id = ids.get(i);
                    mnuBase = new JMenuItem(id.hashCode() + ": " +
                            map.races.getName(map.ships.getRace(id.hashCode()))
                            + ": " + id.toString());
                    mnuBase.addActionListener(addOutpost);
                    mnuOutpost.add(mnuBase);
                }
            }
            else
            {
                JMenuItem mnuNone = new JMenuItem("None");
                mnuNone.setEnabled(false);
                mnuOutpost.add(mnuNone);
            }

            addItems.add(mnuOutpost);
        }

        // stellar objects
        if (map.sectors.getStellarObject(sector) < 0 &&
                map.sectors.getSystem(sector) < 0)
        {
            JMenu mnuObject = new JMenu("Stellar object");

            ArrayList<ID> ids  = new ArrayList<ID>();

            ids.add(new ID(map.lexicon.getEntry(94), 0));
            ids.add(new ID(map.lexicon.getEntry(1450), 1));
            ids.add(new ID(map.lexicon.getEntry(802), 2));
            ids.add(new ID(map.lexicon.getEntry(810), 3));
            ids.add(new ID(map.lexicon.getEntry(1447), 10));
            ids.add(new ID(map.lexicon.getEntry(968), 11));

            if (ids.size() > 0)
            {
                JMenuItem mnuItem;
                ID id;
                Collections.sort(ids);

                ActionListener addObject = new ActionListener()
                {
                    @Override
                    public void actionPerformed(ActionEvent e)
                    {
                        try{
                            JMenuItem mnu = (JMenuItem) e.getSource();
                            String text = mnu.getText();
                            short id = Short.parseShort(text.substring(0, text.indexOf(":")));
                            SavGameTools.addObject(map.game, map.objstruc, sector,
                                    id, text.substring(text.indexOf(":") + 2), false);
                            map.refreshMap();
                        } catch (Exception ex) {
                            UE.displayError(ex);
                        }
                    }
                };

                for (int i = 0; i < ids.size(); i++)
                {
                    id = ids.get(i);
                    mnuItem = new JMenuItem(id.hashCode() + ": " + id.toString());
                    mnuItem.addActionListener(addObject);
                    mnuObject.add(mnuItem);
                }
            }
            else
            {
                JMenuItem mnuNone = new JMenuItem("None");
                mnuNone.setEnabled(false);
                mnuObject.add(mnuNone);
            }

            addItems.add(mnuObject);
        }

        if (addItems.size() > 0)
        {
            Collections.sort(addItems, new Comparator<JMenuItem>()
            {
                @Override
                public int compare(JMenuItem o1, JMenuItem o2) {
                    return o1.getText().compareTo(o2.getText());
                }
            });

            for (int i = 0; i < addItems.size(); i++)
            {
                mnuAdd.add(addItems.get(i));
            }

            items.add(mnuAdd);
        }
        
        /////////////////
        // exploration //
        /////////////////
        JMenu mnuExplorationStatus = new JMenu("Exploration status");
        mnuExplorationStatus.setEnabled(empire >= 0);
        
        if (empire >= 0)
        {
            final JCheckBoxMenuItem mnuExplored = new JCheckBoxMenuItem("Explored",
                    map.sectors.getExploredBy(sector, empire));

            mnuExplored.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent e) {
                    boolean value = mnuExplored.isSelected();

                    try
                    {
                        map.sectors.setExploredBy(sector, empire, value);
                        map.refreshMap();
                    } catch (InvalidArgumentsException ex) {
                        UE.displayError(ex);
                    }
                }
            });
            mnuExplorationStatus.add(mnuExplored);

            final JCheckBoxMenuItem mnuScanned = new JCheckBoxMenuItem("Scanned",
                    map.sectors.getScannedBy(sector, empire));

            mnuScanned.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent e) {
                    boolean value = mnuScanned.isSelected();

                    try
                    {
                        map.sectors.setScannedBy(sector, empire, value);
                        map.refreshMap(map.drawOwnership, map.drawObjects,
                                map.drawOutposts || map.drawShips, map.drawPaths);
                    } catch (InvalidArgumentsException ex) {
                        UE.displayError(ex);
                    }
                }
            });
            mnuExplorationStatus.add(mnuScanned);
        }

        items.add(mnuExplorationStatus);

        //////////////////
        // Sector claim //
        //////////////////
        /*
         * Changing claim in sector.lst has no effect in the game (?)
         *
        if (map.sectors.getOutpost(sector) < 0
                && (map.sectors.getSystem(sector) < 0 ||
                map.systems.getPopulation(map.sectors.getSystem(sector)) <= 0))
        {
            JMenu mnuClaim = new JMenu("Ownership claim");

            ArrayList<ID> ids  = new ArrayList<ID>();
            for (int i = 0; i < 5; i++)
            {
                ids.add(new ID(map.races.getName(i), i));
            }

            if (ids.size() > 0)
            {
                JCheckBoxMenuItem mnuRace;
                ID id;

                ActionListener editClaim = new ActionListener()
                {
                    @Override
                    public void actionPerformed(ActionEvent e)
                    {
                        try{
                            JMenuItem mnu = (JMenuItem) e.getSource();
                            String text = mnu.getText();
                            short id = Short.parseShort(text.substring(0, text.indexOf(":")));

                            short mask = map.sectors.getOwnerMask(sector);

                            mask = (short)(mask ^ (1 << id));

                            map.sectors.setOwnerMask(sector, mask, 0);
                            
                            if (mnu.isSelected())
                            {
                                map.sphOfInf[id].setInfluence(sector, 10);
                            }
                            else
                            {
                                map.sphOfInf[id].setInfluence(sector, 0);
                            }

                            map.refreshMap();
                        } catch (Exception ex) {
                            UE.displayError(ex);
                        }
                    }
                };

                for (int i = 0; i < ids.size(); i++)
                {
                    id = ids.get(i);
                    mnuRace = new JCheckBoxMenuItem(id.hashCode() + ": " + id.toString());

                    mnuRace.setSelected((map.sectors.getOwnerMask(sector) & (1 << id.hashCode())) != 0);

                    mnuRace.addActionListener(editClaim);
                    mnuClaim.add(mnuRace);
                }
            }

            items.add(mnuClaim);
        }//*/
        
        /////////////
        // outpost //
        /////////////
        if (map.sectors.getOutpost(sector) >= 0)
        {
            JMenu mnuOutpost = new JMenu("Outpost/Starbase");

            ArrayList<JMenuItem> oItems = getOutpostMenu(map,
                        map.outposts.getIndex(map.sectors.getOutpost(sector)));

            for (int i = 0; i < oItems.size(); i++)
            {
                mnuOutpost.add(oItems.get(i));
            }
            
            items.add(mnuOutpost);
        }

        ////////////////////
        // stellar object //
        ////////////////////
        if (map.sectors.getStellarObject(sector) >= 0)
        {
            JMenu mnuObject = new JMenu("Stellar object");

            ArrayList<JMenuItem> oItems = getStellarObjectMenu(map,
                        map.sectors.getStellarObject(sector));

            for (int i = 0; i < oItems.size(); i++)
            {
                mnuObject.add(oItems.get(i));
            }

            items.add(mnuObject);
        }

        /////////////////
        // star system //
        /////////////////
        if (map.sectors.getSystem(sector) >= 0)
        {
            JMenu mnuSystem = new JMenu("Star system");

            ArrayList<JMenuItem> oItems = getSystemMenu(map,
                        map.sectors.getSystem(sector));

            for (int i = 0; i < oItems.size(); i++)
            {
                mnuSystem.add(oItems.get(i));
            }

            items.add(mnuSystem);
        }

        // move!
        JMenuItem mnuMove = new JMenuItem("Move");
        mnuMove.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                SelectSectorDialog dialog = new SelectSectorDialog(UE.WINDOW, true,
                        map.game, map.res, map.exe);
                dialog.setVisible(true);

                Point selected = dialog.getSelectedSector();

                if (selected != null)
                {
                    try
                    {
                        SavGameTools.switchSectors(map.game, sector, selected.x, selected.y);
                        map.refreshMap();
                    }
                    catch (Exception ex)
                    {
                        UE.displayError(ex);
                    }
                }
            }
        });

        items.add(mnuMove);

        Collections.sort(items, new Comparator<JMenuItem>()
        {
            @Override
            public int compare(JMenuItem o1, JMenuItem o2) {
                return o1.getText().compareTo(o2.getText());
            }
        });

        return items;
    }

    /**
     * Returns the menu show when and outpost is right-clicked
     *
     * @param map
     * @param outpost
     * @return
     * @throws FileNotFoundException
     * @throws InvalidArgumentsException
     */
    public static ArrayList<JMenuItem> getOutpostMenu(final MapPanel map,
            final int outpost)
            throws FileNotFoundException, InvalidArgumentsException
    {
        ArrayList<JMenuItem> items = new ArrayList<JMenuItem>();

        /////////////////
        // edit menues //
        /////////////////
        JMenu mnuEdit = new JMenu("Edit");
        ArrayList<JMenuItem> editItems = new ArrayList<JMenuItem>();
        
        // owner
        JMenu mnuOwner = new JMenu("Owner");
        JCheckBoxMenuItem mnuCheckItem;
        int owner = map.outposts.getOwner(outpost);

        ActionListener editOwner = new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                try{
                    JMenuItem mnu = (JMenuItem) e.getSource();
                    String text = mnu.getText();
                    short id = Short.parseShort(text.substring(0, text.indexOf(":")));

                    map.outposts.setOwner(outpost, id);
                    map.refreshMap();
                } catch (Exception ex) {
                    UE.displayError(ex);
                }
            }
        };

        for (int r = 0; r < 5; r++)
        {
            mnuCheckItem = new JCheckBoxMenuItem(r + ": " + map.races.getName(r));
            mnuCheckItem.setSelected(owner == r);
            mnuCheckItem.addActionListener(editOwner);

            mnuOwner.add(mnuCheckItem);
        }

        editItems.add(mnuOwner);

        // class/type
        JMenu mnuClass = new JMenu("Class");

        ArrayList<ID> ids  = new ArrayList<ID>();
        ID[] outposts = map.ships.getIDs(43); // 37 + 6
        for (int i = 0; i < outposts.length; i++)
        {
            ids.add(outposts[i]);
        }
        outposts = map.ships.getIDs(44); // 37 + 7
        for (int i = 0; i < outposts.length; i++)
        {
            ids.add(outposts[i]);
        }

        if (ids.size() > 0)
        {
            JCheckBoxMenuItem mnuBase;
            ID id;
            Collections.sort(ids);
            int shipID = map.outposts.getShipID(outpost);

            ActionListener editClass = new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent e)
                {
                    try{
                        JMenuItem mnu = (JMenuItem) e.getSource();
                        String text = mnu.getText();
                        short id = Short.parseShort(text.substring(0, text.indexOf(":")));

                        map.outposts.setShipID(outpost, id);
                        map.outposts.setType(outpost, map.ships.getFunction(id));
                        map.outposts.setHull(outpost, map.ships.getHullStr(id));
                        map.outposts.setScanRange(outpost, map.ships.getScanRange(id));

                        map.refreshMap();
                    } catch (Exception ex) {
                        UE.displayError(ex);
                    }
                }
            };

            for (int i = 0; i < ids.size(); i++)
            {
                id = ids.get(i);
                mnuBase = new JCheckBoxMenuItem(id.hashCode() + ": " +
                        map.races.getName(map.ships.getRace(id.hashCode()))
                        + ": " + id.toString());

                mnuBase.setSelected(shipID == id.hashCode());

                mnuBase.addActionListener(editClass);
                mnuClass.add(mnuBase);
            }
        }

        editItems.add(mnuClass);

        // name
        JMenuItem mnuName = new JMenuItem("Name");
        mnuName.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    Object retVal = JOptionPane.showInputDialog(UE.WINDOW,
                            "Enter new name:", map.outposts.getName(outpost));

                    if (retVal != null)
                    {
                        map.outposts.setName(outpost, retVal.toString());
                        map.refreshMap();
                    }
                }
                catch (InvalidArgumentsException ex)
                {
                    UE.displayError(ex);
                }
            }
        });
        editItems.add(mnuName);

        JMenuItem mnuHitPoints = new JMenuItem("Hit Points");
        mnuHitPoints.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    JPanel pnl = new JPanel(new BorderLayout());

                    int max = map.ships.getHullStr(map.outposts.getShipID(outpost));
                    JSpinner spiValue = new JSpinner(new SpinnerNumberModel(
                            map.outposts.getHull(outpost), 0,
                            max, 1));
                    pnl.add(spiValue, BorderLayout.CENTER);
                    pnl.add(new JLabel("Edit hit points:"), BorderLayout.NORTH);
                    pnl.add(new JLabel(" / " + max + " "), BorderLayout.EAST);

                    int retVal = JOptionPane.showConfirmDialog(UE.WINDOW,
                              pnl,
                              UE.APP_NAME,
                              JOptionPane.OK_CANCEL_OPTION);

                    if (retVal == JOptionPane.OK_OPTION)
                    {
                        map.outposts.setHull(outpost, (Integer)spiValue.getValue());
                        map.refreshMap();
                    }
                }
                catch (InvalidArgumentsException ex)
                {
                    UE.displayError(ex);
                }
            }
        });
        editItems.add(mnuHitPoints);

        JMenu mnuConstruction = new JMenu("Contruction status");

        JCheckBoxMenuItem mnuItem = new JCheckBoxMenuItem("Just started");
        mnuItem.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
               try{
                    short id = map.outposts.getShipID(outpost);
                    map.outposts.setProductionInvested(outpost, 0);
                    map.outposts.setProductionCost(outpost, map.ships.getBuildCost(id));
                    
                    map.refreshMap();
                } catch (Exception ex) {
                    UE.displayError(ex);
                }
            }
        });
        mnuItem.setSelected(map.outposts.getProductionCost(outpost) > 0 &&
                map.outposts.getProductionInvested(outpost) == 0);

        mnuConstruction.add(mnuItem);

        mnuItem = new JCheckBoxMenuItem("Partialy complete");
        mnuItem.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
               try
               {
                    short id = map.outposts.getShipID(outpost);
                    JPanel pnl = new JPanel(new BorderLayout());

                    int max = map.ships.getBuildCost(id);
                    int val = (int)map.outposts.getProductionInvested(outpost);
                    val = Math.min(val, max);
                    val = Math.max(val, 0);
                    
                    JSpinner spiValue = new JSpinner(new SpinnerNumberModel(
                            val, 0, max, 1));
                    pnl.add(spiValue, BorderLayout.CENTER);
                    pnl.add(new JLabel("Edit production invested:"), BorderLayout.NORTH);
                    pnl.add(new JLabel(" / " + max + " "), BorderLayout.EAST);

                    int retVal = JOptionPane.showConfirmDialog(UE.WINDOW,
                              pnl,
                              UE.APP_NAME,
                              JOptionPane.OK_CANCEL_OPTION);

                    if (retVal == JOptionPane.OK_OPTION)
                    {
                        map.outposts.setProductionInvested(outpost, ((Integer)spiValue.getValue()).intValue());
                        map.refreshMap();
                    }
                } catch (Exception ex) {
                    UE.displayError(ex);
                }
            }
        });
        mnuItem.setSelected(map.outposts.getProductionCost(outpost) >
                map.outposts.getProductionInvested(outpost) &&
                map.outposts.getProductionInvested(outpost) > 0);
        mnuConstruction.add(mnuItem);

        mnuItem = new JCheckBoxMenuItem("Completed");
        mnuItem.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
               try{
                    short id = map.outposts.getShipID(outpost);
                    map.outposts.setProductionInvested(outpost, map.ships.getBuildCost(id));
                    map.outposts.setProductionCost(outpost, map.ships.getBuildCost(id));

                    map.refreshMap();
                } catch (Exception ex) {
                    UE.displayError(ex);
                }
            }
        });
        mnuItem.setSelected(map.outposts.getProductionCost(outpost) <=
                map.outposts.getProductionInvested(outpost));
        mnuConstruction.add(mnuItem);
        

        editItems.add(mnuConstruction);


        Collections.sort(editItems, new Comparator<JMenuItem>()
        {
            @Override
            public int compare(JMenuItem o1, JMenuItem o2) {
                return o1.getText().compareTo(o2.getText());
            }
        });

        for (int i = 0; i < editItems.size(); i++)
        {
            mnuEdit.add(editItems.get(i));
        }

        items.add(mnuEdit);

        // remove
        JMenuItem mnuRemove = new JMenuItem("Remove");

        mnuRemove.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    Point loc = map.outposts.getPosition(outpost);
                    int sector = loc.y * map.sectors.getHSectors() + loc.x;
                    SavGameTools.removeOutpost(map.game, map.ships, sector);
                    map.refreshMap();
                } catch (Exception ex) {
                    UE.displayError(ex);
                }
            }
        });

        items.add(mnuRemove);

        // move!
        JMenuItem mnuMove = new JMenuItem("Move");
        mnuMove.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                SelectSectorDialog dialog = new SelectSectorDialog(UE.WINDOW, true,
                        map.game, map.res, map.exe);
                dialog.setVisible(true);

                Point selected = dialog.getSelectedSector();

                if (selected != null)
                {
                    try
                    {
                        SavGameTools.moveOutpost(map.game, outpost, selected.x, selected.y, false);
                        map.refreshMap();
                    }
                    catch (Exception ex)
                    {
                        UE.displayError(ex);
                    }
                }
            }
        });

        items.add(mnuMove);

        Collections.sort(items, new Comparator<JMenuItem>()
        {
            @Override
            public int compare(JMenuItem o1, JMenuItem o2) {
                return o1.getText().compareTo(o2.getText());
            }
        });

        return items;
    }

    /**
     * Returns the menu shown when a stellar object is right clicked
     *
     * @param map
     * @param object
     * @return
     * @throws FileNotFoundException
     * @throws InvalidArgumentsException
     */
    public static ArrayList<JMenuItem> getStellarObjectMenu(final MapPanel map,
            final int object)
            throws FileNotFoundException, InvalidArgumentsException
    {
        ArrayList<JMenuItem> items = new ArrayList<JMenuItem>();

        // remove
        JMenuItem mnuRemove = new JMenuItem("Remove");

        mnuRemove.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    int[] pos = map.objects.getPosition(object);
                    int sector = pos[1] * map.sectors.getHSectors() + pos[0];
                    SavGameTools.removeObject(map.game, sector);
                    map.refreshMap();
                } catch (Exception ex) {
                    UE.displayError(ex);
                }
            }
        });

        items.add(mnuRemove);

        /////////////////
        // edit menues //
        /////////////////
        JMenu mnuEdit = new JMenu("Edit");
        JCheckBoxMenuItem mnuItem;
        ArrayList<JMenuItem> editItems = new ArrayList<JMenuItem>();

        if (map.objects.getType(object) == 10)
        {
            ArrayList<Integer> destinations = new ArrayList<Integer>();

            // fill dests
            int[] dest;

            for (int i = 0; i < map.objects.getNumberOfEntries(); i++)
            {
                if (i == object || map.objects.getType(i) != 10)
                {
                    continue;
                }

                destinations.add(i);
            }

            if (!destinations.isEmpty())
            {
                JMenu mnuLink = new JMenu("Endpoint");
                String itemStr;
                int index;

                ActionListener editLink = new ActionListener()
                {
                    @Override
                    public void actionPerformed(ActionEvent e)
                    {
                        try
                        {
                            JMenuItem mnu = (JMenuItem) e.getSource();
                            String text = mnu.getText();
                            short id = Short.parseShort(text.substring(0, text.indexOf(":")));

                            if (id >= 0)
                            {
                                int ret = JOptionPane.showConfirmDialog(UE.WINDOW,
                                        "Create a two-way link between the wormholes?");

                                if (ret == JOptionPane.CANCEL_OPTION)
                                {
                                    return;
                                }
                                else if (ret == JOptionPane.YES_OPTION)
                                {
                                    map.objects.setLink(id, (short)object);
                                }
                            }

                            map.objects.setLink(object, id);

                            map.refreshMap();
                        }
                        catch (InvalidArgumentsException ex)
                        {
                            UE.displayError(ex);
                        }
                    }
                };

                mnuItem = new JCheckBoxMenuItem("-1: None");
                mnuItem.setSelected(map.objects.getLink(object) < 0);
                mnuItem.addActionListener(editLink);
                mnuLink.add(mnuItem);

                for (int i = 0; i < destinations.size(); i++)
                {
                    index = destinations.get(i);

                    dest = map.objects.getPosition(index);
                    itemStr = index + ": " + map.objects.getName(index) + " (" +
                        map.sectors.getDescription(dest[1] * map.sectors.getHSectors() + dest[0], false)
                        + ")";

                    mnuItem = new JCheckBoxMenuItem(itemStr);

                    mnuItem.setSelected(map.objects.getLink(object) == index);
                    mnuItem.addActionListener(editLink);
                    mnuLink.add(mnuItem);
                }
                editItems.add(mnuLink);
            }
        }

        JMenuItem mnuName = new JMenuItem("Name");
        mnuName.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    Object retVal = JOptionPane.showInputDialog(UE.WINDOW,
                            "Enter new name:", map.objects.getName(object));

                    if (retVal != null)
                    {
                        map.objects.setName(object, retVal.toString());
                        map.refreshMap();
                    }
                }
                catch (InvalidArgumentsException ex)
                {
                    UE.displayError(ex);
                }
            }
        });
        editItems.add(mnuName);

        // type
        JMenu mnuType = new JMenu("Type");

        ActionListener editType = new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                try{
                    JMenuItem mnu = (JMenuItem) e.getSource();
                    String text = mnu.getText();
                    short id = Short.parseShort(text.substring(0, text.indexOf(":")));
                    int[] dest = map.objects.getPosition(object);
                    dest[1] *= map.sectors.getHSectors();
                    dest[1] += dest[0];

                    String sector = map.sectors.getDescription(dest[1], false);
                    sector = sector.substring(sector.indexOf(" ") + 1);

                    map.objects.setType(object, id);
                    map.objects.setAni(object, map.objstruc.getGraph(id));
                    map.objects.setName(object, text.substring(text.indexOf(":") + 2)
                            + " " + sector);

                    map.refreshMap();
                } catch (Exception ex) {
                    UE.displayError(ex);
                }
            }
        };

        ArrayList<ID> ids = new ArrayList<ID>();
        ids.add(new ID(map.lexicon.getEntry(94), 0));
        ids.add(new ID(map.lexicon.getEntry(1450), 1));
        ids.add(new ID(map.lexicon.getEntry(802), 2));
        ids.add(new ID(map.lexicon.getEntry(810), 3));
        ids.add(new ID(map.lexicon.getEntry(1447), 10));
        ids.add(new ID(map.lexicon.getEntry(968), 11));
        ID id;

        for (int i = 0; i < ids.size(); i++)
        {
            id = ids.get(i);
            mnuItem = new JCheckBoxMenuItem(id.hashCode() + ": " + id.toString());
            mnuItem.setSelected(map.objects.getType(object) == id.hashCode());
            mnuItem.addActionListener(editType);
            mnuType.add(mnuItem);
        }

        editItems.add(mnuType);

        // graph
        JMenu mnuGraph = new JMenu("Graphics");

        ActionListener editGraph = new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                try{
                    JMenuItem mnu = (JMenuItem) e.getSource();
                    map.objects.setAni(object, mnu.getText());

                    map.refreshMap();
                } catch (Exception ex) {
                    UE.displayError(ex);
                }
            }
        };

        NebulaHCGraphic nhcg = (NebulaHCGraphic)map.exe.getInternalFile(Trek.fileNEBULA_HC_GRAPHIC, false);

        if (map.objects.getType(object) != 2
                || !nhcg.isHCGraphicEnabled())
        {
            Enumeration<String[]> anis = map.res.getFiles(new FilenameFilter()
            {
                @Override
                public boolean accept(File dir, String name)
                {
                    if (name.endsWith(".ani"))
                    {
                        name = name.substring(0, name.lastIndexOf("."));

                        return map.res.hasInternalFile(name + "map.tga") &&
                                map.res.hasInternalFile(name + "maps.tga");
                    }
                    return false;
                }
            });

            String str;
            ArrayList<JMenuItem> graphItems = new ArrayList<JMenuItem>();

            while (anis.hasMoreElements())
            {
                str = anis.nextElement()[0];
                str = str.substring(0, str.lastIndexOf("."));
                mnuItem = new JCheckBoxMenuItem(str);
                mnuItem.setSelected(map.objects.getAni(object).equalsIgnoreCase(str));
                mnuItem.addActionListener(editGraph);
                graphItems.add(mnuItem);
            }

            Collections.sort(graphItems, new Comparator<JMenuItem>()
            {
                @Override
                public int compare(JMenuItem o1, JMenuItem o2) {
                    return o1.getText().compareTo(o2.getText());
                }
            });

            for (int i = 0; i < graphItems.size(); i++)
            {
                mnuGraph.add(graphItems.get(i));
            }

            editItems.add(mnuGraph);
        }

        Collections.sort(editItems, new Comparator<JMenuItem>()
        {
            @Override
            public int compare(JMenuItem o1, JMenuItem o2) {
                return o1.getText().compareTo(o2.getText());
            }
        });

        for (int i = 0; i < editItems.size(); i++)
        {
            mnuEdit.add(editItems.get(i));
        }

        items.add(mnuEdit);

        // move!
        JMenuItem mnuMove = new JMenuItem("Move");
        mnuMove.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                SelectSectorDialog dialog = new SelectSectorDialog(UE.WINDOW, true,
                        map.game, map.res, map.exe);
                dialog.setVisible(true);

                Point selected = dialog.getSelectedSector();

                if (selected != null)
                {
                    try
                    {
                        SavGameTools.moveObject(map.game, object, selected.x, selected.y, false);
                        map.refreshMap();
                    }
                    catch (Exception ex)
                    {
                        UE.displayError(ex);
                    }
                }
            }
        });

        items.add(mnuMove);

        Collections.sort(items, new Comparator<JMenuItem>()
        {
            @Override
            public int compare(JMenuItem o1, JMenuItem o2) {
                return o1.getText().compareTo(o2.getText());
            }
        });

        return items;
    }

    /**
     * Returns the menu show when a system is right-clicked
     *
     * @param map
     * @param system
     * @return
     * @throws FileNotFoundException
     * @throws InvalidArgumentsException
     */
    public static ArrayList<JMenuItem> getSystemMenu(final MapPanel map,
            final int system)
            throws FileNotFoundException, InvalidArgumentsException
    {
        ArrayList<JMenuItem> items = new ArrayList<JMenuItem>();

        // move!
        JMenuItem mnuMove = new JMenuItem("Move");
        mnuMove.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                SelectSectorDialog dialog = new SelectSectorDialog(UE.WINDOW, true,
                        map.game, map.res, map.exe);
                dialog.setVisible(true);

                Point selected = dialog.getSelectedSector();

                if (selected != null)
                {
                    try
                    {
                        SavGameTools.moveSystem(map.game, system, selected.x, selected.y, false);
                        map.refreshMap();
                    }
                    catch (Exception ex)
                    {
                        UE.displayError(ex);
                    }
                }
            }
        });

        items.add(mnuMove);

        /////////////////
        // edit menues //
        /////////////////
        JMenu mnuEdit = new JMenu("Edit");
        JCheckBoxMenuItem mnuItem;
        ArrayList<JMenuItem> editItems = new ArrayList<JMenuItem>();

        JMenuItem mnuName = new JMenuItem("Name");
        mnuName.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    Object retVal = JOptionPane.showInputDialog(UE.WINDOW,
                            "Enter new name:", map.systems.getName(system));

                    if (retVal != null)
                    {
                        map.objects.setName(system, retVal.toString());
                        map.refreshMap();
                    }
                }
                catch (InvalidArgumentsException ex)
                {
                    UE.displayError(ex);
                }
            }
        });
        editItems.add(mnuName);

        // type
        JMenu mnuType = new JMenu("Type");

        ActionListener editType = new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                try{
                    JMenuItem mnu = (JMenuItem) e.getSource();
                    String text = mnu.getText();
                    short id = Short.parseShort(text.substring(0, text.indexOf(":")));

                    map.systems.setType(system, (byte)id);
                    map.systems.setAni(system, map.objstruc.getGraph(id) + ".ani");

                    map.refreshMap();
                } catch (Exception ex) {
                    UE.displayError(ex);
                }
            }
        };

        ArrayList<ID> ids = new ArrayList<ID>();
        ids.add(new ID(map.lexicon.getEntry(94), 0));
        ids.add(new ID(map.lexicon.getEntry(1450), 1));
        ids.add(new ID(map.lexicon.getEntry(802), 2));
        ids.add(new ID(map.lexicon.getEntry(810), 3));
        ids.add(new ID(map.lexicon.getEntry(987), 4));
        ids.add(new ID(map.lexicon.getEntry(889), 5));
        ids.add(new ID(map.lexicon.getEntry(1452), 6));
        ids.add(new ID(map.lexicon.getEntry(1446), 7));
        ids.add(new ID(map.lexicon.getEntry(420), 8));
        ids.add(new ID(map.lexicon.getEntry(96), 9));
        ids.add(new ID(map.lexicon.getEntry(1447), 10));
        ids.add(new ID(map.lexicon.getEntry(968), 11));
        ID id;

        for (int i = 0; i < ids.size(); i++)
        {
            id = ids.get(i);
            mnuItem = new JCheckBoxMenuItem(id.hashCode() + ": " + id.toString());
            mnuItem.setSelected(map.systems.getType(system) == id.hashCode());
            mnuItem.addActionListener(editType);
            mnuType.add(mnuItem);
        }

        editItems.add(mnuType);

        // dilithium
        JMenu mnuDil = new JMenu("Dilithium");

        mnuItem = new JCheckBoxMenuItem("Yes");
        mnuItem.setSelected(map.systems.hasDilithium(system));
        mnuItem.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                try {
                    map.systems.setDilithium(system, true);
                } catch (InvalidArgumentsException ex) {
                    UE.displayError(ex);
                }
            }
        });
        mnuDil.add(mnuItem);

        mnuItem = new JCheckBoxMenuItem("No");
        mnuItem.setSelected(!map.systems.hasDilithium(system));
        mnuItem.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                try {
                    map.systems.setDilithium(system, false);
                } catch (InvalidArgumentsException ex) {
                    UE.displayError(ex);
                }
            }
        });
        mnuDil.add(mnuItem);

        editItems.add(mnuDil);

        // graph
        JMenu mnuGraph = new JMenu("Graphics");

        ActionListener editGraph = new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                try{
                    JMenuItem mnu = (JMenuItem) e.getSource();
                    map.systems.setAni(system, mnu.getText() + ".ani");

                    map.refreshMap();
                } catch (Exception ex) {
                    UE.displayError(ex);
                }
            }
        };

        Enumeration<String[]> anis = map.res.getFiles(new FilenameFilter()
        {
            @Override
            public boolean accept(File dir, String name)
            {
                if (name.endsWith(".ani"))
                {
                    name = name.substring(0, name.lastIndexOf("."));

                    return map.res.hasInternalFile(name + "map.tga") &&
                            map.res.hasInternalFile(name + "maps.tga");
                }
                return false;
            }
        });

        String str;
        ArrayList<JMenuItem> graphItems = new ArrayList<JMenuItem>();

        while (anis.hasMoreElements())
        {
            str = anis.nextElement()[0];
            str = str.substring(0, str.lastIndexOf("."));
            mnuItem = new JCheckBoxMenuItem(str);
            mnuItem.setSelected(map.systems.getGraphic(system).equalsIgnoreCase(str));
            mnuItem.addActionListener(editGraph);
            graphItems.add(mnuItem);
        }

        Collections.sort(graphItems, new Comparator<JMenuItem>()
        {
            @Override
            public int compare(JMenuItem o1, JMenuItem o2) {
                return o1.getText().compareTo(o2.getText());
            }
        });

        for (int i = 0; i < graphItems.size(); i++)
        {
            mnuGraph.add(graphItems.get(i));
        }

        editItems.add(mnuGraph);

        // structure list
        if (map.systems.getPopulation(system) > 0 && map.systems.getResident(system) >= 0)
        {
            JMenuItem mItem = new JMenuItem("Structure list");
            mItem.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent e)
                {
                    UE.WINDOW.showPanel(new StrcInfoGUI(map.game, map.res,
                            map.exe, system));
                }
            });
            editItems.add(mItem);
        }

        Collections.sort(editItems, new Comparator<JMenuItem>()
        {
            @Override
            public int compare(JMenuItem o1, JMenuItem o2) {
                return o1.getText().compareTo(o2.getText());
            }
        });

        for (int i = 0; i < editItems.size(); i++)
        {
            mnuEdit.add(editItems.get(i));
        }

        items.add(mnuEdit);

        Collections.sort(items, new Comparator<JMenuItem>()
        {
            @Override
            public int compare(JMenuItem o1, JMenuItem o2) {
                return o1.getText().compareTo(o2.getText());
            }
        });

        return items;
    }
}
