/*
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2010 Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * MapPanel.java
 *
 * Created on 20.5.2010
 */

package ue.gui.sav;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.ToolTipManager;
import javax.swing.border.Border;
import ue.UE;
import ue.edit.exe.GenericSegment;
import ue.edit.exe.Trek;
import ue.edit.exe.trek.EmpireImgPrefixes;
import ue.edit.exe.trek.PlanetBonusTypes;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.bin.Planboni;
import ue.edit.res.stbof.bin.Planspac;
import ue.edit.res.stbof.bst.Edifice;
import ue.edit.res.stbof.dic.Lexicon;
import ue.edit.res.stbof.est.Environ;
import ue.edit.res.stbof.rst.Race;
import ue.edit.res.stbof.smt.Objstruc;
import ue.edit.res.stbof.sst.ShipList;
import ue.edit.sav.EmpsInfo;
import ue.edit.sav.GShipList;
import ue.edit.sav.OrderInfo;
import ue.edit.sav.SavGame;
import ue.edit.sav.SphereOfInfluence;
import ue.edit.sav.StarBaseInfo;
import ue.edit.sav.StellInfo;
import ue.edit.sav.StrcInfo;
import ue.edit.sav.SystInfo;
import ue.edit.sav.TaskForceList;
import ue.edit.sav.TechInfo;
import ue.edit.sav.lst.Sector;
import ue.exception.InvalidArgumentsException;
import ue.gui.SelectionEvent;
import ue.gui.SelectionListener;

public class MapPanel extends JPanel
{
    SavGame             game;
    Stbof               res;
    Trek                exe;
    // saved game
    Sector              sectors;
    SystInfo            systems;
    StellInfo           objects;
    StarBaseInfo        outposts;
    StrcInfo            strcInfo;
    EmpsInfo            empireInfo;
    TaskForceList       gWTForce;
    TaskForceList       gTForceList;
    OrderInfo           ordInfo;
    GShipList           gShipList;
    TechInfo            techInfo;
    SphereOfInfluence[] sphOfInf;
    // stbof.res
    ShipList            ships;
    Edifice             edifice;
    Lexicon             lexicon;
    Race                races;
    Environ             environ;
    Planspac            planspac;
    Planboni            planboni;
    Objstruc            objstruc;
    // trek.exe
    EmpireImgPrefixes   empImgPrefixes;
    PlanetBonusTypes    planBoniTypes;
    GenericSegment      groundCombatTechMulti;

    Image[] mov_icon = new Image[9];
    Image[] movs_icon = new Image[9];

    boolean drawOwnership = true;
    boolean drawObjects = true;
    boolean drawOutposts = true;
    boolean drawShips = true;
    boolean drawPaths = true;
    boolean selectObjects = true;
    boolean selectOutposts = true;
    boolean selectShips = true;

    int selected_taskforce = -1;
    int view = -1;
    long mask = 0;
    Point selection1 = null;
    Point prev_selection1 = null;
    Point selection2 = null;

    MapButton[][] map;
    Border selectionBorder;
    Border selection2Border;
    Border normalBorder;
    JPanel mapPanel;

    private ArrayList<ActionListener> listeners;
    private ArrayList<SelectionListener> right_click_listeners;

    public MapPanel(SavGame game, Stbof resource, Trek executable)
    {
        this.game = game;
        this.res = resource;
        this.exe = executable;

        mapPanel = new JPanel();
        mapPanel.setAlignmentY(Component.CENTER_ALIGNMENT);

        this.add(mapPanel);

        ToolTipManager.sharedInstance().setDismissDelay(10000);

        try
        {
            ships           = (ShipList)            res.getInternalFile(Stbof.fileSHIPLIST_SST, true);
            edifice         = (Edifice)             res.getInternalFile(Stbof.fileEDIFICE_BST, true);
            lexicon         = (Lexicon)             res.getInternalFile(Stbof.fileLEXICON_DIC, true);
            races           = (Race)                res.getInternalFile(Stbof.fileRACE_RST, true);
            environ         = (Environ)             res.getInternalFile(Stbof.fileENVIRON_EST, true);
            planspac        = (Planspac)            res.getInternalFile(Stbof.filePLANSPAC_BIN, true);
            planboni        = (Planboni)            res.getInternalFile(Stbof.filePLANBONI_BIN, true);
            objstruc        = (Objstruc)            res.getInternalFile(Stbof.fileOBJSTRUC_SMT, true);

            empImgPrefixes          = (EmpireImgPrefixes)exe.getInternalFile(Trek.fileEMPIRE_IMAGE_PREFIXES, true);
            planBoniTypes           = (PlanetBonusTypes)exe.getInternalFile(Trek.filePLANET_BONUS_TYPES, true);
            groundCombatTechMulti   = (GenericSegment)exe.getSegment(1542916, 8);

            sectors         = (Sector)              game.getInternalFile(SavGame.fileSECTOR_LST, true);
            systems         = (SystInfo)            game.getInternalFile(SavGame.fileSYSTINFO, true);
            objects         = (StellInfo)           game.getInternalFile(SavGame.fileSTELLINFO, true);
            outposts        = (StarBaseInfo)        game.getInternalFile(SavGame.fileSTARBASEINFO, true);
            strcInfo        = (StrcInfo)            game.getInternalFile(SavGame.fileSTRCINFO, true);
            empireInfo      = (EmpsInfo)            game.getInternalFile(SavGame.fileEMPSINFO, true);
            gWTForce        = (TaskForceList)       game.getInternalFile(SavGame.fileGWTFORCE, true);
            gTForceList     = (TaskForceList)       game.getInternalFile(SavGame.fileGTFORCELIST, true);
            ordInfo         = (OrderInfo)           game.getInternalFile(SavGame.fileORDINFO, true);
            gShipList       = (GShipList)           game.getInternalFile(SavGame.fileGSHIPLIST, true);
            techInfo        = (TechInfo)            game.getInternalFile(SavGame.fileTECHINFO, true);
            sphOfInf        = new SphereOfInfluence[5];
            sphOfInf[0]     = (SphereOfInfluence)   game.getInternalFile(SavGame.fileSPHOFIN0, true);
            sphOfInf[1]     = (SphereOfInfluence)   game.getInternalFile(SavGame.fileSPHOFIN1, true);
            sphOfInf[2]     = (SphereOfInfluence)   game.getInternalFile(SavGame.fileSPHOFIN2, true);
            sphOfInf[3]     = (SphereOfInfluence)   game.getInternalFile(SavGame.fileSPHOFIN3, true);
            sphOfInf[4]     = (SphereOfInfluence)   game.getInternalFile(SavGame.fileSPHOFIN4, true);

            mov_icon[0]     = res.getTargaImage("mov_n.tga", false).getImage(true);
            mov_icon[1]     = res.getTargaImage("mov_ne.tga", false).getImage(true);
            mov_icon[2]     = res.getTargaImage("mov_e.tga", false).getImage(true);
            mov_icon[3]     = res.getTargaImage("mov_se.tga", false).getImage(true);
            mov_icon[4]     = res.getTargaImage("mov_s.tga", false).getImage(true);
            mov_icon[5]     = res.getTargaImage("mov_sw.tga", false).getImage(true);
            mov_icon[6]     = res.getTargaImage("mov_w.tga", false).getImage(true);
            mov_icon[7]     = res.getTargaImage("mov_nw.tga", false).getImage(true);
            mov_icon[8]     = res.getTargaImage("mov.tga", false).getImage(true);

            movs_icon[0]    = res.getTargaImage("mov_ns.tga", false).getImage(true);
            movs_icon[1]    = res.getTargaImage("mov_nes.tga", false).getImage(true);
            movs_icon[2]    = res.getTargaImage("mov_es.tga", false).getImage(true);
            movs_icon[3]    = res.getTargaImage("mov_ses.tga", false).getImage(true);
            movs_icon[4]    = res.getTargaImage("mov_ss.tga", false).getImage(true);
            movs_icon[5]    = res.getTargaImage("mov_sws.tga", false).getImage(true);
            movs_icon[6]    = res.getTargaImage("mov_ws.tga", false).getImage(true);
            movs_icon[7]    = res.getTargaImage("mov_nws.tga", false).getImage(true);
            movs_icon[8]    = res.getTargaImage("movs.tga", false).getImage(true);
        }
        catch (Exception ex)
        {
            UE.displayError(ex);
            return;
        }

        listeners = new ArrayList<ActionListener>();
        right_click_listeners = new ArrayList<SelectionListener>();

        normalBorder = BorderFactory.createLineBorder(Color.GRAY);
        selectionBorder = BorderFactory.createLineBorder(Color.WHITE, 2);
        selection2Border = BorderFactory.createLineBorder(Color.YELLOW, 2);

        createMap();
    }

    public void selectTaskForcePath(int tf_index, boolean select)
    {
        if (selection1 != null)
        {
            try {
                // deselct
                if (prev_selection1 != null)
                {
                    map[prev_selection1.x][prev_selection1.y].markPaths(false);
                }

                map[selection1.x][selection1.y].markPaths(false);

                // set new index and select
                if (select)
                {
                    selected_taskforce = tf_index;
                    map[selection1.x][selection1.y].markPaths(true);
                }
                else
                {
                    selected_taskforce = -1;
                }

                this.refreshMap(false, false, false, false);
            }
            catch (InvalidArgumentsException ex)
            {
                UE.displayError(ex);
            }
        }
    }

    public void addActionListener(ActionListener l)
    {
        listeners.add(l);
    }

    public void addRightClickListener(SelectionListener l)
    {
        right_click_listeners.add(l);
    }

    private void createMap()
    {
        int vert = sectors.getVSectors();
        int hor = sectors.getHSectors();

        mapPanel.removeAll();

        mapPanel.setLayout(new GridLayout(vert, hor, 0, 0));

        map = new MapButton[hor][vert];

        for (int y = 0; y < vert; y++)
        {
            for (int x = 0; x < hor; x++)
            {
                map[x][y] = new MapButton(this, x, y);
                map[x][y].addActionListener(new MapSelectionListener(x, y));
                map[x][y].addMouseListener(new MapRightClickSelectionListener(x, y, y * hor + x));
                mapPanel.add(map[x][y]);
            }
        }

        refreshMap(true, true, true, true);
    }

    public void refreshMap(boolean ownership, boolean object, boolean military, boolean paths)
    {
        this.setVisible(false);

        if (military)
        {
            paths = true;
        }

        if (paths)
        {
            for (int y = 0; y < map[0].length; y++)
            {
                for (int x = 0; x < map.length; x++)
                {
                    map[x][y].clearMovementFlags(false);
                }
            }
        }

        for (int y = 0; y < map[0].length; y++)
        {
            for (int x = 0; x < map.length; x++)
            {
                try
                {
                    map[x][y].refresh(ownership, object, military, paths);
                }
                catch (Exception ex)
                {
                    UE.displayError(ex);
                }
            }
        }

        if (selection1 != null)
        {
            drawSelected(selection1, true);
        }
        this.setVisible(true);
    }

    public void setView(int view)
    {
        this.view = view;
        mask = (long)Math.pow(2, (view - 1));

        // clear selection
        if (selection1 != null)
        {
            drawSelected(selection1, false);
            selection1 = null;
        }
        selection2 = null;

        refreshMap(false, false, false, true);
    }

    private void drawSelected(Point pnt, boolean selected)
    {
        if (pnt == null || pnt.x >= map.length || pnt.y >= map[0].length)
        {
            return;
        }

        for (int y = 0; y < map[0].length; y++)
        {
            for (int x = 0; x < map.length; x++)
            {
                if (pnt.x != x || pnt.y != y)
                {
                    map[x][y].setBorder(normalBorder);
                }
            }
        }

        if (selected)
        {
            map[pnt.x][pnt.y].setBorder(selectionBorder);
            Point link = map[pnt.x][pnt.y].getLink();
            if (link != null)
            {
                map[link.x][link.y].setBorder(selection2Border);
            }
        }
        else
        {
            map[pnt.x][pnt.y].setBorder(normalBorder);
            Point link = map[pnt.x][pnt.y].getLink();
            if (link != null)
            {
                map[link.x][link.y].setBorder(normalBorder);
            }
        }
    }

    public void zoomIn()
    {
        Dimension dim = map[0][0].getPreferredSize();
        dim.height *= 1.1;
        dim.width = dim.height;
        map[0][0].setPreferredSize(dim);

        Dimension dp = mapPanel.getPreferredSize();
        dp.height = map[0].length * dim.height;
        dp.width = map.length * dim.width;

        mapPanel.setPreferredSize(dp);
        this.setPreferredSize(dp);

        refreshMap(false, false, false, false);
    }

    public void zoomOut()
    {
        Dimension dim = map[0][0].getPreferredSize();
        dim.height /= 1.1;

        if (dim.height < 10) dim.height = 10;

        dim.width = dim.height;
        map[0][0].setPreferredSize(dim);

        Dimension dp = mapPanel.getPreferredSize();
        dp.height = map[0].length * dim.height;
        dp.width = map.length * dim.width;

        mapPanel.setPreferredSize(dp);
        this.setPreferredSize(dp);

        refreshMap(false, false, false, false);
    }

    protected void fireAction(ActionEvent evt)
    {
        for (ActionListener l : listeners)
        {
            l.actionPerformed(evt);
        }
    }

    private void fireRightClick(int x, int y, SelectionEvent selectionEvent)
    {
        Point pnt = new Point(x, y);

        if (selection1 == null || !selection1.equals(pnt))
        {
            map[x][y].doClick();
        }
        
        for (SelectionListener l : right_click_listeners)
        {
            l.selected(selectionEvent);
        }
    }

    public void refreshMap()
    {
        refreshMap(drawOwnership, drawObjects,
                drawOutposts || drawShips, drawPaths);

        if (selection1 != null)
        {
            fireAction(new ActionEvent(map[selection1.x][selection1.y], ActionEvent.ACTION_PERFORMED, null));
        }
    }

    private class MapSelectionListener implements ActionListener
    {
        int x, y;
        
        public MapSelectionListener(int x, int y)
        {
            this.x = x;
            this.y = y;
        }
        @Override
        public void actionPerformed(ActionEvent arg)
        {
            Point pnt = new Point(x, y);

            prev_selection1 = selection1;
            
            if (selection1 != null)
            {
                if (selection1.equals(pnt))
                {
                    drawSelected(selection1, false);
                    selection1 = null;
                }
                else
                {
                    drawSelected(selection1, false);
                    selection1 = pnt;
                    drawSelected(selection1, true);

                    if (UE.DEBUG_MODE)
                    {
                        try {
                            for (int i = 0; i < 5; i++)
                            {
                                System.out.println("Sphere of influence " + i + ":" +
                                        sphOfInf[i].getInfluence(selection1.y * sectors.getHSectors() + selection1.x));
                            }
                        } catch (InvalidArgumentsException ex) {
                            UE.displayError(ex);
                        }
                    }
                }
            }
            else
            {
                selection1 = pnt;
                drawSelected(selection1, true);

                if (UE.DEBUG_MODE)
                {
                    try {
                        for (int i = 0; i < 5; i++)
                        {
                            System.out.println("Sphere of influence " + i + ":" +
                                    sphOfInf[i].getInfluence(selection1.y * sectors.getHSectors() + selection1.x));
                        }
                    } catch (InvalidArgumentsException ex) {
                        UE.displayError(ex);
                    }
                }
            }

            fireAction(new ActionEvent(map[x][y], ActionEvent.ACTION_PERFORMED, null));
        }
    }

    private class MapRightClickSelectionListener extends MouseAdapter
    {
        int x, y;
        int index;

        public MapRightClickSelectionListener(int x, int y, int index)
        {
            this.x = x; this.y = y;
            this.index = index;
        }

        @Override
        public void mouseClicked(MouseEvent e)
        {
            if (e.getButton() == MouseEvent.BUTTON3)
            {
                fireRightClick(x, y, new SelectionEvent(new int[]{-1, index}, e.getLocationOnScreen()));
            }
        }
    }
}
