/**
 * Ultimate Editor (dev)
 * Copyright (C) 2008 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: SectorEditDialog.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 17.11.2008
 */
package ue.gui.sav;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.FilenameFilter;
import java.util.Enumeration;

import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JTextField;

import ue.Language;
import ue.UE;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.rst.Race;
import ue.edit.res.stbof.smt.Objstruc;
import ue.edit.res.stbof.sst.ShipList;
import ue.edit.sav.GalInfo;
import ue.edit.sav.StarBaseInfo;
import ue.edit.sav.StellInfo;
import ue.edit.sav.lst.Sector;
import ue.exception.InvalidArgumentsException;
import ue.util.ID;

/**
 * Edits sector info.
 */
@SuppressWarnings("serial") //$NON-NLS-1$
public class SectorEditDialog extends JDialog implements WindowListener
{
    private Sector SECTOR;
    private StellInfo STELLAR;
    private GalInfo GALINFO;
    private StarBaseInfo BASEINFO;
    private Stbof STBOF;
    private Objstruc OBJ;
    private ShipList SHIPS;
    private Race RACES;
    
    private int SECTOR_INDEX;
    
    private JLabel[] lblLabel;
    private JComboBox cmbObjectType;
    private JComboBox cmbObjectAni;
    private JComboBox cmbBaseFunc;
    private JComboBox cmbBaseOwner;
    private JComboBox cmbBaseClass;
    private JTextField txtObjectName;
    private JTextField txtBaseName;
    
    private static final String strStellarObject = Language.getString("SectorEditDialog.0"); //$NON-NLS-1$
    private static final String strObjectType = Language.getString("SectorEditDialog.1"); //$NON-NLS-1$
    private static final String strObjectName = Language.getString("SectorEditDialog.2"); //$NON-NLS-1$
    private static final String strObjectAni = Language.getString("SectorEditDialog.3"); //$NON-NLS-1$
    private static final String strBaseFunc = Language.getString("SectorEditDialog.17"); //$NON-NLS-1$
    private static final String strBase = Language.getString("SectorEditDialog.18"); //$NON-NLS-1$
    private static final String strBaseOwner = Language.getString("SectorEditDialog.19"); //$NON-NLS-1$
    private static final String strBaseClass = Language.getString("SectorEditDialog.20"); //$NON-NLS-1$
    private static final String strBaseName = Language.getString("SectorEditDialog.21"); //$NON-NLS-1$
    private static final String strOutpost = Language.getString("SectorEditDialog.22"); //$NON-NLS-1$
    private static final String strStarbase = Language.getString("SectorEditDialog.23"); //$NON-NLS-1$
    
    public SectorEditDialog(int sector_index, Sector sector, StellInfo stell,
            GalInfo gal, StarBaseInfo sbinfo, Stbof stbof)
    {
        super();
        
        SECTOR = sector;
        STELLAR = stell;
        GALINFO = gal;
        BASEINFO = sbinfo;
        STBOF = stbof;
        
        if (STBOF != null)
        {
            try
            {
                OBJ = (Objstruc)STBOF.getInternalFile(Stbof.fileOBJSTRUC_SMT, true);
                SHIPS = (ShipList)STBOF.getInternalFile(Stbof.fileSHIPLIST_SST, true);
                RACES = (Race)STBOF.getInternalFile(Stbof.fileRACE_RST, true);
            }
            catch (Exception e)
            {
                OBJ = null;
                SHIPS = null;
            }
        }
        
        SECTOR_INDEX = sector_index;

        setupComponents();
        placeComponents();
        
        // init
        try
        {
            // stellar object
            int objID = SECTOR.getStellarObject(SECTOR_INDEX);
            
            if (objID < 0)
            {
                cmbObjectType.setSelectedIndex(0);
            }
            else
            {
                cmbObjectType.setSelectedIndex(STELLAR.getType(objID) + 1);
                cmbObjectAni.setSelectedItem(STELLAR.getAni(objID));
            }

            cmbObjectAni.setEnabled(objID >= 0);
            txtObjectName.setEnabled(objID >= 0);
            cmbObjectType.setEnabled(SECTOR.getSystem(SECTOR_INDEX) < 0);

            // starbase
            int outID = SECTOR.getOutpost(SECTOR_INDEX);
            
            if (outID < 0)
            {
                cmbBaseFunc.setSelectedIndex(0);
            }
            else
            {
                int index = BASEINFO.getIndex((short)outID);
                cmbBaseFunc.setSelectedIndex(BASEINFO.getType(index) - 5);
                cmbBaseOwner.setSelectedIndex(BASEINFO.getOwner(index));
                cmbBaseClass.setSelectedIndex(BASEINFO.getShipID(index));
            }

            cmbBaseOwner.setEnabled(outID >= 0);
            cmbBaseClass.setEnabled(outID >= 0);
            txtBaseName.setEnabled(outID >= 0);
            cmbBaseFunc.setEnabled(true);
            this.setTitle(UE.APP_NAME + " - " + SECTOR.getDescription(SECTOR_INDEX, false)); //$NON-NLS-1$
        }
        catch (InvalidArgumentsException e)
        {
            UE.displayError(e);
        }

        this.addWindowListener(this);
        
        pack();

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension pref = this.getPreferredSize();
        setLocation(
                new Point(
                        (int)(screenSize.getWidth()-pref.getWidth())/2,
                        (int)(screenSize.getHeight()-pref.getHeight())/2
                        )
                );
        
        this.setIconImage(UE.WINDOW.getIconImage());
    }

    private void setupComponents()
    {
        Font def = UE.SETTINGS.getDefaultFont();

        // labels
        lblLabel = new JLabel[9];
        lblLabel[0] = new JLabel(strStellarObject);
        lblLabel[1] = new JLabel(strObjectType);
        lblLabel[2] = new JLabel(strObjectName);
        lblLabel[3] = new JLabel(strObjectAni);
        lblLabel[4] = new JLabel(strBase);
        lblLabel[5] = new JLabel(strBaseFunc);
        lblLabel[6] = new JLabel(strBaseOwner);
        lblLabel[7] = new JLabel(strBaseClass);
        lblLabel[8] = new JLabel(strBaseName);
        
        for (int i = 0; i < lblLabel.length; i++)
        {
            lblLabel[i].setFont(def);
        }
        
        // object animation
        cmbObjectAni = new JComboBox();
        cmbObjectAni.setFont(def);
        cmbObjectAni.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                if (cmbObjectAni.getSelectedIndex() < 0
                        || !cmbObjectAni.isEnabled())
                {
                    return;
                }
                try
                {
                    String ani = cmbObjectAni.getSelectedItem().toString();
                    int objID = SECTOR.getStellarObject(SECTOR_INDEX);
                    
                    if (objID >= 0)
                    {
                        STELLAR.setAni(objID, ani);
                    }
                }
                catch (Exception x)
                {
                    UE.displayError(x);
                }
            }
        });
        
        // object type
        cmbObjectType = new JComboBox();
        cmbObjectType.setFont(def);
        cmbObjectType.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    if (cmbObjectType.getSelectedIndex() < 0
                            || !cmbObjectType.isEnabled())
                    {
                        return;
                    }
                    
                    int type = cmbObjectType.getSelectedIndex() - 1;
                    
                    if (type >= -1)
                    {
                        int objID = SECTOR.getStellarObject(SECTOR_INDEX);
                        int oldType = -1;
                        
                        if (objID >= 0)
                        {
                            oldType = STELLAR.getType(objID);
                        }
                            
                        if (type != oldType)
                        {
                            if (objID < 0)
                            {
                                int h = SECTOR.getHorizontal(SECTOR_INDEX);
                                int v = SECTOR.getVertical(SECTOR_INDEX);
                                objID = STELLAR.add(h, v);
                                STELLAR.setType(objID, type);
                                String loc = SECTOR.getDescription(SECTOR_INDEX, false).substring(7);
                                txtObjectName.setText(cmbObjectType.getSelectedItem().toString() +
                                        " " + loc); //$NON-NLS-1$

                                if (OBJ != null)
                                {
                                    cmbObjectAni.setSelectedItem(OBJ.getGraph(type));
                                    STELLAR.setAni(objID, OBJ.getGraph(type));
                                }
                            }
                            else
                            {
                                if (type < 0)
                                {
                                    STELLAR.remove(objID);
                                    
                                    int obj;
                                    
                                    for (int i = 0; i < SECTOR.getNumberOfSectors(); i++)
                                    {
                                        obj = SECTOR.getStellarObject(i); 
                                        
                                        if (obj >= 0 && obj > objID)
                                        {
                                            SECTOR.setStellarObject(i, (short)(obj - 1));
                                        }
                                    }

                                    txtObjectName.setText(""); //$NON-NLS-1$
                                    
                                    objID = type;
                                }
                                else
                                {
                                    STELLAR.setType(objID, type);

                                    String loc = SECTOR.getDescription(SECTOR_INDEX, false).substring(7);
                                    if (txtObjectName.getText().length() == 0
                                            || txtObjectName.getText().endsWith(loc))
                                    {
                                        txtObjectName.setText(cmbObjectType.getSelectedItem().toString() +
                                            " " + loc); //$NON-NLS-1$
                                    }

                                    if (OBJ != null)
                                    {
                                        cmbObjectAni.setSelectedItem(OBJ.getGraph(type));
                                    }
                                }
                            }
                            
                            GALINFO.setAnomalyCount((short) STELLAR.getNumberOfEntries());
                            cmbObjectAni.setEnabled(objID >= 0);
                            txtObjectName.setEnabled(objID >= 0);
                            SECTOR.setStellarObject(SECTOR_INDEX, (short)objID);
                        }
                    }
                }
                catch (Exception x)
                {
                    UE.displayError(x);
                }
            }
        });
        
        txtObjectName = new JTextField(24);
        txtObjectName.setFont(def);
        try
        {
            int objID = SECTOR.getStellarObject(SECTOR_INDEX);
            if (objID >= 0)
            {
                txtObjectName.setText(STELLAR.getName(objID));
            }
        }
        catch (Exception e)
        {
            UE.displayError(e);
        }
        
        // base func
        cmbBaseFunc = new JComboBox();
        cmbBaseFunc.setFont(def);
        cmbBaseFunc.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    if (cmbBaseFunc.getSelectedIndex() < 0
                            || !cmbBaseFunc.isEnabled())
                    {
                        return;
                    }
                    
                    short func = (short)(cmbBaseFunc.getSelectedIndex() + 5);

                    if (func >= 5)
                    {
                        int outID = SECTOR.getOutpost(SECTOR_INDEX);
                        
                        outID = BASEINFO.getIndex((short)outID);
                        
                        short oldFunc = 5;
                        
                        if (outID >= 0)
                        {
                            oldFunc = BASEINFO.getType(outID);
                        }
                        
                        if (func != oldFunc)
                        {
                            if (outID < 0)
                            {
                                int h = SECTOR.getHorizontal(SECTOR_INDEX);
                                int v = SECTOR.getVertical(SECTOR_INDEX);

                                outID = BASEINFO.add((short)SECTOR_INDEX, func, v, h);
                                
                                int shipID = 0;
                                int race = 0;

                                if (SHIPS != null)
                                {
                                    ID[] ids = SHIPS.getIDs(37 + func);
                                    
                                    
                                    if (ids.length > 0)
                                    {
                                        shipID = ids[0].hashCode();
                                        race = SHIPS.getRace(shipID);
                                        BASEINFO.setHull(outID, SHIPS.getHullStr(shipID));
                                    }
                                }
                                
                                txtBaseName.setText(cmbBaseFunc.getSelectedItem().toString());
                                BASEINFO.setName(txtBaseName.getText());
                                cmbBaseOwner.setSelectedIndex(race);
                                cmbBaseClass.setSelectedIndex(shipID);
                            }
                            else
                            {
                                if (func < 6)
                                {
                                    BASEINFO.remove(outID);
                                    
                                    txtBaseName.setText(""); //$NON-NLS-1$
                                    
                                    outID = -1;
                                }
                                else
                                {
                                    BASEINFO.setType(outID, func);

                                    int shipID = 0;
                                    short race = (short)cmbBaseOwner.getSelectedIndex();

                                    if (SHIPS != null)
                                    {
                                        ID[] ids = SHIPS.getIDs(37 + cmbBaseFunc.getSelectedIndex() + 5);
                                        
                                        for (ID id : ids)
                                        {
                                            shipID = id.hashCode();
                                            
                                            if (SHIPS.getRace(shipID) == race)
                                            {
                                                cmbBaseClass.setSelectedIndex(shipID);
                                                BASEINFO.setHull(outID, SHIPS.getHullStr(shipID));
                                                break;
                                            }
                                        }
                                    }

                                    String name = BASEINFO.getName(outID);

                                    if (name.equals(strOutpost))
                                    {
                                        name = strStarbase;
                                    }
                                    else if (name.equals(strStarbase))
                                    {
                                        name = strOutpost;
                                    }
                                    
                                    txtBaseName.setText(name);
                                    BASEINFO.setName(outID, name);
                                }
                            }
                            
                            GALINFO.setBaseCount((short)BASEINFO.getNumberOfEntries());
                            cmbBaseOwner.setEnabled(outID >= 0);
                            cmbBaseClass.setEnabled(outID >= 0);
                            txtBaseName.setEnabled(outID >= 0);
                            if (outID < 0)
                            {
                                SECTOR.setStarbase(SECTOR_INDEX, (short)-1);
                            }
                            else
                            {
                                SECTOR.setStarbase(SECTOR_INDEX, (short)SECTOR_INDEX);
                            }
                        }
                    }
                }
                catch (Exception x)
                {
                    UE.displayError(x);
                }
            }
        });

        // base owner
        cmbBaseOwner = new JComboBox();
        cmbBaseOwner.setFont(def);
        cmbBaseOwner.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                if (cmbBaseOwner.getSelectedIndex() < 0
                        || !cmbBaseOwner.isEnabled())
                {
                    return;
                }
                
                try
                {
                    int outID = SECTOR.getOutpost(SECTOR_INDEX);
                    outID = BASEINFO.getIndex((short)outID);
                    
                    if (outID >= 0)
                    {
                        int shipID = 0;
                        short race = (short)cmbBaseOwner.getSelectedIndex();

                        BASEINFO.setOwner(outID, race);

                        if (SHIPS != null)
                        {
                            ID[] ids = SHIPS.getIDs(37 + cmbBaseFunc.getSelectedIndex() + 5);
                            
                            for (ID id : ids)
                            {
                                shipID = id.hashCode();
                                
                                if (SHIPS.getRace(shipID) == race)
                                {
                                    cmbBaseClass.setSelectedIndex(shipID);
                                    BASEINFO.setHull(outID, SHIPS.getHullStr(shipID));
                                    break;
                                }
                            }
                        }
                    }
                }
                catch (Exception x)
                {
                    UE.displayError(x);
                }
            }
        });

        // base class
        cmbBaseClass = new JComboBox();
        cmbBaseClass.setFont(def);
        cmbBaseClass.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                if (cmbBaseOwner.getSelectedIndex() < 0
                        || !cmbBaseOwner.isEnabled()
                        || SHIPS == null)
                {
                    return;
                }
                
                try
                {
                    int outID = SECTOR.getOutpost(SECTOR_INDEX);
                    outID = BASEINFO.getIndex((short)outID);
                    
                    if (outID >= 0)
                    {
                        BASEINFO.setShipID(outID, (short)cmbBaseClass.getSelectedIndex());
                    }
                }
                catch (Exception x)
                {
                    UE.displayError(x);
                }
            }
        });

        txtBaseName = new JTextField(24);
        txtBaseName.setFont(def);
        try
        {
            int outID = SECTOR.getOutpost(SECTOR_INDEX);
            outID = BASEINFO.getIndex((short)outID);

            if (outID >= 0)
            {
                txtBaseName.setText(BASEINFO.getName(outID));
            }
        }
        catch (Exception e)
        {
            UE.displayError(e);
        }

        // fill base class
        cmbBaseClass.setEnabled(false);
        if (SHIPS != null)
        {
            ID[] ids = SHIPS.getIDs(35);
            
            for (int s = 0; s < ids.length; s++)
            {
                cmbBaseClass.addItem(new ID(s + ": " + ids[s].toString(), s)); //$NON-NLS-1$
            }
        }

        // fill base owner
        cmbBaseOwner.setEnabled(false);
        for (int r = 0; r < 5; r++)
        {
            if (RACES == null)
            {
                cmbBaseOwner.addItem(new ID(Integer.toString(r), r));
            }
            else
            {
                try
                {
                    cmbBaseOwner.addItem(new ID(RACES.getName(r), r));
                }
                catch (Exception e)
                {
                    cmbBaseOwner.addItem(new ID(Integer.toString(r), r));
                }
            }
        }
        
        // fill base function
        cmbBaseFunc.setEnabled(false);
        cmbBaseFunc.addItem(new ID(Language.getString("SectorEditDialog.4"), 0)); //$NON-NLS-1$
        cmbBaseFunc.addItem(new ID(strOutpost, 1));
        cmbBaseFunc.addItem(new ID(strStarbase, 2));

        // fill animation
        Enumeration<String[]> en = STBOF.getFiles(new FilenameFilter()
        {
            @Override
            public boolean accept(File dir, String name)
            {
                return name.endsWith("maps.tga"); //$NON-NLS-1$
            }
        });
        
        String str;
        cmbObjectAni.setEnabled(false);
        while (en.hasMoreElements())
        {
            str = en.nextElement()[0];
            str = str.substring(0, str.lastIndexOf("maps.tga")); //$NON-NLS-1$
            cmbObjectAni.addItem(str);
        }

        // fill type
        cmbObjectType.setEnabled(false);
        cmbObjectType.addItem(new ID(Language.getString("SectorEditDialog.4"), 0)); //$NON-NLS-1$
        cmbObjectType.addItem(new ID(Language.getString("SectorEditDialog.5"), 1)); //$NON-NLS-1$
        cmbObjectType.addItem(new ID(Language.getString("SectorEditDialog.6"), 2)); //$NON-NLS-1$
        cmbObjectType.addItem(new ID(Language.getString("SectorEditDialog.7"), 3)); //$NON-NLS-1$
        cmbObjectType.addItem(new ID(Language.getString("SectorEditDialog.8"), 4)); //$NON-NLS-1$
        cmbObjectType.addItem(new ID(Language.getString("SectorEditDialog.9"), 5)); //$NON-NLS-1$
        cmbObjectType.addItem(new ID(Language.getString("SectorEditDialog.10"), 6)); //$NON-NLS-1$
        cmbObjectType.addItem(new ID(Language.getString("SectorEditDialog.11"), 7)); //$NON-NLS-1$
        cmbObjectType.addItem(new ID(Language.getString("SectorEditDialog.12"), 8)); //$NON-NLS-1$
        cmbObjectType.addItem(new ID(Language.getString("SectorEditDialog.13"), 9)); //$NON-NLS-1$
        cmbObjectType.addItem(new ID(Language.getString("SectorEditDialog.14"), 10)); //$NON-NLS-1$
        cmbObjectType.addItem(new ID(Language.getString("SectorEditDialog.15"), 11)); //$NON-NLS-1$
        cmbObjectType.addItem(new ID(Language.getString("SectorEditDialog.16"), 12)); //$NON-NLS-1$
    }

    public void placeComponents()
    {
        setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.anchor = GridBagConstraints.NORTH;
        c.insets = new Insets(5, 5, 5, 5);
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.gridy = 0;
        add(lblLabel[0], c);
        c.gridwidth = 1;
        for (int i = 1; i < 4; i++)
        {
            c.gridy = i;
            add(lblLabel[i], c);
        }
        c.gridwidth = 2;
        c.gridy = 4;
        add(lblLabel[4], c);
        c.gridwidth = 1;
        for (int i = 5; i < 9; i++)
        {
            c.gridy = i;
            add(lblLabel[i], c);
        }
        c.gridx = 1;
        c.gridy = 1;
        add(cmbObjectType, c);
        c.gridy = 2;
        add(txtObjectName, c);
        c.gridy = 3;
        add(cmbObjectAni, c);
        c.gridy = 5;
        add(cmbBaseFunc, c);
        c.gridy = 6;
        add(cmbBaseOwner, c);
        c.gridy = 7;
        if (SHIPS != null)
        {
            add(cmbBaseClass, c);
        }
        c.gridy = 8;
        add(txtBaseName, c);
    }

    /* (non-Javadoc)
     * @see java.awt.event.WindowListener#windowActivated(java.awt.event.WindowEvent)
     */
    @Override
    public void windowActivated(WindowEvent e)
    {
    }

    /* (non-Javadoc)
     * @see java.awt.event.WindowListener#windowClosed(java.awt.event.WindowEvent)
     */
    @Override
    public void windowClosed(WindowEvent e)
    {
        try
        {
            int id = SECTOR.getStellarObject(SECTOR_INDEX);
            if (id >= 0)
                STELLAR.setName(id, txtObjectName.getText());

            id = SECTOR.getOutpost(SECTOR_INDEX);
            if (id >= 0)
            {
                id = BASEINFO.getIndex((short)id);
                BASEINFO.setName(id, txtBaseName.getText());
            }
        }
        catch (InvalidArgumentsException e1)
        {
        }
    }

    /* (non-Javadoc)
     * @see java.awt.event.WindowListener#windowClosing(java.awt.event.WindowEvent)
     */
    public void windowClosing(WindowEvent e)
    {
        try
        {
            int id = SECTOR.getStellarObject(SECTOR_INDEX);
            if (id >= 0)
                STELLAR.setName(id, txtObjectName.getText());

            id = SECTOR.getOutpost(SECTOR_INDEX);
            if (id >= 0)
            {
                id = BASEINFO.getIndex((short)id);
                BASEINFO.setName(id, txtBaseName.getText());
            }
        }
        catch (InvalidArgumentsException e1)
        {
        }
    }

    /* (non-Javadoc)
     * @see java.awt.event.WindowListener#windowDeactivated(java.awt.event.WindowEvent)
     */
    public void windowDeactivated(WindowEvent e)
    {
    }

    /* (non-Javadoc)
     * @see java.awt.event.WindowListener#windowDeiconified(java.awt.event.WindowEvent)
     */
    public void windowDeiconified(WindowEvent e)
    {
    }

    /* (non-Javadoc)
     * @see java.awt.event.WindowListener#windowIconified(java.awt.event.WindowEvent)
     */
    public void windowIconified(WindowEvent e)
    {
    }

    /* (non-Javadoc)
     * @see java.awt.event.WindowListener#windowOpened(java.awt.event.WindowEvent)
     */
    public void windowOpened(WindowEvent e)
    {
    }
}
