/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: StarBaseInfoGUI.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui.sav;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import ue.Language;
import ue.UE;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.rst.Race;
import ue.edit.res.stbof.sst.ShipList;
import ue.edit.sav.StarBaseInfo;
import ue.gui.MainPanel;
import ue.util.ID;
import ue.util.SettingsManager;

/**Used to edit stellar objects info in a saved game.*/
@SuppressWarnings("serial")  //$NON-NLS-1$
public class StarBaseInfoGUI extends MainPanel{
	private StarBaseInfo BASE;
	private Race RACE;
        private ShipList SHIP;
	//items
        private JComboBox jcbBASE = new JComboBox();
        private JComboBox jcbSHIP_ID = new JComboBox();
        private JComboBox jcbTYPE = new JComboBox();
        private JComboBox jcbRACE = new JComboBox();
        private JLabel[] lblLABEL = new JLabel[6];
        private JLabel lblMaxHull = new JLabel();
        private JTextField txtHull = new JTextField();
        private JButton btnEDIT = new JButton(Language.getString("StarBaseInfoGUI.0")); //$NON-NLS-1$

        private int SELECTED = -1;
        
        /**Constructor.*/
	public StarBaseInfoGUI(StarBaseInfo sb, Stbof st)
	{
		if (sb==null || st==null)
                {
			return;
		}
		//associate
		BASE=sb;
		try
                {
			RACE = (Race)st.getInternalFile(Stbof.fileRACE_RST, true);
                        SHIP = (ShipList)st.getInternalFile(Stbof.fileSHIPLIST_SST, true);
		}
                catch (Exception er)
                {
			UE.displayError(er);
			return;
		}
		if (RACE==null) {
			return;
		}
		//build
                lblLABEL[0] = new JLabel(Language.getString("StarBaseInfoGUI.7")); //$NON-NLS-1$
		lblLABEL[1] = new JLabel(Language.getString("StarBaseInfoGUI.1")); //$NON-NLS-1$
		lblLABEL[2] = new JLabel(Language.getString("StarBaseInfoGUI.2")); //$NON-NLS-1$
                lblLABEL[3] = new JLabel(Language.getString("StarBaseInfoGUI.8")); //$NON-NLS-1$
                lblLABEL[4] = new JLabel(Language.getString("StarBaseInfoGUI.9")); //$NON-NLS-1$
		lblLABEL[5] = new JLabel();
		//set up
		Font def = UE.SETTINGS.getDefaultFont();
		jcbBASE.setFont(def);
		btnEDIT.setFont(def);
                jcbSHIP_ID.setFont(def);
		jcbTYPE.setFont(def);
		jcbRACE.setFont(def);
                lblMaxHull.setFont(def);
                txtHull.setFont(def);
		for (int i = 0; i < 6; i++)
                {
			lblLABEL[i].setFont(def);
		}
		//edit
		btnEDIT.addActionListener(new ActionListener()
                {
		    public void actionPerformed(ActionEvent ae)
                    {
		        if (SELECTED >= 0)
                        {
		            try
                            {
		                String n = JOptionPane.showInputDialog(Language.getString("StarBaseInfoGUI.3"), BASE.getName(SELECTED)); //$NON-NLS-1$
		                if (n!=null)
                                {
		                    int ij = SELECTED;
		                    BASE.setName(ij, n);
		                    fillBase();
		                    jcbBASE.setSelectedIndex(ij);
                                }
                            }
                            catch (Exception ze)
                            {
                                UE.displayError(ze);
                            }
                        }
                    }
		});
		//combos
		jcbBASE.addActionListener(new ActionListener()
        {
            @Override
		    public void actionPerformed(ActionEvent ae)
            {
                try
                {
                    finalWarning();
                }
                catch (Exception ex)
                {
                    // really bad stuff happening - bail
                    UE.displayError(ex);
                    return;
                }
                        
		        if (jcbBASE.getSelectedIndex() >= 0)
                        {
		            SELECTED = jcbBASE.getSelectedIndex();
                            
		            try
                            {
		                //set
		                int lu = BASE.getType(SELECTED);
		                if (lu < 7)
                                {
		                    lu=0;
                                }
                                else
                                {
                                    lu = 1;
                                }
		                jcbTYPE.setSelectedIndex(lu);
		                jcbRACE.setSelectedIndex(BASE.getOwner(SELECTED));
		                String msg = Language.getString("StarBaseInfoGUI.4"); //$NON-NLS-1$
		                msg = msg.replace("%1", BASE.getDescription(SELECTED)); //$NON-NLS-1$
		                lblLABEL[5].setText(msg);
		                jcbSHIP_ID.setSelectedItem(new ID("", BASE.getShipID(SELECTED))); //$NON-NLS-1$
		                txtHull.setText(Integer.toString(BASE.getHull(SELECTED)));
                            }
                            catch (Exception ze)
                            {
                                UE.displayError(ze);
                            }
                        }
                    }
		});
                jcbSHIP_ID.addActionListener(new ActionListener()
                {
                    public void actionPerformed(ActionEvent ae)
                    {
                        if (SELECTED >= 0
                                && jcbSHIP_ID.getSelectedIndex() >= 0)
                        {
                            try
                            {
                                short id = (short)((ID)jcbSHIP_ID.getSelectedItem()).hashCode();
                                
                                BASE.setShipID(SELECTED, id);

                                int max = SHIP.getHullStr(id);
                                
                                lblMaxHull.setText(Integer.toString(max));
                                
                                try
                                {
                                    int hull = Integer.parseInt(txtHull.getText());
                                    
                                    if (hull > max || hull <= 0)
                                    {
                                        txtHull.setText(Integer.toString(max));
                                    }
                                }
                                catch (Exception e)
                                {
                                    txtHull.setText(Integer.toString(max));
                                }
                            }
                            catch (Exception ze)
                            {
                                UE.displayError(ze);
                            }
                        }
                    }
                });
		jcbTYPE.addActionListener(new ActionListener()
                {
		    public void actionPerformed(ActionEvent ae)
                    {
		        if (SELECTED >= 0 && jcbTYPE.getSelectedIndex() >= 0)
                        {
		            try
                            {
		                //set
		                BASE.setType(SELECTED, (short)(jcbTYPE.getSelectedIndex()+6));
                            }
                            catch (Exception ze)
                            {
                                UE.displayError(ze);
                            }
                        }
                    }
		});
		jcbRACE.addActionListener(new ActionListener()
                {
		    public void actionPerformed(ActionEvent ae)
                    {
		        if (SELECTED >= 0)
                        {
		            try
                            {
		                //set
		                BASE.setOwner(SELECTED, (short)jcbRACE.getSelectedIndex());
                            }
                            catch (Exception ze)
                            {
                                UE.displayError(ze);
                            }
                        }
                    }
		});
		//lay out
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets=new Insets(5, 5, 5, 5);
		c.fill=GridBagConstraints.BOTH;
		//up
		c.gridx=0;
		c.gridy=0;
		c.gridwidth=2;
		add(jcbBASE, c);
		c.gridwidth=1;
		c.gridx=2;
		add(btnEDIT, c);
		c.gridx=0;
		c.gridheight=1;
		for (int i=0; i<2; i++){
			c.gridy=1+i;
			add(lblLABEL[i], c);
		}
		c.gridy=3;
		add(lblLABEL[2], c);
                c.gridwidth=3;
                c.gridy=4;
                add(lblLABEL[3], c);
                c.gridy=5;
                add(lblLABEL[4], c);
                c.gridy=6;
                add(lblLABEL[5], c);
		c.gridwidth=2;
		c.gridx=1;
		c.gridy=1;
                add(jcbSHIP_ID, c);
                c.gridy=2;
                add(jcbTYPE, c);
		c.gridy=3;
		add(jcbRACE, c);
                c.gridy=4;
                add(lblMaxHull, c);
                c.gridy=5;
                add(txtHull, c);
		//fill
		jcbTYPE.removeAllItems();
		jcbTYPE.addItem(Language.getString("StarBaseInfoGUI.5")); //$NON-NLS-1$
		jcbTYPE.addItem(Language.getString("StarBaseInfoGUI.6")); //$NON-NLS-1$
		jcbRACE.removeAllItems();
		for (int i=0; i<5; i++){
			try{
				jcbRACE.addItem(new ID(RACE.getName(i), i));
			}catch (Exception ze){
				UE.displayError(ze);
			}
		}
                jcbSHIP_ID.removeAllItems();
                ID[] ids = SHIP.getIDs(35);
                for (int i = 0; i < ids.length; i++)
                {
                    try
                    {
                        jcbSHIP_ID.addItem(new ID(ids[i].hashCode() + ": " //$NON-NLS-1$
                                + ids[i].toString(), ids[i].hashCode()));
                    }catch (Exception ze){
                        UE.displayError(ze);
                    }
                }

                fillBase();
	}
	//fill objects
	private void fillBase(){
		String[] n = BASE.getNames();
		if (n.length==0){
			jcbBASE.removeAllItems();
			jcbBASE.setEnabled(false);
			jcbTYPE.setEnabled(false);
			jcbRACE.setEnabled(false);
			btnEDIT.setEnabled(false);
                        jcbSHIP_ID.setEnabled(false);
		}else{
                    jcbSHIP_ID.setEnabled(true);
			jcbBASE.setEnabled(true);
			jcbTYPE.setEnabled(true);
			jcbRACE.setEnabled(true);
			btnEDIT.setEnabled(true);
			jcbBASE.removeAllItems();
			for (int i=0; i<n.length; i++){
			    jcbBASE.addItem(new ID(n[i], i));
			}
		}
	}
	//final warning
	@Override
	public void finalWarning() throws Exception
        {
	    if (SELECTED >= 0)
	    {
            int hull = Integer.parseInt(txtHull.getText());

            short id = (short)((ID)jcbSHIP_ID.getSelectedItem()).hashCode();

            if (SHIP.getHullStr(id) < hull)
            {
                hull = SHIP.getHullStr(id);
            }

            if (hull <= 0)
            {
                hull = 1;
            }

            BASE.setHull(SELECTED, hull);
        }
    }
}
