/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: StellInfoGUI.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui.sav;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import ue.Language;
import ue.UE;
import ue.edit.sav.StellInfo;
import ue.gui.MainPanel;
import ue.util.ID;
import ue.util.SettingsManager;

/**Used to edit stellar objects info in a saved game.*/
@SuppressWarnings("serial")  //$NON-NLS-1$
public class StellInfoGUI extends MainPanel{
	private StellInfo STELLAR;
	private int SELECTED = -1;
	//items
	JComboBox jcbSTELL = new JComboBox();
	JComboBox jcbTYPE = new JComboBox();
	JTextField txtANI = new JTextField();
	JComboBox jcbLINK = new JComboBox();
	JLabel[] lblLABEL = new JLabel[3];
	JButton btnEDIT = new JButton(Language.getString("StellInfoGUI.0")); //$NON-NLS-1$
	/**Constructor.*/
	public StellInfoGUI(StellInfo si){
		//associate
		STELLAR=si;
		//build
		lblLABEL[0] = new JLabel(Language.getString("StellInfoGUI.1")); //$NON-NLS-1$
		lblLABEL[1] = new JLabel(Language.getString("StellInfoGUI.2")); //$NON-NLS-1$
		lblLABEL[2] = new JLabel(Language.getString("StellInfoGUI.3")); //$NON-NLS-1$
		//set up
		Font def = UE.SETTINGS.getDefaultFont();
		jcbSTELL.setFont(def);
		btnEDIT.setFont(def);
		jcbTYPE.setFont(def);
		txtANI.setFont(def);
		jcbLINK.setFont(def);
		for (int i=0; i<3; i++){
			lblLABEL[i].setFont(def);
		}
		//edit
		btnEDIT.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				if (jcbSTELL.getSelectedIndex()!=-1){
					try{
						String n = JOptionPane.showInputDialog(Language.getString("StellInfoGUI.4"), STELLAR.getName(jcbSTELL.getSelectedIndex())); //$NON-NLS-1$
						if (n!=null){
							int ij = jcbSTELL.getSelectedIndex();
							STELLAR.setName(ij, n);
							fillObjects();
							jcbSTELL.setSelectedIndex(ij);
						}
					}catch (Exception ze){
						UE.displayError(ze);
					}
				}
			}
		});
		//combos
		jcbSTELL.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				if (jcbSTELL.getSelectedIndex()!=-1){
					try{
						finalWarning();
						
						SELECTED = jcbSTELL.getSelectedIndex();
						//set
						jcbTYPE.setSelectedIndex(STELLAR.getType(SELECTED));
						txtANI.setText(STELLAR.getAni(SELECTED));
						int al = STELLAR.getLink(SELECTED);
						if (al==-1){
							jcbLINK.setSelectedIndex(jcbLINK.getItemCount()-1);
						}else{
							jcbLINK.setSelectedIndex(al);
						}
					}catch (Exception ze){
						UE.displayError(ze);
					}
				}
			}
		});
		jcbTYPE.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				if (jcbSTELL.getSelectedIndex()!=-1 && jcbTYPE.getSelectedIndex()>-1){
					try{
						//set
						STELLAR.setType(jcbSTELL.getSelectedIndex(), jcbTYPE.getSelectedIndex());
						jcbTYPE.setSelectedIndex(STELLAR.getType(jcbSTELL.getSelectedIndex()));
					}catch (Exception ze){
						UE.displayError(ze);
					}
				}
			}
		});
		jcbLINK.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				if (jcbSTELL.getSelectedIndex()!=-1 && jcbLINK.getSelectedIndex()>-1){
					try{
						//set
						if (jcbLINK.getSelectedIndex()==jcbLINK.getItemCount()-1){
							STELLAR.setLink(jcbSTELL.getSelectedIndex(), (short)-1);
						}else{
							STELLAR.setLink(jcbSTELL.getSelectedIndex(), (short)jcbLINK.getSelectedIndex());
						}
					}catch (Exception ze){
						UE.displayError(ze);
					}
				}
			}
		});
		//lay out
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets=new Insets(5, 5, 5, 5);
		c.fill=GridBagConstraints.BOTH;
		//up
		c.gridx=0;
		c.gridy=0;
		c.gridwidth=2;
		add(jcbSTELL, c);
		c.gridwidth=1;
		c.gridx=2;
		add(btnEDIT, c);
		c.gridx=0;
		c.gridheight=1;
		for (int i=0; i<3; i++){
			c.gridy=1+i;
			add(lblLABEL[i], c);
		}
		c.gridwidth=2;
		c.gridx=1;
		c.gridy=1;
		add(jcbTYPE, c);
		c.gridy=2;
		add(txtANI, c);
		c.gridy=3;
		add(jcbLINK, c);
		//fill combo
		//type
		jcbTYPE.removeAllItems();
		jcbTYPE.addItem(Language.getString("StellInfoGUI.5")); //$NON-NLS-1$
		jcbTYPE.addItem(Language.getString("StellInfoGUI.6")); //$NON-NLS-1$
		jcbTYPE.addItem(Language.getString("StellInfoGUI.7")); //$NON-NLS-1$
		jcbTYPE.addItem(Language.getString("StellInfoGUI.8")); //$NON-NLS-1$
		jcbTYPE.addItem(Language.getString("StellInfoGUI.9")); //$NON-NLS-1$
		jcbTYPE.addItem(Language.getString("StellInfoGUI.10")); //$NON-NLS-1$
		jcbTYPE.addItem(Language.getString("StellInfoGUI.11")); //$NON-NLS-1$
		jcbTYPE.addItem(Language.getString("StellInfoGUI.12")); //$NON-NLS-1$
		jcbTYPE.addItem(Language.getString("StellInfoGUI.13")); //$NON-NLS-1$
		jcbTYPE.addItem(Language.getString("StellInfoGUI.14")); //$NON-NLS-1$
		jcbTYPE.addItem(Language.getString("StellInfoGUI.15")); //$NON-NLS-1$
		jcbTYPE.addItem(Language.getString("StellInfoGUI.16")); //$NON-NLS-1$
		fillObjects();
	}
	//fill objects
	private void fillObjects(){
		String[] n = STELLAR.getNames();
		jcbSTELL.removeAllItems();
		jcbLINK.removeAllItems();
		//link first
		for (int i=0; i<n.length; i++){
			jcbLINK.addItem(new ID(n[i], i));
		}
		jcbLINK.addItem(new ID(Language.getString("StellInfoGUI.17"), jcbLINK.getItemCount())); //$NON-NLS-1$
		for (int i=0; i<n.length; i++){
			jcbSTELL.addItem(new ID(n[i], i));
		}
	}
	//final warning
	@Override
	public void finalWarning() throws Exception{
		if (SELECTED>=0){
            STELLAR.setAni(SELECTED, txtANI.getText());
		}
	}
}
