
/*
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * 
 * File: StrcInfoGUI.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 2.12.2010
 */
package ue.gui.sav;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;


import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.ScrollPaneConstants;
import javax.swing.SpinnerListModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import ue.UE;
import ue.edit.exe.Trek;
import ue.edit.exe.trek.PlanetBonusTypes;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.bin.AIBldReq;
import ue.edit.res.stbof.bin.Planboni;
import ue.edit.res.stbof.bst.Edifice;
import ue.edit.res.stbof.dic.Lexicon;
import ue.edit.res.stbof.est.Environ;
import ue.edit.res.stbof.rst.Race;
import ue.edit.sav.AlienInfo;
import ue.edit.sav.SavGame;
import ue.edit.sav.StrcInfo;
import ue.edit.sav.SystInfo;
import ue.edit.sav.TechInfo;
import ue.exception.InvalidArgumentsException;
import ue.gui.CyclingHorizontalSpinner;
import ue.gui.InvisibleScrollBarUI;
import ue.gui.MainPanel;
import ue.gui.UEComboBoxUI;
import ue.util.ID;

/**Used to edit game info of a saved game.*/
@SuppressWarnings("serial")  //$NON-NLS-1$
public class StrcInfoGUI extends MainPanel
{
    private Edifice             EDIFICE;
    private Environ             ENVIRON;
    private AIBldReq            AIBLDREQ;
    private Race                RACE;
    private Lexicon             LEX;
    private Planboni            PLANBONI;

    private PlanetBonusTypes    PLANBONITYPES;

    private StrcInfo            STRUCT;
    private SystInfo            SYSTEMS;
    private AlienInfo           ALIENINFO;
    private TechInfo            TECHINFO;

    private Stbof               STBOF;
    private Trek                EXE;
    private SavGame             GAME;

    //items
    private CyclingHorizontalSpinner spiSystems;
    private JPanel pnlTable;
    private JLabel lblDilithium;
    private JLabel lblPlanetTypes;
    private JLabel lblPopulation;
    private JLabel lblEnOutput;
    private JLabel lblEnConsumed;
    private JButton btnSystemView;
    private JComboBox cmbSystems;
    private JComboBox cmbFilterSystems;
    private JScrollPane scrHeader;
    private JScrollPane scrTable;
    private StrcHeaderLine header;

    private final Font def;
    private final Font tableDef;

    private ArrayList<StrcLine> tableLines = new ArrayList<StrcLine>();

    public StrcInfoGUI(SavGame game, Stbof res, Trek exe, int systemID)
    {
        this(game, res, exe);
        spiSystems.setValue(new ID("", systemID));
    }

    /**Constructor.*/
    public StrcInfoGUI(SavGame game, Stbof res, Trek exe)
    {
  		def = UE.SETTINGS.getDefaultFont();
  		tableDef = UE.SETTINGS.getDefaultFont();

        if (game == null || res == null || exe == null)
        {
            return;
        }

        GAME = game; STBOF = res; EXE = exe;

        try
        {
            RACE            = (Race) res.getInternalFile(Stbof.fileRACE_RST, true);
            LEX             = (Lexicon) res.getInternalFile(Stbof.fileLEXICON_DIC, true);
            EDIFICE         = (Edifice) res.getInternalFile(Stbof.fileEDIFICE_BST, true);
            AIBLDREQ        = (AIBldReq) res.getInternalFile(Stbof.fileAIBLDREQ_BIN, true);
            ENVIRON         = (Environ) res.getInternalFile(Stbof.fileENVIRON_EST, true);
            PLANBONI        = (Planboni) res.getInternalFile(Stbof.filePLANBONI_BIN, true);

            PLANBONITYPES   = (PlanetBonusTypes)exe.getInternalFile(Trek.filePLANET_BONUS_TYPES, true);
            STRUCT          = (StrcInfo) game.getInternalFile(SavGame.fileSTRCINFO, true);
            SYSTEMS         = (SystInfo) game.getInternalFile(SavGame.fileSYSTINFO, true);
            ALIENINFO       = (AlienInfo) game.getInternalFile(SavGame.fileALIENINFO, true);
            TECHINFO       = (TechInfo) game.getInternalFile(SavGame.fileTECHINFO, true);
        }
        catch (Exception er)
        {
            UE.displayError(er);
            return;
        }

        Font defSys = UE.SETTINGS.getDefaultBigFont();

        spiSystems = new CyclingHorizontalSpinner();

        spiSystems.addChangeListener(new ChangeListener()
        {
            short starID = -21;

            @Override
            public void stateChanged(ChangeEvent e)
            {
                if (starID == (short)spiSystems.getValue().hashCode())
                {
                    return;
                }

                pnlTable.removeAll();
                tableLines.clear();

                starID = (short)spiSystems.getValue().hashCode();
                
                try
                {
                    // general info
                    lblDilithium.setText(SYSTEMS.hasDilithium(starID) ? "yes" : "no");

                    int i = 0;
                    String str = "";

                    for (String en : ENVIRON.toArray())
                    {
                        for (int p = 0; p < SYSTEMS.planets(starID); p++)
                        {
                            if (SYSTEMS.getType(starID, p) == i)
                            {
                                if (str.length() > 0)
                                {
                                    str = str + ", " + en;
                                }
                                else
                                {
                                    str = en;
                                }
                                break;
                            }
                        }

                        i++;
                    }

                    lblPlanetTypes.setText(str);

                    i = SYSTEMS.getPopulation(starID);
                    
                    if (i == 0)
                    {
                        lblPopulation.setText("none");
                    }
                    else
                    {
                        int r = SYSTEMS.getResident(starID);
                        int c = SYSTEMS.getControling(starID);

                        if (r == c || c < 0 || c >= RACE.getNumberOfEntries())
                        {
                            lblPopulation.setText(Integer.toString(i) + " million ("
                                   + RACE.getName(r) + ")");
                        }
                        else
                        {
                            lblPopulation.setText(Integer.toString(i) + " million ("
                                   + RACE.getName(r) + ", " + RACE.getName(c) + ")");
                        }
                    }

                    updateEnergyUsageStrings(starID);
                }
                catch (InvalidArgumentsException ex)
                {
                    lblDilithium.setText("N/A");
                    lblPlanetTypes.setText("N/A");
                    lblPopulation.setText("N/A");
                }

                // buildings
                short[] bids = STRUCT.getBuildings(starID);
                StrcLine line;

                for (short bid : bids)
                {
                    try
                    {
                        line = new StrcLine(starID, bid);

                        tableLines.add(line);
                        pnlTable.add(line);
                    }
                    catch (Exception ex)
                    {
                        UE.displayError(ex);
                    }
                }
                
                try {
                    // add extra line
                    line = new StrcLine(starID, (short) -1);
                    tableLines.add(line);
                    pnlTable.add(line);
                } catch (Exception ex) {
                    UE.displayError(ex);
                }

                cmbSystems.setEnabled(false);
                cmbSystems.setSelectedItem(new ID("", starID));
                cmbSystems.setEnabled(true);

                pnlTable.setVisible(false);
                pnlTable.setVisible(true);
                header.updateSizes();
            }
        });

        spiSystems.setFont(defSys);

        pnlTable = new JPanel();
        pnlTable.setLayout(new BoxLayout(pnlTable, BoxLayout.Y_AXIS));

        lblDilithium = new JLabel();
        lblDilithium.setFont(def);
        lblPlanetTypes = new JLabel();
        lblPlanetTypes.setFont(def);
        lblPopulation = new JLabel();
        lblPopulation.setFont(def);
        lblEnOutput = new JLabel();
        lblEnOutput.setFont(def);
        lblEnConsumed = new JLabel();
        lblEnConsumed.setFont(def);

        btnSystemView = new JButton("View system info");
        btnSystemView.setFont(def);
        btnSystemView.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                ID item = spiSystems.getValue();
                UE.WINDOW.showPanel(new SystemsInfoGUI(GAME, STBOF, EXE, item.hashCode()));
                UE.WINDOW.setTitle(UE.APP_NAME + " - " + SavGame.cmdSYSTEM_INFO);
            }
        });

        cmbSystems = new JComboBox();
        cmbSystems.setFont(defSys);
        cmbSystems.addItemListener(new ItemListener()
        {
            @Override
            public void itemStateChanged(ItemEvent e)
            {
                if (e.getStateChange() == ItemEvent.SELECTED
                        && cmbSystems.isEnabled())
                {
                    ID selected = (ID)cmbSystems.getSelectedItem();
                    spiSystems.setValue(selected);
                }
            }
        });

        cmbFilterSystems = new JComboBox();
        cmbFilterSystems.setFont(defSys);
        cmbFilterSystems.addItemListener(new ItemListener()
        {
            @Override
            public void itemStateChanged(ItemEvent e)
            {
                if (e.getStateChange() == ItemEvent.SELECTED
                        && cmbFilterSystems.isEnabled())
                {
                    try
                    {
                        finalWarning();
                    }
                    catch (Exception ex)
                    {
                        // really bad stuff happening - bail
                        UE.displayError(ex);
                        return;
                    }
                    
                    int filter = cmbFilterSystems.getSelectedIndex() - 1;

                    ID[] bids = SYSTEMS.getSystemIDs(filter, true);

                    if (bids.length == 0)
                    {
                        String str = "N/A";
                        cmbSystems.setEnabled(false);
                        spiSystems.setEnabled(false);
                        spiSystems.setModel(new SpinnerListModel(new ID[]{new ID(str, 0)}));

                        pnlTable.removeAll();
                        tableLines.clear();

                        // general info
                        lblDilithium.setText(str);
                        lblPlanetTypes.setText(str);
                        lblPopulation.setText(str);
                        lblEnOutput.setText(str);
                        lblEnConsumed.setText(str);
                        return;
                    }
                    

                    int controling;

                    Arrays.sort(bids, new Comparator<ID>(){
                        @Override
                        public int compare(ID o1, ID o2)
                        {
                            return o1.toString().compareToIgnoreCase(o2.toString());
                        }
                    });

                    cmbSystems.setEnabled(false);
                    cmbSystems.removeAllItems();
                    
                    for (ID bid : bids)
                    {
                        try
                        {
                            controling = SYSTEMS.getControling(bid.hashCode());

                            if (controling < 0 || controling >= RACE.getNumberOfEntries()) {
                                controling = SYSTEMS.getResident(bid.hashCode());
                            }

                            if (controling >= 0 && controling < RACE.getNumberOfEntries())
                            {
                                bid.setName("%1 (%2)".replace("%1", bid.toString())
                                        .replace("%2", RACE.getName(controling)));
                            }

                            cmbSystems.addItem(bid);
                        }
                        catch (InvalidArgumentsException ex)
                        {
                            UE.displayError(ex);
                        }
                    }
                    cmbSystems.setEnabled(true);

                    spiSystems.setEnabled(false);
                    bids = SYSTEMS.getSystemIDs(filter, true);
                    spiSystems.setModel(new SpinnerListModel(bids));
                    spiSystems.setEnabled(true);

                    cmbSystems.setSelectedIndex(0);
                }
            }
        });

        //lay out
        JPanel top = new JPanel(new BorderLayout());

        JPanel pnl = new JPanel(new BorderLayout());
        pnl.add(spiSystems, BorderLayout.CENTER);
        pnl.add(cmbSystems, BorderLayout.EAST);
        pnl.add(cmbFilterSystems, BorderLayout.WEST);

        top.add(pnl, BorderLayout.NORTH);

        pnl = new JPanel();
        pnl.setLayout(new BoxLayout(pnl, BoxLayout.Y_AXIS));

        addLabel(" ", def, pnl);
        addLabel("Dilithium present:", def, pnl);
        addLabel("Planet types:", def, pnl);
        addLabel(" ", def, pnl);
        addLabel("Population:", def, pnl);
        addLabel("Energy output:", def, pnl);
        addLabel("Energy consumed:", def, pnl);

        top.add(pnl, BorderLayout.WEST);

        pnl = new JPanel(new GridLayout(0, 1, 0, 0));

        addLabel(" ", def, pnl);
        lblDilithium.setAlignmentX(JLabel.LEFT_ALIGNMENT);
        pnl.add(lblDilithium);
        lblPlanetTypes.setAlignmentX(JLabel.LEFT_ALIGNMENT);
        pnl.add(lblPlanetTypes);
        addLabel(" ", def, pnl);
        lblPopulation.setAlignmentX(JLabel.LEFT_ALIGNMENT);
        pnl.add(lblPopulation);
        lblEnOutput.setAlignmentX(JLabel.LEFT_ALIGNMENT);
        pnl.add(lblEnOutput);
        lblEnConsumed.setAlignmentX(JLabel.LEFT_ALIGNMENT);
        pnl.add(lblEnConsumed);

        pnl.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 5));
        top.add(pnl, BorderLayout.CENTER);


        pnl = new JPanel();
        pnl.setLayout(new BoxLayout(pnl, BoxLayout.Y_AXIS));
        addLabel(" ", def, pnl);
        btnSystemView.setAlignmentX(JButton.RIGHT_ALIGNMENT);
        pnl.add(btnSystemView);
        pnl.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 0));

        top.add(pnl, BorderLayout.EAST);

        setLayout(new BorderLayout());

        top.setBorder(BorderFactory.createEmptyBorder(10, 10, 5, 10));
        
        add(top, BorderLayout.NORTH);

        pnl = new JPanel(new BorderLayout());
        pnl.add(pnlTable, BorderLayout.NORTH);

        scrTable = new JScrollPane(pnl);
        scrTable.setPreferredSize(new Dimension(300, 200));
        scrTable.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        header = new StrcHeaderLine();
        scrHeader = new JScrollPane(header);
        scrHeader.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrHeader.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
        scrHeader.getVerticalScrollBar().setUI(new InvisibleScrollBarUI());
        scrHeader.setBorder(BorderFactory.createMatteBorder(0, 1, 0, 0, Color.GRAY));

        scrTable.getHorizontalScrollBar().addAdjustmentListener(new AdjustmentListener()
        {
            @Override
            public void adjustmentValueChanged(AdjustmentEvent e)
            {
                Rectangle rect = scrHeader.getViewport().getViewRect();

                rect.x = scrTable.getViewport().getViewRect().x - rect.x;

                scrHeader.getViewport().scrollRectToVisible(rect);
            }
        });

        pnl = new JPanel(new BorderLayout());
        pnl.add(scrHeader, BorderLayout.NORTH);
        pnl.add(scrTable, BorderLayout.CENTER);
        pnl.setBorder(BorderFactory.createEmptyBorder(5, 10, 10, 10));
        add(pnl, BorderLayout.CENTER);

        // init
        cmbFilterSystems.addItem(new ID("Any", -1));

        for (int i = 0; i < 5; i++)
        {
            try
            {
                cmbFilterSystems.addItem(new ID(RACE.getName(i), i));
            }
            catch (InvalidArgumentsException ex)
            {
                cmbFilterSystems.addItem(new ID("Empire %1".replace("%1", Integer.toString(i)), i));
            }
        }
        
        cmbFilterSystems.addItem(new ID("Neutral", 5));

        // fix combo box
        Dimension pref = spiSystems.getPreferredSize();
        int width = cmbSystems.getPreferredSize().width;
        pref.width = 20;
        cmbSystems.setPreferredSize(pref);
        cmbSystems.setUI(new UEComboBoxUI(width));
    }

    private void updateEnergyUsageStrings(short starID) throws InvalidArgumentsException
    {
        // get energy bonus
        int en_boni = 0;
        int boni;
        int btype;

        for (int p = 0; p < SYSTEMS.planets(starID); p++)
        {
            btype = PLANBONITYPES.getBonusType(SYSTEMS.getType(starID, p));

            if (btype == PlanetBonusTypes.energy)
            {
                boni = PLANBONI.get(
                            SYSTEMS.getType(starID, p),
                            (byte)SYSTEMS.getSize(starID, p),
                            SYSTEMS.getPltBoni(starID, p),
                            SYSTEMS.getUniBoni(starID, p));

                if (boni != 0)
                {
                    boni = boni * 5;
                    en_boni += boni;
                }
            }
        }

        // get energy output
        short[] bids = STRUCT.getBuildings(starID);
        int count = SYSTEMS.getPop2Ene(starID) / 10;
        int output = 0;
        int total = 0;
        int en_cost = 0;

        for (short bid : bids)
        {
            try
            {
                btype = EDIFICE.getBoniType(bid);

                if (EDIFICE.isMainBuilding(bid))
                {
                    if (btype == 2 || btype == 38)
                    {
                        if (count > 0)
                        {
                            output = Math.min(count, STRUCT.getCount(starID, bid));
                            count -= STRUCT.getCount(starID, bid);
                            output = output
                                    * EDIFICE.getBoni(bid, Math.min(SYSTEMS.getControling(starID), 5));
                            total += output;
                        }
                    }
                }
                else
                {
                    output = STRUCT.getCount(starID, bid);

                    if (EDIFICE.getEnCost(bid) > 0)
                    {
                        output = Math.min(output, STRUCT.getPoweredOnCount(starID, bid));

                        en_cost += output * EDIFICE.getEnCost(bid);
                    }

                    if (btype == 2 || btype == 38)
                    {
                        output = output * EDIFICE.getBoni(bid, Math.min(SYSTEMS.getControling(starID), 5));
                        total += output;
                    }
                    else if (btype == 3)
                    {
                        output = output * EDIFICE.getBoni(bid, Math.min(SYSTEMS.getControling(starID), 5));
                        en_boni += output;
                    }
                }
            }
            catch (Exception ex)
            {
                UE.displayError(ex);
            }
        }

        String str = Integer.toString((int)Math.floor(total * (100 + en_boni) / 100.0));

        if (en_boni > 0)
        {
            str = str + " ( " + total + " +" + en_boni + "% )";
        }

        lblEnOutput.setText(str);
        lblEnConsumed.setText(Integer.toString(en_cost));
    }

    //final warning
    @Override
    public void finalWarning() throws Exception
    {
        if (spiSystems.isEnabled())
        {
            spiSystems.commitEdit();

            // rename systems
            List<?> systems = spiSystems.getModel().getList();
            ID sysID;
            Object o;

            for (int i = 0; i < systems.size(); i++)
            {
                o = systems.get(i);

                if (o instanceof ID)
                {
                    sysID = (ID)o;
                    SYSTEMS.setName(sysID.hashCode(), sysID.toString());
                }
            }
        }
    }

    private void addLabel(String str, Font def, JPanel pnl)
    {
        JLabel lbl = new JLabel(str);
        lbl.setFont(def);
        lbl.setAlignmentX(JLabel.LEFT_ALIGNMENT);
        pnl.add(lbl);
    }

    private class StrcHeaderLine extends JPanel implements ActionListener
    {
        private ArrayList<JButton> buttons;

        public StrcHeaderLine()
        {
            setLayout(new GridLayout(1, 0, 0, 0));

            buttons = new ArrayList<JButton>();
            
            JPanel pnl = new JPanel(new GridLayout(1, 0, 0, 0));
            pnl.add(setupButton("Building", def));
            add(pnl);

            pnl = new JPanel(new GridLayout(1, 0, 0, 0));
            pnl.add(setupButton("Count", def));
            pnl.add(setupButton("Powered", def));
            pnl.add(setupButton("Energy req.", def));
            pnl.add(setupButton("AI energy req.", def));
            pnl.add(setupButton("Level", def));
            pnl.add(setupButton("Can be built", def));
            add(pnl);

            pnl = new JPanel(new GridLayout(1, 0, 0, 0));
            pnl.add(setupButton("System req.", def));
            pnl.add(setupButton("Political req.", def));
            pnl.add(setupButton("Build limit", def));
            add(pnl);
        }

        private JButton setupButton(String str, Font def)
        {
            JButton lbl = new JButton(str);
            lbl.setFont(tableDef);
            lbl.setHorizontalAlignment(JLabel.CENTER);
            lbl.setBorder(BorderFactory.createLineBorder(Color.GRAY));
            lbl.addActionListener(this);
            buttons.add(lbl);
            return lbl;
        }

        @Override
        @SuppressWarnings("unchecked")
        public void actionPerformed(ActionEvent e)
        {
            Object c = e.getSource();

            if (tableLines.size() < 3)
            {
                return;
            }

            Comparable o1, o2;


            for (int i = 0; i < buttons.size(); i++)
            {
                if (c == buttons.get(i))
                {
                    final int index = i;

                    o1 = tableLines.get(0).getColumnValue(i);
                    o2 =  tableLines.get(tableLines.size() - 2).getColumnValue(i);

                    // sort
                    if (o1.compareTo(o2) >= 0)
                    {
                        Collections.sort(tableLines, new Comparator<StrcLine>()
                        {
                            @Override
                            public int compare(StrcLine o1, StrcLine o2)
                            {
                                if (o1.getBuildingID() < 0)
                                {
                                    return 1000;
                                }
                                else if (o2.getBuildingID() < 0)
                                {
                                    return -1000;
                                }
                                else
                                {
                                    return o1.getColumnValue(index).compareTo(o2.getColumnValue(index));
                                }
                            }
                        });
                    }
                    else
                    {
                        Collections.sort(tableLines, new Comparator<StrcLine>()
                        {
                            @Override
                            public int compare(StrcLine o1, StrcLine o2)
                            {
                                if (o1.getBuildingID() < 0)
                                {
                                    return 1000;
                                }
                                else if (o2.getBuildingID() < 0)
                                {
                                    return -1000;
                                }
                                else
                                {
                                    return o2.getColumnValue(index).compareTo(o1.getColumnValue(index));
                                }
                            }
                        });
                    }

                    pnlTable.removeAll();

                    for (int k = 0; k < tableLines.size(); k++)
                    {
                        pnlTable.add(tableLines.get(k));
                    }

                    pnlTable.setVisible(false);
                    pnlTable.setVisible(true);

                    break;
                }
            }
        }

        public void updateSizes()
        {
            StrcLine line = tableLines.get(0);

            Dimension pref = line.getPreferredSize();
            
            setMinimumSize(pref);
            setPreferredSize(pref);
            setMaximumSize(pref);
            setSize(pref);


            Dimension hpref = scrTable.getPreferredSize();
            hpref.height = pref.height;

            scrHeader.setMinimumSize(hpref);
            scrHeader.setPreferredSize(hpref);
            scrHeader.setMaximumSize(hpref);
            scrHeader.setSize(hpref);
/*
            pnlHeader.setMinimumSize(hpref);
            pnlHeader.setPreferredSize(hpref);
            pnlHeader.setMaximumSize(hpref);
            pnlHeader.setSize(hpref);
*/
            setVisible(false);
            setVisible(true);
        }
    }

    private class StrcLine extends JPanel
    {
        public JComboBox cmbBuildings;
        public JTextField txtLevel;
        public JTextField txtEnReq;
        public JTextField txtAIEnReq;
        public JTextField txtCanBeBuilt;
        public JTextField txtSysReq;
        public JTextField txtPolReq;
        public JTextField txtLimit;
        public JSpinner spiCount;
        public JSpinner spiPowered;

        private short starID, buildingID;

        public StrcLine(short starID, short buildingID) throws Exception
        {
            this.starID = starID;
            this.buildingID = buildingID;

            // components
            cmbBuildings = new JComboBox();
            cmbBuildings.removeAllItems();

            for (ID bid : EDIFICE.getShortIDs((byte)0, 40))
            {
                bid.setName(bid.hashCode() + ": " + bid.toString());
                cmbBuildings.addItem(bid);
            }

            cmbBuildings.addItem(new ID("None", -1));
            
            cmbBuildings.setSelectedItem(new ID("", buildingID));
            cmbBuildings.setFont(tableDef);

            if (buildingID != (cmbBuildings.getSelectedItem()).hashCode())
            {
                throw new Exception("Unknown building ID encountered.");
            }

            cmbBuildings.addItemListener(new ItemListener()
            {
                @Override
                public void itemStateChanged(ItemEvent e)
                {
                    if (e.getStateChange() == ItemEvent.DESELECTED)
                    {
                        if (e.getItem().hashCode() < 0)
                        {
                            try {
                                // add another empty line
                                StrcLine line = new StrcLine(getStarID(), (short) -1);
                                tableLines.add(line);
                                pnlTable.add(line);

                                pnlTable.setVisible(false);
                                pnlTable.setVisible(true);
                            }
                            catch (Exception ex)
                            {
                                UE.displayError(ex);
                            }
                        }
                    }
                    else if (e.getStateChange() == ItemEvent.SELECTED)
                    {
                        short bid = (short)e.getItem().hashCode();

                        if (bid >= 0)
                        {
                            if (getBuildingID() >= 0)
                            {
                                STRUCT.changeBuildingID(getStarID(), getBuildingID(), bid);
                            }

                            // remove others
                            StrcLine line;

                            for (int i = 0; i < tableLines.size(); i++)
                            {
                                line = tableLines.get(i);

                                if (line.getBuildingID() == bid)
                                {
                                    tableLines.remove(i);
                                    pnlTable.remove(line);
                                    i--;
                                }
                            }

                            pnlTable.setVisible(false);
                            pnlTable.setVisible(true);

                            setBuildingID(bid);

                            try
                            {
                                updateFields();
                                updateEnergyUsageStrings(getStarID());
                            } catch (InvalidArgumentsException ex) {
                                UE.displayError(ex);
                            }
                        }
                        else
                        {
                            bid = getBuildingID();
                            STRUCT.setCount(getStarID(), bid, (short)0);

                            StrcLine line;

                            for (int i = 0; i < tableLines.size(); i++)
                            {
                                line = tableLines.get(i);

                                if (line.getBuildingID() == bid)
                                {
                                    tableLines.remove(i);
                                    pnlTable.remove(line);
                                    i--;
                                }
                            }

                            pnlTable.setVisible(false);
                            pnlTable.setVisible(true);
                        }
                    }
                }
            });

            txtLevel = new JTextField();
            txtLevel.setEditable(false);
            txtLevel.setHorizontalAlignment(JTextField.RIGHT);
            txtLevel.setFont(def);

            txtEnReq = new JTextField();
            txtEnReq.setEditable(false);
            txtEnReq.setHorizontalAlignment(JTextField.RIGHT);
            txtEnReq.setFont(def);

            txtAIEnReq = new JTextField();
            txtAIEnReq.setEditable(false);
            txtAIEnReq.setHorizontalAlignment(JTextField.RIGHT);
            txtAIEnReq.setFont(def);

            spiCount = new JSpinner(new SpinnerNumberModel(0, 0, Short.MAX_VALUE, 1));
            spiCount.setFont(tableDef);
            spiCount.addChangeListener(new ChangeListener()
            {
                @Override
                public void stateChanged(ChangeEvent e)
                {
                    if (!spiCount.isEnabled())
                    {
                        return;
                    }
                    short count = ((Integer)spiCount.getValue()).shortValue();
                    STRUCT.setCount(getStarID(), getBuildingID(), count);

                    try
                    {
                        updateFields();
                        updateEnergyUsageStrings(getStarID());
                    } catch (InvalidArgumentsException ex) {
                        UE.displayError(ex);
                    }
                }
            });

            spiPowered = new JSpinner(new SpinnerNumberModel(0, 0, Short.MAX_VALUE, 1));
            spiPowered.setFont(tableDef);
            spiPowered.addChangeListener(new ChangeListener()
            {
                @Override
                public void stateChanged(ChangeEvent e)
                {
                    if (!spiPowered.isEnabled())
                    {
                        return;
                    }

                    short count = ((Integer)spiPowered.getValue()).shortValue();
                    STRUCT.setPoweredOnCount(getStarID(), getBuildingID(), count);

                    try
                    {
                        updateFields();
                        updateEnergyUsageStrings(getStarID());
                    } catch (InvalidArgumentsException ex) {
                        UE.displayError(ex);
                    }
                }
            });

            txtSysReq = new JTextField();
            txtSysReq.setEditable(false);
            txtSysReq.setHorizontalAlignment(JTextField.CENTER);
            txtSysReq.setFont(tableDef);

            txtPolReq = new JTextField();
            txtPolReq.setEditable(false);
            txtPolReq.setHorizontalAlignment(JTextField.CENTER);
            txtPolReq.setFont(tableDef);

            txtLimit = new JTextField();
            txtLimit.setEditable(false);
            txtLimit.setHorizontalAlignment(JTextField.CENTER);
            txtLimit.setFont(tableDef);

            txtCanBeBuilt = new JTextField();
            txtCanBeBuilt.setEditable(false);
            txtCanBeBuilt.setHorizontalAlignment(JTextField.CENTER);
            txtCanBeBuilt.setFont(tableDef);

            updateFields();
            
            setLayout(new GridLayout(1, 0, 0, 0));

            JPanel pnl = new JPanel(new GridLayout(1, 0, 0, 0));
            pnl.add(cmbBuildings);
            add(pnl);

            pnl = new JPanel(new GridLayout(1, 0, 0, 0));
            pnl.add(spiCount);
            pnl.add(spiPowered);
            pnl.add(txtEnReq);
            pnl.add(txtAIEnReq);
            pnl.add(txtLevel);
            pnl.add(txtCanBeBuilt);
            add(pnl);

            pnl = new JPanel(new GridLayout(1, 0, 0, 0));
            pnl.add(txtSysReq);
            pnl.add(txtPolReq);
            pnl.add(txtLimit);
            add(pnl);
        }

        public short getStarID()
        {
            return starID;
        }

        public short getBuildingID()
        {
            return buildingID;
        }

        public void setBuildingID(short id)
        {
            buildingID = id;
        }

        public void updateFields() throws InvalidArgumentsException
        {
            if (buildingID >= 0)
            {
                txtLevel.setText(Integer.toString(EDIFICE.getTechLvl(buildingID)));
                int encost = EDIFICE.getEnCost(buildingID);
                txtEnReq.setText(Integer.toString(encost));
                try
                {
                    encost = AIBLDREQ.getEnergyReq(buildingID);
                }
                catch (Exception f){}

                txtAIEnReq.setText(Integer.toString(encost));

                int count = STRUCT.getCount(starID, buildingID);

                spiCount.setEnabled(false);
                spiCount.setValue(count);

                spiPowered.setEnabled(false);
                spiPowered.setModel(new SpinnerNumberModel(
                        Math.min(STRUCT.getPoweredOnCount(starID, buildingID), count),
                        0, count, 1));

                int[] req = new int[]{38, 56, 230, 410, 559, 879, 1259, 1415, 872,
                                        47, 48, 239, 1447, 968, 1450, 553};

                count = EDIFICE.getSysReq(buildingID);
                if (count < 0 || count >= req.length)
                {
                    count = req.length - 1;
                }
                if (req[count] != 872)
                {
                    txtSysReq.setText(LEX.getEntry(req[count]));
                }
                else
                {
                    txtSysReq.setText("-");
                }

                req = new int[]{437, 801, 863, 1187, 6, 527, 184, 553, 553,
                                        872, 977, 277, 553};

                count = EDIFICE.getPoliticReq(buildingID);
                if (count < 0 || count >= req.length)
                {
                    count = req.length - 1;
                }
                if (req[count] != 872)
                {
                    txtPolReq.setText(LEX.getEntry(req[count]));
                }
                else
                {
                    txtPolReq.setText("-");
                }

                req = new int[]{872, 885, 884, 553};

                count = EDIFICE.getFreq(buildingID);
                if (count < 0 || count >= req.length)
                {
                    count = req.length - 1;
                }
                if (req[count] != 872)
                {
                    txtLimit.setText(LEX.getEntry(req[count]));
                }
                else
                {
                    txtLimit.setText("-");
                }

                int controling = SYSTEMS.getControling(starID);
                int resident = SYSTEMS.getResident(starID);

                if (controling < 0 || controling > 4)
                {
                    controling = resident;
                }

                // check race req
                boolean canBeBuilt = EDIFICE.canBeBuiltBy(buildingID, controling, resident);

                // check sys req
                count = EDIFICE.getSysReq(buildingID);

                if (count <= 7)
                {
                    for (int p = 0; p < SYSTEMS.planets(starID); p++)
                    {
                        if (SYSTEMS.getType(starID, p) == count)
                        {
                            count = 8;
                            break;
                        }
                    }

                    canBeBuilt = canBeBuilt && (count == 8);
                }
                else if (count == 11)
                {
                    canBeBuilt = canBeBuilt && SYSTEMS.hasDilithium(starID);
                    count = (SYSTEMS.hasDilithium(starID)) ? 8 : 0;
                }

                if (count == 8)
                {
                    txtSysReq.setBackground(cmbBuildings.getBackground());
                }
                else
                {
                    txtSysReq.setBackground(Color.PINK);
                }

                // check pol req
                int ireq = EDIFICE.getPoliticReq(buildingID);
                boolean btmp = true;

                if (ireq == Edifice.POLITIC_REQ_CONQUERED_HOME)
                {
                    btmp = resident == ALIENINFO.getRace(starID) && controling != resident;
                }
                else if (ireq == Edifice.POLITIC_REQ_HOME_SYSTEM)
                {
                    btmp = resident == ALIENINFO.getRace(starID) && controling == resident;
                }
                else if (ireq == Edifice.POLITIC_REQ_NATIVE_MEMBER)
                {
                    btmp = controling == resident;
                }
                else if (ireq == Edifice.POLITIC_REQ_NON_NATIVE_MEMBER)
                {
                    btmp = ALIENINFO.getRaceStatus(resident) == AlienInfo.STATUS_MEMBER
                            && controling != resident;
                }
                else if (ireq == Edifice.POLITIC_REQ_SUBJUGATED)
                {
                    btmp = ALIENINFO.getRaceStatus(resident) == AlienInfo.STATUS_SUBJUGATED;
                }

                if (btmp)
                {
                    txtPolReq.setBackground(cmbBuildings.getBackground());
                }
                else
                {
                    txtPolReq.setBackground(Color.PINK);
                }

                canBeBuilt = canBeBuilt && btmp;

                // check num req
                ireq = EDIFICE.getFreq(buildingID);
                btmp = true;

                if (ireq == Edifice.FREQ_LIMIT_ONE_PER_SYSTEM)
                {
                    btmp = STRUCT.getCount(starID, buildingID) <= 1;
                }
                else if (ireq == Edifice.FREQ_LIMIT_ONE_PER_EMPIRE)
                {
                    ArrayList<Short> stars = STRUCT.getStarsWithBuilding(buildingID);
                    int num = 0;

                    for (int i = 0; i < stars.size(); i++)
                    {
                        if (SYSTEMS.getControling(stars.get(i)) == controling)
                        {
                            num++;

                            if (num > 1)
                            {
                                btmp = false;
                                break;
                            }
                        }
                    }
                }

                if (btmp)
                {
                    txtLimit.setBackground(cmbBuildings.getBackground());
                }
                else
                {
                    txtLimit.setBackground(Color.PINK);
                }

                canBeBuilt = canBeBuilt && btmp;

                // check level
                ireq = EDIFICE.getTechReq(buildingID);

                btmp = EDIFICE.getTechLvl(buildingID) <= TECHINFO.getTechLevel((short)controling, ireq);

                if (btmp)
                {
                    txtLevel.setBackground(cmbBuildings.getBackground());
                }
                else
                {
                    txtLevel.setBackground(Color.PINK);
                }

                canBeBuilt = canBeBuilt && btmp;

                if (canBeBuilt)
                {
                    txtCanBeBuilt.setText("yes");
                    txtCanBeBuilt.setBackground(Color.CYAN);
                }
                else
                {
                    txtCanBeBuilt.setText("no");
                    txtCanBeBuilt.setBackground(Color.PINK);
                }

                txtLevel.setEnabled(true);
                txtEnReq.setEnabled(true);
                txtAIEnReq.setEnabled(true);
                spiCount.setEnabled(true);
                txtSysReq.setEnabled(true);
                txtPolReq.setEnabled(true);
                txtLimit.setEnabled(true);
                txtCanBeBuilt.setEnabled(true);
                spiPowered.setEnabled(EDIFICE.getEnCost(buildingID) > 0
                        && !EDIFICE.isMainBuilding(buildingID));
            }
            else
            {
                txtLevel.setText("N/A");
                txtLevel.setEnabled(false);
                txtEnReq.setText("N/A");
                txtEnReq.setEnabled(false);
                txtAIEnReq.setText("N/A");
                txtAIEnReq.setEnabled(false);
                spiCount.setValue(0);
                spiCount.setEnabled(false);
                spiPowered = new JSpinner(new SpinnerNumberModel(0, 0, Short.MAX_VALUE, 1));
                spiPowered.setEnabled(false);
                txtSysReq.setText("N/A");
                txtSysReq.setEnabled(false);
                txtPolReq.setText("N/A");
                txtPolReq.setEnabled(false);
                txtLimit.setText("N/A");
                txtLimit.setEnabled(false);
                txtCanBeBuilt.setText("N/A");
                txtCanBeBuilt.setEnabled(false);
            }

            if (SYSTEMS.getPopulation(starID) <= 0)
            {
                cmbBuildings.setEnabled(false);
            }
            else
            {
                cmbBuildings.setEnabled(true);
            }
        }

        public Comparable getColumnValue(int index)
        {
            switch (index)
            {
                case 0:
                {
                    return cmbBuildings.getSelectedItem().hashCode();
                }
                case 1:
                {
                    return (Integer)spiCount.getValue();
                }
                case 2:
                {
                    return (Integer)spiPowered.getValue();
                }
                case 3:
                {
                    return Integer.parseInt(txtEnReq.getText());
                }
                case 4:
                {
                    return Integer.parseInt(txtAIEnReq.getText());
                }
                case 5:
                {
                    return Integer.parseInt(txtLevel.getText());
                }
                case 6:
                {
                    return txtCanBeBuilt.getText();
                }
                case 7:
                {
                    return txtSysReq.getText();
                }
                case 8:
                {
                    return txtPolReq.getText();
                }
                default:
                {
                    return txtLimit.getText();
                }
            }
        }
    }
}
