/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: SystemsInfoGUI.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui.sav;

import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Enumeration;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.LineBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import ue.Language;
import ue.UE;
import ue.edit.exe.Trek;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.bin.Planboni;
import ue.edit.res.stbof.bin.Planspac;
import ue.edit.res.stbof.est.Environ;
import ue.edit.res.stbof.rst.Race;
import ue.edit.sav.SavGame;
import ue.edit.sav.SystInfo;
import ue.exception.InvalidArgumentsException;
import ue.gui.AniLabel;
import ue.gui.MainPanel;
import ue.util.ID;

/**Used to edit systems info in a saved game.*/
@SuppressWarnings("serial")  //$NON-NLS-1$
public class SystemsInfoGUI extends MainPanel{
	private SystInfo SYSTEMS;
	private Planspac SPACE;
	private Planboni BONI;
	private Environ ENVIRON;
	private Race RACES;
	private Stbof STBOF;
	private SavGame GAME;
	private Trek EXE;
	private int SELECTED = -1;
	// static strings
	private static String strEDIT = Language.getString("SystemsInfoGUI.0"); //$NON-NLS-1$
	private static String strENTER_NEW_NAME = Language.getString("SystemsInfoGUI.1"); //$NON-NLS-1$
	
	//items-general
	JComboBox jcbSYSTEM = new JComboBox();
	JButton btnEDIT = new JButton(strEDIT);
	JButton btnSTRUCT;
	JButton btnLEFT = new JButton("<<"); //$NON-NLS-1$
	JButton btnRIGHT = new JButton(">>"); //$NON-NLS-1$
	CardLayout CARDS = new CardLayout();
	JPanel panCARDS = new JPanel();
	//items specific
	SystemPanel INFO;
	SystemPlanet PLANETS;

    public SystemsInfoGUI(SavGame game, Stbof st, Trek exe, int systemID)
    {
        this(game, st, exe);
        jcbSYSTEM.setSelectedItem(new ID("", systemID));
    }

    /**Constructor.*/
	public SystemsInfoGUI(SavGame game, Stbof st, Trek exe)
    {
        if (game == null || st == null || exe == null)
        {
            return;
        }

        GAME = game; STBOF = st; EXE = exe;

		try{
            SYSTEMS = (SystInfo) GAME.getInternalFile(SavGame.fileSYSTINFO, true);

			SPACE=(Planspac)STBOF.getInternalFile(Stbof.filePLANSPAC_BIN, true);
			BONI=(Planboni)STBOF.getInternalFile(Stbof.filePLANBONI_BIN, true);
			ENVIRON=(Environ)STBOF.getInternalFile(Stbof.fileENVIRON_EST, true);
			RACES=(Race)STBOF.getInternalFile(Stbof.fileRACE_RST, true);
		}catch (Exception er){
			UE.displayError(er);
			return;
		}
		//build
		INFO = new SystemPanel();
		PLANETS = new SystemPlanet();
		//set up
		panCARDS.setLayout(CARDS);
		panCARDS.add(INFO, "0"); //$NON-NLS-1$
		panCARDS.add(PLANETS, "1"); //$NON-NLS-1$
		Font def = UE.SETTINGS.getDefaultFont();
		jcbSYSTEM.setFont(def);
		btnEDIT.setFont(def);
		btnLEFT.setFont(def);
		btnRIGHT.setFont(def);
		Dimension dimo = new Dimension(30, 20);
		btnLEFT.setSize(dimo);
		btnRIGHT.setSize(dimo);
		//edit
		btnEDIT.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent ae){
				if (SELECTED!=-1){
					try{
						String n = JOptionPane.showInputDialog(strENTER_NEW_NAME,
								SYSTEMS.getName(SELECTED));
						
                        if (n != null)
                        {
							int ij = SELECTED;
							SYSTEMS.setName(SELECTED, n);
							fillSystems();

							jcbSYSTEM.setSelectedItem(new ID("", ij));
						}
					}catch (Exception ze){
						UE.displayError(ze);
					}
				}
			}
		});
		//combo
		jcbSYSTEM.addActionListener(new ActionListener()
        {
            @Override
			public void actionPerformed(ActionEvent ae)
            {
				if (jcbSYSTEM.getSelectedIndex() >= 0)
                {
                    try
                    {
                        finalWarning();
                    }
                    catch (Exception ex)
                    {
                        // really bad stuff happening - bail
                        UE.displayError(ex);
                        return;
                    }
					SELECTED = jcbSYSTEM.getSelectedItem().hashCode();
					INFO.init();
					PLANETS.init();
				}
			}
		});
		//left and right
		btnLEFT.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent ae){
				if (SELECTED!=-1){
                    try
                    {
                        finalWarning();
                    }
                    catch (Exception ex)
                    {
                        // really bad stuff happening - bail
                        UE.displayError(ex);
                        return;
                    }
					INFO.init();
					PLANETS.init();
					CARDS.previous(panCARDS);
				}
			}
		});
		btnRIGHT.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent ae){
				if (SELECTED!=-1)
                {
                    try
                    {
                        finalWarning();
                    }
                    catch (Exception ex)
                    {
                        // really bad stuff happening - bail
                        UE.displayError(ex);
                        return;
                    }
					INFO.init();
					PLANETS.init();
					CARDS.next(panCARDS);
				}
			}
		});
        btnSTRUCT = new JButton("View structure list");
        btnSTRUCT.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                UE.WINDOW.showPanel(new StrcInfoGUI(GAME, STBOF, EXE, SELECTED));
                UE.WINDOW.setTitle(UE.APP_NAME + " - " + SavGame.cmdSTRCINFO);
            }
        });


		CARDS.first(panCARDS);
		fillSystems();
		//lay out
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets=new Insets(5, 5, 5, 5);
		c.fill = GridBagConstraints.NONE;
		c.anchor = GridBagConstraints.CENTER;
		//up
		JPanel pnlSYSTEM = new JPanel(new GridBagLayout());
		c.gridx=0;
		c.gridy=0;
		c.gridwidth=1;
		pnlSYSTEM.add(jcbSYSTEM, c);
		c.gridx=1;
		pnlSYSTEM.add(btnEDIT, c);
		c.gridx=2;
		pnlSYSTEM.add(btnSTRUCT, c);

		c.gridx=0;
		c.gridy=0;
		c.gridwidth=4;
		add(pnlSYSTEM, c);
		c.gridx=0;
		c.gridy=1;
		add(panCARDS, c);
		c.gridx=0;
		c.gridy=2;
		c.gridwidth=2;
		c.anchor = GridBagConstraints.LINE_START;
		add(btnLEFT, c);
		c.gridx=2;
		c.anchor = GridBagConstraints.LINE_END;
		add(btnRIGHT, c);
	}
	//fill systems
	private void fillSystems()
    {
		ID[] items = SYSTEMS.getSystemIDs(-1, false);

        Arrays.sort(items, new Comparator<ID>()
        {
            @Override
            public int compare(ID o1, ID o2)
            {
                return o1.toString().compareToIgnoreCase(o2.toString());
            }
        });

		jcbSYSTEM.removeAllItems();

        for (ID sid : items)
        {
			jcbSYSTEM.addItem(sid);
		}

        SELECTED = jcbSYSTEM.getSelectedItem().hashCode();
	}
	//final warning
	@Override
	public void finalWarning() throws Exception{
		if (INFO!=null) {
			INFO.finalWarning();
		}
		if (PLANETS!=null) {
			PLANETS.finalWarning();
		}
	}
	
	/*This panel is for editing system info only*/
	private class SystemPanel extends MainPanel{
		//other valuable data
		int MAX_POP=0;
		//items
		JLabel[] lblLABEL = new JLabel[11];
		JLabel lblMAXPOP = new JLabel();
		JLabel lblMORALE = new JLabel();
		JLabel lblUNALL = new JLabel();
		JLabel lblBID = new JLabel();
		JComboBox cobTYPE = new JComboBox();
		JComboBox cobSTARANI = new JComboBox();
		JCheckBox chbDIL = new JCheckBox();
		JComboBox cobRESIDENT = new JComboBox();
		JSlider spiPOPUL = new JSlider();
		JSlider sliMORALE = new JSlider(0, 195);
		JSlider[] sliUNITS = new JSlider[5];
		AniLabel aniSTAR = new AniLabel();
		
		public SystemPanel(){
			//create
			lblLABEL[0] = new JLabel(Language.getString("SystemsInfoGUI.2")); //$NON-NLS-1$
			lblLABEL[1] = new JLabel(Language.getString("SystemsInfoGUI.3")); //$NON-NLS-1$
			lblLABEL[2] = new JLabel(Language.getString("SystemsInfoGUI.4")); //$NON-NLS-1$
			lblLABEL[3] = new JLabel(Language.getString("SystemsInfoGUI.5")); //$NON-NLS-1$
			lblLABEL[4] = new JLabel(Language.getString("SystemsInfoGUI.6")); //$NON-NLS-1$
			lblLABEL[5] = new JLabel(Language.getString("SystemsInfoGUI.7")); //$NON-NLS-1$
			lblLABEL[6] = new JLabel(Language.getString("SystemsInfoGUI.8")); //$NON-NLS-1$
			lblLABEL[7] = new JLabel(Language.getString("SystemsInfoGUI.9")); //$NON-NLS-1$
			lblLABEL[8] = new JLabel(Language.getString("SystemsInfoGUI.10")); //$NON-NLS-1$
			lblLABEL[9] = new JLabel(Language.getString("SystemsInfoGUI.11")); //$NON-NLS-1$
			lblLABEL[10] = new JLabel(Language.getString("SystemsInfoGUI.12")); //$NON-NLS-1$
			sliUNITS[0] = new JSlider();
			sliUNITS[1] = new JSlider();
			sliUNITS[2] = new JSlider();
			sliUNITS[3] = new JSlider();
			sliUNITS[4] = new JSlider();
			//set up
			Font def = UE.SETTINGS.getDefaultFont();
			for (int i=0; i<11; i++){
				lblLABEL[i].setFont(def);
				if (i<5){
					sliUNITS[i].setFont(def);
					sliUNITS[i].setMinorTickSpacing(10);
					sliUNITS[i].setMajorTickSpacing(10);
					sliUNITS[i].setSnapToTicks(true);
					sliUNITS[i].setPreferredSize(new Dimension(100, 16));
				}
			}
			lblUNALL.setFont(def);
			lblMAXPOP.setFont(def);
			lblMORALE.setFont(def);
			cobTYPE.setFont(def);
			cobSTARANI.setFont(def);
			chbDIL.setFont(def);
			cobRESIDENT.setFont(def);
			spiPOPUL.setFont(def);
			spiPOPUL.setPreferredSize(new Dimension(100, 16));
			sliMORALE.setFont(def);
			sliMORALE.setPreferredSize(new Dimension(100, 16));
			//type
			cobTYPE.addActionListener(new ActionListener(){
                @Override
				public void actionPerformed(ActionEvent ae){
					if (SELECTED!=-1 && cobTYPE.getSelectedIndex()>=0){
						try{
							SYSTEMS.setType(SELECTED, (byte)cobTYPE.getSelectedIndex());
						}catch (Exception ze){
							UE.displayError(ze);
						}
					}
				}
			});
			//animation
			cobSTARANI.addActionListener(new ActionListener(){
                @Override
				public void actionPerformed(ActionEvent ae){
					if (SELECTED!=-1 && cobSTARANI.getSelectedIndex()>=0){
						try{ 
							SYSTEMS.setAni(SELECTED, (String)cobSTARANI.getSelectedItem());
							aniSTAR.setAnimation(STBOF.getAnimation(SYSTEMS.getAni(SELECTED), true), true);
						}catch (Exception ze){
							UE.displayError(ze);
						}
					}
				}
			});
			//dilithium
			chbDIL.addChangeListener(new ChangeListener(){
				public void stateChanged(ChangeEvent e){
					if (SELECTED!=-1){
						try{
							SYSTEMS.setDilithium(SELECTED, chbDIL.isSelected());
							if (SYSTEMS.hasDilithium(SELECTED)){
								lblBID.setIcon(new ImageIcon((STBOF.getTargaImage("bid.tga", true)).getImage(true))); //$NON-NLS-1$
							}else{
								lblBID.setIcon(null);
							}
						}catch (InvalidArgumentsException ze){
							UE.displayError(ze);
						}catch (Exception ze){
							UE.displayError(ze);
							lblBID.setIcon(null);
						}
					}
				}
			});
			//resident race
			cobRESIDENT.addActionListener(new ActionListener(){
                @Override
				public void actionPerformed(ActionEvent ae){
					if (SELECTED!=-1 && cobRESIDENT.getSelectedIndex()>=0){
						try{
							SYSTEMS.setResident(SELECTED, (short)cobRESIDENT.getSelectedIndex());
						}catch (Exception ze){
							UE.displayError(ze);
						}
					}
				}
			});
			//population
			spiPOPUL.addChangeListener(new ChangeListener(){
                @Override
				public void stateChanged(ChangeEvent e){
					if (SELECTED != -1){
						try{
							SYSTEMS.setPopulation(SELECTED, spiPOPUL.getValue());

                            int alloc = SYSTEMS.getPop2Ene(SELECTED) + SYSTEMS.getPop2Food(SELECTED)
                                    + SYSTEMS.getPop2Ind(SELECTED) + SYSTEMS.getPop2Intel(SELECTED)
                                    + SYSTEMS.getPop2Res(SELECTED);

                            if (alloc > spiPOPUL.getValue())
                            {
                                alloc = sliUNITS[4].getValue();
                                sliUNITS[4].setValue(0);
                                sliUNITS[4].setValue(alloc);
                                alloc = sliUNITS[3].getValue();
                                sliUNITS[3].setValue(0);
                                sliUNITS[3].setValue(alloc);
                                alloc = sliUNITS[1].getValue();
                                sliUNITS[1].setValue(0);
                                sliUNITS[1].setValue(alloc);
                                alloc = sliUNITS[2].getValue();
                                sliUNITS[2].setValue(0);
                                sliUNITS[2].setValue(alloc);
                                alloc = sliUNITS[0].getValue();
                                sliUNITS[0].setValue(0);
                                sliUNITS[0].setValue(alloc);
                            }

							updatePopText();
						}catch (Exception ze){
							UE.displayError(ze);
						}
					}
				}
			});
			//morale
			sliMORALE.addChangeListener(new ChangeListener(){
                @Override
				public void stateChanged(ChangeEvent e){
					if (SELECTED != -1 && cobRESIDENT.getSelectedIndex()>=0){
						try{
							SYSTEMS.setMorale(SELECTED, (short)sliMORALE.getValue());
							updateMoraleText();
						}catch (Exception ze){
							UE.displayError(ze);
						}
					}
				}
			});
			//population 2 food
			sliUNITS[0].addChangeListener(new ChangeListener(){
                @Override
				public void stateChanged(ChangeEvent e){
					if (SELECTED != -1){
						try{
							//ok, ppl
							int ava = SYSTEMS.getPopulation(SELECTED) - SYSTEMS.getPop2Ene(SELECTED) - SYSTEMS.getPop2Ind(SELECTED)
							- SYSTEMS.getPop2Intel(SELECTED) - SYSTEMS.getPop2Res(SELECTED);
							
                            if (sliUNITS[0].getValue()>ava)
                            {
								int lo = (ava / 10) * 10;

                                if (SYSTEMS.getPopulation(SELECTED) < 10)
                                {
                                    lo = 10;
                                }
								sliUNITS[0].setValue(lo);
							}
							//next please
							SYSTEMS.setPop2Food(SELECTED, sliUNITS[0].getValue());
							updatePopText();
						}catch (Exception ze){
							UE.displayError(ze);
						}
					}
				}
			});
			//population 2 industry
			sliUNITS[1].addChangeListener(new ChangeListener(){
                @Override
				public void stateChanged(ChangeEvent e){
					if (SELECTED != -1){
						try{
							//ok, ppl
							int ava = SYSTEMS.getPopulation(SELECTED) - SYSTEMS.getPop2Ene(SELECTED) - SYSTEMS.getPop2Food(SELECTED)
							- SYSTEMS.getPop2Intel(SELECTED) - SYSTEMS.getPop2Res(SELECTED);
                            if (sliUNITS[1].getValue()>ava)
                            {
								int lo = (ava / 10) * 10;

                                if (SYSTEMS.getPopulation(SELECTED) < 10)
                                {
                                    lo = 10;
                                }
								sliUNITS[1].setValue(lo);
							}
							//next please
							SYSTEMS.setPop2Ind(SELECTED, sliUNITS[1].getValue());
							updatePopText();
						}catch (Exception ze){
							UE.displayError(ze);
						}
					}
				}
			});
			//population 2 energy
			sliUNITS[2].addChangeListener(new ChangeListener(){
                @Override
				public void stateChanged(ChangeEvent e){
					if (SELECTED != -1){
						try{
							//ok, ppl
							int ava = SYSTEMS.getPopulation(SELECTED) - SYSTEMS.getPop2Ind(SELECTED) - SYSTEMS.getPop2Food(SELECTED)
							- SYSTEMS.getPop2Intel(SELECTED) - SYSTEMS.getPop2Res(SELECTED);

                            if (sliUNITS[2].getValue()>ava)
                            {
								int lo = (ava / 10) * 10;

                                if (SYSTEMS.getPopulation(SELECTED) < 10)
                                {
                                    lo = 10;
                                }
								sliUNITS[2].setValue(lo);
							}
							//next please
							SYSTEMS.setPop2Ene(SELECTED, sliUNITS[2].getValue());
							updatePopText();
						}catch (Exception ze){
							UE.displayError(ze);
						}
					}
				}
			});
			//population 2 intel
			sliUNITS[3].addChangeListener(new ChangeListener(){
                @Override
				public void stateChanged(ChangeEvent e){
					if (SELECTED != -1){
						try{
							//ok, ppl
							int ava = SYSTEMS.getPopulation(SELECTED) - SYSTEMS.getPop2Ind(SELECTED) - SYSTEMS.getPop2Food(SELECTED)
							- SYSTEMS.getPop2Ene(SELECTED) - SYSTEMS.getPop2Res(SELECTED);
                            if (sliUNITS[3].getValue()>ava)
                            {
								int lo = (ava / 10) * 10;

                                if (SYSTEMS.getPopulation(SELECTED) < 10)
                                {
                                    lo = 10;
                                }
								sliUNITS[3].setValue(lo);
							}
							//next please
							SYSTEMS.setPop2Intel(SELECTED, sliUNITS[3].getValue());
							updatePopText();
						}catch (Exception ze){
							UE.displayError(ze);
						}
					}
				}
			});
			//population 2 research
			sliUNITS[4].addChangeListener(new ChangeListener(){
                @Override
				public void stateChanged(ChangeEvent e){
					if (SELECTED != -1){
						try{
							//ok, ppl
							int ava = SYSTEMS.getPopulation(SELECTED) - SYSTEMS.getPop2Ind(SELECTED) - SYSTEMS.getPop2Food(SELECTED)
							- SYSTEMS.getPop2Ene(SELECTED) - SYSTEMS.getPop2Intel(SELECTED);

                            if (sliUNITS[4].getValue() > ava)
                            {
								int lo = (ava / 10) * 10;

                                if (SYSTEMS.getPopulation(SELECTED) < 10)
                                {
                                    lo = 10;
                                }

								sliUNITS[4].setValue(lo);
							}

							//next please
							SYSTEMS.setPop2Res(SELECTED, sliUNITS[4].getValue());
							updatePopText();
						}catch (Exception ze){
							UE.displayError(ze);
						}
					}
				}
			});
			//lay out
			setLayout(new GridBagLayout());
			GridBagConstraints c = new GridBagConstraints();
			c.insets=new Insets(5, 5, 5, 5);
			c.fill=GridBagConstraints.BOTH;
				//o_o add to panels
				JPanel pnlPOPU = new JPanel(new GridBagLayout());
				JPanel pnlSYST = new JPanel(new GridBagLayout());
					//syste
					c.gridx=0;
					c.gridy=0;
					c.gridwidth=1;
					c.gridheight=1;
					pnlSYST.add(lblBID, c); //dilithium sign
					c.gridwidth=2;
					c.gridheight=2;
					pnlSYST.add(aniSTAR, c); //animation
					c.gridwidth=1;
					c.gridheight=1;
					c.gridy=2;
					pnlSYST.add(lblLABEL[0], c); //type
					c.gridy=3;
					pnlSYST.add(lblLABEL[1], c); //ani
					c.gridy=4;
					pnlSYST.add(lblLABEL[10], c); //has dil
					c.gridx=1;
					c.gridy=2;
					pnlSYST.add(cobTYPE, c); //type
					c.gridy=3;
					pnlSYST.add(cobSTARANI, c); //star ani
					c.gridy=4;
					pnlSYST.add(chbDIL, c); //ani
					c.gridx=0;
					c.gridy=5;
					c.gridwidth=2;
					pnlSYST.add(lblMAXPOP, c); //max pop
					//popul
					c.gridx=0;
					c.gridy=0;
					c.gridwidth=1;
					c.gridheight=1;
					pnlPOPU.add(lblLABEL[2], c); //resident
					c.gridy=1;
					pnlPOPU.add(lblLABEL[3], c); //pop
					c.gridy=2;
					pnlPOPU.add(lblMORALE, c); //morale
					c.gridx=1;
					c.gridy=0;
					pnlPOPU.add(cobRESIDENT, c); //resident
					c.gridy=1;
					pnlPOPU.add(spiPOPUL, c); //pop
					c.gridy=2;
					pnlPOPU.add(sliMORALE, c); //morale
					c.gridx=0;
					c.gridy=3;
					pnlPOPU.add(lblLABEL[4], c); //alloc
					for (int i=0; i<5; i++){
						c.gridy=4+i;
						pnlPOPU.add(lblLABEL[i+5], c); //alloc
					}
					c.gridy=9;
					c.gridwidth=2;
					pnlPOPU.add(lblUNALL, c); //unalloc
					c.gridwidth=1;
					c.gridx=1;
					for (int i=0; i<5; i++){
						c.gridy=4+i;
						pnlPOPU.add(sliUNITS[i], c); //alloc
					}
				pnlPOPU.setBorder(new LineBorder(Color.BLACK)); 
				pnlSYST.setBorder(new LineBorder(Color.BLACK)); 
			c.gridx=0;
			c.gridy=0;
			c.gridwidth=1;
			c.gridheight=1;
			add(pnlSYST, c);
			c.gridx=1;
			add(pnlPOPU, c);
			//fill them up
			//type
			cobTYPE.removeAllItems();
			cobTYPE.addItem(Language.getString("SystemsInfoGUI.13")); //$NON-NLS-1$
			cobTYPE.addItem(Language.getString("SystemsInfoGUI.14")); //$NON-NLS-1$
			cobTYPE.addItem(Language.getString("SystemsInfoGUI.15")); //$NON-NLS-1$
			cobTYPE.addItem(Language.getString("SystemsInfoGUI.16")); //$NON-NLS-1$
			cobTYPE.addItem(Language.getString("SystemsInfoGUI.17")); //$NON-NLS-1$
			cobTYPE.addItem(Language.getString("SystemsInfoGUI.18")); //$NON-NLS-1$
			cobTYPE.addItem(Language.getString("SystemsInfoGUI.19")); //$NON-NLS-1$
			cobTYPE.addItem(Language.getString("SystemsInfoGUI.20")); //$NON-NLS-1$
			cobTYPE.addItem(Language.getString("SystemsInfoGUI.21")); //$NON-NLS-1$
			cobTYPE.addItem(Language.getString("SystemsInfoGUI.22")); //$NON-NLS-1$
			cobTYPE.addItem(Language.getString("SystemsInfoGUI.23")); //$NON-NLS-1$
			cobTYPE.addItem(Language.getString("SystemsInfoGUI.24")); //$NON-NLS-1$
			//Star ani
			cobSTARANI.removeAllItems();
			Enumeration<String[]> anis = STBOF.getFiles(new java.io.FilenameFilter(){
				public boolean accept(File dir, String name){
					if (name.toLowerCase().endsWith(".ani")) { //$NON-NLS-1$
						return true;
					} else {
						return false; //$NON-NLS-1$
					}
				}
			});
			while (anis.hasMoreElements()){
				cobSTARANI.addItem(anis.nextElement()[0]);
			}
			//resident
			cobRESIDENT.removeAllItems();
			int ij=RACES.getNumberOfEntries();
			for (int i=0; i<ij; i++){
				try{
					cobRESIDENT.addItem(RACES.getName(i));
				}catch (Exception ze){
					UE.displayError(ze);
				}
			}
		}
		//should be called everytime SELECTED changes or this class gains focus
		public void init()
        {
			if (SELECTED==-1)
            {
				return;
			}

			try
            {
				//calculate maximum population
				int j = SYSTEMS.planets(SELECTED);
				MAX_POP = 0;
				byte uni;
				byte pop;
				byte typ;
				byte siz;
				for (int i=0; i<j; i++){
					uni = SYSTEMS.getUniBoni(SELECTED, i);
					pop = SYSTEMS.getPopBoni(SELECTED, i);
					typ = SYSTEMS.getType(SELECTED, i);
					siz = (byte)SYSTEMS.getSize(SELECTED, i);
					MAX_POP += SPACE.get(typ, siz, uni, pop);
				}
				//set limits
				int temp = SELECTED;
				SELECTED = -1;
				for (int i=0; i<5; i++){
					sliUNITS[i].setMinimum(0);
					sliUNITS[i].setMaximum(MAX_POP);
				}
				spiPOPUL.setMinimum(0);
				spiPOPUL.setMaximum(MAX_POP);
				SELECTED = temp;
				//set cobRESIDENT
				short ku = SYSTEMS.getResident(SELECTED);
				if (ku < 0)
                {
                    btnSTRUCT.setEnabled(false);

					SYSTEMS.setResident(SELECTED, (short)-1);
					//morale
					sliMORALE.setEnabled(false);
					SYSTEMS.setMorale(SELECTED, (short)0);
					//population
					spiPOPUL.setEnabled(false);
					SYSTEMS.setPopulation(SELECTED, 0);
					//units
					for (int i=0; i<5; i++){
						sliUNITS[i].setEnabled(false);
						sliUNITS[i].setValue(0);
					}
					cobRESIDENT.setSelectedIndex(-1);
					cobRESIDENT.setEnabled(false);
				}else{
                    btnSTRUCT.setEnabled(true);
					cobRESIDENT.setEnabled(true);
					cobRESIDENT.setSelectedIndex(ku);
					//morale
					sliMORALE.setEnabled(true);
					//population
					spiPOPUL.setEnabled(true);
					//units
					for (int i=0; i<5; i++){
						sliUNITS[i].setEnabled(true);
					}
				}
				//set lblMORALE and sliMORALE
				updateMoraleText();
				//set cobTYPE
				cobTYPE.setSelectedIndex(SYSTEMS.getType(SELECTED));
				//set cobSTARANI
				cobSTARANI.setSelectedItem(SYSTEMS.getAni(SELECTED));
				//set chbDIL
				chbDIL.setSelected(SYSTEMS.hasDilithium(SELECTED));
				//set spiPOPUL
				spiPOPUL.setValue(SYSTEMS.getPopulation(SELECTED));
				//set pop2food
				sliUNITS[0].setValue(SYSTEMS.getPop2Food(SELECTED));
				//set pop2ind
				sliUNITS[1].setValue(SYSTEMS.getPop2Ind(SELECTED));
				//set pop2ene
				sliUNITS[2].setValue(SYSTEMS.getPop2Ene(SELECTED));
				//set pop2intel
				sliUNITS[3].setValue(SYSTEMS.getPop2Intel(SELECTED));
				//set pop2res
				sliUNITS[4].setValue(SYSTEMS.getPop2Res(SELECTED));
				//set labels
				updatePopText();
			}catch (Exception ze){
				UE.displayError(ze);
			}
		}
		
		@Override
		public void finalWarning() throws Exception{
		}
		
		private void updatePopText() throws InvalidArgumentsException{
			String msg = Language.getString("SystemsInfoGUI.31"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(SYSTEMS.getPopulation(SELECTED))); //$NON-NLS-1$
			lblLABEL[3].setText(msg);
			msg = Language.getString("SystemsInfoGUI.25"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(MAX_POP)); //$NON-NLS-1$
			lblMAXPOP.setText(msg);
			msg = Language.getString("SystemsInfoGUI.26"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(sliUNITS[0].getValue())); //$NON-NLS-1$
			lblLABEL[5].setText(msg);
			msg = Language.getString("SystemsInfoGUI.27"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(sliUNITS[1].getValue())); //$NON-NLS-1$
			lblLABEL[6].setText(msg);
			msg = Language.getString("SystemsInfoGUI.28"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(sliUNITS[2].getValue())); //$NON-NLS-1$
			lblLABEL[7].setText(msg);
			msg = Language.getString("SystemsInfoGUI.29"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(sliUNITS[3].getValue())); //$NON-NLS-1$
			lblLABEL[8].setText(msg);
			msg = Language.getString("SystemsInfoGUI.30"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(sliUNITS[4].getValue())); //$NON-NLS-1$
			lblLABEL[9].setText(msg);
			int ava = SYSTEMS.getPopulation(SELECTED) - sliUNITS[0].getValue() - sliUNITS[1].getValue() - sliUNITS[2].getValue()
			 - sliUNITS[3].getValue() - sliUNITS[4].getValue();
			msg = Language.getString("SystemsInfoGUI.32"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(ava)); //$NON-NLS-1$
			lblUNALL.setText(msg);
		}
		
		private void updateMoraleText() throws InvalidArgumentsException{
			int morale = SYSTEMS.getMorale(SELECTED);
			sliMORALE.setValue(morale);
			String msg = Language.getString("SystemsInfoGUI.33"); //$NON-NLS-1$
			msg = msg.replace("%1", Integer.toString(morale)); //$NON-NLS-1$
			if (morale<=0){
				msg = msg.replace("%2", Language.getString("SystemsInfoGUI.34")); //$NON-NLS-1$ //$NON-NLS-2$
			}else if (morale<25){
				msg = msg.replace("%2", Language.getString("SystemsInfoGUI.35")); //$NON-NLS-1$ //$NON-NLS-2$
			}else if (morale<51){
				msg = msg.replace("%2", Language.getString("SystemsInfoGUI.36")); //$NON-NLS-1$ //$NON-NLS-2$
			}else if (morale<76){
				msg = msg.replace("%2", Language.getString("SystemsInfoGUI.37")); //$NON-NLS-1$ //$NON-NLS-2$
			}else if (morale<100){
				msg = msg.replace("%2", Language.getString("SystemsInfoGUI.38")); //$NON-NLS-1$ //$NON-NLS-2$
			}else if (morale<126){
				msg = msg.replace("%2", Language.getString("SystemsInfoGUI.39")); //$NON-NLS-1$ //$NON-NLS-2$
			}else if (morale<151){
				msg = msg.replace("%2", Language.getString("SystemsInfoGUI.40")); //$NON-NLS-1$ //$NON-NLS-2$
			}else if (morale<176){
				msg = msg.replace("%2", Language.getString("SystemsInfoGUI.41")); //$NON-NLS-1$ //$NON-NLS-2$
			}else{
				msg = msg.replace("%2", Language.getString("SystemsInfoGUI.42")); //$NON-NLS-1$ //$NON-NLS-2$
			}
			lblMORALE.setText(msg);
		}
	}
	/*To edit planets*/
	private class SystemPlanet extends MainPanel{
		//-_-
		private int SEL=-1;
		//elements
		private JLabel[] LABEL = new JLabel[8];
		private JButton btnADD = new JButton(Language.getString("SystemsInfoGUI.43")); //$NON-NLS-1$
		private JButton btnEDIT = new JButton(Language.getString("SystemsInfoGUI.0")); //$NON-NLS-1$
		private JButton btnREMOVE = new JButton(Language.getString("SystemsInfoGUI.44")); //$NON-NLS-1$
		private JButton btnMOVEUP = new JButton(Language.getString("SystemsInfoGUI.45")); //$NON-NLS-1$
		private JButton btnMOVEDW = new JButton(Language.getString("SystemsInfoGUI.46")); //$NON-NLS-1$
		private JTextField jtfPOINT = new JTextField(10);
		private JTextField jtfTERRA = new JTextField(10);
		private JList jcbNAME = new JList();
		private JComboBox jcbANI = new JComboBox();
		private JComboBox jcbTYPE = new JComboBox();
		private JComboBox jcbATMO = new JComboBox();
		private JComboBox jcbSIZE = new JComboBox();
		private JComboBox[] jcbBONUS = new JComboBox[3];
		private AniLabel aniANIM = new AniLabel();
		private JLabel jlPOP = new JLabel();
		private JLabel jlBONI = new JLabel();
		private JLabel jlGROWTH = new JLabel(Language.getString("SystemsInfoGUI.47")); // unused! //$NON-NLS-1$
		/*constructor*/
		public SystemPlanet(){
			//build elements
			LABEL[0] = new JLabel(Language.getString("SystemsInfoGUI.48")); //$NON-NLS-1$
			LABEL[1] = new JLabel(Language.getString("SystemsInfoGUI.49")); //$NON-NLS-1$
			LABEL[2] = new JLabel(Language.getString("SystemsInfoGUI.50")); //$NON-NLS-1$
			LABEL[3] = new JLabel(Language.getString("SystemsInfoGUI.51")); //$NON-NLS-1$
			LABEL[4] = new JLabel(Language.getString("SystemsInfoGUI.52")); //$NON-NLS-1$
			LABEL[5] = new JLabel(Language.getString("SystemsInfoGUI.53")); //$NON-NLS-1$
			LABEL[6] = new JLabel(Language.getString("SystemsInfoGUI.54")); //$NON-NLS-1$
			LABEL[7] = new JLabel(Language.getString("SystemsInfoGUI.55")); //$NON-NLS-1$
			jcbBONUS[0] = new JComboBox();
			jcbBONUS[1] = new JComboBox();
			jcbBONUS[2] = new JComboBox();
			//set font
			Font def = UE.SETTINGS.getDefaultFont();
			for (int i=0; i<8; i++){
				LABEL[i].setFont(def);
				if (i<3) {
					jcbBONUS[i].setFont(def);
				}
			}
			btnEDIT.setFont(def);
			jcbNAME.setFont(def);
			jcbANI.setFont(def);
			jcbTYPE.setFont(def);
			jcbATMO.setFont(def);
			jcbSIZE.setFont(def);
			jlPOP.setFont(def);
			jlBONI.setFont(def);
			jlGROWTH.setFont(def);
			jtfPOINT.setFont(def);
			jtfTERRA.setFont(def);
			btnADD.setFont(def);
			btnREMOVE.setFont(def);
			btnMOVEUP.setFont(def);
			btnMOVEDW.setFont(def);
			jcbNAME.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			//add listeners
			btnEDIT.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					if (SELECTED==-1 || jcbNAME.getSelectedIndex()<0) {
						return;
					}
					try{
						String n = JOptionPane.showInputDialog(strENTER_NEW_NAME,
								SYSTEMS.getName(SELECTED, jcbNAME.getSelectedIndex()));
						if (n!=null){
							int t = jcbNAME.getSelectedIndex();
							finalWarning();
							SYSTEMS.setName(SELECTED, t, n);
							fillPlanets();
							jcbNAME.setSelectedIndex(t);
						}
					}catch (Exception ze){
						UE.displayError(ze);
					}
				}
			});
			btnADD.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					if (SELECTED==-1) {
						return;
					}
					try{
						finalWarning();
						SYSTEMS.addPlanet(SELECTED);
						fillPlanets();
					}catch (Exception ze){
						UE.displayError(ze);
					}
				}
			});
			btnREMOVE.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					if (SELECTED==-1 || jcbNAME.getSelectedIndex()<0) {
						return;
					}
					try{
						finalWarning();
						SYSTEMS.removePlanet(SELECTED, (short)jcbNAME.getSelectedIndex());
						fillPlanets();
					}catch (Exception ze){
						UE.displayError(ze);
					}
				}
			});
			btnMOVEUP.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					if (SELECTED==-1 || jcbNAME.getSelectedIndex()<0) {
						return;
					}
					try{
						finalWarning();
						SYSTEMS.movePlanetUp(SELECTED, (short)jcbNAME.getSelectedIndex());
						fillPlanets();
					}catch (Exception ze){
						UE.displayError(ze);
					}
				}
			});
			btnMOVEDW.addActionListener(new ActionListener(){
                @Override
				public void actionPerformed(ActionEvent ae){
					if (SELECTED==-1 || jcbNAME.getSelectedIndex()<0) {
						return;
					}
					try{
						finalWarning();
						SYSTEMS.movePlanetDown(SELECTED, (short)jcbNAME.getSelectedIndex());
						fillPlanets();
					}catch (Exception ze){
						UE.displayError(ze);
					}
				}
			});
			jcbNAME.addListSelectionListener(new ListSelectionListener(){
				public void valueChanged(ListSelectionEvent e){
					if (SELECTED<0) {
						return;
					}
					//save
                    try
                    {
                        finalWarning();
                    }
                    catch (Exception ex)
                    {
                        // really bad stuff happening - bail
                        UE.displayError(ex);
                        return;
                    }
					//show
					SEL = (short)jcbNAME.getSelectedIndex();
					try{
						if (SEL>=0) {
							jcbTYPE.setEnabled(true);
							jcbATMO.setEnabled(true);
							jcbSIZE.setEnabled(true);
							jcbANI.setEnabled(true);
							jcbTYPE.setEnabled(true);
							jcbBONUS[0].setEnabled(true);
							jcbBONUS[1].setEnabled(true);
							jcbBONUS[2].setEnabled(true);
							jtfPOINT.setEnabled(true);
							jtfTERRA.setEnabled(true);
							btnEDIT.setEnabled(true);
							//btnREMOVE.setEnabled(true);
							btnMOVEUP.setEnabled(true);
							btnMOVEDW.setEnabled(true);
							jcbTYPE.setSelectedIndex(SYSTEMS.getType(SELECTED, SEL));
							jcbATMO.setSelectedIndex(SYSTEMS.getAtmosphere(SELECTED, SEL)-8);
							jcbSIZE.setSelectedIndex(SYSTEMS.getSize(SELECTED, SEL));
							jcbANI.setSelectedItem(SYSTEMS.getAnimation(SELECTED, SEL));
							jcbBONUS[0].setSelectedIndex(SYSTEMS.getPltBoni(SELECTED, SEL));
							jcbBONUS[1].setSelectedIndex(SYSTEMS.getUniBoni(SELECTED, SEL));
							jcbBONUS[2].setSelectedIndex(SYSTEMS.getPopBoni(SELECTED, SEL));
							jtfPOINT.setText(Integer.toString(SYSTEMS.getTerraPoints(SELECTED, SEL)));
							jtfTERRA.setText(Integer.toString(SYSTEMS.getTerraformed(SELECTED, SEL)));
						}else{
							jcbTYPE.setEnabled(false);
							jcbATMO.setEnabled(false);
							jcbSIZE.setEnabled(false);
							jcbANI.setEnabled(false);
							jcbTYPE.setEnabled(false);
							jcbBONUS[0].setEnabled(false);
							jcbBONUS[1].setEnabled(false);
							jcbBONUS[2].setEnabled(false);
							jtfPOINT.setEnabled(false);
							jtfTERRA.setEnabled(false);
							btnEDIT.setEnabled(false);
							btnREMOVE.setEnabled(false);
							btnMOVEUP.setEnabled(false);
							btnMOVEDW.setEnabled(false);
							aniANIM.setAnimation(null, false);
						}
						updateText();
					}catch (Exception iae){
						UE.displayError(iae);
					}
				}
			});
			jcbANI.addActionListener(new ActionListener(){
                @Override
				public void actionPerformed(ActionEvent ae){
					if (SELECTED==-1 || jcbNAME.getSelectedIndex()<0) {
						return;
					}
					try{
						int t = jcbNAME.getSelectedIndex();
						SYSTEMS.setAnimation(SELECTED, t, (String)jcbANI.getSelectedItem());
						jcbANI.setSelectedItem(SYSTEMS.getAnimation(SELECTED, t));
						aniANIM.setAnimation(STBOF.getAnimation(SYSTEMS.getAnimation(SELECTED, t), true), true);
					}catch (InvalidArgumentsException iae){
						UE.displayError(iae);
					}catch (Exception fnf){
						UE.displayError(fnf);
						aniANIM.setAnimation(null, false);
					}
				}
			});
			jcbTYPE.addActionListener( new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					if (SELECTED==-1 || jcbNAME.getSelectedIndex()<0) {
						return;
					}
					try{
						int t = jcbNAME.getSelectedIndex();
						byte lol = (byte)jcbTYPE.getSelectedIndex();
						if (lol==-1) {
							return;
						}
						SYSTEMS.setType(SELECTED, t, lol);
						updateText();
					}catch (Exception iae){
						UE.displayError(iae);
					}
				}
			});
			jcbATMO.addActionListener( new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					if (SELECTED==-1 || jcbNAME.getSelectedIndex()<0) {
						return;
					}
					try{
						int t = jcbNAME.getSelectedIndex();
						byte lol = (byte)jcbATMO.getSelectedIndex();
						if (lol==-1) {
							return;
						}
						SYSTEMS.setAtmosphere(SELECTED, t, (byte)(lol+8));
					}catch (Exception iae){
						UE.displayError(iae);
					}
				}
			});
			jcbSIZE.addActionListener( new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					if (SELECTED==-1 || jcbNAME.getSelectedIndex()<0) {
						return;
					}
					try{
						int t = jcbNAME.getSelectedIndex();
						short lol = (short)jcbSIZE.getSelectedIndex();
						if (lol==-1) {
							return;
						}
						SYSTEMS.setSize(SELECTED, t, lol);
						jcbSIZE.setSelectedItem(SYSTEMS.getSize(SELECTED, t));
						updateText();
					}catch (Exception iae){
						UE.displayError(iae);
					}
				}
			});
			jcbBONUS[0].addActionListener( new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					if (SELECTED==-1 || jcbNAME.getSelectedIndex()<0) {
						return;
					}
					try{
						int t = jcbNAME.getSelectedIndex();
						byte lol = (byte)jcbBONUS[0].getSelectedIndex();
						if (lol==-1) {
							return;
						}
						SYSTEMS.setPltBoni(SELECTED, t, lol);
						jcbBONUS[0].setSelectedItem(SYSTEMS.getPltBoni(SELECTED, t));
						updateText();
					}catch (Exception iae){
						UE.displayError(iae);
					}
				}
			});
			jcbBONUS[1].addActionListener( new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					if (SELECTED==-1 || jcbNAME.getSelectedIndex()<0) {
						return;
					}
					try{
						int t = jcbNAME.getSelectedIndex();
						byte lol = (byte)jcbBONUS[1].getSelectedIndex();
						if (lol==-1) {
							return;
						}
						SYSTEMS.setUniBoni(SELECTED, t, lol);
						jcbBONUS[1].setSelectedItem(SYSTEMS.getUniBoni(SELECTED, t));
						updateText();
					}catch (Exception iae){
						UE.displayError(iae);
					}
				}
			});
			jcbBONUS[2].addActionListener( new ActionListener(){
                @Override
				public void actionPerformed(ActionEvent ae){
					if (SELECTED==-1 || jcbNAME.getSelectedIndex()<0) {
						return;
					}
					try{
						int t = jcbNAME.getSelectedIndex();
						byte lol = (byte)jcbBONUS[2].getSelectedIndex();
						if (lol==-1) {
							return;
						}
						SYSTEMS.setPopBoni(SELECTED, t, lol);
						jcbBONUS[2].setSelectedItem(SYSTEMS.getPopBoni(SELECTED, t));
						updateText();
					}catch (Exception iae){
						UE.displayError(iae);
					}
				}
			});
			//fill combos
			Enumeration<String[]> enu = STBOF.getFiles(new java.io.FilenameFilter(){
				public boolean accept(File dir, String name){
					if (name.toLowerCase().endsWith(".ani")) { //$NON-NLS-1$
						return true;
					} else {
						return false; //$NON-NLS-1$
					}
				}
			});
			String str = Language.getString("SystemsInfoGUI.56"); //$NON-NLS-1$
			while (enu.hasMoreElements()){
				str = enu.nextElement()[0];
				jcbANI.addItem(str);
			}
			String[] str2 = ENVIRON.toArray();
			for (int i=0; i<str2.length && i<8; i++){
				jcbTYPE.addItem(str2[i]);
			}
			for (int i=8; i<str2.length; i++){
				jcbATMO.addItem(str2[i]);
			}
			jcbSIZE.addItem(Language.getString("SystemsInfoGUI.57")); //$NON-NLS-1$
			jcbSIZE.addItem(Language.getString("SystemsInfoGUI.58")); //$NON-NLS-1$
			jcbSIZE.addItem(Language.getString("SystemsInfoGUI.59")); //$NON-NLS-1$
			for (int i=0; i<3; i++){
				if (i==0) {
					str=Language.getString("SystemsInfoGUI.60"); //$NON-NLS-1$
				}
				if (i==1) {
					str=Language.getString("SystemsInfoGUI.61"); //$NON-NLS-1$
				}
				if (i==2) {
					str=Language.getString("SystemsInfoGUI.62"); //$NON-NLS-1$
				}
				jcbBONUS[0].addItem(str);
				jcbBONUS[1].addItem(str);
				jcbBONUS[2].addItem(str);
			}
			fillPlanets();
			//place
			setLayout(new GridBagLayout());
			GridBagConstraints c = new GridBagConstraints();
			c.insets=new Insets(5, 5, 5, 5);
			c.fill=GridBagConstraints.BOTH;
			JPanel jpPREV = new JPanel(new GridBagLayout());
			JPanel jpEDIT = new JPanel(new GridBagLayout());
				//preview
				c.gridx=0;
				c.gridy=0;
				c.gridwidth=1;
				c.gridheight=6;
				JScrollPane scrolly = new JScrollPane(jcbNAME);
				scrolly.setPreferredSize(new Dimension(200, 150));
				jpPREV.add(scrolly, c);
				c.gridx=1;
				c.gridheight=1;
				jpPREV.add(btnADD, c);
				c.gridy=1;
				jpPREV.add(btnEDIT, c);
				c.gridy=2;
				jpPREV.add(btnREMOVE, c);
				c.gridy=3;
				jpPREV.add(btnMOVEUP, c);
				c.gridy=4;
				jpPREV.add(btnMOVEDW, c);
				c.gridx=2;
				c.gridy=0;
				c.gridheight=2;
				jpPREV.add(aniANIM, c);
				c.gridheight=1;
				c.gridy = 2;
				jpPREV.add(jlPOP, c);
				c.gridy = 3;
				jpPREV.add(jlBONI, c);
				c.gridy = 4;
//				jpPREV.add(jlGROWTH, c);
				//edit
				c.gridx=0;
				for (int i=0; i<8; i++){
					if (i==5) {
						c.gridx=2;
					}
					if (i<5) {
						c.gridy=i;
					} else {
						c.gridy=i-5;
					}
					jpEDIT.add(LABEL[i], c);
				}
				c.gridy=3;
				JLabel lbl = new JLabel(Language.getString("SystemsInfoGUI.63")); //$NON-NLS-1$
				lbl.setFont(def);
				jpEDIT.add(lbl, c);
				c.gridx=1;
				c.gridy=0;
				jpEDIT.add(jcbANI, c);
				c.gridy=1;
				jpEDIT.add(jcbTYPE, c);
				c.gridy=2;
				jpEDIT.add(jcbSIZE, c);
				c.gridy=3;
				jpEDIT.add(jtfPOINT, c);
				c.gridy=4;
				jpEDIT.add(jtfTERRA, c);
				c.gridx=3;
				c.gridy=0;
				jpEDIT.add(jcbBONUS[0], c);
				c.gridy=1;
				jpEDIT.add(jcbBONUS[1], c);
				c.gridy=2;
				jpEDIT.add(jcbBONUS[2], c);
				c.gridy=3;
				jpEDIT.add(jcbATMO, c);
			jpPREV.setBorder(new LineBorder(Color.BLACK)); 
			jpEDIT.setBorder(new LineBorder(Color.BLACK)); 
			c.gridx=0;
			c.gridy=0;
			c.gridwidth=1;
			c.gridheight=1;
			add(jpPREV, c);
			c.gridy=1;
			add(jpEDIT, c);
			
		}
		
		private void updateText() throws InvalidArgumentsException{
			String msg;
			if (SEL>=0) {
				int pot = SPACE.get(SYSTEMS.getType(SELECTED, SEL), (byte)SYSTEMS.getSize(SELECTED, SEL),
				SYSTEMS.getUniBoni(SELECTED, SEL), SYSTEMS.getPopBoni(SELECTED, SEL));
				msg = Language.getString("SystemsInfoGUI.64"); //$NON-NLS-1$
				msg = msg.replace("%1", Integer.toString(pot)); //$NON-NLS-1$
				jlPOP.setText(msg);
				float pov = BONI.get(SYSTEMS.getType(SELECTED, SEL), (byte)SYSTEMS.getSize(SELECTED, SEL),
				SYSTEMS.getPltBoni(SELECTED, SEL), SYSTEMS.getUniBoni(SELECTED, SEL));
				msg = Language.getString("SystemsInfoGUI.65"); //$NON-NLS-1$
				msg = msg.replace("%1", (pov*100*0.05) + "%"); //$NON-NLS-1$ //$NON-NLS-2$
				jlBONI.setText(msg);
			}else{
				msg = Language.getString("SystemsInfoGUI.66"); //$NON-NLS-1$
				jlPOP.setText(msg);
				msg = Language.getString("SystemsInfoGUI.67"); //$NON-NLS-1$
				jlBONI.setText(msg);
			}
		}
		
		private void fillPlanets(){
			if (SELECTED==-1) {
				return;
			}
			try{
				String[] str = SYSTEMS.getPlanetNames(SELECTED);
				jcbNAME.setListData(str);
				jcbTYPE.setEnabled(false);
				jcbSIZE.setEnabled(false);
				jcbANI.setEnabled(false);
				jcbATMO.setEnabled(false);
				jcbBONUS[0].setEnabled(false);
				jcbBONUS[1].setEnabled(false);
				jcbBONUS[2].setEnabled(false);
				jtfPOINT.setEnabled(false);
				jtfTERRA.setEnabled(false);
				updateText();
				btnADD.setEnabled(false);
				btnEDIT.setEnabled(false);
				btnREMOVE.setEnabled(false);
				btnMOVEUP.setEnabled(false);
				btnMOVEDW.setEnabled(false);
				aniANIM.setAnimation(null, false);
				updateText();
			}catch (Exception ze){
				UE.displayError(ze);
			}
		}
		
		//should be called everytime SELECTED changes or this class gains focus
		public void init(){
			if (SELECTED==-1) {
				return;
			}
			SEL = -1;
			fillPlanets();
		}
		
		@Override
		public void finalWarning() throws Exception{
			if (SELECTED<0 || SEL<0) {
				return;
			}
            SYSTEMS.setTerraPoints(SELECTED, SEL, Integer.parseInt(jtfPOINT.getText()));
            SYSTEMS.setTerraformed(SELECTED, SEL, Integer.parseInt(jtfTERRA.getText()));
		}
	}

	@Override
	public String getHelpFileName(){
		return "SystemsInfo.html"; //$NON-NLS-1$
	}
}
