/**
 * Ultimate Editor (dev)
 * Copyright (C) 2009 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: SwitchRaceGUI.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 3.5.2009
 */
package ue.gui.snd;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.Timer;

import ue.Language;
import ue.UE;
import ue.edit.CanBeEdited;
import ue.edit.FileManager;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.rst.Race;
import ue.edit.snd.English;
import ue.exception.InvalidArgumentsException;
import ue.gui.MainPanel;
import ue.util.ID;
import ue.util.SettingsManager;

/**
 * @author Alan Podlesek
 */
@SuppressWarnings("serial") //$NON-NLS-1$
public class SwitchRaceGUI extends MainPanel
{
    private Race RACE;
    private English ENGLISH;
    
    private JComboBox[] cmbRace;
    private JButton btnSwitch;
    private JButton btnSet;
    private JLabel lblUse;
    private JLabel lblSet;
    private JLabel lblSwitch;
    
    private Timer tReEnableButtons;
        
    public SwitchRaceGUI(English english)
    {
        ENGLISH = english;
        
        try
        {
            CanBeEdited cbe = UE.FILES.getFile(FileManager.P_STBOF);
            
            if (cbe != null)
            {
                RACE = (Race)cbe.getInternalFile(Stbof.fileRACE_RST, true);
            }
            
            if (RACE == null)
            {
                return;
            }
    
            tReEnableButtons = new Timer(2000, new ActionListener() // 2 sec cool-off time
            {
                @Override
                public void actionPerformed(ActionEvent arg0)
                {
                    btnSwitch.setEnabled(true);
                    btnSet.setEnabled(true);
                }
            });
            
            setupComponents();
            placeComponents();
        }
        catch (Exception et)
        {
            UE.displayError(et);
            return;
        }
    }
    
    private void placeComponents()
    {
        this.setLayout(new BorderLayout());
        JPanel mothership = new JPanel(new GridLayout(0, 1));
        JPanel line1 = new JPanel(new FlowLayout(FlowLayout.CENTER));
        JPanel line2 = new JPanel(new FlowLayout(FlowLayout.CENTER));
        
        line1.add(lblSwitch);
        line1.add(cmbRace[0]);
        line1.add(cmbRace[1]);
        line1.add(btnSwitch);
        line2.add(lblSet);
        line2.add(cmbRace[2]);
        line2.add(lblUse);
        line2.add(cmbRace[3]);
        line2.add(btnSet);
        
        mothership.add(line1);
        mothership.add(line2);
        
        add(mothership, BorderLayout.NORTH);
        add(Box.createVerticalGlue(), BorderLayout.CENTER);
    }

    private void setupComponents() throws InvalidArgumentsException
    {
        Font fnt = UE.SETTINGS.getDefaultFont();
        
        // combos
        cmbRace = new JComboBox[4];
        
        for (int i = 0; i < cmbRace.length; i++)
        {
            cmbRace[i] = new JComboBox();
            cmbRace[i].setFocusable(false);
            cmbRace[i].setFont(fnt);

            for (int r = 0; r < 5; r++)
            {
                cmbRace[i].addItem(new ID(RACE.getName(r), r));
            }
        }
        
        // button
        btnSwitch = new JButton(Language.getString("SwitchRaceGUI.0")); //$NON-NLS-1$
        btnSwitch.setFont(fnt);
        btnSwitch.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                // prevent click happyness
                btnSwitch.setEnabled(false);
                btnSet.setEnabled(false);
                tReEnableButtons.start();
                
                if (cmbRace[0].getSelectedIndex() >= 0 || cmbRace[1].getSelectedIndex() >= 0)
                {
                    try
                    {
                        ENGLISH.switchRaces(
                                (short)(cmbRace[0].getSelectedIndex()),
                                (short)(cmbRace[1].getSelectedIndex()),
                                true);
                    }
                    catch (Exception f)
                    {
                        UE.displayError(f);
                    }
                }
            }
        });
        
        // button 2
        btnSet = new JButton(Language.getString("SwitchRaceGUI.1")); //$NON-NLS-1$
        btnSet.setFont(fnt);
        btnSet.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                // prevent click happyness
                btnSwitch.setEnabled(false);
                btnSet.setEnabled(false);
                tReEnableButtons.start();

                if (cmbRace[2].getSelectedIndex() >= 0 || cmbRace[3].getSelectedIndex() >= 0)
                {
                    try
                    {
                        ENGLISH.switchRaces(
                                (short)(cmbRace[2].getSelectedIndex()),
                                (short)(cmbRace[3].getSelectedIndex()),
                                false);
                    }
                    catch (Exception f)
                    {
                        UE.displayError(f);
                    }
                }
            }
        });
        
        // labels
        lblUse = new JLabel(Language.getString("SwitchRaceGUI.2")); //$NON-NLS-1$
        lblUse.setFont(fnt);
        lblSet = new JLabel(Language.getString("SwitchRaceGUI.1")); //$NON-NLS-1$
        lblSet.setFont(fnt);
        lblSwitch = new JLabel(Language.getString("SwitchRaceGUI.3")); //$NON-NLS-1$
        lblSwitch.setFont(fnt);
        
        // clean up
        // is supposed to hint GC it's ok to remove the item
        // in any case it can't hurt
        fnt = null;
    }

    /* (non-Javadoc)
     * @see ue.gui.MainPanel#finalWarning()
     */
    @Override
    public void finalWarning() throws Exception
    {
        RACE = null;
        ENGLISH = null;
        cmbRace = null;
        btnSwitch = null;
        btnSet = null;
        lblUse = null;
        lblSet = null;
        lblSwitch = null;
        tReEnableButtons = null;
    }    
}
