/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: CalcGUI.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui.special;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JTextField;

import ue.Language;
import ue.UE;
import ue.gui.MainPanel;
import ue.util.SettingsManager;

/**Interface for converting numbers.*/
@SuppressWarnings("serial")  //$NON-NLS-1$
public class CalcGUI extends MainPanel
{
    private JTextField txtDEC;
    private JTextField txtHEX;
    private ButtonGroup btgTYPE = new ButtonGroup();
    private ButtonGroup btgEND = new ButtonGroup();
    private JCheckBox chkSHT;
    private JCheckBox chkINT;
    private JCheckBox chkLNG;
    private JCheckBox chkFLT;
    private JCheckBox chkDBL;
    private JCheckBox chkLIT;
    private JCheckBox chkBIG;
    // Strings
    private static String strCONVERT = Language.getString("CalcGUI.0"); //$NON-NLS-1$
    private static String strTYPE_DOUBLE = Language.getString("CalcGUI.1"); //$NON-NLS-1$
    private static String strTYPE_FLOAT = Language.getString("CalcGUI.2"); //$NON-NLS-1$
    private static String strTYPE_INTEGER = Language.getString("CalcGUI.3"); //$NON-NLS-1$
    private static String strTYPE_LONG = Language.getString("CalcGUI.4"); //$NON-NLS-1$
    private static String strTYPE_SHORT = Language.getString("CalcGUI.5"); //$NON-NLS-1$
    private static String strLITTLE_ENDIAN = Language.getString("CalcGUI.6"); //$NON-NLS-1$
    private static String strBIG_ENDIAN = Language.getString("CalcGUI.7"); //$NON-NLS-1$
    private static String strDECIMAL = Language.getString("CalcGUI.8"); //$NON-NLS-1$
    private static String strHEX = Language.getString("CalcGUI.9"); //$NON-NLS-1$
    private static String strTYPE = Language.getString("CalcGUI.10"); //$NON-NLS-1$
    private static String errINVALID_NUMBER_STRING = Language.getString("CalcGUI.11"); //$NON-NLS-1$
    private static String errHEX_STRING_TOO_BIG = Language.getString("CalcGUI.12"); //$NON-NLS-1$
    
    public CalcGUI()
    {
        /*CREATE*/
        txtDEC = new JTextField(24);
        txtHEX = new JTextField(24);
        JButton btnDEC = new JButton(strCONVERT);
        JButton btnHEX = new JButton(strCONVERT);
        chkSHT = new JCheckBox(strTYPE_SHORT, true);
        chkINT = new JCheckBox(strTYPE_INTEGER);
        chkLNG = new JCheckBox(strTYPE_LONG);
        chkFLT = new JCheckBox(strTYPE_FLOAT);
        chkDBL = new JCheckBox(strTYPE_DOUBLE);
        chkLIT = new JCheckBox(strLITTLE_ENDIAN, true);
        chkBIG = new JCheckBox(strBIG_ENDIAN, false);
        
        /*SETUP*/
        Font def = UE.SETTINGS.getDefaultFont();
        txtDEC.setFont(def);
        txtHEX.setFont(def);
        btnDEC.setFont(def);
        btnHEX.setFont(def);
        chkINT.setFont(def);
        chkLNG.setFont(def);
        chkFLT.setFont(def);
        chkDBL.setFont(def);
        chkLIT.setFont(def);
        chkBIG.setFont(def);
        chkSHT.setFont(def);
        btgTYPE.add(chkSHT);
        btgTYPE.add(chkINT);
        btgTYPE.add(chkLNG);
        btgTYPE.add(chkFLT);
        btgTYPE.add(chkDBL);
        btgEND.add(chkLIT);
        btgEND.add(chkBIG);
        
        btnDEC.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                String hex;
                int len;
                
                try
                {
                    if (chkSHT.isSelected())
                    {
                        len = 2;
                        int num = Integer.parseInt(txtDEC.getText());

                        if (num < Short.MIN_VALUE || num > 65535)
                        {
                            UE.displayError(new Exception(errINVALID_NUMBER_STRING));
                            return;
                        }
                        else
                        {
                            if (chkLIT.isSelected())
                            {
                                num = Integer.reverseBytes(num);
                            }
                                
                            hex = Integer.toHexString(num);
                        }
                    }
                    else if (chkINT.isSelected())
                    {
                        len = 4;
                        int num = Integer.parseInt(txtDEC.getText());
                        
                        if (chkLIT.isSelected())
                        {
                            num = Integer.reverseBytes(num);
                        }
                        
                        hex = Integer.toHexString(num);
                    }
                    else if (chkLNG.isSelected())
                    {
                        len = 8;
                        long num = Long.parseLong(txtDEC.getText());
                        
                        if (chkLIT.isSelected())
                        {
                            num = Long.reverseBytes(num);
                        }
                        
                        hex = Long.toHexString(num);
                    }
                    else if (chkFLT.isSelected())
                    {
                        len = 4;
                        float flt = Float.parseFloat(txtDEC.getText());
                        
                        int num = Float.floatToRawIntBits(flt);
                        
                        if (chkLIT.isSelected())
                        {
                            num = Integer.reverseBytes(num);
                        }
                        
                        hex = Integer.toHexString(num);
                    }
                    else
                    {
                        len = 8;
                        double dbl = Double.parseDouble(txtDEC.getText());

                        long num = Double.doubleToRawLongBits(dbl);
                        
                        if (chkLIT.isSelected())
                        {
                            num = Long.reverseBytes(num);
                        }
                        
                        hex = Long.toHexString(num);
                    }

                    // fix formatting
                    if (hex.length() % 2 != 0)
                    {
                        hex = "0" + hex;
                    }
                    
                    while (hex.length() < Math.max(len, 4) * 2)
                    {
                        hex = "00" + hex;
                    }

                    while (hex.length() > len * 2)
                    {
                        if (chkLIT.isSelected()) // big numbers are on the right
                        {
                            hex = hex.substring(0, hex.length() - 2);
                        }
                        else
                        {
                            hex = hex.substring(2, hex.length());
                        }
                    }

                    for (int i = 0; i < len - 1; i++)
                    {
                        hex = hex.substring(0, (i+1)*2+ i) + " " + hex.substring((i+1)*2+ i);
                    }
                    
                    txtHEX.setText(hex.toUpperCase());
                }
                catch (Exception x)
                {
                    UE.displayError(new Exception(errINVALID_NUMBER_STRING));
                }
            }
        });

        btnHEX.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    int len = 8;
                    boolean negative;
                    
                    if (chkSHT.isSelected() || chkINT.isSelected() || chkFLT.isSelected())
                    {
                        len = 4;
                    }
                    
                    String result = txtHEX.getText().replace(" ", "");

                    while (result.length() < len * 2)
                    {
                        result = result + "00";
                    }
                    
                    if (chkLIT.isSelected())
                    {
                        for (int i = 0; i < result.length() - 2; i+=2)
                        {
                            result = result.substring(0, i) + result.substring(result.length() - 2, result.length()) + result.substring(i, result.length() - 2);
                        }
                    }
                    
                    String c = result.substring(0, 1);
                    negative = c.toUpperCase().equals("F") || c.toUpperCase().equals("E")
                    || c.toUpperCase().equals("D") || c.toUpperCase().equals("B") || c.toUpperCase().equals("A")
                    || c.toUpperCase().equals("9") || c.toUpperCase().equals("8");

                    result = "0x" + result;
                        
                    if (chkSHT.isSelected())
                    {
                        if (negative)
                        {
                            result = "0x7" + result.substring(3);
                        }
                        
                        int sh = Integer.decode(result);

                        if (negative)
                        {
                            sh = sh | 0x80000000 + Short.MAX_VALUE*2 + 1;
                        }
                        
                        if (sh < Short.MIN_VALUE || sh > 65535)
                        {
                            throw new Exception(errHEX_STRING_TOO_BIG);
                        }

                        int num = sh;
                        
                        if (num > 0x7FFF)
                        {
                            num = num - 65536;
                        }

                        if (num != sh)
                        {
                            result = Language.getString("CalcGUI.13"); //$NON-NLS-1$
                            result = result.replace("%1", Integer.toString(num)); //$NON-NLS-1$
                            result = result.replace("%2", Integer.toString(sh)); //$NON-NLS-1$
                        }
                        else
                        {
                            result = Integer.toString(sh);
                        }
                    }
                    else if (chkINT.isSelected())
                    {
                        if (negative)
                        {
                            result = "0x7" + result.substring(3);
                        }

                        int num = Integer.decode(result);

                        if (negative)
                        {
                            num = num | 0x80000000;
                        }
                        
                        result = Integer.toString(num);
                    }
                    else if (chkLNG.isSelected())
                    {
                        if (negative)
                        {
                            result = "0x7" + result.substring(3);
                        }

                        long num = Long.decode(result);
                        
                        if (negative)
                        {
                            num = num | 0x8000000000000000L;
                        }

                        result = Long.toString(num);
                    }
                    else if (chkFLT.isSelected())
                    {
                        if (negative)
                        {
                            result = "0x7" + result.substring(3);
                        }

                        int num = Integer.decode(result);

                        if (negative)
                        {
                            num = num | 0x80000000;
                        }
                        
                        result = Float.toString(Float.intBitsToFloat(num));
                    }
                    else
                    {
                        if (negative)
                        {
                            result = "0x7" + result.substring(3);
                        }

                        long num = Long.decode(result);
                        
                        if (negative)
                        {
                            num = num | 0x8000000000000000L;
                        }

                        result = Double.toString(Double.longBitsToDouble(num));
                    }

                    txtDEC.setText(result);
                }
                catch (Exception z)
                {
                    UE.displayError(z);
                }
            }
        });
		/*PLACE*/
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets=new Insets(5, 5, 5, 5);
		c.fill=GridBagConstraints.BOTH;
		c.gridheight=1;
		c.gridwidth=1;
		c.gridx=0;
		c.gridy=0;
		JLabel lbl = new JLabel(strDECIMAL);
		lbl.setFont(def);
		add(lbl, c);
		c.gridy=1;
		lbl = new JLabel(strTYPE);
		lbl.setFont(def);
		add(lbl, c);
		c.gridy=6;
		lbl = new JLabel(strHEX);
		lbl.setFont(def);
		add(lbl, c);
		c.gridx=1;
		c.gridy=0;
		add(txtDEC, c);
		c.gridy=1;
		add(chkSHT, c);
		c.gridy=2;
		add(chkINT, c);
		c.gridy=3;
		add(chkLNG, c);
		c.gridy=4;
		add(chkFLT, c);
		c.gridy=5;
		add(chkDBL, c);
		c.gridy=6;
		add(txtHEX, c);
		c.gridx=2;
		c.gridy=0;
		add(btnDEC, c);
		c.gridy=1;
		lbl = new JLabel(Language.getString("CalcGUI.14")); //$NON-NLS-1$
		lbl.setFont(def);
		add(lbl, c);
		c.gridy=2;
		add(chkLIT, c);
		c.gridy=3;
		add(chkBIG, c);
		c.gridy=6;
		add(btnHEX, c);
	}

	@Override
	public void finalWarning() throws Exception{}
}
