/**
 * Ultimate Editor (dev)
 * Copyright (C) 2009 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: ConvertAddressGUI.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 27.4.2009
 */
package ue.gui.special;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import ue.Language;
import ue.UE;
import ue.gui.MainPanel;
import ue.util.SettingsManager;

/**
 * @author Alan Podlesek
 */
@SuppressWarnings("serial") //$NON-NLS-1$
public class ConvertAddressGUI extends MainPanel
{
    private static final String strHexPrefix = "0x"; //$NON-NLS-1$

    private final String strConvert = Language.getString("ConvertAddressGUI.0"); //$NON-NLS-1$
    private final String strDecimalAddress = Language.getString("ConvertAddressGUI.1"); //$NON-NLS-1$
    private final String strHexAddress = Language.getString("ConvertAddressGUI.2"); //$NON-NLS-1$
    private final String strAsmAddress = Language.getString("ConvertAddressGUI.3"); //$NON-NLS-1$
    private final String strError = Language.getString("ConvertAddressGUI.4"); //$NON-NLS-1$
    private final String strOutOfRange = Language.getString("ConvertAddressGUI.5"); //$NON-NLS-1$
    
    private JTextField txtDec;
    private JTextField txtHex;
    private JTextField txtAsm;
    private JLabel lblDec;
    private JLabel lblHex;
    private JLabel lblAsm;
    private JButton btnDec = new JButton(strConvert);
    private JButton btnHex = new JButton(strConvert);
    private JButton btnAsm = new JButton(strConvert);

    public ConvertAddressGUI()
    {
        setupComponents();
        placeComponents();
    }
    
    private void setupComponents()
    {
        Font fnt = UE.SETTINGS.getDefaultFont();
        
        txtDec = new JTextField(14);
        txtHex = new JTextField(14);
        txtAsm = new JTextField(14);
        
        lblDec = new JLabel(strDecimalAddress);
        lblHex = new JLabel(strHexAddress);
        lblAsm = new JLabel(strAsmAddress);

        txtDec.setFont(fnt);
        txtDec.setMaximumSize(new Dimension(txtDec.getPreferredSize().width, 100));
        txtHex.setFont(fnt);
        txtHex.setMaximumSize(new Dimension(txtHex.getPreferredSize().width, 100));
        txtAsm.setFont(fnt);
        txtAsm.setMaximumSize(new Dimension(txtAsm.getPreferredSize().width, 100));

        btnDec.setFont(fnt);
        btnDec.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    long dec = Long.parseLong(txtDec.getText());
                    
                    txtHex.setText(strHexPrefix + Long.toHexString(dec).toUpperCase());
                    
                    if (dec >= 1024 && dec <= 1518080)
                    {
                        dec += 4197376;
                    }
                    else if (dec >= 1523200 && dec <= 1700864)
                    {
                        dec += 4203008;
                    }
                    else if (dec >= 1700864 && dec <= 1781248)
                    {
                        dec += 5155840;
                    }
                    else
                    {
                        txtAsm.setText(strOutOfRange);
                        return;
                    }

                    txtAsm.setText(Long.toHexString(dec));
                }
                catch (Exception x)
                {
                    txtHex.setText(strError);
                    txtAsm.setText(strError);
                }
            }
        });
        
        btnHex.setFont(fnt);
        btnHex.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    String txt = txtHex.getText();
                    
                    if (!txt.startsWith(strHexPrefix))
                    {
                        txt = strHexPrefix + txt;
                    }
                    
                    long dec = Long.decode(txt);
                    
                    txtDec.setText(Long.toString(dec));
                    
                    if (dec >= 1024 && dec <= 1518080)
                    {
                        dec += 4197376;
                    }
                    else if (dec >= 1523200 && dec <= 1700864)
                    {
                        dec += 4203008;
                    }
                    else if (dec >= 1700864 && dec <= 1781248)
                    {
                        dec += 5155840;
                    }
                    else
                    {
                        txtAsm.setText(strOutOfRange);
                        return;
                    }

                    txtAsm.setText(Long.toHexString(dec));
                }
                catch (Exception x)
                {
                    txtDec.setText(strError);
                    txtAsm.setText(strError);
                }
            }
        });

        btnAsm.setFont(fnt);
        btnAsm.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    String txt = txtAsm.getText();
                    
                    if (!txt.startsWith(strHexPrefix))
                    {
                        txt = strHexPrefix + txt;
                    }
                    
                    long dec = Long.decode(txt);
                    
                    if (dec >= 4198400 && dec <= 5715456)
                    {
                        dec -= 4197376;
                    }
                    else if (dec >= 5726208 && dec <= 5903872)
                    {
                        dec -= 4203008;
                    }
                    else if (dec >= 6856704 && dec <= 6937088)
                    {
                        dec -= 5155840;
                    }
                    else
                    {
                        txtDec.setText(strOutOfRange);
                        txtHex.setText(strOutOfRange);
                        return;
                    }

                    txtDec.setText(Long.toString(dec));
                    
                    txtHex.setText(strHexPrefix + Long.toHexString(dec));
                }
                catch (Exception x)
                {
                    txtDec.setText(strError);
                    txtHex.setText(strError);
                }
            }
        });

        lblDec.setFont(fnt);
        lblHex.setFont(fnt);
        lblAsm.setFont(fnt);
    }
    
    private void placeComponents()
    {
        Dimension dim = new Dimension(5, 5);
        
        setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

        JPanel all = new JPanel();
        all.setLayout(new BoxLayout(all, BoxLayout.Y_AXIS));

        all.add(Box.createVerticalStrut(5));
        
        JPanel pnl = new JPanel();
        pnl.setLayout(new BoxLayout(pnl, BoxLayout.X_AXIS));

        pnl.add(Box.createHorizontalGlue());
        pnl.add(Box.createRigidArea(dim));
        pnl.add(lblDec);
        pnl.add(Box.createRigidArea(dim));
        pnl.add(txtDec);
        pnl.add(Box.createRigidArea(dim));
        pnl.add(btnDec);
        pnl.add(Box.createRigidArea(dim));

        all.add(pnl);
        all.add(Box.createRigidArea(dim));
        
        pnl = new JPanel();
        pnl.setLayout(new BoxLayout(pnl, BoxLayout.X_AXIS));

        pnl.add(Box.createHorizontalGlue());
        pnl.add(Box.createRigidArea(dim));
        pnl.add(lblHex);
        pnl.add(Box.createRigidArea(dim));
        pnl.add(txtHex);
        pnl.add(Box.createRigidArea(dim));
        pnl.add(btnHex);
        pnl.add(Box.createRigidArea(dim));

        all.add(pnl);
        all.add(Box.createRigidArea(dim));

        pnl = new JPanel();
        pnl.setLayout(new BoxLayout(pnl, BoxLayout.X_AXIS));

        pnl.add(Box.createHorizontalGlue());
        pnl.add(Box.createRigidArea(dim));
        pnl.add(lblAsm);
        pnl.add(Box.createRigidArea(dim));
        pnl.add(txtAsm);
        pnl.add(Box.createRigidArea(dim));
        pnl.add(btnAsm);
        pnl.add(Box.createRigidArea(dim));

        all.add(pnl);
        all.add(Box.createVerticalGlue());
        
        add(all);
        add(Box.createHorizontalGlue());
    }
    
    /* (non-Javadoc)
     * @see ue.gui.MainPanel#finalWarning()
     */
    @Override
    public void finalWarning() throws Exception
    {
        // nothing to do
    }
}
