/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: Image2TgaGUI.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui.special;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.ArrayList;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;

import ue.Language;
import ue.UE;
import ue.exception.InvalidArgumentsException;
import ue.gui.MainPanel;
import ue.util.SettingsManager;
import ue.util.TargaImage;

/**Interface for converting images to tga.*/
@SuppressWarnings("serial")  //$NON-NLS-1$
public class Image2TgaGUI extends MainPanel implements ActionListener
{
    //add, remove, clear, convert, browse
    private JButton[] BTN;
    private JList LIST;
    private JTextField PATH;
    private JLabel LBL = new JLabel(Language.getString("Image2TgaGUI.0")); //$NON-NLS-1$
    //data
    private Vector<String> NAMES = new Vector<String>();
    private ArrayList<File> FULLPATHS = new ArrayList<File>();
    private boolean dest_locked = false;
    // strings
    private static String strADD = Language.getString("Image2TgaGUI.1"); //$NON-NLS-1$
    private static String strREMOVE = Language.getString("Image2TgaGUI.2"); //$NON-NLS-1$
    private static String strCLEAR = Language.getString("Image2TgaGUI.3"); //$NON-NLS-1$
    private static String strCONVERT = Language.getString("Image2TgaGUI.4"); //$NON-NLS-1$
    private static String strBROWSE = Language.getString("Image2TgaGUI.5"); //$NON-NLS-1$

    public Image2TgaGUI()
    {
        BTN = new JButton[5];
        BTN[0] = new JButton(strADD);
        BTN[1] = new JButton(strREMOVE);
        BTN[2] = new JButton(strCLEAR);
        BTN[3] = new JButton(strCONVERT);
        BTN[4] = new JButton(strBROWSE);
        LIST = new JList();
        PATH = new JTextField(10);
        //font
        Font def = UE.SETTINGS.getDefaultFont();
        BTN[0].setFont(def);
        BTN[1].setFont(def);
        BTN[2].setFont(def);
        BTN[3].setFont(def);
        BTN[4].setFont(def);
        LIST.setFont(def);
        PATH.setFont(def);
        LBL.setFont(def);
        //set other things
        LIST.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        BTN[1].setEnabled(false);
        BTN[2].setEnabled(false);
        BTN[3].setEnabled(false);
        PATH.setText(UE.SETTINGS.getProperty(SettingsManager.WORK_PATH)); //$NON-NLS-1$
        //action
        for (int i = 0; i < 5; i++){ BTN[i].addActionListener(this);}
        //lay
        setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.insets=new Insets(5, 5, 5, 5);
        c.fill=GridBagConstraints.BOTH;
        //list
        c.gridheight=5;
        c.gridwidth=1;
        c.gridx=0;
        c.gridy=0;
        JScrollPane scrolly = new JScrollPane(LIST);
        scrolly.setPreferredSize(new Dimension(160, 160));
        add(scrolly, c);
        //buttons
        c.gridheight=1;
        c.gridx=1;
        add(BTN[0], c);
        c.gridy=1;
        add(BTN[1], c);
        c.gridy=2;
        add(BTN[2], c);
        c.gridy=3;
        add(BTN[3], c);
        c.gridy=6;
        add(BTN[4], c);
        //label
        c.gridy=5;
        c.gridx=0;
        add(LBL, c);
        //textfield
        c.gridy=6;
        add(PATH, c);
    }
    
    @Override
    public void actionPerformed(ActionEvent ae)
    {
        if ((ae.getActionCommand()).equals(strADD))
        {
            JFileChooser jfc = new JFileChooser(UE.SETTINGS.getProperty(SettingsManager.WORK_PATH));
            javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter()
            {
                @Override
                public boolean accept(File f)
                {
                    if (f.isDirectory())
                    {
                        return true;
                    }
                    String name = f.getName();
                    name=name.toLowerCase();
	            
                    if (name.endsWith(".tga") || name.endsWith(".jpg") //$NON-NLS-1$ //$NON-NLS-2$
                            || name.endsWith(".jpeg") || name.endsWith(".png") //$NON-NLS-1$ //$NON-NLS-2$
                            || name.endsWith(".gif") || name.endsWith(".bmp")) //$NON-NLS-1$
                    {
                        return true; //$NON-NLS-1$
                    }
                    else 
                    {
                        return false;
                    }
                }
                
                @Override
                public String getDescription()
                {
                    return Language.getString("Image2TgaGUI.6") + " (*.tga, *.jpg/*.jpeg, *.gif, *.png, *.bmp)"; //$NON-NLS-1$ //$NON-NLS-2$
                }
            };
            jfc.setFileFilter(filter);
            jfc.setDialogTitle(Language.getString("Image2TgaGUI.7")); //$NON-NLS-1$
            jfc.setFileSelectionMode(JFileChooser.FILES_ONLY);
            jfc.setMultiSelectionEnabled(true);
            int returnVal = jfc.showDialog(this, strADD);
            
            if(returnVal == JFileChooser.APPROVE_OPTION)
            {
                //open and get options
                File[] fil = jfc.getSelectedFiles();
                
                File last_file = fil[0];
                
                if (last_file.isFile())
                {
                    last_file = last_file.getParentFile();
                }
                
                UE.SETTINGS.setProperty(SettingsManager.WORK_PATH, last_file.getAbsolutePath());
                if (!dest_locked) PATH.setText(last_file.getAbsolutePath());
                
                for (int i=0; i<fil.length; i++)
                {
                    NAMES.add(fil[i].getName());
                    FULLPATHS.add(fil[i]);
                }
                LIST.setListData(NAMES);
                BTN[1].setEnabled(true);
                BTN[2].setEnabled(true);
                BTN[3].setEnabled(true);
            }
        }
        else if ((ae.getActionCommand()).equals(strREMOVE))
        {
            if (LIST.getSelectedIndex() >= 0)
            {
                Object[] ut = LIST.getSelectedValues();
                
                for (int i = 0; i < ut.length; i++)
                {
                    int hin = NAMES.indexOf(ut[i]);
                    NAMES.remove(hin);
                    FULLPATHS.remove(hin);
                }
                if (NAMES.isEmpty())
                {
                    BTN[1].setEnabled(false);
                    BTN[2].setEnabled(false);
                    BTN[3].setEnabled(false);
                    LIST.setModel(new DefaultListModel());
                }
                else
                {
                    BTN[1].setEnabled(true);
                    BTN[2].setEnabled(true);
                    BTN[3].setEnabled(true);
                    LIST.setListData(NAMES);
                }
            }
        }
        else if ((ae.getActionCommand()).equals(strCLEAR))
        {
            NAMES = new Vector<String>();
            FULLPATHS = new ArrayList<File>();
            BTN[1].setEnabled(false);
            BTN[2].setEnabled(false);
            BTN[3].setEnabled(false);
            LIST.setModel(new DefaultListModel());
            dest_locked = false;
        }
        else if ((ae.getActionCommand()).equals(strBROWSE))
        {
            JFileChooser fc = new JFileChooser();
            fc.setCurrentDirectory(new File(PATH.getText()));
            fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
            fc.setMultiSelectionEnabled(false);
            fc.setDialogTitle(Language.getString("Image2TgaGUI.8")); //$NON-NLS-1$
            int returnVal=fc.showDialog(UE.WINDOW, Language.getString("Image2TgaGUI.9")); //$NON-NLS-1$
            
            if(returnVal == JFileChooser.APPROVE_OPTION)
            {
                File fil = fc.getSelectedFile();
		
                if (fil!=null)
                {
                    dest_locked = true;
                    PATH.setText(fil.getPath());
                }
            }
        }
        else if ((ae.getActionCommand()).equals(strCONVERT))
        {
            if (!FULLPATHS.isEmpty())
            {
                int j = FULLPATHS.size();
                String name;
                String sub;
                File path;
                TargaImage targa;
                
                for (int i = 0; i < j; i++)
                {
                    //read
                    targa = null;
                    name = NAMES.get(i);
                    path = FULLPATHS.get(i);
                    sub = new String(name);
                    sub.toLowerCase();
			
                    try
                    {
                        if (sub.endsWith(".tga")){ //$NON-NLS-1$
                            targa = new TargaImage(path);
                            if (targa == null)
                            {
                                String msg = Language.getString("Image2TgaGUI.10"); //$NON-NLS-1$
                                msg = msg.replace("%1", name); //$NON-NLS-1$
                                UE.displayError(new InvalidArgumentsException(msg));
                                continue;
                            }
                        }
                        else
                        {
                            BufferedImage bi = ImageIO.read(path);
                            if (bi == null)
                            {
                                String msg = Language.getString("Image2TgaGUI.10"); //$NON-NLS-1$
                                msg = msg.replace("%1", name); //$NON-NLS-1$
                                UE.displayError(new InvalidArgumentsException(msg));
                                continue;
                            }
                            else
                            {
                                // Get all the pixels
                                int w = bi.getWidth(null);
                                int h = bi.getHeight(null);
                                int[] rgbs = new int[w*h];
                                bi.getRGB(0, 0, w, h, rgbs, 0, w);	
                                targa = new TargaImage(rgbs, w, h);
                            }
                        }
                        //save
                        if (targa == null)
                        {
                            String msg = Language.getString("Image2TgaGUI.10"); //$NON-NLS-1$
                            msg = msg.replace("%1", name); //$NON-NLS-1$
                            UE.displayError(new InvalidArgumentsException(msg));
                            continue;
                        }
                        else
                        {
                            int ind = name.indexOf("."); //$NON-NLS-1$
                            if (ind > 0)
                            {
                                name = name.substring(0, ind);
                            }
                            name = name + ".tga"; //$NON-NLS-1$
                            File g = new File(PATH.getText(), name);
                            if (g.exists())
                            {
                                g.delete();
                            }
                            targa.save16(g);
                            //	clean
                            if (!g.equals(path))
                            {
                                path.delete();
                            }
                        }
                    }
                    catch (Exception ef)
                    {
                        UE.displayError(ef);
                    }
                }
                NAMES = new Vector<String>();
                FULLPATHS = new ArrayList<File>();
                BTN[1].setEnabled(false);
                BTN[2].setEnabled(false);
                BTN[3].setEnabled(false);
                LIST.setModel(new DefaultListModel());
                dest_locked = false;
            }
        }
    }
    
    @Override
    public void finalWarning() throws Exception{}
}
