/**
 * Ultimate Editor
 * Copyright (C) 2008 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: AIBldReqGUI.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 10.11.2008
 */
package ue.gui.stbof;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.InputVerifier;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import ue.Language;
import ue.UE;
import ue.edit.CanBeEdited;
import ue.edit.FileManager;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.bin.AIBldReq;
import ue.edit.res.stbof.bst.Edifice;
import ue.exception.InvalidArgumentsException;
import ue.gui.MainPanel;
import ue.util.ID;

/**
 * Allows editing aibldreq.bin
 */
@SuppressWarnings("serial") //$NON-NLS-1$
public class AIBldReqGUI extends MainPanel
{
    private JButton btnMoveUp;
    private JButton btnMoveDown;
    private JButton btnAdd;
    private JButton btnRemove;
    private JLabel[] lblDesc;
    private JLabel lblName;
    private JLabel lblID;
    private JTextField[] txtReq;
    private JList lstBld;
    
    private AIBldReq AIBLDREQ;
    private Edifice EDIFICE = null;
    
    private ID SELECTED = null;
    
    public AIBldReqGUI(AIBldReq aibldreq)
    {
        AIBLDREQ = aibldreq;
    
        CanBeEdited file;
        
        if ((file = UE.FILES.getFile(FileManager.P_STBOF)) != null)
        {
            try
            {
                EDIFICE = (Edifice)file.getInternalFile(Stbof.fileEDIFICE_BST, true);
            }
            catch (Exception e)
            {
                // do nothing
            }
        }
        
        setupComponents();
        placeComponents();
        
        fillList();
    }
    
    private void setupComponents()
    {
        Font def = UE.SETTINGS.getDefaultFont();
        
        // moving
        btnMoveUp = new JButton(Language.getString("AIBldReqGUI.0")); //$NON-NLS-1$
        btnMoveUp.setEnabled(false);
        btnMoveUp.setFont(def);
        btnMoveUp.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                if (SELECTED != null)
                {
                    try
                    {
                        AIBLDREQ.moveUp(SELECTED.hashCode());
                    }
                    catch (InvalidArgumentsException e1)
                    {
                        UE.displayError(e1);
                    }
                    fillList();
                }
            }
        });
        
        btnMoveDown = new JButton(Language.getString("AIBldReqGUI.1")); //$NON-NLS-1$
        btnMoveDown.setEnabled(false);
        btnMoveDown.setFont(def);
        btnMoveDown.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                if (SELECTED != null)
                {
                    try
                    {
                        finalWarning();
                    }
                    catch (Exception ex)
                    {
                        // really bad stuff happening - bail
                        UE.displayError(ex);
                        return;
                    }
                    try
                    {
                        AIBLDREQ.moveDown(SELECTED.hashCode());
                    }
                    catch (InvalidArgumentsException e1)
                    {
                        UE.displayError(e1);
                    }
                    fillList();
                }
            }
        });
        
        // add building
        btnAdd = new JButton(Language.getString("AIBldReqGUI.2")); //$NON-NLS-1$
        btnAdd.setEnabled(EDIFICE != null);
        btnAdd.setFont(def);
        btnAdd.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    finalWarning();
                }
                catch (Exception ex)
                {
                    // really bad stuff happening - bail
                    UE.displayError(ex);
                    return;
                }
                if (EDIFICE != null)
                {
                    ID[] ids = EDIFICE.getShortIDs(Edifice.MASK_ALL, 20);
                    
                    ArrayList<ID> l = new ArrayList<ID>(); 
                    
                    for (int i = 0; i < ids.length; i++)
                    {
                        try
                        {
                            if (!EDIFICE.isMainBuilding(ids[i].hashCode())
                                    && !AIBLDREQ.contains(ids[i].hashCode())
//                                    && EDIFICE.getBoniType(ids[i].hashCode()) != Edifice.BONUS_BUILDS_SHIPS
                                    && EDIFICE.getBoniType(ids[i].hashCode()) != Edifice.BONUS_MORALE_MARTIAL_LAW)
                            {
                                l.add(ids[i]);
                            }
                        }
                        catch (InvalidArgumentsException e1)
                        {
                            UE.displayError(e1);
                        }
                    }
                    
                    Object ret = JOptionPane.showInputDialog(UE.WINDOW,
                            Language.getString("AIBldReqGUI.3"), //$NON-NLS-1$
                            Language.getString("AIBldReqGUI.4"), //$NON-NLS-1$
                            JOptionPane.PLAIN_MESSAGE,
                            null,
                            l.toArray(),
                            null);
                    
                    if (ret != null)
                    {
                        AIBLDREQ.add(ret.hashCode());
                        
                        fillList();
                        lstBld.setSelectedValue(ret, true);
                    }
                }
            }
        });
        
        // remove
        btnRemove = new JButton(Language.getString("AIBldReqGUI.5")); //$NON-NLS-1$
        btnRemove.setEnabled(false);
        btnRemove.setFont(def);
        btnRemove.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                if (SELECTED != null)
                {
                    AIBLDREQ.remove(SELECTED.hashCode());
                    fillList();
                }
            }
        });

        // labels
        lblDesc = new JLabel[5];
        lblDesc[0] = new JLabel(Language.getString("AIBldReqGUI.6")); //$NON-NLS-1$
        lblDesc[1] = new JLabel(Language.getString("AIBldReqGUI.7")); //$NON-NLS-1$
        lblDesc[2] = new JLabel(Language.getString("AIBldReqGUI.8")); //$NON-NLS-1$
        lblDesc[3] = new JLabel(Language.getString("AIBldReqGUI.9")); //$NON-NLS-1$
        lblDesc[4] = new JLabel(Language.getString("AIBldReqGUI.10")); //$NON-NLS-1$
        for (JLabel lbl : lblDesc)
        {
            lbl.setFont(def);
        }
        
        lblName = new JLabel();
        lblName.setFont(def);

        lblID = new JLabel();
        lblID.setFont(def);

        // textfields
        // labels
        txtReq = new JTextField[3];
        txtReq[0] = new JTextField(5);
        txtReq[1] = new JTextField(5);
        txtReq[2] = new JTextField(5);
        
        // makes sure entered values are numbers equal or higher than 0
        InputVerifier iv = new InputVerifier()
        {
            @Override
            public boolean verify(JComponent input)
            {
                if (input instanceof JTextField)
                {
                    String str = ((JTextField)input).getText();
                    
                    try
                    {
                        int val = Integer.parseInt(str);
                        
                        return val >= 0;
                    }
                    catch (Exception e)
                    {
                        // do nothing
                    }
                }
                return false;
            }
        };
        
        for (JTextField txt : txtReq)
        {
            txt.setFont(def);
            txt.setInputVerifier(iv);
        }
        
        // list
        lstBld = new JList();
        lstBld.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lstBld.setFont(def);
        lstBld.addListSelectionListener(new ListSelectionListener()
        {
            @Override
            public void valueChanged(ListSelectionEvent e)
            {
                if (SELECTED != null)
                {
                    try
                    {
                        finalWarning();
                    }
                    catch (Exception ex)
                    {
                        // really bad stuff happening - bail
                        UE.displayError(ex);
                        return;
                    }
                }
                
                if (lstBld.getSelectedIndex() < 0)
                {
                    SELECTED = null;

                    // disable everything
                    btnMoveUp.setEnabled(false);
                    btnMoveDown.setEnabled(false);
                    btnRemove.setEnabled(false);
                    lblName.setText(null);
                    lblID.setText(null);
                    for (JTextField txt : txtReq)
                    {
                        txt.setEnabled(false);
                        txt.setText(null);
                    }
                }
                else
                {
                    ID id = (ID)lstBld.getSelectedValue();
                        
                    // enable and fille everything
                    btnMoveUp.setEnabled(true);
                    btnMoveDown.setEnabled(true);
                    btnAdd.setEnabled(EDIFICE != null);
                    btnRemove.setEnabled(true);
                    
                    if (EDIFICE != null)
                    {
                        try
                        {
                            lblName.setText(EDIFICE.getName(id.hashCode()));
                        }
                        catch (InvalidArgumentsException e1)
                        {
                            UE.displayError(e1);
                            
                            lblName.setText(Language.getString("AIBldReqGUI.11")); // N/A //$NON-NLS-1$
                        }
                    }
                    else
                    {
                        lblName.setText(Language.getString("AIBldReqGUI.11")); // N/A //$NON-NLS-1$
                    }
                    
                    lblID.setText(Integer.toString(id.hashCode()));

                    for (JTextField txt : txtReq)
                    {
                        txt.setEnabled(true);
                    }
                    
                    try
                    {
                        txtReq[0].setText(Integer.toString(AIBLDREQ.getEnergyReq(id.hashCode())));
                    }
                    catch (InvalidArgumentsException e1)
                    {
                        UE.displayError(e1);
                        txtReq[0].setText("0"); //$NON-NLS-1$
                    }
                    
                    try
                    {
                        txtReq[1].setText(Integer.toString(AIBLDREQ.getPopReq(id.hashCode())));
                    }
                    catch (InvalidArgumentsException e1)
                    {
                        UE.displayError(e1);
                        txtReq[1].setText("0"); //$NON-NLS-1$
                    }

                    try
                    {
                        txtReq[2].setText(Integer.toString(AIBLDREQ.getMaxPopReq(id.hashCode())));
                    }
                    catch (InvalidArgumentsException e1)
                    {
                        UE.displayError(e1);
                        txtReq[2].setText("0"); //$NON-NLS-1$
                    }

                    SELECTED = id;
                }
            }
        });
    }

    private void placeComponents()
    {
        JPanel pnlEdit = new JPanel(new GridLayout(5, 2, 10, 10));
        pnlEdit.setAlignmentY(JComponent.TOP_ALIGNMENT);
        
        pnlEdit.add(lblDesc[0]);
        pnlEdit.add(lblName);
        pnlEdit.add(lblDesc[1]);
        pnlEdit.add(lblID);
        pnlEdit.add(lblDesc[2]);
        pnlEdit.add(txtReq[0]);
        pnlEdit.add(lblDesc[3]);
        pnlEdit.add(txtReq[1]);
        pnlEdit.add(lblDesc[4]);
        pnlEdit.add(txtReq[2]);
        
        JPanel pnlList = new JPanel(new GridBagLayout());
        pnlList.setAlignmentY(JComponent.TOP_ALIGNMENT);

        GridBagConstraints c = new GridBagConstraints();
        c.insets=new Insets(5, 5, 5, 5);
        c.fill=GridBagConstraints.BOTH;
        c.anchor=GridBagConstraints.NORTH;      
        c.gridwidth = 1;
        c.gridheight = 5;
        c.gridx = 0;
        c.gridy = 0;
        JScrollPane scrollPane = new JScrollPane(lstBld);
        pnlList.add(scrollPane, c);
        c.gridheight = 1;
        c.gridx = 1;
        c.gridy = 0;
        pnlList.add(btnAdd, c);
        c.gridy = 1;
        pnlList.add(btnRemove, c);
        c.gridy = 2;
        pnlList.add(btnMoveUp, c);
        c.gridy = 3;
        pnlList.add(btnMoveDown, c);

        setLayout(new GridBagLayout());
        c.gridwidth = 1;
        c.gridheight = 1;
        c.gridx = 0;
        c.gridy = 0;
        add(pnlEdit, c);
        c.gridheight = 2;
        c.gridx = 1;
        add(pnlList, c);
    }

    private void fillList()
    {
        ID id = SELECTED;
        SELECTED = null;
        
        Vector<ID> l = new Vector<ID>();

        if (EDIFICE != null)
        {
            String[] names = EDIFICE.getShortNames();
            int[] hm = AIBLDREQ.getIDList();
            
            for (int i = 0; i < hm.length; i++)
            {
                if (hm[i] >= 0 && hm[i] < names.length)
                {
                    l.add(new ID(names[hm[i]], hm[i]));
                }
            }
            
        }
        else
        {
            int[] ids = AIBLDREQ.getIDList();
            
            for (int i = 0; i < ids.length; i++)
            {
                l.add(new ID(Integer.toString(ids[i]), ids[i]));
            }
        }

        lstBld.setListData(l);
        
        if (id != null)
        {
            lstBld.setSelectedValue(id, true);
        }
        else
        {
            lstBld.setSelectedValue(new ID("", 0), true); //$NON-NLS-1$
        }
    }
    
    /* (non-Javadoc)
     * @see ue.gui.MainPanel#finalWarning()
     */
    @Override
    public void finalWarning() throws Exception
    {
        if (SELECTED == null) {
            return;
        }
        
        int val = Integer.parseInt(txtReq[0].getText());
            
        AIBLDREQ.setEnergyReq(SELECTED.hashCode(), val);
        
        val = Integer.parseInt(txtReq[1].getText());

        AIBLDREQ.setPopReq(SELECTED.hashCode(), val);

        val = Integer.parseInt(txtReq[2].getText());
            
        AIBLDREQ.setMaxPopReq(SELECTED.hashCode(), val);
    }
    
}
