/*
Ultimate Editor for Birth Of The Federation 1.02
Copyright (C) 2005-2007  Alan Podlesek

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


File: AniGUI.java

Written by: Alan Podlesek <seqdcer@yahoo.com>

Last modified: 6.9.2007
 */
package ue.gui.stbof;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.ListSelectionModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import ue.Language;
import ue.UE;
import ue.edit.res.Stbof;
import ue.gui.AniLabel;
import ue.gui.MainPanel;
import ue.util.AniOrCur;
import ue.util.SettingsManager;
import ue.util.TargaImage;
import ue.util.Tools;
import ue.util.Tools;
import ue.util.UEWorker;

@SuppressWarnings("serial")  //$NON-NLS-1$
public class AniGUI extends MainPanel {

    private Stbof STBOF;
    //0-1 tip, 2-8 stat names, 9-14 stats
    private JLabel[] LABELS = new JLabel[15];
    private JList LIST = new JList();
    private AniLabel ANI = new AniLabel();
    private JSpinner SPEED;
    private JButton btnEXT;
    private AniOrCur AOC = null;
    private JCheckBox chkALPHA;

    public AniGUI(Stbof st)
    {
        STBOF = st;

        //create labels
        LABELS[0] = new JLabel(Language.getString("AniGUI.0")); //$NON-NLS-1$
        LABELS[1] = new JLabel(Language.getString("AniGUI.1")); //$NON-NLS-1$
        LABELS[2] = new JLabel(Language.getString("AniGUI.2")); //$NON-NLS-1$
        LABELS[3] = new JLabel(Language.getString("AniGUI.3")); //$NON-NLS-1$
        LABELS[4] = new JLabel(Language.getString("AniGUI.4")); //$NON-NLS-1$
        LABELS[5] = new JLabel(Language.getString("AniGUI.5")); //$NON-NLS-1$
        LABELS[6] = new JLabel(Language.getString("AniGUI.6")); //$NON-NLS-1$
        LABELS[7] = new JLabel(Language.getString("AniGUI.7")); //$NON-NLS-1$
        LABELS[8] = new JLabel(Language.getString("AniGUI.8")); //$NON-NLS-1$
        
        for (int i = 9; i <= 14; i++)
        {
            LABELS[i] = new JLabel();
            LABELS[i].setHorizontalAlignment(SwingConstants.RIGHT);
        }
        btnEXT = new JButton(Language.getString("AniGUI.9")); //$NON-NLS-1$

        //create textfield
        SPEED = new JSpinner(new SpinnerNumberModel(0, 0, Short.MAX_VALUE, 1));
        SPEED.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent e)
            {
                if (AOC != null)
                {
                    short speed = (short) ((Integer) SPEED.getValue()).intValue();

                    if (AOC.setDelay(speed))
                    {
                        STBOF.addInternalFile(AOC);
                    }

                    ANI.setAnimation(AOC, chkALPHA.isSelected());
                }
            }
        });
        //set font
        Font def = UE.SETTINGS.getDefaultFont();
        for (int i = 0; i <= 14; i++) {
            LABELS[i].setFont(def);
        }
        LIST.setFont(def);
        ANI.setFont(def);
        SPEED.setFont(def);
        btnEXT.setFont(def);
        //set other things
        LIST.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        ANI.setHorizontalAlignment(SwingConstants.CENTER);

        // use alpha
        chkALPHA = new JCheckBox("Show transparency", true);
        chkALPHA.setFont(def);
        chkALPHA.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                int sel = LIST.getSelectedIndex();
                LIST.clearSelection();

                if (sel >= 0)
                {
                    LIST.setSelectedIndex(sel);
                }
            }
        });

        //fill list
        fillList();
        //action listeners
        LIST.addListSelectionListener(new ListSelectionListener()
        {
            @Override
            public void valueChanged(ListSelectionEvent e)
            {
                if (LIST.getSelectedIndex() >= 0)
                {
                    try
                    {
                        finalWarning();
                    }
                    catch (Exception ex)
                    {
                        // really bad stuff happening - bail
                        UE.displayError(ex);
                        return;
                    }

                    try {
                        btnEXT.setEnabled(true);
                        String d = (String) LIST.getSelectedValue();
                        AOC = STBOF.getAnimation(d, false);

                        Dimension dim = AOC.getFrameDim();
                        LABELS[9].setText(Double.toString(dim.getWidth()));
                        LABELS[10].setText(Double.toString(dim.getHeight()));
                        LABELS[11].setText(Short.toString(AOC.getNumFrames()));
                        SPEED.setValue((int)(AOC.getDelay()));
                        dim = AOC.getTgaDim();
                        LABELS[12].setText(Double.toString(dim.getWidth()));
                        LABELS[13].setText(Double.toString(dim.getHeight()));
                        LABELS[14].setText(Integer.toString(AOC.getTgaSize()));

                        if (AOC.isCursor())
                        {
                            SPEED.setEnabled(true);
                        } else {
                            SPEED.setEnabled(false);
                        }
                        ANI.setAnimation(AOC, chkALPHA.isSelected());
                    }
                    catch (FileNotFoundException fnfe) {
                        btnEXT.setEnabled(false);
                        ANI.setAnimation(null, false);
                        UE.displayError(fnfe);
                    }
                }
            }
        });

        btnEXT.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent ae)
            {
                if (LIST.getSelectedIndex() >= 0)
                {
                    btnEXT.setEnabled(false);

                    final String name = (String) LIST.getSelectedValue();

                    //select path
                    JFileChooser fc = new JFileChooser();
                    fc.setCurrentDirectory(new File(UE.SETTINGS.getProperty(SettingsManager.WORK_PATH)));
                    fc.setDialogTitle(Language.getString("AniGUI.10")); //$NON-NLS-1$
                    fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                    fc.setMultiSelectionEnabled(false);
                    int returnVal = fc.showDialog(UE.WINDOW, Language.getString("AniGUI.11")); //$NON-NLS-1$

                    if (returnVal == JFileChooser.APPROVE_OPTION)
                    {
                        final File path = fc.getSelectedFile();
                        UE.SETTINGS.setProperty(SettingsManager.WORK_PATH, path.getPath());

                        UEWorker<Void> worker = new UEWorker<Void>()
                        {
                            @Override
                            protected Void doInBackground() throws Exception
                            {
                                int num = AOC.getNumFrames();
                                int i = -1;
                                String padding = ".000";
                                File dest;
                                TargaImage targ;

                                try
                                {
                                    this.setProgressVisible(true);
                                    this.setProgress(0);
                                    this.updateStatus(UEWorker.STATUS_WORKING);

                                    for (int j = 0; j < num; j++)
                                    {
                                        i++;

                                        // fix padding
                                        while (padding.length() > 1 && (padding + i).length() > 5)
                                        {
                                            padding = padding.substring(0, padding.length() - 1);
                                        }

                                        // construct destination file
                                        dest = new File(path, name + padding + i + ".tga"); //$NON-NLS-1$

                                        // get frame
                                        targ = AOC.getTgaFrame(j);

                                        // check file existance and fix filename if need be
                                        while (dest.exists())
                                        {
                                            i++;

                                            while (padding.length() > 1 && (padding + i).length() > 5)
                                            {
                                                padding = padding.substring(0, padding.length() - 1);
                                            }

                                            dest = new File(path, name + padding + i + ".tga"); //$NON-NLS-1$
                                        }

                                        this.setProgress((100 * j) / num);
                                        this.sendMessage("Extracting %1... %2%"
                                                .replace("%1", dest.getName())
                                                .replace("%2", Integer.toString(this.getProgress()))
                                                );
                                        
                                        // write
                                        targ.save16(dest);

                                    }
                                }
                                catch (Exception tr)
                                {
                                    //this should not happen
                                    this.reportError(tr);
                                    return null;
                                }

                                this.updateStatus(UEWorker.STATUS_DONE);

                                return null;
                            }
                        };

                        Tools.runUEWorker(worker);
                    }

                    btnEXT.setEnabled(true);
                }
            }
        });
        //lay
        setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.insets = new Insets(5, 5, 5, 5);
        c.fill = GridBagConstraints.BOTH;
        c.anchor = GridBagConstraints.NORTH;
        //labels
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        add(LABELS[0], c);
        c.gridwidth = 2;
        c.gridx = 1;
        add(LABELS[1], c);
        c.gridwidth = 1;
        c.gridy = 1;
        add(LABELS[2], c);
        c.gridy = 2;
        add(LABELS[3], c);
        c.gridy = 3;
        add(LABELS[4], c);
        c.gridy = 4;
        add(LABELS[5], c);
        c.gridy = 5;
        add(LABELS[6], c);
        c.gridy = 6;
        add(LABELS[7], c);
        c.gridy = 7;
        add(LABELS[8], c);
        c.gridx = 2;
        c.gridy = 1;
        add(LABELS[9], c);
        c.gridy = 2;
        add(LABELS[10], c);
        c.gridy = 3;
        add(LABELS[11], c);
        c.gridy = 5;
        add(LABELS[12], c);
        c.gridy = 6;
        add(LABELS[13], c);
        c.gridy = 7;
        add(LABELS[14], c);
        //textfield
        c.gridy = 4;
        add(SPEED, c);
        //list
        c.gridheight = 7;
        c.gridx = 0;
        c.gridy = 1;
        JScrollPane scrolly = new JScrollPane(LIST);
        scrolly.setPreferredSize(new Dimension(160, 160));
        add(scrolly, c);
        //animation
        c.gridheight = 1;
        c.gridwidth = 4;
        c.gridx = 0;
        c.gridy = 8;
        add(ANI, c);
        //extract button
        c.gridheight = 1;
        c.gridwidth = 1;
        c.gridx = 3;
        c.gridy = 1;
        add(btnEXT, c);
        c.gridy = 2;
        add(chkALPHA, c);
        //init
        LIST.setSelectedIndex(0);
    }

    private void fillList() {
        Enumeration<String[]> enu = STBOF.getFiles(new java.io.FilenameFilter() {

            @Override
            public boolean accept(File dir, String name) {
                String str = new String(name);
                if (str.endsWith(".ani") || str.endsWith(".cur")) { //$NON-NLS-1$ //$NON-NLS-2$
                    return true;
                } else {
                    return false; //$NON-NLS-1$ //$NON-NLS-2$
                }
            }
        });
        Vector<String> vec = new Vector<String>();
        while (enu.hasMoreElements()) {
            vec.add(enu.nextElement()[0]);
        }
        LIST.setListData(vec);
    }

    @Override
    public void finalWarning() throws Exception
    {
        if (AOC != null) {
            short speed = (short) ((Integer) SPEED.getValue()).intValue();
            if (AOC.setDelay(speed)) {
                STBOF.addInternalFile(AOC);
            }
        }
    }
}
