/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: BldSetGUI.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui.stbof;


import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import ue.Language;
import ue.UE;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.bin.BldSet;
import ue.edit.res.stbof.bst.Edifice;
import ue.edit.res.stbof.dic.Lexicon;
import ue.edit.res.stbof.rst.Race;
import ue.gui.MainPanel;
import ue.util.ID;
import ue.util.SettingsManager;

/**This class is used to display the about window.*/
@SuppressWarnings("serial")  //$NON-NLS-1$
public class BldSetGUI extends MainPanel{
	//edited
	BldSet SET;
	Edifice BUILDING;
	Stbof STBOF;
	Race RACE;
	Lexicon LEX;
	//gui
	private JComboBox cmbERA;
	private JComboBox cmbSYS;
	private JList lstBLD;
	private JButton btnADD; //add
	private JButton btnREP; //replace
	private JButton btnREM; //remove
	private JTextField txtNUM; //added buildins
	private JList lstALL; //all buildings
	private JComboBox cmbBUILD; //what type of building to show
	private JComboBox cmbLVL; //what level of building to show
	//other
	private int SELECTED_ERA = -1; //selected era
	private int SELECTED_SYS = -1; //selected system
	private int SELECTED_BLD = -1; //selected building - on the left list
	private int SELECTED_ADD = -1; //selected building - on the right list
    
	private ActionListener actSYS = new ActionListener(){
        @Override
		public void actionPerformed(ActionEvent ae){
			if (SELECTED_ERA<0) {
				return;
			}
			try{
				//save
				finalWarning();
				//load list o buildings
				SELECTED_BLD = -1;
				SELECTED_SYS = cmbSYS.getSelectedIndex();
				if (SELECTED_SYS<0) {
					return;
				}
				fillList();
				fillBuildList();
			}catch (Exception e){
				UE.displayError(e);
			}
		}
	};
	
	public BldSetGUI(Stbof st){
		//associate
		if (st==null) {
			return;
		}
		STBOF = st;
		try{
			BUILDING = (Edifice)STBOF.getInternalFile(Stbof.fileEDIFICE_BST, true);
			LEX = (Lexicon)STBOF.getInternalFile(Stbof.fileLEXICON_DIC, true);
			RACE = (Race)STBOF.getInternalFile(Stbof.fileRACE_RST, true);
		}catch (Exception fng){
			UE.displayError(fng);
		}
		if (BUILDING==null || LEX==null) {
			return;
		}
		//build
		cmbERA = new JComboBox();
		cmbSYS = new JComboBox();
		cmbBUILD = new JComboBox();
		cmbLVL = new JComboBox();
		lstBLD = new JList();
		lstALL = new JList();
		btnADD = new JButton(Language.getString("BldSetGUI.0")); //$NON-NLS-1$
		btnREP = new JButton(Language.getString("BldSetGUI.1")); //$NON-NLS-1$
		btnREM = new JButton(Language.getString("BldSetGUI.2")); //$NON-NLS-1$
		txtNUM = new JTextField(4);
        txtNUM.setHorizontalAlignment(JTextField.CENTER);
		//setup
		Font def = UE.SETTINGS.getDefaultFont();
		//list
		lstBLD.setFont(def);
		lstBLD.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		lstBLD.addListSelectionListener(new ListSelectionListener(){
			public void valueChanged(ListSelectionEvent e){
				if (SELECTED_ERA<0 || SELECTED_SYS<0) {
					return;
				}
				try{
					//save old
					finalWarning();
					//go
					if (lstBLD.getSelectedIndex()<0){
						btnREM.setEnabled(false);
						btnREP.setEnabled(false);
						txtNUM.setEnabled(false);
						return;
					}else{
						btnREM.setEnabled(true);
						if (lstALL.getSelectedIndex()>=0){
							btnREP.setEnabled(true);
						}
						txtNUM.setEnabled(true);
					}
					ID id = (ID)lstBLD.getSelectedValue();
					SELECTED_BLD = id.hashCode();
					if (SELECTED_BLD<0) {
						return;
					}
					int f = SET.getNum((short)SELECTED_SYS, (short)SELECTED_BLD);
					txtNUM.setText(Integer.toString(f));
				}catch (Exception h){
					UE.displayError(h);
				}
			}
		});
		//era
		cmbERA.setFont(def);
		cmbERA.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				try{
					//save
					finalWarning();
					//load file
					SELECTED_ERA = cmbERA.getSelectedIndex();
					int g = SELECTED_ERA+1;
					SET = (BldSet)STBOF.getInternalFile("bldset" + g + ".bin", true); //$NON-NLS-1$ //$NON-NLS-2$
					SELECTED_BLD = -1;
					lstBLD.clearSelection();
					actSYS.actionPerformed(null);
					fillBuildList();
				}catch (Exception e){
					UE.displayError(e);
				}
			}
		});
		//sys
		cmbSYS.setFont(def);
		cmbSYS.addActionListener(actSYS);
		//add
		btnADD.setFont(def);
		btnADD.setHorizontalAlignment(SwingConstants.LEFT);
		btnADD.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent ae){
				if (SELECTED_ERA<0 || SELECTED_SYS<0 || SELECTED_ADD<0) {
					return;
				}
				try{
					//get input
					SET.setNum((short)SELECTED_SYS, (short)SELECTED_ADD, 1);
					SELECTED_BLD = SELECTED_ADD;
                    SELECTED_ADD = -1;
                    txtNUM.setText(Integer.toString(1));
					fillList();
					fillBuildList();
				}catch (Exception e){
					UE.displayError(e);
				}
			}
		});
		//remove
		btnREM.setFont(def);
		btnREM.setHorizontalAlignment(SwingConstants.RIGHT);
		btnREM.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent ae){
				if (SELECTED_ERA < 0 || SELECTED_SYS < 0 || SELECTED_BLD < 0) {
					return;
				}
				try
                {
					SET.setNum((short)SELECTED_SYS, (short)SELECTED_BLD, 0);
					SELECTED_ADD = SELECTED_BLD;
                    SELECTED_BLD = -1;
                    txtNUM.setText(Integer.toString(0));
					fillList();
					fillBuildList();
				}catch (Exception e){
					UE.displayError(e);
				}
			}
		});
		//other
		txtNUM.setFont(def);
		//replace
		btnREP.setFont(def);
		btnREP.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent ae){
				if (SELECTED_ERA<0 || SELECTED_SYS<0 || SELECTED_BLD<0 || SELECTED_ADD<0) {
					return;
				}
				try{
                    int num = SET.getNum((short)SELECTED_SYS, (short)SELECTED_BLD);
					SET.setNum((short)SELECTED_SYS, (short)SELECTED_BLD, 0);
					SET.setNum((short)SELECTED_SYS, (short)SELECTED_ADD, num);

					int gor = SELECTED_BLD;
					SELECTED_BLD = SELECTED_ADD;
					SELECTED_ADD = gor;
					fillList();
					fillBuildList();
				}catch (Exception e){
					UE.displayError(e);
				}
			}
		});
		//all
		lstALL.setFont(def);
		lstALL.addListSelectionListener(new ListSelectionListener(){
            @Override
			public void valueChanged(ListSelectionEvent e){
				if (SELECTED_ERA<0 || SELECTED_SYS<0) {
					return;
				}
				try{
					//go
					if (lstALL.getSelectedIndex()<0){
						btnADD.setEnabled(false);
						btnREP.setEnabled(false);
						return;
					}else{
						btnADD.setEnabled(true);
						if (lstBLD.getSelectedIndex()>=0){
							btnREP.setEnabled(true);
						}
					}
					ID id = (ID)lstALL.getSelectedValue();
					SELECTED_ADD = id.hashCode();
				}catch (Exception h){
					UE.displayError(h);
				}
			}
		});
		//combo
		cmbBUILD.setFont(def);
		cmbBUILD.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent ae){
				if (cmbBUILD.getSelectedIndex()<0 || cmbLVL.getSelectedIndex()<0) {
					return;
				}
				fillBuildList();
			}
		});
		cmbLVL.setFont(def);
		cmbLVL.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent ae){
				if (cmbBUILD.getSelectedIndex()<0 || cmbLVL.getSelectedIndex()<0) {
					return;
				}
				fillBuildList();
			}
		});
		//place
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets=new Insets(5, 5, 5, 5);
		c.fill=GridBagConstraints.BOTH;
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		JLabel lbl = new JLabel(Language.getString("BldSetGUI.3")); //$NON-NLS-1$
		lbl.setFont(def);
		add(lbl, c);
		c.gridx = 1;
		lbl = new JLabel(Language.getString("BldSetGUI.4")); //$NON-NLS-1$
		lbl.setFont(def);
		add(lbl, c);
		c.gridy = 1;
		c.gridx = 0;
		add(cmbERA, c);
		c.gridx = 1;
		add(cmbSYS, c);
		c.gridy = 2;
		c.gridx = 0;
		c.gridheight = 6;
		JScrollPane jspBLD = new JScrollPane(lstBLD);
		jspBLD.setPreferredSize(new Dimension(160, 320));
		jspBLD.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		add(jspBLD, c);
		c.gridheight = 1;
		c.gridx = 1;
		add(btnADD, c);
		c.gridy = 3;
		add(btnREP, c);
		c.gridy = 4;
		add(btnREM, c);
		c.gridy = 5;
		lbl = new JLabel(Language.getString("BldSetGUI.5")); //$NON-NLS-1$
		lbl.setFont(def);
		add(lbl, c);
		c.gridy = 6;
		add(txtNUM, c);
		c.gridwidth = 2;
		c.gridx = 2;
		c.gridy = 0;
		lbl = new JLabel(Language.getString("BldSetGUI.6")); //$NON-NLS-1$
		lbl.setFont(def);
		add(lbl, c);
		c.gridwidth = 1;
		c.gridy = 1;
		add(cmbBUILD, c);
		c.gridx = 3;
		add(cmbLVL, c);
		c.gridwidth = 2;
		c.gridx = 2;
		c.gridy = 2;
		c.gridheight = 6;
		JScrollPane jspALL = new JScrollPane(lstALL);
		jspALL.setPreferredSize(new Dimension(160, 320));
		jspALL.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		add(jspALL, c);
		//fill
		//sys
		try{
			cmbSYS.addItem(new ID(LEX.getEntry(121), 0));
			cmbSYS.addItem(new ID(LEX.getEntry(1160), 1));
			cmbSYS.addItem(new ID(LEX.getEntry(357), 2));
			cmbSYS.addItem(new ID(LEX.getEntry(958), 3));
			cmbSYS.addItem(new ID(LEX.getEntry(1092), 4));
		}catch (Exception t){
			UE.displayError(t);
		}
		//era
		try{
			cmbERA.addItem(new ID(LEX.getEntry(88), 0));
			cmbERA.addItem(new ID(LEX.getEntry(252), 1));
			cmbERA.addItem(new ID(LEX.getEntry(234), 2));
			cmbERA.addItem(new ID(LEX.getEntry(312), 3));
			cmbERA.addItem(new ID(LEX.getEntry(4), 4));
		}catch (Exception t){
			UE.displayError(t);
		}
		//level
		try{
			cmbLVL.addItem(Language.getString("BldSetGUI.7")); //$NON-NLS-1$
			String msg;
			for (int i=0; i<10; i++){
				msg = Language.getString("BldSetGUI.8"); //$NON-NLS-1$
				msg = msg.replace("%1", Integer.toString(i)); //$NON-NLS-1$
				cmbLVL.addItem(msg);
			}
		}catch (Exception t){
			UE.displayError(t);
		}
		//mask combo
		try{
			cmbBUILD.addItem(new ID(Language.getString("BldSetGUI.9"),0)); //$NON-NLS-1$
			for (int n=0; n<5; n++){
				cmbBUILD.addItem(new ID(RACE.getName(n), n+1));
			}
			cmbBUILD.addItem(new ID(Language.getString("BldSetGUI.10"), 6)); //$NON-NLS-1$
			String msg;
			for (int n=0; n<5; n++){
				msg = Language.getString("BldSetGUI.11"); //$NON-NLS-1$
				msg = msg.replace("%1", RACE.getName(n)); //$NON-NLS-1$
				cmbBUILD.addItem(new ID(msg, n+7));
			}
			cmbBUILD.addItem(new ID(Language.getString("BldSetGUI.12"), 12)); //$NON-NLS-1$
		}catch (Exception t){
			UE.displayError(t);
		}
		//false
	}
	private void fillList()
    {
        try
        {
            finalWarning();
        }
        catch (Exception ex)
        {
            // really bad stuff happening - bail
            UE.displayError(ex);
            return;
        }

        try
        {
			btnREM.setEnabled(false);
			btnREP.setEnabled(false);
			int sel = SELECTED_BLD;
			SELECTED_BLD = -1;
			short[] id = SET.getBuildings((short)SELECTED_SYS);
			ID[] hm = new ID[id.length];
			for (int i=0; i<id.length; i++){
				hm[i] = new ID(BUILDING.getName(id[i]), id[i]);
			}
			lstBLD.setListData(hm);
			lstBLD.setSelectedValue(new ID("", sel), true); //$NON-NLS-1$
		}catch (Exception e){
			UE.displayError(e);
		}
	}
	private void fillBuildList()
    {
        try
        {
            finalWarning();
        }
        catch (Exception ex)
        {
            // really bad stuff happening - bail
            UE.displayError(ex);
            return;
        }
        
		try{
			btnADD.setEnabled(false);
			btnREP.setEnabled(false);
			int i = SELECTED_ADD;
			SELECTED_ADD = -1;
			if (cmbBUILD.getSelectedIndex()>=0){
				byte msk = (byte)cmbBUILD.getSelectedIndex();
				ID[] hm = BUILDING.getShortIDs(msk, 20);
				ArrayList<ID> accept = new ArrayList<ID>();
				short[] id = SET.getBuildings((short)SELECTED_SYS);
				for (int j=0; j<hm.length; j++){
					if (cmbLVL.getSelectedIndex()!=0){
						int lvl = BUILDING.getTechLvl(hm[j].hashCode());
						if (lvl!=cmbLVL.getSelectedIndex()-1) {
							continue;
						}
					}
					boolean go = true;
					for (int k=0; k<id.length && go; k++){
						if (id[k]==hm[j].hashCode()){
							go = false;
						}
					}
					if (go){
						accept.add(hm[j]);
					}
				}
				hm = new ID[accept.size()];
				for (int j=0; j<accept.size(); j++){
					hm[j] = accept.get(j);
				}
				lstALL.setListData(hm);
				lstALL.setSelectedValue(new ID("", i), true); //$NON-NLS-1$
			}
		}catch (Exception e){
			UE.displayError(e);
		}
	}
	@Override
	public void finalWarning() throws Exception{
		if (SELECTED_ERA<0 || SELECTED_SYS<0 || SELECTED_BLD<0) {
			return;
		}
		int num = Integer.parseInt(txtNUM.getText());

        if (!(num<0)) {
            SET.setNum((short)SELECTED_SYS, (short)SELECTED_BLD, num);
        }
	}
}
