/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: BuildingsGUI.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui.stbof;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileNotFoundException;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.Timer;
import javax.swing.border.LineBorder;

import ue.Language;
import ue.UE;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.bst.Edifbnft;
import ue.edit.res.stbof.bst.Edifice;
import ue.edit.res.stbof.dic.Lexicon;
import ue.edit.res.stbof.rst.Race;
import ue.gui.MainPanel;
import ue.util.ID;
import ue.util.SettingsManager;

/**This class is used to display the about window.*/
@SuppressWarnings("serial")  //$NON-NLS-1$
public class BuildingsMultiSetGUI extends MainPanel{
	//edited
	Edifice BUILDING = null;
	Edifbnft BONUS = null;
	Stbof STBOF = null;
	Lexicon LEX = null;
	Race RACE = null;

	//gui
	private JPanel pnlFILTER;
	private JPanel pnlCONF;
	private JTextField txtVALUE[];
	private JComboBox cmbBONI;
	private JComboBox cmbSET;
	private JComboBox cmbEMPIRE;
	private JButton btnEXECUTE;
	private JLabel lblALL;
	
	//other
	Timer tCOOLER = new Timer(5000, new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                btnEXECUTE.setEnabled(true);
            }
	});
	
	/**Costructor*/
	public BuildingsMultiSetGUI(Stbof st){
		/*PRE*/
			if (st==null) {
				return;
			} else {
				STBOF = st;
			}
			try{
				BUILDING = (Edifice)STBOF.getInternalFile(Stbof.fileEDIFICE_BST, true);
				BONUS = (Edifbnft)STBOF.getInternalFile(Stbof.fileEDIFBNFT_BST, true);
				LEX = (Lexicon)STBOF.getInternalFile(Stbof.fileLEXICON_DIC, true);
				RACE = (Race)STBOF.getInternalFile(Stbof.fileRACE_RST, true);
			}catch (FileNotFoundException fnf){
				UE.displayError(fnf);
				return;
			}
			if (BUILDING == null || BONUS == null || LEX == null) {
				return;
			}

		/*CREATE*/
			pnlFILTER = new JPanel(new GridBagLayout());
			pnlCONF = new JPanel(new GridBagLayout());

			txtVALUE = new JTextField[6];
			btnEXECUTE = new JButton(Language.getString("BuildingsMultiSetGUI.0")); //$NON-NLS-1$
			cmbBONI = new JComboBox();
			cmbSET = new JComboBox();
			cmbEMPIRE = new JComboBox();
			
		/*SETUP*/
			//font
			Font def = UE.SETTINGS.getDefaultFont();

			// timer
			tCOOLER.setRepeats(false);
			
			//filter panel
			pnlFILTER.setBorder(new LineBorder(Color.BLACK));

			// set panel
			pnlCONF.setBorder(new LineBorder(Color.BLACK));			
			
			// text fields
			for (int i = 0; i < 6; i++){
				txtVALUE[i] = new JTextField("5*L^(1-2/1+1)+(L-L)"); //$NON-NLS-1$
				txtVALUE[i].setFont(def);
			}

			//bonus type combo
			cmbBONI.setFont(def);
			for (int i = 0; i < BONUS.getNumberOfEntries(); i++){
				try
                {
                    cmbBONI.addItem(new ID(i + ": " + BONUS.getBoniDesc(i), i));
				}catch (Exception f){
					UE.displayError(f);
					cmbBONI.addItem(new ID(Integer.toString(i), i));
				}
			}
			
			// set combo
			cmbSET.setFont(def);
			cmbSET.addActionListener(new ActionListener(){
            @Override
				public void actionPerformed(ActionEvent ae){
					if (cmbSET.getSelectedIndex() < 0) {
						return;
					}
					if (cmbSET.getSelectedIndex() < 3){
						// industry cost, energy, morale
						for (int i = 0; i < 5; i++){
							txtVALUE[i].setEnabled(false);
						}
						lblALL.setText(Language.getString("BuildingsMultiSetGUI.1")); //$NON-NLS-1$
					}else{
						// bonus, unrest
						for (int i = 0; i < 5; i++){
							txtVALUE[i].setEnabled(true);
						}
						lblALL.setText(Language.getString("BuildingsMultiSetGUI.2")); //$NON-NLS-1$
					}
				}
			});

			// empire combo
			cmbEMPIRE.setFont(def);
                        for (int i = 0; i < 5; i++){
                                try{
                                        cmbEMPIRE.addItem(new ID(RACE.getName(i), i));
                                }catch (Exception f){
                                        UE.displayError(f);
                                        cmbEMPIRE.addItem(new ID(Integer.toString(i), i));
                                }
                        }
                        cmbEMPIRE.addItem(new ID(Language.getString("BuildingsMultiSetGUI.12"), 5)); //$NON-NLS-1$
                        cmbEMPIRE.setSelectedIndex(5);
			
			//add button
			btnEXECUTE.setFont(def);
			btnEXECUTE.addActionListener(new ActionListener(){
            @Override
				public void actionPerformed(ActionEvent ae){
					btnEXECUTE.setEnabled(false);
					tCOOLER.start();
					try{
						int em = cmbEMPIRE.getSelectedIndex();
						byte empire;
						if (em < 5)
							empire = (byte)Math.pow(2, em);
						else
							empire = (byte)0xE0;
						
						String val[] = new String[6];
						for (int i = 0; i < 6; i++){
							val[i] = txtVALUE[i].getText();
						}
						
						switch (cmbSET.getSelectedIndex()){
							case 0:{
								// industry
								BUILDING.multiSetIndustryCost((byte)cmbBONI.getSelectedIndex(), val[5], empire);
								break;
							}
							case 1:{
								// energy
								BUILDING.multiSetEnergy((byte)cmbBONI.getSelectedIndex(), val[5], empire);
								break;
							}
							case 2:{
								// morale
								BUILDING.multiSetMorale((byte)cmbBONI.getSelectedIndex(), val[5], empire);
								break;
							}
							case 3:{
								// bonus
								BUILDING.multiSetBonus((byte)cmbBONI.getSelectedIndex(), val, empire);
								break;
							}
							case 4:{
								// unrest
								BUILDING.multiSetUnrestBonus((byte)cmbBONI.getSelectedIndex(), val, empire);
								break;
							}
						}
					}catch (Exception tu){
						UE.displayError(tu);
					}
				}
			});      
			
		/*PLACE*/
			setLayout(new GridBagLayout());
			GridBagConstraints c = new GridBagConstraints();
			c.insets=new Insets(5, 5, 5, 5);
			c.fill=GridBagConstraints.BOTH;
			c.gridx = 0;
			c.gridy = 0;
			c.gridwidth = 1;
			c.gridheight = 1;
			// FILTER PANEL
				JLabel lbl = new JLabel(Language.getString("BuildingsMultiSetGUI.3")); //$NON-NLS-1$
				lbl.setFont(def);
				pnlFILTER.add(lbl, c);
				c.gridx = 1;
				pnlFILTER.add(cmbBONI, c);
				c.gridy = 1;
				c.gridx = 0;
				lbl = new JLabel(Language.getString("BuildingsMultiSetGUI.13")); //$NON-NLS-1$
				lbl.setFont(def);
				pnlFILTER.add(lbl, c);
				c.gridx = 1;
				pnlFILTER.add(cmbEMPIRE, c);
			//SET PANEL
				c.gridx = 0;
				c.gridy = 0;
				c.gridwidth = 1;
				c.gridheight = 1;
				lbl = new JLabel(Language.getString("BuildingsMultiSetGUI.4")); //$NON-NLS-1$
				lbl.setFont(def);
				pnlCONF.add(lbl, c);
				c.gridx = 1;
				pnlCONF.add(cmbSET, c);
				c.gridy = 1;
				c.gridx = 0;
				c.gridwidth = 2;
				lbl = new JLabel(Language.getString("BuildingsMultiSetGUI.5")); //$NON-NLS-1$
				lbl.setFont(def);
				pnlCONF.add(lbl, c);
				
				c.gridwidth = 1;
				for (int i = 0; i < 5; i++){
					c.gridy++;
					c.gridx = 0;
					try{
						lbl = new JLabel(RACE.getName(i) + ":"); //$NON-NLS-1$
					}catch (Exception f){
						UE.displayError(f);
						lbl = new JLabel(Integer.toString(i));
					}
					lbl.setFont(def);
					pnlCONF.add(lbl, c);
					c.gridx = 1;
					pnlCONF.add(txtVALUE[i], c);
				}
				c.gridy = 7;
				c.gridx = 0;
				lblALL = new JLabel(Language.getString("BuildingsMultiSetGUI.1")); //$NON-NLS-1$
				lblALL.setFont(def);
				pnlCONF.add(lblALL, c);
				c.gridx = 1;
				pnlCONF.add(txtVALUE[5], c);

			// 
			c.gridx = 0;
			c.gridy = 0;
			add(pnlFILTER, c);
			c.gridy = 1;
			add(pnlCONF, c);
			c.gridy = 2;
			c.fill = GridBagConstraints.NONE;
			Dimension dim = btnEXECUTE.getPreferredSize();
			if (dim.width < 60) dim.width = 60;
			btnEXECUTE.setPreferredSize(dim);
			add(btnEXECUTE, c);
			
			// init
			cmbSET.addItem(Language.getString("BuildingsMultiSetGUI.7")); //$NON-NLS-1$
			cmbSET.addItem(Language.getString("BuildingsMultiSetGUI.8")); //$NON-NLS-1$
			cmbSET.addItem(Language.getString("BuildingsMultiSetGUI.9")); //$NON-NLS-1$
			cmbSET.addItem(Language.getString("BuildingsMultiSetGUI.10")); //$NON-NLS-1$
			cmbSET.addItem(Language.getString("BuildingsMultiSetGUI.11")); //$NON-NLS-1$
			
	}
	
        @Override
        public String getHelpFileName(){
                return "BuildingsMultiSet.html"; //$NON-NLS-1$
        }

        @Override
	public void finalWarning() throws Exception{
	}
}
