/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: HobFile.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui.stbof;

import javax.swing.*;
import java.io.*;
import java.util.Arrays;
import java.awt.*;
import java.awt.event.*;
import ue.edit.res.stbof.hob.*;
import ue.edit.res.stbof.lst.*;
import ue.edit.res.*;
import ue.edit.*;
import ue.gui.*;
import ue.util.*;
import ue.*;

@SuppressWarnings("serial")  //$NON-NLS-1$
public class HobGUI extends MainPanel{
	//files
	Stbof STBOF;
	HobFile CURRENT_HOB;
	Texture TEXTURE;
	ID[] textures;
	//components
    JLabel lblMSIZE;
	JComboBox cmbHOB;
	JComboBox[] cmbTEXTURE;
	JPanel pnlTEXTURE;
	JPanel pnlPLACEMENT;
	JTextField txtMSIZE;
	JTextField[] txtMPLACEMENT;
	// strings
	private static String strRADIUS = Language.getString("HobGUI.0"); //$NON-NLS-1$
	
	/**constructor*/
	public HobGUI(Stbof st, Texture tex){
		try{
			STBOF = st;
			TEXTURE = tex;
			CURRENT_HOB = null;

			String[] t = tex.getEntries();
                        Arrays.sort(t);
                        textures = new ID[t.length];
                        for (int i = 0; i < t.length; i++)
                        {
                            textures[i] = new ID(t[i], tex.getTextureIndex(t[i]));
                        }
			//CREATE
			cmbHOB = new JComboBox();
			pnlTEXTURE = new JPanel();
			pnlPLACEMENT = new JPanel();
			txtMSIZE = new JTextField();
			lblMSIZE = new JLabel();
			txtMPLACEMENT = new JTextField[6];
			for (int i = 0; i < 6; i++){
			    txtMPLACEMENT[i] = new JTextField();
			}
			//SETUP
			Font def = UE.SETTINGS.getDefaultFont();
			cmbHOB.setFont(def);
			pnlTEXTURE.setLayout(new GridBagLayout());
			pnlPLACEMENT.setLayout(new GridBagLayout());
			

			cmbHOB.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent ae){
				    try{
				        //SAVE OLD
				        finalWarning();
				        if (CURRENT_HOB!=null){
				            if (CURRENT_HOB.madeChanges()){
				                STBOF.addInternalFile(CURRENT_HOB);
				                for (int i=0; i<cmbTEXTURE.length; i++){
				                    cmbTEXTURE[i].setEnabled(false);
				                }
				            }
				        }
				        //SETUP NEW
				        //load the hob file from stbof.res
				        InternalFile tmp = STBOF.getInternalFile((String)cmbHOB.getSelectedItem(), true);
				        if (tmp instanceof GenericFile){
				            GenericFile gen = (GenericFile)tmp;
				            ByteArrayInputStream bais = new ByteArrayInputStream(gen.getData());
				            CURRENT_HOB = new HobFile(bais);
				            CURRENT_HOB.setName(gen.getName());
				            bais.close();
				        }else{
				            CURRENT_HOB = (HobFile)tmp;
				        }
				        
				        // size
				        float size = CURRENT_HOB.getModelSize();
				        float combat_distance = size * (float)0.3268 + (float)949.0173;
				        txtMSIZE.setText(Float.toString(size));
				        lblMSIZE.setText(strRADIUS.replace("%1", Float.toString(combat_distance))); //$NON-NLS-1$
				        // placement
				        float[] place = CURRENT_HOB.getModelPlacement();
				        for (int i = 0; i < 6; i+=2){
				            txtMPLACEMENT[i].setText(Float.toString(place[i]));
				            txtMPLACEMENT[i+1].setText(Float.toString(place[i+1]));
				        }
				        //make
				        cmbTEXTURE = new JComboBox[CURRENT_HOB.numberOfTextures()];
				        //setup
				        Font def = UE.SETTINGS.getDefaultFont();
				        for (int i=0; i < cmbTEXTURE.length; i++){
				            cmbTEXTURE[i] = new JComboBox();
				            cmbTEXTURE[i].setFont(def);
				        }
				        for (int i=0; i<textures.length; i++){
				            for (int j=0; j<cmbTEXTURE.length; j++){
				                cmbTEXTURE[j].addItem(textures[i]);
				            }
				        }
				        //set cmbTextures and show
				        pnlTEXTURE.removeAll();
				        GridBagConstraints c = new GridBagConstraints();
				        c.gridx=0;
				        c.gridy=0;
				        c.gridwidth=1;
				        c.gridheight=1;

				        for (int i = 0; i < cmbTEXTURE.length; i++)
                                        {
				            int id = CURRENT_HOB.getTextureID(i);
				            
                                            if (id > TEXTURE.getMaxIndex() || id < 0)
                                            {
				                id = 0;
				            }

				            cmbTEXTURE[i].addActionListener(new TexAction(i));
				            cmbTEXTURE[i].setSelectedItem(new ID("", id));
				            
				            if (i%3==0){
				                pnlTEXTURE.add(cmbTEXTURE[i], c);
				            }else{
				                c.gridx++;
				                pnlTEXTURE.add(cmbTEXTURE[i], c);
				                if (c.gridx==2){
				                    c.gridy++;
				                    c.gridx=0;
				                }
				            }
				        }
				        //repaint
				        repaint();
				        validate();
				    }catch (Exception e){
				        UE.displayError(e);
				    }
				}
			});

			// model size
			txtMSIZE.setFont(def);
			txtMSIZE.addActionListener(new ActionListener(){
                @Override
                public void actionPerformed(ActionEvent ae){
                    String t = txtMSIZE.getText();
            		try{
			            Float size = Float.parseFloat(t);
                        
                        if (size > 0){
                            float combat_distance = size * (float)0.3268 + (float)949.0173;
                            lblMSIZE.setText(strRADIUS.replace("%1", Float.toString(combat_distance))); //$NON-NLS-1$
                			CURRENT_HOB.setModelSize(size);
                            return;
                        }
        		    }catch (Exception e){
                        lblMSIZE.setText(Language.getString("HobGUI.1")); //$NON-NLS-1$
                    }
                }
            });

			lblMSIZE.setFont(def);

            // placement
            for (int i = 0; i < 6; i++){
                txtMPLACEMENT[i].setFont(def);
            }

            // pnlPLACEMENT
    			GridBagConstraints c = new GridBagConstraints();
    			c.insets=new Insets(5, 5, 5, 5);
    			c.fill=GridBagConstraints.HORIZONTAL;
    			c.gridwidth=1;
    			c.gridheight=1;

    			JLabel lbl = new JLabel(Language.getString("HobGUI.2")); //$NON-NLS-1$
    			lbl.setFont(def);
    			c.gridx=0;
    			c.gridy=0;
    			pnlPLACEMENT.add(lbl,c);
    			c.gridx=1;
    			pnlPLACEMENT.add(txtMPLACEMENT[0],c);
    			lbl = new JLabel("-"); //$NON-NLS-1$
    			lbl.setFont(def);
    			c.gridx=2;
    			pnlPLACEMENT.add(lbl,c);
    			c.gridx=3;
    			pnlPLACEMENT.add(txtMPLACEMENT[1],c);

    			lbl = new JLabel(Language.getString("HobGUI.3")); //$NON-NLS-1$
    			lbl.setFont(def);
    			c.gridx=0;
    			c.gridy=1;
    			pnlPLACEMENT.add(lbl,c);
    			c.gridx=1;
    			pnlPLACEMENT.add(txtMPLACEMENT[2],c);
    			lbl = new JLabel("-"); //$NON-NLS-1$
    			lbl.setFont(def);
    			c.gridx=2;
    			pnlPLACEMENT.add(lbl,c);
    			c.gridx=3;
    			pnlPLACEMENT.add(txtMPLACEMENT[3],c);

    			lbl = new JLabel(Language.getString("HobGUI.4")); //$NON-NLS-1$
    			lbl.setFont(def);
    			c.gridx=0;
    			c.gridy=2;
    			pnlPLACEMENT.add(lbl,c);
    			c.gridx=1;
    			pnlPLACEMENT.add(txtMPLACEMENT[4],c);
    			lbl = new JLabel("-"); //$NON-NLS-1$
    			lbl.setFont(def);
    			c.gridx=2;
    			pnlPLACEMENT.add(lbl,c);
    			c.gridx=3;
    			pnlPLACEMENT.add(txtMPLACEMENT[5],c);


			//fill cmbHOB
			java.util.Enumeration<String[]> hob_files = STBOF.getFiles(new java.io.FilenameFilter(){
				public boolean accept(File dir, String name){
					name = name.toLowerCase();
					return name.endsWith(".hob"); //$NON-NLS-1$
				}
			});
			
			while (hob_files.hasMoreElements()) {
				cmbHOB.addItem(hob_files.nextElement()[0]);
			}

			//SHOW
			setLayout(new GridBagLayout());
			
			c.gridx=0;
			c.gridy=0;
			c.gridwidth=1;
			c.gridheight=1;
			lbl = new JLabel(Language.getString("HobGUI.5")); //$NON-NLS-1$
			lbl.setFont(def);
			add(lbl,c);

			c.gridx=1;
			c.gridy=1;
			add(cmbHOB,c);

			c.gridx=0;
			c.gridy=2;
			c.gridwidth=4;
			add(lblMSIZE,c);

			c.gridx=1;
			c.gridy=3;
			c.gridwidth=1;
			add(txtMSIZE, c);

			lbl = new JLabel(Language.getString("HobGUI.6")); //$NON-NLS-1$
			lbl.setFont(def);
			c.gridx=0;
			c.gridy=4;
			add(lbl,c);
			c.gridx=1;
			c.gridy=5;
			c.gridwidth=4;
			c.gridheight=3;
			add(pnlPLACEMENT,c);


			lbl = new JLabel(Language.getString("HobGUI.7")); //$NON-NLS-1$
			lbl.setFont(def);
			c.gridwidth=1;
			c.gridheight=1;
			c.gridx=0;
			c.gridy=8;
			add(lbl,c);

			c.gridx=1;
			c.gridy=9;
			c.gridwidth=4;
			add(pnlTEXTURE, c);

			pnlTEXTURE.setVisible(true);
			setVisible(true);
		}catch (Exception e){
			UE.displayError(e);
		}
	}
	
	@Override
	public void finalWarning() throws Exception {
		if (CURRENT_HOB == null) {
			return;
		}

        float[] place = new float[6];
        String str;

        str = txtMSIZE.getText();
        float size = Float.parseFloat(str);
        if (size > 0) {
            CURRENT_HOB.setModelSize(size);
        }

        for (int i = 0; i < 6; i++)
        {
    		str = txtMPLACEMENT[i].getText();
            place[i] = Float.parseFloat(str);
        }

        CURRENT_HOB.setModelPlacement(place);
	}

	private class TexAction implements ActionListener{
		private int INDEX;						
		public TexAction(int i){
			INDEX = i;
		}
        @Override
		public void actionPerformed(ActionEvent ae){
			try{
				//set texture
				CURRENT_HOB.setTextureID(INDEX, cmbTEXTURE[INDEX].getSelectedItem().hashCode());
				if (CURRENT_HOB.madeChanges()) {
					STBOF.addInternalFile(CURRENT_HOB);
				}
			}catch (Exception e){
				UE.displayError(e);
			}
		}
	}
}
