/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2009  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: HtmlReportGUI.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 12.5.2009
*/

package ue.gui.stbof;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileOutputStream;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;

import ue.Language;
import ue.UE;
import ue.edit.res.stbof.bst.Edifbnft;
import ue.edit.res.stbof.bst.Edifice;
import ue.edit.res.stbof.dic.Lexicon;
import ue.edit.res.stbof.rst.Race;
import ue.edit.res.stbof.sst.ShipList;
import ue.edit.res.stbof.sst.ShipTech;
import ue.edit.res.stbof.tec.TecField;
import ue.exception.InvalidArgumentsException;
import ue.gui.MainPanel;
import ue.util.ID;
import ue.util.SettingsManager;
import ue.util.Tools;
import ue.util.UEWorker;

/**
 * Creates html reports based on input files.
 * 
 * Code for generating the html report for buildings is based on Gowron's BuildView2 tool.
 */
@SuppressWarnings("serial")   //$NON-NLS-1$
public class HtmlReportGUI extends MainPanel
{
    private Edifice edifice;
    private Edifbnft edifbnft;
    private Race race;
    private Lexicon lexicon;
    private TecField tecfield;
    private ShipList shiplist;
    private ShipTech shiptech;
    
    //displays report
    private JEditorPane htmlReport;
    private JButton btnSave;
    private JScrollPane scrollPane;
    
    private String report = null;
    
    /**
     * Displays report on buildings.
     */
    public HtmlReportGUI(Edifice ice, Edifbnft bnft, Race rst, Lexicon lex, TecField tf)
    {
        edifice = ice;
        edifbnft = bnft;
        race = rst;
        lexicon = lex;
        tecfield = tf;
        
        // generate report
        Font def = UE.SETTINGS.getDefaultFont();
        report = generateReport(0);
        
        setupComponents(def);
        placeComponents();
    }

    /**
     * Displays report on ships.
     */
    public HtmlReportGUI(ShipList ships, ShipTech tech, Race rst, Lexicon lex)
    {
        shiplist = ships;
        shiptech = tech;
        race = rst;
        lexicon = lex;

        // generate report
        Font def = UE.SETTINGS.getDefaultFont();

        report = generateReport(1);

        setupComponents(def);
        placeComponents();
    }
    
    private void setupComponents(Font def)
    {
        // editor pane
        this.htmlReport = new JEditorPane("text/html", report); //$NON-NLS-1$
        this.htmlReport.setEditable(false);
        htmlReport.setCaretPosition(0);
        
        // scrollpane
        scrollPane = new JScrollPane(this.htmlReport);
        scrollPane.setPreferredSize(new Dimension(560, 340));
        scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
        
        // save button
        btnSave = new JButton(Language.getString("HtmlReportGUI.0")); //$NON-NLS-1$
        btnSave.setFont(def);
        btnSave.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                JFileChooser jfc = new JFileChooser(new File(UE.SETTINGS.getProperty(SettingsManager.WORK_PATH)));

                int returnVal = jfc.showSaveDialog(UE.WINDOW);
                
                if(returnVal == JFileChooser.APPROVE_OPTION)
                {
                    File file = jfc.getSelectedFile();

                    if (!file.getName().endsWith(".html")) //$NON-NLS-1$
                    {
                        file = new File(file.getParentFile(), file.getName() + ".html"); //$NON-NLS-1$
                    }
                    
                    if (file.exists())
                    {
                        String msg = Language.getString("HtmlReportGUI.1"); //$NON-NLS-1$
                        msg = msg.replace("%1", file.getName()); //$NON-NLS-1$
                        int respon = JOptionPane.showConfirmDialog(UE.WINDOW, msg, 
                                UE.APP_NAME, JOptionPane.YES_NO_OPTION,
                                JOptionPane.QUESTION_MESSAGE);
                        
                        if (respon != JOptionPane.YES_OPTION)
                        {
                            return;
                        }
                    }
                    
                    FileOutputStream out = null;
                    
                    try
                    {
                        out = new FileOutputStream(file);
                        
                        int i = 0;
                        int j = 0;
                        int len = 10000;
                        int todo = report.length();
                        
                        while (todo > 0)
                        {
                            if (len > todo)
                            {
                                len = todo;
                            }
                            
                            j += len;

                            out.write(report.substring(i, j).getBytes());
                            
                            i += len;
                            todo -= len;
                        }
                        
                        out.close();
                    }
                    catch (Exception x)
                    {
                        UE.displayError(x);
                        
                        try
                        {
                            if (out != null)
                            {
                                out.close();
                            }
                        }
                        catch (Exception f)
                        {}
                    }
                }
            }
        });
    }
    
    private void placeComponents()
    {
        JPanel pnl = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        pnl.add(btnSave);
        
        this.setLayout(new BorderLayout());
        this.add(scrollPane, BorderLayout.CENTER);
        this.add(pnl, BorderLayout.SOUTH);
    }
    
    /**
     * @param type
     * @return
     * @throws InvalidArgumentsException 
     */
    private String generateReport(int type)
    {
        ReportGenerator gen = new ReportGenerator(type);
        return (String)Tools.runUEWorker(gen);
    }

    @Override
    public void finalWarning() throws Exception {}
    
    private class ReportGenerator extends UEWorker<String>
    {
        int type;

        // html strings
        private static final String strHeaderStart = "<H1 align=center>"; //$NON-NLS-1$
        private static final String strHeaderEnd = "</H1>\n<br>\n<br>\n\n"; //$NON-NLS-1$
        private static final String strTableHeadline =
            "<table border cellspacing=1 cellpadding=1 align=center> \n \n" + //$NON-NLS-1$
            "<TR>\n" + //$NON-NLS-1$
            "<TH align=center bgcolor=silver>CATEGORY</TH>\n" + //$NON-NLS-1$
            "<TH align=center bgcolor=silver>NAME</TH>\n" + //$NON-NLS-1$
            "<TH align=center bgcolor=silver>REQUIRED TECH</TH>\n" + //$NON-NLS-1$
            "<TH colspan=2 align=center bgcolor=silver>COST / EN</TH>\n" + //$NON-NLS-1$
            "<TH colspan=2 align=center bgcolor=silver>OUTPUT / MORALE</TH>\n" + //$NON-NLS-1$
            "<TH align=center bgcolor=silver>LIMIT</TH> \n" + //$NON-NLS-1$
            "<TH align=center bgcolor=silver>SYSTEM REQ.</TH> \n" + //$NON-NLS-1$
            "<TH align=center bgcolor=silver>POLITICAL REQ.</TH> \n" + //$NON-NLS-1$
            "</TR>\n\n"; //$NON-NLS-1$
        
        private static final String strTableHeadlineMinors =
            "<table border cellspacing=1 cellpadding=1 align=center> \n \n" + //$NON-NLS-1$
            "<TR>\n" + //$NON-NLS-1$
            "<TH align=center bgcolor=silver>RACE</TH>\n" + //$NON-NLS-1$
            "<TH align=center bgcolor=silver>CATEGORY</TH>\n" + //$NON-NLS-1$
            "<TH align=center bgcolor=silver>NAME</TH>\n" + //$NON-NLS-1$
            "<TH align=center bgcolor=silver>REQUIRED TECH</TH>\n" + //$NON-NLS-1$
            "<TH colspan=2 align=center bgcolor=silver>COST / EN</TH>\n" + //$NON-NLS-1$
            "<TH colspan=2 align=center bgcolor=silver>OUTPUT / MORALE</TH>\n" + //$NON-NLS-1$
            "<TH align=center bgcolor=silver>LIMIT</TH> \n" + //$NON-NLS-1$
            "<TH align=center bgcolor=silver>SYSTEM REQ.</TH> \n" + //$NON-NLS-1$
            "<TH align=center bgcolor=silver>POLITICAL REQ.</TH> \n" + //$NON-NLS-1$
            "</TR>\n\n"; //$NON-NLS-1$

        private static final String strShipListTableHeadline =
            "<table border cellspacing=1 cellpadding=1 width=\"150%\">\n" +
            "<TR>\n" +
            "<TH align=center bgcolor=silver>ID</TH>\n" +
            "<TH align=center bgcolor=silver>NAME</TH>\n" +
            "<TH align=center bgcolor=silver>CLASS</TH>\n" +
            "<TH align=center bgcolor=silver>FUNCTION</TH>\n" +
            "<TH colspan=2 align=center bgcolor=silver>COST/MNT</TH>\n" +
            "<TH align=center bgcolor=silver>SHIELDS</TH>\n" +
            "<TH align=center bgcolor=silver>HULL</TH>\n" +
            "<TH align=center bgcolor=silver>BEAMS</TH>\n" +
            "<TH align=center bgcolor=silver>B. RANGE</TH>\n" +
            "<TH align=center bgcolor=silver>TORPS</TH>\n" +
            "<TH align=center bgcolor=silver>T. RANGE</TH>\n" +
            "<TH align=center bgcolor=silver>ACCURACY</TH>\n" +
            "<TH align=center bgcolor=silver>DEF</TH>\n" +
            "<TH align=center bgcolor=silver>AGILITY</TH>\n" +
            "<TH align=center bgcolor=silver>RANGE</TH>\n" +
            "<TH align=center bgcolor=silver>SPEED</TH>\n" +
            "<TH align=center bgcolor=silver>SCAN</TH>\n" +
            "<TH align=center bgcolor=silver>CLOAK</TH>\n" +
            "<TH align=center bgcolor=silver>PROD</TH>\n" +
            "<TH colspan=6 align=center bgcolor=silver>REQUIRED TECH</TH>\n" +
            "</TR>\n\n";

        private static final String strShipListMinorsTableHeadline =
            "<table border cellspacing=1 cellpadding=1 width=\"150%\">\n" +
            "<TR>\n" +
            "<TH align=center bgcolor=silver>RACE</TH>\n" +
            "<TH align=center bgcolor=silver>ID</TH>\n" +
            "<TH align=center bgcolor=silver>NAME</TH>\n" +
            "<TH align=center bgcolor=silver>CLASS</TH>\n" +
            "<TH align=center bgcolor=silver>FUNCTION</TH>\n" +
            "<TH colspan=2 align=center bgcolor=silver>COST/MNT</TH>\n" +
            "<TH align=center bgcolor=silver>SHIELDS</TH>\n" +
            "<TH align=center bgcolor=silver>HULL</TH>\n" +
            "<TH align=center bgcolor=silver>BEAMS</TH>\n" +
            "<TH align=center bgcolor=silver>B. RANGE</TH>\n" +
            "<TH align=center bgcolor=silver>TORPS</TH>\n" +
            "<TH align=center bgcolor=silver>T. RANGE</TH>\n" +
            "<TH align=center bgcolor=silver>ACCURACY</TH>\n" +
            "<TH align=center bgcolor=silver>DEF</TH>\n" +
            "<TH align=center bgcolor=silver>AGILITY</TH>\n" +
            "<TH align=center bgcolor=silver>RANGE</TH>\n" +
            "<TH align=center bgcolor=silver>SPEED</TH>\n" +
            "<TH align=center bgcolor=silver>SCAN</TH>\n" +
            "<TH align=center bgcolor=silver>CLOAK</TH>\n" +
            "<TH align=center bgcolor=silver>PROD</TH>\n" +
            "<TH colspan=6 align=center bgcolor=silver>REQUIRED TECH</TH>\n" +
            "</TR>\n\n";

        private static final String strShipListTablePadding =
            "<TR>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD align=center bgcolor=silver>---</TD>\n" +
            "<TD colspan=6 align=center bgcolor=silver>---</TD>\n" +
            "</TR>\n\n";

        public ReportGenerator(int t)
        {
            super();
            type = t;
        }
        
        @Override
        public String doInBackground()
        {
            RESULT = ""; //$NON-NLS-1$
            
            this.updateStatus(UEWorker.STATUS_WORKING);
            
            String msg;
            
            try
            {
                StringBuffer report = new StringBuffer();
                
                switch (type)
                {
                    case 0:     // edifice.bst
                    {
                        // check for errors first and abort if any are found
                        Vector<String> test = new Vector<String>();
                        msg = Language.getString("HtmlReportGUI.2"); //$NON-NLS-1$
                        
                        // edifice
                        this.sendMessage(msg.replace("%1", edifice.getName())); //$NON-NLS-1$
                        test.addAll(edifice.check());

                        if (isCancelled())
                        {
                            this.updateStatus(UEWorker.STATUS_CANCEL);
                            return RESULT;
                        }
                        
                        // edifbnft
                        this.sendMessage(msg.replace("%1", edifbnft.getName())); //$NON-NLS-1$
                        test.addAll(edifbnft.check());

                        if (isCancelled())
                        {
                            this.updateStatus(UEWorker.STATUS_CANCEL);
                            return RESULT;
                        }

                        // race
                        this.sendMessage(msg.replace("%1", race.getName())); //$NON-NLS-1$
                        test.addAll(race.check());

                        if (isCancelled())
                        {
                            this.updateStatus(UEWorker.STATUS_CANCEL);
                            return RESULT;
                        }

                        // lexicon
                        this.sendMessage(msg.replace("%1", lexicon.getName())); //$NON-NLS-1$
                        test.addAll(lexicon.check());

                        if (isCancelled())
                        {
                            this.updateStatus(UEWorker.STATUS_CANCEL);
                            return RESULT;
                        }

                        // tecfield
                        this.sendMessage(msg.replace("%1", tecfield.getName())); //$NON-NLS-1$
                        test.addAll(tecfield.check());

                        if (isCancelled())
                        {
                            this.updateStatus(UEWorker.STATUS_CANCEL);
                            return RESULT;
                        }

                        boolean errors = false;
                        
                        for (int i = 0; i < test.size(); i++)
                        {
                            if (test.get(i).startsWith(Language.getString("HtmlReportGUI.3"))) //$NON-NLS-1$
                            {
                                if (!errors)
                                {
                                    report.append(Language.getString("HtmlReportGUI.4") + "<br><br>"); //$NON-NLS-1$
                                }

                                report.append(test.get(i) + "<br>"); //$NON-NLS-1$
                                errors = true;
                            }
                        }

                        if (errors)
                        {
                            break;
                        }
                        
                        // stuff
                        long emp_mask;
                        String[] category = new String[]{Language.getString("HtmlReportGUI.5"), Language.getString("HtmlReportGUI.6"), Language.getString("HtmlReportGUI.7"), Language.getString("HtmlReportGUI.8"), Language.getString("HtmlReportGUI.9")}; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
                        String[] limit = new String[]{lexicon.getEntry(872), lexicon.getEntry(885), lexicon.getEntry(884)};
                        String[] sys_req = new String[]{
                                lexicon.getEntry(38),
                                lexicon.getEntry(56),
                                lexicon.getEntry(230),
                                lexicon.getEntry(410),
                                lexicon.getEntry(559),
                                lexicon.getEntry(879),
                                lexicon.getEntry(1259),
                                lexicon.getEntry(1415),
                                lexicon.getEntry(872),
                                lexicon.getEntry(47),
                                lexicon.getEntry(48),
                                lexicon.getEntry(239),
                                lexicon.getEntry(1447),
                                lexicon.getEntry(968),
                                lexicon.getEntry(1450)     
                        };
                        String[] pol_req = new String[]{
                                lexicon.getEntry(437),
                                lexicon.getEntry(801),
                                lexicon.getEntry(863),
                                lexicon.getEntry(1187),
                                lexicon.getEntry(6),
                                lexicon.getEntry(527),
                                lexicon.getEntry(184),
                                null,
                                null,
                                lexicon.getEntry(872),
                                lexicon.getEntry(977),
                                lexicon.getEntry(277)
                        };
                        
                        msg = Language.getString("HtmlReportGUI.10"); //$NON-NLS-1$
                        this.sendMessage(msg);

                        for (int empire = 0; empire < 5; empire++)
                        {
                            emp_mask = (long)Math.pow(2, empire);
                            
                            report.append(strHeaderStart);
                            
                            // empire
                            report.append(race.getName(empire));
                            report.append(strHeaderEnd);
                            
                            // headline
                            report.append(strTableHeadline);
                            
                            // sort by category
                            for (int cat = 0; cat < 5; cat++)
                            {
                                for (int i = 0; i < edifice.getNumberOfEntries(); i++)
                                {
                                    // ignore basic shipyard
                                    // this is likely set in trek.exe
                                    if (i == 15)
                                    {
                                        continue;
                                    }
                                    
                                    // ignore wrong category
                                    if (edifice.getCategory(i) != cat)
                                    {
                                        continue;
                                    }
                                    
                                    // ignore not buildable
                                    if (((int)emp_mask & edifice.getExEmpire(i)) != 0)
                                    {
                                        continue;
                                    }
                                    
                                    // ignore main buildings
                                    // and buildings that don't list the empire as needed race on system
                                    if (
                                            edifice.isMainBuilding(i)
                                            || (edifice.getReqRace(i) & emp_mask) != emp_mask
                                    )
                                    {
                                        continue;
                                    }
                                    
                                    // ignore trade goods and emergency morale programs
                                    if (
                                            edifice.getBoniType(i) == 4 ||
                                            edifice.getBoniType(i) == 36
                                    )
                                    {
                                        continue;
                                    }
                                    
                                    // accepted building
                                    // add to report
                                    
                                    // category
                                    report.append("<TR>\n"); //$NON-NLS-1$
                                    report.append("<TD align=center>"); //$NON-NLS-1$
                                    report.append(category[cat]);
                                    report.append("</TD>\n"); //$NON-NLS-1$
                                    
                                    // name
                                    report.append("<TH align=center bgcolor=#00E0FF>"); //$NON-NLS-1$
                                    report.append(edifice.getName(i));
                                    report.append("</TH>\n"); //$NON-NLS-1$
                                    
                                    // tech requirement
                                    report.append("<TD align=center>"); //$NON-NLS-1$

                                    if (edifice.getTechReq(i) >= 0 && edifice.getTechReq(i) < tecfield.getFields().length - 1)
                                    {
                                        report.append(
                                                tecfield.getField(edifice.getTechReq(i)).substring(0, 1).toUpperCase()
                                                + tecfield.getField(edifice.getTechReq(i)).substring(1).toLowerCase());
                                    }
                                    else
                                    {
                                        report.append(Language.getString("HtmlReportGUI.11")); //$NON-NLS-1$
                                    }

                                    report.append(" "); //$NON-NLS-1$
                                    report.append(edifice.getTechLvl(i));
                                    report.append("</TD>\n"); //$NON-NLS-1$
                                    
                                    // cost
                                    report.append("<TD align=right bgcolor=silver>"); //$NON-NLS-1$
                                    report.append(edifice.getIndCost(i));
                                    report.append("</TD>\n"); //$NON-NLS-1$
                                    
                                    // energy
                                    report.append("<TD align=right bgcolor=silver>"); //$NON-NLS-1$
                                    report.append(edifice.getEnCost(i));
                                    report.append("</TD>\n"); //$NON-NLS-1$
                                    
                                    // output
                                    report.append("<TD align=center>"); //$NON-NLS-1$
                                    
                                    if (edifice.getBoniType(i) != 18)   // no output value for shipyards
                                    {
                                        if (edifice.getBoni(i, empire) > 0)
                                        {
                                            report.append("+");        //$NON-NLS-1$
                                        }
                                        
                                        report.append(edifice.getBoni(i, empire));
                                    }
                                        
                                    report.append(edifbnft.getBoniDesc(edifice.getBoniType(i)));
                                    report.append("</TD>\n"); //$NON-NLS-1$
                                    
                                    // morale
                                    report.append("<TH align=center"); //$NON-NLS-1$
                                    
                                    if (edifice.getMoraleBoni(i) < 0)
                                    {
                                        report.append(" bgcolor=#FF4020>"); //$NON-NLS-1$
                                        report.append(edifice.getMoraleBoni(i));
                                    }
                                    else if (edifice.getMoraleBoni(i) == 0)
                                    {
                                        report.append(">-----");            //$NON-NLS-1$
                                    }
                                    else
                                    {
                                        report.append(" bgcolor=#00FF00>+"); //$NON-NLS-1$
                                        report.append(edifice.getMoraleBoni(i));
                                    }
                                    
                                    report.append("</TH>\n"); //$NON-NLS-1$
                                    
                                    // limit
                                    report.append("<TD align=center bgcolor=silver>"); //$NON-NLS-1$
                                    report.append(limit[edifice.getFreq(i)]);           
                                    report.append("</TD>\n"); //$NON-NLS-1$
                                    
                                    // requirement
                                    report.append("<TD align=center>"); //$NON-NLS-1$
                                    if (edifice.getSysReq(i) != 8)
                                    {
                                        report.append(sys_req[edifice.getSysReq(i)]);
                                    }
                                    else
                                    {
                                        report.append("-----"); //$NON-NLS-1$
                                    }
                                    report.append("</TD>\n"); //$NON-NLS-1$
                                    
                                    report.append("<TD align=center>"); //$NON-NLS-1$
                                    if (edifice.getPoliticReq(i) != 9)
                                    {
                                        report.append(pol_req[edifice.getPoliticReq(i)]);
                                    }
                                    else
                                    {
                                        report.append("-----"); //$NON-NLS-1$
                                    }
                                    report.append("</TD>\n"); //$NON-NLS-1$

                                    report.append("</TR>\n\n"); //$NON-NLS-1$
                                }
                                
                                if (cat < 4)
                                {
                                    report.append("<TR>\n"); //$NON-NLS-1$
                                    report.append("<TD align=center bgcolor=silver>-----</TD>\n"); //$NON-NLS-1$
                                    report.append("<TD align=center bgcolor=silver>-----</TD>\n"); //$NON-NLS-1$
                                    report.append("<TD align=center bgcolor=silver>-----</TD>\n"); //$NON-NLS-1$
                                    report.append("<TD colspan=2 align=center bgcolor=silver>-----</TD>\n"); //$NON-NLS-1$
                                    report.append("<TD colspan=2 align=center bgcolor=silver>-----</TD>\n"); //$NON-NLS-1$
                                    report.append("<TD align=center bgcolor=silver>-----</TD>\n"); //$NON-NLS-1$
                                    report.append("<TD align=center bgcolor=silver>-----</TD>\n"); //$NON-NLS-1$
                                    report.append("<TD align=center bgcolor=silver>-----</TD>\n"); //$NON-NLS-1$
                                    report.append("</TR>\n\n"); //$NON-NLS-1$
                                }
                            }
                            
                            report.append("\n\n</table>\n\n"); //$NON-NLS-1$
                            report.append("\n<br>\n<br>\n<br>\n\n"); //$NON-NLS-1$
                        }

                        // minor races
                        report.append(strHeaderStart);
                        
                        report.append(Language.getString("HtmlReportGUI.12")); //$NON-NLS-1$
                        report.append(strHeaderEnd);
                        
                        // headline
                        report.append(strTableHeadlineMinors);
                        
                        long r_mask = 0;
                        
                        // sort by race
                        for (int r = 5; r < race.getNumberOfEntries(); r++)
                        {
                            r_mask = (long)Math.pow(2, r);
                        
                            for (int i = 0; i < edifice.getNumberOfEntries(); i++)
                            {
                                // ignore basic shipyard
                                // this is likely set in trek.exe
                                if (i == 15)
                                {
                                    continue;
                                }
                                
                                // ignore if not buildable by this race
                                // and i not minor race structure
                                if ( (edifice.getReqRace(i) & r_mask) != r_mask
                                        || (edifice.getReqRace(i) & 31) != 0)
                                {
                                    continue;
                                }
                                        
                                // ignore trade goods and emergency morale programs
                                if (
                                        edifice.getBoniType(i) == 4 ||
                                        edifice.getBoniType(i) == 36
                                )
                                {
                                    continue;
                                }
                                
                                // accepted building
                                // add to report
                                
                                // race
                                report.append("<TR>\n"); //$NON-NLS-1$
                                report.append("<TD align=center>"); //$NON-NLS-1$
                                report.append(race.getName(r));
                                report.append("</TD>\n"); //$NON-NLS-1$
                                
                                // category
                                report.append("<TD align=center>"); //$NON-NLS-1$
                                report.append(category[edifice.getCategory(i)]);
                                report.append("</TD>\n"); //$NON-NLS-1$
                                
                                // name
                                report.append("<TH align=center bgcolor=#00E0FF>"); //$NON-NLS-1$
                                report.append(edifice.getName(i));
                                report.append("</TH>\n"); //$NON-NLS-1$
                                
                                // tech requirement
                                report.append("<TD align=center>"); //$NON-NLS-1$
                                
                                if (edifice.getTechReq(i) >= 0 && edifice.getTechReq(i) < tecfield.getFields().length - 1)
                                {
                                    report.append(
                                            tecfield.getField(edifice.getTechReq(i)).substring(0, 1).toUpperCase()
                                            + tecfield.getField(edifice.getTechReq(i)).substring(1).toLowerCase());
                                }
                                else
                                {
                                    report.append(Language.getString("HtmlReportGUI.11")); //$NON-NLS-1$
                                }
                                
                                report.append(" "); //$NON-NLS-1$
                                report.append(edifice.getTechLvl(i));
                                report.append("</TD>\n"); //$NON-NLS-1$
                                
                                // cost
                                report.append("<TD align=right bgcolor=silver>"); //$NON-NLS-1$
                                report.append(edifice.getIndCost(i));
                                report.append("</TD>\n"); //$NON-NLS-1$
                                
                                // energy
                                report.append("<TD align=right bgcolor=silver>"); //$NON-NLS-1$
                                report.append(edifice.getEnCost(i));
                                report.append("</TD>\n"); //$NON-NLS-1$
                                
                                // output
                                report.append("<TD align=center>"); //$NON-NLS-1$
                                
                                if (edifice.getBoniType(i) != 18)   // no output value for shipyards
                                {
                                    if (edifice.getBoni(i, 5) > 0)
                                    {
                                        report.append("+");        //$NON-NLS-1$
                                    }
                                    
                                    report.append(edifice.getBoni(i, 5));
                                }
                                            
                                report.append(edifbnft.getBoniDesc(edifice.getBoniType(i)));
                                report.append("</TD>\n"); //$NON-NLS-1$
                                
                                // morale
                                report.append("<TH align=center"); //$NON-NLS-1$
                                
                                if (edifice.getMoraleBoni(i) < 0)
                                {
                                    report.append(" bgcolor=#FF4020>"); //$NON-NLS-1$
                                    report.append(edifice.getMoraleBoni(i));
                                }
                                else if (edifice.getMoraleBoni(i) == 0)
                                {
                                    report.append(">-----");            //$NON-NLS-1$
                                }
                                else
                                {
                                    report.append(" bgcolor=#00FF00>+"); //$NON-NLS-1$
                                    report.append(edifice.getMoraleBoni(i));
                                }
                                
                                report.append("</TH>\n"); //$NON-NLS-1$
                                
                                // limit
                                report.append("<TD align=center bgcolor=silver>"); //$NON-NLS-1$
                                report.append(limit[edifice.getFreq(i)]);           
                                report.append("</TD>\n"); //$NON-NLS-1$
                                
                                // requirement
                                report.append("<TD align=center>"); //$NON-NLS-1$
                                if (edifice.getSysReq(i) != 8)
                                {
                                    report.append(sys_req[edifice.getSysReq(i)]);
                                }
                                else
                                {
                                    report.append("-----"); //$NON-NLS-1$
                                }
                                report.append("</TD>\n"); //$NON-NLS-1$
                                
                                report.append("<TD align=center>"); //$NON-NLS-1$
                                if (edifice.getPoliticReq(i) != 9)
                                {
                                    report.append(pol_req[edifice.getPoliticReq(i)]);
                                }
                                else
                                {
                                    report.append("-----"); //$NON-NLS-1$
                                }
                                report.append("</TD>\n"); //$NON-NLS-1$

                                report.append("</TR>\n\n"); //$NON-NLS-1$
                            }
                        }

                        report.append("\n\n</table>\n\n"); //$NON-NLS-1$
                        report.append("\n<br>\n<br>\n<br>\n\n"); //$NON-NLS-1$


                        // main buildings
                        report.append(strHeaderStart);

                        report.append(Language.getString("HtmlReportGUI.14")); //$NON-NLS-1$
                        report.append(strHeaderEnd);

                        // headline
                        report.append(strTableHeadlineMinors);

                        r_mask = 0;

                        // sort by race
                        for (int r = 0; r < 5; r++)
                        {
                            r_mask = (long)Math.pow(2, r);

                            for (int i = 0; i < edifice.getNumberOfEntries(); i++)
                            {
                                // ignore basic shipyard
                                // this is likely set in trek.exe
                                if (i == 15)
                                {
                                    continue;
                                }

                                // ignore not buildable
                                if (((int)r_mask & edifice.getExEmpire(i)) != 0)
                                {
                                    continue;
                                }

                                // ignore main buildings
                                // and buildings that don't list the empire as needed race on system
                                if (
                                        !edifice.isMainBuilding(i)
                                        || (edifice.getReqRace(i) & r_mask) != r_mask
                                )
                                {
                                    continue;
                                }

                                // ignore trade goods and emergency morale programs
                                if (
                                        edifice.getBoniType(i) == 4 ||
                                        edifice.getBoniType(i) == 36
                                )
                                {
                                    continue;
                                }

                                // accepted building
                                // add to report

                                // race
                                report.append("<TR>\n"); //$NON-NLS-1$
                                report.append("<TD align=center>"); //$NON-NLS-1$
                                report.append(race.getName(r));
                                report.append("</TD>\n"); //$NON-NLS-1$

                                // category
                                report.append("<TD align=center>"); //$NON-NLS-1$
                                report.append(category[edifice.getCategory(i)]);
                                report.append("</TD>\n"); //$NON-NLS-1$

                                // name
                                report.append("<TH align=center bgcolor=#00E0FF>"); //$NON-NLS-1$
                                report.append(edifice.getName(i));
                                report.append("</TH>\n"); //$NON-NLS-1$

                                // tech requirement
                                report.append("<TD align=center>"); //$NON-NLS-1$

                                if (edifice.getTechReq(i) >= 0 && edifice.getTechReq(i) < tecfield.getFields().length - 1)
                                {
                                    report.append(
                                            tecfield.getField(edifice.getTechReq(i)).substring(0, 1).toUpperCase()
                                            + tecfield.getField(edifice.getTechReq(i)).substring(1).toLowerCase());
                                }
                                else
                                {
                                    report.append(Language.getString("HtmlReportGUI.11")); //$NON-NLS-1$
                                }

                                report.append(" "); //$NON-NLS-1$
                                report.append(edifice.getTechLvl(i));
                                report.append("</TD>\n"); //$NON-NLS-1$

                                // cost
                                report.append("<TD align=right bgcolor=silver>"); //$NON-NLS-1$
                                report.append(edifice.getIndCost(i));
                                report.append("</TD>\n"); //$NON-NLS-1$

                                // energy
                                report.append("<TD align=right bgcolor=silver>"); //$NON-NLS-1$
                                report.append(edifice.getEnCost(i));
                                report.append("</TD>\n"); //$NON-NLS-1$

                                // output
                                report.append("<TD align=center>"); //$NON-NLS-1$

                                if (edifice.getBoniType(i) != 18)   // no output value for shipyards
                                {
                                    if (edifice.getBoni(i, 5) > 0)
                                    {
                                        report.append("+");        //$NON-NLS-1$
                                    }

                                    report.append(edifice.getBoni(i, 5));
                                }

                                report.append(edifbnft.getBoniDesc(edifice.getBoniType(i)));
                                report.append("</TD>\n"); //$NON-NLS-1$

                                // morale
                                report.append("<TH align=center"); //$NON-NLS-1$

                                if (edifice.getMoraleBoni(i) < 0)
                                {
                                    report.append(" bgcolor=#FF4020>"); //$NON-NLS-1$
                                    report.append(edifice.getMoraleBoni(i));
                                }
                                else if (edifice.getMoraleBoni(i) == 0)
                                {
                                    report.append(">-----");            //$NON-NLS-1$
                                }
                                else
                                {
                                    report.append(" bgcolor=#00FF00>+"); //$NON-NLS-1$
                                    report.append(edifice.getMoraleBoni(i));
                                }

                                report.append("</TH>\n"); //$NON-NLS-1$

                                // limit
                                report.append("<TD align=center bgcolor=silver>"); //$NON-NLS-1$
                                report.append(limit[edifice.getFreq(i)]);
                                report.append("</TD>\n"); //$NON-NLS-1$

                                // requirement
                                report.append("<TD align=center>"); //$NON-NLS-1$
                                if (edifice.getSysReq(i) != 8)
                                {
                                    report.append(sys_req[edifice.getSysReq(i)]);
                                }
                                else
                                {
                                    report.append("-----"); //$NON-NLS-1$
                                }
                                report.append("</TD>\n"); //$NON-NLS-1$

                                report.append("<TD align=center>"); //$NON-NLS-1$
                                if (edifice.getPoliticReq(i) != 9)
                                {
                                    report.append(pol_req[edifice.getPoliticReq(i)]);
                                }
                                else
                                {
                                    report.append("-----"); //$NON-NLS-1$
                                }
                                report.append("</TD>\n"); //$NON-NLS-1$

                                report.append("</TR>\n\n"); //$NON-NLS-1$
                            }
                        }

                        report.append("\n\n</table>\n\n"); //$NON-NLS-1$
                        report.append("\n<br>\n<br>\n<br>\n\n"); //$NON-NLS-1$

                        break;
                    }
                    case 1:     // shiplist.sst
                    {
                        // check for errors first and abort if any are found
                        Vector<String> test = new Vector<String>();
                        msg = Language.getString("HtmlReportGUI.2"); //$NON-NLS-1$

                        // shiplist.sst
                        this.sendMessage(msg.replace("%1", shiplist.getName())); //$NON-NLS-1$
                        test.addAll(shiplist.check());

                        if (isCancelled())
                        {
                            this.updateStatus(UEWorker.STATUS_CANCEL);
                            return RESULT;
                        }

                        // shiptech
                        this.sendMessage(msg.replace("%1", shiptech.getName())); //$NON-NLS-1$
                        test.addAll(shiptech.check());

                        if (isCancelled())
                        {
                            this.updateStatus(UEWorker.STATUS_CANCEL);
                            return RESULT;
                        }

                        // race
                        this.sendMessage(msg.replace("%1", race.getName())); //$NON-NLS-1$
                        test.addAll(race.check());

                        if (isCancelled())
                        {
                            this.updateStatus(UEWorker.STATUS_CANCEL);
                            return RESULT;
                        }

                        // lexicon
                        this.sendMessage(msg.replace("%1", lexicon.getName())); //$NON-NLS-1$
                        test.addAll(lexicon.check());

                        if (isCancelled())
                        {
                            this.updateStatus(UEWorker.STATUS_CANCEL);
                            return RESULT;
                        }

                                                boolean errors = false;

                        for (int i = 0; i < test.size(); i++)
                        {
                            if (test.get(i).startsWith(Language.getString("HtmlReportGUI.3"))) //$NON-NLS-1$
                            {
                                if (!errors)
                                {
                                    report.append(Language.getString("HtmlReportGUI.4") + "<br><br>"); //$NON-NLS-1$
                                }

                                report.append(test.get(i) + "<br>"); //$NON-NLS-1$
                                errors = true;
                            }
                        }

                        if (errors)
                        {
                            break;
                        }


                        // for each major race, plus minors, plus aliens
                        String[] function = new String[]{
                                lexicon.getEntry(1103),
                                lexicon.getEntry(232),
                                lexicon.getEntry(204),
                                lexicon.getEntry(1183),
                                lexicon.getEntry(173),
                                lexicon.getEntry(166),
                                lexicon.getEntry(895),
                                lexicon.getEntry(57),
                                Language.getString("ShipGUI.1"),
                                lexicon.getEntry(1366)
                        };
                        String[] cloak = new String[]{
                                Language.getString("ShipGUI.2"),
                                Language.getString("ShipGUI.4"),
                                Language.getString("ShipGUI.5"),
                                Language.getString("ShipGUI.6"),
                                Language.getString("ShipGUI.7"),
                                Language.getString("ShipGUI.8"),
                                Language.getString("ShipGUI.9")
                        };
                        String[] range_plus1 = new String[]{
                                Language.getString("ShipGUI.2"),
                                lexicon.getEntry(1147),
                                lexicon.getEntry(627),
                                lexicon.getEntry(611),
                                Language.getString("ShipGUI.3")
                        };
                        String[] range_color = new String[]{
                                "FFFFFF",
                                "00FF00",
                                "FFFF00",
                                "FF4020",
                                "0000FF"
                        };


                        byte[] tech_req;
                        ID[] ids;
                        int id, tmp, shLen = 0;
                        String stmp;
                        boolean found = false;

                        // find largest shield recharge value
                        for (int i = 0; i < shiplist.getNumberOfEntries(); i++)
                        {
                            tmp = Integer.toString(shiplist.getShieldRec(i)).length();

                            if (tmp > shLen)
                            {
                                shLen = tmp;
                            }
                        }

                        msg = Language.getString("HtmlReportGUI.10"); //$NON-NLS-1$
                        this.sendMessage(msg);

                        // get max tech level
                        int maxLvl = shiptech.getMaxTechLevel();

                        for (int empire = 0; empire < 5; empire++)
                        {
                            found = false;
                            
                            ids = shiplist.getIDs(empire);

                            report.append(strHeaderStart);

                            // empire
                            report.append(race.getName(empire));
                            report.append(strHeaderEnd);

                            report.append(strShipListTableHeadline);

                            // ascending order tech wise
                            for (byte max = 0; max <= maxLvl; max++)
                            {
                                if (found)
                                {
                                    report.append(strShipListTablePadding);
                                }
                                found = false;
                                
                                for (byte min = 0; min <= max; min++)
                                {
                                    for (int i = 0; i < ids.length; i++)
                                    {
                                        tech_req = shiptech.getTechLvls(ids[i].hashCode());

                                        if (max == Tools.getMax(tech_req) && min == Tools.getMin(tech_req))
                                        {
                                            found = true;

                                            id = ids[i].hashCode();

                                            report.append("<TR>\n");
                                            
                                            // ID
                                            report.append("<TD align=right>" + id + "</TD>\n");
                                            // name
                                            report.append("<TD align=center>" + shiplist.getName(id) + "</TD>\n");
                                            // class
                                            report.append("<TD align=center>" + shiplist.getClass(id) + "</TD>\n");
                                            // function
                                            tmp = shiplist.getFunction(id);

                                            if (tmp < 0 && tmp >= function.length)
                                            {
                                                report.append("<TD align=center> ? </TD>\n");
                                            }
                                            else
                                            {
                                                report.append("<TD align=center>" + function[tmp] + "</TD>\n");
                                            }
                                            // build cost
                                            report.append("<TD align=right bgcolor=silver>" + shiplist.getBuildCost(id) + "</TD>\n");
                                            // maintenance
                                            report.append("<TD align=right bgcolor=silver>" + shiplist.getMaintenance(id) + "</TD>\n");
                                            // shields
                                            stmp = Integer.toString(shiplist.getShieldRec(id));

                                            tmp = stmp.length();
                                            while (tmp < shLen)
                                            {
                                                stmp = "&nbsp " + stmp;
                                                tmp++;
                                            }

                                            report.append("<TD align=right><b>" + shiplist.getShieldStr(id)
                                                    + "</b> (" + stmp + ")</TD>\n");
                                            // hull
                                            report.append("<TD align=right><b>" + shiplist.getHullStr(id)
                                                    + "</b></TD>\n");
                                            // beams
                                            report.append("<TD align=right>"
                                                    + shiplist.getBeamNum(id)
                                                    + " x " + shiplist.getBeamDamage(id)
                                                    + "</TD>\n");
                                            // beam range
                                            report.append("<TD align=center>"
                                                    + shiplist.getBeamMin(id)
                                                    + " - " + shiplist.getBeamMax(id)
                                                    + "</TD>\n");
                                            // torps
                                            report.append("<TD align=right>"
                                                    + shiplist.getTorpNum(id)
                                                    + " x " + shiplist.getTorpDamage(id)
                                                    + "</TD>\n");
                                            // torps range
                                            report.append("<TD align=center>"
                                                    + shiplist.getTorpMin(id)
                                                    + " - " + shiplist.getTorpMax(id)
                                                    + "</TD>\n");
                                            // accuracy
                                            report.append("<TD align=center>"
                                                    + shiplist.getBeamAccu(id)
                                                    + " / " + shiplist.getTorpAccu(id)
                                                    + "</TD>\n");
                                            // defense
                                            report.append("<TD align=center>"
                                                    + shiplist.getDefense(id)
                                                    + "</TD>\n");
                                            // agility
                                            report.append("<TD align=center>"
                                                    + shiplist.getAgility(id)
                                                    + "</TD>\n");
                                            // range
                                            tmp = (int)shiplist.getMapRange(id) + 1;

                                            if (tmp < 0 && tmp >= function.length)
                                            {
                                                report.append("<TD align=center bgcolor=#"
                                                        + range_color[0] + "> ? </TD>\n");
                                            }
                                            else
                                            {
                                                report.append("<TD align=center bgcolor=#"
                                                        + range_color[tmp] + ">"
                                                        + range_plus1[tmp] + "</TD>\n");
                                            }
                                            // speed
                                            report.append("<TD align=center bgcolor=#"
                                                        + range_color[tmp] + ">"
                                                        + (int)shiplist.getMapSpeed(id)
                                                        + "</TD>\n");
                                            // scan
                                            report.append("<TD align=center>"
                                                    + shiplist.getScanRange(id) + "</TD>\n");
                                            // cloak
                                            tmp = shiplist.getCloak(id);

                                            if (tmp < 0 && tmp >= cloak.length)
                                            {
                                                report.append("<TD align=center>?</TD>\n");
                                            }
                                            else
                                            {
                                                report.append("<TD align=center>"
                                                        + cloak[tmp] + "</TD>\n");
                                            }
                                            // production
                                            tmp = shiplist.getProduction(id);

                                            if (tmp == 0)
                                            {
                                                report.append("<TD align=center> - </TD>\n");
                                            }
                                            else if (tmp < 0)
                                            {
                                                report.append("<TD align=center> ? </TD>\n");
                                            }
                                            else
                                            {
                                                report.append("<TD align=center>"
                                                        + tmp + "</TD>\n");
                                            }
                                            // tech req
                                            for (int t = 0; t < 6; t++)
                                            {
                                                report.append("<TD align=center bgcolor=#00E0FF><b>"
                                                        + tech_req[t] + "</b></TD>\n");
                                            }

                                            report.append("</TR>\n\n");
                                        }
                                    }
                                }
                            }

                            report.append("</table><br><br>\n\n");
                        }

                        // minors
                        String minor;
                        report.append(strHeaderStart);
                        report.append(Language.getString("HtmlReportGUI.12"));
                        report.append(strHeaderEnd);
                        report.append(strShipListMinorsTableHeadline);

                        for (int rid = 5; rid < race.getNumberOfEntries(); rid++)
                        {
                            minor = race.getName(rid);
                            ids = shiplist.getIDs(rid);

                            for (byte max = 0; max <= 11; max++)
                            {
                                for (byte min = 0; min <= max; min++)
                                {
                                    for (int i = 0; i < ids.length; i++)
                                    {
                                        tech_req = shiptech.getTechLvls(ids[i].hashCode());

                                        if (max == Tools.getMax(tech_req) && min == Tools.getMin(tech_req))
                                        {
                                            id = ids[i].hashCode();

                                            report.append("<TR>\n");

                                            // race
                                            report.append("<TD align=center>" + minor + "</TD>\n");
                                            // ID
                                            report.append("<TD align=right>" + id + "</TD>\n");
                                            // name
                                            report.append("<TD align=center>" + shiplist.getName(id) + "</TD>\n");
                                            // class
                                            report.append("<TD align=center>" + shiplist.getClass(id) + "</TD>\n");
                                            // function
                                            tmp = shiplist.getFunction(id);

                                            if (tmp < 0 && tmp >= function.length)
                                            {
                                                report.append("<TD align=center> ? </TD>\n");
                                            }
                                            else
                                            {
                                                report.append("<TD align=center>" + function[tmp] + "</TD>\n");
                                            }
                                            // build cost
                                            report.append("<TD align=right bgcolor=silver>" + shiplist.getBuildCost(id) + "</TD>\n");
                                            // maintenance
                                            report.append("<TD align=right bgcolor=silver>" + shiplist.getMaintenance(id) + "</TD>\n");
                                            // shields
                                            stmp = Integer.toString(shiplist.getShieldRec(id));

                                            tmp = stmp.length();
                                            while (tmp < shLen)
                                            {
                                                stmp = "&nbsp " + stmp;
                                                tmp++;
                                            }

                                            report.append("<TD align=right><b>" + shiplist.getShieldStr(id)
                                                    + "</b> (" + stmp + ")</TD>\n");
                                            // hull
                                            report.append("<TD align=right><b>" + shiplist.getHullStr(id)
                                                    + "</b></TD>\n");
                                            // beams
                                            report.append("<TD align=right>"
                                                    + shiplist.getBeamNum(id)
                                                    + " x " + shiplist.getBeamDamage(id)
                                                    + "</TD>\n");
                                            // beam range
                                            report.append("<TD align=center>"
                                                    + shiplist.getBeamMin(id)
                                                    + " - " + shiplist.getBeamMax(id)
                                                    + "</TD>\n");
                                            // torps
                                            report.append("<TD align=right>"
                                                    + shiplist.getTorpNum(id)
                                                    + " x " + shiplist.getTorpDamage(id)
                                                    + "</TD>\n");
                                            // torps range
                                            report.append("<TD align=center>"
                                                    + shiplist.getTorpMin(id)
                                                    + " - " + shiplist.getTorpMax(id)
                                                    + "</TD>\n");
                                            // accuracy
                                            report.append("<TD align=center>"
                                                    + shiplist.getBeamAccu(id)
                                                    + " / " + shiplist.getTorpAccu(id)
                                                    + "</TD>\n");
                                            // defense
                                            report.append("<TD align=center>"
                                                    + shiplist.getDefense(id)
                                                    + "</TD>\n");
                                            // agility
                                            report.append("<TD align=center>"
                                                    + shiplist.getAgility(id)
                                                    + "</TD>\n");
                                            // range
                                            tmp = (int)shiplist.getMapRange(id) + 1;

                                            if (tmp < 0 && tmp >= function.length)
                                            {
                                                report.append("<TD align=center bgcolor=#"
                                                        + range_color[0] + "> ? </TD>\n");
                                            }
                                            else
                                            {
                                                report.append("<TD align=center bgcolor=#"
                                                        + range_color[tmp] + ">"
                                                        + range_plus1[tmp] + "</TD>\n");
                                            }
                                            // speed
                                            report.append("<TD align=center bgcolor=#"
                                                        + range_color[tmp] + ">"
                                                        + (int)shiplist.getMapSpeed(id)
                                                        + "</TD>\n");
                                            // scan
                                            report.append("<TD align=center>"
                                                    + shiplist.getScanRange(id) + "</TD>\n");
                                            // cloak
                                            tmp = shiplist.getCloak(id);

                                            if (tmp < 0 && tmp >= cloak.length)
                                            {
                                                report.append("<TD align=center>?</TD>\n");
                                            }
                                            else
                                            {
                                                report.append("<TD align=center>"
                                                        + cloak[tmp] + "</TD>\n");
                                            }
                                            // production
                                            tmp = shiplist.getProduction(id);

                                            if (tmp == 0)
                                            {
                                                report.append("<TD align=center> - </TD>\n");
                                            }
                                            else if (tmp < 0)
                                            {
                                                report.append("<TD align=center> ? </TD>\n");
                                            }
                                            else
                                            {
                                                report.append("<TD align=center>"
                                                        + tmp + "</TD>\n");
                                            }
                                            // tech req
                                            for (int t = 0; t < 6; t++)
                                            {
                                                report.append("<TD align=center bgcolor=#00E0FF><b>"
                                                        + tech_req[t] + "</b></TD>\n");
                                            }

                                            report.append("</TR>\n\n");
                                        }
                                    }
                                }
                            }
                        }
                        report.append("</table><br><br>\n\n");

                        // aliens
                        report.append(strHeaderStart);
                        report.append(Language.getString("HtmlReportGUI.13"));
                        report.append(strHeaderEnd);

                        ids = shiplist.getIDs(race.getNumberOfEntries() + 1);

                        report.append(strShipListTableHeadline);

                        // ok
                        for (int i = 0; i < ids.length; i++)
                        {
                            tech_req = shiptech.getTechLvls(ids[i].hashCode());

                            id = ids[i].hashCode();

                            report.append("<TR>\n");

                            // ID
                            report.append("<TD align=right>" + id + "</TD>\n");
                            // name
                            report.append("<TD align=center>" + shiplist.getName(id) + "</TD>\n");
                            // class
                            report.append("<TD align=center>" + shiplist.getClass(id) + "</TD>\n");
                            // function
                            tmp = shiplist.getFunction(id);

                            if (tmp < 0 && tmp >= function.length)
                            {
                                report.append("<TD align=center> ? </TD>\n");
                            }
                            else
                            {
                                report.append("<TD align=center>" + function[tmp] + "</TD>\n");
                            }
                            // build cost
                            report.append("<TD align=right bgcolor=silver>" + shiplist.getBuildCost(id) + "</TD>\n");
                            // maintenance
                            report.append("<TD align=right bgcolor=silver>" + shiplist.getMaintenance(id) + "</TD>\n");
                            // shields
                            stmp = Integer.toString(shiplist.getShieldRec(id));

                            tmp = stmp.length();
                            while (tmp < shLen)
                            {
                                stmp = "&nbsp " + stmp;
                                tmp++;
                            }

                            report.append("<TD align=right><b>" + shiplist.getShieldStr(id)
                                    + "</b> (" + stmp + ")</TD>\n");
                            // hull
                            report.append("<TD align=right><b>" + shiplist.getHullStr(id)
                                    + "</b></TD>\n");
                            // beams
                            report.append("<TD align=right>"
                                    + shiplist.getBeamNum(id)
                                    + " x " + shiplist.getBeamDamage(id)
                                    + "</TD>\n");
                            // beam range
                            report.append("<TD align=center>"
                                    + shiplist.getBeamMin(id)
                                    + " - " + shiplist.getBeamMax(id)
                                    + "</TD>\n");
                            // torps
                            report.append("<TD align=right>"
                                    + shiplist.getTorpNum(id)
                                    + " x " + shiplist.getTorpDamage(id)
                                    + "</TD>\n");
                            // torps range
                            report.append("<TD align=center>"
                                    + shiplist.getTorpMin(id)
                                    + " - " + shiplist.getTorpMax(id)
                                    + "</TD>\n");
                            // accuracy
                            report.append("<TD align=center>"
                                    + shiplist.getBeamAccu(id)
                                    + " / " + shiplist.getTorpAccu(id)
                                    + "</TD>\n");
                            // defense
                            report.append("<TD align=center>"
                                    + shiplist.getDefense(id)
                                    + "</TD>\n");
                            // agility
                            report.append("<TD align=center>"
                                    + shiplist.getAgility(id)
                                    + "</TD>\n");
                            // range
                            tmp = (int)shiplist.getMapRange(id) + 1;

                            if (tmp < 0 && tmp >= function.length)
                            {
                                report.append("<TD align=center bgcolor=#"
                                        + range_color[0] + "> ? </TD>\n");
                            }
                            else
                            {
                                report.append("<TD align=center bgcolor=#"
                                        + range_color[tmp] + ">"
                                        + range_plus1[tmp] + "</TD>\n");
                            }
                            // speed
                            report.append("<TD align=center bgcolor=#"
                                        + range_color[tmp] + ">"
                                        + (int)shiplist.getMapSpeed(id)
                                        + "</TD>\n");
                            // scan
                            report.append("<TD align=center>"
                                    + shiplist.getScanRange(id) + "</TD>\n");
                            // cloak
                            tmp = shiplist.getCloak(id);

                            if (tmp < 0 && tmp >= cloak.length)
                            {
                                report.append("<TD align=center>?</TD>\n");
                            }
                            else
                            {
                                report.append("<TD align=center>"
                                        + cloak[tmp] + "</TD>\n");
                            }
                            // production
                            tmp = shiplist.getProduction(id);

                            if (tmp == 0)
                            {
                                report.append("<TD align=center> - </TD>\n");
                            }
                            else if (tmp < 0)
                            {
                                report.append("<TD align=center> ? </TD>\n");
                            }
                            else
                            {
                                report.append("<TD align=center>"
                                        + tmp + "</TD>\n");
                            }
                            // tech req
                            for (int t = 0; t < 6; t++)
                            {
                                report.append("<TD align=center bgcolor=#00E0FF><b>"
                                        + tech_req[t] + "</b></TD>\n");
                            }

                            report.append("</TR>\n\n");
                        }
                        report.append("</table><br><br>\n\n");

                        break;
                    }
                }

                RESULT = report.toString();
            }
            catch (Exception e)
            {
                this.reportError(e);
                RESULT = e.getLocalizedMessage();
                return RESULT;
            }
            
            this.updateStatus(UEWorker.STATUS_DONE);
            return RESULT;
        }
    }
}
