/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: MaxPopOrBonHUI.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui.stbof;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import ue.*;
import ue.edit.res.stbof.bin.*;
import ue.edit.res.stbof.est.*;
import ue.gui.*;
import ue.util.ID;

/**Used to edit either planet max populations or planet bonuses.*/
@SuppressWarnings("serial")  //$NON-NLS-1$
public class MaxPopOrBonGUI extends MainPanel{
	private Planspac SPACE = null;
	private Planboni BONI = null;
	private Environ ENVI = null;
	
	private JLabel lblPlanType = new JLabel(Language.getString("MaxPopOrBonGUI.0")); //$NON-NLS-1$
	private JLabel lblPlanSize = new JLabel(Language.getString("MaxPopOrBonGUI.1")); //$NON-NLS-1$
	private JComboBox jcbPlanType = new JComboBox();
	private JComboBox jcbPlanSize = new JComboBox();
	
	private JLabel lblDesc;
	private JLabel[] lblBoni = new JLabel[3];
	private JSpinner[] jspValue = new JSpinner[9];
	
	private JLabel[] lblUni = new JLabel[3];
	
	private int SELECTED_TYPE = -1;
	private int SELECTED_SIZE = -1;
	
	/**Use this constructor if you want to edit max populations.
	@param spac the file to be edited.*/
	public MaxPopOrBonGUI(Planspac spac, Environ env){
		SPACE = spac;
		ENVI = env;
		if (SPACE==null || ENVI==null) {
			return;
		}
		Font def = UE.SETTINGS.getDefaultFont();
		lblDesc = new JLabel(Language.getString("MaxPopOrBonGUI.2").trim()); //$NON-NLS-1$
		lblBoni[0] = new JLabel(Language.getString("MaxPopOrBonGUI.3").trim()); //$NON-NLS-1$
		lblBoni[1] = new JLabel(Language.getString("MaxPopOrBonGUI.4").trim()); //$NON-NLS-1$
		lblBoni[2] = new JLabel(Language.getString("MaxPopOrBonGUI.5").trim()); //$NON-NLS-1$
		lblDesc.setFont(def);
		lblBoni[0].setFont(def);
		lblBoni[1].setFont(def);
		lblBoni[2].setFont(def);

		//listeners
		for (int i = 0; i < 9; i++)
        {
			jspValue[i] = new JSpinner(new SpinnerNumberModel(new Integer(0), new Integer(0), new Integer(Integer.MAX_VALUE), new Integer(1)));
			//[i].setPreferredSize(new Dimension(60, jspValue[i].getPreferredSize().height));
			jspValue[i].setFont(def);
		}
		stage_two();
	}
	/**Use this constructor if you want to edit planet bonuses.
	@param bon the file to be edited.*/
	public MaxPopOrBonGUI(Planboni bon, Environ env){
		BONI = bon;
		ENVI = env;
		if (BONI==null || ENVI==null) {
			return;
		}
		lblDesc = new JLabel(Language.getString("MaxPopOrBonGUI.6").trim()); //$NON-NLS-1$
		lblBoni[0] = new JLabel(Language.getString("MaxPopOrBonGUI.7").trim()); //$NON-NLS-1$
		lblBoni[1] = new JLabel(Language.getString("MaxPopOrBonGUI.8").trim()); //$NON-NLS-1$
		lblBoni[2] = new JLabel(Language.getString("MaxPopOrBonGUI.9").trim()); //$NON-NLS-1$
		Font def = UE.SETTINGS.getDefaultFont();
		lblDesc.setFont(def);
		lblBoni[0].setFont(def);
		lblBoni[1].setFont(def);
		lblBoni[2].setFont(def);
		//listeners
		for (int i=0; i<9; i++){
			jspValue[i] = new JSpinner(new SpinnerNumberModel(new Float(0), new Float(-1000), new Float(1000), new Float(0.05)));
			jspValue[i].setFont(def);
			// [i].setPreferredSize(new Dimension(60, jspValue[i].getPreferredSize().height));
		}
		stage_two();
	}
	
	private void stage_two(){
		Font def = UE.SETTINGS.getDefaultFont();
		lblUni[0] = new JLabel(Language.getString("MaxPopOrBonGUI.10")); //$NON-NLS-1$
		lblUni[1] = new JLabel(Language.getString("MaxPopOrBonGUI.11")); //$NON-NLS-1$
		lblUni[2] = new JLabel(Language.getString("MaxPopOrBonGUI.12")); //$NON-NLS-1$
		lblUni[0].setFont(def);
		lblUni[1].setFont(def);
		lblUni[2].setFont(def);
		lblPlanType.setFont(def);
		lblPlanSize.setFont(def);
		jcbPlanType.setFont(def);
		jcbPlanSize.setFont(def);
		jcbPlanType.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent e){
				//save
                try
                {
                    finalWarning();
                }
                catch (Exception ex)
                {
                    // really bad stuff happening - bail
                    UE.displayError(ex);
                    return;
                }

				SELECTED_TYPE = new Integer(jcbPlanType.getSelectedIndex());
				SELECTED_SIZE = new Integer(jcbPlanSize.getSelectedIndex());
				if (SELECTED_TYPE>=0 && SELECTED_SIZE>=0){
					try{
						if (SPACE!=null){
							if (!SPACE.contains((byte)SELECTED_TYPE,(byte)SELECTED_SIZE)) {
								SPACE.add((byte)SELECTED_TYPE,(byte)SELECTED_SIZE);
							}
							int[] ai = SPACE.getValues((byte)SELECTED_TYPE,(byte)SELECTED_SIZE);
							for (int i=0; i<9; i++){
								jspValue[i].setValue(new Integer(ai[i]));
							}
						}else{
							if (!BONI.contains((byte)SELECTED_TYPE,(byte)SELECTED_SIZE)) {
								BONI.add((byte)SELECTED_TYPE,(byte)SELECTED_SIZE);
							}
							int[] ai = BONI.getValues((byte)SELECTED_TYPE,(byte)SELECTED_SIZE);
							for (int i=0; i<9; i++){
								jspValue[i].setValue(new Float(ai[i] * 0.05));
							}
						}
					}catch (Exception rf){
						UE.displayError(rf);
					}
				}
			}
		});
		jcbPlanSize.addActionListener(new ActionListener()
        {
            @Override
			public void actionPerformed(ActionEvent e)
            {
				//save
                try
                {
    				finalWarning();
                }
                catch (Exception ex)
                {
                    // really bad stuff happening - bail
                    UE.displayError(ex);
                    return;
                }

                SELECTED_TYPE = new Integer(jcbPlanType.getSelectedIndex());
				SELECTED_SIZE = new Integer(jcbPlanSize.getSelectedIndex());
				if (SELECTED_TYPE>=0 && SELECTED_SIZE>=0){
					try{
						if (SPACE!=null){
							if (!SPACE.contains((byte)SELECTED_TYPE,(byte)SELECTED_SIZE)) {
								SPACE.add((byte)SELECTED_TYPE,(byte)SELECTED_SIZE);
							}
							int[] ai = SPACE.getValues((byte)SELECTED_TYPE,(byte)SELECTED_SIZE);
							for (int i=0; i<9; i++){
								jspValue[i].setValue(new Integer(ai[i]));
							}
						}else{
							if (!BONI.contains((byte)SELECTED_TYPE,(byte)SELECTED_SIZE)) {
								BONI.add((byte)SELECTED_TYPE,(byte)SELECTED_SIZE);
							}
							int[] ai = BONI.getValues((byte)SELECTED_TYPE,(byte)SELECTED_SIZE);
							for (int i=0; i<9; i++){
								jspValue[i].setValue(new Float(ai[i] * 0.05));
							}
						}
					}catch (Exception rf){
						UE.displayError(rf);
					}
				}
			}
		});
		//lay
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets=new Insets(5, 5, 5, 5);
		c.fill=GridBagConstraints.BOTH;
		c.anchor=GridBagConstraints.WEST;
		//labels
		c.gridx=0;
		c.gridy=0;
		c.gridwidth=1;
		c.gridheight=1;
		add(lblPlanType, c);
		c.gridx=1;
		add(jcbPlanType, c);
		c.gridx=2;
		add(lblPlanSize, c);
		c.gridx=3;
		add(jcbPlanSize, c);
		c.gridy=2;
		c.gridx=0;
		add(lblDesc, c);
		c.gridx=1;
		add(lblUni[0], c);
		c.gridx=2;
		add(lblUni[1], c);
		c.gridx=3;
		add(lblUni[2], c);
		c.gridy=3;
		c.gridx=0;
		add(lblBoni[0], c);
		c.gridx=1;
		c.anchor=GridBagConstraints.CENTER;
		c.fill=GridBagConstraints.BOTH;
		add(jspValue[0], c);
		c.gridx=2;
		add(jspValue[1], c);
		c.gridx=3;
		add(jspValue[2], c);
		c.anchor=GridBagConstraints.WEST;
		c.fill=GridBagConstraints.BOTH;
		c.gridy=4;
		c.gridx=0;
		add(lblBoni[1], c);
		c.gridx=1;
		c.anchor=GridBagConstraints.CENTER;
		c.fill=GridBagConstraints.BOTH;
		add(jspValue[3], c);
		c.gridx=2;
		add(jspValue[4], c);
		c.gridx=3;
		add(jspValue[5], c);
		c.anchor=GridBagConstraints.WEST;
		c.fill=GridBagConstraints.BOTH;
		c.gridy=5;
		c.gridx=0;
		add(lblBoni[2], c);
		c.gridx=1;
		c.anchor=GridBagConstraints.CENTER;
		c.fill=GridBagConstraints.BOTH;
		add(jspValue[6], c);
		c.gridx=2;
		add(jspValue[7], c);
		c.gridx=3;
		add(jspValue[8], c);
		fillList();
	}
	
	private void fillList(){
		//types
		String[] str = ENVI.toArray();
		jcbPlanType.removeAllItems();

        for (int i = 0; i < str.length && i < 8; i++)
        {
			jcbPlanType.addItem(new ID(str[i].trim(), i));
		}
		//sizes
		jcbPlanSize.removeAllItems();
		jcbPlanSize.addItem(new ID(Language.getString("MaxPopOrBonGUI.13").trim(), 0)); //$NON-NLS-1$
		jcbPlanSize.addItem(new ID(Language.getString("MaxPopOrBonGUI.14").trim(), 1)); //$NON-NLS-1$
		jcbPlanSize.addItem(new ID(Language.getString("MaxPopOrBonGUI.15").trim(), 2)); //$NON-NLS-1$
	}
	
	@Override
	public void finalWarning() throws Exception
    {
		if (SELECTED_SIZE < 0 || SELECTED_TYPE < 0)
        {
			return;
		}

        for (int index = 0; index < 9; index++)
        {
            jspValue[index].commitEdit();
            if (SPACE == null)
            {
                Float vi = (Float)jspValue[index].getValue();
                int boni1 = index%3;
                int boni2 = ((index-boni1)/3);//planet bonus
                BONI.set((byte)SELECTED_TYPE,(byte)SELECTED_SIZE, boni2, boni1, (int)(vi.floatValue()*100000)/4999);
                int ai = BONI.get((byte)SELECTED_TYPE,(byte)SELECTED_SIZE, boni2, boni1);
                jspValue[index].setValue(new Float(ai * 0.05));
            }
            else
            {
                Integer vi = (Integer)jspValue[index].getValue();
                int boni1 = index%3;//universal
                int boni2 = ((index-boni1)/3); //population
                SPACE.set((byte)SELECTED_TYPE,(byte)SELECTED_SIZE, boni1, boni2, vi.intValue());
                int ai = SPACE.get((byte)SELECTED_TYPE,(byte)SELECTED_SIZE, boni1, boni2);
                jspValue[index].setValue(new Integer(ai));
            }
        }
	}
}
