/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	btnAPPLY WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: ObjstrucGUI.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui.stbof;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import ue.edit.res.stbof.smt.*;
import ue.edit.res.stbof.dic.*;
import ue.gui.*;
import ue.util.SettingsManager;
import ue.*;

@SuppressWarnings("serial")  //$NON-NLS-1$
public class ObjstrucGUI extends MainPanel{
	private Objstruc OBJECT;
	private Lexicon LEXICON;
	private int INDEX = -1;
	private JComboBox cmbOBJECTS = new JComboBox(); //indexes
	private JTextField txtGRAPH = new JTextField(10); //graphic file (e.g. blkh)
	private JTextField txtFREQ = new JTextField(10); //frequency
	private JLabel GRAPHICS = new JLabel(); //listed graphic files used
	private JLabel tip1 = new JLabel(Language.getString("ObjstrucGUI.0")); //$NON-NLS-1$
	private JLabel tip2 = new JLabel(Language.getString("ObjstrucGUI.1")); //$NON-NLS-1$
	private JButton btnAPPLY = new JButton(Language.getString("ObjstrucGUI.2")); //$NON-NLS-1$
	private JLabel lblSUM = new JLabel();
	// strings
	private static String strSUM = Language.getString("ObjstrucGUI.3"); //$NON-NLS-1$
	
	public ObjstrucGUI(Objstruc l, Lexicon lec){
		if (l == null || lec == null) {
			return;
		}
		OBJECT = l;
		LEXICON = lec;
		//set font
		Font def = UE.SETTINGS.getDefaultFont();
		cmbOBJECTS.setFont(def);
		txtGRAPH.setFont(def);
		cmbOBJECTS.setFont(def);
		txtFREQ.setFont(def);
		GRAPHICS.setFont(def);
		tip1.setFont(def);
		tip2.setFont(def);
		btnAPPLY.setFont(def);
		lblSUM.setFont(def);
		//action
		cmbOBJECTS.addActionListener( new ActionListener () {
			public void actionPerformed(ActionEvent ae){
				try{
					INDEX = cmbOBJECTS.getSelectedIndex();
					if (INDEX!=-1) {
							//graphics
							txtGRAPH.setText(OBJECT.getGraph(INDEX));
							GRAPHICS.setText(Language.getString("ObjstrucGUI.4") + " " //$NON-NLS-1$ //$NON-NLS-2$
							+ txtGRAPH.getText() + ".ani " //$NON-NLS-1$
							+ txtGRAPH.getText() + "map.tga " //$NON-NLS-1$
							+ txtGRAPH.getText() + "maps.tga"); //$NON-NLS-1$
							//frequency
							txtFREQ.setText(Float.toString(OBJECT.getFreq(INDEX)));
							lblSUM.setText(strSUM.replace("%1", Float.toString(OBJECT.allFreq()))); //$NON-NLS-1$
					}
				}catch (Exception g){
					UE.displayError(g);
				}
			}
		});
		btnAPPLY.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				if (INDEX > -1){
					try{
						OBJECT.setFreq(INDEX, Float.parseFloat(txtFREQ.getText()));
						OBJECT.setGraph(INDEX, txtGRAPH.getText());
						GRAPHICS.setText(Language.getString("ObjstrucGUI.4") + " " //$NON-NLS-1$ //$NON-NLS-2$
						+ txtGRAPH.getText() + ".ani " //$NON-NLS-1$
						+ txtGRAPH.getText() + "map.tga " //$NON-NLS-1$
						+ txtGRAPH.getText() + "maps.tga"); //$NON-NLS-1$
						lblSUM.setText(strSUM.replace("%1", Float.toString(OBJECT.allFreq()))); //$NON-NLS-1$
					}catch (Exception g){
						UE.displayError(g);
					}
				}
			}
		});
		//set up
		try{
			cmbOBJECTS.addItem(LEXICON.getEntry(94));
			cmbOBJECTS.addItem(LEXICON.getEntry(1450));
			cmbOBJECTS.addItem(LEXICON.getEntry(802));
			cmbOBJECTS.addItem(LEXICON.getEntry(810));
			cmbOBJECTS.addItem(LEXICON.getEntry(987));
			cmbOBJECTS.addItem(LEXICON.getEntry(889));
			cmbOBJECTS.addItem(LEXICON.getEntry(1452));
			cmbOBJECTS.addItem(LEXICON.getEntry(1446));
			cmbOBJECTS.addItem(LEXICON.getEntry(420));
			cmbOBJECTS.addItem(LEXICON.getEntry(96));
			cmbOBJECTS.addItem(LEXICON.getEntry(1447));
			cmbOBJECTS.addItem(LEXICON.getEntry(968));
		}catch (Exception tz){
			UE.displayError(tz);
			return;
		}
		//lay
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets=new Insets(5, 5, 5, 5);
		c.fill=GridBagConstraints.BOTH;
		c.gridwidth = 2;
		c.gridheight = 1;
		c.gridx = 0;
		c.gridy = 0;
		add(cmbOBJECTS, c);
		c.gridy = 1;
		add(tip1, c);
		c.gridy = 2;
		add(txtGRAPH, c);
		c.gridy = 3;
		add(GRAPHICS, c);
		c.gridy = 4;
		c.gridwidth = 1;
		add(tip2, c);
		c.gridy = 5;
		c.gridwidth = 1;
		add(txtFREQ, c);
		c.gridx = 1;
		add(btnAPPLY, c);
		c.gridwidth = 2;
		c.gridx = 0;
		c.gridy = 6;
		add(lblSUM, c);
	}
	
	@Override
	public void finalWarning() throws Exception
    {
		if (INDEX > -1)
        {
            OBJECT.setFreq(INDEX, Float.parseFloat(txtFREQ.getText()));
		}
	}
}
