/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: PlanetGUI.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui.stbof;

//standard
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.io.*;
//user defined
import ue.util.*;
import ue.gui.*;
import ue.edit.res.*;
import ue.edit.res.stbof.bin.*;
import ue.edit.res.stbof.pst.*;
import ue.edit.res.stbof.est.*;
import ue.exception.*;
import ue.*;
import ue.edit.res.stbof.rst.Race;

@SuppressWarnings("serial")  //$NON-NLS-1$
public class PlanetGUI extends MainPanel{
	private Planet PLANET;
	private Planspac SPACE;
	private Planboni BONI;
	private Race RACE;
	private Meplanet MEPLANET;
	private Stbof STBOF;
	private Environ ENVIRONMENT;
	private short SELECTED = -1;
	private boolean PAUSE = false;
	//elements
	private JLabel[] LABEL = new JLabel[6];
	private JButton btnEDIT = new JButton(Language.getString("PlanetGUI.0")); //$NON-NLS-1$
	private JButton btnADD = new JButton("Add");
	private JButton btnREMOVE = new JButton("Remove");
        private JComboBox jcbMASK = new JComboBox();
	private JComboBox jcbANI = new JComboBox();
	private JComboBox jcbTYPE = new JComboBox();
	private JComboBox jcbSIZE = new JComboBox();
	private JComboBox[] jcbBONUS = new JComboBox[3];
	private AniLabel alANIM = new AniLabel();
	private JLabel jlPOP = new JLabel();
	private JLabel jlBONI = new JLabel();
	private JLabel jlINDEX = new JLabel();
	private JTextField txtNAME = new JTextField(12);
	private JList lstPLANET = new JList();
	private JComboBox jcbATMO = new JComboBox();
	// strings
	private static String strMAX_POP = Language.getString("PlanetGUI.2"); //$NON-NLS-1$
	private static String strBONUS = Language.getString("PlanetGUI.3"); //$NON-NLS-1$
	private static String strINDEX = Language.getString("PlanetGUI.4"); //$NON-NLS-1$
	
	/**constructor*/
	public PlanetGUI(Planet pl, Planspac sp, Planboni bon, Stbof st, Environ en){
		//associate
		if (pl==null || sp==null || bon==null || st==null || en==null) {
			return;
		}
		PLANET = pl;
		SPACE = sp;
		BONI = bon;
		STBOF = st;
		ENVIRONMENT = en;

                try
                {
                    RACE = (Race)STBOF.getInternalFile(Stbof.fileRACE_RST, true);
                    MEPLANET = (Meplanet)STBOF.getInternalFile(Stbof.fileMEPLANET_BIN, true);
                }
                catch (Exception ex)
                {
                    UE.displayError(ex);
                    return;
                }
                
		//build elements
		LABEL[0] = new JLabel(Language.getString("PlanetGUI.5")); //$NON-NLS-1$
		LABEL[1] = new JLabel(Language.getString("PlanetGUI.6")); //$NON-NLS-1$
		LABEL[2] = new JLabel(Language.getString("PlanetGUI.7")); //$NON-NLS-1$
		LABEL[3] = new JLabel(Language.getString("PlanetGUI.8")); //$NON-NLS-1$
		LABEL[4] = new JLabel(Language.getString("PlanetGUI.9")); //$NON-NLS-1$
		LABEL[5] = new JLabel(Language.getString("PlanetGUI.10")); //$NON-NLS-1$
		jcbBONUS[0] = new JComboBox();
		jcbBONUS[1] = new JComboBox();
		jcbBONUS[2] = new JComboBox();
		//set font
		Font def = UE.SETTINGS.getDefaultFont();
		for (int i=0; i<6; i++){
			LABEL[i].setFont(def);
			if (i<3) {
				jcbBONUS[i].setFont(def);
			}
		}
		btnEDIT.setFont(def);
		jcbMASK.setFont(def);
		jcbANI.setFont(def);
		jcbTYPE.setFont(def);
		jcbSIZE.setFont(def);
		jlPOP.setFont(def);
		jlBONI.setFont(def);
		jlINDEX.setFont(def);
		lstPLANET.setFont(def);
		txtNAME.setFont(def);
		jcbATMO.setFont(def);
		btnADD.setFont(def);
		btnREMOVE.setFont(def);

		//add listeners
		jcbANI.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent ae){
				if (SELECTED!=-1) {
					try{
						int index = PLANET.getIndex(SELECTED);
						int lol = jcbANI.getSelectedIndex();
						if (lol==-1) {
							return;
						}
						PLANET.setAnimation(index, (String)jcbANI.getSelectedItem());
						jcbANI.setSelectedItem(PLANET.getAnimation(index));
						alANIM.setAnimation(STBOF.getAnimation(PLANET.getAnimation(index), true), true);
					}catch (InvalidArgumentsException iae){
						UE.displayError(iae);
					}catch (Exception fnf){
						UE.displayError(fnf);
						alANIM.setAnimation(null, false);
					}
				}
			}
		});
		jcbTYPE.addActionListener( new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent ae){
				if (SELECTED!=-1) {
					try{
						int index = PLANET.getIndex(SELECTED);
						byte lol = (byte)jcbTYPE.getSelectedIndex();
						if (lol==-1) {
							return;
						}
						if (PLANET.setType(index, lol)){
							PLANET.sortEntries();
							fillPlanets();
						}
						index = PLANET.getIndex(SELECTED);
						int pot = SPACE.get(PLANET.getType(index), (byte)PLANET.getSize(index),
						PLANET.getUniBoni(index), PLANET.getPopBoni(index));
						jlPOP.setText(strMAX_POP.replace("%1", Integer.toString(pot))); //$NON-NLS-1$
						float pov = BONI.get(PLANET.getType(index), (byte)PLANET.getSize(index),
						PLANET.getPltBoni(index), PLANET.getUniBoni(index));
						jlBONI.setText(strBONUS.replace("%1", Integer.toString((int)pov*5))); //$NON-NLS-1$
					}catch (Exception fnf){
						UE.displayError(fnf);
					}
				}
			}
		});
		jcbATMO.addActionListener( new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				if (SELECTED!=-1) {
					try{
						int index = PLANET.getIndex(SELECTED);
						byte lol = (byte)jcbATMO.getSelectedIndex();
						if (lol==-1) {
							return;
						}
						if (PLANET.setAtmosphere(index, (byte)(lol+8))){
							PLANET.sortEntries();
							fillPlanets();
						}
					}catch (Exception fnf){
						UE.displayError(fnf);
					}
				}
			}
		});
		jcbSIZE.addActionListener( new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				if (SELECTED!=-1) {
					try{
						int index = PLANET.getIndex(SELECTED);
						short lol = (short)jcbSIZE.getSelectedIndex();
						if (lol==-1) {
							return;
						}
						if (PLANET.setSize(index, lol)){
							PLANET.sortEntries();
							fillPlanets();
						}
						index = PLANET.getIndex(SELECTED);
						int pot = SPACE.get(PLANET.getType(index), (byte)PLANET.getSize(index),
						PLANET.getUniBoni(index), PLANET.getPopBoni(index));
						jlPOP.setText(strMAX_POP.replace("%1", Integer.toString(pot))); //$NON-NLS-1$
						float pov = BONI.get(PLANET.getType(index), (byte)PLANET.getSize(index),
						PLANET.getPltBoni(index), PLANET.getUniBoni(index));
						jlBONI.setText(strBONUS.replace("%1", Integer.toString((int)pov*5))); //$NON-NLS-1$
					}catch (Exception fnf){
						UE.displayError(fnf);
					}
				}
			}
		});
		jcbBONUS[0].addActionListener( new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				if (SELECTED!=-1) {
					try{
						int index = PLANET.getIndex(SELECTED);
						byte lol = (byte)jcbBONUS[0].getSelectedIndex();
						if (lol==-1) {
							return;
						}
						PLANET.setPltBoni(index, lol);
						jcbBONUS[0].setSelectedItem(PLANET.getPltBoni(index));
						int pot = SPACE.get(PLANET.getType(index), (byte)PLANET.getSize(index),
						PLANET.getUniBoni(index), PLANET.getPopBoni(index));
						jlPOP.setText(strMAX_POP.replace("%1", Integer.toString(pot))); //$NON-NLS-1$
						float pov = BONI.get(PLANET.getType(index), (byte)PLANET.getSize(index),
						PLANET.getPltBoni(index), PLANET.getUniBoni(index));
						jlBONI.setText(strBONUS.replace("%1", Integer.toString((int)pov*5))); //$NON-NLS-1$
					}catch (Exception fnf){
						UE.displayError(fnf);
					}
				}
			}
		});
		jcbBONUS[1].addActionListener( new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				if (SELECTED!=-1) {
					try{
						int index = PLANET.getIndex(SELECTED);
						byte lol = (byte)jcbBONUS[1].getSelectedIndex();
						if (lol==-1) {
							return;
						}
						PLANET.setUniBoni(index, lol);
						jcbBONUS[1].setSelectedItem(PLANET.getUniBoni(index));
						int pot = SPACE.get(PLANET.getType(index), (byte)PLANET.getSize(index),
						PLANET.getUniBoni(index), PLANET.getPopBoni(index));
						jlPOP.setText(strMAX_POP.replace("%1", Integer.toString(pot))); //$NON-NLS-1$
						float pov = BONI.get(PLANET.getType(index), (byte)PLANET.getSize(index),
						PLANET.getPltBoni(index), PLANET.getUniBoni(index));
						jlBONI.setText(strBONUS.replace("%1", Integer.toString((int)pov*5))); //$NON-NLS-1$
					}catch (Exception fnf){
						UE.displayError(fnf);
					}
				}
			}
		});
		jcbBONUS[2].addActionListener( new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				if (SELECTED>=0) {
					try{
						int index = PLANET.getIndex(SELECTED);
						byte lol = (byte)jcbBONUS[2].getSelectedIndex();
						if (lol==-1) {
							return;
						}
						PLANET.setPopBoni(index, lol);
						jcbBONUS[2].setSelectedItem(PLANET.getPopBoni(index));
						int pot = SPACE.get(PLANET.getType(index), (byte)PLANET.getSize(index),
						PLANET.getUniBoni(index), PLANET.getPopBoni(index));
						jlPOP.setText(strMAX_POP.replace("%1", Integer.toString(pot))); //$NON-NLS-1$
						float pov = BONI.get(PLANET.getType(index), (byte)PLANET.getSize(index),
						PLANET.getPltBoni(index), PLANET.getUniBoni(index));
						jlBONI.setText(strBONUS.replace("%1", Integer.toString((int)pov*5))); //$NON-NLS-1$
					}catch (Exception fnf){
						UE.displayError(fnf);
					}
				}
			}
		});
		jcbMASK.addActionListener( new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent ae){
				fillPlanets();
			}
		});
		lstPLANET.addListSelectionListener(new ListSelectionListener(){
            @Override
			public void valueChanged(ListSelectionEvent e){
				if (lstPLANET.getSelectedIndex()<0) {
					return;
				}
				//save
                try
                {
                    finalWarning();
                }
                catch (Exception ex)
                {
                    // really bad stuff happening - bail
                    UE.displayError(ex);
                    return;
                }
				//show
				ID hm = (ID)lstPLANET.getSelectedValue();
				SELECTED = (short)hm.hashCode();
				
				if (SELECTED>=0)
                {
                    btnREMOVE.setEnabled(true);
                                    
					try{
						int index = PLANET.getIndex(SELECTED);
						txtNAME.setText(PLANET.getName(index));
						jcbTYPE.setSelectedIndex(PLANET.getType(index));
						jcbATMO.setSelectedIndex(PLANET.getAtmosphere(index)-8);
						jcbSIZE.setSelectedIndex(PLANET.getSize(index));
						jlINDEX.setText(strINDEX.replace("%1", Integer.toString(SELECTED))); //$NON-NLS-1$
						jcbANI.setSelectedItem(PLANET.getAnimation(index));
						jcbBONUS[0].setSelectedIndex(PLANET.getPltBoni(index));
						jcbBONUS[1].setSelectedIndex(PLANET.getUniBoni(index));
						jcbBONUS[2].setSelectedIndex(PLANET.getPopBoni(index));
						alANIM.setAnimation(STBOF.getAnimation(PLANET.getAnimation(index), true), true);
						int pot = SPACE.get(PLANET.getType(index), (byte)PLANET.getSize(index),
						PLANET.getUniBoni(index), PLANET.getPopBoni(index));
						jlPOP.setText(strMAX_POP.replace("%1", Integer.toString(pot))); //$NON-NLS-1$
						float pov = BONI.get(PLANET.getType(index), (byte)PLANET.getSize(index),
						PLANET.getPltBoni(index), PLANET.getUniBoni(index));
						jlBONI.setText(strBONUS.replace("%1", Integer.toString((int)pov*5))); //$NON-NLS-1$
					}catch (InvalidArgumentsException iae){
						UE.displayError(iae);
					}catch (Exception fnf){
						UE.displayError(fnf);
						alANIM.setAnimation(null, false);
					}
				}
				if (!PAUSE) {
					fillPlanets();
				}
			}
		});
                btnADD.addActionListener(new ActionListener()
                {
                    @Override
                    public void actionPerformed(ActionEvent arg0)
                    {
                        try {
                            byte type = (byte)(jcbMASK.getSelectedIndex() - 1);

                            if (type < 0)
                            {
                                PLANET.addPlanet((byte)Math.min(ENVIRONMENT.entries(), 7));
                            }
                            else
                            {
                                PLANET.addPlanet(type);
                            }

                            fillPlanets();

                            ID[] hm = PLANET.getIDs(jcbMASK.getSelectedIndex());
                            
                            lstPLANET.setSelectedIndex(hm.length - 1);
                            lstPLANET.ensureIndexIsVisible(hm.length - 1);
                        } catch (Exception ex) {
                            UE.displayError(ex);
                        }
                    }
                });
                btnREMOVE.addActionListener(new ActionListener()
                {
                    @Override
                    public void actionPerformed(ActionEvent arg0)
                    {
                        if (SELECTED >= 0)
                        {
                            try
                            {
                                // check minor race usage
                                for (short r = 5; r < RACE.getNumberOfEntries(); r++)
                                {
                                    if (RACE.getPlanet(r) == SELECTED)
                                    {
                                        JOptionPane.showMessageDialog(
                                                null,
                                                "This is the home planet of the %1.".replace("%1", RACE.getName(r)),
                                                "Can not remove planet",
                                                JOptionPane.INFORMATION_MESSAGE);
                                        return;
                                    }
                                }

                                //check major race usage
                                String[] planets;
                                int index = PLANET.getIndex(SELECTED);
                                String p = PLANET.getName(index);

                                for (short r = 0; r < 5; r++)
                                {
                                    planets = MEPLANET.getPlanets(RACE.getName(r));

                                    if (planets != null)
                                    {
                                        for (String planet : planets)
                                        {
                                            if (planet.equals(p))
                                            {
                                                JOptionPane.showMessageDialog(
                                                    null,
                                                    "This planet belongs to the home system of the %1.".replace("%1", RACE.getName(r)),
                                                    "Can not remove planet",
                                                    JOptionPane.INFORMATION_MESSAGE);
                                                return;
                                            }
                                        }
                                    }
                                }

                                PLANET.removePlanet(SELECTED);
                                fillPlanets();
                            } catch (Exception ex) {
                                UE.displayError(ex);
                            }
                        }
                    }
                });

		//fill combos
		Enumeration<String[]> enu = STBOF.getFiles(new java.io.FilenameFilter(){
                        @Override
			public boolean accept(File dir, String name){
				String str = new String(name);
				if (str.endsWith(".ani")) { //$NON-NLS-1$
					return true;
				} else {
					return false; //$NON-NLS-1$
				}
			}
		});
		String str = Language.getString("PlanetGUI.11"); //$NON-NLS-1$
		while (enu.hasMoreElements()){
			str = enu.nextElement()[0];
			jcbANI.addItem(str);
		}
		String[] str2 = ENVIRONMENT.toArray();
		for (int i=0; i<str2.length && i<8; i++){
			jcbTYPE.addItem(new ID(str2[i], i));
		}
		for (int i=8; i<str2.length; i++){
			jcbATMO.addItem(new ID(str2[i], i));
		}
		jcbSIZE.addItem(Language.getString("PlanetGUI.12")); //$NON-NLS-1$
		jcbSIZE.addItem(Language.getString("PlanetGUI.13")); //$NON-NLS-1$
		jcbSIZE.addItem(Language.getString("PlanetGUI.14")); //$NON-NLS-1$
		for (int i=0; i<3; i++){
			if (i==0) {
				str=Language.getString("PlanetGUI.15"); //$NON-NLS-1$
			}
			if (i==1) {
				str=Language.getString("PlanetGUI.16"); //$NON-NLS-1$
			}
			if (i==2) {
				str=Language.getString("PlanetGUI.17"); //$NON-NLS-1$
			}
			jcbBONUS[0].addItem(str);
			jcbBONUS[1].addItem(str);
			jcbBONUS[2].addItem(str);
		}
		str2 = ENVIRONMENT.toArray();
		jcbMASK.addItem(new ID(Language.getString("PlanetGUI.18"), 0)); //$NON-NLS-1$
		for (int i=0; i<str2.length && i<8; i++){
			jcbMASK.addItem(new ID(str2[i], i+1));
		}
		//place
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets=new Insets(5, 5, 5, 5);
		c.fill=GridBagConstraints.BOTH;
		//planet name
		c.gridx=0;
		c.gridy=0;
		add(jcbMASK, c);
		c.gridy=1;
		c.gridwidth=1;
		c.gridheight=10;
		JScrollPane jpplan = new JScrollPane(lstPLANET);
		jpplan.setPreferredSize(new Dimension(150, 300));
		add(jpplan, c);
		c.gridheight=1;
		c.gridy=11;
		c.insets=new Insets(0, 5, 0, 5);
		add(btnADD, c);
		c.gridy=12;
		c.insets=new Insets(0, 5, 5, 5);
		add(btnREMOVE, c);
		c.insets=new Insets(5, 5, 5, 5);
		c.gridy=1;
		c.gridx=1;
		JLabel lbl = new JLabel(Language.getString("PlanetGUI.19")); //$NON-NLS-1$
		lbl.setFont(def);
		add(lbl, c);
		//labels
		for (int i=0; i<6; i++){
			c.gridy=i+2;
			add(LABEL[i], c);
		}
		c.gridy=8;
		lbl = new JLabel(Language.getString("PlanetGUI.20")); //$NON-NLS-1$
		lbl.setFont(def);
		add(lbl, c);
		//edit
		c.gridx=2;
		c.gridy=1;
		add(txtNAME, c);
		//ani
		c.gridy=2;
		add(jcbANI, c);
		//type
		c.gridy=3;
		add(jcbTYPE, c);
		//size
		c.gridy=4;
		add(jcbSIZE, c);
		//bonuses
		c.gridy=5;
		add(jcbBONUS[0], c);
		c.gridy=6;
		add(jcbBONUS[1], c);
		c.gridy=7;
		add(jcbBONUS[2], c);
		c.gridy=8;
		add(jcbATMO, c);
		//animation
		c.insets=new Insets(5, 50, 5, 5);
		c.gridheight=2;
		c.gridx = 3;
		c.gridy = 1;
		add(alANIM, c);
		//descriptive labels
		c.gridheight=1;
		c.gridy = 3;
		add(jlPOP, c);
		c.gridy = 4;
		add(jlBONI, c);
		c.gridy = 5;
		add(jlINDEX, c);
	}
	
	private void fillPlanets(){
		int j = SELECTED;
		SELECTED = -1;
		PAUSE = true;
		ID[] hm = PLANET.getIDs(jcbMASK.getSelectedIndex());
		lstPLANET.setListData(hm);

                if (j >= 0)
                {
			lstPLANET.setSelectedValue(new ID("", j), true); //$NON-NLS-1$
			
                        while (lstPLANET.getSelectedIndex() < 0 && j > 0)
                        {
                            j--;
                            lstPLANET.setSelectedValue(new ID("", j), true); //$NON-NLS-1$
			}

                        if (lstPLANET.getSelectedIndex() < 0)
                        {
                            lstPLANET.setSelectedIndex(0);
                        }
		}
                else
                {
			lstPLANET.setSelectedIndex(0);
		}

                btnREMOVE.setEnabled(SELECTED >= 0);
		PAUSE = false;
	}
	
	@Override
	public void finalWarning() throws Exception{
		if (SELECTED<0) {
			return;
		}
        int index = PLANET.getIndex(SELECTED);
                    String old = PLANET.getName(index);
        PLANET.setName(index, txtNAME.getText());

        // now fix MEPLANET if need be
        MEPLANET.renamePlanet(old, PLANET.getName(index));
	}
}
