/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: RaceInfoGUI.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui.stbof;
//standard
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import ue.edit.FileManager;
import ue.edit.exe.Trek;
import ue.edit.exe.trek.RaceList;
import ue.edit.res.*;
import ue.edit.res.stbof.*;
import ue.edit.res.stbof.rst.*;
import ue.edit.res.stbof.wtf.Aiminor;
import ue.edit.res.stbof.bin.Meplanet;
import ue.edit.res.stbof.pst.*;
import ue.exception.*;
import ue.gui.*;
import ue.util.*;
import ue.*;

/**
 * The race info gui
 */
@SuppressWarnings("serial")
public class RaceInfoGUI extends MainPanel
{
    private Race        RACES;
    private StringFile  DESCRIPTIONS;

    private Stbof       STBOF;
    private Planet      PLANET;
    private Meplanet    MEPlanet;
    private Aiminor     AIMinor;

    private short SELECTED = -1;
    
    //components
    private JButton btnADD = new JButton(Language.getString("RaceInfoGUI.0")); //$NON-NLS-1$
    private JButton btnEDIT = new JButton(Language.getString("RaceInfoGUI.1")); //$NON-NLS-1$
    private JButton btnREMOVE = new JButton(Language.getString("RaceInfoGUI.2")); //$NON-NLS-1$
    private JComboBox cmbRACES = new JComboBox(); //combo box with all races
    private JTextArea txtINTRO = new JTextArea(12, 30); //text area with race description
    private JScrollPane jspINTRO; //scrollpane for txtINTRO
    private JLabel lblRACE_IMAGE = new JLabel(); //label with race image
    private JLabel lblRace_Pic = new JLabel(Language.getString("RaceInfoGUI.3")); //$NON-NLS-1$
    private JComboBox cmbTGA= new JComboBox(); //combo box with all tgas
    private JLabel lblRace_BMorale = new JLabel(Language.getString("RaceInfoGUI.4")); //$NON-NLS-1$
    private JLabel[] lblBMorale = new JLabel[7]; //for naming the first five [major] races
    private MoraleSlider[] jsBASE_MORALE = new MoraleSlider[7];
    private JLabel lblRace_Combat_M = new JLabel(Language.getString("RaceInfoGUI.5")); //$NON-NLS-1$
    private JLabel lblStart_Pop = new JLabel(Language.getString("RaceInfoGUI.6")); //$NON-NLS-1$
    private JLabel lblDilithium = new JLabel(Language.getString("RaceInfoGUI.7")); //$NON-NLS-1$
    private JLabel lblHome_Planet= new JLabel(Language.getString("RaceInfoGUI.8")); //$NON-NLS-1$
    private JTextField txtCOMBAT_M = new JTextField(10);
    private JSpinner jsSTART_POP = new JSpinner(new SpinnerNumberModel(0, 0, Short.MAX_VALUE, 1));
    private JComboBox cmbDILITHIUM= new JComboBox(new String[]{Language.getString("RaceInfoGUI.9"), Language.getString("RaceInfoGUI.10")}); //$NON-NLS-1$ //$NON-NLS-2$
    private JComboBox cmbHOME_PLANET;
    // strings
    private String strRACE = Language.getString("RaceInfoGUI.11"); //$NON-NLS-1$
    private String strNEUTRAL = Language.getString("RaceInfoGUI.12"); //$NON-NLS-1$
    private String strUNREST = Language.getString("RaceInfoGUI.13"); //$NON-NLS-1$
    
    public RaceInfoGUI(Stbof stbf)
    {
        if (stbf == null)
        {
            return;
        }
        
        try
        {
            PLANET          = (Planet)stbf.getInternalFile(Stbof.filePLANET_PST, true);
            RACES           = (Race)stbf.getInternalFile(Stbof.fileRACE_RST, true);
            DESCRIPTIONS    = (StringFile)stbf.getInternalFile(Stbof.fileRACEDESC_RST, true);
            MEPlanet        = (Meplanet)stbf.getInternalFile(Stbof.fileMEPLANET_BIN, true);
            AIMinor         = (Aiminor)stbf.getInternalFile(Stbof.fileAIMINOR_WTF, true);
        }
        catch (Exception e)
        {
            UE.displayError(e);
            return;
        }
        
        // setup
        STBOF = stbf;
        DESCRIPTIONS.reset();
        
        for (int i = 0; i < RACES.getNumberOfEntries(); i++)
        {
            try
            {
                DESCRIPTIONS.setIndex(RACES.getDescAddress(i), i);
            }
            catch (Exception e)
            {
                UE.displayError(e);
            }
        }
        
        this.setLayout(new BorderLayout());
        //default font
        Font def = UE.SETTINGS.getDefaultFont();
        //set font
        lblRace_Pic.setFont(def);
        lblRace_BMorale.setFont(def);
        cmbRACES.setFont(def);
        cmbTGA.setFont(def);
        cmbDILITHIUM.setFont(def);
        txtINTRO.setFont(def);
        lblRace_Combat_M.setFont(def);
        lblStart_Pop.setFont(def);
        lblDilithium.setFont(def);
        txtCOMBAT_M.setFont(def);
        jsSTART_POP.setFont(def);
        btnEDIT.setFont(def);
        btnADD.setFont(def);
        btnREMOVE.setFont(def);
        lblHome_Planet.setFont(def);
        btnADD.setEnabled(false);
        btnREMOVE.setEnabled(false);

        //races combo
        cmbRACES.addActionListener( new ActionListener ()
        {
            @Override
            public void actionPerformed(ActionEvent ae)
            {
                //save
                try
                {
                    finalWarning();
                }
                catch (Exception ex)
                {
                    // really bad stuff happening - bail
                    UE.displayError(ex);
                    return;
                }

				//show
				SELECTED = (short)cmbRACES.getSelectedIndex();
				
                if (SELECTED >= 0)
                {
					try
                    {
						txtINTRO.setText(DESCRIPTIONS.getDesc(SELECTED));
						cmbTGA.setSelectedItem(RACES.getPic(SELECTED));

						for (int i = 0; i < 6; i++)
                        {
							jsBASE_MORALE[i].setValue(RACES.getBaseMorale(SELECTED, i));
						}
						jsBASE_MORALE[6].setValue(RACES.getMoraleThreshold(SELECTED));
						txtCOMBAT_M.setText((new Float(RACES.getCombatMulti(SELECTED))).toString());
						jsSTART_POP.setValue(new Integer(Short.toString(RACES.getStartPop(SELECTED))));
						if (RACES.getDilithium(SELECTED))
                        {
							cmbDILITHIUM.setSelectedIndex(1);
						}
                        else
                        {
							cmbDILITHIUM.setSelectedIndex(0);
						}
						cmbHOME_PLANET.setSelectedIndex(PLANET.getIndex(RACES.getPlanet(SELECTED)));
					}
                    catch (InvalidArgumentsException iae)
                    {
						UE.displayError(iae);
					}
				}
			}
		});
		//JTF_START_POP
		jsSTART_POP.addChangeListener(new ChangeListener()
        {
            @Override
			public void stateChanged(ChangeEvent e)
            {
				if (SELECTED >= 0)
                {
                    try
                    {
                        short pop = ((Integer)jsSTART_POP.getValue()).shortValue();

                        RACES.setStartPop(SELECTED, pop);
                    }
                    catch (Exception x)
                    {
                        UE.displayError(x);
                    }
				}
			}
		});

		//init labels and sliders
		for (int i = 0; i < 6; i++)
        {
			lblBMorale[i] = new JLabel();
            // TODO: get impossible dif penalty from trek.exe
			jsBASE_MORALE[i] = new MoraleSlider(21, 127, i);
			lblBMorale[i].setFont(def);
			jsBASE_MORALE[i].setFont(def);
		}
        
		lblBMorale[6] = new JLabel();
		jsBASE_MORALE[6] = new MoraleSlider(0, 195, 6);
		lblBMorale[6].setFont(def);
		jsBASE_MORALE[6].setFont(def);
		
		cmbTGA.addActionListener( new ActionListener()
        {
            @Override
			public void actionPerformed(ActionEvent ae)
            {
				short sel = (short)cmbTGA.getSelectedIndex();

                if (sel >= 0 && SELECTED >= 0)
                {
					try
                    {
						String n = (String)cmbTGA.getSelectedItem();
						RACES.setPic(SELECTED, n);
                        
						TargaImage img = STBOF.getTargaImage(n, false);

                        if (img == null)
                        {
							lblRACE_IMAGE.setIcon(null);
						}
                        else
                        {
							Dimension dim = img.getImageSize();

							if (dim.width > 160)
                            {
								lblRACE_IMAGE.setIcon(new ImageIcon(img.getThumbnail(160, false, false)));
								return;
							}
                            else if (dim.height > 170)
                            {
								lblRACE_IMAGE.setIcon(new ImageIcon(img.getThumbnail(170, false, false)));
								return;
							}
                            else
                            {
    							lblRACE_IMAGE.setIcon(new ImageIcon(img.getImage(false)));
                                lblRACE_IMAGE.setText(null);
                            }
						}
					}catch (Exception e){
						UE.displayError(e);
						lblRACE_IMAGE.setIcon(null);
                        lblRACE_IMAGE.setText("N/A");
					}
				}
			}
		});
		//dilithium
		cmbDILITHIUM.addActionListener( new ActionListener()
        {
            @Override
			public void actionPerformed(ActionEvent ae)
            {
				int sel = cmbDILITHIUM.getSelectedIndex();

				if (sel >= 0 && SELECTED >= 0)
                {
                    try
                    {
                        RACES.setDilithium(SELECTED, sel == 1);
                    }
                    catch (Exception x)
                    {
                        UE.displayError(x);
                    }
				}
			}
		});
		//planet index
		cmbHOME_PLANET = new JComboBox(PLANET.getNames());
		cmbHOME_PLANET.setFont(def);
		cmbHOME_PLANET.addActionListener( new ActionListener()
        {
            @Override
			public void actionPerformed(ActionEvent ae)
            {
                // TODO_ sort planet list
				short sel = (short)cmbHOME_PLANET.getSelectedIndex();

				if (sel >= 0 && SELECTED >= 0)
                {
					try
                    {
						RACES.setPlanet(SELECTED, PLANET.getID(sel));
					}
                    catch (InvalidArgumentsException iae)
                    {
						UE.displayError(iae);
					}
				}
			}
		});
		//text area
		txtINTRO.setLineWrap(true);
		txtINTRO.setWrapStyleWord(true);
		//scroll
		jspINTRO = new JScrollPane(txtINTRO);
		
		//JBtn_Name button
		btnEDIT.addActionListener(new ButtonPressed());
		btnADD.addActionListener(new ButtonPressed());
		btnREMOVE.addActionListener(new ButtonPressed());
		//fill JCombo_Tgas
		try
        {
			Enumeration<String[]> en= STBOF.getFiles(new java.io.FilenameFilter()
            {
                @Override
				public boolean accept(File dir, String name)
                {
					return name.endsWith(".tga"); //$NON-NLS-1$
				}
			});

            while (en.hasMoreElements())
            {
				cmbTGA.addItem(en.nextElement()[0]);
			}
		}
        catch (Exception e)
        {
			UE.displayError(e);
		}
		//fill races
		this.fillRaces();		
		//ads panels
		this.add(new Intro(), BorderLayout.NORTH);
		this.add(new Edit(), BorderLayout.CENTER);
	}
	
	private void fillRaces()
    {
		//fill JCombo_Races
		try
        {
			int old_sel = SELECTED;
			cmbRACES.removeAllItems();

			for (short x = 0; x < RACES.getNumberOfEntries(); x++)
            {
                cmbRACES.addItem(new ID(RACES.getName(x), x));
            }

			if ((old_sel >= 0) && (old_sel < cmbRACES.getItemCount()))
            {
				cmbRACES.setSelectedIndex(old_sel);
			}
		}catch (Exception e){
			UE.displayError(e);
		}		
	}
	
	@Override
	public void finalWarning() throws Exception
    {
        if (SELECTED >= 0)
        {
            DESCRIPTIONS.setDesc(SELECTED, txtINTRO.getText());

            for (int i = 0; i < RACES.getNumberOfEntries(); i++)
            {
                RACES.setDescAddress(i, DESCRIPTIONS.getAddress((short)i));
            }

            Float flt = Float.parseFloat(txtCOMBAT_M.getText());
            RACES.setCombatMulti(SELECTED, flt);
        }
	}
	
	private class Intro extends JPanel
    {
		public Intro()
        {
			this.setLayout(new GridBagLayout());
			GridBagConstraints c = new GridBagConstraints();
			c.insets=new Insets(5, 5, 5, 5);
			c.fill=GridBagConstraints.BOTH;
			//combo
			c.gridx=0;
			c.gridy=0;
			c.gridwidth=1;
			c.gridheight=1;
			this.add(cmbRACES, c);
			//text area intro
			c.gridx=1;
			c.gridy=1;
			c.gridwidth=5;
			c.gridheight=2;
			this.add(jspINTRO, c);
			//image
			c.gridx=0;
			c.gridy=1;
			c.gridwidth=1;
			c.gridheight=1;
			this.add(lblRACE_IMAGE, c);
			//add button
//			c.gridx=1;
			c.gridy=0;
			c.gridwidth=1;
			c.gridheight=1;
//			add(JBtn_Add, c);
			//change name button
			c.gridx=2;
			this.add(btnEDIT, c);
			//remove name button
			c.gridx=3;
//			add(JBtn_Remove, c);
		}
	
	}
	
	private class Edit extends JPanel
    {
		public Edit()
        {
			this.setLayout(new GridBagLayout());
			GridBagConstraints c = new GridBagConstraints();
			c.insets=new Insets(5, 5, 5, 5);
			c.fill=GridBagConstraints.BOTH;
			//pic label
			c.gridx=0;
			c.gridy=0;
			c.gridwidth=1;
			c.gridheight=1;
			this.add(lblRace_Pic, c);
			//list of images
			c.gridx=1;
			c.gridy=0;
			c.gridwidth=1;
			c.gridheight=1;
			this.add(cmbTGA, c);
			//base morale main label
			c.gridx=2;
			c.gridy=0;
			c.gridwidth=2;
			c.gridheight=1;
			this.add(lblRace_BMorale, c);
			//base morale sliders
			for (int i=0; i<6; i++){
				jsBASE_MORALE[i].setPreferredSize(new Dimension(120, jsBASE_MORALE[i].getPreferredSize().height));
				c.gridx=3;
				c.gridy=i+1;
				c.gridwidth=1;
				c.gridheight=1;
				this.add(jsBASE_MORALE[i], c);
			}
			//base morale sliders
			for (int i=1; i<7; i++){
				c.gridx=2;
				c.gridy=i;
				c.gridwidth=1;
				c.gridheight=1;
				this.add(lblBMorale[i-1], c);
			}
			//base combat label
			c.gridx=0;
			c.gridy=1;
			c.gridwidth=1;
			c.gridheight=1;
			this.add(lblRace_Combat_M, c);
			//base combat
			c.gridx=1;
			c.gridy=1;
			c.gridwidth=1;
			c.gridheight=1;
			this.add(txtCOMBAT_M, c);
			//starting population label
			c.gridx=0;
			c.gridy=2;
			c.gridwidth=1;
			c.gridheight=1;
			this.add(lblStart_Pop, c);
			//starting population
			c.gridx=1;
			c.gridy=2;
			c.gridwidth=1;
			c.gridheight=1;
			this.add(jsSTART_POP, c);
			//dilithium label
			c.gridx=0;
			c.gridy=3;
			c.gridwidth=1;
			c.gridheight=1;
			this.add(lblDilithium, c);
			//dilithium
			c.gridx=1;
			c.gridy=3;
			c.gridwidth=1;
			c.gridheight=1;
			this.add(cmbDILITHIUM, c);
			//planet label
			c.gridx=0;
			c.gridy=4;
			this.add(lblHome_Planet, c);
			//planet
			c.gridx=1;
			this.add(cmbHOME_PLANET, c);
			//threshold label
			c.gridwidth=2;
			c.gridx=0;
			c.gridy=5;
			this.add(lblBMorale[6], c);
			//threshold
			c.gridy=6;
			this.add(jsBASE_MORALE[6], c);
		}
	}

    private void renameRace(int i, String name) throws InvalidArgumentsException
    {
        if (RACES.containsRace(name))
        {
            // fix old one first
            renameRace(RACES.getIndex(name), name + "?");
        }

        if (i < 5)
        {
            // fix meplanet
            if (Arrays.binarySearch(MEPlanet.getOwners(), name) < 0)
            {
                    MEPlanet.setOwner(RACES.getName(i), name);
            }
        }

        // fix exe
        try
        {
            if (UE.FILES.getFile(FileManager.P_TREK) != null)
            {
                RaceList rlst = (RaceList)UE.FILES.getFile(FileManager.P_TREK).getInternalFile(Trek.fileRACE_LIST, true);

                String old = rlst.getRaceString(i);

                if (i > 4)
                {
                    // fix aiminor
                    int m = Arrays.binarySearch(AIMinor.getNames(), old);

                    if (m < 0)
                    {
                        m = AIMinor.add(0);
                    }

                    AIMinor.setName(m, name);
                }

                rlst.setRaceString(i, name);
            }
        }
        catch (Exception e)
        {}

        RACES.setName(i, name);

        // and now fix errors :/
        RACES.check();
        AIMinor.check();
    }
	
	private class ButtonPressed implements ActionListener
    {
        @Override
		public void actionPerformed(ActionEvent ae)
        {
			try
            {
				if (SELECTED < 0)
                {
					return;
				}

				if ((ae.getActionCommand()).equals(Language.getString("RaceInfoGUI.1"))){ //$NON-NLS-1$
					String n = JOptionPane.showInputDialog(Language.getString("RaceInfoGUI.19"), RACES.getName(SELECTED)); //$NON-NLS-1$
					
                    if (n != null)
                    {
                        renameRace(SELECTED, n);

                        RaceInfoGUI.this.fillRaces();
					}
				}
				if ((ae.getActionCommand()).equals(Language.getString("RaceInfoGUI.2"))){ //$NON-NLS-1$
					RACES.remove(SELECTED);
					DESCRIPTIONS.remove(SELECTED);
					RaceInfoGUI.this.fillRaces();
				}
				if ((ae.getActionCommand()).equals(Language.getString("RaceInfoGUI.0"))){ //$NON-NLS-1$
					ID[] possibleValues = new ID[RACES.getNumberOfEntries()];

                    for (short x = 0; x < RACES.getNumberOfEntries(); x++)
                    {
                        possibleValues[x]= new ID(RACES.getName(x), x);
                    }

					Object selectedValue = JOptionPane.showInputDialog(null,
    					Language.getString("RaceInfoGUI.20"), UE.APP_NAME, //$NON-NLS-1$
        				JOptionPane.INFORMATION_MESSAGE, null,
            			possibleValues, possibleValues[0]);
					
					if (selectedValue != null)
                    {
						int ix = selectedValue.hashCode();
						int newRace = RACES.add(ix);
						int newAdd = DESCRIPTIONS.add();
						RACES.setDescAddress(newRace, newAdd);
						RaceInfoGUI.this.fillRaces();
					}
				}
				
			} catch(Exception e){
				UE.displayError(e);
			}
		}
	}
	
	private class MoraleSlider extends JSlider implements ChangeListener
    {
		private int index;
		
		public MoraleSlider(int h, int w, int i){
			super(h, w);
			index = i;
			this.addChangeListener(this);
		}		

        @Override
		public void stateChanged(ChangeEvent ce){
			try{
				if (index < 6)
                {
					if (index < 5)
                    {
						String str = strRACE.replace("%1", Integer.toString(this.getValue())); //$NON-NLS-1$
						str = str.replace("%2", RACES.getName(index)); //$NON-NLS-1$
						lblBMorale[index].setText(str);
					}else{
						lblBMorale[index].setText(strNEUTRAL.replace("%1", Integer.toString(this.getValue()))); //$NON-NLS-1$
					}
					RACES.setBaseMorale(SELECTED, index, (byte)this.getValue());
				}else{
					lblBMorale[index].setText(strUNREST.replace("%1", Integer.toString(this.getValue()))); //$NON-NLS-1$
					RACES.setMoraleThreshold(SELECTED, (short)this.getValue());
				}
			}catch (InvalidArgumentsException e){
				UE.displayError(e);
			}
		}
	}
}

