/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2010 Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: ShipModelGUI.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 
 * Created: 23.7.2008
 */
package ue.gui.stbof;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.Collator;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import ue.Language;
import ue.UE;
import ue.edit.InternalFile;
import ue.edit.exe.Trek;
import ue.edit.exe.trek.ScreensaverModelList;
import ue.edit.exe.trek.ShipModelData;
import ue.edit.exe.trek.ShipModelDataEntry;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.hob.HobFile;
import ue.edit.res.stbof.lst.PaletteList;
import ue.edit.res.stbof.lst.Texture;
import ue.edit.res.stbof.sst.ShipList;
import ue.exception.InvalidArgumentsException;
import ue.gui.InfoList;
import ue.gui.InfoPanel;
import ue.gui.MainPanel;
import ue.util.GenericFile;
import ue.util.GifColorMap;
import ue.util.GifImage;
import ue.util.SettingsManager;
import ue.util.ShipModelPackage;
import ue.util.TargaImage;
import ue.util.Tools;
import ue.util.UEWorker;

/**
 * Used for managing ship models.
 */
@SuppressWarnings("serial") //$NON-NLS-1$
public class ShipModelGUI extends MainPanel
{
    private Stbof STBOF;
    private Trek TREK;
    private ShipModelData DATA = null;
    private ShipModelPackage SHIP_PACKAGE = null;
    private ShipList SHIP_LIST = null;
    
    private InfoList lstINSTALLED;
    private InfoList lstINPACK;
    private JButton btnIMPORT;
    private JButton btnEXPORT;
    private JButton btnRENAME;
    private JButton btnREMOVE;
    private JButton btnRESCALE;
    private JButton btnDELETE_GRAPHICS;
    private JButton btnOPEN;
    private JButton btnCREATE;
    private JButton btnPREVIEW;
    private JButton btnSET_PREVIEW;
    private JButton btnSET_AUTHOR;
    private JLabel lblINSTALLED;
    private JLabel lblINPACK;
    private JScrollPane jspINPACK;
    
    private String strDescInst = Language.getString("ShipModelGUI.0"); //$NON-NLS-1$
    private String strDescInPa = Language.getString("ShipModelGUI.1"); //$NON-NLS-1$

    private ArrayList<byte[]> texture_hashes = new ArrayList<byte[]>();
    private ArrayList<byte[]> texture_pack_hashes = new ArrayList<byte[]>();
    private boolean scrollInPack = false;
    
    public ShipModelGUI(Stbof st, Trek tk)
    {
        this.STBOF = st;
        this.TREK = tk;

        if (this.TREK != null){
            try{
                this.DATA = (ShipModelData)this.TREK.getInternalFile(Trek.fileSHIP_MODEL_DATA, true);
            }catch (Exception e){
                // this should never happen
                UE.displayError(e);
                return;
            }
        }

        try
        {
            SHIP_LIST = (ShipList)STBOF.getInternalFile(Stbof.fileSHIPLIST_SST, true);
        }
        catch (FileNotFoundException e)
        {
            UE.displayError(e);
            return;
        }
        
        this.setupComponents();
        this.placeComponents();
        // activate
        this.fillList(true);
    }
    
    private void setupComponents()
    {
        // create & setup
        Font def = UE.SETTINGS.getDefaultFont();
        
        this.lblINPACK = new JLabel();
        this.lblINPACK.setFont(def);
        this.lblINPACK.setText(this.strDescInPa.replace("%1", Language.getString("ShipModelGUI.2"))); //$NON-NLS-1$ //$NON-NLS-2$
        
        this.lblINSTALLED = new JLabel();
        this.lblINSTALLED.setFont(def);
        this.lblINSTALLED.setText(this.strDescInst.replace("%1", Language.getString("ShipModelGUI.2"))); //$NON-NLS-1$ //$NON-NLS-2$
        
        // make left and right lists the same width
        Dimension d = this.lblINSTALLED.getPreferredSize();
        Dimension d2 = this.lblINPACK.getPreferredSize();
        if (d.width < d2.width)
        {
            d.width = d2.width;
        }

        this.lblINSTALLED.setPreferredSize(d);
        this.lblINPACK.setPreferredSize(d);
        
        this.btnCREATE = new JButton(Language.getString("ShipModelGUI.3")); //$NON-NLS-1$
        this.btnCREATE.setFont(def);
        this.btnCREATE.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                JFileChooser jfc = new JFileChooser(new File(UE.SETTINGS.getProperty(SettingsManager.WORK_PATH)));
                
                javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter(){
                    @Override
                    public boolean accept(File f){
                        if (f.isDirectory())
                        {
                            return true;
                        }
                        
                        String name = f.getName();
                        name = name.toLowerCase();
                        
                        return name.endsWith(".zip"); //$NON-NLS-1$
                    }
                    @Override
                    public String getDescription(){
                        return Language.getString("ShipModelGUI.41"); //$NON-NLS-1$
                    }
                };
                
                jfc.setFileFilter(filter);
                
                int returnVal = jfc.showSaveDialog(UE.WINDOW);
                
                if (returnVal == JFileChooser.APPROVE_OPTION)
                {
                    try
                    {
                        File file = jfc.getSelectedFile();

                        UE.SETTINGS.setProperty(SettingsManager.WORK_PATH, file.getParent());

                        if (!file.getName().endsWith(".zip")) //$NON-NLS-1$
                        {
                            file = new File(file.getParentFile(), file.getName() + ".zip"); //$NON-NLS-1$
                        }
                        
                        if (file.exists())
                        {
                            String msg = Language.getString("ShipModelGUI.4").replace("%1", file.getName()); //$NON-NLS-1$ //$NON-NLS-2$
                            
                            int respon = JOptionPane.showConfirmDialog(UE.WINDOW, msg, 
                                    UE.APP_NAME, JOptionPane.YES_NO_OPTION,
                                    JOptionPane.QUESTION_MESSAGE);
                            
                            if (respon == JOptionPane.YES_OPTION)
                            {
                                file.delete();
                            }
                            else
                            {
                                return;
                            }
                        }
                        
                        ShipModelGUI.this.SHIP_PACKAGE = new ShipModelPackage(file);
                        ShipModelGUI.this.fillList(false);
                    }
                    catch (Exception e)
                    {
                        UE.displayError(e);
                    }
                }
            }
        });
        
        this.btnOPEN = new JButton(Language.getString("ShipModelGUI.5")); //$NON-NLS-1$
        this.btnOPEN.setFont(def);
        this.btnOPEN.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                JFileChooser jfc = new JFileChooser(new File(UE.SETTINGS.getProperty(SettingsManager.WORK_PATH)));
                
                javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter(){
                    @Override
                    public boolean accept(File f)
                    {
                        if (f.isDirectory())
                        {
                            return true;
                        }
                        
                        String name = f.getName();
                        name = name.toLowerCase();
                        return name.endsWith(".zip"); //$NON-NLS-1$
                    }
                    @Override
                    public String getDescription(){
                        return Language.getString("ShipModelGUI.41"); //$NON-NLS-1$
                    }
                };
                
                jfc.setFileFilter(filter);
                
                int returnVal = jfc.showOpenDialog(UE.WINDOW);
                
                if (returnVal == JFileChooser.APPROVE_OPTION)
                {
                    try
                    {
                        File file = jfc.getSelectedFile();
                        UE.SETTINGS.setProperty(SettingsManager.WORK_PATH, file.getPath());

                        ShipModelGUI.this.SHIP_PACKAGE = new ShipModelPackage(file);
                        ShipModelGUI.this.fillList(false);
                    }
                    catch (Exception e)
                    {
                        UE.displayError(e);
                    }
                }
            }
        });

        this.btnREMOVE = new JButton(Language.getString("ShipModelGUI.6")); //$NON-NLS-1$
        this.btnREMOVE.setFont(def);
        this.btnREMOVE.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                try
                {
                    if (ShipModelGUI.this.lstINPACK.getSelectedItemIndex() >= 0)
                    {
                        ShipModelGUI.this.SHIP_PACKAGE.clearGraphicsString(ShipModelGUI.this.lstINPACK.getSelectedItemLine1());
                        ShipModelGUI.this.SHIP_PACKAGE.save(null);
                        scrollInPack = true;
                        ShipModelGUI.this.fillList(false);
                    }
                }
                catch (Exception e)
                {
                    UE.displayError(e);
                }
            }
        });

        this.btnIMPORT = new JButton(Language.getString("ShipModelGUI.7")); //$NON-NLS-1$
        this.btnIMPORT.setFont(def);
        this.btnIMPORT.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                try
                {
                    ShipModelDataEntry src = SHIP_PACKAGE.getShipModelDataEntry(lstINPACK.getSelectedItemLine1());
                
                    String DEST = lstINSTALLED.getSelectedItemLine1();
                    ShipModelDataEntry dest;
                    ArrayList<String> removal = new ArrayList<String>();

                    if (STBOF.isShipGraphicsStringUsed(DEST))
                    {
                        removal.add(DEST);
                    }

                    if (DEST.contains("_")) //$NON-NLS-1$
                    {
                        dest = DATA.getEntry(DEST.substring(0, DEST.indexOf("_"))); //$NON-NLS-1$
                    }
                    else
                    {
                        dest = DATA.getEntry(DEST);
                    }

                    if (!src.isCompatible(dest))
                    {
                        if (STBOF.isShipGraphicsStringUsed(dest.GRAPHICS))
                        {
                            if (!removal.contains(dest.GRAPHICS))
                            {
                                removal.add(dest.GRAPHICS);
                            }
                        }
                        
                        if (dest.GRAPHICS.length() < 3)
                        {
                            if (STBOF.isShipGraphicsStringUsed(dest.GRAPHICS.toLowerCase() + "_")) //$NON-NLS-1$ //$NON-NLS-2$
                            {
                                if (!removal.contains(dest.GRAPHICS + "_"))
                                {
                                    removal.add(dest.GRAPHICS + "_"); //$NON-NLS-1$
                                }
                            }
                            
                            if (dest.GRAPHICS.length() < 2)
                            {
                                if (STBOF.isShipGraphicsStringUsed(dest.GRAPHICS.toLowerCase() + "__")) //$NON-NLS-1$ //$NON-NLS-2$
                                {
                                    if (!removal.contains(dest.GRAPHICS + "__"))
                                    {
                                        removal.add(dest.GRAPHICS + "__"); //$NON-NLS-1$
                                    }
                                }
                            }
                        }
                    }
                    
                    if (removal.size() == 1)
                    {
                        String msg = Language.getString("ShipModelGUI.8"); //$NON-NLS-1$
                        String h = removal.get(0);
                            
                        int answer = JOptionPane.showConfirmDialog(UE.WINDOW, msg.replace("%1", h), //$NON-NLS-1$
                                UE.APP_NAME, JOptionPane.OK_CANCEL_OPTION);
                        
                        if (answer == JOptionPane.OK_OPTION)
                        {
                            STBOF.removeModelGraphicFiles(new String[]{h});
                        }
                        else
                        {
                            return;
                        }
                    }
                    else if (removal.size() == 2)
                    {
                        String msg = Language.getString("ShipModelGUI.9"); //$NON-NLS-1$
                        String h1, h2;
                        
                        h1 = removal.get(0); h2 = removal.get(1);
                        
                        msg = msg.replace("%1", h1); //$NON-NLS-1$
                        msg = msg.replace("%2", h2); //$NON-NLS-1$
                        
                        int answer = JOptionPane.showConfirmDialog(UE.WINDOW, msg,
                                UE.APP_NAME, JOptionPane.OK_CANCEL_OPTION);
                        
                        if (answer == JOptionPane.OK_OPTION)
                        {
                            STBOF.removeModelGraphicFiles(new String[]{h1, h2});
                        }
                        else
                        {
                            return;
                        }
                    }
                    else if (removal.size() == 3)
                    {
                        String msg = Language.getString("ShipModelGUI.40"); //$NON-NLS-1$
                        String h1, h2, h3;
                        
                        h1 = removal.get(0); h2 = removal.get(1); h3 = removal.get(2);
                        
                        msg = msg.replace("%1", h1); //$NON-NLS-1$
                        msg = msg.replace("%2", h2); //$NON-NLS-1$
                        msg = msg.replace("%3", h3); //$NON-NLS-1$
                        
                        int answer = JOptionPane.showConfirmDialog(UE.WINDOW, msg,
                                UE.APP_NAME, JOptionPane.OK_CANCEL_OPTION);
                        
                        if (answer == JOptionPane.OK_OPTION)
                        {
                            STBOF.removeModelGraphicFiles(new String[]{h1, h2, h3});
                        }
                        else
                        {
                            return;
                        }
                    }
                    
                    ImportTask tsk = new ImportTask((String)ShipModelGUI.this.lstINPACK.getSelectedItemLine1(), (String)ShipModelGUI.this.lstINSTALLED.getSelectedItemLine1());
    
                    boolean result = (Boolean)Tools.runUEWorker(tsk);
    
                    if (result)
                    {
                        ShipModelGUI.this.fillList(true);
                    }
                }
                catch (Exception e2)
                {
                    UE.displayError(e2);
                }
            }
        });
        
        this.btnEXPORT = new JButton(Language.getString("ShipModelGUI.10")); //$NON-NLS-1$
        this.btnEXPORT.setFont(def);
        d = this.btnEXPORT.getPreferredSize();
        if (d.width < 120)
        {
            d.width = 120;
        }
        this.btnEXPORT.setPreferredSize(d);
        this.btnEXPORT.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                ExportTask tsk = new ExportTask((String)ShipModelGUI.this.lstINSTALLED.getSelectedItemLine1());
                
                boolean result = (Boolean)Tools.runUEWorker(tsk);

                if (result)
                {
                    ShipModelGUI.this.SHIP_PACKAGE.save(null);
                    
                    scrollInPack = true;
                    
                    ShipModelGUI.this.fillList(false);
                    
                    lstINPACK.setSelectedItemIndex(lstINPACK.getNumberOfItems()-1);
                }
            }
        });
        
        btnPREVIEW = new JButton(Language.getString("ShipModelGUI.44")); //$NON-NLS-1$
        this.btnPREVIEW.setFont(def);
        this.btnPREVIEW.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                if (SHIP_PACKAGE == null || lstINPACK.getSelectedItemIndex() < 0)
                {
                    return;
                }
                
                String prefix = lstINPACK.getSelectedItemLine1();

                JFrame frmPrev = new JFrame();
                frmPrev.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                frmPrev.setMaximumSize(new Dimension(800, 600));
                frmPrev.setMinimumSize(new Dimension(400, 300));
                frmPrev.setLayout(new BorderLayout());
                frmPrev.setIconImage(UE.ICON);
                frmPrev.setTitle(prefix);
                
                try
                {
                    String cl = SHIP_PACKAGE.getClassByPrefix(prefix);
                    
                    if (cl != null)
                    {
                        frmPrev.setTitle(cl);
                    }
                    
                    GenericFile gen = SHIP_PACKAGE.getFile(prefix + ".jpg"); //$NON-NLS-1$
                    frmPrev.add(new JLabel(new ImageIcon(gen.getData())), BorderLayout.CENTER);
                    
                    try
                    {
                        gen = SHIP_PACKAGE.getFile("i_" + prefix + "30.tga"); //$NON-NLS-1$ //$NON-NLS-2$
                        TargaImage icon = new TargaImage(new ByteArrayInputStream(gen.getData()));
                        frmPrev.setIconImage(icon.getThumbnail(50, false, false));
                    }
                    catch (Exception x)
                    {
                    }
                }
                catch (Exception x)
                {
                    JLabel lbl = new JLabel(x.getLocalizedMessage());
                    lbl.setHorizontalAlignment(SwingConstants.CENTER);
                    lbl.setHorizontalTextPosition(SwingConstants.CENTER);
                    frmPrev.add(lbl, BorderLayout.CENTER);
                }

                frmPrev.pack();
                Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
                Dimension size = frmPrev.getPreferredSize();
                frmPrev.setLocation(new Point(
                        (int)(screenSize.getWidth()-size.getWidth())/2,
                        (int)(screenSize.getHeight()-size.getHeight())/2
                        )
                );
                frmPrev.setVisible(true);
            }
        });
        
        btnSET_PREVIEW = new JButton(Language.getString("ShipModelGUI.45")); //$NON-NLS-1$
        this.btnSET_PREVIEW.setFont(def);
        this.btnSET_PREVIEW.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                if (SHIP_PACKAGE == null || lstINPACK.getSelectedItemIndex() < 0)
                {
                    return;
                }
                
                JFileChooser jfc = new JFileChooser(new File(UE.SETTINGS.getProperty(SettingsManager.WORK_PATH)));
                
                javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter()
                {
                    @Override
                    public boolean accept(File f)
                    {
                        if (f.isDirectory())
                        {
                            return true;
                        }
                        
                        String name = f.getName();
                        name = name.toLowerCase();
                        return name.endsWith(".jpg"); //$NON-NLS-1$
                    }
                    @Override
                    public String getDescription()
                    {
                        return "*.jpg"; //$NON-NLS-1$
                    }
                };
                
                jfc.setFileFilter(filter);
                
                int returnVal = jfc.showOpenDialog(UE.WINDOW);
                
                if (returnVal == JFileChooser.APPROVE_OPTION)
                {
                    try
                    {
                        File file = jfc.getSelectedFile();
                        UE.SETTINGS.setProperty(SettingsManager.WORK_PATH, file.getPath());

                        FileInputStream fis = new FileInputStream(file);
                        
                        byte[] b = new byte[fis.available()];
                        int off = 0;
                        int len = 0;
                        
                        while ((len = fis.read(b, off, fis.available())) > 0)
                        {
                            off += len;
                        }
                        
                        GenericFile gen = new GenericFile(lstINPACK.getSelectedItemLine1().toLowerCase() + ".jpg", b); //$NON-NLS-1$
                        
                        SHIP_PACKAGE.addFile(gen);
                        btnPREVIEW.setEnabled(true);
                        SHIP_PACKAGE.save(null);
                    }
                    catch (Exception x)
                    {
                        UE.displayError(x);
                    }
                }
            }
        });

        btnSET_AUTHOR = new JButton(Language.getString("ShipModelGUI.46")); //$NON-NLS-1$
        this.btnSET_AUTHOR.setFont(def);
        this.btnSET_AUTHOR.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                if (SHIP_PACKAGE == null || lstINPACK.getSelectedItemIndex() < 0)
                {
                    return;
                }
                
                String prefix = lstINPACK.getSelectedItemLine1();
                
                String str = SHIP_PACKAGE.getAuthorByPrefix(prefix);
                if (str == null)
                {
                    str = ""; //$NON-NLS-1$
                }
                
                Object obj = JOptionPane.showInputDialog(UE.WINDOW,
                        Language.getString("ShipModelGUI.47"), str); //$NON-NLS-1$
                
                if (obj != null)
                {
                    str = (String)obj;
                    
                    if (str.length() > 0)
                    {
                        SHIP_PACKAGE.setAuthorForPrefix(prefix, str);

                        SHIP_PACKAGE.save(null);
                        fillList(false);
                    }
                }
            }
        });
                
        this.btnRENAME = new JButton(Language.getString("ShipModelGUI.11")); //$NON-NLS-1$
        this.btnRENAME.setFont(def);
        this.btnRENAME.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                if (lstINSTALLED.getSelectedItemIndex() < 0) {
                    return;
                }
                
                String g = lstINSTALLED.getSelectedItemLine1();
                String r = g;
                
                if (r.length() > 1)
                {
                    r = getSuggestedGraphicsString(g.substring(0, r.length() - 1));
                }
                
                // get input
                r = JOptionPane.showInputDialog(UE.WINDOW, Language.getString("ShipModelGUI.12").replace("%1", g), r); //$NON-NLS-1$ //$NON-NLS-2$

                if (r == null) {
                    return;
                }
                
                r = r.toUpperCase();
                
                char c;
                for (int i = 0; i < r.length(); i++)
                {
                    c = r.charAt(i);
                    if (!Character.isLetterOrDigit(c))
                    {
                        return;
                    }
                }
                
                if (!g.equals(r))
                {
                    // check if used
                    try
                    {
                        ShipList ships = (ShipList)STBOF.getInternalFile(Stbof.fileSHIPLIST_SST, true);
                        ArrayList<String> used = ships.getGraphics();

                        
                        if (DATA.contains(new ShipModelDataEntry(r.toUpperCase()))
                                || used.contains(r.toLowerCase())
                                || (r.length() < 3 && used.contains(r.toLowerCase() + "_")) //$NON-NLS-1$
                                || (r.length() < 2 && used.contains(r.toLowerCase() + "__"))) //$NON-NLS-1$
                        {
                            UE.displayError(new Exception(Language.getString("ShipModelGUI.13"))); //$NON-NLS-1$
                            actionPerformed(arg0);
                            return;
                        }
                                            
                        // check for orphans
                        int dif = r.length() - g.length();
    
                        if (dif == 1)
                        {
                            String msg = Language.getString("ShipModelGUI.8"); //$NON-NLS-1$
                            String h;
                                
                            if (g.length() < 2)
                            {
                                h = g + "__"; //$NON-NLS-1$
                            }
                            else
                            {
                                h = g + "_"; //$NON-NLS-1$
                            }
                            
                            int answer = JOptionPane.showConfirmDialog(UE.WINDOW, msg.replace("%1", h), //$NON-NLS-1$
                                    UE.APP_NAME, JOptionPane.OK_CANCEL_OPTION);
                            
                            if (answer == JOptionPane.OK_OPTION)
                            {
                                STBOF.removeModelGraphicFiles(new String[]{h});
                            }
                            else
                            {
                                return;
                            }
                        }
                        else if (dif == 2)
                        {
                            String msg = Language.getString("ShipModelGUI.9"); //$NON-NLS-1$
                            String h1, h2;
                            
                            h1 = g + "_"; h2 = g + "__"; //$NON-NLS-1$ //$NON-NLS-2$
                            
                            msg.replace("%1", h1); //$NON-NLS-1$
                            msg.replace("%2", h2); //$NON-NLS-1$
                            
                            int answer = JOptionPane.showConfirmDialog(UE.WINDOW, msg,
                                    UE.APP_NAME, JOptionPane.OK_CANCEL_OPTION);
                            
                            if (answer == JOptionPane.OK_OPTION)
                            {
                                STBOF.removeModelGraphicFiles(new String[]{h1, h2});
                            }
                            else
                            {
                                return;
                            }
                        }
                        
                        // rename
                        String t = r;
                        
                        ScreensaverModelList sml = (ScreensaverModelList)TREK.getInternalFile(Trek.fileSCREENSAVER_MODEL_LIST, true);
                        
                        while (g.length() < 4 && t.length() < 4)
                        {
                            try
                            {
                                STBOF.renameModelGraphicFiles(g, t);
                                
                                if (sml.getNumberofEntries() <= sml.getMax())
                                {
                                    sml.remove(g.toLowerCase() + ".hob"); //$NON-NLS-1$
                                    
                                    if (sml.getNumberofEntries() < sml.getMax())
                                    {
                                        sml.add(t.toLowerCase() + ".hob"); //$NON-NLS-1$
                                    }
                                }
                                
                                for (int i = 0; i < ships.getNumberOfEntries(); i++)
                                {
                                    if (ships.getImageName(i).toLowerCase().equals(g.toLowerCase()))
                                    {
                                        ships.setImageName(i, t.toLowerCase());
                                    }
                                }
                            }
                            catch (Exception e)
                            {
                                UE.displayError(e);
                            }
                            
                            g = g + "_"; //$NON-NLS-1$
                            t = t + "_"; //$NON-NLS-1$
                        }

                        ShipModelDataEntry entry = DATA.getEntry(lstINSTALLED.getSelectedItemLine1());
                        entry.GRAPHICS = r;
    
                        fillList(true);
                    }
                    catch (Exception e)
                    {
                        UE.displayError(e);
                        actionPerformed(arg0);
                        return;
                    }
                }
            }
        });

        this.btnRESCALE = new JButton(Language.getString("ShipModelGUI.37")); //$NON-NLS-1$
        this.btnRESCALE.setFont(def);
        this.btnRESCALE.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                if (lstINSTALLED.getSelectedItemIndex() < 0) {
                    return;
                }
                
                try
                {
                    String g = lstINSTALLED.getSelectedItemLine1();
                    if (g.contains("_")) {
                        return; //$NON-NLS-1$
                    } //$NON-NLS-1$
                    
                    int index = DATA.getIndex(g);
                
                    // get input
                    String scale = Integer.toString(Math.round(DATA.getScale(index)*100));
                    scale = JOptionPane.showInputDialog(UE.WINDOW, Language.getString("ShipModelGUI.39").replace("%1", g), scale); //$NON-NLS-1$ //$NON-NLS-2$
    
                    if (scale == null) return;
                    
                    try
                    {
                        float s = Integer.parseInt(scale) / (float)100;
                        
                        if (s > 0)
                        {
                            DATA.setScale(index, s);
                            fillList(true);
                        }
                    }
                    catch (NumberFormatException f)
                    {
                        
                    }
                }
                catch (InvalidArgumentsException e)
                {
                    UE.displayError(e);
                }
            }
        });

        this.btnDELETE_GRAPHICS = new JButton(Language.getString("ShipModelGUI.14")); //$NON-NLS-1$
        this.btnDELETE_GRAPHICS.setFont(def);
        this.btnDELETE_GRAPHICS.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent arg0)
            {
                if (lstINSTALLED.getSelectedItemIndex() < 0) return;
                
                try
                {
                    String msg = Language.getString("ShipModelGUI.15"); //$NON-NLS-1$
                    String g = lstINSTALLED.getSelectedItemLine1();
                    
                    int answer = JOptionPane.showConfirmDialog(UE.WINDOW, msg.replace("%1", g), //$NON-NLS-1$
                            UE.APP_NAME, JOptionPane.YES_NO_OPTION);
                    
                    if (answer == JOptionPane.YES_OPTION)
                    {
                        if (STBOF.removeModelGraphicFiles(new String[]{g}))
                        {
                            fillList(true);
                        }
                    }
                }
                catch (Exception e)
                {
                    UE.displayError(e);
                }
            }
        });
        
        this.lstINPACK = new InfoList(new ImageIcon(new BufferedImage(60, 50, BufferedImage.TYPE_INT_RGB)),
                true, 0, 1);
        this.lstINPACK.setFont(def);
        this.lstINPACK.addListSelectionListener(new ListSelectionListener()
        {
            @Override
            public void valueChanged(ListSelectionEvent e)
            {
                if ((ShipModelGUI.this.lstINPACK.getSelectedItemIndex() < 0) || (ShipModelGUI.this.SHIP_PACKAGE == null))
                {
                    btnSET_PREVIEW.setEnabled(false);
                    btnSET_AUTHOR.setEnabled(false);
                    return;
                }
                
                // setup buttons
                ShipModelGUI.this.btnREMOVE.setEnabled(true);
                btnIMPORT.setEnabled(canImport());
                btnSET_PREVIEW.setEnabled(true);
                btnSET_AUTHOR.setEnabled(true);
                
                try
                {
                    btnPREVIEW.setEnabled(SHIP_PACKAGE.containsFile(lstINPACK.getSelectedItemLine1() + ".jpg")); //$NON-NLS-1$
                }
                catch (Exception x)
                {
                    UE.displayError(x);
                    btnPREVIEW.setEnabled(false);
                }
            }
        });

        JPanel pnlINPACK = new JPanel(new BorderLayout());
        pnlINPACK.add(lstINPACK, BorderLayout.NORTH);
        jspINPACK = new JScrollPane(pnlINPACK);
        JScrollBar jsp = jspINPACK.getVerticalScrollBar();
        jsp.setUnitIncrement(50);
        jspINPACK.setPreferredSize(new Dimension(200, 250));
        ChangeListener cl = new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent e)
            {
                if (scrollInPack)
                {
                    int item = lstINPACK.getPreferredSize().height/lstINPACK.getNumberOfItems();
                    JScrollBar jsb = jspINPACK.getVerticalScrollBar();
                    int min = jsb.getValue();
                    int max = jsb.getSize().height + min - item;
                    int val = item * lstINPACK.getSelectedItemIndex();
                    if (val < min || val > max)
                    {
                        jsb.setValue(val);
                    }
                    scrollInPack = false;
                }
            }
        };
        
        
        jspINPACK.getVerticalScrollBar().getModel().addChangeListener(cl);

        this.lstINSTALLED = new InfoList(new ImageIcon(new BufferedImage(60, 50, BufferedImage.TYPE_INT_RGB)),
                true, 0, 1);
        this.lstINSTALLED.setFont(def);
        this.lstINSTALLED.addListSelectionListener(new ListSelectionListener()
        {
            @Override
            public void valueChanged(ListSelectionEvent e)
            {
                if ((ShipModelGUI.this.lstINSTALLED.getSelectedItemIndex() < 0) || (ShipModelGUI.this.DATA == null))
                {
                    return;
                }
                
                ShipModelGUI.this.btnEXPORT.setEnabled(ShipModelGUI.this.SHIP_PACKAGE != null);
                btnIMPORT.setEnabled(canImport());
                btnDELETE_GRAPHICS.setEnabled(lstINSTALLED.getSelectedItemLine3().length() <= 0);
                
                String g = lstINSTALLED.getSelectedItemLine1();
                btnRENAME.setEnabled(!g.contains("_") //$NON-NLS-1$
                        && (!g.startsWith("T0") || g.equals("T00"))); //$NON-NLS-1$ //$NON-NLS-2$
                btnRESCALE.setEnabled(!g.contains("_") && !btnDELETE_GRAPHICS.isEnabled()); //$NON-NLS-1$
            }
        });
    }
    
    private void placeComponents()
    {
        // add to panel
        this.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.insets=new Insets(5, 5, 5, 5);
        c.fill=GridBagConstraints.BOTH;
        c.anchor=GridBagConstraints.NORTH;
        
        c.gridwidth = 1;
        c.gridheight = 1;
        c.gridx = 0;
        c.gridy = 0;
        add(this.lblINSTALLED, c);
        
        c.gridy = 1;
        c.gridheight = 4;
        JPanel pnlINSTALLED = new JPanel(new BorderLayout());
        pnlINSTALLED.add(lstINSTALLED, BorderLayout.NORTH);
        JScrollPane jspInstalled = new JScrollPane(pnlINSTALLED);
        JScrollBar jsp = jspInstalled.getVerticalScrollBar();
        jsp.setUnitIncrement(50);
        jspInstalled.setPreferredSize(new Dimension(200, 250));
        add(jspInstalled, c);
        
        c.gridy = 5;
        c.gridheight = 1;
        add(this.btnRENAME, c);
        
        c.gridy = 6;
        add(this.btnRESCALE, c);

        c.gridy = 7;
        add(this.btnDELETE_GRAPHICS, c);

        c.gridx = 2;
        c.gridy = 0;
        c.gridwidth = 2;
        add(this.lblINPACK, c);

        c.gridy = 1;
        c.gridheight = 4;
        add(jspINPACK, c);
        
        c.gridy = 5;
        c.gridheight = 1;
        add(this.btnREMOVE, c);
        
        c.gridy = 6;
        add(this.btnOPEN, c);
        
        c.gridy = 7;
        add(this.btnCREATE, c);
        
        c.gridy = 8;
        add(btnPREVIEW, c);
        c.gridwidth = 1;
        c.gridy = 9;
        add(btnSET_PREVIEW, c);
        c.gridx = 3;
        add(btnSET_AUTHOR, c);

        c.gridx = 1;
        c.gridy = 1;
        add(btnIMPORT, c);
        c.gridy = 2;
        add(btnEXPORT, c);
    }
    
    private boolean canImport()
    {
        if (lstINPACK.getSelectedItemIndex() >= 0 && lstINSTALLED.getSelectedItemIndex() >= 0)
        {
            try
            {
                ArrayList<String> used = ((ShipList)STBOF.getInternalFile(Stbof.fileSHIPLIST_SST, true)).getGraphics();
                ShipModelDataEntry src = SHIP_PACKAGE.getShipModelDataEntry(lstINPACK.getSelectedItemLine1());
            
                String DEST = lstINSTALLED.getSelectedItemLine1();
                ShipModelDataEntry dest;
                
                if (DEST.contains("_")) //$NON-NLS-1$
                {
                    dest = DATA.getEntry(DEST.substring(0, DEST.indexOf("_"))); //$NON-NLS-1$
                }
                else
                {
                    dest = DATA.getEntry(DEST);
                }
                
                if (!src.isCompatible(dest))
                {
                    if ((!dest.GRAPHICS.equals(DEST) && used.contains(dest.GRAPHICS.toLowerCase())))
                    {
                        return false;
                    }
                    
                    if (dest.GRAPHICS.length() < 3)
                    {
                        if ((!(dest.GRAPHICS + "_").equals(DEST) && used.contains(dest.GRAPHICS.toLowerCase() + "_"))) //$NON-NLS-1$ //$NON-NLS-2$
                        {
                            return false;
                        }
                        
                        if (dest.GRAPHICS.length() < 2)
                        {
                            if ((!(dest.GRAPHICS + "__").equals(DEST) && used.contains(dest.GRAPHICS.toLowerCase() + "__"))) //$NON-NLS-1$ //$NON-NLS-2$
                            {
                                return false;
                            }
                        }
                    }
                }
            }
            catch (Exception e2)
            {
                UE.displayError(e2);
                return false;
            }

            return true;
        }
        else
        {
            return false;
        }
    }
    
    private String getSuggestedGraphicsString(String desired)
    {
        String g = desired;
        int i = 0;
        char c = 'A';
        
        try
        {
            while (DATA.contains(new ShipModelDataEntry(g.toUpperCase()))
                    || STBOF.isShipGraphicsStringUsed(g)
                    || (g.length() < 3 && STBOF.isShipGraphicsStringUsed(g + "_")) //$NON-NLS-1$
                    || (g.length() < 2 && STBOF.isShipGraphicsStringUsed(g + "__"))) //$NON-NLS-1$
            {
                if (i < 10)
                {
                    g = g.substring(0, g.length() - 1) + i++;
                }
                else
                {
                    if (c <= 'Z'){
                        g = g.substring(0, g.length() - 1) + c++;
                    }
                    else
                    {
                        if (g.length() > 1)
                        {
                            i = 0;
                            c = 'A';
                            g = g.substring(0, g.length() - 1);
                        }
                        else
                        {
                            try
                            {
                                return STBOF.getUnusedShipGraphicsString();
                            }
                            catch (Exception e)
                            {
                                UE.displayError(e);
                            }
                        }
                    }
                }
            }
        }
        catch (Exception e)
        {
            UE.displayError(e);
        }
        
        return g;
    }
    
    // fills the model list
    @SuppressWarnings("unchecked") //$NON-NLS-1$
    private void fillList(boolean installed)
    {
        this.btnEXPORT.setEnabled(false);
        this.btnIMPORT.setEnabled(false);
        this.btnREMOVE.setEnabled(false);
        this.btnOPEN.setEnabled(false);
        this.btnCREATE.setEnabled(false);
        btnPREVIEW.setEnabled(false);
        btnSET_PREVIEW.setEnabled(false);
        btnSET_AUTHOR.setEnabled(false);
        btnDELETE_GRAPHICS.setEnabled(false);
        btnRENAME.setEnabled(false);
        btnRESCALE.setEnabled(false);

        LoadTask tsk = new LoadTask(installed);
        
        int index;
        
        if (installed && DATA != null)
        {
            Vector<Object[]> result = (Vector<Object[]>)Tools.runUEWorker(tsk);

            ShipList ships = null;
            try
            {
                ships = (ShipList)STBOF.getInternalFile(Stbof.fileSHIPLIST_SST, true);
            }
            catch (Exception e)
            {
                UE.displayError(e);
            }
            
            index = lstINSTALLED.getSelectedItemIndex();
            lstINSTALLED.setVisible(false);
            lstINSTALLED.removeAllItems();
            String l1, l2, l3, tip;
            boolean tipon;
            int num = 0;
            
            for (int i = 0; i < result.size(); i++)
            {
                tipon = false;
                tip = "<html>"; //$NON-NLS-1$
                Object[] data = result.get(i);
                
                try
                {
                    l1 = (String)data[1];
                    
                    l2 = (String)data[2];
                    
                    l3 = (String)data[3];
                    
                    if (l3.length() > 0) {
                        num++;
                    }
                    
                    this.lstINSTALLED.addItem((ImageIcon)data[0], l1, l2, l3);
                    
                    if (ships != null)
                    {
                        String str = ships.getClassByPrefix(l1);
                        if (str != null)
                        {
                            tip = tip + Language.getString("ShipModelGUI.42").replace("%1", str); //$NON-NLS-1$ //$NON-NLS-2$
                            tipon = true;
                        }
                    }
                    if (data[4] != null)
                    {
                        if (tipon)
                        {
                            tip = tip + "<br>"; //$NON-NLS-1$
                        }
                        else
                        {
                            tipon = true;
                        }
                        tip = tip + Language.getString("ShipModelGUI.43").replace("%1", data[4].toString()); //$NON-NLS-1$ //$NON-NLS-2$
                    }

                    if (STBOF.hasInternalFile(l1 + "_a.hob")) //$NON-NLS-1$
                    {
                        double hobsize = STBOF.getInternalFileSize(l1 + "_a.hob"); //$NON-NLS-1$
                        String sizedesc = Language.getString("ShipModelGUI.48"); //$NON-NLS-1$
    
                        if (hobsize > 1024)
                        {
                            hobsize = hobsize * (double)10;
                            hobsize = Math.round(hobsize / (double)1024) / (double)10;
                            sizedesc = Language.getString("ShipModelGUI.49"); //$NON-NLS-1$
    
                            if (hobsize > 1024)
                            {
                                hobsize = hobsize * (double)10;
                                hobsize = Math.round(hobsize / (double)1024) / (double)10;
                                sizedesc = Language.getString("ShipModelGUI.50"); //$NON-NLS-1$
                            }
                        }
                        
                        if (hobsize > 0)
                        {
                            if (tipon)
                            {
                                tip = tip + "<br>"; //$NON-NLS-1$
                            }
                            tip = tip + Language.getString("ShipModelGUI.51").replace("%1", Double.toString(hobsize)).replace("%2", sizedesc); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                        }
                    }
                    
                    tip = tip + "</html>"; //$NON-NLS-1$
                    
                    if (tipon) lstINSTALLED.setTooltip(i, tip);
                }
                catch (Exception e1)
                {
                    UE.displayError(e1);
                    this.lstINSTALLED.addItem((ImageIcon)null, Language.getString("ShipModelGUI.16"), Language.getString("ShipModelGUI.16"), Language.getString("ShipModelGUI.16")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                }
            }
            this.lblINSTALLED.setText(this.strDescInst.replace("%1", Integer.toString(num))); //$NON-NLS-1$

            /*
            this.lstINSTALLED.sort(new Comparator<InfoPanel>()
            {
                @Override
                public int compare(InfoPanel pnl0, InfoPanel pnl1)
                {
                    try
                    {
                        boolean ua = false, ub = false;
                        String a, b;
                    
                        int id1, id2;
                        String g;
                        if(SHIP_LIST != null)
                        {
                            g = pnl0.getLine1();
                            if (g.indexOf("_") >= 0) //$NON-NLS-1$
                            {
                                ua = true;
                                g = g.substring(0, g.indexOf("_")); //$NON-NLS-1$
                            }
                                
                            id1 = SHIP_LIST.getFirstIDWithPrefix(g);
                            
                            g = pnl1.getLine1();
                            if (g.indexOf("_") >= 0) //$NON-NLS-1$
                            {
                                ub = true;
                                g = g.substring(0, g.indexOf("_")); //$NON-NLS-1$
                            }
    
                            id2 = SHIP_LIST.getFirstIDWithPrefix(g);
                            
                            if (id1 < 0)
                            {
                                a = "ZZZZZZZZZZZZZZZZZZZZZZZZ"; //$NON-NLS-1$
                                id1 = 99;
                            }
                            else
                            {
                                a = SHIP_LIST.getClass(id1);
                                id1 = SHIP_LIST.getRace(id1);
                            }
                            if (id2 < 0)
                            {
                                b = "ZZZZZZZZZZZZZZZZZZZZZZZZ"; //$NON-NLS-1$
                                id2 = 99;
                            }
                            else
                            {
                                b = SHIP_LIST.getClass(id2);
                                id2 = SHIP_LIST.getRace(id2);
                            }
                            
                            a = Integer.toString(id1) + a;
                            b = Integer.toString(id2) + b;
                            if (id1 < 10) a = "0" + a; //$NON-NLS-1$
                            if (id2 < 10) b = "0" + b; //$NON-NLS-1$
                            if (ua) a = a + "a"; //$NON-NLS-1$
                            if (ub) b = b + "a"; //$NON-NLS-1$
                            
                            return (Collator.getInstance()).compare(a, b);
                        }
                    }
                    catch (Exception e)
                    {
                        UE.displayError(e);
                    }
                    return 0;
                }
            });
            */
            lstINSTALLED.setVisible(true);

            if (index < 0 || index >= result.size()) index = 0;
            this.lstINSTALLED.setSelectedItemIndex(index);
            
            if (lstINPACK.getSelectedItemIndex() >= 0)
            {
                int l = lstINPACK.getSelectedItemIndex();
                lstINPACK.clearSelection();
                lstINPACK.setSelectedItemIndex(l);
            }
        }
        else if (!installed)
        {
            Vector<Object[]> result = (Vector<Object[]>)Tools.runUEWorker(tsk);
            index = lstINPACK.getSelectedItemIndex();
            lstINPACK.setVisible(false);
            lstINPACK.removeAllItems();
            String l1, l2, l3, tip;
            boolean tipon;
            
            for (int i = 0; i < result.size(); i++)
            {
                Object[] data = result.get(i);
                tip = "<html>"; //$NON-NLS-1$
                tipon = false;

                try
                {
                    l1 = (String)data[1];
                    
                    l2 = (String)data[2];
                    
                    l3 = (String)data[3];

                    this.lstINPACK.addItem((ImageIcon)data[0], l1, l2, l3);

                    String str = SHIP_PACKAGE.getAuthorByPrefix(l1);
                    if (str != null)
                    {
                        tip = tip + "Author(s): %1".replace("%1", str); //$NON-NLS-1$ //$NON-NLS-2$
                        tipon = true;
                    }
                    str = SHIP_PACKAGE.getClassByPrefix(l1);
                    if (str != null)
                    {
                        if (tipon)
                        {
                            tip = tip + "<br>"; //$NON-NLS-1$
                        }
                        tip = tip + Language.getString("ShipModelGUI.42").replace("%1", str); //$NON-NLS-1$ //$NON-NLS-2$
                        tipon = true;
                    }
                    
                    if (data[4] != null)
                    {
                        if (tipon)
                        {
                            tip = tip + "<br>"; //$NON-NLS-1$
                        }
                        else
                        {
                            tipon = true;
                        }
                        tip = tip + Language.getString("ShipModelGUI.43").replace("%1", data[4].toString()); //$NON-NLS-1$ //$NON-NLS-2$
                    }
                    
                    if (SHIP_PACKAGE.containsFile(l1 + "_a.hob")) //$NON-NLS-1$
                    {
                        double hobsize = SHIP_PACKAGE.getFileSize(l1 + "_a.hob"); //$NON-NLS-1$
                        String sizedesc = Language.getString("ShipModelGUI.48"); //$NON-NLS-1$
    
                        if (hobsize > 1024)
                        {
                            hobsize = hobsize * (double)10;
                            hobsize = Math.round(hobsize / (double)1024) / (double)10;
                            sizedesc = Language.getString("ShipModelGUI.49"); //$NON-NLS-1$
    
                            if (hobsize > 1024)
                            {
                                hobsize = hobsize * (double)10;
                                hobsize = Math.round(hobsize / (double)1024) / (double)10;
                                sizedesc = Language.getString("ShipModelGUI.50"); //$NON-NLS-1$
                            }
                        }
                        
                        if (hobsize > 0)
                        {
                            if (tipon)
                            {
                                tip = tip + "<br>"; //$NON-NLS-1$
                            }
                            tip = tip + Language.getString("ShipModelGUI.51").replace("%1", Double.toString(hobsize)).replace("%2", sizedesc); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                        }                    
                    }
                    
                    tip = tip + "</html>"; //$NON-NLS-1$
                    
                    if (tipon) lstINPACK.setTooltip(i, tip);
                }
                catch (Exception e1)
                {
                    UE.displayError(e1);
                    this.lstINPACK.addItem((ImageIcon)null, Language.getString("ShipModelGUI.16"), Language.getString("ShipModelGUI.16"), Language.getString("ShipModelGUI.16")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                }
            }
            this.lblINPACK.setText(this.strDescInPa.replace("%1", Integer.toString(result.size()))); //$NON-NLS-1$
            lstINPACK.setVisible(true);
            if (index < 0) index = 0;
            if (index >= result.size()) index = result.size() - 1;
            this.lstINPACK.setSelectedItemIndex(index);
            if (lstINSTALLED.getSelectedItemIndex() >= 0)
            {
                int l = lstINSTALLED.getSelectedItemIndex();
                lstINSTALLED.clearSelection();
                lstINSTALLED.setSelectedItemIndex(l);
            }
        }
       
        this.btnOPEN.setEnabled(true);
        this.btnCREATE.setEnabled(true);
    }
    
    private class LoadTask extends UEWorker<Vector<Object[]>>{
        private boolean installed;
        
        public LoadTask(boolean inst){
            super();
            this.installed = inst;
        }

        /* (non-Javadoc)
         * @see javax.swing.SwingWorker#doInBackground()
         */
        @Override
        protected Vector<Object[]> doInBackground()
        {
            String msg;
            ShipModelDataEntry entry;
            this.RESULT = new Vector<Object[]>();
            String p = "", s = ""; //$NON-NLS-1$ //$NON-NLS-2$
            Object[] data;
            
            this.updateStatus(UEWorker.STATUS_WORKING);

            if (this.installed)
            {
                try
                {
                    Texture textures = (Texture)ShipModelGUI.this.STBOF.getInternalFile(Stbof.fileTEXTURE_LST, true);

                    ArrayList<String> used = ((ShipList)STBOF.getInternalFile(Stbof.fileSHIPLIST_SST, true)).getGraphics();

                    msg = Language.getString("ShipModelGUI.17"); //$NON-NLS-1$
                
                    for (int i = 0; i < ShipModelGUI.this.DATA.getNumberOfEntries(); i++)
                    {
                        if (this.isCancelled())
                        {
                            this.updateStatus(UEWorker.STATUS_CANCEL);
                            return this.RESULT;
                        }
                    
                        data = new Object[5];
                    
                        entry = ShipModelGUI.this.DATA.getEntry(i);
                    
                        this.sendMessage(msg.replace("%1", entry.GRAPHICS)); //$NON-NLS-1$
                    
                        // always add the slot name
                        data[1] = entry.GRAPHICS;
                        
                        // phasers and scale are transferred to hack - load once and forget
                        int index;
                        int scale;

                        if (entry.GRAPHICS.indexOf("_") < 0) //$NON-NLS-1$
                        {
                            index = DATA.getIndex(entry.GRAPHICS);
                        }
                        else
                        {
                            index = DATA.getIndex(entry.GRAPHICS.substring(0, entry.GRAPHICS.indexOf("_"))); //$NON-NLS-1$
                        }
                        
                        data[2] = p = Language.getString("ShipModelGUI.18").replace("%1", Integer.toString(DATA.getNumberOfPhasers(index))); //$NON-NLS-1$ //$NON-NLS-2$

                        scale = Math.round(DATA.getScale(index)*100);
                        data[3] = s = Language.getString("ShipModelGUI.19").replace("%1", Integer.toString(scale)); //$NON-NLS-1$ //$NON-NLS-2$

                        // regular
                        if (!used.contains(entry.GRAPHICS.toLowerCase()))
                        {
                            data[2] = Language.getString("ShipModelGUI.20"); //$NON-NLS-1$
                            data[3] = ""; //$NON-NLS-1$
                        }

                        // img
                        try
                        {
                            TargaImage icon = STBOF.getTargaImage("i_" + entry.GRAPHICS + "60.tga", false); //$NON-NLS-1$ //$NON-NLS-2$

                            data[0] = new ImageIcon(icon.getImage(true));
                            
                        }
                        catch (Exception e1)
                        {
                            data[0] = null;
                        }

                        try
                        {
                            GenericFile gf = (GenericFile)STBOF.getInternalFile(entry.GRAPHICS + "_a.hob", false); //$NON-NLS-1$
                            ByteArrayInputStream bais = new ByteArrayInputStream(gf.getData());
                            HobFile hob = new HobFile(bais);
                            bais.close();
                            if (hob.numberOfTextures() > 0)
                            {
                                String t = textures.get(hob.getTextureID(0));
                                gf = (GenericFile)STBOF.getInternalFile(t, false);
                                bais = new ByteArrayInputStream(gf.getData());
                                GifImage gif = new GifImage(bais);
                                bais.close();
                                Dimension d = gif.getSize();
                                data[4] = d.height + "x" + d.width; //$NON-NLS-1$
                            }
                        }
                        catch (Exception e1)
                        {
                            data[4] = null;
                        }

                        this.RESULT.add(data);
                        
                        // now test hacks
                        if (entry.GRAPHICS.length() < 3)
                        {
                            data = new Object[]{null, entry.GRAPHICS + "_", p, s, null}; //$NON-NLS-1$
                            
                            if (!used.contains(entry.GRAPHICS.toLowerCase() + "_")) //$NON-NLS-1$
                            {
                                data[2] = Language.getString("ShipModelGUI.20"); //$NON-NLS-1$
                                data[3] = ""; //$NON-NLS-1$
                            }
                            
                            // img
                            try
                            {
                                TargaImage icon = STBOF.getTargaImage("i_" + entry.GRAPHICS + "_60.tga", false); //$NON-NLS-1$ //$NON-NLS-2$
                                
                                data[0] = new ImageIcon(icon.getImage(true));
                            }
                            catch (Exception e1)
                            {
                                data[0] = null;
                            }

                            try
                            {
                                GenericFile gf = (GenericFile)STBOF.getInternalFile(entry.GRAPHICS + "__a.hob", false); //$NON-NLS-1$
                                ByteArrayInputStream bais = new ByteArrayInputStream(gf.getData());
                                HobFile hob = new HobFile(bais);
                                bais.close();
                                if (hob.numberOfTextures() > 0)
                                {
                                    String t = textures.get(hob.getTextureID(0));
                                    gf = (GenericFile)STBOF.getInternalFile(t, false);
                                    bais = new ByteArrayInputStream(gf.getData());
                                    GifImage gif = new GifImage(bais);
                                    bais.close();
                                    Dimension d = gif.getSize();
                                    data[4] = d.height + "x" + d.width; //$NON-NLS-1$
                                }
                            }
                            catch (Exception e1)
                            {
                                data[4] = null;
                            }

                            this.RESULT.add(data);
                            
                            if (entry.GRAPHICS.length() < 2)
                            {
                                data = new Object[]{null, entry.GRAPHICS + "__", p, s, null}; //$NON-NLS-1$
                                
                                if (!used.contains(entry.GRAPHICS.toLowerCase() + "__")) //$NON-NLS-1$
                                {
                                    data[2] = Language.getString("ShipModelGUI.20"); //$NON-NLS-1$
                                    data[3] = ""; //$NON-NLS-1$
                                }
                                
                                // img
                                try
                                {
                                    TargaImage icon = STBOF.getTargaImage("i_" + entry.GRAPHICS + "__60.tga", false); //$NON-NLS-1$ //$NON-NLS-2$
                                    
                                    data[0] = new ImageIcon(icon.getImage(true));
                                }
                                catch (Exception e1)
                                {
                                    data[0] = null;
                                }

                                try
                                {
                                    GenericFile gf = (GenericFile)STBOF.getInternalFile(entry.GRAPHICS + "___a.hob", false); //$NON-NLS-1$
                                    ByteArrayInputStream bais = new ByteArrayInputStream(gf.getData());
                                    HobFile hob = new HobFile(bais);
                                    bais.close();
                                    if (hob.numberOfTextures() > 0)
                                    {
                                        String t = textures.get(hob.getTextureID(0));
                                        gf = (GenericFile)STBOF.getInternalFile(t, false);
                                        bais = new ByteArrayInputStream(gf.getData());
                                        GifImage gif = new GifImage(bais);
                                        bais.close();
                                        Dimension d = gif.getSize();
                                        data[4] = d.height + "x" + d.width; //$NON-NLS-1$
                                    }
                                }
                                catch (Exception e1)
                                {
                                    data[4] = null;
                                }

                                this.RESULT.add(data);
                            }
                        }
                    }

                    // get texture hashes
                    String tex;
                    msg = "Calculating texture hash: %1";

                    texture_hashes.clear();

                    for (int i = 0; i < textures.getTextureNum(); i++)
                    {
                        if (this.isCancelled())
                        {
                            this.updateStatus(UEWorker.STATUS_CANCEL);
                            return this.RESULT;
                        }

                        if (textures.isTextureValid(i))
                        {
                            tex = textures.get(i);

                            this.sendMessage(msg.replace("%1", tex)); //$NON-NLS-1$

                            try
                            {
                                texture_hashes.add(STBOF.getInternalFileHash(tex));
                            }
                            catch (FileNotFoundException ex)
                            {
                                textures.remove(i);
                                texture_hashes.add(new byte[0]);
                            }
                        }
                        else
                        {
                            texture_hashes.add(new byte[0]);
                        }
                    }
                }
                catch (Exception e)
                {
                    this.reportError(e);
                    return this.RESULT;
                }
            }else
            {
                msg = Language.getString("ShipModelGUI.21"); //$NON-NLS-1$
                this.sendMessage(msg);
                ShipModelDataEntry sml;

                try
                {
                    String[] candidates = ShipModelGUI.this.SHIP_PACKAGE.getCandidates();

                    for (String g : candidates)
                    {
                        data = new Object[5];
                        
                        data[1] = g;
                        
                        // phasers and scale are transferred to hack - load once and forget
                        int scale;

                        if (g.indexOf("_") < 0) //$NON-NLS-1$
                        {
                            sml = SHIP_PACKAGE.getShipModelDataEntry(g);
                        }
                        else
                        {
                            sml = SHIP_PACKAGE.getShipModelDataEntry(g.substring(0, g.indexOf("_"))); //$NON-NLS-1$
                        }
                        
                        
                        data[2] = Language.getString("ShipModelGUI.18").replace("%1", Integer.toString(sml.NUM_PHASERS)); //$NON-NLS-1$ //$NON-NLS-2$

                        scale = Math.round(sml.SCALE*100);
                        data[3] = Language.getString("ShipModelGUI.19").replace("%1", Integer.toString(scale)); //$NON-NLS-1$ //$NON-NLS-2$
                        
                        try
                        {
                            GenericFile gun = ShipModelGUI.this.SHIP_PACKAGE.getFile("i_" + g.toLowerCase() + "60.tga"); //$NON-NLS-1$ //$NON-NLS-2$
                            ByteArrayInputStream baos = new ByteArrayInputStream(gun.getData());
                            TargaImage icon = new TargaImage(baos);
                            data[0] = new ImageIcon(icon.getImage(true));
                        }
                        catch (Exception e1)
                        {
                            data[0] = null;
                            data[2] = Language.getString("ShipModelGUI.22"); //$NON-NLS-1$
                            data[3] = ""; //$NON-NLS-1$
                        }

                        try
                        {
                            HobFile hob = SHIP_PACKAGE.getHobFile(g + "_a.hob"); //$NON-NLS-1$
                            if (hob.numberOfTextures() > 0)
                            {
                                String t = SHIP_PACKAGE.getTexture(hob.getTextureID(0));
                                GenericFile gf = SHIP_PACKAGE.getFile(t);
                                ByteArrayInputStream bais = new ByteArrayInputStream(gf.getData());
                                GifImage gif = new GifImage(bais);
                                bais.close();
                                Dimension d = gif.getSize();
                                data[4] = d.height + "x" + d.width; //$NON-NLS-1$
                            }
                        }
                        catch (Exception e1)
                        {
                            data[4] = null;
                        }

                        this.RESULT.add(data);
                    }

                    // get texture hashes
                    String tex;
                    msg = "Calculating texture hash: %1";

                    texture_pack_hashes.clear();

                    for (int i = 0; i < ShipModelGUI.this.SHIP_PACKAGE.getTextureNum(); i++)
                    {
                        if (this.isCancelled())
                        {
                            this.updateStatus(UEWorker.STATUS_CANCEL);
                            return this.RESULT;
                        }

                        if (SHIP_PACKAGE.isTextureValid(i))
                        {
                            tex = SHIP_PACKAGE.getTexture(i);

                            this.sendMessage(msg.replace("%1", tex)); //$NON-NLS-1$

                            try
                            {

                                texture_pack_hashes.add(SHIP_PACKAGE.getInternalFileHash(tex));
                            }
                            catch (FileNotFoundException ex)
                            {
                                SHIP_PACKAGE.removeTexture(i);
                                texture_pack_hashes.add(new byte[0]);
                            }
                        }
                        else
                        {
                            texture_pack_hashes.add(new byte[0]);
                        }
                    }
                }
                catch (Exception e)
                {
                    this.reportError(e);
                    return this.RESULT;
                }
            }
            
            this.updateStatus(UEWorker.STATUS_DONE);

            return this.RESULT;
        }
    }
    
    private class ExportTask extends UEWorker<Boolean>
    {
        private String GRAPHICS;
        
        public ExportTask(String g){
            super();
            this.GRAPHICS = g;
        }

        /* (non-Javadoc)
         * @see javax.swing.SwingWorker#doInBackground()
         */
        @Override
        protected Boolean doInBackground()
        {
            String msg;
            ShipModelDataEntry entry;
            HobFile[] hob = new HobFile[3];
            InternalFile inf;
            ByteArrayInputStream bais;
            ByteArrayOutputStream baos;
            String g = this.GRAPHICS;
            String file;
            GifColorMap gcm;
            char c = 'a';
            
            this.RESULT = false;
            
            this.updateStatus(UEWorker.STATUS_WORKING);

            try
            {
                Texture textures = (Texture)ShipModelGUI.this.STBOF.getInternalFile(Stbof.fileTEXTURE_LST, true);
                PaletteList palettes = (PaletteList)ShipModelGUI.this.STBOF.getInternalFile(Stbof.filePALETTE_LST, true);
                
                String ge = this.GRAPHICS;
                int index = ge.indexOf("_"); //$NON-NLS-1$
                
                if (index >= 0)
                {
                    ge = ge.substring(0, index);
                }

                // get free graphics string
                if (ShipModelGUI.this.SHIP_PACKAGE.isGraphicsStringUsed(ge))
                {
                    g = ShipModelGUI.this.SHIP_PACKAGE.getUnusedGraphicsString();
                }
                else
                {
                    g = ge;
                }
                
                // export ship model data

                msg = Language.getString("ShipModelGUI.23").replace("%1", ge); //$NON-NLS-1$ //$NON-NLS-2$
                this.sendMessage(msg);
                
                // load hob files
                Hashtable<Integer, Integer> tid = new Hashtable<Integer, Integer>();
                
                for (int i = 0; i < hob.length; i++)
                {
                    if (this.isCancelled())
                    {
                        ShipModelGUI.this.SHIP_PACKAGE.clearGraphicsString(g);
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        return this.RESULT;
                    }
                    
                    inf = ShipModelGUI.this.STBOF.getInternalFile(this.GRAPHICS + "_" + c + ".hob", false); //$NON-NLS-1$ //$NON-NLS-2$
                    baos = new ByteArrayOutputStream();
                    inf.save(baos);
                    bais = new ByteArrayInputStream(baos.toByteArray());
                    hob[i] = new HobFile(bais);
                    hob[i].setName(g.toLowerCase() + "_" + c++ + ".hob"); //$NON-NLS-1$ //$NON-NLS-2$
                    
                    for (int j = 0; j < hob[i].numberOfTextures(); j++)
                    {
                        tid.put(hob[i].getTextureID(j), hob[i].getTextureID(j));
                    }
                }
                
                // export textures from hobs
                msg = Language.getString("ShipModelGUI.24"); //$NON-NLS-1$
                this.sendMessage(msg);

                Integer[] t = tid.values().toArray(new Integer[0]);
                
                for (Integer i : t)
                {
                    if (this.isCancelled())
                    {
                        ShipModelGUI.this.SHIP_PACKAGE.clearGraphicsString(g);
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        return this.RESULT;
                    }

                    file = textures.get(i);

                    byte[] hash = STBOF.getInternalFileHash(file);

                    int k = Tools.getIndex(texture_pack_hashes, hash);

                    if (k < 0)
                    {
                        inf = ShipModelGUI.this.STBOF.getInternalFile(file, false);
                        baos = new ByteArrayOutputStream();
                        inf.save(baos);
                        inf = new GenericFile(inf.getName(), baos.toByteArray());
                        gcm = palettes.getColorMap(i);

                        if (gcm == null)
                        {
                            this.reportError(new Exception(Language.getString("ShipModelGUI.25"))); //$NON-NLS-1$
                            return RESULT;
                        }
                    
                        tid.put(i, ShipModelGUI.this.SHIP_PACKAGE.addTexture((GenericFile)inf, gcm));
                    }
                    else
                    {
                        tid.put(i, k);
                    }
                }
                
                // export hob files
                msg = Language.getString("ShipModelGUI.26"); //$NON-NLS-1$
                this.sendMessage(msg);

                for (HobFile h : hob)
                {
                    if (h == null)
                    {
                        continue;
                    }
                    
                    for (int j = 0; j < h.numberOfTextures(); j++)
                    {
                        h.setTextureID(j, tid.get(h.getTextureID(j)));
                    }
                    
                    ShipModelGUI.this.SHIP_PACKAGE.addHobFile(h);
                }
                
                // export gui graphic files
                msg = Language.getString("ShipModelGUI.27"); //$NON-NLS-1$
                this.sendMessage(msg);
                
                inf = ShipModelGUI.this.STBOF.getInternalFile("i_" + this.GRAPHICS + "120.tga", false); //$NON-NLS-1$ //$NON-NLS-2$
                baos = new ByteArrayOutputStream();
                inf.save(baos);
                inf = new GenericFile("i_" + g + "120.tga", baos.toByteArray()); //$NON-NLS-1$ //$NON-NLS-2$
                ShipModelGUI.this.SHIP_PACKAGE.addFile((GenericFile)inf);

                try{
                    inf = ShipModelGUI.this.STBOF.getInternalFile("i_" + this.GRAPHICS + "w.tga", false); //$NON-NLS-1$ //$NON-NLS-2$
                    baos = new ByteArrayOutputStream();
                    inf.save(baos);
                    inf = new GenericFile("i_" + g + "w.tga", baos.toByteArray()); //$NON-NLS-1$ //$NON-NLS-2$
                    ShipModelGUI.this.SHIP_PACKAGE.addFile((GenericFile)inf);
                }
                catch (Exception f)
                {
                    // make a copy of i_xxx120.tga
                    inf = new GenericFile("i_" + g + "w.tga", baos.toByteArray()); //$NON-NLS-1$ //$NON-NLS-2$
                    ShipModelGUI.this.SHIP_PACKAGE.addFile((GenericFile)inf);
                }
                    
                inf = ShipModelGUI.this.STBOF.getInternalFile("i_" + this.GRAPHICS + "30.tga", false); //$NON-NLS-1$ //$NON-NLS-2$
                baos = new ByteArrayOutputStream();
                inf.save(baos);
                inf = new GenericFile("i_" + g + "30.tga", baos.toByteArray()); //$NON-NLS-1$ //$NON-NLS-2$
                ShipModelGUI.this.SHIP_PACKAGE.addFile((GenericFile)inf);

                inf = ShipModelGUI.this.STBOF.getInternalFile("i_" + this.GRAPHICS + "60.tga", false); //$NON-NLS-1$ //$NON-NLS-2$
                baos = new ByteArrayOutputStream();
                inf.save(baos);
                inf = new GenericFile("i_" + g + "60.tga", baos.toByteArray()); //$NON-NLS-1$ //$NON-NLS-2$
                ShipModelGUI.this.SHIP_PACKAGE.addFile((GenericFile)inf);

                try
                {
                    inf = ShipModelGUI.this.STBOF.getInternalFile("i_" + this.GRAPHICS + "170.tga", false); //$NON-NLS-1$ //$NON-NLS-2$
                    baos = new ByteArrayOutputStream();
                    inf.save(baos);
                    inf = new GenericFile("i_" + g + "170.tga", baos.toByteArray()); //$NON-NLS-1$ //$NON-NLS-2$
                    ShipModelGUI.this.SHIP_PACKAGE.addFile((GenericFile)inf);
                }
                catch (Exception e)
                {
                    // if not found use 120 to make the image
                    inf = ShipModelGUI.this.STBOF.getInternalFile("i_" + this.GRAPHICS + "120.tga", false); //$NON-NLS-1$ //$NON-NLS-2$
                    baos = new ByteArrayOutputStream();
                    inf.save(baos);
                    bais = new ByteArrayInputStream(baos.toByteArray());
                    TargaImage ti = new TargaImage(bais);
                    int[] pixels = new int[24140];
                    int line;
                    
                    for (int i = 141; i >= 0; i--) { // 100
                        line = i * 170; // 170 - 120 = 50
                        for (int j = 0; j < 170; j++)
                        {
                            if ((i < 20) || (i >= 120))
                            {
                                pixels[line + j] = 0;
                            }
                            else
                            {
                                if ((j < 25) || (j >= 145))
                                {
                                    pixels[line + j] = 0;
                                }
                                else
                                {
                                    pixels[line + j] = ti.pixels[(i-20)*120 + j - 25];
                                }
                            }
                        }
                    }

                    ti = new TargaImage(pixels, 170, 141);
                    baos = new ByteArrayOutputStream();
                    ti.save(baos);
                    
                    inf = new GenericFile("i_" + g + "170.tga", baos.toByteArray()); //$NON-NLS-1$ //$NON-NLS-2$
                    ShipModelGUI.this.SHIP_PACKAGE.addFile((GenericFile)inf);
                }
                
                entry = new ShipModelDataEntry(g.toUpperCase(), DATA.getEntry(ge.toUpperCase()));
                ShipModelGUI.this.SHIP_PACKAGE.addShipModelDataEntry(entry);
                
                // export ship class
                try
                {
                    String cls = ((ShipList)STBOF.getInternalFile(Stbof.fileSHIPLIST_SST, true)).getClassByPrefix(this.GRAPHICS);
                    if (cls != null)
                    {
                        SHIP_PACKAGE.setClassForPrefix(entry.GRAPHICS, cls);
                    }
                }
                catch (Exception e)
                {
                }
            }
            catch (Exception e)
            {
                try
                {
                    ShipModelGUI.this.SHIP_PACKAGE.clearGraphicsString(g);
                }
                catch (IOException e1)
                {
                    this.reportError(e);
                }
                this.reportError(e);
                return this.RESULT;
            }
            
            this.updateStatus(UEWorker.STATUS_DONE);

            this.RESULT = true;
            return this.RESULT;
        }
    }
   
    private class ImportTask extends UEWorker<Boolean>
    {
        private String GRAPHICS;
        private String DEST;
        
        public ImportTask(String what, String where){
            super();
            this.GRAPHICS = what;
            this.DEST = where;
        }

        /* (non-Javadoc)
         * @see javax.swing.SwingWorker#doInBackground()
         */
        @Override
        protected Boolean doInBackground()
        {
            String msg;
            ShipModelDataEntry src, dest;
            HobFile[] hob = new HobFile[3];
            GenericFile gf;
            ByteArrayInputStream bais;
            ByteArrayOutputStream baos;
            String file;
            GifColorMap gcm;
            char c = 'a';
            ArrayList<InternalFile> undo_files = new ArrayList<InternalFile>();
            
            this.RESULT = false;
            
            this.updateStatus(UEWorker.STATUS_WORKING);

            try
            {
                Texture textures = (Texture)ShipModelGUI.this.STBOF.getInternalFile(Stbof.fileTEXTURE_LST, true);
                PaletteList palettes = (PaletteList)ShipModelGUI.this.STBOF.getInternalFile(Stbof.filePALETTE_LST, true);
                
                src = SHIP_PACKAGE.getShipModelDataEntry(GRAPHICS);
                
                // see if the two slots are compatible
                this.sendMessage(Language.getString("ShipModelGUI.28")); //$NON-NLS-1$

                if (STBOF.isShipGraphicsStringUsed(DEST))
                {
                    msg = Language.getString("ShipModelGUI.30"); //$NON-NLS-1$
                    msg.replace("%1", GRAPHICS); //$NON-NLS-1$
                    this.reportError(new Exception(msg.replace("%2", DEST))); //$NON-NLS-1$
                    return RESULT;
                }

                if (DEST.contains("_")) //$NON-NLS-1$
                {
                    dest = DATA.getEntry(DEST.substring(0, DEST.indexOf("_"))); //$NON-NLS-1$
                }
                else
                {
                    dest = DATA.getEntry(DEST);
                }
                
                if (dest == null)
                {
                    // bummer
                    this.reportError(new Exception(Language.getString("ShipModelGUI.29").replace("%1", DEST))); //$NON-NLS-1$ //$NON-NLS-2$
                    return RESULT;
                }
                
                if (!src.isCompatible(dest))
                {
                    if ((!dest.GRAPHICS.equals(DEST) && STBOF.isShipGraphicsStringUsed(dest.GRAPHICS)))
                    {
                        msg = Language.getString("ShipModelGUI.30"); //$NON-NLS-1$
                        msg.replace("%1", GRAPHICS); //$NON-NLS-1$
                        this.reportError(new Exception(msg.replace("%2", DEST))); //$NON-NLS-1$
                        return RESULT;
                    }
                    
                    if (dest.GRAPHICS.length() < 3)
                    {
                        if ((!(dest.GRAPHICS + "_").equals(DEST) && STBOF.isShipGraphicsStringUsed(dest.GRAPHICS + "_"))) //$NON-NLS-1$ //$NON-NLS-2$
                        {
                            msg = Language.getString("ShipModelGUI.30"); //$NON-NLS-1$
                            msg.replace("%1", GRAPHICS); //$NON-NLS-1$
                            this.reportError(new Exception(msg.replace("%2", DEST))); //$NON-NLS-1$
                            return RESULT;
                        }
                        
                        if (dest.GRAPHICS.length() < 2)
                        {
                            if ((!(dest.GRAPHICS + "__").equals(DEST) && STBOF.isShipGraphicsStringUsed(dest.GRAPHICS + "__"))) //$NON-NLS-1$ //$NON-NLS-2$
                            {
                                msg = Language.getString("ShipModelGUI.30"); //$NON-NLS-1$
                                msg.replace("%1", GRAPHICS); //$NON-NLS-1$
                                this.reportError(new Exception(msg.replace("%2", DEST))); //$NON-NLS-1$
                                return RESULT;
                            }
                        }
                    }
                }
                
                // load hob files
                msg = Language.getString("ShipModelGUI.31"); //$NON-NLS-1$

                Hashtable<Integer, Integer> textureid = new Hashtable<Integer, Integer>();
                
                for (int i = 0; i < hob.length; i++)
                {
                    if (this.isCancelled())
                    {
                        undo(undo_files);
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        return this.RESULT;
                    }
                    
                    this.sendMessage(msg.replace("%1", GRAPHICS.toLowerCase() + "_" + c + ".hob")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                    
                    if (STBOF.hasInternalFile(DEST.toLowerCase() + "_" + c + ".hob")) //$NON-NLS-1$ //$NON-NLS-2$
                    {
                        undo_files.add(STBOF.getInternalFile(DEST.toLowerCase() + "_" + c + ".hob", false)); //$NON-NLS-1$ //$NON-NLS-2$
                    }
                    
                    hob[i] = new HobFile(SHIP_PACKAGE.getHobFile(GRAPHICS.toLowerCase() + "_" + c + ".hob")); //$NON-NLS-1$ //$NON-NLS-2$
                    hob[i].setName(DEST.toLowerCase() + "_" + c++ + ".hob"); //$NON-NLS-1$ //$NON-NLS-2$
                    
                    for (int j = 0; j < hob[i].numberOfTextures(); j++)
                    {
                        textureid.put(hob[i].getTextureID(j), hob[i].getTextureID(j));
                    }
                }
                
                // import textures from hobs
                msg = Language.getString("ShipModelGUI.32"); //$NON-NLS-1$

                Integer[] t = textureid.values().toArray(new Integer[0]);
                
                String gif = DEST.toLowerCase();

                while (textures.anyTextureFileBeginsWith(gif + "_")) //$NON-NLS-1$
                {
                    gif = Tools.getRandomString(3).toLowerCase();
                }
                
                c = 'a';
                
                for (Integer i : t)
                {
                    if (this.isCancelled())
                    {
                        undo(undo_files);
                        this.updateStatus(UEWorker.STATUS_CANCEL);
                        return this.RESULT;
                    }

                    file = SHIP_PACKAGE.getTexture(i);

                    this.sendMessage(msg.replace("%1", file)); //$NON-NLS-1$

                    // check if we already have this texture in store
                    byte[] hash = SHIP_PACKAGE.getInternalFileHash(file);

                    int index = Tools.getIndex(texture_hashes, hash);

                    if (index < 0)
                    {
                        gf = new GenericFile(gif + "_" + c++ + ".gif", SHIP_PACKAGE.getFile(file).getData()); //$NON-NLS-1$ //$NON-NLS-2$

                        gcm = SHIP_PACKAGE.getColorMap(i);

                        if (gcm == null)
                        {
                            bais = new ByteArrayInputStream(gf.getData());
                            GifImage gi = new GifImage(bais);
                            gcm = gi.getInitialGifColorMap();
                        }

                        if (gcm == null)
                        {
                            // undo
                            undo(undo_files);
                            this.reportError(new Exception(Language.getString("ShipModelGUI.33"))); //$NON-NLS-1$
                            return this.RESULT;
                        }

                        if (textures.getUnusedIndex() >= 0)
                        {
                            index = textures.getUnusedIndex();

                            textures.set(index, gf.getName());
                        }
                        else
                        {
                            index = textures.add(gf.getName());
                        }

                        textureid.put(i, index);
                        palettes.setColorMap(index, gcm);

                        STBOF.addInternalFile(gf);
                    }
                    else
                    {
                        textureid.put(i, index);
                    }
                }
                
                // import hob files
                msg = Language.getString("ShipModelGUI.34"); //$NON-NLS-1$
                this.sendMessage(msg);

                for (HobFile h : hob)
                {
                    if (h == null)
                    {
                        continue;
                    }
                    
                    for (int j = 0; j < h.numberOfTextures(); j++)
                    {
                        h.setTextureID(j, textureid.get(h.getTextureID(j)));
                    }
                    
                    baos = new ByteArrayOutputStream();
                    h.save(baos);
                    baos.close();
                    STBOF.addInternalFile(new GenericFile(h.getName(), baos.toByteArray()));
                }
                
                // import gui graphic files
                msg = Language.getString("ShipModelGUI.35"); //$NON-NLS-1$
                this.sendMessage(msg);
                
                gf = SHIP_PACKAGE.getFile("i_" + this.GRAPHICS + "120.tga"); //$NON-NLS-1$ //$NON-NLS-2$
                gf = new GenericFile("i_" + DEST + "120.tga", gf.getData()); //$NON-NLS-1$ //$NON-NLS-2$
                if (STBOF.hasInternalFile(gf.getName()))
                {
                    undo_files.add(STBOF.getInternalFile(gf.getName(), false));
                }
                STBOF.addInternalFile(gf);

                gf = SHIP_PACKAGE.getFile("i_" + this.GRAPHICS + "w.tga"); //$NON-NLS-1$ //$NON-NLS-2$
                gf = new GenericFile("i_" + DEST + "w.tga", gf.getData()); //$NON-NLS-1$ //$NON-NLS-2$
                if (STBOF.hasInternalFile(gf.getName()))
                {
                    undo_files.add(STBOF.getInternalFile(gf.getName(), false));
                }
                STBOF.addInternalFile(gf);
                    
                gf = SHIP_PACKAGE.getFile("i_" + this.GRAPHICS + "30.tga"); //$NON-NLS-1$ //$NON-NLS-2$
                gf = new GenericFile("i_" + DEST + "30.tga", gf.getData()); //$NON-NLS-1$ //$NON-NLS-2$
                if (STBOF.hasInternalFile(gf.getName()))
                {
                    undo_files.add(STBOF.getInternalFile(gf.getName(), false));
                }
                STBOF.addInternalFile(gf);

                gf = SHIP_PACKAGE.getFile("i_" + this.GRAPHICS + "60.tga"); //$NON-NLS-1$ //$NON-NLS-2$
                gf = new GenericFile("i_" + DEST + "60.tga", gf.getData()); //$NON-NLS-1$ //$NON-NLS-2$
                if (STBOF.hasInternalFile(gf.getName()))
                {
                    undo_files.add(STBOF.getInternalFile(gf.getName(), false));
                }
                STBOF.addInternalFile(gf);

                gf = SHIP_PACKAGE.getFile("i_" + this.GRAPHICS + "170.tga"); //$NON-NLS-1$ //$NON-NLS-2$
                gf = new GenericFile("i_" + DEST + "170.tga", gf.getData()); //$NON-NLS-1$ //$NON-NLS-2$
                if (STBOF.hasInternalFile(gf.getName()))
                {
                    undo_files.add(STBOF.getInternalFile(gf.getName(), false));
                }
                STBOF.addInternalFile(gf);
                
                ScreensaverModelList sml = (ScreensaverModelList)TREK.getInternalFile(Trek.fileSCREENSAVER_MODEL_LIST, true);
                if (sml.getNumberofEntries() <= sml.getMax())
                {
                    sml.remove(GRAPHICS.toLowerCase() + ".hob"); //$NON-NLS-1$
                    
                    if (sml.getNumberofEntries() < sml.getMax())
                    {
                        sml.add(DEST.toLowerCase() + ".hob"); //$NON-NLS-1$
                    }
                }                   
                
                // set model data
                dest.copyAllDataFrom(dest.GRAPHICS, src);
                DATA.updateEntry(dest);
                
                // clean palette.lst
                palettes.trim(textures);
                
                if (palettes.getPalettesNum() > 127)
                {
                    msg = Language.getString("ShipModelGUI.36"); //$NON-NLS-1$
                    this.sendMessage(msg);
                    
                    this.updateStatus(UEWorker.STATUS_DONE_WITH_MSG);

                    this.RESULT = true;
                    return this.RESULT;
                }
            }
            catch (Exception e)
            {
                // undo
                undo(undo_files);
                this.reportError(e);
                return this.RESULT;
            }
            
            this.updateStatus(UEWorker.STATUS_DONE);

            this.RESULT = true;
            return this.RESULT;
        }
        
        private void undo(ArrayList<InternalFile> undo_files)
        {
            for (int i = 0; i < undo_files.size(); i++)
            {
                STBOF.addInternalFile(undo_files.get(i));
            }
        }
    }
    /* (non-Javadoc)
     * @see ue.gui.MainPanel#finalWarning()
     */
    @Override
    public void finalWarning() throws Exception {
        // nothing to do
    }
    
    @Override
    public String getHelpFileName()
    {
            return "ManageShipModels.html"; //$NON-NLS-1$
    }
}
