/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: ShipnameGUI.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Created: 6.9.2007
*/

package ue.gui.stbof;

import java.io.FileNotFoundException;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import ue.*;
import ue.edit.FileManager;
import ue.edit.exe.Trek;
import ue.edit.exe.trek.ShipnameCode;
import ue.edit.res.stbof.bin.*;
import ue.gui.*;
import ue.util.SettingsManager;

/**Used to edit shipnames.*/
@SuppressWarnings("serial")  //$NON-NLS-1$
public class ShipnameGUI extends MainPanel
{
	private Shipname SHIPNAMES;
    private Trek trek;
	//items
	private JComboBox GROUP;
	private JList NAMELIST;
	private JButton btnADD;
	private JButton btnEDIT;
	private JButton btnREMOVE;
	private JButton btnIMPORT;
	private JButton btnEXPORT;
	private JButton btnADD_GROUP;
	private JButton btnREM_GROUP;

    private static final int max = 248;
	// strings
	private static String strGROUP = Language.getString("ShipnameGUI.0"); //$NON-NLS-1$
	
	/**Returns an instance of this object.
	@param ship the Shipname object to work with.*/
	public ShipnameGUI(Shipname ship)
    {
		if (ship==null)
        {
			return;
		}

        if (UE.FILES.getFile(FileManager.P_TREK) != null)
        {
            trek = (Trek)UE.FILES.getFile(FileManager.P_TREK);
        }

		SHIPNAMES = ship;
		//create items
		btnADD = new JButton(Language.getString("ShipnameGUI.1")); //$NON-NLS-1$
		btnEDIT = new JButton(Language.getString("ShipnameGUI.2")); //$NON-NLS-1$
		btnREMOVE = new JButton(Language.getString("ShipnameGUI.3")); //$NON-NLS-1$
        btnIMPORT = new JButton("Import");
        btnEXPORT = new JButton("Export");
		btnADD_GROUP = new JButton("+"); //$NON-NLS-1$
		btnREM_GROUP = new JButton("-"); //$NON-NLS-1$
		GROUP = new JComboBox();
		NAMELIST = new JList();
		//set font
		Font def = UE.SETTINGS.getDefaultFont();
		btnADD.setFont(def);
		btnEDIT.setFont(def);
		btnREMOVE.setFont(def);
		btnIMPORT.setFont(def);
		btnEXPORT.setFont(def);
 		GROUP.setFont(def);
		NAMELIST.setFont(def);
        btnADD_GROUP.setEnabled(SHIPNAMES.getNumberOfGroups() < max
                && trek != null);
        btnREM_GROUP.setEnabled(SHIPNAMES.getNumberOfGroups() > 1
                && trek != null);

		//add listeners
		btnADD.addActionListener(new ActionListener()
        {
            @Override
			public void actionPerformed(ActionEvent e)
            {
				if (GROUP.getSelectedIndex() != -1)
                {
					String n = JOptionPane.showInputDialog(Language.getString("ShipnameGUI.4"), Language.getString("ShipnameGUI.5")); //$NON-NLS-1$ //$NON-NLS-2$
					if (n!=null)
                    {
						try
                        {
							SHIPNAMES.add(GROUP.getSelectedIndex(), n);
							fillNames();
						}
                        catch (Exception g)
                        {
							UE.displayError(g);
						}
					}
				}
			}
		});
		btnEDIT.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent e){
				if (GROUP.getSelectedIndex() != -1 && NAMELIST.getSelectedIndex() != -1){
					String n = JOptionPane.showInputDialog(Language.getString("ShipnameGUI.6"), NAMELIST.getSelectedValue()); //$NON-NLS-1$
					if (n!=null){
						try{
							SHIPNAMES.set(GROUP.getSelectedIndex(), (String)NAMELIST.getSelectedValue(), n);
							fillNames();
						}catch (Exception g){
							UE.displayError(g);
						}
					}
				}
			}
		});
		btnREMOVE.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent e){
				if (GROUP.getSelectedIndex() != -1 && NAMELIST.getSelectedIndex() != -1){
					try{
						SHIPNAMES.remove(GROUP.getSelectedIndex(), (String)NAMELIST.getSelectedValue());
						fillNames();
					}catch (Exception g){
						UE.displayError(g);
					}
				}
			}
		});
		GROUP.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent e){
				if (GROUP.getSelectedIndex() >= 0){
					fillNames();
				}
			}
		});
		NAMELIST.addListSelectionListener(new ListSelectionListener(){
            @Override
				public void valueChanged(ListSelectionEvent e){
					if (NAMELIST.getSelectedIndex()>=0) {
						btnEDIT.setEnabled(true);
						btnREMOVE.setEnabled(true);
					}else{
						btnEDIT.setEnabled(false);
						btnREMOVE.setEnabled(false);
					}
				}
		});
        btnIMPORT.addActionListener(new ActionListener()
        {
            @Override
			public void actionPerformed(ActionEvent e)
            {
                JFileChooser jfc = new JFileChooser(new File(UE.SETTINGS.getProperty(SettingsManager.LAST)));
                javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter()
                {
                    @Override
                    public boolean accept(File f)
                    {
                        if (f.isDirectory())
                        {
                            return true;
                        }
                        return f.getName().toLowerCase().endsWith(".txt");
                    }
                    @Override
                    public String getDescription()
                    {
                        return "*.txt"; //$NON-NLS-1$
                    }
                };
                jfc.setFileFilter(filter);
                jfc.setApproveButtonText("Import");
                jfc.setDialogTitle("Import list of names...");
                jfc.setMultiSelectionEnabled(false);

                int returnVal = jfc.showOpenDialog(UE.WINDOW);

                if (returnVal == JFileChooser.APPROVE_OPTION)
                {
                    BufferedReader in = null;

                    try
                    {
                        in = new BufferedReader(new FileReader(jfc.getSelectedFile()));

                        String str = in.readLine();

                        while (str != null)
                        {
                            str = str.trim();
                            if (str.length() > 0)
                            {
                                SHIPNAMES.add(GROUP.getSelectedIndex(), str);
                            }
                            str = in.readLine();
                        }

                    } catch (Exception ex) {
                        UE.displayError(ex);
                    } finally {
                        try {
                            in.close();
                        } catch (IOException ex) {
                            UE.displayError(ex);
                        }
                    }

                    fillNames();
                }
			}
		});
        btnEXPORT.addActionListener(new ActionListener()
        {
            @Override
			public void actionPerformed(ActionEvent e)
            {
                JFileChooser jfc = new JFileChooser(new File(UE.SETTINGS.getProperty(SettingsManager.LAST)));
                javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter()
                {
                    @Override
                    public boolean accept(File f)
                    {
                        if (f.isDirectory())
                        {
                            return true;
                        }
                        return f.getName().toLowerCase().endsWith(".txt");
                    }
                    @Override
                    public String getDescription()
                    {
                        return "*.txt"; //$NON-NLS-1$
                    }
                };
                jfc.setFileFilter(filter);
                jfc.setApproveButtonText("Export");
                jfc.setDialogTitle("Export list of names...");
                jfc.setMultiSelectionEnabled(false);

                int returnVal = jfc.showSaveDialog(UE.WINDOW);

                if (returnVal == JFileChooser.APPROVE_OPTION)
                {
                    File file = jfc.getSelectedFile();

                    if (!file.getName().toLowerCase().endsWith(".txt"))
                    {
                        file = new File(file.getParentFile(), file.getName() + ".txt");
                    }

                    if (file.exists())
                    {
                        String msg = Language.getString("MainMenu.12");
                        msg = msg.replace("%1", file.getName());
                        int response = JOptionPane.showConfirmDialog(UE.WINDOW, msg,
                                UE.APP_NAME, JOptionPane.YES_NO_OPTION,
                                JOptionPane.QUESTION_MESSAGE);

                        if (response != JOptionPane.YES_OPTION)
                        {
                            return;
                        }
                    }

                    BufferedWriter out = null;

                    try
                    {
                        out = new BufferedWriter(new FileWriter(file));
                        String[] names = SHIPNAMES.getEntries(GROUP.getSelectedIndex());

                        for (int i = 0; i < names.length; i++)
                        {
                            out.write(names[i]);
                            out.newLine();
                        }

                    } catch (Exception ex) {
                        UE.displayError(ex);
                    } finally {
                        try {
                            out.close();
                        } catch (IOException ex) {
                            UE.displayError(ex);
                        }
                    }
                }
			}
		});
		btnADD_GROUP.addActionListener(new ActionListener()
        {
            @Override
			public void actionPerformed(ActionEvent e)
            {
                if (SHIPNAMES.getNumberOfGroups() == 62)
                {
                    try
                    {
                        ShipnameCode fix = (ShipnameCode) trek.getInternalFile(Trek.fileSHIPNAME_CODE, true);
                        
                        if (fix.getActiveOption() != ShipnameCode.OPTION_EXTENDED)
                        {
                            int res = JOptionPane.showConfirmDialog(UE.WINDOW,
                                    "This action requires UE to patch trek.exe with the" +
                                    " extended shipname.bin and shipname appendix fix" +
                                    " by Spocks-cuddly-tribble. Continue?",
                                    UE.APP_NAME, JOptionPane.YES_NO_OPTION);

                            if (res == JOptionPane.YES_OPTION) {
                                fix.setActiveOption(ShipnameCode.OPTION_EXTENDED);
                            }
                            else {
                                return;
                            }
                        }
                    } catch (FileNotFoundException ex) {
                        UE.displayError(ex);
                        return;
                    }
                }

                try
                {
                    SHIPNAMES.addGroup();
                    btnADD_GROUP.setEnabled(SHIPNAMES.getNumberOfGroups() < max);
                    fillGroups();
                    GROUP.setSelectedIndex(SHIPNAMES.getNumberOfGroups() - 1);
                }
                catch (Exception ex)
                {
                    UE.displayError(ex);
                }
			}
		});
		btnREM_GROUP.addActionListener(new ActionListener()
        {
            @Override
			public void actionPerformed(ActionEvent e)
            {
                if (SHIPNAMES.getNumberOfGroups() == 63)
                {
                    try
                    {
                        ShipnameCode fix = (ShipnameCode) trek.getInternalFile(Trek.fileSHIPNAME_CODE, true);

                        if (fix.getActiveOption() == ShipnameCode.OPTION_EXTENDED)
                        {
                            fix.setActiveOption(ShipnameCode.OPTION_APPENDIX_ONLY);
                        }
                    } catch (FileNotFoundException ex) {
                        UE.displayError(ex);
                        return;
                    }
                }

                try
                {
                    SHIPNAMES.removeGroup(GROUP.getSelectedIndex());
                    btnREM_GROUP.setEnabled(SHIPNAMES.getNumberOfGroups() > 1);
                    fillGroups();
                }
                catch (Exception ex)
                {
                    UE.displayError(ex);
                }
			}
		});

		//misc
		NAMELIST.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        fillGroups();

		//add items
        JPanel pnlAll = new JPanel();

        GroupLayout layout = new GroupLayout(pnlAll);
        pnlAll.setLayout(layout);

        layout.setAutoCreateGaps(true);
        layout.setAutoCreateContainerGaps(true);

        JPanel pnlTop = new JPanel(new BorderLayout());
        pnlTop.add(GROUP, BorderLayout.CENTER);

        JPanel pnlCenter = new JPanel(new BorderLayout());
		JScrollPane scrolly = new JScrollPane(NAMELIST);
        pnlCenter.add(scrolly, BorderLayout.CENTER);

        JPanel pnlOptions = new JPanel(new GridLayout(0, 1, 5, 5));
        pnlOptions.add(btnADD);
        pnlOptions.add(btnEDIT);
        pnlOptions.add(btnREMOVE);
        pnlOptions.add(btnIMPORT);
        pnlOptions.add(btnEXPORT);

        JPanel pnlEast = new JPanel(new BorderLayout());
        pnlEast.add(pnlOptions, BorderLayout.NORTH);
        pnlEast.setPreferredSize(pnlEast.getMinimumSize());

        JPanel pnlGRPOPT = new JPanel(new GridLayout(1, 0, 5, 5));
//        pnlGRPOPT.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));
        pnlGRPOPT.add(btnADD_GROUP);
        pnlGRPOPT.add(btnREM_GROUP);

        layout.linkSize(SwingConstants.HORIZONTAL, pnlGRPOPT, pnlEast);

        layout.setHorizontalGroup(
           layout.createSequentialGroup()
              .addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
                   .addComponent(pnlTop, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE)
                   .addComponent(pnlCenter, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
//              .addComponent(pnlEast, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE)
              .addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
                   .addComponent(pnlGRPOPT, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE)
                   .addComponent(pnlEast, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE))
        );

        layout.setVerticalGroup(
           layout.createSequentialGroup()
              .addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
                   .addComponent(pnlTop, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE)
                   .addComponent(pnlGRPOPT, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE))
//              .addComponent(pnlTop, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE)
              .addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
                   .addComponent(pnlCenter, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                   .addComponent(pnlEast, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE))
        );

        setLayout(new BorderLayout());
        add(pnlAll, BorderLayout.CENTER);
	}
    
	/*this is used to apply recent changes before panel is removed
	no need for it in this case tho*/
	@Override
	public void finalWarning() throws Exception{};

    private void fillGroups()
    {
        GROUP.removeAllItems();
		for (int i = 0; i < SHIPNAMES.getNumberOfGroups(); i++)
        {
			GROUP.addItem(strGROUP.replace("%1", Integer.toString(i))); //$NON-NLS-1$
		}
    }

    //fills the names list
	private void fillNames()
    {
		if (GROUP.getSelectedIndex() != -1)
        {
			try{
                int sel = NAMELIST.getSelectedIndex();
                String[] names = SHIPNAMES.getEntries(GROUP.getSelectedIndex());
				NAMELIST.setListData(names);
                if (sel >= 0 && sel < names.length)
                {
                    NAMELIST.setSelectedIndex(sel);
                }
                else
                {
                    btnEDIT.setEnabled(false);
                    btnREMOVE.setEnabled(false);
                }
			}catch (Exception g){
				UE.displayError(g);
			}
		}
	}
	
	@Override
	public String getHelpFileName(){
		return "Shipname.html"; //$NON-NLS-1$
	}
}

