/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: StarnameGUI.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui.stbof;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import ue.Language;
import ue.UE;
import ue.edit.res.stbof.bin.*;
import ue.gui.*;
import ue.util.SettingsManager;

@SuppressWarnings("serial")  //$NON-NLS-1$
public class StarnameGUI extends MainPanel{
	private Starname STARS;
	//items
	private JList NAMELIST;
	private JButton btnAdd;
	private JButton btnEdit;
	private JButton btnRemove;
	private JButton btnExport;
	private JButton btnImport;
    private JLabel lblSum;
	
	public StarnameGUI(Starname star)
    {
		STARS = star;
		if (STARS==null)
        {
			return;
		}
		//create items
		btnAdd = new JButton(Language.getString("StarnameGUI.0")); //$NON-NLS-1$
		btnEdit = new JButton(Language.getString("StarnameGUI.1")); //$NON-NLS-1$
		btnRemove = new JButton(Language.getString("StarnameGUI.2")); //$NON-NLS-1$
        btnExport = new JButton("Export");
        btnImport = new JButton("Import");
		NAMELIST = new JList();
        lblSum = new JLabel();
		//set font
		Font def = UE.SETTINGS.getDefaultFont();
		btnAdd.setFont(def);
		btnEdit.setFont(def);
		btnRemove.setFont(def);
		btnImport.setFont(def);
		btnExport.setFont(def);
        lblSum.setFont(def);
		NAMELIST.setFont(def);
		//add listeners
		btnAdd.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent e)
            {
				String n = JOptionPane.showInputDialog(Language.getString("StarnameGUI.3"), Language.getString("StarnameGUI.4")); //$NON-NLS-1$ //$NON-NLS-2$
				
                if (n != null)
                {
					STARS.add(n);
                    updateData();
				}
			}
		});
		btnEdit.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent e){
				if (NAMELIST.getSelectedIndex() != -1){
					String n = JOptionPane.showInputDialog(Language.getString("StarnameGUI.5"), NAMELIST.getSelectedValue()); //$NON-NLS-1$
					if (n!=null){
						STARS.set((String)NAMELIST.getSelectedValue(), n);
                        updateData();
					}
				}
			}
		});
		btnRemove.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent e){
				if (NAMELIST.getSelectedIndex() != -1){
					STARS.remove((String)NAMELIST.getSelectedValue());
                    updateData();
				}
			}
		});
        btnImport.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent e)
            {
                JFileChooser jfc = new JFileChooser(new File(UE.SETTINGS.getProperty(SettingsManager.LAST)));
                javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter()
                {
                    @Override
                    public boolean accept(File f)
                    {
                        if (f.isDirectory())
                        {
                            return true;
                        }
                        return f.getName().toLowerCase().endsWith(".txt");
                    }
                    @Override
                    public String getDescription()
                    {
                        return "*.txt"; //$NON-NLS-1$
                    }
                };
                jfc.setFileFilter(filter);
                jfc.setApproveButtonText("Import");
                jfc.setDialogTitle("Import list of names...");
                jfc.setMultiSelectionEnabled(false);
                
                int returnVal = jfc.showOpenDialog(UE.WINDOW);

                if (returnVal == JFileChooser.APPROVE_OPTION)
                {
                    BufferedReader in = null;

                    try
                    {
                        in = new BufferedReader(new FileReader(jfc.getSelectedFile()));

                        String str = in.readLine();

                        while (str != null)
                        {
                            str = str.trim();
                            if (str.length() > 0)
                            {
                                STARS.add(str);
                            }
                            str = in.readLine();
                        }

                    } catch (IOException ex) {
                        UE.displayError(ex);
                    } finally {
                        try {
                            in.close();
                        } catch (IOException ex) {
                            UE.displayError(ex);
                        }
                    }

                    updateData();
                }
			}
		});
        btnExport.addActionListener(new ActionListener(){
            @Override
			public void actionPerformed(ActionEvent e)
            {
                JFileChooser jfc = new JFileChooser(new File(UE.SETTINGS.getProperty(SettingsManager.LAST)));
                javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter()
                {
                    @Override
                    public boolean accept(File f)
                    {
                        if (f.isDirectory())
                        {
                            return true;
                        }
                        return f.getName().toLowerCase().endsWith(".txt");
                    }
                    @Override
                    public String getDescription()
                    {
                        return "*.txt"; //$NON-NLS-1$
                    }
                };
                jfc.setFileFilter(filter);
                jfc.setApproveButtonText("Export");
                jfc.setDialogTitle("Export list of names...");
                jfc.setMultiSelectionEnabled(false);

                int returnVal = jfc.showSaveDialog(UE.WINDOW);

                if (returnVal == JFileChooser.APPROVE_OPTION)
                {
                    File file = jfc.getSelectedFile();
                    
                    if (!file.getName().toLowerCase().endsWith(".txt"))
                    {
                        file = new File(file.getParentFile(), file.getName() + ".txt");
                    }

                    if (file.exists())
                    {
                        String msg = Language.getString("MainMenu.12");
                        msg = msg.replace("%1", file.getName());
                        int response = JOptionPane.showConfirmDialog(UE.WINDOW, msg,
                                UE.APP_NAME, JOptionPane.YES_NO_OPTION,
                                JOptionPane.QUESTION_MESSAGE);

                        if (response != JOptionPane.YES_OPTION)
                        {
                            return;
                        }
                    }

                    BufferedWriter out = null;

                    try
                    {
                        out = new BufferedWriter(new FileWriter(file));
                        String[] names = STARS.getEntries();

                        for (int i = 0; i < names.length; i++)
                        {
                            out.write(names[i]);
                            out.newLine();
                        }

                    } catch (IOException ex) {
                        UE.displayError(ex);
                    } finally {
                        try {
                            out.close();
                        } catch (IOException ex) {
                            UE.displayError(ex);
                        }
                    }
                }
			}
		});
		//misc
		NAMELIST.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        updateData();
		//add items
		setLayout(new BorderLayout());
        setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		JScrollPane scrolly = new JScrollPane(NAMELIST);
		scrolly.setPreferredSize(new Dimension(200, 200));
        JPanel pnlCenter = new JPanel(new BorderLayout());
        pnlCenter.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        pnlCenter.add(scrolly, BorderLayout.CENTER);

        add(pnlCenter, BorderLayout.CENTER);

        JPanel pnlOptions = new JPanel(new GridLayout(0, 1, 5, 5));
        pnlOptions.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        pnlOptions.add(btnAdd);
        pnlOptions.add(btnEdit);
        pnlOptions.add(btnRemove);
        pnlOptions.add(btnImport);
        pnlOptions.add(btnExport);

        JPanel pnlEast = new JPanel(new BorderLayout());
        pnlEast.add(pnlOptions, BorderLayout.NORTH);

        add(pnlEast, BorderLayout.EAST);
        lblSum.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        add(lblSum, BorderLayout.SOUTH);

        updateSumString();
	}
	/*this is used to apply recent changes before panel is removed
	no need for it in this case tho*/
	@Override
	public void finalWarning() throws Exception
    {
        // nothing to do
    }

    private void updateData()
    {
        int sel = NAMELIST.getSelectedIndex();
        NAMELIST.setListData(STARS.getEntries());
        if (sel >= 0 && sel < STARS.entries())
        {
            NAMELIST.setSelectedIndex(sel);
        }

        updateSumString();
    }

    private void updateSumString()
    {
        lblSum.setText("Star names: %1".replace("%1", Long.toString(STARS.entries())));
    }
}
