/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


	File: TechGUI.java

	Written by: Alan Podlesek <seqdcer@yahoo.com>

	Created on 16.2.2010, 19:28:29
*/

package ue.gui.stbof;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.Enumeration;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.Timer;
import javax.swing.border.TitledBorder;
import ue.Language;
import ue.UE;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.StringFile;
import ue.edit.res.stbof.tec.TecField;
import ue.edit.res.stbof.tec.TechTree;
import ue.exception.InvalidArgumentsException;
import ue.util.CommonStrings;
import ue.util.ID;
import ue.util.SettingsManager;
import ue.util.Tools;

/**
 *
 * @author Alan
 */
public class TechGUI extends ue.gui.MainPanel
{
    private Stbof STBOF;
    private TecField FIELD;
    private TechTree TECH;
    private StringFile DESC;

    private int sel_field = -1;
    private int sel_level = -1;

    private Thread task = null;

    Timer tCOOLER = new Timer(1000, new ActionListener()
    {
        @Override
        public void actionPerformed(ActionEvent evt)
        {
            if (task == null || !task.isAlive())
            {
                setControlsEnabled(true);
                tCOOLER.stop();
            }
        }
    });

    /** Creates new form TechGUI2 */
    public TechGUI(Stbof st)
    {
        if (st == null)
        {
            return;
        }

        STBOF = st;

        try
        {
            FIELD = (TecField) STBOF.getInternalFile(Stbof.fileTECFIELD_TEC, true);
            TECH = (TechTree) STBOF.getInternalFile(Stbof.fileTECHTREE_TEC, true);
            DESC = (StringFile) STBOF.getInternalFile(Stbof.fileTECHDESC_TEC, true);
        }
        catch (FileNotFoundException ex)
        {
            UE.displayError(ex);
        }

        // gui
        initComponents();
        setFonts();
        loadTranslations();

        if (!UE.SETTINGS.getBooleanProperty(SettingsManager.EXPERIMENTAL))
        {
            disableExperimental();
        }

        // magic
        initMagic();
    }

    private void disableExperimental()
    {
        this.btnAdd.setVisible(false);
    }

    private void initMagic()
    {
        // setup description indexes
        for (int tech = 0; tech < TECH.getNumberOfEntries(); tech++)
        {
            try
            {
                DESC.setIndex(TECH.getDesc(tech), tech);
            }
            catch (Exception e)
            {
                UE.displayError(e);
                break;
            }
        }

        // load image file names
        Enumeration<String[]> images = STBOF.getFiles(new java.io.FilenameFilter()
        {
            @Override
            public boolean accept(File dir, String name)
            {
                return name.endsWith("_b.tga");
            }
        });

        String img;

        cmbImageFile.removeAllItems();
        while (images.hasMoreElements())
        {
            img = images.nextElement()[0];
            img = img.substring(0, img.length() - 6);

            if (STBOF.hasInternalFile(img + "_60.tga")
                    && STBOF.hasInternalFile(img + ".tga"))
            {
                this.cmbImageFile.addItem(img + ".tga");
            }
        }

        // load fields
        String[] fields = FIELD.getFields();

        cmbFields.removeAllItems();
        for (int i = 0; i < fields.length; i++)
        {
            this.cmbFields.addItem(new ID(fields[i], i));
        }
    }

    private void setFonts()
    {
        Font def = UE.SETTINGS.getDefaultFont();

        btnEditTechName.setFont(def);
        btnFieldName.setFont(def);
        btnMathApply.setFont(def);
        cmbFields.setFont(def);
        cmbImageFile.setFont(def);
        lstTechs.setFont(def);
        lblAdd.setFont(def);
        lblDesc.setFont(def);
        lblDiv.setFont(def);
        lblImage.setFont(def);
        lblImageIcon.setFont(def);
        lblMath.setFont(def);
        lblMult.setFont(def);
        lblReqResPoints.setFont(def);
        txtAdd.setFont(def);
        btnCalc.setFont(def);
        txtDesc.setFont(def);
        txtDiv.setFont(def);
        txtMath.setFont(def);
        txtMult.setFont(def);
        txtReqResPoints.setFont(def);
        chkAllFields.setFont(def);
    }

    private void loadTranslations()
    {
        btnEditTechName.setText(CommonStrings.getLocalizedString(CommonStrings.Edit, null));
        btnFieldName.setText(CommonStrings.getLocalizedString(CommonStrings.Edit, null));
        lblAdd.setText(Language.getString("TechGUI.5"));
        lblDesc.setText(Language.getString("TechGUI.3"));
        ((TitledBorder)jPanel2.getBorder()).setTitle(Language.getString("TechGUI.6"));
        lblDiv.setText(Language.getString("TechGUI.7"));
        lblImage.setText(Language.getString("TechGUI.2"));
        lblMath.setText(Language.getString("TechGUI.8"));
        btnCalc.setText(Language.getString("TechGUI.9"));
        btnMathApply.setText(Language.getString("TechGUI.9"));
        lblMult.setText(Language.getString("TechGUI.10"));
        lblReqResPoints.setText(Language.getString("TechGUI.1"));
        chkAllFields.setText(Language.getString("TechGUI.11"));
        btnCalc.setText(Language.getString("TechGUI.9"));
        btnAdd.setText(Language.getString("TechGUI.14"));
    }

    private void fillTechList()
    {
        if (sel_field < 0)
        {
            return;
        }

        sel_level = -1;

        String[] tech = TECH.getTechList((byte)sel_field);

        for (int i = 0; i < tech.length; i++)
        {
            tech[i] = tech[i] + " (level " + i + ")";
        }

        lstTechs.setListData(tech);

        // disable and clear tech specific stuff
        this.txtDesc.setEnabled(false);
        this.txtReqResPoints.setEnabled(false);
        this.cmbImageFile.setEnabled(false);
        this.btnEditTechName.setEnabled(false);
        this.txtDesc.setText(null);
        this.txtReqResPoints.setText(null);
        this.lblImageIcon.setIcon(null);
    }

    private void setControlsEnabled(boolean enabled)
    {
        if (sel_level >= 0)
        {
            this.txtDesc.setEnabled(enabled);
            this.txtReqResPoints.setEnabled(enabled);
            this.cmbImageFile.setEnabled(enabled);
        }
        
        this.cmbFields.setEnabled(enabled);
        this.txtAdd.setEnabled(enabled);
        this.btnCalc.setEnabled(enabled);
        this.txtDesc.setEnabled(enabled);
        this.txtDiv.setEnabled(enabled);
        this.txtMath.setEnabled(enabled);
        this.txtMult.setEnabled(enabled);
        this.btnEditTechName.setEnabled(enabled);
        this.btnFieldName.setEnabled(enabled);
        this.btnMathApply.setEnabled(enabled);
        this.lstTechs.setEnabled(enabled);
        this.chkAllFields.setEnabled(enabled);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        cmbFields = new javax.swing.JComboBox();
        btnFieldName = new javax.swing.JButton();
        btnEditTechName = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        lblReqResPoints = new javax.swing.JLabel();
        txtReqResPoints = new javax.swing.JTextField();
        lblDesc = new javax.swing.JLabel();
        lblImage = new javax.swing.JLabel();
        cmbImageFile = new javax.swing.JComboBox();
        lblImageIcon = new javax.swing.JLabel();
        jScrollPane2 = new javax.swing.JScrollPane();
        txtDesc = new javax.swing.JTextArea();
        jPanel2 = new javax.swing.JPanel();
        lblMult = new javax.swing.JLabel();
        txtMult = new javax.swing.JTextField();
        lblDiv = new javax.swing.JLabel();
        txtDiv = new javax.swing.JTextField();
        lblAdd = new javax.swing.JLabel();
        txtAdd = new javax.swing.JTextField();
        btnCalc = new javax.swing.JButton();
        lblMath = new javax.swing.JLabel();
        txtMath = new javax.swing.JTextField();
        btnMathApply = new javax.swing.JButton();
        chkAllFields = new javax.swing.JCheckBox();
        jScrollPane1 = new javax.swing.JScrollPane();
        lstTechs = new javax.swing.JList();
        btnAdd = new javax.swing.JButton();

        cmbFields.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        cmbFields.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmbFieldsActionPerformed(evt);
            }
        });

        btnFieldName.setText("Edit");
        btnFieldName.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnFieldNameActionPerformed(evt);
            }
        });

        btnEditTechName.setText("Edit");
        btnEditTechName.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnEditTechNameActionPerformed(evt);
            }
        });

        lblReqResPoints.setText("jLabel1");

        lblDesc.setText("jLabel1");

        lblImage.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        lblImage.setText("jLabel1");

        cmbImageFile.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        cmbImageFile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmbImageFileActionPerformed(evt);
            }
        });

        lblImageIcon.setVerticalAlignment(javax.swing.SwingConstants.TOP);

        txtDesc.setColumns(20);
        txtDesc.setLineWrap(true);
        txtDesc.setRows(5);
        txtDesc.setWrapStyleWord(true);
        jScrollPane2.setViewportView(txtDesc);

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(lblDesc, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 117, Short.MAX_VALUE)
                            .addComponent(lblReqResPoints, javax.swing.GroupLayout.DEFAULT_SIZE, 117, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(txtReqResPoints, javax.swing.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                        .addGap(18, 18, 18)
                        .addComponent(lblImage, javax.swing.GroupLayout.DEFAULT_SIZE, 50, Short.MAX_VALUE))
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 326, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(lblImageIcon, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(cmbImageFile, 0, 121, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(lblReqResPoints)
                    .addComponent(txtReqResPoints, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(lblImage)
                    .addComponent(cmbImageFile, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(lblDesc)
                        .addGap(9, 9, 9)
                        .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 73, Short.MAX_VALUE))
                    .addComponent(lblImageIcon, javax.swing.GroupLayout.DEFAULT_SIZE, 96, Short.MAX_VALUE)))
        );

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("Modify all techs"));

        lblMult.setText("jLabel1");

        lblDiv.setText("jLabel1");

        lblAdd.setText("jLabel1");

        btnCalc.setText("Apply");
        btnCalc.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCalcActionPerformed(evt);
            }
        });

        lblMath.setText("jLabel1");

        txtMath.setText("152.4158*((L-1)^4)+(L-1)*(10-L)*31");

        btnMathApply.setText("Apply");
        btnMathApply.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnMathApplyActionPerformed(evt);
            }
        });

        chkAllFields.setText("jCheckBox1");

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPanel2Layout.createSequentialGroup()
                        .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(lblAdd, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 94, Short.MAX_VALUE)
                            .addComponent(lblDiv, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 94, Short.MAX_VALUE)
                            .addComponent(lblMult, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 94, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(txtAdd, javax.swing.GroupLayout.DEFAULT_SIZE, 109, Short.MAX_VALUE)
                            .addComponent(txtDiv, javax.swing.GroupLayout.DEFAULT_SIZE, 109, Short.MAX_VALUE)
                            .addComponent(txtMult, javax.swing.GroupLayout.DEFAULT_SIZE, 109, Short.MAX_VALUE)
                            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel2Layout.createSequentialGroup()
                                .addGap(39, 39, 39)
                                .addComponent(btnCalc))))
                    .addComponent(lblMath, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 213, Short.MAX_VALUE)
                    .addComponent(txtMath, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 213, Short.MAX_VALUE)
                    .addComponent(btnMathApply)
                    .addComponent(chkAllFields, javax.swing.GroupLayout.DEFAULT_SIZE, 213, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(lblMult)
                    .addComponent(txtMult, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(lblDiv)
                    .addComponent(txtDiv, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(lblAdd)
                    .addComponent(txtAdd, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnCalc)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(lblMath)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(txtMath, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnMathApply)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(chkAllFields))
        );

        lstTechs.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        lstTechs.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        lstTechs.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                lstTechsValueChanged(evt);
            }
        });
        jScrollPane1.setViewportView(lstTechs);

        btnAdd.setText("Add");
        btnAdd.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAddActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jPanel1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(jScrollPane1)
                            .addComponent(cmbFields, 0, 165, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addComponent(btnFieldName)
                                .addComponent(btnAdd))
                            .addComponent(btnEditTechName))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(cmbFields, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(btnFieldName))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(btnAdd)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(btnEditTechName))
                            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 210, Short.MAX_VALUE)))
                    .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents

    private void btnFieldNameActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnFieldNameActionPerformed
        try
        {
            int i = sel_field;

            if (i < 0) return;

            String response = JOptionPane.showInputDialog(Language.getString("TechGUI.4"), //$NON-NLS-1$
                    FIELD.getField(i));
            
            if (response != null)
            {
                FIELD.setField(i, response);
                
                cmbFields.setEnabled(false);
                cmbFields.removeAllItems();
                
                String[] fields = FIELD.getFields();

                for (int j = 0; j < fields.length; j++)
                {
                    cmbFields.addItem(new ID(fields[j], j));
                }
                cmbFields.setEnabled(true);
                cmbFields.setSelectedIndex(i);
            }
        }
        catch (InvalidArgumentsException ze)
        {
            UE.displayError(ze);
        }
    }//GEN-LAST:event_btnFieldNameActionPerformed

    private void cmbFieldsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmbFieldsActionPerformed
        // save old
        try
        {
            finalWarning();
        }
        catch (Exception ex)
        {
            // really bad stuff happening - bail
            UE.displayError(ex);
            return;
        }

        // setup new
        sel_field = cmbFields.getSelectedIndex();
        fillTechList();
    }//GEN-LAST:event_cmbFieldsActionPerformed

    private void lstTechsValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_lstTechsValueChanged
        // save old
        try
        {
            finalWarning();
        }
        catch (Exception ex)
        {
            // really bad stuff happening - bail
            UE.displayError(ex);
            return;
        }

        // setup new
        sel_level = lstTechs.getSelectedIndex();

        if (sel_level >= 0)
        {
            // set and enable tech specific stuff
            try
            {
                this.txtDesc.setText(DESC.getDesc((short)TECH.getIndex((byte) sel_field, (byte) sel_level)));
            }
            catch (Exception ex)
            {
                UE.displayError(ex);
            }

            try
            {
                this.txtReqResPoints.setText(Integer.toString(TECH.getCost((byte) sel_field, (byte) sel_level)));
            }
            catch (Exception ex)
            {
                UE.displayError(ex);
            }

            try
            {
                String img = TECH.getImage((byte) sel_field, (byte) sel_level) + ".tga";
                cmbImageFile.setSelectedItem(img);
                this.lblImageIcon.setIcon(new ImageIcon(STBOF.getTargaImage(img, false).getImage(false)));
            }
            catch (Exception ex)
            {
                UE.displayError(ex);
            }

            this.txtDesc.setEnabled(true);
            this.txtReqResPoints.setEnabled(true);
            this.cmbImageFile.setEnabled(true);
            this.btnEditTechName.setEnabled(true);
        }
    }//GEN-LAST:event_lstTechsValueChanged

    private void btnEditTechNameActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnEditTechNameActionPerformed
        if (sel_field < 0 || sel_level < 0)
        {
            return;
        }

        try
        {
            String response = JOptionPane.showInputDialog(Language.getString("TechGUI.4"),
                    TECH.getName((byte)sel_field, (byte)sel_level)); //$NON-NLS-1$

            if (response != null)
            {
                int index = lstTechs.getSelectedIndex();

                TECH.setName((byte)sel_field, (byte)sel_level, response);
		
                fillTechList();

                lstTechs.setSelectedIndex(index);
            }
	}
        catch (InvalidArgumentsException ex)
        {
            UE.displayError(ex);
        }
    }//GEN-LAST:event_btnEditTechNameActionPerformed

    private void btnCalcActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCalcActionPerformed
        try
        {
            finalWarning();
        }
        catch (Exception ex)
        {
            // really bad stuff happening - bail
            UE.displayError(ex);
            return;
        }
        fillTechList();
        this.setControlsEnabled(false);

        task = new Thread(new Runnable()
        {
            @Override
            public void run()
            {
                double num;
                boolean filter = !chkAllFields.isSelected();

                for (int i = 0; i < TECH.getNumberOfEntries(); i++)
                {
                    try
                    {
                        if (filter && TECH.getField(i) != sel_field)
                        {
                            continue;
                        }
                    }
                    catch (Exception ex)
                    {
                        UE.displayError(ex);
                        break;
                    }

                    // multiply
                    try
                    {
                        num = Double.parseDouble(txtMult.getText());
                        TECH.setCost(i, (int)Math.max(TECH.getCost(i) * num, 0));
                    }
                    catch (Exception ex)
                    {}
                    
                    // divide
                    try
                    {
                        num = Double.parseDouble(txtDiv.getText());
                        TECH.setCost(i, (int)Math.max(TECH.getCost(i) / num, 0));
                    }
                    catch (Exception ex)
                    {}
                    
                    // add
                    try
                    {
                        num = Double.parseDouble(txtAdd.getText());
                        TECH.setCost(i, (int)Math.max(TECH.getCost(i) + num, 0));
                    }
                    catch (Exception ex)
                    {}
                }
            }
        });

        task.start();
        tCOOLER.setRepeats(true);
        tCOOLER.start();
    }//GEN-LAST:event_btnCalcActionPerformed

    private void cmbImageFileActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmbImageFileActionPerformed
        if (this.cmbImageFile.getSelectedIndex() >= 0 && sel_field >= 0
                && sel_level >= 0)
        {
            String img = this.cmbImageFile.getSelectedItem().toString();
            try {
                this.lblImageIcon.setIcon(new ImageIcon(STBOF.getTargaImage(img, false).getImage(false)));
                TECH.setImage((byte)sel_field, (byte)sel_level, img.substring(0, img.length() - 4));
            } catch (Exception ex) {
                UE.displayError(ex);
            }
        }
    }//GEN-LAST:event_cmbImageFileActionPerformed

    private void btnMathApplyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnMathApplyActionPerformed
        try
        {
            finalWarning();
        }
        catch (Exception ex)
        {
            // really bad stuff happening - bail
            UE.displayError(ex);
            return;
        }
        fillTechList();
        this.setControlsEnabled(false);

        task = new Thread(new Runnable()
        {
            @Override
            public void run()
            {
                String calc;
                byte level, field;
                boolean filter = !chkAllFields.isSelected();

                for (int i = 0; i < TECH.getNumberOfEntries(); i++)
                {
                    try
                    {
                        level = TECH.getLevel(i);
                        field = TECH.getField(i);

                        if (filter && field != sel_field)
                        {
                            continue;
                        }

                        calc = txtMath.getText();
                        calc = calc.replaceAll("L", Byte.toString(level));

                        TECH.setCost(i, (int)Math.max(Tools.calculate(calc), 0));
                    }
                    catch (Exception ex)
                    {
                        UE.displayError(ex);
                        break;
                    }
                }
            }
        });

        task.start();
        tCOOLER.setRepeats(true);
        tCOOLER.start();
    }//GEN-LAST:event_btnMathApplyActionPerformed

    private void btnAddActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnAddActionPerformed
        if (sel_field < 0)
        {
            return;
        }

        // save old
        try
        {
            finalWarning();
        }
        catch (Exception ex)
        {
            // really bad stuff happening - bail
            UE.displayError(ex);
            return;
        }

        try
        {
            String response = JOptionPane.showInputDialog(Language.getString("TechGUI.12"),
                    Language.getString("TechGUI.13")); //$NON-NLS-1$

            if (response != null)
            {
                int index = TECH.add(response, (byte)sel_field);

                if (index < 0)
                {
                    return;
                }

                int address = DESC.add();

                TECH.setDesc(index, address);

                // reset description indexes
                DESC.reset();
                for (int tech = 0; tech < TECH.getNumberOfEntries(); tech++)
                {
                    try
                    {
                        DESC.setIndex(TECH.getDesc(tech), tech);
                    }
                    catch (Exception e)
                    {
                        UE.displayError(e);
                        break;
                    }
                }

                DESC.setDesc((short)index, response.toUpperCase() + ": " + "No description.");

                index = TECH.getLevel(index);

                fillTechList();

                lstTechs.setSelectedIndex(index);
            }
	}
        catch (InvalidArgumentsException ex)
        {
            UE.displayError(ex);
        }
    }//GEN-LAST:event_btnAddActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnAdd;
    private javax.swing.JButton btnCalc;
    private javax.swing.JButton btnEditTechName;
    private javax.swing.JButton btnFieldName;
    private javax.swing.JButton btnMathApply;
    private javax.swing.JCheckBox chkAllFields;
    private javax.swing.JComboBox cmbFields;
    private javax.swing.JComboBox cmbImageFile;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JLabel lblAdd;
    private javax.swing.JLabel lblDesc;
    private javax.swing.JLabel lblDiv;
    private javax.swing.JLabel lblImage;
    private javax.swing.JLabel lblImageIcon;
    private javax.swing.JLabel lblMath;
    private javax.swing.JLabel lblMult;
    private javax.swing.JLabel lblReqResPoints;
    private javax.swing.JList lstTechs;
    private javax.swing.JTextField txtAdd;
    private javax.swing.JTextArea txtDesc;
    private javax.swing.JTextField txtDiv;
    private javax.swing.JTextField txtMath;
    private javax.swing.JTextField txtMult;
    private javax.swing.JTextField txtReqResPoints;
    // End of variables declaration//GEN-END:variables

    @Override
    public void finalWarning() throws Exception
    {
        if (sel_field < 0)
        {
            return;
        }

        if (sel_level >= 0)
        {
            int points = Math.max(Integer.parseInt(this.txtReqResPoints.getText()), 0);
            TECH.setCost((byte)sel_field, (byte)sel_level, points);

            DESC.setDesc((short) TECH.getIndex((byte)sel_field, (byte)sel_level), this.txtDesc.getText());

            for (short tech = 0; tech < TECH.getNumberOfEntries(); tech++)
            {
                TECH.setDesc(tech, DESC.getAddress(tech));
            }
        }
    }

}
