/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: TextureGUI.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.gui.stbof;

import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.io.*;
import ue.edit.res.stbof.lst.*;
import ue.edit.res.*;
import ue.gui.*;
import ue.util.*;
import ue.*;

@SuppressWarnings("serial")  //$NON-NLS-1$
public class TextureGUI extends MainPanel
{
    //files
    private Stbof STBOF;
    private PaletteList PALETTE;
    private Texture TEXTURE;
    //components
    private JComboBox cmbTEXTURE;
    private JComboBox cmbPALETTE;
    private JButton btnAddTEX;
    private JButton btnUseOWN;
    private JLabel lblTextureIMG;
    private JLabel lblPaletteNUM;
    private JPanel pnlCOLOR;
    private JButton[] btnCOLOR; //256*4
    private JButton btnTRIM;
    private JButton btnSEL_COLOR;
    private JButton btnRemTEX;
    private JSlider jslSEL_COLOR_ALPHA;
    private JLabel lblSEL_COLOR;
    //other
    private GifColorMap gcmGIF;
    private GifImage GIF;
    private int SELECTED = -1;
    private int SELECTED_PIXEL = -1;
    // strings
    private static String strRGB = Language.getString("TextureGUI.0"); //$NON-NLS-1$
    private static String strPALETTES = Language.getString("TextureGUI.1"); //$NON-NLS-1$
    
    /**constructor*/
    public TextureGUI(Stbof st, Texture tex, PaletteList pal)
    {
        try
        {
            STBOF = st;
            TEXTURE = tex;
            PALETTE = pal;
            gcmGIF = null;
            //CREATE
            cmbTEXTURE	= new JComboBox();
            cmbPALETTE	= new JComboBox();
            lblPaletteNUM	= new JLabel();
            lblTextureIMG	= new JLabel();
            btnAddTEX	= new JButton(Language.getString("TextureGUI.2")); //$NON-NLS-1$
            btnUseOWN	= new JButton(Language.getString("TextureGUI.3")); //$NON-NLS-1$
            pnlCOLOR	= new JPanel(new GridLayout(16,16));
            btnCOLOR	= new JButton[256];
            btnTRIM		= new JButton(Language.getString("TextureGUI.4")); //$NON-NLS-1$
            btnSEL_COLOR	= new JButton();
            lblSEL_COLOR	= new JLabel();
            jslSEL_COLOR_ALPHA = new JSlider(0, 255);
            btnRemTEX       = new JButton("Remove");
            //SETUP
            Font def = UE.SETTINGS.getDefaultFont();
            Border defBORDER = new LineBorder(Color.BLACK);
            //jslSEL_COLOR_ALPHA
            jslSEL_COLOR_ALPHA.addChangeListener(new ChangeListener()
            {
                @Override
                public void stateChanged(ChangeEvent ce)
                {
                    try
                    {
                        byte[] b = gcmGIF.getPixelBytes(SELECTED_PIXEL);
                        b[0] = (byte)jslSEL_COLOR_ALPHA.getValue();
                        gcmGIF.setPixelBytes(SELECTED_PIXEL, b);
                        
                        int pi[] = new int[4];
                        for (int i=0; i<4; i++)
                        {				
                            pi[i] = b[i];
                            if (pi[i]<0)
                            {
                                pi[i] = 256 + pi[i];
                            }
                        }
                        String str = strRGB.replace("%1", Integer.toString(pi[1])); //$NON-NLS-1$
                        str = str.replace("%2", Integer.toString(pi[2])); //$NON-NLS-1$
                        str = str.replace("%3", Integer.toString(pi[3])); //$NON-NLS-1$
                        str = str.replace("%4", Integer.toString(pi[0])); //$NON-NLS-1$
                        lblSEL_COLOR.setText(str);
                        
                        //insert colormap and redraw
                        GIF.setGifColorMap(gcmGIF);
                        lblTextureIMG.setIcon(new ImageIcon(GIF.getImage()));
                        //save to palette
                        int t = PALETTE.getPalettesNum();
                        
                        PALETTE.setColorMap(SELECTED, gcmGIF);
                        int z = PALETTE.getPalettesNum();
                        if (z>t)
                        {
                            cmbPALETTE.addItem(Integer.toString(z-1));
                            lblPaletteNUM.setText(strPALETTES.replace("%1", Integer.toString(z))); //$NON-NLS-1$
                            if (z > 127)
                            {
                                lblPaletteNUM.setForeground(Color.RED);
                            }
                            else
                            {
                                lblPaletteNUM.setForeground(Color.BLACK);
                            }
                        }
                        cmbPALETTE.setSelectedIndex(PALETTE.getPaletteIndex(SELECTED));
                        repaint();
                        validate();
                    }
                    catch (Exception e)
                    {
                        UE.displayError(e);
                    }
                }
            });
            //lblSEL_COLOR
            lblSEL_COLOR.setFont(def);
            //btnSEL_COLOR
            btnSEL_COLOR.setIconTextGap(1);
            btnSEL_COLOR.setFont(def);
            btnSEL_COLOR.setPreferredSize(new Dimension(40, 40));
            btnSEL_COLOR.setBorder(defBORDER);
            btnSEL_COLOR.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent ae)
                {
                    try
                    {
                        Color col = JColorChooser.showDialog(pnlCOLOR, Language.getString("TextureGUI.5"), btnSEL_COLOR.getBackground()); //$NON-NLS-1$
                        if (col!=null)
                        {
                            gcmGIF.setPixel(SELECTED_PIXEL, col.getRGB());
                            jslSEL_COLOR_ALPHA.setEnabled(false);			
                            jslSEL_COLOR_ALPHA.setValue(0);						
                            jslSEL_COLOR_ALPHA.setEnabled(true);
                            //save to palette
                            int t = PALETTE.getPalettesNum();
                            
                            PALETTE.setColorMap(SELECTED, gcmGIF);
                            int z = PALETTE.getPalettesNum();
                            if (z>t)
                            {
                                cmbPALETTE.addItem(Integer.toString(z-1));
                                lblPaletteNUM.setText(strPALETTES.replace("%1", Integer.toString(z))); //$NON-NLS-1$
                                if (z > 127)
                                {
                                    lblPaletteNUM.setForeground(Color.RED);
                                }
                                else
                                {
                                    lblPaletteNUM.setForeground(Color.BLACK);
                                }
                            }
                            cmbPALETTE.setSelectedIndex(PALETTE.getPaletteIndex(SELECTED));
                        }
                    }
                    catch (Exception e)
                    {
                        UE.displayError(e);
                    }
                }
            });
            //btnTRIM
            btnTRIM.setFont(def);
            btnTRIM.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent ae)
                {
                    try
                    {
                        PALETTE.setColorMap(SELECTED, gcmGIF);
                        int temp = SELECTED;
                        SELECTED = -1;
                        PALETTE.trim(TEXTURE);
                        
                        int t = PALETTE.getPalettesNum();
                        cmbPALETTE.removeAllItems();
                        for (int i=0; i<t; i++)
                        {
                            cmbPALETTE.addItem(Integer.toString(i));
                        }
                        lblPaletteNUM.setText(strPALETTES.replace("%1", Integer.toString(t))); //$NON-NLS-1$
                        if (t > 127)
                        {
                             lblPaletteNUM.setForeground(Color.RED);
                        }
                        else
                        {
                             lblPaletteNUM.setForeground(Color.BLACK);
                        }
                        SELECTED = temp;
                        cmbPALETTE.setSelectedIndex(PALETTE.getPaletteIndex(SELECTED));
                    }
                    catch (Exception e)
                    {
                        UE.displayError(e);
                    }
                }
            });
            //btnRemTEX
            btnRemTEX.setFont(def);
            btnRemTEX.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ae)
                {
                    try
                    {
                        if (SELECTED < 0) return;
                        STBOF.removeInternalFile(TEXTURE.get(SELECTED));
                        TEXTURE.remove(SELECTED);
                        PALETTE.setColorMap(SELECTED, PALETTE.getColorMap(0));
                        SELECTED = -1;
                        cmbTEXTURE.removeAllItems();
                        String[] str = TEXTURE.getEntries();
                        Arrays.sort(str);
                        for (int i = 0; i < str.length; i++)
                        {
                            cmbTEXTURE.addItem(new ID(str[i], TEXTURE.getTextureIndex(str[i])));
                        }
                    }
                    catch (Exception e)
                    {
                        UE.displayError(e);
                    }
                }
            });
            //btnUseOWN
            //add own, set cmbPALETTE to select own
            btnUseOWN.setFont(def);
            btnUseOWN.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent ae)
                {
                    try
                    {
                        if (SELECTED < 0)
                        {
                            return;
                        }
                        //load GifColorMap
                        int t = PALETTE.getPalettesNum();
                        PALETTE.setColorMap(SELECTED, GIF.getInitialGifColorMap());
                        int t2 = PALETTE.getPalettesNum();
                        
                        if (t2>t)
                        {
                            cmbPALETTE.addItem(Integer.toString(t2-1));
                            lblPaletteNUM.setText(strPALETTES.replace("%1", Integer.toString(t2))); //$NON-NLS-1$
                            if (t2 > 127)
                            {
                                lblPaletteNUM.setForeground(Color.RED);
                            }
                            else
                            {
                                lblPaletteNUM.setForeground(Color.BLACK);
                            }						
                        }
                        cmbPALETTE.setSelectedIndex(PALETTE.getPaletteIndex(SELECTED));
                    }
                    catch (Exception e)
                    {
                        UE.displayError(e);
                    }
                }
            });
            //btnAddTEX
            //
            btnAddTEX.setFont(def);
            btnAddTEX.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent ae)
                {
                    try
                    {
                        java.util.Enumeration<String[]> textures = STBOF.getFiles(new java.io.FilenameFilter(){
                            @Override
                            public boolean accept(File dir, String name){
                                name = name.toLowerCase();
                                return (name.endsWith(".gif") && !TEXTURE.contains(name)); //$NON-NLS-1$
                            }
                        });
                        
                        if (textures.hasMoreElements())
                        {
                            ArrayList<String> opt = new ArrayList<String>();
                            while(textures.hasMoreElements())
                            {
                                opt.add(textures.nextElement()[0]);
                            }

                            String tex = (String)JOptionPane.showInputDialog(UE.WINDOW,
                                    Language.getString("TextureGUI.6"), Language.getString("TextureGUI.7"), //$NON-NLS-1$ //$NON-NLS-2$
                                    JOptionPane.OK_CANCEL_OPTION, null, opt.toArray(new String[0]),
                                    opt.get(0));
                            
                            if (tex != null)
                            {
                                GenericFile gif = (GenericFile)STBOF.getInternalFile(tex, true);
                                ByteArrayInputStream bais = new ByteArrayInputStream(gif.getData());
                                GifImage gi = new GifImage(bais);
                                bais.close();
                                GifColorMap gcm = gi.getInitialGifColorMap();
                                
                                int index = TEXTURE.getUnusedIndex();
                                
                                if (index < 0)
                                {
                                    index = TEXTURE.add(tex);
                                }
                                else
                                {
                                    TEXTURE.set(index, tex);
                                }
                                
                                cmbTEXTURE.addItem(new ID(tex, index));
                                
                                int t = PALETTE.getPalettesNum();
                                PALETTE.setColorMap(index, gcm);
                                int z = PALETTE.getPalettesNum();
                                
                                if (z > t)
                                {
                                    cmbPALETTE.addItem(Integer.toString(z-1));
                                    lblPaletteNUM.setText(strPALETTES.replace("%1", Integer.toString(z))); //$NON-NLS-1$
                                    
                                    if (z > 127)
                                    {
                                        lblPaletteNUM.setForeground(Color.RED);
                                    }else{
                                        lblPaletteNUM.setForeground(Color.BLACK);
                                    }						
                                }
                                
                                cmbTEXTURE.setSelectedIndex(index);
                            }
                        }else{
                            JOptionPane.showMessageDialog(UE.WINDOW, Language.getString("TextureGUI.8"), UE.APP_NAME, JOptionPane.	INFORMATION_MESSAGE); //$NON-NLS-1$
                        }
                    }catch (Exception e){
                        UE.displayError(e);
                    }
                }
            });
            //btnCOLOR
            for (int i=0; i<256; i++){
                btnCOLOR[i] = new JButton();
                btnCOLOR[i].setIconTextGap(1);
                btnCOLOR[i].setFont(def);
                btnCOLOR[i].setPreferredSize(new Dimension(20, 20));
                btnCOLOR[i].setBorder(defBORDER);
                btnCOLOR[i].addActionListener(new ColAction(i));
            }
            //cmbPALETTE
            //only show
            cmbPALETTE.setFont(def);
            cmbPALETTE.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent ae){
                    try{
                        if (SELECTED<0) {
                            return;
                        }
                        //load GifColorMap
                        gcmGIF = PALETTE.getPalette(cmbPALETTE.getSelectedIndex());
                        //show palette
                        int[] pixels = gcmGIF.getMapPixels();
                        for (int i=0; i<256; i++){
                            btnCOLOR[i].setEnabled(false);
                            btnCOLOR[i].setBackground(new Color(pixels[i], false));
                            btnCOLOR[i].setEnabled(true);
                        }
                        //insert colormap and redraw
                        GIF.setGifColorMap(gcmGIF);
                        lblTextureIMG.setIcon(new ImageIcon(GIF.getImage()));
                        if (SELECTED_PIXEL<0) {
                            SELECTED_PIXEL = 0;
                        }						
                        btnCOLOR[SELECTED_PIXEL].doClick();
                        repaint();
                        validate();
                    }catch (Exception e){
                        UE.displayError(e);
                    }
                }
            });
            int t = PALETTE.getPalettesNum();
            //lblPaletteNUM
            lblPaletteNUM.setFont(def);
            lblPaletteNUM.setText(strPALETTES.replace("%1", Integer.toString(t))); //$NON-NLS-1$
            if (t > 127){
                lblPaletteNUM.setForeground(Color.RED);
            }else{
                lblPaletteNUM.setForeground(Color.BLACK);
            }						
            for (int i=0; i<t; i++){
                cmbPALETTE.addItem(Integer.toString(i));
            }
            //cmbTEXTURE
            cmbTEXTURE.setFont(def);
            cmbTEXTURE.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent ae){
                    try{
                        if (cmbTEXTURE.getSelectedIndex() < 0) return;
                        //SAVE OLD
                        if (SELECTED>=0 && gcmGIF!=null){
                            PALETTE.setColorMap(SELECTED, gcmGIF);
                            int z = PALETTE.getPalettesNum();
                            lblPaletteNUM.setText(strPALETTES.replace("%1", Integer.toString(z))); //$NON-NLS-1$
                            if (z > 127){
                                lblPaletteNUM.setForeground(Color.RED);
                            }else{
                                lblPaletteNUM.setForeground(Color.BLACK);
                            }
                        }
                        //SET
                        SELECTED = cmbTEXTURE.getSelectedItem().hashCode();
                        //load image
                        String str = (((ID)cmbTEXTURE.getSelectedItem()).toString()).toLowerCase();
                        GenericFile gif = (GenericFile)STBOF.getInternalFile(str, true);
                        ByteArrayInputStream bais = new ByteArrayInputStream(gif.getData());
                        GIF = new GifImage(bais);
                        bais.close();
                        //select palette
                        try{
                            cmbPALETTE.setSelectedIndex(PALETTE.getPaletteIndex(SELECTED));
                        }catch (Exception e){
                            cmbPALETTE.setSelectedIndex(0);
                        }
                    }catch (Exception e){
                        UE.displayError(e);
                    }
                }
            });
            
            String[] str = TEXTURE.getEntries();

            Arrays.sort(str);
            
            for (int i = 0; i < str.length; i++)
            {
                cmbTEXTURE.addItem(new ID(str[i], TEXTURE.getTextureIndex(str[i])));
            }
            
            cmbPALETTE.setPreferredSize(cmbTEXTURE.getPreferredSize());
            Dimension d = new Dimension(536, 536);
            lblTextureIMG.setPreferredSize(d);
            lblTextureIMG.setHorizontalAlignment(JLabel.CENTER);
            lblTextureIMG.setVerticalAlignment(JLabel.CENTER);
            lblTextureIMG.setFont(def);
            lblTextureIMG.setBorder(
                    BorderFactory.createTitledBorder(
                    BorderFactory.createLineBorder(Color.GRAY),
                    Language.getString("TextureGUI.12"),
                    TitledBorder.DEFAULT_JUSTIFICATION,
                    TitledBorder.DEFAULT_POSITION, def
                    )); //$NON-NLS-1$
            //SHOW
            setLayout(new GridBagLayout());
            
            GridBagConstraints c = new GridBagConstraints();
            c.insets=new Insets(5, 5, 5, 5);
            c.gridx=0;
            c.gridy=0;
            c.gridwidth=1;
            c.gridheight=1;

            GridLayout layout = new GridLayout(0,6);
            layout.setHgap(5);
            layout.setVgap(5);
            JPanel pnlTexOpt = new JPanel(layout);
            
            JLabel lbl = new JLabel(Language.getString("TextureGUI.9")); //$NON-NLS-1$
            lbl.setFont(def);

            pnlTexOpt.add(new JLabel());
            pnlTexOpt.add(lbl);
            pnlTexOpt.add(new JLabel());
            
            lbl = new JLabel(Language.getString("TextureGUI.10")); //$NON-NLS-1$
            lbl.setFont(def);
            
            pnlTexOpt.add(new JLabel());
            pnlTexOpt.add(lbl);
            pnlTexOpt.add(new JLabel());

            pnlTexOpt.add(new JLabel());
            pnlTexOpt.add(cmbTEXTURE);
            pnlTexOpt.add(btnRemTEX, c);
            
            pnlTexOpt.add(new JLabel());
            pnlTexOpt.add(cmbPALETTE);
            pnlTexOpt.add(new JLabel());
            
            pnlTexOpt.add(new JLabel());
            pnlTexOpt.add(btnAddTEX);
            pnlTexOpt.add(new JLabel());

            lblPaletteNUM.setHorizontalAlignment(JLabel.CENTER);
            
            pnlTexOpt.add(new JLabel());
            pnlTexOpt.add(lblPaletteNUM);
            pnlTexOpt.add(new JLabel());

            pnlTexOpt.add(new JLabel());
            pnlTexOpt.add(btnUseOWN);
            pnlTexOpt.add(new JLabel());
            
            pnlTexOpt.add(new JLabel());
            pnlTexOpt.add(btnTRIM);
            pnlTexOpt.add(new JLabel());
            
            c.gridx=0;
            c.gridy=0;
            c.gridheight=4;
            c.gridwidth=5;
            add(pnlTexOpt, c);
            c.gridy=4;
            c.gridx=0;
            c.gridwidth=1;
            add(lblTextureIMG, c);
            
            for (int i=0; i<256; i++){
                pnlCOLOR.add(btnCOLOR[i]);
            }
            c.gridx=1;
            c.gridy=4;
            c.gridheight=1;
            c.gridwidth=4;
            add(pnlCOLOR, c);
            c.gridwidth=1;
            c.gridy=5;
            c.gridx=1;
            lbl = new JLabel(Language.getString("TextureGUI.11")); //$NON-NLS-1$
            lbl.setFont(def);
            add(lbl, c);
            c.gridx=2;
            add(jslSEL_COLOR_ALPHA, c);
            c.gridx=3;
            add(btnSEL_COLOR, c);
            c.gridx=1;
            c.gridwidth=3;
            c.gridy=6;
            add(lblSEL_COLOR,c);
            
            pnlCOLOR.setVisible(true);
            setVisible(true);
        }catch (Exception e){
            UE.displayError(e);
        }
    }
    
    @Override
    public void finalWarning() throws Exception {
        if (SELECTED>=0 && gcmGIF!=null) {
            PALETTE.setColorMap(SELECTED, gcmGIF);
        }
    }
    
    private class ColAction implements ActionListener{
        private int INDEX;						
       
        public ColAction(int i){
            INDEX = i;
        }
        public void actionPerformed(ActionEvent ae){
            try{
                if (SELECTED<0) {
                    return;
                }
                //open ColorChooser
                if (SELECTED_PIXEL>=0) {
                    btnCOLOR[SELECTED_PIXEL].setBorder(new LineBorder(Color.BLACK));
                }
                btnCOLOR[INDEX].setBorder(new LineBorder(Color.WHITE));
                SELECTED_PIXEL = INDEX;
                
                //show
                int pix = gcmGIF.getPixel(SELECTED_PIXEL);
                btnSEL_COLOR.setBackground(new Color(pix, false));
                byte[] b = gcmGIF.getPixelBytes(SELECTED_PIXEL);
                jslSEL_COLOR_ALPHA.setEnabled(false);
                int pi[] = new int[4];
                for (int i=0; i<4; i++){				
                    pi[i] = b[i];
                    if (pi[i]<0) {
                        pi[i] = 256 + pi[i];
                    }
                }
                jslSEL_COLOR_ALPHA.setValue(pi[0]);
                jslSEL_COLOR_ALPHA.setEnabled(true);
                String str = strRGB.replace("%1", Integer.toString(pi[1])); //$NON-NLS-1$
                str = str.replace("%2", Integer.toString(pi[2])); //$NON-NLS-1$
                str = str.replace("%3", Integer.toString(pi[3])); //$NON-NLS-1$
                str = str.replace("%4", Integer.toString(pi[0])); //$NON-NLS-1$
                lblSEL_COLOR.setText(str);
            }catch (Exception e){
                UE.displayError(e);
            }
        }
    }
    
    @Override
    public String getHelpFileName()
    {
            return "Textures.html"; //$NON-NLS-1$
    }

}
