/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: ListBoxTreeNode.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 4.8.2011
 */

package ue.gui.stbof.wdf;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.Enumeration;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.UIManager;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.wdf.WDFFont;
import ue.edit.res.stbof.wdf.WDFListBox;
import ue.edit.res.stbof.wdf.WDFWidget;
import ue.edit.res.stbof.wdf.WindowDefinitionFile;
import ue.util.ImageCache;
import ue.util.Tools;

/**
 *
 */
public class ListBoxTreeNode extends WidgetTreeNode
{
    private JPanel cellEditor;
    private JCheckBox display;
    private JLabel lbl;

    private WDFListBox listBox;

    private WDFEditableFontTreeNode normalOption;
    private WDFEditableFontTreeNode selectedOption;

    private boolean fontVisible = true;
    private boolean selectedFontVisible = false;

    public ListBoxTreeNode(final WindowDefinitionFile wdf, final WDFListBox listBox,
            Stbof res, ImageCache imgCache)
    {
        super(wdf, listBox, res, imgCache);
        this.listBox = listBox;

        add(new WDFEditableIntegerValueTreeNode("Unknown 1", listBox.getUnknown1(), 5,
                Integer.MIN_VALUE, Integer.MAX_VALUE)
        {
            @Override
            public void setInteger(int value)
            {
                if (listBox.setUnknown1(value))
                {
                    wdf.markChanged();
                }
            }
        });

        ArrayList<String> files = res.getFileList(new FilenameFilter()
        {
            @Override
            public boolean accept(File dir, String name) {
                return name.endsWith(".fnt");
            }
        });
        files.add(0, WDFWidget.NO_IMAGE);

        normalOption = new WDFEditableFontTreeNode("Font", listBox.getFont1(), true,
            true, files)
        {
            @Override
            public void setFont(boolean isEnabled, WDFFont font)
            {
                if (listBox.setFont1(font))
                {
                    wdf.markChanged();
                }

                fontVisible = isEnabled;

                if (fontVisible)
                {
                    selectedOption.setOptionEnabled(false);
                }
            }
        };
        add(normalOption);

        selectedOption = new WDFEditableFontTreeNode("Selected text font", listBox.getFont2(), true,
            false, files)
        {
            @Override
            public void setFont(boolean isEnabled, WDFFont font)
            {
                if (listBox.setFont2(font))
                {
                    wdf.markChanged();
                }

                selectedFontVisible = isEnabled;

                if (selectedFontVisible)
                {
                    normalOption.setOptionEnabled(false);
                }
            }
        };
        add(selectedOption);

        for (int i = 2; i <= 12; i++)
        {
            add(new WDFEditableIntegerValueTreeNode("Unknown " + i, listBox.getUnknown2(i - 2), 5,
                    Integer.MIN_VALUE, Integer.MAX_VALUE)
            {
                @Override
                public void setInteger(int value)
                {
                    if (listBox.setUnknown2(Integer.parseInt(this.getUserObject().toString().substring(8)) - 2, value))
                    {
                        wdf.markChanged();
                    }
                }
            });
        }
    }
    @Override
    public void paintOnImage(int x, int y, int w, int h, BufferedImage image)
    {
        if (this.visible)
        {
            Graphics gimg = image.getGraphics();
            Image img;

            x += listBox.getPosition().x;
            y += listBox.getPosition().y;
            w = Math.min(listBox.getDimension().width, w - listBox.getPosition().x);
            h = Math.min(listBox.getDimension().height, h - listBox.getPosition().y);

            if (this.backgroundVisible)
            {
                if (!listBox.getBackgroundImage().equalsIgnoreCase(WDFWidget.NO_IMAGE))
                {
                    img = imgCache.getImage(listBox.getBackgroundImage());
                    Tools.drawImage(img, x, y, w, h, gimg);
                }
                else
                {
                    gimg.setColor(Color.WHITE);
                    gimg.drawRect(x - 1, y - 1, w + 2, h + 2);
                }
            }

            // font
            WDFFont font = listBox.getFont1();
            WDFFont selfont = listBox.getFont2();
            int leftWidth = 0;

            if (fontVisible)
            {
                if (!font.getFontFile().equalsIgnoreCase(WDFWidget.NO_IMAGE))
                {
                    int color = font.getRGBColor();
                    int[] rgb = new int[]{(color >> 16) & 0xFF, (color >> 8) & 0xFF,
                                            (color & 0xFF)};
                    float[] hsb = new float[3];
                    Color.RGBtoHSB(rgb[0], rgb[1], rgb[2], hsb);

                    Image character;
                    BufferedImage bimg;

                    String text = "Sample list text.";

                    for (int i = 0; i < text.length(); i++)
                    {
                        character = imgCache.getCharacterImage(font.getFontFile(), text.charAt(i));
                        bimg = null;

                        if (character != null)
                        {
                            bimg = new BufferedImage(character.getWidth(null),
                                    character.getHeight(null), BufferedImage.TYPE_INT_ARGB);

                            Tools.drawImage(character, 0, 0, bimg.getWidth(),
                                    bimg.getHeight(), bimg.getGraphics());

                            if (font.overrideExeDefaults())
                            {
                                int pixel;
                                float[] ohsb = new float[3];

                                for (int xi = 0; xi < bimg.getWidth(); xi++)
                                {
                                    for (int yi = 0; yi < bimg.getHeight(); yi++)
                                    {
                                        pixel = bimg.getRGB(xi, yi);

                                        if ((pixel & 0xFFFFFF) != 0)
                                        {
                                            Color.RGBtoHSB((pixel >> 16) & 0xFF,
                                                    (pixel >> 8) & 0xFF, pixel & 0xFF, ohsb);

                                            pixel = Color.HSBtoRGB(hsb[0], hsb[1], ohsb[2]);

                                            bimg.setRGB(xi, yi, pixel);
                                        }
                                    }
                                }
                            }

                            Tools.drawImage(bimg, x + leftWidth, y,
                                    w - leftWidth, h, gimg);
                            leftWidth += bimg.getWidth();
                        }
                    }
                }
            }

            // sel font
            if (selectedFontVisible)
            {
                if (!selfont.getFontFile().equalsIgnoreCase(WDFWidget.NO_IMAGE))
                {
                    int color = selfont.getRGBColor();
                    int[] rgb = new int[]{(color >> 16) & 0xFF, (color >> 8) & 0xFF,
                                            (color & 0xFF)};
                    float[] hsb = new float[3];
                    Color.RGBtoHSB(rgb[0], rgb[1], rgb[2], hsb);

                    Image character;
                    BufferedImage bimg;

                    String text = "Sample list text.";

                    for (int i = 0; i < text.length(); i++)
                    {
                        character = imgCache.getCharacterImage(selfont.getFontFile(), text.charAt(i));
                        bimg = null;

                        if (character != null)
                        {
                            bimg = new BufferedImage(character.getWidth(null),
                                    character.getHeight(null), BufferedImage.TYPE_INT_ARGB);

                            Tools.drawImage(character, 0, 0, bimg.getWidth(),
                                    bimg.getHeight(), bimg.getGraphics());

                            if (font.overrideExeDefaults())
                            {
                                int pixel;
                                float[] ohsb = new float[3];

                                for (int xi = 0; xi < bimg.getWidth(); xi++)
                                {
                                    for (int yi = 0; yi < bimg.getHeight(); yi++)
                                    {
                                        pixel = bimg.getRGB(xi, yi);

                                        if ((pixel & 0xFFFFFF) != 0)
                                        {
                                            Color.RGBtoHSB((pixel >> 16) & 0xFF,
                                                    (pixel >> 8) & 0xFF, pixel & 0xFF, ohsb);

                                            pixel = Color.HSBtoRGB(hsb[0], hsb[1], ohsb[2]);

                                            bimg.setRGB(xi, yi, pixel);
                                        }
                                    }
                                }
                            }

                            Tools.drawImage(bimg, x + leftWidth, y,
                                    w - leftWidth, h, gimg);
                            leftWidth += bimg.getWidth();
                        }
                    }
                }
            }
        }
    }

    @Override
    public Rectangle getRectangle()
    {
        return new Rectangle(listBox.getPosition(), listBox.getDimension());
    }

    @Override
    public Component getTreeCellEditorComponent(boolean selected)
    {
        if (cellEditor == null)
        {
            cellEditor = new JPanel(new BorderLayout());
            cellEditor.setOpaque(false);

            display = new JCheckBox();
            display.setSelected(true);
            display.setOpaque(false);
            display.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent e) {
                    visible = display.isSelected();
                    fireChangeNotification();
                }
            });

            cellEditor.add(display, BorderLayout.WEST);

            lbl = new JLabel(this.getUserObject().toString());
            cellEditor.add(lbl, BorderLayout.CENTER);
        }

        if (selected)
        {
            lbl.setOpaque(true);
            lbl.setBackground(UIManager.getColor("Tree.selectionBackground"));
        }
        else
        {
            lbl.setOpaque(false);
        }

        return cellEditor;
    }

    @Override
    public void commitChanges() {
        Enumeration en = this.children();
        Object child;

        while (en.hasMoreElements())
        {
            child = en.nextElement();

            if (child instanceof WDFEditableTreeNode)
            {
                ((WDFEditableTreeNode)child).commitChanges();
            }
        }
    }
}
