/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: MenuTreeNode.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 1.8.2011
 */

package ue.gui.stbof.wdf;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.Enumeration;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.UIManager;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.wdf.WDFFont;
import ue.edit.res.stbof.wdf.WDFMenu;
import ue.edit.res.stbof.wdf.WDFWidget;
import ue.edit.res.stbof.wdf.WindowDefinitionFile;
import ue.util.ImageCache;
import ue.util.Tools;

/**
 *
 */
public class MenuTreeNode extends WidgetTreeNode
{
    private JPanel cellEditor;
    private JCheckBox display;
    private JLabel lbl;

    private WDFMenu menu;

    private WDFEditableFontTreeNode normalOption;
    private WDFEditableFontTreeNode selectedOption;

    private boolean leftImageVisible = true;
    private boolean topImageVisible = true;
    private boolean rightImageVisible = true;
    private boolean bottomImageVisible = true;
    private boolean fontVisible = true;
    private boolean selectedFontVisible = false;

    public MenuTreeNode(final WindowDefinitionFile wdf, final WDFMenu menu,
            Stbof res, ImageCache imgCache)
    {
        super(wdf, menu, res, imgCache);
        this.menu = menu;

        add(new WDFEditablePointTreeNode("Unknown position (?)", menu.getMenuPosition())
        {
            @Override
            public void setPoint(Point point)
            {
                if (menu.setMenuPosition(point))
                {
                    wdf.markChanged();
                }
            }
        });

        add(new WDFEditableDimensionTreeNode("Unknown dimension (?)", menu.getMenuDimension())
        {
            @Override
            public void setDimension(Dimension dim) {
                if (menu.setMenuDimension(dim))
                {
                    wdf.markChanged();
                }
            }
        });

        ArrayList<String> files = res.getFileList(new FilenameFilter()
        {
            @Override
            public boolean accept(File dir, String name) {
                return name.endsWith(".fnt");
            }
        });

        normalOption = new WDFEditableFontTreeNode("Font", menu.getFont1(), true,
            true, files)
        {
            @Override
            public void setFont(boolean isEnabled, WDFFont font)
            {
                if (menu.setFont1(font))
                {
                    wdf.markChanged();
                }

                fontVisible = isEnabled;

                if (fontVisible)
                {
                    selectedOption.setOptionEnabled(false);
                }
            }
        };
        add(normalOption);

        selectedOption = new WDFEditableFontTreeNode("Selected text font", menu.getFont2(), true,
            false, files)
        {
            @Override
            public void setFont(boolean isEnabled, WDFFont font)
            {
                if (menu.setFont2(font))
                {
                    wdf.markChanged();
                }

                selectedFontVisible = isEnabled;

                if (selectedFontVisible)
                {
                    normalOption.setOptionEnabled(false);
                }
            }
        };
        add(selectedOption);

        files = res.getFileList(new FilenameFilter()
        {
            @Override
            public boolean accept(File dir, String name) {
                return name.endsWith(".mnu");
            }
        });
        files.add(0, WDFWidget.NO_IMAGE);

        add(new WDFEditableOptionTreeNode("Menu file",
                menu.getMenuFile(),
                files, false, true
                )
                {
                    @Override
                    public void setSelectedItem(boolean isEnabled, int index, String item)
                    {
                        if (menu.setMenuFile(item))
                        {
                            wdf.markChanged();
                        }
                    }
                });

        files = res.getFileList(new FilenameFilter()
        {
            @Override
            public boolean accept(File dir, String name) {
                return name.endsWith(".tga");
            }
        });
        files.add(0, WDFWidget.NO_IMAGE);

        add(new WDFEditableOptionTreeNode("Left image",
                menu.getLeftImage(),
                files, true, leftImageVisible
                )
                {
                    @Override
                    public void setSelectedItem(boolean isEnabled, int index, String item)
                    {
                        if (menu.setLeftImage(item))
                        {
                            wdf.markChanged();
                        }

                        leftImageVisible = isEnabled;
                    }
                });

        add(new WDFEditableOptionTreeNode("Top image",
                menu.getTopImage(),
                files, true, topImageVisible
                )
                {
                    @Override
                    public void setSelectedItem(boolean isEnabled, int index, String item)
                    {
                        if (menu.setTopImage(item))
                        {
                            wdf.markChanged();
                        }

                        topImageVisible = isEnabled;
                    }
                });

        add(new WDFEditableOptionTreeNode("Right image",
                menu.getRightImage(),
                files, true, rightImageVisible
                )
                {
                    @Override
                    public void setSelectedItem(boolean isEnabled, int index, String item)
                    {
                        if (menu.setRightImage(item))
                        {
                            wdf.markChanged();
                        }

                        rightImageVisible = isEnabled;
                    }
                });

        add(new WDFEditableOptionTreeNode("Bottom image",
                menu.getBottomImage(),
                files, true, bottomImageVisible
                )
                {
                    @Override
                    public void setSelectedItem(boolean isEnabled, int index, String item)
                    {
                        if (menu.setBottomImage(item))
                        {
                            wdf.markChanged();
                        }

                        bottomImageVisible = isEnabled;
                    }
                });
    }
    @Override
    public void paintOnImage(int x, int y, int w, int h, BufferedImage image)
    {
        if (this.visible)
        {
            Graphics gimg = image.getGraphics();
            Image img;

            x += menu.getPosition().x;
            y += menu.getPosition().y;
            w = Math.min(menu.getDimension().width, w - menu.getPosition().x);
            h = Math.min(menu.getDimension().height, h - menu.getPosition().y);

            if (this.backgroundVisible)
            {
                if (!menu.getBackgroundImage().equalsIgnoreCase(WDFWidget.NO_IMAGE))
                {
                    img = imgCache.getImage(menu.getBackgroundImage());
                    Tools.drawImage(img, x, y, w, h, gimg);
                }
                else
                {
                    gimg.setColor(Color.WHITE);
                    gimg.drawRect(x - 1, y - 1, w + 2, h + 2);
                }
            }

            if (!menu.getTopImage().equalsIgnoreCase(WDFWidget.NO_IMAGE))
            {
                img = imgCache.getImage(menu.getTopImage());

                if (topImageVisible)
                {
                    Tools.drawImage(img, x, y, 800, 600, gimg);
                }

                y += img.getHeight(null);
                w = img.getWidth(null);
            }

            int leftWidth = 0;

            if (!menu.getLeftImage().equalsIgnoreCase(WDFWidget.NO_IMAGE))
            {
                img = imgCache.getImage(menu.getLeftImage());
                if (leftImageVisible)
                {
                    Tools.drawImage(img, x, y, 800, 600, gimg);
                }

                leftWidth = img.getWidth(null);
            }

            if (rightImageVisible)
            {
                if (!menu.getRightImage().equalsIgnoreCase(WDFWidget.NO_IMAGE))
                {
                    img = imgCache.getImage(menu.getRightImage());
                    Tools.drawImage(img, x + w - img.getWidth(null), y,
                            w, h, gimg);
                }
            }

            // font
            WDFFont font = menu.getFont1();
            WDFFont selfont = menu.getFont2();
            int fheight = 0;

            if (!font.getFontFile().equalsIgnoreCase(WDFWidget.NO_IMAGE))
            {
                Image character = imgCache.getCharacterImage(font.getFontFile(), 'S');
                fheight = character.getHeight(null);

                if (fontVisible)
                {
                    int color = font.getRGBColor();
                    int[] rgb = new int[]{(color >> 16) & 0xFF, (color >> 8) & 0xFF,
                                            (color & 0xFF)};
                    float[] hsb = new float[3];
                    Color.RGBtoHSB(rgb[0], rgb[1], rgb[2], hsb);

                    BufferedImage bimg;

                    String text = "Sample menu text.";

                    for (int i = 0; i < text.length(); i++)
                    {
                        character = imgCache.getCharacterImage(font.getFontFile(), text.charAt(i));
                        bimg = null;

                        if (character != null)
                        {
                            bimg = new BufferedImage(character.getWidth(null),
                                    character.getHeight(null), BufferedImage.TYPE_INT_ARGB);

                            Tools.drawImage(character, 0, 0, bimg.getWidth(),
                                    bimg.getHeight(), bimg.getGraphics());

                            if (font.overrideExeDefaults())
                            {
                                int pixel;
                                float[] ohsb = new float[3];

                                for (int xi = 0; xi < bimg.getWidth(); xi++)
                                {
                                    for (int yi = 0; yi < bimg.getHeight(); yi++)
                                    {
                                        pixel = bimg.getRGB(xi, yi);

                                        if ((pixel & 0xFFFFFF) != 0)
                                        {
                                            Color.RGBtoHSB((pixel >> 16) & 0xFF,
                                                    (pixel >> 8) & 0xFF, pixel & 0xFF, ohsb);

                                            pixel = Color.HSBtoRGB(hsb[0], hsb[1], ohsb[2]);

                                            bimg.setRGB(xi, yi, pixel);
                                        }
                                    }
                                }
                            }

                            Tools.drawImage(bimg, x + leftWidth, y,
                                    800, 600, gimg);
                            leftWidth += bimg.getWidth();
                        }
                    }
                }
            }

            // sel font
            if (!selfont.getFontFile().equalsIgnoreCase(WDFWidget.NO_IMAGE))
            {
                Image character = imgCache.getCharacterImage(selfont.getFontFile(), 'S');
                fheight = character.getHeight(null);

                if (selectedFontVisible)
                {
                    int color = selfont.getRGBColor();
                    int[] rgb = new int[]{(color >> 16) & 0xFF, (color >> 8) & 0xFF,
                                            (color & 0xFF)};
                    float[] hsb = new float[3];
                    Color.RGBtoHSB(rgb[0], rgb[1], rgb[2], hsb);

                    BufferedImage bimg;

                    String text = "Sample menu text.";

                    for (int i = 0; i < text.length(); i++)
                    {
                        character = imgCache.getCharacterImage(selfont.getFontFile(), text.charAt(i));
                        bimg = null;

                        if (character != null)
                        {
                            bimg = new BufferedImage(character.getWidth(null),
                                    character.getHeight(null), BufferedImage.TYPE_INT_ARGB);

                            Tools.drawImage(character, 0, 0, bimg.getWidth(),
                                    bimg.getHeight(), bimg.getGraphics());

                            if (font.overrideExeDefaults())
                            {
                                int pixel;
                                float[] ohsb = new float[3];

                                for (int xi = 0; xi < bimg.getWidth(); xi++)
                                {
                                    for (int yi = 0; yi < bimg.getHeight(); yi++)
                                    {
                                        pixel = bimg.getRGB(xi, yi);

                                        if ((pixel & 0xFFFFFF) != 0)
                                        {
                                            Color.RGBtoHSB((pixel >> 16) & 0xFF,
                                                    (pixel >> 8) & 0xFF, pixel & 0xFF, ohsb);

                                            pixel = Color.HSBtoRGB(hsb[0], hsb[1], ohsb[2]);

                                            bimg.setRGB(xi, yi, pixel);
                                        }
                                    }
                                }
                            }

                            Tools.drawImage(bimg, x + leftWidth, y,
                                    800, 600, gimg);
                            leftWidth += bimg.getWidth();
                        }
                    }
                }
            }

            if (bottomImageVisible)
            {
                if (!menu.getBottomImage().equalsIgnoreCase(WDFWidget.NO_IMAGE))
                {
                    img = imgCache.getImage(menu.getBottomImage());
                    Tools.drawImage(img, x, y + fheight,
                            800, 600, gimg);
                }
            }
        }
    }

    @Override
    public Rectangle getRectangle()
    {
        return new Rectangle(menu.getPosition(), menu.getDimension());
    }

    @Override
    public Component getTreeCellEditorComponent(boolean selected)
    {
        if (cellEditor == null)
        {
            cellEditor = new JPanel(new BorderLayout());
            cellEditor.setOpaque(false);

            display = new JCheckBox();
            display.setSelected(true);
            display.setOpaque(false);
            display.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent e) {
                    visible = display.isSelected();
                    fireChangeNotification();
                }
            });

            cellEditor.add(display, BorderLayout.WEST);

            lbl = new JLabel(this.getUserObject().toString());
            cellEditor.add(lbl, BorderLayout.CENTER);
        }

        if (selected)
        {
            lbl.setOpaque(true);
            lbl.setBackground(UIManager.getColor("Tree.selectionBackground"));
        }
        else
        {
            lbl.setOpaque(false);
        }

        return cellEditor;
    }

    @Override
    public void commitChanges() {
        Enumeration en = this.children();
        Object child;

        while (en.hasMoreElements())
        {
            child = en.nextElement();

            if (child instanceof WDFEditableTreeNode)
            {
                ((WDFEditableTreeNode)child).commitChanges();
            }
        }
    }
}
