/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: PictureBoxTreeNode.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 4.8.2011
 */

package ue.gui.stbof.wdf;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.util.Enumeration;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.UIManager;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.wdf.WDFPictureBox;
import ue.edit.res.stbof.wdf.WDFWidget;
import ue.edit.res.stbof.wdf.WindowDefinitionFile;
import ue.util.ImageCache;
import ue.util.Tools;

/**
 *
 */
public class PictureBoxTreeNode extends WidgetTreeNode
{
    private JPanel cellEditor;
    private JCheckBox display;
    private JLabel lbl;

    private WDFPictureBox pictureBox;

    private boolean placeHolderVisible = true;

    public PictureBoxTreeNode(final WindowDefinitionFile wdf, final WDFPictureBox pictureBox,
            Stbof res, ImageCache imgCache)
    {
        super(wdf, pictureBox, res, imgCache);
        this.pictureBox = pictureBox;

        add(new WDFEditableOptionTreeNode("Image placeholder", null, null,
            true, placeHolderVisible)
        {
            @Override
            public void setSelectedItem(boolean isEnabled, int index, String item) {
                placeHolderVisible = isEnabled;
            }
        });

        add(new WDFEditableIntegerValueTreeNode("Unknown", pictureBox.getUnknown1(), 5,
                Integer.MIN_VALUE, Integer.MAX_VALUE)
        {
            @Override
            public void setInteger(int value)
            {
                if (pictureBox.setUnknown1(value))
                {
                    wdf.markChanged();
                }
            }
        });
    }
    @Override
    public void paintOnImage(int x, int y, int w, int h, BufferedImage image)
    {
        if (this.visible)
        {
            Graphics gimg = image.getGraphics();
            Image img;

            x += pictureBox.getPosition().x;
            y += pictureBox.getPosition().y;
            w = Math.min(pictureBox.getDimension().width, w - pictureBox.getPosition().x);
            h = Math.min(pictureBox.getDimension().height, h - pictureBox.getPosition().y);

            if (this.backgroundVisible)
            {
                if (!pictureBox.getBackgroundImage().equalsIgnoreCase(WDFWidget.NO_IMAGE))
                {
                    img = imgCache.getImage(pictureBox.getBackgroundImage());
                    Tools.drawImage(img, x, y, w, h, gimg);
                }
                else
                {
                    gimg.setColor(Color.WHITE);
                    gimg.drawRect(x - 1, y - 1, w + 2, h + 2);
                }
            }

            if (placeHolderVisible)
            {
                gimg.setColor(Color.BLUE);
                gimg.drawRect(x, y, w, h);
                gimg.drawLine(x, y, x + w, y + h);
                gimg.drawLine(x, y + h, x + w, y);
            }
        }
    }

    @Override
    public Rectangle getRectangle()
    {
        return new Rectangle(pictureBox.getPosition(), pictureBox.getDimension());
    }

    @Override
    public Component getTreeCellEditorComponent(boolean selected)
    {
        if (cellEditor == null)
        {
            cellEditor = new JPanel(new BorderLayout());
            cellEditor.setOpaque(false);

            display = new JCheckBox();
            display.setSelected(true);
            display.setOpaque(false);
            display.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent e) {
                    visible = display.isSelected();
                    fireChangeNotification();
                }
            });

            cellEditor.add(display, BorderLayout.WEST);

            lbl = new JLabel(this.getUserObject().toString());
            cellEditor.add(lbl, BorderLayout.CENTER);
        }

        if (selected)
        {
            lbl.setOpaque(true);
            lbl.setBackground(UIManager.getColor("Tree.selectionBackground"));
        }
        else
        {
            lbl.setOpaque(false);
        }

        return cellEditor;
    }

    @Override
    public void commitChanges() {
        Enumeration en = this.children();
        Object child;

        while (en.hasMoreElements())
        {
            child = en.nextElement();

            if (child instanceof WDFEditableTreeNode)
            {
                ((WDFEditableTreeNode)child).commitChanges();
            }
        }
    }
}
