/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: SliderTreeNode.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 3.8.2011
 */

package ue.gui.stbof.wdf;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.Enumeration;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.UIManager;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.wdf.WDFSlider;
import ue.edit.res.stbof.wdf.WDFWidget;
import ue.edit.res.stbof.wdf.WindowDefinitionFile;
import ue.util.ImageCache;
import ue.util.Tools;

/**
 *
 */
public class SliderTreeNode extends WidgetTreeNode
{
    private Stbof stbof;

    private JPanel cellEditor;
    private JCheckBox display;
    private JLabel lbl;

    private WDFSlider slider;

    private WDFEditableOptionTreeNode cellOffOption;
    private WDFEditableOptionTreeNode cellOnOption;
    private WDFEditableOptionTreeNode markerOffOption;
    private WDFEditableOptionTreeNode markerOnOption;
    
    private boolean cellOnImageVisible = false;
    private boolean cellOffImageVisible = true;
    private boolean markerOnImageVisible = true;
    private boolean markerOffImageVisible = false;

    public SliderTreeNode(final WindowDefinitionFile wdf, final WDFSlider slider,
            Stbof res, ImageCache imgCache)
    {
        super(wdf, slider, res, imgCache);
        this.stbof = res;
        this.slider = slider;

        for (int i = 1; i <= 2; i++)
        {
            add(new WDFEditableIntegerValueTreeNode("Unknown " + i, slider.getUnknown1(i - 1), 3,
                    Byte.MIN_VALUE, Byte.MAX_VALUE)
            {
                @Override
                public void setInteger(int value)
                {
                    if (slider.setUnknown1(Byte.parseByte(this.getUserObject().toString().substring(8)) - 1, (byte)value))
                    {
                        wdf.markChanged();
                    }
                }
            });
        }

        for (int i = 3; i <= 6; i++)
        {
            add(new WDFEditableIntegerValueTreeNode("Unknown " + i, slider.getUnknown2(i - 3), 5,
                    Integer.MIN_VALUE, Integer.MAX_VALUE)
            {
                @Override
                public void setInteger(int value)
                {
                    if (slider.setUnknown2(Integer.parseInt(this.getUserObject().toString().substring(8)) - 3, value))
                    {
                        wdf.markChanged();
                    }
                }
            });
        }

        ArrayList<String> files = res.getFileList(new FilenameFilter()
        {
            @Override
            public boolean accept(File dir, String name) {
                return name.endsWith(".tga");
            }
        });
        files.add(0, WDFWidget.NO_IMAGE);

        cellOffOption = new WDFEditableOptionTreeNode("Cell off image",
                slider.getCellOffImage(),
                files, true, cellOffImageVisible
                )
                {
                    @Override
                    public void setSelectedItem(boolean isEnabled, int index, String item)
                    {
                        if (slider.setCellOffImage(item))
                        {
                            wdf.markChanged();
                        }

                        cellOffImageVisible = isEnabled;
                        
                        if (isEnabled)
                        {
                            cellOnImageVisible = false;
                            cellOnOption.setOptionEnabled(false);
                        }
                    }
                };
        add(cellOffOption);

        cellOnOption = new WDFEditableOptionTreeNode("Cell on image",
                slider.getCellOnImage(),
                files, true, cellOnImageVisible
                )
                {
                    @Override
                    public void setSelectedItem(boolean isEnabled, int index, String item)
                    {
                        if (slider.setCellOnImage(item))
                        {
                            wdf.markChanged();
                        }

                        cellOnImageVisible = isEnabled;

                        if (isEnabled)
                        {
                            cellOffImageVisible = false;
                            cellOffOption.setOptionEnabled(false);
                        }
                     }
                };
        add(cellOnOption);

        markerOffOption = new WDFEditableOptionTreeNode("Marker off image",
                slider.getMarkerOffImage(),
                files, true, markerOffImageVisible
                )
                {
                    @Override
                    public void setSelectedItem(boolean isEnabled, int index, String item)
                    {
                        if (slider.setMarkerOffImage(item))
                        {
                            wdf.markChanged();
                        }

                        markerOffImageVisible = isEnabled;

                        if (isEnabled)
                        {
                            markerOnImageVisible = false;
                            markerOffOption.setOptionEnabled(false);
                        }
                    }
                };
        add(markerOffOption);

        markerOnOption = new WDFEditableOptionTreeNode("Marker on image",
                slider.getMarkerOnImage(),
                files, true, markerOnImageVisible
                )
                {
                    @Override
                    public void setSelectedItem(boolean isEnabled, int index, String item)
                    {
                        if (slider.setMarkerOnImage(item))
                        {
                            wdf.markChanged();
                        }

                        markerOffImageVisible = isEnabled;

                        if (isEnabled)
                        {
                            markerOffImageVisible = false;
                            markerOffOption.setOptionEnabled(false);
                        }
                    }
                };
        add(markerOnOption);

        for (int i = 7; i <= 9; i++)
        {
            add(new WDFEditableIntegerValueTreeNode("Unknown " + i, slider.getUnknown2(i - 7), 5,
                    Integer.MIN_VALUE, Integer.MAX_VALUE)
            {
                @Override
                public void setInteger(int value)
                {
                    if (slider.setUnknown2(Integer.parseInt(this.getUserObject().toString().substring(8)) - 7, value))
                    {
                        wdf.markChanged();
                    }
                }
            });
        }
    }
    @Override
    public void paintOnImage(int x, int y, int w, int h, BufferedImage image)
    {
        if (this.visible)
        {
            Graphics gimg = image.getGraphics();
            Image img;

            x += slider.getPosition().x;
            y += slider.getPosition().y;
            w = Math.min(slider.getDimension().width, w - slider.getPosition().x);
            h = Math.min(slider.getDimension().height, h - slider.getPosition().y);

            if (this.backgroundVisible)
            {
                if (!slider.getBackgroundImage().equalsIgnoreCase(WDFWidget.NO_IMAGE))
                {
                    img = imgCache.getImage(slider.getBackgroundImage());
                    Tools.drawImage(img, x, y, w, h, gimg);
                }
                else
                {
                    gimg.setColor(Color.WHITE);
                    gimg.drawRect(x - 1, y - 1, w + 2, h + 2);
                }
            }

            if (cellOffImageVisible)
            {
                if (!slider.getCellOffImage().equalsIgnoreCase(WDFWidget.NO_IMAGE))
                {
                    img = imgCache.getImage(slider.getCellOffImage());
                    Tools.drawImage(img, x, y, w, h, gimg);
                }
            }

            if (cellOnImageVisible)
            {
                if (!slider.getCellOnImage().equalsIgnoreCase(WDFWidget.NO_IMAGE))
                {
                    img = imgCache.getImage(slider.getCellOnImage());
                    Tools.drawImage(img, x, y, w, h, gimg);
                }
            }

            if (cellOffImageVisible)
            {
                if (!slider.getMarkerOffImage().equalsIgnoreCase(WDFWidget.NO_IMAGE))
                {
                    img = imgCache.getImage(slider.getMarkerOffImage());
                    Tools.drawImage(img, x, y, w, h, gimg);
                }
            }

            if (cellOffImageVisible)
            {
                if (!slider.getMarkerOnImage().equalsIgnoreCase(WDFWidget.NO_IMAGE))
                {
                    img = imgCache.getImage(slider.getMarkerOnImage());
                    Tools.drawImage(img, x, y, w, h, gimg);
                }
            }
        }
    }

    @Override
    public Rectangle getRectangle()
    {
        return new Rectangle(slider.getPosition(), slider.getDimension());
    }

    @Override
    public Component getTreeCellEditorComponent(boolean selected)
    {
        if (cellEditor == null)
        {
            cellEditor = new JPanel(new BorderLayout());
            cellEditor.setOpaque(false);

            display = new JCheckBox();
            display.setSelected(true);
            display.setOpaque(false);
            display.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent e) {
                    visible = display.isSelected();
                    fireChangeNotification();
                }
            });

            cellEditor.add(display, BorderLayout.WEST);

            lbl = new JLabel(this.getUserObject().toString());
            cellEditor.add(lbl, BorderLayout.CENTER);
        }

        if (selected)
        {
            lbl.setOpaque(true);
            lbl.setBackground(UIManager.getColor("Tree.selectionBackground"));
        }
        else
        {
            lbl.setOpaque(false);
        }

        return cellEditor;
    }

    @Override
    public void commitChanges() {
        Enumeration en = this.children();
        Object child;

        while (en.hasMoreElements())
        {
            child = en.nextElement();

            if (child instanceof WDFEditableTreeNode)
            {
                ((WDFEditableTreeNode)child).commitChanges();
            }
        }
    }
}
