/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WDFEditableBooleanValueTreeNode.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 1.8.2011
 */

package ue.gui.stbof.wdf;

import java.awt.Component;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.UIManager;
import ue.util.ID;

/**
 *
 */
public abstract class WDFEditableBooleanValueTreeNode extends WDFEditableTreeNode
{
    private JPanel cellEditor;
    private JLabel lblValue;
    private JComboBox cmbValue;

    public WDFEditableBooleanValueTreeNode(String desc, boolean value)
    {
        super(desc);

        cmbValue = new JComboBox();

        cmbValue.addItem(new ID("Yes", 1));
        cmbValue.addItem(new ID("No", 0));

        cmbValue.setSelectedIndex((value) ? 0 : 1);

        cmbValue.addItemListener(new ItemListener()
        {
            @Override
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED)
                {
                    setBoolean(cmbValue.getSelectedIndex() == 0);
                    fireChangeNotification();
                }
            }
        });

        JLabel lblDesc = new JLabel(desc + ": ");

        lblValue = new JLabel(cmbValue.getSelectedItem().toString());

        cellEditor = new JPanel();

        cellEditor.setLayout(new BoxLayout(cellEditor, BoxLayout.X_AXIS));
        cellEditor.setOpaque(false);
        cellEditor.add(lblDesc);
//        cellEditor.add(Box.createHorizontalGlue());
        cellEditor.add(lblValue);
        cellEditor.add(cmbValue);
        cellEditor.setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));
        cellEditor.setBackground(UIManager.getColor("Tree.selectionBackground"));
    }

    @Override
    public Component getTreeCellEditorComponent(boolean selected)
    {
        cellEditor.setOpaque(selected);
        cmbValue.setVisible(selected);
        lblValue.setVisible(!selected);
        lblValue.setText(cmbValue.getSelectedItem().toString());

        return cellEditor;
    }

    public abstract void setBoolean(boolean value);

    @Override
    public void commitChanges()
    {
        setBoolean(cmbValue.getSelectedIndex() == 0);
    }
}
