/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WDFEditableColorTreeNode.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 1.8.2011
 */

package ue.gui.stbof.wdf;

import java.awt.Component;
import java.util.Enumeration;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.UIManager;

/**
 *
 */
public abstract class WDFEditableColorTreeNode extends WDFEditableTreeNode
{
    private int color;
    private JPanel cellEditor;
    private JLabel lbl;

    public WDFEditableColorTreeNode(String name, int value)
    {
        super(name);

        this.color = value;

        cellEditor = new JPanel();
        cellEditor.setLayout(new BoxLayout(cellEditor, BoxLayout.X_AXIS));

        cellEditor.setOpaque(false);

        lbl = new JLabel(name);
        lbl.setBackground(UIManager.getColor("Tree.selectionBackground"));
        cellEditor.add(lbl);

        add(new WDFEditableIntegerValueTreeNode("Red", ((value >> 16) & 0xFF), 3,
                0, 255)
        {
            @Override
            public void setInteger(int value)
            {
                int tmp = (color & 0x00FFFF) | ((value & 0xFF) << 16);
                if (color != tmp)
                {
                    color = tmp;
                    setColor(color);
                }
            }
        });
        add(new WDFEditableIntegerValueTreeNode("Blue", (value & 0xFF), 3,
                0, 255)
        {
            @Override
            public void setInteger(int value)
            {
                int tmp = (color & 0xFFFF00) | (value & 0xFF);
                if (color != tmp)
                {
                    color = tmp;
                    setColor(color);
                }
            }
        });
        add(new WDFEditableIntegerValueTreeNode("Green", ((value >> 8) & 0xFF), 3,
                0, 255)
        {
            @Override
            public void setInteger(int value)
            {
                int tmp = (color & 0xFF00FF) | ((value & 0xFF) << 8);
                if (color != tmp)
                {
                    color = tmp;
                    setColor(color);
                }
            }
        });
    }

    public abstract void setColor(int value);

    @Override
    public void commitChanges() {
        Enumeration en = this.children();
        Object child;

        while (en.hasMoreElements())
        {
            child = en.nextElement();

            if (child instanceof WDFEditableTreeNode)
            {
                ((WDFEditableTreeNode)child).commitChanges();
            }
        }
    }

    @Override
    public Component getTreeCellEditorComponent(boolean selected)
    {
        lbl.setOpaque(selected);

        return cellEditor;

    }
}
