/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WDFEditableFontTreeNode.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 1.8.2011
 */

package ue.gui.stbof.wdf;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Enumeration;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.UIManager;
import ue.edit.res.stbof.wdf.WDFFont;

/**
 *
 */
public abstract class WDFEditableFontTreeNode extends WDFEditableTreeNode
{
    private JPanel cellEditor;
    private JCheckBox chkEnable;
    private JLabel lbl;

    private WDFFont font;

    public WDFEditableFontTreeNode(String desc, WDFFont value, boolean canBeDisabled,
            boolean isEnabled, ArrayList<String> fonts)
    {
        super(desc);

        this.font = value;

        cellEditor = new JPanel();
        cellEditor.setLayout(new BoxLayout(cellEditor, BoxLayout.X_AXIS));

        cellEditor.setOpaque(false);

        if (canBeDisabled)
        {
            chkEnable = new JCheckBox();
            chkEnable.setOpaque(false);
            chkEnable.setSelected(isEnabled);
            chkEnable.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent e) {
                    setFont(chkEnable.isSelected(), font);
                    fireChangeNotification();
                }
            });
            cellEditor.add(chkEnable);
        }

        lbl = new JLabel(desc);
        cellEditor.add(lbl);

        // children
        add(new WDFEditableColorTreeNode("Color", font.getRGBColor())
        {
            @Override
            public void setColor(int value)
            {
                font.setRGBColor(value);
                setFont(chkEnable.isSelected(), font);
            }
        });

        for (int i = 1; i < 8; i++)
        {
            add(new WDFEditableIntegerValueTreeNode("Unknown " + i, font.getUnknown(i - 1), 5,
                    Integer.MIN_VALUE, Integer.MAX_VALUE)
            {
                @Override
                public void setInteger(int value)
                {
                    font.setUnknown(Integer.parseInt(this.getUserObject().toString().substring(8)) - 1, value);
                    setFont(chkEnable.isSelected(), font);
                }
            });
        }

        add(new WDFEditableBooleanValueTreeNode("Use color settings", font.overrideExeDefaults())
        {
            @Override
            public void setBoolean(boolean value) {
                if (font.setOverrideExeDefaults(value))
                {
                    setFont(chkEnable.isSelected(), font);
                }
            }
        });

        if (!fonts.contains(font.getFontFile()))
        {
            fonts.add(font.getFontFile());
        }

        add(new WDFEditableOptionTreeNode("Font file",
                font.getFontFile(),
                fonts, false, true
                )
                {
                    @Override
                    public void setSelectedItem(boolean isEnabled, int index, String item)
                    {
                        if (!font.getFontFile().equalsIgnoreCase(item))
                        {
                            font.setFontFile(item);
                            setFont(chkEnable.isSelected(), font);
                        }
                    }
                });
    }

    @Override
    public Component getTreeCellEditorComponent(boolean selected)
    {
        if (selected)
        {
            lbl.setOpaque(true);
            lbl.setBackground(UIManager.getColor("Tree.selectionBackground"));
        }
        else
        {
            lbl.setOpaque(false);
        }

        return cellEditor;
    }

    public void setOptionEnabled(boolean value)
    {
        chkEnable.setSelected(value);
        setFont(chkEnable.isSelected(), font);
        fireChangeNotification();
    }

    public abstract void setFont(boolean isEnabled, WDFFont font);

    @Override
    public void commitChanges() {
        Enumeration en = this.children();
        Object child;

        while (en.hasMoreElements())
        {
            child = en.nextElement();

            if (child instanceof WDFEditableTreeNode)
            {
                ((WDFEditableTreeNode)child).commitChanges();
            }
        }
    }
}
