/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WDFEditableIntegerValueTreeNode.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 1.8.2011
 */

package ue.gui.stbof.wdf;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.DecimalFormat;
import java.text.ParseException;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.UIManager;

/**
 *
 */
public abstract class WDFEditableIntegerValueTreeNode extends WDFEditableTreeNode
{
    private JPanel cellEditor;
    private JLabel lblValue;
    private JFormattedTextField txtValue;

    private int minVal, maxVal;

    public WDFEditableIntegerValueTreeNode(String desc, int value, int fieldSize,
            final int minVal, final int maxVal)
    {
        super(desc);

        this.minVal = minVal;
        this.maxVal = maxVal;

        txtValue = new JFormattedTextField(new DecimalFormat("#####"));
        txtValue.setColumns(fieldSize);
        txtValue.setFocusLostBehavior(JFormattedTextField.COMMIT_OR_REVERT);
        txtValue.setHorizontalAlignment(JTextField.RIGHT);
        txtValue.setValue(Integer.valueOf(value));

        txtValue.addPropertyChangeListener("value", new PropertyChangeListener()
        {
            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                int value = Integer.parseInt(txtValue.getValue().toString());
                
                if (value < minVal)
                {
                    value = minVal;
                    txtValue.setValue(value);
                }
                else if (value > maxVal)
                {
                    value = maxVal;
                    txtValue.setValue(value);
                }
                
                setInteger(value);
                fireChangeNotification();
            }
        });
        txtValue.addKeyListener(new KeyAdapter()
        {
            @Override
            public void keyTyped(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_ENTER || e.getKeyCode() == KeyEvent.VK_ACCEPT
                        || e.getKeyCode() == KeyEvent.VK_ESCAPE)
                {
                    try {
                        txtValue.commitEdit();
                        fireChangeNotification();
                    } catch (ParseException ex) {
                    }
                }
            }
        });
        /*
         * Disabled because one can see the delay and the the thing seems unresponsive
         * Would be nice to select all on start edit
        txtValue.addFocusListener(new FocusListener()
        {
            @Override
            public void focusGained(FocusEvent e) {
                SwingUtilities.invokeLater(new Runnable()
                {
                    @Override
                    public void run() {
                        txtValue.setCaretPosition(txtValue.getText().length());
                        txtValue.selectAll();
                    }
                });
            }

            @Override
            public void focusLost(FocusEvent e) {
            }
        });
*/

        JLabel lblDesc = new JLabel(desc + ": ");

        lblValue = new JLabel(txtValue.getValue().toString());

        cellEditor = new JPanel();

        cellEditor.setLayout(new BoxLayout(cellEditor, BoxLayout.X_AXIS));
        cellEditor.setMinimumSize(new Dimension(50, 20));
        cellEditor.setOpaque(false);
        cellEditor.add(lblDesc);
//        cellEditor.add(Box.createHorizontalGlue());
        cellEditor.add(lblValue);
        cellEditor.add(txtValue);

        cellEditor.setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));
        cellEditor.setBackground(UIManager.getColor("Tree.selectionBackground"));
    }

    @Override
    public Component getTreeCellEditorComponent(boolean selected) {
        cellEditor.setOpaque(selected);
        txtValue.setVisible(selected);
        lblValue.setVisible(!selected);
        lblValue.setText(txtValue.getValue().toString());

        return cellEditor;
    }

    public abstract void setInteger(int value);

    @Override
    public void commitChanges() {
        int value = Integer.parseInt(txtValue.getValue().toString());

        if (value < minVal)
        {
            value = minVal;
            txtValue.setValue(value);
        }
        else if (value > maxVal)
        {
            value = maxVal;
            txtValue.setValue(value);
        }

        setInteger(value);
    }
}
