/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WDFEditableOptionTreeNode.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 1.8.2011
 */

package ue.gui.stbof.wdf;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.UIManager;

/**
 *
 */
public abstract class WDFEditableOptionTreeNode extends WDFEditableTreeNode
{
    private JPanel      cellEditor;
    private JLabel      lblValue;
    private JComboBox   cmbOptions;
    private JCheckBox   chkEnable;

    public WDFEditableOptionTreeNode(String desc, String value, ArrayList<String> options,
            boolean canBeDisabled, boolean isEnabled)
    {
        super(desc);

        cellEditor = new JPanel();
        cellEditor.setLayout(new BoxLayout(cellEditor, BoxLayout.X_AXIS));

        cellEditor.setOpaque(false);

        if (canBeDisabled)
        {
            chkEnable = new JCheckBox();
            chkEnable.setOpaque(false);
            chkEnable.setSelected(isEnabled);
            chkEnable.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent e) {
                    if (cmbOptions != null)
                    {
                        setSelectedItem(chkEnable.isSelected(),
                                cmbOptions.getSelectedIndex(),
                                cmbOptions.getSelectedItem().toString());
                    }
                    else
                    {
                        setSelectedItem(chkEnable.isSelected(), -1, null);
                    }
                    fireChangeNotification();
                }
            });
            cellEditor.add(chkEnable);
        }

        if (options != null)
        {
            desc = desc + ": ";

            if (!options.contains(value))
            {
                options.add(value);
            }

            cmbOptions = new JComboBox();
            for (int i = 0; i < options.size(); i++)
            {
                cmbOptions.addItem(options.get(i));
            }

            cmbOptions.setSelectedItem(value);

            cmbOptions.addItemListener(new ItemListener()
            {
                @Override
                public void itemStateChanged(ItemEvent e) {
                    if (e.getStateChange() == ItemEvent.SELECTED)
                    {
                        boolean enabled = true;

                        if (chkEnable != null)
                        {
                            enabled = chkEnable.isSelected();
                        }

                        setSelectedItem(enabled,
                                cmbOptions.getSelectedIndex(),
                                cmbOptions.getSelectedItem().toString());
                        fireChangeNotification();
                    }
                }
            });
        }
        cellEditor.add(new JLabel(desc));


        if (options != null)
        {
            lblValue = new JLabel(cmbOptions.getSelectedItem().toString());
            cellEditor.add(lblValue);
            cellEditor.add(cmbOptions);
        }
        
        cellEditor.setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));
        cellEditor.setBackground(UIManager.getColor("Tree.selectionBackground"));
    }

    @Override
    public Component getTreeCellEditorComponent(boolean selected)
    {
        cellEditor.setOpaque(selected);

        if (cmbOptions != null)
        {
            cmbOptions.setVisible(selected);
            lblValue.setText(cmbOptions.getSelectedItem().toString());
            lblValue.setVisible(!selected);
        }

        return cellEditor;
    }

    public void setOptionEnabled(boolean value)
    {
        if (chkEnable.isSelected() != value)
        {
            chkEnable.setSelected(value);

            if (cmbOptions != null)
            {
                setSelectedItem(chkEnable.isSelected(),
                        cmbOptions.getSelectedIndex(),
                        cmbOptions.getSelectedItem().toString());
            }
            else
            {
                setSelectedItem(chkEnable.isSelected(), -1, null);
            }

            fireChangeNotification();
        }
    }

    public abstract void setSelectedItem(boolean isEnabled, int index, String item);

    @Override
    public void commitChanges() {
        if (cmbOptions != null)
        {
            boolean enabled = true;

            if (chkEnable != null)
            {
                enabled = chkEnable.isSelected();
            }

            setSelectedItem(enabled,
                    cmbOptions.getSelectedIndex(),
                    cmbOptions.getSelectedItem().toString());
        }
        else
        {
            setSelectedItem(chkEnable.isSelected(), -1, null);
        }
    }
}
