/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WDFEditableTreeNode.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 31.7.2011
 */

package ue.gui.stbof.wdf;

import java.awt.Component;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.Vector;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;

/**
 *
 */
public abstract class WDFEditableTreeNode extends DefaultMutableTreeNode
{
    private Vector<ActionListener> listeners;

    public WDFEditableTreeNode(Object userObject)
    {
        super(userObject);
        listeners = new Vector<ActionListener>();
    }

    public abstract Component getTreeCellEditorComponent(boolean selected);

    public void addActionListener(ActionListener listener)
    {
        listeners.add(listener);
        recursiveAddListener(this, listener);
    }

    protected void recursiveAddListener(TreeNode node, ActionListener listener)
    {
        Enumeration en = node.children();
        TreeNode child;

        while (en.hasMoreElements())
        {
            child = (TreeNode) en.nextElement();

            if (child instanceof WDFEditableTreeNode)
            {
                ((WDFEditableTreeNode)child).addActionListener(listener);
            }
            else
            {
                recursiveAddListener(child, listener);
            }
        }
    }

    protected void fireChangeNotification()
    {
        for (int i = 0; i < listeners.size(); i++)
        {
            listeners.get(i).actionPerformed(null);
        }
    }

    public abstract void commitChanges();
}
