/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WDFTreeNode.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 31.7.2011
 */

package ue.gui.stbof.wdf;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.util.Enumeration;
import javax.swing.tree.DefaultMutableTreeNode;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.wdf.WindowDefinitionFile;
import ue.util.ImageCache;

/**
 *
 * @author Alan
 */
public class WDFTreeNode extends DefaultMutableTreeNode
{
    private WindowTreeNode window;

    public WDFTreeNode(Stbof res, WindowDefinitionFile wdf, ActionListener listener,
            ImageCache imgCache)
    {
        super(wdf.getName());

        window = new WindowTreeNode(wdf, wdf.getWindow(), res, imgCache);
        add(window);

        window.addActionListener(listener);
    }

    public BufferedImage paintOnImage(BufferedImage image)
    {
        Rectangle size = window.getRectangle();

        int w = Math.min(2560, Math.max(800, size.width + size.x));
        int h = Math.min(2048, Math.max(600, size.height + size.y));

        if (image.getWidth() != w
                || image.getHeight() != h)
        {
            BufferedImage tmp = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);

            Graphics g = tmp.getGraphics();
            g.setColor(Color.BLACK);
            g.fillRect(0, 0, image.getWidth(), image.getHeight());
            g.drawImage(image, 0, 0, null);
            image = tmp;
        }

        Enumeration en = this.children();
        WidgetTreeNode node;

        while (en.hasMoreElements())
        {
            node = (WidgetTreeNode) en.nextElement();
            node.paintOnImage(0, 0, image.getWidth(), image.getHeight(), image);
        }

        return image;
    }

    public Rectangle getRectangle()
    {
        return window.getRectangle();
    }
    
    public void commitChanges() {
        Enumeration en = this.children();
        Object child;

        while (en.hasMoreElements())
        {
            child = en.nextElement();

            if (child instanceof WDFEditableTreeNode)
            {
                ((WDFEditableTreeNode)child).commitChanges();
            }
        }
    }
}