/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WidgetTreeNode.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 31.7.2011
 */
package ue.gui.stbof.wdf;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.wdf.WDFWidget;
import ue.edit.res.stbof.wdf.WindowDefinitionFile;
import ue.util.ImageCache;

/**
 *
 */
public abstract class WidgetTreeNode extends WDFEditableTreeNode
{
    protected boolean visible = true;
    protected boolean backgroundVisible = true;
    protected WindowDefinitionFile wdf;
    protected ImageCache imgCache;

    public WidgetTreeNode(final WindowDefinitionFile wdf, final WDFWidget widget,
            Stbof res, ImageCache imgCache)
    {
        super(widget.getTypeAsString() + "(" + widget.getInstanceID() + ")");

        this.imgCache = imgCache;
        this.wdf = wdf;

        add(new WDFEditableIntegerValueTreeNode("Instance ID", widget.getInstanceID(), 5,
                Integer.MIN_VALUE, Integer.MAX_VALUE)
        {
            @Override
            public void setInteger(int value)
            {
                if (widget.setInstanceID(value))
                {
                    ((WDFEditableTreeNode)this.getParent()).setUserObject(widget.getTypeAsString() + "(" + widget.getInstanceID() + ")");
                    wdf.markChanged();
                }
            }
        });

        add(new WDFEditablePointTreeNode("Position", widget.getPosition())
        {
            @Override
            public void setPoint(Point point)
            {
                if (widget.setPosition(point))
                {
                    wdf.markChanged();
                }
            }
        });
        add(new WDFEditableDimensionTreeNode("Dimension", widget.getDimension())
        {
            @Override
            public void setDimension(Dimension dim)
            {
                if (widget.setDimension(dim))
                {
                    wdf.markChanged();
                }
            }
        });

        ArrayList<String> images = res.getFileList(new FilenameFilter()
                {
                    @Override
                    public boolean accept(File dir, String name) {
                        return name.endsWith(".tga");
                    }
                });

        images.add(0, WDFWidget.NO_IMAGE);

        if (!images.contains(widget.getBackgroundImage()))
        {
            images.add(widget.getBackgroundImage());
        }

        add(new WDFEditableOptionTreeNode("Background",
                widget.getBackgroundImage(),
                images, true, true
                )
                {
                    @Override
                    public void setSelectedItem(boolean isEnabled, int index, String item)
                    {
                        if (!widget.getBackgroundImage().equalsIgnoreCase(item))
                        {
                            widget.setBackgroundImage(item);
                            wdf.markChanged();
                        }

                        backgroundVisible = isEnabled;
                    }
                });
    }

    public abstract void paintOnImage(int x, int y, int w, int h, BufferedImage image);

    public abstract Rectangle getRectangle();
}