/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2011  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: WindowTreeNode.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 31.7.2011
 */

package ue.gui.stbof.wdf;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Enumeration;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.UIManager;
import ue.edit.res.Stbof;
import ue.edit.res.stbof.wdf.WDFButton;
import ue.edit.res.stbof.wdf.WDFListBox;
import ue.edit.res.stbof.wdf.WDFMenu;
import ue.edit.res.stbof.wdf.WDFPictureBox;
import ue.edit.res.stbof.wdf.WDFScrollBar;
import ue.edit.res.stbof.wdf.WDFSlider;
import ue.edit.res.stbof.wdf.WDFTextBox;
import ue.edit.res.stbof.wdf.WDFWidget;
import ue.edit.res.stbof.wdf.WDFWindow;
import ue.edit.res.stbof.wdf.WindowDefinitionFile;
import ue.util.ImageCache;
import ue.util.Tools;

/**
 *
 * @author Alan
 */
public class WindowTreeNode extends WidgetTreeNode
{
    private Stbof stbof;

    private JPanel cellEditor;
    private JCheckBox display;
    private JLabel lbl;

    private WDFWindow window;
    private ArrayList<WidgetTreeNode> widgets;

    public WindowTreeNode(WindowDefinitionFile wdf, WDFWindow window, Stbof res,
      ImageCache imgCache)
    {
        super(wdf, window, res, imgCache);
        this.stbof = res;
        this.window = window;

        widgets = new ArrayList<WidgetTreeNode>();

        WidgetTreeNode widgetNode;
        Enumeration<WDFWidget> enu;

        // windows
        enu = window.getWidgets(WDFWidget.TYPE_WINDOW);

        while (enu.hasMoreElements())
        {
            widgetNode = new WindowTreeNode(wdf, (WDFWindow)enu.nextElement(),
                    res, imgCache);
            widgets.add(widgetNode);
            add(widgetNode);
        }

        // buttons
        enu = window.getWidgets(WDFWidget.TYPE_BUTTON);

        while (enu.hasMoreElements())
        {
            widgetNode = new ButtonTreeNode(wdf, (WDFButton)enu.nextElement(),
                    res, imgCache);
            widgets.add(widgetNode);
            add(widgetNode);
        }

        // sliders
        enu = window.getWidgets(WDFWidget.TYPE_SLIDER);

        while (enu.hasMoreElements())
        {
            widgetNode = new SliderTreeNode(wdf, (WDFSlider)enu.nextElement(),
                    res, imgCache);
            widgets.add(widgetNode);
            add(widgetNode);
        }

        // scrollbars
        enu = window.getWidgets(WDFWidget.TYPE_SCROLLBAR);

        while (enu.hasMoreElements())
        {
            widgetNode = new ScrollBarTreeNode(wdf, (WDFScrollBar)enu.nextElement(),
                    res, imgCache);
            widgets.add(widgetNode);
            add(widgetNode);
        }

        // listboxes
        enu = window.getWidgets(WDFWidget.TYPE_LISTBOX);

        while (enu.hasMoreElements())
        {
            widgetNode = new ListBoxTreeNode(wdf, (WDFListBox)enu.nextElement(),
                    res, imgCache);
            widgets.add(widgetNode);
            add(widgetNode);
        }

        // pictureboxes
        enu = window.getWidgets(WDFWidget.TYPE_PICTUREBOX);

        while (enu.hasMoreElements())
        {
            widgetNode = new PictureBoxTreeNode(wdf, (WDFPictureBox)enu.nextElement(),
                    res, imgCache);
            widgets.add(widgetNode);
            add(widgetNode);
        }

        // textboxes
        enu = window.getWidgets(WDFWidget.TYPE_TEXTBOX);

        while (enu.hasMoreElements())
        {
            widgetNode = new TextBoxTreeNode(wdf, (WDFTextBox)enu.nextElement(),
                    res, imgCache);
            widgets.add(widgetNode);
            add(widgetNode);
        }

        // menus
        enu = window.getWidgets(WDFWidget.TYPE_MENU);

        while (enu.hasMoreElements())
        {
            widgetNode = new MenuTreeNode(wdf, (WDFMenu)enu.nextElement(), res,
                    imgCache);
            widgets.add(widgetNode);
            add(widgetNode);
        }
    }

    @Override
    public void paintOnImage(int x, int y, int w, int h, BufferedImage image)
    {
        if (this.visible)
        {
            Graphics gimg = image.getGraphics();
            Image img;

            x += window.getPosition().x;
            y += window.getPosition().y;
            w = Math.min(window.getDimension().width, w - window.getPosition().x);
            h = Math.min(window.getDimension().height, h - window.getPosition().y);

            if (this.backgroundVisible)
            {
                if (!window.getBackgroundImage().equalsIgnoreCase(WDFWidget.NO_IMAGE))
                {
                    img = imgCache.getImage(window.getBackgroundImage());
                    Tools.drawImage(img, x, y, w, h, gimg);
                }
                else
                {
                    gimg.setColor(Color.WHITE);
                    gimg.drawRect(x - 1, y - 1, w + 2, h + 2);
                }
            }

            for (int i = 0; i < widgets.size(); i++)
            {
                widgets.get(i).paintOnImage(x, y, w, h, image);
            }
        }
    }

    @Override
    public Rectangle getRectangle()
    {
        return new Rectangle(window.getPosition(), window.getDimension());
    }

    @Override
    public Component getTreeCellEditorComponent(boolean selected)
    {
        if (cellEditor == null)
        {
            cellEditor = new JPanel(new BorderLayout());
            cellEditor.setOpaque(false);

            display = new JCheckBox();
            display.setSelected(true);
            display.setOpaque(false);
            display.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent e) {
                    visible = display.isSelected();
                    fireChangeNotification();
                }
            });

            cellEditor.add(display, BorderLayout.WEST);

            lbl = new JLabel(this.getUserObject().toString());
            cellEditor.add(lbl, BorderLayout.CENTER);
        }

        if (selected)
        {
            lbl.setOpaque(true);
            lbl.setBackground(UIManager.getColor("Tree.selectionBackground"));
        }
        else
        {
            lbl.setOpaque(false);
        }

        lbl.setText(this.getUserObject().toString());

        return cellEditor;
    }

    @Override
    public void commitChanges() {
        Enumeration en = this.children();
        Object child;

        while (en.hasMoreElements())
        {
            child = en.nextElement();

            if (child instanceof WDFEditableTreeNode)
            {
                ((WDFEditableTreeNode)child).commitChanges();
            }
        }
    }
}
