/**
 * Ultimate Editor
 * Copyright (C) 2009 Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: CRC16.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created: 24.5.2011
 */

package ue.util;

/**
 * ANSI CRC
 */
public class CRC16
{
    private static final int POLINOMIAL = 0x8005;

    private int crc = 0;

    public void update(byte[] data)
    {
        update(data, 0, data.length);
    }

    public void update(byte[] data, int off, int len)
    {
        int bit;

        len = len + off;

        for (int i = off; i < len && i < data.length; i++)
        {
            for (int b = 7; b >= 0; b--)
            {
               bit = (data[i] >>> b) & 1;

                if ((crc & 0x8000) != 0)
                {
                    crc = ((crc << 1) + bit) ^ POLINOMIAL;
                }
                else
                {
                    crc = (crc << 1) + bit;
                }
            }
        }

        crc = crc & 0xFFFF;
    }

    public void reset()
    {
        crc = 0;
    }

    public int getValue()
    {
        return crc;
    }
}
