/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * File: Fletcher16.java
 *
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 *
 * Created on: 27.5.2011
 */

package ue.util;

/**
 * Attempt at fletcher16 implementation.
 */
public class Fletcher16
{
    private int sum1 = 0, sum2 = 0;

    public void update(byte[] data)
    {
        update(data, 0, data.length);
    }

    public void update(byte[] data, int off, int len)
    {
        len = off + len;

        for (int i = off; i < len && i < data.length; i++)
        {
            sum1 = (sum1 + data[i]) % 255;
            sum2 = (sum1 + sum2) % 255;
        }
    }

    public void reset()
    {
        sum1 = sum2 = 0;
    }

    public int getValue()
    {
        return (sum2 << 8) | sum1;
    }
}
