/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: GameMonitor.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.util;

import java.util.*;
import java.net.*;
import java.io.*;
import java.applet.*;
import javax.swing.*;

import ue.*;

// TODO: This class needs work. More comments would be welcome too.

/**
 * This class is used for monitoring network games.
 */
public class GameMonitor{
	// IPs of friends
	private ArrayList<String> IP = new ArrayList<String>();
	// Ports of friends
	private ArrayList<Integer> PORT = new ArrayList<Integer>();
	// Connections with friends: me to them
	private ArrayList<ConnectionThread> CONNECTION = new ArrayList<ConnectionThread>();
	// Connections with friends: them to me	
	private ArrayList<ServerConnectionThread> SERVERCON = new ArrayList<ServerConnectionThread>();
	// Use this to run botf and monitor if the game is running
	private Process RUNNER = null;
	// Has the game been killed?
	private boolean KILLED = false;
	// The ServerSocket used to listen for connections
	private ServerSocket SERVER = null;
	// Write messages to this JTextArea
	// TODO: This is not good, use listeners instead
	private JTextArea INFO;
	
	private int SETTING = -1;
	// strings
	private static String errPORT = Language.getString("GameMonitor.0"); //$NON-NLS-1$
	private static String strREASON = Language.getString("GameMonitor.1"); //$NON-NLS-1$
	private static String strPORT = Language.getString("GameMonitor.2"); //$NON-NLS-1$
	
	/**
	 * Constructor
	 * @param inf the JTextArea to write messeges too
	 */
	public GameMonitor(JTextArea inf){
		INFO = inf;
	}
	
	/**
	 * Add a peer to the list.
	 * @param ip the peers ip
	 * @param port the peers port
	 */
	public void addPeer(String ip, int port){
		IP.add(ip);
		PORT.add(port);
	}
	/**
	 * Removes a peer from the list.
	 * @param index the index of the peer 
	 */
	public void removePeer(int index){
		IP.remove(index);
		PORT.remove(index);
	}
	/**
	 * Sets a peer on the list.
	 * @param ip the peers ip
	 * @param port the peers port
	 * @param index peers index
	 */
	public void setPeer(String ip, int port, int index){
		IP.set(index, ip);
		PORT.set(index, port);
	}
	/**
	 * Opens port to listen.
	 * @param port the port to listen on
	 */
	public void openPort(int port){
		if (SERVER!=null){
			try{
				INFO.setText(INFO.getText() + Language.getString("GameMonitor.3") + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
				for (int i=0; i<SERVERCON.size(); i++){
					ServerConnectionThread scon = SERVERCON.get(i);
					scon.end();
				}
				SERVER.close();
			}catch (IOException er){
				er.printStackTrace();
				INFO.setText(INFO.getText() + er.getMessage() + "\n"); //$NON-NLS-1$
			}
		}
		try{
			SERVER = new ServerSocket(port, 5);
		}catch (IOException er){
			er.printStackTrace();
			INFO.setText(INFO.getText() + errPORT.replace("%1", Integer.toString(port)) + "\n" //$NON-NLS-1$ //$NON-NLS-2$
					+ strREASON.replace("%1", er.getMessage()) + "\n");			 //$NON-NLS-1$ //$NON-NLS-2$
			return;
		}
		ServerConnectionThread thr = new ServerConnectionThread();
		thr.setPriority(Thread.MIN_PRIORITY);
		thr.setDaemon(true);
		thr.start();
		INFO.setText(INFO.getText() + strPORT.replace("%1", Integer.toString(port)) + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	/**Connects to peers.*/
	public void connect(){
		disconnect();
		INFO.setText(INFO.getText() + Language.getString("GameMonitor.4") + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
		for (int i=0; i<IP.size(); i++){
			ConnectionThread con = new ConnectionThread(IP.get(i), PORT.get(i));
			con.setPriority(Thread.MIN_PRIORITY);
			con.setDaemon(true);
			con.start();
			CONNECTION.add(con);
		}
	}
	/**disConnects from peers.*/
	public void disconnect(){
		for (int i=0; i<CONNECTION.size(); i++){
			if (i==0) {
				INFO.setText(INFO.getText() + Language.getString("GameMonitor.5") + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
			}
			ConnectionThread con = CONNECTION.get(i);
			con.disconnect();
			con.end();
			CONNECTION.remove(con);
			if (i==CONNECTION.size()-1) {
				INFO.setText(INFO.getText() + Language.getString("GameMonitor.6") + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
	}
	/**disConnects from peers.*/
	public void disconnectAll(){
		disconnect();
		if (SERVER!=null){
			try{
				INFO.setText(INFO.getText() + Language.getString("GameMonitor.7") + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
				for (int i=0; i<SERVERCON.size(); i++){
					ServerConnectionThread scon = SERVERCON.get(i);
					scon.end();
				}
				SERVER.close();
			}catch (IOException er){
				er.printStackTrace();
				INFO.setText(INFO.getText() + er.getMessage() + "\n"); //$NON-NLS-1$
			}
		}
	}
	/**reports crash.*/
	public void reportCrash(){
		for (int i=0; i<CONNECTION.size(); i++){
			ConnectionThread con = CONNECTION.get(i);
			con.reportCrash();
		}
	}
	
	private void killBotf(){
		if (RUNNER==null){
			INFO.setText(INFO.getText() + Language.getString("GameMonitor.8") + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
			return;
		}
		KILLED = true;
		RUNNER.destroy();
		RUNNER = null;
		INFO.setText(INFO.getText() + Language.getString("GameMonitor.9") + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	public void playAlert(){
		try{
			URL url = UE.class.getResource("alert.wav"); //$NON-NLS-1$
			AudioClip au = Applet.newAudioClip(url);
			au.play();
		}catch (Exception re){
			re.printStackTrace();
		}
		INFO.setText(INFO.getText() + Language.getString("GameMonitor.10") + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	public void runAndMonitor(){
		RunAndMonitorThread con = new RunAndMonitorThread();
		con.setPriority(Thread.MIN_PRIORITY);
		con.setDaemon(true);
		con.start();
	}
	
	public String getLocalAddress(){
		try{
			String str = ""; //$NON-NLS-1$
			Enumeration<NetworkInterface> netInterfaces=NetworkInterface.getNetworkInterfaces();
			InetAddress ip;
			while(netInterfaces.hasMoreElements()){
				NetworkInterface ni=(NetworkInterface)netInterfaces.nextElement();
				ip=ni.getInetAddresses().nextElement();
				if(!ip.isLoopbackAddress() && ip.getHostAddress().indexOf(":")==-1){ //$NON-NLS-1$
					if (str.length()==0) {
						str = str + ip;
					} else {
						str = str + " " + Language.getString("GameMonitor.11") + " " + ip; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					}
				}else{
					ip=null;
				}
			}
			if (str.length()==0) {
				str = Language.getString("GameMonitor.12"); //$NON-NLS-1$
			}
			return str;
		}catch (Exception ze){
			ze.printStackTrace();
		}
		return Language.getString("GameMonitor.12"); //$NON-NLS-1$
	}
	
	public String[] getPeerList(){
		String[] str = new String[IP.size()];
		for (int i=0; i<IP.size(); i++){
			str[i] = IP.get(i) + ":" + PORT.get(i); //$NON-NLS-1$
		}
		return str;
	}
	
	public void setAction(int i){
		SETTING = i;
	}
	private class RunAndMonitorThread extends Thread{
		@Override
		public void run(){
			if (RUNNER!=null) {
				return;
			}
			Runtime rt = null;
			try{
				rt = Runtime.getRuntime();
				KILLED = false;
				RUNNER = rt.exec(UE.SETTINGS.getProperty(SettingsManager.EXE));
				RUNNER.waitFor();
				if (KILLED){
					return;
				}
				String str = Language.getString("GameMonitor.13"); //$NON-NLS-1$
				str = str.replace("%1", Integer.toString(RUNNER.exitValue())); //$NON-NLS-1$
				INFO.setText(INFO.getText() + str + "\n"); //$NON-NLS-1$
				reportCrash();
				RUNNER = null;
			}catch(Exception e){
				RUNNER = null;
				e.printStackTrace();
				INFO.setText(INFO.getText() + e.getMessage() + "\n"); //$NON-NLS-1$
				return;
			}
		}
	}
	private class ServerConnectionThread extends Thread{
		private boolean END = false;
		public void end(){
			END = true;
		}
		@Override
		public void run(){
			//accept
			Socket sct = null;
			SERVERCON.add(this);
			try{
				sct = SERVER.accept();
			}catch (IOException er){
				if (END) {
					return;
				}
				er.printStackTrace();
				INFO.setText(INFO.getText() + Language.getString("GameMonitor.14") + "\n" + strREASON.replace("%1", er.getMessage()) + "\n");			 //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
				return;
			}
			InetAddress addy = sct.getInetAddress();
			//ready
			ServerConnectionThread thr = new ServerConnectionThread();
			thr.setPriority(Thread.MIN_PRIORITY);
			thr.setDaemon(true);
			thr.start();
			//get stream & wait for input
			String str = Language.getString("GameMonitor.15"); //$NON-NLS-1$
			INFO.setText(INFO.getText() + str.replace("%1", addy.toString()) + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
			while (!END && !sct.isClosed()){
				try{
					InputStream input = null;
					try{
						input = sct.getInputStream();
					}catch (IOException er){
						er.printStackTrace();
						str = Language.getString("GameMonitor.16"); //$NON-NLS-1$
						str = str.replace("%1", addy.toString()); //$NON-NLS-1$
						INFO.setText(INFO.getText() + str.replace("%1", er.getMessage()) + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
					}
					if (input==null){
						try{
							sct.close();
						}catch (IOException er){
							er.printStackTrace();
							INFO.setText(INFO.getText() + er.getMessage() + "\n"); //$NON-NLS-1$
						}
						END = true;
						return;
					}
					if (input.available()>0){
						int h = input.read();
						//remote crash occured
						if (h==1){
							INFO.setText(INFO.getText() + Language.getString("GameMonitor.17") + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
							switch (SETTING){
								case 1:{
									killBotf();
									break;
								}
								case 2:{
									playAlert();
									break;
								}
								case 3:{
									playAlert();
									killBotf();
									break;
								}
							};
						}
						//disconect
						if (h==2){
							str = Language.getString("GameMonitor.18"); //$NON-NLS-1$
							INFO.setText(INFO.getText() + str.replace("%1", addy.toString()) + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
							sct.close();
							SERVERCON.remove(this);
							END=true;
						}
					}
				}catch (IOException er){
					er.printStackTrace();
					INFO.setText(INFO.getText() + er.getMessage() + "\n"); //$NON-NLS-1$
					return;
				}
			}
		}
	}
	private class ConnectionThread extends Thread{
		private boolean END = false;
		private String HOST;
		private int PORT;
		private boolean crash = false;
		private boolean disconect = false;
		public void end(){
			END = true;
		}
		public void reportCrash(){
			crash = true;
		}
		public void disconnect(){
			disconect = true;
		}
		public ConnectionThread(String host, int port){
			HOST = host;
			PORT = port;
		}
		@Override
		public void run(){
			//connect
			int count=0;
			Socket sct = null;
			while (sct==null && count<3){
				try{
					sct = new Socket(InetAddress.getByName(HOST), PORT);
				}catch (Exception er){
					if (END) {
						return;
					}
					er.printStackTrace();
					INFO.setText(INFO.getText() + er.getMessage() + "\n"); //$NON-NLS-1$
				}
				count++;
			}
			String str;
			if (sct==null){
				str = Language.getString("GameMonitor.19"); //$NON-NLS-1$
				INFO.setText(INFO.getText() + str.replace("%1", HOST) + "\n" + Language.getString("GameMonitor.20") + "\n"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
				END=true;
				return;
			}
			InetAddress addy = sct.getInetAddress();
			//get stream
			OutputStream output = null;
			try{
				output = sct.getOutputStream();
			}catch (IOException er){
				er.printStackTrace();
				INFO.setText(INFO.getText() + Language.getString("GameMonitor.21") + "\n" + er.getMessage() + "\n"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			}
			if (output==null){
				try{
					sct.close();
				}catch (IOException er){
					er.printStackTrace();
					INFO.setText(INFO.getText() + er.getMessage() + "\n"); //$NON-NLS-1$
				}
				END = true;
				return;
			}
			//wait for order
			str = Language.getString("GameMonitor.22"); //$NON-NLS-1$
			INFO.setText(INFO.getText() + str.replace("%1", addy.toString()) + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
			while (!END && !sct.isClosed()){
				try{
					if (crash){
						output.write(1);
						crash=false;
					}
					if (disconect){
						output.write(2);
						END = true;
						return;
					}
				}catch (IOException er){
					er.printStackTrace();
					INFO.setText(INFO.getText() + er.getMessage() + "\n"); //$NON-NLS-1$
					return;
				}
			}
			str = Language.getString("GameMonitor.23"); //$NON-NLS-1$
			INFO.setText(str.replace("%1", HOST) + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
}
