/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: GenericFile.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.util;
//standard
import java.util.*;
import java.io.*;
//user defined
import ue.Language;
import ue.edit.*;

/**This class is used to encapsulate files.
It is used by stbof to add modified animations to it and elsewhere too.*/
public class GenericFile extends InternalFile{
	protected byte[] data;
	// strings
	private static String errEMPTY = Language.getString("GenericFile.0"); //$NON-NLS-1$
	private static String errNAME = Language.getString("GenericFile.1"); //$NON-NLS-1$
	
	/**Creates an empty generic file with spefied name.
	@param n the name of the file*/
	public GenericFile(String n){
		setName(n); data = new byte[0];
	}
	/**Creates a generic file with spefied name and data.
	@param n the name of the file
	@param b file data*/
	public GenericFile(String n, byte[] b){
		setName(n); data = b;
	}
	/**Returns the file data.*/
	public byte[] getData() { return data;}
	/**Returns the size of the file's data.*/
	@Override
	public long getSize() { return data.length;}
	/**Sets the data.
	@param b the new data for this generic file*/
	public void setData(byte[] b){
		if (b==null) {
			return;
		}
		data = b;
                this.markChanged();
	}
	/**Tests if the files are equal by comparing their filenames.
	@param o the generic file to be compared with this one
	@return true if files are equal*/
	@Override
	public boolean equals(Object o)
    {
        if (o instanceof InternalFile)
        {
    		InternalFile gu = (InternalFile)o;
        	return getName().equals(gu.getName());
        }

        return false;
	}
	/**Returns the file's hashcode.*/
	@Override
	public int hashCode(){
		return getName().hashCode();
	}
	/**This function checks the file for known problems/errors that may occur while editing the file.
	@return a Vector object containing results of the check as Strings.*/
	@Override
	public Vector<String> check(){
		Vector<String> response = new Vector<String>();
		if (data.length<1) {
			response.add(errEMPTY.replace("%1", getName())); //$NON-NLS-1$
		}
		if (getName().length()<1) {
			response.add(errNAME.replace("%1", getName())); //$NON-NLS-1$
		}
		return response;
	}
	/**Saves changes.
	@return true if save has been successful.
	@throws Exception the function may throw an exception if an error occures.*/
	@Override
	public void save(OutputStream out) throws Exception{
        out.write(data);
	}
}