/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: GifColorMap.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.util;

import java.io.*;

import ue.Language;
import ue.exception.*;

/**A GIF color map.*/
public class GifColorMap{
	private byte[] PIXELS = new byte[1024];
	private int valid;
	
	public GifColorMap(InputStream in, int pixels, boolean alpha) throws IOException{
		if (pixels>256) {
			throw new IOException(Language.getString("GifColorMap.0")); //$NON-NLS-1$
		}
		valid = pixels;
		byte[] b = Tools.toByte((short)-1, true);

		if (alpha){
			byte[] red = new byte[pixels*4];
			in.read(red);
			System.arraycopy(red, 0, PIXELS, 0, pixels*4);
		}else{
			byte[] red = new byte[pixels*3];
			in.read(red);
			
			for (int i=0; i<pixels; i++){
				PIXELS[i*4] = red[i*3];
				PIXELS[i*4+1] = red[i*3+1];
				PIXELS[i*4+2] = red[i*3+2];
				if (red[i*3]==0 && red[i*3+1]==0 && red[i*3+2]==0){
					PIXELS[i*4+3] = 0; //transparent
				}else{
					PIXELS[i*4+3] = b[0]; //opaque
				}
			}
		}
		for (int i=valid; i<256; i++){
			PIXELS[i*4+3] = b[0]; //opaque
		}
	}

	public GifColorMap(GifColorMap map){
		System.arraycopy(map.PIXELS, 0, PIXELS, 0, 1024);
		valid = map.valid;
	}

	public void save(OutputStream out, boolean alpha) throws IOException{
		if (alpha){
			out.write(PIXELS);
		}else{
			for (int i=0; i<256; i++){
				out.write(PIXELS[i*4]);
				out.write(PIXELS[i*4+1]);
				out.write(PIXELS[i*4+2]);
			}
		}
	}
	public byte[] getMapData(){
		byte[] b = new byte[1024];
		System.arraycopy(PIXELS, 0, b, 0, 1024);
		return b;
	}

	public int[] getMapPixels(){
		byte[] b = new byte[4];
		int[] ret = new int[256];		

		int j = 0;
		for (int i=0; i<256; i++){
			b[1] = PIXELS[j++]; //red
			b[2] = PIXELS[j++]; //green
			b[3] = PIXELS[j++]; //blue
			b[0] = PIXELS[j++]; //alpha

			ret[i] = Tools.toInt(b, false);
		}

		return ret;
	}

	public int getPixel(int index) throws InvalidArgumentsException{
		if (index<0 || index >=256) {
			throw new InvalidArgumentsException(Language.getString("GifColorMap.1")); //$NON-NLS-1$
		}
		byte[] b = new byte[4];

		int j = index*4;
		b[1] = PIXELS[j++]; //red
		b[2] = PIXELS[j++]; //green
		b[3] = PIXELS[j++]; //blue
		b[0] = PIXELS[j++]; //alpha

		return Tools.toInt(b, false);
	}

	public byte[] getPixelBytes(int index) throws InvalidArgumentsException{
		if (index<0 || index >=256) {
			throw new InvalidArgumentsException(Language.getString("GifColorMap.1")); //$NON-NLS-1$
		}
		byte[] b = new byte[4];

		int j = index*4;
		b[1] = PIXELS[j++]; //red
		b[2] = PIXELS[j++]; //green
		b[3] = PIXELS[j++]; //blue
		b[0] = PIXELS[j++]; //alpha

		return b;
	}

	public void setPixelBytes(int index, byte[] b) throws InvalidArgumentsException{
		if (index<0 || index >=256) {
			throw new InvalidArgumentsException(Language.getString("GifColorMap.1")); //$NON-NLS-1$
		}

		PIXELS[4*index] = b[1];
		PIXELS[4*index+1] = b[2];
		PIXELS[4*index+2] = b[3];
		PIXELS[4*index+3] = b[0];
	}

	public void setPixel(int index, int pixel) throws InvalidArgumentsException{
		if (index<0 || index >=256) {
			throw new InvalidArgumentsException(Language.getString("GifColorMap.1")); //$NON-NLS-1$
		}
		byte[] b = Tools.toByte(pixel, false);

		PIXELS[4*index] = b[1];
		PIXELS[4*index+1] = b[2];
		PIXELS[4*index+2] = b[3];
		PIXELS[4*index+3] = b[0];
	}

	@Override
	public boolean equals (Object obj){
		if (obj==null) {
			return false;
		}
		if (obj instanceof GifColorMap){
			GifColorMap a = (GifColorMap)obj;
			//int z;
			//if (valid < a.valid) z = valid; else z = a.valid;
			byte[] b = a.getMapData();
			for (int i=0; i<1024; i++){
				if (b[i]!=PIXELS[i]) {
					return false;
				}
			}
			return true;
		}else{
			return false;
		}
	}

	@Override
	public int hashCode(){
		int ret = valid;
		for (int i=0; i<1024; i++) {
			ret +=PIXELS[i];
		}
		return ret;
	}
}
