/*
Ultimate Editor for Birth Of The Federation 1.02
Copyright (C) 2005-2007  Alan Podlesek

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


File: GifImage.java

Written by: Alan Podlesek <seqdcer@yahoo.com>

Last modified: 6.9.2007
 */
package ue.util;

import java.io.*;
import java.awt.*;
import java.awt.image.*;
import javax.imageio.*;

import ue.Language;
import ue.exception.*;

/**
 * This class is used for loading gif images. Not all kinds of gif's are supported.
 * The variables that are static and final have set values with which the gif being
 * loaded must comply.
 */
public class GifImage {
    //HEADER
    private final static String SIGNATURE = "GIF";		// 3 bytes (ASCII) //$NON-NLS-1$
    
    private String VERSION;					            // 3 bytes (ASCII)

    //LOGICAL SCREEN DESCRIPTOR
    
    private int LOGICAL_SCREEN_WIDTH;				    // 2 bytes (unsigned short)
    private int LOGICAL_SCREEN_HEIGHT;				    // 2 bytes (unsigned short)
    final private static boolean GLOBAL_COLOR_TABLE_FLAG = true;			// 1 bit   (boolean)
    private byte COLOR_RESOLUTION;				        // 3 bit
    final private static boolean SORT_FLAG = false;		// 1 bit   (boolean)
    private byte GLOBAL_COLOR_TABLE_SIZE;			    // 3 bit
    private byte BACKGROUND_COLOR_INDEX;				    // 1 byte  (byte)
    final private static byte PIXEL_ASPECT_RATIO = 0x00;		// 1 byte  (byte)
    //GLOBAL COLOR TABLE
    private GifColorMap initialColorMap;
    //IMAGE DESCRIPTOR
    private byte IMAGE_SEPARATOR;							// 1 byte  (byte)
    private int IMAGE_LEFT_POSITION;				        // 2 bytes (unsigned short)
    private int IMAGE_TOP_POSITION;				            // 2 bytes (unsigned short)
    private int IMAGE_WIDTH;					            // 2 bytes (unsigned short)
    private int IMAGE_HEIGHT;					            // 2 bytes (unsigned short)
    final private boolean LOCAL_COLOR_TABLE_FLAG = false;	// 1 bit   (boolean)
    private boolean LOCAL_INTERLACE_FLAG;				    // 1 bit   (boolean)
    final private boolean LOCAL_SORT_FLAG = false;	// 1 bit   (boolean)
    final private byte LOCAL_RESERVED = 0x00;		// 2 bits
    @SuppressWarnings("unused")  //$NON-NLS-1$
    //LOCAL COLOR TABLE - not allowed
    //THE REST OF THE FILE
    private byte[] LZW_SHIT;					            // unknown
    // strings
    private static String errNOTGIF = Language.getString("GifImage.0"); //$NON-NLS-1$
    private static String errVERSION = Language.getString("GifImage.1"); //$NON-NLS-1$
    private GifColorMap currentColorMap;
    @SuppressWarnings("unused")  //$NON-NLS-1$
    private DirectColorModel cm = new DirectColorModel(32, 0xFF0000, 0xFF00, 0xFF, 0xFF000000);

    /*Constructor.*/
    public GifImage(InputStream in) throws UnsupportedImage, IOException {
        /*
        FIXED VALUE means UE will only accept gif's that have only that one allowed value in a variable.
        Try to figure out what gif files botf allows and filter out those it doesn't.
         */
        //HEADER
        byte[] b = new byte[6];
        in.read(b); // <-----------------------------------------READ 6 BYTES
        //SIGNATURE
        if (!Tools.toASCII(b, 0, 3).equals(SIGNATURE)) {
            throw new UnsupportedImage(errNOTGIF.replace("%1", Tools.toASCII(b))); //$NON-NLS-1$
        }
        //VERSION
        VERSION = Tools.toASCII(b, 3, 3);
        if (!VERSION.equals("89a") && !VERSION.equals("87a")) //$NON-NLS-1$ //$NON-NLS-2$
        {
            throw new UnsupportedImage(errVERSION.replace("%1", Tools.toASCII(b))); //$NON-NLS-1$
        }
        //LOGICAL SCREEN DESCRIPTOR
        b = new byte[2];

        //LOGICAL_SCREEN_WIDTH
        in.read(b); // <-----------------------------------------READ 2 BYTES
        LOGICAL_SCREEN_WIDTH = Tools.toShort(b, true);
        if (LOGICAL_SCREEN_WIDTH < 0) {
            LOGICAL_SCREEN_WIDTH = 65535 + LOGICAL_SCREEN_WIDTH;
        }
        //LOGICAL_SCREEN_HEIGHT
        in.read(b); // <-----------------------------------------READ 2 BYTES
        LOGICAL_SCREEN_HEIGHT = Tools.toShort(b, true);
        if (LOGICAL_SCREEN_HEIGHT < 0) {
            LOGICAL_SCREEN_HEIGHT = 65535 + LOGICAL_SCREEN_HEIGHT;
        }
        //GLOBAL_COLOR_TABLE_FLAG
        b = new byte[1];
        in.read(b); // <-----------------------------------------READ 1 BYTE
        if (GLOBAL_COLOR_TABLE_FLAG != ((0x80 & b[0]) == 0x80)) {
            throw new UnsupportedImage(Language.getString("GifImage.2")); //$NON-NLS-1$
            }
        //COLOR_RESOLUTION
            /*
         * You get COLOR_RESOLUTION -1 from file.
         * It tells you how many bits are used by a basic color (red, green or blue)
         */
        COLOR_RESOLUTION = (byte) (((b[0] >> 4) & 0x07) + 1);
        //SORT_FLAG
        // ignored
        //if (SORT_FLAG != ((0x08 & b[0]) == 0x08))
        //	throw new UnsupportedImage("Unsupported gif: global color table is sorted.");
        //global color table size
        GLOBAL_COLOR_TABLE_SIZE = (byte) (0x07 & b[0]);
        //background color
        in.read(b); // <-----------------------------------------READ 1 BYTE
        BACKGROUND_COLOR_INDEX = b[0];
        //pixel aspect ratio
        in.read(b); // <-----------------------------------------READ 1 BYTE
        if (PIXEL_ASPECT_RATIO != b[0]) {
            throw new UnsupportedImage(Language.getString("GifImage.3")); //$NON-NLS-1$
            }
        // GLOBAL COLOR TABLE
        int size = (int) Math.pow(2, (GLOBAL_COLOR_TABLE_SIZE + 1));
        currentColorMap = initialColorMap = new GifColorMap(in, size, false);
        // IMAGE DESCRIPTOR
        // IMAGE_SEPARATOR
        in.read(b); // <-----------------------------------------READ 1 BYTE
        IMAGE_SEPARATOR = b[0];
        //IMAGE_LEFT_POSITION
        b = new byte[2];
        in.read(b); // <-----------------------------------------READ 2 BYTES
        IMAGE_LEFT_POSITION = Tools.toShort(b, true);
        if (IMAGE_LEFT_POSITION < 0) {
            IMAGE_LEFT_POSITION = 65535 + IMAGE_LEFT_POSITION;
        }
        //IMAGE_TOP_POSITION
        in.read(b); // <-----------------------------------------READ 2 BYTES
        IMAGE_TOP_POSITION = Tools.toShort(b, true);
        if (IMAGE_TOP_POSITION < 0) {
            IMAGE_TOP_POSITION = 65535 + IMAGE_TOP_POSITION;
        }
        //IMAGE_WIDTH
        in.read(b); // <-----------------------------------------READ 2 BYTES
        IMAGE_WIDTH = Tools.toShort(b, true);
        if (IMAGE_WIDTH < 0) {
            IMAGE_WIDTH = 65535 + IMAGE_WIDTH;
        }
        //IMAGE_HEIGHT
        in.read(b); // <-----------------------------------------READ 2 BYTES
        IMAGE_HEIGHT = Tools.toShort(b, true);
        if (IMAGE_HEIGHT < 0) {
            IMAGE_HEIGHT = 65535 + IMAGE_HEIGHT;
        }
        //LOCAL_COLOR_TABLE_FLAG
        b = new byte[1];
        in.read(b); // <-----------------------------------------READ 1 BYTE
        if (LOCAL_COLOR_TABLE_FLAG != ((0x80 & b[0]) == 0x80)) {
            throw new UnsupportedImage(Language.getString("GifImage.4")); //$NON-NLS-1$
        }
        //LOCAL_INTERLACE_FLAG - oh boy
        LOCAL_INTERLACE_FLAG = ((0x40 & b[0]) == 0x40);
        //LOCAL_SORT_FLAG
        // don't care
        //LOCAL_RESERVED
        if (LOCAL_RESERVED != (byte) ((b[0] >> 3) & 0x03)) {
            throw new UnsupportedImage(Language.getString("GifImage.5")); //$NON-NLS-1$
        }
        //NOW COMES LZW
        LZW_SHIT = new byte[in.available()];
        in.read(LZW_SHIT);
    }
    /*Saves this gif.*/

    public void save(OutputStream out) throws IOException {
        //HEADER
        //SIGNATURE
        out.write(Tools.toByte(SIGNATURE, SIGNATURE.length(), SIGNATURE.length()));
        //VERSION
        out.write(Tools.toByte(VERSION, VERSION.length(), VERSION.length()));
        //LOGICAL SCREEN DESCRIPTOR
        //LOGICAL_SCREEN_WIDTH
        byte[] b = Tools.toByte(LOGICAL_SCREEN_WIDTH, true);
        out.write(b[0]);
        out.write(b[1]);
        //LOGICAL_SCREEN_HEIGHT
        b = Tools.toByte(LOGICAL_SCREEN_HEIGHT, true);
        out.write(b[0]);
        out.write(b[1]);
        //GLOBAL_COLOR_TABLE_FLAG
        b = new byte[1];
        b[0] = (byte) 0x80;
        //COLOR_RESOLUTION
        b[0] = (byte) (b[0] | ((COLOR_RESOLUTION - 1) << 4));
        //SORT_FLAG
        if (SORT_FLAG) {
            b[0] = (byte) (b[0] & 0x08);
        }
        //GLOBAL_COLOR_TABLE_SIZE
        b[0] = (byte) (b[0] | 0x07);
        out.write(b);
        //BACKGROUND_COLOR_INDEX
        out.write(BACKGROUND_COLOR_INDEX);
        //PIXEL_ASPECT_RATIO
        out.write(PIXEL_ASPECT_RATIO);
        //GLOBAL COLOR TABLE
        currentColorMap.save(out, false);
        //IMAGE DESCRIPTOR
        //IMAGE_SEPARATOR
        out.write(IMAGE_SEPARATOR);
        //IMAGE_LEFT_POSITION
        b = Tools.toByte(IMAGE_LEFT_POSITION, true);
        out.write(b[0]);
        out.write(b[1]);
        //IMAGE_TOP_POSITION
        b = Tools.toByte(IMAGE_TOP_POSITION, true);
        out.write(b[0]);
        out.write(b[1]);
        //IMAGE_WIDTH
        b = Tools.toByte(IMAGE_WIDTH, true);
        out.write(b[0]);
        out.write(b[1]);
        //IMAGE_HEIGHT
        b = Tools.toByte(IMAGE_HEIGHT, true);
        out.write(b[0]);
        out.write(b[1]);
        //LOCAL_COLOR_TABLE_FLAG
        b = new byte[1];
        //is false anyhow
        //LOCAL_INTERLACE_FLAG - oh boy
        if (LOCAL_INTERLACE_FLAG) {
            b[0] = (byte) (0x40 | b[0]);
        }
        //LOCAL_SORT_FLAG
        if (LOCAL_SORT_FLAG) {
            b[0] = (byte) (0x20 | b[0]);
        }
        //LOCAL_RESERVED
        // zeros
        //LOCAL_COLOR_TABLE_SIZE
        // zero
        out.write(b);
        //NOW COMES THE LZW_SHIT
        out.write(LZW_SHIT);
    }

    /**Sets GifColorMap to use when rendering the image.*/
    public void setGifColorMap(GifColorMap map) {
        currentColorMap = map;
    }

    /**Returns the GifColorMap contained in the gif.*/
    public GifColorMap getInitialGifColorMap() {
        return initialColorMap;
    }

    /**Returns image.*/
    public Image getImage() throws Exception {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        save(baos);
        baos.close();
        ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
        return ImageIO.read(bais);
    }

    /**Returns size.*/
    public Dimension getSize() {
        return new Dimension(LOGICAL_SCREEN_WIDTH, LOGICAL_SCREEN_HEIGHT);
    }
}
