/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: RegQuery.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.util;

import java.io.*;

/**This class is used to read the registry.
It calls the windows program "reg" and reads it's output.*/
public class RegQuery
{
    private static final String REGQUERY_UTIL = "reg query "; //$NON-NLS-1$
    private static final String REGSTR_TOKEN = "REG_SZ"; //$NON-NLS-1$
    
    private static final String BOTF_INSTALL_PATH = REGQUERY_UTIL +
    "\"HKLM\\SOFTWARE\\Microprose\\Star Trek: BOTF\" /v InstallPath"; //$NON-NLS-1$
    private static final String BOTF_VERSION_PATH = REGQUERY_UTIL +
    "\"HKLM\\SOFTWARE\\Microprose\\Star Trek: BOTF\" /v Version"; //$NON-NLS-1$
    
    /**
     * Returns BOTF installation path as writen in "HKLM\\SOFTWARE\\Microprose\\Star Trek: BOTF\\InstallPath".
     */
    public static String getBOTFInstallPath(boolean wine)
    {
        if (wine)
        {
            String result = null;
            
            File registry = new File(System.getProperty("user.home") + File.separator + ".wine" + File.separator + "system.reg");
            
            if (registry.exists())
            {
                DataInputStream in = null;
                
                try
                {
                    String line;
                    
                    in = new DataInputStream(new FileInputStream(registry));
                    BufferedReader br = new BufferedReader(new InputStreamReader(in));
                    
                    while (result == null && in.available() > 0)
                    {
                        line = br.readLine();
                        
                        if (line.contains("Star Trek: BOTF"))
                        {
                            while (in.available() > 0)
                            {
                                line = br.readLine();
                                
                                if (line.contains("InstallPath"))
                                {
                                    if (line.contains("="))
                                    {
                                        line = line.substring(line.indexOf("=") + 1, line.length() - 1);
                                    }
                                    
                                    result = line.replaceAll("\"", "");
                                    result = result.replaceAll("\\\\", File.separator);
                                    result = result.replaceAll("//", File.separator);
                                    result = result.substring(0, 1).toLowerCase() + result.substring(1);
                                    break;
                                }
                            }
                        }
                    }
                    
                    in.close();
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                    if (in != null)
                    {
                        try
                        {
                            in.close();
                        }
                        catch (IOException e1)
                        {
                        }
                    }
                }
            }
            
            return result;
        }
        else
        {
            try
            {
                Process process = Runtime.getRuntime().exec(BOTF_INSTALL_PATH);
                StreamReader reader = new StreamReader(process.getInputStream());
                
                reader.start();
                process.waitFor();
                reader.join();
                
                String result = reader.getResult();
                // System.out.println(result);
                
                int p = result.indexOf(REGSTR_TOKEN);
                
                if (p == -1)
                {
                    return null;
                }
                
                return result.substring(p + REGSTR_TOKEN.length()).trim();
            }
            catch (Exception e)
            {
                System.out.println(e.getMessage());
                return null;
            }
        }
    }
    
    /**
     * Returns BOTF version as writen in "HKLM\\SOFTWARE\\Microprose\\Star Trek: BOTF\\Version".
     */
    public static String getBOTFVersion()
    {
        try
        {
            Process process = Runtime.getRuntime().exec(BOTF_VERSION_PATH);
            StreamReader reader = new StreamReader(process.getInputStream());
            
            reader.start();
            process.waitFor();
            reader.join();
            
            String result = reader.getResult();
            int p = result.indexOf(REGSTR_TOKEN);
            
            if (p == -1)
            {
                return null;
            }
            
            return result.substring(p + REGSTR_TOKEN.length()).trim();
        }
        catch (Exception e)
        {
            System.out.println(e.getMessage());
            return null;
        }
    }
    
    private static class StreamReader extends Thread
    {
        private InputStream is;
        private StringWriter sw;
        
        StreamReader(InputStream is)
        {
            this.is = is;
            sw = new StringWriter();
        }
        
        @Override
        public void run()
        {
            try
            {
                int c;
                while ((c = is.read()) != -1)
                {
                    sw.write(c);
                }
            }
            catch (IOException e) {}
        }
        
        String getResult()
        {
            return sw.toString();
        }
    }
}