/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: SettingsManager.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.util;

//import list
import javax.swing.*;
import javax.swing.plaf.FontUIResource;

import java.awt.Font;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.*;
import java.util.*;

import ue.*;
import ue.edit.FileManager;
/**
Manages Ultimate Editor's settings.
Settings are loaded from Settings.ini
*/
public class SettingsManager
{
    /*All settings here are global, therofore static.*/
    //Default properties
    private Properties DEFAULTS;
    //real ones
    private Properties SETTINGS;
    // listeners
    private ArrayList<PropertyChangeListener> listeners = new ArrayList<PropertyChangeListener>();
    // settings file
    private File fileSETTINGS;
    /**
     * PROPERTY FIUELDS
     */
    public static final String STBOF = "stbof"; //$NON-NLS-1$
    public static final String ALT = "alt"; //$NON-NLS-1$
    public static final String ENGLISH = "english"; //$NON-NLS-1$
    public static final String SFX = "sfx"; //$NON-NLS-1$
    public static final String MUSIC = "music"; //$NON-NLS-1$
    public static final String EXE = "exe"; //$NON-NLS-1$
    public static final String LAST = "last"; //$NON-NLS-1$
    public static final String RECENT = "recent"; //$NON-NLS-1$
    public static final String AUTO_STBOF = "autoStbof"; //$NON-NLS-1$
    public static final String AUTO_ALT = "autoAlt"; //$NON-NLS-1$
    public static final String AUTO_ENGLISH = "autoEnglish"; //$NON-NLS-1$
    public static final String AUTO_EXE = "autoExe"; //$NON-NLS-1$
    public static final String AUTO_SFX = "autoSfx"; //$NON-NLS-1$
    public static final String AUTO_MUSIC = "autoMusic"; //$NON-NLS-1$
    public static final String STORE_SND_DESC = "saveSndDescriptionToUserFolder"; //$NON-NLS-1$
    public static final String LANGUAGE = "language"; //$NON-NLS-1$
    public static final String LOG_FILE = "logFile"; //$NON-NLS-1$
    public static final String FONT_NAME = "fontName"; //$NON-NLS-1$
    public static final String FONT_SIZE = "fontSize"; //$NON-NLS-1$
    public static final String FONT_STYLE = "fontStyle"; //$NON-NLS-1$
    public static final String CHECK_FOR_UPDATES = "checkForUpdates"; //$NON-NLS-1$
    public static final String LAST_CHECK_FOR_UPDATES = "lastCheckForUpdates"; //$NON-NLS-1$
    public static final String LOAD_SECONDARY_FILES = "alwaysLoadSecondaryFiles"; //$NON-NLS-1$
    public static final String NUMBER_OF_BACKUPS = "numberOfBackups"; //$NON-NLS-1$
    public static final String WORK_PATH = "workpath"; //$NON-NLS-1$
    public static final String FILTER_STACK_TRACE = "useExceptionFilter"; //$NON-NLS-1$
    public static final String EXPERIMENTAL = "enableExperimental"; //$NON-NLS-1$
    public static final String BACKUP_WARNING = "backupSizeWarning"; //$NON-NLS-1$

    /**
     * Values
     */
    public static final String YES = "yes"; //$NON-NLS-1$
    public static final String NO = "no"; //$NON-NLS-1$
    public static final String ZERO = "0"; //$NON-NLS-1$
    public static final String NEVER = "never"; //$NON-NLS-1$
    public static final String EN = "en"; //$NON-NLS-1$
    
    /**
	Set's the default vaules, loads saved settings.
	Uses RegQuery to get Botf installation path from registry.
     */
    public SettingsManager()
    {
        try
        {
            fileSETTINGS = FileManager.getWritableLocalFile("Settings.ini"); //$NON-NLS-1$

            //set default settings
            DEFAULTS = new Properties();
            String def_path;
            String tmp = null;
            
            if (System.getProperty("os.name").contains("Windows"))
            {
                def_path = "C:";
                tmp = RegQuery.getBOTFInstallPath(false);
            }
            else
            {
                def_path = ".";
                
                // check if wine is installed
                String prefix = System.getProperty("user.home") + File.separator + ".wine" + File.separator + "dosdevices";
                
                File path = new File(prefix);
                
                if (path.exists() && path.isDirectory())
                {
                    def_path = prefix + File.separator + "c:";

                    tmp = RegQuery.getBOTFInstallPath(true);
                    
                    if (tmp != null)
                    {
                        tmp = prefix + File.separator + tmp;
                    }
                }
            }
            
            if (tmp == null)
            {
                tmp = def_path; //$NON-NLS-1$ 
            }

            DEFAULTS.setProperty(SettingsManager.STBOF, tmp + File.separator + "stbof.res"); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.ALT, tmp + File.separator + "alt.res"); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.ENGLISH, tmp + File.separator + "english.snd"); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.SFX, tmp + File.separator + "sfx.snd"); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.MUSIC, tmp + File.separator + "music.snd"); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.EXE, tmp + File.separator + "trek.exe"); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.LAST, tmp); 
            DEFAULTS.setProperty(SettingsManager.WORK_PATH, tmp); 
            DEFAULTS.setProperty(SettingsManager.RECENT + 0, ""); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.RECENT + 1, ""); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.RECENT + 2, ""); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.RECENT + 3, ""); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.RECENT + 4, ""); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.RECENT + 5, ""); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.RECENT + 6, ""); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.RECENT + 7, ""); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.RECENT + 8, ""); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.RECENT + 9, ""); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.AUTO_STBOF, SettingsManager.YES); 
            DEFAULTS.setProperty(SettingsManager.AUTO_EXE, SettingsManager.YES); 
            DEFAULTS.setProperty(SettingsManager.AUTO_ALT, SettingsManager.YES); 
            DEFAULTS.setProperty(SettingsManager.AUTO_ENGLISH, SettingsManager.YES); 
            DEFAULTS.setProperty(SettingsManager.AUTO_SFX, SettingsManager.YES); 
            DEFAULTS.setProperty(SettingsManager.AUTO_MUSIC, SettingsManager.YES); 
            DEFAULTS.setProperty(SettingsManager.STORE_SND_DESC, SettingsManager.NO); 
            DEFAULTS.setProperty(SettingsManager.LANGUAGE, SettingsManager.EN); 
            DEFAULTS.setProperty(SettingsManager.LOG_FILE, "error.log"); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.FONT_NAME, "Times New Roman"); //$NON-NLS-1$ 
            DEFAULTS.setProperty(SettingsManager.CHECK_FOR_UPDATES, SettingsManager.NO); 
            DEFAULTS.setProperty(SettingsManager.LAST_CHECK_FOR_UPDATES, SettingsManager.NEVER); 
            DEFAULTS.setProperty(SettingsManager.LOAD_SECONDARY_FILES, SettingsManager.YES); 
            DEFAULTS.setProperty(SettingsManager.NUMBER_OF_BACKUPS, ZERO);
            DEFAULTS.setProperty(SettingsManager.FILTER_STACK_TRACE, YES);
            DEFAULTS.setProperty(SettingsManager.EXPERIMENTAL, NO);
            DEFAULTS.setProperty(SettingsManager.FONT_SIZE, "12");
            DEFAULTS.setProperty(SettingsManager.BACKUP_WARNING, YES);
            
            SETTINGS = new Properties(DEFAULTS);
            
            //check file
            if (fileSETTINGS.exists())
            {
                this.readSettings();
            }
            else
            {
                fileSETTINGS.createNewFile();
            }
            
            this.updateUIFont();
        }
        catch (Exception e)
        {
            UE.displayError(e);
        }
    }
    
    private void updateUIFont()
    {
        // set font in UIManager
        Font def = getDefaultFont();

        java.util.Enumeration<Object> keys = UIManager.getDefaults().keys();
        while (keys.hasMoreElements())
        {
            Object key = keys.nextElement();
            Object value = UIManager.get(key);
            if (value instanceof FontUIResource)
            {
                UIManager.put(key, def);
            }
        }
    }

    /*
     * Reads settings from Settings.ini
     */
    private void readSettings()
    {
        try
        {
            SETTINGS.load(new FileInputStream(fileSETTINGS));
        }
        catch (FileNotFoundException e)
        {
            UE.displayError(e);
        }
        catch (IOException e)
        {
            e.printStackTrace();
            int n = JOptionPane.showConfirmDialog(UE.WINDOW, Language.getString("SettingsManager.2"), Language.getString("SettingsManager.1"), JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
            switch (n)
            {
                case JOptionPane.YES_OPTION:
                {
                    this.readSettings();
                    return;
                }
                default:
                {
                    return;
                }
            }
        }
        catch (Exception e)
        {
            UE.displayError(e);
        }
    
    }
    /*SETS*/
    public void setProperty(String property, String value)
    {
        String old = SETTINGS.getProperty(property);
        SETTINGS.setProperty(property, value);
        
        if (property.equals(SettingsManager.LANGUAGE))
        {
            Language.loadLanguage(value);
        }
        else if (property.equals(SettingsManager.FONT_NAME))
        {
            this.updateUIFont();
        }
        
        this.firePropertyChangeEvent(new PropertyChangeEvent(this, property, old, value));
    }
    
    public void setProperty(String property, boolean value)
    {
        if (value)
        {
            setProperty(property, YES);
        }
        else
        {
            setProperty(property, NO);
        }
    }
    
    public void setProperty(String property, int value)
    {
        setProperty(property, Integer.toString(value));
    }

    public String getProperty(String property)
    {
        String val = SETTINGS.getProperty(property);
        
        if (val.equalsIgnoreCase(SettingsManager.YES))
        {
            return SettingsManager.YES;
        }
        else if (val.equalsIgnoreCase(SettingsManager.NO))
        {
            return SettingsManager.NO;
        }
        else if (val.equalsIgnoreCase(SettingsManager.EN))
        {
            return SettingsManager.EN;
        }
        else if (val.equalsIgnoreCase(SettingsManager.NEVER))
        {
            return SettingsManager.NEVER;
        }
        else
        {
            return val;
        }
    }

    public boolean getBooleanProperty(String property)
    {
        String str = getProperty(property);
        
        return str.equalsIgnoreCase(YES);
    }
    
    public int getNumberProperty(String property)
    {
        String str = getProperty(property);
        int ret = 0;
        
        try
        {
            ret = Integer.parseInt(str);
        }
        catch (Exception f){}
        
        return ret;
    }

    public Font getDefaultFont()
    {
        int size = 12;

        try
        {
            size = Integer.parseInt(getProperty(SettingsManager.FONT_SIZE));
        }
        catch (Exception e)
        {}

        size = Math.max(8, size);

        int style = Font.PLAIN;
        try
        {
            style = Integer.parseInt(getProperty(SettingsManager.FONT_STYLE));
        }
        catch (Exception e)
        {}

        return new Font(this.getProperty(SettingsManager.FONT_NAME), style, size);
    }

    public Font getDefaultBigFont()
    {
        int size = 14;

        try
        {
            size = Integer.parseInt(getProperty(SettingsManager.FONT_SIZE));
        }
        catch (Exception e)
        {}

        size = Math.max(8, size) + 2;

        int style = Font.PLAIN;
        try
        {
            style = Integer.parseInt(getProperty(SettingsManager.FONT_STYLE));
        }
        catch (Exception e)
        {}
        style = style | Font.BOLD;

        return new Font(this.getProperty(SettingsManager.FONT_NAME), style, size);
    }

    /**
     * @return a String array of recent files full paths.
     * If no recent file paths exist null is returned.
     */
    public String[] getRecentFiles()
    {
        try
        {
            int i = 0;
            for (int o = 0; o < 10; o++)
            {
                if (SETTINGS.getProperty(SettingsManager.RECENT + o).length() != 0) 
                {
                    i++;
                }
            }
            if (i != 0)
            {
                String[] results = new String[i];
                i = 0;
                for (int o = 0; o < 10; o++)
                {
                    if (SETTINGS.getProperty(SettingsManager.RECENT + o).length() != 0)
                    {
                        results[i] = SETTINGS.getProperty(SettingsManager.RECENT + o);
                        i++;
                    }
                }
                return results;
            }
        }
        catch (Exception e)
        {
            UE.displayError(e);
        }
        
        return null;
    }

    /**
     * Adds a file path to the recent files list.
     * Ignores double entries.
     * @param file the file to be added.
     */
    public void addToRecentFiles(String file)
    {
        try
        {
            //get recent files
            String[] recent = this.getRecentFiles();
            //set this one
            SETTINGS.setProperty(SettingsManager.RECENT + 0, file);
            //add rest
            if (recent == null)
            {
                return;
            }
            int i = recent.length;
            int u = 1;
            File added = new File(file);
            for (int n = 0; (n < i) && (u < 10) && (n < 10); n++)
            {
                if (!added.equals(new File(recent[n])))
                {
                    SETTINGS.setProperty(SettingsManager.RECENT + u, recent[n]); 
                    u++;
                }
            }
            for (int n = u; n < 10; n++)
            {
                SETTINGS.remove(SettingsManager.RECENT + n); 
            }
        }
        catch (Exception e)
        {
            UE.displayError(e);
        }
    }
    
    /**
     * Saves settings to "Settings.ini".
     * If the file doesn't exist it is created.
     */
    public void saveSettings()
    {
        try
        {
            if (!fileSETTINGS.exists())
            {
                fileSETTINGS.createNewFile();
            }
            SETTINGS.store(new FileOutputStream(fileSETTINGS, false), "UE settings:"); //$NON-NLS-1$
        }
        catch (FileNotFoundException e)
        {
            UE.displayError(e);
        }
        catch (IOException e)
        {
            e.printStackTrace();
            int n = JOptionPane.showConfirmDialog(UE.WINDOW, Language.getString("SettingsManager.3"), Language.getString("SettingsManager.1"), JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
            switch (n)
            {
                case JOptionPane.YES_OPTION:
                {
                    this.saveSettings();
                    return;
                }
                default:
                {
                    return;
                }
            }
        }
        catch (Exception e)
        {
            UE.displayError(e);
        }
    }
    
    public void addPropertyChangeListener(PropertyChangeListener l)
    {
        listeners.add(l);
    }
    
    public void removePropertyChangeListener(PropertyChangeListener l)
    {
        listeners.remove(l);
    }
    
    protected void firePropertyChangeEvent(PropertyChangeEvent e)
    {
        for (int i = 0; i < listeners.size(); i++)
        {
            listeners.get(i).propertyChange(e);
        }
    }
}
