/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: Tga2Ani.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.util;

import java.io.*;
import java.util.*;

import ue.Language;
import ue.exception.*;

/**This class is used to convert a group of 16-bit tga images to an ani/cur file.*/
public class Tga2Ani
{
	private LinkedList<GenericFile> DATA = new LinkedList<GenericFile>();
	private short frameh; //frame height
	private short framew; //frame width
	int size = 34; //animation size in bytes (34 bytes is for the ani header)
	// strings
	private static String errWIDTH = Language.getString("Tga2Ani.0"); //$NON-NLS-1$
	private static String errHEIGHT = Language.getString("Tga2Ani.1"); //$NON-NLS-1$
	
	/**Adds a frame to the existing group of images.
	@param name the name of the image
	@param tga tga data
	@throws InvalidArgumentsException if the tga file is invalid or if the size doesn't match the other frames*/
	public void addFrame(String name, byte[] tga) throws InvalidArgumentsException{
		//check if image is valid
		if (tga[0]!=0 || tga[1]!=0 || tga[2]!=2 || tga[3]!=0 || tga[4]!=0 
			|| tga[5]!=0 || tga[6]!=0 || tga[7]!=0 || tga[8]!=0 
			|| tga[9]!=0 || tga[10]!=0 || tga[11]!=0 || tga[16]!=16) {
			throw new InvalidArgumentsException(Language.getString("Tga2Ani.2")); //$NON-NLS-1$
		}
		//get dimensions
		byte[] b = new byte[2];
		short shr;
		//set frame width and height
		if (DATA.isEmpty()){
			b[0] = tga[12];
			b[1] = tga[13];
			framew = Tools.toShort(b, true);
			b[0] = tga[14];
			b[1] = tga[15];
			frameh = Tools.toShort(b, true);
		}else{
			b[0] = tga[12];
			b[1] = tga[13];
			shr = Tools.toShort(b, true);
			if (shr != framew){
				throw new InvalidArgumentsException(errWIDTH.replace("%1", Short.toString(shr)).replace("%2", Short.toString(framew))); //$NON-NLS-1$ //$NON-NLS-2$
			}
			b[0] = tga[14];
			b[1] = tga[15];
			shr = Tools.toShort(b, true);
			if (shr != frameh){
				throw new InvalidArgumentsException(errHEIGHT.replace("%1", Short.toString(shr)).replace("%2", Short.toString(frameh))); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
		size += tga.length - 18; //tga size-tga header
		DATA.add(new GenericFile(name, tga));
	}
	/**Creates and returns an animation file from the frames that were added.
	@return an animation file or null if no frames were added yet.*/
	public AniOrCur getAniOrCur(boolean cursor) throws Exception{
		if (DATA.isEmpty()) {
			return null;
		}
		//data block for the animation
		byte[] anim = new byte[size];
		byte[] b;
		//botf header (ani header)
			//frame dimensions
			b = (cursor) ? Tools.toByte((short)(frameh/2), true) : Tools.toByte(frameh, true);
			anim[0] = b[0];
			anim[1] = b[1];
			b = (cursor) ? Tools.toByte((short)(framew/2), true) : Tools.toByte(framew, true);
			anim[2] = b[0];
			anim[3] = b[1];
			//number of frames
			b = Tools.toByte((short)DATA.size(), true);
			anim[4] = b[0];
			anim[5] = b[1];
			//delay in miliseconds (this is generic)
			b = Tools.toByte((short)1000, true);
			anim[6] = b[0];
			anim[7] = b[1];
			//tga dimensions
			//ok, lets not complicate
			//will place data as if it were a cursor
			b = Tools.toByte(framew, true);
			anim[8] = b[0];
			anim[9] = b[1];
			b = Tools.toByte((short)(DATA.size()*frameh), true);
			anim[10] = b[0];
			anim[11] = b[1];
			//size
			int af = size - 16;
			b = Tools.toByte(af, true);
			anim[12] = b[0];
			anim[13] = b[1];
			anim[14] = b[2];
			anim[15] = b[3];
		//tga header
		anim[16] = 0;
		anim[17] = 0;
		anim[18] = 2;
		anim[19] = 0;
		anim[20] = 0;
		anim[21] = 0;
		anim[22] = 0;
		anim[23] = 0;
		anim[24] = 0;
		anim[25] = 0;
		anim[26] = 0;
		anim[27] = 0;
		anim[32] = (byte)16;
		anim[33] = 1;
		b = Tools.toByte(framew, true);
		anim[28] = b[0];
		anim[29] = b[1];
		b = Tools.toByte((short)(DATA.size()*frameh), true);
		anim[30] = b[0];
		anim[31] = b[1];
		//data
		int index = 34;
		GenericFile gen;
		for (int i = DATA.size() - 1; i >= 0; i--){
			gen = DATA.get(i);
			b = gen.getData();
			for (int j = 0; j < frameh*framew; j++){
				anim[index++] = b[2*j+18];
				anim[index++] = b[2*j+19];
			}
		}
		ByteArrayInputStream bais = new ByteArrayInputStream(anim);
		return new AniOrCur(bais, "finito.cur"); //$NON-NLS-1$
	}
	/**Returns a list of frames that were added (their names), null if none.*/
	public String[] getFrameList(){
		int i = DATA.size();
		if (i <= 0) {
			return null;
		}
		String[] str = new String[i];
		for (int j = 0; j < i; j++){
			str[j] = (DATA.get(j)).getName();
		}
		return str;
	}
	/**Removes a frame from the list. This is the oposite of the add function =).
	@param str the name of the image to remove*/
	public void remove(String str){
		GenericFile gen = new GenericFile(str, null);
		int i = DATA.indexOf(gen);
		if (i<0) {
			return;
		}
		gen = DATA.get(i);
		size-= (gen.getData()).length - 18;
		DATA.remove(i);
	}
	/**Clears frames, removes all frames from the list.*/
	public void clear(){
		DATA.clear();
		size = 34;
	}
	/**Moves the image a position up.*/
	public void moveUp(String str){
		GenericFile gen = new GenericFile(str, null);
		GenericFile gun;
		int i = DATA.indexOf(gen);
		if (i<=0) {
			return;
		}
		gen = DATA.get(i);
		gun = DATA.get(i-1);
		DATA.set(i, gun);
		DATA.set(i-1, gen);
	}
	/**Moves the image a position down.*/
	public void moveDown(String str){
		GenericFile gen = new GenericFile(str, null);
		GenericFile gun;
		int i = DATA.indexOf(gen);
		if (i<0 || i+1>=DATA.size()) {
			return;
		}
		gen = DATA.get(i);
		gun = DATA.get(i+1);
		DATA.set(i, gun);
		DATA.set(i+1, gen);
	}

    public void remove(int index)
    {
		if (index < 0) {
			return;
		}

        GenericFile gen = DATA.get(index);
		size -= gen.getSize() - 18;
		DATA.remove(index);
    }
}
