/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007 Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: Convert.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.util;

import java.awt.Graphics;
import java.awt.Image;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Locale;
import java.util.StringTokenizer;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.SourceDataLine;
import javax.swing.JDialog;
import javax.swing.JOptionPane;

import ue.Language;
import ue.UE;
import ue.exception.InvalidArgumentsException;

/** 
 * Converts different types of data.
 */
public class Tools
{
    private static AudioPlayer APLAYER = null;
    
    // The letters used for HEX coding
    private static String[] HEX = new String[]{"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E", "F"}; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$ //$NON-NLS-8$ //$NON-NLS-9$ //$NON-NLS-10$ //$NON-NLS-11$ //$NON-NLS-12$ //$NON-NLS-13$ //$NON-NLS-14$ //$NON-NLS-15$ //$NON-NLS-16$ 
    
    /**
	 * Converts int to a byte array
	 */
	public static final byte[] toByte(int i, boolean flag){
		byte abyte0[] = new byte[4];
		for(byte byte0 = 0; byte0 <= 3; byte0++) {
			abyte0[byte0] = (byte)(i >>> (3 - byte0) * 8);
		}
		if(flag) {
			abyte0 = reverse_order(abyte0, 4);
		}
		return abyte0;
	}
	
	/**
	 * Converts short to a byte array
	 */
	public static final byte[] toByte(short word0, boolean flag){
		byte abyte0[] = new byte[2];
		for(byte byte0 = 0; byte0 <= 1; byte0++) {
			abyte0[byte0] = (byte)(word0 >>> (1 - byte0) * 8);
		}
		
		if(flag) {
			abyte0 = reverse_order(abyte0, 2);
		}
		return abyte0;
	}
	
	/**
	 * Converts long to a byte array
	 */
	public static final byte[] toByte(long l, boolean flag){
		byte abyte0[] = new byte[8];
		for(byte byte0 = 0; byte0 <= 7; byte0++) {
			abyte0[byte0] = (byte)(int)(l >>> (7 - byte0) * 8);
		}
		
		if(flag) {
			abyte0 = reverse_order(abyte0, 8);
		}
		return abyte0;
	}

	/**
	 * Converts float to a byte array
	 */
	public static final byte[] toByte(float f, boolean flag){
		byte abyte0[] = new byte[4];
		int i = Float.floatToIntBits(f);
		abyte0 = toByte(i, flag);
		return abyte0;
	}
	
	/**
	 * Converts double to a byte array
	 */
	public static final byte[] toByte(double d, boolean flag){
		byte abyte0[] = new byte[8];
		long l = Double.doubleToLongBits(d);
		abyte0 = toByte(l, flag);
		return abyte0;
	}

	/**
	 * Converts a byte array to int
	 */
	public static final int toInt(byte abyte0[], boolean flag){
		int i = 0;
		if(flag) {
			abyte0 = reverse_order(abyte0, 4);
		}
		for(byte byte0 = 0; byte0 <= 3; byte0++){
			int j;
			if(abyte0[byte0] < 0){
				abyte0[byte0] = (byte)(abyte0[byte0] & 0x7f);
				j = abyte0[byte0];
				j |= 0x80;
			}else{
				j = abyte0[byte0];
			}
			i |= j;
			if(byte0 < 3) {
				i <<= 8;
			}
		}
		
		return i;
	}

	/**
	 * Converts a byte array to short
	 */
	public static final short toShort(byte data[], boolean flag){
            byte[] b = new byte[]{data[0], data[1]};
            
	    short word0 = 0;
            
            
	    if(flag) {
			b = reverse_order(b, 2);
		}
	    for(byte byte0 = 0; byte0 <= 1; byte0++){
		    short word1;
		    if(b[byte0] < 0){
			b[byte0] = (byte)(b[byte0] & 0x7f);
			word1 = b[byte0];
			word1 |= 0x80;
		    }else{
			    word1 = b[byte0];
		    }
		    word0 |= word1;
		    if(byte0 < 1) {
				word0 <<= 8;
			}
	    }
	    
	    return word0;
    	}

	/**
	 * Converts a byte array to long
	 */
	public static final long toLong(byte abyte0[], boolean flag){
		long l = 0L;
		if(flag) {
			abyte0 = reverse_order(abyte0, 8);
		}
		for(byte byte0 = 0; byte0 <= 7; byte0++){
			long l1;
			if(abyte0[byte0] < 0){
				abyte0[byte0] = (byte)(abyte0[byte0] & 0x7f);
				l1 = abyte0[byte0];
				l1 |= 128L;
			}else{
				l1 = abyte0[byte0];
			}
			l |= l1;
			if(byte0 < 7) {
				l <<= 8;
			}
		}
		
		return l;
	}
	
	/**
	 * Converts a byte array to float
	 */
	public static final float toFloat(byte abyte0[], boolean flag){
		float f = 0.0F;
		int i = toInt(abyte0, flag);
		f = Float.intBitsToFloat(i);
		return f;
	}

	/**
	 * Converts a byte array to double
	 */
	public static final double toDouble(byte abyte0[], boolean flag){
		double d = 0.0D;
		long l = toLong(abyte0, flag);
		d = Double.longBitsToDouble(l);
		return d;
	}
	
    /**
     * Converts a byte array to a string using the ISO-8859-1 charset.
     * @param b the byte array to convert
     */
    public static final String toASCII(byte b[]) throws UnsupportedEncodingException{
        String str;
		str = new String(b, 0, b.length, "ISO-8859-1"); //$NON-NLS-1$
		int lol = str.indexOf(0);
		if (lol != -1){
			str = str.substring(0, lol);
		}
		if (str==null) {
			return ""; //$NON-NLS-1$
		}
		return str;
    }
    
    /**
     * Converts a byte array to a string using the ISO-8859-1 charset.
     * @param b the byte array
     * @param start the starting index
     * @param len the length
     */
    public static final String toASCII(byte b[], int start, int len) throws UnsupportedEncodingException{
        String str;
		str = new String(b, start, len, "ISO-8859-1"); //$NON-NLS-1$
		int lol = str.indexOf(0);
		if (lol != -1){
			str = str.substring(0, lol);
		}
		return str;
    }
    
    /**
     * Converts a string to a byte array containing ISO-8859-1 characters
     * @param str the string to convert
     * @param size the size of the returning array
     * @param strsize the max size of the string
     */
    public static final byte[] toByte(String str, int size, int strsize) throws UnsupportedEncodingException{
    	byte[] ret = new byte[size];
		byte[] r = str.getBytes("ISO-8859-1"); //$NON-NLS-1$
		int len;
		if (r.length > ret.length) {
			len = ret.length;
		} else {
			len = r.length;
		}
		if (len > strsize) {
			len = strsize;
		}
		System.arraycopy(r, 0, ret, 0, len);
		return ret;
    }

    private static final byte[] reverse_order(byte abyte0[], int i){
        byte abyte1[] = new byte[i];
        for(byte byte0 = 0; byte0 <= i - 1; byte0++) {
			abyte1[byte0] = abyte0[i - 1 - byte0];
		}

        return abyte1;
    }
    
    /**
     * Converts a byte array to a hex string.
     */
    public static final String toHex(byte[] b){
	    String ret = ""; //$NON-NLS-1$
	    for (int j=0; j<b.length; j++){
		    int tun2 = (b[j] & 0x000000ff);
		    int tun = (tun2/16);
		    ret = ret + HEX[tun]+ HEX[tun2-tun*16] + " "; //$NON-NLS-1$
	    }
	    return ret;
    }
    
    /**
     * Converts a hex string to byte array. Works for both lower case letters and upper case.
     */
    public static final byte[] hexToByte(String hex, int max_len) throws Exception
    {
	    hex = hex.toUpperCase();
	    int u = 0;
	    byte[] b = new byte[max_len];
	    boolean ten = true;
	    short t = 0;
	    for (int h=0; h<hex.length(); h++){
		    String chr = hex.substring(h, h+1);
		    for (int j=0; j<16; j++){
			    if (chr.equals(HEX[j])){
				    if (ten){
					    ten = false;
					    t = (short)(j*16);
				    }else{
					    if (u==max_len) {
							throw new Exception(Language.getString("Tools.0")); //$NON-NLS-1$
						}
					    ten = true;
					    t = (short)(t + j);
					    byte[] but = Tools.toByte(t, true);
					    b[u] = but[0];
					    u++;
				    }
			    }
		    }
	    }
	    return b;
    }
    
    /**
     * Copies a file to dest. This is because File.renameTo doesn't work as it should
     * @throws IOException 
     */
    public static void copyFile(File src, File dest) throws IOException
    {
    	FileInputStream fis = new FileInputStream(src);
    	FileOutputStream fos = new FileOutputStream(dest);
    	
    	byte[] b = new byte[1024];
    	int len;
    	while (fis.available()>0){
    		len = fis.read(b);
    		fos.write(b, 0, len);
    	}
    	
    	fis.close();
    	fos.close();
    }
 
    /**
     * The calculation can contain double values,
     * operators allowed are ^, /, *, -, +
     * ( and ) are allowed
     * @param calc the calculation
     * @return the vlue of the calculation
     */
    public static double calculate(String calc) throws InvalidArgumentsException{
    	ArrayList<Double> values = new ArrayList<Double>();
    	ArrayList<Integer> operators = new ArrayList<Integer>();
    	
    	// calculate sub-calculations - remove (,)
    	String subCalc;
    	int i, j;
    	i = calc.lastIndexOf("("); //$NON-NLS-1$
		j = calc.indexOf(")", i); //$NON-NLS-1$

		while (i >= 0){
    		if (j < 0) throw new InvalidArgumentsException(Language.getString("Tools.1")); //$NON-NLS-1$
    		
    		subCalc = calc.substring(i + 1, j);
    		double v = calculate(subCalc);
    		
    		if (v < 0){
    			subCalc = calc.substring(0, i) + "~" + Math.abs(v); //$NON-NLS-1$
    		}else{
    			subCalc = calc.substring(0, i) + v;
    		}
    		
    		if (j < calc.length() - 1){
    			calc = subCalc + calc.substring(j + 1, calc.length());
    		}else{
    			calc = subCalc;
    		}
    		i = calc.lastIndexOf("("); //$NON-NLS-1$
    		j = calc.indexOf(")", i); //$NON-NLS-1$
    	}
		if (j >= 0){
    		throw new InvalidArgumentsException(Language.getString("Tools.2")); //$NON-NLS-1$
    	}
    	
		// calculate
    	StringTokenizer tokenizer = new StringTokenizer(calc, " ^/*-+", true); //$NON-NLS-1$
    	
    	String token;
    	boolean op = false;
    	while (tokenizer.hasMoreTokens()){
    		token = tokenizer.nextToken();
    		
    		if (token.equals(" ")){ //$NON-NLS-1$
    			continue;
    		}else if (token.equals("^")){ //$NON-NLS-1$
    			op = true;
    			operators.add(4);
    		}else if (token.equals("/")){ //$NON-NLS-1$
    			op = true;
    			operators.add(3);
    		}else if (token.equals("*")){ //$NON-NLS-1$
    			op = true;
    			operators.add(2);
    		}else if (token.equals("-")){ //$NON-NLS-1$
    			op = true;
    			operators.add(1);
    		}else if (token.equals("+")){ //$NON-NLS-1$
    			op = true;
    			operators.add(0);
    		}else{
    			try{
    				token = token.replaceAll("~", "-"); //$NON-NLS-1$ //$NON-NLS-2$
    				values.add(Double.parseDouble(token));
    			}catch (Exception e){
    				throw new InvalidArgumentsException(Language.getString("Tools.3").replace("%1", token)); //$NON-NLS-1$ //$NON-NLS-2$
    			}
    			op = false;
    		}
    		
    		if (!op && !operators.isEmpty()){
    			int ope = operators.get(operators.size()-1);
    			
    			switch (ope){
    				case 1:{
    					values.set(values.size()-1, -values.get(values.size()-1));
    					operators.set(operators.size()-1, 0);
    					break;
    				}
    				case 3:{
    					values.set(values.size()-1, 1/values.get(values.size()-1));
    					operators.set(operators.size()-1, 2);
    					break;
    				}
    				default:
    			}
    		}
    		
    		if (op && operators.size() > 1){
    			// check priority
    			int ope1 = operators.get(operators.size()-1);
    			int ope2 = operators.get(operators.size()-2);
    			if (ope2 > ope1){
    				if (values.size() < 2) throw new InvalidArgumentsException(Language.getString("Tools.4")); //$NON-NLS-1$
    				
    				operators.remove(operators.size()-2);
    				
    				double val2 = values.remove(values.size() - 1);
    				double val1 = values.remove(values.size() - 1);

    				val1 = calculate(val1, val2, ope2);
    				values.add(val1);
    			}
    		}
    	}
    	
    	while (!operators.isEmpty()){
			if (values.size() < 2) throw new InvalidArgumentsException(Language.getString("Tools.4")); //$NON-NLS-1$
			int ope = operators.remove(operators.size()-1);
			values.add(calculate(values.remove(values.size() - 2), values.remove(values.size() - 1), ope));
    	}
    	
    	if (values.size() != 1) throw new InvalidArgumentsException(Language.getString("Tools.5")); //$NON-NLS-1$
    	return values.get(0);
    }
    
    private static double calculate(double val1, double val2, int operator) throws InvalidArgumentsException{
    	switch (operator){
    		case 0: // +
    			return (val1 + val2);
    		case 1: // -
    			return (val1 - val2);
    		case 2: // *
    			return (val1 * val2);
    		case 3: // /
    			return (val1 / val2);
    		case 4: // ^
    			return Math.pow(val1, val2);
    		default:
    			throw new InvalidArgumentsException(Language.getString("Tools.6")); //$NON-NLS-1$
    	}
    }
    
    public static Object runUEWorker(UEWorker<?> worker)
    {
        Object response = null;
        final String cancel = Language.getString("Tools.7"); //$NON-NLS-1$
        final String ok = Language.getString("Tools.8"); //$NON-NLS-1$
        
        final JOptionPane opti = new JOptionPane(Language.getString("Tools.9"), //$NON-NLS-1$
                        JOptionPane.PLAIN_MESSAGE);

        opti.setOptions(new String[]{cancel});

        final JDialog dia = opti.createDialog(UE.WINDOW, UE.APP_NAME);

        worker.addPropertyChangeListener(new PropertyChangeListener()
        {
            @Override
            public void propertyChange(PropertyChangeEvent evt)
            {
                if (UEWorker.VISIBLE.equals(evt.getPropertyName()))
                {
                    dia.setVisible((Boolean)evt.getNewValue());
                }
                else if (UEWorker.MESSAGE.equals(evt.getPropertyName()))
                {
                    opti.setMessage(evt.getNewValue());
                    if (dia.getPreferredSize().width > dia.getSize().width)
                    {
                        dia.pack();
                    }
                }
                else if (UEWorker.STATUS.equals(evt.getPropertyName()))
                {
                    if (UEWorker.STATUS_DONE.equals(evt.getNewValue())
                            || UEWorker.STATUS_CANCEL.equals(evt.getNewValue()))
                    {
                        dia.dispose(); // close dialog
                    }
                    else if (UEWorker.STATUS_ERROR.equals(evt.getNewValue())
                            || UEWorker.STATUS_DONE_WITH_MSG.equals(evt.getNewValue()))
                    {
                        opti.setOptions(new String[]{ok});
                    }
                }
            }
        });

        worker.execute();

        dia.setVisible(true);

        if (cancel.equals(opti.getValue())){ //$NON-NLS-1$
                worker.cancel(true);
        }

        try{
                response = worker.getResult();
        }catch (Exception e){
                e.printStackTrace();
        }

        return response;
    }
    
    /**
     * 
     * @return a random string of specified length
     */
    public static String getRandomString(int length){
        byte[] b = new byte[length];
        //48-57, 65-90 - ascii
        //10+26=37
        byte g;
        for (int i = 0; i < length; i++){
                g = (byte)(Math.random()*35);
                //0-9
                if (g<10){
                        g = (byte)(g + 48);
                //10-35
                }else{
                        g = (byte)(g + 55);
                }
                b[i] = g;
        }
        try{
                return new String(b, "ASCII"); //$NON-NLS-1$
        }catch (Exception e){
                e.printStackTrace();
        }
        return null;
    }
    
    public static Number getValueWithinBounds(Number val, Number min, Number max)
    {
        return Math.max(min.doubleValue(), Math.min(val.doubleValue(), max.doubleValue()));
    }
    
    public static void checkForUpdate()
    {
        UpdateChecker t = new UpdateChecker();
        t.start();
        SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy"); //$NON-NLS-1$
        UE.SETTINGS.setProperty(SettingsManager.LAST_CHECK_FOR_UPDATES, sdf.format(Calendar.getInstance().getTime()));
    }
    
    public static void openInDefaultBrowser(String url)
    {
        String os = System.getProperty("os.name").toLowerCase(); //$NON-NLS-1$
        Runtime rt = Runtime.getRuntime();
        try
        {
                if (os.indexOf( "win" ) >= 0) //$NON-NLS-1$
                {
                  rt.exec("rundll32 url.dll,FileProtocolHandler \"" + url + "\""); //$NON-NLS-1$
                }
                else if (os.indexOf( "mac" ) >= 0) //$NON-NLS-1$
                {
                    rt.exec( "open " + url); //$NON-NLS-1$
                }
                else
                {
                  String[] browsers = { "firefox", "mozilla", "konqueror", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                      "opera", "epiphany", "links"}; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
     
                  StringBuffer cmd = new StringBuffer();
                  for (int i=0; i<browsers.length; i++)
                  {
                      cmd.append( (i == 0  ? "" : " || " ) + browsers[i] +" \"" + url + "\" "); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
                  }
                  rt.exec(new String[] { "sh", "-c", cmd.toString() }); //$NON-NLS-1$ //$NON-NLS-2$
               }
        }
        catch (IOException e)
        {
            UE.displayError(e);
        }
    }
    
    private static class UpdateChecker extends Thread
    {
        @Override
        public void run()
        {
            try
            {
                String url = "http://www.4shared.com/rss/BmgQzwAM"; //$NON-NLS-1$
                String[] result = checkRSS(url);
                
                if (result == null &&
                        (UE.APP_VER.contains("dev") || UE.APP_VER.contains("rc"))) //$NON-NLS-1$ //$NON-NLS-2$
                {
                    url = "http://www.4shared.com/rss/9yLT-tJ_"; //$NON-NLS-1$
                    result = checkRSS(url);
                }
                
                if (result != null)
                {
                    int respon; 
                    String[] opt;
                    if (result[1] != null)
                    {
                        opt = new String[]{
                                Language.getString("Tools.12"), //$NON-NLS-1$
                                Language.getString("Tools.13"), //$NON-NLS-1$
                                Language.getString("Tools.14") //$NON-NLS-1$
                        };
                    }
                    else
                    {
                        opt = new String[]{
                                Language.getString("Tools.13"), //$NON-NLS-1$
                                Language.getString("Tools.14") //$NON-NLS-1$
                        };
                    }
                    
                    respon = JOptionPane.showOptionDialog(
                            UE.WINDOW,
                            Language.getString("Tools.10").replace("%1", result[0]), //$NON-NLS-1$ //$NON-NLS-2$
                            Language.getString("Tools.11"), //$NON-NLS-1$
                            JOptionPane.NO_OPTION,
                            JOptionPane.INFORMATION_MESSAGE,
                            null,
                            opt,
                            Language.getString("Tools.14") //$NON-NLS-1$
                         );

                    respon += 3 - opt.length;
                    
                    switch (respon)
                    {
                        case 0:
                        {
                            Tools.openInDefaultBrowser(result[1]);
                            break;
                        }
                        case 1:
                        {
                            UE.SETTINGS.setProperty(SettingsManager.CHECK_FOR_UPDATES, false);
                            break;
                        }
                        default:
                        {
                        }
                    }
                }
            }
            catch (Exception f)
            {
                UE.displayError(f);
            }
        }
        
        // version, dllink
        private String[] checkRSS(String url) throws Exception
        {
            URL ue = new URL(url); //$NON-NLS-1$
            BufferedReader in = new BufferedReader(new InputStreamReader(ue.openStream()));

            StringBuffer buf = new StringBuffer();
            String inputLine;
            String link = null;

            Integer my_ver_num = null;
            Integer ex_ver_num = null;
            String my_ver_desc = null;
            String ex_ver_desc = null;
            String ex_ver;
            String my_ver = UE.APP_VER;
            String tmp = null;
            String result = null;
            String[] final_result = null;

            while ((inputLine = in.readLine()) != null)
            {
                buf.append(inputLine);
            }
            
            in.close();
            
            int s = buf.lastIndexOf("<title>"); //$NON-NLS-1$
            int t = buf.lastIndexOf("</title>"); //$NON-NLS-1$
//            int prev = s;
            
            while (s >= 0 && s < t && t > 0)
            {
                my_ver = UE.APP_VER;
                inputLine = buf.substring(s + 7, t);
                
                if (inputLine.contains("source"))
                {
                    buf.delete(s, buf.length() - 1);
                    
                    s = buf.lastIndexOf("<title>"); //$NON-NLS-1$
                    t = buf.lastIndexOf("</title>"); //$NON-NLS-1$

                    continue;
                }
                
                if (inputLine.contains("UE ")) //$NON-NLS-1$
                {
                    s = inputLine.indexOf("UE "); //$NON-NLS-1$
                    t = inputLine.indexOf(".zip"); //$NON-NLS-1$
                    
                    if (s >= 0 && s < t && t > 0)
                    {
                        tmp = inputLine.substring(s + 3, t);
                        
                        ex_ver = tmp;
                        
                        while (my_ver != null && ex_ver != null)
                        {
                            if (Character.isDigit(my_ver.charAt(0))
                                    && Character.isDigit(ex_ver.charAt(0)))
                            {
                                // compare numbers
                                try
                                {
                                    my_ver_num = Integer.parseInt(my_ver.substring(0, 1));
                                    ex_ver_num = Integer.parseInt(ex_ver.substring(0, 1));
                                    my_ver = my_ver.substring(1);
                                    ex_ver = ex_ver.substring(1);
                                    
                                    while (my_ver.length() > 0 && Character.isDigit(my_ver.charAt(0)))
                                    {
                                        my_ver_num = my_ver_num * 10 + Integer.parseInt(my_ver.substring(0, 1));
                                        my_ver = my_ver.substring(1);
                                    }
                                    
                                    while (ex_ver.length() > 0 && Character.isDigit(ex_ver.charAt(0)))
                                    {
                                        ex_ver_num = ex_ver_num * 10 + Integer.parseInt(ex_ver.substring(0, 1));
                                        ex_ver = ex_ver.substring(1);
                                    }
                                    
                                    if (ex_ver_num > my_ver_num)
                                    {
                                        result = tmp;
                                        break;
                                    }
                                    else if (ex_ver_num < my_ver_num)
                                    {
                                        break;
                                    }
                                }
                                catch (Exception e)
                                {
                                    e.printStackTrace();
                                    break;
                                }
                            }
                            else if (!Character.isDigit(my_ver.charAt(0))
                                    && !Character.isDigit(ex_ver.charAt(0)))
                            {
                                // compare strings
                                my_ver_desc = my_ver.substring(0, 1);
                                ex_ver_desc = ex_ver.substring(0, 1);
                                my_ver = my_ver.substring(1);
                                ex_ver = ex_ver.substring(1);
                                
                                while (my_ver.length() > 0 && !Character.isDigit(my_ver.charAt(0)))
                                {
                                    my_ver_desc = my_ver_desc + my_ver.substring(0, 1);
                                    my_ver = my_ver.substring(1);
                                }
                                
                                while (ex_ver.length() > 0 && !Character.isDigit(ex_ver.charAt(0)))
                                {
                                    ex_ver_desc = ex_ver_desc + ex_ver.substring(0, 1);
                                    ex_ver = ex_ver.substring(1);
                                }
                                
                                if (ex_ver_desc.length() < my_ver_desc.length())
                                {
                                    result = tmp;
                                    break;
                                }
                                else if (ex_ver_desc.length() > my_ver_desc.length())
                                {
                                    break;
                                }
                                else
                                {
                                    int c;
                                    
                                    for (c = 0; c < my_ver_desc.length(); c++)
                                    {
                                        if (my_ver_desc.charAt(c) > ex_ver_desc.charAt(c))
                                        {
                                            c++;
                                            break;
                                        }
                                        else if (my_ver_desc.charAt(c) < ex_ver_desc.charAt(c))
                                        {
                                            result = tmp;
                                            break;
                                        }
                                    }
                                    
                                    if (c != my_ver_desc.length())
                                    {
                                        break;
                                    }
                                }
                            }
                            else
                            {
                                // this shouldn't happen
                                // but if it does, I like numbers better :)
                                if (Character.isDigit(my_ver.charAt(0)))
                                {
                                    break;
                                }
                                else
                                {
                                    result = tmp;
                                    break;
                                }
                            }
                            
                            if (my_ver.length() == 0 || ex_ver.length() == 0)
                            {
                                if (my_ver.length() < ex_ver.length())
                                {
                                    result = tmp;
                                }

                                my_ver = ex_ver = null;
                            }
                        }
                    }
                }
                else
                {
                    buf.delete(s, buf.length() - 1);
                    
                    s = buf.lastIndexOf("<title>"); //$NON-NLS-1$
                    t = buf.lastIndexOf("</title>"); //$NON-NLS-1$

                    continue;
                }

                if (result != null)
                {
                    s = buf.lastIndexOf("<link>"); //$NON-NLS-1$
                    t = buf.lastIndexOf("</link>"); //$NON-NLS-1$

                    if (s >= 0 && s < t && t > 0)
                    {
                        link = buf.substring(s + 6, t);
                    }
                    final_result = new String[]{result, link};
                    return final_result;
                }
            }
            
            return final_result;
        }
    }
    
    public static void playSound(File file) throws Exception
    {
        if (APLAYER == null) APLAYER = new AudioPlayer();
        APLAYER.playSoundFile(file);
    }
    
    private static class AudioPlayer extends Thread
    {
        private File soundFile = null;
        private boolean stop = false;
        
        public void playSoundFile(File file) throws Exception
        {
            if (this.isAlive()) this.stop = true;
            this.join();
            APLAYER = new AudioPlayer();
            APLAYER.soundFile = file;
            APLAYER.start();
        }
        
        @Override
        public void run()
        {
            if (soundFile == null) return;
            
            try
            {              
                AudioInputStream audioInputStream = AudioSystem.getAudioInputStream(soundFile);

                AudioFormat format = audioInputStream.getFormat();

                SourceDataLine auline = null;
                DataLine.Info info = new DataLine.Info(SourceDataLine.class, format);

                auline = (SourceDataLine) AudioSystem.getLine(info);
                auline.open(format);

                auline.start();

                int len = 1;
                byte[] data = new byte[524288]; // 128 kB

                try {
                        while ( len > 0)
                        {
                            if (stop) break;
                            
                            len = audioInputStream.read(data, 0, data.length);
                            if (len >= 0)
                                auline.write(data, 0, len);
                        }
                } catch (IOException e) {
                        e.printStackTrace();
                        return;
                } finally {
                        auline.drain();
                        auline.close();
                        audioInputStream.close();
                }
            }
            catch (Exception e)
            {
                UE.displayError(e);
            }
        }
    }

    public static boolean isSensible(String str)
    {
        try
        {
            byte[] b = Tools.toByte(str, str.length(), str.length());
            int c;

            for (int i = 0; i < b.length; i++)
            {
                c = 0xFF & b[i];

                if (
                        c < 32 || (c > 126 && c < 160))
                {
                    return false;
                }
            }

            return true;
        }
        catch (UnsupportedEncodingException ex)
        {
            return false;
        }
    }

    public static byte getMin(byte[] num)
    {
        byte tmp = Byte.MAX_VALUE;

        for (byte val : num)
        {
            if (val < tmp)
            {
                tmp = val;
            }
        }

        return tmp;
    }

    public static byte getMax(byte[] num)
    {
        byte tmp = Byte.MIN_VALUE;

        for (byte val : num)
        {
            if (val > tmp)
            {
                tmp = val;
            }
        }

        return tmp;
    }

    public static int getIndex(ArrayList<byte[]> list, byte[] hash)
    {
        byte[] h;

        for (int i = 0; i < list.size(); i++)
        {
            h = list.get(i);

            if (Arrays.equals(h, hash))
            {
                return i;
            }
        }

        return -1;
    }

    public static void drawImage(Image image, int x, int y, int w, int h, Graphics g)
    {
        int imageW = Math.min(image.getWidth(null), w);
        int imageH = Math.min(image.getHeight(null), h);

        g.drawImage(image, x, y, x + imageW, y + imageH, 0, 0,
                                imageW, imageH, null);
    }

    public static String capitalize(String word)
    {
        word = word.trim().toLowerCase(Locale.ENGLISH);
        word = word.substring(0, 1).toUpperCase(Locale.ENGLISH) + word.substring(1);

        return word;
    }

    public static String capitalizeWords(String str)
    {
        str = capitalize(str);

        int i = str.indexOf(" ", 0);

        while (i >= 0)
        {
            str = str.substring(0, i) + " " + capitalize(str.substring(i));

            i = str.indexOf(" ", i + 1);
        }

        return str;
    }

    public static String formatNumber(double value, int decimals)
    {
        String format = "0.";

        for (; decimals > 0; decimals--)
        {
            format += "#";
        }

        DecimalFormat df = new DecimalFormat(format);
        return df.format(value);
    }
}