/**
 * Ultimate Editor for Birth Of The Federation 1.02
 * Copyright (C) 2005-2008  Alan Podlesek
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 	
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 	
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * File: UEWorker.java
 * 
 * Written by: Alan Podlesek <seqdcer@yahoo.com>
 * 	
 * Last modified: 22.7.2008
 */
package ue.util;

import java.text.SimpleDateFormat;
import java.util.Calendar;

import javax.swing.SwingWorker;

import ue.Language;
import ue.UE;

/**
 * Use this with Tools.runUEWorker()
 */
public abstract class UEWorker<T> extends SwingWorker <T, Object>
{
    private String oldMSG = ""; //$NON-NLS-1$
    private Integer oldSTATUS = 0;
    private Boolean oldVISIBLE = true;
    
    protected T RESULT = null;
    
    public static final String MESSAGE = "msg"; //$NON-NLS-1$
    public static final String STATUS = "status"; //$NON-NLS-1$
    public static final String VISIBLE = "visible"; //$NON-NLS-1$
    public static final Integer STATUS_STARTED = 0;
    public static final Integer STATUS_WORKING = 1;
    public static final Integer STATUS_ERROR = 2;
    public static final Integer STATUS_DONE = 3;
    public static final Integer STATUS_CANCEL = 4;
    public static final Integer STATUS_DONE_WITH_MSG = 5;
    
    /**
     * Send message via property listeners.
     * @param msg the new message
     */
    protected void sendMessage(String msg)
    {
        firePropertyChange(MESSAGE, oldMSG, msg);
        oldMSG = msg;
    }
    
    /**
     * Send status via property listeners.
     * @param status the new status
     */
    protected void updateStatus(int status)
    {
        firePropertyChange(STATUS, oldSTATUS, new Integer(status));
        oldSTATUS = new Integer(status);
    }

    protected void setProgressVisible(boolean b)
    {
        firePropertyChange(VISIBLE, oldVISIBLE, b);
        oldVISIBLE = b;
    }
    
    /**
     * Reports error via property listeners. And cancelles task.
     * @param error the error
     */
    protected void reportError(Exception e)
    {
        // print stack trace
        SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss dd.MM.yyyy"); //$NON-NLS-1$
        System.err.println("\n" + UE.APP_NAME + " " + UE.APP_VER + " - " //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                + sdf.format(Calendar.getInstance().getTime()));
        e.printStackTrace();
        cancel(false);
        sendMessage(Language.getString("UEWorker.0").replace("%1", e.getLocalizedMessage())); //$NON-NLS-1$ //$NON-NLS-2$
        updateStatus(STATUS_ERROR);
    }
    
    /**
     * @return null
     */
    public T getResult()
    {
        return RESULT;
    }    
}
