/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: Wave.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.util;

import java.io.*;

import ue.Language;
import ue.exception.*;

public class Wave{
	//this
	private int SNDTYPE;
	//header
	private final int strRIFF = 1179011410;
	private final int strWAVE = 1163280727;
	private final int strFMT = 544501094;
	private int WAVE_CHUNK_SIZE;
	private short FORMAT;
	private short CHANNELS;
	private int FREQUENCY;
	private int BYTES_PER_SECOND;
	private short BLOCK_ALIGMENT;
	private short BITS_PER_SAMPLE;
	private short EXTRA_FMT;
	private short SAMPLES_PER_BLOCK;
	private final int strFACT = 1952670054;
	private final int intFACT = 4;
	private int FRAMES;
	private final int strDATA = 1635017060;
	
	private byte[] DATA;
	
	/**Constructor to insert raw data
	type:
	1-english
	2-sfx
	3-music*/
	public Wave(int sndtype, byte[] data, int frames) throws UnsupportedWave{
		SNDTYPE = sndtype;
		switch (sndtype){
			case 1:{
				//english.snd
				WAVE_CHUNK_SIZE = 20;
				FORMAT = 17;
				CHANNELS = 1;
				FREQUENCY = 22050;
				BYTES_PER_SECOND = 11100;
				BLOCK_ALIGMENT = 512;
				BITS_PER_SAMPLE = 4;
				EXTRA_FMT = 2;
				SAMPLES_PER_BLOCK = 1017;
				FRAMES = frames;
				DATA = new byte[data.length];
				System.arraycopy(data, 0, DATA, 0, data.length);
				break;
			}
			case 2:{
				//sfx.snd
				WAVE_CHUNK_SIZE = 16;
				FORMAT = 1;
				CHANNELS = 1;
				FREQUENCY = 22050;
				BYTES_PER_SECOND = 44100;
				BLOCK_ALIGMENT = 2;
				BITS_PER_SAMPLE = 16;
				FRAMES = frames;
				DATA = new byte[data.length];
				System.arraycopy(data, 0, DATA, 0, data.length);
				break;
			}
			case 3:{
				//music.snd
				WAVE_CHUNK_SIZE = 20;
				FORMAT = 17;
				CHANNELS = 2;
				FREQUENCY = 22050;
				BYTES_PER_SECOND = 22201;
				BLOCK_ALIGMENT = 1024;
				BITS_PER_SAMPLE = 4;
				EXTRA_FMT = 2;
				SAMPLES_PER_BLOCK = 1017;
				FRAMES = frames;
				DATA = new byte[data.length];
				System.arraycopy(data, 0, DATA, 0, data.length);
				break;
			}
			default:{
				throw new UnsupportedWave(Language.getString("Wave.0")); //$NON-NLS-1$
			}
		}
	}
	/**Constructor#2*/
	public Wave(byte[] data) throws Exception{
		byte[] b = new byte[4];
		int index=0;
		//RIFF 4
		for (int i=0; i<4; i++) {
			b[i] = data[index++];
		}
		if (strRIFF!=Tools.toInt(b, true)) {
			throw new Exception(Language.getString("Wave.1")); //$NON-NLS-1$
		}
		//size-8 8
		for (int i=0; i<4; i++) {
			b[i] = data[index++];
		}
		if (data.length-8!=Tools.toInt(b, true)) {
			throw new Exception(Language.getString("Wave.2")); //$NON-NLS-1$
		}
		//WAVE 12
		for (int i=0; i<4; i++) {
			b[i] = data[index++];
		}
		if (strWAVE!=Tools.toInt(b, true)) {
			throw new Exception(Language.getString("Wave.3")); //$NON-NLS-1$
		}
		//FMT 16
		for (int i=0; i<4; i++) {
			b[i] = data[index++];
		}
		if (strFMT!=Tools.toInt(b, true)) {
			throw new Exception(Language.getString("Wave.4")); //$NON-NLS-1$
		}
		//wave chunk size 20
		for (int i=0; i<4; i++) {
			b[i] = data[index++];
		}
		WAVE_CHUNK_SIZE=Tools.toInt(b, true);
		//switch
		switch (WAVE_CHUNK_SIZE){
			case 20:{
				//english and music
				//format
				for (int i=0; i<2; i++) {
					b[i] = data[index++];
				}
				if (Tools.toShort(b, true)!=17) {
					throw new Exception(Language.getString("Wave.5")); //$NON-NLS-1$
				}
				FORMAT=17;
				//channels
				for (int i=0; i<2; i++) {
					b[i] = data[index++];
				}
				CHANNELS=Tools.toShort(b, true);
				switch (CHANNELS){
					case 1:{
						//english.snd
						//frequency
						for (int i=0; i<4; i++) {
							b[i] = data[index++];
						}
						if (Tools.toInt(b, true)!=22050) {
							throw new Exception(Language.getString("Wave.6")); //$NON-NLS-1$
						}
						FREQUENCY = 22050;
						//bytes per sec
						for (int i=0; i<4; i++) {
							b[i] = data[index++];
						}
						if (Tools.toInt(b, true)!=11100) {
							throw new Exception(Language.getString("Wave.7")); //$NON-NLS-1$
						}
						BYTES_PER_SECOND = 11100;
						//block aligment
						for (int i=0; i<2; i++) {
							b[i] = data[index++];
						}
						if (Tools.toShort(b, true)!=512) {
							throw new Exception(Language.getString("Wave.8")); //$NON-NLS-1$
						}
						BLOCK_ALIGMENT = 512;
						//bits per sample
						for (int i=0; i<2; i++) {
							b[i] = data[index++];
						}
						if (Tools.toShort(b, true)!=4) {
							throw new Exception(Language.getString("Wave.9")); //$NON-NLS-1$
						}
						BITS_PER_SAMPLE = 4;
						//extra chunk size
						for (int i=0; i<2; i++) {
							b[i] = data[index++];
						}
						if (Tools.toShort(b, true)!=2) {
							throw new Exception(Language.getString("Wave.10")); //$NON-NLS-1$
						}
						EXTRA_FMT = 2;
						//samples per block
						for (int i=0; i<2; i++) {
							b[i] = data[index++];
						}
						if (Tools.toShort(b, true)!=1017) {
							throw new Exception(Language.getString("Wave.11")); //$NON-NLS-1$
						}
						SAMPLES_PER_BLOCK = 1017;
						//FACT
						for (int i=0; i<4; i++) {
							b[i] = data[index++];
						}
						if (strFACT!=Tools.toInt(b, true)) {
							throw new Exception(Language.getString("Wave.12")); //$NON-NLS-1$
						}
						//intFACT
						for (int i=0; i<4; i++) {
							b[i] = data[index++];
						}
						if (intFACT!=Tools.toInt(b, true)) {
							throw new Exception(Language.getString("Wave.13")); //$NON-NLS-1$
						}
						//frames/4
						for (int i=0; i<4; i++) {
							b[i] = data[index++];
						}
						FRAMES=Tools.toInt(b, true)*4;
						//DATA
						for (int i=0; i<4; i++) {
							b[i] = data[index++];
						}
						if (strDATA!=Tools.toInt(b, true)) {
							throw new Exception(Language.getString("Wave.14")); //$NON-NLS-1$
						}
						//data size
						for (int i=0; i<4; i++) {
							b[i] = data[index++];
						}
						if (data.length-60<Tools.toInt(b, true)) {
							throw new Exception(Language.getString("Wave.15")); //$NON-NLS-1$
						}
						//copy data
						DATA = new byte[data.length-60];
						System.arraycopy(data, 60, DATA, 0, data.length-60);
						SNDTYPE=1;
						break;
					}
					case 2:{
						//music.snd
						//frequency
						for (int i=0; i<4; i++) {
							b[i] = data[index++];
						}
						if (Tools.toInt(b, true)!=22050) {
							throw new Exception(Language.getString("Wave.16")); //$NON-NLS-1$
						}
						FREQUENCY = 22050;
						//bytes per sec
						for (int i=0; i<4; i++) {
							b[i] = data[index++];
						}
						if (Tools.toInt(b, true)!=22201) {
							throw new Exception(Language.getString("Wave.17")); //$NON-NLS-1$
						}
						BYTES_PER_SECOND = 22201;
						//block aligment
						for (int i=0; i<2; i++) {
							b[i] = data[index++];
						}
						if (Tools.toShort(b, true)!=1024) {
							throw new Exception(Language.getString("Wave.18")); //$NON-NLS-1$
						}
						BLOCK_ALIGMENT = 1024;
						//bits per sample
						for (int i=0; i<2; i++) {
							b[i] = data[index++];
						}
						if (Tools.toShort(b, true)!=4) {
							throw new Exception(Language.getString("Wave.19")); //$NON-NLS-1$
						}
						BITS_PER_SAMPLE = 4;
						//extra chunk size
						for (int i=0; i<2; i++) {
							b[i] = data[index++];
						}
						if (Tools.toShort(b, true)!=2) {
							throw new Exception(Language.getString("Wave.20")); //$NON-NLS-1$
						}
						EXTRA_FMT = 2;
						//samples per block
						for (int i=0; i<2; i++) {
							b[i] = data[index++];
						}
						if (Tools.toShort(b, true)!=1017) {
							throw new Exception(Language.getString("Wave.21")); //$NON-NLS-1$
						}
						SAMPLES_PER_BLOCK = 1017;
						//FACT
						for (int i=0; i<4; i++) {
							b[i] = data[index++];
						}
						if (strFACT!=Tools.toInt(b, true)) {
							throw new Exception(Language.getString("Wave.22")); //$NON-NLS-1$
						}
						//intFACT
						for (int i=0; i<4; i++) {
							b[i] = data[index++];
						}
						if (intFACT!=Tools.toInt(b, true)) {
							throw new Exception(Language.getString("Wave.23")); //$NON-NLS-1$
						}
						//frames/4
						for (int i=0; i<4; i++) {
							b[i] = data[index++];
						}
						FRAMES=Tools.toInt(b, true)*4;
						//DATA
						for (int i=0; i<4; i++) {
							b[i] = data[index++];
						}
						if (strDATA!=Tools.toInt(b, true)) {
							throw new Exception(Language.getString("Wave.24")); //$NON-NLS-1$
						}
						//data size
						for (int i=0; i<4; i++) {
							b[i] = data[index++];
						}
						if (data.length-60<Tools.toInt(b, true)) {
							throw new Exception(Language.getString("Wave.25")); //$NON-NLS-1$
						}
						//copy data
						DATA = new byte[data.length-60];
						System.arraycopy(data, 60, DATA, 0, data.length-60);
						SNDTYPE=3;
						break;
					}
					default:{
						throw new Exception(Language.getString("Wave.26")); //$NON-NLS-1$
					}
				}
				break;
			}
			case 16:{
				//format 22
				for (int i=0; i<2; i++) {
					b[i] = data[index++];
				}
				if (Tools.toShort(b, true)!=1) {
					throw new Exception(Language.getString("Wave.27")); //$NON-NLS-1$
				}
				FORMAT=1;
				//channels 24
				for (int i=0; i<2; i++) {
					b[i] = data[index++];
				}
				if (Tools.toShort(b, true)!=1) {
					throw new Exception(Language.getString("Wave.28")); //$NON-NLS-1$
				}
				CHANNELS = 1;
				//frequency 28
				for (int i=0; i<4; i++) {
					b[i] = data[index++];
				}
				if (Tools.toInt(b, true)!=22050) {
					throw new Exception(Language.getString("Wave.29")); //$NON-NLS-1$
				}
				FREQUENCY = 22050;
				//bytes per sec 32
				for (int i=0; i<4; i++) {
					b[i] = data[index++];
				}
				if (Tools.toInt(b, true)!=44100) {
					throw new Exception(Language.getString("Wave.30")); //$NON-NLS-1$
				}
				BYTES_PER_SECOND = 44100;
				//block aligment 34
				for (int i=0; i<2; i++) {
					b[i] = data[index++];
				}
				if (Tools.toShort(b, true)!=2) {
					throw new Exception(Language.getString("Wave.31")); //$NON-NLS-1$
				}
				BLOCK_ALIGMENT = 2;
				//bits per sample 36
				for (int i=0; i<2; i++) {
					b[i] = data[index++];
				}
				if (Tools.toShort(b, true)!=16) {
					throw new Exception(Language.getString("Wave.32")); //$NON-NLS-1$
				}
				BITS_PER_SAMPLE = 16;
				//DATA 40
				for (int i=0; i<4; i++) {
					b[i] = data[index++];
				}
				if (strDATA!=Tools.toInt(b, true)) {
					throw new Exception(Language.getString("Wave.33")); //$NON-NLS-1$
				}
				//data size 44
				for (int i=0; i<4; i++) {
					b[i] = data[index++];
				}
				if (data.length-44<Tools.toInt(b, true)) {
					throw new Exception(Language.getString("Wave.34")); //$NON-NLS-1$
				}
				//copy data
				DATA = new byte[data.length-44];
				System.arraycopy(data, 44, DATA, 0, data.length-44);
				SNDTYPE=2;
				//System.out.println(data.length-44);
				break;
			}
			default:{
				throw new Exception(Language.getString("Wave.35")); //$NON-NLS-1$
			}
		}
	}
	/**Saves data to OutputStream*/
	public void save(OutputStream out) throws IOException{
		//RIFF
		out.write(Tools.toByte(strRIFF, true));
		if (SNDTYPE==2){
			//size-8	(4)
			out.write(Tools.toByte((DATA.length+36), true));
			//WAVE		(8)
			out.write(Tools.toByte(strWAVE, true));
			//FMT		(12)
			out.write(Tools.toByte(strFMT, true));
			//wave chunk size(16)
			out.write(Tools.toByte(WAVE_CHUNK_SIZE, true));
			//format	(18)
			out.write(Tools.toByte(FORMAT, true));
			//channels	(20)
			out.write(Tools.toByte(CHANNELS, true));
			//frequency	(24)
			out.write(Tools.toByte(FREQUENCY, true));
			//bytes per sec	(28)
			out.write(Tools.toByte(BYTES_PER_SECOND, true));
			//block aligment(30)
			out.write(Tools.toByte(BLOCK_ALIGMENT, true));
			//bits per sample(32)
			out.write(Tools.toByte(BITS_PER_SAMPLE, true));
		}else if (SNDTYPE==1 || SNDTYPE==3){
			//size-8
			out.write(Tools.toByte((DATA.length+52), true));
			//WAVE
			out.write(Tools.toByte(strWAVE, true));
			//FMT
			out.write(Tools.toByte(strFMT, true));
			//wave chunk size
			out.write(Tools.toByte(WAVE_CHUNK_SIZE, true));
			//format
			out.write(Tools.toByte(FORMAT, true));
			//channels
			out.write(Tools.toByte(CHANNELS, true));
			//frequency
			out.write(Tools.toByte(FREQUENCY, true));
			//bytes per sec
			out.write(Tools.toByte(BYTES_PER_SECOND, true));
			//block aligment
			out.write(Tools.toByte(BLOCK_ALIGMENT, true));
			//bits per sample
			out.write(Tools.toByte(BITS_PER_SAMPLE, true));
			//extra chunk size
			out.write(Tools.toByte(EXTRA_FMT, true));
			//samples per block
			out.write(Tools.toByte(SAMPLES_PER_BLOCK, true));
			//FACT
			out.write(Tools.toByte(strFACT, true));
			//intFACT
			out.write(Tools.toByte(intFACT, true));
			//frames/4
			out.write(Tools.toByte((FRAMES/4), true));
		}else{
			throw new IOException(Language.getString("Wave.36")); //$NON-NLS-1$
		}
		//DATA (36)
		out.write(Tools.toByte(strDATA, true));
		//data size(40)
		out.write(Tools.toByte(DATA.length, true));
		//write data
		out.write(DATA);
	}
	public byte[] getRawData(){
		byte[] data = new byte[DATA.length];
		System.arraycopy(DATA, 0, data, 0, DATA.length);
		return data;
	}
	public int getDataSize(){
		return DATA.length;
	}
	public int getSize(){
		int size = DATA.length;
		switch (SNDTYPE){
			case 1:{
				size+=60;
				break;
			}
			case 2:{
				size+=44;
				break;
			}
			case 3:{
				size+=60;
				break;
			}
		}
		return size;
	}
	public int getFrames(){ return FRAMES;}
	public int getSndType(){ return SNDTYPE;}
}
