/*
	Ultimate Editor for Birth Of The Federation 1.02
	Copyright (C) 2005-2007  Alan Podlesek
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
	
	
	File: ZipReader.java
	
	Written by: Alan Podlesek <seqdcer@yahoo.com>
	
	Last modified: 6.9.2007
*/

package ue.util;

import java.io.*;

import ue.Language;

/**Used to encapsulate the InputStream provided by ZipFile. This is because 
that stream reads bytes irregularly. This stream reads the whole underlying stream and then passes data forward as needed.*/
public class ZipReader extends InputStream{
	private byte[] buffer;
	private long index=0;
	/**Constructor.*/
	public ZipReader(InputStream in) throws IOException{
		try{
			buffer = new byte[in.available()];
			int off = 0;
			while (in.available()>0){
				int len = in.read(buffer, off, in.available());
				off+=len;
			}
		}catch (Exception e){
			throw new IOException(e.getMessage());
		}
	}
	/**Returns the number of bytes available in this stream.*/
	@Override
	public int available() throws IOException{
		try{
			if (buffer.length - index<=0) {
				return -1;
			}
			return (buffer.length - (int)index);
		}catch (Exception e){
			throw new IOException(Language.getString("ZipReader.0")); //$NON-NLS-1$
		}
	}
	/**Closes this and the underlying stream.*/
	@Override
	public void close() throws IOException{
		buffer=null;
		super.close();
	}
	
	@Override
	public long skip(long n) throws IOException{
		try{
			if (buffer.length - (index + n) < 0) {
				throw new IOException(Language.getString("ZipReader.1")); //$NON-NLS-1$
			} else {
				index+=n; //$NON-NLS-1$
			}
		}catch (Exception e){
			throw new IOException(Language.getString("ZipReader.0")); //$NON-NLS-1$
		}
		return n;
	}
	
	@Override
	public int read(byte[] b, int off, int len) throws IOException{
		if (len > buffer.length - index) {
			len = buffer.length - (int)index;
		}
		for (int i=0; i<len; i++){
			b[off+i] = buffer[(int)index];
			index++;
		}
		return len;
	}
	
	@Override
	public int read(byte[] b) throws IOException{
		int len=0;
		try{
			if (buffer.length - index > b.length){
				int i;
				for (i=0; i< b.length; i++){
					b[i] = buffer[(int)index];
					index++;
					len++;
				}
			} else {
				int i;
				long dupe = buffer.length - index;
				for (i=0; i < dupe; i++){
					b[i] = buffer[(int)index];
					index++;
					len++;
				}
			}
		}catch (Exception e){
			throw new IOException(Language.getString("ZipReader.0")); //$NON-NLS-1$
		}
		return len;
	}
	
	@Override
	public int read() throws IOException{
		int r=0;
		try{
			r = buffer[(int)index];
			index++;
		}catch (Exception e){
			throw new IOException(Language.getString("ZipReader.0")); //$NON-NLS-1$
		}
		return r;
	}
	
	@Override
	public boolean markSupported(){ return false;}
}

